\page FAQ Frequently Asked Questions

\brief Not really an FAQ but rather a poorly sorted collection of pieces of information.

\tableofcontents


\section SecFAQHardProcess Hard Process


\section SecFAQPartonShower Parton Shower

* **Parton Shower Turn-On**
  
  The \c hdamp parameter in PowhegBox steers the way the parton shower is smoothly switched on on the hard emission side.

  The easiest way of doing the turn-on would be a hard theta function which blocks every emission above a certain scale and allows everything below that same scale. This would still be correct at the given (fixed) order but would introduce sub-leading terms that may show up as steps/discontinuities in distributions. Therefore, one may use a smooth function interpolating between 1 (allowing emissions) below a certain scale and 0 (vetoing emissions) above another (higher) scale. The Powheg way of doing this should be the function

  \f[
    D(q) = \frac{1}{1 + q^2/Q^2}
  \f]

  where \f$ Q \f$ is the \c hdamp parameter and \f$ q \f$ is the scale associated to the current event configuration.

  The functions that control this transition behaviour are called *hard scale profile* in Herwig. There are more choices beyond the PowhegBox way of doing this (called ``HFact`` in Herwig), the additional alternatives being the theta function and the Herwig default (called *resummation scale*). The resummation scale alternative in principle does the same but has a slightly different way of interpolating between the 0 and the 1. Also, the Herwig authors stress that this option correctly reproduces the towers of logarithms generated by the parton shower (according to Simon Plätzer the PowhegBox default choice does not preserve the logs correctly).

  However, the choice of this damping function is something that in principle the authors do for the various shower/matching combinations.

  The snippet ``Powheg-DefaultShower.in`` includes the line

      set /Herwig/Shower/PowhegShowerHandler:HardScaleProfile NULL

  But this only applies to the parton shower beginning with the second emission. The first emission is handled in the MEMatching object.

  The damping function \f$ D \f$ provides a factor that is multiplied on top of the event weight before it is passed to the parton shower - either via an LHE file or internally within Herwig 7.


* **PDF Sudakov Veto Warning**

  When showering with Herwig, warnings similar to the following may appear:

      PDFVeto warning: Ratio > GtobbbarSudakov:PDFmax (by a factor of 1.749) for g to b 
      PDFVeto warning: Ratio > GtobbbarSudakov:PDFmax (by a factor of 1.286) for g to bbar

  They stem from the overweight estimation in the Sudakov veto algorithm that parton shower Monte Carlos use to generate the sequence of emissions. As of Herwig versions at least up until 7.0.3 there is a manual overestimate in place that does not strictly always overestimate the true values. According to the Herwig authors this shouldn't be problematic as long as a fraction of less than about 1% of the events is concerned. Fractions observed for showering PowhegBox and MG5_aMC@NLO LHE files with Herwig 7.0.1 are in the ballpark of < 0.1%.


* **Azimuthal Angle Correlations**

  When showering with Herwig, the warning
  
      Too many tries to generate phi in forward evolution

  may appear. Again, this shouldn't be worrisome as long as less than about 1% of the events are concerned. Fractions observed for showering PowhegBox and MG5_aMC@NLO LHE files with Herwig 7.0.1 are in the ballpark of < 0.2%.


* **Parton Shower Reweighting** ([Herwig Online Documentation](http://herwig.hepforge.org/tutorials/showers/qtilde.html#on-the-fly-reweighting-for-scale-variations))
  
  Scale variations in the parton shower can be done with:

      do {Dipole,}Showerhandler:AddVariation NameOfVariation XiR XiF {Hard,Secondary,All}

  The convergence of the reweighted distributions can be improved by setting a detuning parameter

    * ``set /Herwig/Shower/SplittingGenerator:Detuning 2.0`` for the angular-ordered/default shower (beginning with Herwig 7.0.3)
    * ``set DipoleShowerHandler:Detuning 2.0`` for the dipole shower (beginning with Herwig 7.0.2)

  Further reading:

    * [arXiv:1605.08256 [hep-ph]](http://arxiv.org/abs/1605.08256): Reweighting Parton Showers



\subsection SubSecFAQAngularOrderedShower Angular-Ordered Parton Shower

* **Strong Coupling** \f$ \alpha_s \f$
  
  The initial value of \f$ \alpha_s \f$ is not taken from the PDFs but rather from ``/Herwig/Shower/NLOAlphaS`` where a value for a certain scale can be specified. The default value is obtained from the Herwig authors as the result of tuning of the various shower/matching(?) combinations to LEP data.

* **Kinematics reconstruction**
  
  There are many different options and settings, such as

  * ``/Herwig/Shower/KinematicsReconstructor:ReconstructionOption``

    Since the default parton shower has \f$ 1\to2 \f$ splitting functions some reshuffling of momenta or recoiling is necessary to put the particles back on the appropriate mass shells. While all of the recoils are perfermed at the end of the parton shower and are *global* there are still freedoms in how exactly to do them. The first option ``0/General`` preserves the mass and rapidity of the final state which makes sense for Drell-Yan. ``3/Colour3`` and ``4/Colour4`` are oriented closer to the colour structure of the hard process. In particular, ``3/Colour3`` and ``4/Colour4`` replicate the dipole kinematics for the first parton shower emission.

    * ``0/General``: Use the general solution which ignores the colour structure for all processes
    * ``1/Colour``: Use the colour structure of the process to determine the reconstruction procedure
    * ``2/Colour2``: Make the most use possible of the colour structure of the process to determine the reconstruction procedure. Start with FF, then IF then II colour connections
    * ``3/Colour3``: Make the most use possible of the colour structure of the process to determine the reconstruction procedure. Do the colour connections in order of the pT's emitted in the shower starting with the hardest. The colour partner is fully reconstructed at the same time.
    * ``4/Colour4``: Make the most use possible of the colour structure of the process to determine the reconstruction procedure. Do the colour connections in order of the pT's emitted in the shower starting with the hardest, while leaving the colour partner on mass-shell

    Therefore, in principle ``3/Colour3`` and ``4/Colour4`` are preferable, with ``3/Colour3`` being the default as of Herwig 7.0.0, and are used and should be used for the internal matching. However, when showering externally provided MG5\_aMC\NLO LHE files, still the option ``0/General`` has to be used because that is what MG5\_aMC\NLO provides in their (parton shower-specific) matching subtraction terms.

  * ``/Herwig/Shower/KinematicsReconstructor:InitialInitialBoostOption``

    This setting concerns how the overall boost that the initial state parton shower creates w.r.t. to the hard process is to be treated.

    * ``0/OneBoost``: Apply one boost from old CMS to new CMS
    * ``1/LongTransBoost``: First apply a longitudinal and then a transverse boost

  * ``/Herwig/Shower/KinematicsReconstructor:FinalStateReconOption``

    * `0/Default`: All the momenta are rescaled in the rest frame

      This is the *democratic* option where all three-momenta are rescaled simultaneously in their common rest frame, solving the equation
      \f[
        \sum_{i} \sqrt{\vec{p}_{i, old}^2 + m_{i, old}^2} = \sum_{i} \sqrt{\vec{p}_{i, new}^2 + m_{i, new}^2}
      \f]
      where the index \f$ i \f$ runs over all particles.

    * ``1/MostOffShell``: All particles put on the new-mass shell and then the most off-shell and recoiling system are rescaled to ensure 4-momentum is conserved.

      * Won't have any effect for ``set /Herwig/Shower/KinematicsReconstructor:ReconstructionOption Colour3``

    * ``2/Recursive``: Recursively put on shell by putting the most off-shell particle which hasn't been rescaled on-shell by rescaling the particles and the recoiling system. 
    * ``3/RestMostOffShell``: The most off-shell is put on shell by rescaling it and the recoiling system, the recoiling system is then put on-shell in its rest frame.
    * ``4/RestRecursive``: As 3 but recursive treated the currently most-off shell, only makes a difference if more than 3 partons.

    The other options ``1/MostOffShell``, ``2/Recursive``, ``3/RestMostOffShell`` and ``4/RestRecursive`` are not democratic in the sense that they don't do the rescaling simultaneously for all particles.


* **Showering of LHE Files**

  Discussion of the appropriate settings together with the authors in this [Google Docs spreadsheet](https://docs.google.com/spreadsheets/d/1USUnQed5H8_r0y2f4sLJmY1il8G-yTeW2lnzqqO_5_I/edit?pli=1#gid=0) (at the moment access permission needs to be requested from Daniel Rauch)

  * **Showering of PowhegBox LHE files**

    * ``set /Herwig/Shower/KinematicsReconstructor:ReconstructionOption 3 # [Colour3]``

      * which is the default anyway

    * ``set /Herwig/Shower/KinematicsReconstructor:InitialInitialBoostOption 1 # [LongTransBoost]``

      * which is the default anyway

    * ``set /Herwig/Shower/ShowerHandler:RestrictPhasespace 1 # [On]``

      * which is the default anyway

    * ``set /Herwig/Shower/ShowerHandler:MaxPtIsMuF 1 # [Yes]``

      * still needs testing by Dominic Hirschbühl


  * **Showering of MG5_aMCatNLO LHE files**

    * ``set /Herwig/Shower/KinematicsReconstructor:ReconstructionOption 0 # [General]``

    * ``set /Herwig/Shower/KinematicsReconstructor:InitialInitialBoostOption 1 # [LongTransBoost]``

      * which is the default anyway

    * ``set /Herwig/Shower/KinematicsReconstructor:FinalStateReconOption 0 # [Default]``

      * which is the default anyway

    * ``set /Herwig/Shower/KinematicsReconstructor:InitialStateReconOption 0 # [Rapidity]``

    * ``set /Herwig/Shower/ShowerHandler:RestrictPhasespace 1 # [On]``

      * which is the default anyway

    * ``set /Herwig/Shower/ShowerHandler:MaxPtIsMuF 1 # [Yes]``

      * recommended by the Herwig authors, but according to test by Dominic Hirschbühl this screws up the jet \f$p_T\f$ distributions in \f$pp\to t\bar{t}\f$: [Rivet plots](http://www.atlas.uni-wuppertal.de/~hirsch/RivetAnalyses/ttbarHerwig7ParameterTests/ATLAS_2014_I1304688/index.html). Therefore, currently still left at the default value of ``0 # [No]``



\subsection SubSecFAQDipoleShower Dipole Parton Shower


\section SecFAQDecays Decays

* **Selecting Specific Decay Channels**
  
  Specific decay channels can be selected with commands such as

      do /Herwig/Particles/W+:SelectDecayModes W+->nu_mu,mu+;

  Multiple decay channels can be requested by doing

      do /Herwig/Particles/Z0:SelectDecayModes /Herwig/Particles/Z0/Z0->e-,e+; /Herwig/Particles/Z0/Z0->mu-,mu+;

  The active decay modes can be listed with

      do /Herwig/Particles/W-:PrintDecayModes

  The total cross section indicated by Herwig really is cross section times branching ratio so when selecting only a subset of the possible decays it decreases accordingly. Apparently, there is some class in ``Decay/BranchingRatioReweighter.[h/cc]`` to undo this - but this doesn't seem to be used in the default repository.

  The warning

      Warning: Total BR for Z0 does not add up to 1. sum = 0.067242

  that appears when only selecting a subset of the possible decay modes can be ignored - the active decays are chosen with the correct relative probabilities.

* Colour-charged decay products stemming from (perturbative) decays that are handled in the parton showers are subject to colour-reconnection across the complete event - even if the decaying particle was a colour singlet. By looking at

      get /Herwig/Shower/ShowerHandler:DecayInShower

  one can find the particles that are decayed in the shower - this is the case for \f$ W \f$ and \f$ Z \f$ bosons as well as the \f$ t \f$ quark and the \f$ h_0 \f$ boson.

  Decays of \f$ \tau \f$ leptons are handled differently, the motivation being the much longer lifetime. They are not decayed perturbatively but decay to hadrons directly. Then of course, since these are colour singlets there is no colour rearrangement.

  Also, products from decays with lengths above 1000 fm are excluded from the colour reconnection (of course the Higgs with a decay length \f$ c\tau\approx 48~\text{fm} \f$ is below that limit for now - the Herwig authors are going to discuss and reconsider this value, though).


\section SecFAQHadronization Hadronization


\section SecFAQUnderlyingEvent Underlying Event


\section SecFAQMinBias Minimum Bias

* Currently the Minimum Bias simulation in Herwig uses a dummy matrix element that basically triggers the Underlying Event with many Multi-Parton Interactions, thus generating a number of QCD hard and soft \f$ 2\to2 \f$ scatters. However, a good portion of soft scatters is made up of diffractive scatters, about which in reality a bit more is known than is included in the soft scatters in Herwig: Diffractive events are typically of lower multiplicity, result in particles with lower \f$ p_T \f$ and usually there is a rapidity gap. When comparing to data Herwig always yields a bad description for low multiplicities (approximately \f$ <5 \f$ for \f$ \sqrt{s}=7 \f$ TeV at the LHC). Pythia includes a diffractive contribution added in by hand in that regime. Conversely, the hard part of Min Bias works quite well and thus also the Underlying Event in triggered processes.


\section SecFAQTuning Tuning

* The Herwig authors use a multi-step strategy for tuning the generator:

  1. Tuning to LEP light flavour observables
  2. Tuning to LEP charm observables
  3. Tuning to LEP bottom observables
  4. Tuning of the underlying event

  Parton shower parameters to be tuned are \f$ \alpha_s(M_Z) \f$ and the lower parton shower \f$ p_T \f$ cut-off.

  At the moment the benefit of tuning the parton shower to LHC data as well (LHC data is included for UE tuning, though, of course!) is unclear - mostly because there are no large deviations visible in event shape variables (shower shapes) at the moment with the PS tunes to LEP data that would suggest a real benefit for the CPU- and time effort to be spent.


\section SecFAQRunModes Run Modes

The possible commands and their parameters are:

- For doing the read step in one go:

      Herwig read <infile>   # *.in file with generator settings

- For splitting up the read step into separate build step, possibly several parallel integrate steps and a mergegrids steps:

  - Figure out the subprocesses and build the matrix elements
  
        Herwig build <infile> \    # *.in file with generator settings
          -z/--jobsize=<jobs> \    # number of subprocesses for each integration job
          -y/--maxjobs=<maxjobs>   # number of integration jobs to prepare

  - Integrate the subprocesses

        Herwig integrate <runfile> \   # *.run file that contains the isolated event generator
          -n/--jobid=<jobid> \         # id of integration job to process
          -t/--tag=<tag> \             # this attaches a suffix to the generator name and the outfiles (*.out/log/tex/yoda/hepmc)
          -x/--setupfile=<setupfile>   # modify event generator with some further commands in addition to those in the infile

  - Merge the integration grids

        Herwig mergegrids <runfile> \   # *.run file that contains the isolated event generator
          -t/--tag=<tag> \              # this attaches a suffix to the generator name and the outfiles (*.out/log/tex/yoda/hepmc)
          -x/--setupfile=<setupfile>    # modify event generator with some further commands in addition to those in the infile
                                        # (has to be set if a setupfile was used for integrate step)

- For doing the run step
  
      Herwig run <runfile> \           # *.run file that contains the isolated event generator
        -N/--numevents=<events> \      # number of events to be simulated
        -s/--seed=<seed> \             # seed for the random number generator
        -j/--jobs=<jobs> \             # number of jobs (threads) that should be spawned
        -t/--tag=<tag> \               # this attaches a suffix to the generator name and the outfiles (*.out/log/tex/yoda/hepmc)
        -x/--setupfile=<setupfile> \   # modify event generator with some further commands before doing the event generation

  Only certain settings can be modified via the setupfile mechanism. The reason for this is that for the creation of the runfile the event generator is isolated, i.e. objects in the repository that are not relevant for the run are not kept. Thus, when running with the default (i.e. angular-ordered) parton shower variation of dipole shower scales lead to errors.

  The setupfile mechanism may also only be applied for the run step. Then, however, only changes should be part of the setupfile that don't alter too much the hard process since the grid adaption won't be repeated that way ([Herwig Online Documentation](http://herwig.hepforge.org/tutorials/workflow/eventgeneration.html#modifying-event-generators)).


\section SecFAQRepository Repository Bits and Pieces

- **The EventHandler Class and its Handlers**

  In ``/Herwig/EventHandlers`` there are by default the EventHandlers ``LEPHandler`` and ``LHCHandler`` and the Matchbox snippets set a generic ``EventHandler`` object. All of them are instances of the class ``ThePEG::StandardEventHandler`` which has a number of pointers to further handlers as members:

  Hard Process

  - PostSubProcessHandlers

  Parton Showering

  - PreCascadeHandlers
  - CascadeHandler

    - CascadeHandler:MPIHandler

  - PostCascadeHandlers

  Hadronization

  - PreHadronizationHandlers
  - HadronizationHandlers
  - PostHadronizationHandlers

  Decays

  - PreDecayHandlers
  - DecayHandler
  - PostDecayHandlers

  Multiple Parton Interactions

  - PreMultipleInteractionHandlers
  - MultipleInteractionHandler
  - PostMultipleInteractionHandlers

  .. todo::

    What is the difference between the CascadeHandler:MPIHandler and the MultipleInteractionHandler?

    Apparently the EventHandler:MultipleInteractionHandler is switched off, i.e. set to ``NULL`` by default (at least in Herwig 7). To be checked for Herwig 2.7.1.