/*
  Copyright (C) 2002-2017 CERN for the benefit of the ATLAS collaboration
*/

#ifndef JIVEXML_PIXELRDORETRIEVER_H
#define JIVEXML_PIXELRDORETRIEVER_H

#include "JiveXML/IDataRetriever.h"
#include "AthenaBaseComps/AthAlgTool.h"
#include "InDetJiveXML/IInDetGeoModelTool.h"


namespace JiveXML {
  
  /**
   * @class PixelRDORetriever
   * @brief Retrieve all @c PixelRDORawData information (pixel raw hits)
   *
   *  - @b Properties
   *    - <em>PixelRDOContainer</em><tt>= 'PixelRDOs'</tt> @copydoc m_PixelRDOContainerName
   *
   *  - @b Retrieved @b Data
   *    - <em>id</em>: the identifier of the hit
   *    - <em>x,y,z</em>: coordinates of the spacePoint
   *    - <em>phiModule,etaModule</em>: @f$\eta@f$ and @f$\phi@f$ of module in detector coordinates
   *    .
   */
  class PixelRDORetriever : virtual public IDataRetriever,
                                    public AthAlgTool {
    
  public:
    
    /// Standard constructor
    PixelRDORetriever(const std::string& type,const std::string& name,const IInterface* parent);

      /// Retrieve all the data
      virtual StatusCode retrieve(ToolHandle<IFormatTool> &FormatTool); 

      /// Return the name of the data type
      virtual std::string dataTypeName() const { return typeName; };

      /// Only retrieve geo tool in initialize
      virtual StatusCode initialize() { return geo.retrieve(); };
    
    private:
      
      ///The data type that is generated by this retriever
      const std::string typeName;

      /// A tool handle to the geo model tool
      const ToolHandle<IInDetGeoModelTool> geo;

      /// The StoreGate key for the PixelRDO container
      std::string m_PixelRDOContainerName;

  };
}

#endif
