/*
  Copyright (C) 2002-2017 CERN for the benefit of the ATLAS collaboration
*/

#ifndef JIVEXML_TRTRETRIEVER_H
#define JIVEXML_TRTRETRIEVER_H

#include "JiveXML/IDataRetriever.h"
#include "AthenaBaseComps/AthAlgTool.h"
#include "InDetJiveXML/IInDetGeoModelTool.h"

namespace JiveXML{

  
  /**
   * @class TRTRetriever
   * @brief Retrieves all @c InDet::TRT_DriftCircle objects
   *
   *  - @b Properties
   *    - TRTClusters: @copydoc m_TRTDriftCircleCollName
   *    - TRT_TruthMap: @copydoc m_TRTTruthMapName
   *    .
   *
   *  - @b Retrieved @b Data
   *    - <em>id</em> : unique identifier of the drift circle
   *    - <em>rhoz</em> : @f$\rho@f$ (for barrel) or @f$z@f$ (for endcap) coordinate of the hit
   *    - <em>phi</em> : @f$\phi@f$ coordinate of the hit
   *    - <em>driftR</em> : the drift radius
   *    - <em>threshold</em> : wether the hit passed the high threshold or not
   *    - <em>numBarcodes</em> : number of associated truth tracks
   *    - <em>barcodes</em> : barcodes of the associated truth tracks
   *    - <em>sub</em> : the barrel/endcap subdetector in which the drift circle is
   *    - <em>noise</em> : wether the hit is flagged as noise
   *    - <em>timeOverThreshold</em> : time over threshold of the hit
   *    - <em>bitPattern</em> : the status word of the drift circle
   *    .
   */
  class TRTRetriever : virtual public IDataRetriever,
                                   public AthAlgTool {
    
    public:
      
      /// Standard Constructor
      TRTRetriever(const std::string& type,const std::string& name,const IInterface* parent);
      
      /// Retrieve all the data
      virtual StatusCode retrieve(ToolHandle<IFormatTool> &FormatTool); 

      /// Return the name of the data type
      virtual std::string dataTypeName() const { return typeName; };

      /// initialize only geo model tool
      virtual StatusCode initialize() { return geo.retrieve(); }

    private:
      
      ///The data type that is generated by this retriever
      const std::string typeName;
      
      /// A tool handle to the geo model tool
      const ToolHandle<IInDetGeoModelTool> geo;

      /// The StoreGate key for the TRT Cluster collection to retrieve
      std::string m_TRTDriftCircleCollName; 
      /// The StoreGate key for the TRT MultiTruthMap with the track associations
      std::string m_TRTTruthMapName; 
  };

}
#endif
