/*
  Copyright (C) 2002-2017 CERN for the benefit of the ATLAS collaboration
*/

#ifndef JIVEXML_TRIGRPCDATARETRIEVER_H
#define JIVEXML_TRIGRPCDATARETRIEVER_H

#include <string>

#include "JiveXML/IDataRetriever.h"

#include "AthenaBaseComps/AthAlgTool.h"

#include "MuonReadoutGeometry/MuonDetectorManager.h"
#include "MuonReadoutGeometry/RpcReadoutElement.h"


class RpcIdHelper;
class IRPCcablingSvc;

namespace Muon 
{
    class IRPC_RDO_Decoder;
}


namespace JiveXML {

  class TrigRpcDataRetriever : virtual public IDataRetriever, public AthAlgTool {
    
  public:

    /// Standard Constructor
    TrigRpcDataRetriever(const std::string& type,const std::string& name,const IInterface* parent);

      /// Retrieve all the data
    virtual StatusCode retrieve(ToolHandle<IFormatTool> &FormatTool);

    /// Return the name of the data type
    virtual std::string dataTypeName() const { return m_typeName; };

    ///Default AthAlgTool methods
    StatusCode initialize();
   
  private:
    
   ///The data type that is generated by this retriever
    const std::string m_typeName;

    ///The storegate key for the CSC collection
    std::string m_sgKey;

    /// RPC identifier helper
    const RpcIdHelper* m_rpcIdHelper;

    /// RPC cabling Svc
    const IRPCcablingSvc *m_rpcCabling;

    /// RPC decoder
    ToolHandle<Muon::IRPC_RDO_Decoder> m_rpcDecoder; 
    
    ///Geo Model
    const MuonGM::MuonDetectorManager* m_muonMgr;
  };
  
}
#endif
