/*
  Copyright (C) 2002-2017 CERN for the benefit of the ATLAS collaboration
*/

#ifndef JIVEXML_XAODTRACKPARTICLERETRIEVER_H
#define JIVEXML_XAODTRACKPARTICLERETRIEVER_H

#include <string>
#include <vector>
#include <map>

#include "JiveXML/IDataRetriever.h"
#include "AthenaBaseComps/AthAlgTool.h"

#include "xAODTracking/TrackParticleContainer.h" 

namespace JiveXML{
  
  /**
   * @class xAODTrackParticleRetriever
   * @brief Retrieves all @c TrackParticle @c objects (TrackParticleAODCollection etc.)
   *
   *  - @b Properties
   *    - StoreGateKey: First collection to be retrieved, displayed
   *      in Atlantis without switching. All other collections are 
   *      also retrieved.
   *
   *  - @b Retrieved @b Data
   *    - Usual four-vectors: phi, eta, et etc.
   *    - Associations for clusters and tracks via ElementLink: key/index scheme
   */
  class xAODTrackParticleRetriever : virtual public IDataRetriever,
                                   public AthAlgTool {
    
    public:
      
      /// Standard Constructor
      xAODTrackParticleRetriever(const std::string& type,const std::string& name,const IInterface* parent);
      
      /// Retrieve all the data
      virtual StatusCode retrieve(ToolHandle<IFormatTool> &FormatTool); 
      const DataMap getData(const xAOD::TrackParticleContainer*);
    
      /// Return the name of the data type
      virtual std::string dataTypeName() const { return typeName; };

    private:
      ///The data type that is generated by this retriever
      const std::string typeName;

      //@name Property members
      //@{
      /// First track collections to retrieve, shown as default in Atlantis
      std::string m_sgKey;
      /// Track collections to retrieve in the sequence they are given, all if empty
      std::vector<std::string> m_otherKeys;
      /// Whether to write HLTAutoKey objects
      bool m_doWriteHLT;
      //@}
  };
}
#endif
