/*
  Copyright (C) 2002-2017 CERN for the benefit of the ATLAS collaboration
*/

#ifndef JIVEXML_JETRECJETRETRIEVER_H
#define JIVEXML_JETRECJETRETRIEVER_H

#include "JiveXML/IDataRetriever.h"
#include "AthenaBaseComps/AthAlgTool.h"
#include "TrkParameters/TrackParameters.h"
class JetCollection;


namespace JiveXML{
  
  /**
   * @class JetRecJetRetriever
   * @brief Retrieves all @c Jet @c objects (JetEvent/Jet)
   *
   *  - @b Properties
   *    - FavouriteJetCollection
   *    - OtherJetCollections
   *    - DoWriteHLT
   *
   *  - @b Retrieved @b Data
   *    - phi, eta, et, mass, energy, px, py, pz
   *  . - Association: CaloCell: numCells and cell ID (multiple values per Jet)
   */
  class JetRecJetRetriever : virtual public IDataRetriever,
                                   public AthAlgTool {
    
    public:
      
      /// Standard Constructor
      JetRecJetRetriever(const std::string& type,const std::string& name,const IInterface* parent);
      
      /// Retrieve all the data
      virtual StatusCode retrieve(ToolHandle<IFormatTool> &FormatTool); 
      const DataMap getData(const JetCollection*);
      
      /// Return the name of the data type
      virtual std::string dataTypeName() const { return typeName; };

    private:
      ///The data type that is generated by this retriever
      const std::string typeName;

      std::string m_sgKeyFavourite;
      std::string m_trackCollection;
      std::vector<std::string> m_otherKeys;
      bool m_doWriteHLT;
      bool m_writeJetQuality;
      std::vector<const Trk::Perigee*> perigeeVector;

      virtual StatusCode fillPerigeeList(); 
  };
}
#endif
