# Copyright (C) 2002-2017 CERN for the benefit of the ATLAS collaboration

#=======================================================================
# File:   CTB_G4Sim/share/CaloRunConditionsData.py
#
# Contents:
#        Run conditions to simulate the 'good validated' calo CTB data. 
#
#=======================================================================
"""
 Run conditions to simulate the 'good validated' calo CTB data. 

  The minimal information required per run is the 'SimLayout' that must 
 be equal to 'ctbh8_calodata'. This implies that you will not have IDET 
 or Muon detectors and the ancillary upstream MBPSID it will be set to 
 the configuration of the 'PeriodA'. 
  Note that the IDET Oracle tag is 'InnerDetector-CTB-01' but in here we 
 do not have IDET at all.    
  Energy, type of particle or number of events SHOULD NOT BE WRITTEN HERE!!!. 
  For the moment we will run all the runs with the extended barrel  

  The run number must be with more than 4 digits to avoid problems 
 with the run conditions defined in CtbRunConditionsData.py for the 
 combined period. In this file the run number here corresponds to the real 
 run number.       
 Information collected by A. Solodkov and V. Giangiobbe 

"""
__version__="$Revision: 1.3 $"
#
from G4AtlasApps.SimFlags import RunCondition 

#--- Data -------------------------------------------------------------
RunCond=RunCondition('Local')
BeamCond = RunCondition('Local')

# Calorimeter runs with 3 TileCal barrel modules and 3 ext.barrel modules in July 2004

RunCond.set_Run(1000110, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000113, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000117, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000119, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000141, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000147, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000148, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000150, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000154, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000188, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000190, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000192, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000194, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000196, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000222, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000264, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000265, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000267, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000269, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000271, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000272, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000274, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000277, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000278, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000279, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000280, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000281, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000282, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000283, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000284, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000285, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000286, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000289, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000292, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000296, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000302, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000303, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000305, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000306, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000308, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000310, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000311, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000321, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000323, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000324, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000325, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000327, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000330, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000334, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000335, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000336, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000338, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000339, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000343, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000344, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000345, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000346, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000348, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000352, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000353, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000355, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000356, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000357, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000359, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000360, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000362, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000363, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000364, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000365, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000367, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000370, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000371, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000372, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000373, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000374, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000375, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000376, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000377, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000378, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000431, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000453, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000454, {'SimLayout':'ctbh8_calodata', 'Eta':1.2,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000469, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000470, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000471, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000487, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000488, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000489, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000490, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000491, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000492, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000493, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000494, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000495, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000498, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000499, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000500, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000501, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000502, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000503, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000504, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000505, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000506, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000507, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000508, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000509, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000510, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000511, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000512, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000513, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000514, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000515, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000516, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000518, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000519, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000520, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000521, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000523, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000525, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000526, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000527, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000528, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000529, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000530, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000531, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000533, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000541, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000542, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000545, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000546, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000547, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000548, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000550, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000551, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000552, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000554, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000555, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000556, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000557, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000559, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000560, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000561, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000563, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000564, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000565, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000567, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000568, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000570, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000571, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000573, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000574, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000576, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000579, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000580, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000581, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000584, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000585, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000586, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000587, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000590, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000591, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000592, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000593, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000594, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000663, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000664, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000666, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000668, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000672, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000674, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000715, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000721, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000723, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000726, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000728, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000729, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000730, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000732, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000734, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000736, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000739, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000741, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000755, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000757, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000758, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000759, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000760, {'SimLayout':'ctbh8_calodata', 'Eta':1.0,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000764, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000765, {'SimLayout':'ctbh8_calodata', 'Eta':1.1,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000777, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000778, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000779, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000780, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000783, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000784, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000785, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000786, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000787, {'SimLayout':'ctbh8_calodata', 'Eta':0.8,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000793, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000794, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000795, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000796, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000797, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})
RunCond.set_Run(1000799, {'SimLayout':'ctbh8_calodata', 'Eta':0.7,  'MagnetMBPSIDBz':0.0, 'GeoModelTileVersion':'TileTB-3B3EB-00'})

# Calorimeter runs with 3 TileCal barrel modules in July-August 2004 (no ext.barrel!)

RunCond.set_Run(1002023, {'SimLayout':'ctbh8_calodata', 'Eta':0.45, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002025, {'SimLayout':'ctbh8_calodata', 'Eta':0.45, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002027, {'SimLayout':'ctbh8_calodata', 'Eta':0.45, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002030, {'SimLayout':'ctbh8_calodata', 'Eta':0.45, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002041, {'SimLayout':'ctbh8_calodata', 'Eta':0.45, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002042, {'SimLayout':'ctbh8_calodata', 'Eta':0.45, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002043, {'SimLayout':'ctbh8_calodata', 'Eta':0.45, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002045, {'SimLayout':'ctbh8_calodata', 'Eta':0.45, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002051, {'SimLayout':'ctbh8_calodata', 'Eta':0.433, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002052, {'SimLayout':'ctbh8_calodata', 'Eta':0.45, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002053, {'SimLayout':'ctbh8_calodata', 'Eta':0.45, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002054, {'SimLayout':'ctbh8_calodata', 'Eta':0.45, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002056, {'SimLayout':'ctbh8_calodata', 'Eta':0.45, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002071, {'SimLayout':'ctbh8_calodata', 'Eta':0.45, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002072, {'SimLayout':'ctbh8_calodata', 'Eta':0.45, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002073, {'SimLayout':'ctbh8_calodata', 'Eta':0.45, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002074, {'SimLayout':'ctbh8_calodata', 'Eta':0.45, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002079, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002081, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002083, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002096, {'SimLayout':'ctbh8_calodata', 'Eta':0.183, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002098, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002099, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002103, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002104, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002105, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002106, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002132, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002135, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002139, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002141, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002143, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002144, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002146, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002149, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002150, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002161, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002170, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002172, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002173, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002174, {'SimLayout':'ctbh8_calodata', 'Eta':0.25, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002175, {'SimLayout':'ctbh8_calodata', 'Eta':0.25, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002191, {'SimLayout':'ctbh8_calodata', 'Eta':0.25, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002192, {'SimLayout':'ctbh8_calodata', 'Eta':0.25, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002193, {'SimLayout':'ctbh8_calodata', 'Eta':0.25, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002194, {'SimLayout':'ctbh8_calodata', 'Eta':0.25, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002198, {'SimLayout':'ctbh8_calodata', 'Eta':0.25, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002199, {'SimLayout':'ctbh8_calodata', 'Eta':0.25, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002213, {'SimLayout':'ctbh8_calodata', 'Eta':0.238, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002214, {'SimLayout':'ctbh8_calodata', 'Eta':0.25, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002216, {'SimLayout':'ctbh8_calodata', 'Eta':0.25, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002217, {'SimLayout':'ctbh8_calodata', 'Eta':0.25, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002221, {'SimLayout':'ctbh8_calodata', 'Eta':0.25, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1002223, {'SimLayout':'ctbh8_calodata', 'Eta':0.25, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003002, {'SimLayout':'ctbh8_calodata', 'Eta':0.25, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003003, {'SimLayout':'ctbh8_calodata', 'Eta':0.25, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003005, {'SimLayout':'ctbh8_calodata', 'Eta':0.25, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003007, {'SimLayout':'ctbh8_calodata', 'Eta':0.25, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003008, {'SimLayout':'ctbh8_calodata', 'Eta':0.25, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003009, {'SimLayout':'ctbh8_calodata', 'Eta':0.25, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003010, {'SimLayout':'ctbh8_calodata', 'Eta':0.25, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003012, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003014, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003021, {'SimLayout':'ctbh8_calodata', 'Eta':0.335, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003022, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003023, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003037, {'SimLayout':'ctbh8_calodata', 'Eta':0.337, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003038, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003039, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003040, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003041, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003042, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003043, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003066, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003067, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003068, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003069, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003071, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003072, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003073, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003074, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003075, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003077, {'SimLayout':'ctbh8_calodata', 'Eta':0.339, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003079, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003086, {'SimLayout':'ctbh8_calodata', 'Eta':0.55, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003087, {'SimLayout':'ctbh8_calodata', 'Eta':0.55, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003088, {'SimLayout':'ctbh8_calodata', 'Eta':0.55, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003092, {'SimLayout':'ctbh8_calodata', 'Eta':0.55, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003095, {'SimLayout':'ctbh8_calodata', 'Eta':0.55, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003096, {'SimLayout':'ctbh8_calodata', 'Eta':0.55, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003097, {'SimLayout':'ctbh8_calodata', 'Eta':0.55, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003098, {'SimLayout':'ctbh8_calodata', 'Eta':0.55, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003104, {'SimLayout':'ctbh8_calodata', 'Eta':0.55, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003105, {'SimLayout':'ctbh8_calodata', 'Eta':0.55, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003106, {'SimLayout':'ctbh8_calodata', 'Eta':0.55, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003126, {'SimLayout':'ctbh8_calodata', 'Eta':0.55, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003127, {'SimLayout':'ctbh8_calodata', 'Eta':0.533, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003128, {'SimLayout':'ctbh8_calodata', 'Eta':0.55, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003129, {'SimLayout':'ctbh8_calodata', 'Eta':0.55, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003130, {'SimLayout':'ctbh8_calodata', 'Eta':0.55, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003131, {'SimLayout':'ctbh8_calodata', 'Eta':0.55, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003136, {'SimLayout':'ctbh8_calodata', 'Eta':0.55, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003150, {'SimLayout':'ctbh8_calodata', 'Eta':0.55, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003151, {'SimLayout':'ctbh8_calodata', 'Eta':0.55, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003152, {'SimLayout':'ctbh8_calodata', 'Eta':0.55, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003153, {'SimLayout':'ctbh8_calodata', 'Eta':0.55, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003154, {'SimLayout':'ctbh8_calodata', 'Eta':0.55, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003155, {'SimLayout':'ctbh8_calodata', 'Eta':0.55, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003159, {'SimLayout':'ctbh8_calodata', 'Eta':0.55, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003175, {'SimLayout':'ctbh8_calodata', 'Eta':0.65, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003177, {'SimLayout':'ctbh8_calodata', 'Eta':0.65, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003178, {'SimLayout':'ctbh8_calodata', 'Eta':0.65, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003179, {'SimLayout':'ctbh8_calodata', 'Eta':0.65, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003180, {'SimLayout':'ctbh8_calodata', 'Eta':0.65, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003182, {'SimLayout':'ctbh8_calodata', 'Eta':0.65, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003194, {'SimLayout':'ctbh8_calodata', 'Eta':0.630, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003195, {'SimLayout':'ctbh8_calodata', 'Eta':0.65, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003196, {'SimLayout':'ctbh8_calodata', 'Eta':0.65, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003197, {'SimLayout':'ctbh8_calodata', 'Eta':0.65, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003198, {'SimLayout':'ctbh8_calodata', 'Eta':0.65, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003199, {'SimLayout':'ctbh8_calodata', 'Eta':0.65, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003200, {'SimLayout':'ctbh8_calodata', 'Eta':0.65, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003212, {'SimLayout':'ctbh8_calodata', 'Eta':0.65, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003214, {'SimLayout':'ctbh8_calodata', 'Eta':0.65, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003215, {'SimLayout':'ctbh8_calodata', 'Eta':0.65, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003216, {'SimLayout':'ctbh8_calodata', 'Eta':0.65, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003217, {'SimLayout':'ctbh8_calodata', 'Eta':0.65, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003220, {'SimLayout':'ctbh8_calodata', 'Eta':0.65, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003222, {'SimLayout':'ctbh8_calodata', 'Eta':0.65, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003241, {'SimLayout':'ctbh8_calodata', 'Eta':0.65, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003242, {'SimLayout':'ctbh8_calodata', 'Eta':0.65, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003243, {'SimLayout':'ctbh8_calodata', 'Eta':0.65, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003244, {'SimLayout':'ctbh8_calodata', 'Eta':0.65, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003245, {'SimLayout':'ctbh8_calodata', 'Eta':0.65, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1003246, {'SimLayout':'ctbh8_calodata', 'Eta':0.65, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1004000, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1004002, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1004004, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1004005, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1004006, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1004008, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1004010, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1004012, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1004013, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1004014, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1004173, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1004174, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1004175, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1004179, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1004218, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1004220, {'SimLayout':'ctbh8_calodata', 'Eta':0.35, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1004222, {'SimLayout':'ctbh8_calodata', 'Eta':0.25, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1004223, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1004224, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1004226, {'SimLayout':'ctbh8_calodata', 'Eta':0.45, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1004228, {'SimLayout':'ctbh8_calodata', 'Eta':0.55, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1004230, {'SimLayout':'ctbh8_calodata', 'Eta':0.65, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1004231, {'SimLayout':'ctbh8_calodata', 'Eta':0.65, 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(1004233, {'SimLayout':'ctbh8_calodata', 'Eta':0.20, 'MagnetMBPSIDBz':0.0})

# Calorimeter runs in September 2004 when InDet was not in readout

RunCond.set_Run(2100011, {'SimLayout':'ctbh8_combined', 'Eta':0.35, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100012, {'SimLayout':'ctbh8_combined', 'Eta':0.35, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100013, {'SimLayout':'ctbh8_combined', 'Eta':0.35, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100028, {'SimLayout':'ctbh8_combined', 'Eta':0.20, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100029, {'SimLayout':'ctbh8_combined', 'Eta':0.20, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100030, {'SimLayout':'ctbh8_combined', 'Eta':0.25, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100031, {'SimLayout':'ctbh8_combined', 'Eta':0.25, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100033, {'SimLayout':'ctbh8_combined', 'Eta':0.347, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100034, {'SimLayout':'ctbh8_combined', 'Eta':0.35, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100036, {'SimLayout':'ctbh8_combined', 'Eta':0.45, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100037, {'SimLayout':'ctbh8_combined', 'Eta':0.45, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100039, {'SimLayout':'ctbh8_combined', 'Eta':0.45, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100041, {'SimLayout':'ctbh8_combined', 'Eta':0.45, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100045, {'SimLayout':'ctbh8_combined', 'Eta':0.55, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100046, {'SimLayout':'ctbh8_combined', 'Eta':0.55, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100047, {'SimLayout':'ctbh8_combined', 'Eta':0.65, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100048, {'SimLayout':'ctbh8_combined', 'Eta':0.65, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100049, {'SimLayout':'ctbh8_combined', 'Eta':0.55, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100050, {'SimLayout':'ctbh8_combined', 'Eta':0.55, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100051, {'SimLayout':'ctbh8_combined', 'Eta':0.45, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100052, {'SimLayout':'ctbh8_combined', 'Eta':0.25, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100053, {'SimLayout':'ctbh8_combined', 'Eta':0.25, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100054, {'SimLayout':'ctbh8_combined', 'Eta':0.25, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100055, {'SimLayout':'ctbh8_combined', 'Eta':0.20, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100056, {'SimLayout':'ctbh8_combined', 'Eta':0.20, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100093, {'SimLayout':'ctbh8_combined', 'Eta':0.192, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100094, {'SimLayout':'ctbh8_combined', 'Eta':0.192, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100100, {'SimLayout':'ctbh8_combined', 'Eta':0.20, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100101, {'SimLayout':'ctbh8_combined', 'Eta':0.20, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100102, {'SimLayout':'ctbh8_combined', 'Eta':0.20, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100109, {'SimLayout':'ctbh8_combined', 'Eta':0.20, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100110, {'SimLayout':'ctbh8_combined', 'Eta':0.20, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100132, {'SimLayout':'ctbh8_combined', 'Eta':0.20, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100133, {'SimLayout':'ctbh8_combined', 'Eta':0.20, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100135, {'SimLayout':'ctbh8_combined', 'Eta':0.20, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100161, {'SimLayout':'ctbh8_combined', 'Eta':0.233, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100164, {'SimLayout':'ctbh8_combined', 'Eta':0.25, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100168, {'SimLayout':'ctbh8_combined', 'Eta':0.25, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100169, {'SimLayout':'ctbh8_combined', 'Eta':0.25, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100170, {'SimLayout':'ctbh8_combined', 'Eta':0.25, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100171, {'SimLayout':'ctbh8_combined', 'Eta':0.25, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100172, {'SimLayout':'ctbh8_combined', 'Eta':0.25, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100173, {'SimLayout':'ctbh8_combined', 'Eta':0.25, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100176, {'SimLayout':'ctbh8_combined', 'Eta':0.25, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100177, {'SimLayout':'ctbh8_combined', 'Eta':0.25, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100207, {'SimLayout':'ctbh8_combined', 'Eta':0.338, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100209, {'SimLayout':'ctbh8_combined', 'Eta':0.35, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100227, {'SimLayout':'ctbh8_combined', 'Eta':0.35, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100228, {'SimLayout':'ctbh8_combined', 'Eta':0.35, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100230, {'SimLayout':'ctbh8_combined', 'Eta':0.35, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100249, {'SimLayout':'ctbh8_combined', 'Eta':0.35, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100251, {'SimLayout':'ctbh8_combined', 'Eta':0.35, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100254, {'SimLayout':'ctbh8_combined', 'Eta':0.45, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100257, {'SimLayout':'ctbh8_combined', 'Eta':0.438, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100266, {'SimLayout':'ctbh8_combined', 'Eta':0.45, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100267, {'SimLayout':'ctbh8_combined', 'Eta':0.436, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100269, {'SimLayout':'ctbh8_combined', 'Eta':0.45, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100273, {'SimLayout':'ctbh8_combined', 'Eta':0.45, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100274, {'SimLayout':'ctbh8_combined', 'Eta':0.45, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100275, {'SimLayout':'ctbh8_combined', 'Eta':0.45, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100291, {'SimLayout':'ctbh8_combined', 'Eta':0.45, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100292, {'SimLayout':'ctbh8_combined', 'Eta':0.45, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100300, {'SimLayout':'ctbh8_combined', 'Eta':0.45, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100317, {'SimLayout':'ctbh8_combined', 'Eta':0.435, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100318, {'SimLayout':'ctbh8_combined', 'Eta':0.45, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100325, {'SimLayout':'ctbh8_combined', 'Eta':0.45, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100326, {'SimLayout':'ctbh8_combined', 'Eta':0.45, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100327, {'SimLayout':'ctbh8_combined', 'Eta':0.45, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100328, {'SimLayout':'ctbh8_combined', 'Eta':0.45, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100346, {'SimLayout':'ctbh8_combined', 'Eta':0.35, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100347, {'SimLayout':'ctbh8_combined', 'Eta':0.326, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100348, {'SimLayout':'ctbh8_combined', 'Eta':0.35, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100349, {'SimLayout':'ctbh8_combined', 'Eta':0.35, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100351, {'SimLayout':'ctbh8_combined', 'Eta':0.55, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100352, {'SimLayout':'ctbh8_combined', 'Eta':0.55, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100359, {'SimLayout':'ctbh8_combined', 'Eta':0.55, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100362, {'SimLayout':'ctbh8_combined', 'Eta':0.55, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100363, {'SimLayout':'ctbh8_combined', 'Eta':0.65, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100364, {'SimLayout':'ctbh8_combined', 'Eta':0.65, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100369, {'SimLayout':'ctbh8_combined', 'Eta':0.65, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100372, {'SimLayout':'ctbh8_combined', 'Eta':0.65, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100373, {'SimLayout':'ctbh8_combined', 'Eta':0.65, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100380, {'SimLayout':'ctbh8_combined', 'Eta':0.65, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100382, {'SimLayout':'ctbh8_combined', 'Eta':0.65, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100383, {'SimLayout':'ctbh8_combined', 'Eta':0.65, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100386, {'SimLayout':'ctbh8_combined', 'Eta':0.65, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100388, {'SimLayout':'ctbh8_combined', 'Eta':0.65, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100389, {'SimLayout':'ctbh8_combined', 'Eta':0.65, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100421, {'SimLayout':'ctbh8_combined', 'Eta':0.5276, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100422, {'SimLayout':'ctbh8_combined', 'Eta':0.55, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100424, {'SimLayout':'ctbh8_combined', 'Eta':0.55, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100425, {'SimLayout':'ctbh8_combined', 'Eta':0.55, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100426, {'SimLayout':'ctbh8_combined', 'Eta':0.55, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100427, {'SimLayout':'ctbh8_combined', 'Eta':0.55, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100432, {'SimLayout':'ctbh8_combined', 'Eta':0.45, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100433, {'SimLayout':'ctbh8_combined', 'Eta':0.45, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100435, {'SimLayout':'ctbh8_combined', 'Eta':0.45, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100436, {'SimLayout':'ctbh8_combined', 'Eta':0.45, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100443, {'SimLayout':'ctbh8_combined', 'Eta':0.431, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100444, {'SimLayout':'ctbh8_combined', 'Eta':0.45, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100445, {'SimLayout':'ctbh8_combined', 'Eta':0.328, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100446, {'SimLayout':'ctbh8_combined', 'Eta':0.35, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100449, {'SimLayout':'ctbh8_combined', 'Eta':0.35, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100450, {'SimLayout':'ctbh8_combined', 'Eta':0.35, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100451, {'SimLayout':'ctbh8_combined', 'Eta':0.35, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100452, {'SimLayout':'ctbh8_combined', 'Eta':0.35, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100453, {'SimLayout':'ctbh8_combined', 'Eta':0.25, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100454, {'SimLayout':'ctbh8_combined', 'Eta':0.25, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100455, {'SimLayout':'ctbh8_combined', 'Eta':0.25, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100456, {'SimLayout':'ctbh8_combined', 'Eta':0.25, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100460, {'SimLayout':'ctbh8_combined', 'Eta':0.25, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100461, {'SimLayout':'ctbh8_combined', 'Eta':0.232, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100463, {'SimLayout':'ctbh8_combined', 'Eta':0.20, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100466, {'SimLayout':'ctbh8_combined', 'Eta':0.20, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100467, {'SimLayout':'ctbh8_combined', 'Eta':0.20, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100468, {'SimLayout':'ctbh8_combined', 'Eta':0.20, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100469, {'SimLayout':'ctbh8_combined', 'Eta':0.20, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100476, {'SimLayout':'ctbh8_combined', 'Eta':0.20, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100477, {'SimLayout':'ctbh8_combined', 'Eta':0.20, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100479, {'SimLayout':'ctbh8_combined', 'Eta':0.25, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100480, {'SimLayout':'ctbh8_combined', 'Eta':0.25, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100481, {'SimLayout':'ctbh8_combined', 'Eta':0.25, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100482, {'SimLayout':'ctbh8_combined', 'Eta':0.336, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100492, {'SimLayout':'ctbh8_combined', 'Eta':0.35, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100494, {'SimLayout':'ctbh8_combined', 'Eta':0.45, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100498, {'SimLayout':'ctbh8_combined', 'Eta':0.45, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100501, {'SimLayout':'ctbh8_combined', 'Eta':0.55, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100504, {'SimLayout':'ctbh8_combined', 'Eta':0.55, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100510, {'SimLayout':'ctbh8_combined', 'Eta':0.55, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100513, {'SimLayout':'ctbh8_combined', 'Eta':0.65, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100514, {'SimLayout':'ctbh8_combined', 'Eta':0.65, 'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100813, {'SimLayout':'ctbh8_combined', 'Eta':1.2,  'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100818, {'SimLayout':'ctbh8_combined', 'Eta':1.2,  'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100822, {'SimLayout':'ctbh8_combined', 'Eta':1.2,  'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100823, {'SimLayout':'ctbh8_combined', 'Eta':1.2,  'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100824, {'SimLayout':'ctbh8_combined', 'Eta':1.2,  'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100825, {'SimLayout':'ctbh8_combined', 'Eta':1.1,  'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100826, {'SimLayout':'ctbh8_combined', 'Eta':1.1,  'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100827, {'SimLayout':'ctbh8_combined', 'Eta':1.1,  'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100829, {'SimLayout':'ctbh8_combined', 'Eta':1.1,  'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100830, {'SimLayout':'ctbh8_combined', 'Eta':1.1,  'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100831, {'SimLayout':'ctbh8_combined', 'Eta':1.1,  'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100839, {'SimLayout':'ctbh8_combined', 'Eta':1.0,  'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100841, {'SimLayout':'ctbh8_combined', 'Eta':1.1,  'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})
RunCond.set_Run(2100844, {'SimLayout':'ctbh8_combined', 'Eta':1.2,  'IdetOracleTag':'InnerDetector-CTB-01-01', 'MagnetMBPSIDBz':0.0})

#=======================================================================
BeamCond.set_Run(1002051 ,{'ParticleGeneratorOrders':{'vertX:':' constant -27500.0','vertY:' :' norm 6. 9. ','vertZ:' : ' norm 0. 18.','t:' :' constant -27500.0','momX:' : ' fixed 1','momY:' : ' fixed 0','momZ:' : ' fixed 0'}})
BeamCond.set_Run(1002213 ,{'ParticleGeneratorOrders':{'vertX:':' constant -27500.0','vertY:' :' norm 7. 9. ','vertZ:' : ' norm 0. 12.5','t:' :' constant -27500.0','momX:' : ' fixed 1','momY:' : ' fixed 0','momZ:' : ' fixed 0'}})
BeamCond.set_Run(1003021 ,{'ParticleGeneratorOrders':{'vertX:':' constant -27500.0','vertY:' :' flat -12. 28. ','vertZ:' : ' flat -25. 25.','t:' :' constant -27500.0','momX:' : ' fixed 1','momY:' : ' fixed 0','momZ:' : ' fixed 0'}})
BeamCond.set_Run(1003037 ,{'ParticleGeneratorOrders':{'vertX:':' constant -27500.0','vertY:' :' norm 8. 12. ','vertZ:' : ' flat -30. 30.','t:' :' constant -27500.0','momX:' : ' fixed 1','momY:' : ' fixed 0','momZ:' : ' fixed 0'}})
BeamCond.set_Run(1003077 ,{'ParticleGeneratorOrders':{'vertX:':' constant -27500.0','vertY:' :' norm 15. 6. ','vertZ:' : ' norm 0. 12.','t:' :' constant -27500.0','momX:' : ' fixed 1','momY:' : ' fixed 0','momZ:' : ' fixed 0'}})
BeamCond.set_Run(1003127 ,{'ParticleGeneratorOrders':{'vertX:':' constant -27500.0','vertY:' :' norm 5. 11. ','vertZ:' : ' norm 0. 20.','t:' :' constant -27500.0','momX:' : ' fixed 1','momY:' : ' fixed 0','momZ:' : ' fixed 0'}})
BeamCond.set_Run(1003194 ,{'ParticleGeneratorOrders':{'vertX:':' constant -27500.0','vertY:' :' norm 6. 8.7 ','vertZ:' : ' norm 0. 11.','t:' :' constant -27500.0','momX:' : ' fixed 1','momY:' : ' fixed 0','momZ:' : ' fixed 0'}})
BeamCond.set_Run(2100093 ,{'ParticleGeneratorOrders':{'vertX:':' constant -27500.0','vertY:' :' flat 0. 25. ','vertZ:' : ' norm 0. 7.','t:' :' constant -27500.0','momX:' : ' fixed 1','momY:' : ' fixed 0','momZ:' : ' fixed 0'}})
BeamCond.set_Run(2100161 ,{'ParticleGeneratorOrders':{'vertX:':' constant -27500.0','vertY:' :' flat -5. 20. ','vertZ:' : ' flat -15. 15.','t:' :' constant -27500.0','momX:' : ' fixed 1','momY:' : ' fixed 0','momZ:' : ' fixed 0'}})
BeamCond.set_Run(2100207 ,{'ParticleGeneratorOrders':{'vertX:':' constant -27500.0','vertY:' :' flat -7. 18. ','vertZ:' : ' flat -15. 15.','t:' :' constant -27500.0','momX:' : ' fixed 1','momY:' : ' fixed 0','momZ:' : ' fixed 0'}})
BeamCond.set_Run(2100347 ,{'ParticleGeneratorOrders':{'vertX:':' constant -27500.0','vertY:' :' norm 7.5 11. ','vertZ:' : ' norm 0. 15.5','t:' :' constant -27500.0','momX:' : ' fixed 1','momY:' : ' fixed 0','momZ:' : ' fixed 0'}})
BeamCond.set_Run(2100421 ,{'ParticleGeneratorOrders':{'vertX:':' constant -27500.0','vertY:' :' flat -8. 17. ','vertZ:' : ' norm 0. 17.','t:' :' constant -27500.0','momX:' : ' fixed 1','momY:' : ' fixed 0','momZ:' : ' fixed 0'}})
BeamCond.set_Run(2100482 ,{'ParticleGeneratorOrders':{'vertX:':' constant -27500.0','vertY:' :' norm 13. 12. ','vertZ:' : ' flat -60. 60.','t:' :' constant -27500.0','momX:' : ' fixed 1','momY:' : ' fixed 0','momZ:' : ' fixed 0'}})

