#!/bin/env python

# Copyright (C) 2002-2019 CERN for the benefit of the ATLAS collaboration
#
# TileCalibBlobPython_Ofl_Onl.py
# Lukas Pribyl
# This Python script needs sqlite file with default (empty) ONL01 and OFL02 folders
# Script of this kind is generated by DQ Leader Web tool
# What it does: offline problems are set, online problems are set and finally the ONL01 folder is synchronized with OFL02

from TileCalibBlobPython import TileCalibTools
from TileCalibBlobPython import TileBchTools
from TileCalibBlobPython.TileCalibTools import MINRUN, MINLBK, MAXRUN, MAXLBK
from TileCalibBlobObjs.Classes import *
import os

from TileCalibBlobPython.TileCalibLogger import TileCalibLogger, getLogger
log = getLogger("writeBch")
import logging
log.setLevel(logging.DEBUG)

#===================================================================
#====================== FILL DB BELOW ==============================
#===================================================================
db = TileCalibTools.openDb('SQLITE', 'CONDBR2', 'UPDATE')

#=== ADC status folder
folder = TileCalibTools.getTilePrefix(True,True) + "STATUS/ADC"

#=== specify folder and tag
folderTag = TileCalibUtils.getFullTag(folder, "RUN2-HLT-UPD1-00")

#=== create bad channel manager
mgr = TileBchTools.TileBchMgr()
mgr.setLogLvl(logging.DEBUG)

#=== always initialize with no bad channels
log.info("Initializing with no bad channels at tag=%s and time=%s" % (folderTag, (0, 0)))
mgr.initialize(db, folder, folderTag, (0, 0))

#=== Tuples of empty channels
emptyChannelLongBarrel =     (30, 31, 43)
emptyChannelExtendedBarrel = (18, 19, 24, 25, 26, 27, 28, 29, 33, 34, 42, 43, 44, 45, 46, 47)
#=== Add problems with mgr.addAdcProblem(ros, drawer, channel, adc, problem)

# LBA
# attention! mgr.addAdcProblem() is just an example
mgr.addAdcProblem(1, 0, 23, 0, TileBchPrbs.BadCis)
mgr.addAdcProblem(1, 1, 20, 0, TileBchPrbs.GeneralMaskAdc)

# LBC

# EBA

# EBC

#=== print bad channels
log.info("bad channels after update")
mgr.listBadAdcs()

#=== commit changes
mgr.commitToDb(db, folder, folderTag, TileBchDecoder.BitPat_ofl01, "lpribyl", "test offline", (151950, 0))

#=== ONLINE FOLDER WITH SYNCHRONIZATION
#=== ADC status folder
folder = TileCalibTools.getTilePrefix(False) + "STATUS/ADC"

#=== specify folder and tag
folderTag = ""

#=== create bad channel manager
mgrOnl = TileBchTools.TileBchMgr()
mgrOnl.setLogLvl(logging.DEBUG)

#=== always initialize with no bad channels
log.info("Initializing with no bad channels at tag=%s and time=%s" % (folderTag, (0, 0)))
mgrOnl.initialize(db, folder, folderTag, (0, 0))

#=== Add problems with mgrOnl.addAdcProblem(ros, drawer, channel, adc, problem)

# LBA
# attention! mgrOnl.addAdcProblem() is just an example
mgrOnl.addAdcProblem(1, 1, 33, 1, TileBchPrbs.IgnoredInDsp)
mgrOnl.addAdcProblem(1, 1, 33, 0, TileBchPrbs.IgnoredInDsp)

# LBC

# EBA

# EBC

#--- synchronization with offline folder
for ros in xrange(1, 5):
    for mod in xrange(0, 64):
        for chn in xrange(0, 48):
            statlo = mgr.getAdcStatus(ros, mod, chn, 0)
            stathi = mgr.getAdcStatus(ros, mod, chn, 1)

            #--- add IgnoreInHlt if either of the ADCs has isBad
            #--- add OnlineGeneralMaskAdc if the ADCs has isBad
            if statlo.isBad() and stathi.isBad():
                mgrOnl.addAdcProblem(ros, mod, chn, 0, TileBchPrbs.IgnoredInHlt)
                mgrOnl.addAdcProblem(ros, mod, chn, 0, TileBchPrbs.OnlineGeneralMaskAdc)
                mgrOnl.addAdcProblem(ros, mod, chn, 1, TileBchPrbs.IgnoredInHlt)
                mgrOnl.addAdcProblem(ros, mod, chn, 1, TileBchPrbs.OnlineGeneralMaskAdc)
            elif statlo.isBad():
                mgrOnl.addAdcProblem(ros, mod, chn, 0, TileBchPrbs.IgnoredInHlt)
                mgrOnl.addAdcProblem(ros, mod, chn, 0, TileBchPrbs.OnlineGeneralMaskAdc)
                mgrOnl.addAdcProblem(ros, mod, chn, 1, TileBchPrbs.IgnoredInHlt)
                mgrOnl.delAdcProblem(ros, mod, chn, 1, TileBchPrbs.OnlineGeneralMaskAdc)
            elif stathi.isBad():
                mgrOnl.addAdcProblem(ros, mod, chn, 0, TileBchPrbs.IgnoredInHlt)
                mgrOnl.delAdcProblem(ros, mod, chn, 0, TileBchPrbs.OnlineGeneralMaskAdc)
                mgrOnl.addAdcProblem(ros, mod, chn, 1, TileBchPrbs.IgnoredInHlt)
                mgrOnl.addAdcProblem(ros, mod, chn, 1, TileBchPrbs.OnlineGeneralMaskAdc)
            else:
                #--- delete IgnoreInHlt and OnlineGeneralMaskAdc if both ADCs are not Bad
                mgrOnl.delAdcProblem(ros, mod, chn, 0, TileBchPrbs.IgnoredInHlt)
                mgrOnl.delAdcProblem(ros, mod, chn, 0, TileBchPrbs.OnlineGeneralMaskAdc)
                mgrOnl.delAdcProblem(ros, mod, chn, 1, TileBchPrbs.IgnoredInHlt)
                mgrOnl.delAdcProblem(ros, mod, chn, 1, TileBchPrbs.OnlineGeneralMaskAdc)

            #--- add OnlineBadTiming if either of the ADCs has isBadTiming
            if statlo.isBadTiming() or stathi.isBadTiming():
                mgrOnl.addAdcProblem(ros, mod, chn, 0, TileBchPrbs.OnlineBadTiming)
                mgrOnl.addAdcProblem(ros, mod, chn, 1, TileBchPrbs.OnlineBadTiming)
            else:
                #--- delete OnlineBadTiming if the both ADCs has not isBadTiming
                mgrOnl.delAdcProblem(ros, mod, chn, 0, TileBchPrbs.OnlineBadTiming)
                mgrOnl.delAdcProblem(ros, mod, chn, 1, TileBchPrbs.OnlineBadTiming)

            #--- add OnlineTimingDmuBcOffset if either of the ADCs has isTimingDmuBcOffset
            if statlo.isTimingDmuBcOffset() or stathi.isTimingDmuBcOffset():
                mgrOnl.addAdcProblem(ros, mod, chn, 0, TileBchPrbs.OnlineTimingDmuBcOffset)
                mgrOnl.addAdcProblem(ros, mod, chn, 1, TileBchPrbs.OnlineTimingDmuBcOffset)
            else:
                #--- delete OnlineTimingDmuBcOffset if the both ADCs has not isTimingDmuBcOffset
                mgrOnl.delAdcProblem(ros, mod, chn, 0, TileBchPrbs.OnlineTimingDmuBcOffset)
                mgrOnl.delAdcProblem(ros, mod, chn, 1, TileBchPrbs.OnlineTimingDmuBcOffset)

log.info("============================")
log.info("ONL01 and OFL02 synchronized")
log.info("============================")


#=== print bad channels
log.info("bad channels after update")
mgrOnl.listBadAdcs()

#=== commit changes
mgrOnl.commitToDb(db, folder, folderTag, TileBchDecoder.BitPat_onl01, "lpribyl", "test online", (151950,0))

#=== close DB
db.closeDatabase()
