# Copyright (C) 2002-2017 CERN for the benefit of the ATLAS collaboration

"""
This module bundles the RTT factory functions.
These functions are written as classes to help write test code.
"""

from Factory_BaseClass    import Factory
from ScriptWriter         import *
from RTTSException        import RTTCodingError

# -------------------------------------------------------------------------
# ScriptWriter Factory
# -------------------------------------------------------------------------

class ScriptWriterFactory(Factory):
  def __init__(self, argsForScripts, logger):
    Factory.__init__(self, logger, self.__class__.__name__)
    self.argsForScripts = argsForScripts
    
    self.dict = {
      'ContainerGroup'                                                                           : NullScriptWriter,
      'AthenaJunkJob'                                                                            : JunkScriptWriter,
      'Athena'                                                                                   : AthenaScriptWriter,
      'AthenaAtlfastWatcher'                                                                     : NullScriptWriter,
      'AthenaegammaRecWatcher'                                                                   : NullScriptWriter,
      'Monitoring'                                                                               : MonitoringScriptWriter,
      'AthenaATN'                                                                                : AthenaScriptWriter,
      'AthenaDCubeClient'                                                                        : AthenaScriptWriter,
      'AthenaCoolDozer'                                                                          : AthenaScriptWriter,
      'AthenaHelloWorld'                                                                         : AthenaScriptWriter,
      'AthenaAtlfast'                                                                            : AthenaScriptWriter,
      'AthenaAtlfastPredicateUnitTest'                                                           : AthenaScriptWriter,
      'AthenaFastShower'                                                                         : AthenaScriptWriter,
      'AthenaGenerateFastSim'                                                                    : AthenaScriptWriter,
      'AthenaCscClusterValidation_misal1_csc11.005145.PythiaZmumu.digit.RDO.v12000601_tid006791' : RecExCommonScriptWriter,
      'AthenaCscSegmentValidation_misal1_csc11.005145.PythiaZmumu.digit.RDO.v12000601_tid006791' : RecExCommonScriptWriter,
      'AthenaJiveXML'                                                                            : JiveXMLScriptWriter,
      'AthenaTriggerRelease'                                                                     : TriggerReleaseScriptWriter,
      'AthenaTriggerReleaseBS'                                                                   : TriggerReleaseScriptWriter,
      'AthenaTrigP1TestMonitorHistos'                                                            : TriggerReleaseScriptWriter,
      'AthenaTrigMenuValidation'                                                                 : TriggerReleaseScriptWriter,
      'AthenaTriggerTestBS'                                                                      : TriggerReleaseScriptWriter,
      'AthenaTriggerTestReco'                                                                    : TriggerReleaseScriptWriter,
      'AthenaTriggerTestMonitorHistos'                                                           : TriggerReleaseScriptWriter,
      'AthenaTrigAnalysisTestIOTests'                                                            : TriggerReleaseScriptWriter,
      'AthenaTriggerReleaseReco'                                                                 : TriggerReleaseScriptWriter,
      'AthenaL2Time1E31MC'                                                                       : HLTScriptWriter,
      'AthenaEFTime1E31MC'                                                                       : HLTScriptWriter,
      'AthenaTriggerReleaseID'                                                                   : TrigReleaseIDScriptWriter,
      'AthenaTriggerReleaseCalo'                                                                 : TrigReleaseCaloScriptWriter,
      'AthenaBackTrack_K0S_Et10'                                                                 : RecExCommonScriptWriter,
      'AthenaBackTrack_ttbar'                                                                    : RecExCommonScriptWriter,
      'AthenaBackTrack_Gamma_Conv_Et20'                                                          : RecExCommonScriptWriter,
      'AthenaRecExCommon'                                                                        : RecExCommonScriptWriter,
      'AthenaTestingRecExCommission'                                                             : RecExCommissionScriptWriter,
      'AthenaTestingRecExCommissionSim'                                                          : RecExCommissionSimScriptWriter,
      'AthenaPrimaryVertexingLowLumi'                                                            : RecExCommonScriptWriter,
      'AthenaConversionVertexingLowLumi'                                                         : RecExCommonScriptWriter,
      'AthenaConversionVertexing'                                                                : RecExCommonScriptWriter,
      'AthenaAtlfastAlgs'                                                                        : RecExCommonScriptWriter,
      'AthenaPileupDigiTest'                                                                     : RecExCommonScriptWriter,
      'AthenaAMAValidationRTT-ttbar-mc08-geo-02-01-00'                                           : RecExCommonScriptWriter,
      'AthenaPileupOverlay'                                                                      : RecExCommonScriptWriter,
      'AthenaTrigBjetValidation'                                                                 : RecExCommonScriptWriter,
      'AthenaHIValidation'                                                                       : RecExCommonScriptWriter,
      'AthenaegammaRec'                                                                          : RecExCommonScriptWriter,
      'AthenaegammaPerformance'                                                                  : RecExCommonScriptWriter,
      'AthenaJetRec_J5'                                                                          : RecExCommonScriptWriter,
      'AthenaJetRec_Pythia'                                                                      : RecExCommonScriptWriter,
      'AthenaJetRec_PythiaFixedHardScatter'                                                      : RecExCommonScriptWriter,
      'AthenaJetRec_ESD_to_CBNT'                                                                 : RecExCommonScriptWriter,
      'AthenaJetRec_SinglePion'                                                                  : RecExCommonScriptWriter,
      'AthenaSUSYValidation'                                                                     : RecExCommonScriptWriter,
      'AthenaSCTDigiRTT'                                                                         : RecExCommonScriptWriter,
      'AthenaH4lAnalysis'                                                                        : RecExCommonScriptWriter,
      'AthenaIDAlignMonRTT'                                                                      : RecExCommonScriptWriter,
      'AthenaIDPerfMonRTT'                                                                       : RecExCommonScriptWriter,
      'AthenaIDAlignMon_Zmumu_perfect'                                                           : RecExCommonScriptWriter,
      'AthenaIDAlignMon_Zmumu_aligned'                                                           : RecExCommonScriptWriter,
      'AthenaIDAlignMon_Zmumu_misaligned'                                                        : RecExCommonScriptWriter,
      'AthenaInDetRecValidation'                                                                 : RecExCommonScriptWriter,
      'AthenaInDetRecValidation_Rome-Initial'                                                    : RecExCommonScriptWriter,
      'AthenaInDetRecValidation_DC3'                                                             : RecExCommonScriptWriter,
      'AthenaPixelTrackValidation'                                                               : RecExCommonScriptWriter,
      'AthenaPixelDigitizationValidation'                                                        : RecExCommonScriptWriter,
      'AthenaPixelTrackValidation_ttbar'                                                         : RecExCommonScriptWriter,
      'AthenaPixelTrackValidation_mu100'                                                         : RecExCommonScriptWriter,
      'AthenaPixelTrackValidation_mu3'                                                           : RecExCommonScriptWriter,
      'AthenaMuonDigiExample'                                                                    : RecExCommonScriptWriter,
      'AthenaMuGirlPerformance'                                                                  : RecExCommonScriptWriter,
      'AthenaMuGirlPerformance-SingleMuonsPt10'                                                  : RecExCommonScriptWriter,
      'AthenaMuGirlPerformance-SingleMuonsPt100'                                                 : RecExCommonScriptWriter,
      'AthenaMuGirlPerformance-Zmumu'                                                            : RecExCommonScriptWriter,
      'AthenaMuGirlPerformance-T1'                                                               : RecExCommonScriptWriter,
      'AthenaMuGirlPerformance-directJpsi'                                                       : RecExCommonScriptWriter,   
      'AthenaMuonEvtValidator'                                                                   : RecExCommonScriptWriter,
      'AthenaMuonGeomRTT-HitRelocation'                                                          : RecExCommonScriptWriter,
      'AthenaMuonGeomRTT-MuonGMCheck'                                                            : RecExCommonScriptWriter,
      'AthenaMuonGeomRTT-AmdcMGM'                                                                : RecExCommonScriptWriter,
      'AthenaTrigEgammaValidation'                                                               : RecExCommonScriptWriter,
      'AthenaTrigEgammaNewRingerValidation'                                                      : RecExCommonScriptWriter,
      'AthenaTrigEgammaNewValidation'                                                            : RecExCommonScriptWriter,
      'AthenaTrigEgammaNewNavValidation'                                                         : RecExCommonScriptWriter,
      'AthenaTrigEgammaNewValidationMTPT'                                                        : RecExCommonScriptWriter,
      'AthenaInDetTrigRecExample'                                                                : RecExCommonScriptWriter,
      'AthenaMuonHitTest'                                                                        : RecExCommonScriptWriter,
      'AthenaMuonDigitTest'                                                                      : RecExCommonScriptWriter,
      'AthenatauRec'                                                                             : RecExCommonScriptWriter,
      'AthenaTauValidation'                                                                      : RecExCommonScriptWriter,
      'AthenaTrigTauPerformanceTest'                                                             : RecExCommonScriptWriter,
      'AthenaMuonRecValidator'                                                                   : RecExCommonScriptWriter,
      'AthenaMuonRecValidator-Zmumu-v13002003'                                                   : RecExCommonScriptWriter,
      'AthenaMuonInSituPerformance'                                                              : RecExCommonScriptWriter,
      'AthenaMuonHolesValidationRTT-Mboy-singleMu500gev-csc-01-00-00'                            : RecExCommonScriptWriter,
      'AthenaMuonHolesValidationRTT-Mboy-singleMu100gev-csc-01-00-00'                            : RecExCommonScriptWriter,
      'AthenaMuonHolesValidationRTT-Momu-singleMu500gev-csc-01-00-00'                            : RecExCommonScriptWriter,
      'AthenaMuonHolesValidationRTT-Momu-singleMu100gev-csc-01-00-00'                            : RecExCommonScriptWriter,
      'AthenaMuonHolesValidationRTT-Mboy-zmumu-sf05-csc-01-02-00'                                : RecExCommonScriptWriter,
      'AthenaMuonHolesValidationRTT-Momu-zmumu-sf05-csc-01-02-00'                                : RecExCommonScriptWriter,
      'AthenaMuonHolesValidationRTT-Mboy-Jpsi-csc-01-02-00'                                      : RecExCommonScriptWriter,
      'AthenaMuonHolesValidationRTT-Mboy-singleMu100gev-geo-02-00-00'                            : RecExCommonScriptWriter,
      'AthenaMuonHolesValidationRTT-Mboy-ttbar-geo-02-01-00'                                     : RecExCommonScriptWriter,
      'AthenaMuonHolesValidationRTT-Momu-zmumu-sf05-csc-01-02-00'                                : RecExCommonScriptWriter,
      'AthenaMuonHolesValidationRTT-Moore-Jpsi-csc-01-02-00'                                     : RecExCommonScriptWriter,
      'AthenaMuonHolesValidationRTT-Moore-singleMu100gev-geo-02-00-00'                           : RecExCommonScriptWriter,
      'AthenaMuonHolesValidationRTT-Moore-ttbar-geo-02-01-00'                                    : RecExCommonScriptWriter,
      'AthenaMooPerformance'                                                                     : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMu-mupt10-geo-02-01-00'                                            : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMu-mupt10-hyb-geo-02-01-00'                                        : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMu-mupt10-TGmat-geo-02-01-00'                                      : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMu-mupt100-geo-02-01-00'                                           : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMu-Zprime-geo-02-01-00'                                            : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMu-Jpsi-geo-02-01-00'                                              : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMu-zmumu-geo-02-01-00'                                             : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMu-ttbar-geo-02-01-00'                                             : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMuOnly-mupt100-geo-02-01-00'                                       : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMu-h4lep-geo-02-01-00'                                             : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMu-cosmics-comm-09-00-00'                                          : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMu-mupt10-hyb-csc-01-02-00'                                        : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMu-mupt10-TGmat-csc-01-02-00'                                      : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMu-mupt100-geo-02-00-00'                                           : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMu-segmreco-zmumu-csc-01-02-00-sf05'                               : RecExCommonScriptWriter,
      'AthenaMooPerformance-mupt10-dc3-02'                                                       : RecExCommonScriptWriter,
      'AthenaMooPerformance-mupt100-dc3-02'                                                      : RecExCommonScriptWriter,
      'AthenaMooPerformance-mupt300-dc3-02'                                                      : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMu-ttbar-geo-02-00-00'                                             : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMu-mupt10-geo-02-00-00'                                            : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMu-mupt10-hyb-geo-02-00-00'                                        : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMu-mupt10-TGmat-geo-02-00-00'                                      : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMuOnly-mupt100-geo-02-00-00'                                       : RecExCommonScriptWriter,
      'AthenaMooPerformance-zmumu-dc3-02'                                                        : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMu-ttbar-csc-01-02-00'                                             : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMuOnly-mupt100-csc-01-02-00'                                       : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMuOnly-zmumu-csc-01-02-00-sf05'                                    : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMu-Zprime-csc-01-02-00'                                            : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMu-Jpsi-csc-01-02-00'                                              : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMu-mupt10-csc-01-02-00'                                            : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMu-mupt100-csc-01-02-00'                                           : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMu-mupt500-csc-01-02-00'                                           : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMu-zmumu-csc-01-02-00'                                             : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMu-zmumu-csc-01-02-00-sf05'                                        : RecExCommonScriptWriter,
      'AthenaMooPerformance-MooreOnly-zmumu-csc-01-02-00'                                        : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMuMooreOnly-zmumu-csc-01-02-00'                                    : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMuMoore-mupt10-csc-01-02-00'                                       : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMuMoore-mupt100-csc-01-02-00'                                      : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMuMoore-mupt500-csc-01-02-00'                                      : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMuMoore-zmumu-csc-01-02-00'                                        : RecExCommonScriptWriter,
      'AthenaMooPerformance-MoMuMoore-zmumu-csc-01-02-00-sf05'                                   : RecExCommonScriptWriter,
      'AthenaMooPerformance-zmumu-csc-01-02-00-sf05'                                             : RecExCommonScriptWriter,
      'AthenaMooPerformance-mupt10-csc-01-02-00'                                                 : RecExCommonScriptWriter,
      'AthenaMooPerformance-mupt100-csc-01-02-00'                                                : RecExCommonScriptWriter,
      'AthenaMooPerformance-mupt500-csc-01-02-00'                                                : RecExCommonScriptWriter,
      'AthenaMooPerformance-zmumu-csc-01-02-00'                                                  : RecExCommonScriptWriter,
      'AthenaMooPerformance-mupt100-csc-01-00-00'                                                : RecExCommonScriptWriter,
      'AthenaMooPerformance-zmumu-csc-01-02-00'                                                  : RecExCommonScriptWriter,
      'AthenaMuonRecRTT-zmumu-csc-01-02-00'                                                      : RecExCommonScriptWriter,
      'AthenaMuonRecRTT-T1-csc-01-02-00'                                                         : RecExCommonScriptWriter,
      'AthenaMuonRecRTT'                                                                         : RecExCommonScriptWriter,
      'AthenaMuonDataRec'                                                                        : RecExCommissionScriptWriter,
      'AthenaMuonDataRec_mon'                                                                    : RecExCommissionScriptWriter,
      'AthenaMuonSimRec'                                                                         : RecExCommissionScriptWriter,
      'AthenaMuonSimRec_mon'                                                                     : RecExCommissionScriptWriter,
      'AthenaMuonCommRec'                                                                        : RecExCommissionScriptWriter,
      'AthenaMuonCommRec_mon'                                                                    : RecExCommissionScriptWriter,
      'AthenaMuonCommRecSim'                                                                     : RecExCommonScriptWriter,
      'AthenaMuonCommRecSim_mon'                                                                 : RecExCommonScriptWriter,
      'AthenaCaloRecEx'                                                                          : RecExCommonScriptWriter,
      'AthenaGammaE50'                                                                           : RecExCommonScriptWriter,
      'AthenaGammaE500'                                                                          : RecExCommonScriptWriter,
      'AthenaGammaE5'                                                                            : RecExCommonScriptWriter,
      'AthenaElectronE5'                                                                         : RecExCommonScriptWriter,
      'AthenaElectronSliceDNAKalman'                                                             : RecExCommonScriptWriter,
      'AthenaEle5IDPRTTKalmanDNA'                                                                : RecExCommonScriptWriter,
      'AthenaEle60IDPRTTKalmanDNA'                                                               : RecExCommonScriptWriter,
      'AthenaEle5IDPRTTGaussianSumFilter'                                                        : RecExCommonScriptWriter,
      'AthenaEle60IDPRTTGaussianSumFilter'                                                       : RecExCommonScriptWriter,      
      'AthenaTopJimmy'                                                                           : RecExCommonScriptWriter,
      'AthenaElectronE10'                                                                        : RecExCommonScriptWriter,
      'AthenaCaloTests'                                                                          : RecExCommonScriptWriter,
      'AthenaCaloTestsCTB'                                                                       : RecExCommonScriptWriter,
      'AthenaCaloTestsAANT'                                                                      : RecExCommonScriptWriter,
      'AthenaCaloAnaExTestBeam'                                                                  : RecExCommonScriptWriter,
      'AthenaCaloAnaExESDAOD'                                                                    : RecExCommonScriptWriter,
      'AthenaCaloDigEx'                                                                          : RecExCommonScriptWriter,
      'AthenaCaloSimEx'                                                                          : RecExCommonScriptWriter,
      'AthenaCaloCosEx'                                                                          : RecExCommonScriptWriter,
      'AthenaCaloRec'                                                                            : RecExCommonScriptWriter,
      'AthenaCaloRecExAANT'                                                                      : RecExCommonScriptWriter,
      'AthenaCaloRecExAANTReal'                                                                  : CaloRecExAANTRealScriptWriter,
      'AthenaBPhysValidation'                                                                    : RecExCommonScriptWriter,
      'AthenaBPhysTrigger'                                                                       : RecExCommonScriptWriter,
      'AthenaSimDig'                                                                             : RecExCommonScriptWriter,
      'AthenaBPhysReco_iPatRec'                                                                  : RecExCommonScriptWriter,
      'AthenaBPhysReco_xKalman'                                                                  : RecExCommonScriptWriter,
      'AthenaBPhysReco_newTrk'                                                                   : RecExCommonScriptWriter,
      'AthenaBPhysTrackValidation'                                                               : RecExCommonScriptWriter,
      'AthenaBPhysTrackValidation_DC3'                                                           : RecExCommonScriptWriter,
      'AthenaAnalysisExamples'                                                                   : RecExCommonScriptWriter,
      'AthenaJetTagRTT'                                                                          : RecExCommonScriptWriter,
      'AthenaDigitization'                                                                       : RecExCommonScriptWriter,
      'AthenaG4AtlasApps'                                                                        : AthenaScriptWriter,
      'AthenaG4AtlasAppsRelease'                                                                 : RecExCommonScriptWriter,
      'AthenaGeneratorsRTT'                                                                      : RecExCommonScriptWriter,
      'AthenaHiggsToGamGam'                                                                      : RecExCommonScriptWriter,
      'AthenaQuickGenCheck'                                                                      : RecExCommonScriptWriter,
      'AthenaInDetRTT'                                                                           : RecExCommonScriptWriter,
      'AthenaV0FindingLowLumi'                                                                   : RecExCommonScriptWriter,
      'AthenaPi1IDPRTT'                                                                          : RecExCommonScriptWriter,
      'AthenaPi100IDPRTT'                                                                        : RecExCommonScriptWriter,
      'AthenaEle1IDPRTT'                                                                         : RecExCommonScriptWriter,
      'AthenaEle100IDPRTT'                                                                       : RecExCommonScriptWriter,      
      'AthenaMu1IDPRTT'                                                                          : RecExCommonScriptWriter,
      'AthenaMu1IDPRTTKalman'                                                                    : RecExCommonScriptWriter,
      'AthenaMu100IDPRTT'                                                                        : RecExCommonScriptWriter,
      'AthenaZtomumuIDPRTT'                                                                      : RecExCommonScriptWriter,
      'AthenaZtomumuIDPRTTKalman'                                                                : RecExCommonScriptWriter,
      'AthenaMu100IDPRTTKalman'                                                                  : RecExCommonScriptWriter,
      'AthenaMu1GlobalChiSq'                                                                     : RecExCommonScriptWriter,
      'AthenaMu100GlobalChiSq'                                                                   : RecExCommonScriptWriter,
      'AthenaMu1Kalman'                                                                          : RecExCommonScriptWriter,
      'AthenaMu100Kalman'                                                                        : RecExCommonScriptWriter,
      'AthenaPi1'                                                                                : RecExCommonScriptWriter,
      'AthenaPi100'                                                                              : RecExCommonScriptWriter,
      'AthenaLowPt'                                                                              : RecExCommonScriptWriter,
      'AthenaLowPtKalman'                                                                        : RecExCommonScriptWriter,
      'AthenaInDetSimRTT'                                                                        : RecExCommonScriptWriter,      
      'AthenaPi1IDPRTTKalman'                                                                    : RecExCommonScriptWriter,
      'AthenaPi100IDPRTTKalman'                                                                  : RecExCommonScriptWriter,
      'AthenaEle1IDPRTTKalman'                                                                   : RecExCommonScriptWriter,
      'AthenaEle100IDPRTTKalman'                                                                 : RecExCommonScriptWriter,
      'AthenaTtbarIDPRTT'                                                                        : RecExCommonScriptWriter,
      'AthenaTtbarIDPRTTKalman'                                                                  : RecExCommonScriptWriter,
      'AthenaZtomumuPileupIDPRTT'                                                                : RecExCommonScriptWriter,
      'AthenaZtomumuPileupIDPRTTKalman'                                                          : RecExCommonScriptWriter,      
      'AthenaSingleParticle'                                                                     : RecExCommonScriptWriter,
      'AthenaSingleE'                                                                            : RecExCommonScriptWriter,
      'AthenaSingleMu'                                                                           : RecExCommonScriptWriter,
      'AthenaSingleMuKalman'                                                                     : RecExCommonScriptWriter,
      'AthenaSinglePi'                                                                           : RecExCommonScriptWriter,
      'AthenaExoticsPhysValJet'                                                                  : RecExCommonScriptWriter,
      'AthenaMinBias'                                                                            : RecExCommonScriptWriter,
      'AthenaMinBiasIDPRTTKalman'                                                                : RecExCommonScriptWriter,
      'AthenaMinBiasIDPRTT'                                                                      : RecExCommonScriptWriter,
      'AthenaMinBiasTrig'                                                                        : RecExCommonScriptWriter,
      'AthenaMinbiasValidation'                                                                  : RecExCommonScriptWriter,
      'AthenaZee'                                                                                : RecExCommonScriptWriter,
      'AthenaZmumu'                                                                              : RecExCommonScriptWriter,
      'AthenaZmumuKalman'                                                                        : RecExCommonScriptWriter,
      'AthenaTop'                                                                                : RecExCommonScriptWriter,
      'AthenaTopRTT'                                                                             : RecExCommonScriptWriter,
      'AthenaJ5'                                                                                 : RecExCommonScriptWriter,
      'AthenaJ8'                                                                                 : RecExCommonScriptWriter,
      'AthenaHighPtMu'                                                                           : RecExCommonScriptWriter,
      'AthenaHighPtMuKalman'                                                                     : RecExCommonScriptWriter,
      'AthenaHiggsToGamGam'                                                                      : RecExCommonScriptWriter,
      'AthenaHighPtMuMisaligned'                                                                 : RecExCommonScriptWriter,
      'AthenaZmumuMisaligned'                                                                    : RecExCommonScriptWriter,
      'AthenaSingleMuMisaligned'                                                                 : RecExCommonScriptWriter,
      'AthenaMissingET'                                                                          : RecExCommonScriptWriter,
      'AthenaTRT_RTT'                                                                            : RecExCommonScriptWriter,
      'AthenaTRTstandalone_Gamma_Conv_Et20'                                                      : RecExCommonScriptWriter,
      'AthenaTRT_RTT_BS'                                                                         : RecExCommonScriptWriter,
      'AthenaTRT_RTT_Monitoring'                                                                 : RecExCommonScriptWriter,
      'AthenaTRT_RTT_e_minus'                                                                    : RecExCommonScriptWriter,
      'AthenaTRT_RTT_e_plus'                                                                     : RecExCommonScriptWriter,
      'AthenaTRT_RTT_mu_minus'                                                                   : RecExCommonScriptWriter,
      'AthenaTRT_RTT_mu_plus'                                                                    : RecExCommonScriptWriter,
      'AthenaTRT_RTT_pi_minus'                                                                   : RecExCommonScriptWriter,
      'AthenaTRT_RTT_pi_plus'                                                                    : RecExCommonScriptWriter,
      'AthenaTRT_RTT_e_minus_DIG'                                                                : RecExCommonScriptWriter,
      'AthenaTRT_RTT_e_plus_DIG'                                                                 : RecExCommonScriptWriter,
      'AthenaTRT_RTT_mu_minus_DIG'                                                               : RecExCommonScriptWriter,
      'AthenaTRT_RTT_mu_plus_DIG'                                                                : RecExCommonScriptWriter,
      'AthenaTRT_RTT_pi_minus_DIG'                                                               : RecExCommonScriptWriter,
      'AthenaTRT_RTT_pi_plus_DIG'                                                                : RecExCommonScriptWriter,
      'AthenaTRT_RTT_e_minus_SIM'                                                                : RecExCommonScriptWriter,
      'AthenaTRT_RTT_e_plus_SIM'                                                                 : RecExCommonScriptWriter,
      'AthenaTRT_RTT_mu_minus_SIM'                                                               : RecExCommonScriptWriter,
      'AthenaTRT_RTT_mu_plus_SIM'                                                                : RecExCommonScriptWriter,
      'AthenaTRT_RTT_pi_minus_SIM'                                                               : RecExCommonScriptWriter,
      'AthenaTRT_RTT_pi_plus_SIM'                                                                : RecExCommonScriptWriter,
      'AthenaLArMonTools'                                                                        : RecExCommonScriptWriter,
      'AthenaPhysicsAnalysisTools'                                                               : RecExCommonScriptWriter,
      'AthenaElectronValidation'                                                                 : RecExCommonScriptWriter,
      'AthenaMuonboyAth'                                                                         : RecExCommonScriptWriter,
      'AthenaMboyPerformance'                                                                    : RecExCommonScriptWriter,
      'AthenaMboyPerformance-mupt100-csc-01-00-00'                                               : RecExCommonScriptWriter,
      'AthenaMboyPerformance-mupt100-csc-01-02-00'                                               : RecExCommonScriptWriter,
      'AthenaMboyPerformance-mupt10-csc-01-02-00'                                                : RecExCommonScriptWriter,
      'AthenaMboyPerformance-mupt500-csc-01-02-00'                                               : RecExCommonScriptWriter,
      'AthenaMboyPerformance-zmumu-csc-01-02-00'                                                 : RecExCommonScriptWriter,
      'AthenaMboyPerformance-zmumu-csc-01-02-00-sf05'                                            : RecExCommonScriptWriter,
      'AthenaMboyPerformance-mupt5-geo-02-00-00'                                                 : RecExCommonScriptWriter,
      'AthenaMboyPerformance-mupt10-geo-02-00-00'                                                : RecExCommonScriptWriter,
      'AthenaMboyPerformance-mupt100-geo-02-00-00'                                               : RecExCommonScriptWriter,
      'AthenaMboyPerformance-mupt500-geo-02-00-00'                                               : RecExCommonScriptWriter,
      'AthenaMboyPerformance-mupt100-csc-01-02-00'                                               : RecExCommonScriptWriter,
      'AthenaMboyPerformance-zmumu-csc-01-02-00'                                                 : RecExCommonScriptWriter,
      'AthenaMboyPerformance-zmumu-csc-01-02-00-sf05'                                            : RecExCommonScriptWriter,
      'AthenaTauRec'                                                                             : RecExCommonScriptWriter,
      'AthenaTauDPDMaker'                                                                        : RecExCommonScriptWriter,
      'AthenaTrigInDetValidation'                                                                : RecExCommonScriptWriter,
      'AthenaTestingTileRecEx'                                                                   : TileRecExScriptWriter,
      'AthenaTestingTileSimEx'                                                                   : TileSimExScriptWriter,
      'AthenaAtlasHLT'                                                                           : HLTScriptWriter,
      'AthenaAtlasTrigConf'                                                                      : HLTTrigConfScriptWriter,
      'AthenaAtlasTrigConfPTOnly'                                                                : HLTTrigConfScriptWriter,
      'AthenaAtlasTrigConfMTOnly'                                                                : HLTTrigConfScriptWriter,
      'AthenaAtlasTrigConfMC'                                                                    : HLTTrigConfScriptWriter,
      'JobTransform'                                                                             : JobTransformScriptWriter,
      'SimuJobTransform'                                                                         : JobTransformScriptWriter,
      'PATJobTransform'                                                                          : JobTransformScriptWriter,
      'TCTTransform'                                                                             : JobTransformScriptWriter,
      'FCTTransform'                                                                             : JobTransformScriptWriter,
      'OnlineRecoTests'                                                                          : JobTransformScriptWriter,
      'Digitization_JobTransforms'                                                               : JobTransformScriptWriter,
      'GeneratorJobTransforms'                                                                   : JobTransformScriptWriter,
      'AtlfastJobTransforms'                                                                     : JobTransformScriptWriter,
      'Valgrind_JobTransforms'                                                                   : JobTransformScriptWriter,
      'Overlay_JobTransforms'                                                                    : JobTransformScriptWriter,
      'OverlayAOD_JobTransforms'                                                                 : JobTransformScriptWriter,
      'OverlayAODMC_JobTransforms'                                                               : JobTransformScriptWriter,
      'OverlayReco_JobTransforms'                                                                : JobTransformScriptWriter,
      'OverlayRecoMC_JobTransforms'                                                              : JobTransformScriptWriter,
      'OverlayBS_JobTransforms'                                                                  : JobTransformScriptWriter,
      'OverlayPOOL_JobTransforms'                                                                : JobTransformScriptWriter,
      'RecTrf'                                                                                   : JobTransformScriptWriter,
      'TrfInDetTrigRecExample'                                                                   : JobTransformScriptWriter,
      'InDetSiLevel1Alignment'                                                                   : JobTransformScriptWriter
      }    
    
  def create(self, descArgsForScripts, logger):
    'Returns a  ScriptWriter instance according to jobGroup'
    jobGroup = descArgsForScripts.jobGroup
    if jobGroup not in self.dict.keys():
      msg  = 'Error instantiating Script: unknown job jobGroup: %s ' % jobGroup
      self.logger.error(msg)
      print msg
      raise RTTCodingError(msg)

    argsForScripts = descArgsForScripts
    argsForScripts.__dict__.update(self.argsForScripts.__dict__)

    try:
      script = self.dict[jobGroup](argsForScripts, logger)
    except Exception, e:
      m = 'Error constricting a runscript:\nException:\n%s\nTraceback:\n%s' % (str(e), exc2string2())
      logger.error(m)
      raise RTTCodingError(m)
    
    return script

