/*
  Copyright (C) 2002-2017 CERN for the benefit of the ATLAS collaboration
*/

#ifndef JIVEXML_TRIGSISPACEPOINTRETRIEVER_H
#define JIVEXML_TRIGSISPACEPOINTRETRIEVER_H

#include <string>

#include "JiveXML/IDataRetriever.h"

#include "AthenaBaseComps/AthAlgTool.h"

#include "TrkPrepRawData/PrepRawDataCLASS_DEF.h"

class PixelID;
class SCT_ID;

namespace JiveXML{

  class TrigSiSpacePointRetriever : virtual public IDataRetriever, public AthAlgTool {

  public:

    /// Standard Constructor
    TrigSiSpacePointRetriever(const std::string& type, const std::string& name, const IInterface* parent);

    /// Retrieve all the data
    virtual StatusCode retrieve(ToolHandle<IFormatTool> &FormatTool);

    /// Return the name of the data type
    virtual std::string dataTypeName() const { return m_typeName; };

    virtual StatusCode initialize();

  private:

   ///The data type that is generated by this retriever
    const std::string m_typeName;

    int calcPixelLayer(const Identifier&);
    int calcSCTLayer(const Identifier&);

    const PixelID*      m_pixelHelper;
    const SCT_ID*       m_sctHelper;  

    std::string m_pixelSpContainerName;
    std::string m_SCT_SpContainerName;
  };

}
#endif
