# Copyright (C) 2002-2017 CERN for the benefit of the ATLAS collaboration

#
#######################################################################
# File: CaloClusterCorrection/python/CaloSwCalibHitsCalibration_vX.py
# Created: July 2008, lc db                                                  
# Purpose: Calibration Hits based em cluster calibration, v8.                
#######################################################################
#
# - These corrections were derived by D. Banfi, L.C., L. Mandelli            
# and E. Nebot Del Busto on ATLAS-CSC-05-00-00 simulation.                  
#
# - Corrections are calculated from photons as in
# rel 14.2.0.                                                                
#
# - In v8 calibration coeffs for conv/unconv are separated                  
#
# The final correction CaloSwCalibHitsCalibration_v8_eleXX consists of
# five subcorrections which should be applied together:
# 1) CaloSwCalHitsAccordion_v8_XX (cluster size dependent)
# 2) CaloSWCalHitsOutOfCone_v8_eleXX (cluster size dependent)
# 3) CaloSWCalHitsLongLeakage_v8_ele
# 4) CaloSWCalHitsEfront_offset_v8_ele
# 5) CaloSWCalHitsEfront_slope_v8_ele
#
# Almost the same for photons
# 1) CaloSwCalHitsAccordion_v8_XX_YY (cluster size dependent)
# 2) CaloSWCalHitsOutOfCone_v8_gamXX_YY (cluster size dependent)
# 3) CaloSWCalHitsLongLeakage_v8_gam_YY
# 4) CaloSWCalHitsEfront_offset_v8_gam_YY
# 5) CaloSWCalHitsEfront_slope_v8_gam_YY
#
# Where YY is "conv" for converted or "unconv" for unconverted
#
# - January 2011 : update of all the calibration coefficients based on
# GEO-16-00-00 by LC, H. de la Torre, I. Koletsou, L. Mandelli, J del Peso,
# R. Turra.
#######################################################################

from CaloClusterCorrection.common import *

#######################################################################

# Accordion correction

#######################################################################




CaloSwCalHitsAccordion_v8_35 = [
[ 0.0125,   1.0346e+00,   -5.7124e-03,   2.4414e-04  ],# bin 0  chi2/NDF: 1.5156, copied from bin 2
[ 0.0375,   1.0346e+00,   -5.7124e-03,   2.4414e-04  ],# bin 1  chi2/NDF: 11.588, copied from bin 2
[ 0.0625,   1.0346e+00,   -5.7124e-03,   2.4414e-04  ],# bin 2  chi2/NDF: 3.3503
[ 0.0875,   1.0421e+00,   -6.8787e-03,   2.8967e-04  ],# bin 3  chi2/NDF: 9.6411
[ 0.1125,   1.0334e+00,   -5.4251e-03,   2.3037e-04  ],# bin 4  chi2/NDF: 3.8031
[ 0.1375,   1.0339e+00,   -5.5281e-03,   2.3104e-04  ],# bin 5  chi2/NDF: 12.823
[ 0.1625,   1.0372e+00,   -6.1714e-03,   2.6385e-04  ],# bin 6  chi2/NDF: 12.804
[ 0.1875,   1.0394e+00,   -6.2554e-03,   2.5026e-04  ],# bin 7  chi2/NDF: 3.2001
[ 0.2125,   1.0375e+00,   -6.1547e-03,   2.5011e-04  ],# bin 8  chi2/NDF: 3.1397
[ 0.2375,   1.0302e+00,   -5.3038e-03,   2.2311e-04  ],# bin 9  chi2/NDF: 2.8248
[ 0.2625,   1.0328e+00,   -5.6115e-03,   2.3063e-04  ],# bin 10  chi2/NDF: 1.9465
[ 0.2875,   1.0416e+00,   -7.1770e-03,   2.9240e-04  ],# bin 11  chi2/NDF: 3.5436
[ 0.3125,   1.0280e+00,   -4.8573e-03,   1.9959e-04  ],# bin 12  chi2/NDF: 2.9889
[ 0.3375,   1.0277e+00,   -4.8024e-03,   1.9499e-04  ],# bin 13  chi2/NDF: 4.6955
[ 0.3625,   1.0254e+00,   -4.3519e-03,   1.7235e-04  ],# bin 14  chi2/NDF: 11.736
[ 0.3875,   1.0260e+00,   -4.3688e-03,   1.7521e-04  ],# bin 15  chi2/NDF: 3.167
[ 0.4125,   1.0328e+00,   -4.9181e-03,   1.8280e-04  ],# bin 16  chi2/NDF: 2.9766
[ 0.4375,   1.0171e+00,   -2.6014e-03,   9.3256e-05  ],# bin 17  chi2/NDF: 2.2698
[ 0.4625,   1.0210e+00,   -3.2363e-03,   1.1909e-04  ],# bin 18  chi2/NDF: 2.7753
[ 0.4875,   1.0288e+00,   -4.3734e-03,   1.6069e-04  ],# bin 19  chi2/NDF: 2.8908
[ 0.5125,   1.0273e+00,   -4.2314e-03,   1.5540e-04  ],# bin 20  chi2/NDF: 2.8068
[ 0.5375,   1.0210e+00,   -3.2648e-03,   1.1950e-04  ],# bin 21  chi2/NDF: 1.3996
[ 0.5625,   1.0273e+00,   -4.3181e-03,   1.6136e-04  ],# bin 22  chi2/NDF: 2.7819
[ 0.5875,   1.0222e+00,   -3.4782e-03,   1.2239e-04  ],# bin 23  chi2/NDF: 1.9722
[ 0.6125,   1.0216e+00,   -3.4662e-03,   1.2356e-04  ],# bin 24  chi2/NDF: 3.3177
[ 0.6375,   1.0204e+00,   -3.3653e-03,   1.1966e-04  ],# bin 25  chi2/NDF: 3.9022
[ 0.6625,   1.0223e+00,   -3.6654e-03,   1.3175e-04  ],# bin 26  chi2/NDF: 3.0543
[ 0.6875,   1.0223e+00,   -3.8615e-03,   1.4407e-04  ],# bin 27  chi2/NDF: 3.2167
[ 0.7125,   1.0221e+00,   -3.8607e-03,   1.4513e-04  ],# bin 28  chi2/NDF: 3.4662
[ 0.7375,   1.0159e+00,   -2.3839e-03,   4.6287e-05  ],# bin 29  chi2/NDF: 3.5243
[ 0.7625,   1.0159e+00,   -2.3839e-03,   4.6287e-05  ],# bin 30  chi2/NDF: 2.6551, copied from bin 29
[ 0.7875,   1.0159e+00,   -2.3839e-03,   4.6287e-05  ],# bin 31  chi2/NDF: 2.2192, copied from bin 29
[ 0.8125,   1.0474e+00,   -6.3193e-03,   2.0096e-04  ],# bin 32  chi2/NDF: 1.1427, copied from bin 34
[ 0.8375,   1.0474e+00,   -6.3193e-03,   2.0096e-04  ],# bin 33  chi2/NDF: 1.2068, copied from bin 34
[ 0.8625,   1.0474e+00,   -6.3193e-03,   2.0096e-04  ],# bin 34  chi2/NDF: 2.122
[ 0.8875,   1.0544e+00,   -7.0346e-03,   2.1866e-04  ],# bin 35  chi2/NDF: 0.9393
[ 0.9125,   1.0461e+00,   -5.8139e-03,   1.6841e-04  ],# bin 36  chi2/NDF: 0.78119
[ 0.9375,   1.0398e+00,   -5.0820e-03,   1.5103e-04  ],# bin 37  chi2/NDF: 0.75772
[ 0.9625,   1.0426e+00,   -5.6351e-03,   1.7596e-04  ],# bin 38  chi2/NDF: 1.9735
[ 0.9875,   1.0326e+00,   -3.9968e-03,   1.1126e-04  ],# bin 39  chi2/NDF: 1.3695
[ 1.0125,   1.0378e+00,   -5.0438e-03,   1.5642e-04  ],# bin 40  chi2/NDF: 2.4945
[ 1.0375,   1.0278e+00,   -3.4366e-03,   9.0892e-05  ],# bin 41  chi2/NDF: 1.3354
[ 1.0625,   1.0307e+00,   -3.8764e-03,   1.0738e-04  ],# bin 42  chi2/NDF: 1.7117
[ 1.0875,   1.0303e+00,   -3.7911e-03,   1.0134e-04  ],# bin 43  chi2/NDF: 1.7717
[ 1.1125,   1.0270e+00,   -3.0904e-03,   6.9200e-05  ],# bin 44  chi2/NDF: 1.839
[ 1.1375,   1.0279e+00,   -3.4580e-03,   8.6175e-05  ],# bin 45  chi2/NDF: 1.1506
[ 1.1625,   1.0386e+00,   -5.2288e-03,   1.5885e-04  ],# bin 46  chi2/NDF: 1.7276
[ 1.1875,   1.0635e+00,   -8.4652e-03,   2.7061e-04  ],# bin 47  chi2/NDF: 1.5407
[ 1.2125,   1.0404e+00,   -5.0748e-03,   1.3170e-04  ],# bin 48  chi2/NDF: 2.0927
[ 1.2375,   1.0327e+00,   -3.9260e-03,   9.3256e-05  ],# bin 49  chi2/NDF: 2.4396
[ 1.2625,   1.0362e+00,   -4.3443e-03,   1.0710e-04  ],# bin 50  chi2/NDF: 1.6801
[ 1.2875,   1.0400e+00,   -4.5801e-03,   1.0097e-04  ],# bin 51  chi2/NDF: 1.4746
[ 1.3125,   1.0399e+00,   -4.5531e-03,   9.7479e-05  ],# bin 52  chi2/NDF: 2.0781
[ 1.3375,   1.0387e+00,   -4.0014e-03,   6.5606e-05  ],# bin 53  chi2/NDF: 2.1076
[ 1.3625,   1.0373e+00,   -3.9545e-03,   6.2502e-05  ],# bin 54  chi2/NDF: 1.3469
[ 1.3875,   1.0365e+00,   -2.8827e-03,   -2.5929e-05  ],# bin 55  chi2/NDF: 1.4438
[ 1.4125,   1.1057e+00,   -1.7242e-02,   6.8111e-04  ],# bin 56  chi2/NDF: 1.3465
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack missing par  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack missing par  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack missing par  chi2/NDF: 0
[ 1.5125,   9.7023e-01,   1.0417e-02,   -4.6713e-04  ],# bin 60 crack  chi2/NDF: 8.8305
[ 1.5375,   1.0452e+00,   -6.5337e-03,   2.0134e-04  ],# bin 61 crack  chi2/NDF: 1.942
[ 1.5625,   1.0297e+00,   -4.4907e-03,   1.3560e-04  ],# bin 62  chi2/NDF: 1.4714
[ 1.5875,   1.0233e+00,   -4.2050e-03,   1.4150e-04  ],# bin 63  chi2/NDF: 1.3273
[ 1.6125,   1.0258e+00,   -4.8561e-03,   1.7256e-04  ],# bin 64  chi2/NDF: 1.0333
[ 1.6375,   1.0360e+00,   -6.9712e-03,   2.6520e-04  ],# bin 65  chi2/NDF: 1.4586
[ 1.6625,   1.0330e+00,   -6.6730e-03,   2.5664e-04  ],# bin 66  chi2/NDF: 0.92678
[ 1.6875,   1.0257e+00,   -5.5877e-03,   2.1597e-04  ],# bin 67  chi2/NDF: 1.2232
[ 1.7125,   1.0371e+00,   -7.4257e-03,   2.7866e-04  ],# bin 68  chi2/NDF: 0.72053
[ 1.7375,   1.0315e+00,   -7.4186e-03,   2.9892e-04  ],# bin 69  chi2/NDF: 0.73737
[ 1.7625,   1.0264e+00,   -7.2308e-03,   3.0363e-04  ],# bin 70  chi2/NDF: 0.9632
[ 1.7875,   1.0330e+00,   -8.8254e-03,   3.8276e-04  ],# bin 71  chi2/NDF: 0.73609
[ 1.8125,   1.0652e+00,   -1.3373e-02,   5.4761e-04  ],# bin 72  chi2/NDF: 0.54602
[ 1.8375,   1.0553e+00,   -1.2435e-02,   5.3824e-04  ],# bin 73  chi2/NDF: 1.299
[ 1.8625,   1.0489e+00,   -1.1796e-02,   5.3128e-04  ],# bin 74  chi2/NDF: 2.3165
[ 1.8875,   1.0395e+00,   -1.0452e-02,   4.8209e-04  ],# bin 75  chi2/NDF: 0.98226
[ 1.9125,   1.0408e+00,   -1.0756e-02,   4.9282e-04  ],# bin 76  chi2/NDF: 0.86389
[ 1.9375,   1.0609e+00,   -1.3894e-02,   6.0826e-04  ],# bin 77  chi2/NDF: 0.95084
[ 1.9625,   1.0414e+00,   -1.1259e-02,   5.1223e-04  ],# bin 78  chi2/NDF: 0.77572
[ 1.9875,   1.0585e+00,   -1.3479e-02,   5.8036e-04  ],# bin 79  chi2/NDF: 0.89438
[ 2.0125,   1.0751e+00,   -1.5015e-02,   6.2142e-04  ],# bin 80  chi2/NDF: 0.98832
[ 2.0375,   1.0358e+00,   -9.4872e-03,   4.3545e-04  ],# bin 81  chi2/NDF: 0.98387
[ 2.0625,   1.0174e+00,   -7.2926e-03,   3.4609e-04  ],# bin 82  chi2/NDF: 1.0283
[ 2.0875,   1.0173e+00,   -7.7411e-03,   3.5481e-04  ],# bin 83  chi2/NDF: 1.529
[ 2.1125,   1.0203e+00,   -8.6326e-03,   4.0469e-04  ],# bin 84  chi2/NDF: 1.238
[ 2.1375,   1.0116e+00,   -7.0777e-03,   3.4542e-04  ],# bin 85  chi2/NDF: 1.3005
[ 2.1625,   1.0131e+00,   -7.1610e-03,   3.4295e-04  ],# bin 86  chi2/NDF: 0.60727
[ 2.1875,   1.0281e+00,   -8.8458e-03,   3.9270e-04  ],# bin 87  chi2/NDF: 1.4358
[ 2.2125,   1.0387e+00,   -1.0242e-02,   4.4152e-04  ],# bin 88  chi2/NDF: 2.136
[ 2.2375,   1.0532e+00,   -1.4393e-02,   6.6243e-04  ],# bin 89  chi2/NDF: 1.0744
[ 2.2625,   1.0301e+00,   -1.1059e-02,   5.3248e-04  ],# bin 90  chi2/NDF: 2.1806
[ 2.2875,   1.0346e+00,   -1.2329e-02,   5.9262e-04  ],# bin 91  chi2/NDF: 1.4757
[ 2.3125,   1.0349e+00,   -1.1858e-02,   5.5171e-04  ],# bin 92  chi2/NDF: 2.2964
[ 2.3375,   1.1462e+00,   -2.8337e-02,   1.1635e-03  ],# bin 93  chi2/NDF: 74.656
[ 2.3625,   1.1513e+00,   -2.9349e-02,   1.1939e-03  ],# bin 94  chi2/NDF: 85.944
[ 2.3875,   1.2772e+00,   -4.7863e-02,   1.8850e-03  ],# bin 95  chi2/NDF: 17.771
[ 2.4125,   1.2814e+00,   -4.4712e-02,   1.6322e-03  ],# bin 96  chi2/NDF: 1.7238
[ 2.4375,   1.2946e+00,   -4.4778e-02,   1.5488e-03  ],# bin 97  chi2/NDF: 0.88982
[ 2.4625,   1.2483e+00,   -3.8290e-02,   1.3023e-03  ],# bin 98  chi2/NDF: 1.8824
[ 2.4875,   1.2483e+00,   -3.8290e-02,   1.3023e-03  ],# bin 99  chi2/NDF: 0.4275, copied from bin 98
] # end CaloSwCalHitsAccordion_v8_35


CaloSwCalHitsAccordion_v8_37 = [
[ 0.0125,   1.0612e+00,   -9.9087e-03,   4.1613e-04  ],# bin 0  chi2/NDF: 2.3305, copied from bin 2
[ 0.0375,   1.0612e+00,   -9.9087e-03,   4.1613e-04  ],# bin 1  chi2/NDF: 12.008, copied from bin 2
[ 0.0625,   1.0612e+00,   -9.9087e-03,   4.1613e-04  ],# bin 2  chi2/NDF: 3.5456
[ 0.0875,   1.0693e+00,   -1.1157e-02,   4.6458e-04  ],# bin 3  chi2/NDF: 10.705
[ 0.1125,   1.0623e+00,   -9.9710e-03,   4.1582e-04  ],# bin 4  chi2/NDF: 5.6467
[ 0.1375,   1.0523e+00,   -8.2351e-03,   3.3657e-04  ],# bin 5  chi2/NDF: 14.491
[ 0.1625,   1.0586e+00,   -9.4122e-03,   3.9291e-04  ],# bin 6  chi2/NDF: 15.361
[ 0.1875,   1.0636e+00,   -1.0046e-02,   4.0525e-04  ],# bin 7  chi2/NDF: 3.8009
[ 0.2125,   1.0600e+00,   -9.6080e-03,   3.8816e-04  ],# bin 8  chi2/NDF: 3.7195
[ 0.2375,   1.0563e+00,   -9.3878e-03,   3.8934e-04  ],# bin 9  chi2/NDF: 4.5192
[ 0.2625,   1.0560e+00,   -9.1357e-03,   3.7045e-04  ],# bin 10  chi2/NDF: 2.9253
[ 0.2875,   1.0634e+00,   -1.0414e-02,   4.1841e-04  ],# bin 11  chi2/NDF: 3.4822
[ 0.3125,   1.0500e+00,   -8.1383e-03,   3.2737e-04  ],# bin 12  chi2/NDF: 3.2597
[ 0.3375,   1.0486e+00,   -7.8717e-03,   3.1345e-04  ],# bin 13  chi2/NDF: 5.4042
[ 0.3625,   1.0461e+00,   -7.3501e-03,   2.8662e-04  ],# bin 14  chi2/NDF: 14.48
[ 0.3875,   1.0472e+00,   -7.5577e-03,   3.0088e-04  ],# bin 15  chi2/NDF: 4.2715
[ 0.4125,   1.0556e+00,   -8.2967e-03,   3.1336e-04  ],# bin 16  chi2/NDF: 3.8705
[ 0.4375,   1.0375e+00,   -5.4820e-03,   2.0016e-04  ],# bin 17  chi2/NDF: 2.9474
[ 0.4625,   1.0401e+00,   -5.9262e-03,   2.1897e-04  ],# bin 18  chi2/NDF: 2.7636
[ 0.4875,   1.0459e+00,   -6.6532e-03,   2.4150e-04  ],# bin 19  chi2/NDF: 3.5088
[ 0.5125,   1.0461e+00,   -6.8122e-03,   2.4950e-04  ],# bin 20  chi2/NDF: 2.9344
[ 0.5375,   1.0415e+00,   -6.1254e-03,   2.2455e-04  ],# bin 21  chi2/NDF: 3.1346
[ 0.5625,   1.0478e+00,   -7.1781e-03,   2.6660e-04  ],# bin 22  chi2/NDF: 3.4073
[ 0.5875,   1.0423e+00,   -6.2572e-03,   2.2384e-04  ],# bin 23  chi2/NDF: 2.6651
[ 0.6125,   1.0378e+00,   -5.6088e-03,   1.9968e-04  ],# bin 24  chi2/NDF: 3.9446
[ 0.6375,   1.0374e+00,   -5.6806e-03,   2.0302e-04  ],# bin 25  chi2/NDF: 4.8365
[ 0.6625,   1.0390e+00,   -5.9975e-03,   2.1903e-04  ],# bin 26  chi2/NDF: 3.4527
[ 0.6875,   1.0402e+00,   -6.4294e-03,   2.4199e-04  ],# bin 27  chi2/NDF: 3.9117
[ 0.7125,   1.0335e+00,   -5.3996e-03,   2.0206e-04  ],# bin 28  chi2/NDF: 4.4725
[ 0.7375,   1.0325e+00,   -4.8327e-03,   1.4393e-04  ],# bin 29  chi2/NDF: 5.4582
[ 0.7625,   1.0325e+00,   -4.8327e-03,   1.4393e-04  ],# bin 30  chi2/NDF: 3.8228, copied from bin 29
[ 0.7875,   1.0325e+00,   -4.8327e-03,   1.4393e-04  ],# bin 31  chi2/NDF: 2.5115, copied from bin 29
[ 0.8125,   1.0690e+00,   -9.2798e-03,   3.0939e-04  ],# bin 32  chi2/NDF: 1.287, copied from bin 34
[ 0.8375,   1.0690e+00,   -9.2798e-03,   3.0939e-04  ],# bin 33  chi2/NDF: 1.2325, copied from bin 34
[ 0.8625,   1.0690e+00,   -9.2798e-03,   3.0939e-04  ],# bin 34  chi2/NDF: 2.0643
[ 0.8875,   1.0705e+00,   -9.1220e-03,   2.9192e-04  ],# bin 35  chi2/NDF: 1.0721
[ 0.9125,   1.0628e+00,   -8.0463e-03,   2.5028e-04  ],# bin 36  chi2/NDF: 1.1514
[ 0.9375,   1.0585e+00,   -7.5962e-03,   2.4241e-04  ],# bin 37  chi2/NDF: 1.3244
[ 0.9625,   1.0598e+00,   -7.9792e-03,   2.6309e-04  ],# bin 38  chi2/NDF: 2.8981
[ 0.9875,   1.0479e+00,   -5.9918e-03,   1.8253e-04  ],# bin 39  chi2/NDF: 1.5966
[ 1.0125,   1.0495e+00,   -6.4714e-03,   2.0570e-04  ],# bin 40  chi2/NDF: 1.67
[ 1.0375,   1.0486e+00,   -6.4000e-03,   2.0416e-04  ],# bin 41  chi2/NDF: 1.3229
[ 1.0625,   1.0493e+00,   -6.4562e-03,   2.0447e-04  ],# bin 42  chi2/NDF: 2.0727
[ 1.0875,   1.0472e+00,   -6.1295e-03,   1.8919e-04  ],# bin 43  chi2/NDF: 2.4222
[ 1.1125,   1.0407e+00,   -4.9344e-03,   1.3840e-04  ],# bin 44  chi2/NDF: 2.2926
[ 1.1375,   1.0459e+00,   -5.9782e-03,   1.8203e-04  ],# bin 45  chi2/NDF: 2.0966
[ 1.1625,   1.0495e+00,   -6.5641e-03,   2.0558e-04  ],# bin 46  chi2/NDF: 1.7474
[ 1.1875,   1.0723e+00,   -9.3702e-03,   2.9615e-04  ],# bin 47  chi2/NDF: 1.0699
[ 1.2125,   1.0558e+00,   -7.1901e-03,   2.1235e-04  ],# bin 48  chi2/NDF: 2.82
[ 1.2375,   1.0497e+00,   -6.2887e-03,   1.8249e-04  ],# bin 49  chi2/NDF: 3.1572
[ 1.2625,   1.0524e+00,   -6.5823e-03,   1.9227e-04  ],# bin 50  chi2/NDF: 2.4308
[ 1.2875,   1.0554e+00,   -6.6753e-03,   1.8009e-04  ],# bin 51  chi2/NDF: 1.5826
[ 1.3125,   1.0552e+00,   -6.6167e-03,   1.7477e-04  ],# bin 52  chi2/NDF: 2.3028
[ 1.3375,   1.0523e+00,   -5.7569e-03,   1.2879e-04  ],# bin 53  chi2/NDF: 1.0228
[ 1.3625,   1.0570e+00,   -6.9041e-03,   1.8124e-04  ],# bin 54  chi2/NDF: 1.1173
[ 1.3875,   1.0501e+00,   -4.8731e-03,   5.2569e-05  ],# bin 55  chi2/NDF: 1.7438
[ 1.4125,   1.1134e+00,   -1.8405e-02,   7.3392e-04  ],# bin 56  chi2/NDF: 1.3203
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack missing par  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack missing par  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack missing par  chi2/NDF: 0
[ 1.5125,   9.8416e-01,   9.0479e-03,   -4.2612e-04  ],# bin 60 crack  chi2/NDF: 8.8587
[ 1.5375,   1.0621e+00,   -8.5496e-03,   2.6764e-04  ],# bin 61 crack  chi2/NDF: 2.0576
[ 1.5625,   1.0470e+00,   -6.4492e-03,   1.9626e-04  ],# bin 62  chi2/NDF: 1.5697
[ 1.5875,   1.0344e+00,   -5.3734e-03,   1.7659e-04  ],# bin 63  chi2/NDF: 1.1777
[ 1.6125,   1.0381e+00,   -6.2887e-03,   2.2035e-04  ],# bin 64  chi2/NDF: 0.59904
[ 1.6375,   1.0461e+00,   -8.0670e-03,   3.0031e-04  ],# bin 65  chi2/NDF: 1.125
[ 1.6625,   1.0438e+00,   -7.8734e-03,   2.9603e-04  ],# bin 66  chi2/NDF: 1.5414
[ 1.6875,   1.0359e+00,   -6.6364e-03,   2.4798e-04  ],# bin 67  chi2/NDF: 1.7125
[ 1.7125,   1.0465e+00,   -8.3479e-03,   3.0588e-04  ],# bin 68  chi2/NDF: 1.2992
[ 1.7375,   1.0405e+00,   -8.2445e-03,   3.2072e-04  ],# bin 69  chi2/NDF: 1.161
[ 1.7625,   1.0354e+00,   -8.0852e-03,   3.2723e-04  ],# bin 70  chi2/NDF: 1.0786
[ 1.7875,   1.0446e+00,   -1.0065e-02,   4.2058e-04  ],# bin 71  chi2/NDF: 1.0143
[ 1.8125,   1.0683e+00,   -1.3364e-02,   5.4067e-04  ],# bin 72  chi2/NDF: 0.85238
[ 1.8375,   1.0770e+00,   -1.5092e-02,   6.2632e-04  ],# bin 73  chi2/NDF: 0.95694
[ 1.8625,   1.0673e+00,   -1.4023e-02,   6.0545e-04  ],# bin 74  chi2/NDF: 2.0964
[ 1.8875,   1.0535e+00,   -1.2061e-02,   5.3480e-04  ],# bin 75  chi2/NDF: 0.63197
[ 1.9125,   1.0521e+00,   -1.1944e-02,   5.2936e-04  ],# bin 76  chi2/NDF: 0.91508
[ 1.9375,   1.0715e+00,   -1.5020e-02,   6.4430e-04  ],# bin 77  chi2/NDF: 0.85746
[ 1.9625,   1.0507e+00,   -1.2146e-02,   5.3815e-04  ],# bin 78  chi2/NDF: 0.78328
[ 1.9875,   1.0656e+00,   -1.4024e-02,   5.9355e-04  ],# bin 79  chi2/NDF: 1.2359
[ 2.0125,   1.0865e+00,   -1.6191e-02,   6.5844e-04  ],# bin 80  chi2/NDF: 1.7351
[ 2.0375,   1.0513e+00,   -1.1246e-02,   4.9312e-04  ],# bin 81  chi2/NDF: 1.7558
[ 2.0625,   1.0314e+00,   -8.8113e-03,   3.9439e-04  ],# bin 82  chi2/NDF: 0.92111
[ 2.0875,   1.0288e+00,   -8.9101e-03,   3.9113e-04  ],# bin 83  chi2/NDF: 1.7211
[ 2.1125,   1.0300e+00,   -9.5368e-03,   4.3168e-04  ],# bin 84  chi2/NDF: 1.2286
[ 2.1375,   1.0221e+00,   -8.0938e-03,   3.7610e-04  ],# bin 85  chi2/NDF: 1.0653
[ 2.1625,   1.0275e+00,   -8.7814e-03,   3.9713e-04  ],# bin 86  chi2/NDF: 0.69297
[ 2.1875,   1.0385e+00,   -9.8448e-03,   4.2332e-04  ],# bin 87  chi2/NDF: 1.9004
[ 2.2125,   1.0524e+00,   -1.1655e-02,   4.8471e-04  ],# bin 88  chi2/NDF: 2.1168
[ 2.2375,   1.0657e+00,   -1.5768e-02,   7.1012e-04  ],# bin 89  chi2/NDF: 0.99384
[ 2.2625,   1.0373e+00,   -1.1602e-02,   5.4782e-04  ],# bin 90  chi2/NDF: 2.2745
[ 2.2875,   1.0490e+00,   -1.3897e-02,   6.4447e-04  ],# bin 91  chi2/NDF: 1.7968
[ 2.3125,   1.0464e+00,   -1.2988e-02,   5.8816e-04  ],# bin 92  chi2/NDF: 3.6221
[ 2.3375,   1.1438e+00,   -2.7278e-02,   1.1150e-03  ],# bin 93  chi2/NDF: 72.795
[ 2.3625,   1.1607e+00,   -3.0010e-02,   1.2071e-03  ],# bin 94  chi2/NDF: 72.163
[ 2.3875,   1.3101e+00,   -5.2142e-02,   2.0366e-03  ],# bin 95  chi2/NDF: 16.246
[ 2.4125,   1.3104e+00,   -4.8294e-02,   1.7541e-03  ],# bin 96  chi2/NDF: 1.8213
[ 2.4375,   1.3198e+00,   -4.7720e-02,   1.6450e-03  ],# bin 97  chi2/NDF: 0.80009
[ 2.4625,   1.2645e+00,   -3.9965e-02,   1.3550e-03  ],# bin 98  chi2/NDF: 1.7014
[ 2.4875,   1.2645e+00,   -3.9965e-02,   1.3550e-03  ],# bin 99  chi2/NDF: 0.17861, copied from bin 98
] # end CaloSwCalHitsAccordion_v8_37


CaloSwCalHitsAccordion_v8_55 = [
[ 0.0125,   1.0461e+00,   -6.9085e-03,   2.8668e-04  ],# bin 0  chi2/NDF: 3.5628, copied from bin 2
[ 0.0375,   1.0461e+00,   -6.9085e-03,   2.8668e-04  ],# bin 1  chi2/NDF: 12.743, copied from bin 2
[ 0.0625,   1.0461e+00,   -6.9085e-03,   2.8668e-04  ],# bin 2  chi2/NDF: 4.3036
[ 0.0875,   1.0515e+00,   -7.7187e-03,   3.1696e-04  ],# bin 3  chi2/NDF: 10.705
[ 0.1125,   1.0443e+00,   -6.5335e-03,   2.6983e-04  ],# bin 4  chi2/NDF: 4.4833
[ 0.1375,   1.0451e+00,   -6.6934e-03,   2.7357e-04  ],# bin 5  chi2/NDF: 12.84
[ 0.1625,   1.0512e+00,   -7.8253e-03,   3.2662e-04  ],# bin 6  chi2/NDF: 13.592
[ 0.1875,   1.0565e+00,   -8.4962e-03,   3.4059e-04  ],# bin 7  chi2/NDF: 4.6457
[ 0.2125,   1.0533e+00,   -8.2056e-03,   3.3303e-04  ],# bin 8  chi2/NDF: 4.92
[ 0.2375,   1.0432e+00,   -6.7699e-03,   2.7845e-04  ],# bin 9  chi2/NDF: 4.68
[ 0.2625,   1.0511e+00,   -8.0261e-03,   3.2745e-04  ],# bin 10  chi2/NDF: 11.34
[ 0.2875,   1.0522e+00,   -8.2569e-03,   3.3196e-04  ],# bin 11  chi2/NDF: 4.7179
[ 0.3125,   1.0422e+00,   -6.5428e-03,   2.6332e-04  ],# bin 12  chi2/NDF: 4.6472
[ 0.3375,   1.0403e+00,   -6.1480e-03,   2.4389e-04  ],# bin 13  chi2/NDF: 6.8625
[ 0.3625,   1.0389e+00,   -5.8272e-03,   2.2608e-04  ],# bin 14  chi2/NDF: 13.049
[ 0.3875,   1.0436e+00,   -6.6182e-03,   2.6381e-04  ],# bin 15  chi2/NDF: 4.2813
[ 0.4125,   1.0479e+00,   -6.7806e-03,   2.5584e-04  ],# bin 16  chi2/NDF: 3.6405
[ 0.4375,   1.0267e+00,   -3.4711e-03,   1.2297e-04  ],# bin 17  chi2/NDF: 3.8477
[ 0.4625,   1.0317e+00,   -4.2974e-03,   1.5698e-04  ],# bin 18  chi2/NDF: 4.4082
[ 0.4875,   1.0383e+00,   -5.1601e-03,   1.8544e-04  ],# bin 19  chi2/NDF: 4.1402
[ 0.5125,   1.0379e+00,   -5.2196e-03,   1.8885e-04  ],# bin 20  chi2/NDF: 3.3933
[ 0.5375,   1.0354e+00,   -4.9052e-03,   1.8013e-04  ],# bin 21  chi2/NDF: 3.6189
[ 0.5625,   1.0381e+00,   -5.3315e-03,   1.9566e-04  ],# bin 22  chi2/NDF: 4.1874
[ 0.5875,   1.0328e+00,   -4.4574e-03,   1.5576e-04  ],# bin 23  chi2/NDF: 3.2931
[ 0.6125,   1.0331e+00,   -4.6208e-03,   1.6425e-04  ],# bin 24  chi2/NDF: 4.3343
[ 0.6375,   1.0320e+00,   -4.5721e-03,   1.6375e-04  ],# bin 25  chi2/NDF: 6.0002
[ 0.6625,   1.0349e+00,   -5.0592e-03,   1.8444e-04  ],# bin 26  chi2/NDF: 5.5794
[ 0.6875,   1.0325e+00,   -4.8632e-03,   1.8163e-04  ],# bin 27  chi2/NDF: 5.1946
[ 0.7125,   1.0362e+00,   -5.5746e-03,   2.1170e-04  ],# bin 28  chi2/NDF: 6.0454
[ 0.7375,   1.0347e+00,   -4.9662e-03,   1.5591e-04  ],# bin 29  chi2/NDF: 5.3949
[ 0.7625,   1.0347e+00,   -4.9662e-03,   1.5591e-04  ],# bin 30  chi2/NDF: 3.8459, copied from bin 29
[ 0.7875,   1.0347e+00,   -4.9662e-03,   1.5591e-04  ],# bin 31  chi2/NDF: 3.4262, copied from bin 29
[ 0.8125,   1.0607e+00,   -7.4664e-03,   2.3557e-04  ],# bin 32  chi2/NDF: 1.2941, copied from bin 34
[ 0.8375,   1.0607e+00,   -7.4664e-03,   2.3557e-04  ],# bin 33  chi2/NDF: 2.4077, copied from bin 34
[ 0.8625,   1.0607e+00,   -7.4664e-03,   2.3557e-04  ],# bin 34  chi2/NDF: 1.0221
[ 0.8875,   1.0698e+00,   -8.6775e-03,   2.7815e-04  ],# bin 35  chi2/NDF: 1.2184
[ 0.9125,   1.0650e+00,   -8.0337e-03,   2.5075e-04  ],# bin 36  chi2/NDF: 0.54736
[ 0.9375,   1.0554e+00,   -6.8247e-03,   2.1666e-04  ],# bin 37  chi2/NDF: 1.2677
[ 0.9625,   1.0526e+00,   -6.3937e-03,   1.9900e-04  ],# bin 38  chi2/NDF: 1.3151
[ 0.9875,   1.0514e+00,   -6.2837e-03,   1.9911e-04  ],# bin 39  chi2/NDF: 2.5134
[ 1.0125,   1.0487e+00,   -5.9951e-03,   1.8833e-04  ],# bin 40  chi2/NDF: 2.0437
[ 1.0375,   1.0433e+00,   -5.1971e-03,   1.5959e-04  ],# bin 41  chi2/NDF: 1.997
[ 1.0625,   1.0473e+00,   -5.7956e-03,   1.8008e-04  ],# bin 42  chi2/NDF: 3.0582
[ 1.0875,   1.0461e+00,   -5.6931e-03,   1.7796e-04  ],# bin 43  chi2/NDF: 2.269
[ 1.1125,   1.0395e+00,   -4.3783e-03,   1.1740e-04  ],# bin 44  chi2/NDF: 2.2353
[ 1.1375,   1.0450e+00,   -5.5458e-03,   1.6964e-04  ],# bin 45  chi2/NDF: 2.2728
[ 1.1625,   1.0525e+00,   -6.7000e-03,   2.1261e-04  ],# bin 46  chi2/NDF: 2.4935
[ 1.1875,   1.0809e+00,   -1.0662e-02,   3.6079e-04  ],# bin 47  chi2/NDF: 2.4283
[ 1.2125,   1.0536e+00,   -6.5154e-03,   1.8662e-04  ],# bin 48  chi2/NDF: 2.8567
[ 1.2375,   1.0430e+00,   -4.8975e-03,   1.3022e-04  ],# bin 49  chi2/NDF: 2.8215
[ 1.2625,   1.0489e+00,   -5.6863e-03,   1.5765e-04  ],# bin 50  chi2/NDF: 3.0786
[ 1.2875,   1.0526e+00,   -5.9563e-03,   1.5587e-04  ],# bin 51  chi2/NDF: 2.0316
[ 1.3125,   1.0516e+00,   -5.7185e-03,   1.4076e-04  ],# bin 52  chi2/NDF: 2.8711
[ 1.3375,   1.0520e+00,   -5.4948e-03,   1.2612e-04  ],# bin 53  chi2/NDF: 2.4636
[ 1.3625,   1.0494e+00,   -5.2084e-03,   1.0660e-04  ],# bin 54  chi2/NDF: 1.5375
[ 1.3875,   1.0410e+00,   -2.4301e-03,   -5.2885e-05  ],# bin 55  chi2/NDF: 2.3098
[ 1.4125,   1.0702e+00,   -8.5418e-03,   2.3952e-04  ],# bin 56  chi2/NDF: 1.3537
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack missing par  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack missing par  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack missing par  chi2/NDF: 0
[ 1.5125,   1.0186e+00,   4.7086e-03,   -2.9738e-04  ],# bin 60 crack  chi2/NDF: 9.6197
[ 1.5375,   1.0784e+00,   -1.0459e-02,   3.3723e-04  ],# bin 61 crack  chi2/NDF: 3.1878
[ 1.5625,   1.0444e+00,   -5.7373e-03,   1.7506e-04  ],# bin 62  chi2/NDF: 1.229
[ 1.5875,   1.0299e+00,   -4.3404e-03,   1.4246e-04  ],# bin 63  chi2/NDF: 1.5754
[ 1.6125,   1.0426e+00,   -6.5971e-03,   2.3402e-04  ],# bin 64  chi2/NDF: 1.2534
[ 1.6375,   1.0535e+00,   -8.7673e-03,   3.2928e-04  ],# bin 65  chi2/NDF: 1.6337
[ 1.6625,   1.0494e+00,   -8.2389e-03,   3.1181e-04  ],# bin 66  chi2/NDF: 1.2526
[ 1.6875,   1.0440e+00,   -7.3882e-03,   2.7706e-04  ],# bin 67  chi2/NDF: 2.2277
[ 1.7125,   1.0555e+00,   -9.1850e-03,   3.3587e-04  ],# bin 68  chi2/NDF: 0.9076
[ 1.7375,   1.0468e+00,   -8.7999e-03,   3.4556e-04  ],# bin 69  chi2/NDF: 0.82934
[ 1.7625,   1.0389e+00,   -8.1452e-03,   3.3360e-04  ],# bin 70  chi2/NDF: 1.0453
[ 1.7875,   1.0473e+00,   -9.9872e-03,   4.2185e-04  ],# bin 71  chi2/NDF: 1.0616
[ 1.8125,   1.0800e+00,   -1.4574e-02,   5.8482e-04  ],# bin 72  chi2/NDF: 1.274
[ 1.8375,   1.0801e+00,   -1.5111e-02,   6.3118e-04  ],# bin 73  chi2/NDF: 1.3784
[ 1.8625,   1.0705e+00,   -1.4132e-02,   6.1632e-04  ],# bin 74  chi2/NDF: 2.0142
[ 1.8875,   1.0573e+00,   -1.2168e-02,   5.4215e-04  ],# bin 75  chi2/NDF: 0.86094
[ 1.9125,   1.0524e+00,   -1.1521e-02,   5.1713e-04  ],# bin 76  chi2/NDF: 1.2603
[ 1.9375,   1.0831e+00,   -1.6240e-02,   6.9213e-04  ],# bin 77  chi2/NDF: 1.6202
[ 1.9625,   1.0497e+00,   -1.1517e-02,   5.1937e-04  ],# bin 78  chi2/NDF: 0.99422
[ 1.9875,   1.0765e+00,   -1.5026e-02,   6.3009e-04  ],# bin 79  chi2/NDF: 1.061
[ 2.0125,   1.0917e+00,   -1.6410e-02,   6.6632e-04  ],# bin 80  chi2/NDF: 1.4986
[ 2.0375,   1.0443e+00,   -9.6901e-03,   4.3853e-04  ],# bin 81  chi2/NDF: 0.95461
[ 2.0625,   1.0295e+00,   -8.0769e-03,   3.7243e-04  ],# bin 82  chi2/NDF: 0.6531
[ 2.0875,   1.0336e+00,   -9.1407e-03,   4.0334e-04  ],# bin 83  chi2/NDF: 1.455
[ 2.1125,   1.0347e+00,   -9.7923e-03,   4.4543e-04  ],# bin 84  chi2/NDF: 1.4427
[ 2.1375,   1.0233e+00,   -7.8293e-03,   3.7121e-04  ],# bin 85  chi2/NDF: 1.1802
[ 2.1625,   1.0277e+00,   -8.2951e-03,   3.8282e-04  ],# bin 86  chi2/NDF: 0.70535
[ 2.1875,   1.0458e+00,   -1.0259e-02,   4.3690e-04  ],# bin 87  chi2/NDF: 1.5576
[ 2.2125,   1.0685e+00,   -1.3576e-02,   5.5694e-04  ],# bin 88  chi2/NDF: 2.4528
[ 2.2375,   1.0787e+00,   -1.7140e-02,   7.6044e-04  ],# bin 89  chi2/NDF: 1.7214
[ 2.2625,   1.0603e+00,   -1.4692e-02,   6.7004e-04  ],# bin 90  chi2/NDF: 1.9126
[ 2.2875,   1.0585e+00,   -1.4832e-02,   6.7955e-04  ],# bin 91  chi2/NDF: 1.1024
[ 2.3125,   1.0617e+00,   -1.4709e-02,   6.4960e-04  ],# bin 92  chi2/NDF: 1.942
[ 2.3375,   1.1715e+00,   -3.0895e-02,   1.2474e-03  ],# bin 93  chi2/NDF: 73.16
[ 2.3625,   1.1804e+00,   -3.2510e-02,   1.3056e-03  ],# bin 94  chi2/NDF: 90.832
[ 2.3875,   1.3606e+00,   -5.8605e-02,   2.2583e-03  ],# bin 95  chi2/NDF: 16.584
[ 2.4125,   1.3134e+00,   -4.7865e-02,   1.7144e-03  ],# bin 96  chi2/NDF: 1.5389
[ 2.4375,   1.3203e+00,   -4.8107e-02,   1.6831e-03  ],# bin 97  chi2/NDF: 0.84129
[ 2.4625,   1.2472e+00,   -3.8403e-02,   1.3330e-03  ],# bin 98  chi2/NDF: 1.9496
[ 2.4875,   1.2472e+00,   -3.8403e-02,   1.3330e-03  ],# bin 99  chi2/NDF: 1.9496, copied from bin 98
] # end CaloSwCalHitsAccordion_v8_55


CaloSwCalHitsEfront_offset_v8_ele = [
[ 0.0125,   2.9965e+02,   9.6699e-01,   -4.1614e-04  ],# bin 0  chi2/NDF: 1357.6, copied from bin 2
[ 0.0375,   2.9965e+02,   9.6699e-01,   -4.1614e-04  ],# bin 1  chi2/NDF: 852.59, copied from bin 2
[ 0.0625,   2.9965e+02,   9.6699e-01,   -4.1614e-04  ],# bin 2  chi2/NDF: 1698.4
[ 0.0875,   2.9693e+02,   9.1710e-01,   -4.2709e-04  ],# bin 3  chi2/NDF: 793.77
[ 0.1125,   2.9919e+02,   9.7169e-01,   -4.0382e-04  ],# bin 4  chi2/NDF: 2413.9
[ 0.1375,   2.9628e+02,   1.0966e+00,   -5.9641e-04  ],# bin 5  chi2/NDF: 2167.4
[ 0.1625,   3.0426e+02,   8.7990e-01,   -2.7089e-04  ],# bin 6  chi2/NDF: 2357.6
[ 0.1875,   2.9546e+02,   1.0329e+00,   -4.2663e-04  ],# bin 7  chi2/NDF: 1500.7
[ 0.2125,   3.0223e+02,   1.1205e+00,   -5.1476e-04  ],# bin 8  chi2/NDF: 3043.6
[ 0.2375,   2.9446e+02,   1.0691e+00,   -3.1325e-04  ],# bin 9  chi2/NDF: 2419.2
[ 0.2625,   3.0429e+02,   1.0580e+00,   -3.7737e-04  ],# bin 10  chi2/NDF: 3523
[ 0.2875,   3.1045e+02,   1.2574e+00,   -5.8401e-04  ],# bin 11  chi2/NDF: 2845.2
[ 0.3125,   3.0211e+02,   1.3325e+00,   -6.1472e-04  ],# bin 12  chi2/NDF: 3229.5
[ 0.3375,   3.1821e+02,   1.1825e+00,   -4.6711e-04  ],# bin 13  chi2/NDF: 3878.9
[ 0.3625,   3.0732e+02,   1.2797e+00,   -5.0929e-04  ],# bin 14  chi2/NDF: 2382.2
[ 0.3875,   3.1843e+02,   1.3661e+00,   -6.2637e-04  ],# bin 15  chi2/NDF: 2749.7
[ 0.4125,   3.1153e+02,   1.2691e+00,   -4.4879e-04  ],# bin 16  chi2/NDF: 1038.5
[ 0.4375,   3.1625e+02,   1.4261e+00,   -5.5204e-04  ],# bin 17  chi2/NDF: 982.39
[ 0.4625,   3.2309e+02,   1.3677e+00,   -5.1116e-04  ],# bin 18  chi2/NDF: 1789.6
[ 0.4875,   3.4658e+02,   1.3134e+00,   -4.0360e-04  ],# bin 19  chi2/NDF: 1997.9
[ 0.5125,   3.3983e+02,   1.8261e+00,   -9.3025e-04  ],# bin 20  chi2/NDF: 2157.8
[ 0.5375,   3.7331e+02,   1.4386e+00,   -5.1236e-04  ],# bin 21  chi2/NDF: 2429.4
[ 0.5625,   3.6228e+02,   2.1112e+00,   -1.2083e-03  ],# bin 22  chi2/NDF: 3850.5
[ 0.5875,   3.7074e+02,   1.8564e+00,   -7.8577e-04  ],# bin 23  chi2/NDF: 2502
[ 0.6125,   4.1155e+02,   1.4462e+00,   1.1237e-04  ],# bin 24  chi2/NDF: 4981.2
[ 0.6375,   4.2947e+02,   1.8984e+00,   -2.0537e-04  ],# bin 25  chi2/NDF: 7421.2
[ 0.6625,   4.9190e+02,   1.8803e+00,   -1.6111e-04  ],# bin 26  chi2/NDF: 668.01
[ 0.6875,   5.0630e+02,   2.8252e+00,   -1.2277e-03  ],# bin 27  chi2/NDF: 4085.8
[ 0.7125,   5.5761e+02,   2.6277e+00,   -1.1047e-03  ],# bin 28  chi2/NDF: 2963.1
[ 0.7375,   6.1905e+02,   2.1106e+00,   -3.9072e-04  ],# bin 29  chi2/NDF: 1460.4
[ 0.7625,   6.1905e+02,   2.1106e+00,   -3.9072e-04  ],# bin 30  chi2/NDF: 2226.3, copied from bin 29
[ 0.7875,   6.1905e+02,   2.1106e+00,   -3.9072e-04  ],# bin 31  chi2/NDF: 9438.6, copied from bin 29
[ 0.8125,   9.4037e+02,   2.2631e+00,   -5.9850e-04  ],# bin 32  chi2/NDF: 4939.1, copied from bin 34
[ 0.8375,   9.4037e+02,   2.2631e+00,   -5.9850e-04  ],# bin 33  chi2/NDF: 7350.9, copied from bin 34
[ 0.8625,   9.4037e+02,   2.2631e+00,   -5.9850e-04  ],# bin 34  chi2/NDF: 10811
[ 0.8875,   9.7886e+02,   2.5780e+00,   -6.6133e-04  ],# bin 35  chi2/NDF: 24250
[ 0.9125,   1.0507e+03,   2.5180e+00,   -3.1361e-04  ],# bin 36  chi2/NDF: 17340
[ 0.9375,   1.0945e+03,   2.2520e+00,   -2.9635e-05  ],# bin 37  chi2/NDF: 23819
[ 0.9625,   1.1162e+03,   3.3052e+00,   -6.8868e-04  ],# bin 38  chi2/NDF: 36222
[ 0.9875,   1.1713e+03,   2.9339e+00,   -1.9490e-04  ],# bin 39  chi2/NDF: 41678
[ 1.0125,   1.3203e+03,   2.8099e+00,   -1.3995e-04  ],# bin 40  chi2/NDF: 1408.1
[ 1.0375,   1.3229e+03,   2.6338e+00,   5.2177e-04  ],# bin 41  chi2/NDF: 15662
[ 1.0625,   1.4710e+03,   2.3791e+00,   9.5693e-04  ],# bin 42  chi2/NDF: 35406
[ 1.0875,   1.3723e+03,   3.2987e+00,   4.0659e-04  ],# bin 43  chi2/NDF: 4715.2
[ 1.1125,   1.3653e+03,   4.6545e+00,   -5.2971e-04  ],# bin 44  chi2/NDF: 17565
[ 1.1375,   1.5237e+03,   5.0292e+00,   -9.1748e-04  ],# bin 45  chi2/NDF: 14217
[ 1.1625,   1.5218e+03,   5.3522e+00,   -3.7798e-04  ],# bin 46  chi2/NDF: 18598
[ 1.1875,   1.5993e+03,   7.1033e+00,   -2.2661e-03  ],# bin 47  chi2/NDF: 35149
[ 1.2125,   1.6159e+03,   6.8348e+00,   -8.8127e-04  ],# bin 48  chi2/NDF: 21996
[ 1.2375,   1.8093e+03,   6.1464e+00,   8.9402e-04  ],# bin 49  chi2/NDF: 51588
[ 1.2625,   1.8251e+03,   7.1969e+00,   8.0616e-04  ],# bin 50  chi2/NDF: 73195
[ 1.2875,   1.8807e+03,   1.0137e+01,   -2.5719e-03  ],# bin 51  chi2/NDF: 38124
[ 1.3125,   2.0159e+03,   1.1986e+01,   -3.6586e-03  ],# bin 52  chi2/NDF: 96379
[ 1.3375,   2.1746e+03,   1.1552e+01,   -3.2625e-03  ],# bin 53  chi2/NDF: 124590
[ 1.3625,   2.5406e+03,   1.0243e+01,   1.6689e-03  ],# bin 54  chi2/NDF: 43032
[ 1.3875,   2.7320e+03,   1.1737e+01,   4.2701e-03  ],# bin 55  chi2/NDF: 97652
[ 1.4125,   3.0802e+03,   1.5378e+01,   1.4943e-03  ],# bin 56  chi2/NDF: 245490
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   8.0661e+03,   9.2197e+02,   4.4972e+03  ],# bin 60 crack  chi2/NDF: 1.1506e+08
[ 1.5375,   -7.8137e+03,   -4.7185e+00,   1.4652e+03  ],# bin 61 crack  chi2/NDF: 3.5758e+07
[ 1.5625,   1.7337e+03,   6.4605e+00,   9.7248e+00  ],# bin 62  chi2/NDF: 508730
[ 1.5875,   1.4369e+03,   5.4395e+00,   3.6385e+01  ],# bin 63  chi2/NDF: 146470
[ 1.6125,   1.6725e+03,   5.8199e+00,   -7.0769e+00  ],# bin 64  chi2/NDF: 296830
[ 1.6375,   7.6581e+02,   -2.3983e+00,   3.1151e+02  ],# bin 65  chi2/NDF: 660700
[ 1.6625,   -1.4582e+03,   -7.7391e+00,   6.4670e+02  ],# bin 66  chi2/NDF: 1.1326e+06
[ 1.6875,   -1.7012e+03,   -1.0224e+00,   3.7568e+02  ],# bin 67  chi2/NDF: 3.0407e+06
[ 1.7125,   7.2029e+02,   -1.2813e+00,   1.7054e+02  ],# bin 68  chi2/NDF: 64957
[ 1.7375,   1.0109e+03,   2.1286e+00,   4.4010e+01  ],# bin 69  chi2/NDF: 31288
[ 1.7625,   1.3906e+03,   3.5677e+00,   -2.4007e+01  ],# bin 70  chi2/NDF: 47175
[ 1.7875,   8.8608e+02,   3.4343e+00,   5.6558e+00  ],# bin 71  chi2/NDF: 20212
[ 1.8125,   -1.8848e+03,   9.4947e+02,   9.9905e-02  ],# bin 72  chi2/NDF: 0.30046
[ 1.8375,   1.0280e+03,   9.6109e+02,   -3.8014e-02  ],# bin 73  chi2/NDF: 0.58623
[ 1.8625,   9.3807e+02,   8.1784e+02,   -4.8276e-01  ],# bin 74  chi2/NDF: 1.088
[ 1.8875,   4.7764e+03,   7.8508e+02,   -3.5572e-01  ],# bin 75  chi2/NDF: 0.38757
[ 1.9125,   5.4881e+03,   7.1022e+02,   -1.1778e-01  ],# bin 76  chi2/NDF: 1.4947
[ 1.9375,   4.4544e+02,   7.3381e+02,   -3.1221e-01  ],# bin 77  chi2/NDF: 0.7721
[ 1.9625,   4.3182e+03,   6.1316e+02,   -1.4060e-01  ],# bin 78  chi2/NDF: 0.52913
[ 1.9875,   6.3641e+03,   6.4961e+02,   -2.3371e-01  ],# bin 79  chi2/NDF: 0.50846
[ 2.0125,   2.4273e+03,   7.1795e+02,   -3.9123e-02  ],# bin 80  chi2/NDF: 0.34711
[ 2.0375,   4.6314e+03,   7.1544e+02,   -1.3343e-01  ],# bin 81  chi2/NDF: 1.1745
[ 2.0625,   7.7150e+03,   6.3573e+02,   -1.3540e-01  ],# bin 82  chi2/NDF: 1.4841
[ 2.0875,   4.4087e+03,   7.4999e+02,   -2.4748e-01  ],# bin 83  chi2/NDF: 1.6156
[ 2.1125,   3.7695e+03,   7.0852e+02,   -9.7448e-02  ],# bin 84  chi2/NDF: 0.23106
[ 2.1375,   3.2287e+03,   6.9444e+02,   -2.2229e-01  ],# bin 85  chi2/NDF: 1.3991
[ 2.1625,   2.4391e+03,   6.7433e+02,   -1.8892e-01  ],# bin 86  chi2/NDF: 1.8578
[ 2.1875,   2.9669e+03,   6.6499e+02,   -2.4102e-01  ],# bin 87  chi2/NDF: 0.36959
[ 2.2125,   3.5713e+03,   5.8536e+02,   -1.3404e-01  ],# bin 88  chi2/NDF: 1.1265
[ 2.2375,   3.3721e+03,   6.0370e+02,   -2.9176e-01  ],# bin 89  chi2/NDF: 3.2549
[ 2.2625,   3.4924e+03,   4.9253e+02,   -5.5820e-02  ],# bin 90  chi2/NDF: 0.51159
[ 2.2875,   3.0543e+03,   4.6333e+02,   -5.2531e-02  ],# bin 91  chi2/NDF: 2.1782
[ 2.3125,   1.2251e+03,   2.8991e+02,   -3.7652e-01  ],# bin 92  chi2/NDF: 2.0179
[ 2.3375,   -5.2293e+03,   9.5625e+02,   -3.8900e-01  ],# bin 93  chi2/NDF: 0.6215
[ 2.3625,   4.1147e+02,   7.7342e+02,   1.3063e-01  ],# bin 94  chi2/NDF: 0.36306
[ 2.3875,   -3.5762e+03,   8.8287e+02,   -1.4902e-01  ],# bin 95  chi2/NDF: 0.55649
[ 2.4125,   -1.2545e+04,   9.9160e+02,   -4.1962e-01  ],# bin 96  chi2/NDF: 0.28015
[ 2.4375,   3.0166e+03,   8.3685e+02,   -1.1648e-01  ],# bin 97  chi2/NDF: 0.49617
[ 2.4625,   -1.4114e+03,   9.3402e+02,   -2.4306e-01  ],# bin 98  chi2/NDF: 0.36236
[ 2.4875,   -1.4114e+03,   9.3402e+02,   -2.4306e-01  ],# bin 99 missing par  chi2/NDF: 0, copied from bin 98
] # end CaloSwCalHitsEfront_offset_v8_ele


CaloSwCalHitsEfront_offset_v8_gam_conv = [
[ 0.0125,   5.5321e+02,   -2.0574e-01,   1.1700e-03  ],# bin 0  chi2/NDF: 3011.6, copied from bin 2
[ 0.0375,   5.5321e+02,   -2.0574e-01,   1.1700e-03  ],# bin 1  chi2/NDF: 1732.5, copied from bin 2
[ 0.0625,   5.5321e+02,   -2.0574e-01,   1.1700e-03  ],# bin 2  chi2/NDF: 2405.1
[ 0.0875,   5.7883e+02,   -3.7848e-01,   2.3695e-03  ],# bin 3  chi2/NDF: 1818.6
[ 0.1125,   5.5170e+02,   4.1907e-01,   -4.6457e-06  ],# bin 4  chi2/NDF: 2920.3
[ 0.1375,   5.1982e+02,   7.5476e-01,   -1.5631e-04  ],# bin 5  chi2/NDF: 5500.2
[ 0.1625,   5.5290e+02,   1.4610e-01,   2.2320e-05  ],# bin 6  chi2/NDF: 1764.3
[ 0.1875,   5.3981e+02,   7.1122e-01,   -1.7367e-04  ],# bin 7  chi2/NDF: 4029.6
[ 0.2125,   5.4072e+02,   4.3975e-01,   4.1155e-05  ],# bin 8  chi2/NDF: 1687.2
[ 0.2375,   5.8314e+02,   -3.4251e-01,   1.7822e-03  ],# bin 9  chi2/NDF: 1571.6
[ 0.2625,   5.3974e+02,   4.6017e-01,   1.8209e-04  ],# bin 10  chi2/NDF: 4781.8
[ 0.2875,   5.3258e+02,   1.0949e+00,   -1.6623e-03  ],# bin 11  chi2/NDF: 3826.7
[ 0.3125,   5.7433e+02,   3.3628e-01,   3.4151e-04  ],# bin 12  chi2/NDF: 2736.8
[ 0.3375,   5.4371e+02,   8.2982e-01,   -2.5584e-04  ],# bin 13  chi2/NDF: 2584
[ 0.3625,   5.6660e+02,   7.5424e-01,   -1.8769e-06  ],# bin 14  chi2/NDF: 2220.5
[ 0.3875,   5.8248e+02,   4.6972e-01,   4.1153e-04  ],# bin 15  chi2/NDF: 2920.5
[ 0.4125,   6.0996e+02,   6.1212e-01,   -2.2534e-05  ],# bin 16  chi2/NDF: 4043.3
[ 0.4375,   5.9942e+02,   4.0088e-01,   8.0171e-05  ],# bin 17  chi2/NDF: 3759.7
[ 0.4625,   6.1581e+02,   5.6107e-01,   8.2600e-05  ],# bin 18  chi2/NDF: 2250.2
[ 0.4875,   6.4418e+02,   4.6789e-01,   2.5165e-04  ],# bin 19  chi2/NDF: 2948.3
[ 0.5125,   6.5375e+02,   6.3530e-01,   1.5073e-04  ],# bin 20  chi2/NDF: 4038.8
[ 0.5375,   6.9413e+02,   2.0471e-01,   7.2441e-04  ],# bin 21  chi2/NDF: 3239.5
[ 0.5625,   6.6573e+02,   1.3421e+00,   -3.6264e-04  ],# bin 22  chi2/NDF: 2796.9
[ 0.5875,   6.8243e+02,   1.0482e+00,   -2.8743e-04  ],# bin 23  chi2/NDF: 5137.8
[ 0.6125,   7.3199e+02,   9.4913e-01,   4.2956e-04  ],# bin 24  chi2/NDF: 5700
[ 0.6375,   7.5405e+02,   1.6537e+00,   -4.4818e-04  ],# bin 25  chi2/NDF: 11533
[ 0.6625,   9.2747e+02,   3.6463e-01,   1.1806e-03  ],# bin 26  chi2/NDF: 3754.6
[ 0.6875,   1.0010e+03,   9.9125e-01,   5.5358e-05  ],# bin 27  chi2/NDF: 16942
[ 0.7125,   1.1106e+03,   1.8991e-01,   1.7415e-03  ],# bin 28  chi2/NDF: 4548.4
[ 0.7375,   1.1023e+03,   9.7328e-01,   9.2116e-04  ],# bin 29  chi2/NDF: 21465
[ 0.7625,   1.1023e+03,   9.7328e-01,   9.2116e-04  ],# bin 30  chi2/NDF: 22956, copied from bin 29
[ 0.7875,   1.1023e+03,   9.7328e-01,   9.2116e-04  ],# bin 31  chi2/NDF: 11430, copied from bin 29
[ 0.8125,   1.1541e+03,   8.5819e-01,   7.6531e-04  ],# bin 32  chi2/NDF: 11133, copied from bin 34
[ 0.8375,   1.1541e+03,   8.5819e-01,   7.6531e-04  ],# bin 33  chi2/NDF: 41674, copied from bin 34
[ 0.8625,   1.1541e+03,   8.5819e-01,   7.6531e-04  ],# bin 34  chi2/NDF: 22044
[ 0.8875,   1.1809e+03,   -1.7948e-01,   1.9553e-03  ],# bin 35  chi2/NDF: 31938
[ 0.9125,   1.2677e+03,   2.5821e-01,   1.4841e-03  ],# bin 36  chi2/NDF: 39331
[ 0.9375,   1.3572e+03,   -3.0560e-01,   2.1293e-03  ],# bin 37  chi2/NDF: 23083
[ 0.9625,   1.2652e+03,   8.7609e-01,   1.2753e-03  ],# bin 38  chi2/NDF: 23279
[ 0.9875,   1.5486e+03,   -1.0964e+00,   3.3782e-03  ],# bin 39  chi2/NDF: 22162
[ 1.0125,   1.4864e+03,   6.3135e-01,   2.0962e-03  ],# bin 40  chi2/NDF: 53375
[ 1.0375,   1.5502e+03,   1.6073e+00,   1.1629e-03  ],# bin 41  chi2/NDF: 54419
[ 1.0625,   1.8041e+03,   6.6653e-01,   1.7699e-03  ],# bin 42  chi2/NDF: 25719
[ 1.0875,   1.7732e+03,   2.0535e+00,   9.9050e-04  ],# bin 43  chi2/NDF: 23633
[ 1.1125,   1.8347e+03,   3.9799e+00,   -1.3150e-03  ],# bin 44  chi2/NDF: 43226
[ 1.1375,   2.1132e+03,   2.3128e+00,   1.5411e-03  ],# bin 45  chi2/NDF: 16384
[ 1.1625,   2.0803e+03,   3.0753e+00,   5.2332e-04  ],# bin 46  chi2/NDF: 38975
[ 1.1875,   2.1866e+03,   3.6427e+00,   6.5135e-04  ],# bin 47  chi2/NDF: 68824
[ 1.2125,   2.2090e+03,   5.6680e+00,   -1.2291e-03  ],# bin 48  chi2/NDF: 91095
[ 1.2375,   2.5263e+03,   3.0893e+00,   4.9020e-03  ],# bin 49  chi2/NDF: 14425
[ 1.2625,   2.5874e+03,   5.3571e+00,   3.0307e-04  ],# bin 50  chi2/NDF: 105710
[ 1.2875,   2.6710e+03,   9.1093e+00,   -1.8822e-03  ],# bin 51  chi2/NDF: 149740
[ 1.3125,   2.8561e+03,   8.0201e+00,   -5.3857e-04  ],# bin 52  chi2/NDF: 56138
[ 1.3375,   3.1314e+03,   9.8413e+00,   -2.5386e-03  ],# bin 53  chi2/NDF: 142730
[ 1.3625,   3.2937e+03,   7.3848e+00,   1.4286e-03  ],# bin 54  chi2/NDF: 178400
[ 1.3875,   3.1108e+03,   1.3550e+01,   -9.1510e-04  ],# bin 55  chi2/NDF: 318640
[ 1.4125,   3.9424e+03,   1.1697e+01,   1.3411e-04  ],# bin 56  chi2/NDF: 673810
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack missing par  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack missing par  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -3.4419e+04,   2.8669e+02,   1.8240e+04  ],# bin 60 crack  chi2/NDF: 1.5669e+07
[ 1.5375,   1.8115e+02,   3.2182e+01,   4.5127e+02  ],# bin 61 crack  chi2/NDF: 2.016e+07
[ 1.5625,   2.3604e+03,   5.4500e+00,   2.2558e+00  ],# bin 62  chi2/NDF: 313380
[ 1.5875,   2.0127e+03,   3.2395e+00,   2.5170e+01  ],# bin 63  chi2/NDF: 324840
[ 1.6125,   1.6993e+03,   6.0366e+00,   5.6925e+00  ],# bin 64  chi2/NDF: 298640
[ 1.6375,   1.5470e+03,   3.5355e-02,   2.0880e+02  ],# bin 65  chi2/NDF: 501810
[ 1.6625,   8.8121e+02,   2.0985e+00,   2.7238e+02  ],# bin 66  chi2/NDF: 1.8044e+06
[ 1.6875,   1.4581e+03,   1.7399e+01,   -2.0475e+02  ],# bin 67  chi2/NDF: 6.071e+06
[ 1.7125,   1.4681e+03,   2.4493e+00,   4.3576e+01  ],# bin 68  chi2/NDF: 227710
[ 1.7375,   1.0117e+03,   2.1710e+00,   2.4856e+01  ],# bin 69  chi2/NDF: 68441
[ 1.7625,   1.0634e+03,   3.6009e+00,   -5.8285e+00  ],# bin 70  chi2/NDF: 58045
[ 1.7875,   1.0666e+03,   4.2126e+00,   -3.9208e+01  ],# bin 71  chi2/NDF: 29025
[ 1.8125,   1.9924e+04,   9.0338e+02,   4.9544e-01  ],# bin 72  chi2/NDF: 6.6338
[ 1.8375,   9.5582e+04,   -7.9287e+02,   7.4581e+00  ],# bin 73  chi2/NDF: 7.9888
[ 1.8625,   1.1483e+04,   9.7267e+02,   7.9493e-01  ],# bin 74  chi2/NDF: 12.907
[ 1.8875,   -3.4314e+03,   1.1215e+03,   2.4093e-02  ],# bin 75  chi2/NDF: 21.086
[ 1.9125,   1.6985e+04,   4.8421e+02,   6.5069e-01  ],# bin 76  chi2/NDF: 77.116
[ 1.9375,   9.8813e+03,   8.6159e+02,   -7.5910e-01  ],# bin 77  chi2/NDF: 0.60966
[ 1.9625,   -3.0142e+04,   1.1723e+03,   -3.3746e-01  ],# bin 78  chi2/NDF: 15.2
[ 1.9875,   4.4069e+04,   2.7980e+02,   1.2374e+00  ],# bin 79  chi2/NDF: 4.4854
[ 2.0125,   -7.1297e+03,   1.1660e+03,   -7.4037e-01  ],# bin 80  chi2/NDF: 0.52702
[ 2.0375,   -8.7162e+03,   1.2024e+03,   -5.0711e-01  ],# bin 81  chi2/NDF: 0.18076
[ 2.0625,   3.0541e+04,   8.7639e+02,   -3.1041e-01  ],# bin 82  chi2/NDF: 6.9894
[ 2.0875,   5.3324e+04,   8.2863e+02,   -8.7815e-02  ],# bin 83  chi2/NDF: 2.8093
[ 2.1125,   3.4516e+04,   5.5247e+02,   1.0782e-01  ],# bin 84  chi2/NDF: 36.96
[ 2.1375,   1.3054e+04,   8.5174e+02,   -2.8602e-01  ],# bin 85  chi2/NDF: 9.7571
[ 2.1625,   1.5047e+04,   7.4634e+02,   -2.0054e-01  ],# bin 86  chi2/NDF: 14.58
[ 2.1875,   8.6942e+03,   6.9457e+02,   -4.3801e-02  ],# bin 87  chi2/NDF: 13.779
[ 2.2125,   1.3599e+04,   6.0355e+02,   1.4596e-01  ],# bin 88  chi2/NDF: 9.4118
[ 2.2375,   2.0839e+03,   7.0468e+02,   3.2700e-02  ],# bin 89  chi2/NDF: 10.332
[ 2.2625,   -7.9063e+03,   7.4976e+02,   -2.7776e-01  ],# bin 90  chi2/NDF: 11.865
[ 2.2875,   -1.8433e+04,   1.0038e+03,   -6.5188e-01  ],# bin 91  chi2/NDF: 27.009
[ 2.3125,   7.5749e+03,   3.0792e+02,   -1.3109e-01  ],# bin 92  chi2/NDF: 16.917
[ 2.3375,   -4.1603e+03,   1.2628e+03,   9.8958e-03  ],# bin 93  chi2/NDF: 10.06
[ 2.3625,   9.1389e+03,   8.8378e+02,   4.1171e-01  ],# bin 94  chi2/NDF: 12.386
[ 2.3875,   -2.1595e+04,   1.4967e+03,   -5.3979e-01  ],# bin 95  chi2/NDF: 5.858
[ 2.4125,   -1.6473e+04,   1.3714e+03,   1.0893e-01  ],# bin 96  chi2/NDF: 22.594
[ 2.4375,   -2.0522e+04,   1.5679e+03,   -6.0999e-01  ],# bin 97  chi2/NDF: 11.687
[ 2.4625,   -3.8488e+03,   1.5334e+03,   -1.0906e+00  ],# bin 98  chi2/NDF: 15.092
[ 2.4875,   -3.8488e+03,   1.5334e+03,   -1.0906e+00  ],# bin 99 missing par  chi2/NDF: 0, copied from bin 98
] # end CaloSwCalHitsEfront_offset_v8_gam_conv


CaloSwCalHitsEfront_offset_v8_gam_unconv = [
[ 0.0125,   1.4083e+01,   8.2891e-01,   -3.1246e-04  ],# bin 0  chi2/NDF: 241.01, copied from bin 2
[ 0.0375,   1.4083e+01,   8.2891e-01,   -3.1246e-04  ],# bin 1  chi2/NDF: 810.9, copied from bin 2
[ 0.0625,   1.4083e+01,   8.2891e-01,   -3.1246e-04  ],# bin 2  chi2/NDF: 713.63
[ 0.0875,   2.2797e+01,   7.9932e-01,   -3.3263e-04  ],# bin 3  chi2/NDF: 291.72
[ 0.1125,   1.2695e+01,   8.2138e-01,   -3.5017e-04  ],# bin 4  chi2/NDF: 1226.4
[ 0.1375,   2.1266e+01,   7.3841e-01,   -2.3721e-04  ],# bin 5  chi2/NDF: 1006.4
[ 0.1625,   2.6919e+01,   7.1416e-01,   -2.5580e-04  ],# bin 6  chi2/NDF: 404.21
[ 0.1875,   1.6684e+01,   8.2997e-01,   -3.5395e-04  ],# bin 7  chi2/NDF: 454.94
[ 0.2125,   2.4023e+01,   8.4100e-01,   -3.0022e-04  ],# bin 8  chi2/NDF: 707.75
[ 0.2375,   4.0442e+01,   6.9426e-01,   -1.5222e-04  ],# bin 9  chi2/NDF: 272.61
[ 0.2625,   1.7995e+01,   9.4536e-01,   -3.9290e-04  ],# bin 10  chi2/NDF: 925.62
[ 0.2875,   2.0610e+01,   8.8117e-01,   -3.2385e-04  ],# bin 11  chi2/NDF: 682.28
[ 0.3125,   3.9159e+01,   7.0707e-01,   -4.3116e-05  ],# bin 12  chi2/NDF: 503.67
[ 0.3375,   3.4669e+01,   9.9140e-01,   -4.4342e-04  ],# bin 13  chi2/NDF: 194.6
[ 0.3625,   3.3697e+01,   9.1857e-01,   -2.7806e-04  ],# bin 14  chi2/NDF: 437.88
[ 0.3875,   3.2253e+01,   9.4843e-01,   -3.2963e-04  ],# bin 15  chi2/NDF: 1384.1
[ 0.4125,   4.3348e+01,   9.1399e-01,   -2.5925e-04  ],# bin 16  chi2/NDF: 474.77
[ 0.4375,   3.4489e+01,   1.1048e+00,   -4.7295e-04  ],# bin 17  chi2/NDF: 621.99
[ 0.4625,   3.9301e+01,   9.4636e-01,   -1.7361e-04  ],# bin 18  chi2/NDF: 933.38
[ 0.4875,   3.5790e+01,   1.2090e+00,   -4.7559e-04  ],# bin 19  chi2/NDF: 525.58
[ 0.5125,   1.8297e+01,   1.3905e+00,   -4.6569e-04  ],# bin 20  chi2/NDF: 3225.4
[ 0.5375,   2.2919e+00,   1.5458e+00,   -6.6649e-04  ],# bin 21  chi2/NDF: 4266.9
[ 0.5625,   -9.9184e+00,   1.7893e+00,   -7.4878e-04  ],# bin 22  chi2/NDF: 7550.6
[ 0.5875,   -3.2747e+01,   1.8674e+00,   -7.1481e-04  ],# bin 23  chi2/NDF: 5068
[ 0.6125,   -3.2670e+01,   1.9350e+00,   -8.6834e-04  ],# bin 24  chi2/NDF: 4035.1
[ 0.6375,   -3.5377e+01,   2.3422e+00,   -1.1708e-03  ],# bin 25  chi2/NDF: 4663.7
[ 0.6625,   1.5978e+01,   2.0233e+00,   -6.0834e-04  ],# bin 26  chi2/NDF: 5797
[ 0.6875,   1.7356e+01,   1.7090e+00,   1.9106e-04  ],# bin 27  chi2/NDF: 11201
[ 0.7125,   -2.1596e+01,   2.5900e+00,   -1.0860e-03  ],# bin 28  chi2/NDF: 8599
[ 0.7375,   7.4381e+00,   2.7517e+00,   -1.0233e-03  ],# bin 29  chi2/NDF: 9029.5
[ 0.7625,   7.4381e+00,   2.7517e+00,   -1.0233e-03  ],# bin 30  chi2/NDF: 5240.3, copied from bin 29
[ 0.7875,   7.4381e+00,   2.7517e+00,   -1.0233e-03  ],# bin 31  chi2/NDF: 6240.1, copied from bin 29
[ 0.8125,   8.0274e+01,   2.1695e+00,   -4.5450e-04  ],# bin 32  chi2/NDF: 4814.7, copied from bin 34
[ 0.8375,   8.0274e+01,   2.1695e+00,   -4.5450e-04  ],# bin 33  chi2/NDF: 2095, copied from bin 34
[ 0.8625,   8.0274e+01,   2.1695e+00,   -4.5450e-04  ],# bin 34  chi2/NDF: 10069
[ 0.8875,   9.6025e+01,   2.6549e+00,   -1.0559e-03  ],# bin 35  chi2/NDF: 5696
[ 0.9125,   8.9420e+01,   2.8636e+00,   -8.4316e-04  ],# bin 36  chi2/NDF: 8844
[ 0.9375,   9.1463e+01,   3.1442e+00,   -1.4791e-03  ],# bin 37  chi2/NDF: 9727
[ 0.9625,   1.3079e+02,   2.9760e+00,   -6.1428e-04  ],# bin 38  chi2/NDF: 9863.4
[ 0.9875,   1.3570e+02,   3.1211e+00,   -7.7353e-04  ],# bin 39  chi2/NDF: 6083.7
[ 1.0125,   1.1347e+02,   4.3320e+00,   -1.9747e-03  ],# bin 40  chi2/NDF: 5407.1
[ 1.0375,   1.2360e+02,   3.9877e+00,   -8.4887e-04  ],# bin 41  chi2/NDF: 8039.9
[ 1.0625,   1.7707e+02,   4.3169e+00,   -9.1537e-04  ],# bin 42  chi2/NDF: 12457
[ 1.0875,   1.3828e+02,   5.5387e+00,   -2.2847e-03  ],# bin 43  chi2/NDF: 12112
[ 1.1125,   1.8792e+02,   4.6300e+00,   -1.1309e-03  ],# bin 44  chi2/NDF: 21542
[ 1.1375,   1.4364e+02,   5.8525e+00,   -2.3597e-03  ],# bin 45  chi2/NDF: 15749
[ 1.1625,   2.3410e+02,   5.4349e+00,   -1.6236e-03  ],# bin 46  chi2/NDF: 4184.2
[ 1.1875,   1.9964e+02,   6.5887e+00,   -2.0217e-03  ],# bin 47  chi2/NDF: 20129
[ 1.2125,   2.3890e+02,   6.6639e+00,   -1.9275e-03  ],# bin 48  chi2/NDF: 11472
[ 1.2375,   2.4954e+02,   6.8628e+00,   -2.0626e-03  ],# bin 49  chi2/NDF: 32563
[ 1.2625,   3.1353e+02,   7.0930e+00,   -1.6808e-03  ],# bin 50  chi2/NDF: 17797
[ 1.2875,   3.8832e+02,   6.9912e+00,   -1.5154e-03  ],# bin 51  chi2/NDF: 36937
[ 1.3125,   3.3420e+02,   8.7825e+00,   -2.0674e-03  ],# bin 52  chi2/NDF: 18449
[ 1.3375,   4.0965e+02,   8.0382e+00,   1.6901e-04  ],# bin 53  chi2/NDF: 36529
[ 1.3625,   3.1840e+02,   1.1269e+01,   -2.3549e-03  ],# bin 54  chi2/NDF: 15316
[ 1.3875,   4.5681e+02,   1.1356e+01,   1.3397e-03  ],# bin 55  chi2/NDF: 31555
[ 1.4125,   2.5270e+02,   2.9261e+01,   -3.5129e-02  ],# bin 56  chi2/NDF: 25193
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack missing par  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack missing par  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -4.4721e+03,   3.1254e+02,   6.0963e+03  ],# bin 60 crack  chi2/NDF: 2.4733e+07
[ 1.5375,   -6.8476e+02,   3.5290e+00,   1.6254e+02  ],# bin 61 crack  chi2/NDF: 90355
[ 1.5625,   -6.5114e+01,   6.2766e+00,   -3.7281e+01  ],# bin 62  chi2/NDF: 4312.7
[ 1.5875,   2.8901e+02,   7.9242e+00,   -9.7105e+01  ],# bin 63  chi2/NDF: 92809
[ 1.6125,   -5.9844e+01,   4.9737e+00,   -2.1076e+01  ],# bin 64  chi2/NDF: 13564
[ 1.6375,   3.4886e+02,   7.2173e+00,   -9.8292e+01  ],# bin 65  chi2/NDF: 50305
[ 1.6625,   2.8332e+02,   6.7742e+00,   -8.4482e+01  ],# bin 66  chi2/NDF: 17195
[ 1.6875,   -1.2307e+02,   5.1350e+00,   -2.8911e+01  ],# bin 67  chi2/NDF: 25036
[ 1.7125,   -8.4379e+01,   4.0669e+00,   -4.7765e+00  ],# bin 68  chi2/NDF: 8492.9
[ 1.7375,   -4.3192e+01,   3.9823e+00,   -9.7154e-01  ],# bin 69  chi2/NDF: 15248
[ 1.7625,   -1.9539e+02,   3.1092e+00,   2.0029e+01  ],# bin 70  chi2/NDF: 3632.4
[ 1.7875,   -3.2858e+01,   3.8115e+00,   -7.0615e+00  ],# bin 71  chi2/NDF: 9693.4
[ 1.8125,   1.9071e+04,   7.2315e+02,   -9.3869e-01  ],# bin 72  chi2/NDF: 6.7464
[ 1.8375,   2.0542e+04,   3.2402e+02,   4.1232e-01  ],# bin 73  chi2/NDF: 9.3387
[ 1.8625,   7.6814e+03,   3.2989e+02,   8.4198e-01  ],# bin 74  chi2/NDF: 14.959
[ 1.8875,   2.4152e+04,   2.5269e+01,   1.5749e+00  ],# bin 75  chi2/NDF: 11.93
[ 1.9125,   3.7584e+03,   2.3959e+02,   9.4753e-01  ],# bin 76  chi2/NDF: 4.5588
[ 1.9375,   -6.0392e+03,   4.0756e+02,   5.1422e-01  ],# bin 77  chi2/NDF: 0.19823
[ 1.9625,   -5.4795e+03,   3.3953e+02,   5.8693e-01  ],# bin 78  chi2/NDF: 5.5675
[ 1.9875,   1.4828e+04,   2.3714e+02,   -2.4544e-02  ],# bin 79  chi2/NDF: 0.999
[ 2.0125,   -1.0009e+04,   5.1344e+02,   -2.5315e-01  ],# bin 80  chi2/NDF: 0.20759
[ 2.0375,   6.1679e+03,   2.5754e+02,   2.7681e-02  ],# bin 81  chi2/NDF: 6.9064
[ 2.0625,   1.1289e+04,   2.0301e+02,   -5.8954e-02  ],# bin 82  chi2/NDF: 0.18435
[ 2.0875,   4.8258e+03,   2.6709e+02,   1.2506e-01  ],# bin 83  chi2/NDF: 12.731
[ 2.1125,   3.1027e+04,   -5.2348e+00,   3.6603e-01  ],# bin 84  chi2/NDF: 1.6205
[ 2.1375,   3.9446e+03,   2.5817e+02,   -1.4037e-01  ],# bin 85  chi2/NDF: 2.2029
[ 2.1625,   3.4491e+04,   4.8516e+00,   1.2271e-01  ],# bin 86  chi2/NDF: 1.1567
[ 2.1875,   2.5698e+04,   -6.0640e+01,   2.7242e-01  ],# bin 87  chi2/NDF: 1.0839
[ 2.2125,   3.0451e+04,   -2.6014e+01,   1.4245e-01  ],# bin 88  chi2/NDF: 2.8847
[ 2.2375,   1.6079e+03,   1.8661e+02,   -1.1986e-01  ],# bin 89  chi2/NDF: 11.116
[ 2.2625,   1.4387e+02,   1.2869e+02,   -3.8673e-02  ],# bin 90  chi2/NDF: 0.59658
[ 2.2875,   8.9391e+03,   4.7811e+01,   2.7831e-02  ],# bin 91  chi2/NDF: 3.5844
[ 2.3125,   -8.0668e+02,   1.3689e+02,   -4.3907e-02  ],# bin 92  chi2/NDF: 2.9597
[ 2.3375,   6.2488e+03,   3.4367e+02,   -1.7027e-01  ],# bin 93  chi2/NDF: 1.8668
[ 2.3625,   6.4763e+04,   1.3961e+02,   3.8819e-02  ],# bin 94  chi2/NDF: 1.8668
[ 2.3875,   2.3153e+04,   2.1352e+02,   -2.4058e-02  ],# bin 95  chi2/NDF: 4.3771
[ 2.4125,   4.7758e+04,   -8.2887e+01,   3.6160e-01  ],# bin 96  chi2/NDF: 9.0368
[ 2.4375,   -4.1035e+03,   2.4500e+02,   -4.9870e-02  ],# bin 97  chi2/NDF: 10.392
[ 2.4625,   3.9976e+03,   2.9951e+02,   -2.0264e-01  ],# bin 98  chi2/NDF: 4.5354
[ 2.4875,   3.9976e+03,   2.9951e+02,   -2.0264e-01  ],# bin 99 missing par  chi2/NDF: 0, copied from bin 98
] # end CaloSwCalHitsEfront_offset_v8_gam_unconv


CaloSwCalHitsLongLeakage_v8_ele = [
[ 0.0125,   6.2811e-02,   2.4637e-06,   -2.1394e-01  ],# bin 0  chi2/NDF: 7.1978, copied from bin 2
[ 0.0375,   6.2811e-02,   2.4637e-06,   -2.1394e-01  ],# bin 1  chi2/NDF: 20.466, copied from bin 2
[ 0.0625,   6.2811e-02,   2.4637e-06,   -2.1394e-01  ],# bin 2  chi2/NDF: 21.875
[ 0.0875,   6.2139e-02,   2.4122e-06,   -2.5228e-01  ],# bin 3  chi2/NDF: 10.276
[ 0.1125,   6.1049e-02,   2.3843e-06,   -2.6308e-01  ],# bin 4  chi2/NDF: 4.9308
[ 0.1375,   6.5882e-02,   2.1361e-06,   -3.1679e-01  ],# bin 5  chi2/NDF: 21.822
[ 0.1625,   6.0653e-02,   2.0927e-06,   -2.7354e-01  ],# bin 6  chi2/NDF: 11.11
[ 0.1875,   5.6886e-02,   2.0965e-06,   -2.5152e-01  ],# bin 7  chi2/NDF: 4.3003
[ 0.2125,   5.6550e-02,   1.9220e-06,   -2.5626e-01  ],# bin 8  chi2/NDF: 6.5348
[ 0.2375,   5.8473e-02,   1.6361e-06,   -2.7378e-01  ],# bin 9  chi2/NDF: 24.813
[ 0.2625,   5.8502e-02,   1.5247e-06,   -2.8466e-01  ],# bin 10  chi2/NDF: 21.557
[ 0.2875,   5.1955e-02,   1.4705e-06,   -2.3389e-01  ],# bin 11  chi2/NDF: 4.9063
[ 0.3125,   5.0307e-02,   1.2583e-06,   -2.3113e-01  ],# bin 12  chi2/NDF: 11.911
[ 0.3375,   4.9837e-02,   1.0661e-06,   -2.3761e-01  ],# bin 13  chi2/NDF: 21.788
[ 0.3625,   4.7499e-02,   9.3555e-07,   -2.3043e-01  ],# bin 14  chi2/NDF: 13.995
[ 0.3875,   3.9240e-02,   8.7185e-07,   -1.6305e-01  ],# bin 15  chi2/NDF: 3.4303
[ 0.4125,   3.6459e-02,   7.2659e-07,   -1.4831e-01  ],# bin 16  chi2/NDF: 5.3064
[ 0.4375,   3.4462e-02,   5.9409e-07,   -1.3910e-01  ],# bin 17  chi2/NDF: 3.4097
[ 0.4625,   3.2839e-02,   4.8363e-07,   -1.3534e-01  ],# bin 18  chi2/NDF: 4.3185
[ 0.4875,   3.2612e-02,   3.8081e-07,   -1.4750e-01  ],# bin 19  chi2/NDF: 4.9472
[ 0.5125,   2.6997e-02,   3.1102e-07,   -1.0020e-01  ],# bin 20  chi2/NDF: 4.8791
[ 0.5375,   2.4546e-02,   2.8034e-07,   -8.4540e-02  ],# bin 21  chi2/NDF: 3.3749
[ 0.5625,   2.1645e-02,   2.5195e-07,   -6.1152e-02  ],# bin 22  chi2/NDF: 2.4567
[ 0.5875,   1.7343e-02,   2.3647e-07,   -2.3167e-02  ],# bin 23  chi2/NDF: 2.1507
[ 0.6125,   1.7832e-02,   2.0532e-07,   -3.6692e-02  ],# bin 24  chi2/NDF: 4.3047
[ 0.6375,   1.4788e-02,   2.0100e-07,   -4.6811e-03  ],# bin 25  chi2/NDF: 3.3173
[ 0.6625,   1.2431e-02,   1.9688e-07,   1.8341e-02  ],# bin 26  chi2/NDF: 2.6341
[ 0.6875,   8.8649e-03,   1.9255e-07,   5.6843e-02  ],# bin 27  chi2/NDF: 3.382
[ 0.7125,   6.7277e-03,   1.7094e-07,   7.7879e-02  ],# bin 28  chi2/NDF: 5.4586
[ 0.7375,   8.2810e-04,   1.7751e-07,   1.4099e-01  ],# bin 29  chi2/NDF: 4.6001
[ 0.7625,   8.2810e-04,   1.7751e-07,   1.4099e-01  ],# bin 30  chi2/NDF: 5.6431, copied from bin 29
[ 0.7875,   8.2810e-04,   1.7751e-07,   1.4099e-01  ],# bin 31  chi2/NDF: 5.4784, copied from bin 29
[ 0.8125,   -3.0286e-02,   2.6276e-07,   6.3800e-01  ],# bin 32  chi2/NDF: 17.209, copied from bin 34
[ 0.8375,   -3.0286e-02,   2.6276e-07,   6.3800e-01  ],# bin 33  chi2/NDF: 4.2427, copied from bin 34
[ 0.8625,   -3.0286e-02,   2.6276e-07,   6.3800e-01  ],# bin 34  chi2/NDF: 4.4997
[ 0.8875,   -3.9950e-02,   2.5033e-07,   7.6072e-01  ],# bin 35  chi2/NDF: 5.5305
[ 0.9125,   -4.8321e-02,   2.4960e-07,   8.5386e-01  ],# bin 36  chi2/NDF: 8.2934
[ 0.9375,   -5.5419e-02,   2.4049e-07,   9.3729e-01  ],# bin 37  chi2/NDF: 9.5556
[ 0.9625,   -6.1393e-02,   2.2353e-07,   1.0161e+00  ],# bin 38  chi2/NDF: 9.2981
[ 0.9875,   -7.5417e-02,   2.4009e-07,   1.1785e+00  ],# bin 39  chi2/NDF: 10.401
[ 1.0125,   -7.4540e-02,   2.0344e-07,   1.1691e+00  ],# bin 40  chi2/NDF: 14.863
[ 1.0375,   -8.4359e-02,   2.2636e-07,   1.2764e+00  ],# bin 41  chi2/NDF: 12.703
[ 1.0625,   -7.6217e-02,   1.6364e-07,   1.1891e+00  ],# bin 42  chi2/NDF: 15.456
[ 1.0875,   -9.2692e-02,   2.1574e-07,   1.3652e+00  ],# bin 43  chi2/NDF: 12.18
[ 1.1125,   -8.6213e-02,   1.6971e-07,   1.2905e+00  ],# bin 44  chi2/NDF: 14.42
[ 1.1375,   -8.7398e-02,   1.6718e-07,   1.2995e+00  ],# bin 45  chi2/NDF: 17.004
[ 1.1625,   -9.1353e-02,   1.5149e-07,   1.3571e+00  ],# bin 46  chi2/NDF: 13.489
[ 1.1875,   -1.0767e-01,   1.5863e-07,   1.5578e+00  ],# bin 47  chi2/NDF: 14.407
[ 1.2125,   -1.1419e-01,   1.5448e-07,   1.6479e+00  ],# bin 48  chi2/NDF: 13.597
[ 1.2375,   -1.2228e-01,   1.6631e-07,   1.7496e+00  ],# bin 49  chi2/NDF: 13.789
[ 1.2625,   -1.2273e-01,   1.3160e-07,   1.7961e+00  ],# bin 50  chi2/NDF: 15.786
[ 1.2875,   -1.4681e-01,   2.1482e-07,   2.1131e+00  ],# bin 51  chi2/NDF: 14.793
[ 1.3125,   -1.5380e-01,   3.3855e-07,   2.2922e+00  ],# bin 52  chi2/NDF: 11.702
[ 1.3375,   -1.6367e-01,   7.7025e-07,   2.6350e+00  ],# bin 53  chi2/NDF: 15.777
[ 1.3625,   -1.5335e-01,   2.3098e-06,   3.0803e+00  ],# bin 54  chi2/NDF: 12.487
[ 1.3875,   2.3465e-02,   6.1117e-06,   2.8973e+00  ],# bin 55  chi2/NDF: 13.471
[ 1.4125,   2.5271e-01,   1.3522e-05,   4.6569e+00  ],# bin 56  chi2/NDF: 15.206
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -2.8303e-02,   5.4896e-09,   7.9828e-01  ],# bin 60 crack  chi2/NDF: 2.3601
[ 1.5375,   -8.6978e-03,   1.0594e-08,   3.2141e-01  ],# bin 61 crack  chi2/NDF: 5.5513
[ 1.5625,   6.0430e-03,   5.3517e-09,   8.3926e-02  ],# bin 62  chi2/NDF: 3.6479
[ 1.5875,   2.1762e-03,   2.7117e-08,   1.2745e-01  ],# bin 63  chi2/NDF: 2.8757
[ 1.6125,   2.4218e-03,   2.4324e-08,   1.2502e-01  ],# bin 64  chi2/NDF: 3.7698
[ 1.6375,   -1.4716e-03,   2.9875e-08,   1.7081e-01  ],# bin 65  chi2/NDF: 3.9836
[ 1.6625,   -6.3952e-04,   2.0936e-08,   1.4996e-01  ],# bin 66  chi2/NDF: 3.0434
[ 1.6875,   1.7496e-03,   1.2538e-08,   1.0885e-01  ],# bin 67  chi2/NDF: 3.4537
[ 1.7125,   4.6535e-03,   6.5685e-09,   5.0202e-02  ],# bin 68  chi2/NDF: 3.1185
[ 1.7375,   5.0365e-03,   1.4660e-08,   4.1746e-02  ],# bin 69  chi2/NDF: 1.2673
[ 1.7625,   5.0856e-03,   1.6733e-08,   3.6533e-02  ],# bin 70  chi2/NDF: 1.5404
[ 1.7875,   5.7531e-03,   2.4972e-08,   2.2093e-02  ],# bin 71  chi2/NDF: 1.6053
[ 1.8125,   3.4618e-03,   2.0690e-08,   4.0609e-02  ],# bin 72  chi2/NDF: 0.8201
[ 1.8375,   -4.5397e-04,   3.9169e-08,   8.2871e-02  ],# bin 73  chi2/NDF: 2.5885
[ 1.8625,   1.2224e-03,   4.2622e-08,   6.7589e-02  ],# bin 74  chi2/NDF: 1.4445
[ 1.8875,   4.3438e-03,   4.0548e-08,   2.6537e-02  ],# bin 75  chi2/NDF: 1.5361
[ 1.9125,   -4.8438e-04,   4.3926e-08,   7.8980e-02  ],# bin 76  chi2/NDF: 1.0802
[ 1.9375,   -1.6255e-03,   3.7733e-08,   9.1873e-02  ],# bin 77  chi2/NDF: 1.6149
[ 1.9625,   -3.8622e-04,   3.1560e-08,   7.5423e-02  ],# bin 78  chi2/NDF: 1.6933
[ 1.9875,   -3.5815e-03,   3.2119e-08,   1.1337e-01  ],# bin 79  chi2/NDF: 1.1243
[ 2.0125,   -1.3052e-04,   1.5691e-08,   7.4300e-02  ],# bin 80  chi2/NDF: 1.2855
[ 2.0375,   3.4624e-03,   1.3995e-08,   2.1425e-02  ],# bin 81  chi2/NDF: 1.6771
[ 2.0625,   -1.2663e-02,   2.2016e-08,   2.1681e-01  ],# bin 82  chi2/NDF: 18.588
[ 2.0875,   -2.0742e-03,   9.8776e-09,   8.6691e-02  ],# bin 83  chi2/NDF: 1.1344
[ 2.1125,   -1.8542e-03,   1.2595e-08,   8.0330e-02  ],# bin 84  chi2/NDF: 1.2852
[ 2.1375,   -1.9724e-03,   1.0927e-08,   8.1503e-02  ],# bin 85  chi2/NDF: 1.873
[ 2.1625,   -2.9331e-03,   8.6656e-09,   9.2574e-02  ],# bin 86  chi2/NDF: 1.2629
[ 2.1875,   -2.1414e-04,   5.6119e-09,   5.8294e-02  ],# bin 87  chi2/NDF: 1.3841
[ 2.2125,   5.6830e-05,   5.3068e-09,   5.3837e-02  ],# bin 88  chi2/NDF: 1.7854
[ 2.2375,   -2.3257e-03,   6.5836e-09,   8.5512e-02  ],# bin 89  chi2/NDF: 1.3205
[ 2.2625,   -3.2195e-03,   5.3106e-09,   9.5129e-02  ],# bin 90  chi2/NDF: 1.3572
[ 2.2875,   -5.4958e-03,   4.4960e-09,   1.2849e-01  ],# bin 91  chi2/NDF: 0.97435
[ 2.3125,   -4.5896e-03,   3.8851e-10,   1.2848e-01  ],# bin 92  chi2/NDF: 0.89294
[ 2.3375,   -1.0939e-02,   3.5991e-09,   2.1121e-01  ],# bin 93  chi2/NDF: 1.6611
[ 2.3625,   -9.4604e-03,   1.1650e-09,   1.9944e-01  ],# bin 94  chi2/NDF: 1.8444
[ 2.3875,   -2.0548e-02,   6.9446e-09,   3.4269e-01  ],# bin 95  chi2/NDF: 1.0583
[ 2.4125,   -1.8058e-02,   6.8907e-09,   3.2602e-01  ],# bin 96  chi2/NDF: 1.2076
[ 2.4375,   -2.4310e-02,   8.5950e-09,   4.3734e-01  ],# bin 97  chi2/NDF: 1.058
[ 2.4625,   -3.4439e-02,   1.1880e-08,   6.5178e-01  ],# bin 98  chi2/NDF: 1.7671
[ 2.4875,   -3.4439e-02,   1.1880e-08,   6.5178e-01  ],# bin 99  chi2/NDF: 0.63955, copied from bin 98
] # end CaloSwCalHitsLongLeakage_v8_ele


CaloSwCalHitsLongLeakage_v8_gam_conv = [
[ 0.0125,   7.1846e-02,   2.6945e-06,   -3.0868e-01  ],# bin 0  chi2/NDF: 3.5995, copied from bin 2
[ 0.0375,   7.1846e-02,   2.6945e-06,   -3.0868e-01  ],# bin 1  chi2/NDF: 3.3542, copied from bin 2
[ 0.0625,   7.1846e-02,   2.6945e-06,   -3.0868e-01  ],# bin 2  chi2/NDF: 2.2408
[ 0.0875,   7.3011e-02,   2.4910e-06,   -3.6671e-01  ],# bin 3  chi2/NDF: 2.3016
[ 0.1125,   7.7293e-02,   2.3416e-06,   -4.2346e-01  ],# bin 4  chi2/NDF: 1.2099
[ 0.1375,   7.0857e-02,   2.2335e-06,   -3.6487e-01  ],# bin 5  chi2/NDF: 2.0395
[ 0.1625,   7.0219e-02,   2.1514e-06,   -3.7309e-01  ],# bin 6  chi2/NDF: 1.9293
[ 0.1875,   7.0970e-02,   2.0707e-06,   -3.9102e-01  ],# bin 7  chi2/NDF: 2.0428
[ 0.2125,   6.8256e-02,   1.9491e-06,   -3.7337e-01  ],# bin 8  chi2/NDF: 1.6906
[ 0.2375,   6.9766e-02,   1.7709e-06,   -3.9814e-01  ],# bin 9  chi2/NDF: 1.9386
[ 0.2625,   6.3321e-02,   1.6674e-06,   -3.4536e-01  ],# bin 10  chi2/NDF: 2.223
[ 0.2875,   6.5002e-02,   1.3990e-06,   -3.6512e-01  ],# bin 11  chi2/NDF: 1.1645
[ 0.3125,   5.7149e-02,   1.3236e-06,   -3.0419e-01  ],# bin 12  chi2/NDF: 1.6133
[ 0.3375,   5.6437e-02,   1.1451e-06,   -3.1210e-01  ],# bin 13  chi2/NDF: 2.3785
[ 0.3625,   5.5188e-02,   9.6317e-07,   -3.1584e-01  ],# bin 14  chi2/NDF: 2.7293
[ 0.3875,   4.9877e-02,   8.3810e-07,   -2.7164e-01  ],# bin 15  chi2/NDF: 1.854
[ 0.4125,   4.3988e-02,   7.4195e-07,   -2.2917e-01  ],# bin 16  chi2/NDF: 3.1565
[ 0.4375,   4.5650e-02,   5.8382e-07,   -2.5999e-01  ],# bin 17  chi2/NDF: 2.3783
[ 0.4625,   4.1785e-02,   4.9033e-07,   -2.3215e-01  ],# bin 18  chi2/NDF: 0.81286
[ 0.4875,   3.8339e-02,   3.8895e-07,   -2.0982e-01  ],# bin 19  chi2/NDF: 1.2697
[ 0.5125,   3.6049e-02,   3.1033e-07,   -2.0506e-01  ],# bin 20  chi2/NDF: 2.2806
[ 0.5375,   3.3135e-02,   2.7960e-07,   -1.8257e-01  ],# bin 21  chi2/NDF: 2.7602
[ 0.5625,   2.6862e-02,   2.5207e-07,   -1.2322e-01  ],# bin 22  chi2/NDF: 1.492
[ 0.5875,   2.7348e-02,   2.3005e-07,   -1.4142e-01  ],# bin 23  chi2/NDF: 1.8713
[ 0.6125,   2.1779e-02,   2.1479e-07,   -8.2703e-02  ],# bin 24  chi2/NDF: 0.92853
[ 0.6375,   2.0094e-02,   2.1565e-07,   -6.9448e-02  ],# bin 25  chi2/NDF: 0.8362
[ 0.6625,   1.5366e-02,   2.2589e-07,   -2.2397e-02  ],# bin 26  chi2/NDF: 1.9934
[ 0.6875,   1.3906e-02,   1.9839e-07,   -4.1272e-03  ],# bin 27  chi2/NDF: 2.0438
[ 0.7125,   1.2797e-02,   1.8213e-07,   3.2747e-03  ],# bin 28  chi2/NDF: 2.2182
[ 0.7375,   8.3245e-03,   1.7348e-07,   4.9335e-02  ],# bin 29  chi2/NDF: 2.0795
[ 0.7625,   8.3245e-03,   1.7348e-07,   4.9335e-02  ],# bin 30  chi2/NDF: 1.6772, copied from bin 29
[ 0.7875,   8.3245e-03,   1.7348e-07,   4.9335e-02  ],# bin 31  chi2/NDF: 2.6709, copied from bin 29
[ 0.8125,   -1.3207e-02,   2.5610e-07,   4.0987e-01  ],# bin 32  chi2/NDF: 1.2539, copied from bin 34
[ 0.8375,   -1.3207e-02,   2.5610e-07,   4.0987e-01  ],# bin 33  chi2/NDF: 2.7313, copied from bin 34
[ 0.8625,   -1.3207e-02,   2.5610e-07,   4.0987e-01  ],# bin 34  chi2/NDF: 3.0833
[ 0.8875,   -1.6232e-02,   2.4637e-07,   4.4537e-01  ],# bin 35  chi2/NDF: 2.5706
[ 0.9125,   -2.1237e-02,   2.2407e-07,   5.0000e-01  ],# bin 36  chi2/NDF: 3.7055
[ 0.9375,   -2.4945e-02,   2.0297e-07,   5.4718e-01  ],# bin 37  chi2/NDF: 3.2033
[ 0.9625,   -3.2439e-02,   1.9859e-07,   6.3233e-01  ],# bin 38  chi2/NDF: 3.1686
[ 0.9875,   -3.6961e-02,   1.9326e-07,   6.8200e-01  ],# bin 39  chi2/NDF: 4.1114
[ 1.0125,   -3.8817e-02,   1.6677e-07,   7.0500e-01  ],# bin 40  chi2/NDF: 4.9425
[ 1.0375,   -4.8006e-02,   1.8031e-07,   8.0459e-01  ],# bin 41  chi2/NDF: 3.7915
[ 1.0625,   -4.6564e-02,   1.5699e-07,   7.8838e-01  ],# bin 42  chi2/NDF: 6.5131
[ 1.0875,   -5.1827e-02,   1.7326e-07,   8.3926e-01  ],# bin 43  chi2/NDF: 4.2341
[ 1.1125,   -4.7989e-02,   1.3302e-07,   7.9789e-01  ],# bin 44  chi2/NDF: 6.8028
[ 1.1375,   -5.6942e-02,   1.7329e-07,   8.9016e-01  ],# bin 45  chi2/NDF: 4.9319
[ 1.1625,   -5.8302e-02,   1.2518e-07,   9.2444e-01  ],# bin 46  chi2/NDF: 6.3814
[ 1.1875,   -6.1389e-02,   1.0310e-07,   9.7392e-01  ],# bin 47  chi2/NDF: 5.1806
[ 1.2125,   -7.2526e-02,   1.2141e-07,   1.1107e+00  ],# bin 48  chi2/NDF: 3.9562
[ 1.2375,   -7.6253e-02,   1.0669e-07,   1.1654e+00  ],# bin 49  chi2/NDF: 4.8111
[ 1.2625,   -9.4125e-02,   1.8693e-07,   1.3816e+00  ],# bin 50  chi2/NDF: 3.4148
[ 1.2875,   -9.3565e-02,   1.8057e-07,   1.4218e+00  ],# bin 51  chi2/NDF: 5.4248
[ 1.3125,   -9.6237e-02,   3.1674e-07,   1.5393e+00  ],# bin 52  chi2/NDF: 3.9705
[ 1.3375,   -8.7580e-02,   7.1955e-07,   1.6672e+00  ],# bin 53  chi2/NDF: 5.6995
[ 1.3625,   -2.7486e-02,   1.5607e-06,   1.5139e+00  ],# bin 54  chi2/NDF: 9.7353
[ 1.3875,   1.1427e-02,   6.8663e-06,   2.8922e+00  ],# bin 55  chi2/NDF: 19.726
[ 1.4125,   1.4424e-01,   1.5864e-05,   5.6495e+00  ],# bin 56  chi2/NDF: 13.241
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -2.8386e-02,   2.6201e-09,   8.0959e-01  ],# bin 60 crack  chi2/NDF: 1.8764
[ 1.5375,   -1.0976e-02,   1.0326e-08,   3.4222e-01  ],# bin 61 crack  chi2/NDF: 6.3012
[ 1.5625,   7.3094e-03,   5.8976e-09,   5.2844e-02  ],# bin 62  chi2/NDF: 3.4268
[ 1.5875,   7.6846e-03,   2.1809e-08,   5.3628e-02  ],# bin 63  chi2/NDF: 2.6241
[ 1.6125,   5.0647e-03,   2.0322e-08,   8.5120e-02  ],# bin 64  chi2/NDF: 3.3594
[ 1.6375,   1.6855e-03,   3.2770e-08,   1.1298e-01  ],# bin 65  chi2/NDF: 2.9752
[ 1.6625,   5.3109e-03,   1.9291e-08,   6.3003e-02  ],# bin 66  chi2/NDF: 3.0394
[ 1.6875,   7.5978e-03,   1.1475e-08,   2.6630e-02  ],# bin 67  chi2/NDF: 1.8115
[ 1.7125,   7.5760e-03,   6.7973e-09,   -3.0146e-03  ],# bin 68  chi2/NDF: 2.2831
[ 1.7375,   8.3933e-03,   1.4391e-08,   -1.4325e-02  ],# bin 69  chi2/NDF: 1.4451
[ 1.7625,   8.6647e-03,   1.4785e-08,   -2.0386e-02  ],# bin 70  chi2/NDF: 1.9154
[ 1.7875,   9.6366e-03,   2.1461e-08,   -3.7050e-02  ],# bin 71  chi2/NDF: 0.91747
[ 1.8125,   6.6090e-03,   1.9028e-08,   -1.1832e-02  ],# bin 72  chi2/NDF: 0.67946
[ 1.8375,   8.2992e-03,   3.3182e-08,   -4.1965e-02  ],# bin 73  chi2/NDF: 2.7582
[ 1.8625,   9.6337e-03,   3.9799e-08,   -5.5686e-02  ],# bin 74  chi2/NDF: 0.96666
[ 1.8875,   7.4886e-03,   4.3218e-08,   -3.1575e-02  ],# bin 75  chi2/NDF: 1.0125
[ 1.9125,   6.9621e-03,   3.7377e-08,   -2.4193e-02  ],# bin 76  chi2/NDF: 0.90274
[ 1.9375,   5.1346e-03,   3.5692e-08,   -8.7921e-03  ],# bin 77  chi2/NDF: 1.174
[ 1.9625,   5.2925e-03,   2.8877e-08,   -1.1527e-02  ],# bin 78  chi2/NDF: 0.99416
[ 1.9875,   5.5842e-03,   2.4153e-08,   -1.6614e-02  ],# bin 79  chi2/NDF: 1.1175
[ 2.0125,   5.1859e-03,   1.6427e-08,   -1.6762e-02  ],# bin 80  chi2/NDF: 0.90487
[ 2.0375,   4.4824e-03,   1.5539e-08,   -8.4399e-03  ],# bin 81  chi2/NDF: 0.79162
[ 2.0625,   7.3122e-03,   1.0176e-08,   -5.0974e-02  ],# bin 82  chi2/NDF: 3.083
[ 2.0875,   9.7697e-03,   5.4311e-09,   -8.1063e-02  ],# bin 83  chi2/NDF: 3.2717
[ 2.1125,   4.4747e-03,   9.4034e-09,   -1.2658e-02  ],# bin 84  chi2/NDF: 2.0759
[ 2.1375,   5.3588e-03,   9.9676e-09,   -2.8776e-02  ],# bin 85  chi2/NDF: 2.459
[ 2.1625,   1.9193e-03,   8.8096e-09,   1.2366e-02  ],# bin 86  chi2/NDF: 0.67271
[ 2.1875,   5.8296e-03,   5.1916e-09,   -3.7368e-02  ],# bin 87  chi2/NDF: 2.1745
[ 2.2125,   6.5602e-03,   1.1708e-09,   -4.3373e-02  ],# bin 88  chi2/NDF: 2.2769
[ 2.2375,   4.4691e-03,   5.7159e-09,   -1.9957e-02  ],# bin 89  chi2/NDF: 2.0533
[ 2.2625,   2.4011e-03,   6.6177e-09,   4.5309e-03  ],# bin 90  chi2/NDF: 1.6338
[ 2.2875,   3.2978e-03,   3.7904e-09,   -6.2876e-03  ],# bin 91  chi2/NDF: 1.3037
[ 2.3125,   -2.7530e-04,   2.3338e-09,   3.7219e-02  ],# bin 92  chi2/NDF: 1.8793
[ 2.3375,   -7.7354e-03,   7.2716e-09,   1.3512e-01  ],# bin 93  chi2/NDF: 0.73617
[ 2.3625,   -6.0771e-03,   5.2183e-09,   1.1894e-01  ],# bin 94  chi2/NDF: 0.87004
[ 2.3875,   -5.8575e-03,   2.0483e-09,   1.3132e-01  ],# bin 95  chi2/NDF: 3.2163
[ 2.4125,   -9.1013e-03,   8.8068e-09,   1.8049e-01  ],# bin 96  chi2/NDF: 2.0069
[ 2.4375,   -1.3008e-02,   8.7912e-09,   2.6298e-01  ],# bin 97  chi2/NDF: 1.8509
[ 2.4625,   -2.2383e-02,   1.2880e-08,   4.5896e-01  ],# bin 98  chi2/NDF: 0.5906
[ 2.4875,   -2.2383e-02,   1.2880e-08,   4.5896e-01  ],# bin 99  chi2/NDF: 0.5906, copied from bin 98
] # end CaloSwCalHitsLongLeakage_v8_gam_conv


CaloSwCalHitsLongLeakage_v8_gam_unconv = [
[ 0.0125,   9.6608e-02,   1.2930e-06,   -5.6329e-01  ],# bin 0  chi2/NDF: 3.5602, copied from bin 2
[ 0.0375,   9.6608e-02,   1.2930e-06,   -5.6329e-01  ],# bin 1  chi2/NDF: 7.3931, copied from bin 2
[ 0.0625,   9.6608e-02,   1.2930e-06,   -5.6329e-01  ],# bin 2  chi2/NDF: 11.015
[ 0.0875,   9.9409e-02,   1.2063e-06,   -6.3509e-01  ],# bin 3  chi2/NDF: 9.7436
[ 0.1125,   1.0089e-01,   1.1525e-06,   -6.7153e-01  ],# bin 4  chi2/NDF: 7.2226
[ 0.1375,   9.6428e-02,   1.1051e-06,   -6.3185e-01  ],# bin 5  chi2/NDF: 5.7612
[ 0.1625,   9.2676e-02,   1.0564e-06,   -6.0414e-01  ],# bin 6  chi2/NDF: 23.92
[ 0.1875,   9.0124e-02,   1.0384e-06,   -5.9052e-01  ],# bin 7  chi2/NDF: 8.1235
[ 0.2125,   8.8295e-02,   9.4222e-07,   -5.8110e-01  ],# bin 8  chi2/NDF: 6.7432
[ 0.2375,   8.8057e-02,   8.5190e-07,   -5.8985e-01  ],# bin 9  chi2/NDF: 6.8457
[ 0.2625,   8.3970e-02,   7.8619e-07,   -5.5687e-01  ],# bin 10  chi2/NDF: 5.8065
[ 0.2875,   8.1636e-02,   6.9661e-07,   -5.4784e-01  ],# bin 11  chi2/NDF: 4.7927
[ 0.3125,   7.5461e-02,   6.2344e-07,   -5.0085e-01  ],# bin 12  chi2/NDF: 6.539
[ 0.3375,   7.6651e-02,   5.2580e-07,   -5.2800e-01  ],# bin 13  chi2/NDF: 23.26
[ 0.3625,   6.9946e-02,   4.6090e-07,   -4.7808e-01  ],# bin 14  chi2/NDF: 6.7273
[ 0.3875,   6.1142e-02,   4.1550e-07,   -4.0048e-01  ],# bin 15  chi2/NDF: 5.6234
[ 0.4125,   6.7552e-02,   2.9863e-07,   -4.8659e-01  ],# bin 16  chi2/NDF: 52.548
[ 0.4375,   5.6927e-02,   2.6996e-07,   -3.9104e-01  ],# bin 17  chi2/NDF: 6.2494
[ 0.4625,   5.5817e-02,   2.0925e-07,   -3.9641e-01  ],# bin 18  chi2/NDF: 25.029
[ 0.4875,   5.0353e-02,   1.7372e-07,   -3.5448e-01  ],# bin 19  chi2/NDF: 18.181
[ 0.5125,   4.5134e-02,   1.3743e-07,   -3.1394e-01  ],# bin 20  chi2/NDF: 12.308
[ 0.5375,   3.9700e-02,   1.2610e-07,   -2.6489e-01  ],# bin 21  chi2/NDF: 10.696
[ 0.5625,   3.7831e-02,   1.1522e-07,   -2.5962e-01  ],# bin 22  chi2/NDF: 15.176
[ 0.5875,   3.5003e-02,   1.0038e-07,   -2.3749e-01  ],# bin 23  chi2/NDF: 15.289
[ 0.6125,   2.2758e-02,   1.1168e-07,   -1.0842e-01  ],# bin 24  chi2/NDF: 193.04
[ 0.6375,   2.7095e-02,   8.5460e-08,   -1.5801e-01  ],# bin 25  chi2/NDF: 8.1502
[ 0.6625,   2.6108e-02,   7.7310e-08,   -1.5211e-01  ],# bin 26  chi2/NDF: 13.615
[ 0.6875,   2.4956e-02,   6.8738e-08,   -1.4511e-01  ],# bin 27  chi2/NDF: 14.959
[ 0.7125,   2.1992e-02,   6.2446e-08,   -1.1662e-01  ],# bin 28  chi2/NDF: 9.7583
[ 0.7375,   2.1226e-02,   5.4907e-08,   -1.1321e-01  ],# bin 29  chi2/NDF: 13.426
[ 0.7625,   2.1226e-02,   5.4907e-08,   -1.1321e-01  ],# bin 30  chi2/NDF: 4.7296, copied from bin 29
[ 0.7875,   2.1226e-02,   5.4907e-08,   -1.1321e-01  ],# bin 31  chi2/NDF: 4.2895, copied from bin 29
[ 0.8125,   2.6034e-02,   1.0759e-07,   -1.1216e-01  ],# bin 32  chi2/NDF: 5.737, copied from bin 34
[ 0.8375,   2.6034e-02,   1.0759e-07,   -1.1216e-01  ],# bin 33  chi2/NDF: 5.6118, copied from bin 34
[ 0.8625,   2.6034e-02,   1.0759e-07,   -1.1216e-01  ],# bin 34  chi2/NDF: 12.42
[ 0.8875,   2.5348e-02,   9.7840e-08,   -1.0872e-01  ],# bin 35  chi2/NDF: 8.2389
[ 0.9125,   2.3636e-02,   8.3038e-08,   -9.1237e-02  ],# bin 36  chi2/NDF: 8.6408
[ 0.9375,   2.0045e-02,   7.3766e-08,   -5.3903e-02  ],# bin 37  chi2/NDF: 11.021
[ 0.9625,   2.0181e-02,   6.5386e-08,   -6.5145e-02  ],# bin 38  chi2/NDF: 10.533
[ 0.9875,   1.8811e-02,   5.7307e-08,   -5.8142e-02  ],# bin 39  chi2/NDF: 8.5522
[ 1.0125,   1.3172e-02,   5.5714e-08,   3.4002e-03  ],# bin 40  chi2/NDF: 12.727
[ 1.0375,   3.4083e-03,   6.5601e-08,   1.0799e-01  ],# bin 41  chi2/NDF: 73.717
[ 1.0625,   1.1839e-02,   3.9084e-08,   9.2873e-03  ],# bin 42  chi2/NDF: 8.4596
[ 1.0875,   1.0121e-02,   3.5682e-08,   2.7048e-02  ],# bin 43  chi2/NDF: 4.9679
[ 1.1125,   6.3886e-03,   3.3693e-08,   6.5857e-02  ],# bin 44  chi2/NDF: 9.6513
[ 1.1375,   7.6532e-03,   2.6376e-08,   4.7916e-02  ],# bin 45  chi2/NDF: 5.749
[ 1.1625,   1.7187e-03,   2.8981e-08,   1.1502e-01  ],# bin 46  chi2/NDF: 4.4453
[ 1.1875,   -3.3953e-03,   2.9470e-08,   1.7812e-01  ],# bin 47  chi2/NDF: 3.1062
[ 1.2125,   -3.4648e-03,   2.1068e-08,   1.7999e-01  ],# bin 48  chi2/NDF: 4.129
[ 1.2375,   -5.7513e-03,   2.0548e-08,   2.1236e-01  ],# bin 49  chi2/NDF: 2.9094
[ 1.2625,   -7.6911e-03,   2.4133e-08,   2.4666e-01  ],# bin 50  chi2/NDF: 6.8004
[ 1.2875,   -6.2042e-03,   3.2597e-08,   2.5939e-01  ],# bin 51  chi2/NDF: 3.7724
[ 1.3125,   1.8412e-03,   6.7507e-08,   2.4958e-01  ],# bin 52  chi2/NDF: 2.4706
[ 1.3375,   2.7181e-02,   2.0940e-07,   1.6412e-01  ],# bin 53  chi2/NDF: 1.5722
[ 1.3625,   9.9566e-02,   6.0101e-07,   -1.5249e-01  ],# bin 54  chi2/NDF: 1.8916
[ 1.3875,   3.4241e-01,   1.9251e-06,   -8.5671e-01  ],# bin 55  chi2/NDF: 15.866
[ 1.4125,   7.0854e-01,   3.7710e-06,   -3.1903e-01  ],# bin 56  chi2/NDF: 9.3054
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -2.6300e-02,   1.7844e-09,   6.8287e-01  ],# bin 60 crack  chi2/NDF: 2.3434
[ 1.5375,   -4.1788e-03,   1.8807e-09,   2.4458e-01  ],# bin 61 crack  chi2/NDF: 9.8875
[ 1.5625,   1.8069e-02,   1.3837e-09,   -1.2242e-01  ],# bin 62  chi2/NDF: 4.453
[ 1.5875,   1.7946e-02,   6.3389e-09,   -1.0658e-01  ],# bin 63  chi2/NDF: 3.9307
[ 1.6125,   1.6150e-02,   6.7164e-09,   -9.1973e-02  ],# bin 64  chi2/NDF: 5.4053
[ 1.6375,   1.4082e-02,   7.2120e-09,   -5.9664e-02  ],# bin 65  chi2/NDF: 4.0036
[ 1.6625,   1.4412e-02,   4.1621e-09,   -7.6373e-02  ],# bin 66  chi2/NDF: 2.9043
[ 1.6875,   1.3220e-02,   2.7106e-09,   -6.8893e-02  ],# bin 67  chi2/NDF: 3.7237
[ 1.7125,   1.2784e-02,   1.9741e-09,   -8.6712e-02  ],# bin 68  chi2/NDF: 4.1749
[ 1.7375,   1.2526e-02,   5.3996e-09,   -8.0282e-02  ],# bin 69  chi2/NDF: 4.8085
[ 1.7625,   1.1296e-02,   6.8255e-09,   -6.8728e-02  ],# bin 70  chi2/NDF: 10.666
[ 1.7875,   7.9159e-03,   1.1729e-08,   -3.4991e-02  ],# bin 71  chi2/NDF: 45.692
[ 1.8125,   1.2363e-02,   8.7905e-09,   -1.0085e-01  ],# bin 72  chi2/NDF: 8.5432
[ 1.8375,   1.2067e-02,   1.7463e-08,   -1.0051e-01  ],# bin 73  chi2/NDF: 10.106
[ 1.8625,   1.2194e-02,   2.1049e-08,   -1.0003e-01  ],# bin 74  chi2/NDF: 13.576
[ 1.8875,   1.1668e-02,   2.1456e-08,   -9.6564e-02  ],# bin 75  chi2/NDF: 16.065
[ 1.9125,   9.8641e-03,   1.9364e-08,   -7.6170e-02  ],# bin 76  chi2/NDF: 22.776
[ 1.9375,   8.8113e-03,   1.7152e-08,   -6.3744e-02  ],# bin 77  chi2/NDF: 22.373
[ 1.9625,   9.7801e-03,   1.3273e-08,   -7.9819e-02  ],# bin 78  chi2/NDF: 7.5494
[ 1.9875,   6.2285e-03,   1.3572e-08,   -3.8855e-02  ],# bin 79  chi2/NDF: 39.341
[ 2.0125,   6.8849e-03,   7.4403e-09,   -4.6507e-02  ],# bin 80  chi2/NDF: 6.7153
[ 2.0375,   8.4248e-03,   6.6806e-09,   -7.1876e-02  ],# bin 81  chi2/NDF: 7.2154
[ 2.0625,   8.7023e-03,   4.6591e-09,   -8.0505e-02  ],# bin 82  chi2/NDF: 5.0246
[ 2.0875,   7.8652e-03,   3.0232e-09,   -7.1222e-02  ],# bin 83  chi2/NDF: 6.0708
[ 2.1125,   9.1837e-03,   4.2259e-09,   -9.0505e-02  ],# bin 84  chi2/NDF: 12.902
[ 2.1375,   7.3478e-03,   4.3556e-09,   -6.7934e-02  ],# bin 85  chi2/NDF: 8.3602
[ 2.1625,   7.7446e-03,   2.8231e-09,   -7.4960e-02  ],# bin 86  chi2/NDF: 3.1576
[ 2.1875,   4.0324e-03,   3.4086e-09,   -2.8438e-02  ],# bin 87  chi2/NDF: 6.9766
[ 2.2125,   6.2528e-03,   1.9422e-09,   -5.9161e-02  ],# bin 88  chi2/NDF: 5.2586
[ 2.2375,   5.9257e-03,   3.3554e-09,   -5.4521e-02  ],# bin 89  chi2/NDF: 3.8204
[ 2.2625,   4.7624e-03,   2.6504e-09,   -4.1063e-02  ],# bin 90  chi2/NDF: 6.0422
[ 2.2875,   5.8206e-03,   1.8066e-09,   -5.6867e-02  ],# bin 91  chi2/NDF: 2.6604
[ 2.3125,   3.4230e-03,   1.3957e-09,   -2.7836e-02  ],# bin 92  chi2/NDF: 3.031
[ 2.3375,   3.9482e-03,   1.9590e-09,   -3.5167e-02  ],# bin 93  chi2/NDF: 1.9648
[ 2.3625,   2.5016e-03,   2.0513e-09,   -1.6281e-02  ],# bin 94  chi2/NDF: 4.0853
[ 2.3875,   2.8828e-03,   3.2225e-09,   -1.2901e-02  ],# bin 95  chi2/NDF: 1.6059
[ 2.4125,   1.2561e-03,   3.6699e-09,   1.7070e-02  ],# bin 96  chi2/NDF: 1.8415
[ 2.4375,   3.5814e-03,   4.0301e-09,   1.2685e-02  ],# bin 97  chi2/NDF: 0.86972
[ 2.4625,   -6.9499e-04,   3.7517e-09,   1.3070e-01  ],# bin 98  chi2/NDF: 1.2628
[ 2.4875,   -6.9499e-04,   3.7517e-09,   1.3070e-01  ],# bin 99  chi2/NDF: 1.2628, copied from bin 98
] # end CaloSwCalHitsLongLeakage_v8_gam_unconv


CaloSwCalHitsEfront_slope_v8_ele = [
[ 0.0125,   7.2992e-01,   3.2605e-01,   -3.3898e-03  ],# bin 0  chi2/NDF: 1.1578, copied from bin 2
[ 0.0375,   7.2992e-01,   3.2605e-01,   -3.3898e-03  ],# bin 1  chi2/NDF: 2.5089, copied from bin 2
[ 0.0625,   7.2992e-01,   3.2605e-01,   -3.3898e-03  ],# bin 2  chi2/NDF: 3.2289
[ 0.0875,   7.3172e-01,   3.1360e-01,   -1.9316e-03  ],# bin 3  chi2/NDF: 3.351
[ 0.1125,   7.3074e-01,   3.1896e-01,   -2.9415e-03  ],# bin 4  chi2/NDF: 3.6097
[ 0.1375,   7.4135e-01,   2.9767e-01,   -1.5232e-03  ],# bin 5  chi2/NDF: 4.0262
[ 0.1625,   7.2034e-01,   3.1784e-01,   -1.7951e-03  ],# bin 6  chi2/NDF: 2.217
[ 0.1875,   7.1242e-01,   3.2722e-01,   -3.2658e-03  ],# bin 7  chi2/NDF: 4.0317
[ 0.2125,   7.0931e-01,   2.9653e-01,   -5.4455e-04  ],# bin 8  chi2/NDF: 3.072
[ 0.2375,   7.0040e-01,   3.2569e-01,   -3.3020e-03  ],# bin 9  chi2/NDF: 2.8892
[ 0.2625,   7.2020e-01,   2.8356e-01,   -3.8785e-04  ],# bin 10  chi2/NDF: 2.2321
[ 0.2875,   6.6714e-01,   3.4128e-01,   -2.3493e-03  ],# bin 11  chi2/NDF: 4.2533
[ 0.3125,   7.2478e-01,   2.6906e-01,   5.2988e-04  ],# bin 12  chi2/NDF: 1.0751
[ 0.3375,   6.8409e-01,   3.1979e-01,   -8.0813e-04  ],# bin 13  chi2/NDF: 6.916
[ 0.3625,   7.0678e-01,   3.0454e-01,   -1.2642e-03  ],# bin 14  chi2/NDF: 3.1391
[ 0.3875,   6.8044e-01,   3.2685e-01,   -1.5934e-03  ],# bin 15  chi2/NDF: 5.6579
[ 0.4125,   6.6846e-01,   3.5035e-01,   -3.6100e-03  ],# bin 16  chi2/NDF: 2.717
[ 0.4375,   6.7104e-01,   3.4087e-01,   -3.3472e-03  ],# bin 17  chi2/NDF: 2.1866
[ 0.4625,   6.8160e-01,   3.2336e-01,   -1.6553e-03  ],# bin 18  chi2/NDF: 3.5978
[ 0.4875,   6.7845e-01,   3.3422e-01,   -2.2417e-03  ],# bin 19  chi2/NDF: 2.3802
[ 0.5125,   6.8996e-01,   3.1270e-01,   -1.3099e-03  ],# bin 20  chi2/NDF: 4.7216
[ 0.5375,   6.6638e-01,   3.3909e-01,   -9.6518e-04  ],# bin 21  chi2/NDF: 3.9982
[ 0.5625,   7.3021e-01,   2.5594e-01,   1.3603e-03  ],# bin 22  chi2/NDF: 9.4182
[ 0.5875,   7.0063e-01,   3.1305e-01,   -1.4258e-03  ],# bin 23  chi2/NDF: 4.6752
[ 0.6125,   6.7601e-01,   3.4763e-01,   -3.0813e-03  ],# bin 24  chi2/NDF: 7.0972
[ 0.6375,   7.2357e-01,   3.3771e-01,   -6.0166e-03  ],# bin 25  chi2/NDF: 8.4795
[ 0.6625,   7.5433e-01,   3.2512e-01,   -6.0713e-03  ],# bin 26  chi2/NDF: 6.1183
[ 0.6875,   7.6548e-01,   3.3579e-01,   -8.1204e-03  ],# bin 27  chi2/NDF: 11.81
[ 0.7125,   7.9629e-01,   2.9765e-01,   -5.4957e-03  ],# bin 28  chi2/NDF: 10.083
[ 0.7375,   8.0697e-01,   3.0588e-01,   -6.3040e-03  ],# bin 29  chi2/NDF: 5.0009
[ 0.7625,   8.0697e-01,   3.0588e-01,   -6.3040e-03  ],# bin 30  chi2/NDF: 8.024, copied from bin 29
[ 0.7875,   8.0697e-01,   3.0588e-01,   -6.3040e-03  ],# bin 31  chi2/NDF: 4.204, copied from bin 29
[ 0.8125,   7.8122e-01,   3.9472e-01,   -1.2072e-02  ],# bin 32  chi2/NDF: 5.9647, copied from bin 34
[ 0.8375,   7.8122e-01,   3.9472e-01,   -1.2072e-02  ],# bin 33  chi2/NDF: 9.7644, copied from bin 34
[ 0.8625,   7.8122e-01,   3.9472e-01,   -1.2072e-02  ],# bin 34  chi2/NDF: 14.333
[ 0.8875,   7.9335e-01,   3.7748e-01,   -1.1240e-02  ],# bin 35  chi2/NDF: 7.5112
[ 0.9125,   7.4975e-01,   4.2695e-01,   -1.3459e-02  ],# bin 36  chi2/NDF: 10.375
[ 0.9375,   7.5266e-01,   4.2651e-01,   -1.3526e-02  ],# bin 37  chi2/NDF: 12.112
[ 0.9625,   7.4596e-01,   4.2819e-01,   -1.3375e-02  ],# bin 38  chi2/NDF: 8.7615
[ 0.9875,   7.5987e-01,   4.1173e-01,   -1.2481e-02  ],# bin 39  chi2/NDF: 7.2449
[ 1.0125,   7.3878e-01,   4.2700e-01,   -1.2433e-02  ],# bin 40  chi2/NDF: 7.6001
[ 1.0375,   7.1063e-01,   4.6599e-01,   -1.4796e-02  ],# bin 41  chi2/NDF: 6.9996
[ 1.0625,   6.8131e-01,   4.9018e-01,   -1.4698e-02  ],# bin 42  chi2/NDF: 13.098
[ 1.0875,   7.6134e-01,   4.0985e-01,   -1.2135e-02  ],# bin 43  chi2/NDF: 4.7132
[ 1.1125,   7.3114e-01,   4.2178e-01,   -1.1436e-02  ],# bin 44  chi2/NDF: 16.333
[ 1.1375,   6.8648e-01,   4.6494e-01,   -1.2338e-02  ],# bin 45  chi2/NDF: 11.211
[ 1.1625,   7.1666e-01,   4.4629e-01,   -1.3054e-02  ],# bin 46  chi2/NDF: 10.48
[ 1.1875,   7.0761e-01,   4.3568e-01,   -1.1485e-02  ],# bin 47  chi2/NDF: 12.108
[ 1.2125,   6.9193e-01,   4.3051e-01,   -1.1790e-02  ],# bin 48  chi2/NDF: 9.9623
[ 1.2375,   6.5762e-01,   4.7124e-01,   -1.4030e-02  ],# bin 49  chi2/NDF: 10.779
[ 1.2625,   6.8098e-01,   4.3356e-01,   -1.1669e-02  ],# bin 50  chi2/NDF: 4.9277
[ 1.2875,   6.5231e-01,   4.6957e-01,   -1.3122e-02  ],# bin 51  chi2/NDF: 7.1102
[ 1.3125,   6.7163e-01,   4.5320e-01,   -1.2783e-02  ],# bin 52  chi2/NDF: 6.3206
[ 1.3375,   7.0454e-01,   4.1703e-01,   -1.0666e-02  ],# bin 53  chi2/NDF: 10.825
[ 1.3625,   6.6673e-01,   4.8067e-01,   -1.5365e-02  ],# bin 54  chi2/NDF: 6.3216
[ 1.3875,   6.4972e-01,   5.1319e-01,   -1.7586e-02  ],# bin 55  chi2/NDF: 3.321
[ 1.4125,   6.6201e-01,   4.9857e-01,   -1.5615e-02  ],# bin 56  chi2/NDF: 4.2913
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -2.5552e+00,   9.1734e-01,   -3.7770e-01  ],# bin 60 crack  chi2/NDF: 6.8093
[ 1.5375,   8.4256e+00,   -1.9225e+00,   1.6142e-01  ],# bin 61 crack  chi2/NDF: 17.362
[ 1.5625,   1.9034e+00,   -2.0532e-01,   1.4605e-02  ],# bin 62  chi2/NDF: 6.2939
[ 1.5875,   1.9451e+00,   -1.9376e-01,   1.1036e-02  ],# bin 63  chi2/NDF: 1.3469
[ 1.6125,   1.3699e+00,   -5.5029e-03,   -1.3164e-02  ],# bin 64  chi2/NDF: 5.7608
[ 1.6375,   2.3128e+00,   -4.5931e-01,   5.7059e-02  ],# bin 65  chi2/NDF: 7.54
[ 1.6625,   4.3955e+00,   -1.0274e+00,   1.1375e-01  ],# bin 66  chi2/NDF: 5.8805
[ 1.6875,   5.9344e+00,   -1.2797e+00,   1.2284e-01  ],# bin 67  chi2/NDF: 9.1157
[ 1.7125,   1.0386e+00,   -1.5517e-01,   3.0644e-02  ],# bin 68  chi2/NDF: 3.9793
[ 1.7375,   -1.8339e-01,   1.8422e-01,   -6.1046e-03  ],# bin 69  chi2/NDF: 3.8288
[ 1.7625,   -5.5529e-01,   3.0290e-01,   -1.9753e-02  ],# bin 70  chi2/NDF: 3.1256
[ 1.7875,   6.1085e-01,   3.1329e-02,   2.1180e-03  ],# bin 71  chi2/NDF: 2.6989
[ 1.8125,   3.2420e+02,   -1.1641e+02,   -9.6661e-03  ],# bin 72  chi2/NDF: 0.32016
[ 1.8375,   -1.6582e+02,   -1.2355e+02,   8.3610e-03  ],# bin 73  chi2/NDF: 0.67831
[ 1.8625,   -2.2871e+02,   -1.0631e+02,   6.6238e-02  ],# bin 74  chi2/NDF: 1.2956
[ 1.8875,   -7.3967e+02,   -1.0369e+02,   5.1454e-02  ],# bin 75  chi2/NDF: 0.47393
[ 1.9125,   -8.2031e+02,   -9.3629e+01,   1.8830e-02  ],# bin 76  chi2/NDF: 1.6681
[ 1.9375,   -1.4984e+02,   -9.5910e+01,   4.3179e-02  ],# bin 77  chi2/NDF: 0.97426
[ 1.9625,   -6.7646e+02,   -7.9234e+01,   2.0473e-02  ],# bin 78  chi2/NDF: 0.63551
[ 1.9875,   -1.0070e+03,   -8.3519e+01,   3.2591e-02  ],# bin 79  chi2/NDF: 0.59851
[ 2.0125,   -3.9656e+02,   -9.2087e+01,   8.9244e-03  ],# bin 80  chi2/NDF: 0.43737
[ 2.0375,   -7.3331e+02,   -9.1282e+01,   2.0564e-02  ],# bin 81  chi2/NDF: 1.5167
[ 2.0625,   -1.1399e+03,   -7.9393e+01,   1.9529e-02  ],# bin 82  chi2/NDF: 1.8841
[ 2.0875,   -7.7242e+02,   -9.4192e+01,   3.5186e-02  ],# bin 83  chi2/NDF: 2.0705
[ 2.1125,   -6.1153e+02,   -9.1997e+01,   1.6851e-02  ],# bin 84  chi2/NDF: 0.30563
[ 2.1375,   -5.5261e+02,   -8.9596e+01,   3.2457e-02  ],# bin 85  chi2/NDF: 1.7277
[ 2.1625,   -3.7636e+02,   -8.7898e+01,   2.9525e-02  ],# bin 86  chi2/NDF: 2.1896
[ 2.1875,   -5.5555e+02,   -8.5541e+01,   3.5069e-02  ],# bin 87  chi2/NDF: 0.38907
[ 2.2125,   -6.0220e+02,   -7.5361e+01,   2.0910e-02  ],# bin 88  chi2/NDF: 1.3735
[ 2.2375,   -5.4578e+02,   -8.0773e+01,   4.3183e-02  ],# bin 89  chi2/NDF: 4.2898
[ 2.2625,   -6.4336e+02,   -6.3739e+01,   8.9583e-03  ],# bin 90  chi2/NDF: 0.51672
[ 2.2875,   -5.5874e+02,   -5.8665e+01,   7.4530e-03  ],# bin 91  chi2/NDF: 2.1533
[ 2.3125,   -1.1347e+01,   -3.2721e+01,   4.6806e-02  ],# bin 92  chi2/NDF: 2.5147
[ 2.3375,   6.7468e+02,   -1.2166e+02,   5.4155e-02  ],# bin 93  chi2/NDF: 0.79213
[ 2.3625,   -6.9510e+01,   -9.6333e+01,   -1.4812e-02  ],# bin 94  chi2/NDF: 0.39459
[ 2.3875,   5.1678e+02,   -1.1656e+02,   2.4468e-02  ],# bin 95  chi2/NDF: 0.67809
[ 2.4125,   1.7345e+03,   -1.2842e+02,   5.8043e-02  ],# bin 96  chi2/NDF: 0.29836
[ 2.4375,   -5.0386e+02,   -1.0831e+02,   1.8168e-02  ],# bin 97  chi2/NDF: 0.53531
[ 2.4625,   2.9575e+02,   -1.2020e+02,   3.4624e-02  ],# bin 98  chi2/NDF: 0.44513
[ 2.4875,   2.9575e+02,   -1.2020e+02,   3.4624e-02  ],# bin 99 missing par  chi2/NDF: 0, copied from bin 98
] # end CaloSwCalHitsEfront_slope_v8_ele


CaloSwCalHitsEfront_slope_v8_gam_conv = [
[ 0.0125,   5.3081e-01,   5.6928e-01,   -1.2067e-02  ],# bin 0  chi2/NDF: 1.8952, copied from bin 2
[ 0.0375,   5.3081e-01,   5.6928e-01,   -1.2067e-02  ],# bin 1  chi2/NDF: 2.3856, copied from bin 2
[ 0.0625,   5.3081e-01,   5.6928e-01,   -1.2067e-02  ],# bin 2  chi2/NDF: 1.2245
[ 0.0875,   4.7364e-01,   7.0338e-01,   -2.4425e-02  ],# bin 3  chi2/NDF: 0.85248
[ 0.1125,   4.7316e-01,   6.4881e-01,   -1.3734e-02  ],# bin 4  chi2/NDF: 2.043
[ 0.1375,   4.4769e-01,   7.1716e-01,   -1.9816e-02  ],# bin 5  chi2/NDF: 2.7334
[ 0.1625,   4.7679e-01,   6.0861e-01,   -8.7450e-03  ],# bin 6  chi2/NDF: 3.5886
[ 0.1875,   4.4608e-01,   6.7472e-01,   -1.4646e-02  ],# bin 7  chi2/NDF: 3.0374
[ 0.2125,   4.8158e-01,   5.8547e-01,   -8.7439e-03  ],# bin 8  chi2/NDF: 2.6572
[ 0.2375,   3.8966e-01,   8.0867e-01,   -2.4707e-02  ],# bin 9  chi2/NDF: 0.94693
[ 0.2625,   4.3764e-01,   6.8862e-01,   -1.6798e-02  ],# bin 10  chi2/NDF: 4.0885
[ 0.2875,   4.7681e-01,   5.6035e-01,   -5.5093e-03  ],# bin 11  chi2/NDF: 7.604
[ 0.3125,   4.0161e-01,   7.3631e-01,   -1.5938e-02  ],# bin 12  chi2/NDF: 4.0229
[ 0.3375,   4.6483e-01,   5.9895e-01,   -8.7355e-03  ],# bin 13  chi2/NDF: 4.0122
[ 0.3625,   4.2114e-01,   7.0178e-01,   -1.4970e-02  ],# bin 14  chi2/NDF: 1.7701
[ 0.3875,   4.7651e-01,   6.0438e-01,   -1.1976e-02  ],# bin 15  chi2/NDF: 3.4794
[ 0.4125,   4.3269e-01,   6.5079e-01,   -1.1139e-02  ],# bin 16  chi2/NDF: 5.0647
[ 0.4375,   5.1684e-01,   4.8438e-01,   -2.4336e-03  ],# bin 17  chi2/NDF: 3.119
[ 0.4625,   4.9437e-01,   5.2796e-01,   -5.0092e-03  ],# bin 18  chi2/NDF: 1.9152
[ 0.4875,   4.2678e-01,   6.7010e-01,   -1.1280e-02  ],# bin 19  chi2/NDF: 2.0647
[ 0.5125,   4.4432e-01,   6.3318e-01,   -1.0172e-02  ],# bin 20  chi2/NDF: 6.1405
[ 0.5375,   4.5602e-01,   6.1623e-01,   -9.1264e-03  ],# bin 21  chi2/NDF: 4.1429
[ 0.5625,   4.7732e-01,   5.8062e-01,   -9.0322e-03  ],# bin 22  chi2/NDF: 1.8621
[ 0.5875,   4.7495e-01,   5.8126e-01,   -7.4099e-03  ],# bin 23  chi2/NDF: 3.0524
[ 0.6125,   4.5468e-01,   6.4415e-01,   -1.3041e-02  ],# bin 24  chi2/NDF: 6.4863
[ 0.6375,   5.4221e-01,   5.3010e-01,   -1.0047e-02  ],# bin 25  chi2/NDF: 7.8627
[ 0.6625,   5.6052e-01,   5.3672e-01,   -1.2511e-02  ],# bin 26  chi2/NDF: 1.8516
[ 0.6875,   5.0952e-01,   6.2343e-01,   -1.5011e-02  ],# bin 27  chi2/NDF: 4.2316
[ 0.7125,   5.3172e-01,   6.0227e-01,   -1.5871e-02  ],# bin 28  chi2/NDF: 1.9172
[ 0.7375,   5.7168e-01,   5.5264e-01,   -1.3580e-02  ],# bin 29  chi2/NDF: 2.4328
[ 0.7625,   5.7168e-01,   5.5264e-01,   -1.3580e-02  ],# bin 30  chi2/NDF: 5.1011, copied from bin 29
[ 0.7875,   5.7168e-01,   5.5264e-01,   -1.3580e-02  ],# bin 31  chi2/NDF: 0.99435, copied from bin 29
[ 0.8125,   7.1683e-01,   4.4963e-01,   -1.3238e-02  ],# bin 32  chi2/NDF: 0.84989, copied from bin 34
[ 0.8375,   7.1683e-01,   4.4963e-01,   -1.3238e-02  ],# bin 33  chi2/NDF: 6.2879, copied from bin 34
[ 0.8625,   7.1683e-01,   4.4963e-01,   -1.3238e-02  ],# bin 34  chi2/NDF: 1.8124
[ 0.8875,   6.7905e-01,   5.0290e-01,   -1.5936e-02  ],# bin 35  chi2/NDF: 6.3409
[ 0.9125,   6.5598e-01,   5.0916e-01,   -1.4135e-02  ],# bin 36  chi2/NDF: 5.0804
[ 0.9375,   6.6342e-01,   5.0142e-01,   -1.4033e-02  ],# bin 37  chi2/NDF: 8.5252
[ 0.9625,   7.0920e-01,   4.5213e-01,   -1.2372e-02  ],# bin 38  chi2/NDF: 4.8313
[ 0.9875,   5.7018e-01,   6.2525e-01,   -1.9407e-02  ],# bin 39  chi2/NDF: 4.8929
[ 1.0125,   6.2471e-01,   5.5412e-01,   -1.6782e-02  ],# bin 40  chi2/NDF: 5.7727
[ 1.0375,   6.2813e-01,   5.3803e-01,   -1.5376e-02  ],# bin 41  chi2/NDF: 6.7964
[ 1.0625,   5.8679e-01,   5.7154e-01,   -1.4269e-02  ],# bin 42  chi2/NDF: 5.1941
[ 1.0875,   6.4251e-01,   5.0614e-01,   -1.2643e-02  ],# bin 43  chi2/NDF: 3.5269
[ 1.1125,   6.0105e-01,   5.3763e-01,   -1.2224e-02  ],# bin 44  chi2/NDF: 4.0064
[ 1.1375,   5.7494e-01,   5.6305e-01,   -1.2626e-02  ],# bin 45  chi2/NDF: 2.7405
[ 1.1625,   6.1239e-01,   5.2726e-01,   -1.2038e-02  ],# bin 46  chi2/NDF: 3.6786
[ 1.1875,   5.6541e-01,   5.9345e-01,   -1.5874e-02  ],# bin 47  chi2/NDF: 8.4591
[ 1.2125,   5.9258e-01,   5.2418e-01,   -1.2597e-02  ],# bin 48  chi2/NDF: 8.8426
[ 1.2375,   5.4243e-01,   5.9303e-01,   -1.6722e-02  ],# bin 49  chi2/NDF: 5.1008
[ 1.2625,   4.7016e-01,   6.9606e-01,   -1.8695e-02  ],# bin 50  chi2/NDF: 10.365
[ 1.2875,   4.9332e-01,   6.5759e-01,   -1.7616e-02  ],# bin 51  chi2/NDF: 11.56
[ 1.3125,   5.6171e-01,   5.6258e-01,   -1.4171e-02  ],# bin 52  chi2/NDF: 4.6104
[ 1.3375,   5.0671e-01,   6.4307e-01,   -1.6759e-02  ],# bin 53  chi2/NDF: 7.603
[ 1.3625,   4.4763e-01,   7.8225e-01,   -2.5814e-02  ],# bin 54  chi2/NDF: 8.7292
[ 1.3875,   5.4322e-01,   6.5255e-01,   -2.2819e-02  ],# bin 55  chi2/NDF: 8.0289
[ 1.4125,   4.3208e-01,   8.9670e-01,   -4.3593e-02  ],# bin 56  chi2/NDF: 9.3803
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack missing par  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack missing par  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -2.8528e+00,   1.0212e+00,   -3.6347e-01  ],# bin 60 crack  chi2/NDF: 2.3268
[ 1.5375,   3.5790e+00,   -5.3300e-01,   -9.7976e-03  ],# bin 61 crack  chi2/NDF: 11.915
[ 1.5625,   1.2969e+00,   -7.8294e-02,   5.5692e-03  ],# bin 62  chi2/NDF: 3.4088
[ 1.5875,   1.4325e+00,   -1.0810e-01,   1.0331e-02  ],# bin 63  chi2/NDF: 4.3453
[ 1.6125,   1.7738e+00,   -1.7173e-01,   9.9864e-03  ],# bin 64  chi2/NDF: 4.843
[ 1.6375,   2.1436e+00,   -4.1739e-01,   5.1588e-02  ],# bin 65  chi2/NDF: 4.2978
[ 1.6625,   3.5104e+00,   -7.4314e-01,   7.7167e-02  ],# bin 66  chi2/NDF: 5.7059
[ 1.6875,   4.7045e+00,   -8.4230e-01,   6.0106e-02  ],# bin 67  chi2/NDF: 6.3337
[ 1.7125,   8.8718e-01,   -1.1066e-01,   2.3370e-02  ],# bin 68  chi2/NDF: 5.2185
[ 1.7375,   2.0486e-01,   9.0727e-02,   2.4310e-03  ],# bin 69  chi2/NDF: 3.1605
[ 1.7625,   -3.7719e-01,   2.5488e-01,   -1.7859e-02  ],# bin 70  chi2/NDF: 5.6473
[ 1.7875,   1.4365e-02,   2.0308e-01,   -1.8326e-02  ],# bin 71  chi2/NDF: 1.0927
[ 1.8125,   -2.9699e+03,   -1.0793e+02,   -6.3852e-02  ],# bin 72  chi2/NDF: 7.6089
[ 1.8375,   -1.4254e+04,   1.2817e+02,   -1.0401e+00  ],# bin 73  chi2/NDF: 7.332
[ 1.8625,   -2.0871e+03,   -1.2936e+02,   -1.0320e-01  ],# bin 74  chi2/NDF: 12.85
[ 1.8875,   -3.0356e+00,   -1.4840e+02,   -3.0388e-03  ],# bin 75  chi2/NDF: 22.778
[ 1.9125,   -2.7442e+03,   -5.8590e+01,   -9.5679e-02  ],# bin 76  chi2/NDF: 84.47
[ 1.9375,   -1.7910e+03,   -1.1297e+02,   1.0429e-01  ],# bin 77  chi2/NDF: 0.7028
[ 1.9625,   4.1807e+03,   -1.6027e+02,   5.5343e-02  ],# bin 78  chi2/NDF: 15.971
[ 1.9875,   -6.9685e+03,   -2.8392e+01,   -1.6877e-01  ],# bin 79  chi2/NDF: 5.845
[ 2.0125,   3.7972e+02,   -1.5050e+02,   1.0047e-01  ],# bin 80  chi2/NDF: 0.66128
[ 2.0375,   9.5077e+02,   -1.6053e+02,   7.6345e-02  ],# bin 81  chi2/NDF: 0.25164
[ 2.0625,   -5.2859e+03,   -1.0559e+02,   3.7947e-02  ],# bin 82  chi2/NDF: 8.2064
[ 2.0875,   -8.2983e+03,   -1.0158e+02,   1.0795e-02  ],# bin 83  chi2/NDF: 2.7543
[ 2.1125,   -5.3399e+03,   -6.8194e+01,   -1.3426e-02  ],# bin 84  chi2/NDF: 42.398
[ 2.1375,   -2.3604e+03,   -1.0992e+02,   3.9553e-02  ],# bin 85  chi2/NDF: 11.544
[ 2.1625,   -2.6254e+03,   -9.3961e+01,   2.5849e-02  ],# bin 86  chi2/NDF: 16.26
[ 2.1875,   -1.5748e+03,   -8.8391e+01,   7.8080e-03  ],# bin 87  chi2/NDF: 16.409
[ 2.2125,   -2.1470e+03,   -7.7327e+01,   -1.7047e-02  ],# bin 88  chi2/NDF: 10.289
[ 2.2375,   -4.0120e+02,   -9.6180e+01,   4.7761e-04  ],# bin 89  chi2/NDF: 12.608
[ 2.2625,   9.1154e+02,   -1.0172e+02,   4.2870e-02  ],# bin 90  chi2/NDF: 12.998
[ 2.2875,   2.2216e+03,   -1.3572e+02,   9.3515e-02  ],# bin 91  chi2/NDF: 28.663
[ 2.3125,   -9.2977e+02,   -3.7093e+01,   1.7608e-02  ],# bin 92  chi2/NDF: 17.329
[ 2.3375,   7.4012e+01,   -1.6234e+02,   3.7581e-03  ],# bin 93  chi2/NDF: 10.887
[ 2.3625,   -1.4774e+03,   -1.1239e+02,   -4.7713e-02  ],# bin 94  chi2/NDF: 12.74
[ 2.3875,   2.7216e+03,   -2.0065e+02,   7.9458e-02  ],# bin 95  chi2/NDF: 5.7571
[ 2.4125,   2.0641e+03,   -1.8220e+02,   -6.9711e-03  ],# bin 96  chi2/NDF: 23.681
[ 2.4375,   2.5745e+03,   -2.0991e+02,   8.6039e-02  ],# bin 97  chi2/NDF: 12.238
[ 2.4625,   2.6239e+02,   -2.0275e+02,   1.5053e-01  ],# bin 98  chi2/NDF: 16.507
[ 2.4875,   2.6239e+02,   -2.0275e+02,   1.5053e-01  ],# bin 99 missing par  chi2/NDF: 0, copied from bin 98
] # end CaloSwCalHitsEfront_slope_v8_gam_conv


CaloSwCalHitsEfront_slope_v8_gam_unconv = [
[ 0.0125,   9.3369e-01,   1.8699e-01,   -1.2329e-04  ],# bin 0  chi2/NDF: 5.1963, copied from bin 2
[ 0.0375,   9.3369e-01,   1.8699e-01,   -1.2329e-04  ],# bin 1  chi2/NDF: 11.773, copied from bin 2
[ 0.0625,   9.3369e-01,   1.8699e-01,   -1.2329e-04  ],# bin 2  chi2/NDF: 11.836
[ 0.0875,   8.4771e-01,   2.6608e-01,   -2.5042e-03  ],# bin 3  chi2/NDF: 4.4006
[ 0.1125,   9.6217e-01,   1.4249e-01,   4.1709e-03  ],# bin 4  chi2/NDF: 9.7093
[ 0.1375,   9.1612e-01,   1.9366e-01,   1.5173e-03  ],# bin 5  chi2/NDF: 11.023
[ 0.1625,   8.6074e-01,   2.3879e-01,   7.0141e-04  ],# bin 6  chi2/NDF: 4.2258
[ 0.1875,   8.1056e-01,   2.7691e-01,   -3.9432e-04  ],# bin 7  chi2/NDF: 6.5271
[ 0.2125,   8.5811e-01,   2.2013e-01,   9.0745e-04  ],# bin 8  chi2/NDF: 10.777
[ 0.2375,   7.7682e-01,   3.0173e-01,   -1.9043e-03  ],# bin 9  chi2/NDF: 4.4108
[ 0.2625,   8.0571e-01,   2.6568e-01,   -3.9502e-05  ],# bin 10  chi2/NDF: 8.2366
[ 0.2875,   8.0622e-01,   2.6508e-01,   5.0655e-04  ],# bin 11  chi2/NDF: 3.9582
[ 0.3125,   7.7195e-01,   3.1823e-01,   -4.6078e-03  ],# bin 12  chi2/NDF: 4.8403
[ 0.3375,   7.8826e-01,   2.7958e-01,   -1.5244e-04  ],# bin 13  chi2/NDF: 9.2178
[ 0.3625,   6.9821e-01,   4.0272e-01,   -6.6770e-03  ],# bin 14  chi2/NDF: 7.9597
[ 0.3875,   8.4405e-01,   2.2988e-01,   1.5233e-03  ],# bin 15  chi2/NDF: 10.465
[ 0.4125,   8.3164e-01,   2.3345e-01,   4.6478e-04  ],# bin 16  chi2/NDF: 8.765
[ 0.4375,   8.0056e-01,   2.5852e-01,   1.7517e-04  ],# bin 17  chi2/NDF: 5.3558
[ 0.4625,   7.6838e-01,   2.9819e-01,   -1.8242e-03  ],# bin 18  chi2/NDF: 3.994
[ 0.4875,   7.9478e-01,   2.6052e-01,   1.0778e-03  ],# bin 19  chi2/NDF: 3.8575
[ 0.5125,   8.8055e-01,   2.1813e-01,   -2.7375e-03  ],# bin 20  chi2/NDF: 64.049
[ 0.5375,   1.1037e+00,   2.1879e-02,   5.7713e-03  ],# bin 21  chi2/NDF: 61.003
[ 0.5625,   1.0998e+00,   7.8480e-02,   -2.7061e-03  ],# bin 22  chi2/NDF: 120.41
[ 0.5875,   1.0967e+00,   6.0926e-02,   4.5062e-04  ],# bin 23  chi2/NDF: 57.516
[ 0.6125,   1.2715e+00,   -7.5177e-02,   6.9620e-03  ],# bin 24  chi2/NDF: 28.086
[ 0.6375,   1.4361e+00,   -1.6739e-01,   8.4153e-03  ],# bin 25  chi2/NDF: 22.743
[ 0.6625,   1.2578e+00,   -6.8903e-02,   6.6003e-03  ],# bin 26  chi2/NDF: 28.376
[ 0.6875,   1.2890e+00,   -7.7388e-02,   5.9459e-03  ],# bin 27  chi2/NDF: 34.655
[ 0.7125,   1.4298e+00,   -1.5351e-01,   8.4556e-03  ],# bin 28  chi2/NDF: 21.574
[ 0.7375,   1.3267e+00,   -1.0068e-01,   8.0315e-03  ],# bin 29  chi2/NDF: 24.205
[ 0.7625,   1.3267e+00,   -1.0068e-01,   8.0315e-03  ],# bin 30  chi2/NDF: 12.724, copied from bin 29
[ 0.7875,   1.3267e+00,   -1.0068e-01,   8.0315e-03  ],# bin 31  chi2/NDF: 6.0278, copied from bin 29
[ 0.8125,   1.2036e+00,   -4.2759e-03,   4.4813e-03  ],# bin 32  chi2/NDF: 3.2605, copied from bin 34
[ 0.8375,   1.2036e+00,   -4.2759e-03,   4.4813e-03  ],# bin 33  chi2/NDF: 2.3466, copied from bin 34
[ 0.8625,   1.2036e+00,   -4.2759e-03,   4.4813e-03  ],# bin 34  chi2/NDF: 3.9415
[ 0.8875,   1.1051e+00,   5.6740e-02,   3.4388e-03  ],# bin 35  chi2/NDF: 5.1851
[ 0.9125,   1.1253e+00,   4.2874e-02,   3.3835e-03  ],# bin 36  chi2/NDF: 3.4644
[ 0.9375,   1.1764e+00,   -7.6413e-03,   6.7864e-03  ],# bin 37  chi2/NDF: 6.6324
[ 0.9625,   1.1560e+00,   6.6843e-03,   5.8748e-03  ],# bin 38  chi2/NDF: 5.6948
[ 0.9875,   9.9559e-01,   1.2754e-01,   2.3117e-03  ],# bin 39  chi2/NDF: 2.8238
[ 1.0125,   1.0696e+00,   6.6009e-02,   4.5050e-03  ],# bin 40  chi2/NDF: 5.6469
[ 1.0375,   1.0920e+00,   6.7111e-02,   2.3464e-03  ],# bin 41  chi2/NDF: 6.2621
[ 1.0625,   1.0331e+00,   9.9596e-02,   2.2324e-03  ],# bin 42  chi2/NDF: 4.4302
[ 1.0875,   1.0695e+00,   7.4603e-02,   2.5839e-03  ],# bin 43  chi2/NDF: 5.1017
[ 1.1125,   1.0834e+00,   6.3521e-02,   4.8037e-03  ],# bin 44  chi2/NDF: 12.557
[ 1.1375,   1.0313e+00,   1.0819e-01,   3.0172e-03  ],# bin 45  chi2/NDF: 14.039
[ 1.1625,   1.0361e+00,   1.0596e-01,   2.9499e-03  ],# bin 46  chi2/NDF: 4.0094
[ 1.1875,   1.0699e+00,   8.3562e-02,   1.5185e-03  ],# bin 47  chi2/NDF: 3.4959
[ 1.2125,   1.0291e+00,   7.2469e-02,   3.1324e-03  ],# bin 48  chi2/NDF: 2.8877
[ 1.2375,   1.0407e+00,   6.1519e-02,   3.6952e-03  ],# bin 49  chi2/NDF: 2.6754
[ 1.2625,   9.8081e-01,   1.0301e-01,   3.1029e-03  ],# bin 50  chi2/NDF: 1.4811
[ 1.2875,   9.4122e-01,   1.2916e-01,   3.2766e-03  ],# bin 51  chi2/NDF: 5.216
[ 1.3125,   9.4808e-01,   1.4120e-01,   1.6598e-03  ],# bin 52  chi2/NDF: 2.4815
[ 1.3375,   9.0448e-01,   1.9148e-01,   -8.4436e-04  ],# bin 53  chi2/NDF: 3.3206
[ 1.3625,   9.6238e-01,   1.4574e-01,   3.3439e-04  ],# bin 54  chi2/NDF: 2.7481
[ 1.3875,   1.0025e+00,   1.4127e-01,   -1.4972e-03  ],# bin 55  chi2/NDF: 17.188
[ 1.4125,   9.3067e-01,   2.0634e-01,   -6.5940e-03  ],# bin 56  chi2/NDF: 21.197
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack missing par  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack missing par  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -1.7191e+00,   1.0586e+00,   -5.7447e-01  ],# bin 60 crack  chi2/NDF: 54.489
[ 1.5375,   2.4367e+00,   -4.9620e-01,   5.9202e-02  ],# bin 61 crack  chi2/NDF: 9.6836
[ 1.5625,   1.6561e+00,   -1.6886e-01,   1.5122e-02  ],# bin 62  chi2/NDF: 1.4982
[ 1.5875,   1.8269e+00,   -2.0933e-01,   1.9381e-02  ],# bin 63  chi2/NDF: 6.8504
[ 1.6125,   1.6864e+00,   -1.6004e-01,   1.0980e-02  ],# bin 64  chi2/NDF: 9.5772
[ 1.6375,   1.8029e+00,   -2.1569e-01,   1.9707e-02  ],# bin 65  chi2/NDF: 11.714
[ 1.6625,   1.2298e+00,   -5.0555e-02,   9.8211e-04  ],# bin 66  chi2/NDF: 7.7145
[ 1.6875,   1.7058e+00,   -1.8031e-01,   1.5812e-02  ],# bin 67  chi2/NDF: 3.7195
[ 1.7125,   1.3682e+00,   -1.1860e-01,   9.6204e-03  ],# bin 68  chi2/NDF: 5.6707
[ 1.7375,   1.4413e+00,   -1.6794e-01,   1.7803e-02  ],# bin 69  chi2/NDF: 29.607
[ 1.7625,   1.4674e+00,   -1.9373e-01,   2.3703e-02  ],# bin 70  chi2/NDF: 2.9403
[ 1.7875,   9.2795e-01,   -1.7665e-02,   3.8164e-03  ],# bin 71  chi2/NDF: 4.6557
[ 1.8125,   -2.7636e+03,   -8.3626e+01,   1.0991e-01  ],# bin 72  chi2/NDF: 7.8691
[ 1.8375,   -2.8997e+03,   -3.7191e+01,   -5.9583e-02  ],# bin 73  chi2/NDF: 10.513
[ 1.8625,   -1.1879e+03,   -4.0521e+01,   -1.1106e-01  ],# bin 74  chi2/NDF: 15.604
[ 1.8875,   -3.3916e+03,   -4.7928e-01,   -2.0803e-01  ],# bin 75  chi2/NDF: 13.515
[ 1.9125,   -4.9842e+02,   -3.1371e+01,   -1.1949e-01  ],# bin 76  chi2/NDF: 4.3119
[ 1.9375,   7.3755e+02,   -5.2518e+01,   -6.4468e-02  ],# bin 77  chi2/NDF: 0.2039
[ 1.9625,   5.6655e+02,   -4.2094e+01,   -7.4854e-02  ],# bin 78  chi2/NDF: 5.7062
[ 1.9875,   -2.1540e+03,   -2.8576e+01,   2.8454e-03  ],# bin 79  chi2/NDF: 1.1904
[ 2.0125,   1.1700e+03,   -6.4017e+01,   3.3332e-02  ],# bin 80  chi2/NDF: 0.24872
[ 2.0375,   -1.0170e+03,   -2.9299e+01,   -9.3713e-03  ],# bin 81  chi2/NDF: 8.4159
[ 2.0625,   -1.4873e+03,   -2.4263e+01,   8.3394e-03  ],# bin 82  chi2/NDF: 0.20353
[ 2.0875,   -7.5339e+02,   -3.1744e+01,   -1.3968e-02  ],# bin 83  chi2/NDF: 13.89
[ 2.1125,   -4.2115e+03,   2.4386e+00,   -4.6572e-02  ],# bin 84  chi2/NDF: 2.0537
[ 2.1375,   -6.2989e+02,   -3.1361e+01,   1.7553e-02  ],# bin 85  chi2/NDF: 2.369
[ 2.1625,   -4.8758e+03,   3.2998e+00,   -1.7639e-02  ],# bin 86  chi2/NDF: 1.2391
[ 2.1875,   -3.4983e+03,   1.0020e+01,   -3.5055e-02  ],# bin 87  chi2/NDF: 1.1465
[ 2.2125,   -4.3125e+03,   6.8458e+00,   -2.0275e-02  ],# bin 88  chi2/NDF: 3.6496
[ 2.2375,   1.5531e+02,   -2.5756e+01,   1.9303e-02  ],# bin 89  chi2/NDF: 12.759
[ 2.2625,   -9.8144e+01,   -1.5237e+01,   4.8777e-03  ],# bin 90  chi2/NDF: 0.59143
[ 2.2875,   -1.2015e+03,   -5.0603e+00,   -3.4411e-03  ],# bin 91  chi2/NDF: 3.5533
[ 2.3125,   1.0072e+02,   -1.6750e+01,   6.4155e-03  ],# bin 92  chi2/NDF: 2.8123
[ 2.3375,   -8.9830e+02,   -4.1597e+01,   2.1584e-02  ],# bin 93  chi2/NDF: 2.0369
[ 2.3625,   -9.1373e+03,   -1.0330e+01,   -1.1235e-02  ],# bin 94  chi2/NDF: 2.0369
[ 2.3875,   -3.4074e+03,   -2.3668e+01,   5.5599e-04  ],# bin 95  chi2/NDF: 4.3897
[ 2.4125,   -6.5401e+03,   1.4102e+01,   -4.8280e-02  ],# bin 96  chi2/NDF: 10.698
[ 2.4375,   7.0656e+02,   -3.2075e+01,   8.5828e-03  ],# bin 97  chi2/NDF: 12.66
[ 2.4625,   -5.1267e+02,   -3.7899e+01,   2.7406e-02  ],# bin 98  chi2/NDF: 4.6154
[ 2.4875,   -5.1267e+02,   -3.7899e+01,   2.7406e-02  ],# bin 99 missing par  chi2/NDF: 0, copied from bin 98
] # end CaloSwCalHitsEfront_slope_v8_gam_unconv


CaloSwCalHitsOutOfCone_v8_ele35 = [
[ 0.0125,   1.0434e+01,   -8.0542e-01,   2.4189e-02  ],# bin 0  chi2/NDF: 90.497, copied from bin 2
[ 0.0375,   1.0434e+01,   -8.0542e-01,   2.4189e-02  ],# bin 1  chi2/NDF: 116.75, copied from bin 2
[ 0.0625,   1.0434e+01,   -8.0542e-01,   2.4189e-02  ],# bin 2  chi2/NDF: 75.863
[ 0.0875,   1.0561e+01,   -8.1480e-01,   2.4237e-02  ],# bin 3  chi2/NDF: 180.58
[ 0.1125,   1.0528e+01,   -8.2136e-01,   2.5278e-02  ],# bin 4  chi2/NDF: 76.533
[ 0.1375,   1.0269e+01,   -7.6094e-01,   2.2186e-02  ],# bin 5  chi2/NDF: 105.11
[ 0.1625,   9.8869e+00,   -6.9436e-01,   1.9279e-02  ],# bin 6  chi2/NDF: 181.45
[ 0.1875,   9.9011e+00,   -6.8251e-01,   1.8226e-02  ],# bin 7  chi2/NDF: 101.09
[ 0.2125,   1.0228e+01,   -7.5009e-01,   2.1755e-02  ],# bin 8  chi2/NDF: 128.69
[ 0.2375,   9.7576e+00,   -6.6917e-01,   1.8466e-02  ],# bin 9  chi2/NDF: 98.485
[ 0.2625,   1.0951e+01,   -8.7427e-01,   2.7199e-02  ],# bin 10  chi2/NDF: 62.022
[ 0.2875,   1.0183e+01,   -7.3310e-01,   2.0986e-02  ],# bin 11  chi2/NDF: 92.263
[ 0.3125,   8.9041e+00,   -4.8031e-01,   8.9841e-03  ],# bin 12  chi2/NDF: 297
[ 0.3375,   1.0489e+01,   -7.7732e-01,   2.3097e-02  ],# bin 13  chi2/NDF: 119.42
[ 0.3625,   1.1006e+01,   -8.6056e-01,   2.6545e-02  ],# bin 14  chi2/NDF: 78.241
[ 0.3875,   1.0607e+01,   -7.7829e-01,   2.2817e-02  ],# bin 15  chi2/NDF: 154.44
[ 0.4125,   1.0735e+01,   -7.9841e-01,   2.3693e-02  ],# bin 16  chi2/NDF: 154.56
[ 0.4375,   1.0626e+01,   -7.7396e-01,   2.2723e-02  ],# bin 17  chi2/NDF: 88.74
[ 0.4625,   1.1277e+01,   -8.8513e-01,   2.7694e-02  ],# bin 18  chi2/NDF: 47.304
[ 0.4875,   1.0769e+01,   -7.8044e-01,   2.2836e-02  ],# bin 19  chi2/NDF: 85.968
[ 0.5125,   1.0980e+01,   -8.1350e-01,   2.4395e-02  ],# bin 20  chi2/NDF: 48.56
[ 0.5375,   1.1900e+01,   -9.5584e-01,   2.9995e-02  ],# bin 21  chi2/NDF: 88.094
[ 0.5625,   1.2114e+01,   -9.9056e-01,   3.1398e-02  ],# bin 22  chi2/NDF: 60.848
[ 0.5875,   1.1772e+01,   -9.2690e-01,   2.8778e-02  ],# bin 23  chi2/NDF: 65.776
[ 0.6125,   1.1222e+01,   -8.3590e-01,   2.5027e-02  ],# bin 24  chi2/NDF: 94.34
[ 0.6375,   1.2770e+01,   -1.0747e+00,   3.4521e-02  ],# bin 25  chi2/NDF: 44.506
[ 0.6625,   1.3571e+01,   -1.1717e+00,   3.7226e-02  ],# bin 26  chi2/NDF: 45.922
[ 0.6875,   1.4772e+01,   -1.3587e+00,   4.4859e-02  ],# bin 27  chi2/NDF: 71.97
[ 0.7125,   1.4533e+01,   -1.3067e+00,   4.2092e-02  ],# bin 28  chi2/NDF: 88.358
[ 0.7375,   1.4923e+01,   -1.3530e+00,   4.3708e-02  ],# bin 29  chi2/NDF: 107.51
[ 0.7625,   1.4923e+01,   -1.3530e+00,   4.3708e-02  ],# bin 30  chi2/NDF: 75.088, copied from bin 29
[ 0.7875,   1.4923e+01,   -1.3530e+00,   4.3708e-02  ],# bin 31  chi2/NDF: 92.134, copied from bin 29
[ 0.8125,   3.1154e+01,   -3.1410e+00,   9.5251e-02  ],# bin 32  chi2/NDF: 80.23, copied from bin 34
[ 0.8375,   3.1154e+01,   -3.1410e+00,   9.5251e-02  ],# bin 33  chi2/NDF: 75.657, copied from bin 34
[ 0.8625,   3.1154e+01,   -3.1410e+00,   9.5251e-02  ],# bin 34  chi2/NDF: 41.214
[ 0.8875,   3.3380e+01,   -3.5070e+00,   1.1016e-01  ],# bin 35  chi2/NDF: 60.743
[ 0.9125,   2.9961e+01,   -2.9749e+00,   8.9480e-02  ],# bin 36  chi2/NDF: 82.558
[ 0.9375,   3.2564e+01,   -3.4375e+00,   1.0962e-01  ],# bin 37  chi2/NDF: 126.3
[ 0.9625,   3.1796e+01,   -3.3237e+00,   1.0506e-01  ],# bin 38  chi2/NDF: 75.804
[ 0.9875,   3.1734e+01,   -3.3196e+00,   1.0543e-01  ],# bin 39  chi2/NDF: 55.258
[ 1.0125,   3.4134e+01,   -3.7286e+00,   1.2230e-01  ],# bin 40  chi2/NDF: 76.604
[ 1.0375,   3.3048e+01,   -3.6066e+00,   1.1935e-01  ],# bin 41  chi2/NDF: 112.61
[ 1.0625,   3.2613e+01,   -3.5590e+00,   1.1781e-01  ],# bin 42  chi2/NDF: 27.377
[ 1.0875,   3.3413e+01,   -3.7171e+00,   1.2550e-01  ],# bin 43  chi2/NDF: 110.4
[ 1.1125,   3.1608e+01,   -3.4573e+00,   1.1564e-01  ],# bin 44  chi2/NDF: 122.6
[ 1.1375,   3.3688e+01,   -3.8424e+00,   1.3337e-01  ],# bin 45  chi2/NDF: 137.77
[ 1.1625,   3.4227e+01,   -3.8765e+00,   1.3259e-01  ],# bin 46  chi2/NDF: 148.13
[ 1.1875,   3.6582e+01,   -4.2248e+00,   1.4628e-01  ],# bin 47  chi2/NDF: 175.46
[ 1.2125,   3.7978e+01,   -4.3951e+00,   1.5088e-01  ],# bin 48  chi2/NDF: 145.75
[ 1.2375,   4.0202e+01,   -4.7275e+00,   1.6424e-01  ],# bin 49  chi2/NDF: 190.95
[ 1.2625,   4.1212e+01,   -4.8063e+00,   1.6394e-01  ],# bin 50  chi2/NDF: 153.82
[ 1.2875,   4.5196e+01,   -5.4069e+00,   1.8753e-01  ],# bin 51  chi2/NDF: 161.07
[ 1.3125,   5.0535e+01,   -6.1585e+00,   2.1332e-01  ],# bin 52  chi2/NDF: 154.44
[ 1.3375,   5.2338e+01,   -6.3273e+00,   2.1598e-01  ],# bin 53  chi2/NDF: 96.209
[ 1.3625,   6.1713e+01,   -7.8989e+00,   2.7974e-01  ],# bin 54  chi2/NDF: 142.43
[ 1.3875,   7.3694e+01,   -8.9597e+00,   3.0711e-01  ],# bin 55  chi2/NDF: 127.22
[ 1.4125,   6.3772e+01,   -7.9414e+00,   2.6723e-01  ],# bin 56  chi2/NDF: 45.666
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -2.6317e+01,   7.0140e-01,   3.6150e+02  ],# bin 60 crack  chi2/NDF: 652.91
[ 1.5375,   -3.7988e+01,   1.1553e+00,   3.9992e+02  ],# bin 61 crack  chi2/NDF: 41.775
[ 1.5625,   -2.2834e+01,   5.8680e-01,   3.1038e+02  ],# bin 62  chi2/NDF: 20.394
[ 1.5875,   -2.9316e+01,   9.0334e-01,   3.2449e+02  ],# bin 63  chi2/NDF: 39.715
[ 1.6125,   -2.5536e+01,   8.0695e-01,   2.9151e+02  ],# bin 64  chi2/NDF: 69.612
[ 1.6375,   -2.4586e+01,   7.9595e-01,   2.8089e+02  ],# bin 65  chi2/NDF: 9.8968
[ 1.6625,   -2.9341e+01,   9.8677e-01,   3.1485e+02  ],# bin 66  chi2/NDF: 35.294
[ 1.6875,   -3.7226e+01,   1.2446e+00,   3.7812e+02  ],# bin 67  chi2/NDF: 64.292
[ 1.7125,   -2.7817e+01,   9.1333e-01,   3.1489e+02  ],# bin 68  chi2/NDF: 155.67
[ 1.7375,   -2.0595e+01,   7.4758e-01,   2.4014e+02  ],# bin 69  chi2/NDF: 107.04
[ 1.7625,   -1.8345e+01,   7.0898e-01,   2.1668e+02  ],# bin 70  chi2/NDF: 131.05
[ 1.7875,   -5.3298e+00,   2.7570e-01,   1.2082e+02  ],# bin 71  chi2/NDF: 59.715
[ 1.8125,   5.0307e+00,   -1.0683e-01,   5.5610e+01  ],# bin 72  chi2/NDF: 18.661
[ 1.8375,   -4.3981e+00,   2.2257e-01,   1.2354e+02  ],# bin 73  chi2/NDF: 30.237
[ 1.8625,   1.1912e+00,   4.3472e-02,   8.1609e+01  ],# bin 74  chi2/NDF: 39.011
[ 1.8875,   3.3522e+00,   -1.8909e-02,   6.6289e+01  ],# bin 75  chi2/NDF: 9.2649
[ 1.9125,   1.0807e+00,   8.7981e-02,   7.9157e+01  ],# bin 76  chi2/NDF: 38.831
[ 1.9375,   -7.2316e-01,   1.7904e-01,   8.9206e+01  ],# bin 77  chi2/NDF: 35.301
[ 1.9625,   6.6742e+00,   -8.0215e-02,   3.9916e+01  ],# bin 78  chi2/NDF: 20.994
[ 1.9875,   6.8584e+00,   -8.3248e-02,   4.1354e+01  ],# bin 79  chi2/NDF: 12.177
[ 2.0125,   6.2297e+00,   -4.8908e-02,   4.7558e+01  ],# bin 80  chi2/NDF: 23.773
[ 2.0375,   4.6267e+00,   7.6987e-03,   6.2317e+01  ],# bin 81  chi2/NDF: 5.4702
[ 2.0625,   4.6749e+00,   1.2756e-02,   6.4355e+01  ],# bin 82  chi2/NDF: 24.534
[ 2.0875,   2.8128e+00,   9.5826e-02,   7.7605e+01  ],# bin 83  chi2/NDF: 9.447
[ 2.1125,   2.1661e+00,   1.2726e-01,   8.1638e+01  ],# bin 84  chi2/NDF: 8.887
[ 2.1375,   4.1976e+00,   6.2227e-02,   6.8843e+01  ],# bin 85  chi2/NDF: 19.615
[ 2.1625,   6.1549e+00,   1.2688e-02,   5.4930e+01  ],# bin 86  chi2/NDF: 12.039
[ 2.1875,   6.4194e+00,   1.1782e-02,   5.4345e+01  ],# bin 87  chi2/NDF: 18.292
[ 2.2125,   7.6962e+00,   -2.6069e-02,   4.7254e+01  ],# bin 88  chi2/NDF: 25.953
[ 2.2375,   8.2443e+00,   -3.1156e-02,   4.3183e+01  ],# bin 89  chi2/NDF: 52.692
[ 2.2625,   1.2074e+00,   2.5019e-01,   8.9738e+01  ],# bin 90  chi2/NDF: 160.7
[ 2.2875,   3.9746e+00,   1.4523e-01,   7.6820e+01  ],# bin 91  chi2/NDF: 188.04
[ 2.3125,   1.9452e+01,   -4.4883e-01,   -1.4421e+01  ],# bin 92  chi2/NDF: 232.33
[ 2.3375,   3.8811e+00,   5.5971e-03,   1.1773e+02  ],# bin 93  chi2/NDF: 198.67
[ 2.3625,   -5.7271e+00,   3.4309e-01,   1.9158e+02  ],# bin 94  chi2/NDF: 212.24
[ 2.3875,   7.6288e+00,   -1.5522e-01,   1.0152e+02  ],# bin 95  chi2/NDF: 125.29
[ 2.4125,   2.1724e+01,   -6.5876e-01,   6.5912e+00  ],# bin 96  chi2/NDF: 56.254
[ 2.4375,   1.2411e+01,   -3.6216e-01,   8.1177e+01  ],# bin 97  chi2/NDF: 65.999
[ 2.4625,   -5.9205e+01,   2.1594e+00,   5.9929e+02  ],# bin 98  chi2/NDF: 28.242
[ 2.4875,   -5.9205e+01,   2.1594e+00,   5.9929e+02  ],# bin 99  chi2/NDF: 28.242, copied from bin 98
] # end CaloSwCalHitsOutOfCone_v8_ele35


CaloSwCalHitsOutOfCone_v8_gam35_conv = [
[ 0.0125,   1.2385e+01,   -1.0049e+00,   2.7891e-02  ],# bin 0  chi2/NDF: 29.779, copied from bin 2
[ 0.0375,   1.2385e+01,   -1.0049e+00,   2.7891e-02  ],# bin 1  chi2/NDF: 36.706, copied from bin 2
[ 0.0625,   1.2385e+01,   -1.0049e+00,   2.7891e-02  ],# bin 2  chi2/NDF: 30.949
[ 0.0875,   1.4595e+01,   -1.4072e+00,   4.6112e-02  ],# bin 3  chi2/NDF: 64.836
[ 0.1125,   1.3237e+01,   -1.1239e+00,   3.2160e-02  ],# bin 4  chi2/NDF: 86.829
[ 0.1375,   1.3374e+01,   -1.1679e+00,   3.5055e-02  ],# bin 5  chi2/NDF: 76.278
[ 0.1625,   1.3493e+01,   -1.1977e+00,   3.6648e-02  ],# bin 6  chi2/NDF: 61.413
[ 0.1875,   1.3199e+01,   -1.1239e+00,   3.2856e-02  ],# bin 7  chi2/NDF: 42.054
[ 0.2125,   1.3569e+01,   -1.1884e+00,   3.5798e-02  ],# bin 8  chi2/NDF: 69.196
[ 0.2375,   1.2680e+01,   -1.0553e+00,   3.0961e-02  ],# bin 9  chi2/NDF: 45.744
[ 0.2625,   1.3835e+01,   -1.2537e+00,   3.9390e-02  ],# bin 10  chi2/NDF: 65.298
[ 0.2875,   1.3004e+01,   -1.0968e+00,   3.2469e-02  ],# bin 11  chi2/NDF: 102.4
[ 0.3125,   1.2674e+01,   -1.0308e+00,   2.9547e-02  ],# bin 12  chi2/NDF: 57.642
[ 0.3375,   1.3705e+01,   -1.1877e+00,   3.5798e-02  ],# bin 13  chi2/NDF: 44.944
[ 0.3625,   1.1942e+01,   -8.9623e-01,   2.4008e-02  ],# bin 14  chi2/NDF: 42.579
[ 0.3875,   1.1823e+01,   -8.5730e-01,   2.1974e-02  ],# bin 15  chi2/NDF: 46.792
[ 0.4125,   1.3123e+01,   -1.0485e+00,   2.9158e-02  ],# bin 16  chi2/NDF: 21.822
[ 0.4375,   1.3914e+01,   -1.1913e+00,   3.5725e-02  ],# bin 17  chi2/NDF: 54.186
[ 0.4625,   1.3328e+01,   -1.0798e+00,   3.0897e-02  ],# bin 18  chi2/NDF: 40.745
[ 0.4875,   1.3901e+01,   -1.1563e+00,   3.3695e-02  ],# bin 19  chi2/NDF: 29.982
[ 0.5125,   1.4465e+01,   -1.2395e+00,   3.7060e-02  ],# bin 20  chi2/NDF: 39.252
[ 0.5375,   1.3332e+01,   -1.0503e+00,   2.9427e-02  ],# bin 21  chi2/NDF: 42.722
[ 0.5625,   1.5017e+01,   -1.3094e+00,   3.9223e-02  ],# bin 22  chi2/NDF: 43.618
[ 0.5875,   1.3473e+01,   -1.0474e+00,   2.8746e-02  ],# bin 23  chi2/NDF: 30.969
[ 0.6125,   1.4036e+01,   -1.1586e+00,   3.3779e-02  ],# bin 24  chi2/NDF: 38.817
[ 0.6375,   1.6029e+01,   -1.4585e+00,   4.5285e-02  ],# bin 25  chi2/NDF: 65.515
[ 0.6625,   1.8377e+01,   -1.8139e+00,   5.8528e-02  ],# bin 26  chi2/NDF: 26.472
[ 0.6875,   1.8553e+01,   -1.8107e+00,   5.7651e-02  ],# bin 27  chi2/NDF: 50.128
[ 0.7125,   1.7756e+01,   -1.6618e+00,   5.0631e-02  ],# bin 28  chi2/NDF: 31.68
[ 0.7375,   1.9133e+01,   -1.8847e+00,   5.9887e-02  ],# bin 29  chi2/NDF: 73.712
[ 0.7625,   1.9133e+01,   -1.8847e+00,   5.9887e-02  ],# bin 30  chi2/NDF: 39.365, copied from bin 29
[ 0.7875,   1.9133e+01,   -1.8847e+00,   5.9887e-02  ],# bin 31  chi2/NDF: 19.953, copied from bin 29
[ 0.8125,   4.1973e+01,   -4.7380e+00,   1.5381e-01  ],# bin 32  chi2/NDF: 64.757, copied from bin 34
[ 0.8375,   4.1973e+01,   -4.7380e+00,   1.5381e-01  ],# bin 33  chi2/NDF: 117.67, copied from bin 34
[ 0.8625,   4.1973e+01,   -4.7380e+00,   1.5381e-01  ],# bin 34  chi2/NDF: 61.721
[ 0.8875,   3.8020e+01,   -4.1744e+00,   1.3397e-01  ],# bin 35  chi2/NDF: 76.903
[ 0.9125,   3.4256e+01,   -3.6209e+00,   1.1331e-01  ],# bin 36  chi2/NDF: 42.501
[ 0.9375,   3.6420e+01,   -3.9775e+00,   1.2815e-01  ],# bin 37  chi2/NDF: 101.75
[ 0.9625,   3.2322e+01,   -3.3885e+00,   1.0686e-01  ],# bin 38  chi2/NDF: 112.9
[ 0.9875,   3.5116e+01,   -3.8181e+00,   1.2365e-01  ],# bin 39  chi2/NDF: 69.035
[ 1.0125,   3.5305e+01,   -3.8475e+00,   1.2446e-01  ],# bin 40  chi2/NDF: 33.908
[ 1.0375,   3.6231e+01,   -4.0207e+00,   1.3231e-01  ],# bin 41  chi2/NDF: 39.859
[ 1.0625,   3.6131e+01,   -4.0833e+00,   1.3747e-01  ],# bin 42  chi2/NDF: 97.866
[ 1.0875,   3.5327e+01,   -3.9350e+00,   1.3121e-01  ],# bin 43  chi2/NDF: 87.715
[ 1.1125,   3.3060e+01,   -3.6314e+00,   1.2076e-01  ],# bin 44  chi2/NDF: 96.118
[ 1.1375,   3.3268e+01,   -3.6568e+00,   1.2192e-01  ],# bin 45  chi2/NDF: 16.066
[ 1.1625,   3.3560e+01,   -3.6721e+00,   1.2136e-01  ],# bin 46  chi2/NDF: 75.81
[ 1.1875,   3.8868e+01,   -4.4675e+00,   1.5178e-01  ],# bin 47  chi2/NDF: 53.999
[ 1.2125,   4.1531e+01,   -4.8562e+00,   1.6557e-01  ],# bin 48  chi2/NDF: 38.213
[ 1.2375,   4.0436e+01,   -4.6441e+00,   1.5691e-01  ],# bin 49  chi2/NDF: 84.084
[ 1.2625,   4.2312e+01,   -4.8557e+00,   1.6171e-01  ],# bin 50  chi2/NDF: 63.689
[ 1.2875,   4.5483e+01,   -5.3138e+00,   1.7945e-01  ],# bin 51  chi2/NDF: 47.031
[ 1.3125,   5.0026e+01,   -5.9243e+00,   1.9880e-01  ],# bin 52  chi2/NDF: 54.738
[ 1.3375,   5.2169e+01,   -6.1465e+00,   2.0348e-01  ],# bin 53  chi2/NDF: 19.621
[ 1.3625,   5.2259e+01,   -6.0933e+00,   1.9661e-01  ],# bin 54  chi2/NDF: 92.635
[ 1.3875,   6.9515e+01,   -8.0031e+00,   2.5835e-01  ],# bin 55  chi2/NDF: 158.47
[ 1.4125,   6.2848e+01,   -7.0875e+00,   2.0393e-01  ],# bin 56  chi2/NDF: 71.918
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack missing par  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack missing par  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   6.9086e+00,   -6.6420e-01,   1.8240e+02  ],# bin 60 crack  chi2/NDF: 504.48
[ 1.5375,   -4.7032e+01,   1.4871e+00,   4.6245e+02  ],# bin 61 crack  chi2/NDF: 36.755
[ 1.5625,   -2.5282e+01,   6.6061e-01,   3.2963e+02  ],# bin 62  chi2/NDF: 59.584
[ 1.5875,   -3.2882e+01,   1.0358e+00,   3.4752e+02  ],# bin 63  chi2/NDF: 32.8
[ 1.6125,   -2.4083e+01,   7.6986e-01,   2.7709e+02  ],# bin 64  chi2/NDF: 84.324
[ 1.6375,   -2.6794e+01,   8.9928e-01,   2.9012e+02  ],# bin 65  chi2/NDF: 50.619
[ 1.6625,   -2.0642e+01,   6.7741e-01,   2.5302e+02  ],# bin 66  chi2/NDF: 17.24
[ 1.6875,   -2.0557e+01,   6.4837e-01,   2.6171e+02  ],# bin 67  chi2/NDF: 12.631
[ 1.7125,   -2.4887e+01,   8.3002e-01,   2.8836e+02  ],# bin 68  chi2/NDF: 69.541
[ 1.7375,   -1.7082e+01,   6.3831e-01,   2.1050e+02  ],# bin 69  chi2/NDF: 99.324
[ 1.7625,   -1.3826e+01,   5.5932e-01,   1.8114e+02  ],# bin 70  chi2/NDF: 96.464
[ 1.7875,   -6.5350e+00,   3.4143e-01,   1.2270e+02  ],# bin 71  chi2/NDF: 75.902
[ 1.8125,   8.2028e+00,   -1.8460e-01,   2.3593e+01  ],# bin 72  chi2/NDF: 30.716
[ 1.8375,   9.0388e+00,   -2.1753e-01,   1.9320e+01  ],# bin 73  chi2/NDF: 11.227
[ 1.8625,   1.3142e+01,   -3.5193e-01,   -1.0303e+01  ],# bin 74  chi2/NDF: 41.552
[ 1.8875,   1.7437e+01,   -4.9500e-01,   -3.9421e+01  ],# bin 75  chi2/NDF: 82.51
[ 1.9125,   1.3194e+01,   -3.2340e-01,   -1.1533e+01  ],# bin 76  chi2/NDF: 6.3215
[ 1.9375,   1.3040e+01,   -2.9770e-01,   -1.1395e+01  ],# bin 77  chi2/NDF: 28.184
[ 1.9625,   8.9300e+00,   -1.2792e-01,   1.5520e+01  ],# bin 78  chi2/NDF: 25.713
[ 1.9875,   9.8922e+00,   -1.5207e-01,   9.7571e+00  ],# bin 79  chi2/NDF: 50.736
[ 2.0125,   7.9366e+00,   -6.9362e-02,   2.4246e+01  ],# bin 80  chi2/NDF: 37.872
[ 2.0375,   1.2614e+01,   -2.4380e-01,   -3.2285e+00  ],# bin 81  chi2/NDF: 41.403
[ 2.0625,   3.4706e+00,   9.0105e-02,   6.2763e+01  ],# bin 82  chi2/NDF: 29.3
[ 2.0875,   -1.0658e+01,   5.7741e-01,   1.6866e+02  ],# bin 83  chi2/NDF: 47.542
[ 2.1125,   -1.4094e+01,   7.2208e-01,   1.9004e+02  ],# bin 84  chi2/NDF: 61.309
[ 2.1375,   -1.4843e+01,   7.6114e-01,   1.9601e+02  ],# bin 85  chi2/NDF: 60.238
[ 2.1625,   -1.6074e+01,   7.9532e-01,   2.0998e+02  ],# bin 86  chi2/NDF: 61.139
[ 2.1875,   -1.7306e+01,   8.4639e-01,   2.2022e+02  ],# bin 87  chi2/NDF: 30.93
[ 2.2125,   -1.2139e+01,   6.8035e-01,   1.8359e+02  ],# bin 88  chi2/NDF: 42.243
[ 2.2375,   -1.0291e+01,   6.3389e-01,   1.6907e+02  ],# bin 89  chi2/NDF: 6.4883
[ 2.2625,   -7.7596e+00,   5.3986e-01,   1.5527e+02  ],# bin 90  chi2/NDF: 13.624
[ 2.2875,   4.9253e+00,   9.2923e-02,   7.0082e+01  ],# bin 91  chi2/NDF: 16.498
[ 2.3125,   6.1022e+00,   3.3582e-02,   7.0891e+01  ],# bin 92  chi2/NDF: 22.822
[ 2.3375,   -6.9450e+00,   4.7144e-01,   1.6966e+02  ],# bin 93  chi2/NDF: 28.932
[ 2.3625,   -1.3628e+01,   6.9421e-01,   2.2524e+02  ],# bin 94  chi2/NDF: 30.69
[ 2.3875,   -1.7801e+01,   8.9018e-01,   2.4600e+02  ],# bin 95  chi2/NDF: 142.74
[ 2.4125,   -1.5538e+01,   7.6740e-01,   2.4221e+02  ],# bin 96  chi2/NDF: 93.025
[ 2.4375,   -1.3962e+01,   7.1863e-01,   2.3110e+02  ],# bin 97  chi2/NDF: 40.496
[ 2.4625,   -2.5652e+01,   1.1013e+00,   3.2746e+02  ],# bin 98  chi2/NDF: 57.413
[ 2.4875,   -2.5652e+01,   1.1013e+00,   3.2746e+02  ],# bin 99  chi2/NDF: 57.413, copied from bin 98
] # end CaloSwCalHitsOutOfCone_v8_gam35_conv


CaloSwCalHitsOutOfCone_v8_gam35_unconv = [
[ 0.0125,   1.5915e+00,   6.6637e-01,   -3.6715e-02  ],# bin 0  chi2/NDF: 104.28, copied from bin 2
[ 0.0375,   1.5915e+00,   6.6637e-01,   -3.6715e-02  ],# bin 1  chi2/NDF: 75.637, copied from bin 2
[ 0.0625,   1.5915e+00,   6.6637e-01,   -3.6715e-02  ],# bin 2  chi2/NDF: 92.791
[ 0.0875,   8.7900e-01,   7.9876e-01,   -4.2339e-02  ],# bin 3  chi2/NDF: 40.949
[ 0.1125,   2.8080e+00,   4.4191e-01,   -2.6227e-02  ],# bin 4  chi2/NDF: 80.11
[ 0.1375,   1.4958e+00,   6.8406e-01,   -3.6973e-02  ],# bin 5  chi2/NDF: 102.72
[ 0.1625,   3.4312e+00,   3.4174e-01,   -2.1941e-02  ],# bin 6  chi2/NDF: 70.826
[ 0.1875,   2.2156e+00,   5.3693e-01,   -2.9651e-02  ],# bin 7  chi2/NDF: 193.1
[ 0.2125,   1.8516e+00,   6.0743e-01,   -3.2897e-02  ],# bin 8  chi2/NDF: 51.701
[ 0.2375,   2.6431e+00,   4.6778e-01,   -2.6688e-02  ],# bin 9  chi2/NDF: 75.603
[ 0.2625,   2.2552e+00,   5.3507e-01,   -2.9474e-02  ],# bin 10  chi2/NDF: 17.1
[ 0.2875,   2.6297e+00,   4.6697e-01,   -2.6174e-02  ],# bin 11  chi2/NDF: 26.002
[ 0.3125,   1.9748e+00,   5.7293e-01,   -3.0397e-02  ],# bin 12  chi2/NDF: 67.033
[ 0.3375,   2.1892e+00,   5.2719e-01,   -2.7822e-02  ],# bin 13  chi2/NDF: 53.394
[ 0.3625,   2.2762e+00,   5.1426e-01,   -2.7322e-02  ],# bin 14  chi2/NDF: 24.442
[ 0.3875,   3.9033e+00,   2.4196e-01,   -1.5736e-02  ],# bin 15  chi2/NDF: 30.569
[ 0.4125,   2.5615e+00,   4.5800e-01,   -2.4307e-02  ],# bin 16  chi2/NDF: 98.392
[ 0.4375,   2.7745e+00,   4.2548e-01,   -2.2792e-02  ],# bin 17  chi2/NDF: 79.517
[ 0.4625,   3.7206e+00,   2.6754e-01,   -1.6178e-02  ],# bin 18  chi2/NDF: 53.693
[ 0.4875,   2.1654e+00,   5.0376e-01,   -2.4801e-02  ],# bin 19  chi2/NDF: 56.677
[ 0.5125,   3.8432e+00,   2.3041e-01,   -1.3693e-02  ],# bin 20  chi2/NDF: 74.962
[ 0.5375,   2.7095e+00,   4.1369e-01,   -2.0818e-02  ],# bin 21  chi2/NDF: 212.01
[ 0.5625,   3.6187e+00,   2.7584e-01,   -1.5670e-02  ],# bin 22  chi2/NDF: 47.986
[ 0.5875,   3.7805e+00,   2.4327e-01,   -1.3931e-02  ],# bin 23  chi2/NDF: 162
[ 0.6125,   4.3572e+00,   1.6852e-01,   -1.1598e-02  ],# bin 24  chi2/NDF: 31.336
[ 0.6375,   5.2532e+00,   3.1321e-02,   -6.2110e-03  ],# bin 25  chi2/NDF: 172.61
[ 0.6625,   6.0370e+00,   -7.8040e-02,   -2.5230e-03  ],# bin 26  chi2/NDF: 51.088
[ 0.6875,   6.5207e+00,   -1.5107e-01,   4.2699e-04  ],# bin 27  chi2/NDF: 49.285
[ 0.7125,   6.4138e+00,   -1.3427e-01,   -4.6622e-04  ],# bin 28  chi2/NDF: 18.26
[ 0.7375,   6.9793e+00,   -2.2432e-01,   3.3107e-03  ],# bin 29  chi2/NDF: 83.959
[ 0.7625,   6.9793e+00,   -2.2432e-01,   3.3107e-03  ],# bin 30  chi2/NDF: 54.095, copied from bin 29
[ 0.7875,   6.9793e+00,   -2.2432e-01,   3.3107e-03  ],# bin 31  chi2/NDF: 71.612, copied from bin 29
[ 0.8125,   9.1357e+00,   -2.0053e-01,   -3.2974e-03  ],# bin 32  chi2/NDF: 18.149, copied from bin 34
[ 0.8375,   9.1357e+00,   -2.0053e-01,   -3.2974e-03  ],# bin 33  chi2/NDF: 56.625, copied from bin 34
[ 0.8625,   9.1357e+00,   -2.0053e-01,   -3.2974e-03  ],# bin 34  chi2/NDF: 83.529
[ 0.8875,   9.7624e+00,   -3.1322e-01,   1.5675e-03  ],# bin 35  chi2/NDF: 22.551
[ 0.9125,   8.3195e+00,   -1.1558e-01,   -5.2273e-03  ],# bin 36  chi2/NDF: 62.035
[ 0.9375,   9.5231e+00,   -2.9569e-01,   1.6518e-03  ],# bin 37  chi2/NDF: 41.921
[ 0.9625,   1.1064e+01,   -5.2597e-01,   1.0051e-02  ],# bin 38  chi2/NDF: 34.524
[ 0.9875,   1.1003e+01,   -5.1890e-01,   1.0209e-02  ],# bin 39  chi2/NDF: 22.999
[ 1.0125,   1.1204e+01,   -5.4646e-01,   1.1047e-02  ],# bin 40  chi2/NDF: 28.551
[ 1.0375,   1.1368e+01,   -5.7597e-01,   1.2572e-02  ],# bin 41  chi2/NDF: 42.783
[ 1.0625,   1.2662e+01,   -7.7641e-01,   1.9950e-02  ],# bin 42  chi2/NDF: 14.623
[ 1.0875,   1.2697e+01,   -7.8824e-01,   2.1060e-02  ],# bin 43  chi2/NDF: 37.464
[ 1.1125,   1.2925e+01,   -8.2171e-01,   2.1926e-02  ],# bin 44  chi2/NDF: 13.381
[ 1.1375,   1.4396e+01,   -1.0430e+00,   3.0704e-02  ],# bin 45  chi2/NDF: 23.606
[ 1.1625,   1.4394e+01,   -1.0130e+00,   2.8136e-02  ],# bin 46  chi2/NDF: 66.032
[ 1.1875,   1.4924e+01,   -1.0840e+00,   3.1239e-02  ],# bin 47  chi2/NDF: 120.16
[ 1.2125,   1.5657e+01,   -1.1645e+00,   3.2806e-02  ],# bin 48  chi2/NDF: 46.859
[ 1.2375,   1.8009e+01,   -1.5206e+00,   4.6872e-02  ],# bin 49  chi2/NDF: 38.559
[ 1.2625,   1.9200e+01,   -1.6542e+00,   4.9743e-02  ],# bin 50  chi2/NDF: 39.932
[ 1.2875,   2.2629e+01,   -2.1553e+00,   6.8705e-02  ],# bin 51  chi2/NDF: 24.174
[ 1.3125,   2.3326e+01,   -2.1537e+00,   6.4022e-02  ],# bin 52  chi2/NDF: 69.635
[ 1.3375,   2.7623e+01,   -2.7697e+00,   8.5994e-02  ],# bin 53  chi2/NDF: 9.3196
[ 1.3625,   2.8917e+01,   -2.8887e+00,   8.5148e-02  ],# bin 54  chi2/NDF: 25.875
[ 1.3875,   5.0085e+01,   -5.5787e+00,   1.8342e-01  ],# bin 55  chi2/NDF: 344.08
[ 1.4125,   4.4864e+01,   -5.2390e+00,   1.6765e-01  ],# bin 56  chi2/NDF: 532.64
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack missing par  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack missing par  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -1.9357e+01,   4.7634e-01,   2.6888e+02  ],# bin 60 crack  chi2/NDF: 621.69
[ 1.5375,   -1.9990e+01,   6.5366e-01,   2.3099e+02  ],# bin 61 crack  chi2/NDF: 55.989
[ 1.5625,   -6.6245e+00,   2.5917e-01,   1.2226e+02  ],# bin 62  chi2/NDF: 49.741
[ 1.5875,   -2.5160e+00,   1.6001e-01,   8.0962e+01  ],# bin 63  chi2/NDF: 68.741
[ 1.6125,   -2.1544e+00,   1.5718e-01,   7.9057e+01  ],# bin 64  chi2/NDF: 94.675
[ 1.6375,   -6.3537e+00,   3.0206e-01,   1.1172e+02  ],# bin 65  chi2/NDF: 23.025
[ 1.6625,   -4.8983e+00,   2.3959e-01,   1.0857e+02  ],# bin 66  chi2/NDF: 46.25
[ 1.6875,   -9.8505e+00,   3.9536e-01,   1.5138e+02  ],# bin 67  chi2/NDF: 12.054
[ 1.7125,   -5.2478e+00,   2.8385e-01,   1.1065e+02  ],# bin 68  chi2/NDF: 8.1529
[ 1.7375,   3.2821e+00,   2.8701e-02,   4.0056e+01  ],# bin 69  chi2/NDF: 102.06
[ 1.7625,   5.3204e+00,   -3.5541e-02,   2.6469e+01  ],# bin 70  chi2/NDF: 109.84
[ 1.7875,   1.0523e+01,   -1.9294e-01,   -1.3779e+01  ],# bin 71  chi2/NDF: 38.855
[ 1.8125,   1.8153e+01,   -4.3177e-01,   -7.1546e+01  ],# bin 72  chi2/NDF: 117.46
[ 1.8375,   2.2347e+01,   -5.8985e-01,   -9.6854e+01  ],# bin 73  chi2/NDF: 44.959
[ 1.8625,   3.3059e+01,   -9.4998e-01,   -1.7402e+02  ],# bin 74  chi2/NDF: 132.58
[ 1.8875,   3.0938e+01,   -8.6208e-01,   -1.5851e+02  ],# bin 75  chi2/NDF: 51.273
[ 1.9125,   2.8922e+01,   -7.7819e-01,   -1.4416e+02  ],# bin 76  chi2/NDF: 46.125
[ 1.9375,   3.5169e+01,   -9.8970e-01,   -1.8677e+02  ],# bin 77  chi2/NDF: 102.73
[ 1.9625,   3.2958e+01,   -8.9982e-01,   -1.7067e+02  ],# bin 78  chi2/NDF: 92.882
[ 1.9875,   3.1587e+01,   -8.2619e-01,   -1.6299e+02  ],# bin 79  chi2/NDF: 70.471
[ 2.0125,   2.5605e+01,   -6.1700e-01,   -1.1714e+02  ],# bin 80  chi2/NDF: 25.848
[ 2.0375,   2.5950e+01,   -6.2850e-01,   -1.1617e+02  ],# bin 81  chi2/NDF: 25.041
[ 2.0625,   2.5144e+01,   -5.8770e-01,   -1.0966e+02  ],# bin 82  chi2/NDF: 31.857
[ 2.0875,   2.6774e+01,   -6.1638e-01,   -1.2413e+02  ],# bin 83  chi2/NDF: 24.71
[ 2.1125,   2.2362e+01,   -4.8161e-01,   -8.6027e+01  ],# bin 84  chi2/NDF: 19.299
[ 2.1375,   2.8558e+01,   -6.8171e-01,   -1.3020e+02  ],# bin 85  chi2/NDF: 71.726
[ 2.1625,   2.9778e+01,   -7.0953e-01,   -1.3864e+02  ],# bin 86  chi2/NDF: 74.491
[ 2.1875,   3.1310e+01,   -7.4447e-01,   -1.5034e+02  ],# bin 87  chi2/NDF: 54.662
[ 2.2125,   3.3301e+01,   -7.9927e-01,   -1.6456e+02  ],# bin 88  chi2/NDF: 60.728
[ 2.2375,   3.5441e+01,   -8.8493e-01,   -1.7459e+02  ],# bin 89  chi2/NDF: 38.44
[ 2.2625,   3.5597e+01,   -8.7614e-01,   -1.7561e+02  ],# bin 90  chi2/NDF: 27.366
[ 2.2875,   3.4882e+01,   -8.1012e-01,   -1.7617e+02  ],# bin 91  chi2/NDF: 35.895
[ 2.3125,   3.7231e+01,   -8.8218e-01,   -1.9274e+02  ],# bin 92  chi2/NDF: 13.038
[ 2.3375,   3.9691e+01,   -9.7593e-01,   -2.0423e+02  ],# bin 93  chi2/NDF: 44.048
[ 2.3625,   3.6046e+01,   -8.1657e-01,   -1.8171e+02  ],# bin 94  chi2/NDF: 37.334
[ 2.3875,   2.9075e+01,   -6.3198e-01,   -1.1626e+02  ],# bin 95  chi2/NDF: 45.041
[ 2.4125,   4.3054e+01,   -1.1118e+00,   -2.1459e+02  ],# bin 96  chi2/NDF: 128.59
[ 2.4375,   2.9835e+01,   -6.4951e-01,   -1.1834e+02  ],# bin 97  chi2/NDF: 30.621
[ 2.4625,   5.0006e+01,   -1.3486e+00,   -2.5647e+02  ],# bin 98  chi2/NDF: 82.431
[ 2.4875,   5.0006e+01,   -1.3486e+00,   -2.5647e+02  ],# bin 99 missing par  chi2/NDF: 0, copied from bin 98
] # end CaloSwCalHitsOutOfCone_v8_gam35_unconv


CaloSwCalHitsOutOfCone_v8_ele37 = [
[ 0.0125,   6.2674e+00,   -2.6029e-01,   3.9375e-03  ],# bin 0  chi2/NDF: 63.459, copied from bin 2
[ 0.0375,   6.2674e+00,   -2.6029e-01,   3.9375e-03  ],# bin 1  chi2/NDF: 181.99, copied from bin 2
[ 0.0625,   6.2674e+00,   -2.6029e-01,   3.9375e-03  ],# bin 2  chi2/NDF: 171.04
[ 0.0875,   5.5912e+00,   -1.2426e-01,   -2.3603e-03  ],# bin 3  chi2/NDF: 192.51
[ 0.1125,   5.7097e+00,   -1.4095e-01,   -1.6787e-03  ],# bin 4  chi2/NDF: 175.33
[ 0.1375,   6.0619e+00,   -2.1082e-01,   1.7738e-03  ],# bin 5  chi2/NDF: 195.37
[ 0.1625,   5.9251e+00,   -1.8081e-01,   3.0183e-04  ],# bin 6  chi2/NDF: 96.179
[ 0.1875,   6.0754e+00,   -2.0816e-01,   1.6585e-03  ],# bin 7  chi2/NDF: 148.24
[ 0.2125,   5.6591e+00,   -1.2984e-01,   -1.9993e-03  ],# bin 8  chi2/NDF: 127.08
[ 0.2375,   5.6716e+00,   -1.3659e-01,   -1.3849e-03  ],# bin 9  chi2/NDF: 118.69
[ 0.2625,   5.6174e+00,   -1.2749e-01,   -1.7394e-03  ],# bin 10  chi2/NDF: 144.91
[ 0.2875,   5.9560e+00,   -1.9037e-01,   1.3056e-03  ],# bin 11  chi2/NDF: 167.91
[ 0.3125,   5.7280e+00,   -1.4265e-01,   -1.0041e-03  ],# bin 12  chi2/NDF: 143.37
[ 0.3375,   6.1485e+00,   -2.1739e-01,   2.5682e-03  ],# bin 13  chi2/NDF: 192.47
[ 0.3625,   6.0770e+00,   -2.0296e-01,   1.9377e-03  ],# bin 14  chi2/NDF: 124.43
[ 0.3875,   5.5084e+00,   -9.4374e-02,   -2.8622e-03  ],# bin 15  chi2/NDF: 268.46
[ 0.4125,   6.5109e+00,   -2.6453e-01,   4.3978e-03  ],# bin 16  chi2/NDF: 297.76
[ 0.4375,   6.8466e+00,   -3.3076e-01,   7.7699e-03  ],# bin 17  chi2/NDF: 220.34
[ 0.4625,   6.7154e+00,   -3.0456e-01,   6.6599e-03  ],# bin 18  chi2/NDF: 159.86
[ 0.4875,   6.4100e+00,   -2.4513e-01,   4.1002e-03  ],# bin 19  chi2/NDF: 107.21
[ 0.5125,   6.5794e+00,   -2.7118e-01,   5.2521e-03  ],# bin 20  chi2/NDF: 127.37
[ 0.5375,   6.7768e+00,   -2.9985e-01,   6.3999e-03  ],# bin 21  chi2/NDF: 154.43
[ 0.5625,   7.5042e+00,   -4.2402e-01,   1.1580e-02  ],# bin 22  chi2/NDF: 115.72
[ 0.5875,   7.4526e+00,   -4.0883e-01,   1.0984e-02  ],# bin 23  chi2/NDF: 149.17
[ 0.6125,   7.5367e+00,   -4.1927e-01,   1.1207e-02  ],# bin 24  chi2/NDF: 183.41
[ 0.6375,   8.3804e+00,   -5.4654e-01,   1.6178e-02  ],# bin 25  chi2/NDF: 114.86
[ 0.6625,   9.1324e+00,   -6.5065e-01,   1.9755e-02  ],# bin 26  chi2/NDF: 94.087
[ 0.6875,   9.7133e+00,   -7.3629e-01,   2.3202e-02  ],# bin 27  chi2/NDF: 95.143
[ 0.7125,   1.0279e+01,   -8.2806e-01,   2.6605e-02  ],# bin 28  chi2/NDF: 139.65
[ 0.7375,   1.0178e+01,   -7.9768e-01,   2.5290e-02  ],# bin 29  chi2/NDF: 133.58
[ 0.7625,   1.0178e+01,   -7.9768e-01,   2.5290e-02  ],# bin 30  chi2/NDF: 89.511, copied from bin 29
[ 0.7875,   1.0178e+01,   -7.9768e-01,   2.5290e-02  ],# bin 31  chi2/NDF: 89.429, copied from bin 29
[ 0.8125,   1.9896e+01,   -1.7553e+00,   4.9668e-02  ],# bin 32  chi2/NDF: 63.972, copied from bin 34
[ 0.8375,   1.9896e+01,   -1.7553e+00,   4.9668e-02  ],# bin 33  chi2/NDF: 54.389, copied from bin 34
[ 0.8625,   1.9896e+01,   -1.7553e+00,   4.9668e-02  ],# bin 34  chi2/NDF: 58.764
[ 0.8875,   2.0590e+01,   -1.8813e+00,   5.5297e-02  ],# bin 35  chi2/NDF: 52.063
[ 0.9125,   1.9523e+01,   -1.7111e+00,   4.8377e-02  ],# bin 36  chi2/NDF: 78.37
[ 0.9375,   2.2063e+01,   -2.1457e+00,   6.6818e-02  ],# bin 37  chi2/NDF: 39.094
[ 0.9625,   2.0532e+01,   -1.9140e+00,   5.7822e-02  ],# bin 38  chi2/NDF: 38.771
[ 0.9875,   2.2290e+01,   -2.2025e+00,   6.9760e-02  ],# bin 39  chi2/NDF: 58.982
[ 1.0125,   2.1959e+01,   -2.1606e+00,   6.8172e-02  ],# bin 40  chi2/NDF: 47.937
[ 1.0375,   2.2458e+01,   -2.2795e+00,   7.4466e-02  ],# bin 41  chi2/NDF: 75.287
[ 1.0625,   2.1765e+01,   -2.1764e+00,   7.0049e-02  ],# bin 42  chi2/NDF: 103.53
[ 1.0875,   2.2235e+01,   -2.2908e+00,   7.6496e-02  ],# bin 43  chi2/NDF: 77.323
[ 1.1125,   2.1134e+01,   -2.1346e+00,   7.0413e-02  ],# bin 44  chi2/NDF: 88.718
[ 1.1375,   2.1895e+01,   -2.2849e+00,   7.7911e-02  ],# bin 45  chi2/NDF: 119.54
[ 1.1625,   2.2072e+01,   -2.2743e+00,   7.5603e-02  ],# bin 46  chi2/NDF: 133.45
[ 1.1875,   2.4888e+01,   -2.7349e+00,   9.5186e-02  ],# bin 47  chi2/NDF: 143.35
[ 1.2125,   2.5344e+01,   -2.7678e+00,   9.4513e-02  ],# bin 48  chi2/NDF: 209.97
[ 1.2375,   2.6741e+01,   -2.9819e+00,   1.0356e-01  ],# bin 49  chi2/NDF: 164.07
[ 1.2625,   2.6998e+01,   -2.9715e+00,   1.0060e-01  ],# bin 50  chi2/NDF: 142.69
[ 1.2875,   2.9630e+01,   -3.3802e+00,   1.1744e-01  ],# bin 51  chi2/NDF: 144.73
[ 1.3125,   3.2506e+01,   -3.7630e+00,   1.2908e-01  ],# bin 52  chi2/NDF: 190.43
[ 1.3375,   3.8093e+01,   -4.6728e+00,   1.6639e-01  ],# bin 53  chi2/NDF: 81.771
[ 1.3625,   3.8949e+01,   -4.7405e+00,   1.6430e-01  ],# bin 54  chi2/NDF: 210.86
[ 1.3875,   5.3505e+01,   -6.4180e+00,   2.2518e-01  ],# bin 55  chi2/NDF: 186.52
[ 1.4125,   3.8750e+01,   -4.4688e+00,   1.4321e-01  ],# bin 56  chi2/NDF: 77.911
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -3.6910e+01,   1.2191e+00,   3.8355e+02  ],# bin 60 crack  chi2/NDF: 348.63
[ 1.5375,   -2.8379e+01,   8.8511e-01,   3.0145e+02  ],# bin 61 crack  chi2/NDF: 49.666
[ 1.5625,   -1.8576e+01,   5.3180e-01,   2.4022e+02  ],# bin 62  chi2/NDF: 27.787
[ 1.5875,   -2.2080e+01,   7.2486e-01,   2.4248e+02  ],# bin 63  chi2/NDF: 53.167
[ 1.6125,   -1.6688e+01,   5.5625e-01,   2.0169e+02  ],# bin 64  chi2/NDF: 24.025
[ 1.6375,   -2.0352e+01,   7.1592e-01,   2.2261e+02  ],# bin 65  chi2/NDF: 32.621
[ 1.6625,   -2.3104e+01,   8.1418e-01,   2.4638e+02  ],# bin 66  chi2/NDF: 32.148
[ 1.6875,   -2.5188e+01,   8.6380e-01,   2.6925e+02  ],# bin 67  chi2/NDF: 34.841
[ 1.7125,   -2.2511e+01,   7.8226e-01,   2.5001e+02  ],# bin 68  chi2/NDF: 132.78
[ 1.7375,   -1.5320e+01,   5.9441e-01,   1.8210e+02  ],# bin 69  chi2/NDF: 118.21
[ 1.7625,   -1.4362e+01,   5.9193e-01,   1.7005e+02  ],# bin 70  chi2/NDF: 169.86
[ 1.7875,   -2.3772e+00,   1.8674e-01,   8.3222e+01  ],# bin 71  chi2/NDF: 53.442
[ 1.8125,   6.1910e+00,   -1.2569e-01,   2.8487e+01  ],# bin 72  chi2/NDF: 17.853
[ 1.8375,   6.2189e+00,   -1.3503e-01,   3.1024e+01  ],# bin 73  chi2/NDF: 21.502
[ 1.8625,   5.1562e+00,   -8.1521e-02,   3.6593e+01  ],# bin 74  chi2/NDF: 39.384
[ 1.8875,   9.4616e+00,   -2.3168e-01,   8.3282e+00  ],# bin 75  chi2/NDF: 21.657
[ 1.9125,   6.6813e+00,   -1.0301e-01,   2.3973e+01  ],# bin 76  chi2/NDF: 29.077
[ 1.9375,   7.8212e+00,   -1.2894e-01,   1.5666e+01  ],# bin 77  chi2/NDF: 3.1693
[ 1.9625,   9.0253e+00,   -1.5881e-01,   7.3432e+00  ],# bin 78  chi2/NDF: 11.785
[ 1.9875,   8.9183e+00,   -1.4942e-01,   1.0037e+01  ],# bin 79  chi2/NDF: 1.6666
[ 2.0125,   7.8846e+00,   -9.8870e-02,   1.7942e+01  ],# bin 80  chi2/NDF: 19.246
[ 2.0375,   4.9500e+00,   1.1189e-02,   4.0384e+01  ],# bin 81  chi2/NDF: 16.839
[ 2.0625,   6.3227e+00,   -3.5246e-02,   3.3492e+01  ],# bin 82  chi2/NDF: 11.929
[ 2.0875,   6.1715e+00,   -1.8064e-02,   3.5276e+01  ],# bin 83  chi2/NDF: 7.407
[ 2.1125,   4.6426e+00,   4.4418e-02,   4.5749e+01  ],# bin 84  chi2/NDF: 7.6672
[ 2.1375,   5.3193e+00,   3.0291e-02,   4.1803e+01  ],# bin 85  chi2/NDF: 10.704
[ 2.1625,   6.6804e+00,   -5.2239e-03,   3.2763e+01  ],# bin 86  chi2/NDF: 10.957
[ 2.1875,   8.5230e+00,   -6.1428e-02,   2.0845e+01  ],# bin 87  chi2/NDF: 3.8848
[ 2.2125,   8.1723e+00,   -4.2877e-02,   2.5056e+01  ],# bin 88  chi2/NDF: 7.3224
[ 2.2375,   9.4187e+00,   -8.3578e-02,   1.8041e+01  ],# bin 89  chi2/NDF: 37.13
[ 2.2625,   5.9494e+00,   6.4746e-02,   4.0611e+01  ],# bin 90  chi2/NDF: 146.56
[ 2.2875,   5.4375e+00,   7.6309e-02,   4.9824e+01  ],# bin 91  chi2/NDF: 167.83
[ 2.3125,   1.7263e+01,   -3.8592e-01,   -1.7483e+01  ],# bin 92  chi2/NDF: 159.77
[ 2.3375,   4.8571e+00,   -1.8918e-02,   8.7228e+01  ],# bin 93  chi2/NDF: 119.32
[ 2.3625,   -1.5169e+01,   6.8100e-01,   2.3471e+02  ],# bin 94  chi2/NDF: 84.827
[ 2.3875,   5.9294e+00,   -6.1313e-02,   8.4456e+01  ],# bin 95  chi2/NDF: 180.75
[ 2.4125,   4.1853e+00,   -2.4543e-02,   1.0516e+02  ],# bin 96  chi2/NDF: 81.17
[ 2.4375,   -2.4517e+01,   1.0233e+00,   3.0316e+02  ],# bin 97  chi2/NDF: 11.159
[ 2.4625,   -3.2081e+01,   1.2423e+00,   3.7602e+02  ],# bin 98  chi2/NDF: 145.06
[ 2.4875,   -3.2081e+01,   1.2423e+00,   3.7602e+02  ],# bin 99  chi2/NDF: 145.06, copied from bin 98
] # end CaloSwCalHitsOutOfCone_v8_ele37


CaloSwCalHitsOutOfCone_v8_gam37_conv = [
[ 0.0125,   7.1462e+00,   -3.5107e-01,   5.6628e-03  ],# bin 0  chi2/NDF: 12.743, copied from bin 2
[ 0.0375,   7.1462e+00,   -3.5107e-01,   5.6628e-03  ],# bin 1  chi2/NDF: 34.856, copied from bin 2
[ 0.0625,   7.1462e+00,   -3.5107e-01,   5.6628e-03  ],# bin 2  chi2/NDF: 17.504
[ 0.0875,   7.9768e+00,   -5.0860e-01,   1.3212e-02  ],# bin 3  chi2/NDF: 30.796
[ 0.1125,   6.5941e+00,   -2.4835e-01,   1.2938e-03  ],# bin 4  chi2/NDF: 26.535
[ 0.1375,   8.1077e+00,   -5.3102e-01,   1.4397e-02  ],# bin 5  chi2/NDF: 21.654
[ 0.1625,   6.6909e+00,   -2.6811e-01,   2.3851e-03  ],# bin 6  chi2/NDF: 50.701
[ 0.1875,   7.1613e+00,   -3.4305e-01,   5.4832e-03  ],# bin 7  chi2/NDF: 53.316
[ 0.2125,   6.8183e+00,   -2.8629e-01,   3.2477e-03  ],# bin 8  chi2/NDF: 21.922
[ 0.2375,   7.1392e+00,   -3.4852e-01,   6.2675e-03  ],# bin 9  chi2/NDF: 41.062
[ 0.2625,   7.0057e+00,   -3.1961e-01,   4.8458e-03  ],# bin 10  chi2/NDF: 15.462
[ 0.2875,   7.0617e+00,   -3.3358e-01,   5.7563e-03  ],# bin 11  chi2/NDF: 44.688
[ 0.3125,   6.9315e+00,   -2.9376e-01,   3.4362e-03  ],# bin 12  chi2/NDF: 14.508
[ 0.3375,   6.3453e+00,   -1.9616e-01,   -2.7910e-04  ],# bin 13  chi2/NDF: 18.605
[ 0.3625,   6.8108e+00,   -2.7937e-01,   3.5594e-03  ],# bin 14  chi2/NDF: 33.649
[ 0.3875,   6.9876e+00,   -3.0665e-01,   4.8633e-03  ],# bin 15  chi2/NDF: 16.855
[ 0.4125,   6.6407e+00,   -2.4029e-01,   1.8683e-03  ],# bin 16  chi2/NDF: 34.339
[ 0.4375,   6.6509e+00,   -2.4891e-01,   2.7450e-03  ],# bin 17  chi2/NDF: 17.478
[ 0.4625,   6.9410e+00,   -2.9041e-01,   4.3677e-03  ],# bin 18  chi2/NDF: 14.246
[ 0.4875,   7.8963e+00,   -4.4127e-01,   1.0506e-02  ],# bin 19  chi2/NDF: 10.454
[ 0.5125,   7.4359e+00,   -3.6999e-01,   8.0002e-03  ],# bin 20  chi2/NDF: 4.5538
[ 0.5375,   6.8723e+00,   -2.6587e-01,   3.4737e-03  ],# bin 21  chi2/NDF: 107.52
[ 0.5625,   7.6765e+00,   -3.9805e-01,   8.7522e-03  ],# bin 22  chi2/NDF: 39.96
[ 0.5875,   7.9407e+00,   -4.3830e-01,   1.0626e-02  ],# bin 23  chi2/NDF: 8.5004
[ 0.6125,   7.8880e+00,   -4.3380e-01,   1.0459e-02  ],# bin 24  chi2/NDF: 36.942
[ 0.6375,   9.2899e+00,   -6.5247e-01,   1.9220e-02  ],# bin 25  chi2/NDF: 26.024
[ 0.6625,   1.0854e+01,   -8.8015e-01,   2.7303e-02  ],# bin 26  chi2/NDF: 28.566
[ 0.6875,   1.1479e+01,   -9.7141e-01,   3.1027e-02  ],# bin 27  chi2/NDF: 48.831
[ 0.7125,   1.0687e+01,   -8.1455e-01,   2.3318e-02  ],# bin 28  chi2/NDF: 62.283
[ 0.7375,   1.1793e+01,   -1.0121e+00,   3.2305e-02  ],# bin 29  chi2/NDF: 72.182
[ 0.7625,   1.1793e+01,   -1.0121e+00,   3.2305e-02  ],# bin 30  chi2/NDF: 70.012, copied from bin 29
[ 0.7875,   1.1793e+01,   -1.0121e+00,   3.2305e-02  ],# bin 31  chi2/NDF: 87.131, copied from bin 29
[ 0.8125,   2.3646e+01,   -2.3438e+00,   7.2476e-02  ],# bin 32  chi2/NDF: 38.104, copied from bin 34
[ 0.8375,   2.3646e+01,   -2.3438e+00,   7.2476e-02  ],# bin 33  chi2/NDF: 87.547, copied from bin 34
[ 0.8625,   2.3646e+01,   -2.3438e+00,   7.2476e-02  ],# bin 34  chi2/NDF: 114.52
[ 0.8875,   2.3430e+01,   -2.3205e+00,   7.1963e-02  ],# bin 35  chi2/NDF: 73.365
[ 0.9125,   2.1586e+01,   -2.0633e+00,   6.2828e-02  ],# bin 36  chi2/NDF: 58.347
[ 0.9375,   2.2186e+01,   -2.1674e+00,   6.7456e-02  ],# bin 37  chi2/NDF: 46.678
[ 0.9625,   2.2755e+01,   -2.2693e+00,   7.1618e-02  ],# bin 38  chi2/NDF: 79.734
[ 0.9875,   2.1788e+01,   -2.1456e+00,   6.8064e-02  ],# bin 39  chi2/NDF: 75.488
[ 1.0125,   2.3003e+01,   -2.3342e+00,   7.5136e-02  ],# bin 40  chi2/NDF: 28.264
[ 1.0375,   2.3283e+01,   -2.4100e+00,   7.9448e-02  ],# bin 41  chi2/NDF: 60.683
[ 1.0625,   2.2897e+01,   -2.3762e+00,   7.8865e-02  ],# bin 42  chi2/NDF: 94.573
[ 1.0875,   2.3154e+01,   -2.4540e+00,   8.3574e-02  ],# bin 43  chi2/NDF: 106.14
[ 1.1125,   2.1561e+01,   -2.1709e+00,   7.0772e-02  ],# bin 44  chi2/NDF: 43.447
[ 1.1375,   2.1489e+01,   -2.1891e+00,   7.3037e-02  ],# bin 45  chi2/NDF: 25.552
[ 1.1625,   2.4033e+01,   -2.5847e+00,   8.8108e-02  ],# bin 46  chi2/NDF: 51.498
[ 1.1875,   2.4659e+01,   -2.6683e+00,   9.1526e-02  ],# bin 47  chi2/NDF: 125.46
[ 1.2125,   2.6063e+01,   -2.8281e+00,   9.4961e-02  ],# bin 48  chi2/NDF: 40.252
[ 1.2375,   2.6496e+01,   -2.8934e+00,   9.8367e-02  ],# bin 49  chi2/NDF: 49.052
[ 1.2625,   2.7672e+01,   -3.0504e+00,   1.0283e-01  ],# bin 50  chi2/NDF: 106.18
[ 1.2875,   3.1446e+01,   -3.6354e+00,   1.2649e-01  ],# bin 51  chi2/NDF: 20.691
[ 1.3125,   3.0999e+01,   -3.4703e+00,   1.1563e-01  ],# bin 52  chi2/NDF: 36.215
[ 1.3375,   3.4703e+01,   -4.0494e+00,   1.3842e-01  ],# bin 53  chi2/NDF: 110.63
[ 1.3625,   3.6651e+01,   -4.2869e+00,   1.4320e-01  ],# bin 54  chi2/NDF: 92.866
[ 1.3875,   6.2169e+01,   -7.7427e+00,   2.7685e-01  ],# bin 55  chi2/NDF: 225.93
[ 1.4125,   4.9302e+01,   -6.0586e+00,   2.0334e-01  ],# bin 56  chi2/NDF: 57.365
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack missing par  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack missing par  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -3.5220e+01,   1.2054e+00,   3.6794e+02  ],# bin 60 crack  chi2/NDF: 731.5
[ 1.5375,   -3.3147e+01,   1.0630e+00,   3.3375e+02  ],# bin 61 crack  chi2/NDF: 38.849
[ 1.5625,   -1.7980e+01,   5.1419e-01,   2.3480e+02  ],# bin 62  chi2/NDF: 47.884
[ 1.5875,   -2.2766e+01,   7.6247e-01,   2.4370e+02  ],# bin 63  chi2/NDF: 33.701
[ 1.6125,   -1.9785e+01,   6.8000e-01,   2.1934e+02  ],# bin 64  chi2/NDF: 27.81
[ 1.6375,   -1.9004e+01,   6.7388e-01,   2.1088e+02  ],# bin 65  chi2/NDF: 14.204
[ 1.6625,   -1.7600e+01,   6.2444e-01,   2.0547e+02  ],# bin 66  chi2/NDF: 16.432
[ 1.6875,   -1.8175e+01,   6.1646e-01,   2.1880e+02  ],# bin 67  chi2/NDF: 3.9622
[ 1.7125,   -2.0025e+01,   7.0279e-01,   2.2921e+02  ],# bin 68  chi2/NDF: 35.882
[ 1.7375,   -1.2039e+01,   4.8331e-01,   1.5673e+02  ],# bin 69  chi2/NDF: 59.716
[ 1.7625,   -1.1014e+01,   4.7715e-01,   1.4447e+02  ],# bin 70  chi2/NDF: 85.149
[ 1.7875,   -1.0970e+00,   1.5124e-01,   7.0842e+01  ],# bin 71  chi2/NDF: 27.366
[ 1.8125,   5.5011e+00,   -8.0999e-02,   2.7200e+01  ],# bin 72  chi2/NDF: 19.102
[ 1.8375,   1.1163e+01,   -2.8420e-01,   -1.0992e+01  ],# bin 73  chi2/NDF: 40.514
[ 1.8625,   1.1260e+01,   -2.7978e-01,   -1.1799e+01  ],# bin 74  chi2/NDF: 30.363
[ 1.8875,   1.4716e+01,   -3.8866e-01,   -3.6168e+01  ],# bin 75  chi2/NDF: 42.895
[ 1.9125,   1.2195e+01,   -2.8497e-01,   -1.9097e+01  ],# bin 76  chi2/NDF: 18.423
[ 1.9375,   1.2709e+01,   -2.8788e-01,   -2.3124e+01  ],# bin 77  chi2/NDF: 23.246
[ 1.9625,   9.0012e+00,   -1.3547e-01,   1.3918e+00  ],# bin 78  chi2/NDF: 11.775
[ 1.9875,   9.4947e+00,   -1.4157e-01,   -1.5540e+00  ],# bin 79  chi2/NDF: 11.686
[ 2.0125,   9.8241e+00,   -1.4094e-01,   -3.4890e+00  ],# bin 80  chi2/NDF: 29.131
[ 2.0375,   9.4261e+00,   -1.3471e-01,   3.9631e+00  ],# bin 81  chi2/NDF: 37.174
[ 2.0625,   6.1407e+00,   -8.4375e-03,   2.8443e+01  ],# bin 82  chi2/NDF: 48.712
[ 2.0875,   -3.7083e+00,   3.4253e-01,   1.0049e+02  ],# bin 83  chi2/NDF: 34.569
[ 2.1125,   -3.4876e+00,   3.4499e-01,   9.8668e+01  ],# bin 84  chi2/NDF: 174.62
[ 2.1375,   -4.6608e+00,   4.0193e-01,   1.0678e+02  ],# bin 85  chi2/NDF: 49.116
[ 2.1625,   -3.6829e+00,   3.6887e-01,   1.0247e+02  ],# bin 86  chi2/NDF: 53.571
[ 2.1875,   -1.7179e+00,   3.1188e-01,   8.8810e+01  ],# bin 87  chi2/NDF: 22.424
[ 2.2125,   -1.5809e+00,   3.1659e-01,   8.8793e+01  ],# bin 88  chi2/NDF: 21.213
[ 2.2375,   8.4490e-01,   2.5085e-01,   7.0245e+01  ],# bin 89  chi2/NDF: 14.554
[ 2.2625,   9.4194e+00,   -6.9113e-02,   1.5543e+01  ],# bin 90  chi2/NDF: 15.473
[ 2.2875,   1.0503e+01,   -9.2420e-02,   8.6936e+00  ],# bin 91  chi2/NDF: 13.184
[ 2.3125,   8.4912e+00,   -3.9971e-02,   3.0841e+01  ],# bin 92  chi2/NDF: 5.4147
[ 2.3375,   2.6242e+00,   1.6454e-01,   7.5203e+01  ],# bin 93  chi2/NDF: 22.005
[ 2.3625,   -1.3282e+00,   3.0214e-01,   1.0848e+02  ],# bin 94  chi2/NDF: 114.87
[ 2.3875,   -8.8242e+00,   5.8120e-01,   1.6017e+02  ],# bin 95  chi2/NDF: 139.28
[ 2.4125,   1.3513e+00,   1.9578e-01,   9.6394e+01  ],# bin 96  chi2/NDF: 55.67
[ 2.4375,   -1.6136e+01,   8.1445e-01,   2.2137e+02  ],# bin 97  chi2/NDF: 48.981
[ 2.4625,   -1.2480e+01,   6.5558e-01,   2.0846e+02  ],# bin 98  chi2/NDF: 41.578
[ 2.4875,   -1.2480e+01,   6.5558e-01,   2.0846e+02  ],# bin 99  chi2/NDF: 41.578, copied from bin 98
] # end CaloSwCalHitsOutOfCone_v8_gam37_conv


CaloSwCalHitsOutOfCone_v8_gam37_unconv = [
[ 0.0125,   9.2738e-01,   6.3795e-01,   -3.3512e-02  ],# bin 0  chi2/NDF: 101.76, copied from bin 2
[ 0.0375,   9.2738e-01,   6.3795e-01,   -3.3512e-02  ],# bin 1  chi2/NDF: 56.629, copied from bin 2
[ 0.0625,   9.2738e-01,   6.3795e-01,   -3.3512e-02  ],# bin 2  chi2/NDF: 49.663
[ 0.0875,   1.4322e+00,   5.5795e-01,   -3.0051e-02  ],# bin 3  chi2/NDF: 51.956
[ 0.1125,   1.2521e+00,   5.8642e-01,   -3.1124e-02  ],# bin 4  chi2/NDF: 38.594
[ 0.1375,   4.4229e-01,   7.3012e-01,   -3.7318e-02  ],# bin 5  chi2/NDF: 90.076
[ 0.1625,   1.4042e+00,   5.6253e-01,   -3.0054e-02  ],# bin 6  chi2/NDF: 124.62
[ 0.1875,   1.1871e+00,   5.9612e-01,   -3.1298e-02  ],# bin 7  chi2/NDF: 51.023
[ 0.2125,   9.2805e-01,   6.3653e-01,   -3.2835e-02  ],# bin 8  chi2/NDF: 37.788
[ 0.2375,   2.0276e+00,   4.5086e-01,   -2.4852e-02  ],# bin 9  chi2/NDF: 84.068
[ 0.2625,   1.2566e+00,   5.7473e-01,   -2.9771e-02  ],# bin 10  chi2/NDF: 51.024
[ 0.2875,   1.4904e+00,   5.3531e-01,   -2.7862e-02  ],# bin 11  chi2/NDF: 94.185
[ 0.3125,   1.8241e+00,   4.7944e-01,   -2.5525e-02  ],# bin 12  chi2/NDF: 115.3
[ 0.3375,   1.6410e+00,   4.9711e-01,   -2.5527e-02  ],# bin 13  chi2/NDF: 42.034
[ 0.3625,   1.5460e+00,   5.1167e-01,   -2.6072e-02  ],# bin 14  chi2/NDF: 44.525
[ 0.3875,   2.4447e+00,   3.6875e-01,   -2.0107e-02  ],# bin 15  chi2/NDF: 93.165
[ 0.4125,   2.5825e+00,   3.3290e-01,   -1.8164e-02  ],# bin 16  chi2/NDF: 51.031
[ 0.4375,   1.3001e+00,   5.4025e-01,   -2.6197e-02  ],# bin 17  chi2/NDF: 10.49
[ 0.4625,   1.6138e+00,   4.9287e-01,   -2.4274e-02  ],# bin 18  chi2/NDF: 24.304
[ 0.4875,   1.9012e+00,   4.3496e-01,   -2.1384e-02  ],# bin 19  chi2/NDF: 55.72
[ 0.5125,   1.6924e+00,   4.5809e-01,   -2.1706e-02  ],# bin 20  chi2/NDF: 50.54
[ 0.5375,   1.9398e+00,   4.2565e-01,   -2.0615e-02  ],# bin 21  chi2/NDF: 68.189
[ 0.5625,   1.9539e+00,   4.1893e-01,   -2.0286e-02  ],# bin 22  chi2/NDF: 116.5
[ 0.5875,   3.2211e+00,   2.1743e-01,   -1.2132e-02  ],# bin 23  chi2/NDF: 46.556
[ 0.6125,   2.6207e+00,   3.2084e-01,   -1.6515e-02  ],# bin 24  chi2/NDF: 70.91
[ 0.6375,   3.3702e+00,   2.0877e-01,   -1.2209e-02  ],# bin 25  chi2/NDF: 54.149
[ 0.6625,   4.1669e+00,   8.7672e-02,   -7.7978e-03  ],# bin 26  chi2/NDF: 49.808
[ 0.6875,   4.5444e+00,   2.8393e-02,   -5.2116e-03  ],# bin 27  chi2/NDF: 43.958
[ 0.7125,   3.9126e+00,   1.3409e-01,   -9.9165e-03  ],# bin 28  chi2/NDF: 96.564
[ 0.7375,   5.0845e+00,   -5.7509e-02,   -1.9311e-03  ],# bin 29  chi2/NDF: 95.37
[ 0.7625,   5.0845e+00,   -5.7509e-02,   -1.9311e-03  ],# bin 30  chi2/NDF: 53.757, copied from bin 29
[ 0.7875,   5.0845e+00,   -5.7509e-02,   -1.9311e-03  ],# bin 31  chi2/NDF: 92.372, copied from bin 29
[ 0.8125,   6.1341e+00,   8.0342e-02,   -1.1728e-02  ],# bin 32  chi2/NDF: 91.671, copied from bin 34
[ 0.8375,   6.1341e+00,   8.0342e-02,   -1.1728e-02  ],# bin 33  chi2/NDF: 78.297, copied from bin 34
[ 0.8625,   6.1341e+00,   8.0342e-02,   -1.1728e-02  ],# bin 34  chi2/NDF: 103.08
[ 0.8875,   7.9100e+00,   -2.0817e-01,   -2.2016e-04  ],# bin 35  chi2/NDF: 17.806
[ 0.9125,   6.3243e+00,   3.0895e-02,   -9.3506e-03  ],# bin 36  chi2/NDF: 15.045
[ 0.9375,   7.6809e+00,   -1.9796e-01,   2.8635e-04  ],# bin 37  chi2/NDF: 63.127
[ 0.9625,   7.5622e+00,   -1.7596e-01,   -8.5499e-04  ],# bin 38  chi2/NDF: 100.37
[ 0.9875,   9.9841e+00,   -5.5169e-01,   1.3857e-02  ],# bin 39  chi2/NDF: 48.881
[ 1.0125,   8.5072e+00,   -3.1960e-01,   4.6838e-03  ],# bin 40  chi2/NDF: 8.8817
[ 1.0375,   8.9344e+00,   -3.8993e-01,   7.8063e-03  ],# bin 41  chi2/NDF: 125.14
[ 1.0625,   9.5109e+00,   -4.8358e-01,   1.1123e-02  ],# bin 42  chi2/NDF: 20.404
[ 1.0875,   1.0034e+01,   -5.7705e-01,   1.5572e-02  ],# bin 43  chi2/NDF: 27.421
[ 1.1125,   9.5800e+00,   -4.9601e-01,   1.1702e-02  ],# bin 44  chi2/NDF: 102.58
[ 1.1375,   1.0753e+01,   -6.9105e-01,   2.0108e-02  ],# bin 45  chi2/NDF: 26.793
[ 1.1625,   1.0443e+01,   -6.2061e-01,   1.6101e-02  ],# bin 46  chi2/NDF: 29.012
[ 1.1875,   1.1012e+01,   -7.1268e-01,   2.0507e-02  ],# bin 47  chi2/NDF: 111.86
[ 1.2125,   1.1395e+01,   -7.4827e-01,   2.0550e-02  ],# bin 48  chi2/NDF: 33.509
[ 1.2375,   1.3309e+01,   -1.0322e+00,   3.1699e-02  ],# bin 49  chi2/NDF: 54.142
[ 1.2625,   1.2232e+01,   -8.3063e-01,   2.2188e-02  ],# bin 50  chi2/NDF: 57.506
[ 1.2875,   1.5693e+01,   -1.3352e+00,   4.1219e-02  ],# bin 51  chi2/NDF: 65.411
[ 1.3125,   1.7103e+01,   -1.4993e+00,   4.4549e-02  ],# bin 52  chi2/NDF: 51.15
[ 1.3375,   1.9507e+01,   -1.8455e+00,   5.7137e-02  ],# bin 53  chi2/NDF: 3.5849
[ 1.3625,   2.0372e+01,   -1.9121e+00,   5.4876e-02  ],# bin 54  chi2/NDF: 4.2765
[ 1.3875,   3.4011e+01,   -3.4819e+00,   1.1253e-01  ],# bin 55  chi2/NDF: 330.93
[ 1.4125,   2.8774e+01,   -3.0850e+00,   9.3285e-02  ],# bin 56  chi2/NDF: 443.34
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack missing par  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack missing par  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -6.6721e+00,   6.1002e-02,   1.6242e+02  ],# bin 60 crack  chi2/NDF: 640.57
[ 1.5375,   -1.4141e+01,   4.7744e-01,   1.7190e+02  ],# bin 61 crack  chi2/NDF: 52.448
[ 1.5625,   -2.2660e+00,   1.2924e-01,   7.4870e+01  ],# bin 62  chi2/NDF: 31.156
[ 1.5875,   1.8553e+00,   1.5181e-02,   3.7869e+01  ],# bin 63  chi2/NDF: 42.12
[ 1.6125,   3.8790e-01,   7.3720e-02,   4.9053e+01  ],# bin 64  chi2/NDF: 52.245
[ 1.6375,   -9.1663e-01,   1.1207e-01,   6.2324e+01  ],# bin 65  chi2/NDF: 16.987
[ 1.6625,   -3.0629e+00,   1.8167e-01,   8.2771e+01  ],# bin 66  chi2/NDF: 25.952
[ 1.6875,   -7.6557e+00,   3.2671e-01,   1.2161e+02  ],# bin 67  chi2/NDF: 13.195
[ 1.7125,   -2.8150e+00,   2.0438e-01,   7.9629e+01  ],# bin 68  chi2/NDF: 11.759
[ 1.7375,   3.0797e+00,   2.4180e-02,   3.1633e+01  ],# bin 69  chi2/NDF: 95.044
[ 1.7625,   5.5682e+00,   -5.6591e-02,   1.5134e+01  ],# bin 70  chi2/NDF: 87.911
[ 1.7875,   1.0624e+01,   -2.1491e-01,   -2.3120e+01  ],# bin 71  chi2/NDF: 83.326
[ 1.8125,   1.7701e+01,   -4.4177e-01,   -7.5362e+01  ],# bin 72  chi2/NDF: 39.648
[ 1.8375,   2.7134e+01,   -7.6961e-01,   -1.4121e+02  ],# bin 73  chi2/NDF: 66.101
[ 1.8625,   2.7643e+01,   -7.8338e-01,   -1.4345e+02  ],# bin 74  chi2/NDF: 67.329
[ 1.8875,   2.7291e+01,   -7.6418e-01,   -1.3964e+02  ],# bin 75  chi2/NDF: 46.455
[ 1.9125,   2.3566e+01,   -6.2043e-01,   -1.1331e+02  ],# bin 76  chi2/NDF: 29.65
[ 1.9375,   3.2620e+01,   -9.2786e-01,   -1.7692e+02  ],# bin 77  chi2/NDF: 112.95
[ 1.9625,   2.8336e+01,   -7.6727e-01,   -1.4590e+02  ],# bin 78  chi2/NDF: 55.47
[ 1.9875,   2.7133e+01,   -7.0556e-01,   -1.3867e+02  ],# bin 79  chi2/NDF: 12.142
[ 2.0125,   2.2936e+01,   -5.4871e-01,   -1.0808e+02  ],# bin 80  chi2/NDF: 70.009
[ 2.0375,   2.5786e+01,   -6.5521e-01,   -1.2379e+02  ],# bin 81  chi2/NDF: 38.824
[ 2.0625,   2.4457e+01,   -5.8932e-01,   -1.1539e+02  ],# bin 82  chi2/NDF: 78.093
[ 2.0875,   2.5066e+01,   -5.9181e-01,   -1.2077e+02  ],# bin 83  chi2/NDF: 31.322
[ 2.1125,   2.1364e+01,   -4.7771e-01,   -8.8735e+01  ],# bin 84  chi2/NDF: 41.496
[ 2.1375,   2.2343e+01,   -4.9712e-01,   -9.5956e+01  ],# bin 85  chi2/NDF: 23.671
[ 2.1625,   2.7557e+01,   -6.6846e-01,   -1.3222e+02  ],# bin 86  chi2/NDF: 65.007
[ 2.1875,   2.7848e+01,   -6.5997e-01,   -1.3574e+02  ],# bin 87  chi2/NDF: 42.121
[ 2.2125,   3.0711e+01,   -7.4502e-01,   -1.5638e+02  ],# bin 88  chi2/NDF: 36.574
[ 2.2375,   3.2047e+01,   -8.0279e-01,   -1.6100e+02  ],# bin 89  chi2/NDF: 30.101
[ 2.2625,   3.6982e+01,   -9.5803e-01,   -1.9676e+02  ],# bin 90  chi2/NDF: 33.046
[ 2.2875,   3.5816e+01,   -8.8371e-01,   -1.9279e+02  ],# bin 91  chi2/NDF: 48.627
[ 2.3125,   3.4764e+01,   -8.4092e-01,   -1.8462e+02  ],# bin 92  chi2/NDF: 20.046
[ 2.3375,   4.0497e+01,   -1.0445e+00,   -2.2079e+02  ],# bin 93  chi2/NDF: 36.61
[ 2.3625,   3.8553e+01,   -9.4645e-01,   -2.0948e+02  ],# bin 94  chi2/NDF: 45.705
[ 2.3875,   2.4000e+01,   -4.9578e-01,   -9.1139e+01  ],# bin 95  chi2/NDF: 23.314
[ 2.4125,   2.9967e+01,   -7.0420e-01,   -1.3126e+02  ],# bin 96  chi2/NDF: 81.881
[ 2.4375,   2.7479e+01,   -6.1125e-01,   -1.1255e+02  ],# bin 97  chi2/NDF: 43.85
[ 2.4625,   5.3621e+01,   -1.5075e+00,   -2.9604e+02  ],# bin 98  chi2/NDF: 96.99
[ 2.4875,   5.3621e+01,   -1.5075e+00,   -2.9604e+02  ],# bin 99 missing par  chi2/NDF: 0, copied from bin 98
] # end CaloSwCalHitsOutOfCone_v8_gam37_unconv


CaloSwCalHitsEfront_P2_v8_ele = [
[ 0.0125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 0  chi2/NDF: 0, copied from bin 2
[ 0.0375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 1  chi2/NDF: 0, copied from bin 2
[ 0.0625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 2  chi2/NDF: 0
[ 0.0875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 3  chi2/NDF: 0
[ 0.1125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 4  chi2/NDF: 0
[ 0.1375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 5  chi2/NDF: 0
[ 0.1625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 6  chi2/NDF: 0
[ 0.1875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 7  chi2/NDF: 0
[ 0.2125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 8  chi2/NDF: 0
[ 0.2375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 9  chi2/NDF: 0
[ 0.2625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 10  chi2/NDF: 0
[ 0.2875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 11  chi2/NDF: 0
[ 0.3125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 12  chi2/NDF: 0
[ 0.3375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 13  chi2/NDF: 0
[ 0.3625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 14  chi2/NDF: 0
[ 0.3875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 15  chi2/NDF: 0
[ 0.4125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 16  chi2/NDF: 0
[ 0.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 17  chi2/NDF: 0
[ 0.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 18  chi2/NDF: 0
[ 0.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 19  chi2/NDF: 0
[ 0.5125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 20  chi2/NDF: 0
[ 0.5375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 21  chi2/NDF: 0
[ 0.5625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 22  chi2/NDF: 0
[ 0.5875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 23  chi2/NDF: 0
[ 0.6125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 24  chi2/NDF: 0
[ 0.6375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 25  chi2/NDF: 0
[ 0.6625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 26  chi2/NDF: 0
[ 0.6875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 27  chi2/NDF: 0
[ 0.7125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 28  chi2/NDF: 0
[ 0.7375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 29  chi2/NDF: 0
[ 0.7625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 30  chi2/NDF: 0, copied from bin 29
[ 0.7875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 31  chi2/NDF: 0, copied from bin 29
[ 0.8125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 32  chi2/NDF: 0, copied from bin 34
[ 0.8375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 33  chi2/NDF: 0, copied from bin 34
[ 0.8625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 34  chi2/NDF: 0
[ 0.8875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 35  chi2/NDF: 0
[ 0.9125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 36  chi2/NDF: 0
[ 0.9375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 37  chi2/NDF: 0
[ 0.9625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 38  chi2/NDF: 0
[ 0.9875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 39  chi2/NDF: 0
[ 1.0125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 40  chi2/NDF: 0
[ 1.0375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 41  chi2/NDF: 0
[ 1.0625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 42  chi2/NDF: 0
[ 1.0875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 43  chi2/NDF: 0
[ 1.1125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 44  chi2/NDF: 0
[ 1.1375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 45  chi2/NDF: 0
[ 1.1625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 46  chi2/NDF: 0
[ 1.1875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 47  chi2/NDF: 0
[ 1.2125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 48  chi2/NDF: 0
[ 1.2375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 49  chi2/NDF: 0
[ 1.2625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 50  chi2/NDF: 0
[ 1.2875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 51  chi2/NDF: 0
[ 1.3125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 52  chi2/NDF: 0
[ 1.3375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 53  chi2/NDF: 0
[ 1.3625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 54  chi2/NDF: 0
[ 1.3875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 55  chi2/NDF: 0
[ 1.4125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 56  chi2/NDF: 0
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -6.7651e-07,   5.8097e-08,   7.5030e-04  ],# bin 60 crack  chi2/NDF: 1.731
[ 1.5375,   -8.1594e-06,   3.9720e-08,   4.4138e-02  ],# bin 61 crack  chi2/NDF: 14.617
[ 1.5625,   1.6731e-05,   -1.1942e-08,   8.6095e-03  ],# bin 62  chi2/NDF: 2.5493
[ 1.5875,   1.3437e-05,   -8.3738e-09,   1.7251e-02  ],# bin 63  chi2/NDF: 0.53526
[ 1.6125,   2.6608e-06,   5.3041e-09,   3.1423e-03  ],# bin 64  chi2/NDF: 2.4529
[ 1.6375,   4.8281e-05,   -4.9650e-08,   1.3437e-02  ],# bin 65  chi2/NDF: 5.3729
[ 1.6625,   3.9441e-05,   -3.5362e-08,   2.7269e-02  ],# bin 66  chi2/NDF: 5.7265
[ 1.6875,   2.7606e-05,   -2.0733e-08,   4.5190e-02  ],# bin 67  chi2/NDF: 12.422
[ 1.7125,   7.3579e-05,   -8.0878e-08,   1.9539e-03  ],# bin 68  chi2/NDF: 5.2363
[ 1.7375,   8.8844e-05,   -1.0254e-07,   -1.3353e-02  ],# bin 69  chi2/NDF: 16.493
[ 1.7625,   7.6198e-05,   -8.6477e-08,   -1.9422e-02  ],# bin 70  chi2/NDF: 16.302
[ 1.7875,   4.3972e-05,   -4.3308e-08,   2.7697e-03  ],# bin 71  chi2/NDF: 3.3884
[ 1.8125,   -9.8532e+00,   3.6287e+00,   1.7120e-04  ],# bin 72  chi2/NDF: 0.35321
[ 1.8375,   9.9654e+00,   4.0191e+00,   -4.1381e-04  ],# bin 73  chi2/NDF: 0.79983
[ 1.8625,   1.5221e+01,   3.4878e+00,   -2.2816e-03  ],# bin 74  chi2/NDF: 1.535
[ 1.8875,   3.2487e+01,   3.4624e+00,   -1.8687e-03  ],# bin 75  chi2/NDF: 0.58394
[ 1.9125,   3.4484e+01,   3.1249e+00,   -7.5167e-04  ],# bin 76  chi2/NDF: 1.8742
[ 1.9375,   1.2217e+01,   3.1663e+00,   -1.5046e-03  ],# bin 77  chi2/NDF: 1.2247
[ 1.9625,   3.0000e+01,   2.5901e+00,   -7.5243e-04  ],# bin 78  chi2/NDF: 0.75435
[ 1.9875,   4.3305e+01,   2.7155e+00,   -1.1462e-03  ],# bin 79  chi2/NDF: 0.72206
[ 2.0125,   1.9726e+01,   2.9859e+00,   -4.2887e-04  ],# bin 80  chi2/NDF: 0.55163
[ 2.0375,   3.1968e+01,   2.9500e+00,   -7.9473e-04  ],# bin 81  chi2/NDF: 1.9351
[ 2.0625,   4.5464e+01,   2.5109e+00,   -7.1081e-04  ],# bin 82  chi2/NDF: 2.4049
[ 2.0875,   3.6398e+01,   2.9792e+00,   -1.2402e-03  ],# bin 83  chi2/NDF: 2.5999
[ 2.1125,   2.8509e+01,   3.0192e+00,   -6.9972e-04  ],# bin 84  chi2/NDF: 0.40975
[ 2.1375,   2.6820e+01,   2.9107e+00,   -1.1713e-03  ],# bin 85  chi2/NDF: 2.0862
[ 2.1625,   1.8149e+01,   2.8879e+00,   -1.1235e-03  ],# bin 86  chi2/NDF: 2.5666
[ 2.1875,   2.8199e+01,   2.7709e+00,   -1.2631e-03  ],# bin 87  chi2/NDF: 0.42228
[ 2.2125,   2.8524e+01,   2.4550e+00,   -8.0745e-04  ],# bin 88  chi2/NDF: 1.6704
[ 2.2375,   2.5246e+01,   2.7410e+00,   -1.6050e-03  ],# bin 89  chi2/NDF: 5.6928
[ 2.2625,   3.1853e+01,   2.0859e+00,   -3.5986e-04  ],# bin 90  chi2/NDF: 0.52159
[ 2.2875,   2.7258e+01,   1.8904e+00,   -2.8459e-04  ],# bin 91  chi2/NDF: 2.1906
[ 2.3125,   -7.8318e-01,   9.6248e-01,   -1.4925e-03  ],# bin 92  chi2/NDF: 3.061
[ 2.3375,   -1.7773e+01,   3.9057e+00,   -1.8934e-03  ],# bin 93  chi2/NDF: 1.0055
[ 2.3625,   7.2341e+00,   3.0263e+00,   4.0139e-04  ],# bin 94  chi2/NDF: 0.42907
[ 2.3875,   -1.3768e+01,   3.8829e+00,   -9.8564e-04  ],# bin 95  chi2/NDF: 0.82456
[ 2.4125,   -5.5392e+01,   4.1899e+00,   -2.0190e-03  ],# bin 96  chi2/NDF: 0.32237
[ 2.4375,   2.5779e+01,   3.5323e+00,   -7.1036e-04  ],# bin 97  chi2/NDF: 0.58884
[ 2.4625,   -9.0599e+00,   3.9067e+00,   -1.2508e-03  ],# bin 98  chi2/NDF: 0.54225
[ 2.4875,   -9.0599e+00,   3.9067e+00,   -1.2508e-03  ],# bin 99 missing par  chi2/NDF: 0, copied from bin 98
] # end CaloSwCalHitsEfront_P2_v8_ele


CaloSwCalHitsEfront_P2_v8_gam_conv = [
[ 0.0125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 0  chi2/NDF: 0, copied from bin 2
[ 0.0375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 1  chi2/NDF: 0, copied from bin 2
[ 0.0625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 2  chi2/NDF: 0
[ 0.0875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 3  chi2/NDF: 0
[ 0.1125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 4  chi2/NDF: 0
[ 0.1375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 5  chi2/NDF: 0
[ 0.1625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 6  chi2/NDF: 0
[ 0.1875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 7  chi2/NDF: 0
[ 0.2125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 8  chi2/NDF: 0
[ 0.2375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 9  chi2/NDF: 0
[ 0.2625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 10  chi2/NDF: 0
[ 0.2875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 11  chi2/NDF: 0
[ 0.3125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 12  chi2/NDF: 0
[ 0.3375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 13  chi2/NDF: 0
[ 0.3625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 14  chi2/NDF: 0
[ 0.3875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 15  chi2/NDF: 0
[ 0.4125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 16  chi2/NDF: 0
[ 0.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 17  chi2/NDF: 0
[ 0.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 18  chi2/NDF: 0
[ 0.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 19  chi2/NDF: 0
[ 0.5125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 20  chi2/NDF: 0
[ 0.5375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 21  chi2/NDF: 0
[ 0.5625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 22  chi2/NDF: 0
[ 0.5875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 23  chi2/NDF: 0
[ 0.6125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 24  chi2/NDF: 0
[ 0.6375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 25  chi2/NDF: 0
[ 0.6625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 26  chi2/NDF: 0
[ 0.6875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 27  chi2/NDF: 0
[ 0.7125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 28  chi2/NDF: 0
[ 0.7375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 29  chi2/NDF: 0
[ 0.7625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 30  chi2/NDF: 0, copied from bin 29
[ 0.7875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 31  chi2/NDF: 0, copied from bin 29
[ 0.8125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 32  chi2/NDF: 0, copied from bin 34
[ 0.8375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 33  chi2/NDF: 0, copied from bin 34
[ 0.8625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 34  chi2/NDF: 0
[ 0.8875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 35  chi2/NDF: 0
[ 0.9125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 36  chi2/NDF: 0
[ 0.9375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 37  chi2/NDF: 0
[ 0.9625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 38  chi2/NDF: 0
[ 0.9875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 39  chi2/NDF: 0
[ 1.0125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 40  chi2/NDF: 0
[ 1.0375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 41  chi2/NDF: 0
[ 1.0625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 42  chi2/NDF: 0
[ 1.0875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 43  chi2/NDF: 0
[ 1.1125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 44  chi2/NDF: 0
[ 1.1375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 45  chi2/NDF: 0
[ 1.1625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 46  chi2/NDF: 0
[ 1.1875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 47  chi2/NDF: 0
[ 1.2125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 48  chi2/NDF: 0
[ 1.2375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 49  chi2/NDF: 0
[ 1.2625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 50  chi2/NDF: 0
[ 1.2875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 51  chi2/NDF: 0
[ 1.3125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 52  chi2/NDF: 0
[ 1.3375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 53  chi2/NDF: 0
[ 1.3625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 54  chi2/NDF: 0
[ 1.3875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 55  chi2/NDF: 0
[ 1.4125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 56  chi2/NDF: 0
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack missing par  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack missing par  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -2.6325e-05,   1.0208e-07,   -1.7956e-03  ],# bin 60 crack  chi2/NDF: 1.1448
[ 1.5375,   1.7162e-05,   5.7196e-09,   2.8264e-02  ],# bin 61 crack  chi2/NDF: 10.809
[ 1.5625,   2.6739e-05,   -2.5615e-08,   1.8646e-03  ],# bin 62  chi2/NDF: 1.6666
[ 1.5875,   8.6561e-06,   -1.4819e-09,   -1.5133e-03  ],# bin 63  chi2/NDF: 0.36249
[ 1.6125,   1.4671e-05,   -9.1085e-09,   9.7925e-03  ],# bin 64  chi2/NDF: 4.1416
[ 1.6375,   4.5956e-05,   -4.5506e-08,   1.7247e-02  ],# bin 65  chi2/NDF: 3.1224
[ 1.6625,   3.1173e-05,   -2.5227e-08,   2.7974e-02  ],# bin 66  chi2/NDF: 2.0277
[ 1.6875,   -5.7580e-06,   2.4065e-08,   2.5491e-02  ],# bin 67  chi2/NDF: 2.7534
[ 1.7125,   6.0892e-05,   -6.3725e-08,   -3.2421e-03  ],# bin 68  chi2/NDF: 3.7853
[ 1.7375,   6.1722e-05,   -6.5532e-08,   -1.6870e-02  ],# bin 69  chi2/NDF: 6.48
[ 1.7625,   7.4861e-05,   -8.0499e-08,   -1.4712e-02  ],# bin 70  chi2/NDF: 4.6118
[ 1.7875,   4.9312e-05,   -4.9795e-08,   -8.8403e-03  ],# bin 71  chi2/NDF: 1.693
[ 1.8125,   1.1270e+02,   3.2563e+00,   2.0405e-03  ],# bin 72  chi2/NDF: 8.8331
[ 1.8375,   5.3037e+02,   -4.9952e+00,   3.6172e-02  ],# bin 73  chi2/NDF: 6.695
[ 1.8625,   9.4935e+01,   4.3047e+00,   3.3716e-03  ],# bin 74  chi2/NDF: 12.817
[ 1.8875,   2.1949e+01,   4.9097e+00,   1.4118e-04  ],# bin 75  chi2/NDF: 24.519
[ 1.9125,   1.1111e+02,   1.7502e+00,   3.5846e-03  ],# bin 76  chi2/NDF: 91.601
[ 1.9375,   8.1383e+01,   3.7042e+00,   -3.5474e-03  ],# bin 77  chi2/NDF: 0.8199
[ 1.9625,   -1.4263e+02,   5.5117e+00,   -2.2043e-03  ],# bin 78  chi2/NDF: 16.803
[ 1.9875,   2.7853e+02,   6.1520e-01,   5.8061e-03  ],# bin 79  chi2/NDF: 7.7046
[ 2.0125,   1.2155e+01,   4.8539e+00,   -3.3773e-03  ],# bin 80  chi2/NDF: 0.82175
[ 2.0375,   -2.0936e+01,   5.3810e+00,   -2.8388e-03  ],# bin 81  chi2/NDF: 0.34962
[ 2.0625,   2.2395e+02,   3.1325e+00,   -1.1041e-03  ],# bin 82  chi2/NDF: 9.6644
[ 2.0875,   3.2250e+02,   3.0923e+00,   -2.8921e-04  ],# bin 83  chi2/NDF: 2.7034
[ 2.1125,   2.0832e+02,   2.0929e+00,   4.3777e-04  ],# bin 84  chi2/NDF: 48.22
[ 2.1375,   1.0520e+02,   3.5447e+00,   -1.3444e-03  ],# bin 85  chi2/NDF: 13.306
[ 2.1625,   1.1615e+02,   2.9209e+00,   -7.7130e-04  ],# bin 86  chi2/NDF: 17.814
[ 2.1875,   6.8800e+01,   2.8313e+00,   -3.3315e-04  ],# bin 87  chi2/NDF: 17.579
[ 2.2125,   8.8048e+01,   2.4716e+00,   5.0750e-04  ],# bin 88  chi2/NDF: 10.921
[ 2.2375,   2.1706e+01,   3.2926e+00,   -1.7614e-04  ],# bin 89  chi2/NDF: 15.205
[ 2.2625,   -2.1059e+01,   3.4582e+00,   -1.6180e-03  ],# bin 90  chi2/NDF: 14.01
[ 2.2875,   -5.9295e+01,   4.5662e+00,   -3.2938e-03  ],# bin 91  chi2/NDF: 28.709
[ 2.3125,   2.9721e+01,   1.1506e+00,   -6.1457e-04  ],# bin 92  chi2/NDF: 17.389
[ 2.3375,   1.7438e+01,   5.2164e+00,   -2.6888e-04  ],# bin 93  chi2/NDF: 11.902
[ 2.3625,   6.3574e+01,   3.5553e+00,   1.4046e-03  ],# bin 94  chi2/NDF: 13.017
[ 2.3875,   -8.2031e+01,   6.7360e+00,   -2.8900e-03  ],# bin 95  chi2/NDF: 5.6899
[ 2.4125,   -6.1303e+01,   6.0645e+00,   -1.3094e-05  ],# bin 96  chi2/NDF: 24.806
[ 2.4375,   -7.6994e+01,   7.0356e+00,   -3.0200e-03  ],# bin 97  chi2/NDF: 12.843
[ 2.4625,   2.8235e+00,   6.7109e+00,   -5.1720e-03  ],# bin 98  chi2/NDF: 18.08
[ 2.4875,   2.8235e+00,   6.7109e+00,   -5.1720e-03  ],# bin 99 missing par  chi2/NDF: 0, copied from bin 98
] # end CaloSwCalHitsEfront_P2_v8_gam_conv


CaloSwCalHitsEfront_P2_v8_gam_unconv = [
[ 0.0125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 0  chi2/NDF: 0, copied from bin 2
[ 0.0375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 1  chi2/NDF: 0, copied from bin 2
[ 0.0625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 2  chi2/NDF: 0
[ 0.0875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 3  chi2/NDF: 0
[ 0.1125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 4  chi2/NDF: 0
[ 0.1375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 5  chi2/NDF: 0
[ 0.1625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 6  chi2/NDF: 0
[ 0.1875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 7  chi2/NDF: 0
[ 0.2125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 8  chi2/NDF: 0
[ 0.2375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 9  chi2/NDF: 0
[ 0.2625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 10  chi2/NDF: 0
[ 0.2875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 11  chi2/NDF: 0
[ 0.3125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 12  chi2/NDF: 0
[ 0.3375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 13  chi2/NDF: 0
[ 0.3625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 14  chi2/NDF: 0
[ 0.3875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 15  chi2/NDF: 0
[ 0.4125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 16  chi2/NDF: 0
[ 0.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 17  chi2/NDF: 0
[ 0.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 18  chi2/NDF: 0
[ 0.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 19  chi2/NDF: 0
[ 0.5125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 20  chi2/NDF: 0
[ 0.5375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 21  chi2/NDF: 0
[ 0.5625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 22  chi2/NDF: 0
[ 0.5875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 23  chi2/NDF: 0
[ 0.6125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 24  chi2/NDF: 0
[ 0.6375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 25  chi2/NDF: 0
[ 0.6625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 26  chi2/NDF: 0
[ 0.6875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 27  chi2/NDF: 0
[ 0.7125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 28  chi2/NDF: 0
[ 0.7375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 29  chi2/NDF: 0
[ 0.7625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 30  chi2/NDF: 0, copied from bin 29
[ 0.7875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 31  chi2/NDF: 0, copied from bin 29
[ 0.8125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 32  chi2/NDF: 0, copied from bin 34
[ 0.8375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 33  chi2/NDF: 0, copied from bin 34
[ 0.8625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 34  chi2/NDF: 0
[ 0.8875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 35  chi2/NDF: 0
[ 0.9125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 36  chi2/NDF: 0
[ 0.9375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 37  chi2/NDF: 0
[ 0.9625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 38  chi2/NDF: 0
[ 0.9875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 39  chi2/NDF: 0
[ 1.0125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 40  chi2/NDF: 0
[ 1.0375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 41  chi2/NDF: 0
[ 1.0625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 42  chi2/NDF: 0
[ 1.0875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 43  chi2/NDF: 0
[ 1.1125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 44  chi2/NDF: 0
[ 1.1375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 45  chi2/NDF: 0
[ 1.1625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 46  chi2/NDF: 0
[ 1.1875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 47  chi2/NDF: 0
[ 1.2125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 48  chi2/NDF: 0
[ 1.2375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 49  chi2/NDF: 0
[ 1.2625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 50  chi2/NDF: 0
[ 1.2875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 51  chi2/NDF: 0
[ 1.3125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 52  chi2/NDF: 0
[ 1.3375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 53  chi2/NDF: 0
[ 1.3625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 54  chi2/NDF: 0
[ 1.3875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 55  chi2/NDF: 0
[ 1.4125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 56  chi2/NDF: 0
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack missing par  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack missing par  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   1.5465e-06,   1.1983e-07,   -1.1900e-03  ],# bin 60 crack  chi2/NDF: 43.645
[ 1.5375,   7.0237e-05,   -7.8379e-08,   6.4634e-03  ],# bin 61 crack  chi2/NDF: 7.2773
[ 1.5625,   1.8137e-05,   -1.4564e-08,   1.2811e-02  ],# bin 62  chi2/NDF: 1.0687
[ 1.5875,   9.6029e-06,   -4.8385e-09,   1.6028e-02  ],# bin 63  chi2/NDF: 1.717
[ 1.6125,   3.6056e-07,   1.0817e-08,   -4.6013e-03  ],# bin 64  chi2/NDF: 2.5885
[ 1.6375,   1.0295e-05,   -1.0209e-09,   -2.3860e-03  ],# bin 65  chi2/NDF: 4.9534
[ 1.6625,   2.3244e-05,   -1.9444e-08,   -1.7136e-02  ],# bin 66  chi2/NDF: 2.4184
[ 1.6875,   2.5291e-05,   -2.1284e-08,   -1.1435e-02  ],# bin 67  chi2/NDF: 0.83944
[ 1.7125,   2.5521e-05,   -1.9343e-08,   -1.2792e-03  ],# bin 68  chi2/NDF: 3.0944
[ 1.7375,   3.6020e-05,   -1.8583e-08,   1.4979e-02  ],# bin 69  chi2/NDF: 17.859
[ 1.7625,   4.5697e-05,   -3.6191e-08,   2.9712e-02  ],# bin 70  chi2/NDF: 2.1779
[ 1.7875,   2.5438e-05,   -2.5838e-08,   2.4085e-02  ],# bin 71  chi2/NDF: 1.1872
[ 1.8125,   9.8543e+01,   2.4045e+00,   -3.1605e-03  ],# bin 72  chi2/NDF: 9.1285
[ 1.8375,   1.0203e+02,   1.0518e+00,   2.1631e-03  ],# bin 73  chi2/NDF: 11.974
[ 1.8625,   4.5235e+01,   1.2417e+00,   3.6839e-03  ],# bin 74  chi2/NDF: 16.342
[ 1.8875,   1.1880e+02,   -7.2366e-02,   6.8841e-03  ],# bin 75  chi2/NDF: 15.331
[ 1.9125,   1.6353e+01,   1.0350e+00,   3.7707e-03  ],# bin 76  chi2/NDF: 4.0504
[ 1.9375,   -2.2616e+01,   1.6999e+00,   2.0228e-03  ],# bin 77  chi2/NDF: 0.20947
[ 1.9625,   -1.3850e+01,   1.3096e+00,   2.3921e-03  ],# bin 78  chi2/NDF: 5.8531
[ 1.9875,   7.7581e+01,   8.6108e-01,   -7.0607e-05  ],# bin 79  chi2/NDF: 1.4258
[ 2.0125,   -3.4111e+01,   2.0022e+00,   -1.0902e-03  ],# bin 80  chi2/NDF: 0.30044
[ 2.0375,   4.0374e+01,   8.1810e-01,   5.1417e-04  ],# bin 81  chi2/NDF: 10.367
[ 2.0625,   4.8469e+01,   7.3138e-01,   -2.8572e-04  ],# bin 82  chi2/NDF: 0.22391
[ 2.0875,   2.8171e+01,   9.4688e-01,   3.9044e-04  ],# bin 83  chi2/NDF: 15.334
[ 2.1125,   1.4290e+02,   -1.3249e-01,   1.4920e-03  ],# bin 84  chi2/NDF: 2.64
[ 2.1375,   2.4061e+01,   9.5518e-01,   -5.4353e-04  ],# bin 85  chi2/NDF: 2.5666
[ 2.1625,   1.7158e+02,   -2.2943e-01,   6.3512e-04  ],# bin 86  chi2/NDF: 1.3576
[ 2.1875,   1.1889e+02,   -3.8949e-01,   1.1339e-03  ],# bin 87  chi2/NDF: 1.2466
[ 2.2125,   1.5195e+02,   -3.2928e-01,   7.2230e-04  ],# bin 88  chi2/NDF: 4.5976
[ 2.2375,   -2.0777e+01,   9.1292e-01,   -7.6971e-04  ],# bin 89  chi2/NDF: 14.196
[ 2.2625,   5.2616e+00,   4.5589e-01,   -1.5383e-04  ],# bin 90  chi2/NDF: 0.59147
[ 2.2875,   3.9718e+01,   1.3618e-01,   1.0636e-04  ],# bin 91  chi2/NDF: 3.4708
[ 2.3125,   -3.1449e+00,   5.1635e-01,   -2.2670e-04  ],# bin 92  chi2/NDF: 2.7158
[ 2.3375,   3.1729e+01,   1.2604e+00,   -6.7973e-04  ],# bin 93  chi2/NDF: 2.202
[ 2.3625,   3.2112e+02,   8.3537e-02,   5.8515e-04  ],# bin 94  chi2/NDF: 2.202
[ 2.3875,   1.2449e+02,   6.3087e-01,   8.9179e-05  ],# bin 95  chi2/NDF: 4.4159
[ 2.4125,   2.1815e+02,   -5.3029e-01,   1.5808e-03  ],# bin 96  chi2/NDF: 11.998
[ 2.4375,   -2.9602e+01,   1.0603e+00,   -3.5276e-04  ],# bin 97  chi2/NDF: 15.41
[ 2.4625,   1.6671e+01,   1.1986e+00,   -9.1116e-04  ],# bin 98  chi2/NDF: 4.6424
[ 2.4875,   1.6671e+01,   1.1986e+00,   -9.1116e-04  ],# bin 99 missing par  chi2/NDF: 0, copied from bin 98
] # end CaloSwCalHitsEfront_P2_v8_gam_unconv


CaloSwCalHitsOutOfCone_v8_ele55 = [
[ 0.0125,   7.4082e+00,   -7.5859e-01,   2.6577e-02  ],# bin 0  chi2/NDF: 25.576, copied from bin 2
[ 0.0375,   7.4082e+00,   -7.5859e-01,   2.6577e-02  ],# bin 1  chi2/NDF: 262.72, copied from bin 2
[ 0.0625,   7.4082e+00,   -7.5859e-01,   2.6577e-02  ],# bin 2  chi2/NDF: 17.402
[ 0.0875,   8.1028e+00,   -8.9897e-01,   3.3462e-02  ],# bin 3  chi2/NDF: 27.672
[ 0.1125,   6.6554e+00,   -6.1136e-01,   1.9514e-02  ],# bin 4  chi2/NDF: 133.25
[ 0.1375,   7.5061e+00,   -7.7252e-01,   2.7142e-02  ],# bin 5  chi2/NDF: 45.857
[ 0.1625,   7.5861e+00,   -7.8640e-01,   2.7848e-02  ],# bin 6  chi2/NDF: 14.596
[ 0.1875,   6.9441e+00,   -6.6532e-01,   2.2220e-02  ],# bin 7  chi2/NDF: 56.85
[ 0.2125,   7.7840e+00,   -8.1545e-01,   2.8970e-02  ],# bin 8  chi2/NDF: 47.227
[ 0.2375,   7.3770e+00,   -7.3592e-01,   2.5173e-02  ],# bin 9  chi2/NDF: 26.869
[ 0.2625,   7.3735e+00,   -7.3211e-01,   2.4982e-02  ],# bin 10  chi2/NDF: 39.084
[ 0.2875,   7.4453e+00,   -7.4467e-01,   2.5583e-02  ],# bin 11  chi2/NDF: 43.508
[ 0.3125,   6.6259e+00,   -5.8054e-01,   1.7734e-02  ],# bin 12  chi2/NDF: 83.029
[ 0.3375,   7.2395e+00,   -6.8977e-01,   2.2653e-02  ],# bin 13  chi2/NDF: 14.175
[ 0.3625,   7.0653e+00,   -6.5402e-01,   2.1090e-02  ],# bin 14  chi2/NDF: 45.502
[ 0.3875,   7.7905e+00,   -7.7297e-01,   2.6117e-02  ],# bin 15  chi2/NDF: 46.539
[ 0.4125,   7.7953e+00,   -7.7326e-01,   2.6257e-02  ],# bin 16  chi2/NDF: 47.822
[ 0.4375,   6.8575e+00,   -6.0292e-01,   1.8731e-02  ],# bin 17  chi2/NDF: 70.911
[ 0.4625,   7.1449e+00,   -6.4980e-01,   2.0823e-02  ],# bin 18  chi2/NDF: 96.988
[ 0.4875,   6.9513e+00,   -6.0713e-01,   1.8807e-02  ],# bin 19  chi2/NDF: 66.895
[ 0.5125,   7.0699e+00,   -6.1862e-01,   1.9181e-02  ],# bin 20  chi2/NDF: 202.49
[ 0.5375,   7.5417e+00,   -6.9074e-01,   2.1936e-02  ],# bin 21  chi2/NDF: 147.43
[ 0.5625,   7.5776e+00,   -6.9733e-01,   2.2336e-02  ],# bin 22  chi2/NDF: 95.778
[ 0.5875,   7.1777e+00,   -6.2437e-01,   1.9076e-02  ],# bin 23  chi2/NDF: 131.69
[ 0.6125,   7.5354e+00,   -6.8518e-01,   2.1809e-02  ],# bin 24  chi2/NDF: 103.89
[ 0.6375,   8.7641e+00,   -8.8121e-01,   2.9695e-02  ],# bin 25  chi2/NDF: 46.398
[ 0.6625,   9.3296e+00,   -9.3479e-01,   3.0656e-02  ],# bin 26  chi2/NDF: 33.801
[ 0.6875,   1.0461e+01,   -1.1022e+00,   3.6856e-02  ],# bin 27  chi2/NDF: 38.965
[ 0.7125,   1.0441e+01,   -1.0889e+00,   3.6139e-02  ],# bin 28  chi2/NDF: 49.497
[ 0.7375,   1.0940e+01,   -1.1555e+00,   3.8333e-02  ],# bin 29  chi2/NDF: 101.57
[ 0.7625,   1.0940e+01,   -1.1555e+00,   3.8333e-02  ],# bin 30  chi2/NDF: 70.013, copied from bin 29
[ 0.7875,   1.0940e+01,   -1.1555e+00,   3.8333e-02  ],# bin 31  chi2/NDF: 42.309, copied from bin 29
[ 0.8125,   2.4450e+01,   -2.7035e+00,   8.4436e-02  ],# bin 32  chi2/NDF: 64.572, copied from bin 34
[ 0.8375,   2.4450e+01,   -2.7035e+00,   8.4436e-02  ],# bin 33  chi2/NDF: 98.779, copied from bin 34
[ 0.8625,   2.4450e+01,   -2.7035e+00,   8.4436e-02  ],# bin 34  chi2/NDF: 52.67
[ 0.8875,   2.6306e+01,   -2.9915e+00,   9.5494e-02  ],# bin 35  chi2/NDF: 40.964
[ 0.9125,   2.4245e+01,   -2.6844e+00,   8.4198e-02  ],# bin 36  chi2/NDF: 29.167
[ 0.9375,   2.5496e+01,   -2.9093e+00,   9.3829e-02  ],# bin 37  chi2/NDF: 121.73
[ 0.9625,   2.5405e+01,   -2.9094e+00,   9.4414e-02  ],# bin 38  chi2/NDF: 35.994
[ 0.9875,   2.3912e+01,   -2.6780e+00,   8.5390e-02  ],# bin 39  chi2/NDF: 34.424
[ 1.0125,   2.5771e+01,   -2.9933e+00,   9.8573e-02  ],# bin 40  chi2/NDF: 93.224
[ 1.0375,   2.6062e+01,   -3.0802e+00,   1.0327e-01  ],# bin 41  chi2/NDF: 69.985
[ 1.0625,   2.5336e+01,   -2.9842e+00,   1.0028e-01  ],# bin 42  chi2/NDF: 80.832
[ 1.0875,   2.6851e+01,   -3.2372e+00,   1.1040e-01  ],# bin 43  chi2/NDF: 66.823
[ 1.1125,   2.4242e+01,   -2.8635e+00,   9.7259e-02  ],# bin 44  chi2/NDF: 103.43
[ 1.1375,   2.5534e+01,   -3.0879e+00,   1.0664e-01  ],# bin 45  chi2/NDF: 102.83
[ 1.1625,   2.6339e+01,   -3.1844e+00,   1.0984e-01  ],# bin 46  chi2/NDF: 105.33
[ 1.1875,   2.7493e+01,   -3.3257e+00,   1.1382e-01  ],# bin 47  chi2/NDF: 121.9
[ 1.2125,   3.0230e+01,   -3.7279e+00,   1.2914e-01  ],# bin 48  chi2/NDF: 104.1
[ 1.2375,   3.2325e+01,   -4.0214e+00,   1.3924e-01  ],# bin 49  chi2/NDF: 182.15
[ 1.2625,   3.4325e+01,   -4.2852e+00,   1.4840e-01  ],# bin 50  chi2/NDF: 140.53
[ 1.2875,   3.6407e+01,   -4.5561e+00,   1.5677e-01  ],# bin 51  chi2/NDF: 138.11
[ 1.3125,   4.1038e+01,   -5.2025e+00,   1.7974e-01  ],# bin 52  chi2/NDF: 109.4
[ 1.3375,   4.4278e+01,   -5.6027e+00,   1.9073e-01  ],# bin 53  chi2/NDF: 116.83
[ 1.3625,   5.1188e+01,   -6.6080e+00,   2.3078e-01  ],# bin 54  chi2/NDF: 117.78
[ 1.3875,   5.3695e+01,   -7.1285e+00,   2.4875e-01  ],# bin 55  chi2/NDF: 66.581
[ 1.4125,   4.1598e+01,   -5.1734e+00,   1.7093e-01  ],# bin 56  chi2/NDF: 149.4
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -2.1134e+01,   4.6350e-01,   2.8141e+02  ],# bin 60 crack  chi2/NDF: 104.3
[ 1.5375,   -2.9898e+01,   8.7314e-01,   3.0326e+02  ],# bin 61 crack  chi2/NDF: 23.686
[ 1.5625,   -2.4694e+01,   6.1059e-01,   2.9329e+02  ],# bin 62  chi2/NDF: 74.118
[ 1.5875,   -2.8617e+01,   8.2953e-01,   2.9202e+02  ],# bin 63  chi2/NDF: 44.705
[ 1.6125,   -2.4878e+01,   7.3324e-01,   2.5882e+02  ],# bin 64  chi2/NDF: 67.601
[ 1.6375,   -2.0141e+01,   5.8887e-01,   2.2031e+02  ],# bin 65  chi2/NDF: 43.162
[ 1.6625,   -2.2759e+01,   7.0852e-01,   2.3638e+02  ],# bin 66  chi2/NDF: 67.71
[ 1.6875,   -2.7665e+01,   8.8039e-01,   2.7339e+02  ],# bin 67  chi2/NDF: 42.88
[ 1.7125,   -2.0852e+01,   6.3094e-01,   2.2964e+02  ],# bin 68  chi2/NDF: 87.3
[ 1.7375,   -1.6966e+01,   5.5950e-01,   1.8402e+02  ],# bin 69  chi2/NDF: 44.436
[ 1.7625,   -1.4096e+01,   4.9183e-01,   1.5715e+02  ],# bin 70  chi2/NDF: 74.385
[ 1.7875,   -6.6355e+00,   2.5368e-01,   9.9976e+01  ],# bin 71  chi2/NDF: 66.278
[ 1.8125,   -2.9070e-01,   1.7099e-02,   6.1115e+01  ],# bin 72  chi2/NDF: 51.981
[ 1.8375,   -9.7321e+00,   3.3932e-01,   1.3019e+02  ],# bin 73  chi2/NDF: 31.096
[ 1.8625,   -8.2847e+00,   3.1433e-01,   1.1564e+02  ],# bin 74  chi2/NDF: 19.344
[ 1.8875,   -6.1203e+00,   2.4807e-01,   9.9717e+01  ],# bin 75  chi2/NDF: 41.369
[ 1.9125,   -4.0468e+00,   1.9223e-01,   8.3336e+01  ],# bin 76  chi2/NDF: 76.316
[ 1.9375,   -4.8629e+00,   2.3764e-01,   8.7358e+01  ],# bin 77  chi2/NDF: 54.436
[ 1.9625,   -4.1457e+00,   2.2458e-01,   8.2174e+01  ],# bin 78  chi2/NDF: 37.598
[ 1.9875,   1.7823e+00,   1.5691e-03,   4.5394e+01  ],# bin 79  chi2/NDF: 22.952
[ 2.0125,   -4.2730e-01,   9.1774e-02,   6.1829e+01  ],# bin 80  chi2/NDF: 34.066
[ 2.0375,   -1.6711e+00,   1.3213e-01,   7.3656e+01  ],# bin 81  chi2/NDF: 14.985
[ 2.0625,   -2.7170e+00,   1.7680e-01,   8.2160e+01  ],# bin 82  chi2/NDF: 33.48
[ 2.0875,   -3.6155e-01,   9.8564e-02,   6.6980e+01  ],# bin 83  chi2/NDF: 13.421
[ 2.1125,   -3.9224e+00,   2.3718e-01,   9.0429e+01  ],# bin 84  chi2/NDF: 9.9142
[ 2.1375,   1.7675e-02,   1.0309e-01,   6.3434e+01  ],# bin 85  chi2/NDF: 14.717
[ 2.1625,   1.0289e+00,   7.9600e-02,   5.6566e+01  ],# bin 86  chi2/NDF: 8.9215
[ 2.1875,   -9.4727e-01,   1.6020e-01,   7.0357e+01  ],# bin 87  chi2/NDF: 20.124
[ 2.2125,   -1.9060e-02,   1.3038e-01,   6.5719e+01  ],# bin 88  chi2/NDF: 23.716
[ 2.2375,   2.7001e+00,   4.2144e-02,   4.6524e+01  ],# bin 89  chi2/NDF: 79.08
[ 2.2625,   -2.5291e+00,   2.4447e-01,   8.2675e+01  ],# bin 90  chi2/NDF: 119.74
[ 2.2875,   -3.0334e+00,   2.5485e-01,   9.1293e+01  ],# bin 91  chi2/NDF: 166.56
[ 2.3125,   8.6173e+00,   -1.7777e-01,   1.9683e+01  ],# bin 92  chi2/NDF: 199.55
[ 2.3375,   -4.4081e+00,   1.8102e-01,   1.3502e+02  ],# bin 93  chi2/NDF: 237.22
[ 2.3625,   -1.9493e+01,   6.9310e-01,   2.4984e+02  ],# bin 94  chi2/NDF: 300.1
[ 2.3875,   -1.0912e+01,   3.9514e-01,   1.8648e+02  ],# bin 95  chi2/NDF: 239.64
[ 2.4125,   -4.7957e+00,   1.7289e-01,   1.4749e+02  ],# bin 96  chi2/NDF: 115.32
[ 2.4375,   5.3510e+00,   -2.2128e-01,   8.4986e+01  ],# bin 97  chi2/NDF: 138.51
[ 2.4625,   -7.0086e+01,   2.3804e+00,   6.5092e+02  ],# bin 98  chi2/NDF: 11.179
[ 2.4875,   -7.0086e+01,   2.3804e+00,   6.5092e+02  ],# bin 99  chi2/NDF: 11.179, copied from bin 98
] # end CaloSwCalHitsOutOfCone_v8_ele55


CaloSwCalHitsOutOfCone_v8_gam55_conv = [
[ 0.0125,   1.1993e+01,   -1.4492e+00,   5.2748e-02  ],# bin 0  chi2/NDF: 32.128, copied from bin 2
[ 0.0375,   1.1993e+01,   -1.4492e+00,   5.2748e-02  ],# bin 1  chi2/NDF: 65.747, copied from bin 2
[ 0.0625,   1.1993e+01,   -1.4492e+00,   5.2748e-02  ],# bin 2  chi2/NDF: 111.99
[ 0.0875,   1.3131e+01,   -1.6558e+00,   6.1926e-02  ],# bin 3  chi2/NDF: 103.15
[ 0.1125,   1.3520e+01,   -1.6876e+00,   6.1964e-02  ],# bin 4  chi2/NDF: 81.446
[ 0.1375,   1.4978e+01,   -1.9687e+00,   7.5250e-02  ],# bin 5  chi2/NDF: 43.924
[ 0.1625,   9.9633e+00,   -1.0674e+00,   3.5172e-02  ],# bin 6  chi2/NDF: 82.987
[ 0.1875,   1.4417e+01,   -1.8319e+00,   6.7935e-02  ],# bin 7  chi2/NDF: 64.347
[ 0.2125,   1.2334e+01,   -1.4820e+00,   5.3332e-02  ],# bin 8  chi2/NDF: 119.14
[ 0.2375,   1.1907e+01,   -1.3792e+00,   4.7876e-02  ],# bin 9  chi2/NDF: 27.735
[ 0.2625,   1.3586e+01,   -1.6645e+00,   5.9985e-02  ],# bin 10  chi2/NDF: 38.804
[ 0.2875,   1.0797e+01,   -1.1886e+00,   3.9879e-02  ],# bin 11  chi2/NDF: 134.64
[ 0.3125,   1.2197e+01,   -1.4032e+00,   4.8194e-02  ],# bin 12  chi2/NDF: 26.462
[ 0.3375,   1.1942e+01,   -1.3663e+00,   4.7084e-02  ],# bin 13  chi2/NDF: 58.853
[ 0.3625,   1.0009e+01,   -1.0329e+00,   3.3002e-02  ],# bin 14  chi2/NDF: 61.504
[ 0.3875,   1.1278e+01,   -1.2152e+00,   3.9559e-02  ],# bin 15  chi2/NDF: 50.296
[ 0.4125,   1.4693e+01,   -1.7937e+00,   6.4094e-02  ],# bin 16  chi2/NDF: 46.152
[ 0.4375,   1.0243e+01,   -1.0635e+00,   3.4522e-02  ],# bin 17  chi2/NDF: 84.753
[ 0.4625,   1.2150e+01,   -1.3511e+00,   4.5454e-02  ],# bin 18  chi2/NDF: 86.01
[ 0.4875,   1.3688e+01,   -1.5903e+00,   5.4902e-02  ],# bin 19  chi2/NDF: 43.013
[ 0.5125,   1.4028e+01,   -1.6407e+00,   5.7032e-02  ],# bin 20  chi2/NDF: 56.643
[ 0.5375,   1.4675e+01,   -1.6876e+00,   5.6794e-02  ],# bin 21  chi2/NDF: 51.618
[ 0.5625,   1.2233e+01,   -1.3073e+00,   4.2053e-02  ],# bin 22  chi2/NDF: 75.097
[ 0.5875,   1.3523e+01,   -1.5181e+00,   5.0802e-02  ],# bin 23  chi2/NDF: 39.521
[ 0.6125,   1.2383e+01,   -1.3415e+00,   4.4044e-02  ],# bin 24  chi2/NDF: 119.1
[ 0.6375,   1.2548e+01,   -1.3431e+00,   4.3208e-02  ],# bin 25  chi2/NDF: 62.906
[ 0.6625,   1.3928e+01,   -1.5542e+00,   5.1414e-02  ],# bin 26  chi2/NDF: 84.003
[ 0.6875,   1.4534e+01,   -1.6376e+00,   5.4300e-02  ],# bin 27  chi2/NDF: 83.079
[ 0.7125,   1.7424e+01,   -2.0909e+00,   7.2051e-02  ],# bin 28  chi2/NDF: 76.515
[ 0.7375,   1.7543e+01,   -2.0870e+00,   7.1104e-02  ],# bin 29  chi2/NDF: 79.841
[ 0.7625,   1.7543e+01,   -2.0870e+00,   7.1104e-02  ],# bin 30  chi2/NDF: 68.642, copied from bin 29
[ 0.7875,   1.7543e+01,   -2.0870e+00,   7.1104e-02  ],# bin 31  chi2/NDF: 9.3978, copied from bin 29
[ 0.8125,   2.9280e+01,   -3.3872e+00,   1.0849e-01  ],# bin 32  chi2/NDF: 52.415, copied from bin 34
[ 0.8375,   2.9280e+01,   -3.3872e+00,   1.0849e-01  ],# bin 33  chi2/NDF: 120.82, copied from bin 34
[ 0.8625,   2.9280e+01,   -3.3872e+00,   1.0849e-01  ],# bin 34  chi2/NDF: 145.84
[ 0.8875,   3.0091e+01,   -3.5408e+00,   1.1523e-01  ],# bin 35  chi2/NDF: 105.72
[ 0.9125,   2.7677e+01,   -3.1996e+00,   1.0329e-01  ],# bin 36  chi2/NDF: 80.512
[ 0.9375,   2.9923e+01,   -3.5517e+00,   1.1687e-01  ],# bin 37  chi2/NDF: 190.62
[ 0.9625,   2.7892e+01,   -3.2768e+00,   1.0776e-01  ],# bin 38  chi2/NDF: 126.63
[ 0.9875,   3.1055e+01,   -3.7360e+00,   1.2433e-01  ],# bin 39  chi2/NDF: 70.219
[ 1.0125,   2.6788e+01,   -3.1002e+00,   1.0088e-01  ],# bin 40  chi2/NDF: 105.85
[ 1.0375,   2.9316e+01,   -3.5076e+00,   1.1696e-01  ],# bin 41  chi2/NDF: 71.743
[ 1.0625,   3.1992e+01,   -3.9532e+00,   1.3548e-01  ],# bin 42  chi2/NDF: 43.263
[ 1.0875,   2.8366e+01,   -3.3940e+00,   1.1376e-01  ],# bin 43  chi2/NDF: 109.7
[ 1.1125,   2.8286e+01,   -3.4334e+00,   1.1737e-01  ],# bin 44  chi2/NDF: 84.166
[ 1.1375,   2.9853e+01,   -3.7068e+00,   1.2882e-01  ],# bin 45  chi2/NDF: 75.39
[ 1.1625,   2.9377e+01,   -3.5744e+00,   1.2219e-01  ],# bin 46  chi2/NDF: 168.48
[ 1.1875,   2.9862e+01,   -3.5970e+00,   1.2099e-01  ],# bin 47  chi2/NDF: 80.742
[ 1.2125,   3.3551e+01,   -4.1459e+00,   1.4195e-01  ],# bin 48  chi2/NDF: 74.39
[ 1.2375,   3.2886e+01,   -3.9110e+00,   1.2817e-01  ],# bin 49  chi2/NDF: 838.23
[ 1.2625,   3.7067e+01,   -4.6138e+00,   1.5792e-01  ],# bin 50  chi2/NDF: 44.896
[ 1.2875,   3.7729e+01,   -4.6451e+00,   1.5638e-01  ],# bin 51  chi2/NDF: 118.24
[ 1.3125,   4.2989e+01,   -5.3867e+00,   1.8275e-01  ],# bin 52  chi2/NDF: 52.948
[ 1.3375,   4.6342e+01,   -5.8124e+00,   1.9511e-01  ],# bin 53  chi2/NDF: 45.116
[ 1.3625,   4.7566e+01,   -5.9039e+00,   1.9797e-01  ],# bin 54  chi2/NDF: 97.604
[ 1.3875,   6.3644e+01,   -8.6195e+00,   3.0512e-01  ],# bin 55  chi2/NDF: 166.61
[ 1.4125,   5.7972e+01,   -7.9013e+00,   2.8619e-01  ],# bin 56  chi2/NDF: 154.46
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack missing par  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack missing par  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -8.1388e+00,   -5.8710e-02,   2.0918e+02  ],# bin 60 crack  chi2/NDF: 60.297
[ 1.5375,   -3.5709e+01,   1.0783e+00,   3.4452e+02  ],# bin 61 crack  chi2/NDF: 65.717
[ 1.5625,   -2.6493e+01,   6.6158e-01,   3.0832e+02  ],# bin 62  chi2/NDF: 54.444
[ 1.5875,   -2.9879e+01,   8.6925e-01,   3.0110e+02  ],# bin 63  chi2/NDF: 27.237
[ 1.6125,   -2.9002e+01,   8.9235e-01,   2.8369e+02  ],# bin 64  chi2/NDF: 77.827
[ 1.6375,   -2.1611e+01,   6.5739e-01,   2.2613e+02  ],# bin 65  chi2/NDF: 62.606
[ 1.6625,   -1.6814e+01,   4.9711e-01,   1.9374e+02  ],# bin 66  chi2/NDF: 20.224
[ 1.6875,   -1.6752e+01,   4.9693e-01,   1.9556e+02  ],# bin 67  chi2/NDF: 8.0597
[ 1.7125,   -2.0968e+01,   6.5360e-01,   2.2461e+02  ],# bin 68  chi2/NDF: 70.031
[ 1.7375,   -1.5882e+01,   5.4020e-01,   1.7098e+02  ],# bin 69  chi2/NDF: 100.73
[ 1.7625,   -1.1801e+01,   4.2942e-01,   1.3552e+02  ],# bin 70  chi2/NDF: 77.07
[ 1.7875,   -8.1570e+00,   3.2952e-01,   1.0455e+02  ],# bin 71  chi2/NDF: 70.444
[ 1.8125,   -4.4624e-01,   5.2471e-02,   5.3924e+01  ],# bin 72  chi2/NDF: 42.866
[ 1.8375,   -2.4638e+00,   1.2675e-01,   6.7862e+01  ],# bin 73  chi2/NDF: 22.621
[ 1.8625,   3.9651e-01,   3.6815e-02,   4.6656e+01  ],# bin 74  chi2/NDF: 103.07
[ 1.8875,   4.0577e+00,   -8.4519e-02,   2.0826e+01  ],# bin 75  chi2/NDF: 54.466
[ 1.9125,   3.6859e+00,   -5.8103e-02,   2.2688e+01  ],# bin 76  chi2/NDF: 13.536
[ 1.9375,   3.1214e+00,   -2.9695e-02,   2.6711e+01  ],# bin 77  chi2/NDF: 22.005
[ 1.9625,   -8.1877e-01,   1.2913e-01,   5.2760e+01  ],# bin 78  chi2/NDF: 9.9077
[ 1.9875,   2.7512e+00,   1.0210e-03,   2.9882e+01  ],# bin 79  chi2/NDF: 80.223
[ 2.0125,   2.6774e+00,   7.0217e-03,   3.2498e+01  ],# bin 80  chi2/NDF: 33.948
[ 2.0375,   2.9623e+00,   -5.0419e-04,   3.2090e+01  ],# bin 81  chi2/NDF: 47.519
[ 2.0625,   -4.6608e+00,   2.7062e-01,   8.8259e+01  ],# bin 82  chi2/NDF: 32.249
[ 2.0875,   -1.7018e+01,   6.8823e-01,   1.8200e+02  ],# bin 83  chi2/NDF: 81.391
[ 2.1125,   -1.9723e+01,   8.0305e-01,   1.9849e+02  ],# bin 84  chi2/NDF: 57.988
[ 2.1375,   -2.0622e+01,   8.3845e-01,   2.0620e+02  ],# bin 85  chi2/NDF: 96.699
[ 2.1625,   -2.4406e+01,   9.6311e-01,   2.3734e+02  ],# bin 86  chi2/NDF: 41.397
[ 2.1875,   -2.5952e+01,   1.0230e+00,   2.4927e+02  ],# bin 87  chi2/NDF: 42.806
[ 2.2125,   -2.5640e+01,   1.0256e+00,   2.4668e+02  ],# bin 88  chi2/NDF: 45.677
[ 2.2375,   -1.7917e+01,   7.7093e-01,   1.8982e+02  ],# bin 89  chi2/NDF: 26.167
[ 2.2625,   -1.6184e+01,   7.0865e-01,   1.8053e+02  ],# bin 90  chi2/NDF: 28.892
[ 2.2875,   -1.1975e+01,   5.5212e-01,   1.5533e+02  ],# bin 91  chi2/NDF: 8.502
[ 2.3125,   -1.2748e+01,   5.6602e-01,   1.6769e+02  ],# bin 92  chi2/NDF: 9.6228
[ 2.3375,   -2.2806e+01,   9.0690e-01,   2.4284e+02  ],# bin 93  chi2/NDF: 28.401
[ 2.3625,   -2.7028e+01,   1.0371e+00,   2.8016e+02  ],# bin 94  chi2/NDF: 80.534
[ 2.3875,   -2.2978e+01,   9.4430e-01,   2.4183e+02  ],# bin 95  chi2/NDF: 209.86
[ 2.4125,   -3.1209e+01,   1.2038e+00,   3.0908e+02  ],# bin 96  chi2/NDF: 180.18
[ 2.4375,   -3.1454e+01,   1.2284e+00,   3.0942e+02  ],# bin 97  chi2/NDF: 532.48
[ 2.4625,   4.0758e+01,   -1.5468e+00,   -1.4007e+02  ],# bin 98  chi2/NDF: 90.132
[ 2.4875,   4.0758e+01,   -1.5468e+00,   -1.4007e+02  ],# bin 99  chi2/NDF: 90.132, copied from bin 98
] # end CaloSwCalHitsOutOfCone_v8_gam55_conv


CaloSwCalHitsOutOfCone_v8_gam55_unconv = [
[ 0.0125,   -2.4433e-02,   4.5548e-01,   -2.2378e-02  ],# bin 0  chi2/NDF: 694.66, copied from bin 2
[ 0.0375,   -2.4433e-02,   4.5548e-01,   -2.2378e-02  ],# bin 1  chi2/NDF: 1546.9, copied from bin 2
[ 0.0625,   -2.4433e-02,   4.5548e-01,   -2.2378e-02  ],# bin 2  chi2/NDF: 2757.8
[ 0.0875,   -1.0750e+00,   6.5066e-01,   -3.1256e-02  ],# bin 3  chi2/NDF: 591.32
[ 0.1125,   -1.1169e+00,   6.5168e-01,   -3.1081e-02  ],# bin 4  chi2/NDF: 1075.5
[ 0.1375,   2.9004e+00,   -6.7232e-02,   9.3635e-04  ],# bin 5  chi2/NDF: 1988.4
[ 0.1625,   -2.4336e+00,   8.7552e-01,   -4.0136e-02  ],# bin 6  chi2/NDF: 4039
[ 0.1875,   -1.7934e+00,   7.7072e-01,   -3.5987e-02  ],# bin 7  chi2/NDF: 689.41
[ 0.2125,   -1.9725e+00,   7.8768e-01,   -3.5899e-02  ],# bin 8  chi2/NDF: 1845.3
[ 0.2375,   8.2808e-02,   4.3433e-01,   -2.1122e-02  ],# bin 9  chi2/NDF: 450.28
[ 0.2625,   -1.0333e+00,   6.2289e-01,   -2.8791e-02  ],# bin 10  chi2/NDF: 2240.1
[ 0.2875,   3.2383e-01,   3.9832e-01,   -1.9782e-02  ],# bin 11  chi2/NDF: 1362.2
[ 0.3125,   -1.7793e+00,   7.4476e-01,   -3.3488e-02  ],# bin 12  chi2/NDF: 1174
[ 0.3375,   -1.0954e+00,   6.2482e-01,   -2.8360e-02  ],# bin 13  chi2/NDF: 840.54
[ 0.3625,   -1.1731e+00,   6.2442e-01,   -2.7646e-02  ],# bin 14  chi2/NDF: 1128.2
[ 0.3875,   3.9032e-01,   3.6803e-01,   -1.7281e-02  ],# bin 15  chi2/NDF: 242.92
[ 0.4125,   -9.0073e-01,   5.6739e-01,   -2.4761e-02  ],# bin 16  chi2/NDF: 2070.9
[ 0.4375,   -1.6342e-01,   4.4770e-01,   -1.9937e-02  ],# bin 17  chi2/NDF: 926.43
[ 0.4625,   2.3655e+00,   1.0928e-02,   -9.6479e-04  ],# bin 18  chi2/NDF: 3052.8
[ 0.4875,   5.3485e-01,   3.3057e-01,   -1.4873e-02  ],# bin 19  chi2/NDF: 976.44
[ 0.5125,   -1.2745e+00,   5.9707e-01,   -2.4179e-02  ],# bin 20  chi2/NDF: 2240.3
[ 0.5375,   -3.8954e-01,   4.5594e-01,   -1.8756e-02  ],# bin 21  chi2/NDF: 2087.9
[ 0.5625,   1.1035e+00,   2.4245e-01,   -1.1272e-02  ],# bin 22  chi2/NDF: 263.15
[ 0.5875,   1.6259e+00,   1.6419e-01,   -8.5950e-03  ],# bin 23  chi2/NDF: 185.97
[ 0.6125,   1.6739e+00,   1.5114e-01,   -7.3659e-03  ],# bin 24  chi2/NDF: 1286.7
[ 0.6375,   1.8651e+00,   1.4736e-01,   -8.4980e-03  ],# bin 25  chi2/NDF: 295.85
[ 0.6625,   3.0199e+00,   -5.4425e-02,   4.7806e-04  ],# bin 26  chi2/NDF: 2433.3
[ 0.6875,   4.1348e+00,   -2.0725e-01,   5.2649e-03  ],# bin 27  chi2/NDF: 62.93
[ 0.7125,   4.6344e+00,   -2.9726e-01,   9.3101e-03  ],# bin 28  chi2/NDF: 296.77
[ 0.7375,   4.7085e+00,   -2.9319e-01,   8.4793e-03  ],# bin 29  chi2/NDF: 75.068
[ 0.7625,   4.7085e+00,   -2.9319e-01,   8.4793e-03  ],# bin 30  chi2/NDF: 192.12, copied from bin 29
[ 0.7875,   4.7085e+00,   -2.9319e-01,   8.4793e-03  ],# bin 31  chi2/NDF: 341.19, copied from bin 29
[ 0.8125,   9.5471e+00,   -8.1241e-01,   2.4659e-02  ],# bin 32  chi2/NDF: 28.44, copied from bin 34
[ 0.8375,   9.5471e+00,   -8.1241e-01,   2.4659e-02  ],# bin 33  chi2/NDF: 14.326, copied from bin 34
[ 0.8625,   9.5471e+00,   -8.1241e-01,   2.4659e-02  ],# bin 34  chi2/NDF: 49.803
[ 0.8875,   7.2664e+00,   -4.8911e-01,   1.3419e-02  ],# bin 35  chi2/NDF: 82.584
[ 0.9125,   4.4395e+00,   -4.2888e-02,   -4.1132e-03  ],# bin 36  chi2/NDF: 135.66
[ 0.9375,   6.6512e+00,   -3.8755e-01,   9.2359e-03  ],# bin 37  chi2/NDF: 476.71
[ 0.9625,   9.3653e+00,   -8.0821e-01,   2.5600e-02  ],# bin 38  chi2/NDF: 119.88
[ 0.9875,   7.4516e+00,   -5.0230e-01,   1.3355e-02  ],# bin 39  chi2/NDF: 220.15
[ 1.0125,   6.8254e+00,   -4.2359e-01,   1.1268e-02  ],# bin 40  chi2/NDF: 992.42
[ 1.0375,   9.0521e+00,   -7.6703e-01,   2.4410e-02  ],# bin 41  chi2/NDF: 244.78
[ 1.0625,   9.3987e+00,   -7.9072e-01,   2.4169e-02  ],# bin 42  chi2/NDF: 82.01
[ 1.0875,   9.1518e+00,   -7.5256e-01,   2.2603e-02  ],# bin 43  chi2/NDF: 251.5
[ 1.1125,   1.0314e+01,   -9.3390e-01,   2.9866e-02  ],# bin 44  chi2/NDF: 50.771
[ 1.1375,   9.4472e+00,   -7.9729e-01,   2.4427e-02  ],# bin 45  chi2/NDF: 189.55
[ 1.1625,   9.7677e+00,   -8.1174e-01,   2.4190e-02  ],# bin 46  chi2/NDF: 158.58
[ 1.1875,   1.1846e+01,   -1.1153e+00,   3.4951e-02  ],# bin 47  chi2/NDF: 31.152
[ 1.2125,   1.4177e+01,   -1.4706e+00,   4.8909e-02  ],# bin 48  chi2/NDF: 48.223
[ 1.2375,   1.4765e+01,   -1.5336e+00,   5.0205e-02  ],# bin 49  chi2/NDF: 110.65
[ 1.2625,   1.5496e+01,   -1.6047e+00,   5.1899e-02  ],# bin 50  chi2/NDF: 181.24
[ 1.2875,   1.7632e+01,   -1.8796e+00,   6.0155e-02  ],# bin 51  chi2/NDF: 82.745
[ 1.3125,   2.2415e+01,   -2.5746e+00,   8.5663e-02  ],# bin 52  chi2/NDF: 28.199
[ 1.3375,   2.3838e+01,   -2.7147e+00,   8.7298e-02  ],# bin 53  chi2/NDF: 24.46
[ 1.3625,   2.5325e+01,   -2.8481e+00,   9.1948e-02  ],# bin 54  chi2/NDF: 103.18
[ 1.3875,   3.3596e+01,   -4.1681e+00,   1.3720e-01  ],# bin 55  chi2/NDF: 277.28
[ 1.4125,   2.5649e+01,   -2.8193e+00,   8.1454e-02  ],# bin 56  chi2/NDF: 136.61
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack missing par  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack missing par  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -1.8707e+01,   4.5405e-01,   2.2383e+02  ],# bin 60 crack  chi2/NDF: 108.71
[ 1.5375,   -1.7253e+01,   5.4532e-01,   1.7659e+02  ],# bin 61 crack  chi2/NDF: 74.33
[ 1.5625,   -8.4150e+00,   2.7089e-01,   1.0826e+02  ],# bin 62  chi2/NDF: 58.59
[ 1.5875,   -7.8877e+00,   2.9044e-01,   9.4265e+01  ],# bin 63  chi2/NDF: 73
[ 1.6125,   -4.7251e+00,   1.8839e-01,   7.1902e+01  ],# bin 64  chi2/NDF: 274.95
[ 1.6375,   -7.6133e+00,   2.8867e-01,   9.3985e+01  ],# bin 65  chi2/NDF: 57.136
[ 1.6625,   -8.5243e+00,   3.1506e-01,   1.0467e+02  ],# bin 66  chi2/NDF: 33.166
[ 1.6875,   -1.2528e+01,   4.4646e-01,   1.3784e+02  ],# bin 67  chi2/NDF: 81.251
[ 1.7125,   -1.0628e+01,   4.0536e-01,   1.2023e+02  ],# bin 68  chi2/NDF: 30.03
[ 1.7375,   -2.2960e+00,   1.5530e-01,   5.1862e+01  ],# bin 69  chi2/NDF: 146.59
[ 1.7625,   -1.1306e+00,   1.2354e-01,   4.3342e+01  ],# bin 70  chi2/NDF: 80.928
[ 1.7875,   4.8790e+00,   -5.9626e-02,   -3.4596e+00  ],# bin 71  chi2/NDF: 251.66
[ 1.8125,   8.7450e+00,   -1.9077e-01,   -3.0591e+01  ],# bin 72  chi2/NDF: 308.25
[ 1.8375,   1.0078e+01,   -2.3360e-01,   -3.9225e+01  ],# bin 73  chi2/NDF: 106.43
[ 1.8625,   2.2829e+01,   -6.7546e-01,   -1.2874e+02  ],# bin 74  chi2/NDF: 164.71
[ 1.8875,   1.3540e+01,   -3.4169e-01,   -6.2899e+01  ],# bin 75  chi2/NDF: 61.268
[ 1.9125,   7.5615e+00,   -1.2517e-01,   -2.0158e+01  ],# bin 76  chi2/NDF: 28.386
[ 1.9375,   1.0341e+01,   -2.1021e-01,   -4.0307e+01  ],# bin 77  chi2/NDF: 109.32
[ 1.9625,   1.8954e+01,   -5.1307e-01,   -9.8850e+01  ],# bin 78  chi2/NDF: 291.99
[ 1.9875,   1.7496e+01,   -4.5783e-01,   -8.7298e+01  ],# bin 79  chi2/NDF: 28.553
[ 2.0125,   1.1907e+01,   -2.5364e-01,   -4.7387e+01  ],# bin 80  chi2/NDF: 70.927
[ 2.0375,   1.2254e+01,   -2.6747e-01,   -4.7223e+01  ],# bin 81  chi2/NDF: 18.824
[ 2.0625,   1.3085e+01,   -2.7748e-01,   -5.4665e+01  ],# bin 82  chi2/NDF: 65.452
[ 2.0875,   1.7603e+01,   -4.2677e-01,   -8.6416e+01  ],# bin 83  chi2/NDF: 27.299
[ 2.1125,   1.6245e+01,   -3.8883e-01,   -7.2670e+01  ],# bin 84  chi2/NDF: 54.338
[ 2.1375,   1.3919e+01,   -2.9717e-01,   -5.6256e+01  ],# bin 85  chi2/NDF: 40.49
[ 2.1625,   1.6657e+01,   -3.7786e-01,   -7.6440e+01  ],# bin 86  chi2/NDF: 120.74
[ 2.1875,   1.8760e+01,   -4.4910e-01,   -8.9697e+01  ],# bin 87  chi2/NDF: 9.5454
[ 2.2125,   1.5024e+01,   -3.1510e-01,   -6.1706e+01  ],# bin 88  chi2/NDF: 19.436
[ 2.2375,   1.6138e+01,   -3.5783e-01,   -6.7159e+01  ],# bin 89  chi2/NDF: 34.525
[ 2.2625,   1.5269e+01,   -3.1012e-01,   -6.1907e+01  ],# bin 90  chi2/NDF: 68.43
[ 2.2875,   1.8786e+01,   -4.1118e-01,   -8.9422e+01  ],# bin 91  chi2/NDF: 56.872
[ 2.3125,   2.2801e+01,   -5.3740e-01,   -1.1904e+02  ],# bin 92  chi2/NDF: 17.652
[ 2.3375,   1.8306e+01,   -3.9918e-01,   -8.0621e+01  ],# bin 93  chi2/NDF: 27.863
[ 2.3625,   2.7134e+01,   -6.7083e-01,   -1.4930e+02  ],# bin 94  chi2/NDF: 36.718
[ 2.3875,   1.1388e+01,   -1.5643e-01,   -2.7060e+01  ],# bin 95  chi2/NDF: 47.305
[ 2.4125,   2.1870e+01,   -5.1785e-01,   -1.0061e+02  ],# bin 96  chi2/NDF: 52.735
[ 2.4375,   2.4720e+01,   -6.1818e-01,   -1.1818e+02  ],# bin 97  chi2/NDF: 55.276
[ 2.4625,   -1.9986e+01,   8.3751e-01,   2.2804e+02  ],# bin 98  chi2/NDF: 139.64
[ 2.4875,   -1.9986e+01,   8.3751e-01,   2.2804e+02  ],# bin 99 missing par  chi2/NDF: 0, copied from bin 98
] # end CaloSwCalHitsOutOfCone_v8_gam55_unconv
#######################################################################
# Corrections for 3x5 cluster size
#######################################################################

CaloSwCalibHitsCalibration_v8_ele35 = [
      CaloSwCalHitsAccordion_v8_35,
      CaloSwCalHitsOutOfCone_v8_ele35,
      CaloSwCalHitsLongLeakage_v8_ele,
      CaloSwCalHitsEfront_offset_v8_ele,
      CaloSwCalHitsEfront_slope_v8_ele,
      CaloSwCalHitsEfront_P2_v8_ele,
]

CaloSwCalibHitsCalibration_v8_gam35_conv = [
      CaloSwCalHitsAccordion_v8_35,
      CaloSwCalHitsOutOfCone_v8_gam35_conv,
      CaloSwCalHitsLongLeakage_v8_gam_conv,
      CaloSwCalHitsEfront_offset_v8_gam_conv,
      CaloSwCalHitsEfront_slope_v8_gam_conv,
      CaloSwCalHitsEfront_P2_v8_gam_conv,
]

CaloSwCalibHitsCalibration_v8_gam35_unconv = [
      CaloSwCalHitsAccordion_v8_35,
      CaloSwCalHitsOutOfCone_v8_gam35_unconv,
      CaloSwCalHitsLongLeakage_v8_gam_unconv,
      CaloSwCalHitsEfront_offset_v8_gam_unconv,
      CaloSwCalHitsEfront_slope_v8_gam_unconv,
      CaloSwCalHitsEfront_P2_v8_gam_unconv,
]

#######################################################################
# Corrections for 3x7 cluster size
#######################################################################

CaloSwCalibHitsCalibration_v8_ele37 = [
      CaloSwCalHitsAccordion_v8_37,
      CaloSwCalHitsOutOfCone_v8_ele37,
      CaloSwCalHitsLongLeakage_v8_ele,
      CaloSwCalHitsEfront_offset_v8_ele,
      CaloSwCalHitsEfront_slope_v8_ele,
      CaloSwCalHitsEfront_P2_v8_ele,
]

CaloSwCalibHitsCalibration_v8_gam37_conv = [
      CaloSwCalHitsAccordion_v8_37,
      CaloSwCalHitsOutOfCone_v8_gam37_conv,
      CaloSwCalHitsLongLeakage_v8_gam_conv,
      CaloSwCalHitsEfront_offset_v8_gam_conv,
      CaloSwCalHitsEfront_slope_v8_gam_conv,
      CaloSwCalHitsEfront_P2_v8_gam_conv,
]

CaloSwCalibHitsCalibration_v8_gam37_unconv = [
      CaloSwCalHitsAccordion_v8_37,
      CaloSwCalHitsOutOfCone_v8_gam37_unconv,
      CaloSwCalHitsLongLeakage_v8_gam_unconv,
      CaloSwCalHitsEfront_offset_v8_gam_unconv,
      CaloSwCalHitsEfront_slope_v8_gam_unconv,
      CaloSwCalHitsEfront_P2_v8_gam_unconv,
]

#######################################################################
# Corrections for 3x5 cluster size
#######################################################################

CaloSwCalibHitsCalibration_v8_ele55 = [
      CaloSwCalHitsAccordion_v8_55,
      CaloSwCalHitsOutOfCone_v8_ele55,
      CaloSwCalHitsLongLeakage_v8_ele,
      CaloSwCalHitsEfront_offset_v8_ele,
      CaloSwCalHitsEfront_slope_v8_ele,
      CaloSwCalHitsEfront_P2_v8_ele,
]

CaloSwCalibHitsCalibration_v8_gam55_conv = [
      CaloSwCalHitsAccordion_v8_55,
      CaloSwCalHitsOutOfCone_v8_gam55_conv,
      CaloSwCalHitsLongLeakage_v8_gam_conv,
      CaloSwCalHitsEfront_offset_v8_gam_conv,
      CaloSwCalHitsEfront_slope_v8_gam_conv,
      CaloSwCalHitsEfront_P2_v8_gam_conv,
]

CaloSwCalibHitsCalibration_v8_gam55_unconv = [
      CaloSwCalHitsAccordion_v8_55,
      CaloSwCalHitsOutOfCone_v8_gam55_unconv,
      CaloSwCalHitsLongLeakage_v8_gam_unconv,
      CaloSwCalHitsEfront_offset_v8_gam_unconv,
      CaloSwCalHitsEfront_slope_v8_gam_unconv,
      CaloSwCalHitsEfront_P2_v8_gam_unconv,
]

samplings_depths = [
[ 0.0125 , 1.60013 , 3.80006 , 14.0009 , 22.8718 ],
[ 0.0375 , 1.60113 , 3.80056 , 14.0077 , 22.8861 ],
[ 0.0625 , 1.60313 , 3.80156 , 14.0215 , 22.9147 ],
[ 0.0875 , 1.60613 , 3.80306 , 14.0421 , 22.9576 ],
[ 0.1125 , 1.61014 , 3.80507 , 14.0697 , 23.0149 ],
[ 0.1375 , 1.61515 , 3.80757 , 14.1041 , 23.0865 ],
[ 0.1625 , 1.62117 , 3.81059 , 14.1456 , 23.1726 ],
[ 0.1875 , 1.62821 , 3.8141  , 14.1939 , 23.2732 ],
[ 0.2125 , 1.63626 , 3.81813 , 14.2493 , 23.3883 ],
[ 0.2375 , 1.64534 , 3.82267 , 14.3117 , 23.518 ],
[ 0.2625 , 1.65544 , 3.82772 , 14.3812 , 23.6625 ],
[ 0.2875 , 1.66658 , 3.83329 , 14.4577 , 23.8217 ],
[ 0.3125 , 1.67876 , 3.83938 , 14.5415 , 23.9958 ],
[ 0.3375 , 1.69199 , 3.846   , 14.6325 , 24.1849 ],
[ 0.3625 , 1.70628 , 3.85314 , 14.7307 , 24.3892 ],
[ 0.3875 , 1.72164 , 3.86082 , 14.8362 , 24.6086 ],
[ 0.4125 , 1.73807 , 3.86903 , 14.9492 , 24.8435 ],
[ 0.4375 , 1.75558 , 3.87779 , 15.0696 , 25.0939 ],
[ 0.4625 , 1.7742  , 3.8871  , 15.1976 , 25.3599 ],
[ 0.4875 , 1.79392 , 3.89696 , 15.3332 , 25.6418 ],
[ 0.5125 , 1.81476 , 3.90738 , 15.5    , 25.9633 ],
[ 0.5375 , 1.83674 , 3.91837 , 15.5    , 26.1263 ],
[ 0.5625 , 1.85987 , 3.92993 , 15.5    , 26.2979 ],
[ 0.5875 , 1.88416 , 3.94208 , 15.5    , 26.4781 ],
[ 0.6125 , 1.90963 , 3.95481 , 15.5    , 26.667 ],
[ 0.6375 , 1.93629 , 3.96814 , 15.5    , 26.8648 ],
[ 0.6625 , 1.96416 , 3.98208 , 15.5    , 27.0716 ],
[ 0.6875 , 1.99326 , 3.99663 , 15.5    , 27.2875 ],
[ 0.7125 , 2.0236  , 4.0118  , 15.5    , 27.5126 ],
[ 0.7375 , 2.05521 , 4.0276  , 15.5    , 27.7471 ],
[ 0.7625 , 2.0881  , 4.04405 , 15.5    , 27.9911 ],
[ 0.7875 , 2.1223  , 4.06115 , 15.5    , 28.2448 ],
[ 0.8125 , 2.15783 , 4.07891 , 15.5    , 25.4314 ],
[ 0.8375 , 2.1947  , 4.09735 , 15.5    , 25.6524 ],
[ 0.8625 , 2.23295 , 4.11647 , 15.5    , 25.8816 ],
[ 0.8875 , 2.27259 , 4.13629 , 15.5    , 26.1192 ],
[ 0.9125 , 2.31365 , 4.15682 , 15.5    , 26.3653 ],
[ 0.9375 , 2.35616 , 4.17808 , 15.5    , 26.62 ],
[ 0.9625 , 2.40014 , 4.20007 , 15.5    , 26.8836 ],
[ 0.9875 , 2.44562 , 4.22281 , 15.5    , 27.1561 ],
[ 1.0125 , 2.49263 , 4.24631 , 15.5    , 27.4378 ],
[ 1.0375 , 2.54119 , 4.2706  , 15.5    , 27.7289 ],
[ 1.0625 , 2.59135 , 4.29567 , 15.5    , 28.0295 ],
[ 1.0875 , 2.64312 , 4.32156 , 15.5    , 28.3398 ],
[ 1.1125 , 2.69655 , 4.34828 , 15.5    , 28.6599 ],
[ 1.1375 , 2.75166 , 4.37583 , 15.5    , 28.9902 ],
[ 1.1625 , 2.8085  , 4.40425 , 15.5    , 29.3308 ],
[ 1.1875 , 2.86709 , 4.43354 , 15.5    , 29.6819 ],
[ 1.2125 , 2.92747 , 4.46373 , 15.5    , 30.0438 ],
[ 1.2375 , 2.98968 , 4.49484 , 15.5    , 30.4166 ],
[ 1.2625 , 3.05375 , 4.52688 , 15.5    , 30.8006 ],
[ 1.2875 , 3.11974 , 4.55987 , 15.5    , 31.196 ],
[ 1.3125 , 3.18768 , 4.59384 , 15.5    , 31.6032 ],
[ 1.3375 , 3.25761 , 4.6288  , 15.5    , 32.0222 ],
[ 1.3625 , 3.32957 , 4.66479 , 15.5    , 32.4535 ],
[ 1.3875 , 3.40362 , 4.70181 , 15.5    , 32.8973 ],
[ 1.4125 , 3.47979 , 4.7399  , 15.5    , 33.3538 ],
[ 1.4375 , 0 , 0 , 0 , 0 ],
[ 1.4625 , 0 , 0 , 0 , 0 ],
[ 1.4875 , 0 , 0 , 0 , 0 ],
[ 1.5125 , 0.0  , 8.85762   , 19.5168 , 30.0369  ], # copied
[ 1.5375 , 0.0  , 8.85762   , 19.5168 , 30.0369  ], # copied
[ 1.5625 , 0.0  , 8.85762   , 19.5168 , 30.0369  ],
[ 1.5875 , 0.0  , 7.24613   , 17.9683 , 28.5757  ],
[ 1.6125 , 0.0  , 7.11249   , 17.9466 , 28.6607  ],
[ 1.6375 , 0.0  , 7.03737   , 17.7263 , 28.4004  ],
[ 1.6625 , 0.0  , 7.44517   , 18.1864 , 28.9547  ],
[ 1.6875 , 0.0  , 7.74611   , 18.5419 , 29.3555  ],
[ 1.7125 , 0.0  , 8.06184   , 18.8982 , 29.8214  ],
[ 1.7375 , 0.0  , 6.90503   , 17.8396 , 28.7947  ],
[ 1.7625 , 0.0  , 6.53893   , 17.5996 , 28.7122  ],
[ 1.7875 , 0.0  , 6.42897   , 17.1062 , 28.3578  ],
[ 1.8125 , 0.0  , 6.41139   , 17.1382 , 28.4191  ],
[ 1.8375 , 0.0  , 5.72471   , 16.4543 , 27.8796  ],
[ 1.8625 , 0.0  , 5.19018   , 16.1606 , 27.7097  ],
[ 1.8875 , 0.0  , 5.04008   , 16.1174 , 27.7649  ],
[ 1.9125 , 0.0  , 5.01225   , 16.184 , 27.9175  ],
[ 1.9375 , 0.0  , 4.95996   , 16.2482 , 28.1358  ],
[ 1.9625 , 0.0  , 4.98242   , 16.3591 , 28.3336  ],
[ 1.9875 , 0.0  , 5.04856   , 16.4194 , 28.5289  ],
[ 2.0125 , 0.0  , 5.32584   , 16.8194 , 29.0619  ],
[ 2.0375 , 0.0  , 5.2971   , 16.8203 , 29.1545  ],
[ 2.0625 , 0.0  , 5.41049   , 17.0891 , 29.5856  ],
[ 2.0875 , 0.0  , 5.48954   , 17.2965 , 29.9571  ],
[ 2.1125 , 0.0  , 5.50876   , 16.8573 , 29.6199  ],
[ 2.1375 , 0.0  , 5.45123   , 16.885 , 29.8321  ],
[ 2.1625 , 0.0  , 5.54906   , 17.1228 , 30.2044  ],
[ 2.1875 , 0.0  , 5.53662   , 17.1833 , 30.4852  ],
[ 2.2125 , 0.0  , 5.46801   , 17.1857 , 30.5842  ],
[ 2.2375 , 0.0  , 5.45581   , 16.6621 , 30.2958  ],
[ 2.2625 , 0.0  , 5.46682   , 16.7899 , 30.5222  ],
[ 2.2875 , 0.0  , 5.52693   , 16.9916 , 30.9557  ],
[ 2.3125 , 0.0  , 5.53211   , 17.0801 , 31.1637  ],
[ 2.3375 , 0.0  , 4.79919   , 16.4387 , 31.5318  ],
[ 2.3625 , 0.0  , 4.80101   , 16.5808 , 31.8533  ],
[ 2.3875 , 0.0  , 4.77726   , 15.9355 , 31.4454  ],
[ 2.4125 , 0.0  , 4.76845   , 16.0665 , 31.8002  ],
[ 2.4375 , 0.0  , 4.5959   , 15.9977 , 32.1175  ],
[ 2.4625 , 0.0  , 4.68513   , 16.2385 , 32.5042  ],
[ 2.4875 , 0.0  , 4.7232   , 16.4144 , 33.028  ]
];

#######################################################################


class CaloSwCalibHitsCalibration_v8_parms:
    eta_start_crack = 1.425
    eta_end_crack = 1.55
    etamax = 2.5
    use_raw_eta = True
    fix_v6_pathologies = False
    update_sampling_energies = False

    sampling_depth = samplings_depths

    region = CALOCORR_COMBINED2

    correction = {'ele55'        : CaloSwCalibHitsCalibration_v8_ele55,
                  'ele35'        : CaloSwCalibHitsCalibration_v8_ele35,
                  'ele37'        : CaloSwCalibHitsCalibration_v8_ele37,
                  'gam55_conv'   : CaloSwCalibHitsCalibration_v8_gam55_conv,
                  'gam35_conv'   : CaloSwCalibHitsCalibration_v8_gam35_conv,
                  'gam37_conv'   : CaloSwCalibHitsCalibration_v8_gam37_conv,
                  'gam55_unconv' : CaloSwCalibHitsCalibration_v8_gam55_unconv,
                  'gam35_unconv' : CaloSwCalibHitsCalibration_v8_gam35_unconv,
                  'gam37_unconv' : CaloSwCalibHitsCalibration_v8_gam37_unconv,

                   # Use 5x5 for cluster sizes that aren't explicitly derived.
                  'ele33'        : CaloSwCalibHitsCalibration_v8_ele55,
                  'ele57'        : CaloSwCalibHitsCalibration_v8_ele55,
                  'ele77'        : CaloSwCalibHitsCalibration_v8_ele55,
                  'gam33_conv'   : CaloSwCalibHitsCalibration_v8_gam55_conv,
                  'gam57_conv'   : CaloSwCalibHitsCalibration_v8_gam55_conv,
                  'gam77_conv'   : CaloSwCalibHitsCalibration_v8_gam55_conv,
                  'gam33_unconv' : CaloSwCalibHitsCalibration_v8_gam55_unconv,
                  'gam57_unconv' : CaloSwCalibHitsCalibration_v8_gam55_unconv,
                  'gam77_unconv' : CaloSwCalibHitsCalibration_v8_gam55_unconv,
                  }
