# Copyright (C) 2002-2017 CERN for the benefit of the ATLAS collaboration

#
#######################################################################
# File: CaloClusterCorrection/python/CaloSwCalibHitsCalibration_vX.py
# Created: July 2008, lc db                                                  
# Purpose: Calibration Hits based em cluster calibration, v9.                
#######################################################################
#
# - These corrections were derived by D. Banfi, L.C., L. Mandelli            
# and E. Nebot Del Busto on ATLAS-CSC-05-00-00 simulation.                  
#
# - Corrections are calculated from photons as in
# rel 14.2.0.                                                                
#
# - In v9 calibration coeffs for conv/unconv are separated                  
#
# The final correction CaloSwCalibHitsCalibration_v9_eleXX consists of
# five subcorrections which should be applied together:
# 1) CaloSwCalHitsAccordion_v9_XX (cluster size dependent)
# 2) CaloSWCalHitsOutOfCone_v9_eleXX (cluster size dependent)
# 3) CaloSWCalHitsLongLeakage_v9_ele
# 4) CaloSWCalHitsEfront_offset_v9_ele
# 5) CaloSWCalHitsEfront_slope_v9_ele
#
# Almost the same for photons
# 1) CaloSwCalHitsAccordion_v9_XX_YY (cluster size dependent)
# 2) CaloSWCalHitsOutOfCone_v9_gamXX_YY (cluster size dependent)
# 3) CaloSWCalHitsLongLeakage_v9_gam_YY
# 4) CaloSWCalHitsEfront_offset_v9_gam_YY
# 5) CaloSWCalHitsEfront_slope_v9_gam_YY
#
# Where YY is "conv" for converted or "unconv" for unconverted
#
# - January 2011 : update of all the calibration coefficients based on
# GEO-16-00-00 by LC, H. de la Torre, I. Koletsou, L. Mandelli, J del Peso,
# R. Turra.
#
# - June 2011 : update of all the calibration coefficients based on
# GEO-18-00-00 by LC, H. de la Torre, I. Koletsou, L. Mandelli, J del Peso,
# R. Turra.
#######################################################################

from CaloClusterCorrection.common import *

#######################################################################

# Accordion correction

#######################################################################




CaloSwCalHitsAccordion_v9leakdata_35 = [
[ 0.0125,   1.0293e+00,   -5.1824e-03,   2.1307e-04  ],# bin 0  chi2/NDF: 2.8998, copied from bin 2
[ 0.0375,   1.0293e+00,   -5.1824e-03,   2.1307e-04  ],# bin 1  chi2/NDF: 4.3876, copied from bin 2
[ 0.0625,   1.0293e+00,   -5.1824e-03,   2.1307e-04  ],# bin 2  chi2/NDF: 4.1267
[ 0.0875,   1.0243e+00,   -4.2642e-03,   1.7169e-04  ],# bin 3  chi2/NDF: 2.8583
[ 0.1125,   1.0259e+00,   -4.5752e-03,   1.8606e-04  ],# bin 4  chi2/NDF: 4.1851
[ 0.1375,   1.0265e+00,   -4.6760e-03,   1.9024e-04  ],# bin 5  chi2/NDF: 4.9092
[ 0.1625,   1.0257e+00,   -4.6401e-03,   1.9185e-04  ],# bin 6  chi2/NDF: 2.5465
[ 0.1875,   1.0264e+00,   -4.4966e-03,   1.6975e-04  ],# bin 7  chi2/NDF: 4.4604
[ 0.2125,   1.0249e+00,   -4.4026e-03,   1.7723e-04  ],# bin 8  chi2/NDF: 3.443
[ 0.2375,   1.0231e+00,   -4.2067e-03,   1.7142e-04  ],# bin 9  chi2/NDF: 3.1742
[ 0.2625,   1.0265e+00,   -4.6867e-03,   1.8820e-04  ],# bin 10  chi2/NDF: 3.2535
[ 0.2875,   1.0184e+00,   -3.2380e-03,   1.1794e-04  ],# bin 11  chi2/NDF: 3.779
[ 0.3125,   1.0237e+00,   -4.2546e-03,   1.7053e-04  ],# bin 12  chi2/NDF: 3.6129
[ 0.3375,   1.0199e+00,   -3.4544e-03,   1.3016e-04  ],# bin 13  chi2/NDF: 4.7358
[ 0.3625,   1.0190e+00,   -3.3113e-03,   1.2380e-04  ],# bin 14  chi2/NDF: 3.3581
[ 0.3875,   1.0207e+00,   -3.6733e-03,   1.3995e-04  ],# bin 15  chi2/NDF: 2.4667
[ 0.4125,   1.0212e+00,   -3.5912e-03,   1.3069e-04  ],# bin 16  chi2/NDF: 3.0034
[ 0.4375,   1.0194e+00,   -3.4068e-03,   1.2377e-04  ],# bin 17  chi2/NDF: 3.5097
[ 0.4625,   1.0148e+00,   -2.6363e-03,   9.2161e-05  ],# bin 18  chi2/NDF: 2.0432
[ 0.4875,   1.0240e+00,   -4.0619e-03,   1.4787e-04  ],# bin 19  chi2/NDF: 3.0091
[ 0.5125,   1.0218e+00,   -3.6943e-03,   1.3025e-04  ],# bin 20  chi2/NDF: 3.2098
[ 0.5375,   1.0206e+00,   -3.5551e-03,   1.2582e-04  ],# bin 21  chi2/NDF: 2.36
[ 0.5625,   1.0190e+00,   -3.1277e-03,   1.0233e-04  ],# bin 22  chi2/NDF: 3.6259
[ 0.5875,   1.0206e+00,   -3.4351e-03,   1.1495e-04  ],# bin 23  chi2/NDF: 3.0192
[ 0.6125,   1.0188e+00,   -3.1752e-03,   1.0492e-04  ],# bin 24  chi2/NDF: 4.016
[ 0.6375,   1.0219e+00,   -3.7135e-03,   1.2635e-04  ],# bin 25  chi2/NDF: 2.8474
[ 0.6625,   1.0177e+00,   -3.0907e-03,   1.0315e-04  ],# bin 26  chi2/NDF: 3.7789
[ 0.6875,   1.0217e+00,   -3.8559e-03,   1.3612e-04  ],# bin 27  chi2/NDF: 4.0067
[ 0.7125,   1.0150e+00,   -2.7898e-03,   9.5528e-05  ],# bin 28  chi2/NDF: 3.7707
[ 0.7375,   1.0119e+00,   -1.9463e-03,   3.1561e-05  ],# bin 29  chi2/NDF: 4.5842
[ 0.7625,   1.0119e+00,   -1.9463e-03,   3.1561e-05  ],# bin 30  chi2/NDF: 3.5936, copied from bin 29
[ 0.7875,   1.0119e+00,   -1.9463e-03,   3.1561e-05  ],# bin 31  chi2/NDF: 2.6118, copied from bin 29
[ 0.8125,   1.0443e+00,   -6.0038e-03,   1.8170e-04  ],# bin 32  chi2/NDF: 1.658, copied from bin 34
[ 0.8375,   1.0443e+00,   -6.0038e-03,   1.8170e-04  ],# bin 33  chi2/NDF: 1.6518, copied from bin 34
[ 0.8625,   1.0443e+00,   -6.0038e-03,   1.8170e-04  ],# bin 34  chi2/NDF: 1.8212
[ 0.8875,   1.0514e+00,   -6.8406e-03,   2.0562e-04  ],# bin 35  chi2/NDF: 0.77236
[ 0.9125,   1.0452e+00,   -6.1953e-03,   1.9168e-04  ],# bin 36  chi2/NDF: 1.4077
[ 0.9375,   1.0351e+00,   -4.4466e-03,   1.1643e-04  ],# bin 37  chi2/NDF: 1.7669
[ 0.9625,   1.0293e+00,   -3.7633e-03,   9.8107e-05  ],# bin 38  chi2/NDF: 0.70834
[ 0.9875,   1.0393e+00,   -5.4461e-03,   1.6803e-04  ],# bin 39  chi2/NDF: 1.3831
[ 1.0125,   1.0362e+00,   -4.9083e-03,   1.4560e-04  ],# bin 40  chi2/NDF: 1.2968
[ 1.0375,   1.0340e+00,   -4.6153e-03,   1.3525e-04  ],# bin 41  chi2/NDF: 1.8571
[ 1.0625,   1.0291e+00,   -3.8294e-03,   1.0470e-04  ],# bin 42  chi2/NDF: 2.0714
[ 1.0875,   1.0266e+00,   -3.2846e-03,   7.6623e-05  ],# bin 43  chi2/NDF: 1.1186
[ 1.1125,   1.0278e+00,   -3.3218e-03,   7.5495e-05  ],# bin 44  chi2/NDF: 1.5644
[ 1.1375,   1.0298e+00,   -3.8644e-03,   1.0313e-04  ],# bin 45  chi2/NDF: 2.7043
[ 1.1625,   1.0287e+00,   -3.5534e-03,   8.4709e-05  ],# bin 46  chi2/NDF: 2.2876
[ 1.1875,   1.0389e+00,   -4.7682e-03,   1.2340e-04  ],# bin 47  chi2/NDF: 2.2995
[ 1.2125,   1.0372e+00,   -4.6465e-03,   1.2227e-04  ],# bin 48  chi2/NDF: 2.043
[ 1.2375,   1.0289e+00,   -3.0122e-03,   4.9475e-05  ],# bin 49  chi2/NDF: 2.3128
[ 1.2625,   1.0280e+00,   -2.7904e-03,   4.0494e-05  ],# bin 50  chi2/NDF: 2.3446
[ 1.2875,   1.0352e+00,   -3.7542e-03,   7.3071e-05  ],# bin 51  chi2/NDF: 2.5842
[ 1.3125,   1.0328e+00,   -3.1184e-03,   3.7471e-05  ],# bin 52  chi2/NDF: 2.5452
[ 1.3375,   1.0303e+00,   -2.2978e-03,   -9.0460e-06  ],# bin 53  chi2/NDF: 1.034
[ 1.3625,   1.0327e+00,   -3.0015e-03,   2.1616e-05  ],# bin 54  chi2/NDF: 2.2769
[ 1.3875,   1.0287e+00,   -1.9734e-03,   -5.2563e-05  ],# bin 55  chi2/NDF: 2.2524
[ 1.4125,   1.0947e+00,   -1.5722e-02,   6.1247e-04  ],# bin 56  chi2/NDF: 1.6119
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   9.9744e-01,   4.6626e-03,   -2.2680e-04  ],# bin 60 crack  chi2/NDF: 7.7791
[ 1.5375,   1.0412e+00,   -7.2218e-03,   2.2997e-04  ],# bin 61 crack  chi2/NDF: 2.8454
[ 1.5625,   1.0251e+00,   -4.6940e-03,   1.4568e-04  ],# bin 62  chi2/NDF: 1.3406
[ 1.5875,   1.0156e+00,   -3.2620e-03,   1.0329e-04  ],# bin 63  chi2/NDF: 1.7289
[ 1.6125,   1.0207e+00,   -4.6145e-03,   1.6225e-04  ],# bin 64  chi2/NDF: 1.005
[ 1.6375,   1.0206e+00,   -5.2713e-03,   1.9414e-04  ],# bin 65  chi2/NDF: 1.8896
[ 1.6625,   1.0145e+00,   -4.1926e-03,   1.4665e-04  ],# bin 66  chi2/NDF: 2.0013
[ 1.6875,   1.0265e+00,   -6.2588e-03,   2.3365e-04  ],# bin 67  chi2/NDF: 0.95389
[ 1.7125,   1.0238e+00,   -5.8797e-03,   2.1416e-04  ],# bin 68  chi2/NDF: 1.1253
[ 1.7375,   1.0212e+00,   -6.1538e-03,   2.4347e-04  ],# bin 69  chi2/NDF: 0.76273
[ 1.7625,   1.0245e+00,   -6.9905e-03,   2.8000e-04  ],# bin 70  chi2/NDF: 0.74339
[ 1.7875,   1.0200e+00,   -6.8562e-03,   2.8888e-04  ],# bin 71  chi2/NDF: 1.3468
[ 1.8125,   1.0299e+00,   -8.6069e-03,   3.6518e-04  ],# bin 72  chi2/NDF: 1.2483
[ 1.8375,   1.0549e+00,   -1.2471e-02,   5.2434e-04  ],# bin 73  chi2/NDF: 1.2511
[ 1.8625,   1.0443e+00,   -1.1410e-02,   5.0517e-04  ],# bin 74  chi2/NDF: 0.73873
[ 1.8875,   1.0398e+00,   -1.0877e-02,   4.8769e-04  ],# bin 75  chi2/NDF: 1.0382
[ 1.9125,   1.0520e+00,   -1.2980e-02,   5.6954e-04  ],# bin 76  chi2/NDF: 1.2497
[ 1.9375,   1.0533e+00,   -1.3257e-02,   5.7563e-04  ],# bin 77  chi2/NDF: 1.0318
[ 1.9625,   1.0375e+00,   -1.1335e-02,   5.0939e-04  ],# bin 78  chi2/NDF: 1.4799
[ 1.9875,   1.0343e+00,   -1.0928e-02,   4.8772e-04  ],# bin 79  chi2/NDF: 0.94369
[ 2.0125,   1.0342e+00,   -1.0119e-02,   4.4461e-04  ],# bin 80  chi2/NDF: 1.0843
[ 2.0375,   1.0370e+00,   -1.0430e-02,   4.6314e-04  ],# bin 81  chi2/NDF: 1.216
[ 2.0625,   1.0248e+00,   -9.1173e-03,   4.0393e-04  ],# bin 82  chi2/NDF: 1.1739
[ 2.0875,   1.0076e+00,   -7.3035e-03,   3.3520e-04  ],# bin 83  chi2/NDF: 0.93466
[ 2.1125,   1.0123e+00,   -8.2845e-03,   3.8479e-04  ],# bin 84  chi2/NDF: 1.861
[ 2.1375,   1.0050e+00,   -6.7326e-03,   3.3018e-04  ],# bin 85  chi2/NDF: 1.1635
[ 2.1625,   1.0101e+00,   -7.3611e-03,   3.4709e-04  ],# bin 86  chi2/NDF: 0.63109
[ 2.1875,   1.0091e+00,   -7.2065e-03,   3.4152e-04  ],# bin 87  chi2/NDF: 1.3647
[ 2.2125,   1.0059e+00,   -6.9175e-03,   3.3402e-04  ],# bin 88  chi2/NDF: 1.2796
[ 2.2375,   9.9990e-01,   -6.2661e-03,   3.2132e-04  ],# bin 89  chi2/NDF: 1.1631
[ 2.2625,   1.0047e+00,   -7.2070e-03,   3.5387e-04  ],# bin 90  chi2/NDF: 1.4154
[ 2.2875,   9.9807e-01,   -6.4269e-03,   3.2370e-04  ],# bin 91  chi2/NDF: 1.6793
[ 2.3125,   1.0048e+00,   -7.6060e-03,   3.7614e-04  ],# bin 92  chi2/NDF: 1.0363
[ 2.3375,   1.0531e+00,   -1.4090e-02,   6.0160e-04  ],# bin 93  chi2/NDF: 1.5705
[ 2.3625,   1.0496e+00,   -1.3780e-02,   5.8325e-04  ],# bin 94  chi2/NDF: 1.8354
[ 2.3875,   1.0378e+00,   -1.2177e-02,   5.2635e-04  ],# bin 95  chi2/NDF: 1.3089
[ 2.4125,   1.1318e+00,   -2.3084e-02,   8.2566e-04  ],# bin 96  chi2/NDF: 1.2147
[ 2.4375,   1.2720e+00,   -4.0884e-02,   1.3800e-03  ],# bin 97  chi2/NDF: 0.93386
[ 2.4625,   1.2746e+00,   -4.1004e-02,   1.3626e-03  ],# bin 98  chi2/NDF: 1.3316
[ 2.4875,   1.6763e+00,   -1.0199e-01,   3.6699e-03  ],# bin 99  chi2/NDF: 87.049
] # end CaloSwCalHitsAccordion_v9_35


CaloSwCalHitsAccordion_v9leakdata_37 = [
[ 0.0125,   1.0467e+00,   -7.7054e-03,   3.1109e-04  ],# bin 0  chi2/NDF: 2.7595, copied from bin 2
[ 0.0375,   1.0467e+00,   -7.7054e-03,   3.1109e-04  ],# bin 1  chi2/NDF: 5.5974, copied from bin 2
[ 0.0625,   1.0467e+00,   -7.7054e-03,   3.1109e-04  ],# bin 2  chi2/NDF: 3.4054
[ 0.0875,   1.0475e+00,   -7.7928e-03,   3.1292e-04  ],# bin 3  chi2/NDF: 4.3634
[ 0.1125,   1.0462e+00,   -7.5581e-03,   3.0181e-04  ],# bin 4  chi2/NDF: 6.2109
[ 0.1375,   1.0468e+00,   -7.6654e-03,   3.0656e-04  ],# bin 5  chi2/NDF: 5.733
[ 0.1625,   1.0479e+00,   -7.9583e-03,   3.2223e-04  ],# bin 6  chi2/NDF: 3.9614
[ 0.1875,   1.0508e+00,   -8.2959e-03,   3.2445e-04  ],# bin 7  chi2/NDF: 5.8441
[ 0.2125,   1.0480e+00,   -7.8689e-03,   3.1361e-04  ],# bin 8  chi2/NDF: 4.8024
[ 0.2375,   1.0479e+00,   -7.9919e-03,   3.2255e-04  ],# bin 9  chi2/NDF: 6.0616
[ 0.2625,   1.0467e+00,   -7.5869e-03,   2.9764e-04  ],# bin 10  chi2/NDF: 4.2928
[ 0.2875,   1.0406e+00,   -6.5454e-03,   2.4781e-04  ],# bin 11  chi2/NDF: 4.9348
[ 0.3125,   1.0453e+00,   -7.4181e-03,   2.9273e-04  ],# bin 12  chi2/NDF: 4.0463
[ 0.3375,   1.0380e+00,   -6.0394e-03,   2.2850e-04  ],# bin 13  chi2/NDF: 5.4628
[ 0.3625,   1.0380e+00,   -6.0472e-03,   2.2843e-04  ],# bin 14  chi2/NDF: 3.7203
[ 0.3875,   1.0380e+00,   -6.1886e-03,   2.3753e-04  ],# bin 15  chi2/NDF: 3.3272
[ 0.4125,   1.0397e+00,   -6.1847e-03,   2.2726e-04  ],# bin 16  chi2/NDF: 3.6363
[ 0.4375,   1.0384e+00,   -6.0180e-03,   2.1935e-04  ],# bin 17  chi2/NDF: 4.9274
[ 0.4625,   1.0356e+00,   -5.5140e-03,   1.9747e-04  ],# bin 18  chi2/NDF: 2.3445
[ 0.4875,   1.0409e+00,   -6.3199e-03,   2.2876e-04  ],# bin 19  chi2/NDF: 3.2765
[ 0.5125,   1.0411e+00,   -6.2756e-03,   2.2234e-04  ],# bin 20  chi2/NDF: 3.6161
[ 0.5375,   1.0379e+00,   -5.8482e-03,   2.0776e-04  ],# bin 21  chi2/NDF: 2.7479
[ 0.5625,   1.0391e+00,   -5.8736e-03,   2.0204e-04  ],# bin 22  chi2/NDF: 3.751
[ 0.5875,   1.0411e+00,   -6.2435e-03,   2.1678e-04  ],# bin 23  chi2/NDF: 3.5884
[ 0.6125,   1.0379e+00,   -5.7940e-03,   2.0051e-04  ],# bin 24  chi2/NDF: 4.9582
[ 0.6375,   1.0396e+00,   -6.1172e-03,   2.1428e-04  ],# bin 25  chi2/NDF: 3.8199
[ 0.6625,   1.0350e+00,   -5.4669e-03,   1.9102e-04  ],# bin 26  chi2/NDF: 5.3234
[ 0.6875,   1.0374e+00,   -5.9837e-03,   2.1473e-04  ],# bin 27  chi2/NDF: 6.1336
[ 0.7125,   1.0269e+00,   -4.3669e-03,   1.5331e-04  ],# bin 28  chi2/NDF: 4.6794
[ 0.7375,   1.0269e+00,   -4.0690e-03,   1.1392e-04  ],# bin 29  chi2/NDF: 6.6419
[ 0.7625,   1.0269e+00,   -4.0690e-03,   1.1392e-04  ],# bin 30  chi2/NDF: 4.7343, copied from bin 29
[ 0.7875,   1.0269e+00,   -4.0690e-03,   1.1392e-04  ],# bin 31  chi2/NDF: 2.914, copied from bin 29
[ 0.8125,   1.0646e+00,   -8.6963e-03,   2.7792e-04  ],# bin 32  chi2/NDF: 1.4264, copied from bin 34
[ 0.8375,   1.0646e+00,   -8.6963e-03,   2.7792e-04  ],# bin 33  chi2/NDF: 1.2935, copied from bin 34
[ 0.8625,   1.0646e+00,   -8.6963e-03,   2.7792e-04  ],# bin 34  chi2/NDF: 1.6936
[ 0.8875,   1.0719e+00,   -9.5598e-03,   3.0306e-04  ],# bin 35  chi2/NDF: 1.2566
[ 0.9125,   1.0621e+00,   -8.3639e-03,   2.6733e-04  ],# bin 36  chi2/NDF: 1.0914
[ 0.9375,   1.0536e+00,   -6.8982e-03,   2.0501e-04  ],# bin 37  chi2/NDF: 1.8863
[ 0.9625,   1.0482e+00,   -6.2913e-03,   1.9044e-04  ],# bin 38  chi2/NDF: 1.5636
[ 0.9875,   1.0553e+00,   -7.5223e-03,   2.4267e-04  ],# bin 39  chi2/NDF: 1.6832
[ 1.0125,   1.0538e+00,   -7.2225e-03,   2.2954e-04  ],# bin 40  chi2/NDF: 1.3683
[ 1.0375,   1.0483e+00,   -6.4397e-03,   2.0055e-04  ],# bin 41  chi2/NDF: 2.4312
[ 1.0625,   1.0452e+00,   -5.9826e-03,   1.8469e-04  ],# bin 42  chi2/NDF: 2.1116
[ 1.0875,   1.0431e+00,   -5.5332e-03,   1.6128e-04  ],# bin 43  chi2/NDF: 2.5129
[ 1.1125,   1.0419e+00,   -5.1780e-03,   1.4438e-04  ],# bin 44  chi2/NDF: 2.075
[ 1.1375,   1.0439e+00,   -5.7101e-03,   1.7106e-04  ],# bin 45  chi2/NDF: 2.89
[ 1.1625,   1.0386e+00,   -4.7135e-03,   1.2500e-04  ],# bin 46  chi2/NDF: 2.3901
[ 1.1875,   1.0526e+00,   -6.5466e-03,   1.8915e-04  ],# bin 47  chi2/NDF: 2.7367
[ 1.2125,   1.0524e+00,   -6.7014e-03,   2.0018e-04  ],# bin 48  chi2/NDF: 2.7892
[ 1.2375,   1.0434e+00,   -4.9479e-03,   1.2152e-04  ],# bin 49  chi2/NDF: 3.0008
[ 1.2625,   1.0398e+00,   -4.2370e-03,   9.2114e-05  ],# bin 50  chi2/NDF: 1.9308
[ 1.2875,   1.0490e+00,   -5.5326e-03,   1.3807e-04  ],# bin 51  chi2/NDF: 2.8464
[ 1.3125,   1.0466e+00,   -4.9262e-03,   1.0492e-04  ],# bin 52  chi2/NDF: 2.9457
[ 1.3375,   1.0431e+00,   -3.9412e-03,   5.0991e-05  ],# bin 53  chi2/NDF: 2.0289
[ 1.3625,   1.0452e+00,   -4.6231e-03,   8.0962e-05  ],# bin 54  chi2/NDF: 2.8811
[ 1.3875,   1.0381e+00,   -3.1680e-03,   -9.9150e-06  ],# bin 55  chi2/NDF: 1.9839
[ 1.4125,   1.1086e+00,   -1.7986e-02,   7.1359e-04  ],# bin 56  chi2/NDF: 1.2236
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   1.0048e+00,   4.2847e-03,   -2.2379e-04  ],# bin 60 crack  chi2/NDF: 6.3558
[ 1.5375,   1.0561e+00,   -8.8858e-03,   2.8217e-04  ],# bin 61 crack  chi2/NDF: 2.7036
[ 1.5625,   1.0355e+00,   -5.6825e-03,   1.7266e-04  ],# bin 62  chi2/NDF: 2.1937
[ 1.5875,   1.0269e+00,   -4.4387e-03,   1.3904e-04  ],# bin 63  chi2/NDF: 1.5895
[ 1.6125,   1.0335e+00,   -6.0391e-03,   2.0736e-04  ],# bin 64  chi2/NDF: 0.85781
[ 1.6375,   1.0321e+00,   -6.5189e-03,   2.3385e-04  ],# bin 65  chi2/NDF: 2.4514
[ 1.6625,   1.0280e+00,   -5.7231e-03,   1.9674e-04  ],# bin 66  chi2/NDF: 2.48
[ 1.6875,   1.0389e+00,   -7.6439e-03,   2.7943e-04  ],# bin 67  chi2/NDF: 1.1726
[ 1.7125,   1.0337e+00,   -6.7752e-03,   2.3843e-04  ],# bin 68  chi2/NDF: 1.1177
[ 1.7375,   1.0315e+00,   -7.1263e-03,   2.7062e-04  ],# bin 69  chi2/NDF: 1.1171
[ 1.7625,   1.0338e+00,   -7.8327e-03,   3.0319e-04  ],# bin 70  chi2/NDF: 1.4099
[ 1.7875,   1.0314e+00,   -8.0568e-03,   3.2639e-04  ],# bin 71  chi2/NDF: 1.1102
[ 1.8125,   1.0381e+00,   -9.2832e-03,   3.8257e-04  ],# bin 72  chi2/NDF: 1.4604
[ 1.8375,   1.0686e+00,   -1.3890e-02,   5.6666e-04  ],# bin 73  chi2/NDF: 1.4561
[ 1.8625,   1.0605e+00,   -1.3263e-02,   5.6533e-04  ],# bin 74  chi2/NDF: 0.95125
[ 1.8875,   1.0542e+00,   -1.2454e-02,   5.3740e-04  ],# bin 75  chi2/NDF: 1.2873
[ 1.9125,   1.0660e+00,   -1.4552e-02,   6.2087e-04  ],# bin 76  chi2/NDF: 1.3929
[ 1.9375,   1.0625e+00,   -1.4114e-02,   6.0067e-04  ],# bin 77  chi2/NDF: 1.1673
[ 1.9625,   1.0503e+00,   -1.2707e-02,   5.5334e-04  ],# bin 78  chi2/NDF: 1.4194
[ 1.9875,   1.0434e+00,   -1.1734e-02,   5.1068e-04  ],# bin 79  chi2/NDF: 2.0249
[ 2.0125,   1.0461e+00,   -1.1313e-02,   4.8125e-04  ],# bin 80  chi2/NDF: 0.83831
[ 2.0375,   1.0480e+00,   -1.1487e-02,   4.9474e-04  ],# bin 81  chi2/NDF: 1.6004
[ 2.0625,   1.0357e+00,   -1.0154e-02,   4.3509e-04  ],# bin 82  chi2/NDF: 1.0144
[ 2.0875,   1.0188e+00,   -8.3680e-03,   3.6698e-04  ],# bin 83  chi2/NDF: 1.2321
[ 2.1125,   1.0248e+00,   -9.5371e-03,   4.2370e-04  ],# bin 84  chi2/NDF: 1.1778
[ 2.1375,   1.0191e+00,   -8.2772e-03,   3.8167e-04  ],# bin 85  chi2/NDF: 1.3747
[ 2.1625,   1.0190e+00,   -8.0971e-03,   3.6846e-04  ],# bin 86  chi2/NDF: 1.0062
[ 2.1875,   1.0193e+00,   -8.1514e-03,   3.7064e-04  ],# bin 87  chi2/NDF: 1.4434
[ 2.2125,   1.0181e+00,   -8.1327e-03,   3.7243e-04  ],# bin 88  chi2/NDF: 1.5432
[ 2.2375,   1.0082e+00,   -6.9340e-03,   3.4071e-04  ],# bin 89  chi2/NDF: 0.76094
[ 2.2625,   1.0152e+00,   -8.1877e-03,   3.8442e-04  ],# bin 90  chi2/NDF: 1.1545
[ 2.2875,   1.0082e+00,   -7.3335e-03,   3.5132e-04  ],# bin 91  chi2/NDF: 1.847
[ 2.3125,   1.0130e+00,   -8.2239e-03,   3.9342e-04  ],# bin 92  chi2/NDF: 1.6252
[ 2.3375,   1.0641e+00,   -1.5015e-02,   6.2697e-04  ],# bin 93  chi2/NDF: 1.784
[ 2.3625,   1.0642e+00,   -1.5222e-02,   6.2725e-04  ],# bin 94  chi2/NDF: 1.9324
[ 2.3875,   1.0537e+00,   -1.3887e-02,   5.8271e-04  ],# bin 95  chi2/NDF: 1.7957
[ 2.4125,   1.1566e+00,   -2.6042e-02,   9.2660e-04  ],# bin 96  chi2/NDF: 1.2275
[ 2.4375,   1.2812e+00,   -4.1516e-02,   1.3947e-03  ],# bin 97  chi2/NDF: 0.85322
[ 2.4625,   1.2875e+00,   -4.2066e-02,   1.3905e-03  ],# bin 98  chi2/NDF: 1.1954
[ 2.4875,   1.6850e+00,   -1.0254e-01,   3.6835e-03  ],# bin 99  chi2/NDF: 84.152
] # end CaloSwCalHitsAccordion_v9_37


CaloSwCalHitsAccordion_v9leakdata_55 = [
[ 0.0125,   1.0389e+00,   -5.9639e-03,   2.3798e-04  ],# bin 0  chi2/NDF: 4.0577, copied from bin 2
[ 0.0375,   1.0389e+00,   -5.9639e-03,   2.3798e-04  ],# bin 1  chi2/NDF: 5.5156, copied from bin 2
[ 0.0625,   1.0389e+00,   -5.9639e-03,   2.3798e-04  ],# bin 2  chi2/NDF: 4.2988
[ 0.0875,   1.0317e+00,   -4.6626e-03,   1.8021e-04  ],# bin 3  chi2/NDF: 3.5834
[ 0.1125,   1.0343e+00,   -5.1050e-03,   1.9884e-04  ],# bin 4  chi2/NDF: 5.2289
[ 0.1375,   1.0348e+00,   -5.2085e-03,   2.0438e-04  ],# bin 5  chi2/NDF: 7.155
[ 0.1625,   1.0377e+00,   -5.7850e-03,   2.3069e-04  ],# bin 6  chi2/NDF: 4.964
[ 0.1875,   1.0396e+00,   -5.8999e-03,   2.2273e-04  ],# bin 7  chi2/NDF: 8.1228
[ 0.2125,   1.0351e+00,   -5.2710e-03,   2.0578e-04  ],# bin 8  chi2/NDF: 5.8163
[ 0.2375,   1.0344e+00,   -5.2730e-03,   2.0898e-04  ],# bin 9  chi2/NDF: 4.9637
[ 0.2625,   1.0387e+00,   -5.8542e-03,   2.2787e-04  ],# bin 10  chi2/NDF: 5.4646
[ 0.2875,   1.0296e+00,   -4.3264e-03,   1.5839e-04  ],# bin 11  chi2/NDF: 6.0606
[ 0.3125,   1.0316e+00,   -4.7430e-03,   1.8313e-04  ],# bin 12  chi2/NDF: 5.8872
[ 0.3375,   1.0272e+00,   -3.8236e-03,   1.3836e-04  ],# bin 13  chi2/NDF: 5.6801
[ 0.3625,   1.0304e+00,   -4.3719e-03,   1.6020e-04  ],# bin 14  chi2/NDF: 6.2024
[ 0.3875,   1.0322e+00,   -4.7098e-03,   1.7541e-04  ],# bin 15  chi2/NDF: 5.0115
[ 0.4125,   1.0344e+00,   -4.8906e-03,   1.7557e-04  ],# bin 16  chi2/NDF: 5.7109
[ 0.4375,   1.0251e+00,   -3.5070e-03,   1.2140e-04  ],# bin 17  chi2/NDF: 4.2654
[ 0.4625,   1.0249e+00,   -3.4492e-03,   1.1788e-04  ],# bin 18  chi2/NDF: 3.3875
[ 0.4875,   1.0388e+00,   -5.7452e-03,   2.1430e-04  ],# bin 19  chi2/NDF: 3.3081
[ 0.5125,   1.0297e+00,   -4.1276e-03,   1.4088e-04  ],# bin 20  chi2/NDF: 4.4184
[ 0.5375,   1.0266e+00,   -3.7282e-03,   1.2773e-04  ],# bin 21  chi2/NDF: 2.9172
[ 0.5625,   1.0299e+00,   -4.0556e-03,   1.3269e-04  ],# bin 22  chi2/NDF: 4.6731
[ 0.5875,   1.0354e+00,   -4.9811e-03,   1.7001e-04  ],# bin 23  chi2/NDF: 5.248
[ 0.6125,   1.0302e+00,   -4.2084e-03,   1.4046e-04  ],# bin 24  chi2/NDF: 5.1818
[ 0.6375,   1.0300e+00,   -4.2622e-03,   1.4483e-04  ],# bin 25  chi2/NDF: 4.9815
[ 0.6625,   1.0266e+00,   -3.7687e-03,   1.2610e-04  ],# bin 26  chi2/NDF: 5.392
[ 0.6875,   1.0314e+00,   -4.6456e-03,   1.6401e-04  ],# bin 27  chi2/NDF: 5.7621
[ 0.7125,   1.0254e+00,   -3.7086e-03,   1.2749e-04  ],# bin 28  chi2/NDF: 6.3149
[ 0.7375,   1.0268e+00,   -3.7930e-03,   1.1105e-04  ],# bin 29  chi2/NDF: 6.8241
[ 0.7625,   1.0268e+00,   -3.7930e-03,   1.1105e-04  ],# bin 30  chi2/NDF: 4.3506, copied from bin 29
[ 0.7875,   1.0268e+00,   -3.7930e-03,   1.1105e-04  ],# bin 31  chi2/NDF: 3.6823, copied from bin 29
[ 0.8125,   1.0603e+00,   -7.5868e-03,   2.3472e-04  ],# bin 32  chi2/NDF: 1.8376, copied from bin 34
[ 0.8375,   1.0603e+00,   -7.5868e-03,   2.3472e-04  ],# bin 33  chi2/NDF: 1.8106, copied from bin 34
[ 0.8625,   1.0603e+00,   -7.5868e-03,   2.3472e-04  ],# bin 34  chi2/NDF: 2.1106
[ 0.8875,   1.0609e+00,   -7.4746e-03,   2.2456e-04  ],# bin 35  chi2/NDF: 0.91049
[ 0.9125,   1.0548e+00,   -6.7882e-03,   2.0529e-04  ],# bin 36  chi2/NDF: 1.0944
[ 0.9375,   1.0533e+00,   -6.5293e-03,   1.9431e-04  ],# bin 37  chi2/NDF: 2.5415
[ 0.9625,   1.0402e+00,   -4.6109e-03,   1.2399e-04  ],# bin 38  chi2/NDF: 1.2851
[ 0.9875,   1.0502e+00,   -6.3254e-03,   1.9693e-04  ],# bin 39  chi2/NDF: 2.0463
[ 1.0125,   1.0462e+00,   -5.6497e-03,   1.6846e-04  ],# bin 40  chi2/NDF: 1.1355
[ 1.0375,   1.0466e+00,   -5.7993e-03,   1.7818e-04  ],# bin 41  chi2/NDF: 2.2283
[ 1.0625,   1.0408e+00,   -4.8701e-03,   1.3980e-04  ],# bin 42  chi2/NDF: 2.9344
[ 1.0875,   1.0380e+00,   -4.3117e-03,   1.1378e-04  ],# bin 43  chi2/NDF: 2.151
[ 1.1125,   1.0405e+00,   -4.5946e-03,   1.2229e-04  ],# bin 44  chi2/NDF: 2.6499
[ 1.1375,   1.0417e+00,   -5.0028e-03,   1.4497e-04  ],# bin 45  chi2/NDF: 3.7207
[ 1.1625,   1.0425e+00,   -4.9607e-03,   1.3532e-04  ],# bin 46  chi2/NDF: 3.0894
[ 1.1875,   1.0488e+00,   -5.5973e-03,   1.5537e-04  ],# bin 47  chi2/NDF: 2.0745
[ 1.2125,   1.0477e+00,   -5.5318e-03,   1.5068e-04  ],# bin 48  chi2/NDF: 2.888
[ 1.2375,   1.0410e+00,   -4.2784e-03,   9.9342e-05  ],# bin 49  chi2/NDF: 2.261
[ 1.2625,   1.0362e+00,   -3.2973e-03,   5.4395e-05  ],# bin 50  chi2/NDF: 2.125
[ 1.2875,   1.0479e+00,   -5.0899e-03,   1.2488e-04  ],# bin 51  chi2/NDF: 3.2058
[ 1.3125,   1.0436e+00,   -4.0986e-03,   7.1911e-05  ],# bin 52  chi2/NDF: 3.2519
[ 1.3375,   1.0443e+00,   -3.9531e-03,   5.9190e-05  ],# bin 53  chi2/NDF: 1.3983
[ 1.3625,   1.0427e+00,   -3.8065e-03,   4.4774e-05  ],# bin 54  chi2/NDF: 2.1198
[ 1.3875,   1.0384e+00,   -2.8006e-03,   -9.4730e-06  ],# bin 55  chi2/NDF: 2.9249
[ 1.4125,   1.0555e+00,   -6.4270e-03,   1.5107e-04  ],# bin 56  chi2/NDF: 1.6701
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   1.0337e+00,   7.6645e-04,   -1.1307e-04  ],# bin 60 crack  chi2/NDF: 5.3119
[ 1.5375,   1.0677e+00,   -9.9799e-03,   3.1739e-04  ],# bin 61 crack  chi2/NDF: 3.7548
[ 1.5625,   1.0368e+00,   -5.4368e-03,   1.6518e-04  ],# bin 62  chi2/NDF: 1.3909
[ 1.5875,   1.0279e+00,   -4.2539e-03,   1.3575e-04  ],# bin 63  chi2/NDF: 1.9153
[ 1.6125,   1.0321e+00,   -5.3812e-03,   1.8510e-04  ],# bin 64  chi2/NDF: 1.2011
[ 1.6375,   1.0358e+00,   -6.6284e-03,   2.3977e-04  ],# bin 65  chi2/NDF: 2.1553
[ 1.6625,   1.0268e+00,   -5.0461e-03,   1.7286e-04  ],# bin 66  chi2/NDF: 2.6012
[ 1.6875,   1.0428e+00,   -7.6870e-03,   2.8049e-04  ],# bin 67  chi2/NDF: 2.0711
[ 1.7125,   1.0392e+00,   -7.0775e-03,   2.5069e-04  ],# bin 68  chi2/NDF: 0.9899
[ 1.7375,   1.0346e+00,   -7.1285e-03,   2.7350e-04  ],# bin 69  chi2/NDF: 0.57767
[ 1.7625,   1.0371e+00,   -7.8488e-03,   3.0675e-04  ],# bin 70  chi2/NDF: 1.0603
[ 1.7875,   1.0337e+00,   -7.9027e-03,   3.2220e-04  ],# bin 71  chi2/NDF: 0.78208
[ 1.8125,   1.0427e+00,   -9.4359e-03,   3.9043e-04  ],# bin 72  chi2/NDF: 1.7801
[ 1.8375,   1.0757e+00,   -1.4459e-02,   5.9007e-04  ],# bin 73  chi2/NDF: 1.3711
[ 1.8625,   1.0571e+00,   -1.2236e-02,   5.2931e-04  ],# bin 74  chi2/NDF: 0.62684
[ 1.8875,   1.0487e+00,   -1.1167e-02,   4.9351e-04  ],# bin 75  chi2/NDF: 1.0728
[ 1.9125,   1.0654e+00,   -1.3960e-02,   6.0259e-04  ],# bin 76  chi2/NDF: 1.1852
[ 1.9375,   1.0678e+00,   -1.4340e-02,   6.1002e-04  ],# bin 77  chi2/NDF: 1.3464
[ 1.9625,   1.0474e+00,   -1.1740e-02,   5.2014e-04  ],# bin 78  chi2/NDF: 1.6093
[ 1.9875,   1.0459e+00,   -1.1588e-02,   5.0840e-04  ],# bin 79  chi2/NDF: 0.78784
[ 2.0125,   1.0516e+00,   -1.1611e-02,   4.9580e-04  ],# bin 80  chi2/NDF: 1.3091
[ 2.0375,   1.0496e+00,   -1.1234e-02,   4.8798e-04  ],# bin 81  chi2/NDF: 1.5696
[ 2.0625,   1.0377e+00,   -9.9039e-03,   4.2824e-04  ],# bin 82  chi2/NDF: 0.74759
[ 2.0875,   1.0222e+00,   -8.3214e-03,   3.6830e-04  ],# bin 83  chi2/NDF: 0.93774
[ 2.1125,   1.0261e+00,   -9.1974e-03,   4.1528e-04  ],# bin 84  chi2/NDF: 1.4459
[ 2.1375,   1.0199e+00,   -7.8606e-03,   3.6887e-04  ],# bin 85  chi2/NDF: 0.88617
[ 2.1625,   1.0262e+00,   -8.6138e-03,   3.8992e-04  ],# bin 86  chi2/NDF: 0.85183
[ 2.1875,   1.0238e+00,   -8.2608e-03,   3.7672e-04  ],# bin 87  chi2/NDF: 0.95846
[ 2.2125,   1.0160e+00,   -7.2763e-03,   3.4456e-04  ],# bin 88  chi2/NDF: 0.81571
[ 2.2375,   1.0130e+00,   -7.0963e-03,   3.4940e-04  ],# bin 89  chi2/NDF: 0.7679
[ 2.2625,   1.0179e+00,   -8.0313e-03,   3.8250e-04  ],# bin 90  chi2/NDF: 1.0261
[ 2.2875,   1.0132e+00,   -7.5072e-03,   3.6020e-04  ],# bin 91  chi2/NDF: 1.2052
[ 2.3125,   1.0211e+00,   -8.8320e-03,   4.1759e-04  ],# bin 92  chi2/NDF: 1.5031
[ 2.3375,   1.0673e+00,   -1.5040e-02,   6.3308e-04  ],# bin 93  chi2/NDF: 1.6445
[ 2.3625,   1.0662e+00,   -1.4986e-02,   6.2259e-04  ],# bin 94  chi2/NDF: 1.6782
[ 2.3875,   1.0645e+00,   -1.4982e-02,   6.2756e-04  ],# bin 95  chi2/NDF: 2.0463
[ 2.4125,   1.1456e+00,   -2.4315e-02,   8.7918e-04  ],# bin 96  chi2/NDF: 0.92191
[ 2.4375,   1.2732e+00,   -4.0683e-02,   1.3917e-03  ],# bin 97  chi2/NDF: 0.95878
[ 2.4625,   1.2610e+00,   -3.9014e-02,   1.3147e-03  ],# bin 98  chi2/NDF: 1.4546
[ 2.4875,   1.6498e+00,   -9.7749e-02,   3.5158e-03  ],# bin 99  chi2/NDF: 84.281
] # end CaloSwCalHitsAccordion_v9_55


CaloSwCalHitsEfront_offset_v9leakdata_ele = [
[ 0.0125,   3.0639e+02,   9.8775e-01,   -2.3023e-04  ],# bin 0  chi2/NDF: 1127.6, copied from bin 2
[ 0.0375,   3.0639e+02,   9.8775e-01,   -2.3023e-04  ],# bin 1  chi2/NDF: 1227.4, copied from bin 2
[ 0.0625,   3.0639e+02,   9.8775e-01,   -2.3023e-04  ],# bin 2  chi2/NDF: 764.68
[ 0.0875,   3.0012e+02,   1.1090e+00,   -3.5523e-04  ],# bin 3  chi2/NDF: 779.57
[ 0.1125,   2.9608e+02,   1.1251e+00,   -3.0344e-04  ],# bin 4  chi2/NDF: 1280.4
[ 0.1375,   3.0266e+02,   1.1046e+00,   -3.3087e-04  ],# bin 5  chi2/NDF: 1277.6
[ 0.1625,   3.0129e+02,   1.0882e+00,   -2.9827e-04  ],# bin 6  chi2/NDF: 1484.3
[ 0.1875,   2.9100e+02,   1.0744e+00,   -2.6661e-04  ],# bin 7  chi2/NDF: 1133.7
[ 0.2125,   2.8678e+02,   1.1733e+00,   -3.7405e-04  ],# bin 8  chi2/NDF: 824.32
[ 0.2375,   2.9212e+02,   1.1800e+00,   -3.8784e-04  ],# bin 9  chi2/NDF: 1201
[ 0.2625,   2.8908e+02,   1.2267e+00,   -4.1776e-04  ],# bin 10  chi2/NDF: 1165.6
[ 0.2875,   2.9308e+02,   1.2942e+00,   -3.9226e-04  ],# bin 11  chi2/NDF: 1504.9
[ 0.3125,   2.9717e+02,   1.3371e+00,   -4.5206e-04  ],# bin 12  chi2/NDF: 1446.1
[ 0.3375,   3.0089e+02,   1.2906e+00,   -3.3336e-04  ],# bin 13  chi2/NDF: 1409.9
[ 0.3625,   3.0495e+02,   1.4110e+00,   -4.3934e-04  ],# bin 14  chi2/NDF: 1351.1
[ 0.3875,   3.0991e+02,   1.4529e+00,   -4.7962e-04  ],# bin 15  chi2/NDF: 1658.9
[ 0.4125,   3.1582e+02,   1.4886e+00,   -5.1697e-04  ],# bin 16  chi2/NDF: 1941.6
[ 0.4375,   3.2264e+02,   1.4625e+00,   -4.5491e-04  ],# bin 17  chi2/NDF: 1512.8
[ 0.4625,   3.2876e+02,   1.5476e+00,   -4.8269e-04  ],# bin 18  chi2/NDF: 1609.1
[ 0.4875,   3.3664e+02,   1.6230e+00,   -5.2152e-04  ],# bin 19  chi2/NDF: 2014
[ 0.5125,   3.4920e+02,   1.8621e+00,   -5.9503e-04  ],# bin 20  chi2/NDF: 1552.1
[ 0.5375,   3.4757e+02,   1.8423e+00,   -4.8987e-04  ],# bin 21  chi2/NDF: 2160
[ 0.5625,   3.5753e+02,   2.0247e+00,   -5.2669e-04  ],# bin 22  chi2/NDF: 3651.8
[ 0.5875,   3.5723e+02,   2.1842e+00,   -6.0535e-04  ],# bin 23  chi2/NDF: 2026.4
[ 0.6125,   3.7852e+02,   2.0943e+00,   -6.0411e-04  ],# bin 24  chi2/NDF: 1661.7
[ 0.6375,   4.1571e+02,   2.0124e+00,   -3.4699e-04  ],# bin 25  chi2/NDF: 590.98
[ 0.6625,   4.8276e+02,   2.0263e+00,   -6.5929e-05  ],# bin 26  chi2/NDF: 1511.5
[ 0.6875,   5.3228e+02,   2.0506e+00,   3.3832e-05  ],# bin 27  chi2/NDF: 2188.5
[ 0.7125,   5.5349e+02,   2.0211e+00,   2.4184e-04  ],# bin 28  chi2/NDF: 2604
[ 0.7375,   6.0284e+02,   2.0195e+00,   5.1555e-04  ],# bin 29  chi2/NDF: 3061.6
[ 0.7625,   6.0284e+02,   2.0195e+00,   5.1555e-04  ],# bin 30  chi2/NDF: 3140.5, copied from bin 29
[ 0.7875,   6.0284e+02,   2.0195e+00,   5.1555e-04  ],# bin 31  chi2/NDF: 17590, copied from bin 29
[ 0.8125,   8.4474e+02,   1.6232e+00,   1.6704e-03  ],# bin 32  chi2/NDF: 6652, copied from bin 34
[ 0.8375,   8.4474e+02,   1.6232e+00,   1.6704e-03  ],# bin 33  chi2/NDF: 2916.5, copied from bin 34
[ 0.8625,   8.4474e+02,   1.6232e+00,   1.6704e-03  ],# bin 34  chi2/NDF: 7499.6
[ 0.8875,   8.9437e+02,   1.7495e+00,   1.9548e-03  ],# bin 35  chi2/NDF: 18932
[ 0.9125,   9.1605e+02,   1.7800e+00,   2.4101e-03  ],# bin 36  chi2/NDF: 14493
[ 0.9375,   9.3818e+02,   2.3957e+00,   1.6405e-03  ],# bin 37  chi2/NDF: 24561
[ 0.9625,   9.2687e+02,   2.9495e+00,   1.3406e-03  ],# bin 38  chi2/NDF: 15578
[ 0.9875,   8.7190e+02,   3.9865e+00,   9.7678e-04  ],# bin 39  chi2/NDF: 47829
[ 1.0125,   1.2656e+03,   1.6941e+00,   2.6894e-03  ],# bin 40  chi2/NDF: 13587
[ 1.0375,   1.2885e+03,   2.3758e+00,   2.7279e-03  ],# bin 41  chi2/NDF: 10953
[ 1.0625,   1.2802e+03,   3.2294e+00,   2.1044e-03  ],# bin 42  chi2/NDF: 13375
[ 1.0875,   1.2999e+03,   2.4057e+00,   3.3665e-03  ],# bin 43  chi2/NDF: 18253
[ 1.1125,   1.3304e+03,   3.5346e+00,   2.7636e-03  ],# bin 44  chi2/NDF: 9018.1
[ 1.1375,   1.3101e+03,   4.3324e+00,   2.2008e-03  ],# bin 45  chi2/NDF: 24157
[ 1.1625,   1.3894e+03,   4.4210e+00,   2.8862e-03  ],# bin 46  chi2/NDF: 14575
[ 1.1875,   1.4551e+03,   5.3999e+00,   3.3606e-03  ],# bin 47  chi2/NDF: 54060
[ 1.2125,   1.4478e+03,   6.2604e+00,   1.9901e-03  ],# bin 48  chi2/NDF: 51611
[ 1.2375,   1.5010e+03,   6.4925e+00,   1.9314e-03  ],# bin 49  chi2/NDF: 36041
[ 1.2625,   1.6188e+03,   6.5541e+00,   3.7535e-03  ],# bin 50  chi2/NDF: 52054
[ 1.2875,   1.4480e+03,   9.2684e+00,   2.9088e-03  ],# bin 51  chi2/NDF: 39188
[ 1.3125,   1.7364e+03,   8.6095e+00,   3.7468e-03  ],# bin 52  chi2/NDF: 77254
[ 1.3375,   1.6525e+03,   1.0441e+01,   4.3417e-03  ],# bin 53  chi2/NDF: 132840
[ 1.3625,   2.2023e+03,   1.0172e+01,   5.4600e-03  ],# bin 54  chi2/NDF: 47020
[ 1.3875,   2.1174e+03,   1.3507e+01,   6.7342e-03  ],# bin 55  chi2/NDF: 94586
[ 1.4125,   2.5735e+03,   1.7014e+01,   1.6943e-02  ],# bin 56  chi2/NDF: 241130
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -5.8576e+03,   5.4267e+02,   9.6229e+03  ],# bin 60 crack  chi2/NDF: 5.8062e+07
[ 1.5375,   -4.3063e+03,   1.6639e+00,   1.0925e+03  ],# bin 61 crack  chi2/NDF: 2.5816e+07
[ 1.5625,   1.2303e+03,   5.5534e+00,   8.6118e+01  ],# bin 62  chi2/NDF: 302560
[ 1.5875,   1.1665e+03,   4.4574e+00,   8.8927e+01  ],# bin 63  chi2/NDF: 409650
[ 1.6125,   1.7427e+03,   6.6619e+00,   -1.0834e+01  ],# bin 64  chi2/NDF: 260730
[ 1.6375,   9.8856e+02,   -1.8139e+00,   3.1697e+02  ],# bin 65  chi2/NDF: 237010
[ 1.6625,   4.3476e+02,   -3.3199e-01,   3.5701e+02  ],# bin 66  chi2/NDF: 895720
[ 1.6875,   -2.9519e+03,   1.7145e+00,   4.9399e+02  ],# bin 67  chi2/NDF: 4.2485e+06
[ 1.7125,   3.6993e+02,   -4.9901e-01,   2.0529e+02  ],# bin 68  chi2/NDF: 93800
[ 1.7375,   1.0753e+03,   2.4000e+00,   5.0957e+01  ],# bin 69  chi2/NDF: 40570
[ 1.7625,   1.3217e+03,   3.8918e+00,   -5.4443e+00  ],# bin 70  chi2/NDF: 86673
[ 1.7875,   8.0987e+02,   2.4761e+00,   4.0005e+01  ],# bin 71  chi2/NDF: 27116
[ 1.8125,   4.0807e+02,   9.4938e+02,   -1.8620e-01  ],# bin 72  chi2/NDF: 0.62843
[ 1.8375,   5.4402e+03,   1.0540e+03,   -2.1873e-01  ],# bin 73  chi2/NDF: 0.61835
[ 1.8625,   6.0201e+03,   8.3407e+02,   -2.4541e-03  ],# bin 74  chi2/NDF: 0.89471
[ 1.8875,   3.1239e+03,   7.8932e+02,   1.1001e-01  ],# bin 75  chi2/NDF: 0.2028
[ 1.9125,   3.7422e+03,   7.7554e+02,   -7.0441e-02  ],# bin 76  chi2/NDF: 0.9775
[ 1.9375,   3.5071e+03,   7.0076e+02,   -1.4364e-01  ],# bin 77  chi2/NDF: 1.5541
[ 1.9625,   4.3243e+03,   6.4367e+02,   -8.0742e-02  ],# bin 78  chi2/NDF: 0.4991
[ 1.9875,   3.0483e+03,   7.4381e+02,   -2.9454e-01  ],# bin 79  chi2/NDF: 0.3454
[ 2.0125,   4.4329e+03,   7.0688e+02,   -1.8264e-01  ],# bin 80  chi2/NDF: 1.2911
[ 2.0375,   2.1567e+03,   7.2309e+02,   -1.3736e-01  ],# bin 81  chi2/NDF: 1.393
[ 2.0625,   2.1767e+03,   7.1760e+02,   -1.1334e-01  ],# bin 82  chi2/NDF: 0.97489
[ 2.0875,   2.0517e+03,   6.9761e+02,   -1.1064e-01  ],# bin 83  chi2/NDF: 2.1512
[ 2.1125,   4.9195e+03,   6.7800e+02,   -2.2574e-01  ],# bin 84  chi2/NDF: 1.3009
[ 2.1375,   3.7564e+03,   6.9077e+02,   -2.5809e-01  ],# bin 85  chi2/NDF: 1.0785
[ 2.1625,   2.9585e+03,   6.3652e+02,   -1.5227e-01  ],# bin 86  chi2/NDF: 1.9781
[ 2.1875,   1.3760e+03,   6.3513e+02,   -2.0556e-01  ],# bin 87  chi2/NDF: 1.6628
[ 2.2125,   3.1347e+03,   5.8552e+02,   -1.2744e-01  ],# bin 88  chi2/NDF: 0.40297
[ 2.2375,   3.3142e+03,   5.8536e+02,   -2.1149e-01  ],# bin 89  chi2/NDF: 1.6558
[ 2.2625,   3.9845e+03,   5.0625e+02,   -1.6888e-01  ],# bin 90  chi2/NDF: 0.65336
[ 2.2875,   3.8577e+03,   5.3232e+02,   -2.1653e-01  ],# bin 91  chi2/NDF: 0.55551
[ 2.3125,   3.2379e+03,   2.8362e+02,   -3.1432e-01  ],# bin 92  chi2/NDF: 2.6294
[ 2.3375,   4.6006e+03,   7.8905e+02,   3.2207e-02  ],# bin 93  chi2/NDF: 0.74019
[ 2.3625,   2.7990e+03,   8.7415e+02,   -1.3614e-01  ],# bin 94  chi2/NDF: 0.71601
[ 2.3875,   6.2764e+03,   7.9647e+02,   -1.0796e-01  ],# bin 95  chi2/NDF: 1.4005
[ 2.4125,   6.8851e+02,   9.2465e+02,   -3.6336e-01  ],# bin 96  chi2/NDF: 0.67218
[ 2.4375,   -6.7396e+03,   1.0855e+03,   -5.0507e-01  ],# bin 97  chi2/NDF: 0.12427
[ 2.4625,   -1.0651e+04,   1.2899e+03,   -1.0088e+00  ],# bin 98  chi2/NDF: 0.89608
[ 2.4875,   -9.4282e+03,   1.2143e+03,   -1.0057e+00  ],# bin 99  chi2/NDF: 1.4332
] # end CaloSwCalHitsEfront_offset_v9_ele


CaloSwCalHitsEfront_offset_v9leakdata_gam_conv = [
[ 0.0125,   5.3918e+02,   4.8323e-01,   1.5831e-04  ],# bin 0  chi2/NDF: 1.8771, copied from bin 2
[ 0.0375,   5.3918e+02,   4.8323e-01,   1.5831e-04  ],# bin 1  chi2/NDF: 4.2803, copied from bin 2
[ 0.0625,   5.3918e+02,   4.8323e-01,   1.5831e-04  ],# bin 2  chi2/NDF: 2.9964
[ 0.0875,   5.5950e+02,   3.5124e-01,   3.1704e-04  ],# bin 3  chi2/NDF: 2.8223
[ 0.1125,   5.4815e+02,   6.0209e-01,   1.3763e-04  ],# bin 4  chi2/NDF: 2.5699
[ 0.1375,   5.4613e+02,   4.2329e-01,   2.8001e-04  ],# bin 5  chi2/NDF: 2.3098
[ 0.1625,   5.3785e+02,   6.2533e-01,   5.3118e-05  ],# bin 6  chi2/NDF: 3.4828
[ 0.1875,   5.4923e+02,   4.2044e-01,   2.2703e-04  ],# bin 7  chi2/NDF: 3.3542
[ 0.2125,   5.3336e+02,   5.7483e-01,   6.3456e-05  ],# bin 8  chi2/NDF: 2.9153
[ 0.2375,   5.3236e+02,   7.0695e-01,   -4.8836e-05  ],# bin 9  chi2/NDF: 3.4266
[ 0.2625,   5.5475e+02,   4.9206e-01,   2.8874e-04  ],# bin 10  chi2/NDF: 1.6792
[ 0.2875,   5.5371e+02,   6.3447e-01,   1.8475e-04  ],# bin 11  chi2/NDF: 1.1805
[ 0.3125,   5.7785e+02,   4.7362e-01,   3.2834e-04  ],# bin 12  chi2/NDF: 4.703
[ 0.3375,   5.6645e+02,   5.1345e-01,   3.4994e-04  ],# bin 13  chi2/NDF: 3.384
[ 0.3625,   5.8754e+02,   5.4945e-01,   3.8572e-04  ],# bin 14  chi2/NDF: 1.1057
[ 0.3875,   5.7705e+02,   6.8937e-01,   2.4964e-04  ],# bin 15  chi2/NDF: 2.2241
[ 0.4125,   5.6741e+02,   8.8164e-01,   -9.7208e-05  ],# bin 16  chi2/NDF: 4.0586
[ 0.4375,   5.9676e+02,   9.9195e-01,   -1.6954e-04  ],# bin 17  chi2/NDF: 2.1834
[ 0.4625,   6.0803e+02,   9.0017e-01,   5.6309e-05  ],# bin 18  chi2/NDF: 2.0178
[ 0.4875,   6.0924e+02,   8.6595e-01,   1.9935e-04  ],# bin 19  chi2/NDF: 2.6552
[ 0.5125,   6.2057e+02,   1.1151e+00,   -8.3155e-05  ],# bin 20  chi2/NDF: 2.2521
[ 0.5375,   6.2334e+02,   1.2721e+00,   -6.1034e-05  ],# bin 21  chi2/NDF: 2.5716
[ 0.5625,   6.5724e+02,   1.3576e+00,   -9.1201e-06  ],# bin 22  chi2/NDF: 6.2553
[ 0.5875,   6.9553e+02,   1.0250e+00,   2.2545e-04  ],# bin 23  chi2/NDF: 5.2844
[ 0.6125,   6.8985e+02,   1.2430e+00,   1.9069e-04  ],# bin 24  chi2/NDF: 3.737
[ 0.6375,   7.3817e+02,   1.3473e+00,   1.6494e-04  ],# bin 25  chi2/NDF: 4.9913
[ 0.6625,   9.0487e+02,   9.6602e-01,   6.4649e-04  ],# bin 26  chi2/NDF: 3.2438
[ 0.6875,   9.6304e+02,   1.0721e+00,   5.4585e-04  ],# bin 27  chi2/NDF: 1.8986
[ 0.7125,   1.0092e+03,   1.2599e+00,   9.1340e-04  ],# bin 28  chi2/NDF: 3.0223
[ 0.7375,   1.1228e+03,   1.3423e+00,   6.5295e-04  ],# bin 29  chi2/NDF: 4.7124
[ 0.7625,   1.1228e+03,   1.3423e+00,   6.5295e-04  ],# bin 30  chi2/NDF: 1.7618, copied from bin 29
[ 0.7875,   1.1228e+03,   1.3423e+00,   6.5295e-04  ],# bin 31  chi2/NDF: 2.4449, copied from bin 29
[ 0.8125,   1.1440e+03,   5.5577e-01,   1.7503e-03  ],# bin 32  chi2/NDF: 2.8174, copied from bin 34
[ 0.8375,   1.1440e+03,   5.5577e-01,   1.7503e-03  ],# bin 33  chi2/NDF: 1.5927, copied from bin 34
[ 0.8625,   1.1440e+03,   5.5577e-01,   1.7503e-03  ],# bin 34  chi2/NDF: 2.1355
[ 0.8875,   1.1700e+03,   -2.0230e-01,   2.9051e-03  ],# bin 35  chi2/NDF: 3.0669
[ 0.9125,   1.1733e+03,   2.6937e-01,   3.0679e-03  ],# bin 36  chi2/NDF: 3.8502
[ 0.9375,   1.2375e+03,   3.4431e-01,   3.2430e-03  ],# bin 37  chi2/NDF: 4.2204
[ 0.9625,   1.3746e+03,   -1.1919e-01,   3.5204e-03  ],# bin 38  chi2/NDF: 3.4993
[ 0.9875,   1.4297e+03,   -9.0819e-02,   4.0798e-03  ],# bin 39  chi2/NDF: 2.2921
[ 1.0125,   1.4877e+03,   1.5739e-01,   4.0325e-03  ],# bin 40  chi2/NDF: 2.9514
[ 1.0375,   1.4991e+03,   1.1506e+00,   3.1789e-03  ],# bin 41  chi2/NDF: 2.4291
[ 1.0625,   1.5841e+03,   1.2700e+00,   3.7596e-03  ],# bin 42  chi2/NDF: 3.2132
[ 1.0875,   1.7398e+03,   1.2938e+00,   3.9896e-03  ],# bin 43  chi2/NDF: 1.5188
[ 1.1125,   1.7562e+03,   2.4766e+00,   2.8465e-03  ],# bin 44  chi2/NDF: 2.1126
[ 1.1375,   1.9606e+03,   2.8862e+00,   3.6736e-03  ],# bin 45  chi2/NDF: 2.2748
[ 1.1625,   2.0141e+03,   2.9518e+00,   3.5619e-03  ],# bin 46  chi2/NDF: 3.8397
[ 1.1875,   2.1192e+03,   3.5761e+00,   3.6662e-03  ],# bin 47  chi2/NDF: 5.272
[ 1.2125,   1.9520e+03,   5.2546e+00,   2.4139e-03  ],# bin 48  chi2/NDF: 3.724
[ 1.2375,   2.5148e+03,   3.4341e+00,   5.1334e-03  ],# bin 49  chi2/NDF: 2.5146
[ 1.2625,   2.5377e+03,   3.6503e+00,   6.0133e-03  ],# bin 50  chi2/NDF: 2.1923
[ 1.2875,   2.5791e+03,   5.5420e+00,   4.2411e-03  ],# bin 51  chi2/NDF: 1.0705
[ 1.3125,   2.6053e+03,   7.6358e+00,   5.1841e-03  ],# bin 52  chi2/NDF: 0.98199
[ 1.3375,   2.8387e+03,   7.7273e+00,   5.1135e-03  ],# bin 53  chi2/NDF: 3.4263
[ 1.3625,   2.9300e+03,   9.3621e+00,   5.4355e-03  ],# bin 54  chi2/NDF: 3.3738
[ 1.3875,   2.4793e+03,   1.4049e+01,   7.6892e-03  ],# bin 55  chi2/NDF: 25.537
[ 1.4125,   3.4723e+03,   1.1430e+01,   1.7219e-02  ],# bin 56  chi2/NDF: 2.2504
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -1.5735e+04,   4.8673e+02,   1.1568e+04  ],# bin 60 crack  chi2/NDF: 8.3532
[ 1.5375,   3.0399e+03,   3.0942e+01,   -8.1689e+01  ],# bin 61 crack  chi2/NDF: 14.286
[ 1.5625,   2.7088e+03,   6.6047e+00,   -2.1268e+01  ],# bin 62  chi2/NDF: 6.2105
[ 1.5875,   2.4696e+03,   5.2159e+00,   -3.1120e+00  ],# bin 63  chi2/NDF: 2.1417
[ 1.6125,   3.1164e+03,   1.0654e+01,   -1.4928e+02  ],# bin 64  chi2/NDF: 4.4573
[ 1.6375,   1.2576e+03,   2.4542e+00,   2.4049e+02  ],# bin 65  chi2/NDF: 2.8064
[ 1.6625,   1.5204e+03,   3.2295e+00,   2.5235e+02  ],# bin 66  chi2/NDF: 9.0536
[ 1.6875,   1.8556e+03,   1.6581e+01,   -5.9754e+01  ],# bin 67  chi2/NDF: 7.8929
[ 1.7125,   1.7733e+03,   3.3226e+00,   2.7596e+01  ],# bin 68  chi2/NDF: 4.3088
[ 1.7375,   1.0401e+03,   2.3262e+00,   4.7236e+01  ],# bin 69  chi2/NDF: 3.1344
[ 1.7625,   1.1813e+03,   2.6427e+00,   1.2257e+01  ],# bin 70  chi2/NDF: 2.1066
[ 1.7875,   7.3320e+02,   1.8489e+00,   5.0102e+01  ],# bin 71  chi2/NDF: 3.3825
[ 1.8125,   4.7194e+02,   8.8895e+02,   3.7901e-01  ],# bin 72  chi2/NDF: 3.4301
[ 1.8375,   6.9242e+03,   8.4038e+02,   4.0963e-01  ],# bin 73  chi2/NDF: 24.041
[ 1.8625,   2.2893e+03,   7.9790e+02,   5.6307e-01  ],# bin 74  chi2/NDF: 8.8435
[ 1.8875,   3.2886e+03,   8.3436e+02,   9.2533e-02  ],# bin 75  chi2/NDF: 6.5943
[ 1.9125,   -2.3864e+03,   7.9490e+02,   3.4542e-01  ],# bin 76  chi2/NDF: 4.0362
[ 1.9375,   1.1471e+03,   7.7045e+02,   4.4741e-01  ],# bin 77  chi2/NDF: 6.7825
[ 1.9625,   -8.7869e+02,   7.7288e+02,   1.1028e-01  ],# bin 78  chi2/NDF: 11.371
[ 1.9875,   2.4219e+03,   6.6063e+02,   3.3708e-01  ],# bin 79  chi2/NDF: 7.727
[ 2.0125,   7.7450e+03,   6.2767e+02,   3.8937e-01  ],# bin 80  chi2/NDF: 1.1128
[ 2.0375,   3.7989e+03,   7.6016e+02,   2.0880e-01  ],# bin 81  chi2/NDF: 4.2013
[ 2.0625,   1.0795e+04,   6.9161e+02,   3.8479e-01  ],# bin 82  chi2/NDF: 11.581
[ 2.0875,   1.4079e+04,   7.0208e+02,   1.6781e-01  ],# bin 83  chi2/NDF: 16.678
[ 2.1125,   1.6329e+04,   7.3783e+02,   6.4632e-02  ],# bin 84  chi2/NDF: 16.032
[ 2.1375,   1.8340e+04,   7.1436e+02,   5.1319e-02  ],# bin 85  chi2/NDF: 5.9616
[ 2.1625,   9.4894e+03,   7.0014e+02,   -1.2832e-01  ],# bin 86  chi2/NDF: 4.4263
[ 2.1875,   1.0477e+04,   5.9635e+02,   2.0324e-01  ],# bin 87  chi2/NDF: 4.1315
[ 2.2125,   1.1449e+04,   5.8900e+02,   3.4444e-02  ],# bin 88  chi2/NDF: 29.739
[ 2.2375,   1.7028e+04,   5.3374e+02,   1.3276e-01  ],# bin 89  chi2/NDF: 7.7193
[ 2.2625,   3.7243e+03,   5.1282e+02,   -5.2404e-02  ],# bin 90  chi2/NDF: 9.9285
[ 2.2875,   4.6717e+03,   5.6457e+02,   -2.1622e-01  ],# bin 91  chi2/NDF: 36.784
[ 2.3125,   6.2933e+03,   4.8354e+02,   -1.8385e-01  ],# bin 92  chi2/NDF: 20.564
[ 2.3375,   1.4362e+04,   7.5617e+02,   1.0078e+00  ],# bin 93  chi2/NDF: 8.2754
[ 2.3625,   1.8035e+03,   9.1704e+02,   1.0287e+00  ],# bin 94  chi2/NDF: 2.9045
[ 2.3875,   9.2948e+03,   8.5023e+02,   6.4235e-01  ],# bin 95  chi2/NDF: 7.3175
[ 2.4125,   1.1554e+04,   8.8250e+02,   9.3133e-01  ],# bin 96  chi2/NDF: 10.693
[ 2.4375,   -3.5112e+03,   1.2780e+03,   2.7324e-01  ],# bin 97  chi2/NDF: 10.327
[ 2.4625,   4.1014e+03,   1.5701e+03,   2.6625e-01  ],# bin 98  chi2/NDF: 11.574
[ 2.4875,   -1.0816e+04,   1.5789e+03,   3.0379e-01  ],# bin 99  chi2/NDF: 2.3791
] # end CaloSwCalHitsEfront_offset_v9_gam_conv


CaloSwCalHitsEfront_offset_v9leakdata_gam_unconv = [
[ 0.0125,   -2.2209e+01,   1.3272e+00,   -7.4234e-04  ],# bin 0  chi2/NDF: 4.647, copied from bin 2
[ 0.0375,   -2.2209e+01,   1.3272e+00,   -7.4234e-04  ],# bin 1  chi2/NDF: 5.868, copied from bin 2
[ 0.0625,   -2.2209e+01,   1.3272e+00,   -7.4234e-04  ],# bin 2  chi2/NDF: 140.12
[ 0.0875,   -8.1216e+00,   1.2470e+00,   -6.5589e-04  ],# bin 3  chi2/NDF: 102.13
[ 0.1125,   -2.1796e+01,   1.3344e+00,   -7.2653e-04  ],# bin 4  chi2/NDF: 143.48
[ 0.1375,   1.3435e+01,   1.0610e+00,   -4.5060e-04  ],# bin 5  chi2/NDF: 26.576
[ 0.1625,   1.8990e+01,   1.0368e+00,   -4.2624e-04  ],# bin 6  chi2/NDF: 29.979
[ 0.1875,   1.4727e+01,   1.0411e+00,   -4.2705e-04  ],# bin 7  chi2/NDF: 27.451
[ 0.2125,   2.0247e+01,   1.0083e+00,   -4.0683e-04  ],# bin 8  chi2/NDF: 12.562
[ 0.2375,   2.4969e+01,   1.0140e+00,   -4.1477e-04  ],# bin 9  chi2/NDF: 13.977
[ 0.2625,   1.9559e+01,   1.0585e+00,   -4.3214e-04  ],# bin 10  chi2/NDF: 20.987
[ 0.2875,   1.8724e+01,   1.1016e+00,   -4.3949e-04  ],# bin 11  chi2/NDF: 20.67
[ 0.3125,   6.6324e+00,   1.2384e+00,   -5.6390e-04  ],# bin 12  chi2/NDF: 51.178
[ 0.3375,   -2.8957e+01,   1.5819e+00,   -9.1218e-04  ],# bin 13  chi2/NDF: 142.31
[ 0.3625,   1.1014e+01,   1.2755e+00,   -5.9197e-04  ],# bin 14  chi2/NDF: 50.787
[ 0.3875,   7.7173e+00,   1.3606e+00,   -6.4672e-04  ],# bin 15  chi2/NDF: 73.434
[ 0.4125,   1.7417e+01,   1.3563e+00,   -6.1079e-04  ],# bin 16  chi2/NDF: 30.732
[ 0.4375,   2.5490e+01,   1.3135e+00,   -5.4943e-04  ],# bin 17  chi2/NDF: 18.051
[ 0.4625,   -6.1818e+00,   1.6369e+00,   -8.0176e-04  ],# bin 18  chi2/NDF: 118.07
[ 0.4875,   -3.5871e+01,   1.9721e+00,   -1.1497e-03  ],# bin 19  chi2/NDF: 196.1
[ 0.5125,   -1.4132e+01,   1.9563e+00,   -9.6647e-04  ],# bin 20  chi2/NDF: 128.29
[ 0.5375,   -2.9271e+01,   2.1437e+00,   -1.1397e-03  ],# bin 21  chi2/NDF: 146.18
[ 0.5625,   -3.8168e+01,   2.2889e+00,   -1.1484e-03  ],# bin 22  chi2/NDF: 153.2
[ 0.5875,   -6.4476e+01,   2.5337e+00,   -1.3691e-03  ],# bin 23  chi2/NDF: 117.72
[ 0.6125,   -3.0030e+01,   2.2560e+00,   -1.1404e-03  ],# bin 24  chi2/NDF: 88.257
[ 0.6375,   -4.8176e+01,   2.6050e+00,   -1.3943e-03  ],# bin 25  chi2/NDF: 76.64
[ 0.6625,   -3.3270e+01,   2.7560e+00,   -1.3793e-03  ],# bin 26  chi2/NDF: 43.624
[ 0.6875,   4.1706e+00,   2.6102e+00,   -1.1796e-03  ],# bin 27  chi2/NDF: 59.538
[ 0.7125,   2.5777e-01,   2.7439e+00,   -1.0797e-03  ],# bin 28  chi2/NDF: 39.654
[ 0.7375,   -2.9566e+00,   3.0881e+00,   -1.4428e-03  ],# bin 29  chi2/NDF: 37.126
[ 0.7625,   -2.9566e+00,   3.0881e+00,   -1.4428e-03  ],# bin 30  chi2/NDF: 20.148, copied from bin 29
[ 0.7875,   -2.9566e+00,   3.0881e+00,   -1.4428e-03  ],# bin 31  chi2/NDF: 9.3568, copied from bin 29
[ 0.8125,   9.3879e+01,   2.9391e+00,   -1.0121e-03  ],# bin 32  chi2/NDF: 3.4484, copied from bin 34
[ 0.8375,   9.3879e+01,   2.9391e+00,   -1.0121e-03  ],# bin 33  chi2/NDF: 5.5555, copied from bin 34
[ 0.8625,   9.3879e+01,   2.9391e+00,   -1.0121e-03  ],# bin 34  chi2/NDF: 8.3466
[ 0.8875,   6.2731e+01,   3.6058e+00,   -1.7897e-03  ],# bin 35  chi2/NDF: 12.573
[ 0.9125,   8.6198e+01,   3.5436e+00,   -1.2556e-03  ],# bin 36  chi2/NDF: 9.0735
[ 0.9375,   7.4888e+01,   4.0493e+00,   -1.7642e-03  ],# bin 37  chi2/NDF: 13.729
[ 0.9625,   9.4179e+01,   4.0832e+00,   -1.5647e-03  ],# bin 38  chi2/NDF: 7.934
[ 0.9875,   1.0439e+02,   4.3999e+00,   -1.5545e-03  ],# bin 39  chi2/NDF: 11.881
[ 1.0125,   9.5508e+01,   4.8050e+00,   -1.9663e-03  ],# bin 40  chi2/NDF: 14.219
[ 1.0375,   7.9040e+01,   5.5804e+00,   -2.1584e-03  ],# bin 41  chi2/NDF: 16.177
[ 1.0625,   8.5997e+01,   5.6101e+00,   -2.2061e-03  ],# bin 42  chi2/NDF: 18.017
[ 1.0875,   9.7419e+01,   5.9861e+00,   -2.3930e-03  ],# bin 43  chi2/NDF: 9.0194
[ 1.1125,   7.4639e+01,   6.9815e+00,   -2.9981e-03  ],# bin 44  chi2/NDF: 15.515
[ 1.1375,   1.2229e+02,   6.3984e+00,   -2.1038e-03  ],# bin 45  chi2/NDF: 11.662
[ 1.1625,   1.7317e+02,   7.2270e+00,   -2.6934e-03  ],# bin 46  chi2/NDF: 8.803
[ 1.1875,   1.9094e+02,   8.1320e+00,   -2.8870e-03  ],# bin 47  chi2/NDF: 16.091
[ 1.2125,   1.2410e+02,   9.2028e+00,   -3.7618e-03  ],# bin 48  chi2/NDF: 25.846
[ 1.2375,   2.7301e+02,   8.8192e+00,   -4.3496e-03  ],# bin 49  chi2/NDF: 13.521
[ 1.2625,   2.4610e+02,   9.0911e+00,   -2.7376e-03  ],# bin 50  chi2/NDF: 10.338
[ 1.2875,   2.3091e+02,   1.1024e+01,   -4.7915e-03  ],# bin 51  chi2/NDF: 13.584
[ 1.3125,   2.3677e+02,   1.1812e+01,   -4.6197e-03  ],# bin 52  chi2/NDF: 10.551
[ 1.3375,   2.5514e+02,   1.2946e+01,   -5.5382e-03  ],# bin 53  chi2/NDF: 52990
[ 1.3625,   2.9884e+02,   1.3743e+01,   -4.9290e-03  ],# bin 54  chi2/NDF: 9.295
[ 1.3875,   3.5038e+02,   1.5158e+01,   -3.1370e-03  ],# bin 55  chi2/NDF: 14.935
[ 1.4125,   2.6479e+02,   2.2722e+01,   -2.0399e-02  ],# bin 56  chi2/NDF: 5.3711
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -1.1458e+04,   1.0128e+02,   9.1021e+03  ],# bin 60 crack  chi2/NDF: 77.195
[ 1.5375,   4.7191e-01,   1.0012e+01,   -2.0246e+01  ],# bin 61 crack  chi2/NDF: 5.3994
[ 1.5625,   1.6510e+02,   7.4553e+00,   -4.5356e+01  ],# bin 62  chi2/NDF: 11.867
[ 1.5875,   5.0870e+01,   6.6183e+00,   -3.3368e+01  ],# bin 63  chi2/NDF: 12.982
[ 1.6125,   7.7691e+01,   6.0400e+00,   -2.9135e+01  ],# bin 64  chi2/NDF: 9.7583
[ 1.6375,   -1.9128e+02,   5.1795e+00,   3.4422e+00  ],# bin 65  chi2/NDF: 5.4154
[ 1.6625,   -1.2324e+02,   4.8732e+00,   2.2901e+00  ],# bin 66  chi2/NDF: 4.9319
[ 1.6875,   -2.8247e+02,   4.3759e+00,   1.7820e+01  ],# bin 67  chi2/NDF: 6.9673
[ 1.7125,   -7.2362e+00,   5.1493e+00,   -1.6525e+01  ],# bin 68  chi2/NDF: 11.771
[ 1.7375,   1.3579e+02,   5.3600e+00,   -3.1486e+01  ],# bin 69  chi2/NDF: 17.502
[ 1.7625,   -1.8814e+01,   4.0544e+00,   -5.8722e-01  ],# bin 70  chi2/NDF: 15.536
[ 1.7875,   6.4097e+01,   4.5987e+00,   -1.7251e+01  ],# bin 71  chi2/NDF: 14.552
[ 1.8125,   2.1174e+03,   6.7363e+02,   5.3767e-01  ],# bin 72  chi2/NDF: 5.1904
[ 1.8375,   3.1324e+03,   4.8396e+02,   3.2267e-01  ],# bin 73  chi2/NDF: 6.4435
[ 1.8625,   4.0088e+03,   3.3553e+02,   4.2009e-01  ],# bin 74  chi2/NDF: 3.0001
[ 1.8875,   2.7025e+03,   3.5130e+02,   2.3030e-01  ],# bin 75  chi2/NDF: 1.5874
[ 1.9125,   4.2864e+03,   3.1889e+02,   1.0309e-01  ],# bin 76  chi2/NDF: 3.0808
[ 1.9375,   3.1970e+03,   2.6888e+02,   3.5787e-01  ],# bin 77  chi2/NDF: 3.1375
[ 1.9625,   2.5833e+03,   2.5836e+02,   2.8464e-01  ],# bin 78  chi2/NDF: 3.6866
[ 1.9875,   1.3008e+03,   2.8973e+02,   1.7245e-01  ],# bin 79  chi2/NDF: 1.7457
[ 2.0125,   4.8138e+03,   2.5029e+02,   2.1578e-01  ],# bin 80  chi2/NDF: 3.4
[ 2.0375,   1.4896e+03,   2.6065e+02,   1.6225e-01  ],# bin 81  chi2/NDF: 2.5715
[ 2.0625,   4.1111e+03,   2.2794e+02,   1.7670e-01  ],# bin 82  chi2/NDF: 8.9662
[ 2.0875,   2.7136e+03,   2.2122e+02,   1.0579e-01  ],# bin 83  chi2/NDF: 8.5999
[ 2.1125,   5.6830e+03,   2.0910e+02,   1.1179e-01  ],# bin 84  chi2/NDF: 8.3834
[ 2.1375,   2.6772e+03,   2.3255e+02,   -3.8367e-03  ],# bin 85  chi2/NDF: 3.1755
[ 2.1625,   2.4141e+03,   1.9681e+02,   2.1708e-02  ],# bin 86  chi2/NDF: 3.1897
[ 2.1875,   1.7181e+03,   1.7592e+02,   4.1814e-02  ],# bin 87  chi2/NDF: 9.7192
[ 2.2125,   2.3925e+03,   1.4245e+02,   9.1278e-02  ],# bin 88  chi2/NDF: 13.881
[ 2.2375,   4.2364e+03,   1.2518e+02,   1.0456e-01  ],# bin 89  chi2/NDF: 14.703
[ 2.2625,   4.0112e+03,   1.2316e+02,   3.0842e-02  ],# bin 90  chi2/NDF: 2.9437
[ 2.2875,   1.0982e+03,   1.2422e+02,   4.1750e-02  ],# bin 91  chi2/NDF: 9.0276
[ 2.3125,   5.4943e+03,   4.2906e+01,   6.9593e-01  ],# bin 92  chi2/NDF: 21.106
[ 2.3375,   8.2388e+03,   7.8059e+01,   4.0499e+00  ],# bin 93  chi2/NDF: 19.667
[ 2.3625,   4.6771e+03,   2.9088e+02,   6.9851e-02  ],# bin 94  chi2/NDF: 8.7923
[ 2.3875,   1.2954e+03,   3.0681e+02,   -1.0905e-01  ],# bin 95  chi2/NDF: 3.1657
[ 2.4125,   5.6825e+03,   2.4701e+02,   3.7499e-02  ],# bin 96  chi2/NDF: 8.3859
[ 2.4375,   5.2425e+03,   2.0136e+02,   1.1251e-01  ],# bin 97  chi2/NDF: 2.653
[ 2.4625,   5.2956e+03,   2.1254e+02,   6.6501e-02  ],# bin 98  chi2/NDF: 6.4304
[ 2.4875,   4.1128e+03,   2.5032e+02,   1.5302e-03  ],# bin 99  chi2/NDF: 27.812
] # end CaloSwCalHitsEfront_offset_v9_gam_unconv


CaloSwCalHitsLongLeakage_v9leakdata_ele = [
[ 0.0125,   6.2084e-02,   2.6887e-06,   -1.8909e-01  ],# bin 0  chi2/NDF: 1.6775, copied from bin 2
[ 0.0375,   6.2084e-02,   2.6887e-06,   -1.8909e-01  ],# bin 1  chi2/NDF: 5.6059, copied from bin 2
[ 0.0625,   6.2084e-02,   2.6887e-06,   -1.8909e-01  ],# bin 2  chi2/NDF: 6.2104
[ 0.0875,   6.7820e-02,   2.4929e-06,   -2.8363e-01  ],# bin 3  chi2/NDF: 8.8131
[ 0.1125,   6.8736e-02,   2.4029e-06,   -3.1640e-01  ],# bin 4  chi2/NDF: 12.972
[ 0.1375,   6.7668e-02,   2.3045e-06,   -3.1648e-01  ],# bin 5  chi2/NDF: 7.8055
[ 0.1625,   6.5823e-02,   2.2027e-06,   -3.0378e-01  ],# bin 6  chi2/NDF: 7.5956
[ 0.1875,   6.4736e-02,   2.1211e-06,   -3.0374e-01  ],# bin 7  chi2/NDF: 5.7738
[ 0.2125,   6.2932e-02,   1.9535e-06,   -2.9418e-01  ],# bin 8  chi2/NDF: 5.4167
[ 0.2375,   6.2949e-02,   1.8017e-06,   -3.0471e-01  ],# bin 9  chi2/NDF: 8.1157
[ 0.2625,   5.9141e-02,   1.6635e-06,   -2.7431e-01  ],# bin 10  chi2/NDF: 7.7947
[ 0.2875,   5.6884e-02,   1.4966e-06,   -2.6790e-01  ],# bin 11  chi2/NDF: 7.0677
[ 0.3125,   5.5322e-02,   1.3015e-06,   -2.6134e-01  ],# bin 12  chi2/NDF: 6.5123
[ 0.3375,   5.1954e-02,   1.1804e-06,   -2.4471e-01  ],# bin 13  chi2/NDF: 5.3774
[ 0.3625,   4.6163e-02,   1.0437e-06,   -2.0424e-01  ],# bin 14  chi2/NDF: 5.2575
[ 0.3875,   4.4591e-02,   8.9169e-07,   -1.9926e-01  ],# bin 15  chi2/NDF: 5.1197
[ 0.4125,   3.9051e-02,   7.5761e-07,   -1.5824e-01  ],# bin 16  chi2/NDF: 5.1273
[ 0.4375,   3.9805e-02,   6.1271e-07,   -1.8010e-01  ],# bin 17  chi2/NDF: 5.1505
[ 0.4625,   3.5715e-02,   5.0409e-07,   -1.5243e-01  ],# bin 18  chi2/NDF: 11.831
[ 0.4875,   3.2008e-02,   4.1575e-07,   -1.2787e-01  ],# bin 19  chi2/NDF: 4.4608
[ 0.5125,   3.0988e-02,   3.1142e-07,   -1.2916e-01  ],# bin 20  chi2/NDF: 3.4467
[ 0.5375,   2.6322e-02,   3.0321e-07,   -9.5026e-02  ],# bin 21  chi2/NDF: 4.7118
[ 0.5625,   2.5417e-02,   2.6851e-07,   -9.5935e-02  ],# bin 22  chi2/NDF: 6.0747
[ 0.5875,   2.1527e-02,   2.4754e-07,   -6.2988e-02  ],# bin 23  chi2/NDF: 4.4868
[ 0.6125,   1.8114e-02,   2.2452e-07,   -2.9046e-02  ],# bin 24  chi2/NDF: 2.707
[ 0.6375,   1.6624e-02,   2.2026e-07,   -1.8226e-02  ],# bin 25  chi2/NDF: 4.758
[ 0.6625,   1.3854e-02,   2.1193e-07,   1.1949e-02  ],# bin 26  chi2/NDF: 4.5449
[ 0.6875,   1.0815e-02,   2.0704e-07,   4.1589e-02  ],# bin 27  chi2/NDF: 6.0736
[ 0.7125,   7.4049e-03,   1.9797e-07,   7.3816e-02  ],# bin 28  chi2/NDF: 6.8125
[ 0.7375,   3.1661e-03,   1.8703e-07,   1.2158e-01  ],# bin 29  chi2/NDF: 7.6874
[ 0.7625,   3.1661e-03,   1.8703e-07,   1.2158e-01  ],# bin 30  chi2/NDF: 9.9755, copied from bin 29
[ 0.7875,   3.1661e-03,   1.8703e-07,   1.2158e-01  ],# bin 31  chi2/NDF: 5.8787, copied from bin 29
[ 0.8125,   -3.4065e-02,   2.8590e-07,   6.9922e-01  ],# bin 32  chi2/NDF: 2.8415, copied from bin 34
[ 0.8375,   -3.4065e-02,   2.8590e-07,   6.9922e-01  ],# bin 33  chi2/NDF: 3.2845, copied from bin 34
[ 0.8625,   -3.4065e-02,   2.8590e-07,   6.9922e-01  ],# bin 34  chi2/NDF: 2.8411
[ 0.8875,   -3.8883e-02,   2.7214e-07,   7.6369e-01  ],# bin 35  chi2/NDF: 4.3458
[ 0.9125,   -4.9396e-02,   2.6497e-07,   8.8766e-01  ],# bin 36  chi2/NDF: 7.3108
[ 0.9375,   -6.1743e-02,   2.6651e-07,   1.0357e+00  ],# bin 37  chi2/NDF: 4.8781
[ 0.9625,   -6.8489e-02,   2.5746e-07,   1.1116e+00  ],# bin 38  chi2/NDF: 6.6377
[ 0.9875,   -7.7058e-02,   2.4538e-07,   1.2185e+00  ],# bin 39  chi2/NDF: 7.3026
[ 1.0125,   -8.2437e-02,   2.2383e-07,   1.2882e+00  ],# bin 40  chi2/NDF: 7.0037
[ 1.0375,   -8.9131e-02,   2.2085e-07,   1.3629e+00  ],# bin 41  chi2/NDF: 8.2481
[ 1.0625,   -9.9172e-02,   2.2666e-07,   1.4705e+00  ],# bin 42  chi2/NDF: 9.803
[ 1.0875,   -1.0230e-01,   2.1082e-07,   1.5164e+00  ],# bin 43  chi2/NDF: 6.72
[ 1.1125,   -9.7229e-02,   1.7703e-07,   1.4603e+00  ],# bin 44  chi2/NDF: 9.0497
[ 1.1375,   -1.0249e-01,   1.9793e-07,   1.5031e+00  ],# bin 45  chi2/NDF: 7.2088
[ 1.1625,   -1.0639e-01,   1.5058e-07,   1.5807e+00  ],# bin 46  chi2/NDF: 9.3214
[ 1.1875,   -1.2523e-01,   1.9744e-07,   1.7920e+00  ],# bin 47  chi2/NDF: 8.723
[ 1.2125,   -1.2743e-01,   1.3945e-07,   1.8548e+00  ],# bin 48  chi2/NDF: 8.9579
[ 1.2375,   -1.2796e-01,   1.3649e-07,   1.8744e+00  ],# bin 49  chi2/NDF: 9.268
[ 1.2625,   -1.4139e-01,   1.4095e-07,   2.0657e+00  ],# bin 50  chi2/NDF: 8.7624
[ 1.2875,   -1.6313e-01,   2.0155e-07,   2.3873e+00  ],# bin 51  chi2/NDF: 7.7595
[ 1.3125,   -1.7976e-01,   3.7519e-07,   2.6739e+00  ],# bin 52  chi2/NDF: 6.549
[ 1.3375,   -1.8394e-01,   7.8472e-07,   2.9895e+00  ],# bin 53  chi2/NDF: 8.2004
[ 1.3625,   -1.3463e-01,   2.1613e-06,   3.1084e+00  ],# bin 54  chi2/NDF: 9.0225
[ 1.3875,   1.0223e-01,   5.6950e-06,   2.4252e+00  ],# bin 55  chi2/NDF: 12.82
[ 1.4125,   3.7315e-01,   1.2512e-05,   4.1319e+00  ],# bin 56  chi2/NDF: 10.301
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -3.2813e-02,   6.2010e-09,   8.7759e-01  ],# bin 60 crack  chi2/NDF: 1.4449
[ 1.5375,   -3.1269e-03,   9.2589e-09,   2.7900e-01  ],# bin 61 crack  chi2/NDF: 8.8132
[ 1.5625,   1.1163e-02,   4.9774e-09,   4.0502e-02  ],# bin 62  chi2/NDF: 5.619
[ 1.5875,   9.8764e-03,   2.3616e-08,   5.5628e-02  ],# bin 63  chi2/NDF: 6.1668
[ 1.6125,   6.0783e-03,   2.4741e-08,   1.0029e-01  ],# bin 64  chi2/NDF: 6.1363
[ 1.6375,   2.7808e-03,   3.0564e-08,   1.4207e-01  ],# bin 65  chi2/NDF: 4.2793
[ 1.6625,   3.2176e-03,   2.1189e-08,   1.2627e-01  ],# bin 66  chi2/NDF: 4.0198
[ 1.6875,   2.3515e-03,   1.5840e-08,   1.2332e-01  ],# bin 67  chi2/NDF: 3.3293
[ 1.7125,   6.7126e-03,   6.7477e-09,   4.4215e-02  ],# bin 68  chi2/NDF: 3.601
[ 1.7375,   8.8737e-03,   1.4656e-08,   1.2196e-02  ],# bin 69  chi2/NDF: 1.9501
[ 1.7625,   8.3978e-03,   1.7019e-08,   1.3757e-02  ],# bin 70  chi2/NDF: 1.9788
[ 1.7875,   5.2967e-03,   2.5216e-08,   4.7549e-02  ],# bin 71  chi2/NDF: 1.4322
[ 1.8125,   5.3021e-03,   2.1393e-08,   3.3121e-02  ],# bin 72  chi2/NDF: 1.2023
[ 1.8375,   5.2628e-03,   3.9925e-08,   2.4890e-02  ],# bin 73  chi2/NDF: 2.9604
[ 1.8625,   9.1946e-04,   5.2337e-08,   7.8042e-02  ],# bin 74  chi2/NDF: 1.3812
[ 1.8875,   2.6100e-03,   4.9539e-08,   5.8360e-02  ],# bin 75  chi2/NDF: 1.3041
[ 1.9125,   1.9866e-03,   4.2947e-08,   6.4457e-02  ],# bin 76  chi2/NDF: 1.8521
[ 1.9375,   8.7006e-03,   3.5280e-08,   -2.6406e-02  ],# bin 77  chi2/NDF: 3.6906
[ 1.9625,   2.4001e-03,   3.3709e-08,   5.0645e-02  ],# bin 78  chi2/NDF: 1.338
[ 1.9875,   1.1603e-03,   2.8558e-08,   6.8809e-02  ],# bin 79  chi2/NDF: 1.2236
[ 2.0125,   -1.6128e-03,   1.8892e-08,   1.0156e-01  ],# bin 80  chi2/NDF: 0.85324
[ 2.0375,   -9.5360e-04,   1.7665e-08,   8.8498e-02  ],# bin 81  chi2/NDF: 1.2883
[ 2.0625,   -2.1425e-03,   1.3338e-08,   9.9194e-02  ],# bin 82  chi2/NDF: 1.4436
[ 2.0875,   1.7486e-03,   9.2288e-09,   4.3736e-02  ],# bin 83  chi2/NDF: 1.5593
[ 2.1125,   3.9098e-04,   1.3518e-08,   6.1278e-02  ],# bin 84  chi2/NDF: 1.1925
[ 2.1375,   9.6753e-04,   1.1458e-08,   5.0707e-02  ],# bin 85  chi2/NDF: 0.80073
[ 2.1625,   8.6650e-04,   8.8582e-09,   4.3782e-02  ],# bin 86  chi2/NDF: 1.1487
[ 2.1875,   -1.1492e-03,   7.8465e-09,   7.0324e-02  ],# bin 87  chi2/NDF: 1.2308
[ 2.2125,   3.0590e-03,   5.6444e-09,   1.2860e-02  ],# bin 88  chi2/NDF: 2.7021
[ 2.2375,   1.4975e-03,   8.3009e-09,   3.2568e-02  ],# bin 89  chi2/NDF: 1.6697
[ 2.2625,   -3.2658e-03,   8.6118e-09,   9.1960e-02  ],# bin 90  chi2/NDF: 1.1021
[ 2.2875,   -2.0735e-03,   6.5029e-09,   7.8474e-02  ],# bin 91  chi2/NDF: 0.95983
[ 2.3125,   -6.5565e-03,   3.5830e-09,   1.4392e-01  ],# bin 92  chi2/NDF: 1.3358
[ 2.3375,   -1.5398e-02,   8.6308e-09,   2.6215e-01  ],# bin 93  chi2/NDF: 1.5117
[ 2.3625,   -2.0828e-02,   9.4093e-09,   3.4011e-01  ],# bin 94  chi2/NDF: 1.7831
[ 2.3875,   -2.0369e-02,   1.0231e-08,   3.4476e-01  ],# bin 95  chi2/NDF: 1.2188
[ 2.4125,   -2.3500e-02,   7.7208e-09,   4.0838e-01  ],# bin 96  chi2/NDF: 1.298
[ 2.4375,   -2.7584e-02,   7.4816e-09,   4.9648e-01  ],# bin 97  chi2/NDF: 1.1181
[ 2.4625,   -4.5151e-02,   1.2577e-08,   8.1635e-01  ],# bin 98  chi2/NDF: 2.053
[ 2.4875,   -3.4082e-01,   3.6207e-07,   5.1496e+00  ],# bin 99  chi2/NDF: 2.4635
] # end CaloSwCalHitsLongLeakage_v9_ele


CaloSwCalHitsLongLeakage_v9leakdata_gam_conv = [
[ 0.0125,   7.8008e-02,   2.7204e-06,   -3.4901e-01  ],# bin 0  chi2/NDF: 1.6316, copied from bin 2
[ 0.0375,   7.8008e-02,   2.7204e-06,   -3.4901e-01  ],# bin 1  chi2/NDF: 1.5378, copied from bin 2
[ 0.0625,   7.8008e-02,   2.7204e-06,   -3.4901e-01  ],# bin 2  chi2/NDF: 2.1611
[ 0.0875,   7.8868e-02,   2.5712e-06,   -3.9619e-01  ],# bin 3  chi2/NDF: 2.9744
[ 0.1125,   8.4454e-02,   2.3523e-06,   -4.6376e-01  ],# bin 4  chi2/NDF: 4.8046
[ 0.1375,   7.6339e-02,   2.5115e-06,   -4.0918e-01  ],# bin 5  chi2/NDF: 5.2392
[ 0.1625,   7.7043e-02,   2.3027e-06,   -4.2176e-01  ],# bin 6  chi2/NDF: 2.3135
[ 0.1875,   7.0225e-02,   2.2680e-06,   -3.6126e-01  ],# bin 7  chi2/NDF: 1.8422
[ 0.2125,   7.3111e-02,   2.0495e-06,   -3.9842e-01  ],# bin 8  chi2/NDF: 2.4122
[ 0.2375,   7.4864e-02,   1.8324e-06,   -4.2444e-01  ],# bin 9  chi2/NDF: 3.5285
[ 0.2625,   7.0847e-02,   1.7249e-06,   -4.0079e-01  ],# bin 10  chi2/NDF: 2.9203
[ 0.2875,   6.4690e-02,   1.5892e-06,   -3.5284e-01  ],# bin 11  chi2/NDF: 2.4894
[ 0.3125,   6.3867e-02,   1.3828e-06,   -3.5325e-01  ],# bin 12  chi2/NDF: 2.4844
[ 0.3375,   5.9634e-02,   1.2288e-06,   -3.2851e-01  ],# bin 13  chi2/NDF: 2.5233
[ 0.3625,   5.9768e-02,   1.0337e-06,   -3.4329e-01  ],# bin 14  chi2/NDF: 1.9776
[ 0.3875,   5.3691e-02,   9.1899e-07,   -3.0020e-01  ],# bin 15  chi2/NDF: 1.6633
[ 0.4125,   5.0740e-02,   7.4053e-07,   -2.7686e-01  ],# bin 16  chi2/NDF: 3.6627
[ 0.4375,   4.5629e-02,   6.3485e-07,   -2.4407e-01  ],# bin 17  chi2/NDF: 1.6149
[ 0.4625,   4.4455e-02,   5.2658e-07,   -2.4980e-01  ],# bin 18  chi2/NDF: 1.5137
[ 0.4875,   4.0472e-02,   4.2435e-07,   -2.2175e-01  ],# bin 19  chi2/NDF: 1.0387
[ 0.5125,   3.8204e-02,   3.3241e-07,   -2.1390e-01  ],# bin 20  chi2/NDF: 1.6648
[ 0.5375,   3.1843e-02,   3.0594e-07,   -1.5215e-01  ],# bin 21  chi2/NDF: 1.8188
[ 0.5625,   3.1849e-02,   2.6891e-07,   -1.6448e-01  ],# bin 22  chi2/NDF: 1.554
[ 0.5875,   2.5263e-02,   2.5680e-07,   -1.0530e-01  ],# bin 23  chi2/NDF: 1.5274
[ 0.6125,   2.6867e-02,   2.2180e-07,   -1.2993e-01  ],# bin 24  chi2/NDF: 1.8366
[ 0.6375,   2.2598e-02,   2.1735e-07,   -8.5643e-02  ],# bin 25  chi2/NDF: 2.0551
[ 0.6625,   2.0269e-02,   2.0763e-07,   -6.0085e-02  ],# bin 26  chi2/NDF: 2.4842
[ 0.6875,   1.7162e-02,   1.9143e-07,   -2.4156e-02  ],# bin 27  chi2/NDF: 1.8844
[ 0.7125,   1.1896e-02,   2.0831e-07,   2.1005e-02  ],# bin 28  chi2/NDF: 2.0055
[ 0.7375,   9.6063e-03,   2.0079e-07,   4.5077e-02  ],# bin 29  chi2/NDF: 2.0747
[ 0.7625,   9.6063e-03,   2.0079e-07,   4.5077e-02  ],# bin 30  chi2/NDF: 2.4557, copied from bin 29
[ 0.7875,   9.6063e-03,   2.0079e-07,   4.5077e-02  ],# bin 31  chi2/NDF: 4.4666, copied from bin 29
[ 0.8125,   -1.5940e-02,   2.7220e-07,   4.6818e-01  ],# bin 32  chi2/NDF: 0.9585, copied from bin 34
[ 0.8375,   -1.5940e-02,   2.7220e-07,   4.6818e-01  ],# bin 33  chi2/NDF: 2.1739, copied from bin 34
[ 0.8625,   -1.5940e-02,   2.7220e-07,   4.6818e-01  ],# bin 34  chi2/NDF: 1.7521
[ 0.8875,   -3.1331e-02,   2.7168e-07,   6.5821e-01  ],# bin 35  chi2/NDF: 1.6519
[ 0.9125,   -1.6729e-02,   2.2314e-07,   4.7728e-01  ],# bin 36  chi2/NDF: 3.544
[ 0.9375,   -3.6926e-02,   2.3718e-07,   7.1702e-01  ],# bin 37  chi2/NDF: 2.3457
[ 0.9625,   -4.7510e-02,   2.2893e-07,   8.4432e-01  ],# bin 38  chi2/NDF: 1.4956
[ 0.9875,   -4.7903e-02,   1.9845e-07,   8.5535e-01  ],# bin 39  chi2/NDF: 1.9079
[ 1.0125,   -4.3494e-02,   1.7266e-07,   7.9565e-01  ],# bin 40  chi2/NDF: 2.9131
[ 1.0375,   -5.4602e-02,   1.8144e-07,   9.2056e-01  ],# bin 41  chi2/NDF: 2.4451
[ 1.0625,   -8.2023e-02,   2.1774e-07,   1.2482e+00  ],# bin 42  chi2/NDF: 1.078
[ 1.0875,   -6.9568e-02,   1.5800e-07,   1.1197e+00  ],# bin 43  chi2/NDF: 1.8545
[ 1.1125,   -9.0743e-02,   1.6930e-07,   1.3783e+00  ],# bin 44  chi2/NDF: 1.6027
[ 1.1375,   -8.5068e-02,   1.5885e-07,   1.3052e+00  ],# bin 45  chi2/NDF: 2.2893
[ 1.1625,   -8.8507e-02,   1.0607e-07,   1.4026e+00  ],# bin 46  chi2/NDF: 1.6574
[ 1.1875,   -1.1057e-01,   1.3441e-07,   1.6481e+00  ],# bin 47  chi2/NDF: 2.5578
[ 1.2125,   -1.1778e-01,   1.1971e-07,   1.7574e+00  ],# bin 48  chi2/NDF: 2.2841
[ 1.2375,   -1.4658e-01,   1.3576e-07,   2.1186e+00  ],# bin 49  chi2/NDF: 2.265
[ 1.2625,   -1.4478e-01,   1.4699e-07,   2.1421e+00  ],# bin 50  chi2/NDF: 1.1147
[ 1.2875,   -1.6797e-01,   1.9070e-07,   2.4626e+00  ],# bin 51  chi2/NDF: 2.2903
[ 1.3125,   -1.8567e-01,   3.8769e-07,   2.7593e+00  ],# bin 52  chi2/NDF: 1.3152
[ 1.3375,   -1.2091e-01,   5.8215e-07,   2.3001e+00  ],# bin 53  chi2/NDF: 3.4783
[ 1.3625,   -7.6485e-02,   1.7109e-06,   2.3754e+00  ],# bin 54  chi2/NDF: 2.9381
[ 1.3875,   -2.2076e-02,   6.5580e-06,   4.0251e+00  ],# bin 55  chi2/NDF: 6.4742
[ 1.4125,   1.4755e-01,   1.6287e-05,   4.7835e+00  ],# bin 56  chi2/NDF: 3.6238
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -3.5013e-02,   4.4098e-09,   9.2844e-01  ],# bin 60 crack  chi2/NDF: 1.2581
[ 1.5375,   -7.6826e-03,   1.2275e-08,   3.3657e-01  ],# bin 61 crack  chi2/NDF: 3.6251
[ 1.5625,   1.2853e-02,   5.6069e-09,   1.6512e-03  ],# bin 62  chi2/NDF: 3.7445
[ 1.5875,   1.3103e-02,   2.2456e-08,   7.9464e-03  ],# bin 63  chi2/NDF: 4.3635
[ 1.6125,   7.3818e-03,   2.5414e-08,   7.9386e-02  ],# bin 64  chi2/NDF: 2.2239
[ 1.6375,   6.5507e-03,   3.0655e-08,   7.9849e-02  ],# bin 65  chi2/NDF: 3.8393
[ 1.6625,   7.3001e-03,   2.0410e-08,   6.2677e-02  ],# bin 66  chi2/NDF: 2.8006
[ 1.6875,   1.2955e-02,   9.2083e-09,   -1.3184e-02  ],# bin 67  chi2/NDF: 3.856
[ 1.7125,   1.2001e-02,   6.2127e-09,   -3.3057e-02  ],# bin 68  chi2/NDF: 2.1842
[ 1.7375,   1.1070e-02,   1.4511e-08,   -2.5388e-02  ],# bin 69  chi2/NDF: 1.5323
[ 1.7625,   9.1991e-03,   1.7219e-08,   -4.8327e-03  ],# bin 70  chi2/NDF: 1.2114
[ 1.7875,   9.4538e-03,   2.4169e-08,   -1.0469e-02  ],# bin 71  chi2/NDF: 1.21
[ 1.8125,   7.0513e-03,   2.1005e-08,   4.5683e-03  ],# bin 72  chi2/NDF: 1.2706
[ 1.8375,   9.7439e-03,   3.4763e-08,   -4.1748e-02  ],# bin 73  chi2/NDF: 2.1303
[ 1.8625,   5.9604e-03,   4.5444e-08,   1.0676e-02  ],# bin 74  chi2/NDF: 1.3946
[ 1.8875,   -1.3488e-03,   5.2019e-08,   1.0186e-01  ],# bin 75  chi2/NDF: 1.117
[ 1.9125,   7.2715e-03,   4.0023e-08,   -1.0421e-02  ],# bin 76  chi2/NDF: 2.059
[ 1.9375,   8.0878e-03,   4.0185e-08,   -3.1253e-02  ],# bin 77  chi2/NDF: 1.8992
[ 1.9625,   1.5307e-02,   2.4621e-08,   -1.2204e-01  ],# bin 78  chi2/NDF: 2.7909
[ 1.9875,   2.8383e-04,   3.3177e-08,   6.1753e-02  ],# bin 79  chi2/NDF: 2.6547
[ 2.0125,   7.8474e-03,   1.6619e-08,   -3.7434e-02  ],# bin 80  chi2/NDF: 1.5506
[ 2.0375,   8.1209e-03,   1.5429e-08,   -4.3990e-02  ],# bin 81  chi2/NDF: 2.4158
[ 2.0625,   5.2658e-03,   1.2311e-08,   -1.0185e-02  ],# bin 82  chi2/NDF: 0.8567
[ 2.0875,   5.1750e-03,   8.2405e-09,   -1.2537e-02  ],# bin 83  chi2/NDF: 2.4649
[ 2.1125,   3.1328e-03,   1.3463e-08,   1.2039e-02  ],# bin 84  chi2/NDF: 0.84248
[ 2.1375,   6.4122e-03,   1.0995e-08,   -3.6920e-02  ],# bin 85  chi2/NDF: 1.8119
[ 2.1625,   5.8275e-03,   8.1962e-09,   -3.3578e-02  ],# bin 86  chi2/NDF: 1.6131
[ 2.1875,   5.7903e-03,   6.9654e-09,   -3.6793e-02  ],# bin 87  chi2/NDF: 2.004
[ 2.2125,   5.3153e-03,   6.1170e-09,   -2.9905e-02  ],# bin 88  chi2/NDF: 1.2221
[ 2.2375,   1.4175e-03,   1.1687e-08,   1.7958e-02  ],# bin 89  chi2/NDF: 1.0387
[ 2.2625,   5.1453e-04,   9.8698e-09,   2.9483e-02  ],# bin 90  chi2/NDF: 0.71886
[ 2.2875,   4.2478e-03,   4.9866e-09,   -1.7087e-02  ],# bin 91  chi2/NDF: 1.2555
[ 2.3125,   1.5199e-03,   3.4062e-09,   1.6852e-02  ],# bin 92  chi2/NDF: 1.1722
[ 2.3375,   7.6562e-04,   4.3422e-09,   2.7000e-02  ],# bin 93  chi2/NDF: 2.2953
[ 2.3625,   -3.3859e-03,   5.3949e-09,   8.6717e-02  ],# bin 94  chi2/NDF: 0.99233
[ 2.3875,   -1.2599e-02,   1.3276e-08,   2.1592e-01  ],# bin 95  chi2/NDF: 0.95649
[ 2.4125,   -1.0140e-02,   5.3436e-09,   2.0948e-01  ],# bin 96  chi2/NDF: 1.2027
[ 2.4375,   -1.7650e-02,   8.5628e-09,   3.3931e-01  ],# bin 97  chi2/NDF: 1.2504
[ 2.4625,   -4.0536e-02,   1.3698e-08,   7.6125e-01  ],# bin 98  chi2/NDF: 1.0692
[ 2.4875,   -3.0205e-01,   2.8638e-07,   4.6192e+00  ],# bin 99  chi2/NDF: 5.3542
] # end CaloSwCalHitsLongLeakage_v9_gam_conv


CaloSwCalHitsLongLeakage_v9leakdata_gam_unconv = [
[ 0.0125,   9.6835e-02,   1.5218e-06,   -5.4886e-01  ],# bin 0  chi2/NDF: 3.0372, copied from bin 2
[ 0.0375,   9.6835e-02,   1.5218e-06,   -5.4886e-01  ],# bin 1  chi2/NDF: 213.77, copied from bin 2
[ 0.0625,   9.6835e-02,   1.5218e-06,   -5.4886e-01  ],# bin 2  chi2/NDF: 7.1096
[ 0.0875,   1.0045e-01,   1.4119e-06,   -6.2700e-01  ],# bin 3  chi2/NDF: 7.3481
[ 0.1125,   1.0155e-01,   1.3217e-06,   -6.5340e-01  ],# bin 4  chi2/NDF: 6.1784
[ 0.1375,   9.9512e-02,   1.2909e-06,   -6.4877e-01  ],# bin 5  chi2/NDF: 7.5966
[ 0.1625,   9.8216e-02,   1.2285e-06,   -6.4511e-01  ],# bin 6  chi2/NDF: 18.847
[ 0.1875,   8.9706e-02,   1.2152e-06,   -5.6913e-01  ],# bin 7  chi2/NDF: 8.0699
[ 0.2125,   8.6414e-02,   1.1094e-06,   -5.4164e-01  ],# bin 8  chi2/NDF: 5.6342
[ 0.2375,   9.1175e-02,   9.8262e-07,   -5.9971e-01  ],# bin 9  chi2/NDF: 7.8286
[ 0.2625,   8.5303e-02,   9.1641e-07,   -5.5390e-01  ],# bin 10  chi2/NDF: 8.5256
[ 0.2875,   8.5060e-02,   8.0647e-07,   -5.6703e-01  ],# bin 11  chi2/NDF: 6.5178
[ 0.3125,   1.8418e-01,   -2.5956e-08,   -1.5822e+00  ],# bin 12  chi2/NDF: 869.41
[ 0.3375,   8.3286e-02,   5.9285e-07,   -5.8548e-01  ],# bin 13  chi2/NDF: 70.158
[ 0.3625,   6.9817e-02,   5.6003e-07,   -4.6100e-01  ],# bin 14  chi2/NDF: 5.848
[ 0.3875,   6.1749e-02,   4.9123e-07,   -3.9195e-01  ],# bin 15  chi2/NDF: 4.8832
[ 0.4125,   5.8994e-02,   3.9849e-07,   -3.8102e-01  ],# bin 16  chi2/NDF: 8.8242
[ 0.4375,   6.0555e-02,   3.1880e-07,   -4.1804e-01  ],# bin 17  chi2/NDF: 6.8686
[ 0.4625,   5.3954e-02,   2.6350e-07,   -3.6349e-01  ],# bin 18  chi2/NDF: 9.0514
[ 0.4875,   4.9002e-02,   2.1128e-07,   -3.2697e-01  ],# bin 19  chi2/NDF: 11.788
[ 0.5125,   4.6096e-02,   1.6070e-07,   -3.1342e-01  ],# bin 20  chi2/NDF: 14.295
[ 0.5375,   4.3190e-02,   1.4976e-07,   -2.9631e-01  ],# bin 21  chi2/NDF: 12.69
[ 0.5625,   3.8698e-02,   1.3625e-07,   -2.5920e-01  ],# bin 22  chi2/NDF: 17.263
[ 0.5875,   3.4459e-02,   1.2510e-07,   -2.2398e-01  ],# bin 23  chi2/NDF: 24.069
[ 0.6125,   2.9974e-02,   1.1637e-07,   -1.8085e-01  ],# bin 24  chi2/NDF: 19.482
[ 0.6375,   2.9441e-02,   1.0414e-07,   -1.7658e-01  ],# bin 25  chi2/NDF: 15.381
[ 0.6625,   2.7035e-02,   9.6587e-08,   -1.5314e-01  ],# bin 26  chi2/NDF: 20.297
[ 0.6875,   2.6353e-02,   8.2779e-08,   -1.5134e-01  ],# bin 27  chi2/NDF: 19.012
[ 0.7125,   2.5442e-02,   7.4150e-08,   -1.4809e-01  ],# bin 28  chi2/NDF: 17.325
[ 0.7375,   2.2784e-02,   6.9016e-08,   -1.2150e-01  ],# bin 29  chi2/NDF: 12.718
[ 0.7625,   2.2784e-02,   6.9016e-08,   -1.2150e-01  ],# bin 30  chi2/NDF: 8.0853, copied from bin 29
[ 0.7875,   2.2784e-02,   6.9016e-08,   -1.2150e-01  ],# bin 31  chi2/NDF: 6.8883, copied from bin 29
[ 0.8125,   2.3278e-02,   1.3723e-07,   -6.7638e-02  ],# bin 32  chi2/NDF: 7.8226, copied from bin 34
[ 0.8375,   2.3278e-02,   1.3723e-07,   -6.7638e-02  ],# bin 33  chi2/NDF: 4.3779, copied from bin 34
[ 0.8625,   2.3278e-02,   1.3723e-07,   -6.7638e-02  ],# bin 34  chi2/NDF: 8.7284
[ 0.8875,   2.6571e-02,   1.1677e-07,   -1.1029e-01  ],# bin 35  chi2/NDF: 10.656
[ 0.9125,   2.4233e-02,   1.0345e-07,   -8.7408e-02  ],# bin 36  chi2/NDF: 16.389
[ 0.9375,   2.0392e-02,   9.6824e-08,   -4.7730e-02  ],# bin 37  chi2/NDF: 14.676
[ 0.9625,   1.9429e-02,   8.5707e-08,   -4.4158e-02  ],# bin 38  chi2/NDF: 21.771
[ 0.9875,   1.5026e-02,   8.3226e-08,   -8.9149e-04  ],# bin 39  chi2/NDF: 46.545
[ 1.0125,   1.5626e-02,   6.6748e-08,   -1.2395e-02  ],# bin 40  chi2/NDF: 13.298
[ 1.0375,   1.4606e-02,   5.8084e-08,   -4.8105e-03  ],# bin 41  chi2/NDF: 11.595
[ 1.0625,   1.3767e-02,   4.7548e-08,   2.1500e-03  ],# bin 42  chi2/NDF: 14.766
[ 1.0875,   1.0383e-02,   4.6780e-08,   3.5740e-02  ],# bin 43  chi2/NDF: 9.5246
[ 1.1125,   8.6371e-03,   3.9031e-08,   5.3352e-02  ],# bin 44  chi2/NDF: 10.116
[ 1.1375,   7.0866e-03,   3.6213e-08,   6.6675e-02  ],# bin 45  chi2/NDF: 9.6485
[ 1.1625,   2.2136e-03,   3.5718e-08,   1.2355e-01  ],# bin 46  chi2/NDF: 6.5916
[ 1.1875,   9.1551e-04,   2.6953e-08,   1.4194e-01  ],# bin 47  chi2/NDF: 4.4632
[ 1.2125,   -3.7758e-03,   2.7678e-08,   1.9815e-01  ],# bin 48  chi2/NDF: 5.7707
[ 1.2375,   -5.0279e-03,   2.4170e-08,   2.1916e-01  ],# bin 49  chi2/NDF: 6.3269
[ 1.2625,   -6.7808e-03,   2.5666e-08,   2.5669e-01  ],# bin 50  chi2/NDF: 4.0749
[ 1.2875,   -6.0074e-03,   4.0055e-08,   2.8018e-01  ],# bin 51  chi2/NDF: 3.7086
[ 1.3125,   3.7160e-03,   8.6818e-08,   2.5902e-01  ],# bin 52  chi2/NDF: 3.0265
[ 1.3375,   2.9128e-02,   2.7183e-07,   1.9338e-01  ],# bin 53  chi2/NDF: 6.0852
[ 1.3625,   1.1606e-01,   7.0474e-07,   -2.4932e-01  ],# bin 54  chi2/NDF: 4.9763
[ 1.3875,   3.5336e-01,   2.4277e-06,   -8.2869e-01  ],# bin 55  chi2/NDF: 17.756
[ 1.4125,   6.1514e-01,   5.5938e-06,   -9.8273e-01  ],# bin 56  chi2/NDF: 5.7328
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -2.3554e-02,   1.9685e-09,   6.8505e-01  ],# bin 60 crack  chi2/NDF: 3.7243
[ 1.5375,   4.6101e-03,   2.1016e-09,   1.5221e-01  ],# bin 61 crack  chi2/NDF: 23.557
[ 1.5625,   2.1391e-02,   1.6642e-09,   -1.5017e-01  ],# bin 62  chi2/NDF: 5.0913
[ 1.5875,   2.2809e-02,   7.4144e-09,   -1.5502e-01  ],# bin 63  chi2/NDF: 8.2105
[ 1.6125,   2.0967e-02,   7.5745e-09,   -1.3738e-01  ],# bin 64  chi2/NDF: 7.6111
[ 1.6375,   1.9381e-02,   9.1997e-09,   -1.1766e-01  ],# bin 65  chi2/NDF: 5.9691
[ 1.6625,   1.7139e-02,   6.0523e-09,   -9.8797e-02  ],# bin 66  chi2/NDF: 4.0361
[ 1.6875,   1.6082e-02,   3.8581e-09,   -9.3206e-02  ],# bin 67  chi2/NDF: 4.8503
[ 1.7125,   1.4218e-02,   2.6755e-09,   -9.4239e-02  ],# bin 68  chi2/NDF: 13.627
[ 1.7375,   1.2780e-02,   7.1360e-09,   -7.5535e-02  ],# bin 69  chi2/NDF: 10.316
[ 1.7625,   1.2436e-02,   8.5805e-09,   -7.5309e-02  ],# bin 70  chi2/NDF: 10.252
[ 1.7875,   1.1388e-02,   1.3261e-08,   -6.9822e-02  ],# bin 71  chi2/NDF: 20.792
[ 1.8125,   1.2482e-02,   1.1329e-08,   -9.4795e-02  ],# bin 72  chi2/NDF: 12.146
[ 1.8375,   1.0965e-02,   2.1908e-08,   -7.8843e-02  ],# bin 73  chi2/NDF: 24.628
[ 1.8625,   1.0401e-02,   2.7483e-08,   -7.2814e-02  ],# bin 74  chi2/NDF: 17.371
[ 1.8875,   1.0112e-02,   2.7816e-08,   -7.0590e-02  ],# bin 75  chi2/NDF: 21.171
[ 1.9125,   1.1420e-02,   2.2906e-08,   -8.9778e-02  ],# bin 76  chi2/NDF: 22.254
[ 1.9375,   1.0705e-02,   2.0391e-08,   -8.4460e-02  ],# bin 77  chi2/NDF: 26.436
[ 1.9625,   1.0913e-02,   1.6185e-08,   -8.8740e-02  ],# bin 78  chi2/NDF: 14.821
[ 1.9875,   8.9731e-03,   1.4767e-08,   -6.6676e-02  ],# bin 79  chi2/NDF: 20.622
[ 2.0125,   7.7351e-03,   1.0523e-08,   -5.9758e-02  ],# bin 80  chi2/NDF: 17.241
[ 2.0375,   7.3710e-03,   9.4609e-09,   -5.4568e-02  ],# bin 81  chi2/NDF: 13.535
[ 2.0625,   1.0338e-02,   5.5834e-09,   -9.8981e-02  ],# bin 82  chi2/NDF: 12.807
[ 2.0875,   8.6280e-03,   4.5725e-09,   -7.9027e-02  ],# bin 83  chi2/NDF: 7.6555
[ 2.1125,   7.5806e-03,   6.5673e-09,   -6.6080e-02  ],# bin 84  chi2/NDF: 12.655
[ 2.1375,   7.4989e-03,   6.0805e-09,   -6.7852e-02  ],# bin 85  chi2/NDF: 8.4795
[ 2.1625,   6.3242e-03,   4.6221e-09,   -5.4221e-02  ],# bin 86  chi2/NDF: 11.241
[ 2.1875,   5.6935e-03,   3.9726e-09,   -4.7074e-02  ],# bin 87  chi2/NDF: 7.5089
[ 2.2125,   7.1794e-03,   2.6881e-09,   -6.8245e-02  ],# bin 88  chi2/NDF: 6.8403
[ 2.2375,   6.5667e-03,   4.3522e-09,   -6.0154e-02  ],# bin 89  chi2/NDF: 5.9856
[ 2.2625,   5.2090e-03,   3.9111e-09,   -4.4408e-02  ],# bin 90  chi2/NDF: 5.8202
[ 2.2875,   7.0114e-03,   2.2445e-09,   -7.0245e-02  ],# bin 91  chi2/NDF: 5.4954
[ 2.3125,   -8.6932e-04,   3.2272e-09,   3.1683e-02  ],# bin 92  chi2/NDF: 15.962
[ 2.3375,   2.3389e-03,   3.6231e-09,   -1.5356e-02  ],# bin 93  chi2/NDF: 8.6033
[ 2.3625,   2.2057e-03,   2.8186e-09,   -1.1684e-02  ],# bin 94  chi2/NDF: 3.534
[ 2.3875,   3.8333e-03,   3.9427e-09,   -2.6463e-02  ],# bin 95  chi2/NDF: 2.8956
[ 2.4125,   3.4223e-03,   3.5585e-09,   -1.0722e-02  ],# bin 96  chi2/NDF: 2.7659
[ 2.4375,   3.3318e-03,   3.3620e-09,   1.5384e-02  ],# bin 97  chi2/NDF: 1.774
[ 2.4625,   1.2909e-02,   2.5367e-09,   -3.3836e-02  ],# bin 98  chi2/NDF: 2.582
[ 2.4875,   -1.0508e-01,   1.2906e-08,   2.1331e+00  ],# bin 99  chi2/NDF: 5.4966
] # end CaloSwCalHitsLongLeakage_v9_gam_unconv


CaloSwCalHitsEfront_slope_v9leakdata_ele = [
[ 0.0125,   6.7211e-01,   4.1263e-01,   -7.1211e-03  ],# bin 0  chi2/NDF: 1.8211, copied from bin 2
[ 0.0375,   6.7211e-01,   4.1263e-01,   -7.1211e-03  ],# bin 1  chi2/NDF: 2.3179, copied from bin 2
[ 0.0625,   6.7211e-01,   4.1263e-01,   -7.1211e-03  ],# bin 2  chi2/NDF: 2.9199
[ 0.0875,   6.7311e-01,   4.0461e-01,   -6.2275e-03  ],# bin 3  chi2/NDF: 2.4824
[ 0.1125,   6.8101e-01,   4.0282e-01,   -7.1515e-03  ],# bin 4  chi2/NDF: 3.0327
[ 0.1375,   6.5599e-01,   4.2339e-01,   -6.8336e-03  ],# bin 5  chi2/NDF: 1.6453
[ 0.1625,   6.5250e-01,   4.2696e-01,   -7.2228e-03  ],# bin 6  chi2/NDF: 3.5169
[ 0.1875,   6.4426e-01,   4.4293e-01,   -8.4860e-03  ],# bin 7  chi2/NDF: 2.2289
[ 0.2125,   6.4403e-01,   4.1537e-01,   -6.0556e-03  ],# bin 8  chi2/NDF: 1.518
[ 0.2375,   6.3815e-01,   4.2080e-01,   -6.1021e-03  ],# bin 9  chi2/NDF: 1.0291
[ 0.2625,   6.2417e-01,   4.4062e-01,   -7.0942e-03  ],# bin 10  chi2/NDF: 2.0047
[ 0.2875,   6.1581e-01,   4.4894e-01,   -7.6405e-03  ],# bin 11  chi2/NDF: 2.3275
[ 0.3125,   6.1270e-01,   4.4875e-01,   -6.9061e-03  ],# bin 12  chi2/NDF: 4.0028
[ 0.3375,   6.3027e-01,   4.3413e-01,   -6.8611e-03  ],# bin 13  chi2/NDF: 3.2205
[ 0.3625,   6.3571e-01,   4.2487e-01,   -6.6370e-03  ],# bin 14  chi2/NDF: 2.4249
[ 0.3875,   6.2470e-01,   4.3309e-01,   -6.2935e-03  ],# bin 15  chi2/NDF: 5.4071
[ 0.4125,   6.2036e-01,   4.2154e-01,   -5.6275e-03  ],# bin 16  chi2/NDF: 2.8064
[ 0.4375,   6.1792e-01,   4.2176e-01,   -5.1819e-03  ],# bin 17  chi2/NDF: 1.7779
[ 0.4625,   6.2088e-01,   4.2649e-01,   -5.8325e-03  ],# bin 18  chi2/NDF: 2.6157
[ 0.4875,   6.1248e-01,   4.3591e-01,   -6.0627e-03  ],# bin 19  chi2/NDF: 4.2364
[ 0.5125,   6.1918e-01,   4.2396e-01,   -6.0157e-03  ],# bin 20  chi2/NDF: 2.6433
[ 0.5375,   6.3551e-01,   4.1346e-01,   -6.0538e-03  ],# bin 21  chi2/NDF: 3.4795
[ 0.5625,   6.3749e-01,   4.1513e-01,   -6.6083e-03  ],# bin 22  chi2/NDF: 6.3393
[ 0.5875,   6.5262e-01,   3.9792e-01,   -6.1981e-03  ],# bin 23  chi2/NDF: 2.9308
[ 0.6125,   6.6097e-01,   3.9135e-01,   -5.2188e-03  ],# bin 24  chi2/NDF: 2.4527
[ 0.6375,   6.9740e-01,   3.7493e-01,   -6.2163e-03  ],# bin 25  chi2/NDF: 3.5132
[ 0.6625,   7.3732e-01,   3.6236e-01,   -8.1490e-03  ],# bin 26  chi2/NDF: 4.6776
[ 0.6875,   7.6045e-01,   3.5148e-01,   -8.3965e-03  ],# bin 27  chi2/NDF: 5.0528
[ 0.7125,   7.8522e-01,   3.3600e-01,   -8.0078e-03  ],# bin 28  chi2/NDF: 5.1606
[ 0.7375,   7.6001e-01,   3.8145e-01,   -1.0970e-02  ],# bin 29  chi2/NDF: 8.048
[ 0.7625,   7.6001e-01,   3.8145e-01,   -1.0970e-02  ],# bin 30  chi2/NDF: 6.0105, copied from bin 29
[ 0.7875,   7.6001e-01,   3.8145e-01,   -1.0970e-02  ],# bin 31  chi2/NDF: 4.1295, copied from bin 29
[ 0.8125,   8.4532e-01,   3.6245e-01,   -1.3573e-02  ],# bin 32  chi2/NDF: 3.8759, copied from bin 34
[ 0.8375,   8.4532e-01,   3.6245e-01,   -1.3573e-02  ],# bin 33  chi2/NDF: 7.4405, copied from bin 34
[ 0.8625,   8.4532e-01,   3.6245e-01,   -1.3573e-02  ],# bin 34  chi2/NDF: 6.9768
[ 0.8875,   8.1993e-01,   3.8841e-01,   -1.4893e-02  ],# bin 35  chi2/NDF: 4.2931
[ 0.9125,   8.1444e-01,   4.0457e-01,   -1.6571e-02  ],# bin 36  chi2/NDF: 9.2757
[ 0.9375,   8.5148e-01,   3.5902e-01,   -1.3471e-02  ],# bin 37  chi2/NDF: 7.0432
[ 0.9625,   8.6302e-01,   3.4931e-01,   -1.3168e-02  ],# bin 38  chi2/NDF: 5.4649
[ 0.9875,   8.9988e-01,   3.2301e-01,   -1.3294e-02  ],# bin 39  chi2/NDF: 8.136
[ 1.0125,   7.7206e-01,   4.1339e-01,   -1.3229e-02  ],# bin 40  chi2/NDF: 8.7005
[ 1.0375,   7.4447e-01,   4.4483e-01,   -1.5392e-02  ],# bin 41  chi2/NDF: 10.173
[ 1.0625,   7.7171e-01,   4.1578e-01,   -1.4065e-02  ],# bin 42  chi2/NDF: 7.3119
[ 1.0875,   7.8979e-01,   4.0695e-01,   -1.4276e-02  ],# bin 43  chi2/NDF: 5.8204
[ 1.1125,   7.3349e-01,   4.5026e-01,   -1.5177e-02  ],# bin 44  chi2/NDF: 8.3614
[ 1.1375,   7.4712e-01,   4.3449e-01,   -1.4312e-02  ],# bin 45  chi2/NDF: 10.156
[ 1.1625,   7.6184e-01,   4.2391e-01,   -1.4279e-02  ],# bin 46  chi2/NDF: 5.0571
[ 1.1875,   7.1590e-01,   4.6970e-01,   -1.7159e-02  ],# bin 47  chi2/NDF: 8.2411
[ 1.2125,   7.4963e-01,   3.9906e-01,   -1.2885e-02  ],# bin 48  chi2/NDF: 3.8064
[ 1.2375,   7.3534e-01,   4.0748e-01,   -1.2611e-02  ],# bin 49  chi2/NDF: 8.4292
[ 1.2625,   6.8296e-01,   4.6836e-01,   -1.5793e-02  ],# bin 50  chi2/NDF: 5.2217
[ 1.2875,   7.4806e-01,   4.0922e-01,   -1.4984e-02  ],# bin 51  chi2/NDF: 6.1633
[ 1.3125,   7.4440e-01,   4.1945e-01,   -1.4950e-02  ],# bin 52  chi2/NDF: 6.6203
[ 1.3375,   7.8869e-01,   3.9467e-01,   -1.6213e-02  ],# bin 53  chi2/NDF: 8.9271
[ 1.3625,   7.1850e-01,   4.4753e-01,   -1.5762e-02  ],# bin 54  chi2/NDF: 5.6257
[ 1.3875,   7.7177e-01,   4.1281e-01,   -1.6898e-02  ],# bin 55  chi2/NDF: 7.3281
[ 1.4125,   6.7786e-01,   5.3285e-01,   -2.5593e-02  ],# bin 56  chi2/NDF: 6.1035
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -3.0040e+00,   1.0868e+00,   -3.3812e-01  ],# bin 60 crack  chi2/NDF: 4.4161
[ 1.5375,   7.1853e+00,   -1.6136e+00,   1.3848e-01  ],# bin 61 crack  chi2/NDF: 13.494
[ 1.5625,   2.0468e+00,   -2.4231e-01,   2.0404e-02  ],# bin 62  chi2/NDF: 4.8347
[ 1.5875,   1.9880e+00,   -1.9629e-01,   1.2168e-02  ],# bin 63  chi2/NDF: 6.2408
[ 1.6125,   1.6835e+00,   -1.1118e-01,   3.4760e-03  ],# bin 64  chi2/NDF: 1.6689
[ 1.6375,   1.7427e+00,   -2.9001e-01,   3.8370e-02  ],# bin 65  chi2/NDF: 2.6525
[ 1.6625,   2.7414e+00,   -5.2501e-01,   5.9251e-02  ],# bin 66  chi2/NDF: 3.1227
[ 1.6875,   6.1409e+00,   -1.2868e+00,   1.1536e-01  ],# bin 67  chi2/NDF: 6.0017
[ 1.7125,   1.0549e+00,   -1.3854e-01,   2.6833e-02  ],# bin 68  chi2/NDF: 4.9387
[ 1.7375,   -1.6090e-01,   1.8539e-01,   -4.6652e-03  ],# bin 69  chi2/NDF: 2.1799
[ 1.7625,   -8.8389e-01,   3.9604e-01,   -2.8799e-02  ],# bin 70  chi2/NDF: 6.9311
[ 1.7875,   2.9517e-01,   1.0344e-01,   1.4032e-03  ],# bin 71  chi2/NDF: 3.3246
[ 1.8125,   -1.2760e+02,   -1.1267e+02,   2.4063e-02  ],# bin 72  chi2/NDF: 0.68127
[ 1.8375,   -9.8665e+02,   -1.3303e+02,   3.0091e-02  ],# bin 73  chi2/NDF: 0.74445
[ 1.8625,   -1.0682e+03,   -1.0738e+02,   2.6884e-03  ],# bin 74  chi2/NDF: 1.1891
[ 1.8875,   -5.8709e+02,   -1.0241e+02,   -1.3356e-02  ],# bin 75  chi2/NDF: 0.41121
[ 1.9125,   -6.8878e+02,   -1.0038e+02,   1.1408e-02  ],# bin 76  chi2/NDF: 1.3515
[ 1.9375,   -6.3622e+02,   -9.0243e+01,   2.0627e-02  ],# bin 77  chi2/NDF: 1.8862
[ 1.9625,   -7.7111e+02,   -8.1851e+01,   1.1407e-02  ],# bin 78  chi2/NDF: 0.7984
[ 1.9875,   -5.7338e+02,   -9.5886e+01,   4.1223e-02  ],# bin 79  chi2/NDF: 0.54321
[ 2.0125,   -7.6794e+02,   -8.8355e+01,   2.5382e-02  ],# bin 80  chi2/NDF: 1.8707
[ 2.0375,   -4.1661e+02,   -9.1285e+01,   1.9821e-02  ],# bin 81  chi2/NDF: 1.9334
[ 2.0625,   -4.0568e+02,   -8.9462e+01,   1.6451e-02  ],# bin 82  chi2/NDF: 1.5561
[ 2.0875,   -3.6643e+02,   -8.6403e+01,   1.6373e-02  ],# bin 83  chi2/NDF: 2.8567
[ 2.1125,   -8.6769e+02,   -8.5087e+01,   3.0848e-02  ],# bin 84  chi2/NDF: 1.733
[ 2.1375,   -6.9377e+02,   -8.7587e+01,   3.5323e-02  ],# bin 85  chi2/NDF: 1.2996
[ 2.1625,   -5.4312e+02,   -8.0171e+01,   2.1698e-02  ],# bin 86  chi2/NDF: 2.6936
[ 2.1875,   -3.1232e+02,   -8.0429e+01,   2.8640e-02  ],# bin 87  chi2/NDF: 2.0596
[ 2.2125,   -6.1376e+02,   -7.3619e+01,   1.7928e-02  ],# bin 88  chi2/NDF: 0.66653
[ 2.2375,   -6.4049e+02,   -7.6234e+01,   3.0111e-02  ],# bin 89  chi2/NDF: 2.3246
[ 2.2625,   -7.4464e+02,   -6.5495e+01,   2.4081e-02  ],# bin 90  chi2/NDF: 0.8476
[ 2.2875,   -7.4977e+02,   -6.8396e+01,   3.0462e-02  ],# bin 91  chi2/NDF: 0.80992
[ 2.3125,   -3.7732e+02,   -3.1578e+01,   3.8104e-02  ],# bin 92  chi2/NDF: 2.8802
[ 2.3375,   -7.4781e+02,   -9.8049e+01,   -2.1810e-03  ],# bin 93  chi2/NDF: 0.78192
[ 2.3625,   -5.4474e+02,   -1.0752e+02,   1.8698e-02  ],# bin 94  chi2/NDF: 0.885
[ 2.3875,   -1.1030e+03,   -1.0001e+02,   1.4620e-02  ],# bin 95  chi2/NDF: 1.6975
[ 2.4125,   -2.2358e+02,   -1.1636e+02,   4.9331e-02  ],# bin 96  chi2/NDF: 0.8144
[ 2.4375,   9.1336e+02,   -1.3922e+02,   7.0112e-02  ],# bin 97  chi2/NDF: 0.17024
[ 2.4625,   1.4636e+03,   -1.6353e+02,   1.3593e-01  ],# bin 98  chi2/NDF: 0.92416
[ 2.4875,   1.2276e+03,   -1.5137e+02,   1.3374e-01  ],# bin 99  chi2/NDF: 1.5673
] # end CaloSwCalHitsEfront_slope_v9_ele


CaloSwCalHitsEfront_slope_v9leakdata_gam_conv = [
[ 0.0125,   5.5888e-01,   4.9969e-01,   -6.0713e-03  ],# bin 0  chi2/NDF: 1.8401, copied from bin 2
[ 0.0375,   5.5888e-01,   4.9969e-01,   -6.0713e-03  ],# bin 1  chi2/NDF: 3.0211, copied from bin 2
[ 0.0625,   5.5888e-01,   4.9969e-01,   -6.0713e-03  ],# bin 2  chi2/NDF: 2.9671
[ 0.0875,   4.9382e-01,   5.9181e-01,   -8.6621e-03  ],# bin 3  chi2/NDF: 2.3899
[ 0.1125,   5.6220e-01,   4.8184e-01,   -5.3614e-03  ],# bin 4  chi2/NDF: 1.3978
[ 0.1375,   5.3079e-01,   5.3746e-01,   -7.1413e-03  ],# bin 5  chi2/NDF: 2.5718
[ 0.1625,   5.3980e-01,   5.1031e-01,   -5.6693e-03  ],# bin 6  chi2/NDF: 2.2011
[ 0.1875,   5.1140e-01,   5.3824e-01,   -5.3597e-03  ],# bin 7  chi2/NDF: 3.2261
[ 0.2125,   5.3230e-01,   4.9258e-01,   -3.8224e-03  ],# bin 8  chi2/NDF: 1.5651
[ 0.2375,   5.3022e-01,   4.9239e-01,   -3.7875e-03  ],# bin 9  chi2/NDF: 2.4272
[ 0.2625,   4.4763e-01,   6.2957e-01,   -8.5070e-03  ],# bin 10  chi2/NDF: 1.8061
[ 0.2875,   4.8091e-01,   5.5811e-01,   -5.4198e-03  ],# bin 11  chi2/NDF: 1.7369
[ 0.3125,   4.4071e-01,   6.3668e-01,   -8.3653e-03  ],# bin 12  chi2/NDF: 4.718
[ 0.3375,   4.6613e-01,   6.0769e-01,   -8.4632e-03  ],# bin 13  chi2/NDF: 2.9691
[ 0.3625,   4.2685e-01,   6.8377e-01,   -1.1855e-02  ],# bin 14  chi2/NDF: 1.3918
[ 0.3875,   4.5819e-01,   6.1574e-01,   -8.6869e-03  ],# bin 15  chi2/NDF: 1.7252
[ 0.4125,   5.2828e-01,   4.7026e-01,   -1.8020e-03  ],# bin 16  chi2/NDF: 2.3158
[ 0.4375,   5.1357e-01,   4.9015e-01,   -2.7757e-03  ],# bin 17  chi2/NDF: 1.9391
[ 0.4625,   4.8319e-01,   5.4896e-01,   -5.0776e-03  ],# bin 18  chi2/NDF: 1.5182
[ 0.4875,   4.9396e-01,   5.5143e-01,   -6.8359e-03  ],# bin 19  chi2/NDF: 1.8507
[ 0.5125,   5.0657e-01,   5.1584e-01,   -4.1598e-03  ],# bin 20  chi2/NDF: 2.3709
[ 0.5375,   4.9164e-01,   5.5376e-01,   -7.0175e-03  ],# bin 21  chi2/NDF: 2.7438
[ 0.5625,   4.9871e-01,   5.4688e-01,   -7.2945e-03  ],# bin 22  chi2/NDF: 5.8564
[ 0.5875,   4.6344e-01,   6.0621e-01,   -8.2025e-03  ],# bin 23  chi2/NDF: 5.5358
[ 0.6125,   5.1459e-01,   5.3881e-01,   -7.2323e-03  ],# bin 24  chi2/NDF: 4.5059
[ 0.6375,   6.3575e-01,   3.9196e-01,   -3.3527e-03  ],# bin 25  chi2/NDF: 4.0899
[ 0.6625,   6.0060e-01,   4.5997e-01,   -6.5580e-03  ],# bin 26  chi2/NDF: 3.4761
[ 0.6875,   6.7029e-01,   3.8996e-01,   -4.9479e-03  ],# bin 27  chi2/NDF: 1.9225
[ 0.7125,   6.7281e-01,   4.0181e-01,   -6.9728e-03  ],# bin 28  chi2/NDF: 4.0941
[ 0.7375,   6.6559e-01,   4.1655e-01,   -6.9281e-03  ],# bin 29  chi2/NDF: 4.56
[ 0.7625,   6.6559e-01,   4.1655e-01,   -6.9281e-03  ],# bin 30  chi2/NDF: 1.4, copied from bin 29
[ 0.7875,   6.6559e-01,   4.1655e-01,   -6.9281e-03  ],# bin 31  chi2/NDF: 3.8014, copied from bin 29
[ 0.8125,   8.5103e-01,   2.9904e-01,   -6.1914e-03  ],# bin 32  chi2/NDF: 5.6277, copied from bin 34
[ 0.8375,   8.5103e-01,   2.9904e-01,   -6.1914e-03  ],# bin 33  chi2/NDF: 1.8611, copied from bin 34
[ 0.8625,   8.5103e-01,   2.9904e-01,   -6.1914e-03  ],# bin 34  chi2/NDF: 1.9992
[ 0.8875,   7.4673e-01,   4.1882e-01,   -1.1467e-02  ],# bin 35  chi2/NDF: 4.9372
[ 0.9125,   7.6516e-01,   4.0242e-01,   -1.1621e-02  ],# bin 36  chi2/NDF: 4.3233
[ 0.9375,   7.2719e-01,   4.3897e-01,   -1.2729e-02  ],# bin 37  chi2/NDF: 5.9427
[ 0.9625,   6.7164e-01,   4.9262e-01,   -1.3599e-02  ],# bin 38  chi2/NDF: 6.2341
[ 0.9875,   6.7758e-01,   4.9108e-01,   -1.4335e-02  ],# bin 39  chi2/NDF: 4.1335
[ 1.0125,   6.8476e-01,   4.8139e-01,   -1.3962e-02  ],# bin 40  chi2/NDF: 4.7951
[ 1.0375,   7.3330e-01,   4.1772e-01,   -1.0992e-02  ],# bin 41  chi2/NDF: 4.4238
[ 1.0625,   6.8919e-01,   4.7088e-01,   -1.3535e-02  ],# bin 42  chi2/NDF: 4.8865
[ 1.0875,   6.8700e-01,   4.6826e-01,   -1.2748e-02  ],# bin 43  chi2/NDF: 4.4369
[ 1.1125,   6.3073e-01,   5.2417e-01,   -1.4173e-02  ],# bin 44  chi2/NDF: 5.8599
[ 1.1375,   6.2808e-01,   5.2240e-01,   -1.4428e-02  ],# bin 45  chi2/NDF: 3.8866
[ 1.1625,   6.4831e-01,   4.9665e-01,   -1.2650e-02  ],# bin 46  chi2/NDF: 5.9271
[ 1.1875,   6.2237e-01,   5.2690e-01,   -1.4600e-02  ],# bin 47  chi2/NDF: 7.4143
[ 1.2125,   6.8445e-01,   4.3127e-01,   -1.1348e-02  ],# bin 48  chi2/NDF: 3.7913
[ 1.2375,   5.6153e-01,   5.6727e-01,   -1.5050e-02  ],# bin 49  chi2/NDF: 6.7054
[ 1.2625,   5.4269e-01,   5.9819e-01,   -1.6785e-02  ],# bin 50  chi2/NDF: 3.4298
[ 1.2875,   5.8009e-01,   5.4521e-01,   -1.4600e-02  ],# bin 51  chi2/NDF: 2.6835
[ 1.3125,   6.3401e-01,   4.9765e-01,   -1.5042e-02  ],# bin 52  chi2/NDF: 6.437
[ 1.3375,   5.9543e-01,   5.4576e-01,   -1.6195e-02  ],# bin 53  chi2/NDF: 8.891
[ 1.3625,   6.1438e-01,   5.2528e-01,   -1.5548e-02  ],# bin 54  chi2/NDF: 1.9523
[ 1.3875,   7.5114e-01,   4.1561e-01,   -1.6645e-02  ],# bin 55  chi2/NDF: 4.7115
[ 1.4125,   5.7839e-01,   6.3688e-01,   -2.7838e-02  ],# bin 56  chi2/NDF: 1.2186
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -3.0840e+00,   1.0590e+00,   -3.3990e-01  ],# bin 60 crack  chi2/NDF: 2.8522
[ 1.5375,   8.0743e+00,   -2.0097e+00,   2.1262e-01  ],# bin 61 crack  chi2/NDF: 16.034
[ 1.5625,   1.6709e+00,   -1.9930e-01,   2.3135e-02  ],# bin 62  chi2/NDF: 5.8012
[ 1.5875,   1.3687e+00,   -9.7071e-02,   1.1612e-02  ],# bin 63  chi2/NDF: 3.0456
[ 1.6125,   1.0736e+00,   3.6035e-03,   -4.2385e-03  ],# bin 64  chi2/NDF: 5.817
[ 1.6375,   3.0021e+00,   -6.5212e-01,   7.6558e-02  ],# bin 65  chi2/NDF: 2.2939
[ 1.6625,   3.9540e+00,   -8.9289e-01,   1.0062e-01  ],# bin 66  chi2/NDF: 7.8467
[ 1.6875,   5.3952e+00,   -1.1521e+00,   1.0837e-01  ],# bin 67  chi2/NDF: 5.7592
[ 1.7125,   7.8659e-01,   -7.3298e-02,   2.0630e-02  ],# bin 68  chi2/NDF: 6.0614
[ 1.7375,   -1.5903e-01,   1.8398e-01,   -7.2108e-03  ],# bin 69  chi2/NDF: 3.6712
[ 1.7625,   -4.2383e-01,   2.4813e-01,   -9.9637e-03  ],# bin 70  chi2/NDF: 1.3937
[ 1.7875,   1.2251e-01,   1.2120e-01,   1.6072e-03  ],# bin 71  chi2/NDF: 2.3835
[ 1.8125,   -1.5265e+02,   -1.0535e+02,   -4.8230e-02  ],# bin 72  chi2/NDF: 4.1519
[ 1.8375,   -1.1889e+03,   -1.0469e+02,   -5.2553e-02  ],# bin 73  chi2/NDF: 23.676
[ 1.8625,   -4.5981e+02,   -1.0470e+02,   -7.2233e-02  ],# bin 74  chi2/NDF: 11.736
[ 1.8875,   -7.4847e+02,   -1.0914e+02,   -1.0650e-02  ],# bin 75  chi2/NDF: 9.0559
[ 1.9125,   2.2928e+02,   -1.0491e+02,   -4.2960e-02  ],# bin 76  chi2/NDF: 5.5897
[ 1.9375,   -2.8438e+02,   -1.0267e+02,   -5.5105e-02  ],# bin 77  chi2/NDF: 8.1088
[ 1.9625,   -6.5983e+01,   -1.0195e+02,   -1.1341e-02  ],# bin 78  chi2/NDF: 12.114
[ 1.9875,   -4.6876e+02,   -8.5807e+01,   -4.1988e-02  ],# bin 79  chi2/NDF: 9.0416
[ 2.0125,   -1.3392e+03,   -7.8913e+01,   -4.8780e-02  ],# bin 80  chi2/NDF: 1.3786
[ 2.0375,   -7.5234e+02,   -9.7921e+01,   -2.4236e-02  ],# bin 81  chi2/NDF: 5.3997
[ 2.0625,   -1.8013e+03,   -8.7099e+01,   -4.6601e-02  ],# bin 82  chi2/NDF: 14.673
[ 2.0875,   -2.3003e+03,   -8.6956e+01,   -1.9184e-02  ],# bin 83  chi2/NDF: 21.304
[ 2.1125,   -2.7237e+03,   -9.4231e+01,   -5.7945e-03  ],# bin 84  chi2/NDF: 16.728
[ 2.1375,   -3.0922e+03,   -9.1058e+01,   -4.8026e-03  ],# bin 85  chi2/NDF: 6.8514
[ 2.1625,   -1.6358e+03,   -8.8937e+01,   1.9145e-02  ],# bin 86  chi2/NDF: 6.1837
[ 2.1875,   -1.7282e+03,   -7.5826e+01,   -2.4263e-02  ],# bin 87  chi2/NDF: 5.5167
[ 2.2125,   -1.8424e+03,   -7.5397e+01,   -1.3852e-03  ],# bin 88  chi2/NDF: 33.925
[ 2.2375,   -2.8344e+03,   -6.9358e+01,   -1.6057e-02  ],# bin 89  chi2/NDF: 11.278
[ 2.2625,   -6.9796e+02,   -6.7156e+01,   8.9528e-03  ],# bin 90  chi2/NDF: 15.06
[ 2.2875,   -8.1073e+02,   -7.4105e+01,   3.2151e-02  ],# bin 91  chi2/NDF: 41.439
[ 2.3125,   -1.0409e+03,   -6.0262e+01,   2.4056e-02  ],# bin 92  chi2/NDF: 26.013
[ 2.3375,   -2.2984e+03,   -9.4084e+01,   -1.2887e-01  ],# bin 93  chi2/NDF: 9.1849
[ 2.3625,   -4.6053e+02,   -1.1491e+02,   -1.3033e-01  ],# bin 94  chi2/NDF: 3.5069
[ 2.3875,   -1.5278e+03,   -1.1025e+02,   -8.1077e-02  ],# bin 95  chi2/NDF: 9.3548
[ 2.4125,   -1.8220e+03,   -1.1235e+02,   -1.2095e-01  ],# bin 96  chi2/NDF: 11.719
[ 2.4375,   2.6031e+02,   -1.6672e+02,   -3.1602e-02  ],# bin 97  chi2/NDF: 10.867
[ 2.4625,   -9.0413e+02,   -2.0363e+02,   -2.5898e-02  ],# bin 98  chi2/NDF: 14.179
[ 2.4875,   1.3385e+03,   -2.0434e+02,   -3.1009e-02  ],# bin 99  chi2/NDF: 2.8201
] # end CaloSwCalHitsEfront_slope_v9_gam_conv


CaloSwCalHitsEfront_slope_v9leakdata_gam_unconv = [
[ 0.0125,   1.0571e+00,   1.0074e-01,   2.8289e-03  ],# bin 0  chi2/NDF: 2.9279, copied from bin 2
[ 0.0375,   1.0571e+00,   1.0074e-01,   2.8289e-03  ],# bin 1  chi2/NDF: 1.5593, copied from bin 2
[ 0.0625,   1.0571e+00,   1.0074e-01,   2.8289e-03  ],# bin 2  chi2/NDF: 3.4057
[ 0.0875,   9.7664e-01,   1.5356e-01,   1.9533e-03  ],# bin 3  chi2/NDF: 10.066
[ 0.1125,   9.7089e-01,   1.7070e-01,   3.4641e-04  ],# bin 4  chi2/NDF: 6.7396
[ 0.1375,   8.1226e-01,   2.9346e-01,   -1.7218e-03  ],# bin 5  chi2/NDF: 1.9922
[ 0.1625,   7.7817e-01,   3.1696e-01,   -1.7445e-03  ],# bin 6  chi2/NDF: 0.47977
[ 0.1875,   7.7270e-01,   3.2920e-01,   -2.6625e-03  ],# bin 7  chi2/NDF: 2.5987
[ 0.2125,   7.5397e-01,   3.2280e-01,   -9.3580e-04  ],# bin 8  chi2/NDF: 1.5054
[ 0.2375,   7.5124e-01,   3.1952e-01,   2.6505e-04  ],# bin 9  chi2/NDF: 2.0055
[ 0.2625,   7.5561e-01,   3.2187e-01,   -5.9438e-04  ],# bin 10  chi2/NDF: 1.3137
[ 0.2875,   7.1522e-01,   3.6982e-01,   -2.5508e-03  ],# bin 11  chi2/NDF: 2.567
[ 0.3125,   7.5383e-01,   3.3520e-01,   -2.0882e-03  ],# bin 12  chi2/NDF: 2.1585
[ 0.3375,   8.0137e-01,   3.0369e-01,   -2.6348e-03  ],# bin 13  chi2/NDF: 2.8689
[ 0.3625,   7.7991e-01,   2.9987e-01,   8.9505e-04  ],# bin 14  chi2/NDF: 4.2006
[ 0.3875,   7.6499e-01,   3.1934e-01,   -8.2409e-04  ],# bin 15  chi2/NDF: 2.6275
[ 0.4125,   7.6287e-01,   2.9281e-01,   9.8600e-04  ],# bin 16  chi2/NDF: 2.1147
[ 0.4375,   7.4165e-01,   3.1060e-01,   1.1683e-03  ],# bin 17  chi2/NDF: 1.6988
[ 0.4625,   8.8792e-01,   1.8704e-01,   3.1579e-03  ],# bin 18  chi2/NDF: 9.1661
[ 0.4875,   1.1000e+00,   1.5363e-02,   8.5837e-03  ],# bin 19  chi2/NDF: 22.465
[ 0.5125,   9.8014e-01,   1.1062e-01,   4.4044e-03  ],# bin 20  chi2/NDF: 6.828
[ 0.5375,   1.0597e+00,   5.5590e-02,   5.9161e-03  ],# bin 21  chi2/NDF: 18.25
[ 0.5625,   1.0709e+00,   6.3605e-02,   3.9045e-03  ],# bin 22  chi2/NDF: 42.832
[ 0.5875,   1.1562e+00,   1.1462e-02,   5.0229e-03  ],# bin 23  chi2/NDF: 27.005
[ 0.6125,   1.0738e+00,   4.8892e-02,   6.7152e-03  ],# bin 24  chi2/NDF: 18.841
[ 0.6375,   1.2021e+00,   -2.0954e-02,   6.4177e-03  ],# bin 25  chi2/NDF: 15.815
[ 0.6625,   1.1414e+00,   3.2711e-02,   2.9019e-03  ],# bin 26  chi2/NDF: 7.6458
[ 0.6875,   1.0339e+00,   1.0875e-01,   1.4412e-03  ],# bin 27  chi2/NDF: 4.796
[ 0.7125,   1.0764e+00,   8.2113e-02,   1.5408e-03  ],# bin 28  chi2/NDF: 6.416
[ 0.7375,   1.0520e+00,   1.0386e-01,   1.3019e-03  ],# bin 29  chi2/NDF: 3.3686
[ 0.7625,   1.0520e+00,   1.0386e-01,   1.3019e-03  ],# bin 30  chi2/NDF: 7.3909, copied from bin 29
[ 0.7875,   1.0520e+00,   1.0386e-01,   1.3019e-03  ],# bin 31  chi2/NDF: 1.2942, copied from bin 29
[ 0.8125,   9.8984e-01,   1.5539e-01,   -7.3217e-06  ],# bin 32  chi2/NDF: 1.1388, copied from bin 34
[ 0.8375,   9.8984e-01,   1.5539e-01,   -7.3217e-06  ],# bin 33  chi2/NDF: 1.3464, copied from bin 34
[ 0.8625,   9.8984e-01,   1.5539e-01,   -7.3217e-06  ],# bin 34  chi2/NDF: 1.5912
[ 0.8875,   1.1011e+00,   6.9783e-02,   2.8450e-03  ],# bin 35  chi2/NDF: 3.6393
[ 0.9125,   9.8817e-01,   1.6212e-01,   -7.1556e-04  ],# bin 36  chi2/NDF: 4.2569
[ 0.9375,   1.0722e+00,   8.7005e-02,   2.5196e-03  ],# bin 37  chi2/NDF: 2.7081
[ 0.9625,   9.9372e-01,   1.5032e-01,   7.0535e-04  ],# bin 38  chi2/NDF: 1.7492
[ 0.9875,   9.8717e-01,   1.5936e-01,   -6.4269e-04  ],# bin 39  chi2/NDF: 3.4781
[ 1.0125,   1.0166e+00,   1.3428e-01,   6.3072e-04  ],# bin 40  chi2/NDF: 2.3627
[ 1.0375,   1.0110e+00,   1.3884e-01,   -1.1371e-04  ],# bin 41  chi2/NDF: 3.2968
[ 1.0625,   1.0672e+00,   1.0321e-01,   9.2087e-04  ],# bin 42  chi2/NDF: 3.7848
[ 1.0875,   1.0515e+00,   1.1063e-01,   1.2740e-03  ],# bin 43  chi2/NDF: 2.8235
[ 1.1125,   1.0764e+00,   9.3321e-02,   1.5066e-03  ],# bin 44  chi2/NDF: 5.0271
[ 1.1375,   9.8816e-01,   1.6309e-01,   1.8380e-04  ],# bin 45  chi2/NDF: 7.4601
[ 1.1625,   1.0033e+00,   1.4875e-01,   4.4373e-04  ],# bin 46  chi2/NDF: 2.8495
[ 1.1875,   9.9769e-01,   1.3775e-01,   7.0889e-04  ],# bin 47  chi2/NDF: 4.421
[ 1.2125,   1.0399e+00,   8.0461e-02,   1.9336e-03  ],# bin 48  chi2/NDF: 4.2641
[ 1.2375,   -8.9054e-03,   3.5829e+00,   3.3235e-01  ],# bin 49  chi2/NDF: 55.815
[ 1.2625,   8.7732e-01,   2.1087e-01,   -1.1877e-03  ],# bin 50  chi2/NDF: 0.33364
[ 1.2875,   9.4765e-01,   1.4722e-01,   8.2727e-04  ],# bin 51  chi2/NDF: 2.1872
[ 1.3125,   9.5889e-01,   1.5130e-01,   -2.8518e-04  ],# bin 52  chi2/NDF: 2.3997
[ 1.3375,   9.9088e-01,   1.2984e-01,   3.6455e-04  ],# bin 53  chi2/NDF: 3.0698
[ 1.3625,   9.2048e-01,   1.9515e-01,   -2.2262e-03  ],# bin 54  chi2/NDF: 1.0469
[ 1.3875,   9.2418e-01,   2.0727e-01,   -3.1415e-03  ],# bin 55  chi2/NDF: 11.789
[ 1.4125,   9.2291e-01,   2.1966e-01,   -5.7647e-03  ],# bin 56  chi2/NDF: 4.028
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -1.1694e+00,   9.9597e-02,   -1.8453e-01  ],# bin 60 crack  chi2/NDF: 45.213
[ 1.5375,   2.2416e+00,   -4.0168e-01,   4.7058e-02  ],# bin 61 crack  chi2/NDF: 11.528
[ 1.5625,   1.2964e+00,   -5.9345e-02,   2.0515e-03  ],# bin 62  chi2/NDF: 10.275
[ 1.5875,   1.4655e+00,   -8.3868e-02,   2.8267e-03  ],# bin 63  chi2/NDF: 6.9572
[ 1.6125,   1.5971e+00,   -1.3650e-01,   1.1713e-02  ],# bin 64  chi2/NDF: 8.0129
[ 1.6375,   1.7844e+00,   -1.9947e-01,   1.7263e-02  ],# bin 65  chi2/NDF: 5.1658
[ 1.6625,   1.6985e+00,   -1.9379e-01,   2.0910e-02  ],# bin 66  chi2/NDF: 4.4952
[ 1.6875,   2.1347e+00,   -3.1068e-01,   3.3934e-02  ],# bin 67  chi2/NDF: 3.5494
[ 1.7125,   1.4825e+00,   -1.3605e-01,   1.2024e-02  ],# bin 68  chi2/NDF: 5.5883
[ 1.7375,   8.8381e-01,   1.3945e-02,   -4.0274e-03  ],# bin 69  chi2/NDF: 9.3919
[ 1.7625,   9.2229e-01,   -1.5182e-02,   2.4693e-03  ],# bin 70  chi2/NDF: 4.2707
[ 1.7875,   9.6656e-01,   1.2359e-02,   -7.2592e-03  ],# bin 71  chi2/NDF: 8.5816
[ 1.8125,   -3.3359e+02,   -8.0892e+01,   -6.2467e-02  ],# bin 72  chi2/NDF: 6.0554
[ 1.8375,   -4.7900e+02,   -6.0125e+01,   -3.9227e-02  ],# bin 73  chi2/NDF: 6.5955
[ 1.8625,   -5.9440e+02,   -4.2210e+01,   -5.3324e-02  ],# bin 74  chi2/NDF: 3.4264
[ 1.8875,   -4.1775e+02,   -4.4563e+01,   -2.8976e-02  ],# bin 75  chi2/NDF: 2.1635
[ 1.9125,   -6.4347e+02,   -3.9865e+01,   -1.3448e-02  ],# bin 76  chi2/NDF: 3.7974
[ 1.9375,   -4.7477e+02,   -3.3770e+01,   -4.5229e-02  ],# bin 77  chi2/NDF: 3.5959
[ 1.9625,   -3.9941e+02,   -3.2109e+01,   -3.6064e-02  ],# bin 78  chi2/NDF: 3.8876
[ 1.9875,   -2.2379e+02,   -3.6270e+01,   -2.1101e-02  ],# bin 79  chi2/NDF: 1.9246
[ 2.0125,   -6.8616e+02,   -3.0239e+01,   -2.6274e-02  ],# bin 80  chi2/NDF: 4.2262
[ 2.0375,   -2.3572e+02,   -3.2010e+01,   -1.9135e-02  ],# bin 81  chi2/NDF: 2.8736
[ 2.0625,   -5.7419e+02,   -2.7260e+01,   -2.0928e-02  ],# bin 82  chi2/NDF: 9.7153
[ 2.0875,   -3.8196e+02,   -2.6239e+01,   -1.1949e-02  ],# bin 83  chi2/NDF: 9.1416
[ 2.1125,   -8.0684e+02,   -2.5054e+01,   -1.3560e-02  ],# bin 84  chi2/NDF: 12.19
[ 2.1375,   -4.0763e+02,   -2.8266e+01,   9.6856e-04  ],# bin 85  chi2/NDF: 3.4452
[ 2.1625,   -3.5788e+02,   -2.3543e+01,   -1.9748e-03  ],# bin 86  chi2/NDF: 4.5037
[ 2.1875,   -2.5566e+02,   -2.1081e+01,   -4.5409e-03  ],# bin 87  chi2/NDF: 12.759
[ 2.2125,   -3.3343e+02,   -1.6995e+01,   -1.0582e-02  ],# bin 88  chi2/NDF: 19.197
[ 2.2375,   -6.0194e+02,   -1.5163e+01,   -1.2860e-02  ],# bin 89  chi2/NDF: 17.036
[ 2.2625,   -5.7317e+02,   -1.4819e+01,   -3.4112e-03  ],# bin 90  chi2/NDF: 3.2238
[ 2.2875,   -1.6887e+02,   -1.5013e+01,   -4.6430e-03  ],# bin 91  chi2/NDF: 10.206
[ 2.3125,   -7.1012e+02,   -4.5752e+00,   -8.0697e-02  ],# bin 92  chi2/NDF: 20.811
[ 2.3375,   -1.0680e+03,   -8.6906e+00,   -4.9240e-01  ],# bin 93  chi2/NDF: 18.569
[ 2.3625,   -6.3897e+02,   -3.5097e+01,   -6.8842e-03  ],# bin 94  chi2/NDF: 9.2157
[ 2.3875,   -2.0436e+02,   -3.8260e+01,   1.4719e-02  ],# bin 95  chi2/NDF: 4.0514
[ 2.4125,   -7.8792e+02,   -2.9966e+01,   -4.4648e-03  ],# bin 96  chi2/NDF: 9.1533
[ 2.4375,   -7.2920e+02,   -2.4246e+01,   -1.3917e-02  ],# bin 97  chi2/NDF: 3.3668
[ 2.4625,   -7.3052e+02,   -2.5254e+01,   -7.9504e-03  ],# bin 98  chi2/NDF: 8.5938
[ 2.4875,   -5.8134e+02,   -3.0015e+01,   5.9546e-04  ],# bin 99  chi2/NDF: 32.343
] # end CaloSwCalHitsEfront_slope_v9_gam_unconv


CaloSwCalHitsOutOfCone_v9leakdata_ele35 = [
[ 0.0125, 1.148100e+01,    -8.1450e-01,    2.4305e-02  ], # bin 0  chi2/NDF: 61.565  copied from bin 2  
[ 0.0375, 1.148100e+01,    -8.1450e-01,    2.4305e-02  ], # bin 1  chi2/NDF: 152.21  copied from bin 2  
[ 0.0625, 1.148100e+01,    -8.1450e-01,    2.4305e-02  ], # bin 2  chi2/NDF: 135.98  
[ 0.0875, 1.171800e+01,    -8.3758e-01,    2.4968e-02  ], # bin 3  chi2/NDF: 134.13  
[ 0.1125, 1.170200e+01,    -8.2199e-01,    2.3876e-02  ], # bin 4  chi2/NDF: 185.74  
[ 0.1375, 1.165100e+01,    -8.1610e-01,    2.3912e-02  ], # bin 5  chi2/NDF: 125.92  
[ 0.1625, 1.166200e+01,    -8.1075e-01,    2.3387e-02  ], # bin 6  chi2/NDF: 109.89  
[ 0.1875, 1.173400e+01,    -8.2014e-01,    2.3860e-02  ], # bin 7  chi2/NDF: 155.06  
[ 0.2125, 1.158000e+01,    -8.0468e-01,    2.3643e-02  ], # bin 8  chi2/NDF: 168.29  
[ 0.2375, 1.174700e+01,    -8.3155e-01,    2.4912e-02  ], # bin 9  chi2/NDF: 77.802  
[ 0.2625, 1.099200e+01,    -6.9660e-01,    1.8946e-02  ], # bin 10  chi2/NDF: 131.06  
[ 0.2875, 1.168200e+01,    -8.1050e-01,    2.3909e-02  ], # bin 11  chi2/NDF: 129.71  
[ 0.3125, 1.103500e+01,    -6.9039e-01,    1.8539e-02  ], # bin 12  chi2/NDF: 168.09  
[ 0.3375, 1.136500e+01,    -7.4589e-01,    2.1156e-02  ], # bin 13  chi2/NDF: 167.17  
[ 0.3625, 1.139600e+01,    -7.4368e-01,    2.0886e-02  ], # bin 14  chi2/NDF: 151.18  
[ 0.3875, 1.272300e+01,    -9.6430e-01,    3.0232e-02  ], # bin 15  chi2/NDF: 146.48  
[ 0.4125, 1.148500e+01,    -7.3399e-01,    1.9889e-02  ], # bin 16  chi2/NDF: 132.1  
[ 0.4375, 1.140000e+01,    -7.2003e-01,    1.9751e-02  ], # bin 17  chi2/NDF: 133.81  
[ 0.4625, 1.147100e+01,    -7.2843e-01,    2.0173e-02  ], # bin 18  chi2/NDF: 175.94  
[ 0.4875, 1.139000e+01,    -7.1047e-01,    1.9598e-02  ], # bin 19  chi2/NDF: 104.48  
[ 0.5125, 1.120800e+01,    -6.6911e-01,    1.7749e-02  ], # bin 20  chi2/NDF: 122.13  
[ 0.5375, 1.246500e+01,    -8.7913e-01,    2.6546e-02  ], # bin 21  chi2/NDF: 72.506  
[ 0.5625, 1.244300e+01,    -8.7114e-01,    2.6033e-02  ], # bin 22  chi2/NDF: 96.922  
[ 0.5875, 1.303000e+01,    -9.6054e-01,    2.9742e-02  ], # bin 23  chi2/NDF: 125.86  
[ 0.6125, 1.250700e+01,    -8.6756e-01,    2.5564e-02  ], # bin 24  chi2/NDF: 152.22  
[ 0.6375, 1.384100e+01,    -1.0883e+00,    3.5040e-02  ], # bin 25  chi2/NDF: 81.878  
[ 0.6625, 1.459800e+01,    -1.1725e+00,    3.6921e-02  ], # bin 26  chi2/NDF: 125.61  
[ 0.6875, 1.644400e+01,    -1.4667e+00,    4.9012e-02  ], # bin 27  chi2/NDF: 58.138  
[ 0.7125, 1.553500e+01,    -1.2970e+00,    4.1042e-02  ], # bin 28  chi2/NDF: 104.1  
[ 0.7375, 1.640300e+01,    -1.4438e+00,    4.7569e-02  ], # bin 29  chi2/NDF: 67.792  
[ 0.7625, 1.640300e+01,    -1.4438e+00,    4.7569e-02  ], # bin 30  chi2/NDF: 132.14  copied from bin 29  
[ 0.7875, 1.640300e+01,    -1.4438e+00,    4.7569e-02  ], # bin 31  chi2/NDF: 107.18  copied from bin 29  
[ 0.8125, 3.512000e+01,    -3.4585e+00,    1.0769e-01  ], # bin 32  chi2/NDF: 74.916  copied from bin 34  
[ 0.8375, 3.512000e+01,    -3.4585e+00,    1.0769e-01  ], # bin 33  chi2/NDF: 52.674  copied from bin 34  
[ 0.8625, 3.512000e+01,    -3.4585e+00,    1.0769e-01  ], # bin 34  chi2/NDF: 45.832  
[ 0.8875, 3.394600e+01,    -3.3039e+00,    1.0277e-01  ], # bin 35  chi2/NDF: 32.972  
[ 0.9125, 3.276600e+01,    -3.1270e+00,    9.5780e-02  ], # bin 36  chi2/NDF: 196.59  
[ 0.9375, 3.459200e+01,    -3.4393e+00,    1.0922e-01  ], # bin 37  chi2/NDF: 56.503  
[ 0.9625, 3.460600e+01,    -3.4559e+00,    1.1011e-01  ], # bin 38  chi2/NDF: 70.179  
[ 0.9875, 3.495000e+01,    -3.5402e+00,    1.1480e-01  ], # bin 39  chi2/NDF: 56.118  
[ 1.0125, 3.647300e+01,    -3.7955e+00,    1.2507e-01  ], # bin 40  chi2/NDF: 83.373  
[ 1.0375, 3.591300e+01,    -3.7341e+00,    1.2375e-01  ], # bin 41  chi2/NDF: 107.38  
[ 1.0625, 3.663800e+01,    -3.8955e+00,    1.3129e-01  ], # bin 42  chi2/NDF: 73.27  
[ 1.0875, 3.528900e+01,    -3.7112e+00,    1.2548e-01  ], # bin 43  chi2/NDF: 151.43  
[ 1.1125, 3.701500e+01,    -4.0806e+00,    1.4324e-01  ], # bin 44  chi2/NDF: 123.85  
[ 1.1375, 3.495700e+01,    -3.7562e+00,    1.3062e-01  ], # bin 45  chi2/NDF: 123.75  
[ 1.1625, 3.629300e+01,    -3.9069e+00,    1.3411e-01  ], # bin 46  chi2/NDF: 142.67  
[ 1.1875, 3.765000e+01,    -4.0885e+00,    1.4104e-01  ], # bin 47  chi2/NDF: 170.65  
[ 1.2125, 3.893400e+01,    -4.2477e+00,    1.4534e-01  ], # bin 48  chi2/NDF: 191.6  
[ 1.2375, 4.109000e+01,    -4.5674e+00,    1.5805e-01  ], # bin 49  chi2/NDF: 137.14  
[ 1.2625, 4.345200e+01,    -4.8881e+00,    1.6824e-01  ], # bin 50  chi2/NDF: 203.82  
[ 1.2875, 4.624600e+01,    -5.2874e+00,    1.8350e-01  ], # bin 51  chi2/NDF: 165.07  
[ 1.3125, 5.167200e+01,    -6.0193e+00,    2.0704e-01  ], # bin 52  chi2/NDF: 452.77  
[ 1.3375, 5.668700e+01,    -6.7706e+00,    2.3529e-01  ], # bin 53  chi2/NDF: 195.8  
[ 1.3625, 6.008500e+01,    -7.2227e+00,    2.4740e-01  ], # bin 54  chi2/NDF: 204.44  
[ 1.3875, 6.947500e+01,    -7.9890e+00,    2.6995e-01  ], # bin 55  chi2/NDF: 109.82  
[ 1.4125, 6.186400e+01,    -7.3766e+00,    2.4484e-01  ], # bin 56  chi2/NDF: 79.236  
[ 1.4375, 1.100000e+00,    0.0000e+00,    0.0000e+00  ], # bin 57 crack  chi2/NDF: 0  
[ 1.4625, 1.100000e+00,    0.0000e+00,    0.0000e+00  ], # bin 58 crack  chi2/NDF: 0  
[ 1.4875, 1.600000e+00,    0.0000e+00,    0.0000e+00  ], # bin 59 crack  chi2/NDF: 0  
[ 1.5125, -2.519800e+01,    7.0928e-01,    3.7149e+02  ], # bin 60 crack  chi2/NDF: 659.74  
[ 1.5375, -3.690400e+01,    1.2080e+00,    4.0641e+02  ], # bin 61 crack  chi2/NDF: 38.953  
[ 1.5625, -3.011900e+01,    9.1543e-01,    3.7924e+02  ], # bin 62  chi2/NDF: 22.519  
[ 1.5875, -2.972000e+01,    1.0057e+00,    3.4278e+02  ], # bin 63  chi2/NDF: 20.216  
[ 1.6125, -2.508300e+01,    8.6521e-01,    3.0681e+02  ], # bin 64  chi2/NDF: 301.35  
[ 1.6375, -2.554700e+01,    9.1734e-01,    3.0404e+02  ], # bin 65  chi2/NDF: 30.872  
[ 1.6625, -2.576600e+01,    9.2398e-01,    3.1131e+02  ], # bin 66  chi2/NDF: 20.523  
[ 1.6875, -3.040400e+01,    1.0783e+00,    3.5056e+02  ], # bin 67  chi2/NDF: 35.592  
[ 1.7125, -2.792900e+01,    9.9289e-01,    3.3663e+02  ], # bin 68  chi2/NDF: 112.08  
[ 1.7375, -1.916100e+01,    7.8099e-01,    2.4943e+02  ], # bin 69  chi2/NDF: 92.567  
[ 1.7625, -1.752200e+01,    7.6033e-01,    2.3124e+02  ], # bin 70  chi2/NDF: 74.662  
[ 1.7875, -1.241600e+00,    2.1959e-01,    1.1123e+02  ], # bin 71  chi2/NDF: 60.471  
[ 1.8125, 9.440400e+00,    -1.7176e-01,    4.3454e+01  ], # bin 72  chi2/NDF: 21.662  
[ 1.8375, 6.866100e+00,    -9.3557e-02,    6.5567e+01  ], # bin 73  chi2/NDF: 30.318  
[ 1.8625, 1.256400e+01,    -2.9158e-01,    2.5853e+01  ], # bin 74  chi2/NDF: 37.886  
[ 1.8875, 6.954000e+00,    -5.9195e-02,    6.1585e+01  ], # bin 75  chi2/NDF: 36.894  
[ 1.9125, 8.009600e+00,    -8.2323e-02,    5.3832e+01  ], # bin 76  chi2/NDF: 10.095  
[ 1.9375, 1.126650e+01,    -1.6994e-01,    2.8862e+01  ], # bin 77  chi2/NDF: 13.18  
[ 1.9625, 1.043870e+01,    -1.2346e-01,    3.4526e+01  ], # bin 78  chi2/NDF: 20.386  
[ 1.9875, 8.655400e+00,    -4.4162e-02,    4.7713e+01  ], # bin 79  chi2/NDF: 10.862  
[ 2.0125, 8.653200e+00,    -3.7753e-02,    5.0509e+01  ], # bin 80  chi2/NDF: 21.671  
[ 2.0375, 7.478000e+00,    5.3614e-03,    6.2674e+01  ], # bin 81  chi2/NDF: 13.315  
[ 2.0625, 6.371300e+00,    5.1076e-02,    7.3135e+01  ], # bin 82  chi2/NDF: 10.152  
[ 2.0875, 7.952300e+00,    1.7370e-02,    6.1737e+01  ], # bin 83  chi2/NDF: 15.437  
[ 2.1125, 6.755100e+00,    5.7833e-02,    7.1803e+01  ], # bin 84  chi2/NDF: 22.266  
[ 2.1375, 5.205600e+00,    1.4325e-01,    8.0250e+01  ], # bin 85  chi2/NDF: 18.729  
[ 2.1625, 6.139500e+00,    1.2590e-01,    7.3625e+01  ], # bin 86  chi2/NDF: 25.381  
[ 2.1875, 4.923900e+00,    1.8682e-01,    8.2395e+01  ], # bin 87  chi2/NDF: 19.288  
[ 2.2125, 8.967900e+00,    4.4131e-02,    5.6872e+01  ], # bin 88  chi2/NDF: 40.624  
[ 2.2375, 9.309600e+00,    5.9547e-02,    5.2162e+01  ], # bin 89  chi2/NDF: 7.9254  
[ 2.2625, 1.340100e+01,    -9.3826e-02,    2.8334e+01  ], # bin 90  chi2/NDF: 15.169  
[ 2.2875, 1.341600e+01,    -9.8338e-02,    3.4458e+01  ], # bin 91  chi2/NDF: 13.37  
[ 2.3125, 2.206900e+01,    -3.9645e-01,    -2.0150e+01  ], # bin 92  chi2/NDF: 19.138  
[ 2.3375, 1.830100e+01,    -3.5825e-01,    3.0765e+01  ], # bin 93  chi2/NDF: 88.35  
[ 2.3625, 9.751000e+00,    -1.0350e-01,    1.0731e+02  ], # bin 94  chi2/NDF: 67.624  
[ 2.3875, 2.569200e+01,    -7.7085e-01,    1.7651e+01  ], # bin 95  chi2/NDF: 379.48  
[ 2.4125, -9.329000e+00,    4.8118e-01,    2.6780e+02  ], # bin 96  chi2/NDF: 69.855  
[ 2.4375, -3.120100e+01,    1.2008e+00,    4.3774e+02  ], # bin 97  chi2/NDF: 44.419  
[ 2.4625, -9.361600e+01,    3.3453e+00,    9.0459e+02  ], # bin 98  chi2/NDF: 146.98  
[ 2.4875, -3.553900e+02,    1.2010e+01,    2.8878e+03  ], # bin 99  chi2/NDF: 479.24  
] # end CaloSwCalHitsOutOfCone_v9_ele35


CaloSwCalHitsOutOfCone_v9leakdata_gam35_conv = [
[ 0.0125, 1.733800e+01,    -1.6691e+00,    5.5225e-02  ], # bin 0  chi2/NDF: 22.719  copied from bin 2  
[ 0.0375, 1.733800e+01,    -1.6691e+00,    5.5225e-02  ], # bin 1  chi2/NDF: 31.924  copied from bin 2  
[ 0.0625, 1.733800e+01,    -1.6691e+00,    5.5225e-02  ], # bin 2  chi2/NDF: 39.772  
[ 0.0875, 1.721900e+01,    -1.6388e+00,    5.3932e-02  ], # bin 3  chi2/NDF: 36.651  
[ 0.1125, 1.809500e+01,    -1.7879e+00,    6.0345e-02  ], # bin 4  chi2/NDF: 44.599  
[ 0.1375, 1.529600e+01,    -1.2919e+00,    3.8625e-02  ], # bin 5  chi2/NDF: 32.578  
[ 0.1625, 1.518400e+01,    -1.2710e+00,    3.7768e-02  ], # bin 6  chi2/NDF: 24.21  
[ 0.1875, 1.565100e+01,    -1.3655e+00,    4.2501e-02  ], # bin 7  chi2/NDF: 145.14  
[ 0.2125, 1.528600e+01,    -1.2695e+00,    3.7167e-02  ], # bin 8  chi2/NDF: 131.49  
[ 0.2375, 1.748000e+01,    -1.6660e+00,    5.5137e-02  ], # bin 9  chi2/NDF: 67.92  
[ 0.2625, 1.660200e+01,    -1.5079e+00,    4.8124e-02  ], # bin 10  chi2/NDF: 34.722  
[ 0.2875, 1.485400e+01,    -1.2108e+00,    3.5826e-02  ], # bin 11  chi2/NDF: 58  
[ 0.3125, 1.455900e+01,    -1.1525e+00,    3.3260e-02  ], # bin 12  chi2/NDF: 41.624  
[ 0.3375, 1.487600e+01,    -1.1878e+00,    3.4399e-02  ], # bin 13  chi2/NDF: 53.653  
[ 0.3625, 1.561500e+01,    -1.3046e+00,    3.9125e-02  ], # bin 14  chi2/NDF: 50.453  
[ 0.3875, 1.555600e+01,    -1.2966e+00,    3.9215e-02  ], # bin 15  chi2/NDF: 82.615  
[ 0.4125, 1.564100e+01,    -1.2914e+00,    3.8529e-02  ], # bin 16  chi2/NDF: 74.597  
[ 0.4375, 1.602900e+01,    -1.3313e+00,    3.9552e-02  ], # bin 17  chi2/NDF: 46.062  
[ 0.4625, 1.649800e+01,    -1.4090e+00,    4.3071e-02  ], # bin 18  chi2/NDF: 47.711  
[ 0.4875, 1.422500e+01,    -1.0241e+00,    2.7354e-02  ], # bin 19  chi2/NDF: 45.299  
[ 0.5125, 1.808800e+01,    -1.6211e+00,    5.0626e-02  ], # bin 20  chi2/NDF: 24.063  
[ 0.5375, 1.509000e+01,    -1.1489e+00,    3.2336e-02  ], # bin 21  chi2/NDF: 47.516  
[ 0.5625, 1.546700e+01,    -1.2125e+00,    3.4945e-02  ], # bin 22  chi2/NDF: 11.803  
[ 0.5875, 1.592200e+01,    -1.2625e+00,    3.6500e-02  ], # bin 23  chi2/NDF: 65.201  
[ 0.6125, 1.839600e+01,    -1.6653e+00,    5.2575e-02  ], # bin 24  chi2/NDF: 63.542  
[ 0.6375, 1.949700e+01,    -1.8457e+00,    6.0088e-02  ], # bin 25  chi2/NDF: 42.717  
[ 0.6625, 2.113400e+01,    -2.0770e+00,    6.8030e-02  ], # bin 26  chi2/NDF: 33.836  
[ 0.6875, 1.978300e+01,    -1.8695e+00,    6.0475e-02  ], # bin 27  chi2/NDF: 91.85  
[ 0.7125, 2.162500e+01,    -2.1582e+00,    7.1374e-02  ], # bin 28  chi2/NDF: 54.626  
[ 0.7375, 2.404600e+01,    -2.5732e+00,    8.9334e-02  ], # bin 29  chi2/NDF: 49.231  
[ 0.7625, 2.404600e+01,    -2.5732e+00,    8.9334e-02  ], # bin 30  chi2/NDF: 62.74  copied from bin 29  
[ 0.7875, 2.404600e+01,    -2.5732e+00,    8.9334e-02  ], # bin 31  chi2/NDF: 91.045  copied from bin 29  
[ 0.8125, 4.198100e+01,    -4.4250e+00,    1.4134e-01  ], # bin 32  chi2/NDF: 58.939  copied from bin 34  
[ 0.8375, 4.198100e+01,    -4.4250e+00,    1.4134e-01  ], # bin 33  chi2/NDF: 44.395  copied from bin 34  
[ 0.8625, 4.198100e+01,    -4.4250e+00,    1.4134e-01  ], # bin 34  chi2/NDF: 66.211  
[ 0.8875, 4.070600e+01,    -4.2560e+00,    1.3588e-01  ], # bin 35  chi2/NDF: 305.78  
[ 0.9125, 3.790600e+01,    -3.9055e+00,    1.2511e-01  ], # bin 36  chi2/NDF: 104.8  
[ 0.9375, 3.835000e+01,    -3.9927e+00,    1.2918e-01  ], # bin 37  chi2/NDF: 20.517  
[ 0.9625, 3.788700e+01,    -3.9310e+00,    1.2692e-01  ], # bin 38  chi2/NDF: 41.703  
[ 0.9875, 3.810600e+01,    -3.9936e+00,    1.3076e-01  ], # bin 39  chi2/NDF: 184.92  
[ 1.0125, 3.846100e+01,    -4.0699e+00,    1.3423e-01  ], # bin 40  chi2/NDF: 50.141  
[ 1.0375, 3.893300e+01,    -4.1589e+00,    1.3836e-01  ], # bin 41  chi2/NDF: 61.284  
[ 1.0625, 3.849200e+01,    -4.1235e+00,    1.3779e-01  ], # bin 42  chi2/NDF: 28.34  
[ 1.0875, 3.919400e+01,    -4.2692e+00,    1.4516e-01  ], # bin 43  chi2/NDF: 75.351  
[ 1.1125, 3.673600e+01,    -3.9140e+00,    1.3199e-01  ], # bin 44  chi2/NDF: 63.187  
[ 1.1375, 3.817100e+01,    -4.1627e+00,    1.4298e-01  ], # bin 45  chi2/NDF: 38.299  
[ 1.1625, 4.046200e+01,    -4.4634e+00,    1.5224e-01  ], # bin 46  chi2/NDF: 51.585  
[ 1.1875, 4.256000e+01,    -4.8157e+00,    1.6783e-01  ], # bin 47  chi2/NDF: 66.447  
[ 1.2125, 4.522500e+01,    -5.1885e+00,    1.8051e-01  ], # bin 48  chi2/NDF: 46.6  
[ 1.2375, 4.679100e+01,    -5.4101e+00,    1.8922e-01  ], # bin 49  chi2/NDF: 60.745  
[ 1.2625, 4.707000e+01,    -5.3610e+00,    1.8327e-01  ], # bin 50  chi2/NDF: 18.413  
[ 1.2875, 5.048500e+01,    -5.8764e+00,    2.0385e-01  ], # bin 51  chi2/NDF: 36.128  
[ 1.3125, 5.602300e+01,    -6.6365e+00,    2.2884e-01  ], # bin 52  chi2/NDF: 22.373  
[ 1.3375, 6.057500e+01,    -7.3095e+00,    2.5378e-01  ], # bin 53  chi2/NDF: 18.732  
[ 1.3625, 6.007100e+01,    -7.1302e+00,    2.4032e-01  ], # bin 54  chi2/NDF: 31.066  
[ 1.3875, 8.482600e+01,    -1.0367e+01,    3.6169e-01  ], # bin 55  chi2/NDF: 103.04  
[ 1.4125, 7.268000e+01,    -9.3645e+00,    3.3415e-01  ], # bin 56  chi2/NDF: 52.958  
[ 1.4375, 1.100000e+00,    0.0000e+00,    0.0000e+00  ], # bin 57 crack  chi2/NDF: 0  
[ 1.4625, 1.100000e+00,    0.0000e+00,    0.0000e+00  ], # bin 58 crack  chi2/NDF: 0  
[ 1.4875, 1.600000e+00,    0.0000e+00,    0.0000e+00  ], # bin 59 crack  chi2/NDF: 0  
[ 1.5125, -4.240100e+00,    -5.6488e-02,    2.3929e+02  ], # bin 60 crack  chi2/NDF: 556.55  
[ 1.5375, -4.004900e+01,    1.3044e+00,    4.3101e+02  ], # bin 61 crack  chi2/NDF: 43.606  
[ 1.5625, -2.655000e+01,    7.9559e-01,    3.5141e+02  ], # bin 62  chi2/NDF: 32.202  
[ 1.5875, -3.208200e+01,    1.0924e+00,    3.5703e+02  ], # bin 63  chi2/NDF: 33.667  
[ 1.6125, -2.183800e+01,    7.6306e-01,    2.7964e+02  ], # bin 64  chi2/NDF: 32.611  
[ 1.6375, -2.376200e+01,    8.6379e-01,    2.8762e+02  ], # bin 65  chi2/NDF: 17.777  
[ 1.6625, -2.381900e+01,    8.6987e-01,    2.9233e+02  ], # bin 66  chi2/NDF: 11.884  
[ 1.6875, -2.626900e+01,    9.3758e-01,    3.1887e+02  ], # bin 67  chi2/NDF: 15.104  
[ 1.7125, -2.155400e+01,    7.8472e-01,    2.8603e+02  ], # bin 68  chi2/NDF: 64.933  
[ 1.7375, -1.556100e+01,    6.6155e-01,    2.2064e+02  ], # bin 69  chi2/NDF: 25.814  
[ 1.7625, -9.746000e+00,    4.9078e-01,    1.7333e+02  ], # bin 70  chi2/NDF: 54.271  
[ 1.7875, 2.355930e+00,    1.0242e-01,    8.1812e+01  ], # bin 71  chi2/NDF: 15.464  
[ 1.8125, 1.233600e+01,    -2.3425e-01,    1.1527e+01  ], # bin 72  chi2/NDF: 15.762  
[ 1.8375, 1.011950e+01,    -1.6366e-01,    3.0551e+01  ], # bin 73  chi2/NDF: 14.827  
[ 1.8625, 1.596000e+01,    -3.6602e-01,    -1.0053e+01  ], # bin 74  chi2/NDF: 12.458  
[ 1.8875, 1.389400e+01,    -2.8047e-01,    5.0822e+00  ], # bin 75  chi2/NDF: 8.3354  
[ 1.9125, 1.753500e+01,    -3.8967e-01,    -2.1588e+01  ], # bin 76  chi2/NDF: 42.376  
[ 1.9375, 1.662000e+01,    -3.4410e-01,    -1.4380e+01  ], # bin 77  chi2/NDF: 37.068  
[ 1.9625, 1.452100e+01,    -2.4845e-01,    -2.2502e-01  ], # bin 78  chi2/NDF: 19.191  
[ 1.9875, 1.109870e+01,    -1.0815e-01,    2.3580e+01  ], # bin 79  chi2/NDF: 32.239  
[ 2.0125, 1.505000e+01,    -2.3149e-01,    -4.0561e+00  ], # bin 80  chi2/NDF: 13.308  
[ 2.0375, 1.245000e+01,    -1.3022e-01,    1.6532e+01  ], # bin 81  chi2/NDF: 37.92  
[ 2.0625, 1.076880e+01,    -6.4253e-02,    3.0844e+01  ], # bin 82  chi2/NDF: 3.9405  
[ 2.0875, 6.739500e+00,    9.2159e-02,    6.0859e+01  ], # bin 83  chi2/NDF: 29.507  
[ 2.1125, 1.250390e+00,    2.9663e-01,    9.9301e+01  ], # bin 84  chi2/NDF: 42.313  
[ 2.1375, -1.323200e+00,    4.0117e-01,    1.1846e+02  ], # bin 85  chi2/NDF: 71.626  
[ 2.1625, -3.044200e+00,    4.6817e-01,    1.3274e+02  ], # bin 86  chi2/NDF: 23.61  
[ 2.1875, -2.694600e+00,    4.7535e-01,    1.2992e+02  ], # bin 87  chi2/NDF: 82.982  
[ 2.2125, -2.013900e+00,    4.4919e-01,    1.2892e+02  ], # bin 88  chi2/NDF: 11.438  
[ 2.2375, -4.662000e-01,    4.2302e-01,    1.1525e+02  ], # bin 89  chi2/NDF: 26.412  
[ 2.2625, 9.342700e+00,    4.8532e-02,    5.5243e+01  ], # bin 90  chi2/NDF: 27.663  
[ 2.2875, 1.946100e+00,    3.2213e-01,    1.0963e+02  ], # bin 91  chi2/NDF: 41.437  
[ 2.3125, 2.319960e+00,    2.9513e-01,    1.1633e+02  ], # bin 92  chi2/NDF: 20.686  
[ 2.3375, -7.204500e+00,    6.0922e-01,    1.9073e+02  ], # bin 93  chi2/NDF: 30.829  
[ 2.3625, 9.740000e-02,    3.0756e-01,    1.5516e+02  ], # bin 94  chi2/NDF: 32.34  
[ 2.3875, -9.489000e+00,    6.5738e-01,    2.2148e+02  ], # bin 95  chi2/NDF: 148.97  
[ 2.4125, -8.127200e+00,    5.5712e-01,    2.2755e+02  ], # bin 96  chi2/NDF: 65.593  
[ 2.4375, -2.355400e+01,    1.0955e+00,    3.4161e+02  ], # bin 97  chi2/NDF: 74.291  
[ 2.4625, -7.115800e+01,    2.7299e+00,    6.9488e+02  ], # bin 98  chi2/NDF: 335.89  
[ 2.4875, -2.207600e+02,    7.7441e+00,    1.8194e+03  ], # bin 99  chi2/NDF: 1133.1  
] # end CaloSwCalHitsOutOfCone_v9_gam35_conv


CaloSwCalHitsOutOfCone_v9leakdata_gam35_unconv = [
[ 0.0125, 1.215350e+00,    9.0633e-01,    -4.7279e-02  ], # bin 0  chi2/NDF: 49.623  copied from bin 2  
[ 0.0375, 1.215350e+00,    9.0633e-01,    -4.7279e-02  ], # bin 1  chi2/NDF: 99.022  copied from bin 2  
[ 0.0625, 1.215350e+00,    9.0633e-01,    -4.7279e-02  ], # bin 2  chi2/NDF: 133.84  
[ 0.0875, 1.553800e+00,    8.4663e-01,    -4.4383e-02  ], # bin 3  chi2/NDF: 138.92  
[ 0.1125, 1.315700e+00,    8.8886e-01,    -4.6100e-02  ], # bin 4  chi2/NDF: 72.481  
[ 0.1375, 1.199690e+00,    9.1076e-01,    -4.6939e-02  ], # bin 5  chi2/NDF: 57.724  
[ 0.1625, 1.815600e+00,    8.0088e-01,    -4.2117e-02  ], # bin 6  chi2/NDF: 35.787  
[ 0.1875, 1.955400e+00,    7.7551e-01,    -4.0811e-02  ], # bin 7  chi2/NDF: 64.63  
[ 0.2125, 2.337600e+00,    7.0663e-01,    -3.7836e-02  ], # bin 8  chi2/NDF: 70.373  
[ 0.2375, 2.075200e+00,    7.4275e-01,    -3.8785e-02  ], # bin 9  chi2/NDF: 68.016  
[ 0.2625, 1.875200e+00,    7.6731e-01,    -3.9453e-02  ], # bin 10  chi2/NDF: 36.723  
[ 0.2875, 2.579300e+00,    6.4284e-01,    -3.3825e-02  ], # bin 11  chi2/NDF: 60.294  
[ 0.3125, 2.184600e+00,    7.1357e-01,    -3.6829e-02  ], # bin 12  chi2/NDF: 81.629  
[ 0.3375, 1.504800e+00,    8.2751e-01,    -4.1164e-02  ], # bin 13  chi2/NDF: 53.364  
[ 0.3625, 2.356600e+00,    6.6932e-01,    -3.4110e-02  ], # bin 14  chi2/NDF: 62.383  
[ 0.3875, 8.784000e-01,    9.0729e-01,    -4.3145e-02  ], # bin 15  chi2/NDF: 36.125  
[ 0.4125, 1.598000e+00,    7.8577e-01,    -3.7985e-02  ], # bin 16  chi2/NDF: 50.357  
[ 0.4375, 2.388200e+00,    6.4689e-01,    -3.1821e-02  ], # bin 17  chi2/NDF: 158.02  
[ 0.4625, 2.403900e+00,    6.4576e-01,    -3.1551e-02  ], # bin 18  chi2/NDF: 91.059  
[ 0.4875, 3.217500e+00,    5.1098e-01,    -2.5805e-02  ], # bin 19  chi2/NDF: 58.92  
[ 0.5125, 2.571000e+00,    5.9891e-01,    -2.8515e-02  ], # bin 20  chi2/NDF: 79.868  
[ 0.5375, 3.296000e+00,    4.7228e-01,    -2.3102e-02  ], # bin 21  chi2/NDF: 111.93  
[ 0.5625, 2.146100e+00,    6.6079e-01,    -3.0660e-02  ], # bin 22  chi2/NDF: 88.802  
[ 0.5875, 3.480500e+00,    4.5933e-01,    -2.2869e-02  ], # bin 23  chi2/NDF: 73.001  
[ 0.6125, 4.750300e+00,    2.6708e-01,    -1.5777e-02  ], # bin 24  chi2/NDF: 192.31  
[ 0.6375, 5.515600e+00,    1.5034e-01,    -1.1090e-02  ], # bin 25  chi2/NDF: 84.032  
[ 0.6625, 5.495600e+00,    1.7002e-01,    -1.2569e-02  ], # bin 26  chi2/NDF: 191.74  
[ 0.6875, 6.375600e+00,    3.3505e-02,    -7.1391e-03  ], # bin 27  chi2/NDF: 134  
[ 0.7125, 6.711800e+00,    -2.3390e-02,    -5.0697e-03  ], # bin 28  chi2/NDF: 94.261  
[ 0.7375, 7.677500e+00,    -1.7936e-01,    1.4470e-03  ], # bin 29  chi2/NDF: 75.618  
[ 0.7625, 7.677500e+00,    -1.7936e-01,    1.4470e-03  ], # bin 30  chi2/NDF: 112.45  copied from bin 29  
[ 0.7875, 7.677500e+00,    -1.7936e-01,    1.4470e-03  ], # bin 31  chi2/NDF: 57.446  copied from bin 29  
[ 0.8125, 1.048640e+01,    -1.0490e-01,    -7.0597e-03  ], # bin 32  chi2/NDF: 47.762  copied from bin 34  
[ 0.8375, 1.048640e+01,    -1.0490e-01,    -7.0597e-03  ], # bin 33  chi2/NDF: 32.209  copied from bin 34  
[ 0.8625, 1.048640e+01,    -1.0490e-01,    -7.0597e-03  ], # bin 34  chi2/NDF: 109.13  
[ 0.8875, 9.495500e+00,    2.3557e-02,    -1.1050e-02  ], # bin 35  chi2/NDF: 49.31  
[ 0.9125, 9.291000e+00,    5.3980e-02,    -1.2289e-02  ], # bin 36  chi2/NDF: 48.1  
[ 0.9375, 9.562700e+00,    6.3031e-04,    -9.6893e-03  ], # bin 37  chi2/NDF: 131.8  
[ 0.9625, 1.093060e+01,    -2.1650e-01,    -1.4055e-03  ], # bin 38  chi2/NDF: 34.434  
[ 0.9875, 1.178100e+01,    -3.4785e-01,    3.9695e-03  ], # bin 39  chi2/NDF: 43.056  
[ 1.0125, 1.221100e+01,    -3.9613e-01,    5.0841e-03  ], # bin 40  chi2/NDF: 33.328  
[ 1.0375, 1.208700e+01,    -3.9547e-01,    5.9580e-03  ], # bin 41  chi2/NDF: 53.855  
[ 1.0625, 1.422500e+01,    -7.0614e-01,    1.6786e-02  ], # bin 42  chi2/NDF: 6.9722  
[ 1.0875, 1.418700e+01,    -7.2212e-01,    1.8655e-02  ], # bin 43  chi2/NDF: 59.494  
[ 1.1125, 1.323700e+01,    -5.5947e-01,    1.1630e-02  ], # bin 44  chi2/NDF: 17.979  
[ 1.1375, 1.557000e+01,    -9.2630e-01,    2.6327e-02  ], # bin 45  chi2/NDF: 78.643  
[ 1.1625, 1.559600e+01,    -8.9395e-01,    2.3382e-02  ], # bin 46  chi2/NDF: 134.82  
[ 1.1875, 1.774100e+01,    -1.2232e+00,    3.6705e-02  ], # bin 47  chi2/NDF: 147.73  
[ 1.2125, 1.723800e+01,    -1.1009e+00,    3.0021e-02  ], # bin 48  chi2/NDF: 131.61  
[ 1.2375, 1.917100e+01,    -1.4006e+00,    4.2330e-02  ], # bin 49  chi2/NDF: 77.454  
[ 1.2625, 1.908900e+01,    -1.3312e+00,    3.7037e-02  ], # bin 50  chi2/NDF: 106.8  
[ 1.2875, 2.269800e+01,    -1.8682e+00,    5.7790e-02  ], # bin 51  chi2/NDF: 110.03  
[ 1.3125, 2.539100e+01,    -2.1912e+00,    6.5915e-02  ], # bin 52  chi2/NDF: 40.295  
[ 1.3375, 2.857900e+01,    -2.6270e+00,    8.0532e-02  ], # bin 53  chi2/NDF: 57.4  
[ 1.3625, 2.941700e+01,    -2.6745e+00,    7.6752e-02  ], # bin 54  chi2/NDF: 35.171  
[ 1.3875, 5.099700e+01,    -5.4115e+00,    1.7751e-01  ], # bin 55  chi2/NDF: 328.61  
[ 1.4125, 3.544900e+01,    -3.5747e+00,    1.0611e-01  ], # bin 56  chi2/NDF: 189.47  
[ 1.4375, 1.100000e+00,    0.0000e+00,    0.0000e+00  ], # bin 57 crack  chi2/NDF: 0  
[ 1.4625, 1.100000e+00,    0.0000e+00,    0.0000e+00  ], # bin 58 crack  chi2/NDF: 0  
[ 1.4875, 1.600000e+00,    0.0000e+00,    0.0000e+00  ], # bin 59 crack  chi2/NDF: 0  
[ 1.5125, -9.959000e+00,    1.8175e-01,    2.3247e+02  ], # bin 60 crack  chi2/NDF: 426.52  
[ 1.5375, -1.589200e+01,    5.8686e-01,    2.2036e+02  ], # bin 61 crack  chi2/NDF: 76.081  
[ 1.5625, -9.260000e-02,    1.2619e-01,    8.9364e+01  ], # bin 62  chi2/NDF: 33.59  
[ 1.5875, 1.061530e+00,    1.1427e-01,    7.3256e+01  ], # bin 63  chi2/NDF: 16.618  
[ 1.6125, 1.868170e+00,    9.5498e-02,    6.8468e+01  ], # bin 64  chi2/NDF: 27.741  
[ 1.6375, -5.570000e-01,    1.7323e-01,    9.0344e+01  ], # bin 65  chi2/NDF: 45.275  
[ 1.6625, -2.359000e-01,    1.5269e-01,    9.5098e+01  ], # bin 66  chi2/NDF: 14.946  
[ 1.6875, -3.681500e+00,    2.6196e-01,    1.2682e+02  ], # bin 67  chi2/NDF: 14.878  
[ 1.7125, 3.981000e-01,    1.7998e-01,    8.7400e+01  ], # bin 68  chi2/NDF: 70.604  
[ 1.7375, 8.202600e+00,    -6.1507e-02,    2.4803e+01  ], # bin 69  chi2/NDF: 79.85  
[ 1.7625, 8.513500e+00,    -5.9089e-02,    2.2178e+01  ], # bin 70  chi2/NDF: 279.41  
[ 1.7875, 1.424100e+01,    -2.3849e-01,    -2.0728e+01  ], # bin 71  chi2/NDF: 97.896  
[ 1.8125, 2.765600e+01,    -6.7008e-01,    -1.2159e+02  ], # bin 72  chi2/NDF: 107.64  
[ 1.8375, 2.873900e+01,    -7.1886e-01,    -1.2421e+02  ], # bin 73  chi2/NDF: 9.9151  
[ 1.8625, 3.410800e+01,    -9.0513e-01,    -1.6034e+02  ], # bin 74  chi2/NDF: 76.243  
[ 1.8875, 2.772800e+01,    -6.7228e-01,    -1.1364e+02  ], # bin 75  chi2/NDF: 40.743  
[ 1.9125, 2.619600e+01,    -6.1219e-01,    -1.0051e+02  ], # bin 76  chi2/NDF: 32.29  
[ 1.9375, 3.469800e+01,    -8.9161e-01,    -1.6093e+02  ], # bin 77  chi2/NDF: 63.339  
[ 1.9625, 3.631300e+01,    -9.3702e-01,    -1.7128e+02  ], # bin 78  chi2/NDF: 46.35  
[ 1.9875, 2.879000e+01,    -6.6064e-01,    -1.1693e+02  ], # bin 79  chi2/NDF: 74.726  
[ 2.0125, 3.365200e+01,    -8.0444e-01,    -1.5339e+02  ], # bin 80  chi2/NDF: 47.11  
[ 2.0375, 3.302200e+01,    -7.8399e-01,    -1.4481e+02  ], # bin 81  chi2/NDF: 133.35  
[ 2.0625, 3.394200e+01,    -7.9526e-01,    -1.5155e+02  ], # bin 82  chi2/NDF: 46.61  
[ 2.0875, 2.934200e+01,    -6.1771e-01,    -1.1890e+02  ], # bin 83  chi2/NDF: 33.601  
[ 2.1125, 4.100800e+01,    -1.0290e+00,    -1.9786e+02  ], # bin 84  chi2/NDF: 103.71  
[ 2.1375, 3.283700e+01,    -7.3227e-01,    -1.3833e+02  ], # bin 85  chi2/NDF: 74.81  
[ 2.1625, 3.664200e+01,    -8.4701e-01,    -1.6567e+02  ], # bin 86  chi2/NDF: 77.474  
[ 2.1875, 3.765000e+01,    -8.6224e-01,    -1.7331e+02  ], # bin 87  chi2/NDF: 44.947  
[ 2.2125, 3.481400e+01,    -7.6101e-01,    -1.5024e+02  ], # bin 88  chi2/NDF: 37.772  
[ 2.2375, 3.819000e+01,    -8.7771e-01,    -1.7095e+02  ], # bin 89  chi2/NDF: 30.431  
[ 2.2625, 4.276900e+01,    -1.0311e+00,    -2.0176e+02  ], # bin 90  chi2/NDF: 53.664  
[ 2.2875, 3.835900e+01,    -8.4497e-01,    -1.7319e+02  ], # bin 91  chi2/NDF: 7.4291  
[ 2.3125, 4.122300e+01,    -9.0094e-01,    -2.0034e+02  ], # bin 92  chi2/NDF: 45.837  
[ 2.3375, 4.324000e+01,    -9.9468e-01,    -2.0555e+02  ], # bin 93  chi2/NDF: 82.757  
[ 2.3625, 5.120500e+01,    -1.2311e+00,    -2.6721e+02  ], # bin 94  chi2/NDF: 44.363  
[ 2.3875, 3.964000e+01,    -8.7901e-01,    -1.7033e+02  ], # bin 95  chi2/NDF: 65.401  
[ 2.4125, 4.495100e+01,    -1.0595e+00,    -2.0654e+02  ], # bin 96  chi2/NDF: 68.782  
[ 2.4375, 4.890200e+01,    -1.1762e+00,    -2.3534e+02  ], # bin 97  chi2/NDF: 60.763  
[ 2.4625, 4.086500e+01,    -8.7685e-01,    -1.7613e+02  ], # bin 98  chi2/NDF: 124.01  
[ 2.4875, 4.384400e+01,    -1.0070e+00,    -1.8469e+02  ], # bin 99  chi2/NDF: 157.06  
] # end CaloSwCalHitsOutOfCone_v9_gam35_unconv


CaloSwCalHitsOutOfCone_v9leakdata_ele37 = [
[ 0.0125, 6.503700e+00,    -1.2488e-01,    -2.8294e-03  ], # bin 0  chi2/NDF: 75.281  copied from bin 2  
[ 0.0375, 6.503700e+00,    -1.2488e-01,    -2.8294e-03  ], # bin 1  chi2/NDF: 229.9  copied from bin 2  
[ 0.0625, 6.503700e+00,    -1.2488e-01,    -2.8294e-03  ], # bin 2  chi2/NDF: 110.35  
[ 0.0875, 6.385300e+00,    -9.6627e-02,    -4.0279e-03  ], # bin 3  chi2/NDF: 135.38  
[ 0.1125, 6.408100e+00,    -9.3318e-02,    -4.3846e-03  ], # bin 4  chi2/NDF: 156.5  
[ 0.1375, 6.526800e+00,    -1.1665e-01,    -3.1067e-03  ], # bin 5  chi2/NDF: 241.99  
[ 0.1625, 6.413800e+00,    -9.5276e-02,    -4.1493e-03  ], # bin 6  chi2/NDF: 150.53  
[ 0.1875, 6.524900e+00,    -1.1691e-01,    -2.9515e-03  ], # bin 7  chi2/NDF: 187.09  
[ 0.2125, 6.334300e+00,    -8.1830e-02,    -4.5785e-03  ], # bin 8  chi2/NDF: 192.36  
[ 0.2375, 6.401000e+00,    -1.0456e-01,    -2.9757e-03  ], # bin 9  chi2/NDF: 154.66  
[ 0.2625, 6.505300e+00,    -1.1983e-01,    -2.4255e-03  ], # bin 10  chi2/NDF: 290.5  
[ 0.2875, 6.788300e+00,    -1.7675e-01,    5.0447e-04  ], # bin 11  chi2/NDF: 197.32  
[ 0.3125, 6.297300e+00,    -8.0786e-02,    -3.9151e-03  ], # bin 12  chi2/NDF: 191.39  
[ 0.3375, 6.803400e+00,    -1.6773e-01,    1.1624e-05  ], # bin 13  chi2/NDF: 233.45  
[ 0.3625, 7.048200e+00,    -2.1214e-01,    2.0444e-03  ], # bin 14  chi2/NDF: 230.06  
[ 0.3875, 7.090400e+00,    -2.1751e-01,    2.5081e-03  ], # bin 15  chi2/NDF: 133.91  
[ 0.4125, 7.189200e+00,    -2.3317e-01,    3.1762e-03  ], # bin 16  chi2/NDF: 349.48  
[ 0.4375, 6.758000e+00,    -1.5338e-01,    -1.8399e-04  ], # bin 17  chi2/NDF: 185.17  
[ 0.4625, 6.734100e+00,    -1.4452e-01,    -5.6636e-04  ], # bin 18  chi2/NDF: 164.79  
[ 0.4875, 6.809200e+00,    -1.5184e-01,    -2.3782e-04  ], # bin 19  chi2/NDF: 263.09  
[ 0.5125, 6.898600e+00,    -1.6707e-01,    5.9737e-04  ], # bin 20  chi2/NDF: 246.6  
[ 0.5375, 7.973600e+00,    -3.4599e-01,    8.1113e-03  ], # bin 21  chi2/NDF: 202.32  
[ 0.5625, 7.397100e+00,    -2.4764e-01,    3.9177e-03  ], # bin 22  chi2/NDF: 260.08  
[ 0.5875, 8.032800e+00,    -3.5204e-01,    8.3385e-03  ], # bin 23  chi2/NDF: 295.44  
[ 0.6125, 8.283000e+00,    -3.9075e-01,    9.7664e-03  ], # bin 24  chi2/NDF: 172.14  
[ 0.6375, 9.384200e+00,    -5.6792e-01,    1.7201e-02  ], # bin 25  chi2/NDF: 131.63  
[ 0.6625, 9.560000e+00,    -5.6622e-01,    1.5889e-02  ], # bin 26  chi2/NDF: 191.15  
[ 0.6875, 1.058100e+01,    -7.2878e-01,    2.2691e-02  ], # bin 27  chi2/NDF: 142.82  
[ 0.7125, 1.066800e+01,    -7.3432e-01,    2.2304e-02  ], # bin 28  chi2/NDF: 102.88  
[ 0.7375, 1.159600e+01,    -8.9515e-01,    2.9547e-02  ], # bin 29  chi2/NDF: 134.85  
[ 0.7625, 1.159600e+01,    -8.9515e-01,    2.9547e-02  ], # bin 30  chi2/NDF: 167.71  copied from bin 29  
[ 0.7875, 1.159600e+01,    -8.9515e-01,    2.9547e-02  ], # bin 31  chi2/NDF: 116.06  copied from bin 29  
[ 0.8125, 2.315800e+01,    -1.9981e+00,    5.9324e-02  ], # bin 32  chi2/NDF: 75.804  copied from bin 34  
[ 0.8375, 2.315800e+01,    -1.9981e+00,    5.9324e-02  ], # bin 33  chi2/NDF: 22.817  copied from bin 34  
[ 0.8625, 2.315800e+01,    -1.9981e+00,    5.9324e-02  ], # bin 34  chi2/NDF: 52.585  
[ 0.8875, 2.479100e+01,    -2.2854e+00,    7.1497e-02  ], # bin 35  chi2/NDF: 89.379  
[ 0.9125, 2.341300e+01,    -2.0559e+00,    6.2061e-02  ], # bin 36  chi2/NDF: 59.263  
[ 0.9375, 2.391800e+01,    -2.1584e+00,    6.6858e-02  ], # bin 37  chi2/NDF: 66.784  
[ 0.9625, 2.406000e+01,    -2.1983e+00,    6.8963e-02  ], # bin 38  chi2/NDF: 44.013  
[ 0.9875, 2.359100e+01,    -2.1383e+00,    6.7380e-02  ], # bin 39  chi2/NDF: 51.175  
[ 1.0125, 2.626300e+01,    -2.6104e+00,    8.7348e-02  ], # bin 40  chi2/NDF: 54.368  
[ 1.0375, 2.423000e+01,    -2.2829e+00,    7.4490e-02  ], # bin 41  chi2/NDF: 66.974  
[ 1.0625, 2.432100e+01,    -2.3201e+00,    7.6244e-02  ], # bin 42  chi2/NDF: 70.057  
[ 1.0875, 2.550700e+01,    -2.5457e+00,    8.6988e-02  ], # bin 43  chi2/NDF: 177.64  
[ 1.1125, 2.359500e+01,    -2.2646e+00,    7.6072e-02  ], # bin 44  chi2/NDF: 180.49  
[ 1.1375, 2.333800e+01,    -2.2442e+00,    7.6487e-02  ], # bin 45  chi2/NDF: 135.89  
[ 1.1625, 2.510300e+01,    -2.4992e+00,    8.4995e-02  ], # bin 46  chi2/NDF: 149.67  
[ 1.1875, 2.636900e+01,    -2.6936e+00,    9.3490e-02  ], # bin 47  chi2/NDF: 151.33  
[ 1.2125, 2.669000e+01,    -2.7132e+00,    9.2538e-02  ], # bin 48  chi2/NDF: 200.4  
[ 1.2375, 2.853200e+01,    -3.0070e+00,    1.0508e-01  ], # bin 49  chi2/NDF: 186.01  
[ 1.2625, 2.826400e+01,    -2.9135e+00,    9.8672e-02  ], # bin 50  chi2/NDF: 212.86  
[ 1.2875, 3.041700e+01,    -3.2478e+00,    1.1272e-01  ], # bin 51  chi2/NDF: 176.62  
[ 1.3125, 3.346000e+01,    -3.6536e+00,    1.2496e-01  ], # bin 52  chi2/NDF: 213.95  
[ 1.3375, 3.777300e+01,    -4.3182e+00,    1.5076e-01  ], # bin 53  chi2/NDF: 183.85  
[ 1.3625, 3.972500e+01,    -4.5432e+00,    1.5389e-01  ], # bin 54  chi2/NDF: 177.47  
[ 1.3875, 5.562800e+01,    -6.4469e+00,    2.2594e-01  ], # bin 55  chi2/NDF: 138.99  
[ 1.4125, 3.920500e+01,    -4.2897e+00,    1.3563e-01  ], # bin 56  chi2/NDF: 77.761  
[ 1.4375, 1.000000e+00,    0.0000e+00,    0.0000e+00  ], # bin 57 crack  chi2/NDF: 0  
[ 1.4625, 1.000000e+00,    0.0000e+00,    0.0000e+00  ], # bin 58 crack  chi2/NDF: 0  
[ 1.4875, 1.500000e+00,    0.0000e+00,    0.0000e+00  ], # bin 59 crack  chi2/NDF: 0  
[ 1.5125, -3.731100e+01,    1.2816e+00,    4.0128e+02  ], # bin 60 crack  chi2/NDF: 704.98  
[ 1.5375, -2.728500e+01,    9.2818e-01,    3.0742e+02  ], # bin 61 crack  chi2/NDF: 48.161  
[ 1.5625, -2.070200e+01,    6.8098e-01,    2.7023e+02  ], # bin 62  chi2/NDF: 12.687  
[ 1.5875, -2.041000e+01,    7.4454e-01,    2.4557e+02  ], # bin 63  chi2/NDF: 15.033  
[ 1.6125, -1.609700e+01,    6.1314e-01,    2.1246e+02  ], # bin 64  chi2/NDF: 5.6596  
[ 1.6375, -1.955100e+01,    7.5636e-01,    2.3416e+02  ], # bin 65  chi2/NDF: 69.184  
[ 1.6625, -2.019700e+01,    7.7611e-01,    2.4430e+02  ], # bin 66  chi2/NDF: 70.766  
[ 1.6875, -2.716600e+01,    1.0129e+00,    2.9941e+02  ], # bin 67  chi2/NDF: 69.136  
[ 1.7125, -1.971500e+01,    7.5590e-01,    2.4852e+02  ], # bin 68  chi2/NDF: 104.87  
[ 1.7375, -1.108000e+01,    5.2020e-01,    1.7005e+02  ], # bin 69  chi2/NDF: 116.79  
[ 1.7625, -1.014000e+01,    5.1051e-01,    1.5982e+02  ], # bin 70  chi2/NDF: 136.95  
[ 1.7875, 1.830320e+00,    1.1973e-01,    7.1065e+01  ], # bin 71  chi2/NDF: 60.944  
[ 1.8125, 1.241800e+01,    -2.6270e-01,    1.8499e+00  ], # bin 72  chi2/NDF: 17.114  
[ 1.8375, 1.147580e+01,    -2.4254e-01,    1.2701e+01  ], # bin 73  chi2/NDF: 21.651  
[ 1.8625, 1.562000e+01,    -3.8930e-01,    -1.5138e+01  ], # bin 74  chi2/NDF: 18.508  
[ 1.8875, 1.472600e+01,    -3.3707e-01,    -1.0226e+01  ], # bin 75  chi2/NDF: 16.185  
[ 1.9125, 1.299600e+01,    -2.5695e-01,    5.1994e-01  ], # bin 76  chi2/NDF: 7.5169  
[ 1.9375, 1.385100e+01,    -2.6031e-01,    -7.6513e+00  ], # bin 77  chi2/NDF: 31.829  
[ 1.9625, 1.252400e+01,    -2.0007e-01,    1.8206e+00  ], # bin 78  chi2/NDF: 27.816  
[ 1.9875, 1.089130e+01,    -1.2596e-01,    1.3484e+01  ], # bin 79  chi2/NDF: 8.1002  
[ 2.0125, 1.128090e+01,    -1.3621e-01,    1.3340e+01  ], # bin 80  chi2/NDF: 28.365  
[ 2.0375, 9.350900e+00,    -5.9204e-02,    2.8926e+01  ], # bin 81  chi2/NDF: 12.575  
[ 2.0625, 9.064100e+00,    -4.4215e-02,    3.3308e+01  ], # bin 82  chi2/NDF: 10.01  
[ 2.0875, 1.177200e+01,    -1.2295e-01,    1.4559e+01  ], # bin 83  chi2/NDF: 14.239  
[ 2.1125, 9.661900e+00,    -4.8411e-02,    3.0852e+01  ], # bin 84  chi2/NDF: 11.83  
[ 2.1375, 9.021200e+00,    -6.6653e-04,    3.3843e+01  ], # bin 85  chi2/NDF: 18.292  
[ 2.1625, 9.853400e+00,    -1.7330e-02,    2.8097e+01  ], # bin 86  chi2/NDF: 50.704  
[ 2.1875, 1.013070e+01,    -1.1380e-02,    2.6321e+01  ], # bin 87  chi2/NDF: 24.75  
[ 2.2125, 1.107130e+01,    -4.4124e-02,    2.2303e+01  ], # bin 88  chi2/NDF: 35.579  
[ 2.2375, 9.920700e+00,    2.0239e-02,    2.8850e+01  ], # bin 89  chi2/NDF: 13.631  
[ 2.2625, 1.571400e+01,    -1.9381e-01,    -7.3324e+00  ], # bin 90  chi2/NDF: 7.6034  
[ 2.2875, 1.563000e+01,    -1.9325e-01,    -1.5143e+00  ], # bin 91  chi2/NDF: 13.753  
[ 2.3125, 2.189800e+01,    -4.0355e-01,    -4.1550e+01  ], # bin 92  chi2/NDF: 10.823  
[ 2.3375, 1.637800e+01,    -2.7823e-01,    1.6189e+01  ], # bin 93  chi2/NDF: 63.12  
[ 2.3625, -1.733300e+00,    2.9775e-01,    1.6251e+02  ], # bin 94  chi2/NDF: 21.877  
[ 2.3875, -4.116000e+00,    3.6970e-01,    1.8339e+02  ], # bin 95  chi2/NDF: 79.215  
[ 2.4125, -1.792200e+01,    8.2027e-01,    2.9353e+02  ], # bin 96  chi2/NDF: 47.951  
[ 2.4375, -4.378200e+01,    1.7078e+00,    4.8555e+02  ], # bin 97  chi2/NDF: 40.369  
[ 2.4625, -8.370800e+01,    3.0557e+00,    7.9160e+02  ], # bin 98  chi2/NDF: 32.761  
[ 2.4875, -1.087000e+02,    3.8267e+00,    1.0010e+03  ], # bin 99  chi2/NDF: 663.5  
] # end CaloSwCalHitsOutOfCone_v9_ele37


CaloSwCalHitsOutOfCone_v9leakdata_gam37_conv = [
[ 0.0125, 7.875900e+00,    -3.0258e-01,    2.4970e-03  ], # bin 0  chi2/NDF: 20.705  copied from bin 2  
[ 0.0375, 7.875900e+00,    -3.0258e-01,    2.4970e-03  ], # bin 1  chi2/NDF: 27.101  copied from bin 2  
[ 0.0625, 7.875900e+00,    -3.0258e-01,    2.4970e-03  ], # bin 2  chi2/NDF: 42.615  
[ 0.0875, 7.944600e+00,    -3.2026e-01,    3.8853e-03  ], # bin 3  chi2/NDF: 43.118  
[ 0.1125, 7.490100e+00,    -2.2905e-01,    -4.9020e-04  ], # bin 4  chi2/NDF: 29.274  
[ 0.1375, 7.711300e+00,    -2.6934e-01,    1.4771e-03  ], # bin 5  chi2/NDF: 34.969  
[ 0.1625, 7.616800e+00,    -2.4784e-01,    3.6739e-04  ], # bin 6  chi2/NDF: 32.415  
[ 0.1875, 8.031600e+00,    -3.2449e-01,    3.9771e-03  ], # bin 7  chi2/NDF: 18.254  
[ 0.2125, 7.652800e+00,    -2.4751e-01,    2.3220e-04  ], # bin 8  chi2/NDF: 55.356  
[ 0.2375, 8.255700e+00,    -3.6485e-01,    6.0099e-03  ], # bin 9  chi2/NDF: 54.414  
[ 0.2625, 7.968200e+00,    -3.1576e-01,    3.9036e-03  ], # bin 10  chi2/NDF: 23.096  
[ 0.2875, 8.124600e+00,    -3.4297e-01,    5.3366e-03  ], # bin 11  chi2/NDF: 21.571  
[ 0.3125, 7.357100e+00,    -2.1343e-01,    -5.4243e-05  ], # bin 12  chi2/NDF: 22.472  
[ 0.3375, 8.110300e+00,    -3.3929e-01,    5.5418e-03  ], # bin 13  chi2/NDF: 18.999  
[ 0.3625, 7.477400e+00,    -2.1761e-01,    -9.8368e-05  ], # bin 14  chi2/NDF: 23.264  
[ 0.3875, 7.928900e+00,    -3.0522e-01,    4.3010e-03  ], # bin 15  chi2/NDF: 32.932  
[ 0.4125, 7.798700e+00,    -2.7688e-01,    2.9962e-03  ], # bin 16  chi2/NDF: 25.211  
[ 0.4375, 8.184700e+00,    -3.3693e-01,    5.5474e-03  ], # bin 17  chi2/NDF: 21.842  
[ 0.4625, 8.485300e+00,    -3.8278e-01,    7.5081e-03  ], # bin 18  chi2/NDF: 12.845  
[ 0.4875, 7.480800e+00,    -2.1533e-01,    8.4080e-04  ], # bin 19  chi2/NDF: 13.25  
[ 0.5125, 8.380700e+00,    -3.5744e-01,    6.6137e-03  ], # bin 20  chi2/NDF: 19.489  
[ 0.5375, 7.673400e+00,    -2.4545e-01,    2.3760e-03  ], # bin 21  chi2/NDF: 20.601  
[ 0.5625, 7.407700e+00,    -1.9112e-01,    -2.8198e-04  ], # bin 22  chi2/NDF: 71.747  
[ 0.5875, 8.408200e+00,    -3.5579e-01,    6.7614e-03  ], # bin 23  chi2/NDF: 11.617  
[ 0.6125, 9.146400e+00,    -4.7653e-01,    1.1496e-02  ], # bin 24  chi2/NDF: 45.132  
[ 0.6375, 1.027810e+01,    -6.5203e-01,    1.8476e-02  ], # bin 25  chi2/NDF: 33.888  
[ 0.6625, 1.161100e+01,    -8.4356e-01,    2.5166e-02  ], # bin 26  chi2/NDF: 19.458  
[ 0.6875, 1.178700e+01,    -8.5873e-01,    2.5610e-02  ], # bin 27  chi2/NDF: 37.378  
[ 0.7125, 1.281500e+01,    -1.0219e+00,    3.1706e-02  ], # bin 28  chi2/NDF: 43.4  
[ 0.7375, 1.476700e+01,    -1.3608e+00,    4.6600e-02  ], # bin 29  chi2/NDF: 62.715  
[ 0.7625, 1.476700e+01,    -1.3608e+00,    4.6600e-02  ], # bin 30  chi2/NDF: 43.515  copied from bin 29  
[ 0.7875, 1.476700e+01,    -1.3608e+00,    4.6600e-02  ], # bin 31  chi2/NDF: 27.118  copied from bin 29  
[ 0.8125, 2.597600e+01,    -2.4032e+00,    7.3523e-02  ], # bin 32  chi2/NDF: 64.173  copied from bin 34  
[ 0.8375, 2.597600e+01,    -2.4032e+00,    7.3523e-02  ], # bin 33  chi2/NDF: 41.283  copied from bin 34  
[ 0.8625, 2.597600e+01,    -2.4032e+00,    7.3523e-02  ], # bin 34  chi2/NDF: 59.311  
[ 0.8875, 2.505800e+01,    -2.2979e+00,    7.0850e-02  ], # bin 35  chi2/NDF: 53.889  
[ 0.9125, 2.485400e+01,    -2.2827e+00,    7.0616e-02  ], # bin 36  chi2/NDF: 45.376  
[ 0.9375, 2.340500e+01,    -2.0883e+00,    6.4309e-02  ], # bin 37  chi2/NDF: 30.438  
[ 0.9625, 2.449500e+01,    -2.2657e+00,    7.1177e-02  ], # bin 38  chi2/NDF: 44.58  
[ 0.9875, 2.600000e+01,    -2.5327e+00,    8.3064e-02  ], # bin 39  chi2/NDF: 33.15  
[ 1.0125, 2.359400e+01,    -2.1269e+00,    6.5836e-02  ], # bin 40  chi2/NDF: 45.768  
[ 1.0375, 2.505200e+01,    -2.4097e+00,    7.9146e-02  ], # bin 41  chi2/NDF: 50.167  
[ 1.0625, 2.439500e+01,    -2.3191e+00,    7.5558e-02  ], # bin 42  chi2/NDF: 44.151  
[ 1.0875, 2.538700e+01,    -2.5037e+00,    8.4296e-02  ], # bin 43  chi2/NDF: 21.846  
[ 1.1125, 2.368600e+01,    -2.2471e+00,    7.4134e-02  ], # bin 44  chi2/NDF: 43.431  
[ 1.1375, 2.459800e+01,    -2.4168e+00,    8.2164e-02  ], # bin 45  chi2/NDF: 20.753  
[ 1.1625, 2.554900e+01,    -2.5368e+00,    8.5298e-02  ], # bin 46  chi2/NDF: 28.339  
[ 1.1875, 2.721100e+01,    -2.8007e+00,    9.6621e-02  ], # bin 47  chi2/NDF: 39.143  
[ 1.2125, 2.895000e+01,    -3.0601e+00,    1.0576e-01  ], # bin 48  chi2/NDF: 48.441  
[ 1.2375, 2.952000e+01,    -3.1284e+00,    1.0849e-01  ], # bin 49  chi2/NDF: 18.642  
[ 1.2625, 2.888200e+01,    -2.9732e+00,    9.9589e-02  ], # bin 50  chi2/NDF: 36.657  
[ 1.2875, 3.171400e+01,    -3.4273e+00,    1.1883e-01  ], # bin 51  chi2/NDF: 16.673  
[ 1.3125, 3.491900e+01,    -3.8561e+00,    1.3182e-01  ], # bin 52  chi2/NDF: 23.131  
[ 1.3375, 3.815800e+01,    -4.3454e+00,    1.5046e-01  ], # bin 53  chi2/NDF: 32.014  
[ 1.3625, 4.010800e+01,    -4.5816e+00,    1.5457e-01  ], # bin 54  chi2/NDF: 12.954  
[ 1.3875, 5.808600e+01,    -6.7203e+00,    2.3203e-01  ], # bin 55  chi2/NDF: 142.88  
[ 1.4125, 4.632200e+01,    -5.6329e+00,    1.9753e-01  ], # bin 56  chi2/NDF: 62.813  
[ 1.4375, 1.000000e+00,    0.0000e+00,    0.0000e+00  ], # bin 57 crack  chi2/NDF: 0  
[ 1.4625, 1.000000e+00,    0.0000e+00,    0.0000e+00  ], # bin 58 crack  chi2/NDF: 0  
[ 1.4875, 1.500000e+00,    0.0000e+00,    0.0000e+00  ], # bin 59 crack  chi2/NDF: 0  
[ 1.5125, -4.212200e+00,    -2.9715e-03,    1.9733e+02  ], # bin 60 crack  chi2/NDF: 771.44  
[ 1.5375, -3.059100e+01,    1.0381e+00,    3.3124e+02  ], # bin 61 crack  chi2/NDF: 45.956  
[ 1.5625, -2.064500e+01,    6.7640e-01,    2.6897e+02  ], # bin 62  chi2/NDF: 7.6611  
[ 1.5875, -2.183600e+01,    7.9933e-01,    2.5318e+02  ], # bin 63  chi2/NDF: 315.68  
[ 1.6125, -1.790500e+01,    6.8619e-01,    2.2178e+02  ], # bin 64  chi2/NDF: 293.26  
[ 1.6375, -1.866900e+01,    7.3858e-01,    2.2365e+02  ], # bin 65  chi2/NDF: 20.802  
[ 1.6625, -1.559100e+01,    6.1679e-01,    2.0919e+02  ], # bin 66  chi2/NDF: 16.866  
[ 1.6875, -1.862100e+01,    7.1750e-01,    2.3661e+02  ], # bin 67  chi2/NDF: 5.825  
[ 1.7125, -1.668200e+01,    6.5811e-01,    2.2341e+02  ], # bin 68  chi2/NDF: 64.049  
[ 1.7375, -1.270300e+01,    5.8852e-01,    1.7763e+02  ], # bin 69  chi2/NDF: 37.223  
[ 1.7625, -7.976100e+00,    4.4377e-01,    1.4103e+02  ], # bin 70  chi2/NDF: 91.42  
[ 1.7875, 2.749100e+00,    9.6383e-02,    6.0957e+01  ], # bin 71  chi2/NDF: 26.719  
[ 1.8125, 1.261800e+01,    -2.3492e-01,    -9.5145e+00  ], # bin 72  chi2/NDF: 37.861  
[ 1.8375, 1.180400e+01,    -2.2229e-01,    1.4929e+00  ], # bin 73  chi2/NDF: 14.755  
[ 1.8625, 1.652400e+01,    -3.8598e-01,    -3.0868e+01  ], # bin 74  chi2/NDF: 17.349  
[ 1.8875, 1.431200e+01,    -2.9580e-01,    -1.4760e+01  ], # bin 75  chi2/NDF: 20.759  
[ 1.9125, 1.549200e+01,    -3.2111e-01,    -2.3788e+01  ], # bin 76  chi2/NDF: 38.462  
[ 1.9375, 1.304800e+01,    -2.0723e-01,    -8.6963e+00  ], # bin 77  chi2/NDF: 47.331  
[ 1.9625, 1.235700e+01,    -1.7521e-01,    -2.5217e+00  ], # bin 78  chi2/NDF: 12.891  
[ 1.9875, 1.335700e+01,    -1.9730e-01,    -8.9443e+00  ], # bin 79  chi2/NDF: 5.5775  
[ 2.0125, 1.317800e+01,    -1.7595e-01,    -7.8141e+00  ], # bin 80  chi2/NDF: 19.21  
[ 2.0375, 1.040190e+01,    -6.4632e-02,    1.2933e+01  ], # bin 81  chi2/NDF: 28.338  
[ 2.0625, 8.959400e+00,    -1.2494e-02,    2.6024e+01  ], # bin 82  chi2/NDF: 15.39  
[ 2.0875, 6.575100e+00,    8.0145e-02,    4.4910e+01  ], # bin 83  chi2/NDF: 53.732  
[ 2.1125, 5.131700e+00,    1.4200e-01,    5.4959e+01  ], # bin 84  chi2/NDF: 31.986  
[ 2.1375, 2.397430e+00,    2.5709e-01,    7.3906e+01  ], # bin 85  chi2/NDF: 92.035  
[ 2.1625, 4.167200e+00,    1.9608e-01,    6.4055e+01  ], # bin 86  chi2/NDF: 48.623  
[ 2.1875, 3.099300e+00,    2.5120e-01,    7.1340e+01  ], # bin 87  chi2/NDF: 75.554  
[ 2.2125, 6.376400e+00,    1.3171e-01,    5.1786e+01  ], # bin 88  chi2/NDF: 58.445  
[ 2.2375, 4.829000e+00,    2.1642e-01,    5.9826e+01  ], # bin 89  chi2/NDF: 24.649  
[ 2.2625, 1.034730e+01,    8.3601e-03,    2.6296e+01  ], # bin 90  chi2/NDF: 8.8948  
[ 2.2875, 6.368600e+00,    1.6575e-01,    5.5288e+01  ], # bin 91  chi2/NDF: 20.938  
[ 2.3125, 1.083800e+01,    4.0319e-03,    2.9389e+01  ], # bin 92  chi2/NDF: 12.075  
[ 2.3375, -9.381000e-01,    3.9241e-01,    1.2099e+02  ], # bin 93  chi2/NDF: 58.237  
[ 2.3625, 6.351000e+00,    1.0531e-01,    8.1410e+01  ], # bin 94  chi2/NDF: 4.7617  
[ 2.3875, -5.841800e+00,    5.3284e-01,    1.6982e+02  ], # bin 95  chi2/NDF: 10.518  
[ 2.4125, 5.932300e+00,    8.8957e-02,    9.6326e+01  ], # bin 96  chi2/NDF: 43.187  
[ 2.4375, -1.677500e+01,    8.8484e-01,    2.6139e+02  ], # bin 97  chi2/NDF: 30.57  
[ 2.4625, -2.747700e+01,    1.2190e+00,    3.5299e+02  ], # bin 98  chi2/NDF: 142.56  
[ 2.4875, -1.428500e+02,    5.1574e+00,    1.2062e+03  ], # bin 99  chi2/NDF: 733.46  
] # end CaloSwCalHitsOutOfCone_v9_gam37_conv


CaloSwCalHitsOutOfCone_v9leakdata_gam37_unconv = [
[ 0.0125, 1.306100e+00,    7.3987e-01,    -3.8583e-02  ], # bin 0  chi2/NDF: 105.89  copied from bin 2  
[ 0.0375, 1.306100e+00,    7.3987e-01,    -3.8583e-02  ], # bin 1  chi2/NDF: 80.745  copied from bin 2  
[ 0.0625, 1.306100e+00,    7.3987e-01,    -3.8583e-02  ], # bin 2  chi2/NDF: 79.188  
[ 0.0875, 8.574800e-01,    8.1774e-01,    -4.1564e-02  ], # bin 3  chi2/NDF: 55.774  
[ 0.1125, 8.614800e-01,    8.1729e-01,    -4.1479e-02  ], # bin 4  chi2/NDF: 133.28  
[ 0.1375, 1.773500e+00,    6.8136e-01,    -3.6362e-02  ], # bin 5  chi2/NDF: 162.72  
[ 0.1625, 8.655400e-01,    8.1722e-01,    -4.1286e-02  ], # bin 6  chi2/NDF: 22.036  
[ 0.1875, 1.157040e+00,    7.6618e-01,    -3.8954e-02  ], # bin 7  chi2/NDF: 107.88  
[ 0.2125, 1.194790e+00,    7.5070e-01,    -3.8026e-02  ], # bin 8  chi2/NDF: 72.835  
[ 0.2375, 1.146230e+00,    7.5915e-01,    -3.8168e-02  ], # bin 9  chi2/NDF: 88.287  
[ 0.2625, 1.199480e+00,    7.3749e-01,    -3.6766e-02  ], # bin 10  chi2/NDF: 26.044  
[ 0.2875, 2.146600e+00,    5.8358e-01,    -3.0351e-02  ], # bin 11  chi2/NDF: 32.561  
[ 0.3125, 1.516400e+00,    6.9209e-01,    -3.4918e-02  ], # bin 12  chi2/NDF: 61.353  
[ 0.3375, 1.955200e+00,    6.0556e-01,    -3.0593e-02  ], # bin 13  chi2/NDF: 35.516  
[ 0.3625, 1.820000e+00,    6.2253e-01,    -3.1065e-02  ], # bin 14  chi2/NDF: 100.35  
[ 0.3875, 9.687900e-01,    7.5699e-01,    -3.5981e-02  ], # bin 15  chi2/NDF: 62.546  
[ 0.4125, 1.430800e+00,    6.7134e-01,    -3.2094e-02  ], # bin 16  chi2/NDF: 82.865  
[ 0.4375, 2.103500e+00,    5.5476e-01,    -2.6966e-02  ], # bin 17  chi2/NDF: 62.599  
[ 0.4625, 1.505100e+00,    6.4540e-01,    -3.0206e-02  ], # bin 18  chi2/NDF: 60.365  
[ 0.4875, 2.211700e+00,    5.3082e-01,    -2.5341e-02  ], # bin 19  chi2/NDF: 54.716  
[ 0.5125, 1.207780e+00,    6.8180e-01,    -3.0774e-02  ], # bin 20  chi2/NDF: 17.793  
[ 0.5375, 1.427700e+00,    6.4603e-01,    -2.9274e-02  ], # bin 21  chi2/NDF: 64.463  
[ 0.5625, 1.917500e+00,    5.7112e-01,    -2.6471e-02  ], # bin 22  chi2/NDF: 0.52292  
[ 0.5875, 2.467500e+00,    4.7880e-01,    -2.2408e-02  ], # bin 23  chi2/NDF: 97.815  
[ 0.6125, 2.608400e+00,    4.6453e-01,    -2.2344e-02  ], # bin 24  chi2/NDF: 211.34  
[ 0.6375, 3.688700e+00,    2.9604e-01,    -1.5614e-02  ], # bin 25  chi2/NDF: 77.473  
[ 0.6625, 4.477700e+00,    1.8618e-01,    -1.2032e-02  ], # bin 26  chi2/NDF: 44.251  
[ 0.6875, 5.402900e+00,    3.4152e-02,    -5.6272e-03  ], # bin 27  chi2/NDF: 28.16  
[ 0.7125, 4.789800e+00,    1.3209e-01,    -9.8042e-03  ], # bin 28  chi2/NDF: 68.597  
[ 0.7375, 5.069400e+00,    8.8740e-02,    -7.8350e-03  ], # bin 29  chi2/NDF: 56.867  
[ 0.7625, 5.069400e+00,    8.8740e-02,    -7.8350e-03  ], # bin 30  chi2/NDF: 56.546  copied from bin 29  
[ 0.7875, 5.069400e+00,    8.8740e-02,    -7.8350e-03  ], # bin 31  chi2/NDF: 42.533  copied from bin 29  
[ 0.8125, 6.913500e+00,    2.2438e-01,    -1.7119e-02  ], # bin 32  chi2/NDF: 52.029  copied from bin 34  
[ 0.8375, 6.913500e+00,    2.2438e-01,    -1.7119e-02  ], # bin 33  chi2/NDF: 29.901  copied from bin 34  
[ 0.8625, 6.913500e+00,    2.2438e-01,    -1.7119e-02  ], # bin 34  chi2/NDF: 97.652  
[ 0.8875, 6.488300e+00,    2.6212e-01,    -1.7594e-02  ], # bin 35  chi2/NDF: 59.047  
[ 0.9125, 7.919800e+00,    5.3375e-02,    -1.0202e-02  ], # bin 36  chi2/NDF: 60.584  
[ 0.9375, 8.792500e+00,    -8.9067e-02,    -4.2954e-03  ], # bin 37  chi2/NDF: 18.265  
[ 0.9625, 6.739700e+00,    2.0152e-01,    -1.4746e-02  ], # bin 38  chi2/NDF: 51.377  
[ 0.9875, 8.538800e+00,    -7.2591e-02,    -4.0710e-03  ], # bin 39  chi2/NDF: 31.625  
[ 1.0125, 8.820100e+00,    -9.8258e-02,    -3.8942e-03  ], # bin 40  chi2/NDF: 95.706  
[ 1.0375, 9.823600e+00,    -2.7434e-01,    3.7975e-03  ], # bin 41  chi2/NDF: 37.218  
[ 1.0625, 1.016050e+01,    -3.2054e-01,    5.0018e-03  ], # bin 42  chi2/NDF: 39.822  
[ 1.0875, 1.219100e+01,    -6.4642e-01,    1.8343e-02  ], # bin 43  chi2/NDF: 43.399  
[ 1.1125, 1.101700e+01,    -4.5772e-01,    1.0436e-02  ], # bin 44  chi2/NDF: 57.445  
[ 1.1375, 1.179100e+01,    -5.9240e-01,    1.6609e-02  ], # bin 45  chi2/NDF: 38.621  
[ 1.1625, 1.138800e+01,    -5.0090e-01,    1.1574e-02  ], # bin 46  chi2/NDF: 82.503  
[ 1.1875, 1.301900e+01,    -7.5325e-01,    2.2018e-02  ], # bin 47  chi2/NDF: 27.147  
[ 1.2125, 1.286800e+01,    -7.1057e-01,    1.9105e-02  ], # bin 48  chi2/NDF: 45.44  
[ 1.2375, 1.491600e+01,    -1.0312e+00,    3.2238e-02  ], # bin 49  chi2/NDF: 27.22  
[ 1.2625, 1.405800e+01,    -8.4661e-01,    2.2604e-02  ], # bin 50  chi2/NDF: 120.28  
[ 1.2875, 1.652500e+01,    -1.2148e+00,    3.7171e-02  ], # bin 51  chi2/NDF: 68.801  
[ 1.3125, 1.811200e+01,    -1.3921e+00,    4.0383e-02  ], # bin 52  chi2/NDF: 30.323  
[ 1.3375, 2.119400e+01,    -1.8568e+00,    5.7857e-02  ], # bin 53  chi2/NDF: 29.7  
[ 1.3625, 2.223100e+01,    -1.9415e+00,    5.5758e-02  ], # bin 54  chi2/NDF: 8.8042  
[ 1.3875, 3.590100e+01,    -3.5112e+00,    1.1491e-01  ], # bin 55  chi2/NDF: 202.14  
[ 1.4125, 2.527600e+01,    -2.3342e+00,    6.5857e-02  ], # bin 56  chi2/NDF: 70.48  
[ 1.4375, 1.000000e+00,    0.0000e+00,    0.0000e+00  ], # bin 57 crack  chi2/NDF: 0  
[ 1.4625, 1.000000e+00,    0.0000e+00,    0.0000e+00  ], # bin 58 crack  chi2/NDF: 0  
[ 1.4875, 1.500000e+00,    0.0000e+00,    0.0000e+00  ], # bin 59 crack  chi2/NDF: 0  
[ 1.5125, 4.052000e+00,    -2.8620e-01,    1.1320e+02  ], # bin 60 crack  chi2/NDF: 527.57  
[ 1.5375, -9.667000e+00,    3.9409e-01,    1.5629e+02  ], # bin 61 crack  chi2/NDF: 51.213  
[ 1.5625, 2.918800e+00,    4.3187e-02,    4.8734e+01  ], # bin 62  chi2/NDF: 34.042  
[ 1.5875, 4.088300e+00,    6.1027e-03,    3.8886e+01  ], # bin 63  chi2/NDF: 16.473  
[ 1.6125, 4.333100e+00,    6.6271e-03,    3.7664e+01  ], # bin 64  chi2/NDF: 35.842  
[ 1.6375, 1.838540e+00,    8.9452e-02,    5.9208e+01  ], # bin 65  chi2/NDF: 29.448  
[ 1.6625, 2.078790e+00,    7.0767e-02,    6.4386e+01  ], # bin 66  chi2/NDF: 14.638  
[ 1.6875, -2.363400e+00,    2.1533e-01,    1.0190e+02  ], # bin 67  chi2/NDF: 20.256  
[ 1.7125, 2.820800e+00,    8.4257e-02,    5.6766e+01  ], # bin 68  chi2/NDF: 50.74  
[ 1.7375, 7.415800e+00,    -5.0976e-02,    1.8500e+01  ], # bin 69  chi2/NDF: 37.642  
[ 1.7625, 8.684100e+00,    -8.4866e-02,    9.4661e+00  ], # bin 70  chi2/NDF: 119.67  
[ 1.7875, 1.261700e+01,    -2.0709e-01,    -1.9543e+01  ], # bin 71  chi2/NDF: 35.51  
[ 1.8125, 2.540500e+01,    -6.1807e-01,    -1.1584e+02  ], # bin 72  chi2/NDF: 43.034  
[ 1.8375, 2.556700e+01,    -6.3932e-01,    -1.1128e+02  ], # bin 73  chi2/NDF: 12.356  
[ 1.8625, 3.069700e+01,    -8.2365e-01,    -1.4435e+02  ], # bin 74  chi2/NDF: 24.906  
[ 1.8875, 2.403000e+01,    -5.7643e-01,    -9.6904e+01  ], # bin 75  chi2/NDF: 21.951  
[ 1.9125, 2.526300e+01,    -6.1365e-01,    -1.0390e+02  ], # bin 76  chi2/NDF: 64.744  
[ 1.9375, 2.757400e+01,    -6.7905e-01,    -1.2007e+02  ], # bin 77  chi2/NDF: 48.455  
[ 1.9625, 3.171500e+01,    -8.1083e-01,    -1.4929e+02  ], # bin 78  chi2/NDF: 51.682  
[ 1.9875, 2.647300e+01,    -6.1606e-01,    -1.1104e+02  ], # bin 79  chi2/NDF: 49.655  
[ 2.0125, 3.049900e+01,    -7.3183e-01,    -1.4186e+02  ], # bin 80  chi2/NDF: 49.929  
[ 2.0375, 3.325400e+01,    -8.2728e-01,    -1.5816e+02  ], # bin 81  chi2/NDF: 110.32  
[ 2.0625, 2.723000e+01,    -6.0409e-01,    -1.1473e+02  ], # bin 82  chi2/NDF: 63.048  
[ 2.0875, 2.953600e+01,    -6.6694e-01,    -1.3088e+02  ], # bin 83  chi2/NDF: 52.42  
[ 2.1125, 3.668800e+01,    -9.2484e-01,    -1.7750e+02  ], # bin 84  chi2/NDF: 125.85  
[ 2.1375, 2.935000e+01,    -6.5299e-01,    -1.2532e+02  ], # bin 85  chi2/NDF: 21.261  
[ 2.1625, 2.725800e+01,    -5.7303e-01,    -1.0869e+02  ], # bin 86  chi2/NDF: 21.578  
[ 2.1875, 3.377000e+01,    -7.7699e-01,    -1.5670e+02  ], # bin 87  chi2/NDF: 46.527  
[ 2.2125, 3.357100e+01,    -7.6803e-01,    -1.5242e+02  ], # bin 88  chi2/NDF: 73.948  
[ 2.2375, 3.619500e+01,    -8.5870e-01,    -1.6824e+02  ], # bin 89  chi2/NDF: 92.668  
[ 2.2625, 3.787700e+01,    -9.1239e-01,    -1.7862e+02  ], # bin 90  chi2/NDF: 80.8  
[ 2.2875, 4.131900e+01,    -9.9686e-01,    -2.0667e+02  ], # bin 91  chi2/NDF: 37.994  
[ 2.3125, 4.498500e+01,    -1.0860e+00,    -2.3883e+02  ], # bin 92  chi2/NDF: 103.8  
[ 2.3375, 3.437100e+01,    -7.4254e-01,    -1.5377e+02  ], # bin 93  chi2/NDF: 101.91  
[ 2.3625, 4.355100e+01,    -1.0275e+00,    -2.2250e+02  ], # bin 94  chi2/NDF: 20.286  
[ 2.3875, 4.076700e+01,    -9.8037e-01,    -1.8929e+02  ], # bin 95  chi2/NDF: 49.484  
[ 2.4125, 5.399300e+01,    -1.4233e+00,    -2.8502e+02  ], # bin 96  chi2/NDF: 167.87  
[ 2.4375, 4.234200e+01,    -1.0061e+00,    -2.0078e+02  ], # bin 97  chi2/NDF: 36.234  
[ 2.4625, 3.741600e+01,    -8.0223e-01,    -1.6650e+02  ], # bin 98  chi2/NDF: 81.233  
[ 2.4875, 2.154900e+01,    -2.5408e-01,    -4.7237e+01  ], # bin 99  chi2/NDF: 409.91  
] # end CaloSwCalHitsOutOfCone_v9_gam37_unconv

CaloSwCalHitsEfront_P2_v9leakdata_ele = [
[ 0.0125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 0  chi2/NDF: 0, copied from bin 2
[ 0.0375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 1  chi2/NDF: 0, copied from bin 2
[ 0.0625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 2  chi2/NDF: 0
[ 0.0875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 3  chi2/NDF: 0
[ 0.1125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 4  chi2/NDF: 0
[ 0.1375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 5  chi2/NDF: 0
[ 0.1625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 6  chi2/NDF: 0
[ 0.1875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 7  chi2/NDF: 0
[ 0.2125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 8  chi2/NDF: 0
[ 0.2375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 9  chi2/NDF: 0
[ 0.2625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 10  chi2/NDF: 0
[ 0.2875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 11  chi2/NDF: 0
[ 0.3125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 12  chi2/NDF: 0
[ 0.3375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 13  chi2/NDF: 0
[ 0.3625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 14  chi2/NDF: 0
[ 0.3875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 15  chi2/NDF: 0
[ 0.4125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 16  chi2/NDF: 0
[ 0.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 17  chi2/NDF: 0
[ 0.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 18  chi2/NDF: 0
[ 0.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 19  chi2/NDF: 0
[ 0.5125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 20  chi2/NDF: 0
[ 0.5375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 21  chi2/NDF: 0
[ 0.5625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 22  chi2/NDF: 0
[ 0.5875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 23  chi2/NDF: 0
[ 0.6125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 24  chi2/NDF: 0
[ 0.6375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 25  chi2/NDF: 0
[ 0.6625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 26  chi2/NDF: 0
[ 0.6875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 27  chi2/NDF: 0
[ 0.7125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 28  chi2/NDF: 0
[ 0.7375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 29  chi2/NDF: 0
[ 0.7625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 30  chi2/NDF: 0, copied from bin 29
[ 0.7875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 31  chi2/NDF: 0, copied from bin 29
[ 0.8125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 32  chi2/NDF: 0, copied from bin 34
[ 0.8375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 33  chi2/NDF: 0, copied from bin 34
[ 0.8625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 34  chi2/NDF: 0
[ 0.8875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 35  chi2/NDF: 0
[ 0.9125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 36  chi2/NDF: 0
[ 0.9375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 37  chi2/NDF: 0
[ 0.9625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 38  chi2/NDF: 0
[ 0.9875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 39  chi2/NDF: 0
[ 1.0125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 40  chi2/NDF: 0
[ 1.0375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 41  chi2/NDF: 0
[ 1.0625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 42  chi2/NDF: 0
[ 1.0875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 43  chi2/NDF: 0
[ 1.1125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 44  chi2/NDF: 0
[ 1.1375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 45  chi2/NDF: 0
[ 1.1625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 46  chi2/NDF: 0
[ 1.1875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 47  chi2/NDF: 0
[ 1.2125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 48  chi2/NDF: 0
[ 1.2375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 49  chi2/NDF: 0
[ 1.2625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 50  chi2/NDF: 0
[ 1.2875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 51  chi2/NDF: 0
[ 1.3125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 52  chi2/NDF: 0
[ 1.3375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 53  chi2/NDF: 0
[ 1.3625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 54  chi2/NDF: 0
[ 1.3875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 55  chi2/NDF: 0
[ 1.4125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 56  chi2/NDF: 0
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -2.5021e-05,   9.7110e-08,   -1.4571e-03  ],# bin 60 crack  chi2/NDF: 3.2657
[ 1.5375,   3.1258e-06,   2.5670e-08,   3.7526e-02  ],# bin 61 crack  chi2/NDF: 10.27
[ 1.5625,   1.6815e-05,   -1.2735e-08,   9.7192e-03  ],# bin 62  chi2/NDF: 3.6496
[ 1.5875,   8.5490e-06,   -1.3465e-09,   1.1374e-02  ],# bin 63  chi2/NDF: 5.3151
[ 1.6125,   1.1464e-05,   -6.5654e-09,   9.9516e-03  ],# bin 64  chi2/NDF: 0.47091
[ 1.6375,   5.0906e-05,   -5.7487e-08,   1.0678e-02  ],# bin 65  chi2/NDF: 2.0843
[ 1.6625,   4.3645e-05,   -4.7866e-08,   1.6411e-02  ],# bin 66  chi2/NDF: 2.1636
[ 1.6875,   1.6981e-05,   -6.0957e-09,   4.5503e-02  ],# bin 67  chi2/NDF: 6.804
[ 1.7125,   7.6508e-05,   -9.1232e-08,   1.1252e-02  ],# bin 68  chi2/NDF: 6.5176
[ 1.7375,   7.8909e-05,   -9.6034e-08,   -2.0254e-02  ],# bin 69  chi2/NDF: 14.981
[ 1.7625,   8.6455e-05,   -1.0812e-07,   -2.7184e-02  ],# bin 70  chi2/NDF: 22.281
[ 1.7875,   5.6020e-05,   -6.7408e-08,   -4.4482e-03  ],# bin 71  chi2/NDF: 6.1524
[ 1.8125,   1.0845e+01,   3.3729e+00,   -7.8190e-04  ],# bin 72  chi2/NDF: 0.68127
[ 1.8375,   4.6718e+01,   4.2235e+00,   -1.0303e-03  ],# bin 73  chi2/NDF: 0.74445
[ 1.8625,   4.9873e+01,   3.4787e+00,   -1.6758e-04  ],# bin 74  chi2/NDF: 1.1891
[ 1.8875,   3.0471e+01,   3.3482e+00,   3.9798e-04  ],# bin 75  chi2/NDF: 0.41121
[ 1.9125,   3.4431e+01,   3.2730e+00,   -4.4960e-04  ],# bin 76  chi2/NDF: 1.3515
[ 1.9375,   3.1671e+01,   2.9290e+00,   -7.3887e-04  ],# bin 77  chi2/NDF: 1.8862
[ 1.9625,   3.6966e+01,   2.6226e+00,   -4.0242e-04  ],# bin 78  chi2/NDF: 0.7984
[ 1.9875,   2.9365e+01,   3.1176e+00,   -1.4430e-03  ],# bin 79  chi2/NDF: 0.54321
[ 2.0125,   3.6124e+01,   2.7791e+00,   -8.7614e-04  ],# bin 80  chi2/NDF: 1.8707
[ 2.0375,   2.2803e+01,   2.9040e+00,   -7.1061e-04  ],# bin 81  chi2/NDF: 1.9334
[ 2.0625,   2.1699e+01,   2.8123e+00,   -5.9299e-04  ],# bin 82  chi2/NDF: 1.5561
[ 2.0875,   1.9492e+01,   2.6987e+00,   -5.9791e-04  ],# bin 83  chi2/NDF: 2.8567
[ 2.1125,   4.1176e+01,   2.6855e+00,   -1.0496e-03  ],# bin 84  chi2/NDF: 1.733
[ 2.1375,   3.4699e+01,   2.7952e+00,   -1.2053e-03  ],# bin 85  chi2/NDF: 1.2996
[ 2.1625,   2.7471e+01,   2.5436e+00,   -7.6854e-04  ],# bin 86  chi2/NDF: 2.6936
[ 2.1875,   1.8835e+01,   2.5645e+00,   -9.9359e-04  ],# bin 87  chi2/NDF: 2.0596
[ 2.2125,   3.1570e+01,   2.3267e+00,   -6.2368e-04  ],# bin 88  chi2/NDF: 0.66653
[ 2.2375,   3.2827e+01,   2.4983e+00,   -1.0664e-03  ],# bin 89  chi2/NDF: 2.3246
[ 2.2625,   3.7080e+01,   2.1321e+00,   -8.5290e-04  ],# bin 90  chi2/NDF: 0.8476
[ 2.2875,   3.8367e+01,   2.2118e+00,   -1.0668e-03  ],# bin 91  chi2/NDF: 0.80992
[ 2.3125,   1.5076e+01,   8.9009e-01,   -1.1604e-03  ],# bin 92  chi2/NDF: 2.8802
[ 2.3375,   3.3914e+01,   3.0647e+00,   2.3675e-06  ],# bin 93  chi2/NDF: 0.78192
[ 2.3625,   2.8803e+01,   3.3221e+00,   -6.3914e-04  ],# bin 94  chi2/NDF: 0.885
[ 2.3875,   5.2107e+01,   3.1526e+00,   -4.9527e-04  ],# bin 95  chi2/NDF: 1.6975
[ 2.4125,   1.7835e+01,   3.6850e+00,   -1.6805e-03  ],# bin 96  chi2/NDF: 0.8144
[ 2.4375,   -2.4748e+01,   4.4950e+00,   -2.4356e-03  ],# bin 97  chi2/NDF: 0.17024
[ 2.4625,   -4.3935e+01,   5.2197e+00,   -4.5873e-03  ],# bin 98  chi2/NDF: 0.92416
[ 2.4875,   -3.3273e+01,   4.7471e+00,   -4.4480e-03  ],# bin 99  chi2/NDF: 1.5673
] # end CaloSwCalHitsEfront_P2_v9_ele


CaloSwCalHitsEfront_P2_v9leakdata_gam_conv = [
[ 0.0125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 0  chi2/NDF: 0, copied from bin 2
[ 0.0375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 1  chi2/NDF: 0, copied from bin 2
[ 0.0625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 2  chi2/NDF: 0
[ 0.0875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 3  chi2/NDF: 0
[ 0.1125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 4  chi2/NDF: 0
[ 0.1375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 5  chi2/NDF: 0
[ 0.1625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 6  chi2/NDF: 0
[ 0.1875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 7  chi2/NDF: 0
[ 0.2125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 8  chi2/NDF: 0
[ 0.2375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 9  chi2/NDF: 0
[ 0.2625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 10  chi2/NDF: 0
[ 0.2875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 11  chi2/NDF: 0
[ 0.3125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 12  chi2/NDF: 0
[ 0.3375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 13  chi2/NDF: 0
[ 0.3625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 14  chi2/NDF: 0
[ 0.3875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 15  chi2/NDF: 0
[ 0.4125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 16  chi2/NDF: 0
[ 0.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 17  chi2/NDF: 0
[ 0.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 18  chi2/NDF: 0
[ 0.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 19  chi2/NDF: 0
[ 0.5125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 20  chi2/NDF: 0
[ 0.5375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 21  chi2/NDF: 0
[ 0.5625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 22  chi2/NDF: 0
[ 0.5875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 23  chi2/NDF: 0
[ 0.6125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 24  chi2/NDF: 0
[ 0.6375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 25  chi2/NDF: 0
[ 0.6625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 26  chi2/NDF: 0
[ 0.6875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 27  chi2/NDF: 0
[ 0.7125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 28  chi2/NDF: 0
[ 0.7375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 29  chi2/NDF: 0
[ 0.7625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 30  chi2/NDF: 0, copied from bin 29
[ 0.7875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 31  chi2/NDF: 0, copied from bin 29
[ 0.8125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 32  chi2/NDF: 0, copied from bin 34
[ 0.8375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 33  chi2/NDF: 0, copied from bin 34
[ 0.8625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 34  chi2/NDF: 0
[ 0.8875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 35  chi2/NDF: 0
[ 0.9125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 36  chi2/NDF: 0
[ 0.9375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 37  chi2/NDF: 0
[ 0.9625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 38  chi2/NDF: 0
[ 0.9875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 39  chi2/NDF: 0
[ 1.0125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 40  chi2/NDF: 0
[ 1.0375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 41  chi2/NDF: 0
[ 1.0625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 42  chi2/NDF: 0
[ 1.0875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 43  chi2/NDF: 0
[ 1.1125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 44  chi2/NDF: 0
[ 1.1375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 45  chi2/NDF: 0
[ 1.1625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 46  chi2/NDF: 0
[ 1.1875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 47  chi2/NDF: 0
[ 1.2125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 48  chi2/NDF: 0
[ 1.2375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 49  chi2/NDF: 0
[ 1.2625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 50  chi2/NDF: 0
[ 1.2875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 51  chi2/NDF: 0
[ 1.3125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 52  chi2/NDF: 0
[ 1.3375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 53  chi2/NDF: 0
[ 1.3625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 54  chi2/NDF: 0
[ 1.3875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 55  chi2/NDF: 0
[ 1.4125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 56  chi2/NDF: 0
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   -1.8996e-05,   9.7434e-08,   -3.6141e-03  ],# bin 60 crack  chi2/NDF: 2.1359
[ 1.5375,   7.7575e-06,   1.7691e-08,   4.4895e-02  ],# bin 61 crack  chi2/NDF: 15.035
[ 1.5625,   2.6843e-05,   -2.7407e-08,   1.1766e-02  ],# bin 62  chi2/NDF: 3.7383
[ 1.5875,   1.9801e-05,   -1.8409e-08,   8.9284e-03  ],# bin 63  chi2/NDF: 2.6456
[ 1.6125,   2.0824e-05,   -1.9008e-08,   -2.8901e-03  ],# bin 64  chi2/NDF: 4.8091
[ 1.6375,   4.4235e-05,   -4.4862e-08,   2.7399e-02  ],# bin 65  chi2/NDF: 1.5133
[ 1.6625,   2.7928e-05,   -2.3101e-08,   2.1606e-02  ],# bin 66  chi2/NDF: 2.8469
[ 1.6875,   1.2841e-05,   1.4632e-09,   4.6978e-02  ],# bin 67  chi2/NDF: 2.5405
[ 1.7125,   5.8295e-05,   -6.5051e-08,   -1.4598e-02  ],# bin 68  chi2/NDF: 5.1873
[ 1.7375,   7.7708e-05,   -9.1413e-08,   -1.4062e-02  ],# bin 69  chi2/NDF: 5.6818
[ 1.7625,   8.1722e-05,   -1.0007e-07,   -2.0870e-02  ],# bin 70  chi2/NDF: 5.926
[ 1.7875,   6.3961e-05,   -7.1852e-08,   -6.3518e-03  ],# bin 71  chi2/NDF: 3.5905
[ 1.8125,   1.1292e+01,   3.1444e+00,   1.5325e-03  ],# bin 72  chi2/NDF: 4.1519
[ 1.8375,   5.2420e+01,   3.2751e+00,   1.6915e-03  ],# bin 73  chi2/NDF: 23.676
[ 1.8625,   2.3661e+01,   3.4602e+00,   2.3112e-03  ],# bin 74  chi2/NDF: 11.736
[ 1.8875,   4.0051e+01,   3.5856e+00,   3.0559e-04  ],# bin 75  chi2/NDF: 9.0559
[ 1.9125,   -1.3457e+00,   3.4835e+00,   1.3309e-03  ],# bin 76  chi2/NDF: 5.5897
[ 1.9375,   1.7111e+01,   3.4479e+00,   1.6780e-03  ],# bin 77  chi2/NDF: 8.1088
[ 1.9625,   1.2291e+01,   3.3796e+00,   2.6927e-04  ],# bin 78  chi2/NDF: 12.114
[ 1.9875,   2.4018e+01,   2.8039e+00,   1.3046e-03  ],# bin 79  chi2/NDF: 9.0416
[ 2.0125,   5.8900e+01,   2.4913e+00,   1.5320e-03  ],# bin 80  chi2/NDF: 1.3786
[ 2.0375,   3.6814e+01,   3.1695e+00,   6.9556e-04  ],# bin 81  chi2/NDF: 5.3997
[ 2.0625,   7.5901e+01,   2.7545e+00,   1.4105e-03  ],# bin 82  chi2/NDF: 14.673
[ 2.0875,   9.4763e+01,   2.6964e+00,   5.5351e-04  ],# bin 83  chi2/NDF: 21.304
[ 2.1125,   1.1406e+02,   3.0155e+00,   1.1312e-04  ],# bin 84  chi2/NDF: 16.728
[ 2.1375,   1.3051e+02,   2.9046e+00,   1.0552e-04  ],# bin 85  chi2/NDF: 6.8514
[ 2.1625,   7.0751e+01,   2.8329e+00,   -6.9356e-04  ],# bin 86  chi2/NDF: 6.1837
[ 2.1875,   7.2069e+01,   2.4195e+00,   7.2387e-04  ],# bin 87  chi2/NDF: 5.5167
[ 2.2125,   7.5483e+01,   2.4227e+00,   -5.1428e-05  ],# bin 88  chi2/NDF: 33.925
[ 2.2375,   1.1896e+02,   2.2576e+00,   4.8673e-04  ],# bin 89  chi2/NDF: 11.278
[ 2.2625,   3.3596e+01,   2.2112e+00,   -3.6197e-04  ],# bin 90  chi2/NDF: 15.06
[ 2.2875,   3.7075e+01,   2.4416e+00,   -1.1748e-03  ],# bin 91  chi2/NDF: 41.439
[ 2.3125,   4.5877e+01,   1.8885e+00,   -7.8533e-04  ],# bin 92  chi2/NDF: 26.013
[ 2.3375,   9.2035e+01,   2.9312e+00,   4.1254e-03  ],# bin 93  chi2/NDF: 9.1849
[ 2.3625,   2.4831e+01,   3.6091e+00,   4.1330e-03  ],# bin 94  chi2/NDF: 3.5069
[ 2.3875,   6.4060e+01,   3.5817e+00,   2.5586e-03  ],# bin 95  chi2/NDF: 9.3548
[ 2.4125,   7.3012e+01,   3.5822e+00,   3.9330e-03  ],# bin 96  chi2/NDF: 11.719
[ 2.4375,   1.1403e+00,   5.4496e+00,   8.9962e-04  ],# bin 97  chi2/NDF: 10.867
[ 2.4625,   4.5269e+01,   6.6108e+00,   5.6852e-04  ],# bin 98  chi2/NDF: 14.179
[ 2.4875,   -3.8676e+01,   6.6294e+00,   7.3642e-04  ],# bin 99  chi2/NDF: 2.8201
] # end CaloSwCalHitsEfront_P2_v9_gam_conv


CaloSwCalHitsEfront_P2_v9leakdata_gam_unconv = [
[ 0.0125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 0  chi2/NDF: 0, copied from bin 2
[ 0.0375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 1  chi2/NDF: 0, copied from bin 2
[ 0.0625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 2  chi2/NDF: 0
[ 0.0875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 3  chi2/NDF: 0
[ 0.1125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 4  chi2/NDF: 0
[ 0.1375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 5  chi2/NDF: 0
[ 0.1625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 6  chi2/NDF: 0
[ 0.1875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 7  chi2/NDF: 0
[ 0.2125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 8  chi2/NDF: 0
[ 0.2375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 9  chi2/NDF: 0
[ 0.2625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 10  chi2/NDF: 0
[ 0.2875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 11  chi2/NDF: 0
[ 0.3125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 12  chi2/NDF: 0
[ 0.3375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 13  chi2/NDF: 0
[ 0.3625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 14  chi2/NDF: 0
[ 0.3875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 15  chi2/NDF: 0
[ 0.4125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 16  chi2/NDF: 0
[ 0.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 17  chi2/NDF: 0
[ 0.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 18  chi2/NDF: 0
[ 0.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 19  chi2/NDF: 0
[ 0.5125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 20  chi2/NDF: 0
[ 0.5375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 21  chi2/NDF: 0
[ 0.5625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 22  chi2/NDF: 0
[ 0.5875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 23  chi2/NDF: 0
[ 0.6125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 24  chi2/NDF: 0
[ 0.6375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 25  chi2/NDF: 0
[ 0.6625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 26  chi2/NDF: 0
[ 0.6875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 27  chi2/NDF: 0
[ 0.7125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 28  chi2/NDF: 0
[ 0.7375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 29  chi2/NDF: 0
[ 0.7625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 30  chi2/NDF: 0, copied from bin 29
[ 0.7875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 31  chi2/NDF: 0, copied from bin 29
[ 0.8125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 32  chi2/NDF: 0, copied from bin 34
[ 0.8375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 33  chi2/NDF: 0, copied from bin 34
[ 0.8625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 34  chi2/NDF: 0
[ 0.8875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 35  chi2/NDF: 0
[ 0.9125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 36  chi2/NDF: 0
[ 0.9375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 37  chi2/NDF: 0
[ 0.9625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 38  chi2/NDF: 0
[ 0.9875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 39  chi2/NDF: 0
[ 1.0125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 40  chi2/NDF: 0
[ 1.0375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 41  chi2/NDF: 0
[ 1.0625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 42  chi2/NDF: 0
[ 1.0875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 43  chi2/NDF: 0
[ 1.1125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 44  chi2/NDF: 0
[ 1.1375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 45  chi2/NDF: 0
[ 1.1625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 46  chi2/NDF: 0
[ 1.1875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 47  chi2/NDF: 0
[ 1.2125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 48  chi2/NDF: 0
[ 1.2375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 49  chi2/NDF: 0
[ 1.2625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 50  chi2/NDF: 0
[ 1.2875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 51  chi2/NDF: 0
[ 1.3125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 52  chi2/NDF: 0
[ 1.3375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 53  chi2/NDF: 0
[ 1.3625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 54  chi2/NDF: 0
[ 1.3875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 55  chi2/NDF: 0
[ 1.4125,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 56  chi2/NDF: 0
[ 1.4375,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 57 crack  chi2/NDF: 0
[ 1.4625,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 58 crack  chi2/NDF: 0
[ 1.4875,   0.0000e+00,   0.0000e+00,   0.0000e+00  ],# bin 59 crack  chi2/NDF: 0
[ 1.5125,   1.8242e-05,   5.2320e-08,   -5.0746e-03  ],# bin 60 crack  chi2/NDF: 20.899
[ 1.5375,   6.8278e-05,   -8.2260e-08,   9.5859e-03  ],# bin 61 crack  chi2/NDF: 12.322
[ 1.5625,   1.2803e-05,   -8.9073e-09,   -3.7663e-03  ],# bin 62  chi2/NDF: 3.3169
[ 1.5875,   8.4141e-06,   -2.2218e-09,   1.2419e-02  ],# bin 63  chi2/NDF: 2.0231
[ 1.6125,   7.6500e-06,   -2.8618e-09,   4.0294e-03  ],# bin 64  chi2/NDF: 1.5433
[ 1.6375,   2.5554e-05,   -2.3801e-08,   1.6610e-02  ],# bin 65  chi2/NDF: 2.7306
[ 1.6625,   3.6436e-05,   -3.9694e-08,   1.3617e-02  ],# bin 66  chi2/NDF: 1.0782
[ 1.6875,   3.8393e-05,   -4.0765e-08,   2.2286e-02  ],# bin 67  chi2/NDF: 1.8963
[ 1.7125,   2.5994e-05,   -2.3166e-08,   1.6466e-03  ],# bin 68  chi2/NDF: 1.0138
[ 1.7375,   3.8833e-05,   -4.1749e-08,   -8.0378e-03  ],# bin 69  chi2/NDF: 3.4644
[ 1.7625,   5.3233e-05,   -6.0297e-08,   9.0470e-03  ],# bin 70  chi2/NDF: 3.8365
[ 1.7875,   2.0427e-05,   4.7311e-09,   3.2635e-02  ],# bin 71  chi2/NDF: 4.7658
[ 1.8125,   1.2339e+01,   2.4381e+00,   1.8165e-03  ],# bin 72  chi2/NDF: 6.0554
[ 1.8375,   1.7640e+01,   1.8775e+00,   1.1954e-03  ],# bin 73  chi2/NDF: 6.5955
[ 1.8625,   2.1537e+01,   1.3373e+00,   1.6964e-03  ],# bin 74  chi2/NDF: 3.4264
[ 1.8875,   1.5565e+01,   1.4232e+00,   9.1491e-04  ],# bin 75  chi2/NDF: 2.1635
[ 1.9125,   2.3547e+01,   1.2543e+00,   4.4298e-04  ],# bin 76  chi2/NDF: 3.7974
[ 1.9375,   1.7193e+01,   1.0691e+00,   1.4316e-03  ],# bin 77  chi2/NDF: 3.5959
[ 1.9625,   1.4859e+01,   1.0058e+00,   1.1456e-03  ],# bin 78  chi2/NDF: 3.8876
[ 1.9875,   8.8668e+00,   1.1432e+00,   6.4720e-04  ],# bin 79  chi2/NDF: 1.9246
[ 2.0125,   2.3942e+01,   9.2027e-01,   8.0210e-04  ],# bin 80  chi2/NDF: 4.2262
[ 2.0375,   8.8070e+00,   9.8974e-01,   5.6515e-04  ],# bin 81  chi2/NDF: 2.8736
[ 2.0625,   1.9637e+01,   8.2114e-01,   6.2164e-04  ],# bin 82  chi2/NDF: 9.7153
[ 2.0875,   1.3053e+01,   7.8432e-01,   3.3752e-04  ],# bin 83  chi2/NDF: 9.1416
[ 2.1125,   2.8090e+01,   7.5572e-01,   4.1344e-04  ],# bin 84  chi2/NDF: 12.19
[ 2.1375,   1.4842e+01,   8.6474e-01,   -4.2761e-05  ],# bin 85  chi2/NDF: 3.4452
[ 2.1625,   1.2710e+01,   7.0981e-01,   4.0589e-05  ],# bin 86  chi2/NDF: 4.5037
[ 2.1875,   9.0682e+00,   6.3705e-01,   1.2266e-04  ],# bin 87  chi2/NDF: 12.759
[ 2.2125,   1.1246e+01,   5.1273e-01,   3.0604e-04  ],# bin 88  chi2/NDF: 19.197
[ 2.2375,   2.0969e+01,   4.6404e-01,   3.9624e-04  ],# bin 89  chi2/NDF: 17.036
[ 2.2625,   1.9987e+01,   4.5069e-01,   9.3279e-05  ],# bin 90  chi2/NDF: 3.2238
[ 2.2875,   6.1310e+00,   4.5831e-01,   1.2859e-04  ],# bin 91  chi2/NDF: 10.206
[ 2.3125,   2.2676e+01,   1.3225e-01,   2.3391e-03  ],# bin 92  chi2/NDF: 20.811
[ 2.3375,   3.4265e+01,   2.5256e-01,   1.4962e-02  ],# bin 93  chi2/NDF: 18.569
[ 2.3625,   2.1545e+01,   1.0633e+00,   1.6071e-04  ],# bin 94  chi2/NDF: 9.2157
[ 2.3875,   7.6219e+00,   1.1972e+00,   -4.9285e-04  ],# bin 95  chi2/NDF: 4.0514
[ 2.4125,   2.6956e+01,   9.1264e-01,   1.3305e-04  ],# bin 96  chi2/NDF: 9.1533
[ 2.4375,   2.5056e+01,   7.3334e-01,   4.3121e-04  ],# bin 97  chi2/NDF: 3.3668
[ 2.4625,   2.4934e+01,   7.5288e-01,   2.3950e-04  ],# bin 98  chi2/NDF: 8.5938
[ 2.4875,   2.0258e+01,   9.0418e-01,   -4.1348e-05  ],# bin 99  chi2/NDF: 32.343
] # end CaloSwCalHitsEfront_P2_v9_gam_unconv


CaloSwCalHitsOutOfCone_v9leakdata_ele55 = [
[ 0.0125, 7.929200e+00,    -7.4952e-01,    2.5501e-02  ], # bin 0  chi2/NDF: 70.604  copied from bin 2  
[ 0.0375, 7.929200e+00,    -7.4952e-01,    2.5501e-02  ], # bin 1  chi2/NDF: 17.933  copied from bin 2  
[ 0.0625, 7.929200e+00,    -7.4952e-01,    2.5501e-02  ], # bin 2  chi2/NDF: 31.973  
[ 0.0875, 9.229400e+00,    -9.9620e-01,    3.6869e-02  ], # bin 3  chi2/NDF: 31.799  
[ 0.1125, 7.670500e+00,    -6.8680e-01,    2.2037e-02  ], # bin 4  chi2/NDF: 295.36  
[ 0.1375, 7.038800e+00,    -5.6641e-01,    1.6410e-02  ], # bin 5  chi2/NDF: 171.64  
[ 0.1625, 7.854500e+00,    -7.1472e-01,    2.3123e-02  ], # bin 6  chi2/NDF: 106.21  
[ 0.1875, 8.640400e+00,    -8.6672e-01,    3.0491e-02  ], # bin 7  chi2/NDF: 36.939  
[ 0.2125, 8.380600e+00,    -8.2996e-01,    2.9327e-02  ], # bin 8  chi2/NDF: 30.932  
[ 0.2375, 6.699600e+00,    -5.0067e-01,    1.3548e-02  ], # bin 9  chi2/NDF: 240.05  
[ 0.2625, 7.949800e+00,    -7.4445e-01,    2.5302e-02  ], # bin 10  chi2/NDF: 43.5  
[ 0.2875, 7.301800e+00,    -6.0542e-01,    1.8302e-02  ], # bin 11  chi2/NDF: 171.19  
[ 0.3125, 8.139400e+00,    -7.6208e-01,    2.5722e-02  ], # bin 12  chi2/NDF: 36.224  
[ 0.3375, 7.595700e+00,    -6.5507e-01,    2.0627e-02  ], # bin 13  chi2/NDF: 92.562  
[ 0.3625, 8.219700e+00,    -7.6112e-01,    2.5327e-02  ], # bin 14  chi2/NDF: 27.295  
[ 0.3875, 8.414500e+00,    -7.8672e-01,    2.6226e-02  ], # bin 15  chi2/NDF: 61.386  
[ 0.4125, 7.496500e+00,    -6.2284e-01,    1.9183e-02  ], # bin 16  chi2/NDF: 163.02  
[ 0.4375, 8.055600e+00,    -7.0986e-01,    2.2665e-02  ], # bin 17  chi2/NDF: 88.491  
[ 0.4625, 8.031000e+00,    -7.0006e-01,    2.2231e-02  ], # bin 18  chi2/NDF: 85.372  
[ 0.4875, 7.717800e+00,    -6.3714e-01,    1.9384e-02  ], # bin 19  chi2/NDF: 167.47  
[ 0.5125, 7.508800e+00,    -5.9983e-01,    1.7980e-02  ], # bin 20  chi2/NDF: 140.37  
[ 0.5375, 7.960500e+00,    -6.6940e-01,    2.0647e-02  ], # bin 21  chi2/NDF: 152.34  
[ 0.5625, 8.598300e+00,    -7.8346e-01,    2.5753e-02  ], # bin 22  chi2/NDF: 49.398  
[ 0.5875, 8.835300e+00,    -8.1389e-01,    2.6699e-02  ], # bin 23  chi2/NDF: 69.372  
[ 0.6125, 9.340300e+00,    -8.9706e-01,    3.0223e-02  ], # bin 24  chi2/NDF: 52.147  
[ 0.6375, 1.037100e+01,    -1.0632e+00,    3.6999e-02  ], # bin 25  chi2/NDF: 87.821  
[ 0.6625, 1.105300e+01,    -1.1431e+00,    3.9330e-02  ], # bin 26  chi2/NDF: 108.07  
[ 0.6875, 1.134400e+01,    -1.1571e+00,    3.8609e-02  ], # bin 27  chi2/NDF: 68.234  
[ 0.7125, 1.141800e+01,    -1.1634e+00,    3.8822e-02  ], # bin 28  chi2/NDF: 43.606  
[ 0.7375, 1.201900e+01,    -1.2551e+00,    4.2323e-02  ], # bin 29  chi2/NDF: 28.628  
[ 0.7625, 1.201900e+01,    -1.2551e+00,    4.2323e-02  ], # bin 30  chi2/NDF: 49.791  copied from bin 29  
[ 0.7875, 1.201900e+01,    -1.2551e+00,    4.2323e-02  ], # bin 31  chi2/NDF: 80.929  copied from bin 29  
[ 0.8125, 2.679900e+01,    -2.9464e+00,    9.3334e-02  ], # bin 32  chi2/NDF: 48.135  copied from bin 34  
[ 0.8375, 2.679900e+01,    -2.9464e+00,    9.3334e-02  ], # bin 33  chi2/NDF: 66.352  copied from bin 34  
[ 0.8625, 2.679900e+01,    -2.9464e+00,    9.3334e-02  ], # bin 34  chi2/NDF: 80.588  
[ 0.8875, 2.490900e+01,    -2.6826e+00,    8.4136e-02  ], # bin 35  chi2/NDF: 141.3  
[ 0.9125, 2.700800e+01,    -3.0012e+00,    9.6185e-02  ], # bin 36  chi2/NDF: 107.64  
[ 0.9375, 2.504400e+01,    -2.7310e+00,    8.6846e-02  ], # bin 37  chi2/NDF: 75.359  
[ 0.9625, 2.692700e+01,    -3.0556e+00,    1.0068e-01  ], # bin 38  chi2/NDF: 37.964  
[ 0.9875, 2.547700e+01,    -2.8320e+00,    9.1978e-02  ], # bin 39  chi2/NDF: 114.86  
[ 1.0125, 2.761000e+01,    -3.1805e+00,    1.0619e-01  ], # bin 40  chi2/NDF: 72.065  
[ 1.0375, 2.669800e+01,    -3.0600e+00,    1.0209e-01  ], # bin 41  chi2/NDF: 118.21  
[ 1.0625, 2.705700e+01,    -3.1481e+00,    1.0678e-01  ], # bin 42  chi2/NDF: 70.931  
[ 1.0875, 2.755300e+01,    -3.2297e+00,    1.0979e-01  ], # bin 43  chi2/NDF: 163.25  
[ 1.1125, 2.632400e+01,    -3.0811e+00,    1.0581e-01  ], # bin 44  chi2/NDF: 165.33  
[ 1.1375, 2.496400e+01,    -2.8771e+00,    9.7898e-02  ], # bin 45  chi2/NDF: 134.98  
[ 1.1625, 2.727900e+01,    -3.2224e+00,    1.1122e-01  ], # bin 46  chi2/NDF: 115.85  
[ 1.1875, 2.904400e+01,    -3.4663e+00,    1.1952e-01  ], # bin 47  chi2/NDF: 140.25  
[ 1.2125, 3.071200e+01,    -3.6974e+00,    1.2803e-01  ], # bin 48  chi2/NDF: 184.65  
[ 1.2375, 3.488700e+01,    -4.2829e+00,    1.4820e-01  ], # bin 49  chi2/NDF: 600.74  
[ 1.2625, 3.488300e+01,    -4.2907e+00,    1.4955e-01  ], # bin 50  chi2/NDF: 144.98  
[ 1.2875, 3.665800e+01,    -4.4953e+00,    1.5456e-01  ], # bin 51  chi2/NDF: 135.66  
[ 1.3125, 4.357400e+01,    -5.5517e+00,    1.9569e-01  ], # bin 52  chi2/NDF: 91.011  
[ 1.3375, 4.745000e+01,    -6.0567e+00,    2.1055e-01  ], # bin 53  chi2/NDF: 261.16  
[ 1.3625, 5.156800e+01,    -6.6066e+00,    2.3264e-01  ], # bin 54  chi2/NDF: 130.88  
[ 1.3875, 5.543000e+01,    -7.3908e+00,    2.6233e-01  ], # bin 55  chi2/NDF: 94.879  
[ 1.4125, 3.764100e+01,    -4.4100e+00,    1.3890e-01  ], # bin 56  chi2/NDF: 160.1  
[ 1.4375, 2.000000e-01,    0.0000e+00,    0.0000e+00  ], # bin 57 crack  chi2/NDF: 0  
[ 1.4625, 2.000000e-01,    0.0000e+00,    0.0000e+00  ], # bin 58 crack  chi2/NDF: 0  
[ 1.4875, 6.000000e-01,    0.0000e+00,    0.0000e+00  ], # bin 59 crack  chi2/NDF: 0  
[ 1.5125, -2.318000e+01,    5.5497e-01,    3.0480e+02  ], # bin 60 crack  chi2/NDF: 123.44  
[ 1.5375, -3.239900e+01,    1.0012e+00,    3.2640e+02  ], # bin 61 crack  chi2/NDF: 27.728  
[ 1.5625, -2.825300e+01,    7.6899e-01,    3.2556e+02  ], # bin 62  chi2/NDF: 43.083  
[ 1.5875, -2.889900e+01,    8.7962e-01,    3.0038e+02  ], # bin 63  chi2/NDF: 40.54  
[ 1.6125, -2.636200e+01,    8.2613e-01,    2.7576e+02  ], # bin 64  chi2/NDF: 37.885  
[ 1.6375, -2.265600e+01,    7.2293e-01,    2.4395e+02  ], # bin 65  chi2/NDF: 19.081  
[ 1.6625, -2.184400e+01,    7.0331e-01,    2.4011e+02  ], # bin 66  chi2/NDF: 8.3968  
[ 1.6875, -2.876500e+01,    9.5300e-01,    2.9040e+02  ], # bin 67  chi2/NDF: 53.641  
[ 1.7125, -2.224000e+01,    7.0788e-01,    2.5080e+02  ], # bin 68  chi2/NDF: 63.165  
[ 1.7375, -1.355900e+01,    4.7658e-01,    1.6919e+02  ], # bin 69  chi2/NDF: 53.729  
[ 1.7625, -1.264400e+01,    4.7276e-01,    1.5751e+02  ], # bin 70  chi2/NDF: 47.101  
[ 1.7875, -3.220900e+00,    1.7018e-01,    8.5628e+01  ], # bin 71  chi2/NDF: 52.647  
[ 1.8125, 1.486440e+00,    -7.9533e-03,    5.8669e+01  ], # bin 72  chi2/NDF: 33.462  
[ 1.8375, -3.735300e+00,    1.6796e-01,    9.7674e+01  ], # bin 73  chi2/NDF: 49.001  
[ 1.8625, -1.669200e+00,    1.1066e-01,    8.0975e+01  ], # bin 74  chi2/NDF: 10.465  
[ 1.8875, -1.547800e+00,    1.1748e-01,    7.9441e+01  ], # bin 75  chi2/NDF: 32.55  
[ 1.9125, -8.793000e-01,    1.1258e-01,    7.3125e+01  ], # bin 76  chi2/NDF: 101.38  
[ 1.9375, 1.979500e+00,    2.6716e-02,    5.1862e+01  ], # bin 77  chi2/NDF: 21.509  
[ 1.9625, 2.455700e+00,    2.6061e-02,    4.7913e+01  ], # bin 78  chi2/NDF: 22.031  
[ 1.9875, 1.398420e+00,    7.0134e-02,    5.6293e+01  ], # bin 79  chi2/NDF: 29.364  
[ 2.0125, -7.375000e-02,    1.2524e-01,    6.9402e+01  ], # bin 80  chi2/NDF: 10.52  
[ 2.0375, -1.595400e+00,    1.7212e-01,    8.4102e+01  ], # bin 81  chi2/NDF: 9.317  
[ 2.0625, -5.974000e-01,    1.5061e-01,    7.7539e+01  ], # bin 82  chi2/NDF: 45.436  
[ 2.0875, 3.785300e-01,    1.2230e-01,    7.2053e+01  ], # bin 83  chi2/NDF: 18.717  
[ 2.1125, -2.722000e+00,    2.4636e-01,    9.2228e+01  ], # bin 84  chi2/NDF: 14.677  
[ 2.1375, 5.346540e-01,    1.3896e-01,    6.9752e+01  ], # bin 85  chi2/NDF: 15.037  
[ 2.1625, 1.684200e+00,    1.1743e-01,    6.0472e+01  ], # bin 86  chi2/NDF: 15.054  
[ 2.1875, 6.519810e-01,    1.6124e-01,    6.8623e+01  ], # bin 87  chi2/NDF: 213.64  
[ 2.2125, 4.427700e-01,    1.8112e-01,    7.0134e+01  ], # bin 88  chi2/NDF: 20.683  
[ 2.2375, 3.203500e+00,    9.0292e-02,    5.1001e+01  ], # bin 89  chi2/NDF: 11.407  
[ 2.2625, 3.848000e+00,    7.4423e-02,    4.8095e+01  ], # bin 90  chi2/NDF: 19.987  
[ 2.2875, 5.591700e+00,    -6.7569e-04,    4.2059e+01  ], # bin 91  chi2/NDF: 22.029  
[ 2.3125, 1.107200e+01,    -1.9180e-01,    8.7517e+00  ], # bin 92  chi2/NDF: 42.114  
[ 2.3375, 1.015910e+00,    8.7603e-02,    9.8791e+01  ], # bin 93  chi2/NDF: 149.82  
[ 2.3625, -1.071300e+01,    4.4855e-01,    1.9849e+02  ], # bin 94  chi2/NDF: 141.39  
[ 2.3875, -1.679600e+01,    6.2453e-01,    2.4977e+02  ], # bin 95  chi2/NDF: 87.769  
[ 2.4125, -3.720100e+01,    1.2985e+00,    4.0895e+02  ], # bin 96  chi2/NDF: 141.29  
[ 2.4375, -3.697000e+01,    1.2391e+00,    4.2165e+02  ], # bin 97  chi2/NDF: 93.777  
[ 2.4625, -1.221800e+02,    4.1678e+00,    1.0616e+03  ], # bin 98  chi2/NDF: 27.563  
[ 2.4875, -2.739700e+02,    9.1635e+00,    2.2684e+03  ], # bin 99  chi2/NDF: 328.99  
] # end CaloSwCalHitsOutOfCone_v9_ele55


CaloSwCalHitsOutOfCone_v9leakdata_gam55_conv = [
[ 0.0125, 1.432000e+01,    -1.7294e+00,    6.3045e-02  ], # bin 0  chi2/NDF: 25.304  copied from bin 2  
[ 0.0375, 1.432000e+01,    -1.7294e+00,    6.3045e-02  ], # bin 1  chi2/NDF: 62.133  copied from bin 2  
[ 0.0625, 1.432000e+01,    -1.7294e+00,    6.3045e-02  ], # bin 2  chi2/NDF: 91.689  
[ 0.0875, 1.320500e+01,    -1.5330e+00,    5.4279e-02  ], # bin 3  chi2/NDF: 86.784  
[ 0.1125, 1.500200e+01,    -1.8331e+00,    6.6938e-02  ], # bin 4  chi2/NDF: 57.494  
[ 0.1375, 1.613900e+01,    -2.0489e+00,    7.7126e-02  ], # bin 5  chi2/NDF: 55.59  
[ 0.1625, 1.456700e+01,    -1.7509e+00,    6.3346e-02  ], # bin 6  chi2/NDF: 80.816  
[ 0.1875, 1.341800e+01,    -1.5512e+00,    5.4704e-02  ], # bin 7  chi2/NDF: 62.601  
[ 0.2125, 1.399600e+01,    -1.6457e+00,    5.8606e-02  ], # bin 8  chi2/NDF: 66.201  
[ 0.2375, 1.457000e+01,    -1.7595e+00,    6.4211e-02  ], # bin 9  chi2/NDF: 151.17  
[ 0.2625, 1.604600e+01,    -1.9874e+00,    7.3046e-02  ], # bin 10  chi2/NDF: 34.831  
[ 0.2875, 1.225500e+01,    -1.3421e+00,    4.5772e-02  ], # bin 11  chi2/NDF: 79.801  
[ 0.3125, 1.282200e+01,    -1.4140e+00,    4.8055e-02  ], # bin 12  chi2/NDF: 72.848  
[ 0.3375, 1.403700e+01,    -1.6035e+00,    5.5571e-02  ], # bin 13  chi2/NDF: 48.636  
[ 0.3625, 1.375100e+01,    -1.5514e+00,    5.3418e-02  ], # bin 14  chi2/NDF: 81.72  
[ 0.3875, 1.391900e+01,    -1.5697e+00,    5.3945e-02  ], # bin 15  chi2/NDF: 102.01  
[ 0.4125, 1.428700e+01,    -1.6226e+00,    5.6032e-02  ], # bin 16  chi2/NDF: 77.311  
[ 0.4375, 1.242200e+01,    -1.3000e+00,    4.2459e-02  ], # bin 17  chi2/NDF: 93.383  
[ 0.4625, 1.301100e+01,    -1.3673e+00,    4.4298e-02  ], # bin 18  chi2/NDF: 120.35  
[ 0.4875, 1.218800e+01,    -1.2339e+00,    3.9174e-02  ], # bin 19  chi2/NDF: 49.331  
[ 0.5125, 1.479500e+01,    -1.6389e+00,    5.5079e-02  ], # bin 20  chi2/NDF: 66.795  
[ 0.5375, 1.541700e+01,    -1.7277e+00,    5.8297e-02  ], # bin 21  chi2/NDF: 20.945  
[ 0.5625, 1.259300e+01,    -1.2916e+00,    4.1606e-02  ], # bin 22  chi2/NDF: 69.132  
[ 0.5875, 1.444300e+01,    -1.5631e+00,    5.1614e-02  ], # bin 23  chi2/NDF: 99.585  
[ 0.6125, 1.710000e+01,    -1.9887e+00,    6.8597e-02  ], # bin 24  chi2/NDF: 81.012  
[ 0.6375, 1.661100e+01,    -1.9213e+00,    6.6250e-02  ], # bin 25  chi2/NDF: 48.359  
[ 0.6625, 1.793200e+01,    -2.1220e+00,    7.4006e-02  ], # bin 26  chi2/NDF: 70.41  
[ 0.6875, 1.886100e+01,    -2.2421e+00,    7.7758e-02  ], # bin 27  chi2/NDF: 51.518  
[ 0.7125, 1.943200e+01,    -2.3449e+00,    8.2353e-02  ], # bin 28  chi2/NDF: 68.627  
[ 0.7375, 1.992100e+01,    -2.4325e+00,    8.6221e-02  ], # bin 29  chi2/NDF: 97.691  
[ 0.7625, 1.992100e+01,    -2.4325e+00,    8.6221e-02  ], # bin 30  chi2/NDF: 47.622  copied from bin 29  
[ 0.7875, 1.992100e+01,    -2.4325e+00,    8.6221e-02  ], # bin 31  chi2/NDF: 57.749  copied from bin 29  
[ 0.8125, 3.642100e+01,    -4.3521e+00,    1.4445e-01  ], # bin 32  chi2/NDF: 57.987  copied from bin 34  
[ 0.8375, 3.642100e+01,    -4.3521e+00,    1.4445e-01  ], # bin 33  chi2/NDF: 166.17  copied from bin 34  
[ 0.8625, 3.642100e+01,    -4.3521e+00,    1.4445e-01  ], # bin 34  chi2/NDF: 100.89  
[ 0.8875, 3.447300e+01,    -4.0914e+00,    1.3571e-01  ], # bin 35  chi2/NDF: 110.69  
[ 0.9125, 3.069200e+01,    -3.5517e+00,    1.1655e-01  ], # bin 36  chi2/NDF: 115.97  
[ 0.9375, 3.169200e+01,    -3.7205e+00,    1.2330e-01  ], # bin 37  chi2/NDF: 56.162  
[ 0.9625, 3.032300e+01,    -3.5494e+00,    1.1830e-01  ], # bin 38  chi2/NDF: 101.77  
[ 0.9875, 3.206200e+01,    -3.8176e+00,    1.2852e-01  ], # bin 39  chi2/NDF: 85.347  
[ 1.0125, 3.077000e+01,    -3.6581e+00,    1.2403e-01  ], # bin 40  chi2/NDF: 112.71  
[ 1.0375, 3.069800e+01,    -3.6367e+00,    1.2260e-01  ], # bin 41  chi2/NDF: 190.41  
[ 1.0625, 3.145400e+01,    -3.7912e+00,    1.3009e-01  ], # bin 42  chi2/NDF: 64.276  
[ 1.0875, 3.271800e+01,    -3.9925e+00,    1.3780e-01  ], # bin 43  chi2/NDF: 49.051  
[ 1.1125, 3.128700e+01,    -3.8036e+00,    1.3201e-01  ], # bin 44  chi2/NDF: 36.522  
[ 1.1375, 3.206400e+01,    -3.9432e+00,    1.3786e-01  ], # bin 45  chi2/NDF: 36.543  
[ 1.1625, 3.439200e+01,    -4.2745e+00,    1.5006e-01  ], # bin 46  chi2/NDF: 43.925  
[ 1.1875, 3.423500e+01,    -4.2239e+00,    1.4702e-01  ], # bin 47  chi2/NDF: 54.534  
[ 1.2125, 3.636700e+01,    -4.5356e+00,    1.5906e-01  ], # bin 48  chi2/NDF: 78.913  
[ 1.2375, 3.692600e+01,    -4.5717e+00,    1.5842e-01  ], # bin 49  chi2/NDF: 57.735  
[ 1.2625, 3.878300e+01,    -4.8304e+00,    1.6809e-01  ], # bin 50  chi2/NDF: 28.442  
[ 1.2875, 4.222400e+01,    -5.2772e+00,    1.8185e-01  ], # bin 51  chi2/NDF: 20.562  
[ 1.3125, 4.607500e+01,    -5.8157e+00,    2.0115e-01  ], # bin 52  chi2/NDF: 31.39  
[ 1.3375, 5.097900e+01,    -6.5109e+00,    2.2464e-01  ], # bin 53  chi2/NDF: 17.301  
[ 1.3625, 5.418300e+01,    -6.9480e+00,    2.4269e-01  ], # bin 54  chi2/NDF: 34.943  
[ 1.3875, 6.432200e+01,    -8.7520e+00,    3.1397e-01  ], # bin 55  chi2/NDF: 112.18  
[ 1.4125, 5.895300e+01,    -8.0530e+00,    2.9179e-01  ], # bin 56  chi2/NDF: 55.787  
[ 1.4375, 2.000000e-01,    0.0000e+00,    0.0000e+00  ], # bin 57 crack  chi2/NDF: 0  
[ 1.4625, 2.000000e-01,    0.0000e+00,    0.0000e+00  ], # bin 58 crack  chi2/NDF: 0  
[ 1.4875, 6.000000e-01,    0.0000e+00,    0.0000e+00  ], # bin 59 crack  chi2/NDF: 0  
[ 1.5125, -1.459300e+01,    2.1498e-01,    2.5618e+02  ], # bin 60 crack  chi2/NDF: 124.77  
[ 1.5375, -3.508400e+01,    1.0865e+00,    3.4647e+02  ], # bin 61 crack  chi2/NDF: 58.518  
[ 1.5625, -3.200500e+01,    8.8934e-01,    3.5298e+02  ], # bin 62  chi2/NDF: 45.34  
[ 1.5875, -3.365200e+01,    1.0476e+00,    3.3222e+02  ], # bin 63  chi2/NDF: 36.385  
[ 1.6125, -2.761600e+01,    8.7298e-01,    2.8243e+02  ], # bin 64  chi2/NDF: 45.928  
[ 1.6375, -2.053900e+01,    6.5453e-01,    2.2608e+02  ], # bin 65  chi2/NDF: 36.02  
[ 1.6625, -2.001700e+01,    6.5031e-01,    2.2290e+02  ], # bin 66  chi2/NDF: 28.772  
[ 1.6875, -2.050100e+01,    6.6364e-01,    2.3044e+02  ], # bin 67  chi2/NDF: 14.065  
[ 1.7125, -2.060500e+01,    6.6920e-01,    2.3287e+02  ], # bin 68  chi2/NDF: 42.061  
[ 1.7375, -1.504800e+01,    5.4228e-01,    1.7527e+02  ], # bin 69  chi2/NDF: 25.776  
[ 1.7625, -1.130900e+01,    4.4440e-01,    1.4242e+02  ], # bin 70  chi2/NDF: 64.66  
[ 1.7875, -2.318500e+00,    1.5226e-01,    7.4606e+01  ], # bin 71  chi2/NDF: 15.182  
[ 1.8125, 1.817700e+00,    1.4462e-02,    4.6662e+01  ], # bin 72  chi2/NDF: 20.056  
[ 1.8375, -2.584000e-02,    8.0473e-02,    6.0130e+01  ], # bin 73  chi2/NDF: 13.108  
[ 1.8625, 3.008000e+00,    -1.2784e-02,    3.7317e+01  ], # bin 74  chi2/NDF: 7.6726  
[ 1.8875, 1.777300e+00,    4.0656e-02,    4.5674e+01  ], # bin 75  chi2/NDF: 7.2866  
[ 1.9125, 6.661700e+00,    -1.2311e-01,    1.1824e+01  ], # bin 76  chi2/NDF: 45.977  
[ 1.9375, 2.954800e+00,    2.2355e-02,    3.7119e+01  ], # bin 77  chi2/NDF: 24.329  
[ 1.9625, 4.753500e+00,    -3.5399e-02,    2.6003e+01  ], # bin 78  chi2/NDF: 18.043  
[ 1.9875, 1.382850e+00,    9.6793e-02,    4.9269e+01  ], # bin 79  chi2/NDF: 30.842  
[ 2.0125, 4.274700e+00,    3.5686e-03,    3.0100e+01  ], # bin 80  chi2/NDF: 17.505  
[ 2.0375, 4.838500e+00,    -1.6578e-02,    2.8518e+01  ], # bin 81  chi2/NDF: 21.793  
[ 2.0625, -1.916000e-02,    1.6584e-01,    6.3675e+01  ], # bin 82  chi2/NDF: 13.684  
[ 2.0875, 1.404600e+00,    1.1359e-01,    5.6798e+01  ], # bin 83  chi2/NDF: 37.39  
[ 2.1125, -6.612500e+00,    4.0992e-01,    1.1239e+02  ], # bin 84  chi2/NDF: 69.863  
[ 2.1375, -5.603300e+00,    3.7549e-01,    1.0726e+02  ], # bin 85  chi2/NDF: 59.575  
[ 2.1625, -1.239500e+01,    6.2420e-01,    1.5611e+02  ], # bin 86  chi2/NDF: 66.018  
[ 2.1875, -9.885000e+00,    5.4130e-01,    1.3955e+02  ], # bin 87  chi2/NDF: 97.27  
[ 2.2125, -8.805200e+00,    5.1083e-01,    1.3322e+02  ], # bin 88  chi2/NDF: 45.557  
[ 2.2375, -7.450400e+00,    4.7443e-01,    1.2293e+02  ], # bin 89  chi2/NDF: 33.451  
[ 2.2625, -5.848600e+00,    4.1573e-01,    1.1545e+02  ], # bin 90  chi2/NDF: 18.891  
[ 2.2875, -1.329900e+01,    6.7043e-01,    1.7262e+02  ], # bin 91  chi2/NDF: 73.909  
[ 2.3125, -1.282500e+01,    6.3958e-01,    1.7671e+02  ], # bin 92  chi2/NDF: 12.158  
[ 2.3375, -1.652000e+01,    7.5693e-01,    2.0729e+02  ], # bin 93  chi2/NDF: 42.859  
[ 2.3625, -2.572400e+01,    1.0516e+00,    2.8340e+02  ], # bin 94  chi2/NDF: 73.855  
[ 2.3875, -1.479400e+01,    6.8962e-01,    2.0177e+02  ], # bin 95  chi2/NDF: 203.63  
[ 2.4125, -3.869300e+01,    1.4780e+00,    3.8627e+02  ], # bin 96  chi2/NDF: 133.34  
[ 2.4375, -3.986700e+01,    1.5097e+00,    3.9964e+02  ], # bin 97  chi2/NDF: 133.54  
[ 2.4625, 1.505200e+01,    -6.6769e-01,    8.0165e+01  ], # bin 98  chi2/NDF: 436.6  
[ 2.4875, -1.844400e+02,    6.1401e+00,    1.5924e+03  ], # bin 99  chi2/NDF: 314.46  
] # end CaloSwCalHitsOutOfCone_v9_gam55_conv


CaloSwCalHitsOutOfCone_v9leakdata_gam55_unconv = [
[ 0.0125, -2.461700e+00,    9.5391e-01,    -4.3266e-02  ], # bin 0  chi2/NDF: 762.12  copied from bin 2  
[ 0.0375, -2.461700e+00,    9.5391e-01,    -4.3266e-02  ], # bin 1  chi2/NDF: 642.49  copied from bin 2  
[ 0.0625, -2.461700e+00,    9.5391e-01,    -4.3266e-02  ], # bin 2  chi2/NDF: 1779.4  
[ 0.0875, 2.481000e+00,    1.0344e-01,    -7.1898e-03  ], # bin 3  chi2/NDF: 636.62  
[ 0.1125, 1.782000e+00,    2.3701e-01,    -1.3396e-02  ], # bin 4  chi2/NDF: 370.1  
[ 0.1375, 9.037000e-01,    3.8236e-01,    -1.9367e-02  ], # bin 5  chi2/NDF: 393.38  
[ 0.1625, 7.346300e-01,    4.1319e-01,    -2.0763e-02  ], # bin 6  chi2/NDF: 555.79  
[ 0.1875, -1.894400e-01,    5.7489e-01,    -2.7559e-02  ], # bin 7  chi2/NDF: 1129.5  
[ 0.2125, 6.170900e+00,    -5.7590e-01,    2.4047e-02  ], # bin 8  chi2/NDF: 1052.9  
[ 0.2375, 1.274800e+00,    3.1492e-01,    -1.6236e-02  ], # bin 9  chi2/NDF: 393.56  
[ 0.2625, -7.722400e-01,    6.5790e-01,    -3.0238e-02  ], # bin 10  chi2/NDF: 916.2  
[ 0.2875, 1.414550e-01,    5.1493e-01,    -2.4859e-02  ], # bin 11  chi2/NDF: 459.93  
[ 0.3125, -6.672600e-01,    6.3778e-01,    -2.9225e-02  ], # bin 12  chi2/NDF: 472.85  
[ 0.3375, 4.151300e-01,    4.3427e-01,    -1.9811e-02  ], # bin 13  chi2/NDF: 1183.4  
[ 0.3625, 1.044370e+00,    3.2720e-01,    -1.5217e-02  ], # bin 14  chi2/NDF: 1479.2  
[ 0.3875, -5.874000e-02,    5.1343e-01,    -2.2999e-02  ], # bin 15  chi2/NDF: 701.09  
[ 0.4125, 1.530700e-01,    4.7016e-01,    -2.0654e-02  ], # bin 16  chi2/NDF: 1803.2  
[ 0.4375, 3.840600e-01,    4.3145e-01,    -1.9077e-02  ], # bin 17  chi2/NDF: 1058.2  
[ 0.4625, 2.284000e+00,    1.1566e-01,    -6.0061e-03  ], # bin 18  chi2/NDF: 247.85  
[ 0.4875, 6.969200e-01,    3.6430e-01,    -1.5403e-02  ], # bin 19  chi2/NDF: 660.27  
[ 0.5125, 4.085600e+00,    -2.0619e-01,    8.3972e-03  ], # bin 20  chi2/NDF: 6568.7  
[ 0.5375, 2.153900e+00,    1.2520e-01,    -5.6445e-03  ], # bin 21  chi2/NDF: 1095.9  
[ 0.5625, 2.329800e+00,    1.1494e-01,    -5.9467e-03  ], # bin 22  chi2/NDF: 1046  
[ 0.5875, -7.907200e-01,    5.9075e-01,    -2.3641e-02  ], # bin 23  chi2/NDF: 2876.6  
[ 0.6125, 2.958600e+00,    3.1527e-02,    -3.2016e-03  ], # bin 24  chi2/NDF: 430.57  
[ 0.6375, 3.770600e+00,    -1.0262e-01,    2.2358e-03  ], # bin 25  chi2/NDF: 730.21  
[ 0.6625, 3.493800e+00,    -3.9126e-02,    -8.5343e-04  ], # bin 26  chi2/NDF: 719.09  
[ 0.6875, 4.347700e+00,    -1.7683e-01,    4.5987e-03  ], # bin 27  chi2/NDF: 481.76  
[ 0.7125, 4.110300e+00,    -1.3450e-01,    2.8149e-03  ], # bin 28  chi2/NDF: 615.25  
[ 0.7375, 5.838400e+00,    -4.2502e-01,    1.4915e-02  ], # bin 29  chi2/NDF: 621.26  
[ 0.7625, 5.838400e+00,    -4.2502e-01,    1.4915e-02  ], # bin 30  chi2/NDF: 112.27  copied from bin 29  
[ 0.7875, 5.838400e+00,    -4.2502e-01,    1.4915e-02  ], # bin 31  chi2/NDF: 260.83  copied from bin 29  
[ 0.8125, 7.627700e+00,    -3.9349e-01,    8.0571e-03  ], # bin 32  chi2/NDF: 41.683  copied from bin 34  
[ 0.8375, 7.627700e+00,    -3.9349e-01,    8.0571e-03  ], # bin 33  chi2/NDF: 86.276  copied from bin 34  
[ 0.8625, 7.627700e+00,    -3.9349e-01,    8.0571e-03  ], # bin 34  chi2/NDF: 76.765  
[ 0.8875, 8.307200e+00,    -5.1571e-01,    1.3381e-02  ], # bin 35  chi2/NDF: 11.983  
[ 0.9125, 6.254800e+00,    -2.0383e-01,    1.7023e-03  ], # bin 36  chi2/NDF: 19.065  
[ 0.9375, 5.834300e+00,    -1.6025e-01,    8.2730e-04  ], # bin 37  chi2/NDF: 459.06  
[ 0.9625, 6.727500e+00,    -2.8473e-01,    5.1794e-03  ], # bin 38  chi2/NDF: 24.885  
[ 0.9875, 7.594400e+00,    -4.1994e-01,    1.0281e-02  ], # bin 39  chi2/NDF: 14.274  
[ 1.0125, 7.032700e+00,    -3.2761e-01,    6.8162e-03  ], # bin 40  chi2/NDF: 74.485  
[ 1.0375, 1.096000e+01,    -9.4007e-01,    3.0294e-02  ], # bin 41  chi2/NDF: 365.58  
[ 1.0625, 9.493800e+00,    -6.9768e-01,    2.0648e-02  ], # bin 42  chi2/NDF: 13.009  
[ 1.0875, 9.199200e+00,    -6.4543e-01,    1.8326e-02  ], # bin 43  chi2/NDF: 243.91  
[ 1.1125, 9.866600e+00,    -7.7543e-01,    2.4784e-02  ], # bin 44  chi2/NDF: 54.968  
[ 1.1375, 1.023900e+01,    -8.1199e-01,    2.5066e-02  ], # bin 45  chi2/NDF: 290.94  
[ 1.1625, 1.336900e+01,    -1.2829e+00,    4.3163e-02  ], # bin 46  chi2/NDF: 559.45  
[ 1.1875, 1.350700e+01,    -1.2829e+00,    4.2144e-02  ], # bin 47  chi2/NDF: 161.16  
[ 1.2125, 1.238800e+01,    -1.0764e+00,    3.3591e-02  ], # bin 48  chi2/NDF: 224.51  
[ 1.2375, 1.431600e+01,    -1.3523e+00,    4.3126e-02  ], # bin 49  chi2/NDF: 298.07  
[ 1.2625, 1.566300e+01,    -1.5141e+00,    4.8074e-02  ], # bin 50  chi2/NDF: 111.43  
[ 1.2875, 1.885200e+01,    -1.9770e+00,    6.4547e-02  ], # bin 51  chi2/NDF: 150.03  
[ 1.3125, 2.161700e+01,    -2.3395e+00,    7.6570e-02  ], # bin 52  chi2/NDF: 87.217  
[ 1.3375, 2.541600e+01,    -2.8595e+00,    9.2938e-02  ], # bin 53  chi2/NDF: 11.937  
[ 1.3625, 2.535500e+01,    -2.7450e+00,    8.8211e-02  ], # bin 54  chi2/NDF: 116.16  
[ 1.3875, 3.398900e+01,    -4.1493e+00,    1.3687e-01  ], # bin 55  chi2/NDF: 398.67  
[ 1.4125, 3.067700e+01,    -3.7104e+00,    1.2262e-01  ], # bin 56  chi2/NDF: 118.01  
[ 1.4375, 2.000000e-01,    0.0000e+00,    0.0000e+00  ], # bin 57 crack  chi2/NDF: 0  
[ 1.4625, 2.000000e-01,    0.0000e+00,    0.0000e+00  ], # bin 58 crack  chi2/NDF: 0  
[ 1.4875, 6.000000e-01,    0.0000e+00,    0.0000e+00  ], # bin 59 crack  chi2/NDF: 0  
[ 1.5125, -1.643100e+01,    3.8377e-01,    2.2272e+02  ], # bin 60 crack  chi2/NDF: 235.6  
[ 1.5375, -1.501700e+01,    4.9282e-01,    1.7035e+02  ], # bin 61 crack  chi2/NDF: 90.373  
[ 1.5625, -6.879500e+00,    2.5500e-01,    1.0421e+02  ], # bin 62  chi2/NDF: 25.255  
[ 1.5875, -6.126000e+00,    2.6335e-01,    8.9650e+01  ], # bin 63  chi2/NDF: 214.22  
[ 1.6125, -7.085000e-01,    8.4071e-02,    5.1290e+01  ], # bin 64  chi2/NDF: 178.28  
[ 1.6375, -6.416200e+00,    2.8284e-01,    9.3760e+01  ], # bin 65  chi2/NDF: 105.79  
[ 1.6625, -3.542300e+00,    1.7416e-01,    7.8616e+01  ], # bin 66  chi2/NDF: 88.683  
[ 1.6875, -8.255900e+00,    3.3106e-01,    1.1694e+02  ], # bin 67  chi2/NDF: 34.961  
[ 1.7125, -7.867800e+00,    3.4983e-01,    1.0860e+02  ], # bin 68  chi2/NDF: 25.327  
[ 1.7375, 2.932100e+00,    1.2704e-02,    2.3355e+01  ], # bin 69  chi2/NDF: 504.19  
[ 1.7625, 3.371500e-01,    1.1453e-01,    4.0989e+01  ], # bin 70  chi2/NDF: 486.16  
[ 1.7875, 4.313200e+00,    -7.3257e-03,    1.0455e+01  ], # bin 71  chi2/NDF: 197.16  
[ 1.8125, 1.779500e+01,    -4.5824e-01,    -8.7547e+01  ], # bin 72  chi2/NDF: 118.76  
[ 1.8375, 1.674600e+01,    -4.2124e-01,    -7.8762e+01  ], # bin 73  chi2/NDF: 43.74  
[ 1.8625, 1.368800e+01,    -3.0681e-01,    -5.6047e+01  ], # bin 74  chi2/NDF: 151.67  
[ 1.8875, 2.706700e+01,    -7.8723e-01,    -1.4674e+02  ], # bin 75  chi2/NDF: 279  
[ 1.9125, 2.007100e+01,    -5.3555e-01,    -9.5955e+01  ], # bin 76  chi2/NDF: 55.907  
[ 1.9375, 2.121000e+01,    -5.6398e-01,    -1.0408e+02  ], # bin 77  chi2/NDF: 77.036  
[ 1.9625, 1.934400e+01,    -4.8348e-01,    -9.1530e+01  ], # bin 78  chi2/NDF: 46.998  
[ 1.9875, 2.029500e+01,    -5.0974e-01,    -9.7746e+01  ], # bin 79  chi2/NDF: 223.98  
[ 2.0125, 2.208500e+01,    -5.4874e-01,    -1.1206e+02  ], # bin 80  chi2/NDF: 135.62  
[ 2.0375, 2.059800e+01,    -5.0545e-01,    -9.7576e+01  ], # bin 81  chi2/NDF: 123.93  
[ 2.0625, 1.884700e+01,    -4.2771e-01,    -8.5650e+01  ], # bin 82  chi2/NDF: 56.342  
[ 2.0875, 1.733700e+01,    -3.7205e-01,    -7.3120e+01  ], # bin 83  chi2/NDF: 96.93  
[ 2.1125, 2.095300e+01,    -4.9352e-01,    -9.7428e+01  ], # bin 84  chi2/NDF: 71.271  
[ 2.1375, 1.862600e+01,    -4.0861e-01,    -7.9360e+01  ], # bin 85  chi2/NDF: 79.298  
[ 2.1625, 1.997100e+01,    -4.4023e-01,    -8.9502e+01  ], # bin 86  chi2/NDF: 43.225  
[ 2.1875, 2.301900e+01,    -5.3693e-01,    -1.1049e+02  ], # bin 87  chi2/NDF: 81.414  
[ 2.2125, 1.939600e+01,    -4.0440e-01,    -8.3372e+01  ], # bin 88  chi2/NDF: 33.438  
[ 2.2375, 1.876900e+01,    -3.8985e-01,    -7.5371e+01  ], # bin 89  chi2/NDF: 18.321  
[ 2.2625, 2.326600e+01,    -5.2926e-01,    -1.0792e+02  ], # bin 90  chi2/NDF: 54.257  
[ 2.2875, 2.254000e+01,    -4.8946e-01,    -1.0384e+02  ], # bin 91  chi2/NDF: 38.222  
[ 2.3125, 2.048500e+01,    -3.9971e-01,    -9.0723e+01  ], # bin 92  chi2/NDF: 40.204  
[ 2.3375, 1.918000e+01,    -3.5340e-01,    -7.8308e+01  ], # bin 93  chi2/NDF: 73.124  
[ 2.3625, 3.261800e+01,    -7.8768e-01,    -1.7955e+02  ], # bin 94  chi2/NDF: 51.973  
[ 2.3875, 2.072100e+01,    -4.0633e-01,    -8.4734e+01  ], # bin 95  chi2/NDF: 166  
[ 2.4125, 2.144800e+01,    -4.2237e-01,    -8.9479e+01  ], # bin 96  chi2/NDF: 98.158  
[ 2.4375, 3.268100e+01,    -8.0174e-01,    -1.6915e+02  ], # bin 97  chi2/NDF: 45.829  
[ 2.4625, -6.216400e+00,    4.4275e-01,    1.3714e+02  ], # bin 98  chi2/NDF: 183.22  
[ 2.4875, -1.864000e+01,    9.2693e-01,    2.6862e+02  ], # bin 99  chi2/NDF: 207.03  
] # end CaloSwCalHitsOutOfCone_v9_gam55_unconv
#######################################################################
# Corrections for 3x5 cluster size
#######################################################################

CaloSwCalibHitsCalibration_v9leakdata_ele35 = [
      CaloSwCalHitsAccordion_v9leakdata_35,
      CaloSwCalHitsOutOfCone_v9leakdata_ele35,
      CaloSwCalHitsLongLeakage_v9leakdata_ele,
      CaloSwCalHitsEfront_offset_v9leakdata_ele,
      CaloSwCalHitsEfront_slope_v9leakdata_ele,
      CaloSwCalHitsEfront_P2_v9leakdata_ele,
]

CaloSwCalibHitsCalibration_v9leakdata_gam35_conv = [
      CaloSwCalHitsAccordion_v9leakdata_35,
      CaloSwCalHitsOutOfCone_v9leakdata_gam35_conv,
      CaloSwCalHitsLongLeakage_v9leakdata_gam_conv,
      CaloSwCalHitsEfront_offset_v9leakdata_gam_conv,
      CaloSwCalHitsEfront_slope_v9leakdata_gam_conv,
      CaloSwCalHitsEfront_P2_v9leakdata_gam_conv,
]

CaloSwCalibHitsCalibration_v9leakdata_gam35_unconv = [
      CaloSwCalHitsAccordion_v9leakdata_35,
      CaloSwCalHitsOutOfCone_v9leakdata_gam35_unconv,
      CaloSwCalHitsLongLeakage_v9leakdata_gam_unconv,
      CaloSwCalHitsEfront_offset_v9leakdata_gam_unconv,
      CaloSwCalHitsEfront_slope_v9leakdata_gam_unconv,
      CaloSwCalHitsEfront_P2_v9leakdata_gam_unconv,
]

#######################################################################
# Corrections for 3x7 cluster size
#######################################################################

CaloSwCalibHitsCalibration_v9leakdata_ele37 = [
      CaloSwCalHitsAccordion_v9leakdata_37,
      CaloSwCalHitsOutOfCone_v9leakdata_ele37,
      CaloSwCalHitsLongLeakage_v9leakdata_ele,
      CaloSwCalHitsEfront_offset_v9leakdata_ele,
      CaloSwCalHitsEfront_slope_v9leakdata_ele,
      CaloSwCalHitsEfront_P2_v9leakdata_ele,
]

CaloSwCalibHitsCalibration_v9leakdata_gam37_conv = [
      CaloSwCalHitsAccordion_v9leakdata_37,
      CaloSwCalHitsOutOfCone_v9leakdata_gam37_conv,
      CaloSwCalHitsLongLeakage_v9leakdata_gam_conv,
      CaloSwCalHitsEfront_offset_v9leakdata_gam_conv,
      CaloSwCalHitsEfront_slope_v9leakdata_gam_conv,
      CaloSwCalHitsEfront_P2_v9leakdata_gam_conv,
]

CaloSwCalibHitsCalibration_v9leakdata_gam37_unconv = [
      CaloSwCalHitsAccordion_v9leakdata_37,
      CaloSwCalHitsOutOfCone_v9leakdata_gam37_unconv,
      CaloSwCalHitsLongLeakage_v9leakdata_gam_unconv,
      CaloSwCalHitsEfront_offset_v9leakdata_gam_unconv,
      CaloSwCalHitsEfront_slope_v9leakdata_gam_unconv,
      CaloSwCalHitsEfront_P2_v9leakdata_gam_unconv,
]

#######################################################################
# Corrections for 3x5 cluster size
#######################################################################

CaloSwCalibHitsCalibration_v9leakdata_ele55 = [
      CaloSwCalHitsAccordion_v9leakdata_55,
      CaloSwCalHitsOutOfCone_v9leakdata_ele55,
      CaloSwCalHitsLongLeakage_v9leakdata_ele,
      CaloSwCalHitsEfront_offset_v9leakdata_ele,
      CaloSwCalHitsEfront_slope_v9leakdata_ele,
      CaloSwCalHitsEfront_P2_v9leakdata_ele,
]

CaloSwCalibHitsCalibration_v9leakdata_gam55_conv = [
      CaloSwCalHitsAccordion_v9leakdata_55,
      CaloSwCalHitsOutOfCone_v9leakdata_gam55_conv,
      CaloSwCalHitsLongLeakage_v9leakdata_gam_conv,
      CaloSwCalHitsEfront_offset_v9leakdata_gam_conv,
      CaloSwCalHitsEfront_slope_v9leakdata_gam_conv,
      CaloSwCalHitsEfront_P2_v9leakdata_gam_conv,
]

CaloSwCalibHitsCalibration_v9leakdata_gam55_unconv = [
      CaloSwCalHitsAccordion_v9leakdata_55,
      CaloSwCalHitsOutOfCone_v9leakdata_gam55_unconv,
      CaloSwCalHitsLongLeakage_v9leakdata_gam_unconv,
      CaloSwCalHitsEfront_offset_v9leakdata_gam_unconv,
      CaloSwCalHitsEfront_slope_v9leakdata_gam_unconv,
      CaloSwCalHitsEfront_P2_v9leakdata_gam_unconv,
]

samplings_depths = [
[ 0.0125 , 1.60013 , 3.80006 , 14.0009 , 22.8718 ],
[ 0.0375 , 1.60113 , 3.80056 , 14.0077 , 22.8861 ],
[ 0.0625 , 1.60313 , 3.80156 , 14.0215 , 22.9147 ],
[ 0.0875 , 1.60613 , 3.80306 , 14.0421 , 22.9576 ],
[ 0.1125 , 1.61014 , 3.80507 , 14.0697 , 23.0149 ],
[ 0.1375 , 1.61515 , 3.80757 , 14.1041 , 23.0865 ],
[ 0.1625 , 1.62117 , 3.81059 , 14.1456 , 23.1726 ],
[ 0.1875 , 1.62821 , 3.8141  , 14.1939 , 23.2732 ],
[ 0.2125 , 1.63626 , 3.81813 , 14.2493 , 23.3883 ],
[ 0.2375 , 1.64534 , 3.82267 , 14.3117 , 23.518 ],
[ 0.2625 , 1.65544 , 3.82772 , 14.3812 , 23.6625 ],
[ 0.2875 , 1.66658 , 3.83329 , 14.4577 , 23.8217 ],
[ 0.3125 , 1.67876 , 3.83938 , 14.5415 , 23.9958 ],
[ 0.3375 , 1.69199 , 3.846   , 14.6325 , 24.1849 ],
[ 0.3625 , 1.70628 , 3.85314 , 14.7307 , 24.3892 ],
[ 0.3875 , 1.72164 , 3.86082 , 14.8362 , 24.6086 ],
[ 0.4125 , 1.73807 , 3.86903 , 14.9492 , 24.8435 ],
[ 0.4375 , 1.75558 , 3.87779 , 15.0696 , 25.0939 ],
[ 0.4625 , 1.7742  , 3.8871  , 15.1976 , 25.3599 ],
[ 0.4875 , 1.79392 , 3.89696 , 15.3332 , 25.6418 ],
[ 0.5125 , 1.81476 , 3.90738 , 15.5    , 25.9633 ],
[ 0.5375 , 1.83674 , 3.91837 , 15.5    , 26.1263 ],
[ 0.5625 , 1.85987 , 3.92993 , 15.5    , 26.2979 ],
[ 0.5875 , 1.88416 , 3.94208 , 15.5    , 26.4781 ],
[ 0.6125 , 1.90963 , 3.95481 , 15.5    , 26.667 ],
[ 0.6375 , 1.93629 , 3.96814 , 15.5    , 26.8648 ],
[ 0.6625 , 1.96416 , 3.98208 , 15.5    , 27.0716 ],
[ 0.6875 , 1.99326 , 3.99663 , 15.5    , 27.2875 ],
[ 0.7125 , 2.0236  , 4.0118  , 15.5    , 27.5126 ],
[ 0.7375 , 2.05521 , 4.0276  , 15.5    , 27.7471 ],
[ 0.7625 , 2.0881  , 4.04405 , 15.5    , 27.9911 ],
[ 0.7875 , 2.1223  , 4.06115 , 15.5    , 28.2448 ],
[ 0.8125 , 2.15783 , 4.07891 , 15.5    , 25.4314 ],
[ 0.8375 , 2.1947  , 4.09735 , 15.5    , 25.6524 ],
[ 0.8625 , 2.23295 , 4.11647 , 15.5    , 25.8816 ],
[ 0.8875 , 2.27259 , 4.13629 , 15.5    , 26.1192 ],
[ 0.9125 , 2.31365 , 4.15682 , 15.5    , 26.3653 ],
[ 0.9375 , 2.35616 , 4.17808 , 15.5    , 26.62 ],
[ 0.9625 , 2.40014 , 4.20007 , 15.5    , 26.8836 ],
[ 0.9875 , 2.44562 , 4.22281 , 15.5    , 27.1561 ],
[ 1.0125 , 2.49263 , 4.24631 , 15.5    , 27.4378 ],
[ 1.0375 , 2.54119 , 4.2706  , 15.5    , 27.7289 ],
[ 1.0625 , 2.59135 , 4.29567 , 15.5    , 28.0295 ],
[ 1.0875 , 2.64312 , 4.32156 , 15.5    , 28.3398 ],
[ 1.1125 , 2.69655 , 4.34828 , 15.5    , 28.6599 ],
[ 1.1375 , 2.75166 , 4.37583 , 15.5    , 28.9902 ],
[ 1.1625 , 2.8085  , 4.40425 , 15.5    , 29.3308 ],
[ 1.1875 , 2.86709 , 4.43354 , 15.5    , 29.6819 ],
[ 1.2125 , 2.92747 , 4.46373 , 15.5    , 30.0438 ],
[ 1.2375 , 2.98968 , 4.49484 , 15.5    , 30.4166 ],
[ 1.2625 , 3.05375 , 4.52688 , 15.5    , 30.8006 ],
[ 1.2875 , 3.11974 , 4.55987 , 15.5    , 31.196 ],
[ 1.3125 , 3.18768 , 4.59384 , 15.5    , 31.6032 ],
[ 1.3375 , 3.25761 , 4.6288  , 15.5    , 32.0222 ],
[ 1.3625 , 3.32957 , 4.66479 , 15.5    , 32.4535 ],
[ 1.3875 , 3.40362 , 4.70181 , 15.5    , 32.8973 ],
[ 1.4125 , 3.47979 , 4.7399  , 15.5    , 33.3538 ],
[ 1.4375 , 0 , 0 , 0 , 0 ],
[ 1.4625 , 0 , 0 , 0 , 0 ],
[ 1.4875 , 0 , 0 , 0 , 0 ],
[ 1.5125 , 0.0  , 8.85762   , 19.5168 , 30.0369  ], # copied
[ 1.5375 , 0.0  , 8.85762   , 19.5168 , 30.0369  ], # copied
[ 1.5625 , 0.0  , 8.85762   , 19.5168 , 30.0369  ],
[ 1.5875 , 0.0  , 7.24613   , 17.9683 , 28.5757  ],
[ 1.6125 , 0.0  , 7.11249   , 17.9466 , 28.6607  ],
[ 1.6375 , 0.0  , 7.03737   , 17.7263 , 28.4004  ],
[ 1.6625 , 0.0  , 7.44517   , 18.1864 , 28.9547  ],
[ 1.6875 , 0.0  , 7.74611   , 18.5419 , 29.3555  ],
[ 1.7125 , 0.0  , 8.06184   , 18.8982 , 29.8214  ],
[ 1.7375 , 0.0  , 6.90503   , 17.8396 , 28.7947  ],
[ 1.7625 , 0.0  , 6.53893   , 17.5996 , 28.7122  ],
[ 1.7875 , 0.0  , 6.42897   , 17.1062 , 28.3578  ],
[ 1.8125 , 0.0  , 6.41139   , 17.1382 , 28.4191  ],
[ 1.8375 , 0.0  , 5.72471   , 16.4543 , 27.8796  ],
[ 1.8625 , 0.0  , 5.19018   , 16.1606 , 27.7097  ],
[ 1.8875 , 0.0  , 5.04008   , 16.1174 , 27.7649  ],
[ 1.9125 , 0.0  , 5.01225   , 16.184 , 27.9175  ],
[ 1.9375 , 0.0  , 4.95996   , 16.2482 , 28.1358  ],
[ 1.9625 , 0.0  , 4.98242   , 16.3591 , 28.3336  ],
[ 1.9875 , 0.0  , 5.04856   , 16.4194 , 28.5289  ],
[ 2.0125 , 0.0  , 5.32584   , 16.8194 , 29.0619  ],
[ 2.0375 , 0.0  , 5.2971   , 16.8203 , 29.1545  ],
[ 2.0625 , 0.0  , 5.41049   , 17.0891 , 29.5856  ],
[ 2.0875 , 0.0  , 5.48954   , 17.2965 , 29.9571  ],
[ 2.1125 , 0.0  , 5.50876   , 16.8573 , 29.6199  ],
[ 2.1375 , 0.0  , 5.45123   , 16.885 , 29.8321  ],
[ 2.1625 , 0.0  , 5.54906   , 17.1228 , 30.2044  ],
[ 2.1875 , 0.0  , 5.53662   , 17.1833 , 30.4852  ],
[ 2.2125 , 0.0  , 5.46801   , 17.1857 , 30.5842  ],
[ 2.2375 , 0.0  , 5.45581   , 16.6621 , 30.2958  ],
[ 2.2625 , 0.0  , 5.46682   , 16.7899 , 30.5222  ],
[ 2.2875 , 0.0  , 5.52693   , 16.9916 , 30.9557  ],
[ 2.3125 , 0.0  , 5.53211   , 17.0801 , 31.1637  ],
[ 2.3375 , 0.0  , 4.79919   , 16.4387 , 31.5318  ],
[ 2.3625 , 0.0  , 4.80101   , 16.5808 , 31.8533  ],
[ 2.3875 , 0.0  , 4.77726   , 15.9355 , 31.4454  ],
[ 2.4125 , 0.0  , 4.76845   , 16.0665 , 31.8002  ],
[ 2.4375 , 0.0  , 4.5959   , 15.9977 , 32.1175  ],
[ 2.4625 , 0.0  , 4.68513   , 16.2385 , 32.5042  ],
[ 2.4875 , 0.0  , 4.7232   , 16.4144 , 33.028  ]
];

#######################################################################


class CaloSwCalibHitsCalibration_v9leakdata_parms:
    eta_start_crack = 1.425
    eta_end_crack = 1.55
    etamax = 2.5
    use_raw_eta = True
    fix_v6_pathologies = False
    update_sampling_energies = False

    sampling_depth = samplings_depths

    region = CALOCORR_COMBINED2

    correction = {'ele55'        : CaloSwCalibHitsCalibration_v9leakdata_ele55,
                  'ele35'        : CaloSwCalibHitsCalibration_v9leakdata_ele35,
                  'ele37'        : CaloSwCalibHitsCalibration_v9leakdata_ele37,
                  'gam55_conv'   : CaloSwCalibHitsCalibration_v9leakdata_gam55_conv,
                  'gam35_conv'   : CaloSwCalibHitsCalibration_v9leakdata_gam35_conv,
                  'gam37_conv'   : CaloSwCalibHitsCalibration_v9leakdata_gam37_conv,
                  'gam55_unconv' : CaloSwCalibHitsCalibration_v9leakdata_gam55_unconv,
                  'gam35_unconv' : CaloSwCalibHitsCalibration_v9leakdata_gam35_unconv,
                  'gam37_unconv' : CaloSwCalibHitsCalibration_v9leakdata_gam37_unconv,

                   # Use 5x5 for cluster sizes that aren't explicitly derived.
                  'ele33'        : CaloSwCalibHitsCalibration_v9leakdata_ele55,
                  'ele57'        : CaloSwCalibHitsCalibration_v9leakdata_ele55,
                  'ele77'        : CaloSwCalibHitsCalibration_v9leakdata_ele55,
                  'gam33_conv'   : CaloSwCalibHitsCalibration_v9leakdata_gam55_conv,
                  'gam57_conv'   : CaloSwCalibHitsCalibration_v9leakdata_gam55_conv,
                  'gam77_conv'   : CaloSwCalibHitsCalibration_v9leakdata_gam55_conv,
                  'gam33_unconv' : CaloSwCalibHitsCalibration_v9leakdata_gam55_unconv,
                  'gam57_unconv' : CaloSwCalibHitsCalibration_v9leakdata_gam55_unconv,
                  'gam77_unconv' : CaloSwCalibHitsCalibration_v9leakdata_gam55_unconv,
                  }
