# Copyright (C) 2002-2017 CERN for the benefit of the ATLAS collaboration

#
# $Id: CaloSwEtaoff_v4.py,v 1.1 2006-11-10 03:47:26 ssnyder Exp $
#
# File: CaloClusterCorrection/python/CaloSwEtaoff_v4.py
# Created: Nov 2006, sss
# Purpose: Eta offset (s-shape) corrections, v4.
#
# These corrections were derived using 12.0.03 simulation and reconstruction,
# for both electrons and photons, 5x5, 3x5, 3x7 clusters.
#
# This correction was in CaloClusterCorrection-00-02-61, in 12.0.4.
#
# See CaloSwEtaoff_v3.h for more details.
#

from CaloClusterCorrection.common import *
from AthenaCommon.SystemOfUnits import GeV

###############################################################################
# Tables for v4 sshape correction, layer 1, 5x5 electron clusters
#


ele5_layer1_etaoffv4_55_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00036316604,   0.00033194472 ],
  [               0,               0,               0,               0,       3.1457724,        1.693693 ],
  [               0,               0,               0,               0,      -1.8731917,       1.9189105 ],
  [               0,               0,               0,               0,      -5.9478442,       1.7171704 ],
  [               0,               0,               0,               0,      0.54120794,     -0.31416782 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele5_layer1_etaoffv4_55_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer1_etaoffv4_55_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.62745592,      0.50918063 ],
  [               0,               0,               0,               0,    -0.076049132,      0.21558698 ],
  [               0,               0,               0,               0,     -0.66309945,     0.049198092 ],
  [               0,               0,               0,               0,     -0.65261242,     0.046720701 ],
  [               0,               0,               0,               0,      0.26184014,     -0.38055362 ],
  [               0,               0,               0,               0,      -1.4643352,       1.2011958 ],
  [               0,               0,               0,               0,     -0.10077029,      0.03554099 ],
  [               0,               0,               0,               0,        0.114762,       1.4088469 ],
 ],
 [
  [               0,               0,               0,     0.010064971,   -0.0077932963,    0.0019314992 ],
  [               0,               0,               0,       12.424576,      -23.159619,       11.665143 ],
  [               0,               0,               0,        1.952054,      -3.6040659,       1.1821103 ],
  [               0,               0,               0,       18.932457,      -24.497342,       7.3435718 ],
  [               0,               0,               0,       2.5684593,      -2.8463212,      0.63919263 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.41198389,      0.33158288 ],
  [               0,               0,               0,               0,       599.11895,       -475.9058 ],
  [               0,               0,               0,               0,      -354.90172,       281.94302 ],
  [               0,               0,               0,               0,       119.62462,      -95.127484 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,        -55.3964,        9.760889,       76.080725 ],
  [               0,               0,               0,    -0.032748656,       0.2189245,     -0.14560857 ],
  [               0,               0,               0,     0.032792271,     -0.21896194,      0.14568343 ],
  [               0,               0,               0,     0.032478848,     -0.21828292,      0.14523813 ],
  [               0,               0,               0,  -0.00015971159,   0.00030510501,  -0.00017129343 ],
  [               0,               0,               0,    0.0052640884,    -0.011203126,    0.0066071346 ],
  [               0,               0,               0,       1.3551261,      -3.9417143,       3.6224242 ],
  [               0,               0,               0,      -3.1151856,       2.7649101,     -0.24397629 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0010677736,   -0.0011664606,    0.0017081841,    0.0045140647 ],
  [               0,               0,       10.689291,      -218.69898,       304.44864,       13.073684 ],
  [               0,               0,       -19.60109,      -1.7827911,       5.9297525,       53.822274 ],
  [               0,               0,        6.826385,       4.1823329,      -4.2280875,      -23.165731 ],
  [               0,               0,       4.8053093,       2.1216018,      -3.5180177,      -14.944272 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer1_etaoffv4_55_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00046112408 ],
  [               0,               0,               0,               0,               0,    0.0078545729 ],
  [               0,               0,               0,               0,               0,       2.2767694 ],
  [               0,               0,               0,               0,               0,       6.3893681 ],
  [               0,               0,               0,               0,               0,       1.4027317 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer1_etaoffv4_55_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   5.5177688e-05,   0.00028771463 ],
  [               0,               0,               0,               0,      0.61769567,       5.0392343 ],
  [               0,               0,               0,               0,      -304.01432,       79.994235 ],
  [               0,               0,               0,               0,      -298.93681,       78.368908 ],
  [               0,               0,               0,               0,       149.63405,      -94.682717 ],
  [               0,               0,               0,               0,    0.0096859611,     0.096478198 ],
  [               0,               0,               0,               0,     -0.29773722,     0.077375282 ],
  [               0,               0,               0,               0,      0.44694515,       1.2843081 ],
 ],
 [
  [               0,               0,               0,    -0.030518263,   -0.0046070893,     0.055827563 ],
  [               0,               0,               0,      -1.5525982,       1.5365341,      0.10245596 ],
  [               0,               0,               0,       1.6301807,      -1.5981695,    -0.057134209 ],
  [               0,               0,               0,       1.5934553,      -1.5727034,    -0.064471187 ],
  [               0,               0,               0,    -0.035565889,     0.028746209,   -0.0075053916 ],
  [               0,               0,               0,   0.00069645785,   -0.0012280723,    0.0004678505 ],
  [               0,               0,               0,       47.828783,      -59.941431,       22.565517 ],
  [               0,               0,               0,      -135.88512,       171.47915,      -47.771306 ],
 ],
 [
  [               0,               0,               0,               0,      -11.327228,       9.0850945 ],
  [               0,               0,               0,               0,        48.67749,       -38.39338 ],
  [               0,               0,               0,               0,      -56.950611,       44.988192 ],
  [               0,               0,               0,               0,       10.617998,      -8.4447053 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,       2.0426454,      -3.5398508,       1.5685005 ],
  [               0,               0,               0,       5.1754532,      -12.147745,       7.1245468 ],
  [               0,               0,               0,      -5.0099385,       11.735229,       -6.868916 ],
  [               0,               0,               0,      -4.8048151,       11.207725,      -6.5297027 ],
  [               0,               0,               0,     -0.00982907,     0.022532385,    -0.012751835 ],
  [               0,               0,               0,    -0.010420414,     0.020084687,   -0.0097694574 ],
  [               0,               0,               0,      -5.9959082,       9.8022113,      -2.0537335 ],
  [               0,               0,               0,      -2.7111581,       9.6149167,      -6.4237773 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0026741949 ],
  [               0,               0,               0,               0,               0,       3.0836187 ],
  [               0,               0,               0,               0,               0,      -1.7115852 ],
  [               0,               0,               0,               0,               0,     0.082368676 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.041714117,     0.025187486,    -0.022865326,     -0.12960433 ],
  [               0,               0,       138.95675,      -213.68869,      -285.96187,       442.93102 ],
  [               0,               0,       925.35883,      -1332.9738,      -1916.2031,       2758.6874 ],
  [               0,               0,      -359.10104,       499.22772,        701.2443,      -975.48994 ],
  [               0,               0,        70.91591,      -95.940132,      -139.75586,       188.34098 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer1_etaoffv4_55_e = [
 [
  [               0,               0,               0,               0,               0,      0.00130986 ],
  [               0,               0,               0,               0,               0,      -39.174261 ],
  [               0,               0,               0,               0,               0,       1.1256412 ],
  [               0,               0,               0,               0,               0,       4.2254451 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00087256193 ],
  [               0,               0,               0,               0,               0,       39.840099 ],
  [               0,               0,               0,               0,               0,      -4.9331153 ],
  [               0,               0,               0,               0,               0,      -5.3868791 ],
  [               0,               0,               0,               0,               0,     -0.20295202 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,       175063.39 ],
  [               0,               0,               0,               0,               0,    0.0055058367 ],
  [               0,               0,               0,               0,               0,   -0.0055057789 ],
  [               0,               0,               0,               0,               0,   1.7685361e-09 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.066999009 ],
  [               0,               0,               0,               0,               0,      -1.0747876 ],
  [               0,               0,               0,               0,               0,      0.84804786 ],
  [               0,               0,               0,               0,               0,      0.90737341 ],
  [               0,               0,               0,               0,               0,     0.088738128 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.062477537,    -0.089432068,     -0.17435535,      0.25373885 ],
  [               0,               0,       9.1420946,       1.6427592,      -91.706737,       107.37487 ],
  [               0,               0,      -16.372438,       82.102281,      -135.73448,       73.091092 ],
  [               0,               0,       15.501584,      -101.87256,       211.78519,      -142.38525 ],
  [               0,               0,      -7.9469167,       36.437149,      -54.898767,       27.078442 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0015161706,   -0.0021200542 ],
  [               0,               0,               0,               0,      -5.9766588,       13.265732 ],
  [               0,               0,               0,               0,      -2.2753991,       4.7188236 ],
  [               0,               0,               0,               0,      -2.1222771,       4.8007715 ],
  [               0,               0,               0,               0,     -0.42886029,      0.68840868 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.025198056,    -0.060942979,     -0.11911806,      0.29135392 ],
  [               0,               0,      -23.874196,       154.96159,      -328.06683,       228.03557 ],
  [               0,               0,       40.290361,      -230.75241,       434.37777,      -268.56343 ],
  [               0,               0,      -21.681047,        157.5257,      -375.83573,       294.48854 ],
  [               0,               0,       13.034364,      -85.463751,       186.31967,      -135.09407 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       120480.24,      -276441.33 ],
  [               0,               0,               0,               0,    -0.019876127,     0.065964324 ],
  [               0,               0,               0,               0,     0.019688955,    -0.065533324 ],
  [               0,               0,               0,               0,  -0.00032866259,   0.00075828209 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0005626932 ],
  [               0,               0,               0,               0,               0,       5.0823515 ],
  [               0,               0,               0,               0,               0,       1.5422821 ],
  [               0,               0,               0,               0,               0,     -0.10567946 ],
  [               0,               0,               0,               0,               0,     -0.08992569 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0095679175,    -0.062601245,      0.10392355 ],
  [               0,               0,               0,       492.21926,      -2399.2808,        2928.029 ],
  [               0,               0,               0,       76.828758,        -363.344,       427.39145 ],
  [               0,               0,               0,       19.652638,       65.085274,      -268.55114 ],
  [               0,               0,               0,      0.25718629,    -0.017170165,       -1.430905 ],
  [               0,               0,               0,       56.388368,      -280.36746,       348.45753 ],
  [               0,               0,               0,       3.1267874,      -9.1880222,      0.76535425 ],
 ],
 [
  [               0,               0,               0,               0,     0.006193463,    -0.013959896 ],
  [               0,               0,               0,               0,      -4270.6215,       10610.426 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,               0,       349.70065,      -867.29075 ],
  [               0,               0,               0,               0,     -0.22959825,     -0.13983008 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer1_etaoffv4_55_b = [
 [
  [               0,               0,      -62618.448,       756.64179,      -3.2377948,    0.0039930576 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00024863799,   0.00027620924 ],
  [               0,               0,               0,               0,      -6.3823865,       7.5219521 ],
  [               0,               0,               0,               0,      0.27548019,       0.3237314 ],
  [               0,               0,               0,               0,      -3.0451151,       1.0907515 ],
  [               0,               0,               0,               0,     -0.62653329,   -0.0085334775 ],
  [               0,               0,               0,               0,  -7.1884626e-05,   4.2751555e-05 ],
  [               0,               0,               0,               0,       9.4062376,      0.87670955 ],
  [               0,               0,               0,               0,      -11.345538,        4.356147 ],
 ],
 [
  [               0,               0,               0,    0.0097607524,    -0.013773748,    0.0052737294 ],
  [               0,               0,               0,      -46.070198,       53.758733,      -11.962648 ],
  [               0,               0,               0,       32.865013,      -43.763671,       11.566533 ],
  [               0,               0,               0,       35.337418,       -46.44007,       12.157887 ],
  [               0,               0,               0,      -8.6127274,       10.232483,      -3.0317091 ],
  [               0,               0,               0,       0.0124686,    -0.020073953,     0.008975749 ],
  [               0,               0,               0,      0.55123957,       1.2700589,     -0.61071379 ],
  [               0,               0,               0,       2.6883236,      -3.6418895,     -0.43533323 ],
 ],
 [
  [               0,               0,               0,               0,     0.022570008,    0.0070234877 ],
  [               0,               0,               0,               0,       21.822337,      -16.867702 ],
  [               0,               0,               0,               0,      -35.720024,       27.951035 ],
  [               0,               0,               0,               0,       19.229395,      -15.292257 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.010467907,    0.0052781982,   -0.0061990319 ],
  [               0,               0,               0,      -0.9106829,       1.3494726,      0.49612788 ],
  [               0,               0,               0,       0.9437976,      -1.6602229,    -0.039496473 ],
  [               0,               0,               0,        1.462601,      -2.9128081,      0.69266542 ],
  [               0,               0,               0,     0.022613814,    -0.034630783,   -0.0056157109 ],
  [               0,               0,               0,   5.0628268e-05,  -0.00034896639,   0.00033800815 ],
  [               0,               0,               0,      -2.4713284,       12.225901,      -7.6527472 ],
  [               0,               0,               0,      -3.4892293,       6.2552537,      0.65495548 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0043275767,    -0.020265643,   -0.0081073497,     0.073122118 ],
  [               0,               0,       1257.2698,      -1808.9401,      -2512.2284,       3618.5177 ],
  [               0,               0,        29.87593,       41.119789,      -448.42107,        468.9465 ],
  [               0,               0,       687.73327,      -948.27205,      -1566.8918,       2165.8677 ],
  [               0,               0,       1849.9329,      -2635.0136,      -3751.3169,       5342.0526 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer1_etaoffv4_55_e = [
 [
  [               0,               0,               0,               0,    -0.015528361,      0.02244791 ],
  [               0,               0,               0,               0,       13.078812,        20.29491 ],
  [               0,               0,               0,               0,      -3260.7418,       4675.1352 ],
  [               0,               0,               0,               0,       3048.0643,      -4339.4479 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,       495100.18 ],
  [               0,               0,               0,               0,               0,     0.003877012 ],
  [               0,               0,               0,               0,               0,   -0.0038770038 ],
  [               0,               0,               0,               0,               0,   -0.0038769926 ],
  [               0,               0,               0,               0,               0,  -1.1611798e-09 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.48233557,     -0.71096569 ],
  [               0,               0,               0,               0,       484.37686,       -715.5952 ],
  [               0,               0,               0,               0,       120.70593,      -181.59249 ],
  [               0,               0,               0,               0,      -51.192077,       76.211497 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0018671103,   -0.0068567782,  -0.00068510419,     0.045512582 ],
  [               0,               0,      -5.1683212,       33.322519,        -67.5799,       44.768053 ],
  [               0,               0,       6.6968589,      -38.880492,       73.665433,      -46.429275 ],
  [               0,               0,      -12.560075,       55.829948,      -81.546855,       38.329752 ],
  [               0,               0,      -1.7719049,       8.4478365,      -13.421816,        7.105665 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.028428903,     0.059589749 ],
  [               0,               0,               0,               0,       3.5728066,      -5.4266582 ],
  [               0,               0,               0,               0,      -1.9571412,       2.7552972 ],
  [               0,               0,               0,               0,     -0.66063542,      0.35985059 ],
  [               0,               0,               0,               0,      0.24459071,     -0.46455437 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.031862328,    -0.075653117,     -0.15536801,      0.37221159 ],
  [               0,               0,        12.93182,      -70.371238,       129.77874,      -80.612466 ],
  [               0,               0,       34.390106,      -210.03682,       424.60424,      -284.84699 ],
  [               0,               0,      -12.636454,       83.236432,      -183.22336,       133.82241 ],
  [               0,               0,       9.7565541,      -64.583268,       142.33108,      -104.43101 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0042248571,     0.012257516 ],
  [               0,               0,               0,               0,      -59.962106,       140.55222 ],
  [               0,               0,               0,               0,       58.569443,      -136.14202 ],
  [               0,               0,               0,               0,       4.8297203,      -10.675595 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0086702525,     0.021772989 ],
  [               0,               0,               0,               0,       15.149852,      -32.069476 ],
  [               0,               0,               0,               0,       8.7077256,      -21.243329 ],
  [               0,               0,               0,               0,       6.8892883,      -16.873902 ],
  [               0,               0,               0,               0,       1.5581691,      -3.6189078 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.011679787,    0.0070966799,     0.058105785 ],
  [               0,               0,               0,      -6.0576579,       26.209449,      -21.965041 ],
  [               0,               0,               0,      -99.570514,       459.84215,      -531.40717 ],
  [               0,               0,               0,      -37.418514,       197.72392,      -259.23425 ],
  [               0,               0,               0,    -0.011068117,    0.0034609319,     0.055695024 ],
  [               0,               0,               0,        3.064568,      -18.169388,       27.148855 ],
  [               0,               0,               0,      -57.352424,       282.52525,      -422.66011 ],
 ],
 [
  [               0,               0,               0,    0.0044309179,    0.0014077779,    0.0036248336 ],
  [               0,               0,               0,       1.9328896,       4.7974332,       15.908923 ],
  [               0,               0,               0,       534.24118,      -31095.918,       524092.11 ],
  [               0,               0,               0,       24.412299,      -78.128552,       43.882626 ],
  [               0,               0,               0,      -1.1728143,      -5.9069032,        23.44352 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer1_etaoffv4_55_b = [
 [
  [               0,               0,      -46492.677,       570.95337,      -2.3195044,    0.0024221593 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -1.7718501e-05,   0.00047728016 ],
  [               0,               0,               0,               0,      -4.5245073,       5.1483225 ],
  [               0,               0,               0,               0,     -0.23263043,     -0.22154904 ],
  [               0,               0,               0,               0,       1.2610861,     -0.49243848 ],
  [               0,               0,               0,               0,     0.043406978,    -0.087749765 ],
  [               0,               0,               0,               0,  -0.00039322735,   5.1459327e-05 ],
  [               0,               0,               0,               0,     -0.70669497,       1.7785975 ],
  [               0,               0,               0,               0,     -0.40990274,       1.2002848 ],
 ],
 [
  [               0,               0,               0,   0.00023505358,    0.0016231784,   -3.522457e-05 ],
  [               0,               0,               0,       11.305873,      -21.031164,       11.698238 ],
  [               0,               0,               0,      -3.9468718,       5.6002088,      -2.8499207 ],
  [               0,               0,               0,       7.6617942,      -9.4224277,       1.8853422 ],
  [               0,               0,               0,       2.5650587,      -3.1645257,      0.86427692 ],
  [               0,               0,               0,    0.0018164387,   -0.0016450012,   0.00029220748 ],
  [               0,               0,               0,      -6.7931079,       3.2503558,       2.1447099 ],
  [               0,               0,               0,      -32.136554,       32.334458,        -8.98842 ],
 ],
 [
  [               0,               0,               0,               0,     0.025143742,    0.0055338082 ],
  [               0,               0,               0,               0,       5.1487806,      -3.5349253 ],
  [               0,               0,               0,               0,       -19.46238,       14.971914 ],
  [               0,               0,               0,               0,        15.95587,      -12.688927 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   0.00015001411,     0.001232743,    0.0048708191 ],
  [               0,               0,               0,       3.3631689,      -6.7236824,       4.4633055 ],
  [               0,               0,               0,       -3.094833,       6.3261766,       -4.018397 ],
  [               0,               0,               0,      -2.0391763,       4.1171178,      -2.9649319 ],
  [               0,               0,               0,     0.032672624,     -0.01951617,    -0.058782324 ],
  [               0,               0,               0,   -0.0030974271,    0.0068481286,   -0.0037914758 ],
  [               0,               0,               0,      -1.8756512,       7.3710472,       -4.573541 ],
  [               0,               0,               0,      0.56526382,       4.9502017,       -2.622456 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.021210726,     0.014100609,   -0.0019533324,    -0.032413873 ],
  [               0,               0,      -9.8526764,      -1.1074205,       13.163348,        42.51548 ],
  [               0,               0,      -11.294054,      -5.1242706,        7.616068,       30.728332 ],
  [               0,               0,               0,               0,               0,              -1 ],
  [               0,               0,      -0.1413098,     -0.17452985,     -0.28102285,      -0.8409228 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0065904298,    -0.070949652,    -0.056470924,      0.21507842 ],
  [               0,               0,       1150.8897,      -1645.9281,      -2295.7653,       3286.9226 ],
  [               0,               0,      -331.45964,        555.3236,       318.82265,      -622.96947 ],
  [               0,               0,       483.57381,      -623.07092,      -1275.2544,       1680.7078 ],
  [               0,               0,       1955.4726,       -2783.838,      -3978.1352,       5662.0805 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer1_etaoffv4_55_e = [
 [
  [               0,               0,               0,               0,               0,       253520.59 ],
  [               0,               0,               0,               0,               0,   -0.0036692117 ],
  [               0,               0,               0,               0,               0,    0.0036691831 ],
  [               0,               0,               0,               0,               0,   1.6102194e-08 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.000953268 ],
  [               0,               0,               0,               0,               0,       26.169063 ],
  [               0,               0,               0,               0,               0,       2.2923782 ],
  [               0,               0,               0,               0,               0,       4.1086099 ],
  [               0,               0,               0,               0,               0,     -0.59710057 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.18694945,      0.28388626 ],
  [               0,               0,               0,               0,      -90.322749,       138.44433 ],
  [               0,               0,               0,               0,       31.751583,      -48.418638 ],
  [               0,               0,               0,               0,       25.729851,      -38.038542 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0093224718,    -0.018576474,    -0.028818209,     0.058418689 ],
  [               0,               0,      0.33936342,       41.177804,      -130.18732,       103.27969 ],
  [               0,               0,      -7.8852943,       39.568993,      -65.860852,        35.26413 ],
  [               0,               0,       5.6196629,      -19.874718,       20.368894,      -5.7656656 ],
  [               0,               0,     -0.41407502,       2.4123164,      -4.6028885,       2.8899655 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0053858923,   -0.0086510436 ],
  [               0,               0,               0,               0,      -6.3189012,       14.532781 ],
  [               0,               0,               0,               0,     -0.37241955,     -0.23856062 ],
  [               0,               0,               0,               0,     -0.67824686,       0.4112844 ],
  [               0,               0,               0,               0,     -0.05075915,     0.056037561 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0090319584,    -0.021878887,    -0.043611702,      0.10691452 ],
  [               0,               0,       7.1412772,      -15.748957,      -19.475278,        46.89984 ],
  [               0,               0,       8.8519685,       -30.91589,       14.551104,       22.668546 ],
  [               0,               0,       0.7630445,       10.019961,         -49.303,       51.282714 ],
  [               0,               0,      0.10799811,      -5.9849374,       23.979079,      -24.940708 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00029894437,    0.0036478136 ],
  [               0,               0,               0,               0,      -115.24278,       266.63859 ],
  [               0,               0,               0,               0,       41.842519,        -97.4987 ],
  [               0,               0,               0,               0,       4.2154414,      -9.3856481 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.010201912,     0.026115144 ],
  [               0,               0,               0,               0,       9.2703823,      -19.428258 ],
  [               0,               0,               0,               0,      -1.1984576,       1.9690678 ],
  [               0,               0,               0,               0,       1.4039456,      -4.1831053 ],
  [               0,               0,               0,               0,       1.4262685,      -3.3446192 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0027746284,   0.00066896431,     0.022246801 ],
  [               0,               0,               0,      0.74264627,      -2.6893529,       7.8382636 ],
  [               0,               0,               0,      0.89200019,      -6.6387724,       9.1057779 ],
  [               0,               0,               0,       16.812442,        8.956315,      -81.730561 ],
  [               0,               0,               0,     0.018773681,     0.040047878,     0.085422615 ],
  [               0,               0,               0,       7.4942948,      -36.690242,       44.944099 ],
  [               0,               0,               0,      0.98940535,      -4.8560713,       2.8384603 ],
 ],
 [
  [               0,               0,               0,               0,    0.0072986314,     0.009540806 ],
  [               0,               0,               0,               0,      -1.1132394,        8.629819 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,               0,      -48.733181,       119.84762 ],
  [               0,               0,               0,               0,      0.19003899,      0.10509579 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer1_etaoffv4_55_b = [
 [
  [               0,               0,      -67701.032,       784.27269,      -2.9797544,    0.0028301493 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   3.0528824e-05,    0.0010513572 ],
  [               0,               0,               0,               0,      -2.0392913,       3.5440035 ],
  [               0,               0,               0,               0,     -0.69386282,     -0.49909736 ],
  [               0,               0,               0,               0,       1.2529882,      -1.2341354 ],
  [               0,               0,               0,               0,      0.32526639,     -0.18107316 ],
  [               0,               0,               0,               0,  -0.00039088977,   0.00023598651 ],
  [               0,               0,               0,               0,      -1.1842021,       1.1879426 ],
  [               0,               0,               0,               0,       2.2722995,      0.82284738 ],
 ],
 [
  [               0,               0,               0,     0.011408854,    -0.011266345,     0.003454787 ],
  [               0,               0,               0,      -13.276705,       11.610588,       1.1381634 ],
  [               0,               0,               0,       4.6589347,      -9.1352846,       3.3318445 ],
  [               0,               0,               0,      -6.8652367,       10.448222,      -5.1061301 ],
  [               0,               0,               0,     -0.30664666,       1.2104078,     -0.97464503 ],
  [               0,               0,               0,    0.0034101976,   -0.0041509967,    0.0013928918 ],
  [               0,               0,               0,      -10.626825,       8.1350391,      -0.3010329 ],
  [               0,               0,               0,       25.514772,       -27.77993,       7.1194806 ],
 ],
 [
  [               0,               0,               0,               0,     0.028307203,   -0.0038080195 ],
  [               0,               0,               0,               0,      -23.579076,       19.401908 ],
  [               0,               0,               0,               0,      -16.221625,       12.324018 ],
  [               0,               0,               0,               0,       23.818793,      -18.939229 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.003115965,   -0.0092363164,     0.007000566 ],
  [               0,               0,               0,      -7.3119829,       12.765879,      -3.2454487 ],
  [               0,               0,               0,       6.0557079,      -21.689238,       11.814418 ],
  [               0,               0,               0,       16.991835,      -46.719737,       24.678537 ],
  [               0,               0,               0,      0.22278633,       1.3271417,      -2.5567452 ],
  [               0,               0,               0,    0.0065721642,    -0.020215222,     0.016653775 ],
  [               0,               0,               0,      -1.1715338,        3.738676,      -2.1512814 ],
  [               0,               0,               0,      0.29146061,      -1.6094643,    -0.044855028 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.13507229,     -0.19395241,     -0.27581838,      0.40448021 ],
  [               0,               0,       1229.7523,      -1775.9654,      -2474.8611,       3578.2053 ],
  [               0,               0,       148.78626,       -137.2074,      -688.24952,       828.75143 ],
  [               0,               0,       649.17086,      -894.33641,      -1504.6994,       2079.1433 ],
  [               0,               0,       2095.8734,      -2983.3109,      -4254.5128,       6054.5537 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer1_etaoffv4_55_e = [
 [
  [               0,               0,               0,               0,      0.68486548,      -0.9752838 ],
  [               0,               0,               0,               0,       143.95087,      -209.42813 ],
  [               0,               0,               0,               0,      -216.01255,       311.57588 ],
  [               0,               0,               0,               0,       64.858066,      -92.663983 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      -2.5554692,         3.77759 ],
  [               0,               0,               0,               0,      -36.317086,       54.544987 ],
  [               0,               0,               0,               0,       25.334776,       -37.97663 ],
  [               0,               0,               0,               0,      -29.477584,       41.612923 ],
  [               0,               0,               0,               0,      0.42087977,     -0.65603097 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.015405172,     0.045685308 ],
  [               0,               0,               0,               0,       -34.56941,       54.017483 ],
  [               0,               0,               0,               0,       69.794514,       -105.0226 ],
  [               0,               0,               0,               0,      -55.231607,       82.214448 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0077353873,    -0.014974941,    -0.022770363,     0.044990932 ],
  [               0,               0,       23.046939,      -30.214454,      -80.278441,       113.31302 ],
  [               0,               0,       -7.602164,       16.336877,       11.985181,      -30.751079 ],
  [               0,               0,       1.3686077,      -3.8581831,       4.0566674,      -3.1702567 ],
  [               0,               0,      0.28588582,        1.222566,      -6.5872141,       6.2294664 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.007326938,    -0.011996512 ],
  [               0,               0,               0,               0,      -2.0339896,       6.2427426 ],
  [               0,               0,               0,               0,      -2.1650205,        3.124784 ],
  [               0,               0,               0,               0,      -2.6552552,       4.0828769 ],
  [               0,               0,               0,               0,     -0.22256114,      0.39944369 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0004825089,  -0.00045638475,   0.00090644936,    0.0078823988 ],
  [               0,               0,       2.9910584,      -5.5740241,      -13.797628,       27.904554 ],
  [               0,               0,    -0.050704314,     -0.16573043,       1.7989856,      -3.5760065 ],
  [               0,               0,     -0.11681577,      0.79310146,      -1.2023529,      -0.8743558 ],
  [               0,               0,     -0.31939614,      0.47879296,       2.3819387,      -4.1555463 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00020517163,    0.0014023756 ],
  [               0,               0,               0,               0,      -292.23196,        674.6479 ],
  [               0,               0,               0,               0,        78.37187,      -181.95102 ],
  [               0,               0,               0,               0,       15.218098,      -34.387699 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00094951548,    0.0028027656 ],
  [               0,               0,               0,               0,       2.1246255,      -3.2917448 ],
  [               0,               0,               0,               0,      -3.2714294,       6.7398587 ],
  [               0,               0,               0,               0,      0.32626534,      -1.8249746 ],
  [               0,               0,               0,               0,      0.61840634,        -1.46251 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -4.1081254e-05,   0.00034291997 ],
  [               0,               0,               0,               0,      -7.1266232,      -1.8996083 ],
  [               0,               0,               0,               0,     0.011939617,       2.5408125 ],
  [               0,               0,               0,               0,       4.3465433,     -0.46246577 ],
  [               0,               0,               0,               0,      -1.0185337,      -2.7433761 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     -0.01085023,     0.015503143,     0.033576755 ],
  [               0,               0,               0,       4.6160299,      -22.330122,        33.01859 ],
  [               0,               0,               0,      -95.020152,         445.726,       -523.7618 ],
  [               0,               0,               0,      -33.823162,       176.65955,       -229.3819 ],
  [               0,               0,               0,   -0.0094088666,    0.0018113507,     0.050251351 ],
  [               0,               0,               0,       4.0583776,      -15.827229,       15.393078 ],
  [               0,               0,               0,      -6.3976773,        15.60092,      0.77106138 ],
 ],
 [
  [               0,               0,               0,               0,          1.4898,      -3.6879381 ],
  [               0,               0,               0,               0,      -3277.7501,       8152.7182 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,               0,      -266.19094,       659.17966 ],
  [               0,               0,               0,               0,       1.2747169,      -2.7839471 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer1_etaoffv4_55_b = [
 [
  [               0,               0,       13465.599,       65.274517,        -1.24299,    0.0017834073 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00016298908,   0.00064654707 ],
  [               0,               0,               0,               0,      0.13538622,       3.2932792 ],
  [               0,               0,               0,               0,       0.2253316,      -0.4394158 ],
  [               0,               0,               0,               0,      0.42626653,     -0.57611726 ],
  [               0,               0,               0,               0,     -0.20440817,    -0.045824605 ],
  [               0,               0,               0,               0,   4.7651137e-05,   4.6908772e-05 ],
  [               0,               0,               0,               0,      -1.0641321,       2.4013215 ],
  [               0,               0,               0,               0,     -0.31077916,      -1.5868798 ],
 ],
 [
  [               0,               0,               0,    0.0010868522,  -0.00010176241,  -2.7048233e-05 ],
  [               0,               0,               0,       11.015419,       -21.80627,       13.179968 ],
  [               0,               0,               0,       5.0656537,      -17.699239,       9.8344836 ],
  [               0,               0,               0,        30.07069,      -43.551004,       16.160169 ],
  [               0,               0,               0,      0.28338791,       1.2636658,      -1.2141533 ],
  [               0,               0,               0,   -0.0023015868,    0.0028420097,  -0.00099945089 ],
  [               0,               0,               0,      -11.003383,       13.206618,      -2.1604248 ],
  [               0,               0,               0,      -5.1884408,       4.0006475,      0.71963949 ],
 ],
 [
  [               0,               0,               0,               0,     0.021607363,    0.0066059151 ],
  [               0,               0,               0,               0,      -12.699721,       10.708419 ],
  [               0,               0,               0,               0,      -15.966982,       12.147496 ],
  [               0,               0,               0,               0,       18.726399,      -14.887963 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0011645714,    -0.003302323,    0.0023948626 ],
  [               0,               0,               0,       18.589823,      -46.756314,       28.227279 ],
  [               0,               0,               0,     -0.49787779,        15.36951,      -11.666123 ],
  [               0,               0,               0,        27.50435,      -36.288664,       8.8701323 ],
  [               0,               0,               0,      -3.7544313,       6.2255944,      -3.5809674 ],
  [               0,               0,               0,   0.00038199764,  -0.00083672457,   0.00083797387 ],
  [               0,               0,               0,      -1.8681154,       4.1642367,      -1.0289061 ],
  [               0,               0,               0,       1.6462344,      -4.9995722,       1.8586033 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0027261707,    0.0012195662,   -0.0018689979,   -0.0071762709 ],
  [               0,               0,      -5.2654983,      -5.0823381,      -6.9370323,      -5.4638547 ],
  [               0,               0,      -23.580858,      -20.149562,      -128.08257,      -54.434204 ],
  [               0,               0,      -5.0222302,       10703.035,       23251.289,       9039.1068 ],
  [               0,               0,       15.382704,       43.002124,       101.37967,       27.896735 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,       0.2741278,     -0.45575342,     -0.60941772,       1.0104433 ],
  [               0,               0,        527.1294,      -757.21535,      -1034.5444,       1490.9112 ],
  [               0,               0,      -786.50461,       1208.3862,       1206.9495,      -1897.5194 ],
  [               0,               0,       347.12366,      -403.69761,      -1138.7888,       1435.1371 ],
  [               0,               0,       2740.0419,      -3904.5022,      -5567.9824,       7932.5478 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer1_etaoffv4_55_e = [
 [
  [               0,               0,               0,               0,     0.066272607,    -0.094259055 ],
  [               0,               0,               0,               0,      -957.67838,       1404.6929 ],
  [               0,               0,               0,               0,       609.81998,      -878.86093 ],
  [               0,               0,               0,               0,       -913.9062,       1319.9591 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.005039955,     0.015193405 ],
  [               0,               0,               0,               0,      -179.52278,       264.92361 ],
  [               0,               0,               0,               0,         99.9346,      -147.55574 ],
  [               0,               0,               0,               0,      -114.04645,       164.29232 ],
  [               0,               0,               0,               0,      0.17870167,     -0.29216064 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.10307495,      0.15583014 ],
  [               0,               0,               0,               0,      -310.01664,       469.30859 ],
  [               0,               0,               0,               0,      -214.08767,       317.29068 ],
  [               0,               0,               0,               0,       304.27694,      -451.95927 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0011549761,   0.00011759141,    0.0072708682,     0.031070332 ],
  [               0,               0,      -2.4973592,       13.291452,      -23.542972,       14.487025 ],
  [               0,               0,       5.1664806,       -26.31828,       44.642253,      -25.774062 ],
  [               0,               0,      -5.5052796,       26.676767,       -42.90567,       22.338942 ],
  [               0,               0,      -1.8433543,       9.0948104,      -14.932731,       8.1592916 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.011045885,    -0.018437442 ],
  [               0,               0,               0,               0,      -6.5584038,       14.872319 ],
  [               0,               0,               0,               0,      0.68718493,      -2.4942366 ],
  [               0,               0,               0,               0,      0.14472033,      -1.4249789 ],
  [               0,               0,               0,               0,     -0.20732799,      0.39519995 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00033673979,  -0.00018193228,   0.00089149653,    0.0054040149 ],
  [               0,               0,        1.126142,      -1.8090835,      -4.9276588,       10.202844 ],
  [               0,               0,        1.486883,      -3.2739378,      -6.7184136,       13.661622 ],
  [               0,               0,     -0.50207207,       1.2168776,       1.7202191,      -5.4223192 ],
  [               0,               0,     -0.36937232,      0.73867024,       1.9722178,      -3.9738794 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.49622135,       1.1451022 ],
  [               0,               0,               0,               0,       90.462887,      -205.91523 ],
  [               0,               0,               0,               0,      -74.885367,       170.74318 ],
  [               0,               0,               0,               0,        65.64466,      -150.52997 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0013583941,    0.0039804555 ],
  [               0,               0,               0,               0,     -0.20410111,       2.0177159 ],
  [               0,               0,               0,               0,      -0.6553479,        0.522363 ],
  [               0,               0,               0,               0,      0.40020969,      -1.9542521 ],
  [               0,               0,               0,               0,      0.19504585,     -0.45366358 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0030090356,    0.0013560993 ],
  [               0,               0,               0,               0,      -15.873411,        44.22558 ],
  [               0,               0,               0,               0,      -9.8904624,       22.492984 ],
  [               0,               0,               0,               0,       35.781884,      -86.608885 ],
  [               0,               0,               0,               0,      0.29088645,     -0.70443587 ],
  [               0,               0,               0,               0,        4.368255,      -10.446525 ],
  [               0,               0,               0,               0,       -17.50608,       39.870574 ],
 ],
 [
  [               0,               0,               0,               0,     0.025184981,    -0.034987992 ],
  [               0,               0,               0,               0,      -1.6610713,       12.357495 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,               0,      -26.559284,       65.080078 ],
  [               0,               0,               0,               0,       1.2258454,      -2.2757781 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer1_etaoffv4_55_b = [
 [
  [               0,               0,      -32826.708,       494.68791,      -2.1948017,    0.0018011644 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00067057985,    0.0022148181 ],
  [               0,               0,               0,               0,      0.55020785,       1.6890591 ],
  [               0,               0,               0,               0,      0.10169743,     -0.96566573 ],
  [               0,               0,               0,               0,     -0.21736461,     -0.92876463 ],
  [               0,               0,               0,               0,     -0.13390407,   -0.0010144104 ],
  [               0,               0,               0,               0,   1.3705422e-05,   5.7601437e-05 ],
  [               0,               0,               0,               0,      0.17742345,       2.5844842 ],
  [               0,               0,               0,               0,     -0.10836631,      -1.7021713 ],
 ],
 [
  [               0,               0,               0,     0.010415108,   -0.0049154411,   0.00096706266 ],
  [               0,               0,               0,      -9.2166093,       3.0275178,       3.6551296 ],
  [               0,               0,               0,       19.347802,      -22.185857,       5.3943449 ],
  [               0,               0,               0,       20.776809,      -23.154532,       5.2917096 ],
  [               0,               0,               0,     -0.46189088,      0.83459844,     -0.41796757 ],
  [               0,               0,               0,     0.003089224,    -0.003336802,   0.00092685699 ],
  [               0,               0,               0,      -14.636271,       16.040266,      -1.8576882 ],
  [               0,               0,               0,       2.9001071,      -3.5686033,     -0.75402756 ],
 ],
 [
  [               0,               0,               0,               0,     0.024171841,    0.0071665565 ],
  [               0,               0,               0,               0,      -4.3548221,       4.0365706 ],
  [               0,               0,               0,               0,      -16.907584,       12.920793 ],
  [               0,               0,               0,               0,       16.484203,      -13.107879 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.006600663,    -0.010080813,     0.005009543 ],
  [               0,               0,               0,        3.696579,      -12.877868,       11.017024 ],
  [               0,               0,               0,     -0.78609164,       3.0842851,      -2.9685316 ],
  [               0,               0,               0,       2.4331592,      -3.6697336,      0.36545478 ],
  [               0,               0,               0,      0.33809083,     -0.47888739,   -0.0033676891 ],
  [               0,               0,               0,    0.0010853735,   -0.0017464658,   0.00076731882 ],
  [               0,               0,               0,      -1.1944761,       1.3181954,       1.7143496 ],
  [               0,               0,               0,      -8.1949117,       19.585282,      -7.1984942 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.18869841,     -0.33456409,     -0.43517167,      0.76295187 ],
  [               0,               0,       826.00155,      -1173.1021,      -1624.4441,       2311.7619 ],
  [               0,               0,      -562.67211,       914.04082,       655.59002,      -1161.6236 ],
  [               0,               0,       165.26701,      -112.95653,      -898.44025,       1028.2322 ],
  [               0,               0,       3095.1712,      -4410.8258,      -6290.5078,       8962.4295 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer1_etaoffv4_55_e = [
 [
  [               0,               0,               0,               0,    -0.047802426,     0.094097629 ],
  [               0,               0,               0,               0,      -2.6440661,       5.3880727 ],
  [               0,               0,               0,               0,       36.094658,      -53.061867 ],
  [               0,               0,               0,               0,      -36.315763,        52.47831 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0033612628,     0.020886425 ],
  [               0,               0,               0,               0,       -42.88072,       65.029125 ],
  [               0,               0,               0,               0,        42.89027,      -64.274674 ],
  [               0,               0,               0,               0,      -71.851358,       103.11929 ],
  [               0,               0,               0,               0,      0.10741059,     -0.15781253 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0072966438,     0.019615942 ],
  [               0,               0,               0,               0,      -214.46444,       322.99773 ],
  [               0,               0,               0,               0,        27.31764,      -42.001993 ],
  [               0,               0,               0,               0,       6.1686663,       -8.952902 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.020359307,    -0.027200711,    -0.053018328,     0.073076107 ],
  [               0,               0,      -44.097128,        226.7714,      -393.78313,       232.25116 ],
  [               0,               0,      -27.426737,       151.69019,      -276.33329,       165.09532 ],
  [               0,               0,      -1.4516585,       18.663171,      -50.399858,       37.867175 ],
  [               0,               0,       -8.512643,       42.393062,       -70.47976,        39.13283 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0055907634,   -0.0084564565 ],
  [               0,               0,               0,               0,      -3.3357532,       8.9863138 ],
  [               0,               0,               0,               0,      -0.7362307,      0.26093029 ],
  [               0,               0,               0,               0,       -1.420673,       1.5904268 ],
  [               0,               0,               0,               0,     -0.23678617,      0.44503029 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.017273516,     -0.03742714,    -0.079442057,      0.17512624 ],
  [               0,               0,      -3.1890682,      -14.044147,       104.52879,      -125.44827 ],
  [               0,               0,       22.764802,      -139.65591,       284.43979,      -193.45269 ],
  [               0,               0,      -4.7491316,       28.994776,      -58.707851,       38.307129 ],
  [               0,               0,       10.552016,      -69.778576,        153.6184,      -112.56947 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00037983335,    0.0043291638 ],
  [               0,               0,               0,               0,      -115.96809,       268.41858 ],
  [               0,               0,               0,               0,       44.694341,      -104.06295 ],
  [               0,               0,               0,               0,      0.52450219,     -0.92244288 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.021538833,    -0.046329999 ],
  [               0,               0,               0,               0,      -5.0662121,       14.099613 ],
  [               0,               0,               0,               0,      -1.4059234,       2.1600492 ],
  [               0,               0,               0,               0,      0.60355672,       -2.519058 ],
  [               0,               0,               0,               0,      0.17682795,     -0.38888872 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0024957083,    -0.016124492,     0.061564615 ],
  [               0,               0,               0,        6.791811,      -42.282843,        69.33228 ],
  [               0,               0,               0,      -91.538367,       428.34095,      -502.08266 ],
  [               0,               0,               0,      -46.594038,       238.25713,      -303.64634 ],
  [               0,               0,               0,    -0.011693968,     0.017654286,     0.025215964 ],
  [               0,               0,               0,   -0.0073521007,       1.6572128,      -2.9996777 ],
  [               0,               0,               0,      -3.5457061,     -0.36578887,       22.844426 ],
 ],
 [
  [               0,               0,               0,               0,    0.0054235941,      0.01973819 ],
  [               0,               0,               0,               0,       1.1404046,       9.0118254 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,               0,      -17.130663,        41.88254 ],
  [               0,               0,               0,               0,      0.21615196,      0.41495505 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer1_etaoffv4_55_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00049317604,    0.0016581461 ],
  [               0,               0,               0,               0,     0.052588838,        2.375441 ],
  [               0,               0,               0,               0,      0.36159912,      -1.1133115 ],
  [               0,               0,               0,               0,      0.15358281,      -1.1349332 ],
  [               0,               0,               0,               0,     -0.14734961,    -0.014050353 ],
  [               0,               0,               0,               0,   2.8970769e-05,   3.8241631e-05 ],
  [               0,               0,               0,               0,      -1.6159941,       3.1920282 ],
  [               0,               0,               0,               0,      -1.6108851,      -1.2895794 ],
 ],
 [
  [               0,               0,               0,               0,   0.00043793218,   0.00068193991 ],
  [               0,               0,               0,               0,      -1.0134247,        3.702398 ],
  [               0,               0,               0,               0,    -0.015340094,     -0.77540887 ],
  [               0,               0,               0,               0,     -0.11513422,     -0.89864593 ],
  [               0,               0,               0,               0,     -0.20064747,    -0.026324336 ],
  [               0,               0,               0,               0,  -5.0752086e-05,   8.8413308e-05 ],
  [               0,               0,               0,               0,     -0.40486416,       2.3577404 ],
  [               0,               0,               0,               0,      -1.9026116,      -1.0501516 ],
 ],
 [
  [               0,               0,               0,               0,     0.025804519,   -0.0010976758 ],
  [               0,               0,               0,               0,      -5.5884432,       5.0790536 ],
  [               0,               0,               0,               0,      -25.919901,       20.043151 ],
  [               0,               0,               0,               0,       23.960513,      -19.055131 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0002739144,   0.00041863642,    0.0023615389 ],
  [               0,               0,               0,      -1.1133048,       2.4509576,      0.67915068 ],
  [               0,               0,               0,      0.34298004,      -0.5537048,     -0.76648361 ],
  [               0,               0,               0,      0.98905765,      -2.2527412,    0.0057479951 ],
  [               0,               0,               0,    -0.087076783,        0.130389,     -0.15432627 ],
  [               0,               0,               0,   0.00026227783,  -0.00059622937,    0.0003415398 ],
  [               0,               0,               0,       2.6680696,      -1.0305803,       1.4591911 ],
  [               0,               0,               0,      -5.4813536,       1.8246159,       0.1539797 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.32991815,      -0.5592755,     -0.74314818,       1.2514284 ],
  [               0,               0,         648.067,      -925.78102,      -1250.3481,       1790.9356 ],
  [               0,               0,      -922.33705,       1483.1933,       1210.0705,      -2066.5968 ],
  [               0,               0,      -71.324425,       248.04713,      -532.75175,       458.58638 ],
  [               0,               0,       3535.0891,      -5034.7216,      -7195.4689,       10245.763 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer1_etaoffv4_55_e = [
 [
  [               0,               0,               0,               0,     0.085435085,    -0.099409824 ],
  [               0,               0,               0,               0,      -6.5709524,        10.74198 ],
  [               0,               0,               0,               0,       11.149968,      -17.161303 ],
  [               0,               0,               0,               0,      -16.794282,       24.437807 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.027874067,     0.042252202 ],
  [               0,               0,               0,               0,       87.537917,      -118.91323 ],
  [               0,               0,               0,               0,      -842.70539,       1240.8826 ],
  [               0,               0,               0,               0,       584.97822,      -849.26085 ],
  [               0,               0,               0,               0,      0.28426427,      -0.2067162 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.23056262,      0.34877563 ],
  [               0,               0,               0,               0,      -247.71892,       374.54694 ],
  [               0,               0,               0,               0,      -41.195627,       59.879132 ],
  [               0,               0,               0,               0,       67.033314,      -99.368733 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0052148301,    0.0065919614,     0.010483895,     0.022332556 ],
  [               0,               0,      0.39884015,      -0.5762378,      -1.2980855,       2.4090633 ],
  [               0,               0,       1.7308229,      -9.8385218,       18.229224,       -11.53428 ],
  [               0,               0,      -3.0548294,       13.927433,      -21.058934,       10.087229 ],
  [               0,               0,      -1.0028547,       4.9650732,      -8.1846634,        4.492564 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0059365475,   -0.0092357263 ],
  [               0,               0,               0,               0,      -4.3470443,       11.052711 ],
  [               0,               0,               0,               0,     -0.34088084,      -0.5104814 ],
  [               0,               0,               0,               0,      -1.0095951,      0.80235764 ],
  [               0,               0,               0,               0,     -0.27628115,      0.53126716 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.014512689,    -0.035657571,    -0.069549935,      0.17395016 ],
  [               0,               0,       11.526361,      -61.727375,       112.86738,      -69.445616 ],
  [               0,               0,       34.309024,       -213.2768,       440.82475,      -304.10058 ],
  [               0,               0,      -12.365669,       85.945626,      -197.28648,       148.65614 ],
  [               0,               0,       11.145775,      -71.988042,       155.02469,       -111.2868 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00028000904,    0.0026944874 ],
  [               0,               0,               0,               0,      -199.72435,       461.63403 ],
  [               0,               0,               0,               0,       54.879215,      -127.86106 ],
  [               0,               0,               0,               0,       8.1281339,      -18.178498 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00011235988,    0.0018094352 ],
  [               0,               0,               0,               0,       -8.179415,       22.750854 ],
  [               0,               0,               0,               0,       2.3390709,      -6.5705326 ],
  [               0,               0,               0,               0,       2.9417523,      -7.8858254 ],
  [               0,               0,               0,               0,      0.27309986,     -0.58893494 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.010314274,     0.025716516 ],
  [               0,               0,               0,               0,       -513.5499,       1232.3454 ],
  [               0,               0,               0,               0,       316.90523,      -759.99791 ],
  [               0,               0,               0,               0,       1435.2783,      -3439.3531 ],
  [               0,               0,               0,               0,      -0.2993501,     -0.69282939 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0039630203,    0.0056712357,    -0.028843064 ],
  [               0,               0,               0,        -11.0976,       14.481729,       37.105104 ],
  [               0,               0,               0,      -41.299087,       201.68783,       -247.7905 ],
  [               0,               0,               0,      -26.896035,       132.02447,      -161.89538 ],
  [               0,               0,               0,    0.0026162003,    0.0038951718,    -0.025654958 ],
  [               0,               0,               0,        12.34949,      0.88021073,      -73.827786 ],
  [               0,               0,               0,        10.78507,       1.1112843,      -64.825608 ],
 ],
 [
  [               0,               0,               0,               0,       0.0232382,     0.014598274 ],
  [               0,               0,               0,               0,       4.0289891,       29.981895 ],
  [               0,               0,               0,               0,   1.5801974e-05,     -0.49675512 ],
  [               0,               0,               0,               0,     -0.67052675,       1.4589371 ],
  [               0,               0,               0,               0,       1.9875145,      -3.5795564 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]


ele_layer1_etaoffv4_55_b = [
    ele5_layer1_etaoffv4_55_b,
    ele10_layer1_etaoffv4_55_b,
    ele25_layer1_etaoffv4_55_b,
    ele50_layer1_etaoffv4_55_b,
    ele75_layer1_etaoffv4_55_b,
    ele100_layer1_etaoffv4_55_b,
    ele200_layer1_etaoffv4_55_b,
    ele500_layer1_etaoffv4_55_b,
    ele1000_layer1_etaoffv4_55_b,
    ]

ele_layer1_etaoffv4_55_e = [
    ele5_layer1_etaoffv4_55_e,
    ele10_layer1_etaoffv4_55_e,
    ele25_layer1_etaoffv4_55_e,
    ele50_layer1_etaoffv4_55_e,
    ele75_layer1_etaoffv4_55_e,
    ele100_layer1_etaoffv4_55_e,
    ele200_layer1_etaoffv4_55_e,
    ele500_layer1_etaoffv4_55_e,
    ele1000_layer1_etaoffv4_55_e,
    ]



###############################################################################
# Tables for v4 sshape correction, layer 1, 5x5 photon clusters
#


gam5_layer1_etaoffv4_55_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       6192.4517,       1506.0787 ],
  [               0,               0,               0,               0,    -0.017446621,     0.015150117 ],
  [               0,               0,               0,               0,     0.017444071,    -0.015148884 ],
  [               0,               0,               0,               0,      0.01744678,    -0.015149518 ],
  [               0,               0,               0,               0,   1.9329236e-07,  -7.9719603e-08 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0024924883,   -0.0033726223,    0.0010016224,   0.00019303074 ],
  [               0,               0,       94.274266,      -3.1249093,      -11.089837,       3.6967966 ],
  [               0,               0,       112.67033,      -42.067894,      -39.903322,       16.692946 ],
  [               0,               0,      -1.5086705,      -3.9798087,       5.5921278,       7.2935098 ],
  [               0,               0,       19.680405,      -13.918577,      -9.8459386,       5.6737252 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam5_layer1_etaoffv4_55_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer1_etaoffv4_55_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.017369102,     0.008208377 ],
  [               0,               0,               0,               0,       2.3756268,      0.69407823 ],
  [               0,               0,               0,               0,      -1243.6104,       172.75915 ],
  [               0,               0,               0,               0,      -1243.5532,       172.75698 ],
  [               0,               0,               0,               0,        1122.339,      -156.62115 ],
  [               0,               0,               0,               0,      -284.42881,       39.691312 ],
  [               0,               0,               0,               0,    -0.024148453,     0.011414641 ],
  [               0,               0,               0,               0,     0.068544957,       1.5384022 ],
 ],
 [
  [               0,               0,               0,  -0.00012354535,   0.00017835288,   9.1376626e-05 ],
  [               0,               0,               0,      -208.50355,       229.86934,      -57.703665 ],
  [               0,               0,               0,      -728.47342,       793.56106,      -260.13184 ],
  [               0,               0,               0,      -640.42726,       709.96573,      -242.65259 ],
  [               0,               0,               0,       269.68054,      -296.87077,       77.076557 ],
  [               0,               0,               0,    -0.030656306,     0.032408689,    -0.018298443 ],
  [               0,               0,               0,        3.099243,      -3.3479217,       1.1500957 ],
  [               0,               0,               0,      -3.9343011,       4.3274011,      0.43624188 ],
 ],
 [
  [               0,               0,               0,               0,     -0.01063283,    0.0088999069 ],
  [               0,               0,               0,               0,       22.501346,       21.996904 ],
  [               0,               0,               0,               0,       827.10218,       -661.4584 ],
  [               0,               0,               0,               0,       337.06041,      -267.23649 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   0.00080137947,   -0.0012120209,   0.00080468936 ],
  [               0,               0,               0,      -49.906068,       132.24271,      -81.876726 ],
  [               0,               0,               0,      -87.621972,       212.28248,      -125.97158 ],
  [               0,               0,               0,      -52.628239,       117.37278,      -65.283328 ],
  [               0,               0,               0,       6.5185698,      -20.623289,       13.834042 ],
  [               0,               0,               0,     0.011553096,    -0.032160364,     0.021391349 ],
  [               0,               0,               0,      -1.7023328,       3.5922262,      -1.0935879 ],
  [               0,               0,               0,      -7.4354517,       13.283491,      -7.7238091 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.082830122,     -0.11010497,      -0.1629889,       0.2246616 ],
  [               0,               0,       2964.9951,      -4287.5362,      -5931.8722,       8580.8707 ],
  [               0,               0,       74.170327,      -24.519041,      -576.21512,       656.44808 ],
  [               0,               0,       1360.3316,      -1906.4155,      -2892.8394,       4055.1162 ],
  [               0,               0,       2241.7828,      -3197.6552,      -4529.2374,       6459.1129 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer1_etaoffv4_55_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,      0.04583902 ],
  [               0,               0,               0,               0,               0,      0.62062488 ],
  [               0,               0,               0,               0,               0,      -0.5801149 ],
  [               0,               0,               0,               0,               0,     -0.56510165 ],
  [               0,               0,               0,               0,               0,  -0.00013924911 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer1_etaoffv4_55_b = [
 [
  [               0,               0,      -74321.378,        720.9807,      -2.2127318,    0.0020710417 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -3.5958781e-06,   0.00092988107 ],
  [               0,               0,               0,               0,     -0.30568607,       2.7548138 ],
  [               0,               0,               0,               0,      -1.1734854,     -0.51848414 ],
  [               0,               0,               0,               0,       1.8404291,      -1.3065796 ],
  [               0,               0,               0,               0,       73.321511,       -18.13896 ],
  [               0,               0,               0,               0,    -0.068039527,     0.016777048 ],
  [               0,               0,               0,               0,     -0.50389288,      0.10798958 ],
  [               0,               0,               0,               0,     0.010716917,    -0.013577018 ],
 ],
 [
  [               0,               0,               0,    0.0065531752,   -0.0062063697,     0.003660672 ],
  [               0,               0,               0,      -12.567402,       14.226983,      -2.1277973 ],
  [               0,               0,               0,       6.8330122,      -8.0439046,       1.3727727 ],
  [               0,               0,               0,       4.7064285,      -5.1902098,      0.33035789 ],
  [               0,               0,               0,     -0.27670136,      0.31073063,     -0.15237035 ],
  [               0,               0,               0,    0.0019174504,   -0.0023616781,   0.00068035619 ],
  [               0,               0,               0,       21.675178,      -9.0270087,      -1.3515112 ],
  [               0,               0,               0,      -76.797328,       109.75755,      -33.268098 ],
 ],
 [
  [               0,               0,               0,               0,       1248.4672,      -204.42613 ],
  [               0,               0,               0,               0,    -0.058239119,     0.063641719 ],
  [               0,               0,               0,               0,     0.057869157,     -0.06334608 ],
  [               0,               0,               0,               0,   0.00047318746,  -0.00037611287 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.049145146,    -0.078816382,      0.03331051 ],
  [               0,               0,               0,     -0.18280549,      -2.0169376,       3.8685302 ],
  [               0,               0,               0,       2.5448794,      -5.0690636,       1.5123313 ],
  [               0,               0,               0,       2.6482033,      -4.8630958,       1.0277174 ],
  [               0,               0,               0,     -0.37394695,       1.0617116,     -0.76689502 ],
  [               0,               0,               0,    0.0013703957,   -0.0030830088,    0.0016173394 ],
  [               0,               0,               0,       37.169372,      -82.861896,       48.088227 ],
  [               0,               0,               0,       6.5269345,      -14.487333,        12.06461 ],
 ],
 [
  [               0,               0,               0,               0,               0,       1805.5826 ],
  [               0,               0,               0,               0,               0,     0.016153244 ],
  [               0,               0,               0,               0,               0,    -0.016151735 ],
  [               0,               0,               0,               0,               0,   5.1505338e-08 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.039210146,     0.026887851,    -0.019109172,     -0.13128812 ],
  [               0,               0,       188.90819,      -287.30393,      -388.95086,       595.08309 ],
  [               0,               0,       869.03312,      -1254.5073,      -1799.1438,       2595.6519 ],
  [               0,               0,      -794.81444,       1120.5861,       1566.9643,      -2210.1557 ],
  [               0,               0,       76.726024,      -103.65823,      -151.75307,       204.21142 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer1_etaoffv4_55_e = [
 [
  [               0,               0,               0,               0,  -0.00033249526,   0.00048269546 ],
  [               0,               0,               0,               0,       12.319142,       22.026044 ],
  [               0,               0,               0,               0,       59302.119,      -85739.315 ],
  [               0,               0,               0,               0,       19056.126,      -25871.779 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,       673839.84 ],
  [               0,               0,               0,               0,               0,    0.0035457549 ],
  [               0,               0,               0,               0,               0,   -0.0035457453 ],
  [               0,               0,               0,               0,               0,   -0.0035457463 ],
  [               0,               0,               0,               0,               0,  -1.0481573e-09 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      -6816.7964,       10400.833 ],
  [               0,               0,               0,               0,      -1.0073125,       1.5431935 ],
  [               0,               0,               0,               0,       1.0062408,      -1.5415476 ],
  [               0,               0,               0,               0,  -0.00035632638,   0.00053124954 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0012124571 ],
  [               0,               0,               0,               0,               0,        -39.7599 ],
  [               0,               0,               0,               0,               0,       1.5376736 ],
  [               0,               0,               0,               0,               0,       3.1235315 ],
  [               0,               0,               0,               0,               0,       3.3804243 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0038719612,   0.00075272996,     0.011960982 ],
  [               0,               0,               0,      -24.766764,       85.144249,      -71.033256 ],
  [               0,               0,               0,       37.096186,      -120.23321,       96.348758 ],
  [               0,               0,               0,       32.454067,      -106.28296,        85.89861 ],
  [               0,               0,               0,      -5.4237078,       17.375196,       -13.94384 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0043336587,   -0.0058184048 ],
  [               0,               0,               0,               0,      -1.6365845,       4.9406039 ],
  [               0,               0,               0,               0,      -1.4943382,       1.9549248 ],
  [               0,               0,               0,               0,      0.64269169,      -2.0278854 ],
  [               0,               0,               0,               0,      0.29256346,     -0.60845735 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00065121739,  -0.00047130078,    0.0012992614,    0.0089747376 ],
  [               0,               0,       4.7363582,      -9.1125514,      -21.652194,       43.835696 ],
  [               0,               0,       1.7714645,      -3.7053102,      -8.2694725,       16.157468 ],
  [               0,               0,      -1.1488749,       2.5729315,       4.4628503,       -11.05169 ],
  [               0,               0,     -0.66765345,       1.3172696,        3.488483,      -6.9610951 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0036673778,     0.013038611 ],
  [               0,               0,               0,               0,      -46.839527,       108.50195 ],
  [               0,               0,               0,               0,       37.639256,      -87.247176 ],
  [               0,               0,               0,               0,      -3.7368487,       8.6454164 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.017152068,     0.042813427 ],
  [               0,               0,               0,               0,        6.085748,      -11.714072 ],
  [               0,               0,               0,               0,       2.8349963,      -7.6727357 ],
  [               0,               0,               0,               0,       2.6913417,      -7.4027474 ],
  [               0,               0,               0,               0,      -0.7435594,       1.7010945 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.016300249,     0.067539356,    -0.063408354 ],
  [               0,               0,               0,      -38.983078,       162.05256,         -155.39 ],
  [               0,               0,               0,      -76.560867,       340.56426,      -377.40199 ],
  [               0,               0,               0,      -4.0190361,       39.740461,      -72.646626 ],
  [               0,               0,               0,    0.0022872785,    0.0062325501,    -0.026665353 ],
  [               0,               0,               0,      -6.7098028,       34.614736,      -43.486293 ],
  [               0,               0,               0,      -7.9246835,       17.920329,      -17.398537 ],
 ],
 [
  [               0,               0,               0,               0,     0.086490687,     -0.20763167 ],
  [               0,               0,               0,               0,      -3061.9989,       7612.7918 ],
  [               0,               0,               0,               0,       1.0364862,      -22.264035 ],
  [               0,               0,               0,               0,      -105.61102,       259.95206 ],
  [               0,               0,               0,               0,     0.066084657,     -0.30933475 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer1_etaoffv4_55_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -2.3801285e-05,   0.00039640971 ],
  [               0,               0,               0,               0,    -0.008237546,    0.0092723274 ],
  [               0,               0,               0,               0,      0.61044581,       1.8309083 ],
  [               0,               0,               0,               0,      -1.6775315,       2.4533886 ],
  [               0,               0,               0,               0,     -0.43686557,    -0.018077209 ],
  [               0,               0,               0,               0,   7.2387503e-05,   0.00018566275 ],
  [               0,               0,               0,               0,     -0.34159134,       1.7916162 ],
  [               0,               0,               0,               0,      0.51636915,      -1.7795412 ],
 ],
 [
  [               0,               0,               0,    0.0033632365,    -0.004520433,    0.0025017081 ],
  [               0,               0,               0,       -8.535722,       11.123784,     -0.52127589 ],
  [               0,               0,               0,      -1.2147942,       2.5664661,      -1.9847148 ],
  [               0,               0,               0,      -1.2395128,      0.96920979,      -1.1862508 ],
  [               0,               0,               0,      0.17268866,     -0.80659624,      0.29121009 ],
  [               0,               0,               0,  -0.00084383287,    0.0013771974,  -0.00051733248 ],
  [               0,               0,               0,       10.333966,      -5.2685048,       1.4624738 ],
  [               0,               0,               0,       4.4460087,      -19.151625,        7.876527 ],
 ],
 [
  [               0,               0,               0,               0,     -0.98959563,      0.81367804 ],
  [               0,               0,               0,               0,      -41.963098,       33.889844 ],
  [               0,               0,               0,               0,       15.041561,       -12.44152 ],
  [               0,               0,               0,               0,       16.768697,      -13.332924 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,  -0.00025689148,   0.00038216166,   0.00090467436 ],
  [               0,               0,               0,       10.165832,      -18.185699,       11.051614 ],
  [               0,               0,               0,      -2.0210307,       3.6880121,      -2.3852707 ],
  [               0,               0,               0,     -0.82488707,      0.95423742,      -1.4039663 ],
  [               0,               0,               0,     -0.21370303,      0.33997945,     -0.37119334 ],
  [               0,               0,               0,   0.00057496572,   -0.0010062726,   0.00047086036 ],
  [               0,               0,               0,      -2.1979113,        3.219309,       2.1860361 ],
  [               0,               0,               0,       3.5816277,      -9.6433398,        8.051025 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.26745062,     -0.35158612,     -0.52524642,       0.6956793 ],
  [               0,               0,       1323.4906,      -1920.1451,      -2668.6715,       3876.8657 ],
  [               0,               0,       760.57597,      -1040.6873,      -1874.9643,       2584.2015 ],
  [               0,               0,       226.48812,      -290.72148,      -667.79615,         883.832 ],
  [               0,               0,       1936.0692,      -2755.6389,      -3922.0451,        5580.794 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer1_etaoffv4_55_e = [
 [
  [               0,               0,               0,               0,      -2.4632805,       3.5598822 ],
  [               0,               0,               0,               0,      -105.62756,       154.19075 ],
  [               0,               0,               0,               0,       134.53779,      -194.27055 ],
  [               0,               0,               0,               0,      -87.733603,       126.32614 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.002993942 ],
  [               0,               0,               0,               0,               0,       5.0945134 ],
  [               0,               0,               0,               0,               0,     -0.89008956 ],
  [               0,               0,               0,               0,               0,     -0.12119421 ],
  [               0,               0,               0,               0,               0,     -0.16858572 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.24422116,      0.37340563 ],
  [               0,               0,               0,               0,      -196.58702,       296.35967 ],
  [               0,               0,               0,               0,       105.93962,       -158.7751 ],
  [               0,               0,               0,               0,       -56.97412,       84.894958 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.027324235,    -0.051330502,    -0.072339784,      0.13862558 ],
  [               0,               0,       -212.2553,       1079.5113,      -1822.0806,       1022.4725 ],
  [               0,               0,      -39.341012,       169.76614,      -237.91484,       106.46995 ],
  [               0,               0,      -72.527005,       334.51613,      -509.53932,       255.21629 ],
  [               0,               0,      -3.6804371,       16.500282,      -24.384897,       11.847784 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00098037062,    0.0040618312 ],
  [               0,               0,               0,               0,        1.296627,     -0.20220249 ],
  [               0,               0,               0,               0,     -0.13875586,     -0.86109032 ],
  [               0,               0,               0,               0,     -0.23114515,     -0.56232245 ],
  [               0,               0,               0,               0,     0.036473449,    -0.082823149 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0004928521,   -0.0002857829,    0.0010513544,    0.0064539567 ],
  [               0,               0,       2.1504308,      -3.6953765,      -10.101532,       20.174223 ],
  [               0,               0,       1.3715372,      -2.8427395,      -5.8078619,       10.882054 ],
  [               0,               0,       3.5369116,      -7.0045505,      -17.274795,       33.227742 ],
  [               0,               0,      0.11710877,     -0.33208515,    -0.058036102,      0.49120029 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0017309799,    0.0052257798 ],
  [               0,               0,               0,               0,      -46.457289,       107.97708 ],
  [               0,               0,               0,               0,       24.511945,      -57.365593 ],
  [               0,               0,               0,               0,      0.36383027,     -0.68634395 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.017764734,    -0.037719375 ],
  [               0,               0,               0,               0,      -9.4008656,       24.176356 ],
  [               0,               0,               0,               0,       1.7998621,      -5.3989646 ],
  [               0,               0,               0,               0,       1.7134268,      -5.0125937 ],
  [               0,               0,               0,               0,      0.48272686,      -1.1142322 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.005143505,    0.0019340059,    -0.026119778 ],
  [               0,               0,               0,      -15.231317,       44.369027,       -12.58642 ],
  [               0,               0,               0,      -53.034539,       258.54427,      -316.48101 ],
  [               0,               0,               0,      -7.4467765,        38.56575,      -49.749113 ],
  [               0,               0,               0,   5.4865402e-06,   -0.0027271138,    0.0072395098 ],
  [               0,               0,               0,        7.740195,      -16.594533,       -5.008875 ],
  [               0,               0,               0,       8.3832714,       11.187664,      -77.574238 ],
 ],
 [
  [               0,               0,               0,               0,      0.97489622,      -2.4134559 ],
  [               0,               0,               0,               0,       -3619.988,       9001.1059 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,               0,      -260.55938,       645.50377 ],
  [               0,               0,               0,               0,      0.77424325,      -1.7211516 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer1_etaoffv4_55_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0015791677,   0.00048430663 ],
  [               0,               0,               0,               0,      -3.5733642,       4.2764567 ],
  [               0,               0,               0,               0,      0.29950998,      -1.0219212 ],
  [               0,               0,               0,               0,       2.0910523,      -1.6359621 ],
  [               0,               0,               0,               0,       0.9990544,      -0.3563343 ],
  [               0,               0,               0,               0,   -0.0012771142,   0.00035484974 ],
  [               0,               0,               0,               0,       0.2540882,       0.5301167 ],
  [               0,               0,               0,               0,       1.2425433,      -1.2220135 ],
 ],
 [
  [               0,               0,               0,   4.7814469e-05,    0.0010148273,   0.00045281165 ],
  [               0,               0,               0,      -16.384542,       16.153047,     -0.56025758 ],
  [               0,               0,               0,       12.372784,      -14.037296,       3.0069897 ],
  [               0,               0,               0,      -5.6473964,       6.9063922,      -3.2610272 ],
  [               0,               0,               0,       -3.739256,       4.2800378,      -1.3459571 ],
  [               0,               0,               0,    0.0048203633,   -0.0056538147,    0.0016179404 ],
  [               0,               0,               0,       22.852794,      -19.680771,       2.9981567 ],
  [               0,               0,               0,      -41.155354,       45.714485,       -11.33126 ],
 ],
 [
  [               0,               0,               0,               0,     -0.12388873,      0.12772526 ],
  [               0,               0,               0,               0,      -1.9052401,       2.0205231 ],
  [               0,               0,               0,               0,      -14.193073,       10.819433 ],
  [               0,               0,               0,               0,       14.994212,      -11.924988 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0012047607,  -0.00034940141,    0.0015883252 ],
  [               0,               0,               0,       3.2661119,      -6.5677786,       5.2531753 ],
  [               0,               0,               0,      -2.2013625,       4.2532328,      -3.0459616 ],
  [               0,               0,               0,     -0.99201749,       1.6723091,      -1.9015917 ],
  [               0,               0,               0,     0.077708945,     -0.11799567,    -0.059275939 ],
  [               0,               0,               0,    0.0010651571,   -0.0020015457,   0.00095672133 ],
  [               0,               0,               0,       4.5888962,      -11.321529,       9.0523033 ],
  [               0,               0,               0,       9.5604188,      -22.629402,       14.819107 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,       0.0233942,     0.010819559,    -0.015315712,    -0.063253224 ],
  [               0,               0,      -3.9190835,      -4.3360493,      -6.7570687,      -6.3957628 ],
  [               0,               0,      -3.5681699,      -3.9516839,       -4.088321,      -4.1259774 ],
  [               0,               0,               0,               0,               0,              -1 ],
  [               0,               0,       2.1604993,       3.0537556,       4.3095055,       5.7723354 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.053357637,     -0.13546625,      -0.1517287,      0.34676381 ],
  [               0,               0,        622.0606,      -891.07613,      -1215.2304,       1744.9471 ],
  [               0,               0,      -384.72333,       640.29635,       390.24757,      -743.03835 ],
  [               0,               0,       286.53019,      -355.70791,      -867.44503,       1126.6452 ],
  [               0,               0,       2129.4017,      -3033.5995,      -4327.5313,       6163.7054 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer1_etaoffv4_55_e = [
 [
  [               0,               0,               0,               0,               0,   0.00014822962 ],
  [               0,               0,               0,               0,               0,       39.905918 ],
  [               0,               0,               0,               0,               0,      -34.234817 ],
  [               0,               0,               0,               0,               0,       39.553129 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0033718324 ],
  [               0,               0,               0,               0,               0,       4.3566355 ],
  [               0,               0,               0,               0,               0,     -0.37387997 ],
  [               0,               0,               0,               0,               0,     -0.73700087 ],
  [               0,               0,               0,               0,               0,     -0.26917971 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.23961276,     -0.35344647 ],
  [               0,               0,               0,               0,       573.00406,      -847.47038 ],
  [               0,               0,               0,               0,      -7.8395852,       8.2356028 ],
  [               0,               0,               0,               0,       154.77824,      -229.71982 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.016331194,    -0.022641615,    -0.045055185,     0.064223022 ],
  [               0,               0,      -46.401275,       275.39708,      -539.33959,       350.68657 ],
  [               0,               0,      -23.571974,       116.65495,      -193.46657,       106.61379 ],
  [               0,               0,       2.2869966,      -5.1472871,      -4.0448325,       9.5947689 ],
  [               0,               0,      -9.7177929,       47.784198,      -78.222501,       42.632208 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.021476306,    -0.031773171 ],
  [               0,               0,               0,               0,      -1.4371392,       3.8884936 ],
  [               0,               0,               0,               0,      0.66003522,      -2.1271359 ],
  [               0,               0,               0,               0,       0.7174091,      -2.2436486 ],
  [               0,               0,               0,               0,    -0.028591444,     0.048908771 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00010998131,   6.0799973e-05,   0.00058190978,    0.0022764828 ],
  [               0,               0,     -0.77323214,       1.4110388,       3.4680005,      -3.6144034 ],
  [               0,               0,      0.90975805,      -1.9364607,      -3.5948392,       6.4973136 ],
  [               0,               0,    -0.086986469,      0.30357154,      0.49457623,      -2.6733078 ],
  [               0,               0,     -0.27268469,      0.52492469,       1.5312848,      -2.9897812 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0010372729,    0.0027542015 ],
  [               0,               0,               0,               0,        -109.091,       252.53596 ],
  [               0,               0,               0,               0,       39.930933,      -92.984885 ],
  [               0,               0,               0,               0,       6.0097892,       -13.55995 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00090852611,    0.0025457398 ],
  [               0,               0,               0,               0,      0.27329818,       1.4418813 ],
  [               0,               0,               0,               0,      0.10914493,      -1.4661558 ],
  [               0,               0,               0,               0,     -0.18161443,      -0.7025365 ],
  [               0,               0,               0,               0,     0.033959642,    -0.061494414 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0055988582,    0.0016765759,     0.035972178 ],
  [               0,               0,               0,      0.29624605,       3.1234208,      -2.6893519 ],
  [               0,               0,               0,      -6.9753485,       17.501534,      -3.2016192 ],
  [               0,               0,               0,       1.6493851,       5.1069656,      -21.903778 ],
  [               0,               0,               0,      0.00977266,  -4.1459577e-05,    -0.056369704 ],
  [               0,               0,               0,      -2.1955194,       2.5234576,       7.8440205 ],
  [               0,               0,               0,      -3.4866048,      0.29793407,       17.530176 ],
 ],
 [
  [               0,               0,               0,               0,   0.00084757394,    0.0066255394 ],
  [               0,               0,               0,               0,      -3466.8538,       8622.1623 ],
  [               0,               0,               0,               0,      -18.297241,      -47.032478 ],
  [               0,               0,               0,               0,      -11.150621,       26.617609 ],
  [               0,               0,               0,               0,      0.16255968,     0.085591295 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer1_etaoffv4_55_b = [
 [
  [               0,               0,      -47728.898,       659.62513,      -2.9651092,    0.0035435454 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00086360049,    0.0005884703 ],
  [               0,               0,               0,               0,      -1.9074918,       4.1871948 ],
  [               0,               0,               0,               0,      -2.5636621,     -0.44054339 ],
  [               0,               0,               0,               0,      0.48065623,     -0.94103923 ],
  [               0,               0,               0,               0,      0.40811884,     -0.11497789 ],
  [               0,               0,               0,               0,  -0.00046303236,   6.5649947e-05 ],
  [               0,               0,               0,               0,      -1.6813077,       1.9368159 ],
  [               0,               0,               0,               0,       2.7156147,     -0.72429002 ],
 ],
 [
  [               0,               0,               0,    0.0013634175,   0.00097441983,   4.4343763e-05 ],
  [               0,               0,               0,      -11.070396,       8.1137789,       2.7220729 ],
  [               0,               0,               0,       7.7927382,      -9.9095683,       1.9609435 ],
  [               0,               0,               0,       8.0162259,      -10.592121,       2.2031523 ],
  [               0,               0,               0,     -0.18613841,      0.16147468,     -0.13549043 ],
  [               0,               0,               0,    0.0017146938,   -0.0020178801,   0.00052614598 ],
  [               0,               0,               0,       6.3507737,      -6.5235334,       3.6644492 ],
  [               0,               0,               0,       12.562292,      -19.279739,       6.0539555 ],
 ],
 [
  [               0,               0,               0,               0,      0.02139811,     0.011273294 ],
  [               0,               0,               0,               0,       -25.69683,       20.999864 ],
  [               0,               0,               0,               0,      -1.2148574,      0.45272136 ],
  [               0,               0,               0,               0,       15.551206,      -12.366719 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.011177316,    -0.015904151,    0.0068790771 ],
  [               0,               0,               0,      0.51746856,      -3.2754433,       5.0725157 ],
  [               0,               0,               0,      0.93854781,      -2.1509098,      0.10971413 ],
  [               0,               0,               0,      0.96697519,      -2.1237721,      -0.1131661 ],
  [               0,               0,               0,     -0.29409494,      0.84119466,     -0.65288847 ],
  [               0,               0,               0,   0.00059636056,    -0.001497064,   0.00081381255 ],
  [               0,               0,               0,       3.8478905,      -8.0129206,       5.9738667 ],
  [               0,               0,               0,       7.5957697,      -19.689837,        17.41011 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.11111499,      -0.2277706,     -0.27779419,      0.54743088 ],
  [               0,               0,       2300.7996,      -3296.2525,      -4596.5324,       6588.5897 ],
  [               0,               0,       -241.8239,       468.33276,      -59.458198,      -166.14653 ],
  [               0,               0,       391.42735,      -476.73288,      -1200.7475,       1543.5863 ],
  [               0,               0,       2913.7799,      -4149.4112,      -5926.2137,       8437.5369 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer1_etaoffv4_55_e = [
 [
  [               0,               0,               0,               0,    -0.034812053,      0.05030771 ],
  [               0,               0,               0,               0,      -22.229296,       70.850521 ],
  [               0,               0,               0,               0,        171.9302,      -257.47606 ],
  [               0,               0,               0,               0,       189.63438,      -254.24097 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.043487483,      0.06415259 ],
  [               0,               0,               0,               0,      -377.88313,       589.50634 ],
  [               0,               0,               0,               0,      -294.05039,       455.76889 ],
  [               0,               0,               0,               0,       1361.7176,       -1979.836 ],
  [               0,               0,               0,               0,     -0.58442868,     -0.46268689 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.032639448,     0.049437103 ],
  [               0,               0,               0,               0,       -1001.208,       1516.7833 ],
  [               0,               0,               0,               0,      -842.41243,       1252.4948 ],
  [               0,               0,               0,               0,       1082.9198,      -1609.1415 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0026790403,  -0.00028032783,     0.008366523,     0.032241874 ],
  [               0,               0,      -2.9679523,       14.937967,      -25.060423,       14.674241 ],
  [               0,               0,      -2.1579405,       10.530774,      -17.041319,        8.543413 ],
  [               0,               0,      0.28713907,      -1.0607457,       1.1703947,     -0.93679986 ],
  [               0,               0,      -0.2683457,       1.4346716,      -2.5509193,       1.5072807 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0034339355,    0.0078041498 ],
  [               0,               0,               0,               0,        7.449553,      -10.784067 ],
  [               0,               0,               0,               0,       1.4291382,      -3.6173273 ],
  [               0,               0,               0,               0,      0.36394886,      -1.6235053 ],
  [               0,               0,               0,               0,  -0.00015359454,   -0.0076211853 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.018540514,    -0.040994797,    -0.082389881,      0.18699293 ],
  [               0,               0,       -24.13275,       148.01664,      -301.43162,       205.63662 ],
  [               0,               0,     -0.61382644,       1.5252332,       2.0983923,       -6.527922 ],
  [               0,               0,      -26.810316,       169.91963,      -358.35808,       250.39821 ],
  [               0,               0,       6.4150288,      -41.401413,       88.958744,      -63.638447 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00015192817,      0.00389615 ],
  [               0,               0,               0,               0,      -204.98828,       473.40801 ],
  [               0,               0,               0,               0,       69.873227,      -161.96911 ],
  [               0,               0,               0,               0,     -0.15416827,      0.70381093 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0005311474,    0.0012237477 ],
  [               0,               0,               0,               0,       4.8743451,      -8.3786581 ],
  [               0,               0,               0,               0,      -2.7167163,       5.2178009 ],
  [               0,               0,               0,               0,      -2.0072309,       3.5849021 ],
  [               0,               0,               0,               0,     -0.32222505,      0.78633697 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.009024758 ],
  [               0,               0,               0,               0,               0,       5.3275766 ],
  [               0,               0,               0,               0,               0,      -1.4466162 ],
  [               0,               0,               0,               0,               0,      0.04866743 ],
  [               0,               0,               0,               0,               0,  -0.00022306121 ],
  [               0,               0,               0,               0,               0,       3.0910714 ],
  [               0,               0,               0,               0,               0,       -3.464235 ],
 ],
 [
  [               0,               0,   0.00075440378,   0.00097485645,   0.00017460036,   -0.0046353275 ],
  [               0,               0,      -160.36309,      -131.72083,       331.80961,       2457.0436 ],
  [               0,               0,     -0.40535911,      -1.0297288,      -2.4727115,      -7.1405577 ],
  [               0,               0,      -4.8647458,      -1.2345485,       13.389194,       47.143842 ],
  [               0,               0,       0.0124747,     0.031600342,     0.071104281,     -0.14993868 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer1_etaoffv4_55_b = [
 [
  [               0,               0,       17071.041,      -37.171177,     -0.62721763,   0.00094626917 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0010908311,    0.0055153198 ],
  [               0,               0,               0,               0,     0.065191959,       2.2230369 ],
  [               0,               0,               0,               0,     -0.20006875,      -1.1395211 ],
  [               0,               0,               0,               0,     0.055083992,       -1.317512 ],
  [               0,               0,               0,               0,     0.028135054,    -0.038554091 ],
  [               0,               0,               0,               0,  -0.00046515244,   0.00083858699 ],
  [               0,               0,               0,               0,     0.072639258,        1.245903 ],
  [               0,               0,               0,               0,      0.38918618,       1.3750117 ],
 ],
 [
  [               0,               0,               0,  -0.00062810034,    0.0017788199,   0.00021896801 ],
  [               0,               0,               0,      -1.5248335,       2.4533134,       2.6824803 ],
  [               0,               0,               0,      -2.0760099,       1.4202344,      -1.1057353 ],
  [               0,               0,               0,         7.93706,      -10.253417,       1.9598305 ],
  [               0,               0,               0,      0.98311062,      -1.2709702,      0.25936838 ],
  [               0,               0,               0,    0.0014636903,   -0.0015274766,   0.00042055813 ],
  [               0,               0,               0,      -22.158919,        25.87125,      -5.0514308 ],
  [               0,               0,               0,       26.260477,      -37.469586,       9.1323533 ],
 ],
 [
  [               0,               0,               0,               0,     0.022947348,    0.0069738814 ],
  [               0,               0,               0,               0,       3.3138547,      -2.0777339 ],
  [               0,               0,               0,               0,      -24.425986,       18.916053 ],
  [               0,               0,               0,               0,       18.370003,      -14.610061 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.013409823,    -0.019347241,    0.0089278888 ],
  [               0,               0,               0,        1.125572,      -4.3881504,       5.0585596 ],
  [               0,               0,               0,      0.45395604,     -0.71752045,     -0.70985174 ],
  [               0,               0,               0,      0.60858689,     -0.76321965,      -1.0425819 ],
  [               0,               0,               0,     -0.17842826,      0.55285508,     -0.46656077 ],
  [               0,               0,               0,   0.00010388918,  -0.00028624527,   0.00018721939 ],
  [               0,               0,               0,       -9.720239,       33.994889,      -15.760964 ],
  [               0,               0,               0,       19.173233,       -44.42773,       23.159315 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.21548152,     -0.40603127,     -0.51250454,      0.94398691 ],
  [               0,               0,      -445.34388,        628.1601,       965.87109,      -1359.0144 ],
  [               0,               0,      -935.84719,       1468.3282,        1381.488,      -2241.9658 ],
  [               0,               0,      -61.400538,       208.25456,      -437.72192,       375.60998 ],
  [               0,               0,       2863.2387,       -4083.113,      -5812.7561,       8287.5274 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer1_etaoffv4_55_e = [
 [
  [               0,               0,               0,               0,     0.082504413,     -0.11724318 ],
  [               0,               0,               0,               0,       1404.3473,      -2001.7609 ],
  [               0,               0,               0,               0,       232.58296,      -337.49053 ],
  [               0,               0,               0,               0,      -461.44959,       668.29261 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.06920063,      0.10209618 ],
  [               0,               0,               0,               0,         2022.74,      -2940.3203 ],
  [               0,               0,               0,               0,      -346.77081,       522.33545 ],
  [               0,               0,               0,               0,       512.89252,      -742.70331 ],
  [               0,               0,               0,               0,      0.18498591,     -0.19474122 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.40332268,       0.6172372 ],
  [               0,               0,               0,               0,      -196.60352,       295.52666 ],
  [               0,               0,               0,               0,       93.614598,      -140.28339 ],
  [               0,               0,               0,               0,      -49.606377,        73.90059 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0024645354,    0.0026596115,    0.0043499287,  -0.00096540134 ],
  [               0,               0,      -19.183769,        107.0757,      -191.98512,       113.44003 ],
  [               0,               0,      -27.700862,       140.71605,      -238.22965,       133.33904 ],
  [               0,               0,       23.163487,      -104.69306,       155.84012,      -77.190089 ],
  [               0,               0,      -9.6161719,       47.247403,      -77.332969,        42.18747 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0019893214,   -0.0022746348 ],
  [               0,               0,               0,               0,       -4.006278,       10.827339 ],
  [               0,               0,               0,               0,      -1.2699935,        1.419021 ],
  [               0,               0,               0,               0,     -0.44722224,     -0.17974706 ],
  [               0,               0,               0,               0,      0.13724117,     -0.27142043 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00025744271,  -0.00019849277,   0.00060569348,    0.0045889236 ],
  [               0,               0,       1.1506057,      -2.0472882,      -5.1166429,       12.173234 ],
  [               0,               0,       1.5842112,      -3.3966734,       -6.715038,       13.193202 ],
  [               0,               0,    -0.040549911,      0.49881591,     -0.38051785,      -2.1204774 ],
  [               0,               0,     -0.41537744,      0.81761204,       2.3584151,      -4.6859859 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,      0.01663545 ],
  [               0,               0,               0,               0,               0,      0.88985874 ],
  [               0,               0,               0,               0,               0,     -0.81842778 ],
  [               0,               0,               0,               0,               0,     0.093265019 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00053015122,    0.0017463368 ],
  [               0,               0,               0,               0,      -1.8169958,       7.0466588 ],
  [               0,               0,               0,               0,      0.21328214,      -1.7134291 ],
  [               0,               0,               0,               0,       2.0005532,      -5.7811485 ],
  [               0,               0,               0,               0,      0.17652957,     -0.37542222 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0049748897,     0.019702409 ],
  [               0,               0,               0,               0,      -45.506156,       118.07947 ],
  [               0,               0,               0,               0,      -7.7814852,        17.26645 ],
  [               0,               0,               0,               0,       5.4500004,      -13.163057 ],
  [               0,               0,               0,               0,     0.020123245,    -0.048374913 ],
  [               0,               0,               0,               0,      -56.204549,       137.68278 ],
  [               0,               0,               0,               0,       127.75116,      -312.38406 ],
 ],
 [
  [               0,               0,               0,               0,     0.017115431,    -0.028237758 ],
  [               0,               0,               0,               0,      -2991.1554,       7444.1832 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,               0,      -1.9347005,       3.9947101 ],
  [               0,               0,               0,               0,       3.3114968,      -7.4587681 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer1_etaoffv4_55_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00072598665,    0.0014223578 ],
  [               0,               0,               0,               0,      -1.2480366,       3.7621801 ],
  [               0,               0,               0,               0,       1.2775253,      -1.5052301 ],
  [               0,               0,               0,               0,      -1.2312463,      -1.1814752 ],
  [               0,               0,               0,               0,     -0.51470273,       0.0431296 ],
  [               0,               0,               0,               0,   -8.270997e-05,  -0.00013703584 ],
  [               0,               0,               0,               0,     -0.31479765,       1.5649703 ],
  [               0,               0,               0,               0,      -3.0441922,      -1.1255156 ],
 ],
 [
  [               0,               0,               0,    0.0026182356,    -0.002229567,    0.0011973373 ],
  [               0,               0,               0,      -9.9998413,       7.4367123,       3.2936555 ],
  [               0,               0,               0,      0.24511536,       3.9594454,      -4.0522206 ],
  [               0,               0,               0,       9.6092255,      -7.2171261,      -1.9021541 ],
  [               0,               0,               0,       1.7268746,       -1.306725,     -0.42977563 ],
  [               0,               0,               0,  -0.00015138769,   -0.0024446939,    0.0022513312 ],
  [               0,               0,               0,     -0.88680482,      0.18267024,      -0.3271198 ],
  [               0,               0,               0,      -2.8061615,       2.3887837,      0.76748577 ],
 ],
 [
  [               0,               0,               0,               0,     0.022133672,     0.010298319 ],
  [               0,               0,               0,               0,        12.93069,      -9.7170249 ],
  [               0,               0,               0,               0,      -29.480014,       22.928039 ],
  [               0,               0,               0,               0,       16.529459,      -13.147049 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0055618097,     0.013399646,   -0.0060677375 ],
  [               0,               0,               0,        6.601296,       -14.70756,       10.652393 ],
  [               0,               0,               0,     -0.35389431,      0.25797625,     -0.84651733 ],
  [               0,               0,               0,      0.51923468,      -1.3701002,     -0.51233951 ],
  [               0,               0,               0,      -0.3970829,      0.95789468,     -0.72879942 ],
  [               0,               0,               0,   0.00095000978,   -0.0019629485,    0.0010346769 ],
  [               0,               0,               0,      -3.1525278,       6.0281241,     -0.38385686 ],
  [               0,               0,               0,       4.8514537,      -14.082326,       5.8948582 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.14311012,     -0.30074629,     -0.36294264,      0.72633842 ],
  [               0,               0,       545.49577,      -784.84056,      -1033.2633,       1491.6253 ],
  [               0,               0,      -688.27256,        1112.763,       857.81591,      -1489.9336 ],
  [               0,               0,       5.3455473,       74.193483,      -470.65278,       501.43027 ],
  [               0,               0,       3054.9464,      -4353.1144,      -6213.3726,       8851.7762 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer1_etaoffv4_55_e = [
 [
  [               0,               0,               0,               0,     -0.74160641,       1.0890261 ],
  [               0,               0,               0,               0,       29.480861,       -40.40325 ],
  [               0,               0,               0,               0,       40.612035,      -59.712117 ],
  [               0,               0,               0,               0,      -46.394146,       66.988913 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.033673664,    -0.048068213 ],
  [               0,               0,               0,               0,      -1557.6852,       2305.3317 ],
  [               0,               0,               0,               0,      -710.79269,       1045.4897 ],
  [               0,               0,               0,               0,       559.63174,      -808.95904 ],
  [               0,               0,               0,               0,      0.39613847,     0.038911409 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.29615006,      0.45018534 ],
  [               0,               0,               0,               0,      -85.704616,       131.69329 ],
  [               0,               0,               0,               0,       13.573847,      -21.555445 ],
  [               0,               0,               0,               0,       0.6374745,     -0.74733465 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.013446281,    -0.018303056,    -0.035966594,     0.050726906 ],
  [               0,               0,      -14.865793,       66.682703,      -103.38168,       58.045005 ],
  [               0,               0,       3.5432278,       4.6711448,      -45.810798,       45.786327 ],
  [               0,               0,      0.13965892,       13.227356,       -47.10432,       40.153987 ],
  [               0,               0,       -1.050004,       3.7550556,      -4.0087598,       1.1548495 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0053088743,   -0.0072934711 ],
  [               0,               0,               0,               0,      -4.2384678,       10.460797 ],
  [               0,               0,               0,               0,      0.91268839,      -2.9416947 ],
  [               0,               0,               0,               0,      0.76224047,      -2.6209981 ],
  [               0,               0,               0,               0,     -0.11618439,      0.22542535 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00048156532,  -0.00026631799,    0.0011308642,     0.006691209 ],
  [               0,               0,      0.56443414,     -0.54093647,      -2.2819428,       4.6601738 ],
  [               0,               0,       1.9263907,       -4.219119,      -8.2337202,       16.804443 ],
  [               0,               0,       1.1308364,      -2.0646045,      -6.1081253,       10.298952 ],
  [               0,               0,     -0.59001984,       1.1709044,        3.144018,       -6.265902 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0036629522 ],
  [               0,               0,               0,               0,               0,       2.1368774 ],
  [               0,               0,               0,               0,               0,      -1.6059158 ],
  [               0,               0,               0,               0,               0,      0.47477029 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00037686692,    0.0019004861 ],
  [               0,               0,               0,               0,      -6.2072896,       17.707475 ],
  [               0,               0,               0,               0,       2.2237205,      -6.4518761 ],
  [               0,               0,               0,               0,       2.0360997,      -5.8818858 ],
  [               0,               0,               0,               0,      0.46719244,      -1.0542003 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00076242793,     0.025821253 ],
  [               0,               0,               0,               0,      -26.769919,       66.042579 ],
  [               0,               0,               0,               0,       273.18559,      -656.66376 ],
  [               0,               0,               0,               0,      -251.43558,       600.72621 ],
  [               0,               0,               0,               0,     0.027467645,      -0.1230591 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0020723159,     0.039623133,    -0.075350287 ],
  [               0,               0,               0,       -33.49337,       108.91444,      -59.741354 ],
  [               0,               0,               0,       178.32661,      -871.53585,       1063.1637 ],
  [               0,               0,               0,      -120.74975,       591.91058,      -725.21444 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       2.2877258,      -5.6638179 ],
  [               0,               0,               0,               0,      -3350.3078,       8332.6706 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,               0,       -385.5108,       954.33471 ],
  [               0,               0,               0,               0,      0.10328274,     -0.21861581 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer1_etaoffv4_55_b = [
 [
  [               0,               0,      -34017.343,       457.46683,      -1.9296932,     0.001776524 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00032698415,   0.00075199765 ],
  [               0,               0,               0,               0,      -3.3976935,       5.6598253 ],
  [               0,               0,               0,               0,      0.20091685,      -1.0759453 ],
  [               0,               0,               0,               0,    -0.098917354,      -1.1846193 ],
  [               0,               0,               0,               0,    -0.099302495,    -0.070355333 ],
  [               0,               0,               0,               0,   0.00024907839,  -5.9043695e-05 ],
  [               0,               0,               0,               0,       6.7236943,     0.081204566 ],
  [               0,               0,               0,               0,     -0.46491246,      -2.2592928 ],
 ],
 [
  [               0,               0,               0,   0.00061769854,    0.0013501826,   3.7980345e-06 ],
  [               0,               0,               0,       10.569873,      -20.476004,       12.504441 ],
  [               0,               0,               0,       1.5484834,       -2.375035,    -0.085873855 ],
  [               0,               0,               0,       5.1789884,      -6.7745568,      0.78453964 ],
  [               0,               0,               0,     -0.28209064,      0.34147804,     -0.28232873 ],
  [               0,               0,               0,   0.00076758056,  -0.00097917712,   0.00031666028 ],
  [               0,               0,               0,      -8.2907735,    -0.091766392,      -2.7428213 ],
  [               0,               0,               0,      -2.7011659,     -0.63394138,      -1.0201109 ],
 ],
 [
  [               0,               0,               0,               0,     0.022590919,    0.0074366008 ],
  [               0,               0,               0,               0,       35.948096,      -28.039792 ],
  [               0,               0,               0,               0,      -46.677158,       36.619121 ],
  [               0,               0,               0,               0,       16.594009,      -13.199805 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0009920352,   0.00065669891,  -9.2741166e-05 ],
  [               0,               0,               0,       2.8614675,      -8.6277681,         8.54819 ],
  [               0,               0,               0,       0.4321727,      -1.2203948,     -0.13508988 ],
  [               0,               0,               0,      0.40741702,     -0.90571813,     -0.87057735 ],
  [               0,               0,               0,     -0.21840386,      0.64369649,     -0.61265678 ],
  [               0,               0,               0,   0.00024018585,  -0.00056766259,   0.00036697218 ],
  [               0,               0,               0,     -0.11554479,     -0.25688815,       2.8056224 ],
  [               0,               0,               0,     -0.31522574,       -1.161001,      -1.5328697 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,       0.2637326,     -0.52791954,     -0.65308978,        1.252595 ],
  [               0,               0,      -1274.1249,       1809.9674,       2692.5698,      -3820.0196 ],
  [               0,               0,      -913.24007,       1496.1676,       1164.2203,      -2053.5996 ],
  [               0,               0,      -330.36991,        732.0394,      -408.47109,       48.087055 ],
  [               0,               0,       3968.0732,      -5655.7711,       -8075.131,       11507.219 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer1_etaoffv4_55_e = [
 [
  [               0,               0,               0,               0,      0.71876417,      -1.0172552 ],
  [               0,               0,               0,               0,      -43.680512,       64.387304 ],
  [               0,               0,               0,               0,       38.047211,      -55.817662 ],
  [               0,               0,               0,               0,      -39.765509,       57.543406 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.11837017,      0.17780377 ],
  [               0,               0,               0,               0,      -66.638555,       102.94221 ],
  [               0,               0,               0,               0,      -142.26203,       208.52751 ],
  [               0,               0,               0,               0,      -15.038562,        20.99224 ],
  [               0,               0,               0,               0,        0.174528,     -0.13532617 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.16529658,      0.25124781 ],
  [               0,               0,               0,               0,      -343.37809,        517.7196 ],
  [               0,               0,               0,               0,      -53.244177,       77.680599 ],
  [               0,               0,               0,               0,       81.039527,      -120.15428 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0009770272,  -0.00049610281,    0.0012504545,    0.0064535432 ],
  [               0,               0,       1.3164594,      0.21118566,      -8.6727834,        9.801076 ],
  [               0,               0,         2.23048,      -2.4907394,      -9.7358449,       11.634824 ],
  [               0,               0,   -0.0063172969,      0.19592217,     -0.77748573,     -0.28719236 ],
  [               0,               0,     -0.33267302,       0.6277148,      0.50686995,      -1.0370005 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0051479176,   -0.0074941732 ],
  [               0,               0,               0,               0,      -4.5096602,        11.48591 ],
  [               0,               0,               0,               0,     -0.24539585,     -0.83422908 ],
  [               0,               0,               0,               0,     0.047431748,      -1.2425764 ],
  [               0,               0,               0,               0,     -0.18108163,      0.36351791 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   0.00018640764,   0.00032313424,   0.00024958121,  -0.00055234063 ],
  [               0,               0,      -1.0828939,       1.8296563,       4.6273974,      -4.7953192 ],
  [               0,               0,      0.73440267,      -1.6393356,      -3.0366622,       5.5308244 ],
  [               0,               0,      0.89019953,      -1.8308278,      -4.3700267,       7.8309257 ],
  [               0,               0,    -0.032393451,    -0.015614278,      0.49405468,     -0.63913168 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0010595157,   0.00099488986 ],
  [               0,               0,               0,               0,      -204.08386,       471.63812 ],
  [               0,               0,               0,               0,       102.87574,      -238.05849 ],
  [               0,               0,               0,               0,      0.68015488,      -1.0522913 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0041215608,     0.013225152 ],
  [               0,               0,               0,               0,      -1.4149668,       6.1900774 ],
  [               0,               0,               0,               0,      0.48252926,      -2.4573526 ],
  [               0,               0,               0,               0,       1.6392842,       -5.012789 ],
  [               0,               0,               0,               0,      0.45542162,      -1.0204959 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0082061038,     0.020087501 ],
  [               0,               0,               0,               0,       6.1175843,       25.191789 ],
  [               0,               0,               0,               0,       305.98849,      -735.76041 ],
  [               0,               0,               0,               0,       662.66368,      -1590.1766 ],
  [               0,               0,               0,               0,     -0.49071718,      -1.2556929 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.023553695,    0.0026995225,      0.13927784 ],
  [               0,               0,               0,       29.921172,       5.2116548,      -179.64997 ],
  [               0,               0,               0,      -12.950958,       28.509739,       4.9309052 ],
  [               0,               0,               0,       2.2263193,       8.0339779,      -32.283673 ],
  [               0,               0,               0,       0.0116528,  -2.0431874e-05,      -0.0671108 ],
  [               0,               0,               0,      -0.2532166,      0.78721722,      0.71430495 ],
  [               0,               0,               0,       2.5583986,       1.3038162,      -20.722365 ],
 ],
 [
  [               0,               0,               0,               0,       1.1742213,       -2.904434 ],
  [               0,               0,               0,               0,      -3850.2858,       9576.6918 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,               0,       -126.4164,       312.42586 ],
  [               0,               0,               0,               0,       1.0599719,      -2.2315951 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]


gam_layer1_etaoffv4_55_b = [
    gam5_layer1_etaoffv4_55_b,
    gam10_layer1_etaoffv4_55_b,
    gam25_layer1_etaoffv4_55_b,
    gam50_layer1_etaoffv4_55_b,
    gam75_layer1_etaoffv4_55_b,
    gam100_layer1_etaoffv4_55_b,
    gam200_layer1_etaoffv4_55_b,
    gam500_layer1_etaoffv4_55_b,
    gam1000_layer1_etaoffv4_55_b,
    ]

gam_layer1_etaoffv4_55_e = [
    gam5_layer1_etaoffv4_55_e,
    gam10_layer1_etaoffv4_55_e,
    gam25_layer1_etaoffv4_55_e,
    gam50_layer1_etaoffv4_55_e,
    gam75_layer1_etaoffv4_55_e,
    gam100_layer1_etaoffv4_55_e,
    gam200_layer1_etaoffv4_55_e,
    gam500_layer1_etaoffv4_55_e,
    gam1000_layer1_etaoffv4_55_e,
    ]



###############################################################################
# Tables for v4 sshape correction, layer 1, 3x5 electron clusters
#

ele5_layer1_etaoffv4_35_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       1485.6164,       1964.6602 ],
  [               0,               0,               0,               0,   -0.0035766507,     0.012951714 ],
  [               0,               0,               0,               0,     0.003577058,    -0.012951097 ],
  [               0,               0,               0,               0,    0.0035753432,    -0.012950902 ],
  [               0,               0,               0,               0,  -6.0703627e-08,  -1.7938661e-08 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele5_layer1_etaoffv4_35_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer1_etaoffv4_35_b = [
 [
  [               0,               0,       25550.025,      -190.17632,     -0.18069955,    0.0018563224 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      -869.70975,       795.91106 ],
  [               0,               0,               0,               0,    -0.017540696,      0.02101475 ],
  [               0,               0,               0,               0,     0.017539508,    -0.021012269 ],
  [               0,               0,               0,               0,     0.017543497,    -0.021012642 ],
  [               0,               0,               0,               0,   1.0950724e-06,  -3.0488439e-07 ],
  [               0,               0,               0,               0,  -0.00042961285,   0.00019425007 ],
  [               0,               0,               0,               0,        6.441829,    -0.037916768 ],
  [               0,               0,               0,               0,       16.456214,       -1.433561 ],
 ],
 [
  [               0,               0,               0,     0.015319527,    -0.012254344,    0.0030002985 ],
  [               0,               0,               0,       18.803529,      -29.072085,       12.551405 ],
  [               0,               0,               0,      -3.7358534,       3.5628466,      -1.2380001 ],
  [               0,               0,               0,       7.6379878,      -10.618021,         3.00074 ],
  [               0,               0,               0,       1.6506826,      -1.7899857,      0.37572644 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.16363317,     -0.12883039 ],
  [               0,               0,               0,               0,       174.94891,      -136.75636 ],
  [               0,               0,               0,               0,      -242.69986,       192.27083 ],
  [               0,               0,               0,               0,       145.65241,      -115.73814 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,  -0.00048601692,    0.0002560548,    0.0004234006 ],
  [               0,               0,               0,      -77.315604,       157.64007,      -77.045373 ],
  [               0,               0,               0,      -44.926272,       84.375795,      -29.573934 ],
  [               0,               0,               0,       30.003284,      -69.321105,       33.932132 ],
  [               0,               0,               0,      -5.5963821,       7.7811882,      -6.2585706 ],
  [               0,               0,               0,    0.0026631494,   -0.0053646845,    0.0033996925 ],
  [               0,               0,               0,      -1.3024845,       1.4883903,      0.99353856 ],
  [               0,               0,               0,      -2.5442961,       1.1516068,      0.69698667 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.44328614,      0.30322322,    -0.026619858,     -0.67758127 ],
  [               0,               0,       50.982561,      -72.936976,      -95.979784,        137.5755 ],
  [               0,               0,      -33.819268,       46.198021,       67.537463,      -92.408673 ],
  [               0,               0,      -52.463621,       72.893461,       105.62005,      -146.93355 ],
  [               0,               0,      -5.6994451,       8.1983896,       11.461718,      -16.489077 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer1_etaoffv4_35_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,        150756.4 ],
  [               0,               0,               0,               0,               0,    0.0043672517 ],
  [               0,               0,               0,               0,               0,   -0.0043672387 ],
  [               0,               0,               0,               0,               0,   -0.0043672236 ],
  [               0,               0,               0,               0,               0,   3.7235949e-09 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer1_etaoffv4_35_b = [
 [
  [               0,               0,       5880.6124,      -1.6200975,     -0.47993088,   0.00086525386 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0015100362,   0.00099462467 ],
  [               0,               0,               0,               0,       11.579998,       1.5902962 ],
  [               0,               0,               0,               0,     -0.86294819,      -0.4476852 ],
  [               0,               0,               0,               0,       1.6722812,     -0.35646896 ],
  [               0,               0,               0,               0,     -0.40471533,      0.16502256 ],
  [               0,               0,               0,               0,  -0.00078757386,   0.00037119931 ],
  [               0,               0,               0,               0,       6.0710517,      -0.1397741 ],
  [               0,               0,               0,               0,      0.80370541,       1.9010169 ],
 ],
 [
  [               0,               0,               0,     -0.03461487,    -0.002827941,     0.054930847 ],
  [               0,               0,               0,      -1.2877346,       1.2651665,      0.17444266 ],
  [               0,               0,               0,       1.3893225,      -1.3487543,     -0.12343752 ],
  [               0,               0,               0,       1.3490079,      -1.3205064,     -0.13135381 ],
  [               0,               0,               0,    -0.038600714,     0.031462644,   -0.0081477217 ],
  [               0,               0,               0,   0.00072493021,       -0.001263,   0.00047812708 ],
  [               0,               0,               0,       47.735771,      -59.788074,       22.529186 ],
  [               0,               0,               0,       -137.3576,       173.23132,      -48.257203 ],
 ],
 [
  [               0,               0,               0,               0,     0.042000651,    -0.011829701 ],
  [               0,               0,               0,               0,       -19.88645,       16.406858 ],
  [               0,               0,               0,               0,      -16.606083,       12.693382 ],
  [               0,               0,               0,               0,       20.173134,      -16.044245 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,       2.3033721,      -3.9920264,       1.7686142 ],
  [               0,               0,               0,       4.9162283,      -11.544014,         6.77826 ],
  [               0,               0,               0,      -4.7704602,        11.17987,      -6.5518066 ],
  [               0,               0,               0,      -4.5855081,       10.706004,      -6.2483899 ],
  [               0,               0,               0,   -0.0092461686,     0.021162259,    -0.012048804 ],
  [               0,               0,               0,    -0.010203752,     0.019720801,   -0.0096148344 ],
  [               0,               0,               0,      -6.3192409,       10.466782,      -2.3697794 ],
  [               0,               0,               0,      -2.5311862,       9.3184577,      -6.3091685 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0026741949 ],
  [               0,               0,               0,               0,               0,       3.0836187 ],
  [               0,               0,               0,               0,               0,      -1.7115852 ],
  [               0,               0,               0,               0,               0,     0.082368676 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.03481956,     0.029792329,    -0.012547484,     -0.13387376 ],
  [               0,               0,      -32.347937,        37.71969,       63.904898,      -70.611646 ],
  [               0,               0,       1118.0326,      -1611.1529,      -2319.6596,       3341.0361 ],
  [               0,               0,      -405.08106,       568.46753,       788.13666,      -1106.9655 ],
  [               0,               0,       130.61524,      -179.65959,       -259.7219,       356.45435 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer1_etaoffv4_35_e = [
 [
  [               0,               0,               0,               0,   -0.0070233323,     0.010220651 ],
  [               0,               0,               0,               0,       12.480059,       21.863737 ],
  [               0,               0,               0,               0,       148.27853,      -168.85216 ],
  [               0,               0,               0,               0,       249.11074,       -361.9339 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00044858608 ],
  [               0,               0,               0,               0,               0,      -39.493141 ],
  [               0,               0,               0,               0,               0,      -5.5236804 ],
  [               0,               0,               0,               0,               0,       6.6340823 ],
  [               0,               0,               0,               0,               0,       3.1223521 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0087496418 ],
  [               0,               0,               0,               0,               0,       2.6204401 ],
  [               0,               0,               0,               0,               0,      -1.3541119 ],
  [               0,               0,               0,               0,               0,      0.12558853 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,       2.9820923,      -5.5249982,       -9.235312,       17.289994 ],
  [               0,               0,      -52.695104,        265.8971,      -445.90872,        248.7959 ],
  [               0,               0,        51.85341,       -261.5245,       438.40205,      -244.52858 ],
  [               0,               0,       50.895431,      -256.94942,       431.14552,      -240.70542 ],
  [               0,               0,     -0.28422863,       1.3726823,      -2.2062159,       1.1800172 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.001479037,   -0.0020697166 ],
  [               0,               0,               0,               0,      -6.8227695,       15.001058 ],
  [               0,               0,               0,               0,      -2.0424783,       4.2738432 ],
  [               0,               0,               0,               0,       -1.794443,       4.1675957 ],
  [               0,               0,               0,               0,     -0.44136058,      0.70631397 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.023642443,    -0.057331348,     -0.11168995,      0.27399019 ],
  [               0,               0,      -27.221293,       173.84553,      -362.65984,       248.43406 ],
  [               0,               0,       43.300529,      -246.57803,       460.86455,      -282.27818 ],
  [               0,               0,      -23.846342,       174.29353,      -417.70538,       328.51137 ],
  [               0,               0,       14.107507,      -92.406235,       201.26858,      -145.80948 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       160981.84,      -369372.11 ],
  [               0,               0,               0,               0,    -0.017553331,      0.05875662 ],
  [               0,               0,               0,               0,     0.017413368,    -0.058434329 ],
  [               0,               0,               0,               0,  -0.00024603365,   0.00056764251 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00070926542 ],
  [               0,               0,               0,               0,               0,       4.0094661 ],
  [               0,               0,               0,               0,               0,       1.0241648 ],
  [               0,               0,               0,               0,               0,     -0.42785675 ],
  [               0,               0,               0,               0,               0,    -0.092247143 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0009372359,   0.00010843384,     0.013373458 ],
  [               0,               0,               0,       31.882164,      -164.68187,        216.4264 ],
  [               0,               0,               0,      -47.199541,       240.07889,      -306.51861 ],
  [               0,               0,               0,       5.0623052,       170.60134,      -436.71769 ],
  [               0,               0,               0,      0.31263572,    0.0026788815,      -1.7906631 ],
  [               0,               0,               0,        31.90958,      -159.58331,       199.47654 ],
  [               0,               0,               0,      -2.1917603,       13.554032,      -23.110782 ],
 ],
 [
  [               0,               0,               0,               0,    0.0038676708,   -0.0078712239 ],
  [               0,               0,               0,               0,      -4424.8373,       10992.148 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,               0,       152.62912,      -379.97311 ],
  [               0,               0,               0,               0,     -0.24439835,     -0.58726935 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer1_etaoffv4_35_b = [
 [
  [               0,               0,       25209.729,      -138.10644,     -0.49329319,    0.0015121006 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00038113841,    0.0003069402 ],
  [               0,               0,               0,               0,      -5.8950928,       6.4916076 ],
  [               0,               0,               0,               0,     -0.28540868,      0.13952303 ],
  [               0,               0,               0,               0,      -2.5237066,      0.62906174 ],
  [               0,               0,               0,               0,     -0.40981532,    -0.039516345 ],
  [               0,               0,               0,               0,  -8.8156576e-05,   4.3207992e-05 ],
  [               0,               0,               0,               0,       7.7535596,       1.0757868 ],
  [               0,               0,               0,               0,      -13.434381,       4.6371393 ],
 ],
 [
  [               0,               0,               0,    0.0095010694,    -0.013256797,    0.0051014382 ],
  [               0,               0,               0,       -41.83007,       48.697401,      -10.594034 ],
  [               0,               0,               0,       19.304157,      -23.199656,       4.6328619 ],
  [               0,               0,               0,       23.219399,      -27.205743,        5.490796 ],
  [               0,               0,               0,      -9.0128959,         11.3705,      -3.6166738 ],
  [               0,               0,               0,     0.010357613,    -0.017393745,    0.0079025528 ],
  [               0,               0,               0,       3.7509343,      -2.6693833,      0.59834288 ],
  [               0,               0,               0,       3.3465581,      -4.8326601,     0.047859703 ],
 ],
 [
  [               0,               0,               0,               0,     0.022570008,    0.0070234877 ],
  [               0,               0,               0,               0,       21.822337,      -16.867702 ],
  [               0,               0,               0,               0,      -35.720024,       27.951035 ],
  [               0,               0,               0,               0,       19.229395,      -15.292257 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0024493402,     0.012471224,   -0.0065541896 ],
  [               0,               0,               0,      -1.6912792,       2.9368625,      0.19101528 ],
  [               0,               0,               0,       1.8710671,       -3.932844,       1.1702999 ],
  [               0,               0,               0,       2.3858632,      -5.0761415,       1.7219706 ],
  [               0,               0,               0,    -0.091940098,      0.27740961,     -0.24588643 ],
  [               0,               0,               0,   0.00044106981,   -0.0011604821,   0.00072689026 ],
  [               0,               0,               0,      -9.0066123,       26.914615,      -15.784986 ],
  [               0,               0,               0,       -33.22953,        70.49601,      -31.598148 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.018419858,   -0.0093335504,     0.014172257,     0.060271661 ],
  [               0,               0,       1304.3587,      -1875.0106,      -2590.4326,       3727.5059 ],
  [               0,               0,       78.509916,      -31.122124,      -560.12947,       634.23467 ],
  [               0,               0,       767.05182,      -1053.1049,      -1758.0074,       2421.4599 ],
  [               0,               0,       2065.7514,      -2941.6608,      -4190.1132,       5965.3938 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer1_etaoffv4_35_e = [
 [
  [               0,               0,               0,               0,    -0.037227254,     0.053768321 ],
  [               0,               0,               0,               0,       14.804258,       18.129068 ],
  [               0,               0,               0,               0,      -180.48005,       258.16885 ],
  [               0,               0,               0,               0,       102.14184,      -138.88327 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0034816656 ],
  [               0,               0,               0,               0,               0,       2.3672005 ],
  [               0,               0,               0,               0,               0,       -1.503973 ],
  [               0,               0,               0,               0,               0,      0.23429757 ],
  [               0,               0,               0,               0,               0,    -0.021171274 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.32544169,     -0.48007175 ],
  [               0,               0,               0,               0,       767.18289,       -1134.417 ],
  [               0,               0,               0,               0,       104.97343,      -158.59076 ],
  [               0,               0,               0,               0,       -17.48454,       26.132983 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,       58090.642 ],
  [               0,               0,               0,               0,               0,   -0.0061715456 ],
  [               0,               0,               0,               0,               0,    0.0061715354 ],
  [               0,               0,               0,               0,               0,    0.0061714979 ],
  [               0,               0,               0,               0,               0,   5.0314822e-08 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0072939781,   -0.0047953209,     0.012398151,      0.06861252 ],
  [               0,               0,     -0.16593527,         12.1231,      -39.119856,       32.797299 ],
  [               0,               0,       1.2216275,        -16.3093,         44.3311,      -34.697792 ],
  [               0,               0,      -5.3736447,       15.992197,      -8.3812057,      -6.0346289 ],
  [               0,               0,     -0.89026869,       4.2870953,      -6.8786559,       3.6770918 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.028060589,     0.058830854 ],
  [               0,               0,               0,               0,       3.5998143,      -5.4701838 ],
  [               0,               0,               0,               0,      -1.9749921,       2.7856979 ],
  [               0,               0,               0,               0,     -0.64327621,      0.32540316 ],
  [               0,               0,               0,               0,      0.25073992,     -0.47600416 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.034870127,    -0.082717692,     -0.16987279,       0.4064674 ],
  [               0,               0,       12.093115,       -64.39723,       115.66937,      -69.601471 ],
  [               0,               0,       32.776404,      -200.98281,       407.92526,      -274.79053 ],
  [               0,               0,      -12.864736,       83.979025,      -183.41762,       133.08018 ],
  [               0,               0,       9.6315021,      -63.523973,       139.52294,      -102.04801 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0042248571,     0.012257516 ],
  [               0,               0,               0,               0,      -59.962106,       140.55222 ],
  [               0,               0,               0,               0,       58.569443,      -136.14202 ],
  [               0,               0,               0,               0,       4.8297203,      -10.675595 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0087059466,     0.021859536 ],
  [               0,               0,               0,               0,       15.159325,      -32.097594 ],
  [               0,               0,               0,               0,       8.7197111,      -21.271894 ],
  [               0,               0,               0,               0,       6.8742091,      -16.838858 ],
  [               0,               0,               0,               0,       1.5477114,       -3.594429 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.010790695,    0.0043176925,     0.059329535 ],
  [               0,               0,               0,       -5.377332,       24.111307,      -20.773483 ],
  [               0,               0,               0,      -104.12708,       480.12273,      -553.90552 ],
  [               0,               0,               0,      -32.244379,       173.62857,      -231.18883 ],
  [               0,               0,               0,     0.011133523,   -0.0047721947,    -0.052662775 ],
  [               0,               0,               0,     -0.30599388,      -3.3472149,       11.116963 ],
  [               0,               0,               0,    -0.028532297,        1.020177,      -4.8245976 ],
 ],
 [
  [               0,               0,               0,    0.0035661759,    0.0026257186,    0.0044749033 ],
  [               0,               0,               0,       1.9419578,       4.8205927,       15.968033 ],
  [               0,               0,               0,       674.92949,      -383762.29,      -52341.905 ],
  [               0,               0,               0,       17.425409,      -39.576851,      -9.1837942 ],
  [               0,               0,               0,        8.604218,      -18.170697,      -6.5384125 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer1_etaoffv4_35_b = [
 [
  [               0,               0,      -12176.376,        209.1362,      -1.1835476,    0.0013290093 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -5.5209313e-05,   0.00094017402 ],
  [               0,               0,               0,               0,      -1.6458524,       3.5926893 ],
  [               0,               0,               0,               0,      0.25070498,     -0.77608673 ],
  [               0,               0,               0,               0,     -0.86026054,      -0.5444419 ],
  [               0,               0,               0,               0,     -0.30997123,     0.031460992 ],
  [               0,               0,               0,               0,   -0.0005246859,   0.00023593725 ],
  [               0,               0,               0,               0,       -1.240897,       1.1620944 ],
  [               0,               0,               0,               0,     0.067053542,       1.7288516 ],
 ],
 [
  [               0,               0,               0,   0.00035445305,    0.0014973805,  -2.8142592e-05 ],
  [               0,               0,               0,       10.283691,      -19.680676,       11.275397 ],
  [               0,               0,               0,      -3.2990708,       4.6817725,      -2.5165612 ],
  [               0,               0,               0,       8.2812548,      -10.331937,       2.2247329 ],
  [               0,               0,               0,       2.5605089,      -3.1565556,      0.85892851 ],
  [               0,               0,               0,    0.0018220591,   -0.0016756905,   0.00031172888 ],
  [               0,               0,               0,      -7.1055283,       3.5723701,       2.0755882 ],
  [               0,               0,               0,      -34.018647,       34.412794,      -9.4884596 ],
 ],
 [
  [               0,               0,               0,               0,     0.025143742,    0.0055338082 ],
  [               0,               0,               0,               0,       5.1487806,      -3.5349253 ],
  [               0,               0,               0,               0,       -19.46238,       14.971914 ],
  [               0,               0,               0,               0,        15.95587,      -12.688927 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.049426436,     -0.12687787,      0.10965315 ],
  [               0,               0,               0,       4.2183785,       -8.960237,        5.126488 ],
  [               0,               0,               0,      -3.7353907,       7.9634052,      -4.5827774 ],
  [               0,               0,               0,      -4.1558044,       8.8517617,      -5.0364302 ],
  [               0,               0,               0,    -0.060579236,      0.12647933,    -0.069101831 ],
  [               0,               0,               0,   -0.0028295716,    0.0060578036,    -0.003069888 ],
  [               0,               0,               0,       9.7967291,      -21.535311,       13.553918 ],
  [               0,               0,               0,        7.989959,      -15.716082,       11.548222 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      -3.1001214,      -1.4434504,       2.0354436,       8.5300926 ],
  [               0,               0,      0.10511334,       4.1089408,       8.5321548,        19.70814 ],
  [               0,               0,      -10.458145,      -4.7435462,       7.0575244,       28.437424 ],
  [               0,               0,               0,               0,               0,              -1 ],
  [               0,               0,     -0.13981885,     -0.17421617,     -0.27797111,     -0.82593078 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.041067899,    -0.025817579,     0.025411095,      0.14637042 ],
  [               0,               0,       444.23948,      -635.81165,      -854.31324,       1226.3113 ],
  [               0,               0,      -304.22934,       507.69521,       299.15693,      -577.11144 ],
  [               0,               0,       339.80941,      -428.05357,      -958.40207,        1249.258 ],
  [               0,               0,       1818.5676,      -2588.4272,      -3700.3612,       5265.6829 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer1_etaoffv4_35_e = [
 [
  [               0,               0,               0,               0,               0,    0.0024947745 ],
  [               0,               0,               0,               0,               0,       -39.96352 ],
  [               0,               0,               0,               0,               0,      0.96222176 ],
  [               0,               0,               0,               0,               0,      0.12583867 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0019098072 ],
  [               0,               0,               0,               0,               0,       8.7445788 ],
  [               0,               0,               0,               0,               0,     -0.45109681 ],
  [               0,               0,               0,               0,               0,      -0.3780125 ],
  [               0,               0,               0,               0,               0,     -0.25352181 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.047248489,     0.081609891 ],
  [               0,               0,               0,               0,      0.72005034,       2.3848546 ],
  [               0,               0,               0,               0,       69.339062,      -104.57344 ],
  [               0,               0,               0,               0,      -53.023748,       79.002159 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0057777365,   -0.0083869715,    -0.031738393,     0.051310085 ],
  [               0,               0,       -9.224672,       86.866922,      -203.10504,       142.22813 ],
  [               0,               0,      -13.827889,       69.252401,      -115.05459,       62.313163 ],
  [               0,               0,        8.524092,      -32.870961,       39.634658,      -15.223654 ],
  [               0,               0,      -0.7626887,       4.1938286,      -7.6343881,       4.6076318 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0053761277,   -0.0086366737 ],
  [               0,               0,               0,               0,        -6.39041,        14.67051 ],
  [               0,               0,               0,               0,     -0.34595032,     -0.28681666 ],
  [               0,               0,               0,               0,     -0.68325789,      0.42197862 ],
  [               0,               0,               0,               0,    -0.053732469,     0.061488086 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0090655515,    -0.021959418,    -0.043771646,      0.10730168 ],
  [               0,               0,        7.146863,      -15.777959,      -19.446222,       46.909318 ],
  [               0,               0,       8.8471411,      -30.995188,       14.934814,       22.257268 ],
  [               0,               0,      0.73505113,       10.133963,       -49.42412,       51.293265 ],
  [               0,               0,      0.11925354,      -6.0397273,       24.059781,      -24.972865 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0010841192,    0.0068717127 ],
  [               0,               0,               0,               0,      -114.10987,       264.02382 ],
  [               0,               0,               0,               0,       41.561165,       -96.84645 ],
  [               0,               0,               0,               0,       4.2744235,      -9.5243296 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.010066011,      0.02578293 ],
  [               0,               0,               0,               0,       9.2592335,      -19.394626 ],
  [               0,               0,               0,               0,        -1.20874,       1.9945828 ],
  [               0,               0,               0,               0,       1.4272377,      -4.2362493 ],
  [               0,               0,               0,               0,       1.4342769,       -3.363434 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0015435522,    -0.002855834,    0.0062032291 ],
  [               0,               0,               0,       0.6271023,      -20.660462,       51.702554 ],
  [               0,               0,               0,      -45.558251,       222.79906,       -273.8627 ],
  [               0,               0,               0,      -23.570318,       115.52517,      -141.46218 ],
  [               0,               0,               0,    -0.004746711,  -0.00043015563,     0.029599619 ],
  [               0,               0,               0,       15.094343,      -1.9239235,      -82.258869 ],
  [               0,               0,               0,      -31.057292,     -0.60621997,       185.49653 ],
 ],
 [
  [               0,               0,               0,               0,    0.0077308179,    0.0082912069 ],
  [               0,               0,               0,               0,      -1.1484379,       8.6846982 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,               0,      -47.929648,       117.85781 ],
  [               0,               0,               0,               0,      0.18966583,      0.10524248 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer1_etaoffv4_35_b = [
 [
  [               0,               0,      -9800.6678,       300.23586,      -1.8428957,     0.001827828 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00021987016,    0.0012995333 ],
  [               0,               0,               0,               0,      -1.8743677,       3.3230599 ],
  [               0,               0,               0,               0,     -0.42368674,     -0.67084057 ],
  [               0,               0,               0,               0,       1.3825385,      -1.3590562 ],
  [               0,               0,               0,               0,       0.2604272,     -0.15296598 ],
  [               0,               0,               0,               0,   -0.0004733068,   0.00026802583 ],
  [               0,               0,               0,               0,      -1.1017621,       1.2810663 ],
  [               0,               0,               0,               0,       1.9801286,      0.89102194 ],
 ],
 [
  [               0,               0,               0,     0.011295569,    -0.011135977,    0.0034188212 ],
  [               0,               0,               0,       -13.12863,       11.435406,       1.1873907 ],
  [               0,               0,               0,       4.5889775,      -9.0500835,       3.3082392 ],
  [               0,               0,               0,      -6.8518204,       10.426061,      -5.0964711 ],
  [               0,               0,               0,     -0.37286838,       1.2951326,     -0.99885509 ],
  [               0,               0,               0,    0.0034181609,   -0.0041702077,     0.001399408 ],
  [               0,               0,               0,      -11.018765,        8.601433,     -0.43062674 ],
  [               0,               0,               0,       25.355337,      -27.569107,       7.0580285 ],
 ],
 [
  [               0,               0,               0,               0,     0.028307203,   -0.0038080195 ],
  [               0,               0,               0,               0,      -23.579076,       19.401908 ],
  [               0,               0,               0,               0,      -16.221625,       12.324018 ],
  [               0,               0,               0,               0,       23.818793,      -18.939229 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   0.00010599908,   -0.0016487792,    0.0025552761 ],
  [               0,               0,               0,      -1.2386223,      0.41630568,       3.0278674 ],
  [               0,               0,               0,       2.8920714,      -18.883076,       10.896143 ],
  [               0,               0,               0,       12.659844,      -39.832217,       21.053653 ],
  [               0,               0,               0,       1.8974733,     -0.21289631,      -2.9670465 ],
  [               0,               0,               0,   -0.0024177983,   0.00083762604,    0.0071785718 ],
  [               0,               0,               0,     -0.78701614,        2.397079,      -1.3011731 ],
  [               0,               0,               0,       0.2229485,      -1.2986309,     -0.30866677 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.10386626,     -0.15458219,      -0.2158098,       0.3294321 ],
  [               0,               0,       1837.3616,      -2644.8427,      -3675.3106,       5294.3173 ],
  [               0,               0,       395.06665,       -490.0634,      -1219.4032,       1589.6531 ],
  [               0,               0,       818.98779,      -1128.0988,      -1868.9855,       2581.4597 ],
  [               0,               0,        2289.964,      -3257.9482,      -4650.7734,       6615.1517 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer1_etaoffv4_35_e = [
 [
  [               0,               0,               0,               0,     0.041152663,    -0.057161402 ],
  [               0,               0,               0,               0,      -17.744796,      -13.865008 ],
  [               0,               0,               0,               0,      -98.565963,       144.65815 ],
  [               0,               0,               0,               0,      -80.735619,       115.90992 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      -1.8064912,        2.701963 ],
  [               0,               0,               0,               0,       -61.74872,       91.171721 ],
  [               0,               0,               0,               0,       33.389023,      -49.350525 ],
  [               0,               0,               0,               0,       29.157495,      -43.545809 ],
  [               0,               0,               0,               0,       1.2232493,      -1.8152372 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0017354092,     0.032184631 ],
  [               0,               0,               0,               0,      -4.4966248,       9.0526366 ],
  [               0,               0,               0,               0,        59.12372,      -89.155087 ],
  [               0,               0,               0,               0,      -54.108243,       80.530557 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.010996508,   -0.0052541849,     0.017368425,      0.07951444 ],
  [               0,               0,      0.20488123,       13.944279,      -47.999897,       40.815137 ],
  [               0,               0,      0.90508582,      -18.292424,       53.343581,      -42.740821 ],
  [               0,               0,      -7.2576031,       22.662585,      -15.093331,      -4.6556672 ],
  [               0,               0,      -1.8007723,       8.9578632,      -14.838669,       8.1849458 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0074226838,    -0.012157166 ],
  [               0,               0,               0,               0,      -2.0440115,       6.2457407 ],
  [               0,               0,               0,               0,      -2.1229117,       3.0435022 ],
  [               0,               0,               0,               0,      -2.6177886,       4.0102866 ],
  [               0,               0,               0,               0,     -0.22299408,      0.40053513 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.011326288,     -0.02796115,    -0.055389064,      0.13898948 ],
  [               0,               0,       4.1937408,      -7.2693694,      -19.974163,       36.955026 ],
  [               0,               0,       18.627067,       -115.8265,       239.96458,      -166.62585 ],
  [               0,               0,      -15.232986,       100.69843,      -220.99038,       160.09338 ],
  [               0,               0,       8.5822816,      -56.266447,       122.79502,      -89.210566 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00054823262,    0.0031331435 ],
  [               0,               0,               0,               0,      -291.83375,       673.73281 ],
  [               0,               0,               0,               0,       78.366475,      -181.93792 ],
  [               0,               0,               0,               0,       15.308499,      -34.595462 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00095946302,    0.0028026563 ],
  [               0,               0,               0,               0,       2.2383578,      -3.5606158 ],
  [               0,               0,               0,               0,       -3.329749,       6.8758878 ],
  [               0,               0,               0,               0,      0.28583703,      -1.7299735 ],
  [               0,               0,               0,               0,      0.61480976,      -1.4540372 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -3.7435478e-05,   0.00033552651 ],
  [               0,               0,               0,               0,       -7.265301,      -1.3814123 ],
  [               0,               0,               0,               0,    -0.088651035,       2.7761125 ],
  [               0,               0,               0,               0,       4.7304707,      -1.4436581 ],
  [               0,               0,               0,               0,      -1.0121309,      -2.7280418 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0049223387,    0.0023346107,    -0.026579383 ],
  [               0,               0,               0,       15.063132,      -106.07472,       174.38333 ],
  [               0,               0,               0,       57.053741,      -276.68169,       333.98246 ],
  [               0,               0,               0,      -132.20696,       647.53765,       -792.8125 ],
  [               0,               0,               0,    0.0088554765,    -0.048346633,       0.0656046 ],
  [               0,               0,               0,       9.8545393,       3.8528337,      -66.424274 ],
  [               0,               0,               0,      -2.6023919,       37.037606,      -75.288893 ],
 ],
 [
  [               0,               0,               0,               0,       1.4714764,      -3.6424686 ],
  [               0,               0,               0,               0,       -3283.424,       8166.7632 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,               0,      -270.04887,       668.72422 ],
  [               0,               0,               0,               0,       1.2490449,      -2.7333983 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer1_etaoffv4_35_b = [
 [
  [               0,               0,      -59059.982,       759.46249,      -2.9241615,    0.0020111221 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00023796995,   0.00079559977 ],
  [               0,               0,               0,               0,     -0.34139661,       3.1383417 ],
  [               0,               0,               0,               0,      0.44071386,     -0.69076829 ],
  [               0,               0,               0,               0,      0.60530937,     -0.81067725 ],
  [               0,               0,               0,               0,     -0.17180873,    -0.038083613 ],
  [               0,               0,               0,               0,   7.4603359e-05,   3.1246391e-05 ],
  [               0,               0,               0,               0,      -1.4170014,       2.6185155 ],
  [               0,               0,               0,               0,      -0.3853332,       -1.572141 ],
 ],
 [
  [               0,               0,               0,    0.0011322986,  -0.00013601557,  -2.0262835e-05 ],
  [               0,               0,               0,       10.740341,      -21.401802,        13.02691 ],
  [               0,               0,               0,       5.5670155,      -18.296523,       9.9890437 ],
  [               0,               0,               0,       30.172004,      -43.708765,       16.194214 ],
  [               0,               0,               0,       0.2238457,       1.3324613,      -1.2314199 ],
  [               0,               0,               0,   -0.0022945587,     0.002837077,  -0.00099852913 ],
  [               0,               0,               0,      -10.973834,       13.170919,      -2.1496052 ],
  [               0,               0,               0,      -5.2725395,       4.0830963,      0.70006146 ],
 ],
 [
  [               0,               0,               0,               0,      0.02240193,    0.0072315089 ],
  [               0,               0,               0,               0,      -12.367092,       10.431909 ],
  [               0,               0,               0,               0,      -14.844928,       11.264171 ],
  [               0,               0,               0,               0,       17.785304,      -14.139778 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0011068397,   -0.0031312692,     0.002265579 ],
  [               0,               0,               0,        19.79786,      -49.444638,       29.704838 ],
  [               0,               0,               0,      -0.8801289,       16.478365,      -12.189081 ],
  [               0,               0,               0,       28.977311,      -38.841889,       9.9133454 ],
  [               0,               0,               0,      -3.8610188,       6.3066963,      -3.6366377 ],
  [               0,               0,               0,    0.0003202473,   -0.0006879017,    0.0007540627 ],
  [               0,               0,               0,      -1.7721666,       3.9348861,     -0.90039556 ],
  [               0,               0,               0,       1.6359342,      -4.9720275,       1.8445886 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0027261707,    0.0012195662,   -0.0018689979,   -0.0071762709 ],
  [               0,               0,      -5.2654983,      -5.0823381,      -6.9370323,      -5.4638547 ],
  [               0,               0,      -23.580858,      -20.149562,      -128.08257,      -54.434204 ],
  [               0,               0,      -5.0222302,       10703.035,       23251.289,       9039.1068 ],
  [               0,               0,       15.382704,       43.002124,       101.37967,       27.896735 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.13856374,     -0.29177509,     -0.35201574,      0.70469113 ],
  [               0,               0,       464.25969,       -669.6939,      -881.85448,       1276.9987 ],
  [               0,               0,      -753.21575,       1168.5498,       1116.6844,      -1784.3279 ],
  [               0,               0,       268.64874,      -296.41424,      -974.04205,       1209.6255 ],
  [               0,               0,       2763.3204,      -3937.2065,      -5616.5936,       8000.8816 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer1_etaoffv4_35_e = [
 [
  [               0,               0,               0,               0,       8.3279524,      -12.129765 ],
  [               0,               0,               0,               0,      -44.657743,       64.199258 ],
  [               0,               0,               0,               0,       37.682955,      -54.152936 ],
  [               0,               0,               0,               0,       5.2866687,      -7.6099283 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0028310585,     0.028127092 ],
  [               0,               0,               0,               0,      -147.85822,       218.31971 ],
  [               0,               0,               0,               0,       92.852817,      -137.19458 ],
  [               0,               0,               0,               0,       -49.87434,       70.811173 ],
  [               0,               0,               0,               0,     0.043497087,    -0.081464735 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.11537027,      0.17386908 ],
  [               0,               0,               0,               0,      -67.957201,       109.89466 ],
  [               0,               0,               0,               0,      -296.14937,       439.38582 ],
  [               0,               0,               0,               0,       375.10066,      -557.22933 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0017407107,   0.00010765617,    0.0096260423,      0.04019385 ],
  [               0,               0,      -1.8356224,       9.9155148,      -17.800636,       11.181521 ],
  [               0,               0,       3.9877563,       -20.42516,       34.835735,      -20.304296 ],
  [               0,               0,      -4.4301075,       21.377812,      -34.223975,       17.649016 ],
  [               0,               0,      -1.4534798,       7.1711856,      -11.774284,       6.4334906 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.011038693,    -0.018425278 ],
  [               0,               0,               0,               0,       -6.562461,       14.880952 ],
  [               0,               0,               0,               0,      0.68655128,        -2.49306 ],
  [               0,               0,               0,               0,      0.14414312,      -1.4239593 ],
  [               0,               0,               0,               0,     -0.20733496,      0.39519569 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00034343881,  -0.00018647621,   0.00090336298,    0.0054775899 ],
  [               0,               0,       1.1698072,      -1.8910614,      -5.1346922,       10.586219 ],
  [               0,               0,       1.4874812,      -3.2756943,      -6.7203826,       13.668424 ],
  [               0,               0,     -0.50345624,       1.2210771,       1.7146268,      -5.4156101 ],
  [               0,               0,     -0.37128747,      0.74188861,       1.9833653,       -3.993609 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.15796111,      0.36904118 ],
  [               0,               0,               0,               0,      -91.040642,       210.93818 ],
  [               0,               0,               0,               0,       14.622224,      -34.793598 ],
  [               0,               0,               0,               0,       9.4202672,       -21.42171 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.001353532,    0.0039685188 ],
  [               0,               0,               0,               0,      -0.2124083,       2.0394962 ],
  [               0,               0,               0,               0,     -0.65008826,      0.50944732 ],
  [               0,               0,               0,               0,      0.40348533,      -1.9624431 ],
  [               0,               0,               0,               0,      0.19503376,     -0.45361284 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.064883091,      0.16674305 ],
  [               0,               0,               0,               0,        48.44553,      -111.97531 ],
  [               0,               0,               0,               0,      -5.9646842,       13.029304 ],
  [               0,               0,               0,               0,       4.4099964,      -10.680603 ],
  [               0,               0,               0,               0,    -0.013979887,     0.033809983 ],
  [               0,               0,               0,               0,       9.9685377,      -21.975109 ],
  [               0,               0,               0,               0,       25.504245,      -64.525912 ],
 ],
 [
  [               0,               0,               0,               0,    0.0075784578,    0.0080833954 ],
  [               0,               0,               0,               0,      -2.1062405,       14.985023 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,               0,       -16.84755,       41.097109 ],
  [               0,               0,               0,               0,       3.1734274,      -7.0185443 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer1_etaoffv4_35_b = [
 [
  [               0,               0,       16543.388,         30.0752,      -1.0255302,    0.0012863146 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00064483886,    0.0018499693 ],
  [               0,               0,               0,               0,      0.44316569,       1.9403792 ],
  [               0,               0,               0,               0,      0.29752205,      -1.0208722 ],
  [               0,               0,               0,               0,    -0.075792914,     -0.97955785 ],
  [               0,               0,               0,               0,     -0.16190984,   -0.0014044975 ],
  [               0,               0,               0,               0,   4.0671266e-05,   4.5319767e-05 ],
  [               0,               0,               0,               0,     -0.22368983,       2.6896889 ],
  [               0,               0,               0,               0,    -0.051002088,       -1.733666 ],
 ],
 [
  [               0,               0,               0,     0.010382962,   -0.0048856304,   0.00096167865 ],
  [               0,               0,               0,      -9.2060799,       3.0270933,       3.6506685 ],
  [               0,               0,               0,       19.294457,      -22.119524,       5.3738284 ],
  [               0,               0,               0,       20.726347,      -23.093302,       5.2733309 ],
  [               0,               0,               0,     -0.45959066,      0.83093738,     -0.41654604 ],
  [               0,               0,               0,    0.0030832389,   -0.0033298406,   0.00092486849 ],
  [               0,               0,               0,      -14.600836,       15.994034,      -1.8425614 ],
  [               0,               0,               0,       2.8938962,      -3.5593282,     -0.75752705 ],
 ],
 [
  [               0,               0,               0,               0,     0.024171841,    0.0071665565 ],
  [               0,               0,               0,               0,      -4.3548221,       4.0365706 ],
  [               0,               0,               0,               0,      -16.907584,       12.920793 ],
  [               0,               0,               0,               0,       16.484203,      -13.107879 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.006574853,    -0.010088082,    0.0050256276 ],
  [               0,               0,               0,       3.8565244,      -13.210274,       11.191018 ],
  [               0,               0,               0,     -0.97994605,       3.4803082,      -3.1638121 ],
  [               0,               0,               0,       2.3334426,      -3.4903918,      0.29132274 ],
  [               0,               0,               0,      0.35491231,     -0.51600578,     0.015562857 ],
  [               0,               0,               0,    0.0010716735,   -0.0017196084,    0.0007553512 ],
  [               0,               0,               0,       -1.183531,       1.3003918,       1.7177606 ],
  [               0,               0,               0,      -8.2182576,       19.628661,       -7.214416 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0040673777,     -0.11885671,    -0.079245337,      0.37766775 ],
  [               0,               0,      -219.91881,       327.41037,       518.57934,      -762.46292 ],
  [               0,               0,         -105.68,       259.75771,      -211.64918,       80.347422 ],
  [               0,               0,      -148.44107,       321.49148,      -247.17365,       125.85467 ],
  [               0,               0,       2866.7595,      -4086.7189,       -5829.058,       8307.7805 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer1_etaoffv4_35_e = [
 [
  [               0,               0,               0,               0,   -0.0032026952,     0.006016222 ],
  [               0,               0,               0,               0,      -19.085696,      -12.206435 ],
  [               0,               0,               0,               0,      -59.414523,       86.770385 ],
  [               0,               0,               0,               0,       -50.98897,       74.537717 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00016644288,      0.01771457 ],
  [               0,               0,               0,               0,       -216.1848,       319.01869 ],
  [               0,               0,               0,               0,       94.104473,      -139.01024 ],
  [               0,               0,               0,               0,       -127.5335,       183.87291 ],
  [               0,               0,               0,               0,       0.0810283,     -0.13143318 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0075744617,     0.021939825 ],
  [               0,               0,               0,               0,      -117.88978,       179.08779 ],
  [               0,               0,               0,               0,       35.662015,      -54.469838 ],
  [               0,               0,               0,               0,      -15.966391,       23.934952 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.020158189,    -0.026940263,    -0.052483905,     0.072349468 ],
  [               0,               0,      -45.741945,       235.27901,      -408.48831,       240.75251 ],
  [               0,               0,      -28.236928,       155.99034,      -283.93583,       169.57062 ],
  [               0,               0,      -1.1168918,       17.153283,      -48.197735,       36.840786 ],
  [               0,               0,      -8.5107822,       42.379798,       -70.45382,       39.118053 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0056476734,   -0.0085435396 ],
  [               0,               0,               0,               0,      -3.2732484,        8.849955 ],
  [               0,               0,               0,               0,     -0.73760504,      0.26402156 ],
  [               0,               0,               0,               0,      -1.4012816,       1.5525128 ],
  [               0,               0,               0,               0,     -0.23240489,      0.43655827 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.017207977,    -0.041972428,    -0.084015244,      0.20766374 ],
  [               0,               0,       7.2934708,      -36.662246,       63.941973,      -38.204173 ],
  [               0,               0,       26.172888,      -154.67521,       301.82255,      -195.26714 ],
  [               0,               0,      -3.7277292,       28.555481,      -71.068161,       56.752521 ],
  [               0,               0,       11.707887,      -76.384893,       165.99235,      -120.13182 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00071799622,    0.0035518244 ],
  [               0,               0,               0,               0,        -115.968,       268.41857 ],
  [               0,               0,               0,               0,       44.730926,       -104.1471 ],
  [               0,               0,               0,               0,      0.49791745,     -0.86134894 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.021546557,    -0.046347884 ],
  [               0,               0,               0,               0,      -5.0678949,       14.103542 ],
  [               0,               0,               0,               0,      -1.4069034,        2.162375 ],
  [               0,               0,               0,               0,      0.60484037,       -2.522115 ],
  [               0,               0,               0,               0,      0.17718643,     -0.38974311 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.059391938,      0.18488969,    -0.088147206 ],
  [               0,               0,               0,      -21.279012,       122.27817,      -165.91069 ],
  [               0,               0,               0,       324.95088,       -1591.177,       1945.9096 ],
  [               0,               0,               0,      -197.33991,       966.87593,       -1183.997 ],
  [               0,               0,               0,    -0.018212244,      0.10121543,     -0.13991403 ],
  [               0,               0,               0,       41.970107,      -206.67407,       255.46179 ],
  [               0,               0,               0,      -24.935854,       160.57478,      -243.34604 ],
 ],
 [
  [               0,               0,               0,               0,     0.002427126,      0.02710609 ],
  [               0,               0,               0,               0,       1.0332582,       9.1723418 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,               0,      -17.440919,       42.646591 ],
  [               0,               0,               0,               0,      0.21589873,      0.41091395 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer1_etaoffv4_35_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00036675568,    0.0013663252 ],
  [               0,               0,               0,               0,      0.18112197,       2.6661081 ],
  [               0,               0,               0,               0,      0.35813118,      -1.0976483 ],
  [               0,               0,               0,               0,      0.11208698,      -1.1239729 ],
  [               0,               0,               0,               0,     -0.17555605,    -0.017205569 ],
  [               0,               0,               0,               0,   2.0280562e-05,   3.3744726e-05 ],
  [               0,               0,               0,               0,      -1.5317852,       3.1417475 ],
  [               0,               0,               0,               0,      -1.8832201,      -1.2482188 ],
 ],
 [
  [               0,               0,               0,               0,   0.00043820109,   0.00068191447 ],
  [               0,               0,               0,               0,      -1.0141699,       3.7026686 ],
  [               0,               0,               0,               0,    -0.014870854,     -0.77574762 ],
  [               0,               0,               0,               0,     -0.11531593,     -0.89866077 ],
  [               0,               0,               0,               0,     -0.20069608,    -0.026285897 ],
  [               0,               0,               0,               0,  -5.0770947e-05,   8.8407566e-05 ],
  [               0,               0,               0,               0,       -0.402557,       2.3565911 ],
  [               0,               0,               0,               0,      -1.9026914,      -1.0501791 ],
 ],
 [
  [               0,               0,               0,               0,     0.025804519,   -0.0010976758 ],
  [               0,               0,               0,               0,      -5.5884432,       5.0790536 ],
  [               0,               0,               0,               0,      -25.919901,       20.043151 ],
  [               0,               0,               0,               0,       23.960513,      -19.055131 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0041903835,   -0.0090374245,    0.0071440369 ],
  [               0,               0,               0,      -2.1648982,       4.6466018,     -0.40514138 ],
  [               0,               0,               0,     -0.28313694,      0.80874797,      -1.4841397 ],
  [               0,               0,               0,      0.86742198,      -1.9963424,     -0.12045031 ],
  [               0,               0,               0,      0.13469736,     -0.34465467,      0.09053137 ],
  [               0,               0,               0,   -0.0002216438,   0.00052387372,  -0.00031561498 ],
  [               0,               0,               0,       15.607177,       -30.25333,       17.113543 ],
  [               0,               0,               0,      -33.143495,       63.660362,      -36.034745 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.10041953,     -0.29759747,      -0.3181929,      0.78053525 ],
  [               0,               0,      -178.94454,       260.79843,       449.23631,      -647.28679 ],
  [               0,               0,      -16.105062,        193.7248,      -580.04289,       480.37006 ],
  [               0,               0,      -388.70339,       681.44633,       147.51849,       -472.3771 ],
  [               0,               0,       3241.6802,      -4618.7493,      -6600.6015,       9402.5209 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer1_etaoffv4_35_e = [
 [
  [               0,               0,               0,     0.015596169,   0.00021276498,    -0.031489082 ],
  [               0,               0,               0,       -3.852798,      -10.185115,      -16.773742 ],
  [               0,               0,               0,      -18.596888,      0.38942442,        39.71429 ],
  [               0,               0,               0,      -66.194473,      0.86952576,       137.94874 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.17615402,     -0.25479894 ],
  [               0,               0,               0,               0,       1028.9221,       -1497.421 ],
  [               0,               0,               0,               0,      -126.07479,       184.15589 ],
  [               0,               0,               0,               0,      -383.33025,       562.26514 ],
  [               0,               0,               0,               0,      0.24094459,     -0.22617147 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.22840417,      0.34542939 ],
  [               0,               0,               0,               0,      -232.77042,       352.38528 ],
  [               0,               0,               0,               0,      -49.945453,       72.878781 ],
  [               0,               0,               0,               0,       73.515502,      -109.00267 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0055180655,    0.0070095158,      0.01115041,     0.023599437 ],
  [               0,               0,      0.38944302,     -0.55959548,      -1.2779641,       2.3629433 ],
  [               0,               0,       1.6182493,      -9.2586487,       17.239255,      -10.965368 ],
  [               0,               0,      -2.8954571,       13.156724,      -19.816097,        9.427296 ],
  [               0,               0,     -0.94588062,       4.6829954,      -7.7196702,       4.2373285 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0059375733,   -0.0092373819 ],
  [               0,               0,               0,               0,       -4.346825,       11.052095 ],
  [               0,               0,               0,               0,     -0.34058595,     -0.51105696 ],
  [               0,               0,               0,               0,      -1.0094733,      0.80207588 ],
  [               0,               0,               0,               0,     -0.27628083,      0.53126036 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.014524781,    -0.035684017,    -0.069610443,      0.17408075 ],
  [               0,               0,       11.483394,      -61.423519,       112.16603,      -68.914765 ],
  [               0,               0,       34.319168,       -213.3335,       440.92825,      -304.16182 ],
  [               0,               0,       -12.41421,       86.266329,      -197.99052,       149.16987 ],
  [               0,               0,       11.157139,      -72.060569,       155.17894,      -111.39611 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00028000904,    0.0026944874 ],
  [               0,               0,               0,               0,      -199.72435,       461.63403 ],
  [               0,               0,               0,               0,       54.879215,      -127.86106 ],
  [               0,               0,               0,               0,       8.1281339,      -18.178498 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00011357294,    0.0018075561 ],
  [               0,               0,               0,               0,      -8.1901037,       22.774573 ],
  [               0,               0,               0,               0,       2.3422551,      -6.5780148 ],
  [               0,               0,               0,               0,       2.9434248,      -7.8899911 ],
  [               0,               0,               0,               0,      0.27262939,     -0.58789992 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.010314274,     0.025716516 ],
  [               0,               0,               0,               0,       -513.5499,       1232.3454 ],
  [               0,               0,               0,               0,       316.90523,      -759.99791 ],
  [               0,               0,               0,               0,       1435.2783,      -3439.3531 ],
  [               0,               0,               0,               0,      -0.2993501,     -0.69282939 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0034774424,    0.0054027134,    -0.025352482 ],
  [               0,               0,               0,      -10.551745,       13.653603,       35.920544 ],
  [               0,               0,               0,       -38.88406,       189.81639,      -233.19995 ],
  [               0,               0,               0,      -25.137775,       123.40457,      -151.33125 ],
  [               0,               0,               0,   -0.0025296492,   -0.0039459232,     0.025274767 ],
  [               0,               0,               0,       12.071494,      0.43325654,      -71.137769 ],
  [               0,               0,               0,        11.10461,      0.52306894,      -68.437882 ],
 ],
 [
  [               0,               0,               0,               0,       0.0232382,     0.014598274 ],
  [               0,               0,               0,               0,       4.0289891,       29.981895 ],
  [               0,               0,               0,               0,   1.5801974e-05,     -0.49675512 ],
  [               0,               0,               0,               0,     -0.67052675,       1.4589371 ],
  [               0,               0,               0,               0,       1.9875145,      -3.5795564 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]


ele_layer1_etaoffv4_35_b = [
    ele5_layer1_etaoffv4_35_b,
    ele10_layer1_etaoffv4_35_b,
    ele25_layer1_etaoffv4_35_b,
    ele50_layer1_etaoffv4_35_b,
    ele75_layer1_etaoffv4_35_b,
    ele100_layer1_etaoffv4_35_b,
    ele200_layer1_etaoffv4_35_b,
    ele500_layer1_etaoffv4_35_b,
    ele1000_layer1_etaoffv4_35_b,
    ]

ele_layer1_etaoffv4_35_e = [
    ele5_layer1_etaoffv4_35_e,
    ele10_layer1_etaoffv4_35_e,
    ele25_layer1_etaoffv4_35_e,
    ele50_layer1_etaoffv4_35_e,
    ele75_layer1_etaoffv4_35_e,
    ele100_layer1_etaoffv4_35_e,
    ele200_layer1_etaoffv4_35_e,
    ele500_layer1_etaoffv4_35_e,
    ele1000_layer1_etaoffv4_35_e,
    ]



###############################################################################
# Tables for v4 sshape correction, layer 1, 3x5 photon clusters
#

gam5_layer1_etaoffv4_35_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      -10544.275,       8747.6533 ],
  [               0,               0,               0,               0,     0.010542072,    0.0077734237 ],
  [               0,               0,               0,               0,    -0.010541924,   -0.0077731546 ],
  [               0,               0,               0,               0,    -0.010540447,   -0.0077734452 ],
  [               0,               0,               0,               0,   1.7136428e-08,  -2.0337495e-08 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      -776.66682,      -206.84814,        377.6615,       436.48822 ],
  [               0,               0,       2.3001041,      -4.0627086,       2.4047525,     -0.45688129 ],
  [               0,               0,      -2.3000517,       4.0627236,      -2.4048078,      0.45689967 ],
  [               0,               0,      -2.2993228,       4.0613709,       -2.404004,      0.45674894 ],
  [               0,               0,    0.0001368241,  -0.00024000796,   0.00013511793,  -2.4508774e-05 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam5_layer1_etaoffv4_35_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer1_etaoffv4_35_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0004197841,   0.00024737138 ],
  [               0,               0,               0,               0,       23.929604,      -3.5844094 ],
  [               0,               0,               0,               0,       -27.37512,       4.8898472 ],
  [               0,               0,               0,               0,      -20.826207,       6.7949695 ],
  [               0,               0,               0,               0,       1.5466536,      0.15196835 ],
  [               0,               0,               0,               0,   -0.0012173942,  -0.00012145146 ],
  [               0,               0,               0,               0,      -1.9295596,       1.4684282 ],
  [               0,               0,               0,               0,     -0.24486431,       1.3668677 ],
 ],
 [
  [               0,               0,               0,     0.031448625,     -0.02358779,    0.0044524221 ],
  [               0,               0,               0,       6.9370188,      -13.963743,       8.1252725 ],
  [               0,               0,               0,       19.794322,        -26.6793,       7.8221711 ],
  [               0,               0,               0,        3.812373,      -5.6199675,      0.93702148 ],
  [               0,               0,               0,      -5.1378337,       7.2230127,      -2.5820127 ],
  [               0,               0,               0,    0.0087745326,    -0.011765433,    0.0038111557 ],
  [               0,               0,               0,       14.187213,      -9.0363553,       2.1201129 ],
  [               0,               0,               0,      -11.788459,       13.046899,       1.1486231 ],
 ],
 [
  [               0,               0,               0,               0,    -0.022516254,     0.018354002 ],
  [               0,               0,               0,               0,       22.405618,       21.903161 ],
  [               0,               0,               0,               0,       795.75526,      -636.31496 ],
  [               0,               0,               0,               0,       377.42627,         -299.41 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0068884055,    -0.014378319,    0.0080663638 ],
  [               0,               0,               0,      -37.652606,       100.48017,      -63.153788 ],
  [               0,               0,               0,       11.974727,         -35.405,       24.799431 ],
  [               0,               0,               0,       2.9943393,      -12.974795,       10.793422 ],
  [               0,               0,               0,       0.4757691,    -0.051522551,      -0.9576742 ],
  [               0,               0,               0,  -0.00044819463,  -0.00094241538,    0.0017865809 ],
  [               0,               0,               0,    -0.095097665,      -2.9168363,        4.072198 ],
  [               0,               0,               0,      -12.959249,       20.808831,      -9.5537555 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.098927409,     -0.11343895,     -0.18109577,      0.20925162 ],
  [               0,               0,       4306.6129,      -6222.0117,      -8643.7868,        12491.32 ],
  [               0,               0,       264.26296,       -315.5625,      -913.89913,       1178.6285 ],
  [               0,               0,       1596.5528,      -2231.6611,      -3360.3021,       4698.2048 ],
  [               0,               0,       1963.4654,      -2792.8375,      -3975.4347,       5653.2433 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer1_etaoffv4_35_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,       3242.2531 ],
  [               0,               0,               0,               0,               0,     0.012575072 ],
  [               0,               0,               0,               0,               0,    -0.012574615 ],
  [               0,               0,               0,               0,               0,    -0.012574439 ],
  [               0,               0,               0,               0,               0,   1.0230612e-08 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer1_etaoffv4_35_b = [
 [
  [               0,               0,      -11819.036,       319.16825,      -2.1973257,    0.0034813935 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0013949935,    0.0013531525 ],
  [               0,               0,               0,               0,     -0.31246337,       2.6449616 ],
  [               0,               0,               0,               0,      -77.677819,       17.002043 ],
  [               0,               0,               0,               0,      -76.820385,       16.745895 ],
  [               0,               0,               0,               0,       8.6985687,       -1.664891 ],
  [               0,               0,               0,               0,    -0.046453513,     0.045890421 ],
  [               0,               0,               0,               0,     -0.73976406,      0.17011883 ],
  [               0,               0,               0,               0,      0.25822574,       1.5228087 ],
 ],
 [
  [               0,               0,               0,   -0.0032662465,    0.0042209163,   8.0153652e-05 ],
  [               0,               0,               0,      -26.131545,       32.845139,       -7.874399 ],
  [               0,               0,               0,       9.1466407,      -10.891484,       2.2691331 ],
  [               0,               0,               0,       7.3061897,      -8.5120659,       1.3468139 ],
  [               0,               0,               0,     -0.63488619,      0.57656075,     -0.21785796 ],
  [               0,               0,               0,    0.0049680248,   -0.0067615483,    0.0022498926 ],
  [               0,               0,               0,       18.791517,       -14.68186,        3.792107 ],
  [               0,               0,               0,       17.283743,      -15.764014,        8.205364 ],
 ],
 [
  [               0,               0,               0,               0,       1248.4672,      -204.42613 ],
  [               0,               0,               0,               0,    -0.058239119,     0.063641719 ],
  [               0,               0,               0,               0,     0.057869157,     -0.06334608 ],
  [               0,               0,               0,               0,   0.00047318746,  -0.00037611287 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,       3.0904345,        4.259613,       1.9812894 ],
  [               0,               0,               0,      -0.3547271,      0.83953312,     -0.40799044 ],
  [               0,               0,               0,      0.35437923,     -0.83891261,      0.40792704 ],
  [               0,               0,               0,      0.35348389,     -0.83648546,      0.40626069 ],
  [               0,               0,               0,  -0.00021316319,   0.00056618351,  -0.00039447047 ],
  [               0,               0,               0,   -0.0019349343,    0.0048190481,    -0.003026512 ],
  [               0,               0,               0,       12.641003,      -21.949999,       10.934369 ],
  [               0,               0,               0,        8.889825,      -14.763325,       8.4944581 ],
 ],
 [
  [               0,               0,               0,               0,               0,       1805.5826 ],
  [               0,               0,               0,               0,               0,     0.016153244 ],
  [               0,               0,               0,               0,               0,    -0.016151735 ],
  [               0,               0,               0,               0,               0,   5.1505338e-08 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.29463242,     -0.38082355,      -0.5760073,      0.74780777 ],
  [               0,               0,       961.72511,      -1403.6103,      -1927.7218,       2818.9271 ],
  [               0,               0,        1251.631,      -1784.5649,      -2769.5024,       3948.5001 ],
  [               0,               0,      -432.46466,       681.76404,       556.29769,      -928.26104 ],
  [               0,               0,       1632.4401,       -2316.231,       -3312.912,       4699.1314 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer1_etaoffv4_35_e = [
 [
  [               0,               0,               0,               0,               0,    0.0020866766 ],
  [               0,               0,               0,               0,               0,   -0.0012909865 ],
  [               0,               0,               0,               0,               0,       -1.562415 ],
  [               0,               0,               0,               0,               0,       1.0666001 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.019690918 ],
  [               0,               0,               0,               0,               0,       1.2711296 ],
  [               0,               0,               0,               0,               0,      -1.0421823 ],
  [               0,               0,               0,               0,               0,      -1.0277137 ],
  [               0,               0,               0,               0,               0,   -0.0098637407 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.022106919,    -0.030862538 ],
  [               0,               0,               0,               0,       168.12332,      -246.58689 ],
  [               0,               0,               0,               0,      -81.589436,        119.5534 ],
  [               0,               0,               0,               0,       148.15249,      -219.95211 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.069213324,     -0.22774272,      0.18871808 ],
  [               0,               0,               0,      -89.283115,       293.68793,      -238.63718 ],
  [               0,               0,               0,       21.953649,      -71.748934,       57.637238 ],
  [               0,               0,               0,       20.062027,      -65.621569,       52.584052 ],
  [               0,               0,               0,     -0.91434656,        3.097937,      -2.7039745 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0046834742,   -0.0063842012 ],
  [               0,               0,               0,               0,      -1.6654163,        4.956653 ],
  [               0,               0,               0,               0,       -1.417367,       1.8055261 ],
  [               0,               0,               0,               0,      0.64331839,       -2.031992 ],
  [               0,               0,               0,               0,      0.27977528,     -0.58142816 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00056109513,  -0.00048872941,    0.0010445699,    0.0084887435 ],
  [               0,               0,       5.6040709,      -11.012848,      -25.386313,       52.078245 ],
  [               0,               0,         1.59216,      -3.4329549,      -6.8648442,        13.68161 ],
  [               0,               0,      -1.6733046,       3.8077092,       6.5113853,      -15.915212 ],
  [               0,               0,     -0.72442061,       1.3980773,       3.9245321,      -7.7076988 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0038113946,     0.012946699 ],
  [               0,               0,               0,               0,      -46.461038,       107.62754 ],
  [               0,               0,               0,               0,       37.431474,      -86.766623 ],
  [               0,               0,               0,               0,      -3.6951411,       8.5489112 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.015666544,     0.039304669 ],
  [               0,               0,               0,               0,        5.752124,      -10.965921 ],
  [               0,               0,               0,               0,       2.2241684,      -6.2313971 ],
  [               0,               0,               0,               0,       2.5568792,      -7.0640181 ],
  [               0,               0,               0,               0,     -0.69266393,       1.5839826 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.010196145,      0.04053848,    -0.033338032 ],
  [               0,               0,               0,      -30.997003,        119.8391,      -100.67236 ],
  [               0,               0,               0,      -76.313104,       344.23721,      -387.65852 ],
  [               0,               0,               0,      -7.1920618,       52.839252,      -85.790177 ],
  [               0,               0,               0,    0.0030896364,    0.0030047431,    -0.023801706 ],
  [               0,               0,               0,      -4.3276374,       23.332806,      -30.143613 ],
  [               0,               0,               0,      -15.960201,       53.514674,      -68.965119 ],
 ],
 [
  [               0,               0,               0,               0,    0.0084124449,    -0.014599909 ],
  [               0,               0,               0,               0,      -3663.8925,       9108.6516 ],
  [               0,               0,               0,               0,      -4.5657833,      -12.295078 ],
  [               0,               0,               0,               0,      -89.974064,       221.20263 ],
  [               0,               0,               0,               0,     0.038303471,     -0.25976287 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer1_etaoffv4_35_b = [
 [
  [               0,               0,       47744.669,      -360.44527,      0.17633589,   0.00096753831 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00073334877,   0.00063387956 ],
  [               0,               0,               0,               0,       3.9532162,       2.2713021 ],
  [               0,               0,               0,               0,      -2.1902402,     -0.27713021 ],
  [               0,               0,               0,               0,      -0.9060456,     -0.70865837 ],
  [               0,               0,               0,               0,      0.17992628,     -0.12850672 ],
  [               0,               0,               0,               0,  -0.00053275804,   0.00014711658 ],
  [               0,               0,               0,               0,       1.7171259,       1.3648506 ],
  [               0,               0,               0,               0,       1.4516782,      -1.5133207 ],
 ],
 [
  [               0,               0,               0,    0.0022198194,   -0.0030379459,    0.0020246883 ],
  [               0,               0,               0,       -6.930364,       9.1635251,     0.052845192 ],
  [               0,               0,               0,     -0.79223576,       2.0206981,      -1.8048168 ],
  [               0,               0,               0,      -1.2879695,       1.0075405,      -1.1813665 ],
  [               0,               0,               0,    -0.059955183,     -0.51470597,      0.20118687 ],
  [               0,               0,               0,  -0.00082125724,    0.0013444756,  -0.00050564327 ],
  [               0,               0,               0,       15.109848,      -10.970917,       3.0868955 ],
  [               0,               0,               0,       4.3745289,      -19.410772,       8.1149593 ],
 ],
 [
  [               0,               0,               0,               0,      -3.8112341,       3.0566406 ],
  [               0,               0,               0,               0,       24.369051,      -18.910914 ],
  [               0,               0,               0,               0,      -53.816915,       42.355199 ],
  [               0,               0,               0,               0,       29.834737,      -23.724162 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0014070847,   -0.0038871008,    0.0034326979 ],
  [               0,               0,               0,       12.774046,      -21.932015,       12.242415 ],
  [               0,               0,               0,      -3.7650435,       8.2082233,      -5.0628609 ],
  [               0,               0,               0,      -1.7701548,        3.279172,      -2.7182338 ],
  [               0,               0,               0,     0.014656475,     -0.38106705,      0.10185688 ],
  [               0,               0,               0,   0.00056531102,   -0.0010228279,   0.00049484764 ],
  [               0,               0,               0,      -6.1644967,       12.660581,      -3.2184975 ],
  [               0,               0,               0,       3.9711163,      -10.720319,       8.7870928 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.24974784,     -0.34228219,     -0.50123679,       0.6936513 ],
  [               0,               0,       1556.7953,      -2245.7685,      -3109.6897,       4490.8269 ],
  [               0,               0,       795.73434,      -1076.0589,      -2028.1261,       2772.8219 ],
  [               0,               0,       461.21692,      -613.95105,      -1189.0296,       1603.4934 ],
  [               0,               0,       2418.8544,      -3443.5384,      -4902.2637,        6977.199 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer1_etaoffv4_35_e = [
 [
  [               0,               0,               0,               0,      -1.4337416,       2.1490954 ],
  [               0,               0,               0,               0,       18.304158,      -27.000824 ],
  [               0,               0,               0,               0,      -17.846268,       26.232562 ],
  [               0,               0,               0,               0,        2.875826,      -4.1303291 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0059827319 ],
  [               0,               0,               0,               0,               0,        3.139492 ],
  [               0,               0,               0,               0,               0,      -1.4695341 ],
  [               0,               0,               0,               0,               0,      -1.1953401 ],
  [               0,               0,               0,               0,               0,   -0.0057060043 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.34816457,       0.5267308 ],
  [               0,               0,               0,               0,      -174.88554,       264.36182 ],
  [               0,               0,               0,               0,       84.544263,      -127.06939 ],
  [               0,               0,               0,               0,      -49.448055,       73.734994 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,       1546.9136 ],
  [               0,               0,               0,               0,               0,    -0.013029269 ],
  [               0,               0,               0,               0,               0,     0.013031681 ],
  [               0,               0,               0,               0,               0,     0.013029008 ],
  [               0,               0,               0,               0,               0,   1.7471292e-06 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,       7.1075435,      -12.209789,      -21.751482,       39.563899 ],
  [               0,               0,      -13.135229,       67.801657,      -116.56709,       66.868984 ],
  [               0,               0,       12.986253,      -67.054498,         115.319,      -66.173836 ],
  [               0,               0,       12.934571,      -66.804341,       114.91688,      -65.959177 ],
  [               0,               0,   -0.0050431933,     0.022567488,    -0.033159104,     0.015936239 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0030224364,    0.0093312839 ],
  [               0,               0,               0,               0,      0.60252237,      0.62354995 ],
  [               0,               0,               0,               0,     0.017060488,      -1.1218908 ],
  [               0,               0,               0,               0,       0.1669188,      -1.3367469 ],
  [               0,               0,               0,               0,     0.042746405,    -0.089953942 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00050188586,  -0.00030026015,    0.0010556978,    0.0065972612 ],
  [               0,               0,       2.4964248,       -4.394529,      -11.684667,       23.365844 ],
  [               0,               0,       1.3868374,      -2.9000469,      -5.7573862,       10.891143 ],
  [               0,               0,       3.5910588,      -7.0917382,      -17.602366,       33.798847 ],
  [               0,               0,       0.1049402,     -0.31511297,     0.039852106,      0.32366454 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.34846479,      0.81077088 ],
  [               0,               0,               0,               0,      -28.938197,       67.672243 ],
  [               0,               0,               0,               0,       8.0387425,      -19.476366 ],
  [               0,               0,               0,               0,       6.5405062,      -14.891375 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00061513982,    0.0024340283 ],
  [               0,               0,               0,               0,      -4.6072306,        12.92486 ],
  [               0,               0,               0,               0,       1.1639977,      -3.9100431 ],
  [               0,               0,               0,               0,       1.7463622,      -5.0807962 ],
  [               0,               0,               0,               0,      0.60186003,      -1.3914726 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0062979366,    0.0029433139,    -0.035384776 ],
  [               0,               0,               0,      -19.505369,       61.804308,      -29.658128 ],
  [               0,               0,               0,      -53.746944,       262.06683,      -320.83879 ],
  [               0,               0,               0,      -6.7738181,       35.363881,      -45.943015 ],
  [               0,               0,               0,   0.00024359106,   -0.0030371393,    0.0065930063 ],
  [               0,               0,               0,       -7.086793,       14.602301,       6.0192181 ],
  [               0,               0,               0,      -8.9049565,      -10.794454,       79.631517 ],
 ],
 [
  [               0,               0,               0,               0,    0.0044390144,    0.0080272192 ],
  [               0,               0,               0,               0,       7.1362497,       21.714377 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,               0,       120.35859,      -297.95393 ],
  [               0,               0,               0,               0,      0.34942019,       0.7316779 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer1_etaoffv4_35_b = [
 [
  [               0,               0,       41307.126,      -298.20018,      0.13287768,   0.00047375828 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0014584926,   0.00049756309 ],
  [               0,               0,               0,               0,      -3.1423509,       4.2027364 ],
  [               0,               0,               0,               0,     -0.06064257,     -0.91353652 ],
  [               0,               0,               0,               0,       1.9432638,      -1.5810226 ],
  [               0,               0,               0,               0,      0.91039387,     -0.33166286 ],
  [               0,               0,               0,               0,   -0.0010780366,   0.00029908734 ],
  [               0,               0,               0,               0,      0.38043435,      0.57656592 ],
  [               0,               0,               0,               0,       1.2505773,      -1.1869871 ],
 ],
 [
  [               0,               0,               0,    0.0011752571,  -0.00034068944,   0.00085696158 ],
  [               0,               0,               0,      -4.1819583,       1.6330154,       3.6633418 ],
  [               0,               0,               0,       16.225422,      -18.607133,        4.409869 ],
  [               0,               0,               0,      0.97576458,     -0.68811917,      -1.0772191 ],
  [               0,               0,               0,      -18.745983,       18.577387,      -4.8699819 ],
  [               0,               0,               0,     0.023246665,    -0.022659191,    0.0056686588 ],
  [               0,               0,               0,       10.566172,      -13.629331,       4.4495219 ],
  [               0,               0,               0,      -8.5495894,       9.6863072,      -2.1124351 ],
 ],
 [
  [               0,               0,               0,               0,     -0.12718407,      0.13162406 ],
  [               0,               0,               0,               0,      -1.7906056,       1.9214087 ],
  [               0,               0,               0,               0,      -13.616335,       10.367123 ],
  [               0,               0,               0,               0,       14.362069,      -11.422243 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   0.00024112842,   2.6125923e-05,    0.0018769564 ],
  [               0,               0,               0,       1.9042613,      -3.1574421,       3.4073936 ],
  [               0,               0,               0,      -1.6711477,       3.1886518,      -2.5368112 ],
  [               0,               0,               0,    -0.041967042,     -0.42085841,     -0.82368547 ],
  [               0,               0,               0,      0.12626525,     -0.28037325,     0.040045164 ],
  [               0,               0,               0,   0.00060651621,   -0.0010958644,   0.00052124623 ],
  [               0,               0,               0,       3.1636972,      -7.8113385,       7.0708453 ],
  [               0,               0,               0,       10.305015,      -24.138337,       15.539542 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.12835916,     0.078689618,    -0.032163153,     -0.24258545 ],
  [               0,               0,       97.596405,       45.354255,      -63.737574,      -259.13923 ],
  [               0,               0,       7.6952456,       2.9607612,      -5.5447527,      -19.974462 ],
  [               0,               0,               0,               0,               0,              -1 ],
  [               0,               0,     -0.48168483,     -0.27228646,      0.22082116,      0.89957843 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.057728413,     -0.13724083,     -0.15754244,      0.34556266 ],
  [               0,               0,       426.34601,      -612.75876,      -810.50491,       1169.5112 ],
  [               0,               0,      -278.91654,       487.79831,       153.72372,      -402.36546 ],
  [               0,               0,       359.63361,      -457.08004,      -1035.0271,       1359.6011 ],
  [               0,               0,       2378.4115,      -3387.2511,      -4833.5965,        6882.272 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer1_etaoffv4_35_e = [
 [
  [               0,               0,               0,               0,    0.0050556853,   -0.0062010979 ],
  [               0,               0,               0,               0,      -17.966578,      -13.762012 ],
  [               0,               0,               0,               0,       -274.7974,       401.31532 ],
  [               0,               0,               0,               0,      -146.20259,       208.44827 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,      0.57312472 ],
  [               0,               0,               0,               0,               0,      0.39184122 ],
  [               0,               0,               0,               0,               0,     -0.37561211 ],
  [               0,               0,               0,               0,               0,     -0.37518556 ],
  [               0,               0,               0,               0,               0,   -0.0010264174 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.31550813,      -0.4653708 ],
  [               0,               0,               0,               0,       481.21127,      -711.28817 ],
  [               0,               0,               0,               0,       27.490816,      -43.848473 ],
  [               0,               0,               0,               0,       73.315233,       -108.6722 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   0.00035434882,  -0.00017262385,  -0.00037960371,    0.0019569092 ],
  [               0,               0,       7.5429616,      -11.545692,      -24.948815,       40.389535 ],
  [               0,               0,      -6.2411371,       10.784896,       15.803123,      -28.302451 ],
  [               0,               0,      -2.5604728,       4.2144455,       7.0394293,      -12.501728 ],
  [               0,               0,    -0.043209128,      0.21677513,      -0.4349147,      0.32157977 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0036804992,    0.0066588553 ],
  [               0,               0,               0,               0,     -0.34900696,        1.638016 ],
  [               0,               0,               0,               0,      0.21853872,      -1.2080464 ],
  [               0,               0,               0,               0,      0.29401162,      -1.3552925 ],
  [               0,               0,               0,               0,    -0.015427988,     0.025617222 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   0.00017435457,   0.00025109242,   0.00016926362,  -0.00015550551 ],
  [               0,               0,     -0.27105007,   -0.0021668497,      0.62941382,       3.8366842 ],
  [               0,               0,      0.91784774,      -1.9172834,      -3.8837122,       6.9362834 ],
  [               0,               0,      0.27333796,      -0.5194909,      -1.1207331,      0.99542454 ],
  [               0,               0,     -0.21438105,      0.41700349,       1.1741902,      -2.3056517 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0011523479,    0.0024803277 ],
  [               0,               0,               0,               0,       -109.5262,       253.53834 ],
  [               0,               0,               0,               0,       40.053802,      -93.268069 ],
  [               0,               0,               0,               0,       6.0193247,       -13.58135 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00097491986,    0.0026598476 ],
  [               0,               0,               0,               0,      0.33060248,       1.2456758 ],
  [               0,               0,               0,               0,     0.058532265,      -1.3371299 ],
  [               0,               0,               0,               0,    -0.088199393,     -0.91455633 ],
  [               0,               0,               0,               0,     0.035160134,    -0.065378451 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.005321371,     0.023366124,    -0.018982275 ],
  [               0,               0,               0,      -41.380938,       177.64158,      -180.23981 ],
  [               0,               0,               0,      -128.76972,       609.62678,      -722.80633 ],
  [               0,               0,               0,      -24.585198,       131.57392,      -174.34318 ],
  [               0,               0,               0,     0.007164493,   -0.0033855539,     -0.03279047 ],
  [               0,               0,               0,       -22.07011,       105.89226,      -125.94314 ],
  [               0,               0,               0,      -6.5209895,       8.7541556,       15.230433 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0021707554,     0.014252734 ],
  [               0,               0,               0,               0,      -3434.4611,       8541.9761 ],
  [               0,               0,               0,               0,      -19.290575,      -48.618397 ],
  [               0,               0,               0,               0,      -9.5917872,       22.787675 ],
  [               0,               0,               0,               0,      0.16796005,      0.10070097 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer1_etaoffv4_35_b = [
 [
  [               0,               0,       55391.294,      -358.54096,    -0.054956486,    0.0012889158 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00090833659,   0.00052339281 ],
  [               0,               0,               0,               0,      -2.1747335,       4.3864506 ],
  [               0,               0,               0,               0,      -2.4308367,     -0.32569964 ],
  [               0,               0,               0,               0,     -0.46771003,     -0.76823425 ],
  [               0,               0,               0,               0,       0.2439773,     -0.11339569 ],
  [               0,               0,               0,               0,  -0.00037692589,   6.6920009e-05 ],
  [               0,               0,               0,               0,      -2.1459924,       1.9811834 ],
  [               0,               0,               0,               0,       4.5605019,     -0.99226907 ],
 ],
 [
  [               0,               0,               0,  -0.00053814442,    0.0029169426,  -0.00036042551 ],
  [               0,               0,               0,      -15.715837,       14.671036,      0.46677261 ],
  [               0,               0,               0,       8.5713108,      -10.593627,        2.027709 ],
  [               0,               0,               0,       8.6293649,      -11.194061,       2.2979567 ],
  [               0,               0,               0,     -0.26487615,      0.19956218,     -0.12152256 ],
  [               0,               0,               0,   -0.0021383511,    0.0025167981,  -0.00065707411 ],
  [               0,               0,               0,       6.5491831,       -6.794773,       3.7128403 ],
  [               0,               0,               0,       15.954143,      -22.865208,       3.7150588 ],
 ],
 [
  [               0,               0,               0,               0,      0.02139811,     0.011273294 ],
  [               0,               0,               0,               0,       -25.69683,       20.999864 ],
  [               0,               0,               0,               0,      -1.2148574,      0.45272136 ],
  [               0,               0,               0,               0,       15.551206,      -12.366719 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.010152623,    -0.014250622,    0.0062111985 ],
  [               0,               0,               0,       0.5623783,      -3.3015367,       5.0921979 ],
  [               0,               0,               0,      0.88863056,      -2.0668061,     0.065171446 ],
  [               0,               0,               0,      0.59266845,      -1.3702965,     -0.49023402 ],
  [               0,               0,               0,     -0.32700885,      0.90398256,     -0.68230036 ],
  [               0,               0,               0,   0.00056295522,    -0.001432239,   0.00077922526 ],
  [               0,               0,               0,       4.1542268,      -8.6665214,       6.3186548 ],
  [               0,               0,               0,       4.8434421,       -13.51096,       14.009585 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0067121165,    -0.060221956,     -0.03599063,       0.2044543 ],
  [               0,               0,       1598.7097,      -2291.2459,       -3173.146,       4551.0604 ],
  [               0,               0,       96.694496,      -27.127117,      -723.61362,       806.64818 ],
  [               0,               0,        274.7808,      -319.97243,      -936.28492,       1185.9914 ],
  [               0,               0,       2811.3929,       -4002.667,      -5717.4741,       8138.4266 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer1_etaoffv4_35_e = [
 [
  [               0,               0,               0,               0,    -0.055773823,     0.077829967 ],
  [               0,               0,               0,               0,      -2.4465403,       43.420455 ],
  [               0,               0,               0,               0,       106.33563,      -155.69684 ],
  [               0,               0,               0,               0,       45.260654,      -65.085075 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0082557011,     0.012446267 ],
  [               0,               0,               0,               0,      -1511.0781,       2236.9268 ],
  [               0,               0,               0,               0,      -3646.9366,       5376.8311 ],
  [               0,               0,               0,               0,       3085.7496,      -4483.7057 ],
  [               0,               0,               0,               0,       0.5340102,      -2.2400921 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.047014795,     0.070929186 ],
  [               0,               0,               0,               0,       1438.8875,      -2118.3654 ],
  [               0,               0,               0,               0,      -806.38082,       1198.8529 ],
  [               0,               0,               0,               0,       978.39808,      -1453.8191 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0029239994,  -0.00037398873,     0.008865241,     0.034373148 ],
  [               0,               0,      -2.8253015,       14.174454,       -23.70098,       13.858769 ],
  [               0,               0,      -2.0527442,       10.059825,      -16.353288,       8.2252062 ],
  [               0,               0,      0.28302181,      -1.0144413,       1.0475369,     -0.83398142 ],
  [               0,               0,     -0.25614047,       1.3688578,      -2.4330398,       1.4371912 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0034480052,    0.0078336746 ],
  [               0,               0,               0,               0,       7.4328569,      -10.756734 ],
  [               0,               0,               0,               0,       1.4296653,      -3.6193825 ],
  [               0,               0,               0,               0,      0.36591063,      -1.6284961 ],
  [               0,               0,               0,               0,   -0.0003077709,   -0.0073364684 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0055607694,   -0.0092179753,    -0.024543974,     0.045084845 ],
  [               0,               0,      -17.861241,       122.10927,      -279.82253,       216.53961 ],
  [               0,               0,      -2.8177243,       14.709521,      -23.703043,       9.9127674 ],
  [               0,               0,      -23.003433,       144.89037,      -303.21883,       209.71249 ],
  [               0,               0,        5.320079,      -34.270783,       73.499968,      -52.481974 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00015192817,      0.00389615 ],
  [               0,               0,               0,               0,      -204.98828,       473.40801 ],
  [               0,               0,               0,               0,       69.873227,      -161.96911 ],
  [               0,               0,               0,               0,     -0.15416827,      0.70381093 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0005758879,    0.0012815507 ],
  [               0,               0,               0,               0,       5.5819575,      -10.159444 ],
  [               0,               0,               0,               0,      -3.4821686,       6.9964781 ],
  [               0,               0,               0,               0,      -2.2355735,       4.1089729 ],
  [               0,               0,               0,               0,     -0.25506399,      0.62720311 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0087996494 ],
  [               0,               0,               0,               0,               0,        5.514198 ],
  [               0,               0,               0,               0,               0,      -1.4535947 ],
  [               0,               0,               0,               0,               0,     0.049099892 ],
  [               0,               0,               0,               0,               0,  -0.00022731861 ],
  [               0,               0,               0,               0,               0,       3.0056238 ],
  [               0,               0,               0,               0,               0,      -3.2863879 ],
 ],
 [
  [               0,               0,   0.00048229416,   0.00074314052,   0.00071278491,  -0.00031608266 ],
  [               0,               0,      -159.79616,      -131.25717,       330.64688,       2448.4856 ],
  [               0,               0,     -0.23740085,      -1.1687322,      -2.9667247,      -7.9892868 ],
  [               0,               0,      -4.8679989,      -1.3753932,       13.225781,       48.468944 ],
  [               0,               0,     0.012577039,     0.031803227,      0.07178883,     -0.14912447 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer1_etaoffv4_35_b = [
 [
  [               0,               0,       48616.961,      -302.79842,     -0.19692046,    0.0013121264 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00040319492,     0.001051446 ],
  [               0,               0,               0,               0,      -1.0555275,       3.6970718 ],
  [               0,               0,               0,               0,     -0.85835889,     -0.66395112 ],
  [               0,               0,               0,               0,      0.38354394,      -1.3890293 ],
  [               0,               0,               0,               0,      0.16751602,     -0.17066955 ],
  [               0,               0,               0,               0,  -0.00032104642,   0.00018499371 ],
  [               0,               0,               0,               0,     -0.27875852,       1.1866487 ],
  [               0,               0,               0,               0,       1.3547948,      0.67702925 ],
 ],
 [
  [               0,               0,               0,    0.0023214589,   -0.0015631141,    0.0010268678 ],
  [               0,               0,               0,      -13.944623,       17.575719,      -2.1596868 ],
  [               0,               0,               0,       1.6283381,      -3.0741822,      0.41060735 ],
  [               0,               0,               0,       2.5345904,      -4.7146344,      0.86710188 ],
  [               0,               0,               0,     -0.12609917,    -0.061534689,    -0.061848925 ],
  [               0,               0,               0,  -0.00043323003,   0.00073914441,  -0.00033527322 ],
  [               0,               0,               0,       -1.684385,       2.8494737,       1.0161924 ],
  [               0,               0,               0,      -8.6518031,       3.7591214,      -4.9406092 ],
 ],
 [
  [               0,               0,               0,               0,     0.018040321,    0.0047996766 ],
  [               0,               0,               0,               0,       3.6505078,      -2.2830824 ],
  [               0,               0,               0,               0,       -31.51808,       24.509561 ],
  [               0,               0,               0,               0,       24.209189,      -19.254055 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0013397077,   0.00070978505,   0.00084157345 ],
  [               0,               0,               0,      0.16433952,      -1.4237752,        3.097812 ],
  [               0,               0,               0,      0.19509736,     -0.25301837,     -0.91361369 ],
  [               0,               0,               0,      0.68028587,      -1.2336531,     -0.65909366 ],
  [               0,               0,               0,    -0.094527577,      0.27811074,     -0.28020268 ],
  [               0,               0,               0,   2.1777391e-05,  -2.6440581e-06,  -2.6795907e-05 ],
  [               0,               0,               0,       4.7117064,       1.4447784,       1.7830478 ],
  [               0,               0,               0,       -1.054129,      0.20426353,        3.121566 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.064042917,     -0.25672115,     -0.24989621,      0.70626774 ],
  [               0,               0,      -1219.7955,       1735.7866,       2552.7743,      -3628.2672 ],
  [               0,               0,      -390.84874,       715.53426,       235.71537,      -657.43224 ],
  [               0,               0,       6.3166445,       109.12936,      -608.35646,       623.94874 ],
  [               0,               0,       3110.1002,       -4436.744,      -6315.7826,       9007.9494 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer1_etaoffv4_35_e = [
 [
  [               0,               0,               0,               0,     -0.39428286,      0.56890471 ],
  [               0,               0,               0,               0,       54.925678,      -80.159262 ],
  [               0,               0,               0,               0,      -132.49135,       189.94193 ],
  [               0,               0,               0,               0,       77.623283,      -110.06877 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.036985416,      0.05481384 ],
  [               0,               0,               0,               0,       1861.8813,      -2705.6406 ],
  [               0,               0,               0,               0,      -1711.1272,       2524.8785 ],
  [               0,               0,               0,               0,       930.94443,      -1349.4011 ],
  [               0,               0,               0,               0,       0.5983869,     -0.83526909 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.21838476,     -0.30546561 ],
  [               0,               0,               0,               0,      -129.56512,       195.53149 ],
  [               0,               0,               0,               0,       68.074046,      -102.46009 ],
  [               0,               0,               0,               0,      -43.651729,       65.036228 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0024402826,    0.0026397335,    0.0042436581,  -0.00085166874 ],
  [               0,               0,      -19.728456,       109.71922,      -196.25254,        115.7342 ],
  [               0,               0,      -27.891428,       141.66858,      -239.81085,       134.21068 ],
  [               0,               0,       23.023136,      -103.98697,       154.66052,      -76.535515 ],
  [               0,               0,      -9.6399565,       47.363144,      -77.520801,       42.289212 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00018438512,     0.001185947 ],
  [               0,               0,               0,               0,      -1.2813298,       5.5881101 ],
  [               0,               0,               0,               0,     -0.91083833,      0.72478265 ],
  [               0,               0,               0,               0,    0.0030438627,      -1.0490242 ],
  [               0,               0,               0,               0,      0.16300814,      -0.3210172 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.011704335,    -0.028738819,     -0.05645819,      0.14108573 ],
  [               0,               0,        4.922742,      -16.643325,       10.758854,       7.5100268 ],
  [               0,               0,       25.243078,      -152.51944,       306.19522,      -205.39017 ],
  [               0,               0,      -18.568821,       124.77424,      -277.48583,        203.2928 ],
  [               0,               0,       10.170054,      -66.292223,       144.01272,      -104.24073 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.016497908 ],
  [               0,               0,               0,               0,               0,      0.89331532 ],
  [               0,               0,               0,               0,               0,     -0.82111159 ],
  [               0,               0,               0,               0,               0,     0.094058595 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00056740813,    0.0018536202 ],
  [               0,               0,               0,               0,      -1.8568681,       7.0382583 ],
  [               0,               0,               0,               0,      0.11496428,      -1.4778922 ],
  [               0,               0,               0,               0,       1.9475954,      -5.6672741 ],
  [               0,               0,               0,               0,      0.16103944,     -0.34112051 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.031167205,     -0.06797971 ],
  [               0,               0,               0,               0,      -62.180791,       158.43006 ],
  [               0,               0,               0,               0,        2.037708,      -6.5621613 ],
  [               0,               0,               0,               0,     -0.22017078,      0.65307183 ],
  [               0,               0,               0,               0,    -0.015725388,     0.039028926 ],
  [               0,               0,               0,               0,       70.700349,      -170.41062 ],
  [               0,               0,               0,               0,       68.356241,      -167.64835 ],
 ],
 [
  [               0,               0,               0,               0,     0.024474572,     -0.03475453 ],
  [               0,               0,               0,               0,        -6.64892,       23.322529 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,               0,      -36.301794,       88.965102 ],
  [               0,               0,               0,               0,       5.1600102,      -12.249794 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer1_etaoffv4_35_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00047106208,    0.0016744229 ],
  [               0,               0,               0,               0,     -0.93214695,       3.5546794 ],
  [               0,               0,               0,               0,       1.2354946,      -1.5120264 ],
  [               0,               0,               0,               0,       -1.050481,      -1.2480707 ],
  [               0,               0,               0,               0,     -0.47393367,     0.035258377 ],
  [               0,               0,               0,               0,  -4.4989363e-05,  -0.00017869963 ],
  [               0,               0,               0,               0,     -0.32890023,       1.5619302 ],
  [               0,               0,               0,               0,      -2.5290838,      -1.2485177 ],
 ],
 [
  [               0,               0,               0,    0.0025079153,   -0.0028763096,    0.0016038706 ],
  [               0,               0,               0,      -14.358551,       14.389036,      0.76620416 ],
  [               0,               0,               0,       29.181768,      -29.519423,       3.4729437 ],
  [               0,               0,               0,       39.660035,      -42.780282,       6.4839436 ],
  [               0,               0,               0,      0.83708445,        1.337226,      -2.1700017 ],
  [               0,               0,               0,   -0.0048222515,   -0.0027873979,     0.006151804 ],
  [               0,               0,               0,    -0.057846882,     -0.63389536,     0.065159935 ],
  [               0,               0,               0,      -1.1982804,       1.4193207,       0.9195404 ],
 ],
 [
  [               0,               0,               0,               0,     0.022133672,     0.010298319 ],
  [               0,               0,               0,               0,        12.93069,      -9.7170249 ],
  [               0,               0,               0,               0,      -29.480014,       22.928039 ],
  [               0,               0,               0,               0,       16.529459,      -13.147049 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0050206602,     0.012243053,   -0.0054126754 ],
  [               0,               0,               0,       5.1253514,       -11.71451,       9.0882982 ],
  [               0,               0,               0,     -0.43588385,      0.51836583,      -1.0234218 ],
  [               0,               0,               0,       1.0316102,      -2.3143703,    -0.074777347 ],
  [               0,               0,               0,     -0.24400681,      0.63639049,     -0.55757916 ],
  [               0,               0,               0,    0.0008835689,   -0.0018354884,   0.00097292787 ],
  [               0,               0,               0,      -4.4194079,       8.5782268,      -1.6252531 ],
  [               0,               0,               0,       3.5257018,       -10.86994,       4.1665391 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.044519573,     -0.16023571,     -0.16058824,      0.43821392 ],
  [               0,               0,      -226.87428,       318.11837,       535.29105,      -748.07612 ],
  [               0,               0,       -601.8609,       986.00822,       690.88043,      -1244.7479 ],
  [               0,               0,      -40.744367,        135.9344,      -369.58503,       365.41565 ],
  [               0,               0,       3005.2246,      -4281.4676,      -6112.7485,        8706.792 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer1_etaoffv4_35_e = [
 [
  [               0,               0,               0,               0,    -0.033611743,     0.048538831 ],
  [               0,               0,               0,               0,      -16.253926,      -16.346651 ],
  [               0,               0,               0,               0,      -23.905257,       26.978339 ],
  [               0,               0,               0,               0,       20.578771,      -17.622565 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.023450455,    -0.033135416 ],
  [               0,               0,               0,               0,      -1581.2721,       2339.5851 ],
  [               0,               0,               0,               0,      -748.33953,       1100.9378 ],
  [               0,               0,               0,               0,       693.03187,       -1003.787 ],
  [               0,               0,               0,               0,      0.40492559,     0.044772369 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.42902283,      0.64894846 ],
  [               0,               0,               0,               0,      -39.463637,       62.752024 ],
  [               0,               0,               0,               0,       23.800438,       -36.73971 ],
  [               0,               0,               0,               0,       -11.05193,       16.618178 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.013457402,    -0.018318716,    -0.035995953,     0.050769495 ],
  [               0,               0,      -14.851999,       66.606741,      -103.24214,       57.958953 ],
  [               0,               0,       3.5435593,       4.6638531,      -45.786222,       45.763959 ],
  [               0,               0,      0.14249347,        13.20906,      -47.063218,       40.122965 ],
  [               0,               0,      -1.0487815,       3.7493843,      -3.9998359,        1.150044 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0047004249,   -0.0066442281 ],
  [               0,               0,               0,               0,      -4.9627724,        12.13199 ],
  [               0,               0,               0,               0,      0.72955435,      -2.5937255 ],
  [               0,               0,               0,               0,      0.32745628,      -1.7776031 ],
  [               0,               0,               0,               0,     -0.16525395,      0.32075746 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00048755304,  -0.00026928647,    0.0011379653,     0.006724676 ],
  [               0,               0,      0.52859898,     -0.42999341,      -2.1115919,       4.1575637 ],
  [               0,               0,       1.9696332,       -4.304626,      -8.4812181,        17.30011 ],
  [               0,               0,       1.2231656,      -2.2567081,      -6.5253584,       11.166346 ],
  [               0,               0,     -0.59367365,       1.1841362,        3.144722,      -6.2915767 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0036629522 ],
  [               0,               0,               0,               0,               0,       2.1368774 ],
  [               0,               0,               0,               0,               0,      -1.6059158 ],
  [               0,               0,               0,               0,               0,      0.47477029 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00044867429,    0.0019251693 ],
  [               0,               0,               0,               0,      -5.2045716,       15.202744 ],
  [               0,               0,               0,               0,       1.9914075,      -5.9102444 ],
  [               0,               0,               0,               0,       2.0422137,      -5.9017605 ],
  [               0,               0,               0,               0,      0.45969545,      -1.0376599 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00076242793,     0.025821253 ],
  [               0,               0,               0,               0,      -26.769919,       66.042579 ],
  [               0,               0,               0,               0,       273.18559,      -656.66376 ],
  [               0,               0,               0,               0,      -251.43558,       600.72621 ],
  [               0,               0,               0,               0,     0.027467645,      -0.1230591 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0048147802,     0.057097878,     -0.10160256 ],
  [               0,               0,               0,      -49.942069,        186.7424,      -151.72507 ],
  [               0,               0,               0,        170.9219,      -835.26582,       1018.7715 ],
  [               0,               0,               0,      -127.46126,       624.62002,      -765.06699 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,        2.185562,      -5.4107969 ],
  [               0,               0,               0,               0,      -3391.0211,       8433.4448 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,               0,      -379.87773,       940.01342 ],
  [               0,               0,               0,               0,       23.242997,      -57.820724 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer1_etaoffv4_35_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00030230434,   0.00072815116 ],
  [               0,               0,               0,               0,      -3.8734222,       5.8300124 ],
  [               0,               0,               0,               0,      0.39820534,       -1.121706 ],
  [               0,               0,               0,               0,      0.11146598,       -1.186532 ],
  [               0,               0,               0,               0,     -0.12412319,    -0.057664851 ],
  [               0,               0,               0,               0,   0.00023846979,   -4.529268e-05 ],
  [               0,               0,               0,               0,       7.3526014,     -0.30779819 ],
  [               0,               0,               0,               0,     -0.10410716,      -2.3030715 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0014690628,  -0.00039486268 ],
  [               0,               0,               0,               0,      0.63278819,      -3.1660138 ],
  [               0,               0,               0,               0,      0.47908203,      0.60199209 ],
  [               0,               0,               0,               0,      0.99765516,       0.5225431 ],
  [               0,               0,               0,               0,      0.17637901,     0.025398185 ],
  [               0,               0,               0,               0,  -0.00019699146,   0.00018827212 ],
  [               0,               0,               0,               0,       0.8961857,       1.9019248 ],
  [               0,               0,               0,               0,      -1.3055129,      -1.2919235 ],
 ],
 [
  [               0,               0,               0,               0,     0.022590919,    0.0074366008 ],
  [               0,               0,               0,               0,       35.948096,      -28.039792 ],
  [               0,               0,               0,               0,      -46.677158,       36.619121 ],
  [               0,               0,               0,               0,       16.594009,      -13.199805 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0008587593,   0.00073269551,  -5.0705777e-05 ],
  [               0,               0,               0,       2.8114247,      -8.6010506,       8.5927676 ],
  [               0,               0,               0,      0.19685833,     -0.58641537,     -0.51840728 ],
  [               0,               0,               0,      0.46580029,      -1.0113839,     -0.83497566 ],
  [               0,               0,               0,      -0.1667526,      0.50763082,     -0.53309498 ],
  [               0,               0,               0,     0.000212543,  -0.00052439534,   0.00034768011 ],
  [               0,               0,               0,      0.12017268,     -0.27316748,         2.68896 ],
  [               0,               0,               0,     -0.26563084,     -0.84132012,      -1.9006067 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,      0.04857836,     -0.10322644,     -0.19968823,    -0.072512014,      0.62454927 ],
  [               0,      -872.54374,       602.81772,       1793.1527,       1329.8601,      -3678.0797 ],
  [               0,      -625.71137,       573.24872,       1123.1262,       431.60378,      -1973.5489 ],
  [               0,       61.545335,       93.446361,      -328.29829,      -641.88408,        1053.801 ],
  [               0,       1551.4543,      -1103.3056,      -3156.5159,      -2255.7752,       6417.2015 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer1_etaoffv4_35_e = [
 [
  [               0,               0,               0,     0.028880141,  -0.00013520751,    -0.058106945 ],
  [               0,               0,               0,       52.302397,       -9.773091,      -132.29431 ],
  [               0,               0,               0,      -70.319486,        2.055071,       146.72411 ],
  [               0,               0,               0,      -33.345304,      0.80139311,       68.488558 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.44655418,      0.66334511 ],
  [               0,               0,               0,               0,      -358.21598,       528.10749 ],
  [               0,               0,               0,               0,       90.917513,      -134.00128 ],
  [               0,               0,               0,               0,      -172.60585,       249.41195 ],
  [               0,               0,               0,               0,      0.11894164,     -0.13705291 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.18045922,      0.27410203 ],
  [               0,               0,               0,               0,      -281.06094,       424.82379 ],
  [               0,               0,               0,               0,      -42.346258,       61.461967 ],
  [               0,               0,               0,               0,       66.873644,      -99.103189 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0010123612,  -0.00050639769,    0.0013007182,    0.0066480424 ],
  [               0,               0,       1.4164548,       0.3094563,      -9.8751666,       11.026705 ],
  [               0,               0,       1.9285814,      -1.8429294,      -9.4490758,       10.765512 ],
  [               0,               0,     -0.11633136,      0.40559465,     -0.59516714,     -0.66029384 ],
  [               0,               0,     -0.30926374,      0.61461252,      0.36434424,     -0.87263712 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0051439616,   -0.0074890022 ],
  [               0,               0,               0,               0,      -4.5160007,       11.500199 ],
  [               0,               0,               0,               0,     -0.24681167,     -0.83161958 ],
  [               0,               0,               0,               0,     0.046206454,      -1.2402158 ],
  [               0,               0,               0,               0,     -0.18129829,      0.36393338 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0001480603,    0.0002911792,   0.00030262069,  -0.00014465388 ],
  [               0,               0,      -1.1015782,       1.9335637,       4.7337345,      -5.3271531 ],
  [               0,               0,      0.62734566,      -1.4125904,      -2.5289475,       4.4624921 ],
  [               0,               0,      0.74881733,      -1.4845086,      -3.8213448,       6.4628403 ],
  [               0,               0,    -0.028653264,    -0.023572478,      0.48925773,     -0.62991809 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0010595157,   0.00099488986 ],
  [               0,               0,               0,               0,      -204.08386,       471.63812 ],
  [               0,               0,               0,               0,       102.87574,      -238.05849 ],
  [               0,               0,               0,               0,      0.68015488,      -1.0522913 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0041290225,     0.013280814 ],
  [               0,               0,               0,               0,      -1.3124372,       5.9257853 ],
  [               0,               0,               0,               0,      0.37953142,      -2.2116188 ],
  [               0,               0,               0,               0,       1.6537022,      -5.0482043 ],
  [               0,               0,               0,               0,      0.47737763,      -1.0733627 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0082061038,     0.020087501 ],
  [               0,               0,               0,               0,       6.1175843,       25.191789 ],
  [               0,               0,               0,               0,       305.98849,      -735.76041 ],
  [               0,               0,               0,               0,       662.66368,      -1590.1766 ],
  [               0,               0,               0,               0,     -0.49071718,      -1.2556929 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0057513944,    0.0023154854,     0.036407507 ],
  [               0,               0,               0,      -5.5309944,       6.2502179,       25.609276 ],
  [               0,               0,               0,      -5.9862429,       21.197731,      -17.838798 ],
  [               0,               0,               0,      0.16081919,       5.2334594,      -13.566336 ],
  [               0,               0,               0,    0.0020988754,   -0.0003086499,    -0.010977594 ],
  [               0,               0,               0,       12.943778,      -1.3626909,      -75.260556 ],
  [               0,               0,               0,       24.175219,      -1.0886161,      -146.14434 ],
 ],
 [
  [               0,               0,               0,               0,       1.1742213,       -2.904434 ],
  [               0,               0,               0,               0,      -3850.2858,       9576.6918 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,               0,       -126.4164,       312.42586 ],
  [               0,               0,               0,               0,       1.0599719,      -2.2315951 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]


gam_layer1_etaoffv4_35_b = [
    gam5_layer1_etaoffv4_35_b,
    gam10_layer1_etaoffv4_35_b,
    gam25_layer1_etaoffv4_35_b,
    gam50_layer1_etaoffv4_35_b,
    gam75_layer1_etaoffv4_35_b,
    gam100_layer1_etaoffv4_35_b,
    gam200_layer1_etaoffv4_35_b,
    gam500_layer1_etaoffv4_35_b,
    gam1000_layer1_etaoffv4_35_b,
    ]

gam_layer1_etaoffv4_35_e = [
    gam5_layer1_etaoffv4_35_e,
    gam10_layer1_etaoffv4_35_e,
    gam25_layer1_etaoffv4_35_e,
    gam50_layer1_etaoffv4_35_e,
    gam75_layer1_etaoffv4_35_e,
    gam100_layer1_etaoffv4_35_e,
    gam200_layer1_etaoffv4_35_e,
    gam500_layer1_etaoffv4_35_e,
    gam1000_layer1_etaoffv4_35_e,
    ]



###############################################################################
# Tables for v4 sshape correction, layer 1, 3x7 electron clusters
#

ele5_layer1_etaoffv4_37_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.003275674,    0.0076192958 ],
  [               0,               0,               0,               0,     0.011661608,      0.85914389 ],
  [               0,               0,               0,               0,      0.12522465,     -0.68496869 ],
  [               0,               0,               0,               0,     -0.28184804,     -0.64742752 ],
  [               0,               0,               0,               0,     -0.01756061,   -0.0052649761 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele5_layer1_etaoffv4_37_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer1_etaoffv4_37_b = [
 [
  [               0,               0,      -58031.736,       665.85347,      -2.6316053,    0.0034586861 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00068202406,   0.00050019009 ],
  [               0,               0,               0,               0,      0.79538897,       3.1483245 ],
  [               0,               0,               0,               0,      -651.13524,       212.87518 ],
  [               0,               0,               0,               0,      -640.38146,       210.09854 ],
  [               0,               0,               0,               0,        331.4421,      -1121.1552 ],
  [               0,               0,               0,               0,      -2.0691807,       1.5214725 ],
  [               0,               0,               0,               0,    -0.073087958,      0.02370492 ],
  [               0,               0,               0,               0,      0.11853324,       1.1933791 ],
 ],
 [
  [               0,               0,               0,    -0.057688479,     0.081405798,    -0.016398008 ],
  [               0,               0,               0,       1.6313911,      -2.9388843,       1.8931385 ],
  [               0,               0,               0,     -0.17567762,       1.0635245,      -1.1628999 ],
  [               0,               0,               0,     0.092788916,      0.67736106,      -1.0352424 ],
  [               0,               0,               0,   -0.0072214172,    0.0011322041,   -0.0072726994 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.13030975,      0.10511988 ],
  [               0,               0,               0,               0,       1540.1439,      -1224.4183 ],
  [               0,               0,               0,               0,      -656.61989,       522.01859 ],
  [               0,               0,               0,               0,       287.71388,      -228.69785 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      -49.299981,       33.625337,       35.859811 ],
  [               0,               0,               0,       0.4096796,     -0.56944619,      0.12350989 ],
  [               0,               0,               0,     -0.41004768,      0.57039874,     -0.12397144 ],
  [               0,               0,               0,     -0.41038335,      0.57121548,     -0.12460588 ],
  [               0,               0,               0,  -0.00037285275,   0.00075756394,  -0.00042358787 ],
  [               0,               0,               0,    0.0090855059,    -0.020025928,     0.011645674 ],
  [               0,               0,               0,      -3.3112735,       6.1946371,      -1.6121162 ],
  [               0,               0,               0,      -5.7445731,        9.227307,      -4.1831458 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,       406.49739,      -576.31647,      -799.34348,       1134.8201 ],
  [               0,               0,     -0.66075873,       3.7824114,      0.51192315,       -6.305742 ],
  [               0,               0,       10.617581,      -16.600964,       -24.80835,       38.148965 ],
  [               0,               0,      -1.9533586,       1.2245275,       1.4558078,      0.90237813 ],
  [               0,               0,       -3.699773,        5.273615,       7.6035934,      -10.838357 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer1_etaoffv4_37_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,       152240.57 ],
  [               0,               0,               0,               0,               0,    0.0045117602 ],
  [               0,               0,               0,               0,               0,   -0.0045117452 ],
  [               0,               0,               0,               0,               0,   -0.0045117311 ],
  [               0,               0,               0,               0,               0,   3.5805419e-09 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer1_etaoffv4_37_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.001322449,   0.00092811855 ],
  [               0,               0,               0,               0,       10.501879,       1.7239594 ],
  [               0,               0,               0,               0,     -0.40254929,     -0.52381169 ],
  [               0,               0,               0,               0,       1.6845739,     -0.39391989 ],
  [               0,               0,               0,               0,     -0.42264845,       0.1489575 ],
  [               0,               0,               0,               0,  -0.00060385261,   0.00029464241 ],
  [               0,               0,               0,               0,       6.4601019,     -0.22518066 ],
  [               0,               0,               0,               0,      0.94748167,        1.900031 ],
 ],
 [
  [               0,               0,               0,     0.032713134,    -0.011039273,      0.02253864 ],
  [               0,               0,               0,      -1.0942935,      0.77013475,      0.47784751 ],
  [               0,               0,               0,       1.1682575,     -0.92015432,     -0.35890785 ],
  [               0,               0,               0,       1.0908056,     -0.83316032,     -0.39073133 ],
  [               0,               0,               0,    -0.026529783,     0.022740782,   -0.0087974686 ],
  [               0,               0,               0,   -0.0010913027,     0.001655506,  -0.00058549951 ],
  [               0,               0,               0,       43.375317,       -54.59396,       21.056332 ],
  [               0,               0,               0,      -147.39508,       185.23795,      -54.761759 ],
 ],
 [
  [               0,               0,               0,               0,     0.069892863,    -0.039523839 ],
  [               0,               0,               0,               0,      -19.638955,       16.285951 ],
  [               0,               0,               0,               0,      -29.060712,          22.553 ],
  [               0,               0,               0,               0,       27.208503,       -21.64028 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      -2.6046412,     0.016042627,       5.6667337 ],
  [               0,               0,               0,      -1.0477454,       2.6297845,      -1.4869203 ],
  [               0,               0,               0,       1.0564339,      -2.6446105,       1.4933873 ],
  [               0,               0,               0,       1.0485613,      -2.6303138,       1.4872007 ],
  [               0,               0,               0,   -0.0025428687,     0.004785619,   -0.0022562437 ],
  [               0,               0,               0,    0.0012786936,   -0.0018448143,   0.00046449404 ],
  [               0,               0,               0,       5.4198466,      -5.0975068,      -1.7087753 ],
  [               0,               0,               0,      -1.3097056,        2.864711,       4.4723904 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0016815645 ],
  [               0,               0,               0,               0,               0,       4.4093638 ],
  [               0,               0,               0,               0,               0,      -2.0673423 ],
  [               0,               0,               0,               0,               0,     0.018192942 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0098958893,   -0.0045943985,    0.0064165702,     0.027311019 ],
  [               0,               0,      -2.7467972,      -3.6194166,      -8.0869818,      -11.518955 ],
  [               0,               0,      -3.2149909,      -7.8617339,      -15.640951,      -28.687554 ],
  [               0,               0,               0,               0,               0,              -1 ],
  [               0,               0,       4.7163025,        6.667852,       9.4488408,       13.119298 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.035371577,     0.029712831,    -0.013180095,     -0.13439028 ],
  [               0,               0,      -14.694883,       12.055709,       27.486126,       -17.67718 ],
  [               0,               0,       1096.3556,       -1579.679,        -2275.22,       3276.5169 ],
  [               0,               0,      -391.96329,       549.54388,       762.60418,      -1070.1072 ],
  [               0,               0,       129.53663,      -178.26127,      -257.62583,       353.75596 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer1_etaoffv4_37_e = [
 [
  [               0,               0,               0,               0,    -0.004008232,    0.0058288138 ],
  [               0,               0,               0,               0,        12.27305,       21.820002 ],
  [               0,               0,               0,               0,       3196.0487,      -4489.5427 ],
  [               0,               0,               0,               0,        916.2186,      -1304.8563 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,       52526.167 ],
  [               0,               0,               0,               0,               0,    0.0057286575 ],
  [               0,               0,               0,               0,               0,    -0.005728634 ],
  [               0,               0,               0,               0,               0,   -0.0057286842 ],
  [               0,               0,               0,               0,               0,   5.7797815e-09 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0044556007 ],
  [               0,               0,               0,               0,               0,       5.3554828 ],
  [               0,               0,               0,               0,               0,      -1.5401329 ],
  [               0,               0,               0,               0,               0,      0.18343489 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.010813544 ],
  [               0,               0,               0,               0,               0,      -1.6806503 ],
  [               0,               0,               0,               0,               0,       1.0939862 ],
  [               0,               0,               0,               0,               0,       1.1748165 ],
  [               0,               0,               0,               0,               0,      0.38554748 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.011513984,    -0.020183503,    -0.034887798,     0.061348731 ],
  [               0,               0,       192.43466,      -270.84042,      -626.04468,       904.60667 ],
  [               0,               0,      -72.700798,       143.95218,       134.13807,      -282.10979 ],
  [               0,               0,      -81.496117,       100.84639,       352.50375,      -484.37534 ],
  [               0,               0,      -6.7922329,       19.506788,      -10.080368,      -6.0160686 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00019552726,  -0.00025464991 ],
  [               0,               0,               0,               0,      -203.15164,       405.81471 ],
  [               0,               0,               0,               0,       -16.88266,       41.551818 ],
  [               0,               0,               0,               0,      -18.994857,       48.052185 ],
  [               0,               0,               0,               0,      -2.4622072,       3.6881044 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.016156026,    -0.038648624,     -0.07755055,      0.18724963 ],
  [               0,               0,      -22.318304,       157.94782,      -360.38735,       268.64408 ],
  [               0,               0,       56.980846,      -326.57629,       618.41487,      -387.02017 ],
  [               0,               0,      -29.750066,       207.94946,      -478.59681,       362.65546 ],
  [               0,               0,       27.536394,      -180.45592,        393.1379,      -284.81366 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       19237.153,       -44140.16 ],
  [               0,               0,               0,               0,      0.66673105,      -1.4964474 ],
  [               0,               0,               0,               0,     -0.66782595,       1.4989701 ],
  [               0,               0,               0,               0,   -0.0019307984,    0.0044551333 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0005051076 ],
  [               0,               0,               0,               0,               0,       5.1376486 ],
  [               0,               0,               0,               0,               0,       2.2732323 ],
  [               0,               0,               0,               0,               0,     0.019136179 ],
  [               0,               0,               0,               0,               0,     -0.19686334 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0053782293,    0.0036452201,     0.030916946 ],
  [               0,               0,               0,       27.177566,      -135.14304,       172.37711 ],
  [               0,               0,               0,       6.5328191,      -22.267204,       13.641043 ],
  [               0,               0,               0,       2.5743895,       34.533564,      -97.322613 ],
  [               0,               0,               0,     0.065025943,   -0.0081625603,     -0.35046637 ],
  [               0,               0,               0,       78.185917,      -389.73565,       485.63938 ],
  [               0,               0,               0,      -4.3805971,       26.749378,      -42.233025 ],
 ],
 [
  [               0,               0,               0,               0,    0.0033989506,   -0.0070954027 ],
  [               0,               0,               0,               0,       -4494.329,       11164.152 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,               0,       353.89024,       -878.1124 ],
  [               0,               0,               0,               0,     -0.30714334,      -0.3036536 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer1_etaoffv4_37_b = [
 [
  [               0,               0,       18069.771,      -21.663528,     -0.92853013,    0.0015669649 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00037698683,   0.00055671882 ],
  [               0,               0,               0,               0,      -5.7527754,       5.3928432 ],
  [               0,               0,               0,               0,       5.5352872,      -1.3810599 ],
  [               0,               0,               0,               0,      -1.2152994,      0.16441582 ],
  [               0,               0,               0,               0,      -1.2904798,      0.18652125 ],
  [               0,               0,               0,               0,  -0.00053557267,   0.00013531193 ],
  [               0,               0,               0,               0,      0.95292915,        1.524471 ],
  [               0,               0,               0,               0,      -1.7142551,       2.7357985 ],
 ],
 [
  [               0,               0,               0,     0.012639065,   -0.0078240133,    0.0012602849 ],
  [               0,               0,               0,      0.81299421,      -16.969516,       12.532874 ],
  [               0,               0,               0,       31.749141,       -38.29217,       10.742526 ],
  [               0,               0,               0,       42.152368,      -50.103181,       13.938215 ],
  [               0,               0,               0,      0.61212741,     -0.20008322,     -0.20984839 ],
  [               0,               0,               0,     0.010824694,    -0.012499954,    0.0035916425 ],
  [               0,               0,               0,      -11.854525,       14.322704,      -2.8350569 ],
  [               0,               0,               0,      -6.2225779,        6.662116,      -3.4368584 ],
 ],
 [
  [               0,               0,               0,               0,      0.10771548,    -0.061285153 ],
  [               0,               0,               0,               0,       26.024107,      -20.211107 ],
  [               0,               0,               0,               0,      -35.583781,       27.838712 ],
  [               0,               0,               0,               0,         18.6364,       -14.81947 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.020693074,     0.002238425,   -0.0084103913 ],
  [               0,               0,               0,       -0.530047,      0.44349396,      0.92367827 ],
  [               0,               0,               0,      0.79405951,      -1.2690628,     -0.22905596 ],
  [               0,               0,               0,      0.88798613,      -1.4609423,     -0.15061697 ],
  [               0,               0,               0,    -0.045139595,      0.12913906,    -0.099270211 ],
  [               0,               0,               0,   0.00011417956,   -0.0004454772,    0.0003598135 ],
  [               0,               0,               0,      -17.273961,        42.39603,      -22.862037 ],
  [               0,               0,               0,      -96.364344,       206.80998,      -101.53097 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.011460791,    -0.013161313,    0.0042180644,       0.0617588 ],
  [               0,               0,       1446.9805,      -2079.9496,       -2879.786,       4143.3253 ],
  [               0,               0,       127.83421,      -101.60414,      -661.41659,       778.95948 ],
  [               0,               0,       787.96387,      -1083.9502,      -1792.2651,       2472.4075 ],
  [               0,               0,       2022.1119,      -2879.2521,       -4101.254,       5838.3253 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer1_etaoffv4_37_e = [
 [
  [               0,               0,               0,               0,   -0.0096261762,     0.013903692 ],
  [               0,               0,               0,               0,       10.732189,       23.706077 ],
  [               0,               0,               0,               0,       1891.0731,      -2668.4271 ],
  [               0,               0,               0,               0,      -1199.6546,       1720.1989 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.022463037 ],
  [               0,               0,               0,               0,               0,       1.3912156 ],
  [               0,               0,               0,               0,               0,      -1.1608373 ],
  [               0,               0,               0,               0,               0,      -0.9676233 ],
  [               0,               0,               0,               0,               0,   -0.0075482643 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       1.1581741,       -1.708264 ],
  [               0,               0,               0,               0,       220.17547,      -324.10004 ],
  [               0,               0,               0,               0,       127.60568,      -191.52512 ],
  [               0,               0,               0,               0,      -96.008079,       142.84793 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0033965477,    -0.008999528,    0.0012242891,     0.066351182 ],
  [               0,               0,     -0.79420231,        19.91189,      -59.594477,       48.253684 ],
  [               0,               0,       4.2345179,      -35.227287,       82.101388,        -59.0831 ],
  [               0,               0,      -9.6499963,       33.276231,      -30.518091,       2.6117286 ],
  [               0,               0,      -1.6171883,       7.8414952,      -12.674555,       6.8284518 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00051263753,    0.0019061662 ],
  [               0,               0,               0,               0,    -0.004255191,        1.755768 ],
  [               0,               0,               0,               0,      -1.2541901,       1.3692732 ],
  [               0,               0,               0,               0,       0.5445952,      -1.9877797 ],
  [               0,               0,               0,               0,      0.41906226,     -0.81624828 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.025727498,    -0.061214486,     -0.12564326,      0.30170355 ],
  [               0,               0,       18.949357,      -108.11062,       209.81535,      -137.88307 ],
  [               0,               0,       42.324311,      -257.40713,        518.2476,       -346.0908 ],
  [               0,               0,      -17.096272,       115.25719,      -258.88034,       192.76231 ],
  [               0,               0,       13.566073,      -89.029214,       194.56629,      -141.60407 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.12633928,      0.29228418 ],
  [               0,               0,               0,               0,       231.56443,       -528.5521 ],
  [               0,               0,               0,               0,       13.348047,      -32.376604 ],
  [               0,               0,               0,               0,       62.608142,      -143.30627 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0085879778,     0.021504977 ],
  [               0,               0,               0,               0,       16.604707,      -35.528812 ],
  [               0,               0,               0,               0,       8.5667884,      -20.834596 ],
  [               0,               0,               0,               0,       8.3271719,      -20.133114 ],
  [               0,               0,               0,               0,       1.8085307,      -4.2149101 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.010070177,   -0.0020916311,     0.070457739 ],
  [               0,               0,               0,      -2.6874181,       11.002441,      -4.8402578 ],
  [               0,               0,               0,      -95.205563,       441.73397,      -513.21129 ],
  [               0,               0,               0,      -26.818672,       145.03202,      -193.78838 ],
  [               0,               0,               0,     0.008871727,   -0.0029365914,    -0.043850813 ],
  [               0,               0,               0,     -0.82370381,      -2.9688815,       13.294311 ],
  [               0,               0,               0,      0.62315654,      0.28392299,      -6.7810026 ],
 ],
 [
  [               0,               0,               0,    0.0010663075,    0.0016219122,    0.0019889581 ],
  [               0,               0,               0,       1.9234648,        4.776072,       15.859214 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,       51.145003,      0.31333423,      -314.08654 ],
  [               0,               0,               0,      0.11178103,      0.27709434,      0.56289179 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer1_etaoffv4_37_b = [
 [
  [               0,               0,       12360.185,      -40.024539,     -0.49226633,   0.00097110915 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0013368465,    0.0016426939 ],
  [               0,               0,               0,               0,     -0.32367577,       2.8927609 ],
  [               0,               0,               0,               0,        1.012705,     -0.99350372 ],
  [               0,               0,               0,               0,      0.23206928,     -0.87046512 ],
  [               0,               0,               0,               0,     -0.25907789,     0.018250419 ],
  [               0,               0,               0,               0,  -0.00049460394,   0.00037405772 ],
  [               0,               0,               0,               0,      -1.1751172,       1.1718404 ],
  [               0,               0,               0,               0,     -0.34539765,       1.6696379 ],
 ],
 [
  [               0,               0,               0,     0.023406504,    -0.029119537,     0.010089122 ],
  [               0,               0,               0,      -18.462206,       16.874794,     -0.27974444 ],
  [               0,               0,               0,      -4.2949643,       8.0584259,      -4.3326823 ],
  [               0,               0,               0,       -1.897989,       4.1026921,      -2.7728348 ],
  [               0,               0,               0,       1.0965862,       -1.682163,      0.53374298 ],
  [               0,               0,               0,   -0.0036275594,     0.005113468,   -0.0018211686 ],
  [               0,               0,               0,       8.2275936,       -8.555485,       4.0400747 ],
  [               0,               0,               0,       26.288803,      -23.296833,       3.8447074 ],
 ],
 [
  [               0,               0,               0,               0,     0.023255947,    0.0045587859 ],
  [               0,               0,               0,               0,       7.1392545,      -5.0977967 ],
  [               0,               0,               0,               0,      -22.100118,       17.054439 ],
  [               0,               0,               0,               0,       17.628653,      -14.019141 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   0.00073974674,   -0.0058675084,     0.018839319 ],
  [               0,               0,               0,       6.8825445,      -14.786902,       8.3901932 ],
  [               0,               0,               0,      -5.5008529,       11.869564,      -6.7869667 ],
  [               0,               0,               0,      -6.5308995,       14.054181,      -7.9085051 ],
  [               0,               0,               0,      -0.1667669,      0.35421979,     -0.19515806 ],
  [               0,               0,               0,    -0.003019573,    0.0064900578,   -0.0033040262 ],
  [               0,               0,               0,       11.056741,      -24.108157,       14.844347 ],
  [               0,               0,               0,       7.6107517,       -14.92966,       11.190083 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      -71.637066,       135.47305,       41.362049,      -126.86103 ],
  [               0,               0,       51.150255,      -7.0486291,      -58.060566,      -83.981669 ],
  [               0,               0,      -298.92816,       61.522398,       260.98066,       330.87874 ],
  [               0,               0,               0,               0,               0,              -1 ],
  [               0,               0,       20.848979,      -7.6408907,      -16.106861,      -17.980146 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.046648709,    -0.015672661,     0.038523776,      0.12300807 ],
  [               0,               0,       436.38664,      -624.91849,      -837.56703,       1203.1138 ],
  [               0,               0,      -331.67001,        547.9609,       352.27819,       -655.1541 ],
  [               0,               0,        336.0343,      -421.07869,      -956.66435,       1243.5334 ],
  [               0,               0,       1835.3947,      -2612.3737,      -3734.7882,       5314.6645 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer1_etaoffv4_37_e = [
 [
  [               0,               0,               0,               0,               0,    0.0071938944 ],
  [               0,               0,               0,               0,               0,       -2.431761 ],
  [               0,               0,               0,               0,               0,       1.1552159 ],
  [               0,               0,               0,               0,               0,    -0.075719225 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0061737049 ],
  [               0,               0,               0,               0,               0,       2.9272827 ],
  [               0,               0,               0,               0,               0,      -1.1583216 ],
  [               0,               0,               0,               0,               0,       -1.098175 ],
  [               0,               0,               0,               0,               0,    -0.082129248 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.037185691,     0.069206915 ],
  [               0,               0,               0,               0,      -23.435874,       37.912479 ],
  [               0,               0,               0,               0,       67.334071,      -101.52388 ],
  [               0,               0,               0,               0,      -52.632834,       78.404961 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0054676122,    -0.011250771,    -0.017676288,     0.036936364 ],
  [               0,               0,       5.3311136,       22.475953,      -108.46288,       96.190555 ],
  [               0,               0,      -4.2178225,       25.468655,      -48.311437,       28.255428 ],
  [               0,               0,       7.2070789,      -19.191749,       6.2942132,       8.4989458 ],
  [               0,               0,     -0.13800956,       1.3035894,      -3.2448081,       2.4216331 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.00552506,   -0.0090101281 ],
  [               0,               0,               0,               0,      -6.4641182,       14.845572 ],
  [               0,               0,               0,               0,     -0.77000864,       0.5694629 ],
  [               0,               0,               0,               0,      -1.4954637,       2.0084773 ],
  [               0,               0,               0,               0,    -0.061753556,     0.060916755 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.011003035,     -0.02669223,    -0.053441965,      0.13129169 ],
  [               0,               0,       6.6888287,      -19.012409,       -2.945636,       30.321225 ],
  [               0,               0,       9.7262794,       -45.16013,       61.965502,      -22.170824 ],
  [               0,               0,       -4.911263,       36.240886,      -85.161646,       63.484837 ],
  [               0,               0,       1.6981839,      -15.326355,       42.083613,       -36.49983 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00065648785,    0.0027241856 ],
  [               0,               0,               0,               0,      -114.53777,       265.05319 ],
  [               0,               0,               0,               0,       40.770276,      -95.043729 ],
  [               0,               0,               0,               0,       4.1702733,      -9.2778691 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00030036752,    0.0014170701 ],
  [               0,               0,               0,               0,      0.90247963,      0.18746229 ],
  [               0,               0,               0,               0,      -2.0262438,       3.9313709 ],
  [               0,               0,               0,               0,        1.214671,      -3.7552248 ],
  [               0,               0,               0,               0,       1.4038334,      -3.3047843 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   0.00091896342,  -0.00050219054,    0.0046933305 ],
  [               0,               0,               0,      -40.184531,       176.62051,      -187.01566 ],
  [               0,               0,               0,       85.457629,       -417.8837,       509.40327 ],
  [               0,               0,               0,      -129.61765,       634.98504,      -777.58505 ],
  [               0,               0,               0,   -0.0026237051,     0.015144685,    -0.021009234 ],
  [               0,               0,               0,       4.1493635,      -13.541641,       11.119826 ],
  [               0,               0,               0,      -3.8944721,      -9.4719987,       45.944971 ],
 ],
 [
  [               0,               0,               0,               0,     0.019954981,    -0.039213903 ],
  [               0,               0,               0,               0,      -3645.7961,       9068.4334 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,               0,       1.3250172,        -3.91498 ],
  [               0,               0,               0,               0,      0.22679605,      0.26673697 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer1_etaoffv4_37_b = [
 [
  [               0,               0,      -16666.606,       414.57374,      -2.4107555,    0.0026496559 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00024513033,    0.0012968786 ],
  [               0,               0,               0,               0,      -1.7786584,       3.2962003 ],
  [               0,               0,               0,               0,     -0.40515451,     -0.67008844 ],
  [               0,               0,               0,               0,        1.356575,      -1.3575708 ],
  [               0,               0,               0,               0,      0.24585414,     -0.15261914 ],
  [               0,               0,               0,               0,  -0.00046975957,    0.0002617271 ],
  [               0,               0,               0,               0,      -1.1345146,       1.2846008 ],
  [               0,               0,               0,               0,       1.9392112,      0.88007468 ],
 ],
 [
  [               0,               0,               0,     0.011028852,     -0.01080436,    0.0032993316 ],
  [               0,               0,               0,      -12.138951,       10.001149,       1.7239379 ],
  [               0,               0,               0,       4.1906313,      -8.7249958,       3.2966574 ],
  [               0,               0,               0,      -6.3879898,       9.9754777,      -5.0039281 ],
  [               0,               0,               0,     -0.14307879,       1.0858159,     -0.97059598 ],
  [               0,               0,               0,    0.0032432727,   -0.0039672114,    0.0013433508 ],
  [               0,               0,               0,      -10.855617,       8.5276739,     -0.43021329 ],
  [               0,               0,               0,       26.009294,      -28.188115,       7.1951371 ],
 ],
 [
  [               0,               0,               0,               0,     0.029035609,   -0.0031885289 ],
  [               0,               0,               0,               0,      -23.178767,        19.06923 ],
  [               0,               0,               0,               0,      -14.466413,       10.939079 ],
  [               0,               0,               0,               0,       22.424713,      -17.830976 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0055457253,     -0.01507019,     0.010466561 ],
  [               0,               0,               0,      -26.938897,       47.967427,      -19.153617 ],
  [               0,               0,               0,       9.5626743,      -29.457304,       16.208267 ],
  [               0,               0,               0,       23.889797,      -60.813826,       32.052206 ],
  [               0,               0,               0,     -0.59629951,       3.7114512,      -4.0592706 ],
  [               0,               0,               0,   -0.0094344948,     0.026048559,    -0.019648947 ],
  [               0,               0,               0,      -1.8230561,       5.2340169,      -2.9703918 ],
  [               0,               0,               0,        1.702735,      -4.9227392,       4.9960206 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.099178243,     -0.14988066,     -0.20737765,       0.3215611 ],
  [               0,               0,       1843.5675,      -2653.2104,      -3686.8331,       5309.7256 ],
  [               0,               0,        359.3254,      -439.80434,      -1138.5883,       1475.8336 ],
  [               0,               0,       800.29317,      -1103.0509,       -1824.929,        2521.948 ],
  [               0,               0,       2244.6142,      -3193.2252,      -4559.9069,       6485.4985 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer1_etaoffv4_37_e = [
 [
  [               0,               0,               0,               0,     0.088839693,     -0.12495485 ],
  [               0,               0,               0,               0,      -18.414606,      -13.089262 ],
  [               0,               0,               0,               0,      -68.087631,       100.09043 ],
  [               0,               0,               0,               0,      -55.616389,       79.985734 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      -1.1093789,       1.7133362 ],
  [               0,               0,               0,               0,      -50.098128,       73.899944 ],
  [               0,               0,               0,               0,       32.088139,      -47.357031 ],
  [               0,               0,               0,               0,       35.901168,      -53.134732 ],
  [               0,               0,               0,               0,       1.3531823,      -1.9944946 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       1.2791424,      -1.8735083 ],
  [               0,               0,               0,               0,      -32.722076,       51.012853 ],
  [               0,               0,               0,               0,        63.88437,      -96.271138 ],
  [               0,               0,               0,               0,      -52.612879,        78.31474 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.56042799,     -0.99833994,      -1.5758531,       3.1296963 ],
  [               0,               0,      -24.748043,       127.31071,      -217.83213,       124.25418 ],
  [               0,               0,       24.420436,      -125.63679,       214.99133,       -122.6469 ],
  [               0,               0,       23.151687,      -119.26206,       204.32538,      -116.70442 ],
  [               0,               0,     -0.26399354,       1.3149437,      -2.1809097,       1.2043955 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0078285928,    -0.012579148 ],
  [               0,               0,               0,               0,      -1.4645747,       4.9225804 ],
  [               0,               0,               0,               0,      -1.6038216,       2.0342803 ],
  [               0,               0,               0,               0,      -2.1025982,       3.0075186 ],
  [               0,               0,               0,               0,     -0.16564064,      0.28922243 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0079492594,     0.006536704,     -0.16012049,      0.23866006 ],
  [               0,               0,        6.658839,      -22.266724,       10.000131,       17.187626 ],
  [               0,               0,       14.919316,      -93.034286,       193.15075,      -134.47543 ],
  [               0,               0,      -11.994896,       77.310041,      -165.81102,       117.38577 ],
  [               0,               0,       9.4285706,      -61.952495,        135.4485,      -98.547768 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00020779989,     0.001242692 ],
  [               0,               0,               0,               0,      -342.29362,       790.03549 ],
  [               0,               0,               0,               0,       88.765215,      -205.88324 ],
  [               0,               0,               0,               0,       15.901116,      -35.919142 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00089105364,     0.002586792 ],
  [               0,               0,               0,               0,       2.7581897,      -4.7297065 ],
  [               0,               0,               0,               0,      -3.4798387,       7.2271958 ],
  [               0,               0,               0,               0,    -0.063370325,     -0.91111836 ],
  [               0,               0,               0,               0,      0.63309158,      -1.4982401 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -4.9634671e-05,   0.00039718945 ],
  [               0,               0,               0,               0,       -4.318032,     -0.28148145 ],
  [               0,               0,               0,               0,      0.45891681,      0.88391475 ],
  [               0,               0,               0,               0,       2.4499294,       4.9278917 ],
  [               0,               0,               0,               0,     -0.88697384,      -2.4277713 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.013323011,     0.005052078,    -0.082446516 ],
  [               0,               0,               0,      -28.439718,       79.146404,      -18.879454 ],
  [               0,               0,               0,      -104.72726,       517.50576,      -640.60488 ],
  [               0,               0,               0,      -29.095816,        143.3032,      -176.40099 ],
  [               0,               0,               0,    0.0062496631,    0.0014806723,    -0.041232388 ],
  [               0,               0,               0,       9.9297273,     -0.14002886,      -57.616128 ],
  [               0,               0,               0,       16.723644,      -1.0117308,      -99.800359 ],
 ],
 [
  [               0,               0,               0,               0,       1.4333831,      -3.5482385 ],
  [               0,               0,               0,               0,      -3294.1139,       8193.2213 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,               0,      -250.73365,       620.90325 ],
  [               0,               0,               0,               0,       1.1790168,      -2.5247202 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer1_etaoffv4_37_b = [
 [
  [               0,               0,      -59206.016,       766.36488,      -3.0287847,    0.0023360047 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00022268868,   0.00072963452 ],
  [               0,               0,               0,               0,     -0.19288391,       3.2817543 ],
  [               0,               0,               0,               0,      0.39525311,     -0.61427522 ],
  [               0,               0,               0,               0,      0.54955109,      -0.7370121 ],
  [               0,               0,               0,               0,     -0.19402101,    -0.040408561 ],
  [               0,               0,               0,               0,   7.1673538e-05,   3.2651887e-05 ],
  [               0,               0,               0,               0,      -1.5290802,       2.5785757 ],
  [               0,               0,               0,               0,     -0.38370121,      -1.5723343 ],
 ],
 [
  [               0,               0,               0,   0.00090307045,   0.00030408035,  -0.00013505774 ],
  [               0,               0,               0,       2.9436487,      -11.099125,       9.4712145 ],
  [               0,               0,               0,      0.53106804,      -8.8339965,       5.6405947 ],
  [               0,               0,               0,       29.624128,      -41.538183,       14.609232 ],
  [               0,               0,               0,       1.7624032,      -1.0673692,      -0.2909194 ],
  [               0,               0,               0,   -0.0026726702,    0.0032848293,   -0.0011153771 ],
  [               0,               0,               0,      -11.799139,       14.214312,        -2.45483 ],
  [               0,               0,               0,      -5.1317993,       4.0451273,       0.6378125 ],
 ],
 [
  [               0,               0,               0,               0,     0.022421291,    0.0070128875 ],
  [               0,               0,               0,               0,      -12.273536,       10.360109 ],
  [               0,               0,               0,               0,       -15.06802,       11.439695 ],
  [               0,               0,               0,               0,       17.921052,      -14.247746 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0011622283,   -0.0033367275,     0.002447635 ],
  [               0,               0,               0,        18.10169,      -45.083097,       27.026585 ],
  [               0,               0,               0,       -1.017899,       16.042897,      -11.872581 ],
  [               0,               0,               0,       27.478337,      -35.054205,       7.9261923 ],
  [               0,               0,               0,         -3.6997,       6.4932909,      -3.8131907 ],
  [               0,               0,               0,   0.00071950159,   -0.0015954172,    0.0012390159 ],
  [               0,               0,               0,      -2.2808283,       5.0985632,      -1.5167753 ],
  [               0,               0,               0,       1.8358002,      -5.4967516,       2.1486804 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0024389051,    0.0010918635,   -0.0017026664,   -0.0063790651 ],
  [               0,               0,      -5.2120185,      -5.0205176,      -6.8573491,      -5.3626144 ],
  [               0,               0,      -37.851628,      -51.148971,      -69.045604,      -93.500407 ],
  [               0,               0,               0,               0,               0,              -1 ],
  [               0,               0,       31.054531,       43.995263,       62.309082,       87.919703 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.14966484,     -0.30807284,     -0.37513551,      0.73859689 ],
  [               0,               0,        545.2182,      -785.21504,      -1046.7809,       1512.3048 ],
  [               0,               0,      -772.72475,       1196.4504,       1156.2294,      -1840.8729 ],
  [               0,               0,        285.9544,      -320.90245,      -1009.5012,       1259.7988 ],
  [               0,               0,       2779.2015,      -3959.9387,      -5648.7798,       8046.9469 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer1_etaoffv4_37_e = [
 [
  [               0,               0,               0,               0,      0.11428176,      -0.1644239 ],
  [               0,               0,               0,               0,      -34.962339,       89.683432 ],
  [               0,               0,               0,               0,       -140.6151,       202.95131 ],
  [               0,               0,               0,               0,       203.87171,      -297.81064 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       5.6984112,      -8.2582541 ],
  [               0,               0,               0,               0,      -94.855112,       140.24611 ],
  [               0,               0,               0,               0,        75.48023,      -111.65563 ],
  [               0,               0,               0,               0,      -11.026726,       14.560611 ],
  [               0,               0,               0,               0,      0.08533148,     -0.13317324 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.11321731,      0.17063607 ],
  [               0,               0,               0,               0,       -130.3425,       202.57014 ],
  [               0,               0,               0,               0,      -310.93156,       461.34523 ],
  [               0,               0,               0,               0,       389.04352,       -577.9323 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0019169001,   0.00018750942,  -0.00017828217,    0.0086059802 ],
  [               0,               0,      -7.4737293,       37.441863,      -62.870572,       36.188246 ],
  [               0,               0,       10.732421,      -53.047924,       87.473692,       -48.82029 ],
  [               0,               0,      -10.036924,       50.029316,      -82.699924,       44.634098 ],
  [               0,               0,      -3.7198964,       18.352188,      -30.130601,       16.462386 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.010725294,    -0.017896068 ],
  [               0,               0,               0,               0,      -6.8047362,       15.389719 ],
  [               0,               0,               0,               0,      0.71805932,      -2.5552232 ],
  [               0,               0,               0,               0,      0.13394932,      -1.4143812 ],
  [               0,               0,               0,               0,      -0.2021274,      0.38174781 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00039549025,  -0.00021923871,   0.00099850812,    0.0060315929 ],
  [               0,               0,       1.3564296,      -2.2309382,      -6.0230012,       12.159859 ],
  [               0,               0,       1.5981494,      -3.4970701,      -7.2379464,       14.711879 ],
  [               0,               0,     -0.44935799,        1.112393,       1.3781841,      -4.7281222 ],
  [               0,               0,     -0.37842508,      0.75201315,       2.0187637,      -4.0495285 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       1.2123438,      -2.7796313 ],
  [               0,               0,               0,               0,      -163.00771,        376.3565 ],
  [               0,               0,               0,               0,       106.04366,      -245.01114 ],
  [               0,               0,               0,               0,      -27.137255,       62.609779 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.061618194,     -0.13726218 ],
  [               0,               0,               0,               0,      -5.6928017,       14.910107 ],
  [               0,               0,               0,               0,      0.84683556,      -2.9988924 ],
  [               0,               0,               0,               0,       1.2045253,      -3.8414834 ],
  [               0,               0,               0,               0,     0.043723389,      -0.1002125 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.068527572,       0.1756666 ],
  [               0,               0,               0,               0,       50.294169,      -116.57715 ],
  [               0,               0,               0,               0,      -5.8256699,       12.699615 ],
  [               0,               0,               0,               0,        4.393914,      -10.645195 ],
  [               0,               0,               0,               0,    -0.014951199,     0.036165797 ],
  [               0,               0,               0,               0,       11.818522,      -26.453056 ],
  [               0,               0,               0,               0,       23.815792,      -60.394735 ],
 ],
 [
  [               0,               0,               0,               0,     0.007405472,     0.009053192 ],
  [               0,               0,               0,               0,      -3.4262787,       18.545387 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,               0,       -14.42748,       35.110417 ],
  [               0,               0,               0,               0,       4.4956176,      -10.281828 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer1_etaoffv4_37_b = [
 [
  [               0,               0,      -4941.3079,       266.21438,      -1.8062383,     0.001960713 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00069250806,     0.001970821 ],
  [               0,               0,               0,               0,       0.4559762,       1.8668133 ],
  [               0,               0,               0,               0,      0.25733517,      -1.0130511 ],
  [               0,               0,               0,               0,    -0.098880677,     -0.97329584 ],
  [               0,               0,               0,               0,     -0.15313216,   -0.0011781769 ],
  [               0,               0,               0,               0,   3.6883323e-05,   4.7459119e-05 ],
  [               0,               0,               0,               0,     -0.18815505,         2.68961 ],
  [               0,               0,               0,               0,    -0.052228735,      -1.7291143 ],
 ],
 [
  [               0,               0,               0,    0.0034585107,   0.00071423253,   -7.575584e-05 ],
  [               0,               0,               0,      -8.4991684,       2.6253095,       3.7161106 ],
  [               0,               0,               0,       14.519245,      -16.101485,       3.5173252 ],
  [               0,               0,               0,        17.05165,      -18.492302,       3.8522201 ],
  [               0,               0,               0,     -0.19308299,      0.42047219,     -0.27492634 ],
  [               0,               0,               0,    0.0025083301,   -0.0026151019,   0.00070623489 ],
  [               0,               0,               0,       -7.976495,       7.4798853,      0.81295696 ],
  [               0,               0,               0,      0.22532783,  -0.00033105827,      -1.9272888 ],
 ],
 [
  [               0,               0,               0,               0,     0.023322058,    0.0069046497 ],
  [               0,               0,               0,               0,      -4.1232068,       3.8604246 ],
  [               0,               0,               0,               0,      -17.771897,       13.601942 ],
  [               0,               0,               0,               0,       17.055023,      -13.561782 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0054187749,     0.001922394,    0.0012689234 ],
  [               0,               0,               0,      -1.2042178,       2.7465677,     -0.59690287 ],
  [               0,               0,               0,      0.95345443,      -2.3215253,      0.65323476 ],
  [               0,               0,               0,       1.2416377,        -2.98074,      0.96549753 ],
  [               0,               0,               0,    -0.014359809,     0.053940185,    -0.069006494 ],
  [               0,               0,               0,   0.00048708766,   -0.0012927683,   0.00087351716 ],
  [               0,               0,               0,       4.5850768,      -7.2360398,        4.914524 ],
  [               0,               0,               0,       7.8869589,      -15.223721,        11.61094 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0046889924,     -0.11836409,    -0.078416548,      0.37737793 ],
  [               0,               0,      -169.22712,       253.97898,       414.75959,       -612.1511 ],
  [               0,               0,      -165.14533,        346.1348,      -96.209438,      -87.481643 ],
  [               0,               0,      -139.62598,       311.47976,      -271.16419,       154.86297 ],
  [               0,               0,       2877.1556,       -4101.398,      -5850.3768,       8337.8863 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer1_etaoffv4_37_e = [
 [
  [               0,               0,               0,               0,      -0.1303837,      0.18878488 ],
  [               0,               0,               0,               0,        128.4908,      -223.10233 ],
  [               0,               0,               0,               0,       109.37631,      -154.86219 ],
  [               0,               0,               0,               0,      -83.353844,       120.28648 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       1.2413417,       -1.799047 ],
  [               0,               0,               0,               0,      -195.55124,       288.92679 ],
  [               0,               0,               0,               0,       91.364722,      -135.25928 ],
  [               0,               0,               0,               0,      -185.60316,       268.92706 ],
  [               0,               0,               0,               0,     0.095484131,     -0.14338407 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.038652977,     -0.04640539 ],
  [               0,               0,               0,               0,       -128.2024,       194.30553 ],
  [               0,               0,               0,               0,       38.633875,      -58.882471 ],
  [               0,               0,               0,               0,      -17.086207,       25.599753 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.021563925,     -0.02879128,    -0.056067107,      0.07727507 ],
  [               0,               0,      -36.473605,       187.31996,      -325.55272,       192.77573 ],
  [               0,               0,      -23.559171,       130.94123,      -239.25687,       143.03844 ],
  [               0,               0,      -2.8328422,        24.63511,      -58.555464,       41.265131 ],
  [               0,               0,      -8.3314584,       41.510356,      -69.034951,       38.336431 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0056135168,   -0.0084213474 ],
  [               0,               0,               0,               0,      -3.1145812,       8.5081616 ],
  [               0,               0,               0,               0,     -0.71590029,      0.22484355 ],
  [               0,               0,               0,               0,      -1.2675128,       1.2904359 ],
  [               0,               0,               0,               0,      -0.2054133,      0.38204395 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.011298069,      0.11867633,     -0.35770539,      0.33534148 ],
  [               0,               0,      -34.687347,        191.1681,      -338.83181,       192.51057 ],
  [               0,               0,       14.815586,       -88.24613,       173.77215,      -114.14479 ],
  [               0,               0,      -17.273451,       108.02881,      -224.60959,       154.16635 ],
  [               0,               0,       21.645613,      -141.29713,       307.17959,      -222.38324 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00070440638,    0.0034555645 ],
  [               0,               0,               0,               0,      -119.64134,       276.89396 ],
  [               0,               0,               0,               0,       43.837939,      -102.11134 ],
  [               0,               0,               0,               0,       1.3013429,      -2.6999042 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00081064368,    0.0023562098 ],
  [               0,               0,               0,               0,       1.0453111,     -0.27087945 ],
  [               0,               0,               0,               0,      -1.8292892,       3.1535633 ],
  [               0,               0,               0,               0,      0.34880542,      -1.9234935 ],
  [               0,               0,               0,               0,      0.29845779,     -0.67591548 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0029706052,    0.0018767848,    -0.013634123 ],
  [               0,               0,               0,          -5.709,       2.5893094,       33.658314 ],
  [               0,               0,               0,      -6.2799924,       19.961816,      -13.005483 ],
  [               0,               0,               0,      0.25187972,        6.705485,      -17.742649 ],
  [               0,               0,               0,    0.0030404705,   2.4301381e-06,    -0.017020185 ],
  [               0,               0,               0,       5.8633353,      0.76251159,      -36.106703 ],
  [               0,               0,               0,      -18.884056,      0.43756446,       109.84529 ],
 ],
 [
  [               0,               0,               0,               0,    0.0045559679,     0.023859215 ],
  [               0,               0,               0,               0,      0.57874863,       11.677779 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,               0,      -12.740872,       31.068181 ],
  [               0,               0,               0,               0,      0.35565508,      0.13149093 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer1_etaoffv4_37_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00035076571,    0.0013552761 ],
  [               0,               0,               0,               0,       0.1615133,       2.6834144 ],
  [               0,               0,               0,               0,      0.35725971,      -1.0967521 ],
  [               0,               0,               0,               0,      0.11511972,      -1.1248931 ],
  [               0,               0,               0,               0,     -0.17541705,    -0.017711678 ],
  [               0,               0,               0,               0,   1.9985109e-05,   3.3430194e-05 ],
  [               0,               0,               0,               0,      -1.5525024,       3.1467893 ],
  [               0,               0,               0,               0,      -1.8700393,      -1.2517377 ],
 ],
 [
  [               0,               0,               0,               0,   0.00043203385,    0.0006858163 ],
  [               0,               0,               0,               0,      -1.0120201,       3.6868066 ],
  [               0,               0,               0,               0,   -0.0075856349,     -0.77608112 ],
  [               0,               0,               0,               0,     -0.10598757,     -0.89833302 ],
  [               0,               0,               0,               0,     -0.20194897,    -0.025476184 ],
  [               0,               0,               0,               0,  -4.9059144e-05,    8.840047e-05 ],
  [               0,               0,               0,               0,     -0.41158824,       2.3598176 ],
  [               0,               0,               0,               0,      -1.8476435,      -1.0734813 ],
 ],
 [
  [               0,               0,               0,               0,     0.026291757,   -0.0009703991 ],
  [               0,               0,               0,               0,      -5.8343456,        5.268368 ],
  [               0,               0,               0,               0,      -25.092382,       19.389715 ],
  [               0,               0,               0,               0,       23.409807,      -18.617192 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   0.00022061987,   0.00056708319,    0.0016806734 ],
  [               0,               0,               0,      0.16489824,      -1.0261725,       2.9137401 ],
  [               0,               0,               0,     -0.27849804,      0.83935699,      -1.5418993 ],
  [               0,               0,               0,      0.22970202,     -0.34423373,      -1.1626292 ],
  [               0,               0,               0,    -0.080414256,      0.18381065,     -0.21840784 ],
  [               0,               0,               0,   8.4653582e-05,  -0.00022639063,   0.00014232471 ],
  [               0,               0,               0,       1.3632502,      0.55788229,       3.8923051 ],
  [               0,               0,               0,      -5.4449189,      -1.1001471,     -0.56367213 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.10084708,     -0.29841361,     -0.31922276,       0.7824431 ],
  [               0,               0,      -179.67855,       261.72228,       450.65538,      -649.06508 ],
  [               0,               0,      -24.688132,        205.9626,      -562.47791,       455.32477 ],
  [               0,               0,      -387.50131,       679.60746,       145.62944,      -469.42843 ],
  [               0,               0,       3240.8171,       -4617.521,      -6598.8372,       9400.0114 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer1_etaoffv4_37_e = [
 [
  [               0,               0,               0,     0.016969091,  -0.00084752263,    -0.032761625 ],
  [               0,               0,               0,       41.880347,      -10.708156,      -109.54664 ],
  [               0,               0,               0,      -13.517708,       1.3043279,       28.149191 ],
  [               0,               0,               0,       -76.10451,       1.4274429,       157.58282 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.075894123,     -0.10824753 ],
  [               0,               0,               0,               0,       825.02531,      -1199.8762 ],
  [               0,               0,               0,               0,      -210.65942,        308.8805 ],
  [               0,               0,               0,               0,      -1.4906043,       4.6541086 ],
  [               0,               0,               0,               0,      0.31791259,     -0.32607331 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.22941172,      0.34693986 ],
  [               0,               0,               0,               0,      -237.16808,       358.87345 ],
  [               0,               0,               0,               0,      -50.130781,       73.149421 ],
  [               0,               0,               0,               0,        74.32909,      -110.20263 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0044173087,    0.0053557312,    0.0080242766,     0.016641573 ],
  [               0,               0,      0.41405219,     -0.55394905,      -1.4935908,       2.6441415 ],
  [               0,               0,       2.2340202,      -12.395506,       22.543047,      -13.986363 ],
  [               0,               0,      -3.7756113,       17.453328,      -26.808396,       13.177442 ],
  [               0,               0,      -1.2611324,       6.2442094,      -10.294006,       5.6508257 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0059636285,   -0.0092838946 ],
  [               0,               0,               0,               0,      -4.3791998,        11.11845 ],
  [               0,               0,               0,               0,     -0.37116643,     -0.45206665 ],
  [               0,               0,               0,               0,     -0.99369378,      0.76644178 ],
  [               0,               0,               0,               0,     -0.26831805,      0.51390198 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.013624304,    -0.030000585,    -0.061752663,       0.1395798 ],
  [               0,               0,       3.2339663,      -40.675404,       130.21414,      -121.55632 ],
  [               0,               0,       27.981183,      -178.13259,         377.548,      -267.61143 ],
  [               0,               0,      -11.541892,       74.469275,      -159.62566,       112.54251 ],
  [               0,               0,       9.1762717,      -60.421491,       132.55102,      -96.858009 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00030747972,    0.0026252208 ],
  [               0,               0,               0,               0,      -199.02723,       460.03249 ],
  [               0,               0,               0,               0,       53.623164,      -124.97365 ],
  [               0,               0,               0,               0,       8.5304382,      -19.102819 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00010136671,    0.0018028297 ],
  [               0,               0,               0,               0,      -8.3205206,       23.124601 ],
  [               0,               0,               0,               0,       2.3961178,      -6.6954653 ],
  [               0,               0,               0,               0,       3.0188998,      -8.0588787 ],
  [               0,               0,               0,               0,      0.26877778,     -0.57926512 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.030780963,      0.12883254,     -0.12304488 ],
  [               0,               0,               0,      -161.18626,       772.07073,      -917.40541 ],
  [               0,               0,               0,      -158.37345,       762.84961,       -920.0676 ],
  [               0,               0,               0,        60.65609,      -289.08268,       344.35363 ],
  [               0,               0,               0,     0.001186443,     0.017122051,    -0.048058932 ],
  [               0,               0,               0,       6.0601786,      -89.605384,       184.17245 ],
  [               0,               0,               0,      -21.585142,      -15.316403,         164.668 ],
 ],
 [
  [               0,               0,               0,               0,     0.026213145,    0.0089722121 ],
  [               0,               0,               0,               0,       2.4329392,        33.94703 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,               0,    -0.092037583,    0.0042242724 ],
  [               0,               0,               0,               0,         4.14453,      -8.9399486 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]


ele_layer1_etaoffv4_37_b = [
    ele5_layer1_etaoffv4_37_b,
    ele10_layer1_etaoffv4_37_b,
    ele25_layer1_etaoffv4_37_b,
    ele50_layer1_etaoffv4_37_b,
    ele75_layer1_etaoffv4_37_b,
    ele100_layer1_etaoffv4_37_b,
    ele200_layer1_etaoffv4_37_b,
    ele500_layer1_etaoffv4_37_b,
    ele1000_layer1_etaoffv4_37_b,
    ]

ele_layer1_etaoffv4_37_e = [
    ele5_layer1_etaoffv4_37_e,
    ele10_layer1_etaoffv4_37_e,
    ele25_layer1_etaoffv4_37_e,
    ele50_layer1_etaoffv4_37_e,
    ele75_layer1_etaoffv4_37_e,
    ele100_layer1_etaoffv4_37_e,
    ele200_layer1_etaoffv4_37_e,
    ele500_layer1_etaoffv4_37_e,
    ele1000_layer1_etaoffv4_37_e,
    ]



###############################################################################
# Tables for v4 sshape correction, layer 1, 3x7 photon clusters
#

gam5_layer1_etaoffv4_37_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      -11303.479,       8797.1883 ],
  [               0,               0,               0,               0,    0.0074320017,    0.0084438477 ],
  [               0,               0,               0,               0,    -0.007431814,   -0.0084435722 ],
  [               0,               0,               0,               0,   -0.0074304981,   -0.0084438525 ],
  [               0,               0,               0,               0,   4.4571897e-09,   -2.025953e-08 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,       4.9852462,       63.997628,       51.124815,      -19.518041 ],
  [               0,               0,        6.180438,      -11.250739,       6.7193328,      -1.2623022 ],
  [               0,               0,      -6.1814193,       11.252887,      -6.7208213,       1.2626623 ],
  [               0,               0,      -6.1787531,       11.249282,      -6.7197872,        1.262786 ],
  [               0,               0,  -0.00024851629,   0.00067736046,  -0.00056067075,   0.00014238205 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam5_layer1_etaoffv4_37_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer1_etaoffv4_37_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0029994169,    0.0014807587 ],
  [               0,               0,               0,               0,       10.298699,      -1.6952133 ],
  [               0,               0,               0,               0,      -10.671929,        1.688054 ],
  [               0,               0,               0,               0,        -8.54844,       2.4056293 ],
  [               0,               0,               0,               0,      0.55707862,     0.088376369 ],
  [               0,               0,               0,               0,  -0.00090707016,  -0.00017566015 ],
  [               0,               0,               0,               0,      -2.0339513,        1.547754 ],
  [               0,               0,               0,               0,      0.71183775,      0.91067046 ],
 ],
 [
  [               0,               0,               0,       11.066526,     0.082819799,      -1.1254793 ],
  [               0,               0,               0,      0.13821385,       -0.243689,      0.21808522 ],
  [               0,               0,               0,     -0.12875994,      0.23018905,     -0.21289613 ],
  [               0,               0,               0,     -0.14122589,      0.24666976,      -0.2183139 ],
  [               0,               0,               0,   -0.0032362525,    0.0044024331,   -0.0015305879 ],
  [               0,               0,               0,   -0.0035874248,    0.0051572438,   -0.0017967838 ],
  [               0,               0,               0,       22.129588,      -16.925899,        3.930003 ],
  [               0,               0,               0,      -55.634207,       59.080718,      -13.130285 ],
 ],
 [
  [               0,               0,               0,               0,     0.065466885,    -0.030111486 ],
  [               0,               0,               0,               0,      -42.155802,       34.229894 ],
  [               0,               0,               0,               0,       30.052582,      -24.526579 ],
  [               0,               0,               0,               0,       11.020243,      -8.7547588 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,       0.0059585,    -0.010210969,    0.0045016996 ],
  [               0,               0,               0,       1.8571056,      -3.0601661,       3.9635526 ],
  [               0,               0,               0,      -6.2171551,       9.9658217,      -3.3845884 ],
  [               0,               0,               0,       16.531868,      -49.517573,       35.402044 ],
  [               0,               0,               0,      -6.9909214,       18.349307,      -12.165559 ],
  [               0,               0,               0,   -0.0019284746,    0.0034950645,   -0.0013055459 ],
  [               0,               0,               0,      -6.5473614,       11.806917,       -3.985813 ],
  [               0,               0,               0,        -21.2039,       37.624907,      -18.518674 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.15936198,     -0.20547242,     -0.30806727,      0.40288905 ],
  [               0,               0,       3776.5737,      -5455.7752,      -7600.0526,       10981.814 ],
  [               0,               0,      -392.60543,       645.25462,       375.61038,      -709.11919 ],
  [               0,               0,       1716.7661,      -2404.5923,      -3610.1243,       5057.4332 ],
  [               0,               0,       2225.0316,       -3168.126,      -4504.0279,       6411.6254 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer1_etaoffv4_37_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,       801.22113 ],
  [               0,               0,               0,               0,               0,     0.020880975 ],
  [               0,               0,               0,               0,               0,    -0.020879059 ],
  [               0,               0,               0,               0,               0,    -0.020878362 ],
  [               0,               0,               0,               0,               0,   4.0206178e-08 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer1_etaoffv4_37_b = [
 [
  [               0,               0,      -33421.467,       570.70338,      -3.1966305,    0.0049243435 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    9.518306e-08,    0.0010860709 ],
  [               0,               0,               0,               0,     -0.57847771,       2.5546688 ],
  [               0,               0,               0,               0,      0.04214244,     -0.85001727 ],
  [               0,               0,               0,               0,       1.2033495,      -1.2004793 ],
  [               0,               0,               0,               0,       4881.8162,      -1008.7078 ],
  [               0,               0,               0,               0,      -5.3022232,       1.0955141 ],
  [               0,               0,               0,               0,    -0.038894304,    0.0089836097 ],
  [               0,               0,               0,               0,     0.026681937,   -0.0087677776 ],
 ],
 [
  [               0,               0,               0,    0.0023836974,   -0.0040134336,     0.002666009 ],
  [               0,               0,               0,      -32.629071,       37.807196,      -7.4656853 ],
  [               0,               0,               0,       11.414549,      -12.087043,       2.0129441 ],
  [               0,               0,               0,       8.1661038,      -8.1409817,      0.68311555 ],
  [               0,               0,               0,      -1.0091566,       0.8602618,     -0.27497976 ],
  [               0,               0,               0,  -0.00021452768,   9.7796406e-05,   9.1599874e-05 ],
  [               0,               0,               0,       26.188066,      -28.593112,       9.8221682 ],
  [               0,               0,               0,        6.060271,       -5.327243,       2.8874968 ],
 ],
 [
  [               0,               0,               0,      -18.767535,       1.0745318,       13.045117 ],
  [               0,               0,               0,       42.990238,      -75.240628,        32.78547 ],
  [               0,               0,               0,       14.616381,       -16.81471,       3.9905884 ],
  [               0,               0,               0,       1.1626662,      -1.6636587,      0.58781655 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0017779259,   -0.0050489725,    0.0038413223 ],
  [               0,               0,               0,       31.923059,       -49.50215,       22.809626 ],
  [               0,               0,               0,       3.9818696,      -3.4385437,     -0.86254856 ],
  [               0,               0,               0,        10.31126,      -18.150489,       6.6370636 ],
  [               0,               0,               0,     -0.37585427,     -0.46001925,      0.36735189 ],
  [               0,               0,               0,   0.00019680954,  -0.00051674342,    0.0002328591 ],
  [               0,               0,               0,       5.2059913,      -10.408481,       8.3135179 ],
  [               0,               0,               0,       5.6901725,      -12.471305,       11.159789 ],
 ],
 [
  [               0,               0,               0,               0,               0,       69.982657 ],
  [               0,               0,               0,               0,               0,     0.037929853 ],
  [               0,               0,               0,               0,               0,    -0.037927076 ],
  [               0,               0,               0,               0,               0,   1.6927866e-06 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,       0.3161676,     -0.40983329,     -0.61915829,      0.80584113 ],
  [               0,               0,       1319.0587,      -1917.4941,        -2647.23,        3853.609 ],
  [               0,               0,       1315.9063,      -1876.9396,      -2907.6646,       4146.9993 ],
  [               0,               0,      -359.19511,        576.3532,        410.2902,      -718.19956 ],
  [               0,               0,       1694.2008,      -2403.9403,      -3438.5783,       4877.5594 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer1_etaoffv4_37_e = [
 [
  [               0,               0,               0,               0,     -0.84595855,       1.2030711 ],
  [               0,               0,               0,               0,      -123.72471,       179.74033 ],
  [               0,               0,               0,               0,       122.03155,       -177.0407 ],
  [               0,               0,               0,               0,      -17.088553,       24.603567 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0009314843 ],
  [               0,               0,               0,               0,               0,        8.275986 ],
  [               0,               0,               0,               0,               0,      -2.1223895 ],
  [               0,               0,               0,               0,               0,     -0.14765332 ],
  [               0,               0,               0,               0,               0,     -0.22484177 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.12691182,      0.19093198 ],
  [               0,               0,               0,               0,       164.42909,      -232.12021 ],
  [               0,               0,               0,               0,       294.51583,      -438.57046 ],
  [               0,               0,               0,               0,      -60.099019,       89.676966 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      -963.31267,        3107.312,      -2451.5439 ],
  [               0,               0,               0,      0.74350917,      -2.5079857,       2.1667359 ],
  [               0,               0,               0,     -0.74194031,       2.5028705,      -2.1625257 ],
  [               0,               0,               0,       -0.742424,       2.5043978,      -2.1637308 ],
  [               0,               0,               0,  -0.00037009482,    0.0012057214,  -0.00098348949 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0026383303,   -0.0033383524 ],
  [               0,               0,               0,               0,    -0.060126511,       2.2325022 ],
  [               0,               0,               0,               0,      -4.2149669,       7.2341622 ],
  [               0,               0,               0,               0,     -0.21075974,     -0.25060956 ],
  [               0,               0,               0,               0,      0.52918474,      -1.0919483 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00038312357,  -0.00032583139,   0.00071052509,    0.0060994402 ],
  [               0,               0,       6.3599299,      -12.482852,       -30.08529,       61.689935 ],
  [               0,               0,       1.5040944,      -2.8132297,      -7.4560313,       13.024879 ],
  [               0,               0,      -1.9796535,       4.4623224,       7.7788557,      -18.568133 ],
  [               0,               0,     -0.79578324,       1.5732737,       4.0210314,      -8.0341363 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0046679177,     0.014916191 ],
  [               0,               0,               0,               0,      -37.704057,       87.440041 ],
  [               0,               0,               0,               0,       33.642517,       -78.01224 ],
  [               0,               0,               0,               0,      -3.4372344,       7.9460561 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.011494586,     0.029089336 ],
  [               0,               0,               0,               0,       3.7614368,      -6.1601537 ],
  [               0,               0,               0,               0,       2.0822874,      -5.7942354 ],
  [               0,               0,               0,               0,       3.7460094,       -9.750823 ],
  [               0,               0,               0,               0,     -0.42637662,       0.9504984 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.015242481,    0.0012268469,     0.095813775 ],
  [               0,               0,               0,     -0.47319204,       2.1670675,       2.8870643 ],
  [               0,               0,               0,     0.027036797,      -8.3224141,       18.607175 ],
  [               0,               0,               0,       1.6988047,      -3.2126221,      -2.1818972 ],
  [               0,               0,               0,   -0.0083420037,  -7.1173722e-05,     0.050016462 ],
  [               0,               0,               0,      -2.5272011,     -0.42621294,       16.521586 ],
  [               0,               0,               0,       22.894909,       4.0980703,      -154.04606 ],
 ],
 [
  [               0,               0,               0,               0,     0.026001124,    -0.010034416 ],
  [               0,               0,               0,               0,       17.094778,      -39.849592 ],
  [               0,               0,               0,               0,       32802.214,      -81619.352 ],
  [               0,               0,               0,               0,      -64.148494,       157.71948 ],
  [               0,               0,               0,               0,     0.044426627,   -0.0013539272 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer1_etaoffv4_37_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.001665587,   0.00023113791 ],
  [               0,               0,               0,               0,      -4.2010895,       4.8288782 ],
  [               0,               0,               0,               0,       88.060298,       -40.19008 ],
  [               0,               0,               0,               0,       87.655941,      -40.219981 ],
  [               0,               0,               0,               0,      -9.6544702,       1.5986983 ],
  [               0,               0,               0,               0,     0.082837736,     0.013124686 ],
  [               0,               0,               0,               0,      -0.5460363,       0.2440235 ],
  [               0,               0,               0,               0,      0.18750856,       -1.603917 ],
 ],
 [
  [               0,               0,               0,    0.0068614865,   -0.0074020522,    0.0029910886 ],
  [               0,               0,               0,      -9.4607323,       10.210049,      0.16544957 ],
  [               0,               0,               0,       5.6640645,      -7.0188428,       1.2322002 ],
  [               0,               0,               0,      -5.9740059,       7.3396403,      -3.2680628 ],
  [               0,               0,               0,      -1.8014797,       2.0694843,     -0.70269811 ],
  [               0,               0,               0,     0.001639006,    -0.002221044,   0.00070310065 ],
  [               0,               0,               0,       57.060923,      -60.966638,       17.079247 ],
  [               0,               0,               0,      -59.157223,         68.7795,      -18.999682 ],
 ],
 [
  [               0,               0,               0,               0,       -42.64963,       34.261625 ],
  [               0,               0,               0,               0,     -0.30597814,      0.44400174 ],
  [               0,               0,               0,               0,      -2.0617285,       1.4413361 ],
  [               0,               0,               0,               0,       1.9590102,      -1.5577493 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   0.00021427669,   -0.0010930737,    0.0016732084 ],
  [               0,               0,               0,       15.757205,      -27.454041,       15.378928 ],
  [               0,               0,               0,      -2.1165708,       5.0847452,       -3.483813 ],
  [               0,               0,               0,   -0.0010286726,     -0.26408659,      -0.9965911 ],
  [               0,               0,               0,     -0.28981357,      0.20382061,      -0.2398628 ],
  [               0,               0,               0,   0.00024154463,  -0.00037027074,   0.00018073924 ],
  [               0,               0,               0,      -4.1442357,       8.4001282,        -1.03012 ],
  [               0,               0,               0,        3.843791,      -9.9309913,       7.8913485 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.23659863,     -0.31876611,      -0.4709746,      0.64086067 ],
  [               0,               0,       1715.2436,      -2475.2814,      -3425.8433,       4948.8178 ],
  [               0,               0,       988.38155,      -1351.8549,      -2433.9484,       3353.6822 ],
  [               0,               0,       485.13872,      -647.37597,      -1242.5083,       1678.3199 ],
  [               0,               0,       2496.2815,      -3553.4883,      -5058.8475,       7199.5097 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer1_etaoffv4_37_e = [
 [
  [               0,               0,               0,               0,       -3518.517,       5946.1941 ],
  [               0,               0,               0,               0,      0.16469362,     -0.26042114 ],
  [               0,               0,               0,               0,     -0.16505472,      0.26093818 ],
  [               0,               0,               0,               0,   4.9518524e-05,  -7.1996998e-05 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0077324521 ],
  [               0,               0,               0,               0,               0,       2.0668147 ],
  [               0,               0,               0,               0,               0,      -1.1083984 ],
  [               0,               0,               0,               0,               0,     -0.87289719 ],
  [               0,               0,               0,               0,               0,     -0.03476762 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.34666775,      0.52557112 ],
  [               0,               0,               0,               0,      -198.37164,       299.48435 ],
  [               0,               0,               0,               0,       114.11229,      -171.09103 ],
  [               0,               0,               0,               0,      -75.405675,       112.35138 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00099834349 ],
  [               0,               0,               0,               0,               0,      -39.878049 ],
  [               0,               0,               0,               0,               0,       5.9196599 ],
  [               0,               0,               0,               0,               0,       1.5818959 ],
  [               0,               0,               0,               0,               0,       2.1973237 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,       5.6557418,      -9.3326959,      -17.305952,       30.315498 ],
  [               0,               0,      -14.521723,       74.354299,      -126.83249,       72.218982 ],
  [               0,               0,       14.358439,      -73.538635,       125.47488,       -71.46506 ],
  [               0,               0,       14.263741,      -73.072904,       124.71317,      -71.050765 ],
  [               0,               0,   -0.0075367084,     0.034577308,    -0.052401296,     0.026183796 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0019820811,    0.0060083633 ],
  [               0,               0,               0,               0,      0.87516566,      0.52692102 ],
  [               0,               0,               0,               0,      0.27343129,      -1.6175185 ],
  [               0,               0,               0,               0,      0.51096058,      -1.9644054 ],
  [               0,               0,               0,               0,     0.055977549,     -0.12798975 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.014565993,     0.031673815,     0.061722543,     -0.13082109 ],
  [               0,               0,       9.5068675,       -36.11588,       24.247965,        22.56984 ],
  [               0,               0,         31.7772,       -197.2764,       407.84524,      -281.92709 ],
  [               0,               0,       -14.21107,       104.25958,      -249.87718,       195.36734 ],
  [               0,               0,       13.916744,      -89.800187,       193.12425,      -138.43147 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0017583423,    0.0053763146 ],
  [               0,               0,               0,               0,       -48.61397,       112.92374 ],
  [               0,               0,               0,               0,       23.456561,      -54.924819 ],
  [               0,               0,               0,               0,      0.45492601,     -0.90006888 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.019568575,    -0.042529519 ],
  [               0,               0,               0,               0,      -11.733789,       29.823703 ],
  [               0,               0,               0,               0,       1.7644614,      -5.3387333 ],
  [               0,               0,               0,               0,      0.67123743,      -2.5282215 ],
  [               0,               0,               0,               0,      0.41969654,     -0.96381871 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.015764307,    0.0077593556,    -0.096177243 ],
  [               0,               0,               0,      -140.98451,       666.12091,      -782.00205 ],
  [               0,               0,               0,       36.300932,      -174.54222,       208.51061 ],
  [               0,               0,               0,      -58.022827,       283.70091,      -346.79439 ],
  [               0,               0,               0,   -0.0052096815,    0.0033825463,     0.019685382 ],
  [               0,               0,               0,       15.295208,       -74.55499,       91.949822 ],
  [               0,               0,               0,       1.3978811,      -11.644588,       18.255076 ],
 ],
 [
  [               0,               0,               0,       285.56223,      -1414.0622,       1750.5574 ],
  [               0,               0,               0,      -776.52043,       1630.8365,       747.78705 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,      -1431.6255,       6485.6333,      -7279.8785 ],
  [               0,               0,               0,      -10.025789,      -14.080912,        96.89227 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer1_etaoffv4_37_b = [
 [
  [               0,               0,       59931.198,      -503.77679,      0.80592885,  -6.5215907e-05 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0018842324,   0.00040464144 ],
  [               0,               0,               0,               0,      -4.7138553,       4.5443654 ],
  [               0,               0,               0,               0,       9.9372634,      -5.3571691 ],
  [               0,               0,               0,               0,       10.950405,      -5.8519021 ],
  [               0,               0,               0,               0,     -0.88207707,     -0.66757716 ],
  [               0,               0,               0,               0,    0.0072809841,    0.0024911491 ],
  [               0,               0,               0,               0,     -0.68135271,      0.30424546 ],
  [               0,               0,               0,               0,      0.28297132,      -1.4779947 ],
 ],
 [
  [               0,               0,               0,   -0.0015885422,    0.0038662895,  -0.00049678454 ],
  [               0,               0,               0,      -8.9223188,       6.1766882,       2.4531107 ],
  [               0,               0,               0,       11.131682,       -12.72148,       2.6299607 ],
  [               0,               0,               0,      -1.6412581,       2.1197568,      -1.8623257 ],
  [               0,               0,               0,       -2.663945,       3.0889969,       -1.008789 ],
  [               0,               0,               0,    0.0045147879,   -0.0052399988,    0.0014730795 ],
  [               0,               0,               0,       30.496135,      -28.410767,       5.2640919 ],
  [               0,               0,               0,      -47.685818,       53.034654,      -13.136248 ],
 ],
 [
  [               0,               0,               0,               0,     -0.35280473,      0.31186332 ],
  [               0,               0,               0,               0,      -10.699497,        9.005675 ],
  [               0,               0,               0,               0,      -7.1631097,       5.2365756 ],
  [               0,               0,               0,               0,       14.159006,      -11.260592 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.025574144,    -0.058716302,       0.0409184 ],
  [               0,               0,               0,      -2.8718115,        6.174576,       -2.177345 ],
  [               0,               0,               0,      -5.4510441,       10.700971,      -6.2102705 ],
  [               0,               0,               0,      -5.2665901,       10.257986,       -6.034181 ],
  [               0,               0,               0,       1.6758606,      -2.5500942,      0.33788239 ],
  [               0,               0,               0,     0.040073878,    -0.094344548,     0.063923708 ],
  [               0,               0,               0,      -1.9930994,       3.9543346,      -1.9946157 ],
  [               0,               0,               0,       1.7488844,      -2.9553383,       2.3592099 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.018237794,    0.0084255956,    -0.011966425,    -0.049258457 ],
  [               0,               0,      -5.3253339,      -5.3145576,      -6.7866708,      -4.6228539 ],
  [               0,               0,      -3.5661193,      -4.7042216,      -6.1862558,      -8.5068536 ],
  [               0,               0,               0,               0,               0,              -1 ],
  [               0,               0,       2.8049334,       3.9679079,       5.6109166,       7.6313127 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.068548544,     -0.15133392,      -0.1788557,      0.37314269 ],
  [               0,               0,       560.18033,      -804.29895,      -1080.8178,       1556.3507 ],
  [               0,               0,      -270.34116,       475.57848,       135.10362,      -375.82558 ],
  [               0,               0,       373.18656,      -475.91131,      -1062.3823,       1397.6023 ],
  [               0,               0,        2380.141,      -3389.7506,      -4836.9788,       6887.1597 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer1_etaoffv4_37_e = [
 [
  [               0,               0,               0,               0,    -0.018917491,     0.028400346 ],
  [               0,               0,               0,               0,       -17.91858,      -13.100793 ],
  [               0,               0,               0,               0,       -198.9738,       289.32831 ],
  [               0,               0,               0,               0,      -104.04638,       148.88401 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,       60285.684 ],
  [               0,               0,               0,               0,               0,    0.0082712904 ],
  [               0,               0,               0,               0,               0,   -0.0082711372 ],
  [               0,               0,               0,               0,               0,    -0.008271141 ],
  [               0,               0,               0,               0,               0,  -1.1459953e-08 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       0.4012745,     -0.59189202 ],
  [               0,               0,               0,               0,       464.54198,       -686.4751 ],
  [               0,               0,               0,               0,       82.136113,      -124.91165 ],
  [               0,               0,               0,               0,       8.3252692,      -12.064394 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.012894583,    -0.017779767,    -0.035092944,      0.04985122 ],
  [               0,               0,      -56.612962,       326.11415,      -623.67338,       397.78783 ],
  [               0,               0,      -27.122354,       136.37796,      -229.95709,        129.0814 ],
  [               0,               0,       8.4679968,       -33.31236,       38.055208,      -10.962122 ],
  [               0,               0,      -14.030392,       69.346431,      -114.09356,       62.484717 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0043632514,    0.0079110752 ],
  [               0,               0,               0,               0,     -0.17879717,       1.2495849 ],
  [               0,               0,               0,               0,     0.088532786,     -0.92611861 ],
  [               0,               0,               0,               0,      0.15385769,       -1.053836 ],
  [               0,               0,               0,               0,    -0.012361479,     0.019816438 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   0.00035813527,   0.00036218068,  -0.00014915546,   -0.0019553621 ],
  [               0,               0,       1.1536952,       -3.522692,      -6.0867576,       20.441996 ],
  [               0,               0,       1.0136823,      -2.1081747,      -4.3106422,       7.7988253 ],
  [               0,               0,      0.66727828,      -1.4771631,      -2.4794406,       4.4313794 ],
  [               0,               0,     -0.22456859,      0.42047351,       1.2924041,      -2.4779916 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0010766582,     0.002745024 ],
  [               0,               0,               0,               0,      -105.26849,       243.73396 ],
  [               0,               0,               0,               0,       42.076288,      -97.893484 ],
  [               0,               0,               0,               0,       5.3881135,      -12.140849 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.023641069,      0.05985033 ],
  [               0,               0,               0,               0,       6.8271596,      -13.849212 ],
  [               0,               0,               0,               0,     -0.66118924,      0.33738651 ],
  [               0,               0,               0,               0,     -0.29066027,     -0.45372113 ],
  [               0,               0,               0,               0,     0.080076944,     -0.17155214 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0054874341,     0.032159273,    -0.039521895 ],
  [               0,               0,               0,      -63.734634,       279.30494,      -295.35486 ],
  [               0,               0,               0,      -122.38895,       579.59022,      -687.43083 ],
  [               0,               0,               0,      -19.888698,       108.61088,      -146.30802 ],
  [               0,               0,               0,    0.0059202497,   -0.0018078861,    -0.029249988 ],
  [               0,               0,               0,       3.4048335,      -19.353035,       27.971064 ],
  [               0,               0,               0,      -8.0999372,       13.549838,       13.025528 ],
 ],
 [
  [               0,               0,               0,               0,     0.047454801,     -0.11745008 ],
  [               0,               0,               0,               0,       7.0760292,       21.579308 ],
  [               0,               0,               0,               0,      -119.24371,       -300.2747 ],
  [               0,               0,               0,               0,       7169.8969,      -17730.764 ],
  [               0,               0,               0,               0,      -2.8163105,       22.590614 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer1_etaoffv4_37_b = [
 [
  [               0,               0,       58431.782,       -401.5487,     0.081998877,    0.0012374675 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00089914618,   0.00050957512 ],
  [               0,               0,               0,               0,      -2.6476289,       4.5723909 ],
  [               0,               0,               0,               0,      -2.7016551,     -0.28001203 ],
  [               0,               0,               0,               0,    -0.092296392,      -0.8186658 ],
  [               0,               0,               0,               0,      0.35379069,     -0.13103671 ],
  [               0,               0,               0,               0,  -0.00041819464,   7.1149417e-05 ],
  [               0,               0,               0,               0,      -1.7910539,       1.9189131 ],
  [               0,               0,               0,               0,        3.949475,     -0.85406805 ],
 ],
 [
  [               0,               0,               0,   -0.0038734645,    0.0052358323,   -0.0010270807 ],
  [               0,               0,               0,       3.0451713,      -9.6092153,       8.4377873 ],
  [               0,               0,               0,      -15.280069,        16.84562,      -6.2221096 ],
  [               0,               0,               0,      -6.6651048,       9.3309179,      -5.8771923 ],
  [               0,               0,               0,       2.1493629,      -0.5884603,      -1.0844109 ],
  [               0,               0,               0,    0.0025020556,   -0.0031328188,    0.0017731057 ],
  [               0,               0,               0,       2.0649347,      -1.9040562,      0.88377894 ],
  [               0,               0,               0,      -6.3256972,       6.2950732,      -2.6864121 ],
 ],
 [
  [               0,               0,               0,               0,      0.13125215,    -0.072484382 ],
  [               0,               0,               0,               0,      -21.001586,       17.240141 ],
  [               0,               0,               0,               0,      -1.4740584,      0.67871148 ],
  [               0,               0,               0,               0,       13.576061,      -10.796109 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.015447574,    -0.023111598,     0.010050971 ],
  [               0,               0,               0,      -2.0543507,       2.3479662,       1.9657481 ],
  [               0,               0,               0,       1.6935096,      -3.7982607,      0.98173775 ],
  [               0,               0,               0,       3.3766244,      -6.9913065,       2.2906862 ],
  [               0,               0,               0,    -0.011983789,      0.27541673,     -0.36623958 ],
  [               0,               0,               0,   0.00023270476,  -0.00090180634,   0.00057880168 ],
  [               0,               0,               0,       -1.719083,       3.0862748,      0.53812295 ],
  [               0,               0,               0,       20.507883,      -46.995219,       31.653642 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.013359341,    -0.058591299,    -0.027881282,      0.20923185 ],
  [               0,               0,       1611.3573,      -2309.0835,      -3199.2868,       4587.8749 ],
  [               0,               0,       23.921061,       77.683942,      -571.88069,       588.17148 ],
  [               0,               0,       267.35236,      -310.28575,       -918.2714,       1162.1739 ],
  [               0,               0,       2790.8479,      -3973.6749,       -5675.924,       8079.8189 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer1_etaoffv4_37_e = [
 [
  [               0,               0,               0,               0,     0.066422477,    -0.093526307 ],
  [               0,               0,               0,               0,      -90.701827,        111.3671 ],
  [               0,               0,               0,               0,      -87.473397,       128.69454 ],
  [               0,               0,               0,               0,      -99.075065,       142.54464 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.020025024,    -0.013042074 ],
  [               0,               0,               0,               0,       110.43687,      -158.98872 ],
  [               0,               0,               0,               0,       41.983827,      -63.235478 ],
  [               0,               0,               0,               0,      -37.269154,       53.125941 ],
  [               0,               0,               0,               0,       0.1935365,     -0.30677611 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.034160577,     0.051505214 ],
  [               0,               0,               0,               0,       257.17001,      -345.95928 ],
  [               0,               0,               0,               0,      -1347.5334,       2004.3611 ],
  [               0,               0,               0,               0,       1582.1495,      -2351.1493 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0027481992,  -0.00020191606,    0.0089755701,     0.034301417 ],
  [               0,               0,       -2.704092,       13.301719,      -21.863936,       12.652169 ],
  [               0,               0,      -1.9057029,         9.49033,      -15.632014,       7.9308145 ],
  [               0,               0,      0.40123996,      -1.3216827,       1.1388481,     -0.67477484 ],
  [               0,               0,     -0.30309672,       1.6107079,      -2.8471187,       1.6727478 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0040317291,    0.0091522554 ],
  [               0,               0,               0,               0,       7.6403511,      -11.475565 ],
  [               0,               0,               0,               0,      0.46807056,      -1.8603958 ],
  [               0,               0,               0,               0,     -0.41647552,     -0.22121004 ],
  [               0,               0,               0,               0,     0.038003263,    -0.087011341 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0067622114,    -0.011685562,    -0.030319464,     0.056485226 ],
  [               0,               0,      -13.497735,       94.367399,      -221.73809,         176.641 ],
  [               0,               0,      -1.8438556,       8.8191938,      -11.914437,       2.1087215 ],
  [               0,               0,      -21.842872,       137.28093,      -286.49351,       197.37122 ],
  [               0,               0,       4.9017315,       -31.64356,       68.029122,      -48.709698 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00022984668,    0.0037108332 ],
  [               0,               0,               0,               0,      -204.59046,        472.4957 ],
  [               0,               0,               0,               0,       70.153329,      -162.61743 ],
  [               0,               0,               0,               0,     -0.69225858,       1.9435394 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00062606593,   0.00068853852 ],
  [               0,               0,               0,               0,       7.5280732,      -14.384478 ],
  [               0,               0,               0,               0,      -4.8106936,       10.176615 ],
  [               0,               0,               0,               0,      -2.2614759,       4.2343158 ],
  [               0,               0,               0,               0,     -0.19026585,      0.47793025 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0090995946 ],
  [               0,               0,               0,               0,               0,       5.2335262 ],
  [               0,               0,               0,               0,               0,      -1.4399008 ],
  [               0,               0,               0,               0,               0,     0.045819057 ],
  [               0,               0,               0,               0,               0,  -0.00025601246 ],
  [               0,               0,               0,               0,               0,        3.005257 ],
  [               0,               0,               0,               0,               0,      -3.4080168 ],
 ],
 [
  [               0,               0,    0.0058343102,    0.0051534344,    -0.010451119,    -0.083353922 ],
  [               0,               0,      -161.03951,      -132.27158,       333.19226,        2467.177 ],
  [               0,               0,      -0.4656767,      -1.1122788,      -2.8822356,      -7.4822682 ],
  [               0,               0,      -3.4651732,      0.48798351,       11.242478,       20.711518 ],
  [               0,               0,     0.013241552,     0.032618022,     0.074801093,     -0.12605878 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer1_etaoffv4_37_b = [
 [
  [               0,               0,       80736.879,      -674.36376,      0.99322331,   0.00064799575 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0007332973,   0.00068696456 ],
  [               0,               0,               0,               0,      -1.4584713,       4.1421376 ],
  [               0,               0,               0,               0,      0.14502351,     -0.95064919 ],
  [               0,               0,               0,               0,     -0.85831111,     -0.92672425 ],
  [               0,               0,               0,               0,     -0.34135984,    -0.011949796 ],
  [               0,               0,               0,               0,  -0.00025054806,   6.0829753e-05 ],
  [               0,               0,               0,               0,      -4.9418916,      0.65933439 ],
  [               0,               0,               0,               0,      -5.3968714,      -1.3587135 ],
 ],
 [
  [               0,               0,               0,  -0.00069436458,    0.0021263716,   3.0382063e-05 ],
  [               0,               0,               0,      0.54481613,     0.080144026,       3.4302838 ],
  [               0,               0,               0,      -1.9060353,      0.99421127,      -0.9307253 ],
  [               0,               0,               0,        7.848849,      -10.483296,       2.1178699 ],
  [               0,               0,               0,      0.83276126,      -1.0855492,      0.20378458 ],
  [               0,               0,               0,    0.0017493508,   -0.0018326667,   0.00050399219 ],
  [               0,               0,               0,      -18.652194,       22.182434,      -4.1949365 ],
  [               0,               0,               0,       26.373379,       -37.77938,       9.1877651 ],
 ],
 [
  [               0,               0,               0,               0,     0.022002923,    0.0059854239 ],
  [               0,               0,               0,               0,       3.3855596,      -2.1192207 ],
  [               0,               0,               0,               0,      -26.104048,       20.238771 ],
  [               0,               0,               0,               0,       19.741318,      -15.700672 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0099716797,    0.0063427238,     0.044908957 ],
  [               0,               0,               0,      -1.4951779,       3.8189355,      -1.7878631 ],
  [               0,               0,               0,       1.3747882,      -3.4816696,       1.6186833 ],
  [               0,               0,               0,       1.4566883,      -3.7027979,       1.7453873 ],
  [               0,               0,               0,   -0.0011739422,   -0.0062552522,    0.0013511182 ],
  [               0,               0,               0,   0.00068408844,   -0.0018859159,    0.0012820223 ],
  [               0,               0,               0,       12.591775,      -19.664642,       9.5749638 ],
  [               0,               0,               0,       5.0225747,      -10.610134,       9.7084901 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.063483176,     -0.25651414,     -0.24915127,      0.70646144 ],
  [               0,               0,      -1225.5961,       1743.8778,       2564.1402,      -3644.1161 ],
  [               0,               0,      -407.59141,       739.46737,       269.98174,      -706.41673 ],
  [               0,               0,       7.8505737,       106.79579,      -610.60546,       627.45665 ],
  [               0,               0,         3107.05,      -4432.3923,      -6309.5825,       8999.1077 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer1_etaoffv4_37_e = [
 [
  [               0,               0,               0,               0,      0.18893627,     -0.26704629 ],
  [               0,               0,               0,               0,      -88.644339,       126.50124 ],
  [               0,               0,               0,               0,       93.327703,      -133.18077 ],
  [               0,               0,               0,               0,      -120.97721,       174.42904 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.070439061,      0.10397524 ],
  [               0,               0,               0,               0,      -12.986899,       34.203184 ],
  [               0,               0,               0,               0,      -759.36419,       1126.5446 ],
  [               0,               0,               0,               0,       593.52293,      -861.40205 ],
  [               0,               0,               0,               0,      0.28149187,     -0.44675554 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.31600207,     -0.44888905 ],
  [               0,               0,               0,               0,      -121.06324,       182.80155 ],
  [               0,               0,               0,               0,       69.497517,       -104.5747 ],
  [               0,               0,               0,               0,      -45.970929,       68.480174 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00058838236,  -0.00052450456,   0.00051304281,    0.0049535272 ],
  [               0,               0,       4.7055002,      -5.1330033,      -15.376386,       20.552262 ],
  [               0,               0,       1.9384025,      -2.4707084,      -7.5446294,       9.4042277 ],
  [               0,               0,        5.466625,      -8.7385528,      -14.926954,       22.886289 ],
  [               0,               0,      0.11147429,    -0.080610701,      -0.6919928,      0.88716111 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0016183551,   -0.0016388542 ],
  [               0,               0,               0,               0,      -3.5735902,       10.099982 ],
  [               0,               0,               0,               0,      -1.5291461,       1.9355064 ],
  [               0,               0,               0,               0,     -0.32342861,     -0.40495819 ],
  [               0,               0,               0,               0,      0.21172622,     -0.41730946 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.012046407,    -0.029600122,    -0.058059658,      0.14519964 ],
  [               0,               0,       6.1615139,      -23.508996,       23.020425,      0.51118964 ],
  [               0,               0,       25.744522,      -156.01829,       314.20377,      -211.41284 ],
  [               0,               0,      -18.215324,        121.7012,      -269.31047,       196.41032 ],
  [               0,               0,       10.297445,      -67.216866,       146.19762,      -105.93201 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.017794896 ],
  [               0,               0,               0,               0,               0,      0.86355033 ],
  [               0,               0,               0,               0,               0,     -0.79834687 ],
  [               0,               0,               0,               0,               0,     0.087405061 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00036626959,      0.00154238 ],
  [               0,               0,               0,               0,      -2.1019388,       8.2235831 ],
  [               0,               0,               0,               0,     -0.20475867,     -0.73605389 ],
  [               0,               0,               0,               0,       2.3410801,      -6.5461809 ],
  [               0,               0,               0,               0,      0.22354789,     -0.47461914 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.051704102,     -0.11754801 ],
  [               0,               0,               0,               0,      -80.359543,       202.57244 ],
  [               0,               0,               0,               0,       1.9150011,      -6.2263971 ],
  [               0,               0,               0,               0,     -0.27322787,      0.77754202 ],
  [               0,               0,               0,               0,    -0.023468408,     0.057645648 ],
  [               0,               0,               0,               0,       70.931007,      -171.15748 ],
  [               0,               0,               0,               0,       114.24352,      -278.80307 ],
 ],
 [
  [               0,               0,               0,               0,     0.014083466,    -0.020830907 ],
  [               0,               0,               0,               0,      -2991.2261,       7444.3474 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,               0,      0.47247112,      -2.0458886 ],
  [               0,               0,               0,               0,       10.717717,      -25.871757 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer1_etaoffv4_37_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00087587598,    0.0029385462 ],
  [               0,               0,               0,               0,     -0.26041614,       3.0702522 ],
  [               0,               0,               0,               0,       0.7830766,      -1.5220504 ],
  [               0,               0,               0,               0,     -0.70613459,      -1.3380183 ],
  [               0,               0,               0,               0,     -0.31454744,     0.026511873 ],
  [               0,               0,               0,               0,   0.00035912936,  -0.00043682638 ],
  [               0,               0,               0,               0,       0.0879107,       1.4675737 ],
  [               0,               0,               0,               0,      -1.3227188,      -1.3911709 ],
 ],
 [
  [               0,               0,               0,   -0.0069858197,    0.0082803686,  -0.00089658428 ],
  [               0,               0,               0,       2.9970831,      -5.3116283,       5.1304018 ],
  [               0,               0,               0,       4.4689196,       -7.893696,       2.4358879 ],
  [               0,               0,               0,       10.624055,      -15.539656,       4.1597738 ],
  [               0,               0,               0,     -0.88984871,      0.68279324,      -0.3887755 ],
  [               0,               0,               0,    -0.011365845,     0.015581575,   -0.0056198314 ],
  [               0,               0,               0,      -6.9320264,       7.9193743,       -1.646486 ],
  [               0,               0,               0,       -15.23661,       12.216375,      -4.2166714 ],
 ],
 [
  [               0,               0,               0,               0,     0.024197859,     0.011011914 ],
  [               0,               0,               0,               0,       12.560429,      -9.4398217 ],
  [               0,               0,               0,               0,      -27.820922,       21.621896 ],
  [               0,               0,               0,               0,       15.268354,      -12.144026 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0024716234,     0.013025952,   -0.0051948543 ],
  [               0,               0,               0,     -0.98078256,       1.1902791,      0.77787035 ],
  [               0,               0,               0,      0.99288239,      -1.6116604,     -0.14312843 ],
  [               0,               0,               0,       1.2654547,      -2.0822045,    -0.015539024 ],
  [               0,               0,               0,   -0.0045151877,     0.050599001,    -0.074809883 ],
  [               0,               0,               0,   0.00082558679,   -0.0016728463,    0.0009073363 ],
  [               0,               0,               0,      -3.1917694,       5.3764358,      0.34547672 ],
  [               0,               0,               0,      -1.6113886,       3.1803353,      -3.5456614 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.049769327,     -0.16706589,      -0.1709074,      0.45154149 ],
  [               0,               0,      -192.48046,       268.90188,       465.75251,       -648.5715 ],
  [               0,               0,      -601.85944,       986.22114,       689.11528,      -1242.6612 ],
  [               0,               0,      -34.795724,       127.70521,      -382.59945,       383.45041 ],
  [               0,               0,       3019.3631,      -4301.6159,      -6141.4531,         8747.69 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer1_etaoffv4_37_e = [
 [
  [               0,               0,               0,               0,      -0.2146522,      0.31045213 ],
  [               0,               0,               0,               0,      -1184.9782,       1682.8885 ],
  [               0,               0,               0,               0,      -116.24686,       168.07905 ],
  [               0,               0,               0,               0,       110.26628,      -157.46468 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.037039341,    -0.052995664 ],
  [               0,               0,               0,               0,       13.626659,       7.9591031 ],
  [               0,               0,               0,               0,      -694.12579,       1020.9185 ],
  [               0,               0,               0,               0,       545.89578,      -788.81644 ],
  [               0,               0,               0,               0,      0.40517235,    0.0081076725 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.45096625,      0.68164706 ],
  [               0,               0,               0,               0,      -39.235193,       62.373599 ],
  [               0,               0,               0,               0,       23.443947,      -36.203967 ],
  [               0,               0,               0,               0,      -10.094441,       15.198409 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.011141945,    -0.015205643,    -0.029752985,     0.042113199 ],
  [               0,               0,      -10.845615,       44.137758,      -62.106101,       33.465409 ],
  [               0,               0,        5.241302,      -1.8490212,      -38.647791,        44.08796 ],
  [               0,               0,       2.0282931,        4.743984,        -35.1537,       35.077946 ],
  [               0,               0,     -0.92856945,       2.9056176,      -2.2266777,     -0.01142243 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0057292444,   -0.0082159112 ],
  [               0,               0,               0,               0,      -4.0438462,       10.171167 ],
  [               0,               0,               0,               0,      0.55590054,      -2.2783902 ],
  [               0,               0,               0,               0,      0.30725122,      -1.7523566 ],
  [               0,               0,               0,               0,     -0.13084705,      0.25480001 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00045464188,  -0.00024399905,    0.0010799818,    0.0063298944 ],
  [               0,               0,      0.38377652,      -0.1792667,      -1.4439417,       3.0505733 ],
  [               0,               0,       2.0515916,      -4.4710564,      -8.8570195,       18.058602 ],
  [               0,               0,       1.1147957,      -2.0124177,      -6.0175957,       10.027916 ],
  [               0,               0,     -0.63212197,       1.2666738,       3.3274294,      -6.6825413 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0039422243 ],
  [               0,               0,               0,               0,               0,       2.0232001 ],
  [               0,               0,               0,               0,               0,      -1.5464229 ],
  [               0,               0,               0,               0,               0,      0.43971861 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00056231177,    0.0020765481 ],
  [               0,               0,               0,               0,      -4.8577537,       14.144775 ],
  [               0,               0,               0,               0,       1.8450186,      -5.5606317 ],
  [               0,               0,               0,               0,       2.6990507,      -7.4866646 ],
  [               0,               0,               0,               0,      0.47749289,      -1.0890441 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.038475306,     0.093880449 ],
  [               0,               0,               0,               0,      -168.31139,       406.06957 ],
  [               0,               0,               0,               0,       119.34505,      -287.46847 ],
  [               0,               0,               0,               0,       283.46281,      -681.10767 ],
  [               0,               0,               0,               0,     -0.12541825,     -0.50925895 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0059208628,     0.061409764,     -0.10550782 ],
  [               0,               0,               0,      -54.740229,       212.97391,      -187.21379 ],
  [               0,               0,               0,       160.39475,      -784.49626,       957.58794 ],
  [               0,               0,               0,      -134.37485,       658.52774,      -806.64035 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       2.2904723,      -5.6706813 ],
  [               0,               0,               0,               0,      -3399.9276,       8455.5283 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,               0,      -398.15725,       985.50428 ],
  [               0,               0,               0,               0,       7.3271302,      -18.248182 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer1_etaoffv4_37_b = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0003465666,    0.0007100539 ],
  [               0,               0,               0,               0,      -2.8212581,       5.8173963 ],
  [               0,               0,               0,               0,      0.34094339,      -1.0943692 ],
  [               0,               0,               0,               0,     -0.38766174,      -1.1381934 ],
  [               0,               0,               0,               0,     -0.27899135,    -0.030261982 ],
  [               0,               0,               0,               0,   4.8844853e-06,   1.3086901e-05 ],
  [               0,               0,               0,               0,      -97.081231,       1.7951167 ],
  [               0,               0,               0,               0,    -0.099633412,     -0.83849051 ],
 ],
 [
  [               0,               0,               0,               0,     -0.00162983,  -0.00048306544 ],
  [               0,               0,               0,               0,      0.46307988,      -2.8791361 ],
  [               0,               0,               0,               0,      0.39193898,      0.70337624 ],
  [               0,               0,               0,               0,      0.89462683,        0.620881 ],
  [               0,               0,               0,               0,      0.16625096,     0.016865149 ],
  [               0,               0,               0,               0,  -0.00018752999,   0.00017944834 ],
  [               0,               0,               0,               0,        1.234276,       1.8265235 ],
  [               0,               0,               0,               0,      -1.3570603,      -1.3119147 ],
 ],
 [
  [               0,               0,               0,               0,     0.022114775,    0.0076988323 ],
  [               0,               0,               0,               0,       34.728855,      -27.068087 ],
  [               0,               0,               0,               0,      -45.261406,       35.491132 ],
  [               0,               0,               0,               0,       16.452956,      -13.087449 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0035101078,    0.0051219876,    0.0011897931 ],
  [               0,               0,               0,     -0.59221541,      0.55739658,       1.1340343 ],
  [               0,               0,               0,      0.53255805,     -0.75329485,     -0.61215375 ],
  [               0,               0,               0,      0.73951448,      -1.1226834,     -0.54241471 ],
  [               0,               0,               0,    0.0048952303,     0.013453907,    -0.050791166 ],
  [               0,               0,               0,   0.00081565561,   -0.0017561097,   0.00098084056 ],
  [               0,               0,               0,      -6.4259057,       12.352053,      -2.8406662 ],
  [               0,               0,               0,     -0.60153889,       1.1097147,      -2.5701264 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.18829711,     -0.46464484,     -0.53042945,       1.1719609 ],
  [               0,               0,      -2203.3885,       3108.6393,       4575.5908,      -6452.1559 ],
  [               0,               0,       -1758.266,       2711.4301,       2856.8919,      -4488.2745 ],
  [               0,               0,       177.38169,      -61.160808,      -1217.6752,       1342.6985 ],
  [               0,               0,       4051.5466,      -5772.4654,      -8245.0116,       11744.834 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer1_etaoffv4_37_e = [
 [
  [               0,               0,               0,     0.028880869,   0.00035922442,    -0.058624292 ],
  [               0,               0,               0,      -59.947929,      -8.5346109,       99.199248 ],
  [               0,               0,               0,      -36.837458,       1.0520537,       77.685052 ],
  [               0,               0,               0,        -36.6778,      0.16496245,       76.566845 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.39743887,      0.59121584 ],
  [               0,               0,               0,               0,      -354.33538,        522.4064 ],
  [               0,               0,               0,               0,       81.992257,      -120.89026 ],
  [               0,               0,               0,               0,      -162.47471,       234.70853 ],
  [               0,               0,               0,               0,      0.12111468,     -0.13682782 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.18809445,       0.2855723 ],
  [               0,               0,               0,               0,      -285.38114,       431.13478 ],
  [               0,               0,               0,               0,      -38.640093,          55.954 ],
  [               0,               0,               0,               0,         63.8468,      -94.598786 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0010894807,  -0.00062035237,    0.0013392846,    0.0074997032 ],
  [               0,               0,       1.3117349,       1.9342265,      -14.315514,       14.313365 ],
  [               0,               0,        1.359982,     -0.27586614,      -10.136279,       10.203616 ],
  [               0,               0,     -0.54374915,       1.3658408,     -0.44691889,      -1.5901159 ],
  [               0,               0,     -0.33067262,      0.73078513,      0.16396652,     -0.76290775 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0052595613,   -0.0077085354 ],
  [               0,               0,               0,               0,      -4.8173304,       12.039185 ],
  [               0,               0,               0,               0,     -0.12286532,      -1.0456703 ],
  [               0,               0,               0,               0,      0.12401095,      -1.3768218 ],
  [               0,               0,               0,               0,     -0.18755309,      0.37459952 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   0.00056455718,   0.00076535719,  -6.8265028e-05,   -0.0050207915 ],
  [               0,               0,      0.36506221,       -1.866265,      -1.4601635,       10.705603 ],
  [               0,               0,       1.1898464,      -3.0548313,      -3.1919645,       7.8904688 ],
  [               0,               0,       1.5046611,      -3.1082022,      -7.5029324,       14.343992 ],
  [               0,               0,      0.10734578,     -0.54678491,      0.79928002,     -0.23626707 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   0.00031109099,   0.00047268323,    0.0010279984 ],
  [               0,               0,               0,      -39.111491,     -0.56073242,       210.31958 ],
  [               0,               0,               0,       17.166922,      -0.2405355,      -91.743301 ],
  [               0,               0,               0,      0.77513537,      0.11130638,      -3.8707121 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0045944746,     0.014353436 ],
  [               0,               0,               0,               0,     -0.48500041,       3.9909141 ],
  [               0,               0,               0,               0,    -0.084768367,      -1.1200853 ],
  [               0,               0,               0,               0,       1.6239865,      -4.9777088 ],
  [               0,               0,               0,               0,      0.54095813,      -1.2226457 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0001769901,   0.00088067453 ],
  [               0,               0,               0,               0,       5.7747022,       26.071543 ],
  [               0,               0,               0,               0,      -1.4509223,       2.3355822 ],
  [               0,               0,               0,               0,       1.9511125,      -8.2430584 ],
  [               0,               0,               0,               0,     -0.46180848,      -1.4077926 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.034250878,      0.12710334,    -0.097494094 ],
  [               0,               0,               0,      -263.26822,       1292.9688,      -1579.9779 ],
  [               0,               0,               0,       189.95283,      -930.32613,       1137.4131 ],
  [               0,               0,               0,      -150.66989,       738.29559,      -904.25086 ],
  [               0,               0,               0,    0.0092068459,    -0.017046827,     -0.01335116 ],
  [               0,               0,               0,       22.980751,      -133.57395,        186.7775 ],
  [               0,               0,               0,       11.998918,       17.959042,      -117.65638 ],
 ],
 [
  [               0,               0,               0,               0,     0.029125901,    -0.056895336 ],
  [               0,               0,               0,               0,      -3895.5603,       9688.7926 ],
  [               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,               0,      -148.51394,       366.60134 ],
  [               0,               0,               0,               0,        2.507429,      -5.9432176 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]


gam_layer1_etaoffv4_37_b = [
    gam5_layer1_etaoffv4_37_b,
    gam10_layer1_etaoffv4_37_b,
    gam25_layer1_etaoffv4_37_b,
    gam50_layer1_etaoffv4_37_b,
    gam75_layer1_etaoffv4_37_b,
    gam100_layer1_etaoffv4_37_b,
    gam200_layer1_etaoffv4_37_b,
    gam500_layer1_etaoffv4_37_b,
    gam1000_layer1_etaoffv4_37_b,
    ]

gam_layer1_etaoffv4_37_e = [
    gam5_layer1_etaoffv4_37_e,
    gam10_layer1_etaoffv4_37_e,
    gam25_layer1_etaoffv4_37_e,
    gam50_layer1_etaoffv4_37_e,
    gam75_layer1_etaoffv4_37_e,
    gam100_layer1_etaoffv4_37_e,
    gam200_layer1_etaoffv4_37_e,
    gam500_layer1_etaoffv4_37_e,
    gam1000_layer1_etaoffv4_37_e,
    ]



###############################################################################
# Tables for v4 sshape correction, layer 2, 5x5 electron clusters
#

ele5_layer2_etaoffv4_55_b = [
 [
  [               0,               0,               0,        32728.13,      -22543.504,       4134.0753 ],
  [               0,               0,               0,       0.0331433,     0.054574156,    0.0075082149 ],
  [               0,               0,               0,     -0.03305254,    -0.054580923,   -0.0075077266 ],
  [               0,               0,               0,   2.2801847e-05,  -4.6665531e-06,   1.5323096e-07 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele5_layer2_etaoffv4_55_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer2_etaoffv4_55_b = [
 [
  [               0,               0,               0,     0.018905235,   -0.0085785128,    0.0027521988 ],
  [               0,               0,               0,       22.263292,      -7.8608823,       4.1418133 ],
  [               0,               0,               0,        54.95979,      -23.619685,       1.1349939 ],
  [               0,               0,               0,      -27.097116,       11.935862,      -1.1926088 ],
  [               0,               0,               0,     0.006171815,   -0.0034294231,    0.0021541214 ],
  [               0,               0,               0,      -2.0221914,      0.72082634,      0.98115941 ],
  [               0,               0,               0,       32.161705,      -13.233199,     -0.18566721 ],
 ],
 [
  [               0,               0,               0,    0.0041626773,   -0.0062683374,    0.0024483995 ],
  [               0,               0,               0,        691.7636,      -846.60953,       256.23291 ],
  [               0,               0,               0,       469.13713,      -424.89755,       89.565238 ],
  [               0,               0,               0,      -93.044805,       92.610627,      -19.587646 ],
  [               0,               0,               0,   -0.0047932324,    0.0054508833,   0.00069076016 ],
  [               0,               0,               0,      -1.8324746,       2.0615225,      0.46017109 ],
  [               0,               0,               0,       4.0185777,      -1.0571575,      -2.2676988 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0037905577 ],
  [               0,               0,               0,               0,               0,       3.1695195 ],
  [               0,               0,               0,               0,               0,      -1.3320928 ],
  [               0,               0,               0,               0,               0,    0.0046315754 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.011966383,    -0.027018109,     0.019250646 ],
  [               0,               0,               0,      -5.1244054,        9.419187,      -1.2212901 ],
  [               0,               0,               0,      0.93252758,      -1.9366118,     -0.39543127 ],
  [               0,               0,               0,     -0.99835809,       2.9817608,      -1.6082679 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.01706983,     0.023678473 ],
  [               0,               0,               0,               0,       8.4006365,       28.171519 ],
  [               0,               0,               0,               0,       374.72342,      -511.73348 ],
  [               0,               0,               0,               0,       170.62868,      -224.73523 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.03299456,    -0.041141868 ],
  [               0,               0,               0,               0,       507.60359,      -711.99655 ],
  [               0,               0,               0,               0,       -240.6339,       335.74732 ],
  [               0,               0,               0,               0,     -0.40360193,      0.84537815 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer2_etaoffv4_55_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer2_etaoffv4_55_b = [
 [
  [               0,               0,               0,    -0.042609467,     0.020902281,    0.0070357814 ],
  [               0,               0,               0,       12.794897,      -6.8574872,       3.6460166 ],
  [               0,               0,               0,      -1.3770269,      0.48322511,      -1.2793287 ],
  [               0,               0,               0,    -0.019236591,      0.16977322,    -0.021555361 ],
  [               0,               0,               0,     0.009867834,   -0.0045906372,   0.00042263554 ],
  [               0,               0,               0,       24.995183,        -14.7186,       2.3646978 ],
  [               0,               0,               0,      -161.91678,       9.8011888,       1.0994348 ],
 ],
 [
  [               0,               0,               0,    0.0056785127,    0.0076434069,    0.0015651107 ],
  [               0,               0,               0,       3.9238295,       -9.859054,       7.2061506 ],
  [               0,               0,               0,       1.5841987,      -1.3877232,      -0.9797001 ],
  [               0,               0,               0,     -0.23117135,      0.56084095,     -0.18827902 ],
  [               0,               0,               0,    0.0019128403,   -0.0030082792,    0.0015331931 ],
  [               0,               0,               0,       1.3291961,      0.12963486,      0.48697237 ],
  [               0,               0,               0,       8.1508794,      -7.9914153,      0.67702712 ],
 ],
 [
  [               0,               0,               0,               0,      0.16119432,     -0.11959093 ],
  [               0,               0,               0,               0,     -0.93416521,       3.1265404 ],
  [               0,               0,               0,               0,       6.2896756,      -6.2870203 ],
  [               0,               0,               0,               0,      -3.7715815,        3.046924 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.028536991,    -0.066699322,     0.046872366 ],
  [               0,               0,               0,      -5.1189428,       11.066513,      -3.4809125 ],
  [               0,               0,               0,        1.327011,      -3.2617419,      0.62030367 ],
  [               0,               0,               0,    -0.069362433,      0.78168999,     -0.54935895 ],
  [               0,               0,               0,    0.0060120425,    -0.011384726,    0.0052584645 ],
  [               0,               0,               0,       8.5479621,      -25.010872,       18.252405 ],
  [               0,               0,               0,       -24.51307,       54.512411,        -33.2547 ],
 ],
 [
  [               0,               0,               0,    -0.010523591,  -0.00077372056,     0.030849262 ],
  [               0,               0,               0,      -400.46633,       1063.7666,      -703.60825 ],
  [               0,               0,               0,      -237.09599,       671.45728,      -476.03733 ],
  [               0,               0,               0,      -32.542446,       74.425339,      -40.649741 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,       6.8487746,      -19.333555,       13.646138 ],
  [               0,               0,               0,      -403.04279,       223.88513,       516.69714 ],
  [               0,               0,               0,      -268.20046,       134.93712,         333.224 ],
  [               0,               0,               0,      -2.7238328,       1.5381747,       4.6976842 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer2_etaoffv4_55_e = [
 [
  [               0,               0,     -0.77259779,       1.1097729,       1.4749924,      -2.1145091 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.63743677,      0.95576076 ],
  [               0,               0,               0,               0,      -55.461841,        82.62672 ],
  [               0,               0,               0,               0,       18.053596,      -27.182628 ],
  [               0,               0,               0,               0,      -9.8824266,       14.408711 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0020704154,    0.0041226582 ],
  [               0,               0,               0,               0,       11.734109,       22.105847 ],
  [               0,               0,               0,               0,       16.320934,      -26.384989 ],
  [               0,               0,               0,               0,       6.6114944,      -10.669237 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     -0.25450628,      0.87113102,     -0.73253859 ],
  [               0,               0,               0,       37.552091,      -125.41218,       106.33873 ],
  [               0,               0,               0,      -9.2309842,       32.249211,      -29.213128 ],
  [               0,               0,               0,      -1.9828426,       6.1997828,      -4.8024285 ],
  [               0,               0,               0,     0.017388993,    -0.059575595,     0.051608079 ],
  [               0,               0,               0,      -4.8602621,       14.305387,      -9.2772949 ],
  [               0,               0,               0,      -30.349731,       90.597062,      -69.972671 ],
 ],
 [
  [               0,               0,               0,      0.02985545,    0.0033309759,     -0.11892103 ],
  [               0,               0,               0,       -10.53283,       44.936346,      -46.914543 ],
  [               0,               0,               0,       8.2385783,      -36.235165,        39.11719 ],
  [               0,               0,               0,      -1.6120498,       7.3269795,       -8.334016 ],
  [               0,               0,               0,   -0.0025922631,    0.0018740054,    0.0088390571 ],
  [               0,               0,               0,        4.034238,      -15.347853,       15.616439 ],
  [               0,               0,               0,       3.9068347,      -10.693872,      -8.1156409 ],
 ],
 [
  [               0,               0,               0,      -0.0211052,    0.0019953596,      0.13655031 ],
  [               0,               0,               0,       37.616084,       -175.4486,       205.68226 ],
  [               0,               0,               0,      -4.9600592,       22.118814,      -25.424799 ],
  [               0,               0,               0,       -10.05201,       48.010303,      -57.308511 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.010232738 ],
  [               0,               0,               0,               0,               0,       1.3261438 ],
  [               0,               0,               0,               0,               0,     -0.98459595 ],
  [               0,               0,               0,               0,               0,      0.11455977 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer2_etaoffv4_55_b = [
 [
  [               0,               0,               0,   -0.0048128147,   -0.0054895025,    0.0098624686 ],
  [               0,               0,               0,     -0.12628185,         2.29373,       2.8232157 ],
  [               0,               0,               0,       1.2175981,     -0.63889401,      -1.2164347 ],
  [               0,               0,               0,        25.82177,       -34.99687,       5.5557022 ],
  [               0,               0,               0,      0.39710266,     -0.37213659,     0.054844301 ],
  [               0,               0,               0,    0.0066279116,     -0.20119619,     0.041470013 ],
  [               0,               0,               0,      -2.5642236,        1.641147,      -3.1719318 ],
 ],
 [
  [               0,               0,               0,               0,    0.0069311046,    0.0019594372 ],
  [               0,               0,               0,               0,      -3.7407705,       5.9658534 ],
  [               0,               0,               0,               0,     -0.45890504,      -1.2589771 ],
  [               0,               0,               0,               0,      0.57477146,     -0.16608334 ],
  [               0,               0,               0,               0,  -0.00099543534,    0.0014039153 ],
  [               0,               0,               0,               0,     -0.35235291,       1.0796581 ],
  [               0,               0,               0,               0,       -3.218932,     -0.23725533 ],
 ],
 [
  [               0,               0,               0,               0,      0.25676927,     -0.19720096 ],
  [               0,               0,               0,               0,      -82.197903,       69.502874 ],
  [               0,               0,               0,               0,       15.211324,      -13.552287 ],
  [               0,               0,               0,               0,      -2.9223764,       2.3665459 ],
  [               0,               0,               0,               0,      0.02056707,    -0.016776605 ],
  [               0,               0,               0,               0,      -8.6187764,       8.5358714 ],
  [               0,               0,               0,               0,       80.721265,      -65.861673 ],
 ],
 [
  [               0,               0,               0,    0.0095423464,    -0.023943489,     0.018907537 ],
  [               0,               0,               0,       1.6952353,      -4.0383814,       5.9490971 ],
  [               0,               0,               0,      -3.3770707,       6.5483654,       -4.588453 ],
  [               0,               0,               0,       1.4944287,      -2.2528894,      0.96417304 ],
  [               0,               0,               0,   -0.0052519983,     0.011932548,   -0.0055588353 ],
  [               0,               0,               0,      -1.0083933,       2.1629797,     -0.16145085 ],
  [               0,               0,               0,      -7.4097262,       15.948726,      -9.9546168 ],
 ],
 [
  [               0,               0,    -0.032650207,    -0.019171158,     0.020736593,      0.10975876 ],
  [               0,               0,       121.27786,      -162.42066,      -253.33984,       341.60357 ],
  [               0,               0,      -91.277154,       131.61741,       162.37198,      -235.81006 ],
  [               0,               0,      -11.214839,       13.714838,       18.158056,      -21.662142 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     -0.20275702,    0.0034542894,      0.44260717 ],
  [               0,               0,               0,      -225.68237,       10.734099,       467.36821 ],
  [               0,               0,               0,       2.3131528,       3.5384625,      -11.654703 ],
  [               0,               0,               0,     -0.26463993,      0.14697007,      0.88819592 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer2_etaoffv4_55_e = [
 [
  [               0,               0,      -1.9783485,       2.8287292,        3.912411,      -5.5901657 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,       10.595563,      -47.265156,       70.274364,      -34.824493 ],
  [               0,               0,       180.08113,      -91.284007,      -149.99215,      -137.35092 ],
  [               0,               0,       12.089506,      -3.9436303,      -12.415073,      -16.239447 ],
  [               0,               0,       25.138977,      -16.453677,      -22.131053,      -14.658997 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.037164226,      0.05676924 ],
  [               0,               0,               0,               0,       1706.6795,      -2562.6862 ],
  [               0,               0,               0,               0,      -453.65258,       687.25099 ],
  [               0,               0,               0,               0,       594.92989,      -899.61029 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0022656324,   0.00010228532,     0.014784971 ],
  [               0,               0,               0,       4.7919793,      -14.477346,       13.290323 ],
  [               0,               0,               0,      0.53075654,      -2.4053842,       1.3006323 ],
  [               0,               0,               0,      -1.2168373,       2.9048441,     -0.96380986 ],
  [               0,               0,               0,   -0.0039277146,   -0.0011530231,     0.017286165 ],
  [               0,               0,               0,      -0.2218273,       1.6706878,      -1.9600365 ],
  [               0,               0,               0,       10.961314,      -36.382734,       20.719889 ],
 ],
 [
  [               0,               0,               0,     0.023812271,    -0.084271075,     0.080365673 ],
  [               0,               0,               0,       6.4724724,      -31.503954,       39.905314 ],
  [               0,               0,               0,       1.6159714,      -6.0505383,       4.3989955 ],
  [               0,               0,               0,      -62.262375,       260.72077,      -271.55571 ],
  [               0,               0,               0,     0.013866806,    -0.020018776,    0.0037567143 ],
  [               0,               0,               0,    -0.069138118,      0.25788634,     -0.22864187 ],
  [               0,               0,               0,       23.822328,      -99.802301,       102.43888 ],
 ],
 [
  [               0,               0,               0,    -0.015591163,  -0.00012810986,      0.10131993 ],
  [               0,               0,               0,       47.198276,      -218.38045,       254.19118 ],
  [               0,               0,               0,       1.4560071,      -8.1967606,       10.203119 ],
  [               0,               0,               0,      -9.9501766,       47.612487,      -56.928194 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       1.9124837,       -4.659969 ],
  [               0,               0,               0,               0,      -36.688016,       91.651236 ],
  [               0,               0,               0,               0,       34.054686,      -84.759521 ],
  [               0,               0,               0,               0,      -20.225762,       49.853999 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   0.00051425587,   0.00043998076,  -7.2096851e-05,     -0.01025718 ],
  [               0,               0,      -1.1876803,      -1.3856162,      -3.4053852,      -4.4959162 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,       78.317748,       57.379657,      -130.86714,       -1152.194 ],
  [               0,               0,      0.95650372,       6.5419227,       5.5014119,       3.0073275 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer2_etaoffv4_55_b = [
 [
  [               0,               0,               0,     0.046132837,    -0.013019211,    0.0087329121 ],
  [               0,               0,               0,      -12.207144,       3.3867325,       3.3636863 ],
  [               0,               0,               0,       1.0712887,     -0.35022841,      -1.3060303 ],
  [               0,               0,               0,       7.6293334,      -5.2934196,      0.94284063 ],
  [               0,               0,               0,     0.068318602,    -0.046871904,     0.008151655 ],
  [               0,               0,               0,       10.886735,      -2.1763741,      0.18074237 ],
  [               0,               0,               0,      -24.734982,       7.4037211,      -3.2507759 ],
 ],
 [
  [               0,               0,               0,   -0.0029036888,    0.0043479655,    0.0068144399 ],
  [               0,               0,               0,      -3.2532979,       2.5657013,       2.9495765 ],
  [               0,               0,               0,      -2.1553605,       2.2910139,      -1.9762193 ],
  [               0,               0,               0,       95.037066,      -72.440778,       14.455529 ],
  [               0,               0,               0,      0.79974048,     -0.61017627,      0.12135679 ],
  [               0,               0,               0,       1.0763107,      -1.5158697,      0.57173956 ],
  [               0,               0,               0,      0.72719287,      -1.1060686,      -2.7057844 ],
 ],
 [
  [               0,               0,               0,               0,      0.19642458,     -0.15117956 ],
  [               0,               0,               0,               0,      -34.356373,       31.383254 ],
  [               0,               0,               0,               0,        14.35326,      -13.149676 ],
  [               0,               0,               0,               0,      -2.0090724,        1.701671 ],
  [               0,               0,               0,               0,   -0.0017360384,      0.00261575 ],
  [               0,               0,               0,               0,      -34.147955,       28.232874 ],
  [               0,               0,               0,               0,      -109.41583,       84.780001 ],
 ],
 [
  [               0,               0,               0,   -0.0026634116,    0.0076478407,    0.0033246181 ],
  [               0,               0,               0,       1.9291933,      -5.6440966,       6.7018333 ],
  [               0,               0,               0,     -0.49949991,      0.72154472,      -1.6122805 ],
  [               0,               0,               0,     -0.27067596,       0.3731905,      0.58729636 ],
  [               0,               0,               0,   -0.0021450917,   0.00067759245,    0.0061815087 ],
  [               0,               0,               0,      0.94498239,      -1.5631639,      0.73206973 ],
  [               0,               0,               0,       1.0715246,      -2.6570849,      -1.4371535 ],
 ],
 [
  [               0,               0,               0,   -0.0082581753,   0.00011109436,     0.027961267 ],
  [               0,               0,               0,      -83.020153,       204.04804,      -121.59092 ],
  [               0,               0,               0,      -121.05099,       344.52713,       -246.0618 ],
  [               0,               0,               0,      -24.607734,       56.529555,      -31.081425 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      0.23370118,     -0.01135024,      -0.4416912 ],
  [               0,               0,               0,      -76.235386,      -10.622252,       204.29003 ],
  [               0,               0,               0,      -2843.3119,       8028.5705,       -5671.789 ],
  [               0,               0,               0,      -1.8498826,       2.9482721,      0.48484681 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer2_etaoffv4_55_e = [
 [
  [               0,               0,       1.4199871,      -1.9791468,      -3.0713498,       4.2871743 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   0.00015732412,  -0.00095084944,  -0.00095875677,    0.0030082782 ],
  [               0,               0,       6.5171933,       6.8039593,       2.9270284,      -1.7820522 ],
  [               0,               0,       99.920302,       47.412719,      -73.932223,      -316.35392 ],
  [               0,               0,       37.309035,       16.390527,      -31.672707,      -127.71129 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.075038435,      0.11764866 ],
  [               0,               0,               0,               0,       -241.5679,       368.75853 ],
  [               0,               0,               0,               0,       70.946559,      -108.14436 ],
  [               0,               0,               0,               0,      -13.460991,       20.410863 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0081355025,    -0.030456412,     0.035870861 ],
  [               0,               0,               0,       2.6702256,      -7.8154273,       8.3323595 ],
  [               0,               0,               0,      -7.3871886,       24.856269,      -22.085529 ],
  [               0,               0,               0,       4.2649031,      -14.234615,       11.858785 ],
  [               0,               0,               0,     0.021938133,    -0.073910958,     0.062429602 ],
  [               0,               0,               0,       10.635057,      -37.628229,       33.936474 ],
  [               0,               0,               0,      -38.748597,       132.69146,      -121.63442 ],
 ],
 [
  [               0,               0,               0,   0.00065082281,   0.00073897616,    0.0065083381 ],
  [               0,               0,               0,      -1.4038973,       4.5100769,       -1.179814 ],
  [               0,               0,               0,      0.47137803,      -1.6781528,      0.31711184 ],
  [               0,               0,               0,      -1.3365144,       3.1347476,     -0.18335353 ],
  [               0,               0,               0,   -0.0063597909,    0.0011873098,     0.030839371 ],
  [               0,               0,               0,       3.2914643,      -13.062855,        12.91547 ],
  [               0,               0,               0,       4.1854276,      -15.126854,       10.161264 ],
 ],
 [
  [               0,               0,               0,   -0.0027927196,   -0.0048888461,     0.036379118 ],
  [               0,               0,               0,       55.734181,      -263.24148,       312.94821 ],
  [               0,               0,               0,      -21.767531,       103.62486,      -124.47425 ],
  [               0,               0,               0,      -12.617979,       71.335536,      -94.482048 ],
  [               0,               0,               0,     0.011002525,    0.0020890463,    -0.032892025 ],
  [               0,               0,               0,       3.2745937,      -16.341869,       20.384614 ],
  [               0,               0,               0,       16.189819,      -73.684911,       80.693189 ],
 ],
 [
  [               0,               0,               0,               0,    0.0055870857,     0.014879175 ],
  [               0,               0,               0,               0,      -113.87318,         282.243 ],
  [               0,               0,               0,               0,       74.085478,      -183.31568 ],
  [               0,               0,               0,               0,      -37.763308,       93.075121 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   0.00056918487,     0.002129978,    0.0010679213,     -0.02436755 ],
  [               0,               0,     -0.93730921,      -1.3937337,      -3.4533343,      -4.5530625 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,       4.0053803,       26.615307,      -19.459222,      -53.200924 ],
  [               0,               0,      -6.4736976,      0.98106926,      -48.426532,       312.11059 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer2_etaoffv4_55_b = [
 [
  [               0,               0,     0.060863207,    -0.022861504,   0.00057532836,    0.0088899383 ],
  [               0,               0,       20.089287,      -19.842508,       4.8959022,       3.2152975 ],
  [               0,               0,      -8.9151437,       6.6572457,      -1.4664808,        -1.20619 ],
  [               0,               0,       5.1149046,      -3.7322197,       0.8583665,    -0.057809082 ],
  [               0,               0,      0.10142413,    -0.066333579,     0.012055438,  -0.00049556604 ],
  [               0,               0,       743.31724,      -279.55442,         39.8008,     -0.05813348 ],
  [               0,               0,      -871.88397,       271.03902,      -32.217924,      -9.5271751 ],
 ],
 [
  [               0,               0,               0,     0.020246932,     -0.01736466,     0.010443164 ],
  [               0,               0,               0,      -10.777224,       10.347656,       1.3117246 ],
  [               0,               0,               0,       4.6509701,      -5.8876003,      0.39242349 ],
  [               0,               0,               0,      -1.1422784,       1.9382599,     -0.63626063 ],
  [               0,               0,               0,    0.0037002936,   -0.0044779205,    0.0019103338 ],
  [               0,               0,               0,      -3.9070795,       3.1882885,      0.32627794 ],
  [               0,               0,               0,       9.0705125,      -16.031876,       4.0128872 ],
 ],
 [
  [               0,               0,               0,               0,     0.046424237,    -0.032641174 ],
  [               0,               0,               0,               0,       4.6781419,      0.71132629 ],
  [               0,               0,               0,               0,      -14.851843,       10.465961 ],
  [               0,               0,               0,               0,       16.122117,      -12.815172 ],
  [               0,               0,               0,               0,     0.041514883,    -0.031521182 ],
  [               0,               0,               0,               0,      -19.864998,       16.923164 ],
  [               0,               0,               0,               0,      -65.831085,       51.126438 ],
 ],
 [
  [               0,               0,               0,    -0.015320369,     0.030265279,   -0.0055007043 ],
  [               0,               0,               0,       4.0959946,      -9.0230462,       7.7195423 ],
  [               0,               0,               0,     -0.77250583,       1.2010104,      -1.7551565 ],
  [               0,               0,               0,     -0.49874854,      0.38282368,      0.87716097 ],
  [               0,               0,               0,   -0.0068643338,    0.0033123848,    0.0098635592 ],
  [               0,               0,               0,      0.65983275,     -0.90735154,      0.30470094 ],
  [               0,               0,               0,       3.8974205,      -8.5918335,       1.6671064 ],
 ],
 [
  [               0,               0,               0,   -0.0051602972,  -0.00068091897,     0.022240343 ],
  [               0,               0,               0,      -50.764738,       108.38919,      -50.867031 ],
  [               0,               0,               0,      -91.478156,       266.16771,      -194.24409 ],
  [               0,               0,               0,      -20.169807,       42.937892,      -20.793373 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      0.19530785,      0.03297099,     -0.42174602 ],
  [               0,               0,               0,       5.6998082,      -282.61647,       424.51135 ],
  [               0,               0,               0,      -14.738379,     0.051955152,        26.14758 ],
  [               0,               0,               0,      0.19992358,      0.20163828,      0.10939632 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer2_etaoffv4_55_e = [
 [
  [               0,               0,       1.0272128,      -1.4179031,      -2.2557618,       3.1214933 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.031528304,    -0.017014118,     0.023264803,      0.11363519 ],
  [               0,               0,       23.174882,      -27.873133,      -36.676315,        41.34357 ],
  [               0,               0,       54.167366,      -86.525621,      -121.59293,       193.13358 ],
  [               0,               0,       19.797174,      -27.803872,       -41.88663,        58.66593 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0052208936,  -0.00018208823 ],
  [               0,               0,               0,               0,       69.883117,      -103.45866 ],
  [               0,               0,               0,               0,      -8.4517234,       11.669302 ],
  [               0,               0,               0,               0,      -1.7167471,       2.5978467 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0089420722,    0.0015930773,     0.038355482 ],
  [               0,               0,               0,       6.2571763,      -19.763716,       17.858287 ],
  [               0,               0,               0,      -1.3584671,       4.1761199,      -4.2716705 ],
  [               0,               0,               0,      0.71039387,      -2.2396827,       1.7277738 ],
  [               0,               0,               0,   -0.0023155045,    0.0018788277,    0.0049955368 ],
  [               0,               0,               0,       3.2983362,      -10.661932,        9.699242 ],
  [               0,               0,               0,       2.0069231,      -6.6497324,      0.58325005 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0026346057,     0.014955882 ],
  [               0,               0,               0,               0,     -0.77368613,       3.9581937 ],
  [               0,               0,               0,               0,     0.047576672,      -1.2570537 ],
  [               0,               0,               0,               0,      -2.4941046,       4.7479382 ],
  [               0,               0,               0,               0,     0.005267163,   -0.0033602556 ],
  [               0,               0,               0,               0,     0.079720519,     -0.14732995 ],
  [               0,               0,               0,               0,       3.3496067,      -7.9193756 ],
 ],
 [
  [               0,               0,               0,    -0.013080522,     0.055935482,    -0.049296552 ],
  [               0,               0,               0,       38.522936,      -185.05023,       224.34527 ],
  [               0,               0,               0,       3.1979521,      -15.513472,       17.681053 ],
  [               0,               0,               0,        3.050906,       15.619486,      -56.416277 ],
  [               0,               0,               0,     -0.06523715,     0.014427354,      0.35523187 ],
  [               0,               0,               0,       3.4388008,      -15.734383,       18.004062 ],
  [               0,               0,               0,      -14.034068,       62.424245,       -69.32876 ],
 ],
 [
  [               0,               0,               0,               0,     0.016890861,   -0.0029755623 ],
  [               0,               0,               0,               0,      -26.967377,       67.518252 ],
  [               0,               0,               0,               0,        13.92269,      -35.129734 ],
  [               0,               0,               0,               0,      -2.4175854,       6.0237496 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0044848618,     0.003724632,   -0.0091633116,    -0.068360216 ],
  [               0,               0,      -1.1812447,      -1.3879751,      -3.4387663,      -4.5214019 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,      -97.148096,      -78.815658,       199.99921,       1490.9957 ],
  [               0,               0,      0.24290507,       1.3500959,       1.5344315,       2.0780378 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer2_etaoffv4_55_b = [
 [
  [               0,               0,               0,    -0.019357932,    0.0087147278,    0.0070123929 ],
  [               0,               0,               0,       6.6355342,      -3.2407277,       4.2215569 ],
  [               0,               0,               0,      -1.1598424,      0.36991696,      -1.3096668 ],
  [               0,               0,               0,       1.7060687,      -0.5910143,   -0.0043294211 ],
  [               0,               0,               0,   -0.0036709145,     0.001297741,   0.00028152183 ],
  [               0,               0,               0,       9.6509653,      -4.5672714,      0.98137454 ],
  [               0,               0,               0,      -44.836697,       17.245363,      -2.0904754 ],
 ],
 [
  [               0,               0,               0,    -0.012255765,     0.013584843,    0.0051505447 ],
  [               0,               0,               0,      -1.3401438,      0.85024117,        3.413713 ],
  [               0,               0,               0,      -2.2781502,       2.2315781,      -1.8497602 ],
  [               0,               0,               0,       4.5411082,      -2.7203282,      0.27904136 ],
  [               0,               0,               0,     0.020685769,   -0.0038588431,   -0.0028217291 ],
  [               0,               0,               0,      -1.1489068,       2.0337767,     -0.70230769 ],
  [               0,               0,               0,       6.8456116,      -9.6604153,       -5.928665 ],
 ],
 [
  [               0,               0,               0,               0,     0.032371219,     -0.02131419 ],
  [               0,               0,               0,               0,       8.8487425,      -2.6606564 ],
  [               0,               0,               0,               0,       3.1977282,      -3.9007834 ],
  [               0,               0,               0,               0,      -3.7867931,        3.003029 ],
  [               0,               0,               0,               0,    0.0025898385,  -0.00079235435 ],
  [               0,               0,               0,               0,     -0.49867338,        1.442982 ],
  [               0,               0,               0,               0,      -7.9448981,       4.7781295 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00047056625,    0.0091518512 ],
  [               0,               0,               0,               0,     -0.64082589,       3.8179965 ],
  [               0,               0,               0,               0,     -0.40821006,     -0.98130189 ],
  [               0,               0,               0,               0,       2.9263425,       1.3550067 ],
  [               0,               0,               0,               0,     0.020572549,      0.01588847 ],
  [               0,               0,               0,               0,     0.077080744,     -0.02554755 ],
  [               0,               0,               0,               0,    0.0029585185,        -3.11399 ],
 ],
 [
  [               0,               0,    -0.025054828,    -0.025169494,    0.0081840175,      0.11219866 ],
  [               0,               0,       407.64628,       -560.4263,       -773.2318,       1065.2725 ],
  [               0,               0,      -114.02287,       161.50272,       208.19722,      -296.39506 ],
  [               0,               0,      -54.001293,       70.933741,       98.434784,      -128.99558 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.066738517,    0.0087457715,     -0.11443655 ],
  [               0,               0,               0,       6.0043459,       8.2268968,       15.779178 ],
  [               0,               0,               0,      -182.53302,       497.38142,      -340.99906 ],
  [               0,               0,               0,     -0.05559893,      0.61117095,     0.077187263 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer2_etaoffv4_55_e = [
 [
  [               0,               0,      0.31736799,     -0.39974461,     -0.78820494,         1.01671 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0060137688,    -0.013823643,     0.010713908 ],
  [               0,               0,               0,       557.77417,      -1548.8976,       1073.3279 ],
  [               0,               0,               0,       426.92485,      -1303.3425,       992.67222 ],
  [               0,               0,               0,       147.77753,       -422.7091,        301.9177 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.010155291,    -0.012494703 ],
  [               0,               0,               0,               0,       159.73733,      -237.92021 ],
  [               0,               0,               0,               0,      -111.80152,       168.00807 ],
  [               0,               0,               0,               0,       100.94013,      -152.62413 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0027765648,   -0.0012307155,     0.014290229 ],
  [               0,               0,               0,       38.078277,      -124.31457,       105.13503 ],
  [               0,               0,               0,       3.1522492,       -10.58212,       7.6403097 ],
  [               0,               0,               0,     -0.82909869,       2.9294237,       -2.586223 ],
  [               0,               0,               0,     0.002814117,   -0.0052866258,    0.0021760491 ],
  [               0,               0,               0,      -1.2322841,       4.7853892,      -3.6545972 ],
  [               0,               0,               0,       1.1092144,      -3.5557145,      -11.230954 ],
 ],
 [
  [               0,               0,               0,    -0.048356638,      0.19498899,     -0.18532695 ],
  [               0,               0,               0,       6.3169243,      -26.650873,        30.35665 ],
  [               0,               0,               0,     -0.79085726,       3.3530824,      -4.7162818 ],
  [               0,               0,               0,       25.115528,      -59.595433,       17.883431 ],
  [               0,               0,               0,      0.12207751,    -0.019983295,     -0.45833598 ],
  [               0,               0,               0,    -0.065396222,      0.27327282,     -0.27990574 ],
  [               0,               0,               0,      0.61448101,      0.51194994,       -6.790505 ],
 ],
 [
  [               0,               0,               0,    -0.003704666,    0.0016419514,     0.029307153 ],
  [               0,               0,               0,       4.9616199,      -22.903448,        28.40545 ],
  [               0,               0,               0,       6.1920954,      -29.585212,       34.217684 ],
  [               0,               0,               0,      -10.949266,       52.102835,      -61.943719 ],
  [               0,               0,               0,  -0.00095793246,   -0.0016626932,    0.0092090388 ],
  [               0,               0,               0,      -2.0844504,       6.1142715,      -2.2470558 ],
  [               0,               0,               0,        6.145131,       10.672294,      -60.139626 ],
 ],
 [
  [               0,               0,               0,               0,    0.0038059912,    0.0022667459 ],
  [               0,               0,               0,               0,      -56.530585,       141.28702 ],
  [               0,               0,               0,               0,       13.084483,      -33.494201 ],
  [               0,               0,               0,               0,       2.1695637,      -5.1021455 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0036097137,    0.0029354565,    -0.007537366,     -0.05424718 ],
  [               0,               0,      -1.1870648,      -1.3925457,      -3.4503495,      -4.5490221 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,       11.085585,       10.702128,       -16.00623,      -147.26494 ],
  [               0,               0,      0.53509843,       1.3485009,       3.3912803,        8.085858 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer2_etaoffv4_55_b = [
 [
  [               0,               0,               0,    0.0010418149,    0.0078224038,    0.0055952763 ],
  [               0,               0,               0,      0.77514708,      -3.2630595,       4.8613837 ],
  [               0,               0,               0,      -2.1033948,       1.1498416,       -1.397896 ],
  [               0,               0,               0,        1.246732,     -0.56394295,    0.0047672349 ],
  [               0,               0,               0,   0.00091530062,   -0.0023410643,    0.0008914251 ],
  [               0,               0,               0,       4.1705013,      -2.0131312,       1.1396369 ],
  [               0,               0,               0,       4.4476654,       3.0807813,      -1.7645274 ],
 ],
 [
  [               0,               0,               0,      0.01798457,    -0.022890469,     0.013685181 ],
  [               0,               0,               0,      -10.441159,       11.297385,       1.3126334 ],
  [               0,               0,               0,       2.5853604,      -3.4334389,     -0.29527559 ],
  [               0,               0,               0,     -0.65080432,        1.104803,     -0.38587258 ],
  [               0,               0,               0,   -0.0050856538,    0.0069728317,   -0.0016429213 ],
  [               0,               0,               0,     -0.39547697,        0.383966,      -1.0584743 ],
  [               0,               0,               0,      -15.942664,       20.006945,      -23.248054 ],
 ],
 [
  [               0,               0,               0,               0,     0.035983441,    -0.024841084 ],
  [               0,               0,               0,               0,      -13.041253,        15.47029 ],
  [               0,               0,               0,               0,       21.219213,      -18.513192 ],
  [               0,               0,               0,               0,      -12.495153,       10.039206 ],
  [               0,               0,               0,               0,   -0.0054036328,    0.0061026705 ],
  [               0,               0,               0,               0,       2.7102685,      -1.1128956 ],
  [               0,               0,               0,               0,       9.6338316,      -9.4720624 ],
 ],
 [
  [               0,               0,               0,    0.0014912402,    -0.001620743,    0.0078147645 ],
  [               0,               0,               0,       1.2350965,       -3.588796,       5.9095608 ],
  [               0,               0,               0,       1.0228413,       -2.493185,     0.070948249 ],
  [               0,               0,               0,     -0.42967221,       1.3311098,      -0.8166358 ],
  [               0,               0,               0,   0.00065203689,  -0.00073516983,   0.00051130906 ],
  [               0,               0,               0,      -1.4066319,       2.2168738,    -0.015278758 ],
  [               0,               0,               0,         3.29396,      -9.4219271,       3.9582669 ],
 ],
 [
  [               0,               0,   -0.0063183675,    -0.031609784,    -0.014094154,      0.10581809 ],
  [               0,               0,       475.89898,      -657.52618,      -902.63148,       1249.6054 ],
  [               0,               0,      -125.89964,       182.71223,       219.55334,      -321.24857 ],
  [               0,               0,      -29.128717,       37.005288,        49.89505,      -62.800822 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.29396059,     -0.38871052 ],
  [               0,               0,               0,               0,       12.408737,       22.168625 ],
  [               0,               0,               0,               0,      -20.398288,       26.216755 ],
  [               0,               0,               0,               0,      0.35018656,      0.23983729 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer2_etaoffv4_55_e = [
 [
  [     0.054034444,    0.0060553626,    -0.065225796,     -0.16238537,     -0.28380801,      0.58057718 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     -0.10111094,    -0.047537594,     0.078279692,      0.33249007 ],
  [               0,               0,      -82.985841,       130.03136,       193.19766,      -300.29286 ],
  [               0,               0,        187.7606,      -280.50003,      -406.93349,       607.15169 ],
  [               0,               0,       56.724199,      -83.198571,      -122.79806,       180.07831 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.010726285,    -0.015506031 ],
  [               0,               0,               0,               0,      -550.12435,        840.8058 ],
  [               0,               0,               0,               0,      -1024.1607,       1547.9139 ],
  [               0,               0,               0,               0,       997.77955,      -1508.5715 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.019759006,    -0.047886053,     0.032711998 ],
  [               0,               0,               0,       5.5489937,      -22.671281,       25.583211 ],
  [               0,               0,               0,      0.42049291,      -1.4088343,     -0.12759251 ],
  [               0,               0,               0,      -1.0187623,         3.61165,      -3.1250801 ],
  [               0,               0,               0,     0.012613669,    -0.044658585,     0.039886794 ],
  [               0,               0,               0,       16.592251,      -56.698265,       49.080127 ],
  [               0,               0,               0,      -9.1693857,        23.81632,      -16.413052 ],
 ],
 [
  [               0,               0,               0,  -0.00025082661,    0.0018258315,    0.0070231827 ],
  [               0,               0,               0,      -0.3019288,     -0.29219119,       4.4794804 ],
  [               0,               0,               0,      0.22596084,     -0.49403996,      -1.2015853 ],
  [               0,               0,               0,      -2.0573356,       4.7229603,      0.21996292 ],
  [               0,               0,               0,   -0.0084449795,    0.0019694521,     0.043295043 ],
  [               0,               0,               0,      0.69122325,      -3.0526797,       3.4281539 ],
  [               0,               0,               0,       5.8020149,      -22.390171,       18.514022 ],
 ],
 [
  [               0,               0,               0,      0.22914759,      -1.0364854,       1.1716945 ],
  [               0,               0,               0,      -185.67841,       883.64968,      -1047.8142 ],
  [               0,               0,               0,       74.927656,       -364.4551,       441.93067 ],
  [               0,               0,               0,       29.400311,      -140.30328,       167.37754 ],
  [               0,               0,               0,   -0.0095936261,     0.023895623,  -0.00060259694 ],
  [               0,               0,               0,      -28.850798,        134.3987,      -155.47145 ],
  [               0,               0,               0,      -32.076233,       153.30218,        -184.559 ],
 ],
 [
  [               0,               0,               0,               0,    0.0044570764,    0.0019363614 ],
  [               0,               0,               0,               0,      -42.134326,       105.82777 ],
  [               0,               0,               0,               0,       8.1205805,      -21.265703 ],
  [               0,               0,               0,               0,       1.2662368,      -2.9087743 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0031075684,    0.0024504472,   -0.0064499008,     -0.04627956 ],
  [               0,               0,      -1.2042481,       -1.379957,      -3.4194582,      -4.4732122 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,       341.16132,       286.09609,       -684.9597,      -5172.9997 ],
  [               0,               0,       0.8626743,       1.9383992,       5.2313502,       16.647022 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer2_etaoffv4_55_b = [
 [
  [               0,               0,               0,    0.0094431003,   -0.0046832025,    0.0086197533 ],
  [               0,               0,               0,      -6.1493145,       2.5794754,       3.9638047 ],
  [               0,               0,               0,        1.004742,     -0.47231967,      -1.2849899 ],
  [               0,               0,               0,    -0.051756293,     0.030053687,    -0.015903339 ],
  [               0,               0,               0,    0.0050156206,   -0.0026668551,   0.00023992725 ],
  [               0,               0,               0,       75.257205,      -30.071098,      0.65880278 ],
  [               0,               0,               0,      -120.29172,       54.435452,      -5.1115884 ],
 ],
 [
  [               0,               0,               0,   -0.0025021602,  -0.00032170294,    0.0090150017 ],
  [               0,               0,               0,       3.4318689,      -4.3477532,       5.4186296 ],
  [               0,               0,               0,      -3.6330681,       3.5958027,      -2.1480203 ],
  [               0,               0,               0,       2.2529514,      -2.2222266,      0.50229269 ],
  [               0,               0,               0,    0.0086424734,   -0.0093275039,    0.0026986226 ],
  [               0,               0,               0,       5.8765403,      -7.6839565,       3.3525843 ],
  [               0,               0,               0,       22.159135,      -36.544046,       12.537076 ],
 ],
 [
  [               0,               0,               0,               0,     0.018364473,   -0.0057601335 ],
  [               0,               0,               0,               0,      0.31286098,       3.3379464 ],
  [               0,               0,               0,               0,       5.7883301,      -5.8377065 ],
  [               0,               0,               0,               0,      -3.8953729,       0.0353627 ],
  [               0,               0,               0,               0,    -0.059507932,     0.020545624 ],
  [               0,               0,               0,               0,       1.0126495,     -0.75630466 ],
  [               0,               0,               0,               0,      0.54002603,      -3.5823442 ],
 ],
 [
  [               0,               0,               0,   0.00027047016,   -0.0011489733,    0.0095667008 ],
  [               0,               0,               0,      0.65796985,      -1.7646711,       4.5498739 ],
  [               0,               0,               0,      0.17472527,     -0.66623816,     -0.82972606 ],
  [               0,               0,               0,       34.440616,       3.5736584,      -23.941704 ],
  [               0,               0,               0,      0.26134884,     0.096226623,      -0.2354744 ],
  [               0,               0,               0,    -0.046848419,      0.11688538,    -0.054661253 ],
  [               0,               0,               0,      0.15066059,     -0.39465795,      -2.8668605 ],
 ],
 [
  [               0,               0,    -0.002811563,    -0.036921981,    -0.021225728,      0.11591723 ],
  [               0,               0,       573.28037,      -789.37234,      -1084.5303,       1496.0965 ],
  [               0,               0,      -91.429995,       134.95985,       155.94866,      -233.15684 ],
  [               0,               0,      -59.688397,       78.644643,       107.41204,      -141.17542 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      0.12618762,    0.0085235761,     -0.24262417 ],
  [               0,               0,               0,       5.8444535,       8.3467489,         15.9215 ],
  [               0,               0,               0,      -47.201479,       103.09872,      -54.112941 ],
  [               0,               0,               0,      0.11453421,       0.2812497,     0.031306401 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer2_etaoffv4_55_e = [
 [
  [     0.076659462,  -0.00085901673,      -0.1036242,     -0.21739039,     -0.30285702,      0.72617426 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,    -0.010788295,   -0.0077437464,   0.00074219194,     0.019192343,     0.055710588 ],
  [               0,       9.0894796,      -1.6918176,      -12.518021,      -15.037536,       13.761006 ],
  [               0,       24.947376,      -19.844626,       -56.26814,      -39.855345,       125.89138 ],
  [               0,       6.0192017,      -4.0902063,       -12.58546,      -9.6199307,       26.188601 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.001464668,     0.014789918 ],
  [               0,               0,               0,               0,       24.985861,      -35.618025 ],
  [               0,               0,               0,               0,       26.995548,      -41.976377 ],
  [               0,               0,               0,               0,      -29.992303,        45.38724 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0084666801,     0.027534499,    -0.018105533 ],
  [               0,               0,               0,       13.934745,      -48.827621,       47.211257 ],
  [               0,               0,               0,      -20.038784,       67.694224,      -58.159325 ],
  [               0,               0,               0,        10.26887,      -34.549401,       28.932146 ],
  [               0,               0,               0,  -0.00054077247,    0.0028185506,   -0.0016191861 ],
  [               0,               0,               0,      0.36715381,      -1.2515721,       2.0338818 ],
  [               0,               0,               0,      -20.018056,       67.736614,        -58.4498 ],
 ],
 [
  [               0,               0,               0,    0.0021671524,   -0.0038464164,    0.0080979571 ],
  [               0,               0,               0,       0.2587054,      -3.4000475,       8.6609644 ],
  [               0,               0,               0,      0.53806563,      -1.9908133,      0.57604753 ],
  [               0,               0,               0,      -17.902601,       83.076374,      -88.664871 ],
  [               0,               0,               0,    0.0065666923,     0.012544988,     0.019607413 ],
  [               0,               0,               0,     0.076889594,     -0.31488651,      0.32490849 ],
  [               0,               0,               0,      0.69469376,      -4.9712128,       4.7508328 ],
 ],
 [
  [               0,               0,               0,    0.0037801597,   -0.0035461273,   -0.0018503093 ],
  [               0,               0,               0,       21.639535,      -105.82813,       131.52866 ],
  [               0,               0,               0,      -10.831549,       52.461196,       -64.68147 ],
  [               0,               0,               0,      -9.7399605,       31.900246,      -6.1409379 ],
  [               0,               0,               0,     0.010157554,     0.021476401,     0.053722498 ],
  [               0,               0,               0,      0.48988057,      -2.5956558,       3.4172265 ],
  [               0,               0,               0,       9.8638584,      -45.602551,       49.582372 ],
 ],
 [
  [               0,               0,               0,               0,    0.0027457372,    0.0059352432 ],
  [               0,               0,               0,               0,      -51.556363,       129.11487 ],
  [               0,               0,               0,               0,       8.8402101,      -23.037779 ],
  [               0,               0,               0,               0,       1.6578601,      -3.8738489 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0018432084,    0.0022364609,   -0.0046471281,    -0.030275014 ],
  [               0,               0,      -1.1636885,      -1.4096337,       -3.496094,       -4.660086 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,       551.91304,       468.59711,      -1112.1394,      -8383.4648 ],
  [               0,               0,       1.4272174,       2.3292209,       6.1928961,       42.917679 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]


ele_layer2_etaoffv4_55_b = [
    ele5_layer2_etaoffv4_55_b,
    ele10_layer2_etaoffv4_55_b,
    ele25_layer2_etaoffv4_55_b,
    ele50_layer2_etaoffv4_55_b,
    ele75_layer2_etaoffv4_55_b,
    ele100_layer2_etaoffv4_55_b,
    ele200_layer2_etaoffv4_55_b,
    ele500_layer2_etaoffv4_55_b,
    ele1000_layer2_etaoffv4_55_b,
    ]

ele_layer2_etaoffv4_55_e = [
    ele5_layer2_etaoffv4_55_e,
    ele10_layer2_etaoffv4_55_e,
    ele25_layer2_etaoffv4_55_e,
    ele50_layer2_etaoffv4_55_e,
    ele75_layer2_etaoffv4_55_e,
    ele100_layer2_etaoffv4_55_e,
    ele200_layer2_etaoffv4_55_e,
    ele500_layer2_etaoffv4_55_e,
    ele1000_layer2_etaoffv4_55_e,
    ]


###############################################################################
# Tables for v4 sshape correction, layer 2, 5x5 photon clusters
#

gam5_layer2_etaoffv4_55_b = [
 [
  [               0,               0,               0,    -0.068084658,     0.046245274,   0.00059831481 ],
  [               0,               0,               0,       30.045176,      -17.833908,       4.0923685 ],
  [               0,               0,               0,      -5.6944588,       2.6739236,      -1.3046577 ],
  [               0,               0,               0,      0.20103283,      0.22222044,    -0.029024971 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.073164082,    -0.063258637,      0.01595789 ],
  [               0,               0,               0,       32.623234,      -51.370754,       21.164518 ],
  [               0,               0,               0,       2.5894293,      -1.3307261,      -1.2486795 ],
  [               0,               0,               0,      -0.8824915,       1.0984612,     -0.20553187 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.044767648,    -0.034555836 ],
  [               0,               0,               0,               0,      -153.06053,       130.61342 ],
  [               0,               0,               0,               0,       48.160825,      -40.994079 ],
  [               0,               0,               0,               0,      -34.237804,       28.017578 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam5_layer2_etaoffv4_55_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer2_etaoffv4_55_b = [
 [
  [               0,               0,               0,    -0.078103043,     0.029864407,     0.004954744 ],
  [               0,               0,               0,       26.099831,      -11.552654,       3.9765334 ],
  [               0,               0,               0,       -4.843859,       1.9486001,      -1.4217403 ],
  [               0,               0,               0,       1.0364256,     -0.27725353,     0.033190825 ],
  [               0,               0,               0,   -0.0031481515,    -0.001460968,   0.00040795815 ],
  [               0,               0,               0,       8.2934816,      -10.589077,       3.3051298 ],
  [               0,               0,               0,       15.072709,      -7.1892293,      -4.0042678 ],
 ],
 [
  [               0,               0,               0,    -0.012254329,     0.007537802,    0.0072561528 ],
  [               0,               0,               0,      -4.2749602,       5.4338782,       1.0086989 ],
  [               0,               0,               0,       1.9242744,      -2.5239069,     -0.44867618 ],
  [               0,               0,               0,       0.6945186,     -0.34014379,     0.044941194 ],
  [               0,               0,               0,     0.012423046,    -0.014147811,     0.003898318 ],
  [               0,               0,               0,       25.220237,      -26.771073,       8.7825909 ],
  [               0,               0,               0,      -104.57014,       47.348885,      -25.151171 ],
 ],
 [
  [               0,               0,               0,               0,      0.42255168,     -0.32808489 ],
  [               0,               0,               0,               0,      -46.073872,       39.068428 ],
  [               0,               0,               0,               0,       27.601817,      -23.445508 ],
  [               0,               0,               0,               0,      -8.2870601,        6.708813 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.013599035,     0.025834396,    -0.003264366 ],
  [               0,               0,               0,      0.72413311,      -1.9384883,       3.3907506 ],
  [               0,               0,               0,      -4.1150339,       8.7475723,      -5.6904565 ],
  [               0,               0,               0,       1.2131242,      -1.8960649,      0.70846184 ],
  [               0,               0,               0,   -0.0076457927,     0.014211795,   -0.0055825744 ],
  [               0,               0,               0,      -1.7510061,       7.3283048,      -4.8076465 ],
  [               0,               0,               0,     -0.50152185,        1.878934,     -0.96720035 ],
 ],
 [
  [               0,               0,   -0.0069551315,   -0.0021714909,    0.0036442784,     0.017668003 ],
  [               0,               0,      -23.464964,      -3.9413319,       24.214821,       72.231673 ],
  [               0,               0,      -64.442417,       132.76293,      -62.282406,     -0.48162247 ],
  [               0,               0,       43.927854,       9.6593161,      -25.325088,       -91.33536 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.035705907,    0.0018637283,    -0.068356432 ],
  [               0,               0,               0,       4.9136426,       8.3365355,       17.796976 ],
  [               0,               0,               0,      -62.274721,     -0.91162548,       120.11212 ],
  [               0,               0,               0,      0.18287757,       0.2067931,      0.27687538 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer2_etaoffv4_55_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,       574.66442 ],
  [               0,               0,               0,               0,               0,     0.027342781 ],
  [               0,               0,               0,               0,               0,    -0.027335686 ],
  [               0,               0,               0,               0,               0,  -1.6137685e-07 ],
  [               0,               0,               0,               0,               0,   0.00071733071 ],
  [               0,               0,               0,               0,               0,       1.5951541 ],
  [               0,               0,               0,               0,               0,      -7.9156723 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer2_etaoffv4_55_b = [
 [
  [               0,               0,               0,    -0.085678212,     0.038508086,    0.0058271236 ],
  [               0,               0,               0,       33.849122,      -15.535736,       4.4811506 ],
  [               0,               0,               0,      -2.7070821,       1.1232336,      -1.3529781 ],
  [               0,               0,               0,    -0.049927867,      0.13225516,    -0.017537064 ],
  [               0,               0,               0,   -0.0028313869,    0.0017033849,  -0.00028430268 ],
  [               0,               0,               0,       72.447143,      -27.154289,       3.6571758 ],
  [               0,               0,               0,      -63.812027,      -11.247591,      -0.4680253 ],
 ],
 [
  [               0,               0,               0,     0.010621683,   -0.0048898279,    0.0052000425 ],
  [               0,               0,               0,      -8.6378279,       8.2774145,       1.7787567 ],
  [               0,               0,               0,       5.8442898,      -7.5130257,       0.8206182 ],
  [               0,               0,               0,     -0.90780241,       1.8417572,     -0.58466175 ],
  [               0,               0,               0,    0.0093563094,    -0.011841738,     0.004527824 ],
  [               0,               0,               0,        -4.85315,       4.4762515,     -0.02615953 ],
  [               0,               0,               0,      -1.7731532,       -2.723406,    -0.034371716 ],
 ],
 [
  [               0,               0,               0,               0,      0.15709656,     -0.12195657 ],
  [               0,               0,               0,               0,       30.155738,      -20.873051 ],
  [               0,               0,               0,               0,       30.270075,      -26.465107 ],
  [               0,               0,               0,               0,       -19.59089,       16.150616 ],
  [               0,               0,               0,               0,    -0.021060095,     0.018845343 ],
  [               0,               0,               0,               0,      -4.0988515,       4.3004936 ],
  [               0,               0,               0,               0,      -23.491416,       4.1111045 ],
 ],
 [
  [               0,               0,               0,     0.015781393,    -0.034550617,     0.021067731 ],
  [               0,               0,               0,      -18.071082,       37.840571,      -14.839284 ],
  [               0,               0,               0,      -17.797987,        39.53129,      -22.860552 ],
  [               0,               0,               0,       7.0044674,      -14.804317,       7.9827241 ],
  [               0,               0,               0,    -0.004325484,    0.0088613817,   -0.0027531345 ],
  [               0,               0,               0,      0.64712648,      -1.3454279,       1.6880815 ],
  [               0,               0,               0,      -12.003314,       27.234852,       -16.48824 ],
 ],
 [
  [               0,               0,      0.84880354,      -1.1974486,      -1.6030609,       2.2692664 ],
  [               0,               0,      -1014.2914,       2140.4882,      -193.45151,      -1141.3741 ],
  [               0,               0,       149.33936,      -193.77708,      -276.17631,       356.69485 ],
  [               0,               0,       17.397448,      -29.951841,      -36.900882,       62.272621 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.15966655,     -0.20367089 ],
  [               0,               0,               0,               0,      -864.02901,       1249.7292 ],
  [               0,               0,               0,               0,      -1.2538091,    -0.053790524 ],
  [               0,               0,               0,               0,       5.3038747,      -7.3009747 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer2_etaoffv4_55_e = [
 [
  [               0,               0,     0.058508416,    -0.052066537,       -0.271135,      0.32748554 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,       147.62065,      -219.84303,       -327.2041,       487.32471 ],
  [               0,               0,      -123.60445,       192.44734,       215.57875,      -339.88438 ],
  [               0,               0,        115.2113,      -159.89069,      -258.43807,       359.20077 ],
  [               0,               0,        2.107751,      -2.2253572,       -7.959155,       9.8073238 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.073548356,    -0.022021072 ],
  [               0,               0,               0,               0,       6.6434849,       -8.581349 ],
  [               0,               0,               0,               0,       39.849367,      -61.255212 ],
  [               0,               0,               0,               0,      -41.165363,       62.258364 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      0.02290946,    0.0024655526,     -0.05293292 ],
  [               0,               0,               0,       5.0500275,      -21.045173,       23.390247 ],
  [               0,               0,               0,      -8.1093448,        30.65025,      -29.917248 ],
  [               0,               0,               0,       3.7372521,      -13.816507,       12.728388 ],
  [               0,               0,               0,     0.002330133,   -0.0043310232,    0.0030839568 ],
  [               0,               0,               0,      -1.9363475,       6.9133499,      -5.1255117 ],
  [               0,               0,               0,       19.084771,      -71.695323,       12.049813 ],
 ],
 [
  [               0,               0,               0,     0.018081792,    -0.048413133,     0.031991932 ],
  [               0,               0,               0,       6.2142208,      -31.171015,       40.174487 ],
  [               0,               0,               0,      0.36564313,     -0.52437344,      -1.6426056 ],
  [               0,               0,               0,      -53.563091,       219.29456,       -218.8896 ],
  [               0,               0,               0,     0.020036077,    -0.031529033,      0.03290799 ],
  [               0,               0,               0,      0.08833738,      -0.3963094,       0.4426444 ],
  [               0,               0,               0,      -3.1395455,       8.1163775,      -6.1963343 ],
 ],
 [
  [               0,               0,               0,   -0.0022057526,   0.00049073262,     0.021671236 ],
  [               0,               0,               0,       15.905762,      -75.495248,       91.557171 ],
  [               0,               0,               0,       16.666366,      -79.574574,       93.849364 ],
  [               0,               0,               0,      -15.412591,       73.970199,      -88.714677 ],
  [               0,               0,               0,    0.0025517052,     -0.00525511,   -0.0017880244 ],
  [               0,               0,               0,      -7.1474332,       8.4504536,       22.043184 ],
  [               0,               0,               0,       2.7404822,       2.6666211,      -24.236865 ],
 ],
 [
  [               0,               0,               0,               0,               0,      0.10130069 ],
  [               0,               0,               0,               0,               0,      0.61308702 ],
  [               0,               0,               0,               0,               0,     -0.56282494 ],
  [               0,               0,               0,               0,               0,     0.019531964 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,    0.0013193135,   -0.0015772144,   -0.0026097995,   -0.0027945093,   -0.0025840219 ],
  [               0,     -0.42259454,     -0.45241709,      -1.1191246,       -2.767266,      -2.8475087 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,      0.28835457,      0.51638989,      0.79370119,      0.75409285,      -2.1516286 ],
  [               0,      0.11325585,      0.27983136,      0.69534537,       1.7220077,       3.9638581 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer2_etaoffv4_55_b = [
 [
  [               0,               0,               0,      -0.0388006,     0.011791606,    0.0076342107 ],
  [               0,               0,               0,       12.173372,      -4.1243129,        3.847318 ],
  [               0,               0,               0,     -0.80543821,      0.39804627,      -1.3396382 ],
  [               0,               0,               0,        2.597582,      -1.3529324,      0.12812228 ],
  [               0,               0,               0,    0.0073121261,   -0.0036915207,   0.00091485647 ],
  [               0,               0,               0,       14.209028,      -4.5917404,      0.41568369 ],
  [               0,               0,               0,      -41.570113,       21.986787,      -3.6015531 ],
 ],
 [
  [               0,               0,               0,     -0.01747803,      0.01645462,    0.0025782487 ],
  [               0,               0,               0,       1.9761602,      -3.9273799,       5.6103323 ],
  [               0,               0,               0,      0.19409806,      0.27253604,      -1.6299667 ],
  [               0,               0,               0,       1.3313362,      -1.5764708,      0.53262706 ],
  [               0,               0,               0,    0.0065515812,    -0.006028882,    0.0019441992 ],
  [               0,               0,               0,      -1.2155877,       2.2163972,    0.0088815666 ],
  [               0,               0,               0,      -5.5839875,       9.8671654,      -11.829875 ],
 ],
 [
  [               0,               0,               0,               0,     0.036754424,    -0.021322932 ],
  [               0,               0,               0,               0,       16.538673,      -9.8245003 ],
  [               0,               0,               0,               0,        3.341785,      -4.0920209 ],
  [               0,               0,               0,               0,      -14.182544,       11.906885 ],
  [               0,               0,               0,               0,    -0.081946988,     0.070162076 ],
  [               0,               0,               0,               0,       10.650916,      -8.2970343 ],
  [               0,               0,               0,               0,      -21.400857,        7.847923 ],
 ],
 [
  [               0,               0,               0,  -0.00091763461,  -0.00050514088,     0.009530821 ],
  [               0,               0,               0,       1.8039029,      -4.1151064,       5.3683317 ],
  [               0,               0,               0,     -0.53222668,      0.90485002,      -1.7275854 ],
  [               0,               0,               0,      -1.4579145,      0.69050063,        2.275703 ],
  [               0,               0,               0,     -0.01410358,    0.0055329372,      0.02031699 ],
  [               0,               0,               0,       1.1285936,      -2.3431003,       1.2399099 ],
  [               0,               0,               0,      -3.0467288,       6.9927323,      -6.8118091 ],
 ],
 [
  [               0,               0,     0.011909717,     0.011772141,   -0.0016289295,     -0.04214083 ],
  [               0,               0,      -203.15009,        281.1896,       357.65751,      -492.49518 ],
  [               0,               0,       278.17196,      -370.63447,      -538.69241,       716.78744 ],
  [               0,               0,      -322.67193,       439.84907,       601.52374,      -819.55621 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      0.15375305,   -0.0069588163,     -0.29131005 ],
  [               0,               0,               0,      -122.41296,       5.4484493,       272.29155 ],
  [               0,               0,               0,      -8937.4436,       25242.208,      -17830.005 ],
  [               0,               0,               0,      -32.452244,       47.293307,     0.025253905 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer2_etaoffv4_55_e = [
 [
  [               0,               0,      0.90655305,      -1.2791465,      -2.0129985,       2.8444358 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      -1221.2969,        5374.322,       -7882.572,       3853.4964 ],
  [               0,               0,        181.7699,      -342.38641,      -110.88322,       326.63467 ],
  [               0,               0,      -48.603454,       112.09193,       6.8548887,      -99.232418 ],
  [               0,               0,       -536.9013,       1098.9897,       263.86263,      -1057.0745 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      -687.44512,       1096.6519 ],
  [               0,               0,               0,               0,        2.589926,       -3.888102 ],
  [               0,               0,               0,               0,      -2.5932539,       3.8931713 ],
  [               0,               0,               0,               0,    0.0066365735,    -0.010046027 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.076138718,      0.25642298,     -0.21160372 ],
  [               0,               0,               0,       86.807055,      -295.75499,       255.46884 ],
  [               0,               0,               0,      -10.089959,       33.848783,      -29.695309 ],
  [               0,               0,               0,       3.9194281,      -12.840168,       10.522856 ],
  [               0,               0,               0,     0.030366365,     -0.10088675,     0.084774439 ],
  [               0,               0,               0,      -1.2890442,       4.7361626,       -3.366826 ],
  [               0,               0,               0,        2.953286,      -9.9513059,       6.6568883 ],
 ],
 [
  [               0,               0,               0,      0.01295945,     -0.04079495,     0.040247132 ],
  [               0,               0,               0,       3.0301707,      -15.436699,        21.05386 ],
  [               0,               0,               0,      -1.2229085,       5.4873372,      -7.2576776 ],
  [               0,               0,               0,      -26.987627,       137.95212,      -165.12442 ],
  [               0,               0,               0,     0.083300341,     0.013569922,     -0.33092436 ],
  [               0,               0,               0,     -0.32795928,       1.6516659,      -2.0121754 ],
  [               0,               0,               0,      -2.0179368,       8.8146285,      -12.742403 ],
 ],
 [
  [               0,               0,               0,    -0.019644528,    -0.011043513,      0.15065445 ],
  [               0,               0,               0,       55.494942,      -264.23415,       316.40338 ],
  [               0,               0,               0,       32.593872,      -151.52133,       174.89756 ],
  [               0,               0,               0,       -26.89356,       126.37326,      -148.40554 ],
  [               0,               0,               0,    0.0091347813,    -0.011993092,    -0.023272692 ],
  [               0,               0,               0,      -29.822247,       138.88097,      -160.51396 ],
  [               0,               0,               0,       18.175943,      -80.935494,       88.700372 ],
 ],
 [
  [               0,               0,               0,               0,    0.0083110471,   -0.0026143171 ],
  [               0,               0,               0,               0,      -62.118281,       154.32669 ],
  [               0,               0,               0,               0,       13.648545,      -34.519803 ],
  [               0,               0,               0,               0,        11.30242,      -27.690123 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0019728125,  -0.00015503786,  -0.00079271084,    -0.027019029 ],
  [               0,               0,      -1.1814395,      -1.3915662,      -3.4474682,     -0.76107568 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,        2.011008,      0.90417886,      0.89073337,       4.2250302 ],
  [               0,               0,       1.8445631,      0.51133324,       1.8112979,       3.2597105 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer2_etaoffv4_55_b = [
 [
  [               0,               0,     -0.81285086,      0.48930012,    -0.079033655,    0.0091840248 ],
  [               0,               0,       310.76977,      -184.23008,         28.9771,       3.2806449 ],
  [               0,               0,      -38.507137,       23.613113,      -3.9219056,      -1.2341799 ],
  [               0,               0,       10.247897,      -6.0803773,       1.0101798,    -0.023925253 ],
  [               0,               0,      0.27291074,     -0.16545894,     0.027087772,  -0.00034960073 ],
  [               0,               0,       85.780699,      -59.740475,       11.610768,       0.3364697 ],
  [               0,               0,       93.522032,      -61.507947,       11.703392,      -2.3797592 ],
 ],
 [
  [               0,               0,               0,   -0.0055111037,    0.0051953345,    0.0079308403 ],
  [               0,               0,               0,   -0.0013323118,     -0.26099661,       3.4877019 ],
  [               0,               0,               0,     -0.40676338,      0.42165878,      -1.4208213 ],
  [               0,               0,               0,      0.74515196,     -0.68692492,       0.1740772 ],
  [               0,               0,               0,    0.0017006358,   -0.0011958566,   8.2481382e-05 ],
  [               0,               0,               0,       53.875338,      -74.474038,       23.701779 ],
  [               0,               0,               0,       52.527938,      -83.980823,        19.46998 ],
 ],
 [
  [               0,               0,               0,               0,     0.026920689,    -0.012821856 ],
  [               0,               0,               0,               0,      0.13465804,       3.2570427 ],
  [               0,               0,               0,               0,       8.1506203,      -7.8235996 ],
  [               0,               0,               0,               0,      -4.7991094,       3.8362015 ],
  [               0,               0,               0,               0,    -0.014762135,     0.012025838 ],
  [               0,               0,               0,               0,       43.106348,      -32.523136 ],
  [               0,               0,               0,               0,      -48.788288,       33.988348 ],
 ],
 [
  [               0,               0,               0,   -0.0017874301,    0.0071331707,    0.0034758879 ],
  [               0,               0,               0,       2.8855614,      -7.7282233,       7.8184573 ],
  [               0,               0,               0,     -0.27831013,      0.57313993,      -1.6248362 ],
  [               0,               0,               0,        1.278371,       2.3595409,      -2.4547613 ],
  [               0,               0,               0,     0.022681983,  -6.4697239e-05,    -0.012739977 ],
  [               0,               0,               0,      0.99283859,       -2.285955,       1.3346627 ],
  [               0,               0,               0,      -2.0087856,       3.9265535,      -4.8572246 ],
 ],
 [
  [               0,               0,     0.015649213,    -0.019535163,    -0.026559446,     0.042354118 ],
  [               0,               0,       525.75981,      -732.65093,      -1002.0489,       1398.4861 ],
  [               0,               0,      -86.151968,        128.7979,       148.74799,      -225.32636 ],
  [               0,               0,      -58.093623,       76.223114,       103.95345,      -135.88983 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.37468988,     -0.51089501 ],
  [               0,               0,               0,               0,      -466.21324,       692.69704 ],
  [               0,               0,               0,               0,      -27.659445,        36.55708 ],
  [               0,               0,               0,               0,       3.8870677,      -4.9838629 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer2_etaoffv4_55_e = [
 [
  [               0,               0,     -0.46997932,      0.70522361,      0.81188881,      -1.2277363 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     -0.15131238,    -0.080032033,      0.11013247,      0.52472851 ],
  [               0,               0,       135.21231,      -192.47586,      -300.72696,       429.06601 ],
  [               0,               0,       8.6981908,      -13.744571,      -19.942532,       30.840681 ],
  [               0,               0,       1.9309097,      -2.6881535,      -4.0945251,       5.6823252 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.13487167,      -0.1131408 ],
  [               0,               0,               0,               0,     -0.21318167,        1.645815 ],
  [               0,               0,               0,               0,       30.249072,      -46.670982 ],
  [               0,               0,               0,               0,      -30.750506,       46.512006 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.078820473,      0.26933737,      -0.2198759 ],
  [               0,               0,               0,       24.116951,      -82.355814,        72.69981 ],
  [               0,               0,               0,    -0.036205677,      0.13491329,      -1.3743564 ],
  [               0,               0,               0,       1.5864222,      -5.2363898,       4.5857978 ],
  [               0,               0,               0,    0.0020407989,   -0.0059766932,      0.00681607 ],
  [               0,               0,               0,       6.3932774,      -21.775886,       18.258135 ],
  [               0,               0,               0,      -12.274572,       41.589902,      -38.407836 ],
 ],
 [
  [               0,               0,               0,     0.041792409,     -0.17371287,      0.18160321 ],
  [               0,               0,               0,      -45.627411,       184.50271,      -180.17542 ],
  [               0,               0,               0,      -5.8963674,       26.170098,      -30.164736 ],
  [               0,               0,               0,     -0.96929882,       3.1299145,      -2.1799866 ],
  [               0,               0,               0,    -0.016600274,     0.068159974,    -0.067628517 ],
  [               0,               0,               0,       -1.225065,       4.8981182,      -3.8990116 ],
  [               0,               0,               0,      -2.5339811,       11.279846,      -13.994149 ],
 ],
 [
  [               0,               0,               0,     -0.27309304,       1.2988516,      -1.5320995 ],
  [               0,               0,               0,       78.297568,      -372.54104,       445.03119 ],
  [               0,               0,               0,      -16.563617,       77.643549,      -92.073193 ],
  [               0,               0,               0,      -4.5191957,       22.517402,      -27.941102 ],
  [               0,               0,               0,    -0.040420589,      0.19688055,     -0.23911982 ],
  [               0,               0,               0,       144.12615,      -682.47514,       808.14718 ],
  [               0,               0,               0,       194.00296,      -951.94029,       1164.4048 ],
 ],
 [
  [               0,               0,               0,               0,      -1.1348609,       2.8147856 ],
  [               0,               0,               0,               0,       10.581741,      -24.282583 ],
  [               0,               0,               0,               0,       11.608259,      -29.646786 ],
  [               0,               0,               0,               0,      -6.6715774,       16.556329 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0028070002,    0.0022768334,    -0.005702811,    -0.042535766 ],
  [               0,               0,      -1.1515828,      -1.4073751,      -3.4859855,      -4.6345442 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,       -57.12324,      -46.672447,       121.62648,       894.07372 ],
  [               0,               0,      0.52558615,       1.3143703,       3.0692261,       6.7525852 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer2_etaoffv4_55_b = [
 [
  [               0,               0,               0,    -0.064024441,     0.028075782,    0.0048369522 ],
  [               0,               0,               0,       18.288827,      -8.4480243,       4.8652245 ],
  [               0,               0,               0,      -1.7526529,       0.6476799,      -1.3868806 ],
  [               0,               0,               0,      0.17115122,     0.042303049,    -0.010051906 ],
  [               0,               0,               0,     0.019521014,    -0.008500026,    0.0010912426 ],
  [               0,               0,               0,       18.429277,      -7.6256177,       1.2209037 ],
  [               0,               0,               0,       5.3484719,      -2.7333743,      -1.5493314 ],
 ],
 [
  [               0,               0,               0,   -0.0044165372,    0.0074958669,    0.0059591247 ],
  [               0,               0,               0,      -4.3390564,       3.1501929,       3.1872908 ],
  [               0,               0,               0,     -0.11235883,      0.13382446,      -1.3507691 ],
  [               0,               0,               0,      0.68599034,     -0.62153313,      0.14751099 ],
  [               0,               0,               0,   -0.0082471133,    0.0091752909,   -0.0024390495 ],
  [               0,               0,               0,      -8.1005457,        6.234763,       0.8305932 ],
  [               0,               0,               0,      -131.39385,       140.43882,      -48.158548 ],
 ],
 [
  [               0,               0,               0,               0,     0.068797495,    -0.050614371 ],
  [               0,               0,               0,               0,      -14.564065,       16.441525 ],
  [               0,               0,               0,               0,       14.709924,      -13.381159 ],
  [               0,               0,               0,               0,      -9.7174345,       7.8784723 ],
  [               0,               0,               0,               0,    -0.010679421,    0.0099376992 ],
  [               0,               0,               0,               0,       1.8153178,     -0.42065692 ],
  [               0,               0,               0,               0,      0.47963202,      -2.2511322 ],
 ],
 [
  [               0,               0,               0,    0.0069627316,    -0.012951915,     0.013941658 ],
  [               0,               0,               0,      0.51253332,      -2.5502786,       5.3461746 ],
  [               0,               0,               0,       1.0219343,      -2.4554651,     0.042741861 ],
  [               0,               0,               0,      -2.7963831,       7.1921267,      -1.5979476 ],
  [               0,               0,               0,    0.0021234739,      0.00791987,     0.011594978 ],
  [               0,               0,               0,     -0.26963673,      0.67323327,     -0.34896127 ],
  [               0,               0,               0,     0.090716054,     -0.40798903,      -2.7346407 ],
 ],
 [
  [               0,               0,               0,   -0.0094698624,  -0.00063501788,     0.029991962 ],
  [               0,               0,               0,      -82.763584,       201.32737,      -117.97401 ],
  [               0,               0,               0,      -141.78632,       402.33637,      -286.35582 ],
  [               0,               0,               0,      -28.713198,       66.197682,      -36.659322 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.64642754,     -0.90019981 ],
  [               0,               0,               0,               0,       12.437644,       22.108017 ],
  [               0,               0,               0,               0,      -49.382072,       66.139645 ],
  [               0,               0,               0,               0,      -3.1505112,        5.400495 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer2_etaoffv4_55_e = [
 [
  [               0,               0,      0.59893802,     -0.80646288,       -1.368896,        1.855435 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,       129.86339,      -386.26368,       287.26405 ],
  [               0,               0,               0,      -155.47713,       470.72049,      -355.48818 ],
  [               0,               0,               0,       117.09605,      -356.06661,       269.95133 ],
  [               0,               0,               0,       14.765689,      -43.019158,       31.319513 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0025828819,     0.022543467 ],
  [               0,               0,               0,               0,       9.8831221,      -13.750732 ],
  [               0,               0,               0,               0,      -1.5190524,       1.4060438 ],
  [               0,               0,               0,               0,       -4.149556,       6.2963561 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,  -0.00012107654,    0.0048178869,     0.001728516 ],
  [               0,               0,               0,       3.9210797,      -13.975056,       15.025285 ],
  [               0,               0,               0,       2.3815853,      -8.1761133,        5.740406 ],
  [               0,               0,               0,       1.0831114,      -12.724483,        20.12904 ],
  [               0,               0,               0,    -0.026357652,    0.0087134645,     0.076808218 ],
  [               0,               0,               0,      -1.6650616,       5.9546239,      -5.1887207 ],
  [               0,               0,               0,       5.7702719,      -19.156717,        12.79116 ],
 ],
 [
  [               0,               0,               0,     0.020366646,    -0.093439682,      0.11556832 ],
  [               0,               0,               0,      -4.3392042,       18.079487,      -16.373696 ],
  [               0,               0,               0,       2.7512934,      -11.500772,       10.763119 ],
  [               0,               0,               0,       -26.76601,       101.81591,      -89.576852 ],
  [               0,               0,               0,    -0.038255143,     0.015924668,      0.19174488 ],
  [               0,               0,               0,       -1.245839,       5.2896733,      -5.5582395 ],
  [               0,               0,               0,       2.6766695,      -10.960147,       8.1078473 ],
 ],
 [
  [               0,               0,               0,     0.061976502,     -0.29801823,      0.35847522 ],
  [               0,               0,               0,       98.515757,      -344.30746,        275.1488 ],
  [               0,               0,               0,      -75.231457,       269.15923,      -218.81302 ],
  [               0,               0,               0,      -24.977461,       175.22736,       -272.9636 ],
  [               0,               0,               0,     0.011079364,    -0.052178439,     0.064246472 ],
  [               0,               0,               0,      -1.9180941,       8.3776594,      -8.0534573 ],
  [               0,               0,               0,       19.090673,      -95.338547,        117.0575 ],
 ],
 [
  [               0,               0,               0,               0,     -0.34689583,      0.87230249 ],
  [               0,               0,               0,               0,      -24.594622,       62.436924 ],
  [               0,               0,               0,               0,       22.224198,      -55.879082 ],
  [               0,               0,               0,               0,      -13.290775,        32.87931 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0014196413,  -0.00078763731,   -0.0046354247,   -0.0052790472 ],
  [               0,               0,       83.027889,      -154.65447,      -140.18407,      -4.6040442 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,       -75.32484,       29.730879,       275.12172,       451.68997 ],
  [               0,               0,       3.3591088,        9.347941,       7.4092744,       2.9982125 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer2_etaoffv4_55_b = [
 [
  [               0,               0,               0,    -0.026247292,     0.013834109,    0.0066208796 ],
  [               0,               0,               0,       9.7523263,      -5.1253675,       4.4689289 ],
  [               0,               0,               0,      -1.8778316,       1.0061587,      -1.4268665 ],
  [               0,               0,               0,      0.39684296,     -0.27522083,     0.030096738 ],
  [               0,               0,               0,    0.0025918886,    -0.003065299,   0.00058519416 ],
  [               0,               0,               0,       2.1896958,      -4.9272419,       1.1541227 ],
  [               0,               0,               0,      -12.120954,        5.360155,      -2.4196259 ],
 ],
 [
  [               0,               0,               0,      0.01366749,    -0.017482475,     0.013476346 ],
  [               0,               0,               0,       -9.560243,        11.21751,      0.68482598 ],
  [               0,               0,               0,     -0.33555534,      0.15579631,      -1.3134764 ],
  [               0,               0,               0,      -1.3760096,       3.1131803,     -0.95822887 ],
  [               0,               0,               0,    0.0012320992,    0.0038293656,   0.00088480534 ],
  [               0,               0,               0,      0.65422187,     -0.48559579,      0.10136807 ],
  [               0,               0,               0,       5.7404188,      -9.8754455,      -5.0795366 ],
 ],
 [
  [               0,               0,               0,               0,      0.03365317,    -0.022709208 ],
  [               0,               0,               0,               0,       -28.34098,       27.643717 ],
  [               0,               0,               0,               0,       26.867128,      -23.010632 ],
  [               0,               0,               0,               0,      -13.999443,       11.247517 ],
  [               0,               0,               0,               0,   -0.0016406797,    0.0028989484 ],
  [               0,               0,               0,               0,       3.1472091,      -1.4821308 ],
  [               0,               0,               0,               0,       18.374777,      -16.443306 ],
 ],
 [
  [               0,               0,               0,   0.00076398259,   -0.0027999921,     0.010513021 ],
  [               0,               0,               0,      0.68541622,      -1.8051874,       4.4388419 ],
  [               0,               0,               0,      0.50933304,      -1.4011019,     -0.47086668 ],
  [               0,               0,               0,      -37.059674,       36.500501,       36.358082 ],
  [               0,               0,               0,     -0.23331432,     0.099684707,      0.43665205 ],
  [               0,               0,               0,     -0.13942778,      0.33576312,     -0.18065436 ],
  [               0,               0,               0,      0.36517678,     -0.78532011,      -2.7032312 ],
 ],
 [
  [               0,               0,    -0.017484668,    -0.032838219,   -0.0044592318,      0.12360076 ],
  [               0,               0,       495.93142,      -680.97627,      -938.62247,       1291.3811 ],
  [               0,               0,      -130.13595,       186.51033,       231.16734,      -333.39924 ],
  [               0,               0,      -44.970901,       58.585022,       80.098684,      -103.93495 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.42499547,     -0.57632486 ],
  [               0,               0,               0,               0,       12.402641,       22.177436 ],
  [               0,               0,               0,               0,      -23.782251,       30.883476 ],
  [               0,               0,               0,               0,      0.34712644,      0.28160211 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer2_etaoffv4_55_e = [
 [
  [               0,               0,      0.78485206,      -1.0594117,      -1.7319314,       2.3481784 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.061855233,      0.00273914,      0.13844073 ],
  [               0,               0,               0,       102.81491,      -247.30364,       142.88391 ],
  [               0,               0,               0,       265.46458,      -802.91837,       605.84703 ],
  [               0,               0,               0,       89.688822,      -259.41044,       187.48496 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0024356451,    0.0034853943 ],
  [               0,               0,               0,               0,       62.337706,      -91.415653 ],
  [               0,               0,               0,               0,      -2.3226486,       2.3972263 ],
  [               0,               0,               0,               0,      -3.6191989,       5.4834453 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0022749693,    0.0067485955,   0.00083190546 ],
  [               0,               0,               0,       2.6238805,       -8.592786,       10.646036 ],
  [               0,               0,               0,      -14.315617,       49.046241,      -43.066001 ],
  [               0,               0,               0,        7.664146,      -26.209925,       22.319371 ],
  [               0,               0,               0,   0.00084393866,   -0.0028053844,    0.0033447607 ],
  [               0,               0,               0,       4.5670277,      -15.576336,       14.179806 ],
  [               0,               0,               0,      -34.835303,       119.56841,      -109.87491 ],
 ],
 [
  [               0,               0,               0,    -0.039142207,      0.16736639,     -0.16864383 ],
  [               0,               0,               0,       6.9015917,      -30.726957,       36.494756 ],
  [               0,               0,               0,     -0.73100612,       3.4858393,       -5.309747 ],
  [               0,               0,               0,      -4.5460395,       18.788552,      -19.190961 ],
  [               0,               0,               0,    -0.036844971,      0.15455852,      -0.1601825 ],
  [               0,               0,               0,       5.9254339,      -25.501583,       27.557362 ],
  [               0,               0,               0,       21.806422,      -89.138582,        88.22786 ],
 ],
 [
  [               0,               0,               0,    0.0028907814,   -0.0018432439,   0.00018944144 ],
  [               0,               0,               0,      -4.0913897,       18.000916,      -17.592222 ],
  [               0,               0,               0,       9.0171426,      -42.728328,       49.478432 ],
  [               0,               0,               0,      -9.1812642,       45.304425,      -55.767738 ],
  [               0,               0,               0,   -0.0048042996,    -0.005063546,     0.039038359 ],
  [               0,               0,               0,      -64.681787,        305.9373,      -361.46381 ],
  [               0,               0,               0,      -10.888402,       67.333755,      -98.410878 ],
 ],
 [
  [               0,               0,               0,               0,    0.0028755719,     0.018204122 ],
  [               0,               0,               0,               0,      -23.412579,       59.545863 ],
  [               0,               0,               0,               0,       35.803759,      -89.316194 ],
  [               0,               0,               0,               0,       -29.46196,       72.655777 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0036260216,    0.0029913536,   -0.0074770597,     -0.05503068 ],
  [               0,               0,      -1.1605402,      -1.4081016,      -3.4881228,      -4.6406893 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,       126.20853,       106.55644,      -250.04904,       -1902.399 ],
  [               0,               0,      0.61039967,       1.4156293,       3.7178923,       10.561519 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer2_etaoffv4_55_b = [
 [
  [               0,               0,               0,    -0.040459706,     0.014694197,    0.0063064285 ],
  [               0,               0,               0,       9.4673339,      -3.3050902,       4.6378685 ],
  [               0,               0,               0,     -0.46838365,      0.50964209,      -1.4135666 ],
  [               0,               0,               0,     0.012275884,     -0.18473466,     0.020250047 ],
  [               0,               0,               0,      0.01735223,   -0.0065141621,   0.00076260623 ],
  [               0,               0,               0,       7.1647877,      -3.1269688,       1.1821362 ],
  [               0,               0,               0,      -5.6595724,       3.9475608,       -2.083728 ],
 ],
 [
  [               0,               0,               0,   -0.0013542731,    0.0074882941,    0.0032404748 ],
  [               0,               0,               0,       5.1663343,      -9.2209245,       7.9289543 ],
  [               0,               0,               0,      -1.9630869,       2.6336116,      -2.1930626 ],
  [               0,               0,               0,       1.1172343,       -1.360076,      0.39760301 ],
  [               0,               0,               0,  -6.6563194e-05,   -0.0012800263,    0.0012002225 ],
  [               0,               0,               0,      -6.1568116,       6.2766876,     -0.61135569 ],
  [               0,               0,               0,       2.3446607,      0.17203928,      -2.4424068 ],
 ],
 [
  [               0,               0,               0,               0,    0.0094720921,   -0.0031111972 ],
  [               0,               0,               0,               0,        4.080595,       1.9144713 ],
  [               0,               0,               0,               0,       13.228125,      -11.935935 ],
  [               0,               0,               0,               0,      -9.5032008,       7.5504211 ],
  [               0,               0,               0,               0,    0.0011241696,     0.000582085 ],
  [               0,               0,               0,               0,      0.66030393,      0.50116495 ],
  [               0,               0,               0,               0,       5.5097102,      -6.0740234 ],
 ],
 [
  [               0,               0,               0,     0.011068281,    -0.021549737,     0.019083234 ],
  [               0,               0,               0,       -3.108048,       5.5368103,       1.1898527 ],
  [               0,               0,               0,      0.69969493,      -1.4798693,     -0.54174631 ],
  [               0,               0,               0,      -0.2956842,      0.76921461,     -0.45098851 ],
  [               0,               0,               0,    0.0011478569,   -0.0022575168,    0.0012206334 ],
  [               0,               0,               0,      -2.7618954,        4.429786,      0.22040954 ],
  [               0,               0,               0,     -0.57626072,      -0.3443966,      -3.8732953 ],
 ],
 [
  [               0,               0,   -0.0096348778,   -0.0047361111,    0.0075376676,     0.033426573 ],
  [               0,               0,      -21.933586,       23.285467,       35.531599,      -32.907711 ],
  [               0,               0,      -69.387018,         98.8573,       133.32723,      -191.10741 ],
  [               0,               0,      -15.174195,       16.951876,       25.937368,      -28.232022 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.45995463,     -0.63026767 ],
  [               0,               0,               0,               0,       12.137613,        22.55061 ],
  [               0,               0,               0,               0,      -31.571997,       41.844752 ],
  [               0,               0,               0,               0,      0.31660215,      0.22759904 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer2_etaoffv4_55_e = [
 [
  [      0.09674899,     -0.00594468,     -0.13598221,     -0.26496125,     -0.32151576,      0.84577929 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.018034678,   -0.0072954426,     0.017078229,     0.065766051 ],
  [               0,               0,      -27.562741,       46.118867,       65.439354,      -107.13294 ],
  [               0,               0,       23.331826,      -37.904017,      -52.255128,       83.839725 ],
  [               0,               0,       7.0915968,      -10.099288,       -14.96136,       21.281446 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.018110624,     0.064994776 ],
  [               0,               0,               0,               0,       4.2765833,      -4.4805644 ],
  [               0,               0,               0,               0,       51.324827,      -78.730528 ],
  [               0,               0,               0,               0,      -50.869816,       76.950521 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.043206745,     -0.14434726,      0.12832221 ],
  [               0,               0,               0,      -8.8522446,       28.850995,      -20.034785 ],
  [               0,               0,               0,      0.47686243,      -1.7567212,      0.28290672 ],
  [               0,               0,               0,        5.759868,      -19.438505,       16.491927 ],
  [               0,               0,               0,     0.065288358,     -0.22365571,       0.1921598 ],
  [               0,               0,               0,      -8.0334492,       28.004057,      -24.035209 ],
  [               0,               0,               0,      -11.448842,       36.998237,      -32.067096 ],
 ],
 [
  [               0,               0,               0,     0.017435892,    -0.075375098,     0.090604642 ],
  [               0,               0,               0,      -3.4393005,       13.409985,      -10.296617 ],
  [               0,               0,               0,       1.4632956,      -5.9073472,       4.6501423 ],
  [               0,               0,               0,      -7.2738124,       27.554291,      -23.060832 ],
  [               0,               0,               0,   -0.0084430649,    0.0024666326,     0.056699087 ],
  [               0,               0,               0,     -0.45786896,       1.8826879,      -1.8607254 ],
  [               0,               0,               0,       1.1712681,       -4.505076,       1.2342793 ],
 ],
 [
  [               0,               0,               0,    -0.015975345,   0.00081618426,     0.099563223 ],
  [               0,               0,               0,       27.742418,      -124.25053,       140.81538 ],
  [               0,               0,               0,        6.822485,      -34.369451,       41.933575 ],
  [               0,               0,               0,      -8.2152415,       39.811202,      -48.129788 ],
  [               0,               0,               0,    0.0020550462,   -0.0014224648,   -0.0077729073 ],
  [               0,               0,               0,    -0.074146078,        1.387135,      -2.2488696 ],
  [               0,               0,               0,       3.2135274,      0.75148534,      -22.642424 ],
 ],
 [
  [               0,               0,               0,               0,    0.0067707448,    0.0013597592 ],
  [               0,               0,               0,               0,      -30.486419,       76.787952 ],
  [               0,               0,               0,               0,       5.8575149,      -15.541959 ],
  [               0,               0,               0,               0,        1.062985,      -2.4762853 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0024672374,    0.0018336317,   -0.0049952545,    -0.036405897 ],
  [               0,               0,      -1.2033242,      -1.3765831,      -3.4114736,       -4.454372 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,       429.79947,       360.56241,       -859.1153,      -6514.3219 ],
  [               0,               0,       1.1086359,       3.2940232,       6.7897664,       20.685622 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer2_etaoffv4_55_b = [
 [
  [               0,               0,               0,   -0.0028563768,    0.0020482784,     0.007888579 ],
  [               0,               0,               0,      0.96496751,      -1.2693483,       4.4556394 ],
  [               0,               0,               0,       1.7156586,     -0.65532308,      -1.2635678 ],
  [               0,               0,               0,     -0.42888516,      0.17015232,    -0.046225565 ],
  [               0,               0,               0,    -0.004683757,    0.0011668686,   8.5418654e-05 ],
  [               0,               0,               0,       46.281926,      -19.327038,       1.8624748 ],
  [               0,               0,               0,      -93.970076,       28.883882,      -1.2981781 ],
 ],
 [
  [               0,               0,               0,  -0.00041684975,     0.001114157,    0.0074977665 ],
  [               0,               0,               0,      -3.4990415,       3.3672845,       3.4892647 ],
  [               0,               0,               0,     -0.23017175,      0.10033838,      -1.2833327 ],
  [               0,               0,               0,       911.50238,      -98.390703,      -819.15549 ],
  [               0,               0,               0,      -8.0507823,       2.0331687,       6.0880978 ],
  [               0,               0,               0,     0.037579699,     -0.05477533,     0.020257303 ],
  [               0,               0,               0,     -0.53237771,      0.45571343,    -0.098847226 ],
 ],
 [
  [               0,               0,               0,               0,     0.035828228,    -0.019372986 ],
  [               0,               0,               0,               0,       -5.323887,       7.8870913 ],
  [               0,               0,               0,               0,       5.3857199,      -5.4410061 ],
  [               0,               0,               0,               0,      -7.3463993,       5.6626535 ],
  [               0,               0,               0,               0,     0.044785216,    -0.034435831 ],
  [               0,               0,               0,               0,      -5.0188709,       4.4628819 ],
  [               0,               0,               0,               0,       11.154663,      -8.9389991 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0027744416,     0.012633874 ],
  [               0,               0,               0,               0,      0.23666645,       3.1679369 ],
  [               0,               0,               0,               0,    -0.016154147,      -1.2837339 ],
  [               0,               0,               0,               0,      0.07037007,     -0.06218772 ],
  [               0,               0,               0,               0,  -0.00063668349,   0.00095453184 ],
  [               0,               0,               0,               0,       1.0190043,      0.17878881 ],
  [               0,               0,               0,               0,      -1.5827159,       -15.46669 ],
 ],
 [
  [               0,               0,   -0.0090361931,   -0.0042476338,    0.0072924159,     0.031190293 ],
  [               0,               0,      -15.192011,       12.745503,       22.327046,      -12.226365 ],
  [               0,               0,      -59.792932,       85.920686,       115.25481,      -166.74422 ],
  [               0,               0,      -12.812853,       13.758024,       21.517259,      -22.272422 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.48639495,     -0.66956206 ],
  [               0,               0,               0,               0,       11.427828,       23.565924 ],
  [               0,               0,               0,               0,      -35.521701,       47.383148 ],
  [               0,               0,               0,               0,      0.29758142,      0.20699242 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer2_etaoffv4_55_e = [
 [
  [       100.84951,      -204.37216,      -161.64573,       241.19944,       583.88969,      -603.95039 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,    0.0057400512,   -0.0063353747,    -0.014863024,   -0.0062487949,     0.049090613 ],
  [               0,      -49.088349,       31.125752,       124.02621,        113.8133,      -303.77833 ],
  [               0,       189.55104,      -145.31003,      -413.53029,      -293.49507,       900.69889 ],
  [               0,      -101.78733,       74.575637,        222.4642,       165.83809,      -486.01163 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.010334233,     -0.01224617 ],
  [               0,               0,               0,               0,      -393.36084,       600.10167 ],
  [               0,               0,               0,               0,      -69.374526,       103.94142 ],
  [               0,               0,               0,               0,       109.44644,      -165.40001 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0047859719,    -0.011661499,     0.014485756 ],
  [               0,               0,               0,       1.8290131,      -7.9622308,       11.618568 ],
  [               0,               0,               0,        0.996369,      -3.1465923,       1.0881996 ],
  [               0,               0,               0,     -0.17158913,      -2.0337722,       4.5913516 ],
  [               0,               0,               0,   -0.0064641656,    0.0015473697,     0.021073836 ],
  [               0,               0,               0,     0.083162366,    -0.058940608,     0.040205015 ],
  [               0,               0,               0,       6.6889956,      -22.526847,       15.938256 ],
 ],
 [
  [               0,               0,               0,   0.00018535998,   0.00068849714,    0.0077475143 ],
  [               0,               0,               0,      0.87525314,      -5.1691787,       9.6112788 ],
  [               0,               0,               0,      0.38870745,      -1.4338133,     0.041729308 ],
  [               0,               0,               0,     -0.39533741,       2.0899264,      -2.1064849 ],
  [               0,               0,               0,    0.0012093915,   0.00036327769,    -0.001112373 ],
  [               0,               0,               0,   9.4937922e-05,    -0.059942411,      0.21898152 ],
  [               0,               0,               0,     -0.15125901,      0.71833112,      -3.7446691 ],
 ],
 [
  [               0,               0,               0,   -0.0015837706,   0.00065556103,     0.018530361 ],
  [               0,               0,               0,       2.0598214,      -10.145405,       14.765102 ],
  [               0,               0,               0,      0.79203078,       -3.358305,       2.2963799 ],
  [               0,               0,               0,       -1.484345,       25.133698,      -36.504279 ],
  [               0,               0,               0,     0.019269179,     0.018396245,     0.012435863 ],
  [               0,               0,               0,     -0.25844125,      0.99913971,     -0.89507567 ],
  [               0,               0,               0,       9.8094825,      -45.688209,       43.787514 ],
 ],
 [
  [               0,               0,               0,               0,    0.0012566541,    0.0079222012 ],
  [               0,               0,               0,               0,      -82.615751,       205.86273 ],
  [               0,               0,               0,               0,       14.411588,      -36.798713 ],
  [               0,               0,               0,               0,       1.5113009,      -3.4785787 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.002070983,       0.0016813,   -0.0044154923,    -0.030858193 ],
  [               0,               0,      -1.1904989,      -1.3924364,      -3.4520293,      -4.5560321 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,       421.85176,       360.26852,      -848.84175,      -6424.7959 ],
  [               0,               0,        1.870918,       2.0186049,       5.3226923,        16.19436 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]


gam_layer2_etaoffv4_55_b = [
    gam5_layer2_etaoffv4_55_b,
    gam10_layer2_etaoffv4_55_b,
    gam25_layer2_etaoffv4_55_b,
    gam50_layer2_etaoffv4_55_b,
    gam75_layer2_etaoffv4_55_b,
    gam100_layer2_etaoffv4_55_b,
    gam200_layer2_etaoffv4_55_b,
    gam500_layer2_etaoffv4_55_b,
    gam1000_layer2_etaoffv4_55_b,
    ]

gam_layer2_etaoffv4_55_e = [
    gam5_layer2_etaoffv4_55_e,
    gam10_layer2_etaoffv4_55_e,
    gam25_layer2_etaoffv4_55_e,
    gam50_layer2_etaoffv4_55_e,
    gam75_layer2_etaoffv4_55_e,
    gam100_layer2_etaoffv4_55_e,
    gam200_layer2_etaoffv4_55_e,
    gam500_layer2_etaoffv4_55_e,
    gam1000_layer2_etaoffv4_55_e,
    ]


###############################################################################
# Tables for v4 sshape correction, layer 2, 3x5 electron clusters
#

ele5_layer2_etaoffv4_35_b = [
 [
  [               0,               0,               0,      0.10987232,    -0.064305045,     0.021083849 ],
  [               0,               0,               0,      -4.9270249,       3.4065712,       1.2462197 ],
  [               0,               0,               0,        3.928984,      -1.9743173,     -0.85701349 ],
  [               0,               0,               0,      0.13066158,      0.03203252,    0.0042413257 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele5_layer2_etaoffv4_35_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer2_etaoffv4_35_b = [
 [
  [               0,               0,               0,      0.17373786,    -0.051306368,     0.012738092 ],
  [               0,               0,               0,      -36.195707,       13.142796,       1.9249914 ],
  [               0,               0,               0,       12.191153,      -4.3942372,     -0.95810195 ],
  [               0,               0,               0,      -2.9721807,       1.0599621,    -0.051496831 ],
  [               0,               0,               0,    0.0059039375,    0.0028217645,  -0.00036504732 ],
  [               0,               0,               0,       1.9600059,      -3.3848351,       1.9630277 ],
  [               0,               0,               0,      -30.764499,       5.5868512,      -3.9092284 ],
 ],
 [
  [               0,               0,               0,   -0.0095880236,     0.018936695,  -0.00010189709 ],
  [               0,               0,               0,      -6.9448105,       1.7545501,       4.2263068 ],
  [               0,               0,               0,       3.4486792,      -2.4069519,      -1.0148344 ],
  [               0,               0,               0,       1.8341909,      -2.4114671,      0.82506839 ],
  [               0,               0,               0,      0.02749479,    -0.029663118,    0.0083647943 ],
  [               0,               0,               0,       3.0713418,      -1.8833006,      0.77685432 ],
  [               0,               0,               0,      -17.948053,       24.438249,      -9.2183401 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.009976959 ],
  [               0,               0,               0,               0,               0,       2.2150727 ],
  [               0,               0,               0,               0,               0,      -1.1854355 ],
  [               0,               0,               0,               0,               0,    0.0068981477 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0047877162,    -0.021419347,     0.025600299 ],
  [               0,               0,               0,       2.8803165,       -3.988123,       3.5099699 ],
  [               0,               0,               0,      -1.2000517,        1.787667,      -1.8657024 ],
  [               0,               0,               0,      0.65215151,     -0.70369799,      0.19175242 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.018976371,    -0.025717068 ],
  [               0,               0,               0,               0,       10.549962,        25.20926 ],
  [               0,               0,               0,               0,      -107.07964,       131.14066 ],
  [               0,               0,               0,               0,      -76.875029,       121.82056 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       4358.8266,      -6110.9603 ],
  [               0,               0,               0,               0,        2.536192,      -3.2276786 ],
  [               0,               0,               0,               0,      -1.5784469,       -1.085276 ],
  [               0,               0,               0,               0,      -79.500452,       114.28176 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer2_etaoffv4_35_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer2_etaoffv4_35_b = [
 [
  [               0,               0,               0,    -0.011374091,    0.0033197982,    0.0071265791 ],
  [               0,               0,               0,       1.0212886,     0.050343366,       3.9126132 ],
  [               0,               0,               0,       1.6589365,      -0.7642071,       -1.297329 ],
  [               0,               0,               0,      0.75888256,      -1.0668278,      0.29037259 ],
  [               0,               0,               0,     0.030701313,    -0.019125205,    0.0035799167 ],
  [               0,               0,               0,     -0.72021753,       1.6287807,     0.057511998 ],
  [               0,               0,               0,       16.516314,      -3.7092462,      -2.1698743 ],
 ],
 [
  [               0,               0,               0,    -0.019900448,     0.033153677,   -0.0040982915 ],
  [               0,               0,               0,       5.6340782,      -11.314056,        8.108847 ],
  [               0,               0,               0,      0.32506838,     -0.35526022,      -1.2339619 ],
  [               0,               0,               0,     -0.69218631,       3.7156603,       -2.414651 ],
  [               0,               0,               0,    -0.013463666,   -0.0003822328,    0.0097043157 ],
  [               0,               0,               0,     -0.31563726,      0.16161357,     0.085934385 ],
  [               0,               0,               0,      -10.796462,       11.518526,      -3.4223143 ],
 ],
 [
  [               0,               0,               0,               0,      0.10741029,    -0.075704737 ],
  [               0,               0,               0,               0,      0.55371607,        2.255595 ],
  [               0,               0,               0,               0,       5.2189123,      -5.4515874 ],
  [               0,               0,               0,               0,       -3.181877,       2.5620109 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0075096141,     0.011136694,     0.003651659 ],
  [               0,               0,               0,     -0.04790662,    -0.063797425,       3.4952542 ],
  [               0,               0,               0,       2.5407966,      -5.9033602,       1.8248984 ],
  [               0,               0,               0,     -0.25968387,      0.89746499,     -0.39268366 ],
  [               0,               0,               0,     0.007941236,     -0.01792843,     0.010707918 ],
  [               0,               0,               0,      -4.7042267,       11.564056,       -6.224576 ],
  [               0,               0,               0,       10.586308,      -20.732923,       7.5527966 ],
 ],
 [
  [               0,    0.0039221261,      0.00108002,  -0.00025351835,   -0.0013790653,    -0.013733667 ],
  [               0,       -159.4477,      -14.096783,       61.681499,       156.00977,       281.43423 ],
  [               0,      -26.855715,      -20.285238,       6.1209654,       1.2465721,       123.72953 ],
  [               0,      -13.270043,      -4.5609471,       14.574276,       13.211977,         22.7311 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0073083333,    0.0016853657,     0.021123916 ],
  [               0,               0,               0,      -438.84062,       4.8769888,       898.14733 ],
  [               0,               0,               0,       34.385807,      -104.13922,       78.302954 ],
  [               0,               0,               0,     -0.12479181,     -0.48209249,     -0.74945935 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer2_etaoffv4_35_e = [
 [
  [               0,               0,      -2.2021902,       3.1738442,       4.4591526,        -6.42265 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.011224927,      0.01671849 ],
  [               0,               0,               0,               0,      -12.000204,        57.35617 ],
  [               0,               0,               0,               0,       476.10156,      -701.16565 ],
  [               0,               0,               0,               0,      -359.81478,       536.06624 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.094526293,     -0.14071397 ],
  [               0,               0,               0,               0,       479.05574,      -719.56166 ],
  [               0,               0,               0,               0,      -245.22214,       369.22744 ],
  [               0,               0,               0,               0,       225.11529,      -340.55224 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.075761525,      0.24294412,     -0.18256514 ],
  [               0,               0,               0,       38.282529,      -127.65171,       108.47081 ],
  [               0,               0,               0,      -6.7928946,       21.823091,      -18.639264 ],
  [               0,               0,               0,       2.6078905,      -7.7452144,       5.6873692 ],
  [               0,               0,               0,    -0.016980732,     0.067563815,     -0.06485556 ],
  [               0,               0,               0,       6.6497945,      -24.505265,       23.074328 ],
  [               0,               0,               0,       30.682626,      -108.67543,       93.065277 ],
 ],
 [
  [               0,               0,               0,   -0.0041627722,  -0.00082540993,     0.031920092 ],
  [               0,               0,               0,      0.11359137,     0.040924643,       1.4945043 ],
  [               0,               0,               0,      -0.9308663,        3.921839,      -5.1921085 ],
  [               0,               0,               0,      -24.138406,       87.258159,      -77.568743 ],
  [               0,               0,               0,    -0.035534876,   -0.0039432636,      0.15577752 ],
  [               0,               0,               0,      -1.6130717,       6.8596124,      -7.2320494 ],
  [               0,               0,               0,      -10.277199,       43.416569,      -68.020742 ],
 ],
 [
  [               0,               0,               0,    -0.017051954,    0.0014449071,       0.1157522 ],
  [               0,               0,               0,       37.077805,      -173.11087,       203.36886 ],
  [               0,               0,               0,      -2.0910955,       8.7011938,      -9.8198094 ],
  [               0,               0,               0,      -9.9699196,       47.579165,      -56.746557 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.012208487 ],
  [               0,               0,               0,               0,               0,        1.633179 ],
  [               0,               0,               0,               0,               0,      -1.0685971 ],
  [               0,               0,               0,               0,               0,     0.042423041 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   0.00035391504,  -0.00060235712,  -0.00068227145 ],
  [               0,               0,               0,      -3.3372888,      -4.7028903,      -7.6841573 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,       42.574453,       199.18689,       274.34267 ],
  [               0,               0,               0,       20.369861,        170.5184,       123.36627 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer2_etaoffv4_35_b = [
 [
  [               0,               0,               0,    -0.030899623,    0.0074871173,     0.008209707 ],
  [               0,               0,               0,       5.4861314,    0.0066712539,       3.6771911 ],
  [               0,               0,               0,     -0.44351283,      0.22966979,      -1.3556629 ],
  [               0,               0,               0,       10.337563,      -8.1441497,       1.2293543 ],
  [               0,               0,               0,     0.043399887,     -0.05965575,      0.01031213 ],
  [               0,               0,               0,       1.3202186,      -1.1829954,      0.18528201 ],
  [               0,               0,               0,     -0.29707571,       3.0072097,      -3.0901691 ],
 ],
 [
  [               0,               0,               0,               0,    0.0062110078,    0.0021336059 ],
  [               0,               0,               0,               0,      -4.3845542,       7.2871222 ],
  [               0,               0,               0,               0,     -0.34597996,      -1.3553303 ],
  [               0,               0,               0,               0,      0.51573552,     -0.14760188 ],
  [               0,               0,               0,               0,   -0.0012413171,    0.0017892393 ],
  [               0,               0,               0,               0,     -0.73838342,       1.2214591 ],
  [               0,               0,               0,               0,      -2.5258438,       -6.842154 ],
 ],
 [
  [               0,               0,               0,               0,      0.17751401,     -0.13525602 ],
  [               0,               0,               0,               0,      -27.053337,       25.910636 ],
  [               0,               0,               0,               0,       29.830312,      -25.659155 ],
  [               0,               0,               0,               0,      -27.095473,       22.362353 ],
  [               0,               0,               0,               0,    -0.027419004,     0.025590263 ],
  [               0,               0,               0,               0,      -7.7174045,       6.5236809 ],
  [               0,               0,               0,               0,      -4.7626483,      -5.3977536 ],
 ],
 [
  [               0,               0,               0,    -0.019771479,     0.044482874,     -0.01723546 ],
  [               0,               0,               0,       8.3928245,      -19.908194,       15.096178 ],
  [               0,               0,               0,     -0.22828117,      0.44406867,      -1.6621984 ],
  [               0,               0,               0,      0.10788378,      0.21394482,     -0.16190104 ],
  [               0,               0,               0,    0.0065020314,     -0.01268958,    0.0068342999 ],
  [               0,               0,               0,      0.49878208,         -2.1767,       2.2284005 ],
  [               0,               0,               0,          5.7512,      -13.021546,       5.0815579 ],
 ],
 [
  [               0,               0,      0.13624849,     0.037908199,    -0.099737998,     -0.27034497 ],
  [               0,               0,      -2310.7123,       3285.9041,       4045.7397,      -5764.6761 ],
  [               0,               0,      -507.44502,       713.18024,       958.93235,      -1349.1611 ],
  [               0,               0,        527.8999,      -723.56762,      -990.90777,       1358.3267 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.049706432,    0.0050013896,    -0.088463618 ],
  [               0,               0,               0,      0.73856073,        8.423863,       26.360639 ],
  [               0,               0,               0,      -23.280809,       4.3762436,       37.577142 ],
  [               0,               0,               0,      0.13024132,      0.19478293,     0.063056079 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer2_etaoffv4_35_e = [
 [
  [               0,               0,      -1.1927782,       1.7461157,       2.3219997,      -3.4009812 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     -0.33208252,       0.4894997,      0.77539928,      -1.1410818 ],
  [               0,               0,      -1379.7683,        1878.986,       2976.0978,      -4039.0601 ],
  [               0,               0,      -2543.5091,       3796.2148,       5642.8025,      -8421.5028 ],
  [               0,               0,       1276.9589,      -1883.8389,      -2839.5116,       4188.9495 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.034681517,    -0.051336049 ],
  [               0,               0,               0,               0,       70.095049,      -105.07081 ],
  [               0,               0,               0,               0,      -546.01509,       823.56059 ],
  [               0,               0,               0,               0,       597.11494,      -902.53438 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.093949795,      0.32159274,     -0.26508285 ],
  [               0,               0,               0,       53.502896,      -182.38278,       158.15098 ],
  [               0,               0,               0,      -6.6838659,       22.890202,      -20.703443 ],
  [               0,               0,               0,       1.2928948,      -4.3678281,       3.6473271 ],
  [               0,               0,               0,    0.0092629674,    -0.031570789,       0.0272196 ],
  [               0,               0,               0,       17.335593,      -59.340203,        52.18739 ],
  [               0,               0,               0,       28.743341,      -99.186555,        79.95813 ],
 ],
 [
  [               0,               0,               0,     0.050590464,     -0.20095583,      0.20572414 ],
  [               0,               0,               0,      -4.3668393,       13.640871,      -6.2284802 ],
  [               0,               0,               0,       3.9363384,      -15.904189,       14.737351 ],
  [               0,               0,               0,      -4.3529903,       17.430038,      -17.399407 ],
  [               0,               0,               0,   -0.0041161936,     0.023235978,    -0.028831138 ],
  [               0,               0,               0,       7.3364277,      -32.384443,       35.874975 ],
  [               0,               0,               0,       18.014209,      -72.593063,       71.528159 ],
 ],
 [
  [               0,               0,               0,    -0.016493148,   0.00039662162,      0.10633255 ],
  [               0,               0,               0,       46.445832,      -213.52077,       247.14346 ],
  [               0,               0,               0,        10.21621,      -49.593332,       59.055599 ],
  [               0,               0,               0,      -12.225625,       58.218885,      -69.286659 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.013659569,     0.034730127 ],
  [               0,               0,               0,               0,       7.3821168,       21.634111 ],
  [               0,               0,               0,               0,      -349.71623,       859.94084 ],
  [               0,               0,               0,               0,       489.93352,      -1206.1495 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   0.00048391576,   0.00023108789,  -0.00035031504,   -0.0079215314 ],
  [               0,               0,      -1.1884292,      -1.3920109,      -3.4523011,      -4.5618167 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,      -142.98585,      -114.19301,       299.30048,       2346.9769 ],
  [               0,               0,       9.8093018,       1.3055879,      -10.657516,       14.137981 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer2_etaoffv4_35_b = [
 [
  [               0,               0,               0,    0.0051238973,   0.00051020933,    0.0069611816 ],
  [               0,               0,               0,      -2.9410994,     -0.16714782,       4.5971475 ],
  [               0,               0,               0,      -1.7589393,      0.95870481,       -1.497857 ],
  [               0,               0,               0,       14.313523,       -5.535988,      0.85373872 ],
  [               0,               0,               0,     0.086783026,    -0.031768981,    0.0059750838 ],
  [               0,               0,               0,       2.7552779,      -1.4764768,       0.2846177 ],
  [               0,               0,               0,      -18.039307,       7.3117193,      -3.1290057 ],
 ],
 [
  [               0,               0,               0,    -0.016895988,     0.020760675,    0.0010829914 ],
  [               0,               0,               0,     0.022266857,      -1.6998801,       5.1975078 ],
  [               0,               0,               0,      -3.4641406,       3.6447296,      -2.3704736 ],
  [               0,               0,               0,       1.4907403,      -1.1916017,       0.2939772 ],
  [               0,               0,               0,    0.0030085272,   -0.0022585634,    0.0010378035 ],
  [               0,               0,               0,       6.4569711,      -7.9426011,       2.9377958 ],
  [               0,               0,               0,        5.380514,      -8.4223078,      -5.5305437 ],
 ],
 [
  [               0,               0,               0,               0,      0.11318289,    -0.082281001 ],
  [               0,               0,               0,               0,      -8.4008787,       10.583237 ],
  [               0,               0,               0,               0,       11.397207,      -10.716297 ],
  [               0,               0,               0,               0,      -4.8762756,       4.1388791 ],
  [               0,               0,               0,               0,    -0.034203261,     0.029986768 ],
  [               0,               0,               0,               0,       17.436053,      -13.443784 ],
  [               0,               0,               0,               0,      -67.766351,       51.346559 ],
 ],
 [
  [               0,               0,               0,   -0.0041361333,    0.0098834575,    0.0020409533 ],
  [               0,               0,               0,       2.8844967,      -7.6774869,       8.4818004 ],
  [               0,               0,               0,     -0.86169117,       1.4583549,      -2.0112131 ],
  [               0,               0,               0,        1.728238,      -1.5213401,       1.5625745 ],
  [               0,               0,               0,    0.0091085727,   -0.0040464082,    0.0082753372 ],
  [               0,               0,               0,      0.43079932,     -0.79110289,      0.42456757 ],
  [               0,               0,               0,      0.50475337,      -1.5340519,      -1.9207911 ],
 ],
 [
  [               0,               0,               0,      0.45615378,     -0.20635108,     -0.55191651 ],
  [               0,               0,               0,       1151.1262,      -3134.1932,        2135.907 ],
  [               0,               0,               0,      -201.97554,       588.25318,       -428.2769 ],
  [               0,               0,               0,       1452.5864,      -3980.9345,       2727.5745 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.013849135,    0.0029374221,     -0.01768418 ],
  [               0,               0,               0,      -368.18282,       3.7670724,       759.53194 ],
  [               0,               0,               0,       43.862818,      -112.52477,       70.363558 ],
  [               0,               0,               0,     0.025139733,     -0.20346987,     -0.44117082 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer2_etaoffv4_35_e = [
 [
  [               0,               0,      -0.8784295,       1.2813701,       1.6499131,      -2.4086638 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.026064392,   -0.0049809283,      0.02549985,     0.062925496 ],
  [               0,               0,       1157.3538,      -1689.4779,      -2477.4108,       3617.6239 ],
  [               0,               0,      -187.78095,       269.78026,       404.81747,      -582.53248 ],
  [               0,               0,      -172.49161,       254.78449,       375.55849,      -554.57313 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.027223694,    -0.040439915 ],
  [               0,               0,               0,               0,       1755.0861,      -2636.3169 ],
  [               0,               0,               0,               0,      -474.73341,       714.96384 ],
  [               0,               0,               0,               0,       655.66942,      -990.94833 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   2.6362684e-05,  -0.00010146573,    0.0062189213 ],
  [               0,               0,               0,       6.6558596,      -22.573365,        22.94443 ],
  [               0,               0,               0,      -4.3524753,       13.671289,      -11.943754 ],
  [               0,               0,               0,       1.6920264,      -4.7921585,       3.2592431 ],
  [               0,               0,               0,     0.015488962,    -0.052617844,     0.045518448 ],
  [               0,               0,               0,      -1.0370322,       4.7660769,      -4.4882648 ],
  [               0,               0,               0,      -5.5975731,       15.056593,      -11.006632 ],
 ],
 [
  [               0,               0,               0,     0.014454959,    -0.065026684,     0.080888659 ],
  [               0,               0,               0,      -6.3962884,       26.552085,      -24.524513 ],
  [               0,               0,               0,       1.4338724,       -5.907271,       4.8226515 ],
  [               0,               0,               0,      -75.205122,       287.12509,      -252.99057 ],
  [               0,               0,               0,    -0.071295271,    0.0054734343,      0.45289774 ],
  [               0,               0,               0,    -0.091807553,      0.39109927,     -0.40669864 ],
  [               0,               0,               0,       5.5464411,      -22.510507,       19.843233 ],
 ],
 [
  [               0,               0,               0,   -0.0029040904,   -0.0039398513,     0.035316287 ],
  [               0,               0,               0,       23.922193,      -111.61065,       132.68095 ],
  [               0,               0,               0,      -12.173855,       58.209465,      -70.800759 ],
  [               0,               0,               0,      -6.4306956,       45.476933,      -63.836302 ],
  [               0,               0,               0,    0.0025254675,      0.00860172,      0.04104122 ],
  [               0,               0,               0,       1.3099511,      -6.7503556,       8.6808462 ],
  [               0,               0,               0,       17.201434,      -79.761371,       89.352686 ],
 ],
 [
  [               0,               0,               0,               0,       3.1589149,      -7.7343055 ],
  [               0,               0,               0,               0,      -22.273966,       56.668134 ],
  [               0,               0,               0,               0,       51.303841,      -127.44998 ],
  [               0,               0,               0,               0,      -41.150917,       101.34924 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0022316966,    0.0017970119,   -0.0038517131,    -0.035471675 ],
  [               0,               0,      -1.1922079,      -1.3841032,      -3.4309828,      -4.5048735 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,      -8.8637632,      -4.9100212,       22.582269,       158.85004 ],
  [               0,               0,       0.6138454,       1.5223724,       3.7446228,       8.8343182 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer2_etaoffv4_35_b = [
 [
  [               0,               0,      0.52133444,     -0.40319463,     0.081836612,    0.0070958763 ],
  [               0,               0,      -78.512493,       62.321317,       -12.39601,       4.3113001 ],
  [               0,               0,       3.2564257,      -3.1706325,      0.55717369,      -1.2989125 ],
  [               0,               0,       1.1099424,     -0.74957875,      0.23296122,    -0.033127981 ],
  [               0,               0,      0.14280864,     -0.11389231,     0.024192845,  -0.00034310119 ],
  [               0,               0,      -12.368504,        22.22359,      -6.5524368,       1.6957543 ],
  [               0,               0,       31.605503,      -16.092716,       3.0909476,       -5.022579 ],
 ],
 [
  [               0,               0,               0,    0.0085024197,   -0.0081143959,    0.0089450557 ],
  [               0,               0,               0,      -10.147887,       10.285901,       1.8917861 ],
  [               0,               0,               0,       1.5110565,      -2.2408509,      -0.6461694 ],
  [               0,               0,               0,       0.4661754,     0.093103891,     -0.14537667 ],
  [               0,               0,               0,    0.0069952263,   -0.0072619175,    0.0024538391 ],
  [               0,               0,               0,      -1.6512757,      0.94030227,      0.70477331 ],
  [               0,               0,               0,       6.2342467,      -11.471531,        2.574183 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0030801497,     0.010324778 ],
  [               0,               0,               0,               0,       21.900847,      -13.485048 ],
  [               0,               0,               0,               0,       1.8300105,      -3.0093329 ],
  [               0,               0,               0,               0,      -3.2943909,       2.8388796 ],
  [               0,               0,               0,               0,    0.0054482736,   -0.0029906182 ],
  [               0,               0,               0,               0,       4.8407524,      -3.3246084 ],
  [               0,               0,               0,               0,       2.1848224,      -4.6662339 ],
 ],
 [
  [               0,               0,               0,    -0.013273765,     0.027334499,   -0.0060494916 ],
  [               0,               0,               0,       5.7626949,      -12.842549,       10.724041 ],
  [               0,               0,               0,      0.32682174,      -1.1206539,      -0.6176404 ],
  [               0,               0,               0,    -0.077685735,      0.75636615,      -0.5751457 ],
  [               0,               0,               0,    0.0062061316,    -0.011788771,    0.0060400926 ],
  [               0,               0,               0,      -1.8967688,       3.5644975,      -1.0874449 ],
  [               0,               0,               0,       10.442432,       -24.64794,       12.167822 ],
 ],
 [
  [               0,               0,               0,      0.42350209,     -0.20094142,     -0.50066572 ],
  [               0,               0,               0,       1577.4266,      -4278.4567,       2903.5981 ],
  [               0,               0,               0,      -250.92133,       716.70072,      -512.54671 ],
  [               0,               0,               0,         926.423,      -2530.7808,       1728.5252 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0020432064,    0.0031193475,     0.014884865 ],
  [               0,               0,               0,      -383.41579,       4.9089834,       790.17692 ],
  [               0,               0,               0,        25.99954,      -64.864307,       38.714769 ],
  [               0,               0,               0,   -0.0087462396,     -0.14177106,      -0.4221815 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer2_etaoffv4_35_e = [
 [
  [               0,               0,      -1.2832212,       1.8559125,       2.4863442,      -3.5961517 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.020358626,    0.0081498705,     0.028820122,    0.0090562032 ],
  [               0,               0,      -156.83975,        262.8208,       365.55827,       -603.4071 ],
  [               0,               0,      -104.52106,       143.52413,       223.04256,      -307.23161 ],
  [               0,               0,      -87.847734,       131.66135,       193.03515,      -289.07284 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.025445758,     -0.03611846 ],
  [               0,               0,               0,               0,         145.051,      -215.32923 ],
  [               0,               0,               0,               0,      -206.88144,       311.46172 ],
  [               0,               0,               0,               0,       210.64196,      -318.38543 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      0.05661839,     -0.19200891,       0.1690949 ],
  [               0,               0,               0,      -19.297714,       65.746523,      -52.054172 ],
  [               0,               0,               0,       5.1415795,       -17.68899,       13.826085 ],
  [               0,               0,               0,      -12.019944,       41.205077,      -34.770437 ],
  [               0,               0,               0,   -0.0037416435,     0.013241429,   -0.0080909384 ],
  [               0,               0,               0,      -5.3051131,       18.067566,      -15.160515 ],
  [               0,               0,               0,       24.170827,      -82.644527,       67.939737 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0027640892,     0.014274999 ],
  [               0,               0,               0,               0,     -0.68001666,       4.3924361 ],
  [               0,               0,               0,               0,    -0.014691404,      -1.2057748 ],
  [               0,               0,               0,               0,      -26.147205,       43.985481 ],
  [               0,               0,               0,               0,      0.23041228,     -0.38505271 ],
  [               0,               0,               0,               0,    -0.022052865,     0.054312805 ],
  [               0,               0,               0,               0,     -0.71027628,       1.1406735 ],
 ],
 [
  [               0,               0,               0,   7.7459868e-05,   -0.0037252691,     0.017912089 ],
  [               0,               0,               0,       49.718538,      -238.72377,       289.14386 ],
  [               0,               0,               0,       10.614272,      -50.800223,       59.573345 ],
  [               0,               0,               0,       3.2417713,      -9.2894017,      -4.0436774 ],
  [               0,               0,               0,   -0.0087944082,     0.014348011,     0.092888251 ],
  [               0,               0,               0,      -1.3835668,       6.5617512,      -7.7671766 ],
  [               0,               0,               0,       10.500887,      -52.903951,       66.147832 ],
 ],
 [
  [               0,               0,               0,               0,     0.014096254,   0.00052273452 ],
  [               0,               0,               0,               0,     -0.63384854,       3.4413347 ],
  [               0,               0,               0,               0,        41.59313,      -103.52332 ],
  [               0,               0,               0,               0,       -38.98291,       96.015972 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.019876924,    -0.030354186,    -0.035076429,    -0.029145916 ],
  [               0,               0,      -1.1725724,      -1.3897228,      -3.4684593,      -4.6505885 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,       1.0764896,       1.3322087,      0.10528477,      -8.8075928 ],
  [               0,               0,      0.14976211,      0.39359606,       1.0755015,       2.4398127 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer2_etaoffv4_35_b = [
 [
  [               0,               0,               0,    0.0071909688,    -0.002686103,    0.0072335273 ],
  [               0,               0,               0,      0.14800075,     -0.52170325,       4.9603491 ],
  [               0,               0,               0,      0.61348682,     -0.29932431,      -1.3134969 ],
  [               0,               0,               0,     -0.92156706,      0.37831232,     -0.06284076 ],
  [               0,               0,               0,     0.011312627,   -0.0045749481,   0.00093624684 ],
  [               0,               0,               0,      -16.663032,       6.7133508,     0.080795286 ],
  [               0,               0,               0,       1.7763594,     -0.19539799,      -1.3368727 ],
 ],
 [
  [               0,               0,               0,   -0.0058927755,    0.0074871636,    0.0056129422 ],
  [               0,               0,               0,      -2.5618054,       1.7382354,       4.1969897 ],
  [               0,               0,               0,       -2.060352,       2.0510053,      -1.8655818 ],
  [               0,               0,               0,       67.902225,      -99.850956,        83.61706 ],
  [               0,               0,               0,      0.27773294,     -0.46719275,      0.56503474 ],
  [               0,               0,               0,       0.1665817,     -0.13673884,     0.030781437 ],
  [               0,               0,               0,       1.0358419,      -1.5479744,      -2.5407659 ],
 ],
 [
  [               0,               0,               0,               0,     0.058677522,    -0.038906315 ],
  [               0,               0,               0,               0,      0.36163671,        3.821631 ],
  [               0,               0,               0,               0,       12.505321,      -11.404365 ],
  [               0,               0,               0,               0,      -20.545612,       18.635859 ],
  [               0,               0,               0,               0,     0.013587091,     0.006835658 ],
  [               0,               0,               0,               0,      -3.9992011,       3.2583549 ],
  [               0,               0,               0,               0,       4.4884538,      -6.5840612 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00056717953,    0.0085820931 ],
  [               0,               0,               0,               0,     -0.60546198,       4.5334042 ],
  [               0,               0,               0,               0,      -0.3712771,      -1.0610306 ],
  [               0,               0,               0,               0,       1.1557833,       129.69416 ],
  [               0,               0,               0,               0,    -0.067047338,       1.1154212 ],
  [               0,               0,               0,               0,     0.015654298,   -0.0054596398 ],
  [               0,               0,               0,               0,   -0.0049111888,      -3.1219546 ],
 ],
 [
  [               0,               0,      0.18725876,   -0.0018694289,     -0.17520206,     -0.21998562 ],
  [               0,               0,       629.32443,      -844.43323,      -1157.9997,        1556.497 ],
  [               0,               0,      -42.668216,       73.518215,       73.103527,      -130.08252 ],
  [               0,               0,       510.57731,      -696.10704,       -960.7316,       1309.9552 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.015726091,    0.0037191993,      0.04469053 ],
  [               0,               0,               0,      -330.89448,       6.3124785,       685.75787 ],
  [               0,               0,               0,       21.585236,      -58.486193,       38.200621 ],
  [               0,               0,               0,      0.03336139,    -0.046947898,      -0.3042582 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer2_etaoffv4_35_e = [
 [
  [               0,               0,      -2.3516099,       3.4053121,       4.6523793,      -6.7373693 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.013564069,    -0.024000543,     0.069863992 ],
  [               0,               0,               0,        -112.638,       418.34416,      -368.19839 ],
  [               0,               0,               0,      -77.148628,       205.27553,      -136.55985 ],
  [               0,               0,               0,      -34.703219,       108.19109,      -83.840208 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.024843386,     0.040860052 ],
  [               0,               0,               0,               0,        207.7276,      -309.80887 ],
  [               0,               0,               0,               0,       -104.1277,       156.34821 ],
  [               0,               0,               0,               0,       107.17986,      -161.98512 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.022879438,     0.066536362,    -0.035718457 ],
  [               0,               0,               0,       20.842781,      -69.308259,       60.657078 ],
  [               0,               0,               0,      -1.3908277,       4.2571773,      -4.4211057 ],
  [               0,               0,               0,     0.095656745,     -0.10982724,    -0.093477407 ],
  [               0,               0,               0,    0.0092731414,    -0.034605431,     0.032640247 ],
  [               0,               0,               0,       5.4867846,      -16.750377,       13.999127 ],
  [               0,               0,               0,       6.3255914,      -21.026087,       12.441988 ],
 ],
 [
  [               0,               0,               0,   -0.0066293657,     0.026851766,    -0.018181277 ],
  [               0,               0,               0,       2.0277242,      -9.9255703,       14.877861 ],
  [               0,               0,               0,      0.19660906,     -0.64669249,     -0.76827166 ],
  [               0,               0,               0,      -25.793458,       107.02685,      -110.74373 ],
  [               0,               0,               0,    0.0029593263,    -0.012807723,     0.024909522 ],
  [               0,               0,               0,      0.28527731,      -1.1288478,       1.1330434 ],
  [               0,               0,               0,       20.761743,      -86.163978,       87.629235 ],
 ],
 [
  [               0,               0,               0,   -0.0049514492,   -0.0066203041,     0.053748215 ],
  [               0,               0,               0,       56.480972,      -264.27058,       311.64228 ],
  [               0,               0,               0,       6.2740804,      -30.510351,       35.857388 ],
  [               0,               0,               0,      -28.202981,       86.573094,      -63.769635 ],
  [               0,               0,               0,    -0.043260476,    -0.025548239,      0.66469947 ],
  [               0,               0,               0,    -0.086583426,       0.4186161,      -0.5050387 ],
  [               0,               0,               0,      -3.9871646,       19.153216,      -24.063887 ],
 ],
 [
  [               0,               0,               0,               0,    0.0011462029,    0.0072245049 ],
  [               0,               0,               0,               0,      0.54584136,       1.3403335 ],
  [               0,               0,               0,               0,       6.6897301,      -17.783603 ],
  [               0,               0,               0,               0,      0.43820546,      -1.0059454 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0016363872,   0.00053429455,   -0.0077787409,   -0.0089035366 ],
  [               0,               0,       -1.197004,      -1.3363845,      -3.3099147,      -4.1986093 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,       4.9611488,       8.7647496,        30.85551,      -79.666584 ],
  [               0,               0,      0.93748005,       5.0638559,       8.2682625,       14.380198 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer2_etaoffv4_35_b = [
 [
  [               0,               0,               0,    -0.025225349,     0.012263965,    0.0060004597 ],
  [               0,               0,               0,       7.6369894,      -4.4780217,       5.5013323 ],
  [               0,               0,               0,      0.60523016,     -0.14687352,      -1.3223874 ],
  [               0,               0,               0,      0.07367651,    -0.055570188,    -0.038623971 ],
  [               0,               0,               0,    0.0076709842,   -0.0035536365,   0.00078492243 ],
  [               0,               0,               0,       13.324669,      -5.6507276,       1.1780536 ],
  [               0,               0,               0,      -12.920296,       6.1344576,      -1.6706137 ],
 ],
 [
  [               0,               0,               0,     0.013441217,    -0.011771577,     0.010855955 ],
  [               0,               0,               0,      -6.8477088,       5.7601452,       3.4988459 ],
  [               0,               0,               0,       1.1653314,      -1.2080678,       -1.054851 ],
  [               0,               0,               0,    -0.063339175,      0.12435049,    -0.050288825 ],
  [               0,               0,               0,   -0.0023378001,    0.0019815218,  -0.00056561802 ],
  [               0,               0,               0,       2.7227287,      -1.7027442,      -1.5649706 ],
  [               0,               0,               0,       14.713267,      -15.635191,       -19.61638 ],
 ],
 [
  [               0,               0,               0,               0,     0.019510378,    -0.010793439 ],
  [               0,               0,               0,               0,     -0.45340261,        5.975245 ],
  [               0,               0,               0,               0,       22.149424,      -19.232224 ],
  [               0,               0,               0,               0,      -12.553067,       10.085252 ],
  [               0,               0,               0,               0,   -0.0059919721,    0.0063867464 ],
  [               0,               0,               0,               0,       2.4617996,      -0.9170814 ],
  [               0,               0,               0,               0,       16.365555,      -14.824196 ],
 ],
 [
  [               0,               0,               0,   -0.0043470387,     0.012915518,   -0.0017207553 ],
  [               0,               0,               0,       4.0478854,      -10.750018,       11.196366 ],
  [               0,               0,               0,      0.53526808,      -1.3692581,     -0.60028304 ],
  [               0,               0,               0,     -0.12970106,      0.62270431,      -0.4275392 ],
  [               0,               0,               0,    0.0032368891,   -0.0071944408,    0.0046277872 ],
  [               0,               0,               0,     -0.79712581,      0.85636679,      0.76045465 ],
  [               0,               0,               0,     -0.13973614,      -1.5717672,     -0.17675062 ],
 ],
 [
  [               0,               0,      0.18421948,    -0.046706709,     -0.20444023,      -0.0924517 ],
  [               0,               0,        649.2703,      -849.89548,      -1182.7743,       1551.0835 ],
  [               0,               0,      -852.09426,       1189.0753,       1588.6436,       -2219.003 ],
  [               0,               0,       740.49358,      -1008.9012,      -1392.4989,       1897.3761 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.017515085,   -0.0038397024 ],
  [               0,               0,               0,               0,        11.11619,      0.51904043 ],
  [               0,               0,               0,               0,      -8.1348035,       9.9380838 ],
  [               0,               0,               0,               0,     0.062227988,     -0.17533504 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer2_etaoffv4_35_e = [
 [
  [       77.471796,      -156.25506,      -124.75444,       183.49973,       448.20937,      -461.22758 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,      0.23731153,     -0.23029168,     -0.42379285,      0.37669533 ],
  [               0,               0,       985.50752,      -1479.6549,      -2047.5387,        3078.677 ],
  [               0,               0,       823.10979,      -1193.8164,      -1856.1163,       2692.0861 ],
  [               0,               0,       -800.9202,       1182.0927,        1735.554,      -2561.0746 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.025774793,    -0.035402165 ],
  [               0,               0,               0,               0,       80.541119,      -117.60904 ],
  [               0,               0,               0,               0,      -138.11465,       207.55172 ],
  [               0,               0,               0,               0,       134.58506,      -203.39896 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   0.00045657751,  -0.00065123339,    0.0082972362 ],
  [               0,               0,               0,        6.869856,      -24.221874,       24.937806 ],
  [               0,               0,               0,     -0.31804482,       1.0862669,      -2.2844205 ],
  [               0,               0,               0,     0.051717288,      0.13386627,      0.21481584 ],
  [               0,               0,               0,   0.00080908571,     0.000198666,    0.0021764411 ],
  [               0,               0,               0,     -0.68396136,       2.1475258,      -1.5126925 ],
  [               0,               0,               0,     -0.29215981,       1.2501898,      -4.1935298 ],
 ],
 [
  [               0,               0,               0,    0.0061229851,    -0.019026861,       0.0218101 ],
  [               0,               0,               0,     -0.13600419,      -2.6319811,       9.3079901 ],
  [               0,               0,               0,      0.54644357,       -1.658116,     -0.23740816 ],
  [               0,               0,               0,        -3.58617,       14.141105,       -13.76795 ],
  [               0,               0,               0,    0.0010437417,   -0.0015112022,   0.00017227562 ],
  [               0,               0,               0,       4.8107867,      -21.079551,       23.226872 ],
  [               0,               0,               0,       12.615829,       -47.68344,       42.531478 ],
 ],
 [
  [               0,               0,               0,     0.060202266,     -0.24565268,      0.24773318 ],
  [               0,               0,               0,       128.28383,      -627.98088,       771.93015 ],
  [               0,               0,               0,       49.915859,      -247.10689,       304.33813 ],
  [               0,               0,               0,      -12.862572,       65.689138,      -83.573738 ],
  [               0,               0,               0,     0.028384816,     -0.15139293,      0.20146773 ],
  [               0,               0,               0,      -6.2008623,       28.172922,      -30.981567 ],
  [               0,               0,               0,      -119.39571,       562.42009,      -663.51331 ],
 ],
 [
  [               0,               0,               0,               0,    0.0020317168,    0.0055205986 ],
  [               0,               0,               0,               0,      -8.6129084,       23.885609 ],
  [               0,               0,               0,               0,       2.8307475,       -8.241935 ],
  [               0,               0,               0,               0,       2.4206261,      -5.8975689 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,  -0.00014178204,  -0.00046195207,     0.020372776,     -0.04553431 ],
  [               0,               0,      -1.2225763,      -1.3634033,      -3.3848603,      -4.4055718 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,      -3.8802456,       50.086802,      -23.537696,       50.790188 ],
  [               0,               0,       13.439137,       4.9905224,      -17.318856,       16.539133 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer2_etaoffv4_35_b = [
 [
  [               0,               0,               0,     -0.01596717,    0.0099711771,    0.0060359385 ],
  [               0,               0,               0,       7.8226669,      -4.9691669,       5.7610333 ],
  [               0,               0,               0,      -1.6520717,       0.5116499,      -1.3133249 ],
  [               0,               0,               0,       1.4542287,     -0.51016917,    -0.039740461 ],
  [               0,               0,               0,    -0.001125186,  -0.00076902337,   0.00070208063 ],
  [               0,               0,               0,      0.99254859,      -1.8725446,       1.0306954 ],
  [               0,               0,               0,      -27.771969,        12.13586,       -1.812984 ],
 ],
 [
  [               0,               0,               0,   -0.0086702094,     0.011658237,    0.0033578489 ],
  [               0,               0,               0,       6.3043451,      -9.5624157,        8.478877 ],
  [               0,               0,               0,      -1.6141916,        1.561252,      -1.6959676 ],
  [               0,               0,               0,      0.94315739,     -0.81540452,      0.12625242 ],
  [               0,               0,               0,   0.00076806019,  -0.00077749367,   0.00062006129 ],
  [               0,               0,               0,     -0.82837933,       0.8923847,      0.57881749 ],
  [               0,               0,               0,      -1.2695042,      -1.0995857,     -0.36840138 ],
 ],
 [
  [               0,               0,               0,               0,       0.0191544,   -0.0069294854 ],
  [               0,               0,               0,               0,      -5.1153648,       8.4830368 ],
  [               0,               0,               0,               0,       7.8202409,       -7.551176 ],
  [               0,               0,               0,               0,      -5.3540232,       4.1165711 ],
  [               0,               0,               0,               0,    0.0072175472,   -0.0044955813 ],
  [               0,               0,               0,               0,       6.1498598,       -4.747294 ],
  [               0,               0,               0,               0,       15.550719,      -12.807871 ],
 ],
 [
  [               0,               0,               0,   -0.0010063367,      0.00141976,    0.0077612869 ],
  [               0,               0,               0,       1.0495263,      -2.5111539,       5.6410335 ],
  [               0,               0,               0,     -0.37476303,      0.61093694,       -1.591934 ],
  [               0,               0,               0,      0.18851593,     -0.33850565,      0.76062412 ],
  [               0,               0,               0,   -0.0025417576,    0.0047193125,    0.0029616856 ],
  [               0,               0,               0,      0.92384047,      -1.9009421,       1.0511798 ],
  [               0,               0,               0,       -2.651171,       5.9861701,       -6.156389 ],
 ],
 [
  [               0,               0,       4.9810088,      -6.6070083,      -9.3364173,        12.39032 ],
  [               0,               0,      -4191.0959,        5815.625,       7783.1148,      -10791.447 ],
  [               0,               0,      -4068.5979,       5711.0758,       7364.7591,      -10350.401 ],
  [               0,               0,      -1711.6789,       2379.0438,       3197.4721,       -4443.215 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      0.16140874,    0.0059233781,     -0.32024143 ],
  [               0,               0,               0,        20.86618,       5.6645266,      -22.573501 ],
  [               0,               0,               0,      -97.234641,       235.92809,      -142.20913 ],
  [               0,               0,               0,    -0.017155537,       0.2547568,      -0.1051026 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer2_etaoffv4_35_e = [
 [
  [       88.919297,      -179.53805,       -142.7821,       210.94202,       513.29881,      -528.79736 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,       280.91196,      -988.42953,       681.44213,       834.92173,      -871.79469 ],
  [               0,      -489.70307,       839.65902,       372.26744,      -236.29863,      -834.62849 ],
  [               0,       467.09525,      -734.48252,      -463.07639,       212.71317,       838.52461 ],
  [               0,       46.156835,      -82.237768,      -31.014707,       33.544387,       63.462979 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0033688341,     0.015661895 ],
  [               0,               0,               0,               0,         30.2516,      -42.811427 ],
  [               0,               0,               0,               0,       19.640006,      -30.958028 ],
  [               0,               0,               0,               0,      -17.737405,       26.884897 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0061604774,     0.019892816,    -0.010832008 ],
  [               0,               0,               0,        12.58216,      -44.443158,       44.163635 ],
  [               0,               0,               0,      -12.534482,        42.35932,      -36.947949 ],
  [               0,               0,               0,       6.5437781,      -22.013583,       18.445277 ],
  [               0,               0,               0,  -0.00099685271,    0.0044830347,     -0.00331486 ],
  [               0,               0,               0,     -0.24663793,       0.9746914,    -0.026894965 ],
  [               0,               0,               0,      -18.491742,        62.50268,      -54.008294 ],
 ],
 [
  [               0,               0,               0,    0.0019355508,   -0.0038348426,    0.0083546775 ],
  [               0,               0,               0,      0.37192729,      -4.2736487,       10.628718 ],
  [               0,               0,               0,      0.53744509,      -2.0472598,      0.64024594 ],
  [               0,               0,               0,      -4.5152871,       21.343722,      -25.288012 ],
  [               0,               0,               0,   -0.0049599971,  -0.00015588269,     0.028386056 ],
  [               0,               0,               0,       1.0056472,      -4.1181475,       4.2524778 ],
  [               0,               0,               0,       3.3591858,      -15.375465,       16.541993 ],
 ],
 [
  [               0,               0,               0,   -0.0010762453,   0.00099127856,     0.013638344 ],
  [               0,               0,               0,        31.40569,      -149.91702,       181.72729 ],
  [               0,               0,               0,       9.7746982,      -45.885932,       52.533419 ],
  [               0,               0,               0,       -3.018459,       37.607679,      -60.103791 ],
  [               0,               0,               0,     0.041102701,   -0.0062089152,     -0.09648523 ],
  [               0,               0,               0,      -3.4760687,        16.30287,      -19.076434 ],
  [               0,               0,               0,       9.1506789,      -42.662134,       46.614314 ],
 ],
 [
  [               0,               0,               0,               0,   0.00064627531,    0.0093934101 ],
  [               0,               0,               0,               0,      -26.069336,       66.915436 ],
  [               0,               0,               0,               0,       5.6263847,      -15.093516 ],
  [               0,               0,               0,               0,      0.21613276,     -0.47549451 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0012078319,    0.0018088218,    -0.003171623,    -0.021582016 ],
  [               0,               0,      -1.1804019,       -1.394585,      -3.4701312,        -4.61861 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,       3.3727051,       9.7544048,       8.9857363,      -15.206654 ],
  [               0,               0,       0.9344602,       1.0551092,       8.7515192,       16.498668 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]


ele_layer2_etaoffv4_35_b = [
    ele5_layer2_etaoffv4_35_b,
    ele10_layer2_etaoffv4_35_b,
    ele25_layer2_etaoffv4_35_b,
    ele50_layer2_etaoffv4_35_b,
    ele75_layer2_etaoffv4_35_b,
    ele100_layer2_etaoffv4_35_b,
    ele200_layer2_etaoffv4_35_b,
    ele500_layer2_etaoffv4_35_b,
    ele1000_layer2_etaoffv4_35_b,
    ]

ele_layer2_etaoffv4_35_e = [
    ele5_layer2_etaoffv4_35_e,
    ele10_layer2_etaoffv4_35_e,
    ele25_layer2_etaoffv4_35_e,
    ele50_layer2_etaoffv4_35_e,
    ele75_layer2_etaoffv4_35_e,
    ele100_layer2_etaoffv4_35_e,
    ele200_layer2_etaoffv4_35_e,
    ele500_layer2_etaoffv4_35_e,
    ele1000_layer2_etaoffv4_35_e,
    ]


###############################################################################
# Tables for v4 sshape correction, layer 2, 3x5 photon clusters
#

gam5_layer2_etaoffv4_35_b = [
 [
  [               0,               0,               0,     -0.15794344,     0.076428685,    0.0045758062 ],
  [               0,               0,               0,       25.127268,      -11.957931,       3.2571577 ],
  [               0,               0,               0,      -1.0835373,      0.47204694,      -1.1661803 ],
  [               0,               0,               0,      -1.1368947,      0.53509251,    -0.035013422 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.067148313,     0.082353288,     -0.01256529 ],
  [               0,               0,               0,       1.4311191,      -3.2269117,       3.3530426 ],
  [               0,               0,               0,     -0.93555117,       1.3187509,       -1.578317 ],
  [               0,               0,               0,        1.144832,      -1.1646229,      0.32659689 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.34488596,     -0.26816395 ],
  [               0,               0,               0,               0,      -117.25956,       97.285493 ],
  [               0,               0,               0,               0,       45.280633,      -37.755019 ],
  [               0,               0,               0,               0,      -7.7661212,       6.2983205 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam5_layer2_etaoffv4_35_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer2_etaoffv4_35_b = [
 [
  [               0,               0,               0,     0.017656588,   -0.0017972813,    0.0061706289 ],
  [               0,               0,               0,      -26.130063,       10.116914,       3.0388606 ],
  [               0,               0,               0,       4.7087667,      -1.6992926,       -1.130427 ],
  [               0,               0,               0,       0.9388487,     -0.47106757,    -0.028088561 ],
  [               0,               0,               0,    0.0035528768,   -0.0021948398,    0.0011296492 ],
  [               0,               0,               0,       24.373224,      -10.683438,       1.5383534 ],
  [               0,               0,               0,      -3.4850102,       3.7485895,      -1.3859508 ],
 ],
 [
  [               0,               0,               0,     0.026664589,    -0.036761892,     0.020335011 ],
  [               0,               0,               0,      -21.151877,       26.075948,      -4.4414583 ],
  [               0,               0,               0,       1.3562725,      -2.0525819,     -0.60681098 ],
  [               0,               0,               0,       1.0221376,     -0.81918074,      0.18483666 ],
  [               0,               0,               0,   -0.0039078791,    0.0055320029,   -0.0017078498 ],
  [               0,               0,               0,       31.081178,      -41.717919,         15.5873 ],
  [               0,               0,               0,       2.3683427,      -2.0707727,      -4.2579449 ],
 ],
 [
  [               0,               0,               0,               0,      0.46376666,     -0.35815735 ],
  [               0,               0,               0,               0,      -49.738058,       42.166125 ],
  [               0,               0,               0,               0,       19.246878,      -16.676448 ],
  [               0,               0,               0,               0,       -3.509259,       2.8329405 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.023634523,    -0.080107996,     0.073880547 ],
  [               0,               0,               0,        4.724172,      -10.191296,       7.5500259 ],
  [               0,               0,               0,     -0.38716594,      0.70473187,      -1.4136332 ],
  [               0,               0,               0,      0.31742836,      -0.3662344,     0.060564166 ],
  [               0,               0,               0,     0.020535495,    -0.050464385,     0.031373071 ],
  [               0,               0,               0,       3.7970945,       -7.584819,       4.7453338 ],
  [               0,               0,               0,      -7.7661203,       12.964995,      -4.0720118 ],
 ],
 [
  [               0,               0,     0.048644905,    -0.046125909,    -0.077151446,     0.070708596 ],
  [               0,               0,      -399.69621,       484.23124,       711.75018,       -853.1248 ],
  [               0,               0,      -1141.7773,       1542.6473,       2105.3578,      -2845.8212 ],
  [               0,               0,      -357.45331,        513.8234,       683.31832,      -980.23623 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.015050992,   -0.0028318715,     0.040083883 ],
  [               0,               0,               0,      -431.39349,        4.631978,       882.15779 ],
  [               0,               0,               0,         5.34879,      -48.651974,       58.572175 ],
  [               0,               0,               0,     -0.39447386,      -0.7151656,      -1.1199976 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer2_etaoffv4_35_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0094274349 ],
  [               0,               0,               0,               0,               0,       2.4860959 ],
  [               0,               0,               0,               0,               0,      -1.1438412 ],
  [               0,               0,               0,               0,               0,    -0.010179396 ],
  [               0,               0,               0,               0,               0,   0.00015571908 ],
  [               0,               0,               0,               0,               0,       2.3874064 ],
  [               0,               0,               0,               0,               0,      -6.3536869 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer2_etaoffv4_35_b = [
 [
  [               0,               0,               0,    0.0091803725,    -0.002070447,    0.0071994078 ],
  [               0,               0,               0,       2.6730458,      -1.7556053,       4.3998023 ],
  [               0,               0,               0,        1.324307,     -0.59582342,      -1.2923896 ],
  [               0,               0,               0,      -1.6453628,      0.71913675,    -0.074340918 ],
  [               0,               0,               0,     0.024460823,    -0.011414056,    0.0018085548 ],
  [               0,               0,               0,      -16.990612,       7.0071118,    -0.088999875 ],
  [               0,               0,               0,       6.1956013,      -1.7809835,        -1.36858 ],
 ],
 [
  [               0,               0,               0,    0.0069896492,   -0.0043160857,    0.0058219986 ],
  [               0,               0,               0,      -9.1154382,       8.6527526,       2.6676161 ],
  [               0,               0,               0,       1.4832348,      -2.3328062,     -0.75184367 ],
  [               0,               0,               0,       1.0491055,     -0.55366844,      0.12479807 ],
  [               0,               0,               0,    0.0060194921,   -0.0075740146,     0.003385217 ],
  [               0,               0,               0,      -2.5982443,       2.3548323,      0.40042959 ],
  [               0,               0,               0,      -4.4190624,       2.4745906,      -8.2942578 ],
 ],
 [
  [               0,               0,               0,               0,     0.084424376,    -0.064049066 ],
  [               0,               0,               0,               0,      -44.229184,         41.5756 ],
  [               0,               0,               0,               0,       27.884875,         -24.462 ],
  [               0,               0,               0,               0,      -16.086791,       13.225422 ],
  [               0,               0,               0,               0,    -0.014673666,     0.013552284 ],
  [               0,               0,               0,               0,      -4.9659622,       4.9269283 ],
  [               0,               0,               0,               0,      -5.7665317,       2.6013929 ],
 ],
 [
  [               0,               0,               0,     -0.04009956,     0.094862998,    -0.047837027 ],
  [               0,               0,               0,       18.664204,      -44.532605,       29.574762 ],
  [               0,               0,               0,      -2.4067422,       5.6292961,       -4.685094 ],
  [               0,               0,               0,      0.78988166,      -1.6210232,      0.98901321 ],
  [               0,               0,               0,     0.018159201,     -0.03998098,     0.022409726 ],
  [               0,               0,               0,      -1.2127564,       1.0086441,      0.91071069 ],
  [               0,               0,               0,       7.4899242,      -16.155806,        6.591265 ],
 ],
 [
  [               0,               0,     0.068768555,     0.022411667,    -0.047791143,      -0.1446025 ],
  [               0,               0,       -485.9246,       621.06675,        826.9951,      -1046.8276 ],
  [               0,               0,      -79.241997,       119.59022,       155.66552,      -235.57376 ],
  [               0,               0,       19.898002,       -22.73047,      -34.022259,       38.556469 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.12977927,     -0.18089191 ],
  [               0,               0,               0,               0,       11.842942,       22.938334 ],
  [               0,               0,               0,               0,      -297.98732,       414.42995 ],
  [               0,               0,               0,               0,     -0.60690751,     -0.11820511 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer2_etaoffv4_35_e = [
 [
  [               0,               0,     -0.28532727,      0.47430145,      0.46350581,     -0.79739789 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0016559138,  -0.00080692272,    0.0011751561,    0.0059925147 ],
  [               0,               0,       3.3752323,       4.1678838,        5.251062,       10.709942 ],
  [               0,               0,      -18.565346,      -8.7046155,       12.679718,       56.787022 ],
  [               0,               0,       -2.583108,      -1.0394347,       2.4126972,       8.1393424 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,        3.069917,      -4.5991194 ],
  [               0,               0,               0,               0,        54.87997,      -80.754834 ],
  [               0,               0,               0,               0,        81.78959,       -124.9464 ],
  [               0,               0,               0,               0,      -79.662195,       120.48176 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.014005333,    -0.050875732,      0.04783664 ],
  [               0,               0,               0,      -37.950266,       139.82639,      -119.72061 ],
  [               0,               0,               0,        49.09883,       -161.5321,       130.15706 ],
  [               0,               0,               0,      -25.787987,       84.911405,      -69.274945 ],
  [               0,               0,               0,    0.0090361186,    -0.029275816,      0.02577976 ],
  [               0,               0,               0,     -0.93375546,        3.219736,       -1.816676 ],
  [               0,               0,               0,       11.364142,       -35.87115,       13.551877 ],
 ],
 [
  [               0,               0,               0,     0.021804978,    -0.068290452,     0.055994197 ],
  [               0,               0,               0,       9.4131229,      -46.462004,       58.832689 ],
  [               0,               0,               0,      0.92331977,      -2.9861811,      0.96575071 ],
  [               0,               0,               0,      -136.28921,       616.20919,      -695.16761 ],
  [               0,               0,               0,    -0.013745106,     -0.15307103,      0.44801362 ],
  [               0,               0,               0,      0.14159065,     -0.59213092,       0.6220089 ],
  [               0,               0,               0,       -7.454285,       26.995543,      -25.095482 ],
 ],
 [
  [               0,               0,               0,     -0.01765741,    0.0024148376,      0.10822322 ],
  [               0,               0,               0,       37.809085,      -174.95397,       204.27135 ],
  [               0,               0,               0,      -30.053097,       142.25261,      -169.47702 ],
  [               0,               0,               0,       -6.648307,       31.956828,      -38.275707 ],
  [               0,               0,               0,   -0.0023364435,    0.0037971298,    0.0057378574 ],
  [               0,               0,               0,        5.343929,       -12.23812,      -1.1792302 ],
  [               0,               0,               0,      -11.513938,       55.913734,      -70.521463 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.026853471 ],
  [               0,               0,               0,               0,               0,       1.2177148 ],
  [               0,               0,               0,               0,               0,     -0.88452158 ],
  [               0,               0,               0,               0,               0,     0.014094307 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.14985585,      -0.3714368 ],
  [               0,               0,               0,               0,      -9.9607596,      -15.050465 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,       1186.8175,      -2927.7275 ],
  [               0,               0,               0,               0,      0.81596207,       4.0917786 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer2_etaoffv4_35_b = [
 [
  [               0,               0,               0,    -0.081142778,     0.037298604,       0.0034818 ],
  [               0,               0,               0,       35.159337,      -17.038657,       6.4058977 ],
  [               0,               0,               0,      -3.1987943,       1.4998402,      -1.5112694 ],
  [               0,               0,               0,       1.4331667,      -0.6016563,     0.046840062 ],
  [               0,               0,               0,     0.023331204,    -0.011056437,    0.0017385566 ],
  [               0,               0,               0,       20.378436,      -8.9381604,       1.2437214 ],
  [               0,               0,               0,      -7.7456415,       3.4664127,      -1.8310038 ],
 ],
 [
  [               0,               0,               0,    -0.026491597,     0.030515346,   0.00019945858 ],
  [               0,               0,               0,        10.41656,      -13.796968,       8.5726345 ],
  [               0,               0,               0,     -0.42435724,      0.39521497,       -1.442861 ],
  [               0,               0,               0,      0.54837967,       -0.450861,     0.098487532 ],
  [               0,               0,               0,    0.0018591571,   -0.0027649168,    0.0013302225 ],
  [               0,               0,               0,       10.601184,      -10.935257,       4.2694453 ],
  [               0,               0,               0,      -8.7460228,       10.210557,      -32.761937 ],
 ],
 [
  [               0,               0,               0,               0,     0.031523739,    -0.017062112 ],
  [               0,               0,               0,               0,       10.502174,      -4.4852362 ],
  [               0,               0,               0,               0,       3.7961988,      -4.4638813 ],
  [               0,               0,               0,               0,      -24.728358,       28.169523 ],
  [               0,               0,               0,               0,     0.062479676,     0.018551604 ],
  [               0,               0,               0,               0,       2.0597637,      -1.5979978 ],
  [               0,               0,               0,               0,      -6.5639496,      -4.1046349 ],
 ],
 [
  [               0,               0,               0,       0.0192625,    -0.042056884,     0.029139684 ],
  [               0,               0,               0,      -5.6759013,       11.482104,      -1.6522211 ],
  [               0,               0,               0,     -0.41246284,      0.75903599,      -1.7906708 ],
  [               0,               0,               0,      0.58492756,     -0.97478764,      0.49866441 ],
  [               0,               0,               0,    -0.002869863,    0.0061901013,   -0.0025812141 ],
  [               0,               0,               0,      -0.8514557,       1.5622463,     0.057561286 ],
  [               0,               0,               0,      -11.271158,       24.199184,      -14.554909 ],
 ],
 [
  [               0,               0,      0.16206914,    -0.023731182,     -0.16770947,     -0.12969924 ],
  [               0,               0,      -4310.5745,       6058.9022,       7744.6384,      -10893.238 ],
  [               0,               0,      -396.62136,       564.59141,       748.51252,       -1067.111 ],
  [               0,               0,       525.92322,       -719.5291,      -988.54221,       1352.5875 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.013314772,    0.0052604481,    -0.011776142 ],
  [               0,               0,               0,      -3.1649675,      0.82787054,       12.663619 ],
  [               0,               0,               0,       3.5877464,       2.7504065,      -12.190231 ],
  [               0,               0,               0,   -0.0036841994,   0.00060573992,     -0.32456644 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer2_etaoffv4_35_e = [
 [
  [               0,               0,      -1.1087056,       1.6012377,       2.1329018,      -3.0798319 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.041075036,     -0.01775846,     0.033396728,      0.13226894 ],
  [               0,               0,       384.92875,      -548.30043,      -840.25815,         1198.44 ],
  [               0,               0,      -81.921935,       115.43174,       172.79587,      -244.23128 ],
  [               0,               0,      -58.952036,       86.638762,       128.02864,      -188.03446 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.040600882,     0.069454255 ],
  [               0,               0,               0,               0,       75.592829,      -113.19577 ],
  [               0,               0,               0,               0,      -88.132628,       132.48955 ],
  [               0,               0,               0,               0,       63.597155,      -96.177223 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0035083861,   -0.0066156219,    0.0075084701 ],
  [               0,               0,               0,       13.208102,       -48.58099,       48.120319 ],
  [               0,               0,               0,       6.3076188,      -20.681985,       15.413911 ],
  [               0,               0,               0,      -2.6833818,       8.3711205,      -6.3044502 ],
  [               0,               0,               0,    0.0063371478,    -0.024373557,     0.024253525 ],
  [               0,               0,               0,       -8.617982,       30.650181,       -26.40463 ],
  [               0,               0,               0,       28.197371,      -92.641027,       67.237307 ],
 ],
 [
  [               0,               0,               0,     -0.01610122,     0.067007046,    -0.060107576 ],
  [               0,               0,               0,       5.7323062,       -25.57282,       31.076749 ],
  [               0,               0,               0,       1.2675084,      -5.1502901,       3.9325228 ],
  [               0,               0,               0,       61.617585,      -255.85325,       303.30285 ],
  [               0,               0,               0,    -0.015024096,     0.068404475,      0.28474695 ],
  [               0,               0,               0,     -0.46967307,       1.9818767,      -2.0683242 ],
  [               0,               0,               0,       1.6229145,      -6.7312407,       3.8573776 ],
 ],
 [
  [               0,               0,               0,    0.0075662097,     0.011220641,    -0.056452052 ],
  [               0,               0,               0,       29.830934,      -147.73774,       184.79566 ],
  [               0,               0,               0,       7.4268669,      -34.694502,       39.355406 ],
  [               0,               0,               0,      -4.1958548,       21.872209,       -28.26013 ],
  [               0,               0,               0,    -0.004626676,     -0.01052703,     0.051122744 ],
  [               0,               0,               0,       68.424747,      -323.31632,       381.60079 ],
  [               0,               0,               0,       58.940193,      -295.78162,       370.08144 ],
 ],
 [
  [               0,               0,               0,               0,    0.0016520464,     0.015701568 ],
  [               0,               0,               0,               0,      -130.48536,       323.34646 ],
  [               0,               0,               0,               0,       64.596741,      -159.90066 ],
  [               0,               0,               0,               0,      -25.257952,       62.214783 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0020796274,     0.016564732,     -0.11989033,      0.16372907 ],
  [               0,               0,      0.30693523,     -0.77788572,      -3.4013198,      -4.4499337 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,       133.57678,       327.15773,      -645.01836,       -17.54488 ],
  [               0,               0,       29.764987,       105.47066,       254.42899,      -47.741128 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer2_etaoffv4_35_b = [
 [
  [               0,               0,               0,    -0.027915664,     0.012939409,    0.0060688516 ],
  [               0,               0,               0,       11.371303,      -5.9369126,       5.3065374 ],
  [               0,               0,               0,      -2.1529548,       1.0828635,      -1.4890519 ],
  [               0,               0,               0,       2.2392158,      -1.1791323,      0.15933668 ],
  [               0,               0,               0,    0.0082592838,   -0.0049186123,    0.0013123251 ],
  [               0,               0,               0,       10.218403,      -3.8732776,      0.60046081 ],
  [               0,               0,               0,       -12.90289,       7.0038826,      -2.6664214 ],
 ],
 [
  [               0,               0,               0,     0.023171261,    -0.025563249,     0.012998003 ],
  [               0,               0,               0,      -12.936885,       13.201794,       1.7170002 ],
  [               0,               0,               0,       1.7940541,      -2.5620075,     -0.58411135 ],
  [               0,               0,               0,     0.048341463,      0.43338554,     -0.21188044 ],
  [               0,               0,               0,   -0.0061301885,     0.007073979,   -0.0011221342 ],
  [               0,               0,               0,      -5.8538231,       6.6901484,      -1.0122333 ],
  [               0,               0,               0,      0.17774363,      -2.5739506,     -0.30979378 ],
 ],
 [
  [               0,               0,               0,               0,      0.17667507,     -0.13395059 ],
  [               0,               0,               0,               0,      -44.558503,       40.016992 ],
  [               0,               0,               0,               0,       21.854318,      -19.088329 ],
  [               0,               0,               0,               0,      -22.198341,       19.158387 ],
  [               0,               0,               0,               0,      0.11089105,    -0.078959733 ],
  [               0,               0,               0,               0,      -10.592725,       8.6357777 ],
  [               0,               0,               0,               0,      -10.574875,       5.4174283 ],
 ],
 [
  [               0,               0,               0,   -0.0019149524,    0.0067134266,    0.0031365314 ],
  [               0,               0,               0,       2.9904495,      -8.1352769,       8.8891506 ],
  [               0,               0,               0,     0.065847493,      -0.4400071,      -1.0114527 ],
  [               0,               0,               0,       1.1755104,      -1.5359771,      0.46141215 ],
  [               0,               0,               0,     0.014069147,    -0.025284043,     0.011860556 ],
  [               0,               0,               0,       2.0005797,       -4.999315,       3.3797617 ],
  [               0,               0,               0,       5.8665946,      -16.770753,       8.8830623 ],
 ],
 [
  [               0,               0,      0.17720943,     0.009251939,      -0.1579038,     -0.23870394 ],
  [               0,               0,       129.90132,      -164.15925,      -241.57979,       308.10815 ],
  [               0,               0,       82.099998,      -97.575974,      -159.75832,       189.30296 ],
  [               0,               0,       444.36915,      -606.76727,      -836.52697,       1142.3457 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.01690578,    -0.010590949 ],
  [               0,               0,               0,               0,      -1098.8808,       1585.6132 ],
  [               0,               0,               0,               0,      -5.4795968,       6.6635712 ],
  [               0,               0,               0,               0,     -0.15648572,     -0.50634018 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer2_etaoffv4_35_e = [
 [
  [               0,               0,      -1.1205548,       1.6373558,       2.1618217,      -3.1610281 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   0.00055365818,   -2.262895e-06,   0.00023150027,   -0.0010071701 ],
  [               0,               0,       2.9055646,       4.0821944,       5.7360824,       12.094891 ],
  [               0,               0,     -0.87165131,     -0.14788445,      -1.3806943,       2.0906786 ],
  [               0,               0,     -0.80167311,     -0.19192843,    -0.050044473,       4.0391279 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.021114721,  -0.00022039692 ],
  [               0,               0,               0,               0,       24.120409,      -34.753866 ],
  [               0,               0,               0,               0,       28.321383,      -43.903558 ],
  [               0,               0,               0,               0,      -33.460542,       50.623091 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.012730144,     0.046987478,    -0.034862323 ],
  [               0,               0,               0,       15.458133,      -53.992274,       50.377761 ],
  [               0,               0,               0,      -2.0287935,       6.8533166,      -7.0989906 ],
  [               0,               0,               0,       10.610281,      -31.758343,        29.55048 ],
  [               0,               0,               0,     0.013929295,    0.0091598623,   -0.0044350144 ],
  [               0,               0,               0,       0.5664656,      -1.8898229,       1.6152973 ],
  [               0,               0,               0,      -1.5012179,       4.8623861,       -6.948251 ],
 ],
 [
  [               0,               0,               0,     0.025770429,     -0.10081031,       0.1040107 ],
  [               0,               0,               0,      -8.4151635,       30.024903,      -22.403695 ],
  [               0,               0,               0,       11.103492,      -46.505232,       47.260024 ],
  [               0,               0,               0,      -6.0147622,       25.360772,      -26.612055 ],
  [               0,               0,               0,    0.0045654971,    -0.019709993,     0.022353173 ],
  [               0,               0,               0,       3.4860598,      -14.482395,       15.847832 ],
  [               0,               0,               0,       21.717278,      -91.798828,       95.142017 ],
 ],
 [
  [               0,               0,               0,   -0.0057278608,     0.022507607,    -0.011257173 ],
  [               0,               0,               0,       55.881359,      -265.55892,       318.06357 ],
  [               0,               0,               0,       10.465511,      -50.188283,       58.935156 ],
  [               0,               0,               0,       6.9955513,      -9.5736115,      -14.950946 ],
  [               0,               0,               0,     0.045427205,   -0.0081777822,     -0.21873978 ],
  [               0,               0,               0,      -3.4047722,       16.421365,       -19.75107 ],
  [               0,               0,               0,       34.298878,      -168.61393,       204.15913 ],
 ],
 [
  [               0,               0,               0,               0,      -1.1368428,       2.8221751 ],
  [               0,               0,               0,               0,       20.645553,       -49.17307 ],
  [               0,               0,               0,               0,       -2.729931,       5.7441873 ],
  [               0,               0,               0,               0,      -1.4405156,       3.5705429 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   0.00049824886,    0.0014477715,   -0.0059257329,   -0.0017889928 ],
  [               0,               0,       -1.161987,      -1.4069891,      -3.4895391,      -4.6547772 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,       442.35217,       380.70552,      -875.30876,       -6669.054 ],
  [               0,               0,       1.2556947,       3.1819669,       32.323674,       50.729121 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer2_etaoffv4_35_b = [
 [
  [               0,               0,               0,    -0.049286879,     0.022536351,     0.004671593 ],
  [               0,               0,               0,       19.885222,      -9.5757144,        5.980248 ],
  [               0,               0,               0,     -0.62203621,      0.12885932,      -1.3596887 ],
  [               0,               0,               0,      0.06655189,      0.11353046,    -0.040832217 ],
  [               0,               0,               0,      0.01494066,   -0.0070159403,    0.0012892742 ],
  [               0,               0,               0,       17.142782,      -7.2593521,       1.1883955 ],
  [               0,               0,               0,       -5.321736,       1.7183941,      -1.5748306 ],
 ],
 [
  [               0,               0,               0,   -0.0095360405,      0.01186364,     0.003948388 ],
  [               0,               0,               0,       0.1020967,      -2.0377536,       5.6351751 ],
  [               0,               0,               0,     -0.86327104,      0.78404132,      -1.5294697 ],
  [               0,               0,               0,      0.78432021,      -0.6314065,      0.12005879 ],
  [               0,               0,               0,    0.0074045387,   -0.0090960919,    0.0032329752 ],
  [               0,               0,               0,       0.8317989,      0.15951029,     0.065182187 ],
  [               0,               0,               0,     -0.74796018,     -0.40274144,      -1.1941461 ],
 ],
 [
  [               0,               0,               0,               0,     0.045664296,    -0.030463867 ],
  [               0,               0,               0,               0,        4.092225,       1.6384886 ],
  [               0,               0,               0,               0,       12.059521,      -11.057426 ],
  [               0,               0,               0,               0,      -8.4719183,       6.7914307 ],
  [               0,               0,               0,               0,    -0.010703103,    0.0094702985 ],
  [               0,               0,               0,               0,     -0.52493136,       1.3258067 ],
  [               0,               0,               0,               0,       15.937227,      -14.338816 ],
 ],
 [
  [               0,               0,               0,    0.0089930966,    -0.015381877,     0.013027812 ],
  [               0,               0,               0,      -1.1117919,     0.027762088,       5.4064167 ],
  [               0,               0,               0,        2.796528,      -6.4651433,       2.2195508 ],
  [               0,               0,               0,     -0.87846762,        2.555599,      -1.5832237 ],
  [               0,               0,               0,    0.0051875961,    -0.010133511,    0.0056343505 ],
  [               0,               0,               0,       -1.252022,       1.6063016,      0.37754827 ],
  [               0,               0,               0,       4.5541565,      -13.353572,       6.9613523 ],
 ],
 [
  [               0,               0,               0,      0.42907645,     -0.28893171,     -0.39141156 ],
  [               0,               0,               0,       1057.8274,      -2884.8816,       1969.8738 ],
  [               0,               0,               0,      -718.92952,       2007.7421,      -1402.7497 ],
  [               0,               0,               0,       1462.9297,      -4007.5342,       2744.5989 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.50426446,     -0.70631998 ],
  [               0,               0,               0,               0,      -1153.4945,       1661.9626 ],
  [               0,               0,               0,               0,        69.46254,      -101.22844 ],
  [               0,               0,               0,               0,    -0.025881914,     -0.52314264 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer2_etaoffv4_35_e = [
 [
  [               0,               0,       -2.158194,       3.1117102,       4.2748282,        -6.16216 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,       -11.04362,       32.652024,      -24.124407 ],
  [               0,               0,               0,       865.12103,      -2526.6782,       1846.5607 ],
  [               0,               0,               0,      -88.380146,       258.19524,      -189.76372 ],
  [               0,               0,               0,      -506.98185,       1493.2333,      -1099.3445 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.010457707,     0.051161213 ],
  [               0,               0,               0,               0,      0.67064615,       0.6348041 ],
  [               0,               0,               0,               0,       44.414469,      -68.211422 ],
  [               0,               0,               0,               0,      -43.406028,        65.66463 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0058618009,     0.021999553,    -0.011611298 ],
  [               0,               0,               0,       6.9421675,      -23.941706,        23.84757 ],
  [               0,               0,               0,       2.5204622,      -8.6370835,       6.0384364 ],
  [               0,               0,               0,     -0.81642119,      -1.7972434,       6.4274902 ],
  [               0,               0,               0,    -0.013282588,    0.0062853083,     0.036516344 ],
  [               0,               0,               0,      -2.7924185,       9.7291447,      -8.2964168 ],
  [               0,               0,               0,       8.6344319,      -28.446906,       20.393316 ],
 ],
 [
  [               0,               0,               0,   -0.0013905227,    0.0056667709,    0.0033614838 ],
  [               0,               0,               0,      0.15036445,      -2.0946919,       6.6488385 ],
  [               0,               0,               0,      -0.2741572,       1.2103317,      -2.6370624 ],
  [               0,               0,               0,       3.0392458,      -3.6480759,       2.0716806 ],
  [               0,               0,               0,     0.017161498,     0.010091774,    -0.025759938 ],
  [               0,               0,               0,    0.0062622307,    0.0060641055,    0.0068592581 ],
  [               0,               0,               0,     -0.33842819,       1.3701698,      -4.4615122 ],
 ],
 [
  [               0,               0,               0,    -0.014640039,   -0.0056171574,      0.10723676 ],
  [               0,               0,               0,      -8.2956478,       44.960024,      -57.474287 ],
  [               0,               0,               0,       31.065417,      -148.75803,       176.76481 ],
  [               0,               0,               0,      -16.318432,       78.208613,      -93.597044 ],
  [               0,               0,               0,    0.0032253158,   -0.0060204215,   -0.0031136245 ],
  [               0,               0,               0,      -0.8181839,       4.2267907,      -4.6347221 ],
  [               0,               0,               0,       17.218257,      -61.850652,       46.734158 ],
 ],
 [
  [               0,               0,               0,               0,    -0.017296028,     0.045880055 ],
  [               0,               0,               0,               0,      -190.61314,       473.76382 ],
  [               0,               0,               0,               0,      -133.61245,       327.61699 ],
  [               0,               0,               0,               0,       166.35877,      -409.44212 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.036465633,    -0.054432428,    -0.068614038,    -0.050245304 ],
  [               0,               0,      -1.1573735,       -1.420055,      -3.5014679,      -4.5822853 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,      0.42402148,      0.59775734,      0.53672346,      -2.2258576 ],
  [               0,               0,     0.032539412,      0.14823273,      0.54642761,       1.1882644 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer2_etaoffv4_35_b = [
 [
  [               0,               0,               0,     0.011225922,   -0.0029972052,    0.0075256318 ],
  [               0,               0,               0,      -1.9813008,     -0.19555194,        4.946565 ],
  [               0,               0,               0,    -0.011565344,     -0.10603248,       -1.310256 ],
  [               0,               0,               0,      0.15153977,      0.17630829,     -0.07705468 ],
  [               0,               0,               0,    0.0070442527,     -0.00474177,    0.0010561672 ],
  [               0,               0,               0,      -11.442656,       5.3515843,    -0.012191216 ],
  [               0,               0,               0,      -18.782003,       4.9194283,      -1.3905397 ],
 ],
 [
  [               0,               0,               0,    0.0057355381,   -0.0069982515,    0.0096410013 ],
  [               0,               0,               0,      -8.5115218,       9.5759016,        2.114361 ],
  [               0,               0,               0,      -0.2732919,      0.15939571,      -1.3697743 ],
  [               0,               0,               0,     0.047373391,       1.2505273,      0.54957331 ],
  [               0,               0,               0,   -0.0054051834,    0.0042911234,    -0.011813561 ],
  [               0,               0,               0,      0.33824373,     -0.30270063,      0.08899082 ],
  [               0,               0,               0,      -1.0983053,    -0.015781971,      -30.523063 ],
 ],
 [
  [               0,               0,               0,               0,    0.0078862262,  -0.00078507644 ],
  [               0,               0,               0,               0,      -4.9052827,       9.2282108 ],
  [               0,               0,               0,               0,       24.756568,      -21.279562 ],
  [               0,               0,               0,               0,      -12.934805,       10.385733 ],
  [               0,               0,               0,               0,  -0.00010803479,     0.001331384 ],
  [               0,               0,               0,               0,       2.4318369,     -0.94556794 ],
  [               0,               0,               0,               0,       31.497796,      -26.944183 ],
 ],
 [
  [               0,               0,               0,   -0.0012081976,    0.0037266635,    0.0049310125 ],
  [               0,               0,               0,       0.4297824,      -1.8993609,       5.6737188 ],
  [               0,               0,               0,     -0.81075092,       1.6125578,       -2.206995 ],
  [               0,               0,               0,      0.43425435,     -0.65228434,      0.28999231 ],
  [               0,               0,               0,    0.0020775694,   -0.0039957121,    0.0024877225 ],
  [               0,               0,               0,       3.4490273,      -8.3453754,       5.4833404 ],
  [               0,               0,               0,      -3.0958687,       5.8241739,      -4.7184266 ],
 ],
 [
  [               0,               0,      0.16060864,    0.0005510226,     -0.14879796,     -0.19539376 ],
  [               0,               0,       272.98317,      -363.69951,      -507.72541,       680.17355 ],
  [               0,               0,       -627.8315,       880.45699,       1172.5101,      -1646.1601 ],
  [               0,               0,       605.16368,      -826.75877,      -1138.3293,       1555.2497 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.065106961,    -0.077624671 ],
  [               0,               0,               0,               0,      -1034.1378,       1495.1123 ],
  [               0,               0,               0,               0,      -2.9315284,       2.9140657 ],
  [               0,               0,               0,               0,     -0.09597357,     -0.43157886 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer2_etaoffv4_35_e = [
 [
  [               0,               0,      -3.0813163,       4.4568688,       6.1290634,      -8.8647555 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.048618434,      0.01496225,     -0.12166619 ],
  [               0,               0,               0,       15.291699,       -133.9383,       168.07804 ],
  [               0,               0,               0,       102.22716,      -289.64247,       203.33055 ],
  [               0,               0,               0,       6.7421452,      -27.799802,        26.56574 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.046144858,     0.070501285 ],
  [               0,               0,               0,               0,       1362.6772,      -2048.0329 ],
  [               0,               0,               0,               0,      -1074.5551,       1630.4423 ],
  [               0,               0,               0,               0,       1340.9815,      -2028.0769 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.003992247,    0.0036196626,     0.018356205 ],
  [               0,               0,               0,        2.840018,       -9.028118,       10.251067 ],
  [               0,               0,               0,     -0.69907218,       2.5588684,      -3.5311021 ],
  [               0,               0,               0,      0.24224032,       -0.899708,      0.82442474 ],
  [               0,               0,               0,  -0.00077504106,   1.7690238e-05,    0.0033624238 ],
  [               0,               0,               0,      -1.9167626,       6.6863389,      -4.6244206 ],
  [               0,               0,               0,      -2.0361651,       5.4813562,      -8.3534798 ],
 ],
 [
  [               0,               0,               0,    0.0014405175,   -0.0011785925,     0.004855016 ],
  [               0,               0,               0,      -2.6237348,        8.654627,      -3.4428344 ],
  [               0,               0,               0,      0.80654402,      -2.8291044,       1.0671779 ],
  [               0,               0,               0,       -3.259406,       13.349453,      -13.522932 ],
  [               0,               0,               0,    -0.013259279,     0.056097749,    -0.058192927 ],
  [               0,               0,               0,       10.278877,      -44.384791,       48.160214 ],
  [               0,               0,               0,       24.649514,      -100.94511,        100.8869 ],
 ],
 [
  [               0,               0,               0,   -0.0046146369,    0.0038498932,     0.027255138 ],
  [               0,               0,               0,       16.852246,       -77.39898,       91.446646 ],
  [               0,               0,               0,       2.8895104,        -14.5322,       16.979777 ],
  [               0,               0,               0,       7.8149648,      -24.586827,       15.281474 ],
  [               0,               0,               0,     0.015136323,    0.0018719201,    -0.078306175 ],
  [               0,               0,               0,       6.6196897,      -30.881253,       36.032741 ],
  [               0,               0,               0,       14.883536,      -79.490772,       102.27224 ],
 ],
 [
  [               0,               0,               0,               0,    0.0022067914,    0.0072769132 ],
  [               0,               0,               0,               0,      -45.705173,       114.91188 ],
  [               0,               0,               0,               0,       5.5282033,      -14.750913 ],
  [               0,               0,               0,               0,       0.9278663,      -2.2367766 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0036434821,    0.0029994205,    -0.007526503,    -0.055152501 ],
  [               0,               0,      -1.1741249,      -1.3985268,      -3.4715311,      -4.6173176 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,       215.44647,        180.8338,      -432.17298,      -3266.9097 ],
  [               0,               0,      0.51718747,       1.1467351,       3.1476731,       9.8511512 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer2_etaoffv4_35_b = [
 [
  [               0,               0,               0,    -0.010531191,    0.0067589052,    0.0065194612 ],
  [               0,               0,               0,       1.9255384,      -1.9679605,        5.450869 ],
  [               0,               0,               0,      -1.4299007,      0.78456586,      -1.4333799 ],
  [               0,               0,               0,     -0.57201872,     -0.24055082,     0.013401756 ],
  [               0,               0,               0,     0.016617918,   -0.0046627067,   0.00067940635 ],
  [               0,               0,               0,       3.8693686,      -3.9801964,       1.1286967 ],
  [               0,               0,               0,      -18.073146,       11.369916,      -2.2854416 ],
 ],
 [
  [               0,               0,               0,    -0.013910897,     0.017715813,    0.0021318807 ],
  [               0,               0,               0,         6.41123,      -9.2830288,       8.0890318 ],
  [               0,               0,               0,      0.17048543,     -0.39655872,      -1.1630274 ],
  [               0,               0,               0,      0.45894239,     -0.25355842,    -0.060053321 ],
  [               0,               0,               0,  -0.00081863706,   0.00046126789,   0.00049732328 ],
  [               0,               0,               0,       8.4764285,      -9.9978995,       3.2610072 ],
  [               0,               0,               0,      -5.2444928,       3.5298257,      -1.2393636 ],
 ],
 [
  [               0,               0,               0,               0,     0.017322224,   -0.0054713329 ],
  [               0,               0,               0,               0,      -2.2551352,       6.1952628 ],
  [               0,               0,               0,               0,       5.6734445,      -5.8580578 ],
  [               0,               0,               0,               0,      -3.3200925,       2.5857126 ],
  [               0,               0,               0,               0,    0.0021850937,    0.0012606458 ],
  [               0,               0,               0,               0,     -0.64660852,      0.54142365 ],
  [               0,               0,               0,               0,      -2.1262583,      0.26342295 ],
 ],
 [
  [               0,               0,               0,    0.0054928284,    -0.010032133,     0.013146981 ],
  [               0,               0,               0,      -2.4732396,       4.1328576,        2.621599 ],
  [               0,               0,               0,      0.21608479,     -0.45746073,      -1.1071775 ],
  [               0,               0,               0,    -0.055988003,      0.22185777,     -0.15428888 ],
  [               0,               0,               0,  -7.4080089e-05,   -2.509022e-05,  -0.00011425424 ],
  [               0,               0,               0,     -0.89523355,        1.055347,       1.6447326 ],
  [               0,               0,               0,      -0.2854821,    -0.017302762,      -1.3141441 ],
 ],
 [
  [               0,               0,      0.18776787,    -0.034189512,      -0.1988574,      -0.1312491 ],
  [               0,               0,       319.36482,      -414.04712,      -585.27767,       762.13269 ],
  [               0,               0,      -419.61002,        594.4528,       778.78487,      -1105.4153 ],
  [               0,               0,       591.95815,      -806.97854,      -1113.3447,       1517.8372 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.027183554,    -0.020320189 ],
  [               0,               0,               0,               0,      -865.68526,       1258.4043 ],
  [               0,               0,               0,               0,       0.3819983,      -1.9403592 ],
  [               0,               0,               0,               0,    0.0062044091,     -0.29360067 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer2_etaoffv4_35_e = [
 [
  [       89.547193,      -180.43588,      -144.17639,       211.60942,       517.30358,      -531.81242 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.039670924,     0.018766398,    -0.027343196,     -0.11544069 ],
  [               0,               0,       830.87201,      -1229.5286,      -1788.0899,       2647.9979 ],
  [               0,               0,       399.26584,      -584.78277,      -885.22673,       1295.3535 ],
  [               0,               0,      -427.00686,       629.51129,       925.98762,      -1364.8386 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0077778913,     0.034281897 ],
  [               0,               0,               0,               0,        4.089608,      -3.7945708 ],
  [               0,               0,               0,               0,       47.933197,      -73.674384 ],
  [               0,               0,               0,               0,      -46.595653,       70.501179 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.048916282,     -0.16534359,      0.14679035 ],
  [               0,               0,               0,      -15.290804,       50.918561,      -38.081126 ],
  [               0,               0,               0,       1.2268654,      -4.4452392,       2.6057608 ],
  [               0,               0,               0,       2.2781029,      -7.4687889,       6.2097872 ],
  [               0,               0,               0,     0.029252088,     -0.10053567,     0.087184895 ],
  [               0,               0,               0,       -10.91069,       38.037949,      -32.606152 ],
  [               0,               0,               0,      -12.236707,       39.489932,      -33.770219 ],
 ],
 [
  [               0,               0,               0,     0.015627147,    -0.067137156,     0.080630888 ],
  [               0,               0,               0,      -4.5014471,       17.728768,      -14.038559 ],
  [               0,               0,               0,      0.89173713,      -3.4857015,       2.0336689 ],
  [               0,               0,               0,      -6.7603731,       25.127334,      -20.688353 ],
  [               0,               0,               0,   -0.0074601731,    0.0021908683,     0.047520478 ],
  [               0,               0,               0,     -0.24213526,      0.93418488,     -0.81856839 ],
  [               0,               0,               0,       2.1687967,      -8.2260611,       4.7373621 ],
 ],
 [
  [               0,               0,               0,     0.044969925,     -0.26186923,      0.37426105 ],
  [               0,               0,               0,       146.01668,       -677.4502,        788.7778 ],
  [               0,               0,               0,       4.5011544,      -24.405775,       31.151966 ],
  [               0,               0,               0,      -10.776367,        52.81435,      -64.520033 ],
  [               0,               0,               0,     0.010730851,    -0.038998431,     0.033472648 ],
  [               0,               0,               0,      -37.491096,       180.38614,      -216.05193 ],
  [               0,               0,               0,       69.496811,      -332.78286,       396.56524 ],
 ],
 [
  [               0,               0,               0,               0,    0.0016491601,     0.021135879 ],
  [               0,               0,               0,               0,      0.39736034,       1.0623069 ],
  [               0,               0,               0,               0,       29.643062,      -74.123215 ],
  [               0,               0,               0,               0,      -28.938127,       71.282157 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   0.00088468143,    0.0003124557,  -0.00077215753,    -0.013444535 ],
  [               0,               0,      -1.2273259,      -1.3604408,      -3.3746176,      -4.3825249 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,       6.2533165,       21.011518,       25.184122,      -50.127282 ],
  [               0,               0,       1.0877614,       16.290361,       3.9091785,      -9.5696222 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer2_etaoffv4_35_b = [
 [
  [               0,               0,               0,    -0.012880141,     0.010990718,     0.005899075 ],
  [               0,               0,               0,        7.886189,      -6.6409493,       6.1565137 ],
  [               0,               0,               0,       0.8297172,     -0.38977563,      -1.3056214 ],
  [               0,               0,               0,      0.38824101,     0.029830938,    -0.052826338 ],
  [               0,               0,               0,    0.0090629639,   -0.0046625827,   0.00080608303 ],
  [               0,               0,               0,       1.1564506,      -3.9261234,       1.1921878 ],
  [               0,               0,               0,      -39.842055,        10.98144,      -1.7682372 ],
 ],
 [
  [               0,               0,               0,     0.015056645,    -0.019884419,     0.013109172 ],
  [               0,               0,               0,      -9.9187546,       12.079941,       1.8555322 ],
  [               0,               0,               0,      -1.5870537,       1.7929666,      -1.8073357 ],
  [               0,               0,               0,      0.92891183,      -1.0083158,      0.21379854 ],
  [               0,               0,               0,   -0.0064811011,    0.0084832122,    -0.002128717 ],
  [               0,               0,               0,      -8.2995318,       10.483326,      -2.3653721 ],
  [               0,               0,               0,       5.1118636,      -7.1606364,       1.1219952 ],
 ],
 [
  [               0,               0,               0,               0,    -0.011685547,     0.017180373 ],
  [               0,               0,               0,               0,      -4.0528265,        7.946806 ],
  [               0,               0,               0,               0,        13.31781,      -11.812822 ],
  [               0,               0,               0,               0,      -8.0462014,       6.3097228 ],
  [               0,               0,               0,               0,     0.026659013,    -0.020632647 ],
  [               0,               0,               0,               0,       6.6910437,      -4.5186883 ],
  [               0,               0,               0,               0,       5.3597864,      -4.8465464 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0027297365,    0.0095215487 ],
  [               0,               0,               0,               0,       0.6371182,       4.1667392 ],
  [               0,               0,               0,               0,     -0.18295869,      -1.1992807 ],
  [               0,               0,               0,               0,      0.14704016,     -0.13472381 ],
  [               0,               0,               0,               0,    0.0010412175,  -0.00038988311 ],
  [               0,               0,               0,               0,       0.1030841,      0.79066408 ],
  [               0,               0,               0,               0,      0.14839069,      -20.715592 ],
 ],
 [
  [               0,               0,      0.16092111,    -0.040099051,     -0.17804873,    -0.083020297 ],
  [               0,               0,       75.759764,      -62.088749,      -114.73437,       84.732886 ],
  [               0,               0,      -1019.5748,       1413.6946,       1912.9494,      -2654.3522 ],
  [               0,               0,       438.14719,      -593.05542,      -820.33468,       1110.5225 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.65628297,     -0.92028738 ],
  [               0,               0,               0,               0,       660.35432,      -929.00847 ],
  [               0,               0,               0,               0,       18.498773,      -29.639664 ],
  [               0,               0,               0,               0,       2.5305386,      -3.6547749 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer2_etaoffv4_35_e = [
 [
  [       101.70822,      -204.96654,      -163.62062,       240.36666,       586.99736,      -603.56254 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,     -0.59894672,      0.47464419,       1.3454482,      0.95317781,      -3.0080189 ],
  [               0,       299.83716,      -241.94752,      -609.44588,      -460.70065,         1364.65 ],
  [               0,       190.50005,      -130.19499,      -426.06057,      -344.16753,       949.31988 ],
  [               0,      -196.45986,       145.27063,       426.23053,       312.57671,      -924.31958 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0032377787,   -0.0015426802 ],
  [               0,               0,               0,               0,       -489.7829,       746.88978 ],
  [               0,               0,               0,               0,      -104.14524,       156.46719 ],
  [               0,               0,               0,               0,       145.91739,      -220.51522 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0099502109,     -0.03009343,     0.030302603 ],
  [               0,               0,               0,     -0.65024574,      0.19496096,       5.7016724 ],
  [               0,               0,               0,      0.91485634,      -2.7707373,      0.63982313 ],
  [               0,               0,               0,      -4.7809208,       13.875567,      -6.3034486 ],
  [               0,               0,               0,   -0.0040302187,    0.0072046246,     0.026544316 ],
  [               0,               0,               0,     -0.48378019,       1.4861722,      -1.0554632 ],
  [               0,               0,               0,       2.7599006,      -9.2696858,       4.7389425 ],
 ],
 [
  [               0,               0,               0,  -3.8312459e-05,   -0.0003647002,    0.0099554015 ],
  [               0,               0,               0,      0.98331465,      -5.4956723,       10.481737 ],
  [               0,               0,               0,        0.296616,      -1.1400545,     -0.22472767 ],
  [               0,               0,               0,     -0.13997407,       1.2224851,       -1.399578 ],
  [               0,               0,               0,    0.0019257715,   0.00058740109,   -0.0040758509 ],
  [               0,               0,               0,    0.0092260764,     -0.07373174,      0.17668931 ],
  [               0,               0,               0,     -0.93608905,       4.3781839,      -7.5652296 ],
 ],
 [
  [               0,               0,               0,    0.0015814196,    0.0018699765,   -0.0030951066 ],
  [               0,               0,               0,       5.4994671,      -29.425728,       41.702666 ],
  [               0,               0,               0,       3.5480289,      -16.015278,       16.737368 ],
  [               0,               0,               0,      -12.009029,       44.793686,      -17.733832 ],
  [               0,               0,               0,     0.011287131,     0.028624864,     0.082831837 ],
  [               0,               0,               0,     -0.12166857,      0.38920381,      -0.2201217 ],
  [               0,               0,               0,       8.1205285,      -37.844926,       40.982187 ],
 ],
 [
  [               0,               0,               0,               0,      0.11964712,     -0.28831326 ],
  [               0,               0,               0,               0,      -111.63353,       278.51839 ],
  [               0,               0,               0,               0,      -24.580691,       59.080216 ],
  [               0,               0,               0,               0,       43.869237,      -107.91175 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   9.0030634e-05,   0.00014499945,     0.013126877,    -0.034762498 ],
  [               0,               0,      -1.1869239,      -1.3920552,      -3.4588416,      -4.5952249 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,      -1.1159611,       17.965781,        39.22872,       15.275349 ],
  [               0,               0,   -0.0084547834,       4.2762783,       25.091879,       11.461104 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]


gam_layer2_etaoffv4_35_b = [
    gam5_layer2_etaoffv4_35_b,
    gam10_layer2_etaoffv4_35_b,
    gam25_layer2_etaoffv4_35_b,
    gam50_layer2_etaoffv4_35_b,
    gam75_layer2_etaoffv4_35_b,
    gam100_layer2_etaoffv4_35_b,
    gam200_layer2_etaoffv4_35_b,
    gam500_layer2_etaoffv4_35_b,
    gam1000_layer2_etaoffv4_35_b,
    ]

gam_layer2_etaoffv4_35_e = [
    gam5_layer2_etaoffv4_35_e,
    gam10_layer2_etaoffv4_35_e,
    gam25_layer2_etaoffv4_35_e,
    gam50_layer2_etaoffv4_35_e,
    gam75_layer2_etaoffv4_35_e,
    gam100_layer2_etaoffv4_35_e,
    gam200_layer2_etaoffv4_35_e,
    gam500_layer2_etaoffv4_35_e,
    gam1000_layer2_etaoffv4_35_e,
    ]


###############################################################################
# Tables for v4 sshape correction, layer 2, 3x7 electron clusters
#

ele5_layer2_etaoffv4_37_b = [
 [
  [               0,               0,               0,      0.43432174,     -0.31061693,     0.064460381 ],
  [               0,               0,               0,       8.2599225,       1.2530767,      0.73305738 ],
  [               0,               0,               0,      0.58551963,      -1.8061877,     -0.59752058 ],
  [               0,               0,               0,      0.41782826,    -0.040887239,    0.0017588449 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele5_layer2_etaoffv4_37_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer2_etaoffv4_37_b = [
 [
  [               0,               0,               0,       0.1994411,    -0.059754595,     0.013981238 ],
  [               0,               0,               0,      -33.216609,       12.025911,       1.8059337 ],
  [               0,               0,               0,       11.532025,       -4.174064,     -0.94032517 ],
  [               0,               0,               0,      -2.7155815,      0.97091372,    -0.048802946 ],
  [               0,               0,               0,    0.0070570991,    0.0024640056,  -0.00038419028 ],
  [               0,               0,               0,       1.4440644,       -3.402288,       1.9866689 ],
  [               0,               0,               0,      -30.408385,       5.1865178,       -3.844747 ],
 ],
 [
  [               0,               0,               0,     -0.13563013,      0.16300132,    -0.036400485 ],
  [               0,               0,               0,       15.604446,      -20.512557,       8.7640517 ],
  [               0,               0,               0,      -3.1456074,       4.1171387,      -2.4754265 ],
  [               0,               0,               0,        4.501123,      -5.1392752,       1.4350054 ],
  [               0,               0,               0,    -0.013977007,     0.015775343,   -0.0035390617 ],
  [               0,               0,               0,       39.642142,      -44.268676,       12.037792 ],
  [               0,               0,               0,      -44.634558,       56.409263,      -17.650883 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0099935021 ],
  [               0,               0,               0,               0,               0,       2.2305112 ],
  [               0,               0,               0,               0,               0,      -1.2004804 ],
  [               0,               0,               0,               0,               0,      0.01327573 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0084179776,     -0.03194614,     0.032639565 ],
  [               0,               0,               0,        3.826203,      -5.2825665,       3.8065473 ],
  [               0,               0,               0,      -1.3452113,       2.0170962,      -1.9506077 ],
  [               0,               0,               0,      0.97285001,      -1.3194866,      0.48911862 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.005465067,   -0.0074103433 ],
  [               0,               0,               0,               0,      -16.664802,      -16.315287 ],
  [               0,               0,               0,               0,      -3535.4757,       4800.8381 ],
  [               0,               0,               0,               0,      -793.98714,       1159.9387 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.072426044,     0.026984939 ],
  [               0,               0,               0,               0,      -58.761076,       86.611007 ],
  [               0,               0,               0,               0,       44.601976,      -63.731002 ],
  [               0,               0,               0,               0,      0.13967105,     -0.33993272 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer2_etaoffv4_37_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer2_etaoffv4_37_b = [
 [
  [               0,               0,               0,    0.0018002291,  -0.00068785037,    0.0074431047 ],
  [               0,               0,               0,      -5.5048083,       2.4076499,       3.7024436 ],
  [               0,               0,               0,      -1.5890242,      0.70481508,      -1.4230602 ],
  [               0,               0,               0,       903.17188,      -214.26152,       27.298905 ],
  [               0,               0,               0,        6.742945,      -1.5964163,      0.20301557 ],
  [               0,               0,               0,       0.3060016,     -0.21362248,     0.041135561 ],
  [               0,               0,               0,      -2.6308694,       1.2078661,      -3.1258886 ],
 ],
 [
  [               0,               0,               0,     0.005983226,   -0.0089559268,    0.0087867306 ],
  [               0,               0,               0,      -11.986476,         11.2267,       1.7479878 ],
  [               0,               0,               0,      -2.2046418,       1.1786611,      -1.4004626 ],
  [               0,               0,               0,       1.9809677,      -1.1727025,      0.11790199 ],
  [               0,               0,               0,    0.0020767328,   -0.0009276425,    0.0008676135 ],
  [               0,               0,               0,      -3.1533722,       4.6072365,     -0.72548797 ],
  [               0,               0,               0,      0.29065152,      -3.0982489,     0.054135104 ],
 ],
 [
  [               0,               0,               0,               0,     0.095215635,     -0.06483648 ],
  [               0,               0,               0,               0,       5.2740596,       -1.749679 ],
  [               0,               0,               0,               0,       3.6330845,      -4.1416989 ],
  [               0,               0,               0,               0,      -2.6692353,       2.1449837 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0098596566,     0.015307302,     0.001957005 ],
  [               0,               0,               0,       1.2688967,      -2.4089721,       4.4622248 ],
  [               0,               0,               0,       1.7663184,      -4.3385656,       1.0549487 ],
  [               0,               0,               0,      0.18682971,    -0.018191485,      0.07558172 ],
  [               0,               0,               0,    0.0089992208,    -0.020323865,     0.012082972 ],
  [               0,               0,               0,      -3.9587033,       9.8422127,      -5.3046609 ],
  [               0,               0,               0,       9.9044337,      -19.298319,       6.8041728 ],
 ],
 [
  [               0,     0.015839212,     0.040334031,     0.040737109,    -0.019342772,     -0.20516683 ],
  [               0,       383.92042,      -222.11508,      -1025.4644,      -1003.0972,       2534.5894 ],
  [               0,       -1172.128,       1497.3514,       1607.3867,      -616.06913,      -1896.4477 ],
  [               0,       1012.9697,      -539.79191,      -2069.9459,      -1820.9471,       4200.1716 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.029835218,    0.0016339447,     0.067024416 ],
  [               0,               0,               0,      -442.89796,       5.2756495,       905.31935 ],
  [               0,               0,               0,       46.003713,      -118.21982,       75.417156 ],
  [               0,               0,               0,      -0.1251737,     -0.53832586,     -0.82794311 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer2_etaoffv4_37_e = [
 [
  [               0,               0,     -0.28241007,      0.45643481,      0.47059739,     -0.77614478 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.012588448,    -0.017637741 ],
  [               0,               0,               0,               0,       11.976514,       21.266092 ],
  [               0,               0,               0,               0,       8.3151909,      -14.719061 ],
  [               0,               0,               0,               0,      -110.52482,       164.70517 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.53222538,     -0.79811247 ],
  [               0,               0,               0,               0,       502.72138,      -755.21052 ],
  [               0,               0,               0,               0,       130.14688,      -198.66446 ],
  [               0,               0,               0,               0,      -86.728862,       131.09431 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     -0.15139306,      0.50978132,     -0.41621859 ],
  [               0,               0,               0,       39.597798,      -133.55805,       114.53813 ],
  [               0,               0,               0,      -11.748285,       39.157512,      -33.715748 ],
  [               0,               0,               0,       3.0168173,       -9.530671,       7.5098529 ],
  [               0,               0,               0,   -0.0083531276,     0.035654403,    -0.035741609 ],
  [               0,               0,               0,       12.547982,      -44.601209,       40.176537 ],
  [               0,               0,               0,       43.430434,      -154.72405,       134.57223 ],
 ],
 [
  [               0,               0,               0,      0.30285816,      -1.2599127,       1.3257794 ],
  [               0,               0,               0,      -12.016535,        49.61505,      -49.443779 ],
  [               0,               0,               0,        3.122772,      -12.807137,       12.102038 ],
  [               0,               0,               0,     -0.10606401,      0.39251434,     -0.36390458 ],
  [               0,               0,               0,    -0.014719311,     0.061819333,    -0.064779407 ],
  [               0,               0,               0,       160.34301,      -668.36548,       699.43187 ],
  [               0,               0,               0,       118.83726,      -507.81853,       503.04268 ],
 ],
 [
  [               0,               0,               0,    -0.026224998,    0.0023042158,      0.16961255 ],
  [               0,               0,               0,       33.483443,      -155.44126,       181.57471 ],
  [               0,               0,               0,      -3.6424622,       15.834176,      -17.968974 ],
  [               0,               0,               0,      -8.3506602,         39.8377,      -47.497843 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,      0.01082804 ],
  [               0,               0,               0,               0,               0,       1.7877043 ],
  [               0,               0,               0,               0,               0,      -1.1023438 ],
  [               0,               0,               0,               0,               0,     0.047962349 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.81764471,      -2.0172219 ],
  [               0,               0,               0,               0,        6.280516,       24.340779 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,       89.437495,      -221.24046 ],
  [               0,               0,               0,               0,      0.37192943,      0.96922228 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer2_etaoffv4_37_b = [
 [
  [               0,               0,               0,    -0.017200938,    0.0043464557,     0.007203211 ],
  [               0,               0,               0,     -0.75624183,       2.0278909,         3.92492 ],
  [               0,               0,               0,       -0.969399,      0.55531793,      -1.4236687 ],
  [               0,               0,               0,       1.5817442,     -0.83220404,      0.10302173 ],
  [               0,               0,               0,     0.013380407,   -0.0047912157,    0.0010778411 ],
  [               0,               0,               0,       8.4046285,      -4.1929462,      0.81228396 ],
  [               0,               0,               0,      -15.724603,       8.5885345,       -2.626535 ],
 ],
 [
  [               0,               0,               0,               0,    0.0065091465,    0.0019602231 ],
  [               0,               0,               0,               0,       -4.469732,       7.3360896 ],
  [               0,               0,               0,               0,     -0.28679154,      -1.3878488 ],
  [               0,               0,               0,               0,      0.50047829,     -0.13742653 ],
  [               0,               0,               0,               0,   -0.0013412716,    0.0018681294 ],
  [               0,               0,               0,               0,     -0.80827962,       1.2499059 ],
  [               0,               0,               0,               0,      -2.4910229,       -6.857467 ],
 ],
 [
  [               0,               0,               0,               0,      0.17049616,     -0.12807508 ],
  [               0,               0,               0,               0,      -8.5609327,       10.567593 ],
  [               0,               0,               0,               0,       17.571446,        -15.6743 ],
  [               0,               0,               0,               0,      -3.9464826,       3.3809779 ],
  [               0,               0,               0,               0,    -0.057626028,     0.050159203 ],
  [               0,               0,               0,               0,       11.611885,      -8.7439672 ],
  [               0,               0,               0,               0,      -92.228622,       64.580134 ],
 ],
 [
  [               0,               0,  -2.5364476e-05,   -0.0055976574,     0.011457406,    0.0053533538 ],
  [               0,               0,      -2.2738662,        8.791962,      -11.349938,       7.4969257 ],
  [               0,               0,       2.4378912,      -7.9068581,       8.3185105,      -4.1237871 ],
  [               0,               0,      -0.6083042,       1.9064313,      -1.7343246,      0.50643386 ],
  [               0,               0,     0.012973126,    -0.040424612,     0.041157834,    -0.013569509 ],
  [               0,               0,        129.9275,      -447.08596,       495.22878,      -181.20208 ],
  [               0,               0,       4.3889099,      -3.4961279,      -7.3602989,       7.7983858 ],
 ],
 [
  [               0,               0,       2.8765489,      -3.6587892,      -5.1370405,        6.526128 ],
  [               0,               0,      -936.00425,       1309.6387,       1569.0845,      -2199.3311 ],
  [               0,               0,       27.713496,      -22.565988,      -47.416933,       34.806672 ],
  [               0,               0,        79.26331,      -106.45325,      -147.95541,        198.9117 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.068966548,    0.0050156441,     -0.12795968 ],
  [               0,               0,               0,      0.36829337,        8.432698,       27.120009 ],
  [               0,               0,               0,      -24.432227,       4.1317066,       40.080058 ],
  [               0,               0,               0,      0.13774249,       0.2050383,     0.085800262 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer2_etaoffv4_37_e = [
 [
  [               0,               0,        -1.29577,       1.8774715,       2.5197208,       -3.651071 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.010049623,     0.012356809,    0.0021088708,    -0.059782053 ],
  [               0,               0,       203.18282,      -395.58774,      -519.71336,       983.81355 ],
  [               0,               0,      -1272.5273,       1933.8518,       2786.8248,      -4236.5071 ],
  [               0,               0,      -271.95706,       391.86405,       578.46212,      -832.53191 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.027255794,     0.041625061 ],
  [               0,               0,               0,               0,       21.882343,      -32.990568 ],
  [               0,               0,               0,               0,      -29.331446,       44.250589 ],
  [               0,               0,               0,               0,       8.6750798,      -13.114139 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.079346959,       0.2712487,     -0.22338519 ],
  [               0,               0,               0,       35.003113,      -118.83446,       103.78423 ],
  [               0,               0,               0,      -1.8848636,       6.2517161,      -6.4835176 ],
  [               0,               0,               0,       8.8715219,      -30.777644,       27.734262 ],
  [               0,               0,               0,    -0.009372797,     0.027727199,    -0.011088041 ],
  [               0,               0,               0,      -1.6989515,       5.9198695,      -4.9872942 ],
  [               0,               0,               0,        10.45529,      -35.832004,        21.36605 ],
 ],
 [
  [               0,               0,               0,    0.0025255065,   -0.0026786241,    0.0022452552 ],
  [               0,               0,               0,       7.9164916,      -37.363311,       46.446382 ],
  [               0,               0,               0,       1.9770408,      -7.6734883,       6.1258183 ],
  [               0,               0,               0,      -6.9344613,       27.335715,      -26.838185 ],
  [               0,               0,               0,    0.0025789043,  -0.00020380932,   -0.0079522942 ],
  [               0,               0,               0,       3.2463595,      -14.124012,        15.46091 ],
  [               0,               0,               0,       11.693681,      -44.740774,       40.917184 ],
 ],
 [
  [               0,               0,               0,     -0.01643974,   0.00070472625,      0.10560255 ],
  [               0,               0,               0,       40.344775,      -184.87137,       213.50413 ],
  [               0,               0,               0,       11.742086,      -56.797743,       67.562103 ],
  [               0,               0,               0,      -12.736611,       60.631054,      -72.134669 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0015890564,    0.0051986317 ],
  [               0,               0,               0,               0,       -188.9095,       467.61938 ],
  [               0,               0,               0,               0,       34.097263,      -84.709199 ],
  [               0,               0,               0,               0,       17.930445,      -44.135019 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   0.00038811872,  -0.00017749363,  -5.6014245e-05,   -0.0046735419 ],
  [               0,               0,      -1.0530668,      -1.3942471,      -3.4565732,      -4.5697452 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,       10.423849,      -12.804315,       70.136889,      -32.381795 ],
  [               0,               0,       2.0590746,       6.3811091,       30.255966,       39.940524 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer2_etaoffv4_37_b = [
 [
  [               0,               0,               0,     0.039268717,    -0.011855082,    0.0076511854 ],
  [               0,               0,               0,      -17.509299,        5.152324,       4.3150847 ],
  [               0,               0,               0,       4.9522678,      -2.0328386,      -1.2546232 ],
  [               0,               0,               0,     -0.73691516,        1.799551,      0.24497682 ],
  [               0,               0,               0,     0.050209888,    -0.010490819,    0.0041753213 ],
  [               0,               0,               0,        -5.08231,       1.9746233,    0.0091694576 ],
  [               0,               0,               0,       7.2428711,      -4.8316108,       -2.081916 ],
 ],
 [
  [               0,               0,               0,     0.014277635,    -0.026159894,     0.020379001 ],
  [               0,               0,               0,      -6.7367889,       8.8614679,       0.8970402 ],
  [               0,               0,               0,      -1.9723698,       2.3169772,      -2.0002885 ],
  [               0,               0,               0,       1.3756016,      -1.5107941,      0.43274576 ],
  [               0,               0,               0,   -0.0036457426,    0.0064356565,   -0.0030883151 ],
  [               0,               0,               0,       5.2467415,      -4.3148833,       2.0826418 ],
  [               0,               0,               0,       -1.975094,    0.0023712694,      -7.2637883 ],
 ],
 [
  [               0,               0,               0,               0,      0.25038521,       -0.187689 ],
  [               0,               0,               0,               0,      -13.167677,       13.831684 ],
  [               0,               0,               0,               0,       19.863417,      -17.361375 ],
  [               0,               0,               0,               0,      -14.381291,       11.619817 ],
  [               0,               0,               0,               0,     0.027110093,    -0.023605105 ],
  [               0,               0,               0,               0,       19.988928,      -14.857544 ],
  [               0,               0,               0,               0,      -63.496726,       49.531932 ],
 ],
 [
  [               0,               0,               0,   -0.0033548671,     0.008235788,    0.0028859859 ],
  [               0,               0,               0,       2.4860993,      -6.8597728,       8.0670492 ],
  [               0,               0,               0,     -0.92694233,       1.5844015,       -2.073645 ],
  [               0,               0,               0,       1.2411392,     -0.16156175,      0.41053224 ],
  [               0,               0,               0,    0.0077599208,   0.00089133259,    0.0025119991 ],
  [               0,               0,               0,      0.46156543,     -0.85438191,      0.46340284 ],
  [               0,               0,               0,      0.59660548,      -1.8339567,      -1.6757749 ],
 ],
 [
  [               0,               0,               0,       26.784001,      -71.988754,       48.372979 ],
  [               0,               0,               0,      -1017.9364,       2775.9611,      -1889.4409 ],
  [               0,               0,               0,      -152.16854,       473.24466,      -364.28261 ],
  [               0,               0,               0,       1442.9423,      -3961.5843,       2719.1605 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      0.02447871,    0.0030901926,    -0.040334804 ],
  [               0,               0,               0,      -390.00981,        4.269198,       802.35462 ],
  [               0,               0,               0,        33.73885,      -85.846545,       52.866588 ],
  [               0,               0,               0,   -0.0068089948,     -0.19496833,     -0.45941123 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer2_etaoffv4_37_e = [
 [
  [               0,               0,     -0.51457772,      0.76142361,      0.90468353,      -1.3441187 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   -0.0026328321,   -0.0015898171,    0.0066402285,    0.0032356439 ],
  [               0,               0,        3.061544,       4.1765743,       5.6678952,       11.690563 ],
  [               0,               0,     -0.39226948,       1.0832326,      -3.5193023,       1.8330113 ],
  [               0,               0,    -0.047383459,      0.93368257,     -0.88127172,     -0.20488224 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.030181464,    -0.045021563 ],
  [               0,               0,               0,               0,       1699.8954,      -2554.1431 ],
  [               0,               0,               0,               0,      -562.48644,       846.40201 ],
  [               0,               0,               0,               0,       849.39896,      -1283.8138 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.001315495,    0.0060817817,   5.2141021e-05 ],
  [               0,               0,               0,       3.5816827,      -12.535231,        14.66684 ],
  [               0,               0,               0,       -0.946016,         2.48097,      -2.7771313 ],
  [               0,               0,               0,      -3.3950093,       12.652472,      -11.659345 ],
  [               0,               0,               0,    0.0059020082,    -0.023075216,     0.023455152 ],
  [               0,               0,               0,       8.3103603,      -26.933528,       22.060168 ],
  [               0,               0,               0,       8.3883995,      -33.073923,        30.20319 ],
 ],
 [
  [               0,               0,               0,   -0.0033373666,      0.01157575,   -0.0012091434 ],
  [               0,               0,               0,      -2.3473822,       9.1075208,      -5.8200844 ],
  [               0,               0,               0,      0.84506679,      -3.3897256,        2.146748 ],
  [               0,               0,               0,      -31.714638,       112.93596,      -89.907647 ],
  [               0,               0,               0,    -0.036493784,   0.00092310377,      0.22761545 ],
  [               0,               0,               0,     0.030916117,     -0.12026938,      0.12489566 ],
  [               0,               0,               0,       7.8347941,       -31.25605,       28.394677 ],
 ],
 [
  [               0,               0,               0,   -0.0087546807,   -0.0041720116,     0.068186735 ],
  [               0,               0,               0,       105.20523,      -489.81444,       572.51055 ],
  [               0,               0,               0,      -1.5320366,       6.0306566,      -6.9115837 ],
  [               0,               0,               0,      -7.0018874,       47.374569,      -54.539398 ],
  [               0,               0,               0,     -0.14461378,      0.01890136,      0.93618691 ],
  [               0,               0,               0,       0.2949316,       -1.554507,       2.0453022 ],
  [               0,               0,               0,       16.678265,      -76.617219,       84.867785 ],
 ],
 [
  [               0,               0,               0,               0,       3.9224402,      -9.6086167 ],
  [               0,               0,               0,               0,      -8.9048991,       23.770734 ],
  [               0,               0,               0,               0,       54.630244,      -135.67911 ],
  [               0,               0,               0,               0,      -46.034619,       113.37617 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     -0.23049957,        0.940306,     -0.39279802,      -1.2932569 ],
  [               0,               0,      -1.1161333,      -1.3461989,      -3.4031602,      -4.5194193 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,      -1.2760879,       16.454094,       31.961845,      -16.718712 ],
  [               0,               0,       1.1500229,       6.4977221,       12.009462,       11.898252 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer2_etaoffv4_37_b = [
 [
  [               0,               0,      0.13762918,    -0.092383494,     0.027354391,    0.0046701659 ],
  [               0,               0,       29.931534,      -13.440946,      -2.3703566,       5.1746517 ],
  [               0,               0,       13.083033,      -10.363983,       2.3367928,        -1.50417 ],
  [               0,               0,      -11.690951,       7.9671231,      -1.4205585,     0.057106951 ],
  [               0,               0,    0.0050748656,    0.0062866488,   -0.0060096745,    0.0014193982 ],
  [               0,               0,      -18.792435,       36.341847,       -10.54794,       1.2289394 ],
  [               0,               0,      -68.443727,      -3.0467477,       5.4246768,      -1.9284811 ],
 ],
 [
  [               0,               0,               0,    0.0080708455,   -0.0077484924,    0.0088728413 ],
  [               0,               0,               0,      -10.404248,       10.644792,       1.7745336 ],
  [               0,               0,               0,       1.4555344,      -2.1852513,     -0.65840373 ],
  [               0,               0,               0,      0.55514391,    0.0011867177,     -0.12281905 ],
  [               0,               0,               0,    0.0074931466,   -0.0078046871,    0.0026151622 ],
  [               0,               0,               0,      -1.6217684,      0.89972002,      0.71035266 ],
  [               0,               0,               0,       5.7952171,      -10.848837,       2.3878452 ],
 ],
 [
  [               0,               0,               0,               0,    0.0030143719,    0.0053719527 ],
  [               0,               0,               0,               0,       20.065115,      -11.988321 ],
  [               0,               0,               0,               0,       2.4618316,      -3.5196204 ],
  [               0,               0,               0,               0,      -3.6873304,       3.1698494 ],
  [               0,               0,               0,               0,    0.0038251505,   -0.0015579874 ],
  [               0,               0,               0,               0,       4.2444442,      -2.8682262 ],
  [               0,               0,               0,               0,        1.419929,      -4.0415598 ],
 ],
 [
  [               0,               0,               0,    -0.015406452,     0.031447167,   -0.0077134707 ],
  [               0,               0,               0,       6.0072876,      -13.296335,        10.85974 ],
  [               0,               0,               0,     -0.23740949,      0.15921003,      -1.3271992 ],
  [               0,               0,               0,      0.47961097,     -0.52761938,      0.20096253 ],
  [               0,               0,               0,    0.0089669791,    -0.018050551,    0.0099118204 ],
  [               0,               0,               0,      -2.0556832,       4.5677548,      -2.1917369 ],
  [               0,               0,               0,       6.6864494,       -14.89441,       5.7670995 ],
 ],
 [
  [               0,               0,               0,      0.42694777,      -0.1563256,     -0.56712601 ],
  [               0,               0,               0,       1579.6531,      -4289.4632,       2914.2778 ],
  [               0,               0,               0,       -190.9177,       547.67143,      -393.56633 ],
  [               0,               0,               0,       740.17792,      -2019.5204,       1377.6752 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,  -0.00041268293,    0.0031612245,     0.011026739 ],
  [               0,               0,               0,      -402.74656,       4.9344531,       828.65975 ],
  [               0,               0,               0,        21.67934,      -51.861553,       29.022547 ],
  [               0,               0,               0,    -0.027700124,     -0.14531729,     -0.44468651 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer2_etaoffv4_37_e = [
 [
  [               0,               0,      -1.2575251,       1.8188758,       2.4374957,      -3.5256546 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.011658926,   -0.0052127336,    0.0094724653,     0.039209768 ],
  [               0,               0,      -209.83916,       333.55466,       470.41804,      -742.26925 ],
  [               0,               0,      -150.89299,       214.33044,       324.28328,      -461.76929 ],
  [               0,               0,      -60.701908,       91.314266,       133.45134,      -200.57227 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.040528382,    -0.058842801 ],
  [               0,               0,               0,               0,       151.68699,       -225.1314 ],
  [               0,               0,               0,               0,      -178.35124,       268.18585 ],
  [               0,               0,               0,               0,       190.63684,      -288.13344 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0083695599,    -0.028200956,     0.029575133 ],
  [               0,               0,               0,    -0.017451939,      0.14520348,       3.8220509 ],
  [               0,               0,               0,       10.524279,      -36.189354,       29.684502 ],
  [               0,               0,               0,      -8.4105895,       28.946981,      -24.786407 ],
  [               0,               0,               0,    0.0036678228,    -0.012597201,     0.011873628 ],
  [               0,               0,               0,      -1.0774863,        3.698149,      -2.5105495 ],
  [               0,               0,               0,       41.996061,      -143.66276,       120.74282 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0027986495,     0.014314662 ],
  [               0,               0,               0,               0,     -0.58910768,       4.2188073 ],
  [               0,               0,               0,               0,    -0.020372113,      -1.1971369 ],
  [               0,               0,               0,               0,      -117.32548,       205.31926 ],
  [               0,               0,               0,               0,       1.3568833,      -2.3727893 ],
  [               0,               0,               0,               0,   -0.0017956967,    0.0046435219 ],
  [               0,               0,               0,               0,     -0.33929966,    -0.043729316 ],
 ],
 [
  [               0,               0,               0,    0.0015636888,  -0.00067427961,    0.0022360487 ],
  [               0,               0,               0,       53.120643,      -256.66568,       312.58205 ],
  [               0,               0,               0,       11.001264,      -52.493223,       61.406424 ],
  [               0,               0,               0,       22.645697,       31.012393,       -225.5998 ],
  [               0,               0,               0,    -0.061879722,     0.037538767,      0.62376834 ],
  [               0,               0,               0,     -0.12887441,      0.61388183,      -0.7300445 ],
  [               0,               0,               0,      -1.8276301,       5.1820017,      -2.8876168 ],
 ],
 [
  [               0,               0,               0,               0,     0.012628814,     0.003917406 ],
  [               0,               0,               0,               0,      -2.1135682,       7.0786284 ],
  [               0,               0,               0,               0,       40.786014,      -101.53081 ],
  [               0,               0,               0,               0,      -37.956349,       93.488108 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0043358598,  -0.00024634371,      -0.0260045 ],
  [               0,               0,               0,      -3.4091645,      -4.6121801,      -7.4537854 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,       780.14672,       25.928461,      -4703.6544 ],
  [               0,               0,               0,       9.5577631,       9.2174374,       26.336334 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer2_etaoffv4_37_b = [
 [
  [               0,               0,               0,    0.0079480548,    -0.002979046,    0.0072382269 ],
  [               0,               0,               0,     -0.21627079,     -0.37679946,       4.9574481 ],
  [               0,               0,               0,      0.64862334,     -0.31614194,      -1.3135523 ],
  [               0,               0,               0,     -0.93869533,      0.39098599,    -0.063630657 ],
  [               0,               0,               0,     0.011280987,     -0.00458729,   0.00095881413 ],
  [               0,               0,               0,      -16.525569,       6.6593439,     0.084607195 ],
  [               0,               0,               0,       2.3752734,     -0.49264856,      -1.3249737 ],
 ],
 [
  [               0,               0,               0,   -0.0064697061,    0.0082808126,    0.0053615868 ],
  [               0,               0,               0,      -2.2910138,       1.3484947,       4.3253895 ],
  [               0,               0,               0,      -2.0859508,       2.0634422,      -1.8684811 ],
  [               0,               0,               0,       33.609422,      -47.338324,       78.984348 ],
  [               0,               0,               0,     -0.12809992,      0.12438267,      0.47318376 ],
  [               0,               0,               0,      0.13633953,     -0.10983683,     0.025444091 ],
  [               0,               0,               0,      0.66588736,      -1.0065948,      -2.7417687 ],
 ],
 [
  [               0,               0,               0,               0,     0.070304677,    -0.048713092 ],
  [               0,               0,               0,               0,       1.0535427,       3.3879344 ],
  [               0,               0,               0,               0,       13.429739,      -12.179846 ],
  [               0,               0,               0,               0,      -8.4980796,       6.8583232 ],
  [               0,               0,               0,               0,    -0.029945988,     0.024510645 ],
  [               0,               0,               0,               0,      -7.0464663,       6.3049724 ],
  [               0,               0,               0,               0,       12.484411,      -12.157404 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00057736827,    0.0085956136 ],
  [               0,               0,               0,               0,     -0.60573658,       4.5329881 ],
  [               0,               0,               0,               0,     -0.38432482,      -1.0516224 ],
  [               0,               0,               0,               0,       1.0373364,       536.79176 ],
  [               0,               0,               0,               0,     -0.30436907,       4.6164354 ],
  [               0,               0,               0,               0,    0.0076998435,   -0.0025690821 ],
  [               0,               0,               0,               0,   -0.0024877968,      -3.1315539 ],
 ],
 [
  [               0,               0,       0.1924443,    0.0095573828,     -0.17176159,     -0.25753132 ],
  [               0,               0,       710.02028,      -960.30525,       -1307.118,       1770.4896 ],
  [               0,               0,       29.549328,      -25.855072,      -63.588185,       58.067153 ],
  [               0,               0,       513.60948,      -700.77595,      -966.69051,       1319.0638 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.010400453,    0.0035329316,     0.034196546 ],
  [               0,               0,               0,      -340.73947,       5.9656352,       705.71881 ],
  [               0,               0,               0,       39.118035,      -107.63187,        72.61375 ],
  [               0,               0,               0,     0.063433783,    -0.085066427,     -0.29992115 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer2_etaoffv4_37_e = [
 [
  [               0,               0,      -2.9996149,        4.332871,       5.9657267,      -8.6169251 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,       3.7126991,      -11.086715,       8.2801562 ],
  [               0,               0,               0,        -982.311,       3018.5738,      -2311.0214 ],
  [               0,               0,               0,       298.45147,      -915.63157,        699.5295 ],
  [               0,               0,               0,       27.696458,      -71.101227,       44.838314 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.042510549,     0.066489169 ],
  [               0,               0,               0,               0,       364.20634,      -545.90057 ],
  [               0,               0,               0,               0,      -255.16182,        385.0999 ],
  [               0,               0,               0,               0,       256.21583,      -387.36353 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     -0.02641642,     0.079421142,    -0.047343345 ],
  [               0,               0,               0,       20.777316,      -69.230259,       60.712787 ],
  [               0,               0,               0,      -1.3951517,       4.3033898,      -4.4888573 ],
  [               0,               0,               0,      0.10084987,     -0.14015068,    -0.056860711 ],
  [               0,               0,               0,    0.0082051691,    -0.030761928,     0.029194003 ],
  [               0,               0,               0,       5.2073333,       -15.88641,       13.349372 ],
  [               0,               0,               0,       6.1144229,      -20.343096,       11.907534 ],
 ],
 [
  [               0,               0,               0,   -0.0023540313,    0.0097110707,   -0.0010298121 ],
  [               0,               0,               0,        1.050751,      -5.9733467,       10.891078 ],
  [               0,               0,               0,     0.071238361,      -0.1277126,      -1.3042861 ],
  [               0,               0,               0,      -99.677876,       301.61079,      -125.60527 ],
  [               0,               0,               0,      -0.2241888,      0.15155521,       1.4277742 ],
  [               0,               0,               0,    0.0024466387,    -0.009132092,    0.0089431387 ],
  [               0,               0,               0,       2.3021868,      -8.7182962,       5.6435219 ],
 ],
 [
  [               0,               0,               0,   -0.0037838666,   0.00024065593,      0.03142692 ],
  [               0,               0,               0,        31.24494,      -147.05425,       175.54622 ],
  [               0,               0,               0,       8.5314467,       -40.94235,       47.908801 ],
  [               0,               0,               0,      -17.889791,       32.040418,       2.0800396 ],
  [               0,               0,               0,     0.016698323,     0.018559106,      0.11989445 ],
  [               0,               0,               0,     -0.22849737,       1.1056193,      -1.3345504 ],
  [               0,               0,               0,      -9.7884235,       47.241866,      -57.336215 ],
 ],
 [
  [               0,               0,               0,               0,   0.00088254122,    0.0075922756 ],
  [               0,               0,               0,               0,      -3.2419235,       10.744295 ],
  [               0,               0,               0,               0,       9.3465249,      -24.336435 ],
  [               0,               0,               0,               0,    -0.054987137,      0.21305655 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0026434575,    0.0018957219,   -0.0054605062,    -0.038248185 ],
  [               0,               0,      -1.1906972,      -1.3849465,      -3.4360638,      -4.5247617 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,       427.97441,       358.57656,      -861.64632,      -6500.6468 ],
  [               0,               0,      0.88628118,       1.9302221,       5.4060526,       17.844273 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer2_etaoffv4_37_b = [
 [
  [               0,               0,               0,    -0.024284498,     0.011830097,    0.0060909614 ],
  [               0,               0,               0,       7.1634842,      -4.2458523,       5.4580282 ],
  [               0,               0,               0,      0.70596829,      -0.1939418,      -1.3187236 ],
  [               0,               0,               0,     0.099135116,    -0.064983015,    -0.038723389 ],
  [               0,               0,               0,    0.0067007265,   -0.0031243621,    0.0007500412 ],
  [               0,               0,               0,       14.235549,      -6.0381041,       1.1811455 ],
  [               0,               0,               0,      -12.848076,       6.0905603,      -1.6522033 ],
 ],
 [
  [               0,               0,               0,     0.013687229,    -0.011883407,     0.010900975 ],
  [               0,               0,               0,      -6.6406463,       5.4990014,       3.5609231 ],
  [               0,               0,               0,       1.2090909,      -1.2553105,      -1.0422405 ],
  [               0,               0,               0,    -0.082682844,      0.14619216,    -0.055973794 ],
  [               0,               0,               0,   -0.0025244288,    0.0021494534,  -0.00060993563 ],
  [               0,               0,               0,      -2.4954083,       1.4777157,       1.6013068 ],
  [               0,               0,               0,      -14.595439,       15.532509,      -24.343407 ],
 ],
 [
  [               0,               0,               0,               0,     0.018287095,   -0.0097498397 ],
  [               0,               0,               0,               0,       1.4955401,        4.360207 ],
  [               0,               0,               0,               0,       21.779678,      -18.917288 ],
  [               0,               0,               0,               0,        -12.3768,       9.9345357 ],
  [               0,               0,               0,               0,   -0.0052262099,    0.0057597732 ],
  [               0,               0,               0,               0,       2.4070447,     -0.87547271 ],
  [               0,               0,               0,               0,       16.635323,      -15.028881 ],
 ],
 [
  [               0,               0,               0,    -0.003839656,     0.011538662,  -0.00078165913 ],
  [               0,               0,               0,       3.7016441,      -9.8852692,       10.652911 ],
  [               0,               0,               0,       0.5327883,      -1.3876296,      -0.5789598 ],
  [               0,               0,               0,     -0.12084657,      0.62004856,     -0.43246079 ],
  [               0,               0,               0,    0.0030332063,   -0.0065985825,    0.0042333497 ],
  [               0,               0,               0,     -0.69655616,      0.66971372,       0.8323444 ],
  [               0,               0,               0,      0.14191345,      -2.1867205,      0.14593389 ],
 ],
 [
  [               0,               0,      0.18522222,    -0.034650041,     -0.19678795,     -0.12698544 ],
  [               0,               0,        657.6878,      -869.68736,      -1202.9641,       1593.8047 ],
  [               0,               0,      -643.10059,       902.30655,       1195.5297,      -1679.5417 ],
  [               0,               0,        735.2134,      -1002.4064,      -1383.2403,       1886.0538 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.15124866,      -0.1988441 ],
  [               0,               0,               0,               0,      -905.32315,       1316.4254 ],
  [               0,               0,               0,               0,      -26.488461,       46.931522 ],
  [               0,               0,               0,               0,       302.96377,      -435.63909 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer2_etaoffv4_37_e = [
 [
  [       72.636589,      -146.56758,      -117.02208,       172.25328,       420.73756,      -433.15224 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,       4.3434315,      -6.3805923,      -8.7920156,       12.925727 ],
  [               0,               0,       715.13549,      -1057.4696,      -1537.3472,       2275.2116 ],
  [               0,               0,       785.24696,      -1154.6373,      -1721.4481,       2529.7432 ],
  [               0,               0,      -778.85298,        1147.911,       1692.1078,      -2493.4563 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.030749021,    -0.042657943 ],
  [               0,               0,               0,               0,       72.273032,      -105.26855 ],
  [               0,               0,               0,               0,       -123.3007,       185.14332 ],
  [               0,               0,               0,               0,       120.72078,      -182.44079 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.009164847,    -0.029880635,     0.032747138 ],
  [               0,               0,               0,       3.6053922,      -13.214683,        15.69618 ],
  [               0,               0,               0,       1.1276522,      -3.8599962,       1.9326489 ],
  [               0,               0,               0,       -4.600991,       15.235142,      -8.8346108 ],
  [               0,               0,               0,   -0.0013210174,     0.005450184,     0.026141826 ],
  [               0,               0,               0,      -1.0276222,       3.4538692,      -2.8330294 ],
  [               0,               0,               0,       1.1788058,       -3.889157,      0.14742518 ],
 ],
 [
  [               0,               0,               0,    0.0012268333,    0.0011889416,    0.0010744272 ],
  [               0,               0,               0,       1.0578407,      -7.5529145,       14.355402 ],
  [               0,               0,               0,      0.38111067,     -0.98028241,     -0.92831093 ],
  [               0,               0,               0,       -3.627822,       14.187083,      -13.671802 ],
  [               0,               0,               0,    0.0015641472,   -0.0038955605,    0.0030865025 ],
  [               0,               0,               0,       4.3363217,      -18.926591,       20.780076 ],
  [               0,               0,               0,       11.969523,      -44.817516,        39.35112 ],
 ],
 [
  [               0,               0,               0,   -0.0094377308,   -0.0012447872,     0.067789894 ],
  [               0,               0,               0,       60.630874,      -283.53273,       333.90016 ],
  [               0,               0,               0,        12.83891,       -62.05923,       73.763395 ],
  [               0,               0,               0,      -15.966693,       75.890903,       -90.14111 ],
  [               0,               0,               0,     0.017965916,    -0.073146038,     0.072558796 ],
  [               0,               0,               0,      -1.4941368,      -1.4862903,       12.836108 ],
  [               0,               0,               0,       93.340145,      -434.66739,       503.85835 ],
 ],
 [
  [               0,               0,               0,               0,     0.068001612,     -0.14995485 ],
  [               0,               0,               0,               0,      -0.7727494,       4.2992707 ],
  [               0,               0,               0,               0,       33.571439,      -83.897275 ],
  [               0,               0,               0,               0,      -29.452993,       72.567787 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0031143912,    -0.003318809,    0.0087671069,    -0.048868784 ],
  [               0,               0,      -1.2153757,      -1.3709136,      -3.4019916,       -4.442577 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,        3.538569,       6.0256587,       13.438161,      -41.130482 ],
  [               0,               0,       1.2918419,       3.1127675,       11.875433,      0.88187071 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer2_etaoffv4_37_b = [
 [
  [               0,               0,               0,    -0.015217677,    0.0096205425,    0.0060762133 ],
  [               0,               0,               0,       7.3099931,      -4.7249968,        5.733494 ],
  [               0,               0,               0,      -1.7161341,       0.5466124,      -1.3168599 ],
  [               0,               0,               0,       1.5255217,     -0.54568118,    -0.036799497 ],
  [               0,               0,               0,   -0.0014512436,  -0.00060000228,   0.00069304223 ],
  [               0,               0,               0,        1.103538,       -1.906771,       1.0290767 ],
  [               0,               0,               0,      -27.708555,       12.114594,       -1.820123 ],
 ],
 [
  [               0,               0,               0,   -0.0085328856,     0.011544694,    0.0034116015 ],
  [               0,               0,               0,       6.1645827,      -9.4128356,       8.4238854 ],
  [               0,               0,               0,      -1.6265533,       1.5731821,      -1.6989236 ],
  [               0,               0,               0,      0.95727923,     -0.82667901,      0.12839628 ],
  [               0,               0,               0,   0.00080826639,  -0.00082266012,   0.00063250065 ],
  [               0,               0,               0,     -0.70083893,      0.74370235,      0.60947886 ],
  [               0,               0,               0,      -1.3534065,      -1.0667721,     -0.36138581 ],
 ],
 [
  [               0,               0,               0,               0,     0.013303332,   -0.0022363552 ],
  [               0,               0,               0,               0,      -2.3863188,       6.2923798 ],
  [               0,               0,               0,               0,       7.9910409,       -7.676505 ],
  [               0,               0,               0,               0,      -3.0180002,     -0.20157288 ],
  [               0,               0,               0,               0,     0.020856837,    0.0051233072 ],
  [               0,               0,               0,               0,         1.77603,      -1.3722873 ],
  [               0,               0,               0,               0,       2.7009352,      -2.2368538 ],
 ],
 [
  [               0,               0,               0,   0.00032153002,   -0.0014312627,    0.0092395742 ],
  [               0,               0,               0,      0.48226721,      -1.3077778,       5.0221024 ],
  [               0,               0,               0,     -0.13297321,    -0.014305578,      -1.2011216 ],
  [               0,               0,               0,      0.53990835,     -0.55182192,     0.037413673 ],
  [               0,               0,               0,    0.0062718815,    -0.011026785,    0.0052732419 ],
  [               0,               0,               0,      0.14795916,     -0.28018664,       0.4099737 ],
  [               0,               0,               0,       5.8051088,      -16.373219,       8.8055733 ],
 ],
 [
  [               0,               0,       5.1255064,      -6.7974255,      -9.6064557,       12.746387 ],
  [               0,               0,      -4881.4709,       6770.7138,       9049.3832,       -12543.93 ],
  [               0,               0,      -3823.2204,       5377.7424,       6896.7984,      -9714.5275 ],
  [               0,               0,      -1564.3679,       2175.8743,       2918.9153,      -4059.1241 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      0.19303798,   -0.0022910488,     -0.37370777 ],
  [               0,               0,               0,       14.219697,       5.5408805,      -9.5395596 ],
  [               0,               0,               0,      -778.65925,       2181.0906,      -1530.8687 ],
  [               0,               0,               0,      -1.2172641,      -1.0222749,       4.2399974 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer2_etaoffv4_37_e = [
 [
  [       80.221772,      -162.08268,      -128.89126,       190.63989,        463.8509,      -478.17369 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,     -0.48926881,      0.37539415,       1.0823091,      0.78049451,      -2.3851899 ],
  [               0,       517.11783,      -391.19529,      -1097.1888,      -853.56619,       2456.0564 ],
  [               0,       61.201204,      -31.893215,      -146.82805,      -146.86435,        347.4065 ],
  [               0,      -94.402268,       70.890542,       203.30713,       145.65786,      -437.60015 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.62996737,      0.96797223 ],
  [               0,               0,               0,               0,       52.916864,       -77.51784 ],
  [               0,               0,               0,               0,       25.936793,      -40.344547 ],
  [               0,               0,               0,               0,      -24.838524,       37.602386 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0057356973,     0.018486569,   -0.0095840023 ],
  [               0,               0,               0,       11.548744,      -40.911675,        41.11659 ],
  [               0,               0,               0,       -11.91578,       40.246058,      -35.156211 ],
  [               0,               0,               0,       6.2917691,      -21.151752,       17.713312 ],
  [               0,               0,               0,  -0.00091032505,    0.0041840649,   -0.0030759453 ],
  [               0,               0,               0,     -0.16260797,      0.69942869,      0.18686316 ],
  [               0,               0,               0,      -18.356432,        62.01535,       -53.57714 ],
 ],
 [
  [               0,               0,               0,    0.0022357263,   -0.0059931711,     0.011531528 ],
  [               0,               0,               0,      0.14864485,      -3.0827143,        9.127721 ],
  [               0,               0,               0,      0.39561825,      -1.4593587,     0.024248358 ],
  [               0,               0,               0,      -14.198211,       53.302891,      -47.427821 ],
  [               0,               0,               0,   -0.0049632522,   -0.0036082836,      0.05058651 ],
  [               0,               0,               0,     0.018559013,    -0.098218588,      0.14037753 ],
  [               0,               0,               0,       9.8942869,      -39.307517,       36.457567 ],
 ],
 [
  [               0,               0,               0,   -0.0019638433,   -0.0049097555,     0.032561149 ],
  [               0,               0,               0,        40.57759,      -191.21004,       228.08187 ],
  [               0,               0,               0,       4.9675672,      -23.318996,       26.058379 ],
  [               0,               0,               0,      -8.1003331,       45.799116,      -57.523903 ],
  [               0,               0,               0,  -0.00019011746,    0.0054802234,     0.042428667 ],
  [               0,               0,               0,      -2.9688472,       13.794107,      -15.966613 ],
  [               0,               0,               0,       11.867049,      -54.738983,       60.016857 ],
 ],
 [
  [               0,               0,               0,               0,    0.0039266001,    0.0057653458 ],
  [               0,               0,               0,               0,      0.30431276,       1.8691958 ],
  [               0,               0,               0,               0,       34.819709,      -86.994031 ],
  [               0,               0,               0,               0,      -29.608191,       72.952216 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.017464999,     -0.02450002,    -0.035682447,    -0.026839166 ],
  [               0,               0,      -1.1382115,      -1.3858335,      -3.5174756,      -4.9297234 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,       1.0698042,       1.3310645,      0.11148877,      -8.7503316 ],
  [               0,               0,      0.13884296,       0.3657476,      0.98967513,       2.2171817 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]


ele_layer2_etaoffv4_37_b = [
    ele5_layer2_etaoffv4_37_b,
    ele10_layer2_etaoffv4_37_b,
    ele25_layer2_etaoffv4_37_b,
    ele50_layer2_etaoffv4_37_b,
    ele75_layer2_etaoffv4_37_b,
    ele100_layer2_etaoffv4_37_b,
    ele200_layer2_etaoffv4_37_b,
    ele500_layer2_etaoffv4_37_b,
    ele1000_layer2_etaoffv4_37_b,
    ]

ele_layer2_etaoffv4_37_e = [
    ele5_layer2_etaoffv4_37_e,
    ele10_layer2_etaoffv4_37_e,
    ele25_layer2_etaoffv4_37_e,
    ele50_layer2_etaoffv4_37_e,
    ele75_layer2_etaoffv4_37_e,
    ele100_layer2_etaoffv4_37_e,
    ele200_layer2_etaoffv4_37_e,
    ele500_layer2_etaoffv4_37_e,
    ele1000_layer2_etaoffv4_37_e,
    ]


###############################################################################
# Tables for v4 sshape correction, layer 2, 3x7 photon clusters
#

gam5_layer2_etaoffv4_37_b = [
 [
  [               0,               0,               0,     -0.16202486,     0.077328508,    0.0045370428 ],
  [               0,               0,               0,       24.667318,      -11.335703,       3.0705685 ],
  [               0,               0,               0,      -3.0943663,       1.1590291,      -1.1791143 ],
  [               0,               0,               0,     -0.41027452,      0.24787922,    -0.015135562 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      -0.1010056,      0.12309076,    -0.026780541 ],
  [               0,               0,               0,       15.948113,      -21.272934,        9.049787 ],
  [               0,               0,               0,      -2.5152731,       3.2773634,      -2.2200531 ],
  [               0,               0,               0,       1.2341873,      -1.2109847,      0.33560943 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.048042695,     -0.02785337 ],
  [               0,               0,               0,               0,       8.4211147,       -4.785101 ],
  [               0,               0,               0,               0,      -3.3062045,       1.4853127 ],
  [               0,               0,               0,               0,      -2.3305239,       1.8952268 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam5_layer2_etaoffv4_37_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer2_etaoffv4_37_b = [
 [
  [               0,               0,               0,     0.019361112,   -0.0027623702,    0.0061747217 ],
  [               0,               0,               0,      -26.389301,       10.339759,       3.0512984 ],
  [               0,               0,               0,       5.0607245,      -2.1054247,      -1.0605774 ],
  [               0,               0,               0,      0.93501634,     -0.39110508,      -0.0593013 ],
  [               0,               0,               0,   0.00044823506,   -0.0010228751,    0.0011227802 ],
  [               0,               0,               0,       21.973373,      -9.9905547,       1.4801854 ],
  [               0,               0,               0,      -2.4220777,        3.012747,      -1.2674834 ],
 ],
 [
  [               0,               0,               0,     0.014354906,     0.003274438,   -0.0013479753 ],
  [               0,               0,               0,       3.1085056,      -11.953794,       9.8280685 ],
  [               0,               0,               0,      -13.262544,       17.562032,      -7.0293715 ],
  [               0,               0,               0,       8.5107737,      -10.713698,       3.3821649 ],
  [               0,               0,               0,   -0.0016799015,   -0.0045317554,    0.0041920797 ],
  [               0,               0,               0,      -7.5047712,        6.987433,     -0.59067665 ],
  [               0,               0,               0,       23.720877,      -19.952437,       2.2930553 ],
 ],
 [
  [               0,               0,               0,               0,       0.4394836,     -0.33888042 ],
  [               0,               0,               0,               0,      -39.074522,       33.536239 ],
  [               0,               0,               0,               0,        16.77042,      -14.681487 ],
  [               0,               0,               0,               0,      -4.0133449,       3.2346935 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0098908708,    -0.016896702,     0.015733401 ],
  [               0,               0,               0,      0.28556376,      -2.6425756,       4.9901086 ],
  [               0,               0,               0,      0.10843463,     0.033883985,      -1.2963418 ],
  [               0,               0,               0,     -0.36014259,       1.0821514,     -0.70062591 ],
  [               0,               0,               0,    0.0013505895,   -0.0027961534,    0.0020643976 ],
  [               0,               0,               0,      -1.5965531,       4.9314249,      -2.4306195 ],
  [               0,               0,               0,     -0.79147825,      0.82826855,     -0.28518582 ],
 ],
 [
  [               0,               0,     0.006717923,    0.0037478126,   -0.0037355864,    -0.018688255 ],
  [               0,               0,      0.29436508,       3.8105378,       8.3349669,       19.761076 ],
  [               0,               0,      -69.678665,      -32.298029,       41.416427,       172.47918 ],
  [               0,               0,      -16.116971,      -6.3660983,        12.78293,       46.627161 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.027564981,     0.001927001,     -0.05061632 ],
  [               0,               0,               0,       -268.8287,       2.6048607,       551.06944 ],
  [               0,               0,               0,       37.015834,      -68.807735,       22.419175 ],
  [               0,               0,               0,     -0.13190514,     -0.42356724,     -0.81956617 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer2_etaoffv4_37_e = [
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.013708599 ],
  [               0,               0,               0,               0,               0,       2.0431716 ],
  [               0,               0,               0,               0,               0,      -1.2394216 ],
  [               0,               0,               0,               0,               0,      0.10995805 ],
  [               0,               0,               0,               0,               0,    0.0013987605 ],
  [               0,               0,               0,               0,               0,      0.67503646 ],
  [               0,               0,               0,               0,               0,      -9.7661667 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer2_etaoffv4_37_b = [
 [
  [               0,               0,               0,   0.00079192214,    0.0017354514,    0.0070185571 ],
  [               0,               0,               0,       5.6906123,      -3.1251259,       4.4365713 ],
  [               0,               0,               0,      0.97333899,     -0.46337253,      -1.2859669 ],
  [               0,               0,               0,      -2.5685628,       1.1328513,     -0.13262238 ],
  [               0,               0,               0,      0.03272161,    -0.015048106,    0.0022167131 ],
  [               0,               0,               0,      -14.389557,       5.9391963,    -0.061225638 ],
  [               0,               0,               0,       5.5280028,      -1.4636802,      -1.2362959 ],
 ],
 [
  [               0,               0,               0,    0.0035702095,    0.0029164478,     0.002561709 ],
  [               0,               0,               0,      -2.8736261,     -0.88547772,        6.163015 ],
  [               0,               0,               0,       2.4525712,      -3.3006122,     -0.51285737 ],
  [               0,               0,               0,       0.5171445,    0.0064829889,    -0.025470551 ],
  [               0,               0,               0,     0.006716054,   -0.0095456866,    0.0043580368 ],
  [               0,               0,               0,      -3.4210699,       3.2039707,      0.20455618 ],
  [               0,               0,               0,      -4.3863423,          2.3492,      -8.1505435 ],
 ],
 [
  [               0,               0,               0,               0,     0.067712233,    -0.050624554 ],
  [               0,               0,               0,               0,      -17.485291,       19.838494 ],
  [               0,               0,               0,               0,      -5.3615625,       2.7067184 ],
  [               0,               0,               0,               0,       9.6878966,      -7.6636954 ],
  [               0,               0,               0,               0,     0.020342718,    -0.014342508 ],
  [               0,               0,               0,               0,      -21.497092,       18.147244 ],
  [               0,               0,               0,               0,       -35.40576,       26.680269 ],
 ],
 [
  [               0,               0,               0,    -0.037520166,     0.090071046,    -0.045845628 ],
  [               0,               0,               0,       18.167946,      -43.763796,       29.360202 ],
  [               0,               0,               0,      -2.3073029,       5.4635858,      -4.6379089 ],
  [               0,               0,               0,      0.70770538,      -1.4584484,      0.92136933 ],
  [               0,               0,               0,     0.019360176,    -0.042599637,     0.023898203 ],
  [               0,               0,               0,      -1.1364496,      0.78034313,       1.0677119 ],
  [               0,               0,               0,       7.6259871,      -16.486858,       6.7939095 ],
 ],
 [
  [               0,               0,     0.071541153,     0.032367204,    -0.043334655,     -0.17579662 ],
  [               0,               0,      -169.27551,       181.15079,        259.4416,      -258.09947 ],
  [               0,               0,      -8.0161774,       24.036445,        23.68763,      -58.485718 ],
  [               0,               0,       8.8237515,      -8.0998895,      -13.650343,       11.646141 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.082012777,     -0.10762695 ],
  [               0,               0,               0,               0,       10.646809,        24.68272 ],
  [               0,               0,               0,               0,      -127.18853,       176.87457 ],
  [               0,               0,               0,               0,     -0.78514133,       0.9894845 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer2_etaoffv4_37_e = [
 [
  [               0,               0,       -1.901094,       2.7532153,       3.7897907,      -5.4868772 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    -0.091612846,      0.14607384,      0.21675244,     -0.34533505 ],
  [               0,               0,      -8581.0246,       13431.278,       15709.333,      -24857.506 ],
  [               0,               0,      -1472.9716,       2160.1393,       3372.0243,      -4944.2313 ],
  [               0,               0,       1869.7781,      -2751.7504,      -4079.0612,       6001.6998 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.45419712,     -0.68117628 ],
  [               0,               0,               0,               0,       378.42392,      -568.48398 ],
  [               0,               0,               0,               0,       54.845523,      -84.395764 ],
  [               0,               0,               0,               0,      -45.922024,       69.335228 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,  -0.00082612074,   -0.0031504212,     0.015490474 ],
  [               0,               0,               0,        10.99513,      -32.050076,       26.317462 ],
  [               0,               0,               0,       1.8928598,      -6.3759031,       4.0166493 ],
  [               0,               0,               0,     -0.12256176,      0.19480206,     0.029541466 ],
  [               0,               0,               0,    0.0069466897,    -0.023671589,     0.020628448 ],
  [               0,               0,               0,      -13.832969,       52.198464,      -47.056462 ],
  [               0,               0,               0,       16.166827,      -60.761331,       14.535896 ],
 ],
 [
  [               0,               0,               0,     0.019086048,    -0.070605419,     0.072198521 ],
  [               0,               0,               0,      -6.1409933,       22.127742,      -16.440443 ],
  [               0,               0,               0,     -0.80030131,       3.7090769,      -5.5491909 ],
  [               0,               0,               0,      -40.802072,       186.97769,      -199.23587 ],
  [               0,               0,               0,       0.0554391,     0.019377062,     -0.17581715 ],
  [               0,               0,               0,      0.45090296,      -1.8747897,       1.9783085 ],
  [               0,               0,               0,      0.28647875,      -1.4409708,      -1.3071951 ],
 ],
 [
  [               0,               0,               0,     0.032599985,     -0.10182279,     0.070755712 ],
  [               0,               0,               0,     -0.73366846,      -2.2027653,       11.537141 ],
  [               0,               0,               0,        13.77009,      -65.027738,       75.618116 ],
  [               0,               0,               0,       -11.61002,       56.103783,      -67.742953 ],
  [               0,               0,               0,      0.09037102,     -0.43064376,      0.51321916 ],
  [               0,               0,               0,      -65.072229,       317.90681,      -387.68354 ],
  [               0,               0,               0,       282.21593,      -1376.8773,       1676.4714 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.027331681 ],
  [               0,               0,               0,               0,               0,       1.2143313 ],
  [               0,               0,               0,               0,               0,     -0.88417258 ],
  [               0,               0,               0,               0,               0,         0.01372 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   0.00097275989,   0.00072147266,   -0.0019684532,    -0.014374165 ],
  [               0,               0,      -1.0734349,      -1.4746888,      -3.6894493,      -5.2313952 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,       502.73388,       418.86105,      -1003.6281,       -7623.916 ],
  [               0,               0,      0.91224243,       4.0508867,       4.7513365,       14.438212 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer2_etaoffv4_37_b = [
 [
  [               0,               0,               0,    -0.081996288,      0.03749189,    0.0035165324 ],
  [               0,               0,               0,       33.990948,      -16.414272,       6.3071337 ],
  [               0,               0,               0,      -3.0978719,       1.4410219,      -1.5034476 ],
  [               0,               0,               0,       1.4117351,     -0.58492019,     0.044830059 ],
  [               0,               0,               0,     0.023288159,    -0.010976976,    0.0017440249 ],
  [               0,               0,               0,       20.465764,      -8.9601795,       1.2434361 ],
  [               0,               0,               0,      -7.8742256,        3.468407,      -1.8261498 ],
 ],
 [
  [               0,               0,               0,    -0.031738044,     0.038267945,    -0.004095827 ],
  [               0,               0,               0,       10.049739,      -14.696344,       9.5126756 ],
  [               0,               0,               0,      -3.6407091,       4.1943649,      -2.5928499 ],
  [               0,               0,               0,       2.4863097,      -2.9571758,      0.96580263 ],
  [               0,               0,               0,    0.0034462053,   -0.0056312971,    0.0029999272 ],
  [               0,               0,               0,       11.105085,       -11.93355,       3.4539361 ],
  [               0,               0,               0,      -2.8573717,       4.4015686,       -3.699648 ],
 ],
 [
  [               0,               0,               0,               0,     0.035468485,      -0.0204234 ],
  [               0,               0,               0,               0,       6.4240429,      -1.1501533 ],
  [               0,               0,               0,               0,       2.7172395,      -3.6129458 ],
  [               0,               0,               0,               0,      -6.9771988,       7.3058765 ],
  [               0,               0,               0,               0,     0.011419226,    0.0046901317 ],
  [               0,               0,               0,               0,        3.887138,      -2.9928478 ],
  [               0,               0,               0,               0,      -11.395699,     -0.15777898 ],
 ],
 [
  [               0,               0,               0,     0.019748229,    -0.043466401,     0.029867196 ],
  [               0,               0,               0,      -5.9742887,       12.195604,      -1.9997012 ],
  [               0,               0,               0,     -0.24171956,       0.3087397,       -1.525891 ],
  [               0,               0,               0,      0.48815526,     -0.71274635,      0.34221097 ],
  [               0,               0,               0,   -0.0033706434,    0.0073981022,   -0.0032150937 ],
  [               0,               0,               0,       -1.457795,       2.9800774,     -0.73029092 ],
  [               0,               0,               0,      -9.4398869,       20.172712,       -12.37691 ],
 ],
 [
  [               0,               0,      0.14484356,   -0.0048928946,     -0.13827801,     -0.16141077 ],
  [               0,               0,      -4769.6495,        6735.996,       8464.8612,      -11970.264 ],
  [               0,               0,      -428.33166,       608.02582,       809.24359,      -1150.3088 ],
  [               0,               0,       505.93232,      -692.34737,      -950.61227,       1301.0215 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.025623972,    0.0058081936,    -0.036006232 ],
  [               0,               0,               0,      -4.8581064,      0.62852024,        15.47783 ],
  [               0,               0,               0,       3.7924466,       3.7158598,      -13.940075 ],
  [               0,               0,               0,   -0.0023618649,    0.0046251856,     -0.32294873 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer2_etaoffv4_37_e = [
 [
  [               0,               0,      -1.6456164,       2.3636506,       3.2296688,      -4.6371715 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0035199422,  -0.00012980764,     0.001418961,    -0.012142331 ],
  [               0,               0,       2.5992327,       3.9460936,       5.9874615,       13.082868 ],
  [               0,               0,      -1.6342895,       2.0301034,       2.2780705,      -5.2910224 ],
  [               0,               0,      -4.2873912,       1.0011902,       -2.013994,       15.285398 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.49971711,      0.77455378 ],
  [               0,               0,               0,               0,       48.263108,      -71.918521 ],
  [               0,               0,               0,               0,      -33.541645,       49.934737 ],
  [               0,               0,               0,               0,       15.906712,       -24.05202 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.020282984,    -0.055008715,     0.042209081 ],
  [               0,               0,               0,       4.5530432,      -20.351081,       24.999819 ],
  [               0,               0,               0,      0.76355494,      -1.8676983,     -0.40558655 ],
  [               0,               0,               0,      -7.5580554,       25.731233,      -21.628538 ],
  [               0,               0,               0,     -0.02005395,     0.070975592,    -0.060730132 ],
  [               0,               0,               0,      0.03522182,      -1.6700776,        3.077023 ],
  [               0,               0,               0,       35.302571,      -120.22563,       93.269274 ],
 ],
 [
  [               0,               0,               0,     0.037129143,     -0.15053146,       0.1616634 ],
  [               0,               0,               0,      -1.9408594,       5.7819444,     -0.87448915 ],
  [               0,               0,               0,      -2.6612079,       11.501415,      -13.659307 ],
  [               0,               0,               0,      -21.413134,       92.597171,      -93.917893 ],
  [               0,               0,               0,    0.0022599789,     0.048668792,    -0.055223781 ],
  [               0,               0,               0,       1.3856613,      -5.8296784,       6.1600548 ],
  [               0,               0,               0,      -3.1004552,       13.069907,       -16.78569 ],
 ],
 [
  [               0,               0,               0,    0.0029622491,   0.00057200241,   -0.0046080578 ],
  [               0,               0,               0,       8.2562593,      -41.914276,       55.118903 ],
  [               0,               0,               0,       9.5628244,      -45.028072,       51.856884 ],
  [               0,               0,               0,      -7.4896424,       35.718461,      -42.572871 ],
  [               0,               0,               0,    0.0021472102,   0.00013855333,    -0.012571005 ],
  [               0,               0,               0,       0.2043006,      0.31386089,      -1.5280344 ],
  [               0,               0,               0,       6.1076815,       8.3132183,       -56.96864 ],
 ],
 [
  [               0,               0,               0,               0,   0.00013351312,     0.019393321 ],
  [               0,               0,               0,               0,       -117.2371,       290.65612 ],
  [               0,               0,               0,               0,       54.213683,      -134.35556 ],
  [               0,               0,               0,               0,      -19.319516,       47.594847 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0021961717,    0.0017616154,   -0.0045543556,    -0.032934723 ],
  [               0,               0,      -1.2006028,      -1.3735785,      -3.4083097,      -4.4571703 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,       16.328129,       14.765841,      -27.608902,      -229.84744 ],
  [               0,               0,      0.46016207,       1.1267474,       2.8066206,       6.8170794 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer2_etaoffv4_37_b = [
 [
  [               0,               0,               0,    -0.024457071,     0.011675807,    0.0061499824 ],
  [               0,               0,               0,       9.1906547,      -5.0479802,       5.2344284 ],
  [               0,               0,               0,       -2.000225,       1.0163952,      -1.4809113 ],
  [               0,               0,               0,       2.3766552,      -1.2223403,      0.15807528 ],
  [               0,               0,               0,    0.0066928212,   -0.0041990486,    0.0012916746 ],
  [               0,               0,               0,       9.3502211,      -3.6729203,       0.5865558 ],
  [               0,               0,               0,      -15.001619,       7.7657468,      -2.6716936 ],
 ],
 [
  [               0,               0,               0,     0.025639127,    -0.029042232,     0.014219863 ],
  [               0,               0,               0,      -14.697325,       15.587306,      0.91345825 ],
  [               0,               0,               0,       1.6265243,      -2.3912889,     -0.62287889 ],
  [               0,               0,               0,       0.1750684,      0.30218906,     -0.17962777 ],
  [               0,               0,               0,   -0.0062573543,    0.0074030393,   -0.0012727539 ],
  [               0,               0,               0,      -6.1121765,       7.0422801,      -1.1422227 ],
  [               0,               0,               0,      0.19900381,       -2.671793,     -0.24788486 ],
 ],
 [
  [               0,               0,               0,               0,       0.1642695,     -0.12405662 ],
  [               0,               0,               0,               0,      -40.819026,       37.008399 ],
  [               0,               0,               0,               0,       19.986346,      -17.568864 ],
  [               0,               0,               0,               0,      -19.237879,       16.739456 ],
  [               0,               0,               0,               0,      0.10598549,    -0.075289534 ],
  [               0,               0,               0,               0,      -10.392441,       8.4759873 ],
  [               0,               0,               0,               0,      -11.251151,       5.9706592 ],
 ],
 [
  [               0,               0,               0,    -0.001555651,      0.00480647,    0.0048794896 ],
  [               0,               0,               0,       2.9094976,      -7.6209137,       8.4007933 ],
  [               0,               0,               0,     -0.71905683,       1.4245582,      -2.0964046 ],
  [               0,               0,               0,       2.8009657,       -1.338587,     -0.42266656 ],
  [               0,               0,               0,     0.021631913,   -0.0081000922,   -0.0052754724 ],
  [               0,               0,               0,       1.1107346,      -2.5639566,       1.5368702 ],
  [               0,               0,               0,      -1.5103228,       2.7569154,      -4.0997224 ],
 ],
 [
  [               0,               0,      0.17528191,     0.014580227,     -0.15236806,     -0.25147192 ],
  [               0,               0,      -150.89759,       223.02026,       268.61431,      -395.48554 ],
  [               0,               0,      -2.4553497,       19.473921,     -0.36914844,       -31.32101 ],
  [               0,               0,       471.84956,      -644.95407,      -888.30806,       1214.2885 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.017389104,    -0.012043707 ],
  [               0,               0,               0,               0,      -1147.3798,       1653.8103 ],
  [               0,               0,               0,               0,      -8.4907274,       10.984497 ],
  [               0,               0,               0,               0,      -0.1960523,     -0.55152646 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer2_etaoffv4_37_e = [
 [
  [               0,               0,     -0.93584095,       1.3794933,       1.7859447,      -2.6368461 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.069683085,    -0.099320295,     -0.14691706,      0.21390493 ],
  [               0,               0,      -1326.3684,       1516.6493,       4235.0855,      -5286.7929 ],
  [               0,               0,       1081.1105,      -1602.4688,      -2364.7046,       3503.1393 ],
  [               0,               0,      -620.50556,        916.8005,         1351.53,      -1996.4434 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0035875212,    0.0086549051 ],
  [               0,               0,               0,               0,       58.405603,       -86.50021 ],
  [               0,               0,               0,               0,      -8.1743733,       11.254955 ],
  [               0,               0,               0,               0,      -3.5176426,       5.3471635 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0061130645,     0.022516242,     -0.01240661 ],
  [               0,               0,               0,       11.985755,      -41.481193,       39.154628 ],
  [               0,               0,               0,       3.2556833,       -10.95537,       7.8149735 ],
  [               0,               0,               0,    -0.027994345,       2.8118789,      -2.8106778 ],
  [               0,               0,               0,    0.0065482192,    0.0025954055,   -0.0080023679 ],
  [               0,               0,               0,      -2.5775196,       8.6515374,      -7.1203369 ],
  [               0,               0,               0,       3.2140904,      -10.988359,       6.3683171 ],
 ],
 [
  [               0,               0,               0,     0.014998515,    -0.057323142,      0.06332592 ],
  [               0,               0,               0,     -0.54728678,     -0.40556211,       6.1455517 ],
  [               0,               0,               0,       1.1583826,        -4.58799,       3.2657674 ],
  [               0,               0,               0,      -16.321925,       66.960335,      -66.132966 ],
  [               0,               0,               0,    -0.031048189,      -0.0021056,       0.1986337 ],
  [               0,               0,               0,     0.025782194,    -0.093478514,     0.087207491 ],
  [               0,               0,               0,     -0.34265708,      0.47623623,      -1.4626276 ],
 ],
 [
  [               0,               0,               0,    0.0049889666,     -0.03122693,     0.055993691 ],
  [               0,               0,               0,       55.606368,      -263.84422,       315.53309 ],
  [               0,               0,               0,       11.084984,      -52.878513,         61.8456 ],
  [               0,               0,               0,       23.115644,      -26.031007,      -56.154803 ],
  [               0,               0,               0,      0.11313218,     0.017676697,     -0.53681636 ],
  [               0,               0,               0,     -0.54793132,       2.6140611,      -3.1141615 ],
  [               0,               0,               0,       7.8555121,      -40.999579,       50.455699 ],
 ],
 [
  [               0,               0,               0,               0,      -1.0415681,       2.5867117 ],
  [               0,               0,               0,               0,         19.4213,      -46.134761 ],
  [               0,               0,               0,               0,      -3.5177259,       7.6762829 ],
  [               0,               0,               0,               0,     -0.37419648,       0.9468089 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0016264834,    -0.001042329,    -0.001396165,    -0.014889294 ],
  [               0,               0,      -1.1654633,      -1.4035217,      -3.4783953,       -4.620579 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,       52.411551,        51.53737,      -82.083852,       -812.4357 ],
  [               0,               0,       1.7174555,       1.8324445,       8.8450853,       13.539836 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer2_etaoffv4_37_b = [
 [
  [               0,               0,               0,    -0.057163897,     0.031161303,    0.0034808255 ],
  [               0,               0,               0,       33.221023,      -18.259262,       7.0395687 ],
  [               0,               0,               0,     -0.79007276,        0.397849,      -1.4074308 ],
  [               0,               0,               0,     -0.46542766,      0.25863373,    -0.038822852 ],
  [               0,               0,               0,     0.050122609,    -0.016674274,    0.0019712192 ],
  [               0,               0,               0,       8.3233725,      -6.0841444,       1.1844017 ],
  [               0,               0,               0,    -0.066040238,     0.074621469,      -1.5722362 ],
 ],
 [
  [               0,               0,               0,    0.0031306233,  -0.00036971805,    0.0081476864 ],
  [               0,               0,               0,      -2.6013575,      0.87841956,        4.318223 ],
  [               0,               0,               0,     -0.20004102,       0.2153354,      -1.3884181 ],
  [               0,               0,               0,      0.48779971,     -0.42606034,     0.092714175 ],
  [               0,               0,               0,   0.00030827789,  -0.00018809187,  -2.2650526e-05 ],
  [               0,               0,               0,      -317.48954,      -8.6474528,       1.1004343 ],
  [               0,               0,               0,       7.6242347,      -35.074905,      0.47065975 ],
 ],
 [
  [               0,               0,               0,               0,      0.15030184,     -0.11300723 ],
  [               0,               0,               0,               0,       -36.47854,       33.793047 ],
  [               0,               0,               0,               0,       14.114434,      -12.763057 ],
  [               0,               0,               0,               0,      -15.957384,       12.941373 ],
  [               0,               0,               0,               0,    -0.073640126,     0.060134748 ],
  [               0,               0,               0,               0,       -18.75345,       15.252459 ],
  [               0,               0,               0,               0,      -3.6215288,      0.46907985 ],
 ],
 [
  [               0,               0,               0,      0.01019368,    -0.017859131,      0.01427843 ],
  [               0,               0,               0,      -1.7190876,       1.2850244,       4.7690066 ],
  [               0,               0,               0,       3.0152392,      -6.9556873,       2.4842004 ],
  [               0,               0,               0,     -0.94495064,       2.7429472,      -1.6990855 ],
  [               0,               0,               0,    0.0056573595,    -0.010948731,     0.006012597 ],
  [               0,               0,               0,      -1.3135426,       1.7310927,      0.30718903 ],
  [               0,               0,               0,       5.0454076,      -14.466315,       7.5703828 ],
 ],
 [
  [               0,               0,               0,      0.43147047,     -0.27098216,     -0.41989028 ],
  [               0,               0,               0,       769.46107,      -2103.9262,       1441.1494 ],
  [               0,               0,               0,      -794.01928,       2215.4101,      -1546.2847 ],
  [               0,               0,               0,       1580.4467,      -4331.4793,       2967.8153 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.19924596,       0.3024931 ],
  [               0,               0,               0,               0,      -1126.7542,       1624.2716 ],
  [               0,               0,               0,               0,       35.584584,      -51.166743 ],
  [               0,               0,               0,               0,      -5.7179919,        7.568405 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer2_etaoffv4_37_e = [
 [
  [               0,               0,      -2.1670038,       3.1292736,       4.2900761,      -6.1945533 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      -40.576627,        119.5924,      -88.097407 ],
  [               0,               0,               0,       698.90212,      -2028.7678,       1473.2821 ],
  [               0,               0,               0,       719.72141,      -2121.7671,       1562.6945 ],
  [               0,               0,               0,      -579.88531,       1709.5119,      -1259.7928 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.011751649,     0.051384629 ],
  [               0,               0,               0,               0,      0.44636516,      0.99534485 ],
  [               0,               0,               0,               0,       46.101927,      -70.769404 ],
  [               0,               0,               0,               0,      -44.924419,       67.960859 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,  -0.00039546556,   -0.0004300376,    0.0098465252 ],
  [               0,               0,               0,       6.9610514,      -23.160463,       22.633789 ],
  [               0,               0,               0,       4.0843282,      -13.948421,       10.506112 ],
  [               0,               0,               0,      -2.1311437,       6.8989866,      -5.4270072 ],
  [               0,               0,               0,   -0.0024707602,    0.0059163534,   -0.0017966901 ],
  [               0,               0,               0,      -6.1138541,       22.074262,      -19.232318 ],
  [               0,               0,               0,       22.246248,       -72.38595,       56.223514 ],
 ],
 [
  [               0,               0,               0,   -0.0020024572,    0.0084018459,   0.00043774373 ],
  [               0,               0,               0,      0.36352986,      -3.0058815,       7.6016971 ],
  [               0,               0,               0,     -0.33069851,       1.4536606,      -2.8958243 ],
  [               0,               0,               0,       2.6147653,      -3.9332943,       2.7096039 ],
  [               0,               0,               0,      0.01377701,    0.0076142683,    -0.021624231 ],
  [               0,               0,               0,      0.03408387,     -0.10678511,      0.12791904 ],
  [               0,               0,               0,     -0.41826206,       1.7057874,      -4.8036658 ],
 ],
 [
  [               0,               0,               0,  -0.00023634614,    -0.068779506,      0.17557886 ],
  [               0,               0,               0,       2.0360247,      -3.1831024,       -1.354722 ],
  [               0,               0,               0,       30.906183,      -147.68611,       175.11039 ],
  [               0,               0,               0,      -18.291087,       87.221879,      -103.86302 ],
  [               0,               0,               0,     0.008314368,    -0.033041097,     0.032463123 ],
  [               0,               0,               0,      -18.218342,       88.145903,      -105.88087 ],
  [               0,               0,               0,       123.38867,      -572.56437,       660.88938 ],
 ],
 [
  [               0,               0,               0,               0,    -0.030846487,     0.078236234 ],
  [               0,               0,               0,               0,      -242.92629,       603.36034 ],
  [               0,               0,               0,               0,       -268.0068,        658.5924 ],
  [               0,               0,               0,               0,       306.63214,      -754.76688 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,    0.0023717463,    0.0017946666,   -0.0048351077,    -0.035099449 ],
  [               0,               0,      -1.1810841,      -1.3981517,       -3.464365,      -4.5840468 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,       123.41571,        104.6939,      -242.10842,      -1850.9347 ],
  [               0,               0,      0.74415431,       1.8623229,       4.5364661,       12.378554 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer2_etaoffv4_37_b = [
 [
  [               0,               0,               0,    -0.022822543,     0.012260541,     0.006141554 ],
  [               0,               0,               0,       12.548188,       -6.502613,       5.5029638 ],
  [               0,               0,               0,      -1.6008266,       0.7802808,      -1.4237939 ],
  [               0,               0,               0,     -0.17161412,     -0.32317426,     0.020714622 ],
  [               0,               0,               0,      0.01780381,   -0.0048607456,   0.00082427911 ],
  [               0,               0,               0,       8.1202775,      -5.5542625,       1.0679428 ],
  [               0,               0,               0,       -14.26545,       8.8620459,       -2.263999 ],
 ],
 [
  [               0,               0,               0,    0.0045165095,   -0.0055048333,    0.0091932858 ],
  [               0,               0,               0,      -7.6552671,       8.5409938,       2.4228928 ],
  [               0,               0,               0,     -0.33606634,      0.23132633,      -1.3908933 ],
  [               0,               0,               0,      0.05446368,      0.29370445,      0.29479174 ],
  [               0,               0,               0,    0.0033899308,  -0.00032600385,   -0.0059144196 ],
  [               0,               0,               0,      0.63760494,     -0.58389677,      0.17020606 ],
  [               0,               0,               0,      -2.4352879,        1.448061,      -24.405632 ],
 ],
 [
  [               0,               0,               0,               0,    0.0061458555,   0.00056332626 ],
  [               0,               0,               0,               0,      -2.2491288,       7.1153415 ],
  [               0,               0,               0,               0,       25.607541,      -21.958888 ],
  [               0,               0,               0,               0,      -13.343095,       10.709657 ],
  [               0,               0,               0,               0,  -0.00015582425,    0.0014090674 ],
  [               0,               0,               0,               0,       2.1936352,     -0.75521571 ],
  [               0,               0,               0,               0,       31.701539,      -27.100967 ],
 ],
 [
  [               0,               0,               0,   -0.0010284176,    0.0033585712,    0.0051209149 ],
  [               0,               0,               0,      0.41582849,      -1.9049124,       5.6961583 ],
  [               0,               0,               0,     -0.78224212,        1.534608,      -2.1606222 ],
  [               0,               0,               0,      0.42643337,     -0.61739755,      0.26583135 ],
  [               0,               0,               0,    0.0020018307,   -0.0037453861,    0.0023344964 ],
  [               0,               0,               0,        3.290455,      -7.9818979,        5.279486 ],
  [               0,               0,               0,      -2.7537417,       5.0278599,      -4.2742755 ],
 ],
 [
  [               0,               0,       0.1652879,     0.011336232,     -0.14547177,     -0.23095986 ],
  [               0,               0,       303.78579,      -408.87688,       -568.1693,       768.29161 ],
  [               0,               0,      -541.63122,       762.22067,       1010.1001,      -1423.3385 ],
  [               0,               0,       612.37483,      -837.22543,      -1152.2778,       1575.4395 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       0.2413547,     -0.33209758 ],
  [               0,               0,               0,               0,      -1073.0866,       1549.9817 ],
  [               0,               0,               0,               0,      -28.225977,       37.915412 ],
  [               0,               0,               0,               0,     -0.10061522,     -0.47307134 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer2_etaoffv4_37_e = [
 [
  [               0,               0,      -3.0031653,       4.3488095,       5.9714148,      -8.6471092 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.061639041,     0.024936476,     -0.16427963 ],
  [               0,               0,               0,       45.456935,       -243.8749,        264.6674 ],
  [               0,               0,               0,       143.54309,       -404.8516,        283.3682 ],
  [               0,               0,               0,      -2.5292202,     -0.60116245,        6.604652 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0059680204,   -0.0013923022 ],
  [               0,               0,               0,               0,       83.804441,      -123.46851 ],
  [               0,               0,               0,               0,       2.0180848,      -4.2643573 ],
  [               0,               0,               0,               0,      -2.9274964,       4.4726072 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,   -0.0044613269,    0.0040765578,     0.019328187 ],
  [               0,               0,               0,       2.8203767,      -8.8999758,       10.058423 ],
  [               0,               0,               0,      -0.7500122,       2.7478097,      -3.7014247 ],
  [               0,               0,               0,      0.26439872,     -0.98400891,      0.90257541 ],
  [               0,               0,               0,  -0.00079369315,  -0.00018326941,    0.0038408285 ],
  [               0,               0,               0,      -1.7154845,       6.0154446,      -4.0701789 ],
  [               0,               0,               0,      -2.0198627,       5.3909736,      -8.2403707 ],
 ],
 [
  [               0,               0,               0,   0.00061698937,    0.0033098628,  -0.00097788977 ],
  [               0,               0,               0,      -2.1563598,       6.4305751,     -0.81083405 ],
  [               0,               0,               0,      0.75052302,      -2.5492431,      0.72445787 ],
  [               0,               0,               0,      -3.4203044,       13.979496,      -14.136638 ],
  [               0,               0,               0,    -0.011754709,     0.049932165,    -0.051841459 ],
  [               0,               0,               0,       10.334577,      -44.715745,        48.60509 ],
  [               0,               0,               0,       25.561466,       -104.5921,       104.56618 ],
 ],
 [
  [               0,               0,               0,   -0.0060259597,     0.014315584,     0.010161277 ],
  [               0,               0,               0,       14.237242,      -65.643377,       78.318237 ],
  [               0,               0,               0,       10.660015,      -51.094787,       59.979255 ],
  [               0,               0,               0,       27.456224,      -10.895369,      -105.40363 ],
  [               0,               0,               0,       0.1180365,     0.032388481,     -0.49174444 ],
  [               0,               0,               0,      -0.7956826,         3.71015,      -4.3262429 ],
  [               0,               0,               0,      -3.1759462,       19.314426,      -25.118938 ],
 ],
 [
  [               0,               0,               0,               0,    0.0017455526,    0.0086583843 ],
  [               0,               0,               0,               0,      -46.428871,       116.67849 ],
  [               0,               0,               0,               0,       5.9075233,      -15.681879 ],
  [               0,               0,               0,               0,      0.78657955,      -1.8878511 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,   0.00095766576,   0.00023913846,    0.0035447893,    -0.024814672 ],
  [               0,               0,      -1.1704672,       -1.399757,      -3.4733497,       -4.619008 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,        714.8533,       597.58485,      -1431.5312,      -10814.897 ],
  [               0,               0,       1.2849115,        5.001732,       14.285764,        38.57571 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer2_etaoffv4_37_b = [
 [
  [               0,               0,               0,   -0.0077811127,    0.0057977569,    0.0065624951 ],
  [               0,               0,               0,    -0.077798059,      -1.2448332,       5.4136146 ],
  [               0,               0,               0,      -1.4249396,      0.79859839,      -1.4344379 ],
  [               0,               0,               0,     -0.69209421,      -0.2614997,     0.015441071 ],
  [               0,               0,               0,     0.017415999,   -0.0045417648,   0.00068052697 ],
  [               0,               0,               0,        3.848315,      -3.9761141,       1.1238341 ],
  [               0,               0,               0,       -17.85059,       11.449177,      -2.2915038 ],
 ],
 [
  [               0,               0,               0,     -0.01583672,     0.020862983,    0.0009774291 ],
  [               0,               0,               0,       7.6930213,       -11.13309,       8.7241597 ],
  [               0,               0,               0,      0.31864984,     -0.55483326,      -1.1241808 ],
  [               0,               0,               0,      -1.0001012,       1.0790761,     -0.34531974 ],
  [               0,               0,               0,     0.013952826,    -0.013465213,      0.00368235 ],
  [               0,               0,               0,       5.0993346,      -7.6481847,       3.0012007 ],
  [               0,               0,               0,       1.4484075,      -1.7324914,     -0.46056454 ],
 ],
 [
  [               0,               0,               0,               0,   0.00014167844,    0.0047695963 ],
  [               0,               0,               0,               0,       5.5263782,       1.6031543 ],
  [               0,               0,               0,               0,       20.234036,      -17.660982 ],
  [               0,               0,               0,               0,      -12.365323,       9.9092079 ],
  [               0,               0,               0,               0,  -0.00011707744,     0.001604484 ],
  [               0,               0,               0,               0,       1.1181884,      0.12265607 ],
  [               0,               0,               0,               0,       14.242908,      -13.136811 ],
 ],
 [
  [               0,               0,               0,     0.010522579,    -0.019523919,     0.015374209 ],
  [               0,               0,               0,      -4.4974637,       7.8334051,       1.5020435 ],
  [               0,               0,               0,       1.1384524,      -2.6354195,     0.089772966 ],
  [               0,               0,               0,     -0.53133329,        1.385758,     -0.82184861 ],
  [               0,               0,               0,    0.0020701556,    -0.003633952,   0.00075720107 ],
  [               0,               0,               0,      -1.2682393,       2.1659904,    -0.002673787 ],
  [               0,               0,               0,       1.4647784,      -4.1562801,      -2.1203608 ],
 ],
 [
  [               0,               0,       0.1925289,    -0.030493249,     -0.20062588,     -0.14716007 ],
  [               0,               0,       413.29525,       -543.7007,      -760.30619,       1003.5688 ],
  [               0,               0,      -346.70844,       494.87833,       639.94913,      -915.75001 ],
  [               0,               0,       618.23727,       -842.9678,      -1163.6795,       1586.7582 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.28957235,     -0.39814289 ],
  [               0,               0,               0,               0,      -906.78431,        1316.259 ],
  [               0,               0,               0,               0,      -20.689359,        27.14265 ],
  [               0,               0,               0,               0,     -0.13019588,     -0.10636708 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer2_etaoffv4_37_e = [
 [
  [       82.322423,      -165.94123,      -132.62461,       194.75368,       476.18155,      -489.72852 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.041428695,     0.026988145,    -0.022910414,     -0.14441133 ],
  [               0,               0,       839.31254,      -1241.5405,      -1805.0277,       2671.9393 ],
  [               0,               0,       436.58427,      -641.12892,        -963.327,       1413.3826 ],
  [               0,               0,      -466.06931,       687.06133,       1011.5963,      -1490.9579 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.27753806,      0.44204462 ],
  [               0,               0,               0,               0,       14.329934,      -19.284236 ],
  [               0,               0,               0,               0,       45.321605,      -69.716677 ],
  [               0,               0,               0,               0,      -45.044555,       68.154406 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,     0.050175199,     -0.16970197,      0.15057302 ],
  [               0,               0,               0,      -15.709379,         52.4078,      -39.404097 ],
  [               0,               0,               0,       1.1689968,      -4.2517357,        2.446786 ],
  [               0,               0,               0,       2.3498399,      -7.6954632,       6.3872476 ],
  [               0,               0,               0,      0.03012543,     -0.10358878,       0.0898876 ],
  [               0,               0,               0,      -10.386689,       36.243603,       -31.08854 ],
  [               0,               0,               0,      -12.661723,       40.873291,       -34.87513 ],
 ],
 [
  [               0,               0,               0,    8.813867e-05,  -0.00062619804,    0.0096458481 ],
  [               0,               0,               0,       0.3050481,      -2.8041524,       7.8405312 ],
  [               0,               0,               0,    -0.077687388,       0.6209427,      -2.2944395 ],
  [               0,               0,               0,      -1.8448691,       5.1802825,      -1.9832461 ],
  [               0,               0,               0,   -0.0048661635,     0.001454232,     0.025040155 ],
  [               0,               0,               0,      0.63725442,      -2.6988447,        2.963112 ],
  [               0,               0,               0,       5.2994865,      -20.210698,       16.296531 ],
 ],
 [
  [               0,               0,               0,    0.0013584227,   -0.0023791074,    0.0079058835 ],
  [               0,               0,               0,       13.139491,      -63.589771,       79.738855 ],
  [               0,               0,               0,    0.0031390674,     -0.32884426,     -0.50523178 ],
  [               0,               0,               0,      -8.7669028,       53.987622,      -76.945339 ],
  [               0,               0,               0,     0.023923539,    0.0002332823,     -0.11689873 ],
  [               0,               0,               0,       5.6293741,      -26.375013,       30.929286 ],
  [               0,               0,               0,      -1.3461272,       2.4708816,      -1.0954897 ],
 ],
 [
  [               0,               0,               0,               0,    0.0020735606,     0.021254308 ],
  [               0,               0,               0,               0,      0.23801921,       1.4611897 ],
  [               0,               0,               0,               0,       32.393047,      -80.896943 ],
  [               0,               0,               0,               0,      -31.325821,       77.161877 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.019512307,     -0.02677718,    -0.039865445,     -0.03359903 ],
  [               0,               0,       -1.087987,      -1.2678994,      -3.6397916,      -6.2537412 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,       1.4708632,       1.7680459,     -0.17803032,      -12.889503 ],
  [               0,               0,      0.15211723,      0.40000409,       1.0840281,       2.4625929 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer2_etaoffv4_37_b = [
 [
  [               0,               0,               0,    -0.013451144,     0.011244586,    0.0059079633 ],
  [               0,               0,               0,       7.9237156,       -6.612458,       6.1297514 ],
  [               0,               0,               0,       0.8526713,     -0.39976996,      -1.3041406 ],
  [               0,               0,               0,      0.38797978,     0.030471112,    -0.053047226 ],
  [               0,               0,               0,    0.0091431968,   -0.0046963073,   0.00081019758 ],
  [               0,               0,               0,       1.1759698,      -3.9301461,       1.1892595 ],
  [               0,               0,               0,      -40.095224,       11.087473,      -1.7768684 ],
 ],
 [
  [               0,               0,               0,     0.012222146,     -0.01660191,     0.012207274 ],
  [               0,               0,               0,      -8.6185068,       10.549668,       2.2785732 ],
  [               0,               0,               0,      -1.4384988,       1.5893756,       -1.741393 ],
  [               0,               0,               0,      0.81055447,     -0.84850646,      0.16251861 ],
  [               0,               0,               0,   -0.0051180668,    0.0068842729,   -0.0016686271 ],
  [               0,               0,               0,      -7.3225922,       9.3210151,      -2.0319002 ],
  [               0,               0,               0,        5.976622,      -8.3959558,       1.5374877 ],
 ],
 [
  [               0,               0,               0,               0,    -0.057205383,     0.053379402 ],
  [               0,               0,               0,               0,       13.535699,      -6.0393479 ],
  [               0,               0,               0,               0,       12.173149,      -10.875383 ],
  [               0,               0,               0,               0,      -7.5196839,       5.8636515 ],
  [               0,               0,               0,               0,     0.025414962,    -0.019480009 ],
  [               0,               0,               0,               0,        15.83869,      -11.911424 ],
  [               0,               0,               0,               0,      -25.871184,       20.243086 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0029095616,    0.0097402364 ],
  [               0,               0,               0,               0,      0.71770134,        4.071839 ],
  [               0,               0,               0,               0,     -0.16726173,      -1.2110171 ],
  [               0,               0,               0,               0,      0.13866089,     -0.12820706 ],
  [               0,               0,               0,               0,    0.0011014829,  -0.00045055494 ],
  [               0,               0,               0,               0,      0.14257211,      0.73893549 ],
  [               0,               0,               0,               0,      0.18494676,      -14.450525 ],
 ],
 [
  [               0,               0,      0.15855234,    -0.030496651,     -0.16904634,     -0.10683953 ],
  [               0,               0,       66.460821,      -55.726772,      -102.24559,       79.307979 ],
  [               0,               0,       -954.9686,        1325.625,        1791.043,      -2488.1345 ],
  [               0,               0,       458.37986,      -621.33209,      -858.92282,       1164.4181 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.10631837,     -0.13205586 ],
  [               0,               0,               0,               0,      -782.75305,       1142.6214 ],
  [               0,               0,               0,               0,      -4.8999933,        5.580953 ],
  [               0,               0,               0,               0,       9.0210961,      -13.078172 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer2_etaoffv4_37_e = [
 [
  [       89.484423,      -180.54853,      -143.88304,       212.03921,       516.93428,      -532.17434 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,     -0.60023856,      0.47616217,       1.3496037,      0.95605028,      -3.0196152 ],
  [               0,       290.14959,      -234.63733,      -590.95523,      -437.70957,       1312.8777 ],
  [               0,       190.66754,      -131.80429,      -424.96085,      -339.03474,       943.73191 ],
  [               0,      -197.42342,       145.83157,       428.53448,       314.75689,      -929.79091 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    0.0039609071,   -0.0026509995 ],
  [               0,               0,               0,               0,      -488.79968,       745.36986 ],
  [               0,               0,               0,               0,      -108.21451,       162.62073 ],
  [               0,               0,               0,               0,       150.42507,      -227.33263 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    0.0029075447,   -0.0056318031,    0.0091345242 ],
  [               0,               0,               0,       2.5304353,      -10.848665,       15.254595 ],
  [               0,               0,               0,     -0.36051816,       1.5463002,      -3.0046653 ],
  [               0,               0,               0,     0.073574793,      -2.4020714,       4.4525875 ],
  [               0,               0,               0,   -0.0048800445,      0.00117229,     0.016530183 ],
  [               0,               0,               0,       1.8917129,       -6.315215,       5.4414793 ],
  [               0,               0,               0,       3.9904148,      -13.154822,       7.9617951 ],
 ],
 [
  [               0,               0,               0,   -0.0023401666,    0.0094520222,  -0.00050333672 ],
  [               0,               0,               0,       1.0702429,      -5.9063028,       10.974367 ],
  [               0,               0,               0,      0.43332805,      -1.6914399,      0.33259465 ],
  [               0,               0,               0,      -6.1033526,       24.135005,      -23.456139 ],
  [               0,               0,               0,   0.00020701872,   -0.0037281324,     0.011754853 ],
  [               0,               0,               0,      0.22172997,     -0.85139121,      0.86008302 ],
  [               0,               0,               0,       12.409815,      -49.306147,        46.72938 ],
 ],
 [
  [               0,               0,               0,    0.0018757448,   0.00024192042,   -0.0010019806 ],
  [               0,               0,               0,       6.1362545,      -32.333588,       45.047712 ],
  [               0,               0,               0,       4.9387653,      -23.393075,       26.430041 ],
  [               0,               0,               0,      -6.5999295,       31.057898,      -30.948696 ],
  [               0,               0,               0,    0.0023772262,    0.0065947812,     0.029127753 ],
  [               0,               0,               0,       2.6220047,      -12.258931,        14.33868 ],
  [               0,               0,               0,      -9.3621712,       42.148723,      -50.179368 ],
 ],
 [
  [               0,               0,               0,               0,      0.19882508,     -0.48159577 ],
  [               0,               0,               0,               0,      -87.843271,       219.67471 ],
  [               0,               0,               0,               0,       7.8022635,      -20.645096 ],
  [               0,               0,               0,               0,       11.796962,      -28.954201 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,     0.015633927,    -0.023137942,    -0.029105356,    -0.023603011 ],
  [               0,               0,      0.11812956,      -1.2920847,      -3.4294786,      -4.5590929 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,       1.0482263,       1.3576062,      0.38342844,      -7.5575932 ],
  [               0,               0,      0.15365084,      0.38671391,       1.0310304,        2.311514 ],
  [               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0 ],
 ],
]


gam_layer2_etaoffv4_37_b = [
    gam5_layer2_etaoffv4_37_b,
    gam10_layer2_etaoffv4_37_b,
    gam25_layer2_etaoffv4_37_b,
    gam50_layer2_etaoffv4_37_b,
    gam75_layer2_etaoffv4_37_b,
    gam100_layer2_etaoffv4_37_b,
    gam200_layer2_etaoffv4_37_b,
    gam500_layer2_etaoffv4_37_b,
    gam1000_layer2_etaoffv4_37_b,
    ]

gam_layer2_etaoffv4_37_e = [
    gam5_layer2_etaoffv4_37_e,
    gam10_layer2_etaoffv4_37_e,
    gam25_layer2_etaoffv4_37_e,
    gam50_layer2_etaoffv4_37_e,
    gam75_layer2_etaoffv4_37_e,
    gam100_layer2_etaoffv4_37_e,
    gam200_layer2_etaoffv4_37_e,
    gam500_layer2_etaoffv4_37_e,
    gam1000_layer2_etaoffv4_37_e,
    ]


###############################################################################
# Region tables.
#

layer1_etaoffv4_regions_b = [
  [ 0.0, 0.00625, 5, 0.003125, 0 ],
  [ 0.00625, 0.4, 11, 0.003125, 0 ],
  [ 0.4, 0.79375, 11, 0.003125, 0 ],
  [ 0.79365, 0.8, 3, 0.003125, 0 ],
  [ 0.8, 1.39375, 11, 0.003125, 0 ],
  [ 1.39375, 1.3955, 3, 0.003125, 0 ],
  [ 1.3955, 1.4, 0, 0.025, 0 ],
  [ 1.4, 1.45, 0, 0.025, 0 ],
]
layer1_etaoffv4_regions_e = [
  [ 1.425, 1.454, 3, 0.025, 0 ],
  [ 1.454, 1.475, 0, 0.025, 0 ],
  [ 1.475, 1.5, 3, 0.025, 0 ],
  [ 1.5, 1.503125, 0, 0.003125, 0 ],
  [ 1.503125, 1.8, 0, 0.003125, 0 ],
  [ 1.8, 2.0, 0, 0.00416666666667, 0 ],
  [ 2.0, 2.29375, 0, 0.00625, 0 ],
  [ 2.29375, 2.30625, 3, 0.00625, 0 ],
  [ 2.30625, 2.39375, 0, 0.00625, 0 ],
  [ 2.39375, 2.4, 0, 0.00625, 0 ],
  [ 2.4, 2.475, 10, 0.025, 0 ],
  [ 2.475, 2.5, 0, 0.025, 0 ],
]

layer2_etaoffv4_regions_b = [
  [ 0.0, 0.4, 10, 0.025, 0 ],
  [ 0.4, 0.775, 10, 0.025, 0 ],
  [ 0.775, 0.825, 10, 0.025, 0 ],
  [ 0.825, 1.35, 10, 0.025, 0 ],
  [ 1.35, 1.4, 3, 0.025, 0 ],
  [ 1.4, 1.437, 3, 0.075, -0.025 ],
]
layer2_etaoffv4_regions_e = [
  [ 1.4, 1.45, 5, 0.025, 0 ],
  [ 1.45, 1.5, 3, 0.025, 0 ],
  [ 1.5, 1.525, 3, 0.025, 0 ],
  [ 1.525, 1.9, 10, 0.025, 0 ],
  [ 1.9, 2.3, 10, 0.025, 0 ],
  [ 2.3, 2.45, 10, 0.025, 0 ],
  [ 2.45, 2.475, 3, 0.025, 0 ],
  [ 2.475, 2.5, 0, 0.025, 0 ],
]


ele_etaoffv4_energies = [
    5*GeV, 10*GeV, 25*GeV, 50*GeV, 75*GeV, 100*GeV, 200*GeV, 500*GeV, 1000*GeV
    ]
gam_etaoffv4_energies = [
    5*GeV, 10*GeV, 25*GeV, 50*GeV, 75*GeV, 100*GeV, 200*GeV, 500*GeV, 1000*GeV
    ]
etaoffv4_energies = {'ele55' : ele_etaoffv4_energies,
                     'ele35' : ele_etaoffv4_energies,
                     'ele37' : ele_etaoffv4_energies,
                     'gam55' : gam_etaoffv4_energies,
                     'gam35' : gam_etaoffv4_energies,
                     'gam37' : gam_etaoffv4_energies,
                     
                     # Use 5x5 for other cluster sizes.
                     'ele33' : ele_etaoffv4_energies,
                     'ele57' : ele_etaoffv4_energies,
                     'ele77' : ele_etaoffv4_energies,
                     'gam33' : gam_etaoffv4_energies,
                     'gam57' : gam_etaoffv4_energies,
                     'gam77' : gam_etaoffv4_energies,
                     }

class CaloSwEtaoff_v4_b1_parms:
    region = CALOCORR_EMB1
    regions = layer1_etaoffv4_regions_b
    energy_degree = 3
    forms = [[]]

    correction = {'ele55' : ele_layer1_etaoffv4_55_b,
                  'ele35' : ele_layer1_etaoffv4_35_b,
                  'ele37' : ele_layer1_etaoffv4_37_b,
                  'gam55' : gam_layer1_etaoffv4_55_b,
                  'gam35' : gam_layer1_etaoffv4_35_b,
                  'gam37' : gam_layer1_etaoffv4_37_b,

                  # Use 5x5 for other cluster sizes.
                  'ele33' : ele_layer1_etaoffv4_55_b,
                  'ele57' : ele_layer1_etaoffv4_55_b,
                  'ele77' : ele_layer1_etaoffv4_55_b,
                  'gam33' : gam_layer1_etaoffv4_55_b,
                  'gam57' : gam_layer1_etaoffv4_55_b,
                  'gam77' : gam_layer1_etaoffv4_55_b,
                  }

    energies = etaoffv4_energies


class CaloSwEtaoff_v4_b2_parms:
    region = CALOCORR_EMB2
    regions = layer2_etaoffv4_regions_b
    energy_degree = 3
    forms = [[]]

    correction = {'ele55' : ele_layer2_etaoffv4_55_b,
                  'ele35' : ele_layer2_etaoffv4_35_b,
                  'ele37' : ele_layer2_etaoffv4_37_b,
                  'gam55' : gam_layer2_etaoffv4_55_b,
                  'gam35' : gam_layer2_etaoffv4_35_b,
                  'gam37' : gam_layer2_etaoffv4_37_b,

                  # Use 5x5 for other cluster sizes.
                  'ele33' : ele_layer2_etaoffv4_55_b,
                  'ele57' : ele_layer2_etaoffv4_55_b,
                  'ele77' : ele_layer2_etaoffv4_55_b,
                  'gam33' : gam_layer2_etaoffv4_55_b,
                  'gam57' : gam_layer2_etaoffv4_55_b,
                  'gam77' : gam_layer2_etaoffv4_55_b,
                  }

    energies = etaoffv4_energies


class CaloSwEtaoff_v4_e1_parms:
    region = CALOCORR_EME1
    regions = layer1_etaoffv4_regions_e
    energy_degree = 3
    forms = [[]]

    correction = {'ele55' : ele_layer1_etaoffv4_55_e,
                  'ele35' : ele_layer1_etaoffv4_35_e,
                  'ele37' : ele_layer1_etaoffv4_37_e,
                  'gam55' : gam_layer1_etaoffv4_55_e,
                  'gam35' : gam_layer1_etaoffv4_35_e,
                  'gam37' : gam_layer1_etaoffv4_37_e,

                  # Use 5x5 for other cluster sizes.
                  'ele33' : ele_layer1_etaoffv4_55_e,
                  'ele57' : ele_layer1_etaoffv4_55_e,
                  'ele77' : ele_layer1_etaoffv4_55_e,
                  'gam33' : gam_layer1_etaoffv4_55_e,
                  'gam57' : gam_layer1_etaoffv4_55_e,
                  'gam77' : gam_layer1_etaoffv4_55_e,
                  }

    energies = etaoffv4_energies


class CaloSwEtaoff_v4_e2_parms:
    region = CALOCORR_EME2
    regions = layer2_etaoffv4_regions_e
    energy_degree = 3
    forms = [[]]

    correction = {'ele55' : ele_layer2_etaoffv4_55_e,
                  'ele35' : ele_layer2_etaoffv4_35_e,
                  'ele37' : ele_layer2_etaoffv4_37_e,
                  'gam55' : gam_layer2_etaoffv4_55_e,
                  'gam35' : gam_layer2_etaoffv4_35_e,
                  'gam37' : gam_layer2_etaoffv4_37_e,

                  # Use 5x5 for other cluster sizes.
                  'ele33' : ele_layer2_etaoffv4_55_e,
                  'ele57' : ele_layer2_etaoffv4_55_e,
                  'ele77' : ele_layer2_etaoffv4_55_e,
                  'gam33' : gam_layer2_etaoffv4_55_e,
                  'gam57' : gam_layer2_etaoffv4_55_e,
                  'gam77' : gam_layer2_etaoffv4_55_e,
                  }

    energies = etaoffv4_energies




CaloSwEtaoff_v4_parms = { EMB1 : CaloSwEtaoff_v4_b1_parms,
                          EMB2 : CaloSwEtaoff_v4_b2_parms,
                          EME1 : CaloSwEtaoff_v4_e1_parms,
                          EME2 : CaloSwEtaoff_v4_e2_parms,
                          }



#
# Notes on the derivation of the above tables.
# The following are the descriptions of the fits used.
# The D constructor has the signature:
#
#     def __init__ (self, sampling, cellsize, lo, hi, degree, form,
#                   ysize = 0.001, prange = (None, None),
#                   psize = None, cond = None,
#                   outlier_sigma = 5,
#                   phase = 0,
#                   initvals = [],
#                   cmds = [],
#                   minevents = MINEVENTS_DEFAULT,
#                   fitter_type = 1,
#                   ignorepars = []):
#
#
# layer1_regions_v31 = [
#     D(EMB1, 0.025/8, 0.0,     0.00625, 3, 5, 0.004, minevents=75),        # 0
#     D(EMB1, 0.025/8, 0.00625, 0.4,     1, 11,                             # 1
#       prange=(0.1, 0.15), psize=0.0001,
#       ignorepars = {'ele5' : [(5,0),(6,0),(7,0)],
#                     'gam5' : [(5,0),(6,0),(7,0)]}),
#     D(EMB1, 0.025/8, 0.4,     0.79375, {'ele1000':1,                      # 2
#                                         'gam1000-35':1,
#                                         'gam1000-37':1,
#                                         'gam5' : 3,
#                                         None: 2},
#                                        11, prange=(0.6, 0.65), psize=0.0001,
#       minevents = {'ele5' : 1e9},
#       ignorepars = {'gam5' : [(5,0),(6,0),(7,0)],
#                     'ele10' : [(5,0),(6,0),(7,0)]}),
#     D(EMB1, 0.025/8, 0.79365, 0.8, {'gam25-37' : 2,                       # 3
#                                     None : 1}, 3),
#     D(EMB1, 0.025/8, 0.8,     1.39375, 2, 11,                             # 4
#       prange=(1.0, 1.05), psize=0.0001,
#       minevents = {'gam5' : 1e9}),
#     D(EMB1, 0.025/8, 1.39375, 1.3955,  0, 3, 0.002, psize=0.0005,         # 5
#       minevents=38),
#     D(EMB1, 0.025,   1.3955,  1.4,     3, 0, 0.01, minevents=32),         # 6
#     D(EMB1, 0.025,   1.4,     1.45,    {'gam1000-35' : 4,                 # 7
#                                         None: 3}, 0, 0.01),  
#     D(EME1, 0.025,   1.425,   1.454,   {'ele25-55' : 0,                   # 8
#                                         'gam25-35' : 0,
#                                         'ele75' : 0,
#                                         'gam75-55' : 0,
#                                         'gam1000-35' : 2,
#                                         'gam1000-37' : 2,
#                                         'ele1000-35' : 2,
#                                         'ele1000-37' : 2,
#                                         None : 1},
#                                        3, 0.015),
#     D(EME1, 0.025,   1.454,   1.475,   {'ele25' : 0,                      # 9
#                                         'gam25' : 0,
#                                         'ele50' : 0,
#                                         'gam50' : 0,
#                                         'ele75' : 0,
#                                         'gam75' : 0,
#                                         None :    1}, 0, 0.01),
#     D(EME1, 0.025,   1.475,   1.5,     {'ele25': 0,                       #10
#                                         None   : 1}, 3, 0.01), 
#     D(EME1, 0.025/8, 1.5,     1.503125,0, 0, 0.005,                       #11
#       minevents = {'ele25-55' : 38,
#                    'ele50-35' : 35,
#                    'ele25-37' : 51,
#                    'gam25-55' : 27,
#                    'gam50-35' : 41,
#                    'gam50-37' : 37,
#                    None: 1e9}),
#     D(EME1, 0.025/8, 1.503125, 1.8, {'ele10' : 0,                         #12
#                                      'gam10' : 0,
#                                      'gam25' : 2,
#                                      None:3}, 0, prange=(1.6, 1.65)),
#     D(EME1, 0.025/6, 1.8,     2.0,     1, 0, prange=(1.85, 1.9),          #13
#       minevents = {'gam10' : 1e9}),
#     D(EME1, 0.025/4, 2.0,     2.29375, 3, 0, prange=(2.1, 2.15)),         #14
#     D(EME1, 0.025/4, 2.29375, 2.30625, {'gam1000-37' : 2,                 #15
#                                         'gam200' : 0,
#                                         'gam500' : 0,
#                                         None: 1}, 3, 0.003),
#     D(EME1, 0.025/4, 2.30625, 2.39375, {'ele25' : 0,                      #16
#                                         None :    1}, 0),
#     D(EME1, 0.025/4, 2.39375, 2.4,     {'ele200' : 0,                     #17
#                                         'gam200' : 0,
#                                         'ele75'  : 0,
#                                         'gam75'  : 0,
#                                         'ele50'  : 0,
#                                         'gam50'  : 0,
#                                         'gam100' : 0,
#                                         None: 1},
#       0, 0.004,
#       minevents = {'ele100' : 93}),
#     D(EME1, 0.025,   2.4,     2.475,   {'gam100' : 0,                     #18
#                                         'ele200' : 1,
#                                         'gam200' : 1,
#                                         None:2},
#       10, 0.008,
#       ignorepars={'gam500' : [(4,0),(5,0),(6,0)]}),
#     D(EME1, 0.025,   2.475,   2.5,     {'gam50-37':2,                     #19
#                                         'gam100':3,
#                                         'ele50' : 2,
#                                         None:1},
#       0, 0.008, outlier_sigma=3),
#     ]


# layer2_regions_v31 = [
#     D(EMB2, 0.025,   0.0,   0.4,       {'gam75-55' : 3,                   # 0
#                                         'ele100'   : 3,
#                                         None       : 2},
#       10, psize=0.0003,
#       fitter_type = {'gam75-55' : 2,
#                      'ele100'   : 2},
#       ignorepars={'ele5' : [(4,0),(5,0),(6,0)],
#                   'gam5' : [(4,0),(5,0),(6,0)]}),
#     D(EMB2, 0.025,   0.4,   0.775,     {'ele50' : 1,                      # 1
#                                         None:2}, 10, psize=0.0003,
#       minevents={'ele5' : 1e9},
#       ignorepars={'gam5' : [(4,0),(5,0),(6,0)]}),
#     D(EMB2, 0.025,   0.775, 0.825,     {'ele10': 0,                       # 2
#                                         None: 1}, 10, 0.005, psize=0.001,
#       ignorepars={'gam5'  : [(4,0),(5,0),(6,0)],
#                   'gam10' : [(4,0),(5,0),(6,0)],
#                   'ele10' : [(4,0),(5,0),(6,0)],
#                   'ele25' : [(4,0),(5,0),(6,0)]}),
#     D(EMB2, 0.025,   0.825, 1.35,      {'ele200' : 1,                     # 3
#                                         'gam1000' : 1,
#                                         'ele50-37'   : 3,
#                                         None:2}, 10, 0.002, psize=0.0003,
#       ignorepars={'ele10' : [(4,0),(5,0),(6,0)]},
#       fitter_type = {'ele50-37' : 2},
#       minevents={'gam5' : 1e9}),
#     D(EMB2, 0.025,   1.35,  1.4,       {'ele25-55' : 2,                   # 4
#                                         'ele25-35' : 4,
#                                         'ele25-37' : 4,
#                                         'ele10' : 1,
#                                         'gam100' : 2,
#                                         'ele100' : 2,
#                                         'ele75'  : 2,
#                                         None: 3}, 3, 0.005, psize=0.0004), 
#     D(EMB2, 0.075,   1.4,   1.437,     {'ele10' : 1,                      # 5
#                                         'gam10' : 2,
#                                         'ele25' : 2,
#                                         'gam25' : 1,
#                                         'ele50' : 2,
#                                         'gam50' : 2,
#                                         'ele75' : 2,
#                                         'gam75' : 1,
#                                         'ele100' : 2,
#                                         'gam100' : 1,
#                                         'ele200' : 2,
#                                         'gam200' : 1,
#                                         'ele500' : 1,
#                                         'gam500' : 1,
#                                         'ele1000' : 2,
#                                         'gam1000' : 1,
#                                         None: 1},
#       3, 0.008, psize=0.005, phase=-0.025),
#     D(EME2, 0.025,   1.4,  1.45,      {'ele500':5,                        # 6
#                                        'gam500':5,
#                                        'ele1000':5,
#                                        'gam1000':5,
#                                        None:3},
#       5, 0.02, psize=0.02, outlier_sigma=2),
#     D(EME2, 0.025,   1.45, 1.5,       {'ele25':1,                         # 7
#                                        'gam100' : 2,
#                                        'ele200' : 2,
#                                        'gam200' : 2,
#                                        'ele1000': 4,
#                                        'gam1000': 4,
#                                        None: 3},
#       3, 0.01, psize=0.005),
#     D(EME2, 0.025,   1.5,    1.525,   1, 3, 0.005, psize=0.003),          # 8
#     D(EME2, 0.025,   1.525,  1.9,     {'gam10' : 0,                       # 9
#                                        None: 2}, 10, 0.005, psize=0.001,
#       minevents = {'ele10' : 1e9}),
#     D(EME2, 0.025,   1.9,  2.3,       {'ele100':1,                        #10
#                                        None:2}, 10, psize=0.0003),
#     D(EME2, 0.025,   2.3,  2.45,      2, 10, psize=0.0003,                #11
#       ignorepars = {'ele25' : [(4,0),(5,0),(6,0)],
#                     'ele50' : [(4,0),(5,0),(6,0)]}),
#     D(EME2, 0.025,   2.45, 2.475,     {'ele25': 0,                        #12
#                                        'gam25': 0,
#                                        None:1},  3, 0.005, psize=0.001,
#       outlier_sigma = {'ele25': 2,
#                        'gam25': 2,
#                        'ele50': 3,
#                        'gam50': 3,
#                        'ele75': 3,
#                        'gam75': 3,
#                        None: 5}),
#     D(EME2, 0.025,   2.475,2.5,       {'ele25-37' : 1,                    #13
#                                        'gam25-35' : 1,
#                                        'gam25-55' : 4,
#                                        'ele25-35' : 2,
#                                        'ele100-37' : 2,
#                                        None : 3}, 0, 0.005,
#       outlier_sigma = 3,
#       ignorepars=[(2,0)]),
#     ]
#
