# Copyright (C) 2002-2017 CERN for the benefit of the ATLAS collaboration

#
# $Id: CaloSwEtaoff_v5.py 579330 2014-01-21 12:59:55Z ehill $
#
# File: CaloClusterCorrection/python/CaloSwEtaoff_v5.py
# Created: November 19 2013, Ewan Hill
# Purpose: Eta offset (s-shape) corrections, v5.
#
# These corrections were derived using ?????? simulation and reconstruction,
# for both electrons and photons, 5x5, 3x5, 3x7 clusters.
#
# The difference from v4_1 is that this is was derived using mc11 with corrected
# endcap positions, and updated upstream material using data.  The outliers
# removal code has also been fixed.
#
# This correction was in CaloClusterCorrection-????????, in ??????.
#
# See CaloSwEtaoff_v3.h for more details.
#

from CaloClusterCorrection.common import *
from AthenaCommon.SystemOfUnits import GeV

###############################################################################
# Tables for v5 sshape correction, 5 GeV electron clusters
#

ele5_layer1_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,     0.072989638,     -0.17049012,      0.12753003,    -0.031186699 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.002453677,    0.0064463459 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.3428451,      0.64136447 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.8296257,     -0.50053791 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.2520104,       -0.384187 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.074186201,     0.011746851 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele5_layer1_etaoffv5_55_e = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele5_layer2_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,    0.0011589521 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       3.7151917 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -1.7117579 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      0.26931734 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,   0.00035313599 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       1.0729783 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -2.2763164 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele5_layer2_etaoffv5_55_e = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele5_layer1_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,     0.072128502,     -0.16456854,      0.12105163,    -0.029435699 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00016559771,   0.00073941062 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.2621124,       1.4095363 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.26408493,     -0.15626361 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.5615259,       0.7180044 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.68414823,      0.10451243 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele5_layer1_etaoffv5_35_e = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele5_layer2_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,    0.0078190315 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       2.3686109 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -1.0889308 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,    -0.095506311 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,   0.00075077582 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      0.48663133 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       -12.84529 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele5_layer2_etaoffv5_35_e = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele5_layer1_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,     0.066155616,     -0.15715806,        0.119526,    -0.029936009 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00095700762,    0.0013978613 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.1228115,       1.2847418 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.10227286,     -0.58476178 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.8273161,    -0.059420491 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.40556575,     0.060674991 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele5_layer1_etaoffv5_37_e = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele5_layer2_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,    0.0022226541 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       3.3230196 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,     -0.81143022 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,     -0.21776971 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,    0.0014621932 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      0.97893612 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -7.5582061 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele5_layer2_etaoffv5_37_e = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]


###############################################################################
# Tables for v5 sshape correction, 10 GeV electron clusters
#


ele10_layer1_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,   -0.0024741163,    0.0066641425,    -0.007169514,    0.0016995319 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00020445012,    0.0009264084 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.074685444,       2.0682551 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.39025051,     -0.61169644 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.45515605,      -0.2200778 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.26456731,      0.02269211 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.020230737,    0.0080422516 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -55.380576,       44.780953 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       42.059886,      -34.112169 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       11.211448,       -8.897884 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00069442328,   -0.0020678887,    0.0016772638 ],
  [               0,               0,               0,               0,               0,               0,               0,       8.1190522,       1.2218475,      -6.3152178 ],
  [               0,               0,               0,               0,               0,               0,               0,      -13.119773,      -4.0244283,       7.6090272 ],
  [               0,               0,               0,               0,               0,               0,               0,      -11.797068,      -6.9770862,        10.36826 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.7971964,       1.6470629,      -2.1831356 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.001361197,   0.00040877844,   0.00089878373 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.34230615,      0.48489269,      0.37996596 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.39395522,     -0.56448379,      -1.6005689 ],
 ],
 [
  [               0,               0,               0,               0,      0.41485244,      -1.1512523,       1.2080311,     -0.58038198,      0.11833037,    -0.011352446 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.067456473,       -0.184482,      0.18660205,    -0.098136241,     0.032049173,   -0.0029795803 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.3310908,      0.48193824 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -19.765645,       30.694383 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -43.057265,       62.625214 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -15.162429,       20.222097 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.1705035,     -0.45845419 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer1_etaoffv5_55_e = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.021437304,     0.040066778 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -5.3845808,       9.6664823 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        6.307003,      -10.607074 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        6.573371,      -11.252949 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.10024449,    -0.057301124 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   2.9101122e-05,   5.7829432e-05,   3.8224698e-05,   0.00031016135 ],
  [               0,               0,               0,               0,               0,               0,      0.04024113,      0.30835312,      0.56950734,       4.2668303 ],
  [               0,               0,               0,               0,               0,               0,      0.55538605,      0.44668477,     -0.62354942,      -4.8823425 ],
  [               0,               0,               0,               0,               0,               0,     -0.92823325,     -0.43472135,       1.2652013,       4.6173903 ],
  [               0,               0,               0,               0,               0,               0,       -0.334399,     -0.28170941,     0.048919254,      0.74558748 ],
  [               0,               0,               0,               0,               0,               0,  -5.2864323e-06,   6.6185226e-05,   0.00024907769,    0.0016684596 ],
  [               0,               0,               0,               0,               0,               0,     -0.16507329,    -0.019758073,      -0.1802477,       1.3945366 ],
  [               0,               0,               0,               0,               0,               0,     -0.45528635,     -0.20818858,      0.62453277,       2.7650833 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer2_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.030237774,  -0.00039176896 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.6789351,       3.3031953 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.1911137,     -0.75887169 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.99233828,     -0.29114467 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0072999844,    0.0023687586 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.83466517,       1.1019434 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.3518388,      -1.7067423 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0016194801,    0.0095759834 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.4752294,       2.7394542 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.32092232,      -1.3333161 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.22761486,    -0.029804406 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00025067591,   0.00041568924 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -5.3504814,       3.2115003 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       8.4909927,      -7.9066108 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0038890792,    0.0040261679 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       19.727572,      -13.723861 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.3042393,      0.62699804 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.020199385,      0.10241972 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0072638498,   -0.0043358143 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.88207332,       1.7329589 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -177.56676,       135.41061 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00014440639,  -0.00030068342,   0.00068390913,   0.00094337624 ],
  [               0,               0,               0,               0,               0,               0,       -2.175244,     -0.66292111,       1.7376843,       6.6083296 ],
  [               0,               0,               0,               0,               0,               0,      0.55555864,     -0.43002052,       0.4227442,      -1.1073279 ],
  [               0,               0,               0,               0,               0,               0,     0.068681648,       1.3596101,       0.4062398,      -1.3063688 ],
  [               0,               0,               0,               0,               0,               0,  -0.00028851141,   0.00046264828,  -0.00020343083,    0.0012428566 ],
  [               0,               0,               0,               0,               0,               0,      0.95288874,      -1.2041108,      -0.3859616,        1.566069 ],
  [               0,               0,               0,               0,               0,               0,      0.11506919,     0.093007905,     -0.12154792,     -0.99409901 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.021426707,     0.033262842 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -81.843021,       114.59331 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       8.6290026,      -13.493567 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.4675796,      -2.2838764 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.17334677,     -0.41325644,      0.34101233,       -0.108675,    0.0077331544,    0.0017280076 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.015204372,    0.0049407883 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.089197226,       4.4386937 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.053462531,      -1.4888645 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.12242142,     -0.16862791 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer2_etaoffv5_55_e = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00047715986,    0.0011581033 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -2.2478498e-05,         3.33233 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00018510431,      -2.3676611 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.001378454,       1.7726902 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0042491659,    0.0011896818,   -0.0072749768 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.47471835,      -2.6923002,       5.3268749 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.2853535,      -14.059534,       10.314271 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.31499097,      -1.6408746,         1.99232 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer1_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,   -0.0067245505,     0.015840171,    -0.013177212,    0.0026359945 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00019734623,   0.00085030043 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.15586767,       2.1042226 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.24761103,     -0.47217727 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.42599518,     -0.17320386 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.26751175,     0.015013511 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.017149751,    0.0082046479 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -42.425897,       34.486113 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       31.766723,      -25.933125 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       13.506041,      -10.722649 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00042082287,   -0.0014666005,    0.0014260553 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.7423223,       2.5775603,      -4.1695843 ],
  [               0,               0,               0,               0,               0,               0,               0,      -9.0586151,      -2.9441339,       8.0554608 ],
  [               0,               0,               0,               0,               0,               0,               0,      -9.6937201,      -1.8747022,       8.4432903 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.3592297,       1.2089487,      -2.1282792 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0019368217,   0.00036431126,  -0.00085051691 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.38257012,      0.23991685,      0.80472476 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.33761533,     -0.35357163,      -1.7772722 ],
 ],
 [
  [               0,               0,               0,               0,      0.34946633,     -0.95656366,      0.99034128,     -0.47279496,     0.096526371,   -0.0099080009 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.027023617,     0.040718511,   -0.0069447012,    -0.024384537,     0.020160231,   -0.0030521156 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00033801441,    0.0045819059 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.3480912,      -4.2986855 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -41.62928,       59.733074 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -21.830368,       29.759211 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.16044518,     -0.45087123 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer1_etaoffv5_35_e = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,  -0.00099756711,   0.00016280156,    0.0032859531 ],
  [               0,               0,               0,               0,               0,               0,               0,      -4.6369399,      0.50215357,       13.501768 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.9126874,     -0.31868868,      -11.307208 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.5409174,       1.9351049,      -15.210236 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.036465113,     -0.11981965,      0.67442495 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0012981616,  -0.00063374644,   -0.0031524356,   0.00024935818 ],
  [               0,               0,               0,               0,               0,               0,       -21.24187,       10.756687,       47.807817,       22.945318 ],
  [               0,               0,               0,               0,               0,               0,       11.722167,        3.636399,      -18.793485,      -64.050079 ],
  [               0,               0,               0,               0,               0,               0,      -2.7822715,        19.22681,       20.972776,      -95.420137 ],
  [               0,               0,               0,               0,               0,               0,      -9.6178405,      -7.4941753,        10.22923,       72.898926 ],
  [               0,               0,               0,               0,               0,               0,    0.0070602359,    0.0020736573,   -0.0096252529,    -0.028509564 ],
  [               0,               0,               0,               0,               0,               0,      -5.3303281,       10.605994,        19.42317,      -38.792375 ],
  [               0,               0,               0,               0,               0,               0,      -16.175936,       31.524235,         43.4076,      -83.854014 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer2_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0034108116,    0.0078223958 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.72049618,       3.0716061 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.76886984,      -1.1084154 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.58405689,     -0.13131687 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00029317622,   0.00076570415 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.85355313,      0.48101477 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -4.2498003,      -6.6821182 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0095252328,    0.0027742834 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.6647311,       5.0107721 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.25044397,      -1.4568847 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.3569333,       1.4927666 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.015224429,     0.014987472 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.056668422,     0.014573485 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.7304189,      -1.5926349 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0049058294,     0.011259745 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       23.757473,      -16.195435 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.0075388,      -5.3221053 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -8.8864374,       7.0831664 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.03977388,    -0.031369736 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -26.628066,       22.851721 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -115.95304,       92.983502 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00020995605,   -0.0013727279,   -0.0004218451,    0.0051358917 ],
  [               0,               0,               0,               0,               0,               0,       1.9535994,     -0.33063706,      -4.0946037,       5.9197352 ],
  [               0,               0,               0,               0,               0,               0,       2.9508206,      -4.7759798,   7.7817177e-05,      0.27055667 ],
  [               0,               0,               0,               0,               0,               0,     -0.24287786,       1.1277471,     0.074552766,     -0.60099141 ],
  [               0,               0,               0,               0,               0,               0,     0.001468333,   -0.0014336555,   -0.0019750426,    0.0033249485 ],
  [               0,               0,               0,               0,               0,               0,      0.19532619,     -0.25124073,     -0.02336537,       1.0728435 ],
  [               0,               0,               0,               0,               0,               0,      0.21134852,      0.19369185,     -0.63756164,        -1.22802 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.018735403,    -0.012209482 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.24341325,       1.9933883 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.0929391,       3.1064338 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.2493339,      -2.8762835 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.05711206,     0.054279059,      0.02493198,    -0.017186011,    0.0010531974,    0.0016067159 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.0303175,      0.04892623 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -417.69526,       598.85119 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -35.553715,       51.517907 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.1297786,      -1.6380577 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer2_etaoffv5_35_e = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0047457705,     0.013429886 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.24523024,       2.4537325 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -45.627149,       68.199696 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -7.7744136,       12.044916 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.003856769,   0.00080521819,   -0.0038139795 ],
  [               0,               0,               0,               0,               0,               0,               0,       7.0678318,      -26.280412,       26.792515 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.2164405,      -10.245854,       6.8845963 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.11228348,     -0.10479318,      0.59839244 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer1_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,     0.048424596,     -0.11228078,     0.084241888,    -0.021592229 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00012324164,   0.00046675662 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.24272283,       2.7370438 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.5795599,      0.19600442 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.56662065,      0.77431194 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.49097424,     0.028494115 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.017348263,    0.0061182135 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -67.508967,       54.530684 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       32.254988,        -26.3601 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       18.202993,      -14.459575 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0082352995,    -0.019643306,     0.011660657 ],
  [               0,               0,               0,               0,               0,               0,               0,       22.010773,      -20.965691,       3.8392738 ],
  [               0,               0,               0,               0,               0,               0,               0,         7.06995,      -14.042723,       7.1100861 ],
  [               0,               0,               0,               0,               0,               0,               0,       16.747725,      -36.966191,       19.366826 ],
  [               0,               0,               0,               0,               0,               0,               0,       -5.082842,       7.3046031,      -3.1941722 ],
  [               0,               0,               0,               0,               0,               0,               0,  -0.00036133907,     0.003066921,   -0.0019259886 ],
  [               0,               0,               0,               0,               0,               0,               0,      -9.5785952,       21.538324,      -11.257882 ],
  [               0,               0,               0,               0,               0,               0,               0,       11.048554,      -25.053819,       12.513679 ],
 ],
 [
  [               0,               0,               0,               0,    0.0038593006,     0.011711304,    -0.006514371,    -0.014704451,    0.0068155076,    -0.004383118 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.048663293,     -0.16863537,      0.20468918,     -0.11892683,      0.03778063,   -0.0040069109 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00032751323,    0.0041769925 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       6.8145288,      -6.1204319 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -53.385493,       76.826444 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -31.011813,       42.838999 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.14307305,     -0.45995273 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer1_etaoffv5_37_e = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0032439496,    0.0061561089 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -13.702693,       24.468767 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        17.30803,      -26.800954 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       16.739123,      -27.016751 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.58411261,     -0.34999924 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   3.7156739e-06,  -1.6917866e-05,   0.00015445907,   0.00032701225 ],
  [               0,               0,               0,               0,               0,               0,      0.14528845,      0.68331668,      0.48408572,       2.6559705 ],
  [               0,               0,               0,               0,               0,               0,    -0.033794527,        0.204937,     -0.26238031,     -0.19526267 ],
  [               0,               0,               0,               0,               0,               0,      0.16218642,     0.034893889,      0.16355439,      -0.9973015 ],
  [               0,               0,               0,               0,               0,               0,     0.039944869,     0.083680831,      0.22923634,     0.057903602 ],
  [               0,               0,               0,               0,               0,               0,  -0.00013642524,   -2.280138e-05,  -6.5092439e-05,   0.00089822198 ],
  [               0,               0,               0,               0,               0,               0,     -0.21512431,     -0.01673126,     -0.03734148,       1.8605027 ],
  [               0,               0,               0,               0,               0,               0,     0.036306844,    -0.060443307,    -0.018313202,     0.017966336 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer2_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.02472175,    0.0027435373 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -4.838579,       4.0665378 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.70909147,      -1.4448432 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.092149401,     0.052372591 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0054194082,     0.001762652 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.6802153,       1.1563866 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.42231115,       -8.041401 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0036301946,     0.010708051,    0.0037025209 ],
  [               0,               0,               0,               0,               0,               0,               0,      -5.0753045,       3.2002832,       2.6122397 ],
  [               0,               0,               0,               0,               0,               0,               0,         1.35284,      -1.4455773,     -0.91286972 ],
  [               0,               0,               0,               0,               0,               0,               0,       -0.836135,       1.5006271,     -0.53275919 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.04249277,    -0.060074923,     0.021437537 ],
  [               0,               0,               0,               0,               0,               0,               0,       8.0865333,      -7.2759632,       1.7019425 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.12882474,     -0.49419902,      -1.1830869 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0085528392,    0.0025987023 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       24.020022,      -16.833103 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.6362938,       -3.276695 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -7.1265252,       5.6389048 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.015650581,    -0.012003602 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -51.124475,       42.330928 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -15.444429,       12.118184 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0012461251,  -0.00042354708,    0.0011937075,    0.0044637771 ],
  [               0,               0,               0,               0,               0,               0,       5.4312569,      -4.6853926,      -7.2916435,       9.3777984 ],
  [               0,               0,               0,               0,               0,               0,       1.5460095,      -1.3584312,      -1.8163115,      0.17462148 ],
  [               0,               0,               0,               0,               0,               0,     -0.38773441,      0.75188367,      0.58638567,     -0.66403155 ],
  [               0,               0,               0,               0,               0,               0,   0.00024365171,  -0.00024456945,  -0.00029949546,    0.0015821088 ],
  [               0,               0,               0,               0,               0,               0,     -0.68792366,       0.9452432,      0.52023171,      0.22998222 ],
  [               0,               0,               0,               0,               0,               0,       0.2456696,      0.31928641,       -0.292294,       -1.740027 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.018012302,    -0.012267348 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -4.4530503,       7.7868973 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.7290383,       2.5905583 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.6638449,      -3.4144672 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.030481139,     0.025685487,     0.009985914,    0.0052798136,   -0.0036657591,    0.0018009177 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.021954206,     0.051129239 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.49065126,       1.7323348 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0054452861,     -0.59632843 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.019724273,     -0.22966422 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele10_layer2_etaoffv5_37_e = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0033208433,     0.015798018 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        28.29423,       -41.55588 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -22.872377,        33.72624 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.4420766,       2.2817906 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0036237369,    0.0012642434,    -0.003761846 ],
  [               0,               0,               0,               0,               0,               0,               0,        5.041916,      -18.969184,       20.170338 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.0209055,      -13.108067,       9.4309909 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.46313574,       1.1434366,     -0.51094196 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]

###############################################################################
# Tables for v5 sshape correction, 25 GeV electron clusters
#

ele25_layer1_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,    -0.002855613,    0.0041279974,   -0.0023973161 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00046088656,    0.0013325708 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.035341311,       1.8419445 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.021870518,     -0.75471143 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.40136901,     -0.63428313 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.15759935,    0.0027095946 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.027940036,    0.0028050891 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -65.68568,       52.840677 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       28.945766,      -23.567375 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       19.125253,      -15.208139 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0011096635,  -0.00067860605,    0.0033583249 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.7553608,      -5.3129036,       1.2310378 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.7326014,      0.23589612,       0.6749024 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.64961371,      -1.9404228,        1.778065 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.1818358,       2.4353652,      -1.2994274 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00013339137,  -9.7229128e-05,   0.00081110077 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.12262829,     -0.42478625,       1.0962573 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.52827981,      -1.0190788,      -1.2290142 ],
 ],
 [
  [               0,               0,               0,               0,     0.026480839,    -0.021990694,    -0.021807961,     0.035884334,    -0.022086619,   0.00015063836 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.07516665,      -0.1916325,      0.19030379,     -0.11084052,      0.04061378,   -0.0041571058 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0011694407,    0.0040107403 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.0892358,       6.8792555 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        20.57544,      -30.418529 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       19.086875,       -28.59778 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.056063253,     -0.31074648 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer1_etaoffv5_55_e = [
 [
  [               0,               0,               0,               0,    -0.027006632,     0.045942963,     0.014326744,    -0.052880681,     0.010306991,    0.0072252282 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.09753753,     0.097965947 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       36.438509,      -53.223919 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -36.101467,       52.745849 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -32.465333,       47.435283 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.1267988,     -0.18918238 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.62836984,     -0.91900622 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       15.276105,      -21.385494 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       13.362971,      -21.119097 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       17.906782,      -26.634261 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.5392092,       23.816865 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.12895152,      0.29512632 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.12871125,     -0.29459436 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.12872291,     -0.29461698 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -1.8436449e-05,   3.3882741e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00010780145,   6.6490282e-05,   8.6265291e-06,   0.00057619063 ],
  [               0,               0,               0,               0,               0,               0,       0.3670748,      -0.5946059,      -1.4976635,       4.6833809 ],
  [               0,               0,               0,               0,               0,               0,      0.14359936,    0.0082677706,     -0.33515484,      -1.2230754 ],
  [               0,               0,               0,               0,               0,               0,     -0.11356404,    -0.028441998,       0.1857893,     -0.36034578 ],
  [               0,               0,               0,               0,               0,               0,     -0.23198227,    -0.035712613,      0.31674686,      0.28740998 ],
  [               0,               0,               0,               0,               0,               0,   5.4126296e-05,   0.00011356649,   9.7922561e-05,   0.00070362881 ],
  [               0,               0,               0,               0,               0,               0,    -0.049786719,     0.024643656,     -0.26249095,      0.55337246 ],
  [               0,               0,               0,               0,               0,               0,     -0.35242388,     -0.19383108,      0.52276781,       2.6734592 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -2.9025852e-05,   -0.0007600871,  -0.00063033131,     0.007133527 ],
  [               0,               0,               0,               0,               0,               0,       1.6594476,      -3.1450554,      -7.5822922,       16.697552 ],
  [               0,               0,               0,               0,               0,               0,       2.1776227,      -4.3516635,      -9.5371498,       18.216486 ],
  [               0,               0,               0,               0,               0,               0,      0.49112484,     -0.55674086,      -2.0923214,       1.6908287 ],
  [               0,               0,               0,               0,               0,               0,     -0.39333457,      0.86447177,       2.0499725,      -4.3971451 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.054102829,      0.12780535 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -46.634328,       109.84207 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       99.474804,      -230.09106 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.5719253,       2.7415667 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       218.58133,       -502.4376 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.81011866,       1.8635758 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -84.052177,       193.27329 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       32.555522,      -74.448858 ],
 ],
 [
  [               0,               0,               0,               0,               0,   9.5958283e-06,   2.8895985e-05,   7.2257226e-05,   0.00015521429,   0.00077229445 ],
  [               0,               0,               0,               0,               0,    -0.035227318,    -0.044052593,    -0.038404937,    0.0018272552,        4.076552 ],
  [               0,               0,               0,               0,               0,    -0.016944277,    -0.014056474,    0.0084552268,     0.072002962,     -0.21600309 ],
  [               0,               0,               0,               0,               0,      0.07955488,     0.098633878,     0.020785419,     -0.45418082,      -3.2639851 ],
  [               0,               0,               0,               0,               0,    0.0023927015,    0.0063086552,      0.01459096,     0.029278294,     -0.25371071 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00062940809,   -0.0018954262 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    -0.012163718,   0.00086904277,     0.076338096 ],
  [               0,               0,               0,               0,               0,               0,               0,       6.8826706,      0.70962674,      -37.451653 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.4288618,       2.7679112,       6.0472645 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.2631637,       2.1171283,      -12.380042 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0008911387,   1.1761396e-05,   -0.0039650065 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.47812979,     -0.14970207,      -2.0128718 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.1142491,      0.33382281,       3.1806757 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,     0.015962019 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       3.5593899 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -37.365945 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,     -0.81749486 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      0.46632479 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer2_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,    0.0074004277 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       3.0438211 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -1.2891902 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,     0.023656122 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,   0.00037984545 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      0.89893628 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       -7.997074 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0028199708,    0.0079915637 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.0992523,       3.2219554 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.031707902,      -1.2487079 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.39250565,     -0.16856774 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0020903662,    0.0012950507 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.6911362,      -1.6732517 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -5.138445,       2.0314009 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0076010537,   0.00049502421 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       40.909551,      -29.986206 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.94354415,     -0.44856113 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -4.8091174,       3.7920266 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0089470971,   -0.0064405376 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -29.19358,       24.399979 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -21.073166,       15.774296 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00044727505,  -0.00027474555,    0.0009938837,     0.003993042 ],
  [               0,               0,               0,               0,               0,               0,        2.674085,      -3.1604874,      -3.8625021,       7.2809345 ],
  [               0,               0,               0,               0,               0,               0,      0.37928237,     -0.45402842,     -0.27689819,      -1.0375145 ],
  [               0,               0,               0,               0,               0,               0,     -0.38795062,      0.41713541,       1.2211626,     -0.98481118 ],
  [               0,               0,               0,               0,               0,               0,   8.9974802e-05,  -0.00015788105,  -0.00017128423,    0.0012531268 ],
  [               0,               0,               0,               0,               0,               0,     -0.53922843,      0.78292493,      0.32819248,      0.44520135 ],
  [               0,               0,               0,               0,               0,               0,      0.24600013,      0.11819915,     -0.31710567,      -1.4355687 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.13072483,      0.18880839 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -50.756443,         71.1325 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       10.864324,      -16.094569 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.0456401,      -1.0436279 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,  -0.00084076272,   0.00076052582,   -0.0051571857,     0.013651849,   -0.0095834057,    0.0015682297 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.061934302,    0.0020539008 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -128.95875,       192.22173 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       42.817597,      -61.448278 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.19355256,     -0.17814552 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.06580158,    -0.076761173 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       38.461055,      -50.083615 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -204.88548,       283.51308 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer2_etaoffv5_55_e = [
 [
  [     0.089646483,     -0.05339543,    -0.082025721,    -0.035489663,     0.035692454,     0.073772939,     0.029814029,    -0.078160126,     0.012906989,    0.0041194008 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   2.6369052e-05,   4.3599434e-05,   6.8986214e-05,   0.00062867807 ],
  [               0,               0,               0,               0,               0,               0,      -2.5458856,      0.50645758,       0.8341711,       5.0041341 ],
  [               0,               0,               0,               0,               0,               0,     0.025913628,     0.052743929,     0.022964793,     -0.45549655 ],
  [               0,               0,               0,               0,               0,               0,    -0.023358406,     -0.84278157,       0.1457845,     -0.50007226 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.064549265,      0.10694925 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       66.344974,      -98.753414 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -10.379788,       14.561885 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.3330192,       3.4330752 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00035105729,    0.0010231275,    0.0054826611 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.7992964,      -15.859528,       15.382274 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.1330166,       3.8364914,      -4.3748729 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.32677561,      -1.1022292,      0.93948098 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00016092542,   0.00019873618,  -0.00084088327 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.6592435,     0.081113899,      -4.3107336 ],
  [               0,               0,               0,               0,               0,               0,               0,      -6.7052745,      -1.6068606,       4.5764126 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00052057502,    0.0013917762,    0.0043573407 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.019407886,    0.0047834417,        2.087558 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0027220214,      0.00832933,      -1.1772006 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.012076607,      0.14069013,     0.064078446 ],
  [               0,               0,               0,               0,               0,               0,               0,  -0.00038268455,  -0.00027730192,    -0.000356548 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.13280205,     -0.24720214,     -0.28124844 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00036413182,   -0.0014164288,    0.0081048099 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    -0.001163365,   -0.0015918411,  -0.00028920186,    0.0092711909,     0.050050502 ],
  [               0,               0,               0,               0,               0,       3.0740692,      -3.4023756,      -16.956126,      -21.219471,       95.445299 ],
  [               0,               0,               0,               0,               0,     -0.23166522,       0.2124455,       1.2176293,       1.6391945,      -7.2605222 ],
  [               0,               0,               0,               0,               0,      -1.0408922,       1.2334392,       5.9092127,       7.0856548,       -33.53635 ],
  [               0,               0,               0,               0,               0,  -0.00016166518,  -0.00025358302,  -0.00016921136,   0.00092770751,    0.0070902066 ],
  [               0,               0,               0,               0,               0,    -0.079849438,    -0.017708777,      0.10842632,      0.19062512,       1.6474764 ],
  [               0,               0,               0,               0,               0,     -0.20040517,     -0.71220427,      -1.2665943,      0.41792936,       15.020778 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0067109699,   -0.0046138296 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.7338795,       10.973536 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        2.294988,      -6.8597312 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.2553353,      -2.8505817 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,      -2.8522667,       2.1374651,     -0.45307257,     0.030518653,    0.0016792577 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer1_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,    -0.028051143,     0.068387388,    -0.053872047,     0.012548778 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00031503482,   0.00086503748 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.16230868,       2.2125834 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.055106062,     -0.53014273 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.66584793,     -0.36454024 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.24185562,    0.0051295111 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.6762806,      -1.3163885 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -79.906214,       64.268825 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        30.15255,      -24.605705 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       22.953106,      -18.253838 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0024398931,   -0.0059682169,     0.014728394 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.0248114,      -6.9269708,       3.9408271 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0059635709,      0.72024648,       -1.751038 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.61064728,     -0.65937023,      -1.0366263 ],
  [               0,               0,               0,               0,               0,               0,               0,      -6.8377531,       11.073284,      -4.4220607 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.014491045,     0.030031286,   -0.0099646672 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.31680353,      -1.1644672,      0.96130932 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.4381548,      -4.3366228,     -0.50062059 ],
 ],
 [
  [               0,               0,               0,               0,     -0.20100458,      0.62345345,     -0.71263639,      0.37656564,    -0.097307628,    0.0059588021 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.080407475,      0.22026508,     -0.20404173,     0.052083915,     0.015697934,   -0.0044942798 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0027489608,    0.0035164191 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        1.888098,      0.58461439 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -9.9557638,       13.451702 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -15.78226,       21.172451 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.10766813,     -0.34646572 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer1_etaoffv5_35_e = [
 [
  [               0,               0,               0,               0,      0.28968245,     -0.80470543,      0.81411366,     -0.34483773,     0.041967641,    0.0038029965 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.17138058,       0.2527324 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -31.915288,       47.422473 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       50.403108,      -73.108506 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       300.23744,      -438.49256 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.1561158,     -0.29481008 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.018455163,    -0.024223042 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -138.57433,       209.29226 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       90.206864,      -136.11763 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       52.076694,      -77.504897 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.9750953,        11.91543 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.24898794,      0.51648952 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.24836071,     -0.51514959 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.24835028,     -0.51514059 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -4.8661457e-05,   8.8163654e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00014044583,   7.3184221e-05,  -3.4805084e-05,   0.00048653301 ],
  [               0,               0,               0,               0,               0,               0,       0.2708975,     -0.31859482,       -1.059992,       3.4057932 ],
  [               0,               0,               0,               0,               0,               0,        0.060785,      0.12187062,    -0.051171305,      -1.4803215 ],
  [               0,               0,               0,               0,               0,               0,    -0.082571059,    -0.065080018,      0.11772801,     -0.15832802 ],
  [               0,               0,               0,               0,               0,               0,     -0.16233013,    -0.010429136,      0.21595333,    -0.079743732 ],
  [               0,               0,               0,               0,               0,               0,   8.4188872e-05,   0.00013548989,    5.327386e-05,   0.00041522907 ],
  [               0,               0,               0,               0,               0,               0,    -0.042108868,     0.017032223,     -0.27617239,      0.56482012 ],
  [               0,               0,               0,               0,               0,               0,     -0.36002172,     -0.18642915,      0.51952144,       2.5235692 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00010375731,  -0.00026151838,   5.3550324e-05,    0.0034787114 ],
  [               0,               0,               0,               0,               0,               0,        2.260762,      -4.3513174,      -10.330575,       22.788172 ],
  [               0,               0,               0,               0,               0,               0,       2.1094851,      -4.0759044,      -9.2135463,       17.176717 ],
  [               0,               0,               0,               0,               0,               0,       1.2751575,      -2.0581581,      -5.4898959,       8.4553054 ],
  [               0,               0,               0,               0,               0,               0,     -0.65651865,       1.4495997,       3.3116134,      -7.1553047 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0010615587,   -0.0011463839 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -9.5274586,       26.098161 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.7668103,       5.3208839 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -23.818235,       54.453867 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.8110276,      -6.4538142 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0057333537,     0.020372955 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -24.361257,       56.134097 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.7679301,       -8.705522 ],
 ],
 [
  [               0,               0,               0,               0,               0,   1.2487613e-05,   2.3830126e-05,   4.2296461e-05,   6.5492516e-05,   0.00057035635 ],
  [               0,               0,               0,               0,               0,    0.0017791579,   -0.0058236136,    -0.043134716,     -0.18432197,       3.3392078 ],
  [               0,               0,               0,               0,               0,    -0.053294889,    -0.061214543,    0.0009619981,      0.32878634,       1.1064459 ],
  [               0,               0,               0,               0,               0,     0.086170103,      0.10699283,     0.025971079,     -0.47047375,      -3.3567832 ],
  [               0,               0,               0,               0,               0,    0.0058397053,    0.0098502125,     0.013608182,     0.008519018,      -0.3375822 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00054163118,   -0.0018548012 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0051450289,   0.00077379454,     0.035749768 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.0371736,    -0.013084996,      -13.318027 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.65856223,      0.47705337,       1.2704842 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.35864465,       0.3090604,      -2.8024417 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0050539888,  -9.5285583e-05,     0.030436391 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.1956539,    -0.084823869,      -17.824975 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.8850264,    -0.086924518,      -24.938792 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,     0.015325481 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,        3.699361 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -59.706324 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,     -0.80310477 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      0.48553648 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer2_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,    0.0068386975 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       3.9535403 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -1.3491017 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      0.01819209 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,   0.00068785431 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      0.63463032 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       -45.66517 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0057275447,    0.0041929482 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.4241844,       5.6367352 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.070840552,      -1.3360375 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.60504743,     -0.26885465 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00060576914,    0.0012051893 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.21257101,      0.45284162 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -3.776229,      0.38210823 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0057962706,    0.0035467033 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       32.896301,      -23.059756 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.69289625,       -2.051034 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.3277343,       2.7951544 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.013115677,     0.011915382 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       26.187826,      -20.426109 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -42.416647,       12.407247 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0003852776,   0.00056740863,   0.00097841018,    0.0063832498 ],
  [               0,               0,               0,               0,               0,               0,       1.1887578,      -1.4148636,       -2.351245,       5.6776276 ],
  [               0,               0,               0,               0,               0,               0,       0.5748699,     -0.87746474,     -0.23084962,     -0.88878379 ],
  [               0,               0,               0,               0,               0,               0,     -0.28187941,      0.40135034,      0.49928343,      -0.4568656 ],
  [               0,               0,               0,               0,               0,               0,    0.0018337924,   -0.0016692596,   -0.0029938242,    0.0033636093 ],
  [               0,               0,               0,               0,               0,               0,      -2.7644526,       3.3281307,       2.3301107,      -2.2779405 ],
  [               0,               0,               0,               0,               0,               0,        1.409238,      -1.1377237,       -2.754118,        0.462493 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0022780045,    0.0056424369 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -86.56751,        120.6273 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       12.995924,      -18.996889 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.6657565,      -2.0193519 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.031268642,      0.03958291,     0.014837751,   -0.0072813326,   -0.0031915881,    0.0011953141 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0016324317,    0.0060465173 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -353.32884,       507.16516 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -18.395235,       25.391423 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.6600859,      -1.1082059 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.28215946,     -0.40314834 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       57.424614,      -78.623724 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -31.115722,       45.435941 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer2_etaoffv5_35_e = [
 [
  [       1.0111368,      -1.2075325,      -0.7962058,      0.51596705,       1.0465173,     0.038775976,      -1.1191078,      0.59828408,    -0.090499717,   0.00070367093 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00035280314,   0.00013252696,    0.0011857784,    0.0037497957 ],
  [               0,               0,               0,               0,               0,               0,      0.15069229,       7.5176897,       2.6329307,      -20.432326 ],
  [               0,               0,               0,               0,               0,               0,      -5.7869378,      -2.8143559,       4.2827838,       18.342032 ],
  [               0,               0,               0,               0,               0,               0,       3.5449316,       1.7636308,      -2.6582318,      -12.198807 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.013331818,    -0.007333941 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       30.537499,      -44.550408 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.74435103,      -2.2578067 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.6493138,        -2.48958 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00028101216,    0.0012083767,    0.0061678823 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.9801366,       -10.06733,       11.444948 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.92687635,       3.1829903,      -3.9148098 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.29799427,      -1.0288338,      0.89867507 ],
  [               0,               0,               0,               0,               0,               0,               0,  -6.0601818e-05,   5.8737895e-05,   0.00034183355 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.32475217,      -1.2867908,       2.8920161 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.062785692,      -1.0483304,      -3.3487649 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0054431385,    -0.022053115,     0.022588263 ],
  [               0,               0,               0,               0,               0,               0,               0,      -65.656429,       22.814982,       289.40221 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.6453236,        32.44383,      -54.827227 ],
  [               0,               0,               0,               0,               0,               0,               0,      -11.383436,       37.663184,      -26.844066 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0017050255,   -0.0082314761,     0.012737189 ],
  [               0,               0,               0,               0,               0,               0,               0,        0.228867,      -1.0632582,       2.2096947 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.3202043,       10.162813,      -12.724865 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   -0.0014646539,   -0.0017882055,   4.5957796e-05,     0.010938667,      0.05362997 ],
  [               0,               0,               0,               0,               0,      0.68232296,     -0.51265616,      -3.1677839,      -4.7571047,       16.711796 ],
  [               0,               0,               0,               0,               0,      -1.1152066,        1.129424,       5.9532114,       7.6786383,      -32.706425 ],
  [               0,               0,               0,               0,               0,     -0.66597051,      0.85608139,       4.0029685,       4.7058262,      -23.851472 ],
  [               0,               0,               0,               0,               0,   0.00021926662,   0.00020789693,  -0.00012478471,   -0.0014407878,   -0.0043066175 ],
  [               0,               0,               0,               0,               0,      0.19914256,     0.038117226,     -0.48202686,      -1.5003689,     -0.22326446 ],
  [               0,               0,               0,               0,               0,     0.081405991,      0.54117446,      0.98168465,     -0.95991312,      -15.939021 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00098833715,     0.012468422 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.83596928,     -0.25039626 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.6165072,      -5.0477864 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.65633804,       1.6567716 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,       -0.438321,      0.59741996,     -0.16909611,     0.013001061,   0.00017884471 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer1_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,    -0.016563147,     0.042894348,    -0.035766879,    0.0084550388 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00034043535,   0.00090892595 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.24958022,       2.0993104 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.0964579,     -0.53311381 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.70329478,     -0.36708153 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.23979458,    0.0055423373 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.026998895,    0.0011069049 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -27.235858,       22.269392 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.50818986,     -0.15251332 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        21.52397,      -17.116518 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0024794264,   -0.0040027183,    0.0042456862 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.18277068,     -0.12058952,       1.4910653 ],
  [               0,               0,               0,               0,               0,               0,               0,      -4.3358991,       9.8666871,      -6.3653448 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.60525326,     -0.47048495,      -1.1105743 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.5145596,       6.3555496,       -4.108984 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0041972888,   0.00039945942,   -0.0022935353 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.6253894,      -8.1639896,       4.6207132 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.8252033,      -12.298545,       4.5656874 ],
 ],
 [
  [               0,               0,               0,               0,     -0.13646951,      0.43287385,     -0.49981026,      0.26609221,    -0.071271366,    0.0038463717 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.096506729,      0.26279202,      -0.2428891,     0.066852554,     0.013300811,   -0.0042965465 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0025815907,    0.0035055439 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.9767001,      0.54401978 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -9.5763971,       12.919507 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -16.315454,       21.930994 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.10754239,     -0.35094985 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer1_etaoffv5_37_e = [
 [
  [               0,               0,               0,               0,   -0.0036480921,    0.0042886662,    0.0030899882,    0.0091560036,    -0.020420152,    0.0062906052 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.12439537,     -0.18094232 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       141.39742,      -203.87847 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -263.11427,       386.91665 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1775.3722,       2593.7854 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.5914855,      -3.7579301 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.092744771,     -0.13596063 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -351.36616,       527.16449 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       206.45203,      -310.92606 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       141.91269,      -211.04031 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.1245382,      0.13271013 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.30352646,      0.67425473 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.30018329,     -0.66737282 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.30088316,     -0.66856999 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00019309428,   0.00035042253 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00037719289,   0.00029514336,  -9.2031477e-05,  -0.00061738442 ],
  [               0,               0,               0,               0,               0,               0,      0.45696258,     -0.97740665,      -2.1116888,       5.9494388 ],
  [               0,               0,               0,               0,               0,               0,     -0.43302241,     -0.26003259,      0.47075252,       2.1784867 ],
  [               0,               0,               0,               0,               0,               0,     -0.49608038,     -0.45857739,      0.48783088,       3.4394093 ],
  [               0,               0,               0,               0,               0,               0,     -0.38967359,    -0.026490724,      0.76343835,       1.6038614 ],
  [               0,               0,               0,               0,               0,               0,  -0.00014665896,  -1.2627561e-05,   0.00034116204,    0.0020725365 ],
  [               0,               0,               0,               0,               0,               0,      -0.3085896,     -0.11552881,     0.091922301,       2.3029513 ],
  [               0,               0,               0,               0,               0,               0,     -0.50081362,     -0.21579687,      0.95551909,       4.3881244 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   2.4845699e-05,  -0.00027875247,  -0.00030614788,    0.0027130148 ],
  [               0,               0,               0,               0,               0,               0,       1.0159476,      -2.0315271,      -4.7218182,       12.854854 ],
  [               0,               0,               0,               0,               0,               0,      0.41819039,     -0.41849079,      -1.2877588,      0.45980498 ],
  [               0,               0,               0,               0,               0,               0,       1.4449632,      -2.4739557,      -6.0953451,       10.435583 ],
  [               0,               0,               0,               0,               0,               0,     -0.55332918,       1.2523421,       2.7847942,      -6.1224784 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    2.135208e-05,    0.0065811864 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.1377422,       9.1996541 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       15.646536,      -36.541632 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -39.568737,       90.614632 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -4.9714941,       11.575292 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00024892825,    0.0026078274 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.0647587,       1.8667553 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.50800656,     -0.30736656 ],
 ],
 [
  [               0,               0,               0,               0,               0,   1.0220686e-05,   2.0095932e-05,   3.7413826e-05,   6.3505583e-05,   0.00058875248 ],
  [               0,               0,               0,               0,               0,    0.0068223032,   0.00061830878,    -0.040430683,     -0.20652229,       3.2217146 ],
  [               0,               0,               0,               0,               0,    -0.055405914,    -0.062551111,    0.0051731206,      0.35324808,        1.191915 ],
  [               0,               0,               0,               0,               0,     0.074256096,     0.092897481,     0.027731506,      -0.3793127,        -2.92663 ],
  [               0,               0,               0,               0,               0,    0.0014916135,    0.0048324474,     0.013654755,     0.035538405,     -0.21300934 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00036664731,   -0.0018245739 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    -0.014892245,    0.0017859736,     0.091329871 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.9019118,      0.83551082,      -20.651455 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.83058511,       1.2931314,      0.38689382 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.60974247,      0.59089612,      -5.0400122 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00063758986,  -2.9324388e-05,   -0.0039885561 ],
  [               0,               0,               0,               0,               0,               0,               0,       10.096356,      -17.092349,      -15.898148 ],
  [               0,               0,               0,               0,               0,               0,               0,      -8.2922267,     -0.64903758,       48.084169 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,     0.014851787 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,        4.292984 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -19.306143 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,     -0.74416844 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      0.57487266 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer2_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,    0.0069639397 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       3.9045844 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -1.3453134 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,     0.018985085 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,   0.00070968821 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       0.5738186 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -58.233274 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0058169837,    0.0042427989 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.4285832,       5.5895828 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.018896314,      -1.3628603 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.69416884,      -0.3078176 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00024036404,     0.001280089 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.21721651,      0.36986918 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.5515997,      0.22954655 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.12836588,    -0.097694495 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -9.3165285,       11.529533 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       9.9964298,      -9.5095844 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.4136997,      -3.4958318 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.018544189,    -0.013391346 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -27.071049,       22.546101 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -50.357201,       19.681978 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00033290323,   3.5380382e-05,    0.0016608187,    0.0058672271 ],
  [               0,               0,               0,               0,               0,               0,       1.6880329,      -1.8452713,      -3.1933754,       6.4374369 ],
  [               0,               0,               0,               0,               0,               0,      0.51705235,     -0.96745712,     0.098096793,      -1.0793303 ],
  [               0,               0,               0,               0,               0,               0,       -0.250081,      0.46586656,       0.3140795,     -0.36792591 ],
  [               0,               0,               0,               0,               0,               0,   0.00043004159,  -0.00042085376,  -0.00076734952,    0.0013893721 ],
  [               0,               0,               0,               0,               0,               0,      -2.1479015,       2.5025243,       2.0781409,      -1.6963702 ],
  [               0,               0,               0,               0,               0,               0,      -0.1807593,       0.2296584,     -0.32247428,      -1.5871777 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0033878958,    0.0041959246 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -95.186694,       132.32795 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       14.342519,      -20.817199 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.7198852,      -3.4454971 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.038176323,      0.04109708,     0.018166169,   -0.0048997727,   -0.0047501996,    0.0014103898 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0079875539,     0.021136838 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.62336623,       1.4471044 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -57.672085,        81.82887 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.65360842,     -0.87592931 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.061053616,    -0.085621746 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       81.601014,      -112.38021 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       24.975104,      -34.289582 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele25_layer2_etaoffv5_37_e = [
 [
  [     0.076273509,    -0.066501087,    -0.071926499,   -0.0051343182,     0.058595786,     0.058269071,     -0.01252718,    -0.070851899,     0.033603698,   -0.0035521337 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00030137474,  -0.00016728538,   0.00014174584,    0.0012502952 ],
  [               0,               0,               0,               0,               0,               0,      -115.60388,      -55.825431,       82.522395,       375.62765 ],
  [               0,               0,               0,               0,               0,               0,      -4.4451146,       -2.398476,        2.672912,       12.813856 ],
  [               0,               0,               0,               0,               0,               0,      -2.2518957,      -3.2944492,      -4.8085309,      -7.3012258 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.020850313,    0.0013216055 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       16.170368,      -23.540168 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -8.5729831,       12.173943 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.0674932,      -6.1526873 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0003918751,    0.0014044404,    0.0060826279 ],
  [               0,               0,               0,               0,               0,               0,               0,        3.001237,      -10.051836,       11.227711 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.53059429,       1.8283006,      -2.7359669 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.20902249,      -0.7463764,       0.6684009 ],
  [               0,               0,               0,               0,               0,               0,               0,   3.8352877e-05,   4.4121682e-05,   0.00012850813 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.043473899,     -0.37178457,       2.3156181 ],
  [               0,               0,               0,               0,               0,               0,               0,       0.1089934,      -1.0850818,      -3.5023384 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00091087781,   -0.0015787439 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -332.60763,       758.44259 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       17.348735,      -39.134589 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -11.769568,       26.575843 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0012592772,    0.0056634759 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.1321644,       1.2612333 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.30248756,      -2.2689947 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -0.00043240269,  -0.00076430793,  -0.00039500853,    0.0042028935,      0.02656472 ],
  [               0,               0,               0,               0,               0,       2.2274118,      -2.6797326,      -12.188937,      -13.943605,       69.059763 ],
  [               0,               0,               0,               0,               0,      0.33882209,     -0.80609349,      -1.2406532,       1.5645669,       2.1417511 ],
  [               0,               0,               0,               0,               0,      -5.1629127,       5.6081978,       28.897787,       36.983227,      -161.09016 ],
  [               0,               0,               0,               0,               0,   0.00044462572,   0.00059027023,   3.0634668e-05,   -0.0039544882,    -0.020108314 ],
  [               0,               0,               0,               0,               0,      0.99301157,      -1.3019111,       -5.488834,      -5.8803974,       30.838243 ],
  [               0,               0,               0,               0,               0,       6.0444481,      -6.9337302,      -32.238726,      -37.496608,       172.05379 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.00120172,     0.013798364 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.77058574,     -0.20526282 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.6708285,       -5.154469 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.77002799,       1.9309936 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     -0.89267068,      0.96420149,     -0.26191432,     0.021400095,   7.3716165e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]

###############################################################################
# Tables for v5 sshape correction, 50 GeV electron clusters
#


ele50_layer1_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,    -0.021365025,     0.056429781,    -0.047167133,     0.011444599 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00029422679,   0.00075903377 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.06000156,       2.7306516 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.066212297,     -0.56014191 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.56392439,     -0.44262999 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.23983011,   -0.0042030622 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.059529655,    -0.038195795 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -52.433486,       42.563533 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -11.537638,       8.4310328 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       47.191969,      -37.526515 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.002462266,   -0.0068053847,    0.0047459397 ],
  [               0,               0,               0,               0,               0,               0,               0,       12.216915,      -13.619958,      0.45070321 ],
  [               0,               0,               0,               0,               0,               0,               0,      -16.273838,        25.27925,       -8.739489 ],
  [               0,               0,               0,               0,               0,               0,               0,       -19.01766,       26.544027,      -7.4857528 ],
  [               0,               0,               0,               0,               0,               0,               0,      -10.763588,       19.257266,      -8.7515425 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00067192241,   -0.0015363126,    0.0016167724 ],
  [               0,               0,               0,               0,               0,               0,               0,        1.252663,      -2.9595936,       2.6448755 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.56988153,       1.3977039,      -2.5116409 ],
 ],
 [
  [               0,               0,               0,               0,    -0.081721859,      0.32353779,     -0.43088738,      0.25575975,    -0.075866302,    0.0050967302 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.02892432,     0.056124107,   -0.0086735996,    -0.050264364,     0.035865954,   -0.0041733006 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0034478421,    0.0032201378 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.2779443,     -0.92670374 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -20.618123,       28.827634 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        -28.3919,       38.971179 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.018286399,     -0.28083456 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer1_etaoffv5_55_e = [
 [
  [               0,               0,               0,               0,      0.15664863,     -0.30406162,      0.19898408,    -0.070163129,    0.0074364132,    0.0055253208 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.011584843,      0.00524826 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        45.99611,      -65.889589 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       8.5227473,      -13.642239 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -15.174056,       21.309371 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.074963614,     -0.15637797 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0049788824,   -0.0018560827 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.3996783,      0.54571016 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       44.662541,       -67.94626 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       15.977093,      -23.657785 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.9732823,       5.7589369 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.11321416,       0.2886153 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.11280157,     -0.28769419 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.11290712,     -0.28787045 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -1.2059336e-05,   3.0770732e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0039204917,    0.0027663176,   -0.0034751251,    -0.023218257 ],
  [               0,               0,               0,               0,               0,               0,    -0.023714601,     -0.46999364,     -0.82226279,       4.5877102 ],
  [               0,               0,               0,               0,               0,               0,  -0.00061558094,   -0.0031016709,    -0.086536055,     -0.59662496 ],
  [               0,               0,               0,               0,               0,               0,     -0.34008938,        0.535407,       1.1690347,      -2.5643117 ],
  [               0,               0,               0,               0,               0,               0,     -0.70702661,      -0.2201369,       1.1667844,       3.8733236 ],
  [               0,               0,               0,               0,               0,               0,    0.0021481549,   0.00099156624,   -0.0024780457,   -0.0084978455 ],
  [               0,               0,               0,               0,               0,               0,      -0.1902944,      0.25838653,       0.2334185,    -0.022760927 ],
  [               0,               0,               0,               0,               0,               0,      -1.2797353,     0.070434829,        2.666487,       5.6767508 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00022873507,   -0.0013032669,  -0.00057161715,     0.012653557 ],
  [               0,               0,               0,               0,               0,               0,     -0.16637069,      -0.8190941,     -0.41113271,       6.9700682 ],
  [               0,               0,               0,               0,               0,               0,       3.1125187,       -6.087015,       -10.49481,       19.615696 ],
  [               0,               0,               0,               0,               0,               0,      -1.8140551,       5.1591943,       7.8026709,      -22.675655 ],
  [               0,               0,               0,               0,               0,               0,       -1.188777,       2.3347733,        6.190917,      -12.236134 ],
  [               0,               0,               0,               0,               0,               0,   0.00015244148,   6.1052469e-05,  -0.00024958887,  -0.00090135191 ],
  [               0,               0,               0,               0,               0,               0,     -0.50187115,      0.57193275,       1.5634369,      0.57060306 ],
  [               0,               0,               0,               0,               0,               0,      0.92445729,      -1.6907104,      -4.3356796,       6.4220385 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0014519851,     0.005818967 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -11.221063,       26.842644 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -4.0193555,       8.2898837 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -5.9902711,       13.163402 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.85578518,      -1.4811586 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00057262055,    0.0018418803 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.7520209,        4.287969 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.32944807,      -2.2263775 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00016095503,   0.00020738509,   7.4342133e-05,  -0.00082188765,   -0.0038320209 ],
  [               0,               0,               0,               0,               0,    -0.034700147,    -0.056793855,     -0.07549625,     0.032588682,        4.391671 ],
  [               0,               0,               0,               0,               0,    0.0089921809,     0.005808734,    -0.019273085,     -0.11624168,     -0.82679698 ],
  [               0,               0,               0,               0,               0,    -0.084806324,    -0.091126751,     0.025647771,       0.5903088,        1.555187 ],
  [               0,               0,               0,               0,               0,     -0.02305139,    -0.022965026,     0.016111438,      0.19445367,      0.50492194 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   5.4230477e-05,   0.00063028261 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.3599195,       4.5056032 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.27156527,      0.25188174 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.44024684,     0.067622835 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.32479894,      -1.0791692 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    -0.002495024,   0.00086755881,     0.019266688 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.6819627,      0.42110967,        25.97869 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.73500175,      0.45468685,       1.5552405 ],
  [               0,               0,               0,               0,               0,               0,               0,        1.564392,      0.24733756,      -9.6785292 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0047087785,    5.070889e-05,    -0.026909718 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.3591181,     0.054444537,        14.72811 ],
  [               0,               0,               0,               0,               0,               0,               0,      -0.8341834,      -2.0819703,      -5.2263389 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.001635551,     0.014579616,    -0.003635442 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.060972694,     0.049924761,        19.88362 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.081962276,   -0.0071927119,       113.03676 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.022263396,     -0.22437111,      0.49407199 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.063322114,    -0.082547626,       1.9010064 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer2_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,     0.006945429 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       3.4971183 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -1.3306274 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,     0.020393538 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,   0.00047314558 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      0.87487878 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -26.871217 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,  -0.00079784598,     0.002815345,    0.0066124914 ],
  [               0,               0,               0,               0,               0,               0,               0,      -4.5121412,       3.9419056,       2.4402885 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.64993541,      0.57605132,      -1.5046951 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.2135829,       -1.010217,      0.28022877 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0019229585,   -0.0017307734,   0.00074758263 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.10776541,     -0.98167802,       1.2071539 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.8224316,      -4.1119193,     -0.73774099 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.064171572,    0.0011988486,    -0.036919591 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.31887237,      -2.5518497,       5.9184956 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.0441593,       -3.391335,      -1.2169324 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.3524192,     -0.61007074,      -1.0323386 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.012969389,  -0.00093760068,   -0.0063968525 ],
  [               0,               0,               0,               0,               0,               0,               0,      -13.872485,      -6.1165102,       14.734133 ],
  [               0,               0,               0,               0,               0,               0,               0,      -33.988614,     -0.42370858,       20.587186 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00024016646,   -0.0017455942,   0.00043363729,    0.0076720907 ],
  [               0,               0,               0,               0,               0,               0,       1.1040792,      -1.3888326,      -1.2002434,       4.4833836 ],
  [               0,               0,               0,               0,               0,               0,      0.95707088,      -2.2831283,       1.2507144,      -1.3907776 ],
  [               0,               0,               0,               0,               0,               0,      0.23077837,      -1.1040648,         2.21037,      -1.1256859 ],
  [               0,               0,               0,               0,               0,               0,   0.00036627137,  -0.00011629955,  -0.00043376857,   0.00081293003 ],
  [               0,               0,               0,               0,               0,               0,     -0.55173676,      0.88678879,      0.18522788,       0.4476668 ],
  [               0,               0,               0,               0,               0,               0,      0.52466926,     -0.27031865,      -0.4814428,      -1.5205903 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.16180197,      0.23023513 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -19.851153,       29.392247 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.6466925,      -9.1133274 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.1400545,      -1.1831133 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0091886169,     0.011667726,   0.00036750031,    0.0042229516,   -0.0078215967,   0.00088324815 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0076905001,    0.0062319988,    0.0020516598,   -0.0064409166 ],
  [               0,               0,               0,               0,               0,               0,       2.8580964,       4.2701975,       6.3597678,       13.483316 ],
  [               0,               0,               0,               0,               0,               0,       -4.277713,      -2.0590131,       2.3876751,       9.9890288 ],
  [               0,               0,               0,               0,               0,               0,      0.10501392,      0.14403888,       0.2119632,     0.032301563 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer2_etaoffv5_55_e = [
 [
  [     -0.01137174,   -0.0087611367,    -0.002569603,    0.0068907464,     0.016701726,     0.018793814,     0.001748802,    -0.024889465,   -0.0016734697,  -0.00011232719 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00076426418,  -0.00040967666,   0.00049951304,    0.0029317915 ],
  [               0,               0,               0,               0,               0,               0,       62.143343,      -92.272317,        13.38908,      -23.847371 ],
  [               0,               0,               0,               0,               0,               0,        14.72913,       7.6419224,      -10.455173,      -49.243968 ],
  [               0,               0,               0,               0,               0,               0,       12.735637,       6.0309253,      -10.376221,      -44.700034 ],
  [               0,               0,               0,               0,               0,               0,    0.0051104318,    0.0024729161,   -0.0038205433,    -0.015642219 ],
  [               0,               0,               0,               0,               0,               0,       -2.004342,     -0.86601878,       1.3099709,       7.7945727 ],
  [               0,               0,               0,               0,               0,               0,      -3.3057097,      -1.5968979,       2.2686791,       10.005828 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0036673114,    0.0050619149 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       36.963611,      -54.322859 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -5.3093776,       6.9442122 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.67116309,       1.0585114 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    -0.034259395,      0.12156858,    -0.098802923 ],
  [               0,               0,               0,               0,               0,               0,               0,       7.6581403,      -26.474306,       25.347035 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.5005931,       4.9213733,      -5.1843291 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.9671985,      -6.2945544,       5.0444197 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0046585103,    -0.011382201,    0.0060425906 ],
  [               0,               0,               0,               0,               0,               0,               0,      -14.332075,       52.587204,      -47.894557 ],
  [               0,               0,               0,               0,               0,               0,               0,      -0.7051257,      -1.0615772,       1.5974784 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,      0.00157318,  -0.00066279383,   -0.0049441535,    -0.006397629,     0.021079143 ],
  [               0,               0,               0,               0,               0,    -0.033099028,      0.49014099,     -0.99542909,      -4.3859997,       11.754468 ],
  [               0,               0,               0,               0,               0,     -0.21332138,       1.2002905,     -0.28482286,      -6.0445327,       5.8556442 ],
  [               0,               0,               0,               0,               0,     -0.31901001,      0.80651621,      0.70092298,      -2.0872605,     0.096771304 ],
  [               0,               0,               0,               0,               0,   0.00051404953,  -0.00043356492,   -0.0022179168,   -0.0027435628,     0.010035513 ],
  [               0,               0,               0,               0,               0,      0.64023815,     -0.82584759,      -2.9993192,      -2.7189425,       14.368133 ],
  [               0,               0,               0,               0,               0,       3.0612063,      -2.9569803,       -13.20852,      -14.569387,        54.53292 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -0.00017425306,   -0.0003656575,  -0.00014828099,     0.002809565,     0.018134933 ],
  [               0,               0,               0,               0,               0,       3.1391294,      -3.7082803,      -17.850233,      -21.621013,       103.39406 ],
  [               0,               0,               0,               0,               0,      -1.6211787,       1.7929468,       9.0591957,       11.322863,      -51.451263 ],
  [               0,               0,               0,               0,               0,      0.32011138,     -0.39833969,      -1.5637882,      -1.4301286,       7.4133079 ],
  [               0,               0,               0,               0,               0,   0.00014485252,    0.0003820594,   0.00048254379,    -0.001057615,   -0.0094937642 ],
  [               0,               0,               0,               0,               0,    -0.089622253,      0.37272825,       1.0022143,      0.17664543,      -7.6732213 ],
  [               0,               0,               0,               0,               0,      0.78540067,      -1.5836387,      -5.7596896,      -4.9462894,       38.191885 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     -0.10050099,   -0.0026182259,      0.62924649 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.46960236,     -0.13118129,      -0.6398208 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.4963966,     0.026566886,      -10.280431 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.31700792,       -0.043607,      -1.5946844 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,       -3.311162,       2.5741246,      -0.5233867,       0.0322458,    0.0011674605 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer1_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,    0.0003881124,    0.0077060888,    -0.011456178,    0.0028066378 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00031901078,   0.00077656097 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.12023892,       2.7122903 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.060976764,     -0.58465971 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.50393386,     -0.49218575 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.22988838,   -0.0051460415 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.046219804,    -0.023786825 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -41.213035,       33.520925 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -3.328658,       1.9852474 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       33.633394,      -26.745142 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0014967471,   -0.0048858166,    0.0040624323 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.14408588,       8.1986643,      -7.9660999 ],
  [               0,               0,               0,               0,               0,               0,               0,       -9.466932,      0.83109683,       5.3931569 ],
  [               0,               0,               0,               0,               0,               0,               0,      -9.8117243,       1.3996506,       5.2345288 ],
  [               0,               0,               0,               0,               0,               0,               0,       -3.544003,       7.6714854,      -4.0539727 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0056100666,     0.012906421,   -0.0053361537 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.85187534,      -2.0456099,       1.8416926 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.39678512,     -0.81466038,      -1.2702824 ],
 ],
 [
  [               0,               0,               0,               0,     0.027480029,    -0.022563078,    -0.024316812,     0.035444229,    -0.021399463,   0.00025698683 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.32266942,      0.80679611,     -0.69448893,      0.21243025,    0.0013145048,   -0.0049611408 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0034459664,    0.0029270156 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.4383882,      -1.0007977 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -21.052644,       29.444282 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -30.177812,       41.512106 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0072329422,     -0.27422165 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer1_etaoffv5_35_e = [
 [
  [               0,               0,               0,               0,    -0.045777011,     0.055888405,     0.011248523,    -0.026357036,    0.0023387136,   -0.0017334707 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0038900867,    0.0042712062 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -6.5824578,       11.337378 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       10.515476,       -16.48412 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       44.349488,       -65.58447 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.031678563,     -0.16113483 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.1505525,     -0.22046815 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -254.82403,       385.19598 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       70.652599,      -107.59673 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       44.507668,      -65.957453 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      -104.49506,       236.79109 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.11882547,      0.24640453 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.11878429,     -0.24630358 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.11880875,     -0.24634427 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   2.2962416e-06,  -2.2478225e-06 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0037053118,    0.0026992547,   -0.0032367847,    -0.022263868 ],
  [               0,               0,               0,               0,               0,               0,     -0.25973383,    -0.013937181,     0.039129325,       2.9441584 ],
  [               0,               0,               0,               0,               0,               0,     -0.15303779,      0.27246718,       0.4693846,      -1.5857821 ],
  [               0,               0,               0,               0,               0,               0,     -0.28620711,      0.39777609,      0.95549384,      -2.0122057 ],
  [               0,               0,               0,               0,               0,               0,     -0.74048427,     -0.26675351,       1.1862831,       4.2660258 ],
  [               0,               0,               0,               0,               0,               0,    0.0022984304,   0.00098904332,   -0.0026773798,   -0.0089320122 ],
  [               0,               0,               0,               0,               0,               0,     -0.10609381,     0.098201647,    -0.061204665,       0.5452692 ],
  [               0,               0,               0,               0,               0,               0,      -1.1476631,      -0.1449627,       2.2486719,       6.3310548 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00044568568,    -0.001890676,  -0.00064369237,     0.018635489 ],
  [               0,               0,               0,               0,               0,               0,       0.2366946,    0.0030701347,     -0.85850532,        1.430375 ],
  [               0,               0,               0,               0,               0,               0,      0.67605556,      -1.9305991,      -2.5396495,       6.8332244 ],
  [               0,               0,               0,               0,               0,               0,      -1.6511821,       3.5445839,       6.4039552,       -14.48651 ],
  [               0,               0,               0,               0,               0,               0,     -0.80706135,       1.7192801,       3.9788383,      -8.4369874 ],
  [               0,               0,               0,               0,               0,               0,   0.00040166565,   0.00059310075,  -0.00041033438,   -0.0065183556 ],
  [               0,               0,               0,               0,               0,               0,     0.070738539,    -0.021369997,      -0.4666496,      0.98988402 ],
  [               0,               0,               0,               0,               0,               0,      0.39159126,       -0.605236,      -1.7256876,       1.0410885 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0017789464,    0.0061056962 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -10.403688,       24.994218 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.4986622,       7.1662988 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -5.1472168,       11.140099 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.6215833,      -1.4900745 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00028619658,    0.0012648708 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        -1.40774,       3.2279405 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.17902632,     -0.57434629 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00018481293,   0.00023117518,   6.7566586e-05,  -0.00095464861,   -0.0043482142 ],
  [               0,               0,               0,               0,               0,    -0.027601564,    -0.046246919,    -0.065594255,     -0.05471731,        4.098713 ],
  [               0,               0,               0,               0,               0,    0.0040318374,   -0.0026734722,    -0.025534249,    -0.085176749,      -0.6164596 ],
  [               0,               0,               0,               0,               0,    -0.086258798,    -0.087293461,     0.037868006,      0.59535588,       1.4412178 ],
  [               0,               0,               0,               0,               0,    -0.022089215,    -0.020433158,     0.019378501,      0.18777738,      0.43603712 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   5.3399261e-05,   0.00062829316 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.5238141,       4.3011403 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.33073129,      0.39227102 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.51217792,      0.23197931 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.33133974,      -1.0949043 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    -0.002824958,   0.00093096626,     0.021170878 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.4908325,       1.0851849,       23.193058 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.9570979,       7.6899297,      -8.8109621 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.0600082,       2.0271828,      -11.017593 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0041423746,    2.551998e-05,    -0.023472275 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.7412447,      0.78329847,       15.207462 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.067122036,       -2.641028,      -21.077626 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0019884265,     0.020714674,   -0.0021607741 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.15347833,     0.088650719,       31.053253 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.10314973,     0.012823992,       102.66172 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.15629885,     -0.26323072,       1.4821667 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.010268092,    -0.073170867,       1.6408336 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer2_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,     -0.05731258,     0.030649892,    0.0035635502 ],
  [               0,               0,               0,               0,               0,               0,               0,       26.512473,      -14.096833,       5.8779562 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.9224975,      0.88820253,      -1.4326909 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.030862268,     -0.23597861,    0.0053514341 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.037683146,    -0.011486198,    0.0017311608 ],
  [               0,               0,               0,               0,               0,               0,               0,       7.4990761,      -5.3671343,       1.0665185 ],
  [               0,               0,               0,               0,               0,               0,               0,      -9.5173159,       4.9725886,      -1.8439227 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0056308118,   -0.0058699866,    0.0088626676 ],
  [               0,               0,               0,               0,               0,               0,               0,      -8.0228493,       7.8624656,       2.2182152 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.1886055,       1.1312376,      -1.6717223 ],
  [               0,               0,               0,               0,               0,               0,               0,        1.496603,       -1.133777,       0.4067287 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0023632417,  -0.00055026732,    0.0014426178 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.46709569,     -0.47624955,      0.33933586 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.9224324,       1.2619228,      -2.6023775 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.016547751,    0.0027270926,   -0.0047915365 ],
  [               0,               0,               0,               0,               0,               0,               0,       17.650206,       -8.915012,     -0.59248467 ],
  [               0,               0,               0,               0,               0,               0,               0,       12.491422,      -8.7588656,      -2.4320334 ],
  [               0,               0,               0,               0,               0,               0,               0,      -19.574388,       2.4505223,       19.964979 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.047220027,     0.030987499,     0.020063034 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.038073013,      -1.2141448,      0.99268196 ],
  [               0,               0,               0,               0,               0,               0,               0,       13.648952,       -19.49612,      -21.359088 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00064085499,   0.00013847415,    0.0019702427,    0.0055557332 ],
  [               0,               0,               0,               0,               0,               0,       1.4566994,      -2.1296403,      -1.6089686,        5.804539 ],
  [               0,               0,               0,               0,               0,               0,      0.91699792,      -2.2246007,       1.2533857,      -1.4247902 ],
  [               0,               0,               0,               0,               0,               0,     -0.11050536,     0.053242672,      0.74505017,     -0.53242523 ],
  [               0,               0,               0,               0,               0,               0,   0.00079081178,  -0.00064960165,   -0.0010933868,    0.0016024741 ],
  [               0,               0,               0,               0,               0,               0,      -1.3172027,       1.4818842,       1.2713597,     -0.67043113 ],
  [               0,               0,               0,               0,               0,               0,      0.53517617,     -0.66911422,      -1.3156013,     -0.52856008 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00021012118,    0.0083023341 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -53.764088,       76.173637 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,         9.34281,      -14.099364 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.1624283,      -2.7384481 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      -0.3847614,      0.88626642,     -0.68445236,      0.22915327,    -0.035007285,    0.0016437853 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,      0.13307333,     0.066898045,    -0.082678346,     -0.37537983 ],
  [               0,               0,               0,               0,               0,               0,       -6.698957,      -3.0954492,       4.6992818,       23.017748 ],
  [               0,               0,               0,               0,               0,               0,     -0.40366214,       7.3195136,       5.3985572,      -22.696422 ],
  [               0,               0,               0,               0,               0,               0,    -0.011314312,     0.021611108,     0.058554744,     -0.33302925 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer2_etaoffv5_35_e = [
 [
  [      0.34385815,     -0.40790432,     -0.30263858,       0.1647236,      0.40659679,     0.051540848,     -0.46038987,      0.23649951,    -0.029202415,   -0.0066462224 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,     -0.00197544,    0.0022475029,    0.0037811147,   -0.0040799372 ],
  [               0,               0,               0,               0,               0,               0,       123586.59,       172431.98,       229963.51,       282293.74 ],
  [               0,               0,               0,               0,               0,               0,       2926.7647,       1431.3211,      -2129.4065,      -9371.8141 ],
  [               0,               0,               0,               0,               0,               0,      -1124.2224,      -564.84602,       783.21792,       3544.3899 ],
  [               0,               0,               0,               0,               0,               0,   -0.0077108922,    -0.025946944,    -0.010630785,      0.10222781 ],
  [               0,               0,               0,               0,               0,               0,        14.50305,      -3.3769859,      -18.637306,      -11.414352 ],
  [               0,               0,               0,               0,               0,               0,       1.4763628,      -10.675606,      -9.4561117,       32.451182 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0034205057,      0.00468423 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       61.236308,      -90.525679 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.23551132,      -1.6338931 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.11388765,      0.26727844 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0006523621,   0.00051874576,    0.0091941683 ],
  [               0,               0,               0,               0,               0,               0,               0,       5.6496773,      -18.504103,       18.243373 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.6862728,       5.4362928,      -5.5976085 ],
  [               0,               0,               0,               0,               0,               0,               0,      -0.1162858,       1.3941943,      -1.8348405 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0006820777,    -0.001146441,    0.0024224873 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.0831104,      -6.5952097,       5.2882085 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.46213257,      -1.9710011,       2.5225199 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0041672609,   -0.0048549886,    -0.014043534,     0.021341567 ],
  [               0,               0,               0,               0,               0,               0,        0.467369,      0.22641024,      -12.011667,       22.584273 ],
  [               0,               0,               0,               0,               0,               0,     -0.78204277,       6.1475906,      -14.470081,        9.273696 ],
  [               0,               0,               0,               0,               0,               0,      -1.6980428,       5.5620955,      -1.5918765,      -5.3200637 ],
  [               0,               0,               0,               0,               0,               0,    0.0011949136,   -0.0019323968,    -0.004926254,    0.0091272065 ],
  [               0,               0,               0,               0,               0,               0,       2.0788378,      -4.5569691,      -10.363576,       22.841597 ],
  [               0,               0,               0,               0,               0,               0,       8.4465177,      -16.826533,      -37.272511,       72.051289 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -8.4561586e-05,   8.7106188e-05,   0.00095877739,    0.0042238527,     0.015533467 ],
  [               0,               0,               0,               0,               0,      0.88555778,      -1.0686978,      -5.1248061,      -6.3122649,       31.823667 ],
  [               0,               0,               0,               0,               0,     0.052445977,     -0.10800809,      -0.2280541,     0.016190157,    -0.032868932 ],
  [               0,               0,               0,               0,               0,     -0.23573386,      0.26072558,       1.3913871,       1.8567613,      -8.2376298 ],
  [               0,               0,               0,               0,               0,   -5.013919e-05,  -8.5411223e-06,   0.00015378806,   0.00056060214,    0.0022621919 ],
  [               0,               0,               0,               0,               0,    -0.013297033,     0.034267874,     0.092199225,    -0.052293297,      0.63276094 ],
  [               0,               0,               0,               0,               0,     -0.05688056,      0.13561811,       0.3571002,     -0.30882755,      -5.9885671 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00058624915,    0.0010942506,    0.0023307926 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.51794083,     -0.18366719,       6.2995209 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.6487041,     -0.11910515,      -10.958894 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.59025632,     0.051098208,      -3.6221666 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     -0.62040175,      0.98500907,     -0.30389208,     0.026606312,  -0.00062485316 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer1_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,   -0.0081443031,     0.027528504,     -0.02679926,    0.0067184332 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00032517058,   0.00081784711 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.044790496,       2.6050848 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.056608416,     -0.61193694 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.52804028,      -0.5067023 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.23005542,   -0.0031307263 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.03729511,    -0.013399373 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -37.803983,        30.74712 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.2066536,      -1.5753627 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       27.221226,      -21.646324 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.001867275,   -0.0050846968,    0.0034942982 ],
  [               0,               0,               0,               0,               0,               0,               0,      -31.440058,        38.73289,      -10.013409 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.9052363,       20.948201,      -16.579436 ],
  [               0,               0,               0,               0,               0,               0,               0,       5.0558467,       8.7494333,      -10.940353 ],
  [               0,               0,               0,               0,               0,               0,               0,      -14.687782,       26.833318,      -12.469139 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.001040936,   -0.0021615049,    0.0016380288 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.5135251,       5.6415918,      -1.9669383 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.22363923,     -0.31009454,      -1.6847655 ],
 ],
 [
  [               0,               0,               0,               0,     0.026202901,     -0.02018546,    -0.023969651,     0.032879053,    -0.020014039,   9.0584576e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.32735452,      0.81807436,     -0.70413812,      0.21600622,   0.00071047774,   -0.0049043374 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0034561228,    0.0029233009 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.4090062,     -0.96593592 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -20.865433,        29.17494 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -29.997448,       41.254276 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0071657906,     -0.27401645 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer1_etaoffv5_37_e = [
 [
  [               0,               0,               0,               0,    -0.064342643,     0.083226045,     0.018227163,    -0.050975328,     0.011926281,    -0.002925229 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0013722671,     0.001866372 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.51753033,       3.8784821 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.6406574,       4.2679569 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.2406591,      -5.7430863 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.022029887,     -0.32107858 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.074350716,     -0.10846169 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -381.56593,       576.27515 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       13.809302,      -23.694787 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        157.4673,       -233.8499 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      -51.638843,       157.40737 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.10677291,      0.22595368 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.10671786,     -0.22583265 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.1067379,     -0.22586672 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -3.0989599e-07,   1.7084433e-06 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,     0.006749279,     0.004057785,   -0.0063096804,    -0.035504736 ],
  [               0,               0,               0,               0,               0,               0,       1.1671672,      -2.5070737,      -5.0416607,       11.495926 ],
  [               0,               0,               0,               0,               0,               0,      0.23229845,     -0.43409449,     -0.93163509,       1.0043287 ],
  [               0,               0,               0,               0,               0,               0,     -0.88477837,       1.6662728,       3.2386627,      -6.8265913 ],
  [               0,               0,               0,               0,               0,               0,     -0.57216848,     -0.13118953,      0.98308923,       2.8291727 ],
  [               0,               0,               0,               0,               0,               0,     0.003559525,    0.0011545753,   -0.0045462219,    -0.011243246 ],
  [               0,               0,               0,               0,               0,               0,     -0.22002946,      0.41374829,      0.27519776,      -0.5191111 ],
  [               0,               0,               0,               0,               0,               0,      -2.2083005,       2.2771912,       6.2380383,      -2.8806022 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00065365901,    -0.002257541,  -0.00086021602,     0.020809997 ],
  [               0,               0,               0,               0,               0,               0,      -2.2690681,       6.2179247,       9.8651696,      -24.976426 ],
  [               0,               0,               0,               0,               0,               0,      -7.2934154,       13.234165,       32.118966,      -59.633322 ],
  [               0,               0,               0,               0,               0,               0,      -2.9606598,       5.8198484,        10.55491,      -21.593592 ],
  [               0,               0,               0,               0,               0,               0,      -1.7081992,       3.3580871,       8.8148172,      -17.029644 ],
  [               0,               0,               0,               0,               0,               0,  -0.00083560657,   0.00027749605,    0.0024434946,    0.0036576792 ],
  [               0,               0,               0,               0,               0,               0,       3.4778384,      -6.4912809,       -15.51727,       29.153369 ],
  [               0,               0,               0,               0,               0,               0,       2.6625847,      -5.2265013,      -10.753978,       18.833355 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.001518159,    0.0056673003 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -11.98144,       28.635139 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.9563952,       8.2171953 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -5.9294785,       12.951406 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.75695307,      -1.6634115 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0001781978,   0.00095195441 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.3260212,       3.0369697 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.45332452,       0.1081038 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00021424615,   0.00027470422,   9.3227793e-05,   -0.0011122897,   -0.0053181692 ],
  [               0,               0,               0,               0,               0,    -0.028859911,    -0.048520754,    -0.069332387,    -0.060643209,       4.0876337 ],
  [               0,               0,               0,               0,               0,   -0.0091318091,    -0.013870961,    -0.015543183,  -0.00066106401,       -0.332749 ],
  [               0,               0,               0,               0,               0,    -0.071896876,    -0.069313637,     0.039518637,      0.49589677,      0.94699738 ],
  [               0,               0,               0,               0,               0,    -0.017400714,    -0.014566746,     0.019911398,      0.15521005,      0.27380646 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   5.4255476e-05,   0.00063037481 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.4880596,       4.3782682 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.2917856,      0.29935187 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.46932897,      0.12533493 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.34333348,      -1.1236677 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0039185757,     0.001036257,     0.027401127 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.7476285,       1.4149198,       17.782559 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.1845875,       5.1662915,      -7.2095128 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.1185831,       1.1043134,        -9.14575 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0033757741,   0.00034779749,    -0.019721069 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.7221453,      0.15518929,        16.61119 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.18060304,     -0.16150191,      -1.3685236 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00045720136,    0.0079207837,    0.0030041474 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.019656402,    -0.060611717,        4.942394 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.027417524,     -0.02701815,        4.912124 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.057207659,    -0.025417512,     -0.28098117 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0072826596,    0.0060623103,      0.65018012 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer2_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0011117023,    0.0073990318 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.57582608,       4.0975656 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.33437844,      -1.2946934 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.62435089,    -0.061920711 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0011303213,    0.0010793412 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.16092535,       0.2116594 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.6845134,      -1.3567233 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0015513435,    0.0029197832,    0.0062095897 ],
  [               0,               0,               0,               0,               0,               0,               0,      -5.0757053,       4.2194856,       3.3167542 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.4885721,       1.4645412,      -1.7636237 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.4289902,      -1.0040306,      0.28584168 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0023028096,  -0.00045489457,   0.00092290128 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.88831487,        -1.05937,      0.58457645 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.63342669,       -2.309171,      -1.2690769 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,     0.029207826,    0.0096940322,   -0.0032204595,    -0.010859784 ],
  [               0,               0,               0,               0,               0,               0,       7.6145075,      -2.0749477,      -3.4345096,       3.8153678 ],
  [               0,               0,               0,               0,               0,               0,       16.259244,       -6.250855,      -8.0989009,      0.68224596 ],
  [               0,               0,               0,               0,               0,               0,       -2.945668,      -2.5036446,      0.22084858,       3.5544381 ],
  [               0,               0,               0,               0,               0,               0,    0.0095806664,    0.0035842862,  -0.00041722979,   -0.0019698525 ],
  [               0,               0,               0,               0,               0,               0,      -2.0095091,      -0.7326501,     -0.37433447,       1.9935879 ],
  [               0,               0,               0,               0,               0,               0,      0.85818805,      0.67944543,      -1.3941434,      -2.6710375 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0023122936,    0.0011566701,    0.0052852197,    0.0040244189 ],
  [               0,               0,               0,               0,               0,               0,       1.2448642,      -1.5553078,      -1.9294826,       5.5162123 ],
  [               0,               0,               0,               0,               0,               0,     -0.26333626,     0.069037462,      0.51287269,      -1.7169046 ],
  [               0,               0,               0,               0,               0,               0,     0.073539966,     0.083063984,      0.23498129,    -0.051248007 ],
  [               0,               0,               0,               0,               0,               0,  -0.00077353372,     0.001640883,    0.0014933621,  -6.3883181e-05 ],
  [               0,               0,               0,               0,               0,               0,      0.67012456,     -0.33996103,      -1.5880764,       1.4146755 ],
  [               0,               0,               0,               0,               0,               0,     -0.73889189,      0.92331498,      0.18887859,      -2.9764178 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0010571589,     0.007242027 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -42.51206,       60.811478 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       8.7890534,      -13.362133 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.9366907,      -2.4235953 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.56983197,       1.2728214,     -0.97270654,      0.32200623,     -0.04648082,    0.0020039377 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,     0.074240468,     0.037112701,     -0.04481903,      -0.2022637 ],
  [               0,               0,               0,               0,               0,               0,      -16.467657,      -7.5295462,       11.471215,       51.240578 ],
  [               0,               0,               0,               0,               0,               0,      -1.6269747,       5.9728299,       5.7084995,      -16.772416 ],
  [               0,               0,               0,               0,               0,               0,    -0.018630676,    0.0048560545,     0.036878202,     -0.33397858 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele50_layer2_etaoffv5_37_e = [
 [
  [      0.70787459,     -0.82816571,     -0.58914766,      0.33811209,      0.76578587,     0.060949278,     -0.82966269,      0.44545219,    -0.069353669,   -0.0047619073 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0042079611,   -0.0021277363,    0.0030251939,     0.014268177 ],
  [               0,               0,               0,               0,               0,               0,      -1.9142525,        7.620692,       21.031534,       49.495264 ],
  [               0,               0,               0,               0,               0,               0,    -0.054020228,      0.59797535,       1.8569536,       3.7391064 ],
  [               0,               0,               0,               0,               0,               0,      -1.3899383,      -1.9749175,      -2.8169853,      -4.3361342 ],
  [               0,               0,               0,               0,               0,               0,     0.004605938,    0.0024523666,   -0.0028370553,    -0.012775487 ],
  [               0,               0,               0,               0,               0,               0,       4.3294871,       2.1444317,      -3.5248139,      -13.017674 ],
  [               0,               0,               0,               0,               0,               0,       1.7472082,      0.80342025,      -1.3103504,      -5.4348126 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0034602092,    0.0053435799 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       52.210275,      -76.960216 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.8774217,      -4.1083853 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.29613524,     -0.35788539 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0003602543,   0.00054641639,    0.0081793268 ],
  [               0,               0,               0,               0,               0,               0,               0,       5.3638657,      -17.789055,       17.866322 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.72777428,       2.2991586,      -3.0348997 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.73536573,       2.5311612,      -2.0359162 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0024856687,  -0.00036024928,     0.010457836 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.7176997,      -8.5840553,        6.831182 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.7922567,       8.0981469,      -7.6116582 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,     0.003914888,   -0.0042195683,     -0.01270039,     0.018172661 ],
  [               0,               0,               0,               0,               0,               0,      0.56128059,     -0.29264549,      -11.041251,       21.942705 ],
  [               0,               0,               0,               0,               0,               0,     -0.68105707,        5.429531,      -12.750033,       7.8939792 ],
  [               0,               0,               0,               0,               0,               0,      -1.6624529,       5.2252241,      -0.6793473,      -6.0773081 ],
  [               0,               0,               0,               0,               0,               0,    0.0010787545,   -0.0016662773,   -0.0043720571,    0.0079601263 ],
  [               0,               0,               0,               0,               0,               0,       2.1256844,      -4.6848304,      -10.423605,       23.097747 ],
  [               0,               0,               0,               0,               0,               0,       8.1929314,      -16.310942,      -36.079884,       69.622431 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   -0.0009487394,   -0.0011964839,   1.0759074e-05,    0.0076240304,     0.038621341 ],
  [               0,               0,               0,               0,               0,       5.3113639,      -5.9389701,      -29.505962,      -36.623852,       166.18131 ],
  [               0,               0,               0,               0,               0,      0.17681118,     -0.13908845,      -1.2726671,      -2.2883662,       7.7193156 ],
  [               0,               0,               0,               0,               0,     -0.58288779,      0.87548476,       3.7028537,       3.8477803,      -23.539961 ],
  [               0,               0,               0,               0,               0,  -0.00067458929,  -0.00094050729,  -0.00022340038,    0.0052315705,     0.029307928 ],
  [               0,               0,               0,               0,               0,   -0.0033006396,    -0.014602858,    -0.058712762,       -0.231182,       1.1223916 ],
  [               0,               0,               0,               0,               0,      -2.2230085,       3.2299509,       13.011061,       12.367149,      -80.637828 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0033848429,    0.0012407923,     0.027192195 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.7176275,      -0.2189909,       13.453107 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.5447566,     -0.11118648,      -10.293021 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.40542936,     0.049202144,      -2.5112502 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,      -1.0209315,       1.3326462,     -0.39920525,     0.036920041,  -0.00092595059 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]

###############################################################################
# Tables for v5 sshape correction, 75 GeV electron clusters
#


ele75_layer1_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,    -0.031796778,     0.078986766,    -0.062858724,      0.01490535 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00028911223,   0.00062421382 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.52695287,       3.3924629 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.094991605,     -0.52745509 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.64298716,     -0.37407723 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.30209355,   -0.0024883943 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.028030739,    0.0058380281 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -51.332839,       41.375498 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       23.459668,      -19.166268 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       15.901256,      -12.646191 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,      0.32195625,     -0.52625852,      0.21625675 ],
  [               0,               0,               0,               0,               0,               0,               0,       21.053198,      -48.332695,        27.76316 ],
  [               0,               0,               0,               0,               0,               0,               0,       12.320667,      -16.579462,       1.4050173 ],
  [               0,               0,               0,               0,               0,               0,               0,       12.305076,      -16.512514,       1.3423767 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.1872428,      -3.1303033,       2.0714057 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.2063196,      -1.9905177,      0.82849612 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.7840598,       5.3413719,      -2.3393677 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.35322226,      0.93264378,      -2.1892008 ],
 ],
 [
  [               0,               0,               0,               0,     0.062971718,    -0.067664641,    -0.042945015,     0.083846442,    -0.043849571,    0.0033866293 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.017908429,    0.0013580287,     0.069185332,     -0.09479379,     0.046069622,   -0.0047313275 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0029316292,    0.0029046716 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.7762933,      0.55492886 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -18.234901,       25.483593 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -25.78054,       35.147915 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.01332924,     -0.30462331 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer1_etaoffv5_55_e = [
 [
  [               0,               0,               0,               0,      0.33007885,     -0.60649726,      0.33733588,    -0.081400264,     0.012260065,    0.0031127662 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.007094599,    0.0064148993 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.11460569,       1.5865624 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -14.884673,       21.082343 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -6.0491113,       7.9093701 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.13569509,      -0.2855649 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,  -8.6171334e-05,    0.0070735305 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.85391037,       5.2877109 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       35.239369,      -53.870526 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       7.2291406,      -10.658583 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0075145206,     -0.00358194 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.98507475,       2.7171302 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.57462953,      -1.8003654 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.57053668,      -1.7939181 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.041896402,     0.081983211 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0002345587,  -3.4499748e-05,   0.00035403155,    0.0012780211 ],
  [               0,               0,               0,               0,               0,               0,      0.46960317,      0.33834081,      -1.1382329,      -2.8192154 ],
  [               0,               0,               0,               0,               0,               0,       -1.133172,       1.0154223,       3.3774052,    -0.076957638 ],
  [               0,               0,               0,               0,               0,               0,      0.48864921,     -0.57547089,     -0.81631136,       3.9769413 ],
  [               0,               0,               0,               0,               0,               0,      0.37136964,      0.83350323,      0.27607501,      -5.2197971 ],
  [               0,               0,               0,               0,               0,               0,   0.00017632253,   0.00010830966,   -0.0002840134,    -0.000564694 ],
  [               0,               0,               0,               0,               0,               0,     -0.15227601,      -0.1444307,    -0.053634119,       2.8940601 ],
  [               0,               0,               0,               0,               0,               0,    -0.047306783,      0.31880826,      0.17246551,      -2.8144846 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0072418565,    -0.017518166,    -0.039262543,     0.096335413 ],
  [               0,               0,               0,               0,               0,               0,       3.5652592,       -8.460498,      -6.7188307,       20.437233 ],
  [               0,               0,               0,               0,               0,               0,      -0.1665786,      0.67916992,      0.70971781,      -4.0867882 ],
  [               0,               0,               0,               0,               0,               0,      -1.3473235,       1.6569049,       12.863962,      -22.919981 ],
  [               0,               0,               0,               0,               0,               0,       -1.759478,       8.0625175,       15.214489,      -49.152653 ],
  [               0,               0,               0,               0,               0,               0,   0.00049191167,   -0.0021492667,  -0.00012476209,     0.016209444 ],
  [               0,               0,               0,               0,               0,               0,      0.38401966,      -0.8588911,      -1.6539114,       3.7152366 ],
  [               0,               0,               0,               0,               0,               0,       5.6155767,      -12.893354,      -27.869677,         61.0647 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0019634614,    0.0066087178 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -10.001166,       24.144634 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -4.8912018,       10.308481 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -5.6649355,       12.236891 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.82433493,      -2.0641296 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00059586587,    0.0018065143 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -1.280696,       2.9376316 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.1150857,     -0.44405889 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00053158208,    0.0018141184 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.59565585,       3.4614156 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.66776324,      -2.5346129 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.12043223,     -0.67398175 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.32379367,      0.78415677 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,    0.0012105213 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -1.8541445 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,     -0.91985235 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -1.7320277 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -1.0901015 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,    0.0020374234 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      0.59479965 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -1.7133168 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00041169726,  -0.00010326119,    0.0017585772,    0.0092382606 ],
  [               0,               0,               0,               0,               0,               0,      -1.0808377,     -0.49628457,       2.7355817,       17.189978 ],
  [               0,               0,               0,               0,               0,               0,     -0.95363319,        1.932926,       5.0545237,      -11.649796 ],
  [               0,               0,               0,               0,               0,               0,      0.15294478,      0.92519728,      0.72614393,       -9.251862 ],
  [               0,               0,               0,               0,               0,               0,   -0.0014812222,    -0.001258609,    0.0026231058,     0.021153412 ],
  [               0,               0,               0,               0,               0,               0,     -0.52733982,     -0.24236829,       1.1052475,       7.0985394 ],
  [               0,               0,               0,               0,               0,               0,     -0.23431681,     -0.14919517,    -0.078585172,      -1.3842656 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0005419694,     0.010495621,   -0.0046528713 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0014031588,    -0.042622105,       5.7622528 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.038249297,    -0.050091221,       55.863457 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.087115564,    -0.041540994,    0.0061295823 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.035948142,    -0.055969119,      0.69545471 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer2_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0073729805,    0.0028620909,    0.0087835718 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.2148777,      -1.4738693,       3.3229372 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.75037351,     -0.40413424,      -1.2317222 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.43137756,      0.26314291,    -0.029197906 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0056200053,   -0.0024131487,   0.00022680218 ],
  [               0,               0,               0,               0,               0,               0,               0,      -47.654559,        17.69882,      0.89876273 ],
  [               0,               0,               0,               0,               0,               0,               0,      -142.42968,      -2.5214648,      -4.5473859 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0024139631,    0.0029574443,    0.0072324893 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.8687018,       3.5197776,       2.5807803 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.79040664,      0.65765637,      -1.4836108 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.4109999,      -1.1630703,      0.28993216 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.00229068,   -0.0012104414,   0.00034533994 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.2106473,      -1.9663335,       1.2184256 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.2859672,      -3.3923017,     -0.99541476 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,     0.082577143,     0.022763358,    -0.014629765,    -0.035888873 ],
  [               0,               0,               0,               0,               0,               0,      -4.6680741,      -4.3876147,     -0.62291009,       8.6472491 ],
  [               0,               0,               0,               0,               0,               0,       5.1971139,     -0.78886778,      -2.2728478,      -1.6513799 ],
  [               0,               0,               0,               0,               0,               0,      -1.3253289,     -0.73016879,       0.2226444,      0.96090912 ],
  [               0,               0,               0,               0,               0,               0,     -0.01195699,   -0.0050212458,    0.0014311239,    0.0079779431 ],
  [               0,               0,               0,               0,               0,               0,      -13.660309,      -10.005791,     -0.11123754,       14.565593 ],
  [               0,               0,               0,               0,               0,               0,      -26.151009,      -5.4984247,       5.4821823,       11.835454 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   9.8436684e-05,   -0.0004462365,     0.001594325,    0.0074221769 ],
  [               0,               0,               0,               0,               0,               0,        0.910476,      -1.5649238,     -0.96883162,       4.3832724 ],
  [               0,               0,               0,               0,               0,               0,      0.34527383,     -0.94768309,      0.62156343,      -1.3925305 ],
  [               0,               0,               0,               0,               0,               0,      0.20439836,     -0.97510339,       1.8033203,     -0.70793581 ],
  [               0,               0,               0,               0,               0,               0,   0.00023156721,  -9.5592642e-06,  -9.4352379e-06,    0.0015602719 ],
  [               0,               0,               0,               0,               0,               0,     -0.21064611,      0.58019849,     -0.31199509,       0.1650462 ],
  [               0,               0,               0,               0,               0,               0,       0.2212931,     -0.21639608,     -0.58677815,      -2.4182466 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.012549144,    -0.007234555 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -65.916385,       92.116363 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       7.9862038,      -12.279942 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.6594226,      -5.9960958 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0051237012,    0.0054735198,  -0.00030416587,    0.0085545795,   -0.0096251398,    0.0008135174 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0044296666,   -0.0027789591,    0.0068360893,     0.018768087 ],
  [               0,               0,               0,               0,               0,               0,     0.039121044,     -0.13429063,   0.00059083295,       20.604966 ],
  [               0,               0,               0,               0,               0,               0,    -0.065364064,    -0.040921785,     0.011686246,      -1.9233027 ],
  [               0,               0,               0,               0,               0,               0,    -0.072748226,    0.0020521427,    0.0071969156,      0.77742717 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer2_etaoffv5_55_e = [
 [
  [    -0.053165201,     0.014931821,     0.042001106,     0.032576968,  -0.00030093947,    -0.031941522,     -0.02677496,     0.034297277,    -0.013304036,   -0.0022131064 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00057760415,  -0.00032120893,   0.00031831828,    0.0021507533 ],
  [               0,               0,               0,               0,               0,               0,       1.5878285,        4.192987,       5.7680796,       11.905105 ],
  [               0,               0,               0,               0,               0,               0,       1.3241406,       1.4140682,        1.283771,      0.30105906 ],
  [               0,               0,               0,               0,               0,               0,      -2.4733171,      -3.9773113,      -6.3576532,      -10.410044 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0014919312,     0.005199435 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       56.379081,      -82.967567 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -4.2731728,       5.2742727 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.7508429,       4.2320129 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0083714142,     0.015197739,     0.019853341,    -0.026780273 ],
  [               0,               0,               0,               0,               0,               0,       2.5832546,      -3.5051096,      -10.783122,       18.174482 ],
  [               0,               0,               0,               0,               0,               0,     -0.28022844,      0.55495239,      0.58582019,      -2.3749689 ],
  [               0,               0,               0,               0,               0,               0,     0.052709441,      -0.1114772,    -0.068567508,       0.2044086 ],
  [               0,               0,               0,               0,               0,               0,   -0.0012203625,    0.0029693949,    0.0016314542,   -0.0051483383 ],
  [               0,               0,               0,               0,               0,               0,      -28.092409,       49.564039,       95.733085,       -167.9658 ],
  [               0,               0,               0,               0,               0,               0,       2.7655347,      -4.9272591,      -7.4064459,       11.650768 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0088718626,    -0.013506104,   5.3991278e-06 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.84674877,      0.41064235,        4.990686 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.75589568,      -2.3693244,      0.50507598 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.79717383,       3.0762262,      -2.8781132 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0012760436,   -0.0082970763,      0.01206488 ],
  [               0,               0,               0,               0,               0,               0,               0,        9.624617,      -43.088004,       48.292257 ],
  [               0,               0,               0,               0,               0,               0,               0,       20.430056,      -85.842312,       87.815608 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0017690823,  -1.7134373e-05,     0.011260139 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.2481149,      -1.2570214,       14.192918 ],
  [               0,               0,               0,               0,               0,               0,               0,       -1.825968,       1.7938343,       4.5363599 ],
  [               0,               0,               0,               0,               0,               0,               0,        1.589941,       1.5484737,      -12.245846 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0003359607,   6.3661264e-06,   0.00014958909 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.030139192,     -0.11616157,       1.0721272 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.30901608,      0.95893324,      -2.1786561 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0054888968,   -0.0015517939 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -30.304281,         76.5726 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       8.3305098,      -21.766651 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.8910244,      -4.4001935 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,       -2.505777,       2.0641906,     -0.42105048,     0.024027874,     0.001253565 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer1_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,    -0.017852535,     0.046760055,     -0.03849489,    0.0088283471 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00031478972,   0.00067037272 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.60448134,       3.2945435 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.11169808,     -0.60074839 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.50163613,     -0.49734559 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.27416122,   -0.0056765926 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.022497629,    -0.010519209 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -292.48073,        233.4081 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       120.34654,      -96.365338 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       68.433393,      -54.429706 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0084127302,    -0.022883893,     0.015684002 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.5498991,      -9.2242505,       7.5359745 ],
  [               0,               0,               0,               0,               0,               0,               0,       5.8535944,      -3.0895164,      -2.3954969 ],
  [               0,               0,               0,               0,               0,               0,               0,       5.8413047,      -4.2236061,      -1.3725279 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.7162299,       6.5441438,      -2.9296216 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0017868114,    -0.003982671,    0.0026960378 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.8765653,       6.5353919,      -2.5593187 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.2314833,       2.8150545,      -3.2980815 ],
 ],
 [
  [               0,               0,               0,               0,     0.038061151,    -0.035637489,    -0.032169389,     0.052008341,    -0.029021075,    0.0013663736 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.31613172,        0.776932,     -0.64495061,      0.17560843,     0.013310456,   -0.0060783479 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0033707933,    0.0028125229 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.3850557,     -0.52489641 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -23.382052,       32.845729 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -33.327644,        45.94205 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.01384668,     -0.28707278 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer1_etaoffv5_35_e = [
 [
  [               0,               0,               0,               0,     -0.02485679,     0.022262426,    0.0069871524,    0.0051274797,     -0.01229943,   -0.0025385988 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0075963237,    0.0016068102 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.7577295,       4.4428681 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -13.050913,       18.180867 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -26.054169,       36.846715 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.075127348,     -0.22718294 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.072235305,      0.11112761 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -166.1351,       252.62547 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -68.598377,       100.61319 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       123.87577,      -184.07232 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.81818359,     -0.39609123 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.28508392,      0.68624187 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.28035816,      -0.6762364 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.28091774,     -0.67719554 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00033869711,    0.0006749102 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -1.4642209e-05,  -6.9213319e-06,  -8.9505707e-06,   0.00042333226 ],
  [               0,               0,               0,               0,               0,               0,        0.238682,      0.17999339,     -0.22368924,       2.4231837 ],
  [               0,               0,               0,               0,               0,               0,     -0.26551611,     -0.11085272,      0.41042187,       1.3033988 ],
  [               0,               0,               0,               0,               0,               0,      0.17671738,      0.15575615,     0.026235577,      -1.3181885 ],
  [               0,               0,               0,               0,               0,               0,        0.014286,     0.056765082,      0.12112627,     -0.12772279 ],
  [               0,               0,               0,               0,               0,               0,   2.1868786e-05,  -1.1928066e-05,  -0.00013474323,   0.00051333983 ],
  [               0,               0,               0,               0,               0,               0,     0.016311122,     0.072618501,     -0.16606169,       1.0655919 ],
  [               0,               0,               0,               0,               0,               0,    0.0026544186,    -0.046226929,     -0.26462992,     -0.99332382 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0042211656,    -0.010689191,    -0.023207233,     0.060488174 ],
  [               0,               0,               0,               0,               0,               0,         0.97802,        -1.33432,      -3.7960618,       6.8421792 ],
  [               0,               0,               0,               0,               0,               0,      0.84414149,      -2.0218476,      -3.3525275,       6.9477824 ],
  [               0,               0,               0,               0,               0,               0,     -0.26556268,      0.67093747,      0.75161755,      -3.1933005 ],
  [               0,               0,               0,               0,               0,               0,     -0.25836436,       1.0023671,       2.1079993,      -5.5921865 ],
  [               0,               0,               0,               0,               0,               0,  -0.00070298004,   0.00064154953,    0.0028077552,    0.0016807299 ],
  [               0,               0,               0,               0,               0,               0,      0.14601351,     -0.20654456,     -0.88142121,       1.4653219 ],
  [               0,               0,               0,               0,               0,               0,     0.094784211,     -0.38752489,     -0.65248148,     -0.25066806 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0019457195,     0.006741003 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -10.458307,        25.10555 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -5.2884419,       11.133992 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -5.2739091,       11.389283 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.4761146,     -0.66120806 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00072354858,    0.0020744641 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.6215941,       4.0036864 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.39327514,      -1.9042076 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00052912984,    0.0018150934 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.58784446,       3.4439091 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.65183307,      -2.4968226 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.12818808,     -0.65427966 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.3216895,      0.77928665 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,   0.00058781209 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -2.0020602 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -1.5982441 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -3.1924456 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -2.2612692 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,    0.0013177633 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      0.62476498 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -1.7829062 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00068025343,  -0.00031972138,     0.002288454,     0.013080387 ],
  [               0,               0,               0,               0,               0,               0,     -0.87462483,     -0.25523572,       2.4171375,       13.554123 ],
  [               0,               0,               0,               0,               0,               0,     -0.97558669,       1.9554821,       5.1416639,      -11.723081 ],
  [               0,               0,               0,               0,               0,               0,        0.103575,      0.94926795,      0.90194645,      -9.1074058 ],
  [               0,               0,               0,               0,               0,               0,    0.0012504791,    0.0010119071,   -0.0024475531,    -0.016816451 ],
  [               0,               0,               0,               0,               0,               0,     -0.56535421,     -0.26475236,       1.1698478,       7.7164636 ],
  [               0,               0,               0,               0,               0,               0,     0.042249767,     -0.15897945,      -3.1034233,      -19.977163 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0039100701,    0.0051698253,    0.0020464519 ],
  [               0,               0,               0,               0,               0,               0,               0,        3.838806,     -0.02331955,      -19.677421 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,            -0.4 ],
  [               0,               0,               0,               0,               0,               0,               0,      -9.1003773,    -0.091697916,       55.125196 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.042848899,      0.10630795,    -0.079425018 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer2_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0072169097,   -0.0029653776,    0.0075800462 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.8151934,       1.5842714,       4.1867991 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.4494148,       -1.151544,      -1.2193879 ],
  [               0,               0,               0,               0,               0,               0,               0,       11.497996,       -4.988719,    -0.063313187 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.11005596,     0.048230394,   3.4642919e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.6034408,      -2.1078725,      0.31103268 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.1962206,      -1.8088079,     -0.39786045 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    -0.003257935,    0.0038390979,    0.0063149883 ],
  [               0,               0,               0,               0,               0,               0,               0,      -4.2795836,       3.7556924,       3.4576778 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.2190644,        1.139327,      -1.6534969 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.1835359,     -0.70978185,      0.17967396 ],
  [               0,               0,               0,               0,               0,               0,               0,  -0.00096428084,  -0.00083735447,  -0.00066333694 ],
  [               0,               0,               0,               0,               0,               0,               0,       0.9000973,     -0.87412505,       0.4219739 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.77641168,      -2.7690093,       -4.121669 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0071769701,    0.0025273364 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       18.404516,      -11.076431 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       6.1411046,      -6.2431449 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -5.0765238,        4.130359 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.3459144,      0.28892246 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.12196291,    -0.085402637 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.6233093,      0.45511642 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,  -0.00074428153,    0.0031868555,    0.0048831941 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.2002961,      -8.5711096,       8.9894412 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.24030968,      -0.7192615,     -0.99459168 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.23318079,      0.89881181,      -0.5126827 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00031802365,  -0.00024641108,   0.00061486697 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.93185525,       1.8534238,     -0.26130624 ],
  [               0,               0,               0,               0,               0,               0,               0,      -0.5635741,      0.24613324,      -1.8439585 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0012341659,     0.010850886 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -54.722631,       77.498916 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       9.1844669,      -13.878968 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        1.470543,       -1.820323 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.32015934,       0.8001066,     -0.66099227,      0.23474092,    -0.037590987,    0.0015414259 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0091247251,    0.0077793492 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -219.35607,       319.06115 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       16.022562,      -23.470868 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.5072752,       -2.722072 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer2_etaoffv5_35_e = [
 [
  [      0.18748966,     -0.24676302,     -0.17301709,      0.11954149,      0.25831481,     0.010812578,     -0.30011704,      0.17345409,    -0.026154232,    -0.006995373 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,     -0.16892624,    -0.078229057,      0.13231877,      0.54921766 ],
  [               0,               0,               0,               0,               0,               0,         8.95307,       7.4890859,       13.363976,      -64.075396 ],
  [               0,               0,               0,               0,               0,               0,    -0.060936416,      -7.8262921,      -13.686316,       36.571281 ],
  [               0,               0,               0,               0,               0,               0,     -0.30101762,      -1.8991282,      -1.6970117,       7.7638409 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0061166983,   -0.0021321038 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       103.72699,       -153.8863 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.9852651,      -10.406383 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.3203389,       5.1453703 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00070176833,   -0.0042168935,   -0.0047970121,     0.023538132 ],
  [               0,               0,               0,               0,               0,               0,       3.9231481,      -5.1720892,      -12.705973,       20.764361 ],
  [               0,               0,               0,               0,               0,               0,       1.3890847,      -3.9008201,        1.225231,       1.0326056 ],
  [               0,               0,               0,               0,               0,               0,     -0.10845822,     -0.87778269,       3.8188899,      -3.3258385 ],
  [               0,               0,               0,               0,               0,               0,    0.0015486112,   -0.0017446136,   -0.0046202837,    0.0061587181 ],
  [               0,               0,               0,               0,               0,               0,      -2.3424498,       5.9473291,       1.7045464,      -7.7568639 ],
  [               0,               0,               0,               0,               0,               0,      0.11405924,      -0.1768995,       2.1659582,      -5.6169404 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0017594007,   0.00069241379,  -0.00062956342 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.26119851,      -2.1036937,       8.4171793 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.68702719,       -2.393002,      0.71943738 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.91442537,       3.7058219,      -3.6400495 ],
  [               0,               0,               0,               0,               0,               0,               0,   3.5551652e-05,  -0.00022415772,    0.0010878068 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.23861997,      -1.6580951,       2.9983762 ],
  [               0,               0,               0,               0,               0,               0,               0,      -0.5164923,        1.598261,      -3.1686474 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   -0.0011816283,   -0.0016646253,  -0.00044352997,    0.0091535867,      0.05083069 ],
  [               0,               0,               0,               0,               0,      0.36025498,     -0.17729454,      -1.4848807,      -2.6535919,       8.0256608 ],
  [               0,               0,               0,               0,               0,      -0.4751538,      0.43727123,       2.4148529,        3.191377,      -13.168424 ],
  [               0,               0,               0,               0,               0,     -0.34081785,      0.47011296,       2.1073392,       2.3810785,      -12.883193 ],
  [               0,               0,               0,               0,               0,    0.0002585632,   0.00029216427,  -6.1609279e-05,   -0.0018108952,   -0.0066891077 ],
  [               0,               0,               0,               0,               0,      0.10145204,     0.056974725,     -0.16959503,     -0.85185433,     -0.51838205 ],
  [               0,               0,               0,               0,               0,     0.064323927,      0.41996298,      0.71497856,     -0.89147409,      -12.298756 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0010925263,    0.0090088285 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.84322408,      0.21964891 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.7483134,      -12.885294 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.21419274,      -0.4641619 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     -0.65768095,       1.0968418,      -0.3661422,     0.037178022,   -0.0011954002 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer1_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0050733406,   -0.0068584147,    0.0010246857 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00031542272,   0.00067682156 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.57899192,        3.264722 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.11815821,     -0.60530342 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.51126748,     -0.49688148 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.27849949,   -0.0051445719 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.4683776,      -1.1438138 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -54.737638,       44.126438 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,         24.0923,       -19.69887 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       16.310418,      -12.972652 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00096516742,   -0.0018021167,   0.00085415142 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.87881486,       7.9583226,      -7.5984836 ],
  [               0,               0,               0,               0,               0,               0,               0,       46.705793,       -108.8103,       57.754276 ],
  [               0,               0,               0,               0,               0,               0,               0,       36.254648,      -100.53463,       66.969128 ],
  [               0,               0,               0,               0,               0,               0,               0,      -32.866655,       86.047459,      -55.889608 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00071491892,  -0.00099875756,    0.0008983788 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.3917977,       2.6434224,     -0.22625852 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.46729628,      0.86869587,      -2.2072961 ],
 ],
 [
  [               0,               0,               0,               0,     0.049458687,    -0.053919675,    -0.031940017,     0.064386065,     -0.03555247,      0.00233569 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.30923848,       0.7607345,     -0.63257485,      0.17232886,     0.013426639,   -0.0060470916 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0033645035,    0.0028823308 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.2470147,     -0.35326418 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -22.521703,       31.596862 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -31.090066,       42.738115 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.015587746,     -0.28158707 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer1_etaoffv5_37_e = [
 [
  [               0,               0,               0,               0,     -0.02587598,     0.022094379,    0.0080767573,    0.0036275698,     -0.01170937,   -0.0029165129 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.005089571,    0.0017194428 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.22244877,       2.6145241 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -12.688288,       17.582538 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -32.196977,       45.755205 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.047462197,     -0.23545266 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.17881331,      0.27196448 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.30166248,       5.1008009 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       35.537952,      -54.257876 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       8.4967387,      -12.510837 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.3633355,    0.0011037989 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.20639565,      0.50406781 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.20453621,     -0.50004949 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.20478457,     -0.50048575 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0001356877,   0.00026966522 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00016908707,  -2.2461296e-05,   0.00023698456,   0.00094279208 ],
  [               0,               0,               0,               0,               0,               0,      0.19290709,       0.5043329,      0.18675536,        1.720571 ],
  [               0,               0,               0,               0,               0,               0,     -0.69478422,      0.88930477,       2.1594134,      -2.5834572 ],
  [               0,               0,               0,               0,               0,               0,       1.2092707,     -0.16474889,      -1.9901166,      -2.1881568 ],
  [               0,               0,               0,               0,               0,               0,      0.66937799,      0.97831772,    -0.086921856,      -6.7977982 ],
  [               0,               0,               0,               0,               0,               0,   0.00020469405,   0.00012049727,  -0.00032624356,  -0.00073952225 ],
  [               0,               0,               0,               0,               0,               0,    -0.025598678,    -0.049761224,     -0.21356585,       1.8480254 ],
  [               0,               0,               0,               0,               0,               0,    -0.033932377,      0.30032808,      0.12916177,      -2.7611359 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,     0.005573362,    -0.012948953,    -0.029612027,      0.06991285 ],
  [               0,               0,               0,               0,               0,               0,       2.7139515,      -4.5556821,      -10.692929,       20.046847 ],
  [               0,               0,               0,               0,               0,               0,       2.4217163,      -5.0518496,      -9.5931568,       18.982838 ],
  [               0,               0,               0,               0,               0,               0,      0.31615471,     -0.12296638,      -1.4149976,     -0.33428841 ],
  [               0,               0,               0,               0,               0,               0,      -1.4150838,         2.98761,       7.0836286,      -14.875211 ],
  [               0,               0,               0,               0,               0,               0,    0.0027953556,   -0.0063248676,    -0.012625567,      0.02854386 ],
  [               0,               0,               0,               0,               0,               0,       5.1890642,      -12.255377,       -25.31912,       58.325847 ],
  [               0,               0,               0,               0,               0,               0,      -34.800352,       74.369724,       183.66488,      -394.65805 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0014514817,    0.0055958779 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -13.18524,       31.571077 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -6.4103165,       13.749856 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -7.331445,        16.04909 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        1.059856,      -2.5691392 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00045158798,    0.0013747772 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.0612895,       2.4361371 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.16971577,     -0.41620134 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00048331164,    0.0017355788 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.0866604,       4.6677714 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.2287075,      -3.8316039 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.13161014,     -0.63774407 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.47685426,       1.1364441 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,    0.0015311319 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       3.8667601 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -1.6596511 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -2.4372158 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -1.0959524 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,  -0.00045309633 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       1.6673843 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,        -1.57651 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0013491623,    0.0018298286,   -0.0010518122,    -0.019864872 ],
  [               0,               0,               0,               0,               0,               0,      -2.4188507,     -0.13223438,       7.0985552,       23.120992 ],
  [               0,               0,               0,               0,               0,               0,      -0.2642274,      0.68101444,       1.5188418,      -5.2841539 ],
  [               0,               0,               0,               0,               0,               0,      0.40151262,      0.38670158,     -0.80660769,      -7.2800977 ],
  [               0,               0,               0,               0,               0,               0,     0.009960392,    0.0094325271,    -0.015257453,     -0.13463598 ],
  [               0,               0,               0,               0,               0,               0,      0.23636659,      -0.5478456,       -1.654451,       3.8683324 ],
  [               0,               0,               0,               0,               0,               0,     -0.10584307,     -0.14294121,     -0.12911812,       1.4530093 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0013242588,     0.003735984,    0.0073266502 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0095480925,   -0.0048914643,       5.7965953 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.036122556,    -0.066916071,       38.968098 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.072077625,    -0.036180861,    -0.072388076 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.028895238,    -0.047580768,       1.0978973 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer2_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0087854587,   -0.0036556241,    0.0076385869 ],
  [               0,               0,               0,               0,               0,               0,               0,      -4.5088441,       1.8965513,        4.155662 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.4980384,      -1.1767865,       -1.215699 ],
  [               0,               0,               0,               0,               0,               0,               0,       10.354937,      -4.4791242,    -0.059451061 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.10211309,     0.044702098,   5.5222429e-06 ],
  [               0,               0,               0,               0,               0,               0,               0,        4.897749,      -2.2426672,      0.33239367 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.3387662,       -1.880304,     -0.42486001 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0030470665,    0.0057008854 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,         -2.7204,       5.7134124 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.13800732,      -1.3322854 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.02862109,      0.58135392 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0029558651,    0.0065236257 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.22151709,    -0.020803654 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -1.275095,      -8.1790415 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0061567028,    0.0031861577,    0.0016650578 ],
  [               0,               0,               0,               0,               0,               0,               0,       14.242526,     -0.38617335,      -5.1520893 ],
  [               0,               0,               0,               0,               0,               0,               0,        8.156032,      -1.7181966,      -5.2756157 ],
  [               0,               0,               0,               0,               0,               0,               0,      -4.2936601,     0.060871063,       3.1476101 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0035881425,   -0.0020227011,   6.0289862e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.4858613,     -0.56307495,       2.9026812 ],
  [               0,               0,               0,               0,               0,               0,               0,       5.3352623,      -3.7179915,      -6.5003036 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00050130955,   0.00021340389,     0.002215468,    0.0061239284 ],
  [               0,               0,               0,               0,               0,               0,       1.2403298,      -2.0903707,      -1.2041377,       5.5109476 ],
  [               0,               0,               0,               0,               0,               0,    -0.031512645,    -0.041640669,    -0.083474975,      -1.2657281 ],
  [               0,               0,               0,               0,               0,               0,    -0.050437217,      0.27766805,      0.30122253,     -0.15199871 ],
  [               0,               0,               0,               0,               0,               0,   0.00086774217,    9.818057e-05,  -0.00010119273,    0.0016697354 ],
  [               0,               0,               0,               0,               0,               0,    -0.070818236,      0.26417042,     -0.16409871,       0.1652257 ],
  [               0,               0,               0,               0,               0,               0,      0.30362164,     -0.33270563,      -1.0482681,      -1.6380893 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00067278337,     0.010301907 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -52.330562,       74.180035 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       9.2671847,      -13.992294 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.3406678,       -1.638618 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.55177207,       1.2977713,      -1.0427672,      0.36125136,    -0.054154413,    0.0021360075 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,      0.14822227,    0.0094900001,     -0.28030058 ],
  [               0,               0,               0,               0,               0,               0,               0,      -13.371178,      0.12055643,       30.782652 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.9203667,       5.0600076,      -14.090266 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0079668156,     0.039086182,     -0.31789983 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele75_layer2_etaoffv5_37_e = [
 [
  [      0.35901905,     -0.45073107,     -0.30060103,      0.20768091,      0.41878779,    0.0061937673,     -0.45841571,       0.2647533,    -0.043013417,   -0.0069489424 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0010639885,  -0.00079600052,   0.00063969037,    0.0051386673 ],
  [               0,               0,               0,               0,               0,               0,       17.079959,       10.555465,      -10.052112,      -48.447815 ],
  [               0,               0,               0,               0,               0,               0,       -47.85969,      -21.088218,       36.359983,       144.31693 ],
  [               0,               0,               0,               0,               0,               0,       46.062225,       24.341311,      -32.643883,      -155.90953 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0051398418,  -0.00094686311 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       118.82688,      -176.61752 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        8.501649,      -14.242367 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.5953995,       4.0524951 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00012635371,  -9.2629909e-06,   0.00064036761,    0.0046996839 ],
  [               0,               0,               0,               0,               0,               0,       1.4727328,      -2.5381494,       -4.761053,       11.795243 ],
  [               0,               0,               0,               0,               0,               0,    -0.072249589,      0.10888338,      0.30246988,      -1.7619682 ],
  [               0,               0,               0,               0,               0,               0,     0.042759501,     -0.12621715,     0.032503041,      0.16314162 ],
  [               0,               0,               0,               0,               0,               0,   0.00010215003,  -0.00021217189,  -0.00036674421,    0.0016523243 ],
  [               0,               0,               0,               0,               0,               0,      0.50563737,     -0.88361615,      -1.3754292,       3.2035574 ],
  [               0,               0,               0,               0,               0,               0,      0.23864569,     -0.35688661,     -0.78690011,     -0.47323379 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0091010605,     0.044013977,    -0.042665509 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.0148483,      -15.280699,        21.46551 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.36458965,      -1.0727456,      -0.5629681 ],
  [               0,               0,               0,               0,               0,               0,               0,       -1.765441,        2.372617,       2.9514523 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.045203042,     -0.18852867,       0.2037691 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.4310219,      -6.0151516,       6.3510207 ],
  [               0,               0,               0,               0,               0,               0,               0,        4.498927,      -12.194427,       4.0563399 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   -5.522341e-05,   0.00011281132,   0.00093250035,    0.0039963507,     0.014704825 ],
  [               0,               0,               0,               0,               0,      0.76559619,     -0.89766992,      -4.3759295,      -5.4946158,       27.266136 ],
  [               0,               0,               0,               0,               0,     0.051992537,     -0.10531556,     -0.23904144,    -0.027555122,     0.086551223 ],
  [               0,               0,               0,               0,               0,     -0.19831965,      0.21815875,       1.1679117,        1.566752,      -6.9062837 ],
  [               0,               0,               0,               0,               0,   1.4235607e-05,   7.2162371e-05,   0.00016730106,   0.00015462276,   0.00022469429 ],
  [               0,               0,               0,               0,               0,     0.038099853,    -0.029200536,     -0.18114093,     -0.34514458,       2.0965743 ],
  [               0,               0,               0,               0,               0,     -0.07963895,      0.15303215,      0.43177075,     -0.21561585,      -6.1199154 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0010998031,    0.0091732746 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.0380484,     -0.27150255 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.0660755,      -13.663023 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.22173873,     -0.48668642 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,      -1.1515893,       1.5191608,     -0.48381973,     0.050630105,   -0.0016901851 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]


###############################################################################
# Tables for v5 sshape correction, 100 GeV electron clusters
#


ele100_layer1_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,     0.013702194,     -0.02736868,     0.018535378,    -0.005479122 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00031961863,   0.00072953396 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.40217889,       3.1978022 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.10143628,     -0.69312791 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.54092834,     -0.58093321 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.2682761,  -0.00022234359 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.3878817,      0.30889369 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       16.129613,      -14.233171 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -300.61283,       239.40222 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -18.840852,        14.65459 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.1053011,    -0.083190168 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       82.411457,      -64.693415 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        408.9075,      -326.61723 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0017545215,    0.0039858375,    0.0044666496 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.14902302,      0.46083605,      0.85389293 ],
  [               0,               0,               0,               0,               0,               0,               0,       -1.681217,       3.3521674,       -2.486939 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.80346313,      -1.6927112,    -0.039298302 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.0029416,     -0.35289909,      -1.4941974 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.011629513,     0.001466804,    -0.011803823 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.0932326,       4.3068224,       -2.188255 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.8565752,      -7.0521055,       -5.162976 ],
 ],
 [
  [               0,               0,               0,               0,     -0.19579339,      0.66570963,     -0.81300758,      0.44989967,     -0.12047849,    0.0088707981 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.040808989,      0.07382169,   -0.0066704474,     -0.06472115,     0.042619447,   -0.0047625936 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0037085714,    0.0030625818 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.6605001,      -1.1053525 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -24.476873,       34.423666 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -43.099006,       59.888183 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.023818185,     -0.27380541 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer1_etaoffv5_55_e = [
 [
  [               0,               0,               0,               0,      0.44734884,     -0.93342433,       0.6317229,     -0.17241297,     0.019216671,    0.0036218687 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00036118587,     0.001123443 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.59074055,       3.2721665 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -10.737949,       -8.660684 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -10.20789,       -12.33618 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.3710793,      -2.2965792 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.019165138,     0.024201871 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.13676655,      0.09355126 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.0515045,      0.07479526 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.078585938,      0.12452329 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       21.532105,      -27.935547 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        25.18805,      -38.842503 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.6705494,       -8.359397 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0047709683,   -0.0020717691 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.84629939,       2.6841602 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.36905009,      -1.5389636 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.27167951,      -1.3689328 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.06625601,      0.13126892 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0003142955,   9.8022171e-05,   0.00062638275,   0.00085955038 ],
  [               0,               0,               0,               0,               0,               0,      0.70984342,     -0.59817429,      -1.6082798,       5.3272947 ],
  [               0,               0,               0,               0,               0,               0,    -0.034108871,      -1.2603924,      -1.1103438,        5.467645 ],
  [               0,               0,               0,               0,               0,               0,       1.0054496,       0.6470739,      -1.0195379,      -6.6489776 ],
  [               0,               0,               0,               0,               0,               0,      0.68370012,      0.34314085,     -0.71411118,      -3.6482545 ],
  [               0,               0,               0,               0,               0,               0,    0.0001972527,   7.1615372e-05,  -0.00035782435,   -0.0004354778 ],
  [               0,               0,               0,               0,               0,               0,       0.1595906,     -0.15505936,     -0.65303977,        1.739213 ],
  [               0,               0,               0,               0,               0,               0,   -0.0062215132,      0.30198232,     0.080431713,      -2.9127756 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0041397596,    -0.010466379,    -0.019101191,     0.051176712 ],
  [               0,               0,               0,               0,               0,               0,      0.36020926,     -0.16774649,      -1.2074995,         2.18906 ],
  [               0,               0,               0,               0,               0,               0,     0.037557892,     -0.32250742,      0.53951261,      -1.1210607 ],
  [               0,               0,               0,               0,               0,               0,      -1.2619046,       2.9052945,       4.9253794,      -12.478003 ],
  [               0,               0,               0,               0,               0,               0,     -0.72365246,       1.6171857,        3.773787,      -8.3126778 ],
  [               0,               0,               0,               0,               0,               0,    0.0018946923,   -0.0029894648,   -0.0090506729,     0.014561019 ],
  [               0,               0,               0,               0,               0,               0,     -0.13291297,      0.29605918,      0.29872626,     -0.55209695 ],
  [               0,               0,               0,               0,               0,               0,      0.33464589,     -0.27989909,     -0.80084454,      -2.4500971 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0023144929,    0.0012650995,       0.0184392 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.3489601,     -0.38565848,       14.791273 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.1711704,     -0.10295698,       10.592557 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.98738852,     0.038725351,        4.437266 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0078809798,     0.073901445,     -0.00479959 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00031988762,  -2.2682287e-05,  -0.00079608816 ],
  [               0,               0,               0,               0,               0,               0,               0,      -8.2702522,     -0.42378458,       43.859464 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.8099651,     -0.30297712,       8.1555293 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00042145247,    0.0017584299 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.66112064,       3.9338127 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.74441272,      0.73536349 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.051596411,     -0.89931728 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.03673434,      0.11412445 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00022444093,   0.00051013573,    0.0016559732 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.19097846,      0.73919092,      -2.0517437 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.64565385,    -0.034223302,      -4.3241435 ],
  [               0,               0,               0,               0,               0,               0,               0,        18.55605,   -0.0025041985,      -107.42805 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0031764173,   -0.0097611912,     -0.32263997 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0038245083,    0.0028304585,     0.023430906 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.7084102,       1.5248898,       17.256265 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.0423683,       2.1891483,     -0.72111994 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.1099831,      0.77786573,      -8.3440998 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0039136298,  -0.00045829169,    -0.021258059 ],
  [               0,               0,               0,               0,               0,               0,               0,       -2.667493,      0.62799059,       15.189465 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.71656675,       -6.685668,      -53.390629 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0011336246,    0.0081453635,    -0.003089907 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.035274966,     -0.05602645,       6.7089062 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.11711705,     0.024868036,      0.00231744 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.014976277,     0.089279315,      -0.7264197 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.066050607,      0.02813464,       1.1602991 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer2_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   3.8552553e-05,    0.0088842702 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.057561125,       3.2852579 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.10420378,      -1.2590394 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.078220353,    -0.017026286 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0050409336,    0.0018943831 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.21972554,      0.53824741 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -125.70121,      -3.1726534 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0053416922,    0.0060180981 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.3873645,       4.5167898 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.031111628,      -1.3316142 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.20814411,     -0.07765029 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   2.9101313e-05,  -8.8900736e-06 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -12.452308,     -0.95249358 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.1731778,      -7.1516761 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.030132173,    -0.018320377 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       29.476583,      -20.012769 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       7.2437147,      -7.1343391 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -7.730908,       6.1505955 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0025556816,    0.0029125196 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -7.1816755,       6.7388528 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.81960745,      -0.9063851 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00018350138,   -0.0009775227,    0.0013764615,    0.0081701662 ],
  [               0,               0,               0,               0,               0,               0,       1.1449258,      -2.2952281,      0.12708525,       3.8344652 ],
  [               0,               0,               0,               0,               0,               0,      0.19220145,     -0.40971277,     -0.06107477,      -1.0954385 ],
  [               0,               0,               0,               0,               0,               0,      0.34989694,      -1.2935839,       2.0825939,     -0.92616028 ],
  [               0,               0,               0,               0,               0,               0,   0.00044471112,   6.5896887e-05,  -0.00020186948,   0.00059298628 ],
  [               0,               0,               0,               0,               0,               0,     -0.34404976,      0.70892303,     -0.22115296,      0.16877752 ],
  [               0,               0,               0,               0,               0,               0,     0.056526056,     -0.15498198,     -0.19844294,      -2.6390614 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.011554211,   -0.0053762142 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -67.469669,       94.260312 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       10.233597,      -15.327735 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.5407762,      -3.1387689 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.015507537,     0.017821715,    0.0046375227,   -0.0012138306,   -0.0065859361,   0.00040190384 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,     0.001307198,     0.010640379,    0.0048656722,    0.0045331971 ],
  [               0,               0,               0,               0,               0,               0,    0.0022924784,     0.065057435,     0.019347211,       24.600814 ],
  [               0,               0,               0,               0,               0,               0,    -0.026663753,    -0.071458773,     -0.21298753,      -1.7320767 ],
  [               0,               0,               0,               0,               0,               0,    -0.011252327,    -0.026751817,    -0.043750963,      0.79141339 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer2_etaoffv5_55_e = [
 [
  [    -0.074866663,     0.025782841,     0.063091224,     0.044962951,   -0.0068606482,    -0.052306071,    -0.038501164,      0.04886493,   -0.0093132607,    -0.003883713 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00015541331,   0.00021549846,   0.00029754038,   0.00090910449 ],
  [               0,               0,               0,               0,               0,               0,        27.14179,       15.711992,       -18.00488,      -92.899866 ],
  [               0,               0,               0,               0,               0,               0,      -1.3192489,     -0.66443224,      0.79965349,        3.237658 ],
  [               0,               0,               0,               0,               0,               0,       2.2538893,       1.1557981,      -1.6001826,      -7.6930018 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0018271569,    0.0047890266 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       43.479671,      -63.440265 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.2569463,       2.2336066 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.3606313,       2.1419588 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0053659512,    0.0070461294,     0.015073566,   -0.0092578931 ],
  [               0,               0,               0,               0,               0,               0,       2.1638671,      -2.8349388,       -8.684568,       14.814417 ],
  [               0,               0,               0,               0,               0,               0,     -0.37484943,      0.85020155,      0.25959684,      -2.2214908 ],
  [               0,               0,               0,               0,               0,               0,      0.10045313,    -0.098666074,     -0.46759385,       0.6123021 ],
  [               0,               0,               0,               0,               0,               0,    0.0022038241,   -0.0030723246,    -0.005997236,    0.0080849401 ],
  [               0,               0,               0,               0,               0,               0,      -2.8829038,       3.8843286,       9.3979652,      -11.981865 ],
  [               0,               0,               0,               0,               0,               0,      0.41143146,     -0.35322809,      -2.3536521,       1.3361291 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0029120553,   -0.0022668417,   -0.0081791728,    0.0096440899 ],
  [               0,               0,               0,               0,               0,               0,      -0.2128016,      0.69020799,      -3.7237258,       9.0350348 ],
  [               0,               0,               0,               0,               0,               0,     -0.12935509,       1.8278862,      -5.0315382,       2.5055527 ],
  [               0,               0,               0,               0,               0,               0,     -0.44303307,       1.6598307,      -1.4362875,     -0.10186255 ],
  [               0,               0,               0,               0,               0,               0,   0.00052743603,   -0.0014769253,   -0.0029773938,    0.0083608414 ],
  [               0,               0,               0,               0,               0,               0,       2.3659317,      -5.1842081,      -10.560622,       23.634296 ],
  [               0,               0,               0,               0,               0,               0,       4.3084893,      -8.7637303,      -20.244361,       38.881728 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -0.00052138284,  -0.00069305046,  -3.1875916e-06,    0.0047228189,     0.024911637 ],
  [               0,               0,               0,               0,               0,       3.0684569,      -3.4963641,       -17.16917,      -21.166516,       98.304476 ],
  [               0,               0,               0,               0,               0,     -0.35231279,      0.37750818,       1.9269618,       2.3849754,      -11.482722 ],
  [               0,               0,               0,               0,               0,    -0.083140792,      0.47786594,       1.4664644,      0.87539763,      -13.324338 ],
  [               0,               0,               0,               0,               0,   6.7356093e-05,   0.00014265718,   0.00032401846,   0.00080396988,    0.0031722019 ],
  [               0,               0,               0,               0,               0,      0.18926917,     -0.21477048,     -0.98392817,      -1.3138656,       5.5462527 ],
  [               0,               0,               0,               0,               0,    -0.077983729,     -0.28223241,     -0.45434693,      0.31249385,        5.739588 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.010299448,    -0.013872905 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -19.698801,       50.371696 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -2.880421,        5.845611 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       11.139558,      -27.164673 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,      -2.0862642,        1.873322,     -0.42698847,      0.03177831,   0.00053003026 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer1_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,    0.0065446912,   -0.0054871752,   -0.0025028296,   0.00099261389 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00031818649,   0.00072191684 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.3963264,       3.2115151 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.099589929,     -0.68515706 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.5569106,     -0.56746151 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.27191415,   0.00036767154 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.02698636,    0.0032028579 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -20.482331,       16.849506 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -4.2318035,       2.8486678 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       19.353009,      -15.392784 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.004102999,    0.0062136241 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.3378046,      -4.4852372 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -10.016461,       7.9368683 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -10.784773,       8.7031087 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.44811034,     -0.37987489 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0072945779,   -0.0048524523 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.95538605,       1.5534147 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.21013172,      -1.9257168 ],
 ],
 [
  [               0,               0,               0,               0,      0.02113575,   -0.0093437074,    -0.028701073,     0.029251811,    -0.017776926,  -0.00016901367 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.34816028,      0.84007291,     -0.67942303,      0.17444405,     0.018191435,   -0.0068152273 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0037345891,    0.0030695615 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.6522436,      -1.0928906 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -23.973368,       33.685564 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -41.64062,       57.807372 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.025404509,     -0.26919476 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer1_etaoffv5_35_e = [
 [
  [               0,               0,               0,               0,    -0.075322653,     0.076675052,     0.033632693,    -0.029563392,   -0.0090996524,   -0.0022022013 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0094851907,    0.0030395174 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.1313241,       3.1855345 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -17.597654,        24.98381 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.3626714,       3.9048237 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.1221965,     -0.26414299 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.28465871,      0.43077741 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.13708099,       4.5013189 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       25.101884,       -38.57765 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.9687052,      -8.7930185 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.59390387,      0.12641482 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.16957905,      0.48442143 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.16707933,     -0.47846233 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.16690811,      -0.4781432 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00027896123,   0.00057341722 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00013968141,  -4.7357349e-05,   0.00016041837,   0.00098112469 ],
  [               0,               0,               0,               0,               0,               0,      0.58719602,     -0.28921667,     -0.98605252,       5.0171514 ],
  [               0,               0,               0,               0,               0,               0,     -0.22847203,      -1.2800032,     -0.68481523,       6.5262466 ],
  [               0,               0,               0,               0,               0,               0,       1.7336126,      0.79619964,      -2.0045342,      -8.7891416 ],
  [               0,               0,               0,               0,               0,               0,      0.96120223,      0.70646884,     -0.80106595,      -6.2265349 ],
  [               0,               0,               0,               0,               0,               0,    0.0001964059,   5.6452315e-05,  -0.00037006104,  -0.00034606854 ],
  [               0,               0,               0,               0,               0,               0,      0.10346068,     -0.16335923,     -0.54192842,       1.9936449 ],
  [               0,               0,               0,               0,               0,               0,    0.0027227848,       0.2405781,    0.0032524139,      -2.5872531 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,     0.006619928,    -0.013277022,    -0.036739722,      0.07504833 ],
  [               0,               0,               0,               0,               0,               0,       6.8135061,      -13.897034,      -29.287803,       63.706424 ],
  [               0,               0,               0,               0,               0,               0,       6.2860265,       -14.58728,      -24.664692,       56.374834 ],
  [               0,               0,               0,               0,               0,               0,       1.3458103,      -2.1977105,      -8.4556073,       13.244138 ],
  [               0,               0,               0,               0,               0,               0,      -4.5968543,       9.9386448,       21.792988,      -46.898128 ],
  [               0,               0,               0,               0,               0,               0,    -0.003345361,    0.0069978792,     0.016498461,    -0.033639009 ],
  [               0,               0,               0,               0,               0,               0,      -3.9734754,       8.9107204,       17.436381,      -38.233822 ],
  [               0,               0,               0,               0,               0,               0,       3.0626164,      -6.9676648,      -13.344389,        28.93286 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   6.0809995e-05,   0.00082441972,    0.0039686725 ],
  [               0,               0,               0,               0,               0,               0,               0,      -4.2928051,      -0.3514288,       25.039152 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.4049934,      -0.1131512,       17.089573 ],
  [               0,               0,               0,               0,               0,               0,               0,       -2.504466,     0.031462667,       12.369829 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.045216762,     0.072134209,     -0.21226919 ],
  [               0,               0,               0,               0,               0,               0,               0,   5.3344713e-05,  -3.7263626e-05,   0.00054544115 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.38776785,    -0.081278545,       2.2372274 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.064547934,     -0.19088183,     -0.53726591 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00042294361,    0.0017607109 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.64576215,       3.8951842 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.72553816,      0.69100671 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.048272585,     -0.90775037 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.034360367,      0.10835192 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   8.1174609e-06,   1.9476953e-05,   0.00054673248 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.0202962,     0.018363768,       4.0460961 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.032156859,    -0.077460456,      -0.5865858 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.16748072,      0.40026735,    -0.043392098 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.061570182,     -0.14765907,     -0.65411882 ],
  [               0,               0,               0,               0,               0,               0,               0,  -2.2484617e-05,  -5.3820674e-05,   0.00087117184 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.049419506,     -0.11855636,       1.7155866 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.37718114,     -0.12986661,     -0.31064233 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0062840383,    0.0033080932,     0.036868088 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.5950672,       3.3737372,       6.1588968 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.2328628,       8.7380872,       -9.613871 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.65044351,       2.1636662,      -8.9934677 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0027556035,  -0.00067231224,    -0.013887294 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.9618659,      -0.6379772,      -16.932976 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.55969265,      -1.3464059,       2.9741672 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.033494766,   -0.0081451416 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.21079147,        39.03604 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.72544483,      -142.15138 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.14051247,      0.25260515 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.4863262,       2.6377761 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer2_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,    -0.045989821,     0.024507482,    0.0044547373 ],
  [               0,               0,               0,               0,               0,               0,               0,       19.510333,      -10.434063,       5.6341285 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.74571919,      0.32320834,      -1.3703991 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.55473062,    -0.062205106,    -0.021789936 ],
  [               0,               0,               0,               0,               0,               0,               0,        0.026785,   -0.0084391198,    0.0013722089 ],
  [               0,               0,               0,               0,               0,               0,               0,         7.61696,      -5.4136186,       1.1047697 ],
  [               0,               0,               0,               0,               0,               0,               0,      -12.559761,       6.7090874,      -1.8637478 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.007042474,    0.0024459833 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -4.3362387,       7.0965494 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.13553621,      -1.3736978 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.34759898,     -0.13028856 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0014206094,    0.0016053123 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.8894972,       1.2592829 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.3313824,      -6.8424601 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.010880267,  -0.00063289164 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       21.100493,       -13.07577 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       11.552308,      -10.669253 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -6.1578119,       5.2874371 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.012419939,   -0.0068204424 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -6.7769619,       5.6590813 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        4.938494,      -6.8504126 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0012904687,   0.00053737347,    0.0034010818,     0.005433317 ],
  [               0,               0,               0,               0,               0,               0,        1.484474,      -2.4075695,       -0.950718,       5.4139027 ],
  [               0,               0,               0,               0,               0,               0,      0.17778663,     -0.73018339,      0.64142818,      -1.5084593 ],
  [               0,               0,               0,               0,               0,               0,     0.036885707,     0.085822672,      0.36595375,     -0.22845776 ],
  [               0,               0,               0,               0,               0,               0,   0.00078201828,   7.2055629e-05,  -0.00018270019,   0.00098300877 ],
  [               0,               0,               0,               0,               0,               0,     0.022235766,     0.024683211,    -0.022762854,      0.21197932 ],
  [               0,               0,               0,               0,               0,               0,      0.50064629,     -0.61214417,      -1.2356073,      -1.2962805 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00060174849,    0.0099283191 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -52.447935,       74.517469 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       8.9792817,      -13.613967 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.0376278,      -1.2408697 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.24463505,      0.65723159,     -0.56669159,      0.20837584,    -0.035098398,     0.001389487 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,      0.12720353,    0.0097213947,     -0.23641177 ],
  [               0,               0,               0,               0,               0,               0,               0,      -12.848536,      0.18896529,       29.902778 ],
  [               0,               0,               0,               0,               0,               0,               0,        4.926601,       4.8485321,      -17.721179 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0094108931,     0.040227544,     -0.33674765 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer2_etaoffv5_35_e = [
 [
  [       -2.222731,       4.8460079,      -1.2761611,      -3.3530681,       0.9959426,       2.7237386,      -2.3619208,      0.73504013,    -0.084020124,   -0.0065278504 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,     -0.23893776,      -0.1136412,      0.18322136,      0.78764639 ],
  [               0,               0,               0,               0,               0,               0,       937.26529,      -1397.0266,      -1916.5451,       2858.5401 ],
  [               0,               0,               0,               0,               0,               0,      -11.216745,       -9.669846,       3.4828066,       52.043564 ],
  [               0,               0,               0,               0,               0,               0,       1.6597474,      0.52209688,       1.7326349,      -9.1694507 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0060071969,   -0.0015072865 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       74.684527,      -110.04442 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.1444188,      -9.1013481 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -2.556005,       3.9916784 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0016503943,   -0.0022274668,   -0.0053350838,     0.015607162 ],
  [               0,               0,               0,               0,               0,               0,      0.41248161,     -0.23593204,      -3.2785621,       7.5913271 ],
  [               0,               0,               0,               0,               0,               0,     -0.21809224,      0.66991668,      -0.4848372,        -1.31331 ],
  [               0,               0,               0,               0,               0,               0,    -0.032108557,    -0.080560427,     0.026056453,      0.57218377 ],
  [               0,               0,               0,               0,               0,               0,   -0.0002789246,   -0.0013586968,  -0.00041974362,    0.0082211066 ],
  [               0,               0,               0,               0,               0,               0,       1.1575295,      -1.6469713,      -3.6345982,       5.3696053 ],
  [               0,               0,               0,               0,               0,               0,     -0.63432978,      0.96590952,       1.6537826,      -4.7861815 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0014599548,    0.0009878292,   -0.0012767768 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.78645681,      0.16011532,       6.2522938 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.19181703,       1.4496507,      -3.4879606 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.60599413,       2.2282021,      -1.9103856 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0002828939,  -1.8812067e-05,    0.0022111434 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.075561017,       -0.526431,       2.1882867 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.099235428,       1.0917975,      -3.5915338 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -0.00070174695,   -0.0011569915,   -0.0007250197,    0.0058826247,     0.041315741 ],
  [               0,               0,               0,               0,               0,       2.6080328,      -2.8971039,      -14.437351,      -18.038994,       82.455209 ],
  [               0,               0,               0,               0,               0,     0.065680164,     -0.29320133,    -0.022921271,       1.5478302,      -2.8709257 ],
  [               0,               0,               0,               0,               0,      -0.2392564,      0.16854167,       1.5326599,       2.7155989,      -9.7103723 ],
  [               0,               0,               0,               0,               0,  -0.00096203261,    0.0013532726,    0.0057807105,    0.0059437756,    -0.034790927 ],
  [               0,               0,               0,               0,               0,      0.68862242,     -0.60161513,      -3.3541997,      -4.6204723,        17.45429 ],
  [               0,               0,               0,               0,               0,       1.7277268,      -2.2593352,      -10.270069,      -11.748878,       59.430961 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00050958468,      0.01035866 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -6.3550278,       17.967714 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       7.5700721,      -19.824066 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.48713539,       1.2680152 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,      0.62033482,     -0.10398164,    0.0077630287,   -0.0073761976,   0.00027039916 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer1_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,    -0.010278066,      0.03272304,    -0.031027275,    0.0079194124 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00031637806,   0.00072270682 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.38004316,       3.2082742 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.10291557,     -0.68366975 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.56947554,     -0.56362627 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.27727591,   0.00056343487 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.6278188,      -3.6807657 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       110.32271,      -87.170713 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.8057478,      -4.7537746 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -80.117339,       64.076734 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.035988163,     -0.17239224,       0.1683199 ],
  [               0,               0,               0,               0,               0,               0,               0,       17.450358,      -35.860975,       17.942979 ],
  [               0,               0,               0,               0,               0,               0,               0,      -17.190644,        35.44954,      -17.806369 ],
  [               0,               0,               0,               0,               0,               0,               0,      -17.006685,       35.004281,      -17.559142 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.017278614,    0.0029761554,    0.0085274969 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0057752242,     0.013827024,   -0.0088996372 ],
  [               0,               0,               0,               0,               0,               0,               0,      -4.3686462,        9.759318,      -4.3974046 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.7183431,       5.7677931,      -7.7958542 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00010023618,   0.00014171912,  -0.00030378901,   6.2846041e-05,   0.00074044082 ],
  [               0,               0,               0,               0,               0,     0.089751009,      0.11252828,      -0.9285336,     -0.12493898,       6.0087877 ],
  [               0,               0,               0,               0,               0,      0.17455003,      0.38142342,     -0.37656717,      -1.9812239,       2.1166401 ],
  [               0,               0,               0,               0,               0,     0.028376655,     0.035258443,    0.0024671681,      0.10695243,     -0.92799963 ],
  [               0,               0,               0,               0,               0,     -0.13494619,     -0.19647988,     -0.12863079,     -0.44637947,      -2.4644281 ],
  [               0,               0,               0,               0,               0,   3.6920838e-05,   5.2998177e-05,   0.00013831694,   0.00011313422,    0.0012172165 ],
  [               0,               0,               0,               0,               0,     -0.32486262,     -0.06244705,     0.099260424,     0.064417258,        2.378671 ],
  [               0,               0,               0,               0,               0,      -0.5484351,   -0.0075089655,     -0.14254413,      0.03104806,     0.014035587 ],
 ],
 [
  [               0,               0,               0,               0,     -0.35143986,       0.8474875,     -0.68533338,       0.1765267,      0.01783237,   -0.0067824246 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0037218633,    0.0030910313 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.6319406,       -1.061459 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -23.742685,       33.352094 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -41.806087,       58.043161 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.025223795,     -0.26934614 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer1_etaoffv5_37_e = [
 [
  [               0,               0,               0,               0,    -0.045075928,     0.041770753,     0.019483258,   -0.0029272313,    -0.016879234,   -0.0022235889 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.008524139,    0.0028373394 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.95417059,       3.0903614 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -13.967018,       19.579013 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.9905069,       1.8648942 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.099664724,     -0.23913462 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.28029972,      0.42414689 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -70.486434,       109.10389 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       25.721961,      -39.495108 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       9.6678701,      -14.271869 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.4812882,      0.45444214 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.11343685,      0.31577396 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.11279127,     -0.31425686 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.11274693,       -0.314176 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -7.0785168e-05,   0.00014365581 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00023657467,   0.00013693836,   0.00051225914,   0.00031153346 ],
  [               0,               0,               0,               0,               0,               0,     -0.14016465,      -0.1686145,      0.35107165,        6.683618 ],
  [               0,               0,               0,               0,               0,               0,     -0.28832192,     -0.55927945,     0.022098909,       3.3498606 ],
  [               0,               0,               0,               0,               0,               0,       1.4488941,       0.9003474,      -1.5019926,      -9.2716357 ],
  [               0,               0,               0,               0,               0,               0,      0.88948964,      0.56867711,     -0.87887499,      -5.4523676 ],
  [               0,               0,               0,               0,               0,               0,   0.00015019649,   9.0815283e-05,  -0.00024871028,  -0.00035409804 ],
  [               0,               0,               0,               0,               0,               0,     0.028051001,     0.052497436,     -0.21413847,        1.082776 ],
  [               0,               0,               0,               0,               0,               0,    -0.097442728,      0.25656228,      0.21690405,      -2.2839926 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0026014241,   -0.0079407283,    -0.014823892,      0.04502495 ],
  [               0,               0,               0,               0,               0,               0,        6.310733,      -11.775073,      -26.001551,       50.484882 ],
  [               0,               0,               0,               0,               0,               0,       5.2371879,      -11.128223,      -22.389692,       46.471793 ],
  [               0,               0,               0,               0,               0,               0,      0.66324692,      -1.2199197,      -3.1154071,       4.7527079 ],
  [               0,               0,               0,               0,               0,               0,      -2.4022019,       4.8959864,       11.729345,      -23.908851 ],
  [               0,               0,               0,               0,               0,               0,    0.0053422096,    -0.011402807,    -0.024349679,     0.051970254 ],
  [               0,               0,               0,               0,               0,               0,       2.5734528,       -6.128112,      -11.038121,       25.912511 ],
  [               0,               0,               0,               0,               0,               0,      -25.284876,       62.486836,       113.19917,      -282.12214 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   3.9957689e-05,   0.00075767099,    0.0037720077 ],
  [               0,               0,               0,               0,               0,               0,               0,      -4.7069889,     -0.34343861,       27.264564 ],
  [               0,               0,               0,               0,               0,               0,               0,       -3.609587,     -0.11545645,       18.161654 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.6486491,     0.031265341,       13.130332 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.027498144,     0.038237938,     -0.27470783 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0001137236,   0.00012718836,   0.00098244005 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.38311798,    -0.081856205,       2.2187891 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.026829395,    -0.081380894,     -0.23312188 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00067863882,    0.0015529897 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.55970109,       3.5174113 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.51683182,      0.20310785 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.024899023,      -1.1023933 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.064749158,      0.16712033 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    6.932767e-06,   1.6617761e-05,    0.0005398324 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.3148852,      0.32531921,       4.7841885 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.034014187,    -0.081734005,     -0.59640117 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.11373888,      0.27199239,     -0.34956575 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.073423005,     -0.17608346,     -0.72228375 ],
  [               0,               0,               0,               0,               0,               0,               0,  -2.1446465e-05,  -5.1356167e-05,   0.00087702179 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.05747114,     -0.13821297,       1.6676008 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.36651402,    -0.098359638,     -0.23541616 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.005966081,     0.001931101,    -0.031597641 ],
  [               0,               0,               0,               0,               0,               0,               0,      -6.5857699,      0.70323969,       41.867309 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.010728803,      0.20865674,      -1.9290874 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.49960744,      0.06729795,      -3.5614925 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.01101505,   0.00093224347,    -0.062568542 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.17994965,     -0.16164808,        1.488941 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.3118614,      0.21820855,       6.5198842 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.010075009,     -0.02495082 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.024300637,      -4.0932003 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.23764255,      -50.557994 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.22240475,       367.01143 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.0190763,       90.788327 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer2_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,    -0.086919399,      0.05885155,    -0.011465472,    0.0079455088 ],
  [               0,               0,               0,               0,               0,               0,       39.843953,      -28.122704,       5.6778821,        4.112669 ],
  [               0,               0,               0,               0,               0,               0,      -6.7871881,        5.994242,      -1.6410387,      -1.2215331 ],
  [               0,               0,               0,               0,               0,               0,       100.83584,       69.046713,      -30.830487,      0.15771725 ],
  [               0,               0,               0,               0,               0,               0,       2.1122851,      -1.1399742,      0.27148464,   -0.0018231815 ],
  [               0,               0,               0,               0,               0,               0,      -4.1382218,        4.142052,      -1.3772888,      0.15789861 ],
  [               0,               0,               0,               0,               0,               0,       69.244834,      -58.419896,       7.9103987,     -0.52147406 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0027069605,    0.0059432839 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.4358589,        5.626263 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.16977616,      -1.3090109 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.098776322,      0.19715133 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0030646981,    0.0040135082 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.38353803,    -0.059489638 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.5116671,      -7.8407384 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0096185085,    0.0028107189,  -0.00032082452 ],
  [               0,               0,               0,               0,               0,               0,               0,       12.122013,     -0.33935789,      -3.7036751 ],
  [               0,               0,               0,               0,               0,               0,               0,        7.822715,      -0.8459099,       -5.761186 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.8400019,      0.13642514,        2.633338 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0084552076,   0.00059666453,   -0.0033796881 ],
  [               0,               0,               0,               0,               0,               0,               0,       -4.423078,     -0.75840548,       3.7127281 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.1371038,     -0.66665653,      -4.3372291 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0012529934,   0.00051567606,    0.0033452701,     0.005502423 ],
  [               0,               0,               0,               0,               0,               0,       1.4069246,      -2.3025738,     -0.93957985,       5.3619082 ],
  [               0,               0,               0,               0,               0,               0,      0.21943776,     -0.88161166,      0.81121584,      -1.5688802 ],
  [               0,               0,               0,               0,               0,               0,     0.064058707,    -0.025148284,      0.49081314,     -0.26085293 ],
  [               0,               0,               0,               0,               0,               0,   0.00069286269,   8.5126349e-05,  -0.00011943997,    0.0010832362 ],
  [               0,               0,               0,               0,               0,               0,    0.0051376332,     0.052491486,    0.0035393279,      0.17139961 ],
  [               0,               0,               0,               0,               0,               0,      0.46600708,     -0.56466497,      -1.1740977,      -1.3609898 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0011690702,    0.0075261526 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -54.908784,       77.853889 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       9.8999253,      -14.873495 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        1.184965,      -1.4334654 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.49937267,       1.2170642,      -1.0051288,      0.35545188,    -0.054279047,    0.0020405272 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,      0.12305275,     0.010068234,     -0.22711662 ],
  [               0,               0,               0,               0,               0,               0,               0,      -16.877733,      0.23030356,       38.022983 ],
  [               0,               0,               0,               0,               0,               0,               0,       5.1968281,       5.8097353,      -19.612134 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.011400996,     0.048531749,     -0.33651623 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele100_layer2_etaoffv5_37_e = [
 [
  [      0.34827521,     -0.43689004,     -0.30763823,      0.20632584,      0.43050304,    0.0051443696,     -0.46588034,      0.26846808,    -0.041897953,   -0.0084615519 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,     -0.41334215,     -0.18963779,      0.32308059,       1.3413161 ],
  [               0,               0,               0,               0,               0,               0,       7.6245844,       4.2562925,      -6.8011011,      -23.194896 ],
  [               0,               0,               0,               0,               0,               0,      -4.4603772,      -2.4789228,       2.9987568,        14.72033 ],
  [               0,               0,               0,               0,               0,               0,     -0.55868047,    -0.027428609,      0.65187429,      0.95201573 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0049801912,   0.00021984948 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       74.822955,       -110.3152 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.0776329,      -9.0069082 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.3799249,       2.2101641 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00014359756,   0.00035546733,    0.0074059674 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.4919523,      -15.486986,         16.6332 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.1756511,       3.8465742,      -4.4044412 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0093727548,       0.7735719,      -1.0659087 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0010651973,  -0.00084872165,    0.0011046003 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.6964849,      -5.4673767,       4.4962473 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.56722151,     -0.73155476,      0.51342076 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,     0.001962519,   -0.0024316678,   -0.0062503369,     0.015316342 ],
  [               0,               0,               0,               0,               0,               0,      0.11367626,      0.44785362,      -6.6093945,       13.572397 ],
  [               0,               0,               0,               0,               0,               0,     0.062085603,      0.84037999,        -3.90878,       2.7046523 ],
  [               0,               0,               0,               0,               0,               0,      -5.0476667,        11.41797,       17.912958,      -40.866021 ],
  [               0,               0,               0,               0,               0,               0,    0.0011927867,   -0.0012332517,   -0.0034203552,    0.0086461669 ],
  [               0,               0,               0,               0,               0,               0,    -0.014707095,      0.20197225,     -0.68800638,      0.72503308 ],
  [               0,               0,               0,               0,               0,               0,        6.669863,      -13.961087,      -28.417879,       57.475742 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0010078698,    -0.003409759,    -0.010954795,   -0.0056283188,      0.10256328 ],
  [               0,               0,               0,               0,               0,       2.3569255,      -2.5897498,      -12.926419,      -16.187867,       73.419147 ],
  [               0,               0,               0,               0,               0,      0.28266603,     -0.43253754,      -1.4568569,      -1.0742947,       6.3998551 ],
  [               0,               0,               0,               0,               0,     -0.33247821,      0.33870875,       1.9658367,       2.7812582,      -11.623942 ],
  [               0,               0,               0,               0,               0,   1.5416199e-05,  -0.00021946286,  -0.00043131883,   0.00018885991,    0.0051062405 ],
  [               0,               0,               0,               0,               0,     0.073322845,      0.12058404,     0.029514507,     -0.66445711,       -1.138328 ],
  [               0,               0,               0,               0,               0,      0.15589904,     -0.25889178,       -1.398695,      -2.2556659,       7.0040502 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00052660965,     0.010079761 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.0243767,     -0.16757079 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       6.9171575,      -18.229461 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.095151728,     -0.16669078 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,      0.97782548,     -0.42263494,      0.11156715,    -0.020327422,   0.00080219882 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]


###############################################################################
# Tables for v5 sshape correction, 200 GeV electron clusters
#


ele200_layer1_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,    -0.015349433,     0.043361576,    -0.037566263,    0.0090142211 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0002900198,   0.00065519477 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -0.609237,       3.7270084 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.14606587,     -0.68420971 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.58582408,      -0.5683999 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.29349815,   -0.0056260994 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,       119.06966,      -188.99434,       74.995345 ],
  [               0,               0,               0,               0,               0,               0,               0,       76.558539,      -2803.2624,       2189.6409 ],
  [               0,               0,               0,               0,               0,               0,               0,       137239.19,      -217143.57,       85884.746 ],
  [               0,               0,               0,               0,               0,               0,               0,       349561.87,      -558160.35,       222810.14 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0077774515,   0.00085820876 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.21241264,       1.2995699 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.33188719,      -1.1190133 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.22026236,     -0.69584519 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0047088092,    -0.048251353 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00035436268,   0.00016227646 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.2244989,      -3.6274595 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.56551814,       3.8122986 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   5.5600668e-05,   7.3882359e-05,   9.5188912e-05,   0.00061782896 ],
  [               0,               0,               0,               0,               0,               0,       3.1060737,       4.3510466,       6.0941968,       12.533852 ],
  [               0,               0,               0,               0,               0,               0,      -6.4868761,      -3.3891009,       3.2277551,       15.284465 ],
  [               0,               0,               0,               0,               0,               0,      0.36653824,      0.68412234,       1.1952802,        1.003289 ],
  [               0,               0,               0,               0,               0,               0,     -0.31099221,     -0.19234496,     0.077157287,      0.30178032 ],
  [               0,               0,               0,               0,               0,               0,   0.00023711836,   0.00038113316,   0.00060514563,    0.0019512758 ],
  [               0,               0,               0,               0,               0,               0,      0.10167215,    -0.043940819,     -0.26817464,       1.3389157 ],
  [               0,               0,               0,               0,               0,               0,      -1.1342155,    -0.049483866,      0.12898104,      0.46449166 ],
 ],
 [
  [               0,               0,               0,               0,     -0.05497757,     0.089130637,    0.0035338876,     -0.08272978,     0.049464166,   -0.0053396157 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0034279912,    0.0030384701 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.5165664,     -0.26114795 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -23.995641,       33.848199 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -46.341202,       64.394285 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.049060431,      -0.3088464 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer1_etaoffv5_55_e = [
 [
  [               0,               0,               0,               0,      0.15689051,     -0.30857239,      0.16699901,    -0.031335741,     0.010022933,    0.0023680503 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0032433541,    0.0043391073 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.5438423,      -5.6056423 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -23.42182,       33.492362 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -28.82535,        40.82825 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.12997158,     -0.32475838 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.08737312,      0.13857415 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -31.365148,       50.747159 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       18.041296,      -28.188267 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.4383222,      -5.0308992 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00083840238,    0.0016832396 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.6398591,       2.8090631 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.13003638,      -1.2629894 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.085554976,      -1.1594417 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.083596277,      0.18343649 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00027643278,      0.00010144,   0.00056077051,   0.00069678967 ],
  [               0,               0,               0,               0,               0,               0,       1.3579767,     -0.62180009,      -3.4968421,     -0.82998288 ],
  [               0,               0,               0,               0,               0,               0,      -0.7895636,      -0.4924515,        1.418482,        7.327709 ],
  [               0,               0,               0,               0,               0,               0,      0.30157816,      0.28838039,      0.38072721,      0.12293074 ],
  [               0,               0,               0,               0,               0,               0,      0.58916378,      0.40866666,      -0.5304618,      -3.7534328 ],
  [               0,               0,               0,               0,               0,               0,   0.00015655182,   9.8280732e-05,  -0.00025679654,  -0.00043714983 ],
  [               0,               0,               0,               0,               0,               0,     -0.24469016,    -0.023278794,      0.24753601,       2.6531038 ],
  [               0,               0,               0,               0,               0,               0,     -0.11154023,      0.24408477,      0.22812031,      -2.1539887 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0013127684,   -0.0062917942,   -0.0090585309,     0.040011569 ],
  [               0,               0,               0,               0,               0,               0,       2.9335782,      -5.2707635,      -13.320469,       25.880571 ],
  [               0,               0,               0,               0,               0,               0,       2.7737196,      -6.0445528,      -11.802334,        24.57517 ],
  [               0,               0,               0,               0,               0,               0,      0.43758399,     -0.56706627,      -2.8584157,       3.3658965 ],
  [               0,               0,               0,               0,               0,               0,      -1.4631379,       3.0148305,       7.5153935,       -15.45757 ],
  [               0,               0,               0,               0,               0,               0,  -0.00028866342,   0.00044499721,    0.0010723985,   -0.0012285981 ],
  [               0,               0,               0,               0,               0,               0,      0.49121298,      -1.0842669,      -2.9652389,       6.0423719 ],
  [               0,               0,               0,               0,               0,               0,       1.3751728,       0.3759348,      -2.7766197,      -12.750543 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00028646049,   0.00095357558,    0.0033698057 ],
  [               0,               0,               0,               0,               0,               0,               0,      -6.8755623,     -0.36718529,       38.642924 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.1608428,     -0.11973946,       -17.66702 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.1246826,     0.045389263,        15.69457 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.83539307,     0.091596147,       4.5309803 ],
  [               0,               0,               0,               0,               0,               0,               0,  -1.9316804e-05,  -3.4503859e-05,   0.00094287433 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.38408823,     -0.14863072,       1.9029954 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.12357912,     -0.46100061,      -1.4600759 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0043729428,     0.013974931 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.1874673,       5.0016533 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.4236982,      -4.4581857 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.83232211,      -3.0548077 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.1941801,      0.48427801 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   5.6339275e-05,   0.00063528267 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.0064688,       4.7996803 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.19947652,     0.078273276 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.09632606,     -0.77076962 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.33222621,      -1.0969086 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00084843364,     0.001082878,  -0.00037013141,   -0.0091999853 ],
  [               0,               0,               0,               0,               0,               0,      -1.6549352,      -1.1342868,       3.6356398,       26.832249 ],
  [               0,               0,               0,               0,               0,               0,     -0.07627008,     0.032239217,      0.16132749,     -0.89508567 ],
  [               0,               0,               0,               0,               0,               0,      0.18367212,      0.18362005,     -0.29862327,      -3.0944745 ],
  [               0,               0,               0,               0,               0,               0,   0.00077611839,   0.00064332869,   -0.0014755449,     -0.01000901 ],
  [               0,               0,               0,               0,               0,               0,     -0.19078967,     -0.14523303,      0.13584954,        3.405998 ],
  [               0,               0,               0,               0,               0,               0,     -0.71644072,     -0.45841917,       1.3349546,       9.0361205 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0065855922,    0.0037166715,    -0.021709965 ],
  [               0,               0,               0,               0,               0,               0,               0,       -2.743902,      0.35125241,       22.913992 ],
  [               0,               0,               0,               0,               0,               0,               0,       -2.644442,      -1.0629248,     -0.82723914 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.3287979,     0.031078064,       19.701735 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.27397549,      0.15047773,       2.1159152 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer2_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,    -0.033849218,    -0.016078655,     0.013919444,      0.00645099 ],
  [               0,               0,               0,               0,               0,               0,       26.382316,      -6.6366414,       -1.951825,       4.0657851 ],
  [               0,               0,               0,               0,               0,               0,      -15.170703,       10.549391,      -2.3423977,       -1.137993 ],
  [               0,               0,               0,               0,               0,               0,       2.0329728,      -1.2236887,      0.42851007,    -0.088441184 ],
  [               0,               0,               0,               0,               0,               0,     0.031911568,    -0.017109281,     0.001370037,   0.00045108288 ],
  [               0,               0,               0,               0,               0,               0,       13.500108,       16.047903,      -9.0601693,       1.2215193 ],
  [               0,               0,               0,               0,               0,               0,      -37.008215,      -10.972483,       7.8721838,      -1.3130948 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0020724288,    0.0011826184,      0.00359951,    0.0054777374 ],
  [               0,               0,               0,               0,               0,               0,      -2.3628079,      0.98242016,     -0.26372662,       4.0071146 ],
  [               0,               0,               0,               0,               0,               0,      0.37935128,      -1.3234736,      0.90457184,      -1.5386228 ],
  [               0,               0,               0,               0,               0,               0,       1.2344674,      -1.0322344,      0.24945596,     0.010657692 ],
  [               0,               0,               0,               0,               0,               0,    0.0021235183,  -0.00090730418,   -0.0011440572,   0.00090352099 ],
  [               0,               0,               0,               0,               0,               0,      -1.0854546,      0.72390097,     -0.47330178,       1.0585475 ],
  [               0,               0,               0,               0,               0,               0,      -1.4027225,     -0.53871277,     -0.66674177,      -1.2361616 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.047014759,   0.00074043648,    -0.022210577 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.1879956,      -1.9904064,       2.1623772 ],
  [               0,               0,               0,               0,               0,               0,               0,       7.4509892,      -1.1972865,      -5.1789208 ],
  [               0,               0,               0,               0,               0,               0,               0,      -5.1538223,    -0.045009023,        3.400405 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0083334461,   -0.0016030236,    0.0074274583 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.36593221,      -14.580203,       11.920502 ],
  [               0,               0,               0,               0,               0,               0,               0,      -16.705294,     -0.36208685,       7.9390769 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0060843745,     0.020581527,     -0.02271419,     0.016975013 ],
  [               0,               0,               0,               0,               0,               0,       3.4664716,      -10.177589,       8.9084621,      0.78304757 ],
  [               0,               0,               0,               0,               0,               0,      -2.4587771,       8.3386264,      -9.6495055,       2.4041247 ],
  [               0,               0,               0,               0,               0,               0,       26.986017,      -89.989786,       100.05037,      -36.601397 ],
  [               0,               0,               0,               0,               0,               0,      0.23298751,     -0.77462898,      0.85630724,     -0.31147716 ],
  [               0,               0,               0,               0,               0,               0,       3.2884698,       -12.40862,       15.556022,      -6.2850743 ],
  [               0,               0,               0,               0,               0,               0,      -12.554642,       43.960497,      -51.226566,       16.809445 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0082544935,   -0.0013674038 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -113.42619,       157.12113 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       16.367231,      -23.658168 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.7526582,      -4.8031845 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.014791834,     0.018493424,    0.0033321053,  -0.00078775654,     -0.00703381,   0.00022217948 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   1.6885992e-05,     0.013305521,   0.00091370334,   -0.0015378003,    0.0030885421 ],
  [               0,               0,               0,               0,               0,    -0.074061645,    -0.059342763,    -0.055483149,     -0.11171753,       38.768394 ],
  [               0,               0,               0,               0,               0,   -0.0036801381,    -0.037140335,    -0.069691467,      0.12629771,      -2.3217771 ],
  [               0,               0,               0,               0,               0,     0.024230454,     0.010041456,     0.025858514,   -0.0055130965,      0.60493116 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer2_etaoffv5_55_e = [
 [
  [     -0.10210438,     0.043898123,     0.095172789,     0.064105163,    -0.017906188,    -0.091003466,    -0.068139627,     0.087228456,   -0.0034125982,   -0.0083724185 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0044859442,   -0.0019237287,    0.0042129268,      0.01730427 ],
  [               0,               0,               0,               0,               0,               0,       7.7183775,       3.9913116,      -5.7871846,      -22.948316 ],
  [               0,               0,               0,               0,               0,               0,      -4.7480391,      -2.3665021,       3.3674559,       14.702024 ],
  [               0,               0,               0,               0,               0,               0,       4.2294403,       2.1550748,      -3.0251227,      -14.182299 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.077483051,     -0.10945021 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       10.318127,      -13.010325 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.57154204,     -0.33771699 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.94508914,       1.5173985 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0046479721,      0.00813472,     0.011785275,    -0.010822884 ],
  [               0,               0,               0,               0,               0,               0,       1.2034976,      -1.2600487,      -6.4437343,       11.377776 ],
  [               0,               0,               0,               0,               0,               0,     -0.29431278,      0.70152152,      0.12415016,      -1.9856708 ],
  [               0,               0,               0,               0,               0,               0,      0.04762723,   -0.0038737368,     -0.36537424,       0.4327066 ],
  [               0,               0,               0,               0,               0,               0,    0.0071281223,    -0.010935688,    -0.021827583,     0.033592087 ],
  [               0,               0,               0,               0,               0,               0,       6.3725478,      -12.017487,      -22.932772,       43.318125 ],
  [               0,               0,               0,               0,               0,               0,      -4.2494595,        7.832682,       13.907226,      -27.124882 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0012684871,   0.00065384044,    -0.001627846 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.73424735,      0.06448926,        6.056264 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.28487458,        1.820806,      -3.8357038 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.70027508,       2.6057346,      -2.2636416 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0004408515,  -6.6518676e-05,    0.0032152351 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.25532059,      -1.4486745,       2.8325482 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.24945485,       1.3051176,       -3.262015 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   -0.0010428626,   -0.0014341836,  -0.00024298365,    0.0085656056,     0.046397953 ],
  [               0,               0,               0,               0,               0,       2.8665506,      -3.1858442,      -15.779841,      -19.637459,       88.944002 ],
  [               0,               0,               0,               0,               0,     -0.34458658,       0.2931692,       1.9518491,       2.9224198,      -11.986959 ],
  [               0,               0,               0,               0,               0,     -0.68652882,      0.79017393,       3.9848982,       5.0282796,      -23.065932 ],
  [               0,               0,               0,               0,               0,   0.00027089407,   2.7798784e-06,  -0.00076553678,   -0.0018367679,   0.00095932193 ],
  [               0,               0,               0,               0,               0,      0.23532865,     -0.39726423,      -1.3689646,      -1.2644587,       8.7575854 ],
  [               0,               0,               0,               0,               0,     -0.17040153,      0.87666674,       2.4436445,      0.63240678,      -24.294877 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0068501244,   -0.0022084758 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -28.672044,        72.39205 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       7.3769791,      -19.351412 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.7364341,      -4.0638075 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     -0.55846192,      0.77592635,     -0.16728426,    0.0066599558,    0.0010907618 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer1_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,    -0.012729511,     0.037732122,    -0.033566284,    0.0080040127 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00029352949,   0.00066212483 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.61857589,       3.7075862 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.14824546,     -0.69421276 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.56649379,     -0.58533997 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.28955476,   -0.0060817435 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.6131085,      -1.2581372 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -55.053852,       44.367419 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        23.24894,      -19.020184 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       16.718875,      -13.299506 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0013403767,    0.0022058357 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.1102926,      -3.2990755 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -5.8383616,       2.5871846 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -6.8834601,        3.379998 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.45852333,      -0.4788258 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0012528942,  -0.00047185695 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.52249011,       1.1516973 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.058517271,      -4.7702908 ],
 ],
 [
  [               0,               0,               0,               0,               0,     0.001346389,    0.0012263995,   0.00078203795,  -0.00023225654,   -0.0017087283 ],
  [               0,               0,               0,               0,               0,     -0.13470547,     -0.11368076,    -0.053467932,     0.074129875,       4.3134482 ],
  [               0,               0,               0,               0,               0,      -21.807699,      -15.311649,     -0.11265719,       29.718954,       83.066788 ],
  [               0,               0,               0,               0,               0,       -4.416096,      -3.1336984,    -0.058919231,       6.0610643,       16.204246 ],
  [               0,               0,               0,               0,               0,      -1.0984438,      -0.7777529,    -0.016220225,       1.4907701,       3.9222221 ],
  [               0,               0,               0,               0,               0,   0.00023599823,   0.00021315562,    0.0001338121,  -4.3728798e-05,   0.00061447436 ],
  [               0,               0,               0,               0,               0,     -0.30928847,     0.041302226,     0.012290261,    -0.046453658,        1.845973 ],
  [               0,               0,               0,               0,               0,     -0.43934779,      0.23095816,      0.45493223,      0.82524349,       1.4248512 ],
 ],
 [
  [               0,               0,               0,               0,     -0.25152655,      0.54847788,     -0.34949995,    0.0011645009,     0.059399709,   -0.0099333427 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0037715036,    0.0031500709 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        3.650584,     -0.68795012 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -24.807179,       34.939757 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -47.746396,       66.421142 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.050501564,     -0.28170404 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer1_etaoffv5_35_e = [
 [
  [               0,               0,               0,               0,      -0.4141635,      0.86472697,     -0.58647222,      0.16187896,    -0.029726931,   -0.0010032138 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0050512069,    0.0041305055 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       6.2455251,      -6.9239052 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -22.108973,       31.492931 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -28.091764,       39.756523 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.16230628,     -0.34510394 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.15446765,      0.23802472 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -42.806338,       67.912673 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       15.775365,      -24.813737 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.2457891,      -7.7074637 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.022491063,    -0.015745877 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.50840745,       1.6463378 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.36377785,      -1.3100115 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.35412276,       -1.289811 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.017715166,     0.035868615 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00029617411,   0.00010739262,   0.00060394619,   0.00074217979 ],
  [               0,               0,               0,               0,               0,               0,      0.47290058,     -0.39464645,      -1.7367976,       1.1697834 ],
  [               0,               0,               0,               0,               0,               0,     -0.30999988,     -0.17755416,      0.81001303,       3.7754907 ],
  [               0,               0,               0,               0,               0,               0,      0.73465954,      0.54324544,     -0.16700364,      -2.7898703 ],
  [               0,               0,               0,               0,               0,               0,      0.56326633,      0.40091975,     -0.49036322,      -3.6124085 ],
  [               0,               0,               0,               0,               0,               0,   0.00016627826,   0.00011037532,  -0.00026381104,  -0.00054151007 ],
  [               0,               0,               0,               0,               0,               0,     -0.26580215,    -0.018794325,      0.29084422,       2.7096391 ],
  [               0,               0,               0,               0,               0,               0,    -0.099424648,      0.28263246,      0.24045342,      -2.4224154 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0033323997,    -0.010988206,    -0.019114553,     0.063332953 ],
  [               0,               0,               0,               0,               0,               0,       3.4185841,      -6.6987151,      -13.543489,       28.121897 ],
  [               0,               0,               0,               0,               0,               0,       1.7281948,      -3.7380786,      -7.5190083,       15.142798 ],
  [               0,               0,               0,               0,               0,               0,      0.45263279,      -1.0823279,      -1.2909163,       2.2234936 ],
  [               0,               0,               0,               0,               0,               0,       -3.364464,       6.9269968,       16.613313,      -34.203875 ],
  [               0,               0,               0,               0,               0,               0,    0.0068773996,     -0.01525895,    -0.031441333,     0.070039421 ],
  [               0,               0,               0,               0,               0,               0,       1.2175932,      -2.5072397,      -6.5390201,       13.346428 ],
  [               0,               0,               0,               0,               0,               0,     -0.64346781,       5.6796776,       8.5366544,      -41.073979 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00032006361,    0.0010140611,    0.0034707164 ],
  [               0,               0,               0,               0,               0,               0,               0,      -4.4151347,     -0.36479006,       25.624622 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.3581393,     -0.10999765,       11.569137 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.1102503,     0.029520034,       10.278924 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.048649958,      0.04710224,     -0.34462591 ],
  [               0,               0,               0,               0,               0,               0,               0,   7.1846552e-05,   4.8710829e-05,    0.0008435022 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.36285811,    -0.079346754,       2.1106483 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.024950244,   -0.0087075206,     -0.17248137 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,  -6.3944159e-05,    0.0038252539 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.2787284,       5.2269211 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.69041481,      -2.7363902 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.095951733,       -1.326604 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.23013178,      0.56892692 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   5.6223679e-05,   0.00063500278 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -2.001885,       4.8008946 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.20180619,     0.083822902 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.090476872,     -0.78477104 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.33382034,      -1.1007306 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00022614034,   0.00029398859,    0.0017710928,    0.0049539916 ],
  [               0,               0,               0,               0,               0,               0,      -1.2052287,     -0.57659225,       3.0152949,       18.847112 ],
  [               0,               0,               0,               0,               0,               0,     -0.28254426,     0.083599428,      0.81394764,    -0.012134093 ],
  [               0,               0,               0,               0,               0,               0,      0.31534358,      0.34612766,      -0.4667631,      -5.3110754 ],
  [               0,               0,               0,               0,               0,               0,    0.0010952051,   0.00089268733,   -0.0021634512,    -0.014865607 ],
  [               0,               0,               0,               0,               0,               0,     -0.54271722,     -0.29064512,       1.0607067,       7.8546184 ],
  [               0,               0,               0,               0,               0,               0,     -0.51426477,     -0.22560029,       0.8309456,       4.1530157 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0080856032,    0.0044214857,    -0.027473532 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.9166783,      0.11228088,        22.86947 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.4359616,     -0.57718008,     -0.63199564 ],
  [               0,               0,               0,               0,               0,               0,               0,       -8.232139,    -0.069405524,       49.815731 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.13224044,      0.12276401,       1.0856633 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer2_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,     0.017803227,    -0.012001582,    0.0022703587,     0.007387129 ],
  [               0,               0,               0,               0,               0,               0,      -4.1544108,        3.033071,     -0.65900823,       4.6045344 ],
  [               0,               0,               0,               0,               0,               0,      -8.3115752,        6.693645,      -1.7008926,      -1.2261587 ],
  [               0,               0,               0,               0,               0,               0,        3.726778,       85.372503,      -20.537773,     0.072939152 ],
  [               0,               0,               0,               0,               0,               0,       2.3467256,      -1.1146045,      0.18638606,   -0.0012180394 ],
  [               0,               0,               0,               0,               0,               0,      -7.1774188,       6.6858484,      -2.0155691,      0.21061041 ],
  [               0,               0,               0,               0,               0,               0,       184.43788,      -128.22194,       16.683011,      -0.8137071 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0013625923,    0.0010627725,   0.00032886055,    0.0073402662 ],
  [               0,               0,               0,               0,               0,               0,      -4.5576117,       3.4711304,     -0.40012539,       4.3869159 ],
  [               0,               0,               0,               0,               0,               0,      -1.1722127,      0.78885352,       0.0437729,      -1.4435713 ],
  [               0,               0,               0,               0,               0,               0,      0.49645701,     -0.24645507,      0.20749235,    -0.038602941 ],
  [               0,               0,               0,               0,               0,               0,    -6.25443e-05,   0.00064919137,  -0.00059241908,    0.0012349098 ],
  [               0,               0,               0,               0,               0,               0,      0.18537436,       1.3868109,      -1.2584756,      0.42809966 ],
  [               0,               0,               0,               0,               0,               0,    -0.018956958,      0.86508286,      -3.3524139,      -6.8354589 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0030527124,    0.0029524763,    0.0040762044 ],
  [               0,               0,               0,               0,               0,               0,               0,       14.426419,      -5.4636888,     -0.95597206 ],
  [               0,               0,               0,               0,               0,               0,               0,       9.1388156,      -3.2884348,      -4.6102144 ],
  [               0,               0,               0,               0,               0,               0,               0,      -6.6397931,    -0.080029075,       4.9560828 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.017909522,    0.0018434579,     0.015627369 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.4224469,     -0.49006122,       2.0955945 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.8364439,      -1.0051969,      -3.3799826 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00036869901,   0.00021454408,    0.0013510666,    0.0047346971 ],
  [               0,               0,               0,               0,               0,               0,       1.5591864,      -2.3637145,      -2.3863728,       7.3014248 ],
  [               0,               0,               0,               0,               0,               0,      0.38588545,     -0.49424675,     -0.54695687,     -0.82735885 ],
  [               0,               0,               0,               0,               0,               0,     -0.21326126,      0.29641591,      0.50770294,      -0.4799246 ],
  [               0,               0,               0,               0,               0,               0,   0.00031949924,  -0.00033568505,  -0.00046975267,    0.0012416427 ],
  [               0,               0,               0,               0,               0,               0,     -0.70439705,      0.64841013,       0.5573358,       0.3365453 ],
  [               0,               0,               0,               0,               0,               0,    -0.090354014,     -0.34656598,     -0.60991644,     -0.85495513 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0015985235,    0.0054786748 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.62267748,       4.4741357 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.41264522,      -1.0588322 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.39440482,      0.15300826 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00091230924,    0.0027740187 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.9290229,       2.9177517 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.9592505,       1.1461376 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.064776998,    -0.019263167,    -0.033927266,     0.024570297,    -0.009640663,   0.00018189369 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,      0.00695348,    0.0058426582,    0.0024788571,   -0.0048733119,    -0.018525645 ],
  [               0,               0,               0,               0,               0,     -0.42404498,     -0.11786228,      0.51002964,       1.6710387,       7.6961172 ],
  [               0,               0,               0,               0,               0,     -0.96538521,      0.39939243,       1.4757717,       1.1211729,      -3.3870685 ],
  [               0,               0,               0,               0,               0,    0.0092748043,     0.015567852,     0.031075035,     0.050717916,     -0.24703127 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer2_etaoffv5_35_e = [
 [
  [      -2.6129557,       5.1122931,     -0.77776526,      -3.3206874,      0.46765491,       2.4645157,      -1.8011697,      0.53985414,    -0.067053662,   -0.0075418673 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,      -1.0367329,     -0.50171055,      0.78651793,        3.427656 ],
  [               0,               0,               0,               0,               0,               0,      -30.981165,       53.431586,       77.630929,      -130.79292 ],
  [               0,               0,               0,               0,               0,               0,       31.185291,      -50.873485,      -71.503258,       115.55149 ],
  [               0,               0,               0,               0,               0,               0,       1.0278859,      -1.8704177,      -2.3914556,       4.3188338 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.084493438,     -0.12012651 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       19.981046,      -27.065373 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.0672395,       -5.961501 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.19308338,      0.41763272 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    -0.005550834,    0.0077946347,     0.011559412,   -0.0082702558 ],
  [               0,               0,               0,               0,               0,               0,       6.7607186,      -10.187335,      -21.219886,       36.113026 ],
  [               0,               0,               0,               0,               0,               0,      0.30330078,     -0.55617045,      -0.8074684,      0.14957419 ],
  [               0,               0,               0,               0,               0,               0,     -0.13074207,      0.16709812,       0.6238088,     -0.81133586 ],
  [               0,               0,               0,               0,               0,               0,    0.0014739195,  -0.00098025369,   -0.0061189663,    0.0068789902 ],
  [               0,               0,               0,               0,               0,               0,      -1.9290117,       4.8957156,       1.4093106,      -6.2046017 ],
  [               0,               0,               0,               0,               0,               0,      0.31610594,     -0.19466368,      -1.0374229,     -0.95999613 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.027972758,    -0.095212606,     0.085229148 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.50281202,       -7.864664,       17.346997 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.13563462,      0.39764265,      -2.7447076 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.71525496,       2.6925022,      -2.3689685 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0042060324,     -0.01976671,     0.023765059 ],
  [               0,               0,               0,               0,               0,               0,               0,       6.8986361,      -30.228993,       33.572683 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.6002158,      -13.376898,       14.317993 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   -0.0006618247,  -0.00096797529,  -0.00024016287,    0.0057539695,     0.032449021 ],
  [               0,               0,               0,               0,               0,       3.5412976,      -3.9602938,      -19.577694,      -24.227481,       110.76205 ],
  [               0,               0,               0,               0,               0,     0.073445796,     -0.34968221,     -0.10681585,       1.6755838,      -2.2337607 ],
  [               0,               0,               0,               0,               0,     -0.71657028,      0.78609424,       4.4365762,       6.2084517,      -26.473475 ],
  [               0,               0,               0,               0,               0,  -0.00020710218,  -0.00017800001,   0.00031865995,    0.0025911844,     0.011884979 ],
  [               0,               0,               0,               0,               0,     -0.19247871,      0.24812415,       1.0104524,      0.85054887,      -4.8287724 ],
  [               0,               0,               0,               0,               0,       1.2412972,      -1.3399012,      -6.4373688,      -7.9944089,       30.842415 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00077324005,    0.0093491428 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.9391912,       0.1296382 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       6.1665392,      -16.409949 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.078539773,     -0.11646442 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,       1.3303524,     -0.71997437,      0.18334423,    -0.027470931,   0.00085452618 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer1_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,    -0.078620743,      0.19204837,     -0.15182629,     0.037529617 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00029658553,   0.00066883706 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.59928055,       3.6741203 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.14900495,     -0.69915464 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.5686321,     -0.58954971 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.28954666,   -0.0059707463 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.6072856,      -1.2538705 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -55.491923,       44.719997 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       23.278376,      -19.046822 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       16.926651,      -13.464789 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0032087351,    0.0050891257 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.6455256,      -2.0702492 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -4.6726224,       1.1228997 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -5.673962,       1.9665685 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.24078387,      -0.1364965 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0034804433,   -0.0012787422 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.49506168,      0.89134929 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.068924609,      -4.8700532 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0014986931,    0.0013421558,   0.00080588065,  -0.00039485399,   -0.0022142135 ],
  [               0,               0,               0,               0,               0,    -0.046677673,    -0.058337816,    -0.071401971,    -0.085136914,       3.9016054 ],
  [               0,               0,               0,               0,               0,      -20.116963,       -14.13184,     -0.12647525,       27.363025,        76.49054 ],
  [               0,               0,               0,               0,               0,      -5.5822435,      -3.9524207,     -0.06302845,       7.6578993,       20.688423 ],
  [               0,               0,               0,               0,               0,      -1.0013815,     -0.70517354,    -0.007935579,        1.365043,       3.5444793 ],
  [               0,               0,               0,               0,               0,   0.00018352549,      0.00016855,    0.0001124356,  -1.5205701e-05,   0.00073757946 ],
  [               0,               0,               0,               0,               0,     -0.27657902,     0.064165554,     0.017483799,    -0.077121008,       1.7493676 ],
  [               0,               0,               0,               0,               0,     -0.42493962,      0.25547748,      0.50149004,      0.90793397,       1.5656797 ],
 ],
 [
  [               0,               0,               0,               0,     -0.25339247,      0.55137778,     -0.34949727,  -0.00012928212,     0.059811505,   -0.0099627013 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0037738013,    0.0031100331 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.6858921,     -0.73163872 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -25.889738,       36.506733 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -47.331885,       65.828294 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.050697976,     -0.28291051 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer1_etaoffv5_37_e = [
 [
  [               0,               0,               0,               0,     -0.48978442,       1.0674751,     -0.76845788,       0.2223456,     -0.03538452,  -0.00082644282 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0058981472,    0.0045068758 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        1.544545,     -0.19471731 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        -19.2621,       27.312348 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -23.377758,       32.905517 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.16845267,     -0.34260004 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.36932352,      0.55738533 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       42.442139,      -59.059067 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.3656725,      -4.7021608 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       22.206589,      -32.935974 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.07437314,    -0.051601495 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.34935256,       1.1047657 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.30496496,      -1.0022377 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.30145425,     -0.99522024 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0055351272,     0.011103812 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00034016308,   0.00018957844,   0.00075823837,   0.00045002463 ],
  [               0,               0,               0,               0,               0,               0,      0.88513519,      0.29202562,       -1.187247,     0.078606623 ],
  [               0,               0,               0,               0,               0,               0,     -0.26574699,     -0.37039644,      0.10189249,       2.2048282 ],
  [               0,               0,               0,               0,               0,               0,      0.75244144,      0.49621511,     -0.64832886,      -4.8477654 ],
  [               0,               0,               0,               0,               0,               0,      0.57664664,      0.44019775,     -0.46807313,      -3.8026662 ],
  [               0,               0,               0,               0,               0,               0,   0.00018306503,   0.00012192761,    -0.000284347,  -0.00066026541 ],
  [               0,               0,               0,               0,               0,               0,    -0.093899869,     0.050219488,     0.020660693,       1.5934021 ],
  [               0,               0,               0,               0,               0,               0,     -0.16508796,      0.50493929,      0.56712037,      -3.4363091 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0046198844,    -0.010426171,    -0.024029026,     0.057320414 ],
  [               0,               0,               0,               0,               0,               0,     0.038788821,      0.35015961,     -0.74105371,       1.9126379 ],
  [               0,               0,               0,               0,               0,               0,     -0.60947448,       1.2847574,       3.1158463,      -7.6579864 ],
  [               0,               0,               0,               0,               0,               0,      -1.5567595,       3.6112806,       6.3486351,      -15.918702 ],
  [               0,               0,               0,               0,               0,               0,      -0.4388227,      0.86884088,        2.405205,      -4.7807973 ],
  [               0,               0,               0,               0,               0,               0,   -0.0021108359,     0.004777397,     0.009280637,     -0.02102861 ],
  [               0,               0,               0,               0,               0,               0,      -20.292594,       34.797927,       128.54162,      -233.97821 ],
  [               0,               0,               0,               0,               0,               0,      -48.264857,       99.563551,       249.01067,      -515.80282 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00036584467,    0.0011629553,    0.0039133097 ],
  [               0,               0,               0,               0,               0,               0,               0,      -5.2181568,     -0.36758854,       29.882348 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.6883597,     -0.10618508,      -9.8151181 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.7097093,     0.019783434,       13.390726 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.57602551,     0.068164024,       3.0533116 ],
  [               0,               0,               0,               0,               0,               0,               0,  -7.3278069e-07,  -0.00013067564,   0.00040197722 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.47943353,     -0.34774011,       1.5013222 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.036349433,     -0.11159056,     -0.32133288 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00011151817,    0.0039350162 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.2853989,       5.2379103 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.75847966,      -2.8854185 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.064025513,      -1.2553435 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.26471403,      0.64456406 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   1.6566375e-05,   3.9788293e-05,   0.00059556124 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.3476805,       1.2452472,       4.5885388 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.067203437,       0.1611233,    -0.013699946 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.029414536,     0.069995108,     -0.83344629 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.095961641,     -0.23019512,     -0.85219676 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00068729666,  -9.1784667e-05,     0.002666473,     0.011695114 ],
  [               0,               0,               0,               0,               0,               0,     -0.79509457,     -0.29005782,       2.1322278,       13.224752 ],
  [               0,               0,               0,               0,               0,               0,     -0.24009045,   -0.0039031637,      0.58450618,      0.47704249 ],
  [               0,               0,               0,               0,               0,               0,      0.31278485,       0.3188698,     -0.49434625,      -5.0649188 ],
  [               0,               0,               0,               0,               0,               0,   0.00095651579,    0.0007884288,    -0.001881201,    -0.012952927 ],
  [               0,               0,               0,               0,               0,               0,     -0.62682943,     -0.37423298,       1.2385249,       9.1713203 ],
  [               0,               0,               0,               0,               0,               0,     -0.37309735,     -0.11572219,      0.54072113,       2.1972757 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00072737032,    0.0094405313,   -0.0014166685 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.060118799,    -0.092016988,       8.0812473 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.20814609,   -0.0064945381,       4.9126453 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.018406049,     0.041668072,     -0.77734909 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.10038119,    -0.020386278,      0.32412697 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer2_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,      0.16282609,      -0.1354817,     0.035617471,    0.0046286855 ],
  [               0,               0,               0,               0,               0,               0,      -59.579242,       51.758576,      -14.259376,       5.7623937 ],
  [               0,               0,               0,               0,               0,               0,       2.7684292,      -1.3976826,     0.081103806,      -1.3298807 ],
  [               0,               0,               0,               0,               0,               0,       5.1898462,      -1.7970166,     0.065406801,    -0.043435129 ],
  [               0,               0,               0,               0,               0,               0,    -0.042331783,     0.048245724,     -0.01056426,    0.0012627993 ],
  [               0,               0,               0,               0,               0,               0,       -26.58681,       25.410302,      -8.6576442,        1.166317 ],
  [               0,               0,               0,               0,               0,               0,       58.095926,       -55.60905,       13.562251,      -1.9541543 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0055249692,    0.0010790385,    0.0044659788,    0.0058174866 ],
  [               0,               0,               0,               0,               0,               0,     -0.60753369,     -0.26627717,     -0.13397206,       4.6859535 ],
  [               0,               0,               0,               0,               0,               0,      -1.3001445,      0.61700424,      0.34524361,      -1.5305307 ],
  [               0,               0,               0,               0,               0,               0,      0.22213789,    -0.076351372,      0.29788949,     -0.12612468 ],
  [               0,               0,               0,               0,               0,               0,  -0.00031741547,   -8.395429e-05,  -1.1924695e-05,    0.0010230804 ],
  [               0,               0,               0,               0,               0,               0,        2.294229,     -0.23095017,       -1.355197,      0.63090808 ],
  [               0,               0,               0,               0,               0,               0,       2.2330019,     -0.81029426,      -4.1258326,      -6.0957825 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0014843218,     0.003021396,    0.0050929905 ],
  [               0,               0,               0,               0,               0,               0,               0,       14.920581,      -4.9122696,      -1.7403773 ],
  [               0,               0,               0,               0,               0,               0,               0,       9.6241119,      -4.0212175,      -4.3291619 ],
  [               0,               0,               0,               0,               0,               0,               0,      -6.3664315,    -0.096438088,       4.9070161 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.016703142,    0.0022940356,     0.015518788 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.3789166,      -0.4999785,       2.0628534 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.8032029,       -1.010442,      -3.3602102 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00071962359,  -0.00010628541,    0.0034216703,    0.0055324729 ],
  [               0,               0,               0,               0,               0,               0,       1.6034383,      -3.0672175,     -0.09880538,       5.2809692 ],
  [               0,               0,               0,               0,               0,               0,     0.064206142,     -0.51428824,      0.57067195,      -1.5297398 ],
  [               0,               0,               0,               0,               0,               0,      0.16132275,    -0.045223902,      0.58441414,     -0.37763341 ],
  [               0,               0,               0,               0,               0,               0,    0.0011695183,   0.00092509835,   0.00015588569,   7.4912783e-05 ],
  [               0,               0,               0,               0,               0,               0,     0.051917232,    -0.020572523,    -0.078218275,      0.23979724 ],
  [               0,               0,               0,               0,               0,               0,      0.59021465,     -0.90265532,     -0.99387305,      -1.4241134 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.031485225,     0.050869507 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -13.074427,       21.467724 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.4389575,      -4.7012013 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.65515168,     -0.74705413 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.043349088,     0.059289894 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       27.623743,      -35.848622 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -57.79255,       78.249276 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.22362074,      0.67035131,     -0.63306882,      0.25413223,    -0.046067102,    0.0018849979 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0091471607,     0.007465694,    0.0026407036,    -0.007720452,    -0.026985807 ],
  [               0,               0,               0,               0,               0,     -0.89467279,     -0.48114243,      0.42235287,       2.1472388,       9.2167032 ],
  [               0,               0,               0,               0,               0,     -0.66193373,      0.41676706,       1.1968724,      0.69563883,      -3.4610805 ],
  [               0,               0,               0,               0,               0,     0.010480736,     0.017387698,      0.03196276,     0.049446137,     -0.24948461 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele200_layer2_etaoffv5_37_e = [
 [
  [     -0.17177076,     0.095454237,      0.14360079,     0.059611492,    -0.060046296,     -0.11721262,    -0.036914227,      0.13452322,    -0.039439821,   -0.0083212524 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,      -13.723958,      -6.6647473,       10.431589,       45.616329 ],
  [               0,               0,               0,               0,               0,               0,       26.373792,      -36.203861,      -50.204541,       68.338284 ],
  [               0,               0,               0,               0,               0,               0,       -4.575155,      -4.2816829,       27.572799,      -16.828025 ],
  [               0,               0,               0,               0,               0,               0,      -6.8153348,       6.9474827,       24.267384,       -29.03806 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.087834273,     -0.12518781 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       22.634091,      -31.063832 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.9238602,      -7.2666623 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.080269494,      0.24661043 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0076408167,     0.011616557,     0.017048961,    -0.018019784 ],
  [               0,               0,               0,               0,               0,               0,       6.5020762,      -9.8339274,      -20.458709,       34.978225 ],
  [               0,               0,               0,               0,               0,               0,        0.409965,     -0.75079675,      -1.0464964,      0.59818242 ],
  [               0,               0,               0,               0,               0,               0,     -0.16196986,      0.20054383,      0.74140264,     -0.95697785 ],
  [               0,               0,               0,               0,               0,               0,    0.0024390086,   -0.0024824501,   -0.0097636327,     0.012596305 ],
  [               0,               0,               0,               0,               0,               0,      -2.2000789,       5.8394445,      0.78968309,      -6.5817714 ],
  [               0,               0,               0,               0,               0,               0,      0.64097216,     -0.69563842,      -2.0527189,      0.58602156 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    -0.018961221,     0.082543216,    -0.080001513 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.7534817,      -22.160041,       28.405805 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.23255716,     -0.67440298,     -0.85310792 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.1425501,       4.2161929,       1.6225379 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0051480102,    -0.012815437,     0.060079734 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.063039938,     -0.20575335,      0.20066582 ],
  [               0,               0,               0,               0,               0,               0,               0,       13.925122,      -55.866115,        53.20383 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -0.00029111317,  -0.00033285461,   0.00023899526,     0.003439381,     0.016504595 ],
  [               0,               0,               0,               0,               0,       1.4965479,      -1.7147587,      -8.3911346,      -10.356114,       49.773098 ],
  [               0,               0,               0,               0,               0,      0.23737545,     -0.36569698,      -1.2511436,     -0.96187754,       5.5332626 ],
  [               0,               0,               0,               0,               0,      -0.3644788,      0.40261562,       2.1405294,        2.834915,      -12.579586 ],
  [               0,               0,               0,               0,               0,   4.2668939e-05,  -3.3718813e-05,  -0.00022776859,  -0.00046542208,    0.0011156263 ],
  [               0,               0,               0,               0,               0,     0.078050684,   -0.0054690935,     -0.22963206,     -0.55298252,       1.7543416 ],
  [               0,               0,               0,               0,               0,    -0.087442987,      0.10962253,      0.44417569,      0.27448182,      -3.7984523 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.16982804,      0.43151787 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -6.7804106,       18.936109 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       6.8491234,      -18.000776 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.4538631,       3.6442373 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,        1.232149,     -0.61281211,      0.15026654,    -0.023277885,   0.00070341457 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]



###############################################################################
# Tables for v5 sshape correction, 500 GeV electron clusters
#


ele500_layer1_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,    -0.035698724,     0.012894804,      0.11057357,     -0.12251235,     0.033961556 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00087898472,    0.0005277886 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.3096679,        4.487706 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.2638838,    -0.099831209 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.9282447,   -0.0086360642 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -5.6773138,       3.0467033 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0080730082,     0.004506823 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.1555839,    0.0040995431 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.69346038,      -4.3439711 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.5931424,       -2.841987 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -68.70891,        55.34296 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       37.930469,      -30.796088 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       17.357193,      -13.806528 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00029016488,     0.002009665 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.019340204,       2.3395083 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.45895125,      -1.2899757 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.064442989,      -1.2704985 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.034228246,     -0.11878201 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -5.0949918e-05,   5.6936295e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.7024978,       2.8159366 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -4.700877,      -2.7267659 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00079891021,   0.00068635787,   0.00034493438,  -0.00039503485,   -0.0013042542 ],
  [               0,               0,               0,               0,               0,      0.15906507,      0.19153745,      0.22548095,      0.25579551,       4.2700331 ],
  [               0,               0,               0,               0,               0,      -37.776452,      -26.539664,     -0.25008514,       51.344119,       143.89073 ],
  [               0,               0,               0,               0,               0,     -0.45265518,     -0.27836992,     0.099096519,      0.81084286,       1.0588782 ],
  [               0,               0,               0,               0,               0,      -1.7554631,      -1.2565657,    -0.044954895,       2.3833211,       6.5269909 ],
  [               0,               0,               0,               0,               0,  -0.00014225307,  -2.3703831e-05,   0.00021949511,   0.00067105106,      0.00246293 ],
  [               0,               0,               0,               0,               0,     -0.23607371,      0.14770004,     0.058776311,     -0.12639289,       1.5288329 ],
  [               0,               0,               0,               0,               0,     -0.79998197,     -0.14767234,    -0.091419243,     0.036127112,      0.28409315 ],
 ],
 [
  [               0,               0,      0.30133875,     -0.56383335,     0.038554888,      0.49112031,     -0.29358177,    -0.015366362,     0.047831652,   -0.0058292479 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0041206255,    0.0033136543 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.3498929,      -1.5506004 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -29.621409,       41.925456 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -70.500811,       98.856867 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.072815501,     -0.29243857 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer1_etaoffv5_55_e = [
 [
  [               0,               0,               0,               0,      0.11749061,      -0.2057251,     0.051546332,     0.035486593,   -0.0061201607,    0.0032321978 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0027482505,    0.0051869743 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -9.1472254,       16.240895 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -29.816873,       42.804034 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -61.327212,       88.047515 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.15007773,     -0.31728754 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.097617901,      0.15346344 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -22.19002,       37.761656 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       15.334429,      -24.220256 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.8060964,      -5.5672763 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0016421584,    0.0018637165 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.21437633,       1.8494237 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00059685838,      -1.0193593 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.09368635,     -0.84127692 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.060142586,      0.13174809 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -9.9602463e-05,  -3.3150309e-05,    0.0001002225,   0.00074487666 ],
  [               0,               0,               0,               0,               0,               0,        1.406561,       1.0119997,      -1.1210904,      -3.8314041 ],
  [               0,               0,               0,               0,               0,               0,     -0.91925835,     0.091824642,       1.9590986,       3.5026466 ],
  [               0,               0,               0,               0,               0,               0,       1.6144978,      0.61697806,      -1.8304849,      -7.0192611 ],
  [               0,               0,               0,               0,               0,               0,      0.90284808,      0.91920809,     -0.55399917,      -7.1679949 ],
  [               0,               0,               0,               0,               0,               0,   0.00017556516,    9.476592e-05,  -0.00029037776,  -0.00046445974 ],
  [               0,               0,               0,               0,               0,               0,     -0.12906343,    0.0061221145,     0.051120566,       2.0225515 ],
  [               0,               0,               0,               0,               0,               0,    -0.039086993,     0.033562971,     -0.10221327,      -1.2295138 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0028114893,   -0.0080694842,    -0.015035384,     0.045233735 ],
  [               0,               0,               0,               0,               0,               0,       1.3664376,      -2.1760958,      -5.6508447,        11.06241 ],
  [               0,               0,               0,               0,               0,               0,       1.4779455,      -3.1542221,      -6.5808941,        12.85447 ],
  [               0,               0,               0,               0,               0,               0,     -0.25956958,      0.68899399,        1.069621,      -3.9778572 ],
  [               0,               0,               0,               0,               0,               0,     -0.48865004,       1.0401591,       2.4365187,      -5.1259034 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00034631582,    0.0012178907,    0.0042677943 ],
  [               0,               0,               0,               0,               0,               0,               0,      -12.482384,     -0.38184167,       68.200936 ],
  [               0,               0,               0,               0,               0,               0,               0,        6.718843,    -0.096152615,      -36.399325 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.36103015,     0.074115357,       1.9297952 ],
  [               0,               0,               0,               0,               0,               0,               0,  -0.00024437982,  -0.00012867142,    0.0017007842 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.29352371,    -0.005753527,       2.1373896 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.24709204,     -0.63988767,      -1.5593878 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00065329769,    0.0023375301 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.31644473,       3.0403228 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.11472534,      -1.4183459 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.43941073,    -0.083824422 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.20482869,      0.51726687 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   4.5132218e-05,   0.00070573597 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -1.668985,        6.217685 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.48661519,       0.5101647 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.15522551,      -2.8324045 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.32187712,      -1.0721051 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -1.3061472e-05,   0.00037103104,    0.0012372134,    0.0027502937 ],
  [               0,               0,               0,               0,               0,               0,      -1.3854745,     -0.72327745,       3.4203596,       21.809578 ],
  [               0,               0,               0,               0,               0,               0,     -0.45383021,      0.38675206,         1.68878,      -1.4855366 ],
  [               0,               0,               0,               0,               0,               0,      0.29384644,      0.40720966,     -0.32756106,      -5.6919347 ],
  [               0,               0,               0,               0,               0,               0,  -0.00098802292,  -0.00087297805,     0.001658736,     0.014375956 ],
  [               0,               0,               0,               0,               0,               0,      0.51055996,      0.12744389,      -1.4976668,      -5.3411885 ],
  [               0,               0,               0,               0,               0,               0,      0.73528073,      0.54502487,      -1.5877503,      -10.673111 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0032040935,    0.0057201526,     0.019252014 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0074319964,    -0.006232612,        22.69471 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.038941566,     -0.39218427,       17.799165 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.070546817,     -0.35707011,      0.20993938 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.23125652,    -0.099433664,      0.11374191 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer2_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00027457023,    0.0079952135 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.08978974,       3.9665269 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.066956974,      -1.2994468 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.5115799,     -0.11050277 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00086055779,   0.00082887564 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.12497271,     0.014319889 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.9768907,     -0.79172944 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0050804658,    0.0010520178,    0.0047452962,    0.0066458602 ],
  [               0,               0,               0,               0,               0,               0,     -0.28134421,         0.14116,     -0.79397672,       4.0710793 ],
  [               0,               0,               0,               0,               0,               0,      0.35343756,      -2.2201116,       2.1871114,      -1.8920536 ],
  [               0,               0,               0,               0,               0,               0,       1.8867049,      -1.9429163,      0.43420856,     0.030316447 ],
  [               0,               0,               0,               0,               0,               0,   -0.0020257895,   0.00086838155,    0.0004276311,   -0.0001200726 ],
  [               0,               0,               0,               0,               0,               0,      -2.1420145,     -0.79073705,     -0.46946625,       1.8013783 ],
  [               0,               0,               0,               0,               0,               0,      -13.573165,      -8.7726241,      -6.0035296,       -4.198627 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,      -5.8958088,       9.4288618,      -3.7618521 ],
  [               0,               0,               0,               0,               0,               0,               0,       294.81455,      -453.63273,       177.91051 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.5702719,       12.792343,      -14.437814 ],
  [               0,               0,               0,               0,               0,               0,               0,       48.653181,      -89.366239,       40.288198 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.7883235,      -2.8663466,       1.1488737 ],
  [               0,               0,               0,               0,               0,               0,               0,       126.86501,       -204.7078,       83.443198 ],
  [               0,               0,               0,               0,               0,               0,               0,       958.29276,       -1497.372,       582.71674 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00060493431,  -0.00098276882,   0.00080879692,    0.0081413864 ],
  [               0,               0,               0,               0,               0,               0,      0.50004727,     -0.83068099,     -0.83630407,       4.3656621 ],
  [               0,               0,               0,               0,               0,               0,      0.89312574,      -2.2373934,       1.3931879,      -1.4187845 ],
  [               0,               0,               0,               0,               0,               0,      0.83554132,      -2.7667027,       3.6908928,      -1.5859664 ],
  [               0,               0,               0,               0,               0,               0,   0.00088195353,   0.00045568724,   -0.0004166817,   7.8289358e-05 ],
  [               0,               0,               0,               0,               0,               0,     -0.90111553,       1.1804144,      0.74654916,     -0.72557147 ],
  [               0,               0,               0,               0,               0,               0,      0.65751413,     -0.90679514,     -0.93505402,       -1.729216 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.005488827,    0.0023339737 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -125.19817,       173.41282 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       16.267626,      -23.539882 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.7625682,      -3.4873829 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.011286281,     0.015990705,  -6.6970427e-05,    0.0032408153,   -0.0088464093,   0.00019446478 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,      0.17491567,     0.012006188,     -0.32917555 ],
  [               0,               0,               0,               0,               0,               0,               0,        3.570935,       15.759217,        41.63658 ],
  [               0,               0,               0,               0,               0,               0,               0,      -5.6177268,       0.7250019,       7.3848449 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.21030856,      0.30138247,      0.14247325 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.00139305,  -6.9746886e-05,   -0.0018017895 ],
  [               0,               0,               0,               0,               0,               0,               0,       20.919197,      0.71018393,       -39.09617 ],
  [               0,               0,               0,               0,               0,               0,               0,      -20.785438,      0.26230545,       41.946596 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer2_etaoffv5_55_e = [
 [
  [    -0.068086936,     0.019632984,     0.057959944,     0.051172143,     0.010256716,    -0.041535767,    -0.060457553,    0.0082000339,     0.033318601,   -0.0093199058 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00094035641,    0.0011381275,     0.001303226,    0.0018560665 ],
  [               0,               0,               0,               0,               0,               0,       4.4967048,       2.1402528,       -3.714368,      -11.989867 ],
  [               0,               0,               0,               0,               0,               0,       -2.183456,      -1.0632204,       1.5160129,       6.2370266 ],
  [               0,               0,               0,               0,               0,               0,       1.3230488,      0.69494625,     -0.89592377,      -4.5556506 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.091063768,     -0.13265593 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.3146791,       2.6952125 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -53.335215,        79.50847 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       54.044629,      -81.659849 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.020870508,    -0.031248459 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.51503271,     -0.36169244 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       31.748813,      -48.287132 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00013430221,    0.0003393252,    0.0012918096,    0.0048994424 ],
  [               0,               0,               0,               0,               0,               0,         1.19616,      -2.2280381,      -4.2118938,         10.7558 ],
  [               0,               0,               0,               0,               0,               0,       -0.115224,      0.26668691,      0.17278058,      -1.7677351 ],
  [               0,               0,               0,               0,               0,               0,     0.039479292,    -0.055985482,     -0.11055255,      0.22786723 ],
  [               0,               0,               0,               0,               0,               0,   0.00018276091,  -0.00026627516,   -0.0005619103,    0.0011276505 ],
  [               0,               0,               0,               0,               0,               0,     -0.13746791,      0.10489622,     -0.13918489,       1.2611962 ],
  [               0,               0,               0,               0,               0,               0,     -0.11060463,     0.016680446,     -0.30089669,      -1.0296378 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0078306711,    -0.012602087,    -0.030055903,     0.055315655 ],
  [               0,               0,               0,               0,               0,               0,     -0.65783589,       1.0559248,      0.45740173,       3.0371859 ],
  [               0,               0,               0,               0,               0,               0,      0.29008134,     -0.30250993,      -1.7653514,       1.0984684 ],
  [               0,               0,               0,               0,               0,               0,     -0.17149353,      0.37232586,      0.46719725,      -0.9292944 ],
  [               0,               0,               0,               0,               0,               0,    0.0013540946,   -0.0031356162,   -0.0067546557,     0.015926725 ],
  [               0,               0,               0,               0,               0,               0,      0.51298378,       -1.364605,      -2.6621642,       7.4948505 ],
  [               0,               0,               0,               0,               0,               0,      0.24109449,     -0.92430281,      -2.1427044,       4.0748044 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -4.7563352e-05,  -3.3578628e-05,   0.00012586742,   0.00082686895,    0.0038087237 ],
  [               0,               0,               0,               0,               0,     0.038829347,    -0.034789915,     -0.21398255,     -0.40754614,       4.2637915 ],
  [               0,               0,               0,               0,               0,    -0.037610339,     0.040700162,      0.19294034,      0.15356993,      -2.0850536 ],
  [               0,               0,               0,               0,               0,    -0.014035471,     0.036706039,       0.1370287,      0.15379846,      -1.0583927 ],
  [               0,               0,               0,               0,               0,    -3.79823e-05,  -4.9072216e-05,   -1.595285e-06,   0.00031008477,    0.0025780007 ],
  [               0,               0,               0,               0,               0,    0.0057250902,     -0.01167831,    -0.058854039,     -0.16396004,       1.6412205 ],
  [               0,               0,               0,               0,               0,    0.0083172291,      0.01839978,    -0.010519151,     -0.26758477,      -1.4646232 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -6.912388e-05,   0.00030118999,    0.0019008023,    0.0080305813 ],
  [               0,               0,               0,               0,               0,               0,      -1.6425094,      -1.4232582,       3.0315906,       27.632948 ],
  [               0,               0,               0,               0,               0,               0,      0.34320404,      0.23490965,     -0.83674104,       -5.729608 ],
  [               0,               0,               0,               0,               0,               0,      0.13058117,      0.11981886,      -0.1942798,      -1.9652431 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,      0.68289609,    -0.072619573,     0.011194939,   -0.0075746378,    0.0012072458 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer1_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,    0.0014848422,     -0.04092025,     0.097539341,    -0.076953061,     0.018183417 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00030772806,    0.0006564329 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.73570132,       4.0566935 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.14920372,     -0.74941624 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.76345777,     -0.57229876 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.30483581,   -0.0052471819 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.0386565,      -1.6033922 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -59.111385,       47.684807 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       18.229309,      -15.091577 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       20.383368,      -16.215115 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0070632725,    -0.020379719,     0.015343072 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.24954715,       2.8645501,     -0.90895568 ],
  [               0,               0,               0,               0,               0,               0,               0,       6.3811168,      -11.613853,       4.3200281 ],
  [               0,               0,               0,               0,               0,               0,               0,       5.3517059,       -10.77952,       4.1732213 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.2628978,       2.1781779,      -1.0326529 ],
  [               0,               0,               0,               0,               0,               0,               0,  -0.00025815411,    0.0005761362,    -0.000318809 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.3258115,       2.2135127,      0.97131245 ],
  [               0,               0,               0,               0,               0,               0,               0,       -9.894673,       21.042845,      -14.387955 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00018777482,   0.00023937826,   0.00026736288,   0.00083675941 ],
  [               0,               0,               0,               0,               0,               0,      0.18105856,        0.629064,     0.072286235,       4.6007207 ],
  [               0,               0,               0,               0,               0,               0,      -2.6879963,      0.86938646,       2.2145482,       2.1658448 ],
  [               0,               0,               0,               0,               0,               0,      0.39552884,      -0.6943081,      0.33148954,      -1.5424444 ],
  [               0,               0,               0,               0,               0,               0,     -0.57680232,    -0.084868497,      0.14901724,     -0.65993208 ],
  [               0,               0,               0,               0,               0,               0,   0.00015524193,   2.0674017e-05,   1.7058005e-06,    0.0013623507 ],
  [               0,               0,               0,               0,               0,               0,     0.039302082,    -0.070606372,      -0.1844129,       1.5876383 ],
  [               0,               0,               0,               0,               0,               0,      -1.0838175,    -0.029488745,      0.17020206,      0.14281682 ],
 ],
 [
  [               0,               0,               0,               0,     0.026474938,     -0.20199372,      0.40223038,     -0.34204003,      0.12881204,     -0.01447498 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0041220929,    0.0034522343 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.1539889,      -1.3033143 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -28.414763,       40.170186 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -67.787679,        94.97263 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.075746787,     -0.28306897 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer1_etaoffv5_35_e = [
 [
  [               0,               0,               0,               0,      0.33365205,      -1.1152255,        1.215247,     -0.47471871,     0.036206847,  -0.00016185934 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0033040202,    0.0048214514 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -7.4400541,       13.714666 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -25.750899,       36.785712 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -62.507639,       89.748361 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.15724047,     -0.32188032 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.12247879,      0.19042421 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -26.576276,       44.313443 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       14.999795,       -23.71358 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.7297408,      -5.4519598 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0017040956,    0.0017078828 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.22807239,       1.8797053 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0014425632,      -1.0245307 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.094489187,     -0.84154743 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.061395033,      0.13419824 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00062375812,  -9.7774287e-05,    0.0010602819,    0.0033804536 ],
  [               0,               0,               0,               0,               0,               0,      -1.3102452,      -2.0776972,       -0.135618,       15.840211 ],
  [               0,               0,               0,               0,               0,               0,       1.8089177,      0.99992203,      -1.9542727,      -10.393752 ],
  [               0,               0,               0,               0,               0,               0,       2.3341545,       1.0563078,      -2.7844077,      -12.325705 ],
  [               0,               0,               0,               0,               0,               0,      0.11824364,      0.22132041,     0.076635119,      -1.5297611 ],
  [               0,               0,               0,               0,               0,               0,   0.00016843506,   2.5671917e-05,  -0.00035739173,  -0.00010871193 ],
  [               0,               0,               0,               0,               0,               0,     -0.30865724,      0.17390869,      0.54315603,       1.9917475 ],
  [               0,               0,               0,               0,               0,               0,     -0.19662616,      0.30498999,      0.44367837,      -2.1698133 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0002453121,  -0.00012476918,    0.0008199806,     0.005027783 ],
  [               0,               0,               0,               0,               0,               0,     0.070807366,     0.011959021,     -0.34909866,       2.3429266 ],
  [               0,               0,               0,               0,               0,               0,    -0.098442723,      0.23993224,      0.47682827,      -2.3105737 ],
  [               0,               0,               0,               0,               0,               0,     0.074010013,    -0.088545153,     -0.35185349,     -0.67245321 ],
  [               0,               0,               0,               0,               0,               0,     -0.25204066,      0.53716335,       1.2540156,      -2.6216727 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0040711645,   0.00052804686,     0.024482802 ],
  [               0,               0,               0,               0,               0,               0,               0,      -7.4287214,      -0.2332252,       41.619287 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.8514517,     -0.11797789,       8.8595322 ],
  [               0,               0,               0,               0,               0,               0,               0,      -4.5523898,     0.039274434,       23.230968 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.23640202,      0.09548874,      -1.1174987 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0048334227,  -0.00017668504,    -0.025346293 ],
  [               0,               0,               0,               0,               0,               0,               0,       16.513681,     -0.32870479,      -88.206292 ],
  [               0,               0,               0,               0,               0,               0,               0,      -36.011081,     -0.24447555,       187.52507 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00066333868,     0.002339233 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.33133223,       3.0670669 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.13121832,      -1.4572744 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.39753004,     -0.18232842 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.19394533,      0.49145943 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   9.0652213e-05,   0.00072169424 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.8144755,       4.5365209 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.33004147,      -0.4147431 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.53514092,      -1.8979474 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.072488805,      -1.3404336 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0004324583,     0.003381883,    0.0031385824 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.67878161,     -0.13964995,       8.8327828 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.43122462,      -3.1091884,       3.3852114 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.38029043,     -0.13033868,      -1.8037004 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0001867598,    0.0004329316,    0.0027264447 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.72209298,     -0.78097135,      -1.2503055 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.38687393,     -0.23786183,      -1.3393291 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0022388788,   -0.0010340125,     0.022223529 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0055554252,    -0.069394195,       12.229812 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.089581675,    -0.044991117,      -35.050583 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.022535408,     0.051996473,     -0.68994111 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.11798514,     -0.20210629,      0.85321437 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer2_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.006098399,    0.0056493939 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.7166667,        5.597813 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.064864161,      -1.3406521 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.024733716,      -0.0397324 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0017471727,   0.00088428598 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.6109604,       1.0768032 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.9897582,      -1.7128517 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0049290041,   0.00072567371,    0.0036943631,    0.0062664452 ],
  [               0,               0,               0,               0,               0,               0,       1.9056759,      -1.0364733,      -1.1001843,       5.2553888 ],
  [               0,               0,               0,               0,               0,               0,      -1.3134002,       1.3333568,     -0.22960708,       -1.361137 ],
  [               0,               0,               0,               0,               0,               0,      0.69257385,     -0.11981569,      -0.3851551,      -1.1763185 ],
  [               0,               0,               0,               0,               0,               0,      0.51923522,     -0.17421558,      -0.2474313,     0.094292612 ],
  [               0,               0,               0,               0,               0,               0,   -0.0011022982,    0.0006529715,    7.124169e-05,  -4.0403957e-05 ],
  [               0,               0,               0,               0,               0,               0,      -6.7631335,       1.9441454,       3.2760149,       1.9479491 ],
  [               0,               0,               0,               0,               0,               0,       10.139004,      -12.743374,      -6.6745787,     -0.41768236 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.042192489,    0.0017627771,    -0.020268971 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.4761837,     -0.75633882,       5.7699691 ],
  [               0,               0,               0,               0,               0,               0,               0,       8.4770405,      0.55693115,      -7.2254876 ],
  [               0,               0,               0,               0,               0,               0,               0,      -6.7072948,      0.24044899,       4.0552584 ],
  [               0,               0,               0,               0,               0,               0,               0,  -0.00097300669,  -0.00039986039,    0.0016445544 ],
  [               0,               0,               0,               0,               0,               0,               0,      -15.930535,      -3.0895472,       12.908716 ],
  [               0,               0,               0,               0,               0,               0,               0,       27.093617,      0.45517795,      -19.051459 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,  -0.00039121358,    0.0018194143,    0.0062396071 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.5800219,      -4.4867045,       6.9486549 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.29086431,     -0.89941163,     -0.81931882 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.16227258,       0.7014565,     -0.45927848 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00069824288,  -0.00072862602,   0.00055537546 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.76328029,       1.4627711,    -0.041709947 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.5014632,      -4.7167963,        1.049799 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0028095104,    0.0044634436 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.25424956,       4.0713405 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.1899984,      -3.1871486 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.53408421,     -0.28184685 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00058693264,    0.0018264832 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.64970574,       1.2089674 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.1309938,      -1.4594451 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,       0.1585826,     -0.12338801,    -0.070546879,     0.096352359,    -0.032769129,    0.0017565965 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00063578553,  -0.00011588925,     0.009847011,     0.015871176 ],
  [               0,               0,               0,               0,               0,               0,     0.012065785,     0.013797436,    -0.011753666,       4.6464522 ],
  [               0,               0,               0,               0,               0,               0,    -0.030236005,     -0.14417227,     0.014340566,      -0.6572065 ],
  [               0,               0,               0,               0,               0,               0,    -0.019433212,     0.091108658,    -0.043386931,      -0.4070181 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer2_etaoffv5_35_e = [
 [
  [      -1.3664608,        1.422802,       1.0796026,     -0.39795818,      -1.1514517,     -0.28250151,       1.0144326,     -0.32231841,    0.0031428626,   -0.0019090764 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0065100239,   -0.0026802832,     0.006420441,      0.02551661 ],
  [               0,               0,               0,               0,               0,               0,       12.311278,       6.1950103,      -9.2301678,       -37.80205 ],
  [               0,               0,               0,               0,               0,               0,      -6.0794726,      -2.8265163,       4.4835887,       18.365987 ],
  [               0,               0,               0,               0,               0,               0,        4.036247,       2.0879208,      -2.8527301,      -13.602879 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0022763392,    0.0037588634 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.20114975,       3.2516894 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.86356477,      -2.4959795 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.30645123,     -0.44504087 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -1.8642886e-05,   0.00092916545 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        -1.63843,       1.8869063 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.25423194,     0.041409537 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -7.6380365e-05,  -8.2558016e-05,     0.001036776,    0.0063321341 ],
  [               0,               0,               0,               0,               0,               0,       0.9890069,      -1.8858878,      -3.3031229,       10.155665 ],
  [               0,               0,               0,               0,               0,               0,    -0.020441671,     0.056325598,    -0.035767467,      -1.3514682 ],
  [               0,               0,               0,               0,               0,               0,    -0.034686914,     0.048942452,      0.17595828,     -0.18313712 ],
  [               0,               0,               0,               0,               0,               0,   0.00015387098,   -0.0001395314,  -0.00041744537,   0.00098118224 ],
  [               0,               0,               0,               0,               0,               0,      0.87631708,      -1.5901966,      -2.0419641,       4.4971778 ],
  [               0,               0,               0,               0,               0,               0,     0.080400157,     -0.10030251,      -0.4607966,      -1.1028764 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0013757141,    0.0023488181,    0.0097021594,   -0.0084892911 ],
  [               0,               0,               0,               0,               0,               0,      0.27785568,     -0.67990317,        -2.68775,       9.0834482 ],
  [               0,               0,               0,               0,               0,               0,       0.0433451,      0.11823572,     -0.82571187,     -0.45419117 ],
  [               0,               0,               0,               0,               0,               0,      -0.9655999,        1.238162,       3.1082235,      -1.9899796 ],
  [               0,               0,               0,               0,               0,               0,    0.0026545023,    -0.010866033,    -0.019583871,     0.075075905 ],
  [               0,               0,               0,               0,               0,               0,      0.21248206,      -1.3613894,       2.9286839,      -2.0765058 ],
  [               0,               0,               0,               0,               0,               0,        3.557187,      -8.0888558,      -10.285459,       21.653523 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -0.00040424854,   -0.0006399541,  -0.00020071212,    0.0040055225,     0.023892378 ],
  [               0,               0,               0,               0,               0,       2.2656394,       -2.557737,      -12.595768,      -15.571197,       72.718271 ],
  [               0,               0,               0,               0,               0,      0.17509434,     -0.30527429,     -0.87940015,     -0.42505227,       3.2743844 ],
  [               0,               0,               0,               0,               0,     -0.80623609,      0.96110456,        4.728319,       5.8364388,      -27.564319 ],
  [               0,               0,               0,               0,               0,   0.00045557679,   0.00031036975,  -0.00053980784,   -0.0029527437,   -0.0066676131 ],
  [               0,               0,               0,               0,               0,      0.11207979,     -0.21094684,     -0.79023157,     -0.84686197,       5.9236312 ],
  [               0,               0,               0,               0,               0,    -0.026117961,       0.3664188,      0.86478437,     -0.22557061,      -10.967851 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00066219618,    0.0094058607 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.98494391,      0.15026185 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.7524165,      -15.420221 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.12805886,     -0.23059355 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,       2.3018468,      -1.5531467,      0.42160936,    -0.055523813,    0.0018925027 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer1_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,   -0.0084870744,    -0.027405878,      0.10092109,    -0.087494505,     0.021655446 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00030269684,   0.00066454171 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.72849981,       4.0338388 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.1805206,     -0.76900641 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.76119547,     -0.57716403 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.30772635,   -0.0036004457 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.018826512,    0.0060459124,   -0.0018105005 ],
  [               0,               0,               0,               0,               0,               0,               0,      -6.9602749,      -1.7538661,       6.4758938 ],
  [               0,               0,               0,               0,               0,               0,               0,      -15.315342,     -0.15599894,       9.2018224 ],
  [               0,               0,               0,               0,               0,               0,               0,       19.600862,      0.12582971,      -12.501452 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00028812324,    0.0024343437 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.042286968,       2.1138083 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.29630372,      -1.1863438 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.13949853,      -1.3332093 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.011352144,     -0.11222021 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   4.0828979e-05,  -4.7866701e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.7651279,       1.8500988 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -2.789771,        5.068915 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00098874077,    0.0011216929,    0.0011955139,    0.0011385686,    0.0013300939 ],
  [               0,               0,               0,               0,               0,       1.7702528,       2.4861749,       3.4916097,       4.9035838,      -24.737947 ],
  [               0,               0,               0,               0,               0,      -5.6826969,      -3.9885291,    -0.027320858,       7.7444524,        21.34278 ],
  [               0,               0,               0,               0,               0,     -0.38030023,     -0.24685629,     0.058668448,      0.65438059,      0.72605679 ],
  [               0,               0,               0,               0,               0,     -0.22278657,    -0.089919812,      0.18926654,         0.71235,       1.3322818 ],
  [               0,               0,               0,               0,               0,    0.0013604034,    0.0023931477,    0.0040418103,    0.0066368631,     0.011675807 ],
  [               0,               0,               0,               0,               0,     -0.73182968,     -0.29494665,    -0.084117462,      0.36087642,       3.2014087 ],
  [               0,               0,               0,               0,               0,     -0.87660581,     -0.22901613,   -0.0062242227,      0.44041218,       1.2580743 ],
 ],
 [
  [               0,               0,               0,               0,    0.0077709702,     -0.15183369,      0.35339273,     -0.32130985,      0.12522201,    -0.014280994 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0041865554,    0.0037069572 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.1955919,    -0.046667814 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -21.073649,       29.531114 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -66.530507,       93.185082 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.076108198,     -0.26465694 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer1_etaoffv5_37_e = [
 [
  [               0,               0,               0,               0,      0.20136006,     -0.80990882,      0.98305449,     -0.41261475,     0.033050359,   -0.0002764181 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0037218626,    0.0048116337 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -5.852502,       11.248317 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -25.868865,       36.962491 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -50.617202,       72.484029 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.16082514,      -0.3206894 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.1653609,      0.25461211 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -36.515858,       58.856991 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       14.415318,      -22.816254 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.6104256,      -5.2669482 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0017244326,    0.0017210915 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.2461507,       1.9066828 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.013358981,       -1.044038 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.084627642,     -0.85833316 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.063417266,      0.13703352 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00035464918,   3.7674347e-05,   0.00065912435,    0.0014100173 ],
  [               0,               0,               0,               0,               0,               0,      0.71825808,      0.17165549,      -1.0257221,       1.0664797 ],
  [               0,               0,               0,               0,               0,               0,     -0.34117849,     -0.16104955,      0.38897527,        1.061481 ],
  [               0,               0,               0,               0,               0,               0,      0.55109005,      0.10364503,     -0.82109502,      -3.0146621 ],
  [               0,               0,               0,               0,               0,               0,      0.33004965,      0.33258223,     -0.17257864,      -2.6438678 ],
  [               0,               0,               0,               0,               0,               0,   0.00018391444,   0.00010644077,  -0.00029612535,  -0.00056178799 ],
  [               0,               0,               0,               0,               0,               0,    -0.038493118,     0.058612379,    -0.080807812,       1.3443445 ],
  [               0,               0,               0,               0,               0,               0,     -0.16239004,      0.30671831,      0.38383441,      -2.3143595 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    -0.000243073,  -0.00012356849,   0.00081545265,     0.005006021 ],
  [               0,               0,               0,               0,               0,               0,     0.064079883,     0.027474149,     -0.31565377,       2.2693451 ],
  [               0,               0,               0,               0,               0,               0,    -0.083777708,      0.20704097,      0.41269486,      -2.1651079 ],
  [               0,               0,               0,               0,               0,               0,     0.051890553,    -0.041235356,     -0.25406272,     -0.88202577 ],
  [               0,               0,               0,               0,               0,               0,     -0.26203829,      0.55765425,       1.3024256,      -2.7220254 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0010855642,   0.00078933276,    0.0098718169 ],
  [               0,               0,               0,               0,               0,               0,               0,      -5.3613407,     -0.26399197,       30.559773 ],
  [               0,               0,               0,               0,               0,               0,               0,       -2.794487,     -0.10005423,       13.921978 ],
  [               0,               0,               0,               0,               0,               0,               0,      -4.2646133,     0.079202131,       21.902042 ],
  [               0,               0,               0,               0,               0,               0,               0,       0.6988894,      0.13749149,      -3.3683313 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00056313934,   0.00028014903,  -0.00068887442 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.42426591,     0.043028944,       1.9128752 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.5405096,     -0.37416185,       6.6089157 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0004914624,     0.002524154 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.049962063,        2.483224 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.24313205,      -1.7150178 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.82477534,      0.81925259 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.30333382,      0.74688258 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.050500195,     -0.12015527 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.4642649,       5.7155555 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.29150325,      0.18874947 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.096798675,      -1.9451544 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.27140991,      -1.1057681 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   6.1092502e-05,    0.0003291911,   0.00097352368,    0.0026381329 ],
  [               0,               0,               0,               0,               0,               0,       -1.450215,     -0.56369148,       3.7926161,       20.756703 ],
  [               0,               0,               0,               0,               0,               0,      -1.1115891,       2.0814765,       5.7045787,      -11.774275 ],
  [               0,               0,               0,               0,               0,               0,      0.12638469,      0.77528561,      0.61520807,      -7.7745727 ],
  [               0,               0,               0,               0,               0,               0,  -0.00094730024,     -0.00082032,    0.0016086113,     0.013659594 ],
  [               0,               0,               0,               0,               0,               0,     -0.46316604,     -0.43546668,      0.86155336,       8.0354258 ],
  [               0,               0,               0,               0,               0,               0,     -0.68802374,     -0.48925087,       1.5168826,       9.8432294 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0027472605,    0.0094996682,   -0.0064510111 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.048198565,     0.015044951,       11.862194 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.095534149,    -0.093101692,        3.076756 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.13517947,      0.03326737,      0.32209036 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.019692209,    -0.048202215,       1.0710624 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer2_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0060527579,    0.0056613031 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.6842737,       5.5905645 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.073138128,      -1.3427272 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.032095689,    -0.038251024 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0017188559,   0.00088863103 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.6045972,       1.0731717 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.9441293,      -1.7153582 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   6.6597117e-05,    0.0028638227,   0.00089468989,    0.0040438945 ],
  [               0,               0,               0,               0,               0,               0,      -6.2251361,        3.589279,     -0.65692532,       5.8209652 ],
  [               0,               0,               0,               0,               0,               0,      -3.2973842,       5.5753702,      -3.4369246,     -0.43941154 ],
  [               0,               0,               0,               0,               0,               0,      0.13045288,       1.7513333,      -2.5149197,     -0.33616911 ],
  [               0,               0,               0,               0,               0,               0,      0.82302219,     -0.59269361,     -0.10095291,      0.09329326 ],
  [               0,               0,               0,               0,               0,               0,   0.00039883123,  -0.00026035897,    0.0011094996,   -0.0012577446 ],
  [               0,               0,               0,               0,               0,               0,     -0.45447252,       1.4853197,      -1.6151592,       1.8806024 ],
  [               0,               0,               0,               0,               0,               0,      0.82074569,      0.44273625,     -0.94851154,      -4.5059824 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,      0.05064865,    0.0014551447,    -0.025294207 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.9126198,      -1.0897651,       7.5630701 ],
  [               0,               0,               0,               0,               0,               0,               0,       9.0263014,      0.40495281,      -7.4691379 ],
  [               0,               0,               0,               0,               0,               0,               0,      -8.2112171,      0.21168724,       5.0476164 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0020168706,  -0.00027961114,    0.0024813512 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.9531316,      -14.078274,       13.961183 ],
  [               0,               0,               0,               0,               0,               0,               0,       36.432763,     0.049671933,      -24.724241 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00014936662,  -0.00011012601,    0.0016906354,    0.0063681687 ],
  [               0,               0,               0,               0,               0,               0,      0.89188828,      -1.4181845,      -1.1724306,       5.7126229 ],
  [               0,               0,               0,               0,               0,               0,      0.25400346,     -0.70183783,       0.3563593,      -1.3321167 ],
  [               0,               0,               0,               0,               0,               0,     0.097671622,     -0.29201437,      0.63132231,     -0.35217264 ],
  [               0,               0,               0,               0,               0,               0,   0.00057874841,  -0.00025619655,  -0.00064551119,   0.00086686974 ],
  [               0,               0,               0,               0,               0,               0,     -0.48377179,      0.59041772,      0.37408069,      0.11410207 ],
  [               0,               0,               0,               0,               0,               0,      0.21195678,      -0.4880946,     -0.86081082,      -1.1062672 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0015412884,    0.0063099768 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.86212651,        4.851884 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -1.165329,     0.019562881 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.1733024,       0.2616154 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00036831407,    0.0025763854 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.8971678,      -2.2834507 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.8815591,     -0.41332124 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.14682296,     -0.10841249,    -0.065676206,     0.085709322,    -0.029301651,    0.0015087314 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00013507106,     0.002688726,    0.0010120797,     0.002289028 ],
  [               0,               0,               0,               0,               0,               0,    0.0013333654,   -0.0063290877,   -0.0023844807,       4.1982631 ],
  [               0,               0,               0,               0,               0,               0,   -0.0016250059,   -0.0025501724,    0.0018694938,      0.79043882 ],
  [               0,               0,               0,               0,               0,               0,  -0.00075660369,   -0.0042499533,    0.0021976663,      -2.4332968 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele500_layer2_etaoffv5_37_e = [
 [
  [      -1.4268175,       1.5313421,       1.1003526,      -0.4736904,      -1.2082069,      -0.2402923,       1.0855786,     -0.38597771,     0.018401817,   -0.0031662525 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0076068187,   -0.0032090205,    0.0072623752,     0.029173108 ],
  [               0,               0,               0,               0,               0,               0,       13.493138,       6.8450593,       -10.00592,      -41.947976 ],
  [               0,               0,               0,               0,               0,               0,      -6.0005939,      -2.7844198,        4.425937,       18.089397 ],
  [               0,               0,               0,               0,               0,               0,       3.8132133,       1.9848311,      -2.6832996,      -12.896243 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0024318758,    0.0039814874 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.4890577,        2.711009 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.4954573,      -4.9761345 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.56922656,     -0.88304513 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   6.1709884e-05,    0.0010578676 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -1.588824,       1.9403367 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.13152412,    0.0094464103 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00053952126,  -0.00067744854,  -0.00060051022,    0.0088289625 ],
  [               0,               0,               0,               0,               0,               0,       1.0646529,       -2.020821,      -3.7719488,       10.715564 ],
  [               0,               0,               0,               0,               0,               0,    -0.036842652,      0.07990128,     0.019363913,      -1.4026921 ],
  [               0,               0,               0,               0,               0,               0,    -0.042880129,     -0.02772647,      0.13883892,      0.22288817 ],
  [               0,               0,               0,               0,               0,               0,  -0.00043841306,  -0.00027290802,   0.00032818863,    0.0037982461 ],
  [               0,               0,               0,               0,               0,               0,       1.3103608,      -2.0764261,      -3.3377321,       5.4048992 ],
  [               0,               0,               0,               0,               0,               0,     -0.29614586,      0.49977806,      0.66715255,      -3.3981482 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00057241974,  -0.00015764566,   -0.0016002152,   -0.0018722317,    0.0096888281 ],
  [               0,               0,               0,               0,               0,     -0.39311899,      0.78905755,      0.62916344,      -2.2859833,       5.6305312 ],
  [               0,               0,               0,               0,               0,     -0.26152508,       1.5453003,     -0.50648891,      -8.1244698,       8.7438515 ],
  [               0,               0,               0,               0,               0,     -0.32686162,       1.0386596,      0.45373252,      -3.6676706,       2.5482184 ],
  [               0,               0,               0,               0,               0,   4.9355496e-05,  -9.9610905e-05,  -0.00036293695,  -0.00034183065,    0.0029958136 ],
  [               0,               0,               0,               0,               0,      0.65092422,     -0.69055106,      -2.9691857,      -3.2484107,       14.183999 ],
  [               0,               0,               0,               0,               0,      0.81164935,     -0.74962989,      -3.7525059,      -4.6540342,       15.473739 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   -0.0005574923,  -0.00074070386,   4.8609252e-06,    0.0051010618,     0.026826587 ],
  [               0,               0,               0,               0,               0,       2.2610608,      -2.5244618,      -12.498969,      -15.533777,       71.729218 ],
  [               0,               0,               0,               0,               0,     0.050945675,     -0.16143762,     -0.17871113,      0.42692775,     -0.66680512 ],
  [               0,               0,               0,               0,               0,     -0.75398923,      0.85052712,       4.3194148,       5.4817342,      -24.701206 ],
  [               0,               0,               0,               0,               0,   0.00016057738,   9.7172794e-05,  -0.00023355794,   -0.0011228779,   -0.0016733266 ],
  [               0,               0,               0,               0,               0,      0.24508433,     -0.41576913,      -1.5837358,      -1.5686945,       10.814744 ],
  [               0,               0,               0,               0,               0,      0.28070966,       0.2027736,     -0.31827785,       -2.102165,      -7.2179803 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00072119266,    0.0094176436 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.94434625,      0.23955362 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.9740419,       -15.96571 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.12103158,     -0.21314995 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,       2.2560667,      -1.4738997,      0.39037032,    -0.050806528,    0.0016885078 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]


###############################################################################
# Tables for v5 sshape correction, 1000 GeV electron clusters
#


ele1000_layer1_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,    -0.072134939,      0.18306949,     -0.15102793,     0.039205342 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00034288839,   0.00076636922 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.83378351,       3.9515033 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.24364911,     -0.92820837 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.74785526,     -0.71008669 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.29362947,    0.0009663283 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,     0.029935228,    0.0093691557,   -0.0040634973,    -0.011914583 ],
  [               0,               0,               0,               0,               0,               0,      -204.43555,      -53.094479,       41.853495,       103.99711 ],
  [               0,               0,               0,               0,               0,               0,       70.901698,       18.834214,      -15.000772,      -36.262521 ],
  [               0,               0,               0,               0,               0,               0,       46.708473,       12.510917,      -9.7110868,      -23.713115 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0084916527,    -0.024291447,     0.018102736 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.97284356,       1.1444513,   -0.0025263032 ],
  [               0,               0,               0,               0,               0,               0,               0,       5.9133656,      -10.794847,       3.9292994 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.9919228,      -8.1617196,        2.901104 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.3174467,       2.2835205,      -1.0720069 ],
  [               0,               0,               0,               0,               0,               0,               0,  -0.00012875005,    0.0002382988,  -0.00010021297 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.9035019,       5.3892044,       1.3425098 ],
  [               0,               0,               0,               0,               0,               0,               0,      -33.867321,        74.11255,      -40.833353 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -7.8457069e-05,   7.0936073e-05,   0.00011141387,   9.8632832e-05,   0.00070592711 ],
  [               0,               0,               0,               0,               0,      0.03287675,      0.21738784,      -1.1985578,      0.24909769,       4.4918187 ],
  [               0,               0,               0,               0,               0,     0.083630062,     -0.14590388,     -0.11371253,      0.16259591,    -0.018813148 ],
  [               0,               0,               0,               0,               0,     0.084455795,    -0.096318642,      0.12893882,       2.3936081,     -0.23372422 ],
  [               0,               0,               0,               0,               0,     -0.47977686,     -0.19519527,     -0.31259894,      0.26528539,     -0.54125346 ],
  [               0,               0,               0,               0,               0,   5.0315985e-05,   0.00067279069,   0.00035488275,   0.00012668417,    0.0010168911 ],
  [               0,               0,               0,               0,               0,     -0.25499576,      0.02492701,     0.051222391,     0.034869539,       2.0248837 ],
  [               0,               0,               0,               0,               0,     -0.43434782,     -0.03339454,     -0.13807903,     -0.22014128,     -0.18060062 ],
 ],
 [
  [               0,               0,     0.048226593,    -0.083106243,    -0.023038942,     0.082352858,     0.037556884,     -0.11778731,     0.062155278,   -0.0066225074 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.004181043,    0.0035930139 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.4708397,      -1.5772536 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -32.341355,       45.902666 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -81.916274,       115.07202 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.079601321,     -0.31783892 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer1_etaoffv5_55_e = [
 [
  [               0,               0,               0,               0,     0.050834666,    -0.057890899,    -0.066539009,     0.078969369,    -0.013097656,     0.003132104 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0037125171,    0.0078153992 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -11.264342,       18.954138 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -18.039005,       25.391668 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -34.387644,       48.783322 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.16477721,      -0.2863061 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.17228649,       0.2648231 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -31.357903,         51.6493 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       14.201566,      -22.500949 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.5032714,      -5.1168784 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00088087697,    0.0012158224 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.19648994,       1.6195551 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.29881116,     -0.60900129 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.42357907,     -0.34810768 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.092190318,      0.20969153 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00010599475,   0.00061065164,   0.00055419368 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.53753519,     -0.37429383,       5.3223293 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.65454551,      0.47694408,      0.12750688 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.24249296,     -0.69823691,      0.96337797 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.062507705,       0.4007743,     -0.46458072 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00084277906,  -0.00015624374,   -0.0025228517 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.48926882,     0.071753989,       2.4145125 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.2396967,      0.15517317,       2.7649618 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00028346386,  -0.00010924118,   0.00092669457,    0.0051124683 ],
  [               0,               0,               0,               0,               0,               0,     0.071383384,     0.039071519,     -0.30909488,       2.2013493 ],
  [               0,               0,               0,               0,               0,               0,     -0.13660214,      0.31125384,      0.63141714,       -2.618637 ],
  [               0,               0,               0,               0,               0,               0,      0.16282884,     -0.27818053,     -0.75296342,      0.16879338 ],
  [               0,               0,               0,               0,               0,               0,     -0.22750544,      0.48871045,       1.1384245,      -2.3907917 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,  -0.00032294837,    0.0005855663,    0.0048955821 ],
  [               0,               0,               0,               0,               0,               0,               0,      -4.7950867,     0.074685437,         26.9821 ],
  [               0,               0,               0,               0,               0,               0,               0,      -7.5464519,     -0.16953488,       38.724225 ],
  [               0,               0,               0,               0,               0,               0,               0,      -7.8471494,      0.08083993,       40.827171 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.90931814,      0.29881569,      -3.7451626 ],
  [               0,               0,               0,               0,               0,               0,               0,  -0.00021444678,   -0.0010292181,   -0.0025997597 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.50624088,      0.10417519,       2.6459585 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.29428425,   3.2893764e-05,       1.5602033 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00088026531,    0.0016557701 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.42090544,        3.449463 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.44703827,     -0.14141255 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.012245991,      -1.0993268 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.056266539,      0.17855094 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   6.5853696e-05,   0.00065845038 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -1.300038,        6.030695 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.043843865,     -0.50068078 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.30688639,      -1.7581679 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.30520383,      -1.0322041 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,  -0.00059663776,   0.00087776098,    0.0082730827 ],
  [               0,               0,               0,               0,               0,               0,               0,      -4.5786999,      0.39783911,       32.952432 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.67530025,    -0.018391459,       2.2012217 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.95134693,      0.10653485,      -5.6603009 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0001260521,   5.2087733e-05,    0.0019962718 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.65974085,     -0.20684541,       5.4042038 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.58341374,     -0.32864017,       1.9037159 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0030597011,     0.052860755 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.1153351,       17.370339 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.23660857,        20.42712 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.031078511,     -0.39275381 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0077012232,       1.1858894 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer2_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0044524697,    0.0066976455 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.5171675,       4.4592864 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.064501384,      -1.3046734 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0032174022,    -0.034373026 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00098379691,   0.00041921901 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.3559083,       1.1424623 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.7412881,      -1.8811012 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.004448769,    0.0043275532 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.8810218,       5.2776985 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.51374498,       -1.024464 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.36379312,     -0.22262772 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0013707729,    0.0013097199 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.31812591,       1.1631133 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.8885554,      -31.331667 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.037520555,    -0.021078416 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.81100488,       4.1399186 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,         6.95833,      -6.8328886 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -5.8094061,       4.5890644 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00040431178,   0.00044580939 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       25.143278,       -18.86201 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -19.017031,       12.261486 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0008959007,   0.00042947459,    0.0021472999,    0.0071876971 ],
  [               0,               0,               0,               0,               0,               0,      0.93211116,       -1.197002,      -1.2544323,        4.741819 ],
  [               0,               0,               0,               0,               0,               0,      0.71338405,       -2.142827,       1.8254216,      -1.7192676 ],
  [               0,               0,               0,               0,               0,               0,      0.22059912,      -1.3225585,        2.649302,       -1.455423 ],
  [               0,               0,               0,               0,               0,               0,   -0.0026973249,    0.0047531797,    0.0024502809,   -0.0039703141 ],
  [               0,               0,               0,               0,               0,               0,      -1.9949731,       3.5397508,      0.11224908,       -1.449699 ],
  [               0,               0,               0,               0,               0,               0,        3.128888,      -4.3163795,      -3.3541833,       1.8243866 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0011234623,    0.0080701758 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -133.51301,       184.92275 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       16.381594,      -23.704004 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.7028854,      -3.4240678 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0051937199,     0.009334245,   -0.0027441906,     0.008617661,    -0.011169852,   0.00031420951 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0058343822,   -0.0054635819,     0.036083836 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.26711313,    -0.013464168,       40.541492 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.089838912,    -0.023855827,      -2.0718159 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.070374227,      0.21538416,      0.52542011 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer2_etaoffv5_55_e = [
 [
  [     -0.26569426,      0.19482554,      0.25323199,     0.043883106,     -0.19716487,     -0.17098828,      0.17143451,    -0.037067439,      0.01193848,   -0.0028322708 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0016482045,    0.0012993361,    0.0002106896,   -0.0017591729 ],
  [               0,               0,               0,               0,               0,               0,       4.9847998,       2.4257709,       -3.943336,      -13.293899 ],
  [               0,               0,               0,               0,               0,               0,      -1.8356227,     -0.90058586,       1.2202858,       4.9835093 ],
  [               0,               0,               0,               0,               0,               0,       1.1242686,      0.60210729,     -0.73010714,      -3.8538839 ],
  [               0,               0,               0,               0,               0,               0,    0.0011324464,   0.00053053203,  -0.00092500861,   -0.0029215651 ],
  [               0,               0,               0,               0,               0,               0,       4.1139136,        2.361047,      -3.1251894,      -13.211566 ],
  [               0,               0,               0,               0,               0,               0,      -1.1383528,      -0.3927215,      0.24624888,       1.6364683 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.065068392,    -0.089123301 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        4.818891,      -4.5866644 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.48622258,     -0.49213968 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.96750285,       1.5507994 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00036035759,   0.00085365234,    0.0066827824 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.9357984,      -14.642814,       16.600806 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.065386956,     -0.20171922,      -1.1458382 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.035236668,     0.019766296,     -0.02282978 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00069636199,   -0.0011119833,   0.00038078453 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.20361068,       0.3966251,      0.34615825 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.13819569,     -0.31307436,      -1.7258691 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00018626467,   7.7195711e-05,  -0.00025833679,  -0.00075793274,   -0.0003520999,    0.0063317053 ],
  [               0,               0,               0,               0,    -0.092040256,       0.2410751,      0.25167353,     -0.87910108,      -2.7642849,        9.066622 ],
  [               0,               0,               0,               0,     -0.11740588,      0.41585466,      0.45746808,      -1.2533101,      -4.2918048,       5.7431557 ],
  [               0,               0,               0,               0,      -0.1045986,      0.26974734,      0.37945972,     -0.57701422,      -2.4493093,       3.3010687 ],
  [               0,               0,               0,               0,   0.00012437171,  -6.7432821e-05,  -0.00048853532,  -0.00099059781,  -0.00043884281,    0.0065007884 ],
  [               0,               0,               0,               0,       0.0196753,    -0.013076924,     -0.12342883,     -0.24695761,     -0.19147037,       2.7852079 ],
  [               0,               0,               0,               0,    0.0013696456,   -0.0083211794,     -0.10807082,     -0.26773062,     -0.28353237,       0.8125766 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00055309039,   -0.0013110979,   -0.0045224664,   -0.0023525886,     0.043161394 ],
  [               0,               0,               0,               0,               0,      0.99360826,      -1.1467017,      -5.6094829,      -6.9977168,       34.046177 ],
  [               0,               0,               0,               0,               0,     -0.28473057,      0.31448259,       1.6005832,       1.9620514,      -9.9607092 ],
  [               0,               0,               0,               0,               0,    -0.079780832,      0.14241405,      0.59966839,      0.64736765,      -4.2545558 ],
  [               0,               0,               0,               0,               0,   0.00040777249,  -0.00059579643,   -0.0026190412,    -0.002851973,     0.017545719 ],
  [               0,               0,               0,               0,               0,      0.17740734,     -0.13683011,     -0.81912551,      -1.3611231,       4.1501273 ],
  [               0,               0,               0,               0,               0,     -0.28096087,     -0.35035461,    -0.055720349,       1.7187093,       8.3751864 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0067300366,  -0.00085979186 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -5.3501873,       15.011656 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.58773046,      -2.6809253 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.1478299,      -2.6294291 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,       -67.20481,       88.140785,      -41.422173,       8.9914573,     -0.85959644,     0.023048713,   0.00086087666 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer1_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.014148654,    -0.022218255,    0.0073481241 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00035577891,     0.000754219 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.91417901,       4.0108215 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.16548407,     -0.88221752 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.68700979,     -0.74168583 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.27245397,    -0.011398266 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.029979611,    0.0070588126 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -64.650618,       51.958883 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       24.713514,      -20.147381 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       17.326118,       -13.78276 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0066739273,    -0.019841525,     0.015524229 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.5742995,     -0.41829753,      0.91843937 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.7668658,      -6.7361975,       2.0272509 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.9119379,      -6.0850363,        1.909333 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.0560914,       1.8241164,     -0.87292757 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00014058877,  -0.00035293293,   0.00020140658 ],
  [               0,               0,               0,               0,               0,               0,               0,      -4.6923419,       6.0090785,       4.4486178 ],
  [               0,               0,               0,               0,               0,               0,               0,       14.252159,        -30.0042,       12.723281 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00017906415,   0.00019469852,   0.00030877164,   0.00088434638 ],
  [               0,               0,               0,               0,               0,               0,     0.072713967,      0.16453341,       0.3145649,       4.4331682 ],
  [               0,               0,               0,               0,               0,               0,      0.35099633,     -0.27950534,      -1.3606181,       3.5225569 ],
  [               0,               0,               0,               0,               0,               0,       -0.905397,      0.23159499,      0.34978156,      -0.4303165 ],
  [               0,               0,               0,               0,               0,               0,     -0.49797607,     -0.24256827,     -0.10675348,     -0.60426563 ],
  [               0,               0,               0,               0,               0,               0,   1.9681146e-05,    5.495965e-05,   0.00019099681,    0.0011179253 ],
  [               0,               0,               0,               0,               0,               0,    0.0030104225,     -0.10365948,     0.018115435,       1.6677152 ],
  [               0,               0,               0,               0,               0,               0,     -0.96913016,     0.051735824,      0.07563726,      0.15491607 ],
 ],
 [
  [               0,               0,               0,               0,      0.49732976,      -1.4354724,       1.5837644,     -0.84635192,      0.22117443,    -0.019861384 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0041965569,    0.0036188762 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.3505985,      -1.4208079 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -31.370059,       44.494021 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -83.854659,       117.86154 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.077578644,     -0.31787052 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer1_etaoffv5_35_e = [
 [
  [               0,               0,               0,               0,      0.55404462,      -1.6462494,       1.6443729,     -0.60533636,      0.04818047,   -3.936736e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0034133552,    0.0067390288 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -4.6130733,       9.3699927 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -25.452395,       36.278886 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -34.220221,       48.569783 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.15767524,     -0.28012927 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.19183944,      0.29385519 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -33.476199,       54.842214 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       14.046407,      -22.264418 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.5333818,      -5.1596688 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0011114486,     0.001010104 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.036386461,       1.9471619 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.17174665,     -0.81612411 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.29368613,     -0.56291143 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.090693841,      0.20371891 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -7.3339408e-05,  -2.6829711e-05,   5.5230042e-05,   0.00060347005 ],
  [               0,               0,               0,               0,               0,               0,       1.6971385,       1.9554352,      0.34401183,       -9.991594 ],
  [               0,               0,               0,               0,               0,               0,     -0.76861707,      0.13178188,       1.9921658,       4.1748946 ],
  [               0,               0,               0,               0,               0,               0,       2.7622215,       1.5112969,      -2.6581556,      -13.524978 ],
  [               0,               0,               0,               0,               0,               0,       1.1676217,       1.0524469,     -0.84910462,      -8.4960937 ],
  [               0,               0,               0,               0,               0,               0,   0.00017119773,     9.41788e-05,  -0.00028710161,  -0.00046876133 ],
  [               0,               0,               0,               0,               0,               0,     -0.18512692,    0.0071701618,      0.16517685,       2.2837857 ],
  [               0,               0,               0,               0,               0,               0,    -0.067112916,     0.013633337,    -0.071048789,       -1.019432 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,     -0.46303493,       3.1242021,      -7.0297411,       5.2776641 ],
  [               0,               0,               0,               0,               0,               0,      -143.20016,       904.64772,      -1896.8709,       1322.2361 ],
  [               0,               0,               0,               0,               0,               0,       4.2275907,      -19.143912,       23.506553,      -5.2749237 ],
  [               0,               0,               0,               0,               0,               0,        16.94745,      -106.21097,       221.56762,      -154.94145 ],
  [               0,               0,               0,               0,               0,               0,       12.274925,      -80.586944,       176.31921,      -128.50701 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    -0.004603652,   0.00048417329,     0.027021779 ],
  [               0,               0,               0,               0,               0,               0,               0,      -10.142597,     -0.32209981,       53.408004 ],
  [               0,               0,               0,               0,               0,               0,               0,      -5.7923755,      0.24041447,       31.345206 ],
  [               0,               0,               0,               0,               0,               0,               0,      -6.2800769,      0.32429272,       33.643617 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.010471898,     0.089351684,     0.043631334 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0051767028,  -0.00025783793,    -0.027547244 ],
  [               0,               0,               0,               0,               0,               0,               0,      -13.015379,      0.21733084,       69.845823 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.0693445,      0.33503052,       12.516353 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00060849579,     0.002366929 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.51354306,       3.6396985 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.11415672,     -0.92346749 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.002608717,      -1.1230275 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.1142328,      0.31383897 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   7.6015331e-05,   0.00068318253 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -1.322725,       5.9767986 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.04994396,     -0.51599597 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.29007763,      -1.7301015 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.28473097,     -0.98323377 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0039986284,    0.0016114378,    -0.019917267 ],
  [               0,               0,               0,               0,               0,               0,               0,      -8.2031946,       1.2261757,       51.971914 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.99126662,      0.10490778,       3.8965584 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.3850269,      0.12126073,      -8.3020113 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0039899683,   0.00016107794,    -0.022825879 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.4986924,    -0.044508079,        9.908612 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.3125728,     -0.11134376,       17.411849 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00023462931,      0.04419453 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.020680948,        16.89785 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.093148507,      -17.240421 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.14600764,     0.039792936 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.020381856,       1.1487929 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer2_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0054486171,    0.0058922706 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -2.501332,       5.6402773 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.077566882,      -1.3342539 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.035487716,    -0.040514823 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0015732634,   0.00077775274 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -1.414852,        1.084722 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.5229833,      -1.7372637 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0036657726,    0.0045651069 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.3187585,       6.4238081 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.48160933,      -1.0752206 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.33043256,     -0.20948414 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0012182587,    0.0013291873 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.4373508,       1.1632804 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.4088489,      -6.4464982 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0049947608,    0.0039904718 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.6322662,     -0.13111357 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       9.7527408,      -9.1240644 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -7.3429325,       5.8309347 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00019914326,   0.00016669393 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       8.3483754,      -5.7129784 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.8595182,      -5.0403875 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0016871317,    0.0088039838 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.14053548,       4.4836554 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.32467191,      -1.0965845 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.26749692,     -0.21746949 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00083971195,  -0.00025286898 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.1467483,      0.67939896 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.28554678,      -7.8568983 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0016872242,    0.0063054211 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.83965671,       2.6659872 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.7837527,      -7.9483752 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.044984961,     0.032900451 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.19794732,     -0.16762267,    -0.087037163,      0.12732711,    -0.042358494,    0.0023757379 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,      0.13560859,     0.024877751,     -0.29789027 ],
  [               0,               0,               0,               0,               0,               0,               0,        4.767962,       8.0278993,       17.182918 ],
  [               0,               0,               0,               0,               0,               0,               0,      -37.777538,      -2.0707347,        74.13497 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.14562121,       0.1437111,      0.13033805 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer2_etaoffv5_35_e = [
 [
  [      -1.2518979,       1.1987367,       1.0487824,     -0.22977548,      -1.0444958,     -0.38718771,      0.90904312,      -0.2313878,    -0.014351215,   0.00033582516 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0013600874,    0.0013592445,    0.0010296235,   0.00054912316 ],
  [               0,               0,               0,               0,               0,               0,       7.4980198,       3.6899618,      -5.8127983,      -21.756652 ],
  [               0,               0,               0,               0,               0,               0,      -2.7120244,      -1.2522374,       1.9385708,        7.641864 ],
  [               0,               0,               0,               0,               0,               0,       1.8193675,      0.95713043,      -1.2441434,       -6.216325 ],
  [               0,               0,               0,               0,               0,               0,     0.001073121,   0.00047125082,  -0.00096800498,   -0.0029155545 ],
  [               0,               0,               0,               0,               0,               0,      0.17463208,      0.10500428,     -0.25687358,       1.0565029 ],
  [               0,               0,               0,               0,               0,               0,      -1.4890428,     -0.82664436,      0.78505878,       4.0940996 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00052188875,    0.0078925667 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.86954921,       2.0476814 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.5617131,      -5.1811033 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.16790563,       0.3738534 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,  -0.00088353802,   4.0217587e-05,     0.010370407 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.0209541,      -10.716666,       13.472455 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.13489188,      0.29989242,      -1.4948483 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.12377165,       0.5058205,     -0.39932008 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00051715083,   -0.0005556447,   1.7040572e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.3174765,      -3.5287272,       2.8738837 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.28788819,     -0.47929229,       -2.026148 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00035220376,  -0.00010908441,   -0.0011263446,   -0.0022480488,  -0.00025354558,     0.017642648 ],
  [               0,               0,               0,               0,     -0.09170908,      0.13067119,      0.25144868,     -0.15963029,      -1.1672336,       5.2092623 ],
  [               0,               0,               0,               0,    -0.098576959,      0.34656683,      0.38714747,      -1.0241876,      -3.5713499,       4.3995641 ],
  [               0,               0,               0,               0,     -0.10936109,      0.22016351,      0.38996404,     -0.24707585,      -1.7820445,        1.529807 ],
  [               0,               0,               0,               0,  -0.00019038288,   0.00010447526,   0.00068615626,    0.0012847487,   0.00036686199,   -0.0062651028 ],
  [               0,               0,               0,               0,      0.10894748,    -0.094640783,     -0.42044174,      -0.6520154,    -0.099260627,       4.7876768 ],
  [               0,               0,               0,               0,      0.35426334,    -0.098567057,      -1.2080065,      -2.7502514,      -1.9836488,       12.618601 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -2.5013865e-05,  -3.1656375e-05,   0.00017563859,    0.0014587409,    0.0075058739 ],
  [               0,               0,               0,               0,               0,      0.70360884,     -0.87577501,      -4.1236459,      -4.9568598,       26.992449 ],
  [               0,               0,               0,               0,               0,      0.16621315,     -0.21843536,     -0.90116144,     -0.97426421,       3.7867742 ],
  [               0,               0,               0,               0,               0,     -0.19659647,      0.27348346,        1.265394,       1.5228728,      -7.7085263 ],
  [               0,               0,               0,               0,               0,   0.00024004772,   0.00024142848,  -4.4897583e-05,   -0.0011742095,   -0.0033423061 ],
  [               0,               0,               0,               0,               0,     0.080852881,    -0.097054421,     -0.44684078,     -0.67929124,        3.010677 ],
  [               0,               0,               0,               0,               0,    -0.048348972,     0.055898953,      0.20976566,    -0.059756509,      -3.1319742 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00022396217,     0.010798768 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.5068877,      -1.1236978 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.8554059,      -13.210476 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.099604609,      0.32877025 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,       4.7524342,      -3.8251802,       1.1493085,     -0.14979602,    0.0058636476 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer1_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,     0.035176172,    -0.071698403,     0.046049444,    -0.010163556 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00034930576,    0.0007880054 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.88413663,       3.9156677 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.20872337,     -0.92435229 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.64656292,     -0.77389347 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.26768868,   -0.0095533535 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.030665209,    0.0060924052 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -66.542543,       53.466476 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       25.636242,      -20.883332 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        17.59915,      -13.999958 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00010912491,   1.9270557e-05,    0.0013739062 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.17416184,     -0.50747686,       3.2861983 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.32109858,        1.175977,      -1.6505308 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.3952812,      -2.6938184,    0.0068287062 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.31718624,      0.70974315,     -0.56744821 ],
  [               0,               0,               0,               0,               0,               0,               0,  -9.7027176e-05,   0.00021112503,  -0.00011263647 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.9562035,     0.010551055,       2.0707874 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.30660118,      0.52118869,      0.83211173 ],
 ],
 [
  [               0,               0,               0,               0,               0,   6.7398797e-05,   8.4135848e-05,    9.860559e-05,   0.00016706337,   0.00072268812 ],
  [               0,               0,               0,               0,               0,    -0.043385168,     -0.12857024,     -0.58723653,     -0.63382423,       7.3475048 ],
  [               0,               0,               0,               0,               0,      0.15304782,    -0.010043702,     0.040536655,      0.28753081,       1.1198736 ],
  [               0,               0,               0,               0,               0,    -0.089958111,     -0.17771999,     -0.32961493,     -0.54400022,      -1.9444121 ],
  [               0,               0,               0,               0,               0,     -0.55768348,     -0.13055458,     -0.18922273,     -0.23226789,      -0.6281683 ],
  [               0,               0,               0,               0,               0,   7.6038016e-05,   5.7028612e-05,   6.4076155e-05,     7.74616e-05,    0.0010689339 ],
  [               0,               0,               0,               0,               0,     -0.35742191,     0.011918888,     0.015906921,   0.00036036483,       2.0215342 ],
  [               0,               0,               0,               0,               0,     -0.55688963,    -0.010585705,    -0.011562675,    -0.012659193,    -0.013720355 ],
 ],
 [
  [               0,               0,               0,               0,      0.49964992,      -1.4407331,       1.5875927,     -0.84713615,      0.22104129,     -0.01982078 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0022499985,    0.0022184798,    0.0022359095 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.3962877,      0.17882959,       1.5631266 ],
  [               0,               0,               0,               0,               0,               0,               0,      -8.4266767,       -0.113478,        16.87983 ],
  [               0,               0,               0,               0,               0,               0,               0,      -29.844968,     -0.25369151,        59.35867 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.02711209,     -0.01824553,     -0.32717397 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer1_etaoffv5_37_e = [
 [
  [               0,               0,               0,               0,      0.53431233,      -1.6140352,       1.6367969,     -0.61176142,     0.049697466,  -6.1014008e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.003367872,    0.0059296003 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.0939383,       5.8296344 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -29.09148,       41.627128 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -37.381032,       53.177175 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.15403521,     -0.28143659 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.1948666,      0.29840806 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -35.001495,       57.077671 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       14.341902,      -22.712945 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.4929969,      -5.0898212 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0017678159,  -1.3214776e-06 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.24597777,       2.2760825 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.14140218,     -0.86531441 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.2713486,     -0.60061213 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.10131419,      0.21995418 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -7.2405543e-05,  -2.8540163e-05,   5.1902078e-05,   0.00060896121 ],
  [               0,               0,               0,               0,               0,               0,       1.5674624,       1.9839612,      0.24252674,      -9.1692302 ],
  [               0,               0,               0,               0,               0,               0,     -0.67332559,      0.29995153,       2.0250777,       3.1943541 ],
  [               0,               0,               0,               0,               0,               0,       2.7054228,       1.4925108,      -2.6078648,      -13.427799 ],
  [               0,               0,               0,               0,               0,               0,       1.1424484,       1.0232854,     -0.84483745,      -8.3393537 ],
  [               0,               0,               0,               0,               0,               0,   0.00016804365,   9.5190798e-05,  -0.00028000776,  -0.00046165335 ],
  [               0,               0,               0,               0,               0,               0,     -0.18558295,    0.0064089973,      0.16470373,       2.2904294 ],
  [               0,               0,               0,               0,               0,               0,    -0.079621604,    0.0063489212,    -0.052610884,     -0.92559748 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00028390485,  -0.00010862233,   0.00092854222,    0.0051134143 ],
  [               0,               0,               0,               0,               0,               0,     0.079659612,     0.030554985,     -0.33637241,       2.2185282 ],
  [               0,               0,               0,               0,               0,               0,      -0.1638734,      0.36750709,      0.74784924,      -2.8560145 ],
  [               0,               0,               0,               0,               0,               0,       0.1895395,     -0.34143967,     -0.87465832,      0.45514713 ],
  [               0,               0,               0,               0,               0,               0,     -0.20978589,      0.45162128,       1.0512819,      -2.2095776 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,  -0.00035764647,   0.00054261649,    0.0048776409 ],
  [               0,               0,               0,               0,               0,               0,               0,      -5.2036694,     0.068882536,       29.114358 ],
  [               0,               0,               0,               0,               0,               0,               0,      -9.7247551,     -0.22598811,       49.976045 ],
  [               0,               0,               0,               0,               0,               0,               0,      -8.0618406,     0.026518622,       41.713187 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.71735522,      0.14990826,      -3.4168373 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00037288989,   0.00013553152,  -0.00034840327 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.14984693,      -0.1906597,     -0.79899768 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.085737835,      0.33024189,       1.0471461 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00091846193,    0.0015660156 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.36846098,       3.3206942 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.43332185,     -0.16718342 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.062383658,      -1.2754445 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.069846084,      0.20819058 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   9.4508097e-05,    0.0007277488 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.4270633,       5.9772551 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.043059492,     -0.50216572 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.28540458,      -1.7292411 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.26852829,     -0.94458805 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.003185688,    0.0018653961,    -0.015858125 ],
  [               0,               0,               0,               0,               0,               0,               0,      -7.4254235,       1.5881612,       46.462215 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.2044339,      0.12445071,        5.148297 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.4874025,      0.12760073,      -8.9472696 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0036985343,   0.00012187537,    -0.021049439 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.5145114,    -0.028004029,       9.9742686 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.9564358,     0.032998822,       21.016202 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0093028405,     0.029474632 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.086813833,       20.122231 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.13117825,       31.475683 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.16628681,     -0.67798303 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.13080178,       1.5844874 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer2_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0054210594,    0.0059048654 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.4757183,       5.6315653 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.084072099,      -1.3357981 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.04230916,    -0.039318812 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0015457566,   0.00078042371 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.4300843,       1.0822236 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.4722113,      -1.7386419 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.003731188,    0.0045595059 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.3544033,       6.4299911 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.47140677,      -1.0812922 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.32833552,     -0.20775587 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0012118029,    0.0013252228 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.44500555,       1.1618418 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.4011322,      -6.4537354 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0076320171,   -0.0026336369 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        11.18845,      -1.6478287 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       16.100786,      -14.030743 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -13.362984,       10.473406 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0014595983,   0.00097775144 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.11720535,      0.97092717 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.8122187,       1.5814956 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   7.7482885e-05,   0.00022961182,    0.0016702818,     0.004967843 ],
  [               0,               0,               0,               0,               0,               0,       1.3099198,      -1.9514128,      -1.9421073,       6.9624805 ],
  [               0,               0,               0,               0,               0,               0,      0.26110806,     -0.36041717,     -0.32751899,      -1.0079356 ],
  [               0,               0,               0,               0,               0,               0,     -0.12459868,      0.11974589,      0.44021796,     -0.37904388 ],
  [               0,               0,               0,               0,               0,               0,  -2.5190158e-05,   -3.372779e-05,  -2.0681912e-05,   0.00074930152 ],
  [               0,               0,               0,               0,               0,               0,     -0.56522821,      0.39942114,      0.66460899,      0.36678881 ],
  [               0,               0,               0,               0,               0,               0,      0.10516732,     -0.49348058,     -0.51628698,     -0.95821907 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0026733455,    0.0045012217 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.37691811,       4.3980268 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.23900296,      -1.8995784 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.31491118,     -0.10699673 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00020355861,    0.0014556445 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.0441514,       1.8505992 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.1325752,      -1.3199234 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.19960165,     -0.16821525,    -0.090017659,      0.13421311,    -0.047171405,    0.0034202568 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     -0.14242071,     0.013526203,      0.31353746 ],
  [               0,               0,               0,               0,               0,               0,               0,       -3.446857,      0.96099473,       13.405175 ],
  [               0,               0,               0,               0,               0,               0,               0,        6.133373,       6.3006982,      -22.464612 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.067928479,      0.13420057,     -0.24613097 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
ele1000_layer2_etaoffv5_37_e = [
 [
  [      -1.3900094,       1.4009448,       1.1392488,     -0.34915844,      -1.2005588,     -0.34785272,       1.1021849,     -0.37192164,     0.016481692,   -0.0014049574 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0028108055,   -0.0013804548,    0.0023032516,     0.010693372 ],
  [               0,               0,               0,               0,               0,               0,      -3.9724081,      -1.8147708,       3.3378146,       17.854242 ],
  [               0,               0,               0,               0,               0,               0,      -3.7450166,      -1.7437526,       2.7646675,       11.191018 ],
  [               0,               0,               0,               0,               0,               0,       1.3556006,      0.72024876,     -0.91077934,      -4.6825759 ],
  [               0,               0,               0,               0,               0,               0,    0.0089300366,    0.0044976018,   -0.0062509476,    -0.027354123 ],
  [               0,               0,               0,               0,               0,               0,       1.5124813,      0.72197138,      -1.2985439,      -3.2940019 ],
  [               0,               0,               0,               0,               0,               0,      -14.458037,      -7.3550245,       10.282842,       47.117965 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00048330034,    0.0080586752 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.87675767,       2.0147325 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.6402719,      -5.2982886 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.18015179,       0.3910058 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,  -0.00093860431,   1.3035787e-06,     0.010485121 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.1576217,      -11.100124,       13.759658 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.02652942,     -0.22811702,      -1.0682091 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.16049413,      0.62097665,      -0.4887035 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00054663293,    -0.000574797,    5.549276e-07 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.2227468,      -3.3350928,       2.8509505 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.30691703,      -0.4788027,      -2.0344599 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   0.00044746397,  -0.00019191173,   -0.0015174839,   -0.0028603283,  -0.00015389187,     0.021519808 ],
  [               0,               0,               0,               0,     -0.10130752,     0.093162706,      0.29340495,      0.19383142,     -0.57433488,       3.1483075 ],
  [               0,               0,               0,               0,     -0.07722653,      0.27472559,      0.30862701,     -0.81238127,      -2.8680664,       3.2428435 ],
  [               0,               0,               0,               0,    -0.096997918,      0.17692142,      0.34488711,     -0.11319507,      -1.3521515,       0.8033225 ],
  [               0,               0,               0,               0,  -0.00027125079,   0.00017328169,   0.00098149954,    0.0017190652,   0.00031627663,   -0.0086836726 ],
  [               0,               0,               0,               0,      0.10796582,    -0.074811359,     -0.42015089,     -0.73412231,     -0.21012094,       4.9666391 ],
  [               0,               0,               0,               0,      0.24266754,    -0.070614895,     -0.80005878,      -1.7712177,      -1.3142133,       7.1074805 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -4.9026154e-05,  -8.3302857e-05,   0.00011367685,    0.0015735366,    0.0086883913 ],
  [               0,               0,               0,               0,               0,      0.89346205,      -1.0879922,      -5.1729958,      -6.2437528,       32.826416 ],
  [               0,               0,               0,               0,               0,      0.14507686,     -0.20363831,     -0.77796279,     -0.76304085,       3.0858883 ],
  [               0,               0,               0,               0,               0,     -0.21789011,       0.4088206,       1.6606315,       1.7026501,      -11.434758 ],
  [               0,               0,               0,               0,               0,   0.00030825367,    0.0003283195,   1.1177088e-05,    -0.001328783,   -0.0042235779 ],
  [               0,               0,               0,               0,               0,      0.14952988,     -0.17020562,     -0.78530322,       -1.084777,       4.5730736 ],
  [               0,               0,               0,               0,               0,     -0.16219835,    -0.043524358,      0.30530531,      0.77135584,     -0.17195944 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00015478714,     0.011457205 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.4918355,      -1.1491522 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        4.532716,      -12.405774 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.21959159,      0.62179253 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,       4.7648715,      -3.7763708,       1.1194631,     -0.14411295,    0.0055657508 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]


###############################################################################
# Tables for v5 sshape correction, 5 GeV photon clusters
#



gam5_layer1_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,    -0.023349015,     0.038756088,    -0.023372344,    0.0055834838 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.016208309,     0.037070064 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.052854279,      0.46092311 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.045889691,     -0.43121374 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.063659698,     -0.42317988 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0060270029,   0.00046567446 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.020211027,     0.014487089 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -36.019798,       29.281047 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       15.764916,      -13.101742 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       12.067401,      -9.5939795 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.011576355,    -0.029737247,     0.019697091 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.4267701,      -1.3626665,     -0.55935784 ],
  [               0,               0,               0,               0,               0,               0,               0,       11.425921,      -21.921437,       9.5602708 ],
  [               0,               0,               0,               0,               0,               0,               0,       5.7608239,      -12.530221,       5.5453375 ],
  [               0,               0,               0,               0,               0,               0,               0,       -2.871685,       5.3168428,      -2.5654529 ],
  [               0,               0,               0,               0,               0,               0,               0,  -0.00035067822,   0.00089336029,  -0.00047875188 ],
  [               0,               0,               0,               0,               0,               0,               0,       8.4544855,      -17.528529,       11.962263 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.2848103,      -6.4839462,       4.0256938 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam5_layer1_etaoffv5_55_e = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam5_layer2_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.025195237,    0.0013931151 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -3.878028,       2.8934135 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.89266656,      -1.3097999 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.017829037,     0.019014628 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0023921896,   0.00076755153 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -7.8956249,       1.4163845 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,         1.17087,      -1.8739204 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0083561126,    0.0025721141 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.1162208,       3.6042783 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.1736592,      -1.7545235 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.02110786,      0.11109195 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0001621146,   0.00015311674 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -5.0750497,       3.1568589 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       10.528544,      -8.6153887 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.094242438,    -0.070723198 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -20.009148,        18.38146 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        5.790387,      -5.9237799 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,         8.70827,      -7.0675352 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.04480979,    -0.037058847 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -8.2749346,       6.5153219 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -59.449015,       36.357536 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0020582525,   -0.0052668703,    -0.002855893,     0.011250957 ],
  [               0,               0,               0,               0,               0,               0,      -5.8256529,       22.323961,      -24.835879,       10.546783 ],
  [               0,               0,               0,               0,               0,               0,       2.0235138,       -4.448521,       1.4548949,     -0.24882621 ],
  [               0,               0,               0,               0,               0,               0,       2.0457957,      -6.6626961,       8.5780588,      -3.7260747 ],
  [               0,               0,               0,               0,               0,               0,   0.00096395663,   -0.0018111809,   -0.0017464385,    0.0025720842 ],
  [               0,               0,               0,               0,               0,               0,      -5.2626519,       5.1609763,       4.4530279,      -3.3144604 ],
  [               0,               0,               0,               0,               0,               0,      -2.6655361,       2.5486568,       4.2337529,      -4.0899393 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam5_layer2_etaoffv5_55_e = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam5_layer1_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,    -0.027058021,     0.060422335,    -0.047075564,     0.011965597 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.009397783,     0.021429758 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.062559713,       0.5595421 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.053862229,     -0.50625754 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.079582606,      -0.4949468 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.010347613,   0.00079359301 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.021589153,     0.015471159 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -34.978722,       28.435758 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       16.303371,      -13.517797 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       11.182543,      -8.8903958 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0003758713,   -0.0019072646,    0.0038515388 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.8533121,       0.3918091,      0.11788181 ],
  [               0,               0,               0,               0,               0,               0,               0,       5.1833922,      -10.355307,       4.2538701 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.5935395,      -4.1387021,        1.368109 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.6676759,        5.353447,      -2.7986572 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00013807325,  -0.00010216217,   1.9175866e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.093047754,      -0.3528675,       3.3098482 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.78633153,      -1.4559091,       1.5363929 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam5_layer1_etaoffv5_35_e = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam5_layer2_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00014887705,     0.010061942 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.2283914,       2.3457349 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.039070007,      -1.2029678 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -22.635735,       8.2814168 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.22829937,    -0.083348278 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.21520762,    -0.061514031 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.56776675,      -6.2534552 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0090862726,    0.0035925206 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.7768263,       4.1004412 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.37152866,      -1.4485935 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.068791347,      0.12974627 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0021531175,  -0.00024874876 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.0005573,      0.67937049 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        2.661343,      -3.3931527 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0088141978,    0.0014649948 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       18.606929,      -12.222193 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       10.373971,      -9.5776462 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -16.453686,       13.168075 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00070766718,   0.00082204307 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.9507169,       4.1788323 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -150.8169,       109.19989 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   3.4067906e-05,   -0.0012462083,   0.00082627287,    0.0070564281 ],
  [               0,               0,               0,               0,               0,               0,       2.1156679,     -0.93608168,      -4.0863635,       5.5485849 ],
  [               0,               0,               0,               0,               0,               0,      0.62135025,       -1.670041,      0.56096029,     -0.78242596 ],
  [               0,               0,               0,               0,               0,               0,     -0.13652277,        -0.43286,       2.0463533,      -1.3485258 ],
  [               0,               0,               0,               0,               0,               0,  -9.5030151e-06,  -0.00022294405,  -0.00034687263,    0.0010998458 ],
  [               0,               0,               0,               0,               0,               0,      -3.0540788,       3.2934358,       2.7439553,      -1.9341978 ],
  [               0,               0,               0,               0,               0,               0,      -2.0228208,      0.67333918,       1.5685164,      -1.5178365 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam5_layer2_etaoffv5_35_e = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam5_layer1_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,   -0.0017330746,    0.0013562496,   -0.0015173641,   0.00040062337 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0081483001,     0.018078489 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.057692006,      0.59873108 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.04923106,     -0.53511252 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.07724392,     -0.52264073 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.011986632,   0.00078719061 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.016009295,      0.01168197 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -41.785994,       33.927876 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       16.643401,      -13.842719 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        14.84958,      -11.805693 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,  -0.00074472974,   -0.0010450506,    0.0037376366 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.27367919,       2.2557883,     -0.41075928 ],
  [               0,               0,               0,               0,               0,               0,               0,        3.527126,      -7.2730459,       2.7794426 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.46556842,    -0.018384785,     -0.69178484 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.1989423,       4.4721225,      -2.3649573 ],
  [               0,               0,               0,               0,               0,               0,               0,   6.2832355e-05,   1.7168594e-05,  -2.4377182e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0010892292,     -0.13668147,       3.0388093 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.64212235,       -1.555158,       1.7239411 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam5_layer1_etaoffv5_37_e = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam5_layer2_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0079894008,     0.013708617 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.1911535,       1.7850683 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.5047399,     -0.99585215 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.16529599,    -0.033369666 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00078287082,   0.00029540287 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       10.234356,       0.2793722 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -8.4255195,      -6.0178725 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.012333235,    0.0025364057 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.8917892,       3.9540656 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.45237524,      -1.4522723 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.26330338,      0.28838377 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.004228653,  -8.1278177e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.34546754,      0.22591847 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.9889968,      -3.0707439 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0092185077,    0.0022008351 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       13.577484,      -8.4707225 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       10.436107,      -9.5510109 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -12.099748,        9.650064 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0018921081,  -0.00042981694 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.4907736,       2.2269751 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -114.8741,        86.61758 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00067026273,   -0.0013210777,    0.0011671535,    0.0073749944 ],
  [               0,               0,               0,               0,               0,               0,       2.7386459,      -1.3009148,       -4.754988,       5.8565766 ],
  [               0,               0,               0,               0,               0,               0,     -0.48177544,    -0.073441242,       1.0306785,      -1.6872745 ],
  [               0,               0,               0,               0,               0,               0,      0.62263349,       -1.819082,       2.3631649,      -1.0568876 ],
  [               0,               0,               0,               0,               0,               0,   -0.0014713308,     0.001399661,    0.0012425184,  -0.00057457268 ],
  [               0,               0,               0,               0,               0,               0,      -3.2576453,       3.9050626,       2.2391963,      -1.8208105 ],
  [               0,               0,               0,               0,               0,               0,      -3.5484306,       2.8802101,       4.0800769,      -4.5389267 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam5_layer2_etaoffv5_37_e = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]


###############################################################################
# Tables for v5 sshape correction, 10 GeV photon clusters
#


gam10_layer1_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,     0.024911435,     -0.05489902,     0.036570331,   -0.0076724567 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0052811251,    0.0095568935 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.020316214,      0.82534332 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.023743881,     -0.69477745 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.057631415,     -0.66305415 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.027770667,    0.0027048589 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.021973216,    0.0099745056 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -41.890669,       33.928364 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       22.518596,      -18.465345 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       13.756537,      -10.939358 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0026246144,       -0.021714,     0.030435728 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.99395159,       -1.075355,      0.97868108 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.30625622,      0.10836874,       -0.522382 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.045507551,     -0.45005015,     -0.25634751 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.054425905,      0.11111108,    -0.082776329 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0010859815,   -0.0024606943,    0.0013738693 ],
  [               0,               0,               0,               0,               0,               0,               0,       11.027017,      -17.166516,       6.5480741 ],
  [               0,               0,               0,               0,               0,               0,               0,      -99.634748,       219.99079,      -115.95212 ],
 ],
 [
  [               0,               0,               0,               0,     0.011906622,   -0.0026533289,    -0.013405756,     0.012624869,     -0.01148824,   -0.0010127419 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.02851832,     0.051749341,    -0.006900468,    -0.045228468,     0.031859856,   -0.0038783166 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.003361877,    0.0018250468 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       6.2542021,      -5.1167906 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -33.200134,       47.113038 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -39.683441,       55.006634 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.06619471,     -0.21656568 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer1_etaoffv5_55_e = [
 [
  [               0,               0,               0,               0,     -0.33671862,        1.042286,      -1.1233909,      0.49641098,     -0.09042577,    0.0064309035 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0022074528,    0.0041409796 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       19.709995,      -2.8610917 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -5.6925196,      -7.5232472 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        4.751104,       4.3304303 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.1101868,      -1.3769222 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.035813415,      0.71222144 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.46007651,       1.0315496 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.0552248,      -1.9217789 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.79753799,      -1.1857321 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.017060107,    -0.016061698 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.72006352,       2.0957547 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.29890723,      -1.2322098 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.2933202,      -1.2241521 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.049856766,     0.096288011 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00018103903,  -5.7428848e-05,   0.00023939205,    0.0012500417 ],
  [               0,               0,               0,               0,               0,               0,       1.6423015,       1.2845784,      -1.5640558,      -7.4910477 ],
  [               0,               0,               0,               0,               0,               0,      -1.0459717,     -0.46644511,       1.0004564,       3.1730895 ],
  [               0,               0,               0,               0,               0,               0,      0.41058155,     0.012174542,     -0.87291113,      -3.2926512 ],
  [               0,               0,               0,               0,               0,               0,      0.58214785,      0.55619608,     -0.35145053,      -4.3079224 ],
  [               0,               0,               0,               0,               0,               0,  -4.0566282e-05,  -2.4639072e-05,    1.481908e-05,    0.0010766048 ],
  [               0,               0,               0,               0,               0,               0,     0.056494568,      0.23828341,     -0.11746861,     -0.13397931 ],
  [               0,               0,               0,               0,               0,               0,    -0.080303952,      0.17458287,     0.094000295,      -1.9031228 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer2_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0070721926,    0.0065237311 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.97323184,       2.8355953 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.18369486,      -1.1356306 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.022328706,    -0.037993707 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00042001916,  -0.00033475288 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.0598888,      -1.2628386 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -4.2972257,      -7.9216507 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.033503488,    -0.007213574 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.8844162,       3.3137445 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.45462991,      -1.4355583 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.012448952,      0.03728802 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0074789971,    0.0037228929 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.22664013,        1.162561 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.69840188,      -1.2036591 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0044104929,    0.0034946809 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       16.523482,      -10.710703 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.8397358,      -5.1291832 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -15.175941,       12.146093 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.022210608,    -0.016517664 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.8626776,       -3.672176 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -174.37044,       140.59231 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00047787228,   0.00084486332,    0.0017918099,    0.0014681811 ],
  [               0,               0,               0,               0,               0,               0,       3.9587921,      -8.7543934,       3.0660245,       4.5025401 ],
  [               0,               0,               0,               0,               0,               0,      -0.4176945,     -0.53487895,       3.2538166,      -3.5374131 ],
  [               0,               0,               0,               0,               0,               0,       0.4764991,      -1.7229063,       2.2433488,     -0.81194997 ],
  [               0,               0,               0,               0,               0,               0,   0.00059806462,  -0.00097333258,  -0.00080224963,    0.0018241449 ],
  [               0,               0,               0,               0,               0,               0,    -0.074559035,      0.39265269,     -0.30743572,        1.010982 ],
  [               0,               0,               0,               0,               0,               0,    -0.030984882,       1.4608415,      0.87708926,      -3.4295253 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0059277577,    -0.002541986 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.7098647,       6.5058615 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.0220859,       -4.396315 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.8554166,      -1.8765236 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.079747155,     -0.21424375,      0.20882743,    -0.083168399,    0.0084347657,    0.0010139046 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.014773985,     0.018921277 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -33.270645,       52.127878 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,         21.2749,      -31.279474 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.13724535,     -0.19274191 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.068216979,    -0.096587031 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       21.453686,      -24.736179 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.0966418,      -6.7142389 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer2_etaoffv5_55_e = [
 [
  [        -2.50874,       5.6251631,      -0.5988837,      -5.6751915,      0.18275529,       7.3845854,      -6.0436485,       1.8472294,     -0.22086938,    0.0024999896 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.017936195,    -0.022265629 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       84.614643,      -125.87014 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.0346769,     0.096921761 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.10057168,      0.40154313 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0090756592,   -0.0069063091 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.23323622,       1.7634012 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.053595573,      -1.0324012 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0020697331,     0.043066645 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer1_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,     -0.01462005,      0.03345416,    -0.026306189,    0.0062188352 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0021117435,    0.0040163625 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.068825714,       1.1095222 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.066057623,     -0.78688189 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.21627803,     -0.73656834 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.064051766,    0.0049419495 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.023393458,      0.01173459 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -36.557316,       29.662606 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       20.804256,      -17.084674 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       12.110257,      -9.6298434 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,  -0.00080077688,   -0.0045963667,     0.011161882 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.6155626,      -2.3002324,       1.8543951 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.42110765,      0.48887114,     -0.87904636 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.16513526,      -0.7282835,     -0.32410689 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.031612103,     0.074254161,    -0.096298899 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0013579041,   -0.0031124041,    0.0017519895 ],
  [               0,               0,               0,               0,               0,               0,               0,        -38.0645,         85.6529,      -44.388309 ],
  [               0,               0,               0,               0,               0,               0,               0,       55.617166,      -125.13897,        67.77863 ],
 ],
 [
  [               0,               0,               0,               0,     0.011816364,   -0.0027744171,    -0.012615134,     0.008960451,   -0.0086565505,   -0.0014843307 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.089536367,      0.21272253,     -0.15562079,    0.0096911009,     0.026684032,   -0.0049047839 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0033818429,     0.001662541 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        6.666359,      -5.6504092 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -35.959292,       51.114669 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -42.360887,       58.828741 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.064736687,     -0.22651462 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer1_etaoffv5_35_e = [
 [
  [               0,               0,               0,               0,     -0.30553034,       1.0210615,      -1.1951232,        0.578696,     -0.10612676,    0.0033696547 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0090031034,   -0.0059927961 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       84.836191,      -122.64762 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       119.90682,      -180.91825 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -17.273818,       25.889513 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.060793962,     0.051072589 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.1497673,      0.61623468 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.12792384,     -0.56709601 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.13085633,     -0.57224155 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0025492538,    0.0052723442 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00036361174,  -0.00086017912,    -0.001602856,    0.0037698528 ],
  [               0,               0,               0,               0,               0,               0,       3.8952663,       4.4462193,      -1.9567891,      -27.941359 ],
  [               0,               0,               0,               0,               0,               0,      -6.4608466,      -2.0182482,        7.331129,       18.087004 ],
  [               0,               0,               0,               0,               0,               0,      -1.4973971,      -4.5554661,      -3.6399888,       17.221944 ],
  [               0,               0,               0,               0,               0,               0,       1.0021356,       2.0431058,      0.48863838,      -12.924605 ],
  [               0,               0,               0,               0,               0,               0,  -0.00061191061,  -0.00032473759,   0.00089137195,    0.0054318654 ],
  [               0,               0,               0,               0,               0,               0,     0.028313284,      0.33635489,   -0.0029816486,     -0.72702994 ],
  [               0,               0,               0,               0,               0,               0,     -0.41842224,       0.7675333,       1.3625279,      -4.0892707 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer2_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,    -0.017237427,    0.0068508481,    0.0095069164 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.3681169,     -0.89190824,        2.978209 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.1635419,      -0.6442162,       -1.176727 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.27516839,      0.19546656,    -0.020644174 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0066935039,   -0.0032103354,   0.00032579981 ],
  [               0,               0,               0,               0,               0,               0,               0,       -136.5003,       14.804913,     -0.53071322 ],
  [               0,               0,               0,               0,               0,               0,               0,      -380.97116,      -64.239888,      -13.508264 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.007254263,    0.0056636146 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.4405535,       4.8981126 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.16216015,      -1.3690966 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.20656456,    -0.075746549 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0012439185,   0.00078422541 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -4.3540348,      0.38736824 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.8581943,     -0.40340157 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.007275834,    0.0026573451 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        23.84822,       -16.19382 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.6941019,      -2.6761149 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -7.5388555,         6.32819 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0020848782,     0.001395007 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -4.5041682,       3.5656663 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -16.237094,      -9.4851146 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00079300784,  -0.00014043708,   0.00028683439,    0.0024116823 ],
  [               0,               0,               0,               0,               0,               0,       3.0455873,      -4.9498355,      -3.6097622,       9.8016026 ],
  [               0,               0,               0,               0,               0,               0,     0.066172549,     0.043627376,       0.2990322,       -1.908584 ],
  [               0,               0,               0,               0,               0,               0,     -0.38500324,      0.53963746,      0.45586776,     -0.39450455 ],
  [               0,               0,               0,               0,               0,               0,  -2.7119516e-05,  -0.00042621361,   -0.0001356853,    0.0020342926 ],
  [               0,               0,               0,               0,               0,               0,    -0.064073869,      0.27946112,     -0.25267296,       1.0055708 ],
  [               0,               0,               0,               0,               0,               0,     -0.29895793,      0.85640285,      0.19567182,      -2.2357869 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.10093456,      0.14521862 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -13.135166,       20.717297 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       6.4307734,      -10.106576 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.8252118,      -2.2337704 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.028163519,     0.032541357,     0.011943559,   -0.0039907875,   -0.0026301923,    0.0012055053 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.076473357,      0.11095029 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       218.75381,      -310.57961 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        8.655074,       2.0927843 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.8121579,      -8.7697958 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.23110459,     -0.32981523 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       77.379126,      -107.52461 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       127.97878,      -180.56368 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer2_etaoffv5_35_e = [
 [
  [       1.8510513,      -2.1129641,      -1.5870602,      0.80841925,       2.0667705,      0.29977353,       -2.330277,       1.1691612,      -0.1618816,   -0.0038523429 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0042253018,       0.0023999 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       43.172858,       -63.04576 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.6838688,      -3.8679793 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.40554928,     -0.48216325 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0060064323,  -0.00011549038 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.091528246,       2.7175407 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.054222953,      -1.0822091 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.016731993,    0.0085516975 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer1_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,    0.0017021323,   -0.0012320365,   -0.0027015186,    0.0011470511 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0021821526,    0.0040679918 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.068313685,       1.0974398 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.064265938,      -0.7814624 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.20714623,      -0.7327964 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.063798403,    0.0050264632 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.020799515,     0.012265448 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -35.803312,       29.082703 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       19.634788,      -16.169959 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       12.804637,      -10.181918 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,  -0.00054256941,  -0.00034947791,    0.0022379687 ],
  [               0,               0,               0,               0,               0,               0,               0,       5.5482497,       -8.720829,       5.9125331 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.13561447,      0.52481432,      -1.5026448 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.4554372,      -8.4273641,       2.7447828 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.34401411,      0.82434976,     -0.71912211 ],
  [               0,               0,               0,               0,               0,               0,               0,   3.3981176e-05,  -0.00011825299,   0.00014385897 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.3501422,     -0.58407405,       1.9587923 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.13598159,      0.10198538,       1.0503852 ],
 ],
 [
  [               0,               0,               0,               0,     0.013435737,   -0.0042547077,    -0.012830866,    0.0088586285,   -0.0081938502,   -0.0015734038 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.099654111,       0.2405533,     -0.18251634,     0.020500904,     0.024975873,    -0.004783911 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0033589325,    0.0015885238 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       6.6319684,      -5.5606189 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -37.475016,       53.317224 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -42.125333,       58.488798 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.064452199,     -0.23232087 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer1_etaoffv5_37_e = [
 [
  [               0,               0,               0,               0,      -0.5462184,       1.4439923,      -1.3556438,      0.51520913,    -0.065613121,   -0.0011537963 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.40296398,     -0.58290956 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -114.01208,       172.22523 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        129.3026,      -194.16808 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -47.688023,       71.009208 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.015554462,    -0.015167471 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.65413152,       2.0450229 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.17867448,      -1.0650363 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.18213697,      -1.0733999 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.059500914,      0.11433604 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00010619466,   0.00028732327,   0.00059068739,    0.0014693507 ],
  [               0,               0,               0,               0,               0,               0,     -0.13095654,      0.19548878,     0.095501671,       1.5673846 ],
  [               0,               0,               0,               0,               0,               0,     -0.34850074,      0.51310649,       1.0183706,      -2.4506855 ],
  [               0,               0,               0,               0,               0,               0,       0.2001807,     -0.43222157,     -0.76116551,       0.6233313 ],
  [               0,               0,               0,               0,               0,               0,     -0.12904955,      0.28940394,      0.54825572,      -1.1529569 ],
  [               0,               0,               0,               0,               0,               0,  -0.00014962278,  -0.00020131372,  -5.5594719e-05,     0.001811319 ],
  [               0,               0,               0,               0,               0,               0,      0.11627919,      0.11812637,     -0.37140824,     0.090021788 ],
  [               0,               0,               0,               0,               0,               0,     -0.47722804,     -0.25878947,      0.76034793,       3.9141829 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer2_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,    -0.061375572,     0.024364653,    0.0083642784 ],
  [               0,               0,               0,               0,               0,               0,               0,       15.334182,      -6.2701772,        3.368024 ],
  [               0,               0,               0,               0,               0,               0,               0,      -0.2964868,    -0.037710125,      -1.2191849 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.35006186,       0.2376926,    -0.025822192 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0084061075,   -0.0036375611,    0.0003411591 ],
  [               0,               0,               0,               0,               0,               0,               0,      -77.425935,       27.905772,      0.89028981 ],
  [               0,               0,               0,               0,               0,               0,               0,      -193.01257,       -6.984948,      -41.990546 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.011630436,    -0.000814792 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -4.9415265,       6.5914486 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.027166758,      -1.3921435 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.30295508,      -0.1003154 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0036362374,    0.0029893687 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -0.781905,       1.3008994 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.79300942,      -1.1549943 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.064794654,     0.060122483 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       33.840943,      -24.097301 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       7.6165153,      -7.3767161 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -17.897862,       14.340265 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.00524508,    0.0060618027 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        7.799184,      -5.1532995 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -184.20845,       123.39609 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00032789541,  -0.00027263963,   0.00039855419,    0.0024567485 ],
  [               0,               0,               0,               0,               0,               0,       2.8485389,      -4.6438796,      -3.0874119,       9.4387668 ],
  [               0,               0,               0,               0,               0,               0,      0.37986575,     -0.46326946,      0.15553115,      -1.6088108 ],
  [               0,               0,               0,               0,               0,               0,     -0.41013676,      0.56464741,      0.74378446,     -0.64869999 ],
  [               0,               0,               0,               0,               0,               0,   2.5008442e-06,  -0.00030619384,  -3.2812899e-05,    0.0019246564 ],
  [               0,               0,               0,               0,               0,               0,     -0.18927888,      0.41075896,     -0.10676041,      0.86133336 ],
  [               0,               0,               0,               0,               0,               0,     -0.39503294,      0.84810697,      0.29307656,      -2.2311951 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0061054765,   -0.0013690722 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -1.052617,       4.3938989 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.5048535,      -7.5795666 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.9414665,      -2.3539479 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.025327909,     0.025591609,    0.0095957598,    0.0038040759,   -0.0049297993,    0.0014723642 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00071700739,    0.0042478286 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -162.30085,       230.83977 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -9.5640817,       15.773921 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.3711563,      -2.1513656 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.12464633,     -0.17471562 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       43.551249,      -59.585256 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -10.632213,       16.364219 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam10_layer2_etaoffv5_37_e = [
 [
  [      0.66861146,     -0.85030065,     -0.57461314,      0.38637486,      0.81608687,     0.055532454,     -0.89622906,      0.44468158,      -0.0553132,   -0.0058103772 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0053160488,    0.0063050172 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       29.786371,      -43.430047 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.5516102,       2.7266598 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -1.369298,       2.1513821 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0067374693,   -0.0011745287 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.08687019,       2.6550603 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.064205223,      -1.0587904 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.016224637,    0.0092259587 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]


###############################################################################
# Tables for v5 sshape correction, 25 GeV photon clusters
#



gam25_layer1_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,    -0.014063934,      0.03686109,     -0.03135376,    0.0077208829 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00072748169,    0.0016491542 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.037660082,       1.8626604 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.042219846,     -0.95314053 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.36635382,     -0.82824695 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.12965791,    0.0059680227 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.98744643,     -0.76469534 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -51.791389,       41.814025 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       21.926421,      -18.004032 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       18.332579,      -14.580673 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,      0.08732895,     -0.13642898,     0.063662767 ],
  [               0,               0,               0,               0,               0,               0,               0,      -0.8777609,       3.9056596,       -2.826317 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.17278936,      -1.7552855,       1.3218598 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.33000766,     -0.84807843,      0.94779951 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.031092774,     -0.12148807,     0.093881191 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0039451978,    -0.012910485,    0.0099172394 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.7621073,      -5.6601136,       2.7112565 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.8649758,       7.2252564,       1.1406869 ],
 ],
 [
  [               0,               0,               0,               0,      -0.1090629,      0.39583577,     -0.50340942,      0.28904106,    -0.082500821,    0.0055104048 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.10802608,      0.22402469,     -0.12827269,    -0.020472082,     0.035698729,   -0.0046788257 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0034751069,    0.0024640186 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.2297819,      -3.4084467 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -26.049812,       36.809172 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -50.543098,       70.479919 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.067779877,     -0.25733541 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer1_etaoffv5_55_e = [
 [
  [               0,               0,               0,               0,      0.20831462,     -0.49731635,      0.43444339,     -0.18259718,     0.029010864,    0.0048209399 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.23927174,     -0.34397199 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -257.54599,       379.12905 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       178.16256,      -256.96935 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.83181054,       2.4377021 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -394.10628,       576.30626 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0018050748,    0.0035204083 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -33.948503,       54.162754 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        45.68488,      -69.515725 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       16.590583,      -24.625593 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.12182867,       1.4112596 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.12169372,      0.38227471 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.11979729,     -0.37782876 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.12044044,     -0.37890738 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   1.2538713e-05,   2.4978777e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0011173877,     0.002167829,    0.0040500074,   -0.0077911763 ],
  [               0,               0,               0,               0,               0,               0,       10.361357,       7.0915658,      -13.257536,      -78.389119 ],
  [               0,               0,               0,               0,               0,               0,     -0.71974715,      0.37650001,       1.5909315,     -0.35282927 ],
  [               0,               0,               0,               0,               0,               0,       8.0347252,       5.7605034,       -6.140225,      -42.916335 ],
  [               0,               0,               0,               0,               0,               0,      0.15542431,      0.71963606,       1.1285321,     -0.60483426 ],
  [               0,               0,               0,               0,               0,               0,   -4.028014e-05,  -0.00027777435,  -0.00025645094,    0.0023603998 ],
  [               0,               0,               0,               0,               0,               0,      0.11833162,      0.24268992,     -0.16392439,     -0.13366726 ],
  [               0,               0,               0,               0,               0,               0,      -1.5925716,        2.512902,       5.1694044,      -9.8308501 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0085844519,    -0.021160806,    -0.043450266,      0.10881345 ],
  [               0,               0,               0,               0,               0,               0,        3.548795,      -6.0440936,      -15.582677,       28.598535 ],
  [               0,               0,               0,               0,               0,               0,       2.1429765,      -4.5704701,       -9.269283,       18.664041 ],
  [               0,               0,               0,               0,               0,               0,       1.4850086,      -2.8090402,      -7.6472398,       13.674355 ],
  [               0,               0,               0,               0,               0,               0,     -0.35199586,      0.69688299,       2.1173728,      -4.2130031 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0026929818,    0.0080491746 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -20.431162,       47.950004 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       8.8096347,      -21.207444 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -5.1950264,       11.281535 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -1.940108,       4.6603118 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0002390548,   0.00080996923 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.74794689,       2.2026385 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.87486143,       1.4169415 ],
 ],
 [
  [               0,               0,               0,               0,               0,   3.5812902e-05,   8.5374665e-05,    0.0002041735,   0.00048976806,    0.0016782259 ],
  [               0,               0,               0,               0,               0,    0.0039593285,    -0.034185486,      -0.1351358,     -0.33322998,       3.4436474 ],
  [               0,               0,               0,               0,               0,    0.0096200541,     0.020830009,     0.010376399,     -0.14415979,        -1.31051 ],
  [               0,               0,               0,               0,               0,     0.030054785,   -0.0027975078,    -0.088571678,     -0.19011277,     -0.91525782 ],
  [               0,               0,               0,               0,               0,   -0.0083235827,   0.00055009792,     0.029979074,     0.094905356,     -0.11858734 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,  -5.4998178e-05,   -0.0011375798 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,  -0.00019256341,    0.0010312119,    0.0066595499 ],
  [               0,               0,               0,               0,               0,               0,               0,      -4.5553835,      0.19410115,       31.980346 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.28561465,     -0.06436807,      0.33380048 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.63920548,     0.073608336,       -3.909909 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.099663658,     -0.18095304 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -27.859074,       71.778407 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.3208812,     -0.44277816 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -64.972015,       159.71006 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.10651028,     -0.14706113 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer2_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,     0.006447639 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       3.6315104 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -1.2997677 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,   -0.0023562466 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,   0.00057531365 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      0.93409576 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -32.938956 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0045132463,    0.0096630826 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.12865055,       3.3035948 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.8691282,     -0.53398222 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.3427549,      -0.6620306 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.33035464,     -0.10090204 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.010233997,    -0.004143902 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.93163761,       1.0806208 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.95507371,      -2.2642617 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.048038995,    -0.030324719 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       13.286534,       -7.674423 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       6.3539145,      -6.4094616 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -12.311761,       10.009335 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.04353996,    -0.036247552 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -18.768404,       14.881517 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -9.7583607,       7.4839728 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,  -9.4451234e-05,   0.00094438924,    0.0044745486 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.5626908,      -4.9105778,       6.6669241 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.46192632,     -0.97869851,     -0.89308415 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.61904163,       1.7723702,     -0.97947609 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00012186509,  -0.00073836149,    0.0013822359 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.52664147,       1.2252654,      0.29298167 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.17016903,      0.39596492,      -2.0678808 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0086028075,   -0.0036247578 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -77.536768,       108.26118 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       9.0676676,      -13.817995 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        3.051425,      -3.7392939 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0032256553,    0.0043465558,   -0.0028785775,     0.010471075,   -0.0098734378,      0.00108152 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,        1.160249,      -2.5830692,       2.0616552,     -0.68468953,     0.056631146,   -0.0044233987 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer2_etaoffv5_55_e = [
 [
  [               0,               0,               0,    -0.020819445,    0.0080772833,     0.013582315,    0.0052932426,    0.0091915225,    -0.024291751,    0.0049705081 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00010116428,    0.0002855988,   0.00062417575,    0.0017223258 ],
  [               0,               0,               0,               0,               0,               0,       5.4475189,      0.23804009,      -7.5384846,      -5.5043845 ],
  [               0,               0,               0,               0,               0,               0,      0.65788383,      0.29105312,     -0.67176337,       -3.158152 ],
  [               0,               0,               0,               0,               0,               0,       1.3690653,      0.67828676,      -1.0083168,      -4.7935782 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.014401901,    -0.012792974 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       50.252572,      -74.351792 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -6.9178757,       9.3142519 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.69234556,       1.0639737 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.001134841,   -0.0038926392,     0.012271915 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.9223651,      -17.165769,       17.534558 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.6695402,       9.5604989,      -9.7444715 ],
  [               0,               0,               0,               0,               0,               0,               0,       0.8313091,      -3.0054476,       2.7448983 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0042126914,      0.01116565,   -0.0066545011 ],
  [               0,               0,               0,               0,               0,               0,               0,      -7.1824604,       21.040896,      -14.082162 ],
  [               0,               0,               0,               0,               0,               0,               0,       7.2641179,      -27.500568,       21.494305 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.020623468,     -0.06663364,     0.059660137 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.1466721,      -20.820111,       27.451147 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.19716005,    -0.094948266,      -1.7782087 ],
  [               0,               0,               0,               0,               0,               0,               0,       -1.749259,        6.804892,      -6.5125132 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0054430121,    -0.021120753,     0.021116799 ],
  [               0,               0,               0,               0,               0,               0,               0,       6.1338053,      -26.439072,       28.525958 ],
  [               0,               0,               0,               0,               0,               0,               0,       27.380319,      -108.14205,       104.40294 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -0.00069279568,  -0.00084442035,   0.00011424406,    0.0058169095,     0.028679286 ],
  [               0,               0,               0,               0,               0,       2.7176118,      -3.0560739,      -15.027372,      -18.571151,       85.336787 ],
  [               0,               0,               0,               0,               0,     -0.19416823,      0.12918421,       1.1238774,       1.8798004,      -7.4602674 ],
  [               0,               0,               0,               0,               0,     -0.86403255,       1.0883167,       5.2373066,       6.3319782,      -31.391637 ],
  [               0,               0,               0,               0,               0,  -0.00047310861,  -0.00068191539,  -0.00035366674,    0.0027010557,     0.017132367 ],
  [               0,               0,               0,               0,               0,     -0.30254934,      0.44522596,       1.4416434,       0.9740439,      -6.6841879 ],
  [               0,               0,               0,               0,               0,      0.27255234,     0.065828667,     -0.69942726,        -2.52571,      -5.4598566 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0072357204,   -0.0048818018 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -9.5772322,       25.378071 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.1244953,      -8.9140596 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.5154891,      -5.9609146 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,       -1.677037,       1.3288661,     -0.26016238,     0.015731623,    0.0010171535 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer1_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,   -0.0038641975,     0.016619949,    -0.018080954,    0.0046482245 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0010106278,    0.0023564989 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.041157607,       1.5880894 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.027384379,     -0.94813329 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.22854396,     -0.87730418 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.09181197,    0.0046682709 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.2289548,     -0.95800591 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -58.474257,       47.147681 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       25.951994,      -21.219423 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       18.769394,      -14.928043 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0034301825,   -0.0046592703,     0.013520599 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.2044074,      -1.2827983,       1.2754337 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.1153046,       1.7120492,      -1.4487189 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.3082712,       1.9243091,      -1.5378042 ],
  [               0,               0,               0,               0,               0,               0,               0,       -1.195734,        2.366254,      -1.1843605 ],
  [               0,               0,               0,               0,               0,               0,               0,  -9.5797637e-06,    -0.001075425,   0.00091773746 ],
  [               0,               0,               0,               0,               0,               0,               0,       5.1845598,      -10.352934,       5.4654694 ],
  [               0,               0,               0,               0,               0,               0,               0,      -15.578861,       39.042666,      -16.593465 ],
 ],
 [
  [               0,               0,               0,               0,    -0.095702123,      0.32412175,     -0.38974149,      0.21174896,    -0.059231791,    0.0030717982 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.26129856,      0.63779268,      -0.5228139,      0.13474673,     0.015660911,   -0.0055070036 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0039058884,    0.0025692852 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.6120585,      -4.2669314 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -29.370767,       41.549981 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -54.093956,       75.582803 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.072731068,     -0.22125604 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer1_etaoffv5_35_e = [
 [
  [               0,               0,               0,               0,    -0.070921465,     0.096475804,     0.025797275,    -0.073378081,     0.018578255,   0.00023432524 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0040920255,    -0.002361237 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -24.577416,       38.790481 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       41.471578,      -62.253063 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       105.13647,      -153.89422 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0061255632,     -0.11128998 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0020167478,  -0.00030294895 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -11.193041,       25.375802 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       97.881896,      -147.93143 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       35.314786,      -52.394833 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.24337814,       8.7772487 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.13705321,      0.33077444 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.13652948,      -0.3296529 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.13675621,     -0.33003764 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   1.2186007e-05,  -1.1898353e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00021984254,   5.6628316e-05,    0.0003915567,   0.00070655376 ],
  [               0,               0,               0,               0,               0,               0,       15.426871,      -31.017659,       -52.17263,       108.56436 ],
  [               0,               0,               0,               0,               0,               0,        46.17535,      -92.668067,      -155.56915,       310.20153 ],
  [               0,               0,               0,               0,               0,               0,       27.469721,      -45.379754,      -89.311976,       149.77059 ],
  [               0,               0,               0,               0,               0,               0,       5.8227325,      -8.2300603,      -15.184257,       20.717481 ],
  [               0,               0,               0,               0,               0,               0,   0.00012261461,   0.00025499882,  -0.00014693985,  -0.00088723772 ],
  [               0,               0,               0,               0,               0,               0,      0.76398255,      -1.6952063,      -2.9782702,       7.6621792 ],
  [               0,               0,               0,               0,               0,               0,      0.50895237,     -0.22502684,      -1.3939138,      -1.8509128 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0068245835,    -0.017289113,    -0.036115106,     0.093304675 ],
  [               0,               0,               0,               0,               0,               0,       2.0771107,      -3.3339884,      -9.1638452,       16.626001 ],
  [               0,               0,               0,               0,               0,               0,       1.1395646,      -2.6032362,      -4.4408237,       9.1587831 ],
  [               0,               0,               0,               0,               0,               0,       1.0366048,      -1.8635859,      -6.0089355,       10.210263 ],
  [               0,               0,               0,               0,               0,               0,     -0.19952622,      0.30578669,       1.6480659,      -2.9249514 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0018758947,    0.0060570929 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -30.716146,       71.835251 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       13.309654,      -31.528508 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -6.3247278,       14.056107 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.6742611,       6.4668975 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00042059015,    0.0013741522 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.88625794,       1.5731557 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.0974922,       1.3491916 ],
 ],
 [
  [               0,               0,               0,               0,               0,   2.2057172e-05,   5.2939907e-05,   0.00012746288,    0.0003077897,    0.0012452243 ],
  [               0,               0,               0,               0,               0,    -0.010141373,    -0.037056417,     -0.09627417,     -0.19576474,       3.7317291 ],
  [               0,               0,               0,               0,               0,     0.015328743,     0.017765129,    -0.012929714,     -0.18522261,      -1.2577884 ],
  [               0,               0,               0,               0,               0,     0.026137863,     0.011668675,    -0.044239098,     -0.16685197,      -1.2945582 ],
  [               0,               0,               0,               0,               0,   -0.0077138552,   -0.0030569873,     0.020137054,     0.092740734,    -0.018804548 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00012144024,   -0.0011179518 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,  -0.00023106497,    0.0010139757,    0.0068033719 ],
  [               0,               0,               0,               0,               0,               0,               0,      -4.6822824,      0.20744696,       32.800515 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.25886896,    -0.062089753,      0.17071245 ],
  [               0,               0,               0,               0,               0,               0,               0,       0.6446788,     0.073954011,      -3.9433065 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.9833728,      -9.8540372 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -37.768199,       96.061313 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2880.7013,      -7174.1541 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -26.529525,       64.663433 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.10094965,    -0.094878458 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer2_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,    0.0069553773 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       4.3360749 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -1.3305701 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,    -0.019289415 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,   0.00063806222 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       0.5284671 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -881.06277 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0089343023,   0.00096214054 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -5.4885967,        7.693545 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.27381456,      -1.5985723 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.91361913,      -1.8192717 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.29147398,     -0.11027601 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0037658669,    0.0030503625 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.24307674,      0.97129707 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.3176495,     -0.96109266 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.042081869,    -0.025820852 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        10.87612,      -5.0598359 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       6.5609338,      -6.6072879 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.0420163,       1.7244107 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.02547546,     0.023429532 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.70104902,     -0.50685451 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -14.563018,       9.7719493 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,  -0.00029286752,     0.001333956,    0.0060485403 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.3591237,      -4.3319298,       6.6418738 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.36022231,      -1.0155468,     -0.77253835 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.29691655,       1.0484578,     -0.63296021 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00047964255,   -0.0010764614,    0.0011401622 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.2031733,       2.6717053,     -0.78793346 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.30030013,     -0.42924382,      -1.0902581 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.077181656,      0.11371946 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -12.181254,       19.681118 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.6852672,      -6.3915827 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.45920195,     -0.43785625 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.04873121,     0.070353802,     0.019354739,    -0.028851678,    0.0015050963,   0.00059447733 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.34352882,      0.94559913,     -0.92232731,      0.41863251,     -0.11713468,    0.0042703369 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer2_etaoffv5_35_e = [
 [
  [               0,               0,               0,     -0.14732256,      0.18833077,     0.070047993,     -0.14603187,     0.014676295,     0.023638118,   -0.0077434126 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0018335735,  -0.00085260769,    0.0015045336,    0.0068365762 ],
  [               0,               0,               0,               0,               0,               0,       16.955565,       28.977759,     -0.20629207,      -115.07939 ],
  [               0,               0,               0,               0,               0,               0,      -29.385958,       -15.11612,       20.809732,        96.02699 ],
  [               0,               0,               0,               0,               0,               0,       23.543075,       11.629905,      -17.448655,       -77.86106 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0091995197,   -0.0039569482 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       52.986831,      -78.040674 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.79137727,       -2.471026 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.99542984,      -1.4425451 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -1.690839e-05,  -0.00039668348,    0.0090443118 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.6361072,      -15.276593,       15.743249 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.2024045,       3.8339845,      -4.2948889 ],
  [               0,               0,               0,               0,               0,               0,               0,      -0.4337236,        2.562525,      -2.7425781 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00081000634,  -0.00066617762,      0.00249178 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.0795412,      -6.6984117,       5.4290436 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.1138075,      0.92774539,     -0.73071021 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0049991902,    -0.012603805,     0.013790834 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.4451187,      -9.0055038,       15.195081 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.76857995,      -2.6642489,      0.99489494 ],
  [               0,               0,               0,               0,               0,               0,               0,      -5.2464571,       20.816375,      -20.496957 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.002914586,   -0.0053064414,   0.00042844791 ],
  [               0,               0,               0,               0,               0,               0,               0,        4.981336,      -21.451301,        23.16548 ],
  [               0,               0,               0,               0,               0,               0,               0,        13.34973,      -50.639636,       45.728552 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0098709328,    -0.018185734,     -0.07112145,     -0.06278907,      0.51946087 ],
  [               0,               0,               0,               0,               0,      0.66802013,     -0.70308511,      -3.5820935,      -4.7740777,        20.79802 ],
  [               0,               0,               0,               0,               0,     -0.17118236,      0.13582871,      0.93465543,       1.4169707,      -5.8360287 ],
  [               0,               0,               0,               0,               0,     -0.10530662,      0.10852337,      0.63432431,      0.91914247,      -3.8546048 ],
  [               0,               0,               0,               0,               0,   0.00079097002,  -0.00098666943,   -0.0046455861,   -0.0055094892,     0.027665985 ],
  [               0,               0,               0,               0,               0,     -0.49996736,      0.62234862,       3.1176152,       3.9229032,      -16.852417 ],
  [               0,               0,               0,               0,               0,       3.0547199,      -3.6348237,       -17.14401,      -20.301803,       94.791413 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0010011896,    0.0097034129 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.0044935,     -0.27351681 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.8161359,      -13.037544 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.33231468,      0.88476006 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     -0.73651436,      0.85459889,     -0.21574188,      0.01182811,  -0.00012504001 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer1_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,    -0.016599313,     0.043976743,    -0.036983606,    0.0088565543 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0012620944,    0.0027416525 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0033038488,       1.4613017 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0056752829,     -0.92137287 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.22239791,      -0.8648699 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.077975758,    0.0032596399 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.1381305,      -0.8769418 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -41.661615,       33.685602 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       21.025837,      -17.233334 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       13.476651,      -10.718277 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,      0.02395193,    -0.067720639,     0.050748127 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.7052455,      -2.1304676,        1.514916 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0047226425,     -0.35853606,     -0.44967592 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.024633389,     -0.51864317,     -0.33321718 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.18763308,      0.33933742,     -0.19883854 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00055268484,   -0.0017551345,    0.0013287432 ],
  [               0,               0,               0,               0,               0,               0,               0,        21.92547,      -41.428479,       19.407008 ],
  [               0,               0,               0,               0,               0,               0,               0,        37.31318,      -76.280622,       38.318748 ],
 ],
 [
  [               0,               0,               0,               0,     -0.15211424,      0.47760297,     -0.54473545,      0.28325683,    -0.074034143,    0.0041749164 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.26211549,      0.64779137,     -0.53886718,      0.14374824,     0.013741019,    -0.005375817 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0039468844,    0.0025476377 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.7070077,      -4.4018028 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -29.773377,       42.128202 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -55.141044,       77.077569 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.072543334,     -0.22117688 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer1_etaoffv5_37_e = [
 [
  [               0,               0,               0,               0,    -0.091451435,      0.12005367,     0.039149397,    -0.094545779,     0.022492328,   0.00061584307 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,       -1.229317,       1.8122227 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -102.0387,       149.53249 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       86.101994,      -125.92619 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       119.03083,      -174.06024 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.34917762,      -0.5510299 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.062781819,    -0.090514814 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -43.007083,       71.041255 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       101.33616,       -153.4413 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        43.75513,       -64.92867 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      -36.888218,       171.09775 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.075036828,      0.16804261 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.074993536,     -0.16795123 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.075011426,     -0.16798164 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   1.1322808e-06,  -1.2605246e-06 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0047190902,   -0.0019413506,    0.0070549992,     0.030010559 ],
  [               0,               0,               0,               0,               0,               0,      -6.2508544,       12.691397,       22.511188,       -44.11631 ],
  [               0,               0,               0,               0,               0,               0,      -8.2226276,       13.140942,        30.38362,      -50.064348 ],
  [               0,               0,               0,               0,               0,               0,       6.5403852,      -11.472852,      -24.415533,       42.187845 ],
  [               0,               0,               0,               0,               0,               0,       1.5593923,      -2.8444467,      -3.1692703,       5.8022549 ],
  [               0,               0,               0,               0,               0,               0,  -0.00069847298,    0.0018230873,    0.0027140413,   -0.0062784247 ],
  [               0,               0,               0,               0,               0,               0,        1.447572,      -1.8862811,      -4.1884423,       5.5402899 ],
  [               0,               0,               0,               0,               0,               0,        1.711371,      0.99255888,      -2.5565643,      -13.483657 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00085467583,   -0.0034286384,   -0.0055568927,     0.022536898 ],
  [               0,               0,               0,               0,               0,               0,        1.057058,      -1.4815994,        -4.55738,       8.3899954 ],
  [               0,               0,               0,               0,               0,               0,       1.0445887,      -2.2301063,      -4.7219906,       8.9833173 ],
  [               0,               0,               0,               0,               0,               0,       0.6554072,      -1.2960825,      -3.3990793,       5.7872011 ],
  [               0,               0,               0,               0,               0,               0,     -0.36631666,      0.76723233,       1.9130342,      -3.9658847 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0016386494,    0.0051526791 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -83.552142,        193.2454 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       57.006477,      -131.77073 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       25.869565,      -59.716233 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -4.6931572,       11.802355 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0017337498,    0.0048654148 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.91477611,       1.8962454 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.5791475,     0.071503961 ],
 ],
 [
  [               0,               0,               0,               0,               0,   2.0772099e-05,   4.9686663e-05,   0.00011999721,   0.00029237883,    0.0012180384 ],
  [               0,               0,               0,               0,               0,    0.0077801797,    -0.024482315,     -0.11632768,     -0.31263845,       3.4037559 ],
  [               0,               0,               0,               0,               0,    0.0066852361,   -0.0012010832,    -0.033284049,     -0.12629093,     -0.75063697 ],
  [               0,               0,               0,               0,               0,     0.018123454,    0.0029702752,    -0.042117261,     -0.11185788,      -1.0550368 ],
  [               0,               0,               0,               0,               0,   -0.0074827968,    -0.004094946,     0.016999259,     0.090879497,    0.0067004817 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    3.390276e-05,   -0.0012310629 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0001700709,   0.00099289529,    0.0063381898 ],
  [               0,               0,               0,               0,               0,               0,               0,      -4.5430742,      0.19893096,       31.935474 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.27005739,    -0.063666488,      0.24172425 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.67470639,     0.075302644,      -4.1328857 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.023354918,     0.016994589 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -24.700065,       63.777601 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.1904375,      -0.4578765 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -62.725686,       154.15174 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.10191962,      -0.1522814 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer2_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,    0.0096600388 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       3.7830278 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -1.3159183 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,   -0.0012627827 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,  -0.00052972514 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       1.3976194 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       -234.0284 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0090015993,   0.00095068078 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -5.4488494,       7.6432549 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.23541313,      -1.5671764 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.96126321,      -1.8527196 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.31786124,     -0.12974815 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0037920949,    0.0030750588 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.3027731,       1.0006129 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -1.283002,     -0.94977153 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.040311299,    -0.024376166 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       12.299717,      -6.2232298 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       6.5476396,      -6.5944868 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.2392147,       2.7327384 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.023186536,     0.020952008 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.3763138,      -1.0148953 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -19.996957,       13.824699 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,  -0.00026917489,    0.0011643653,    0.0061287503 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.3515373,      -4.3763522,       6.6974607 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.30640749,     -0.91787912,     -0.81487792 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.26542696,      0.99791708,     -0.61553616 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00044690364,  -0.00098685946,    0.0011228102 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.4014226,       3.1678334,      -1.0739607 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.23152849,     -0.44563007,       -1.080847 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   3.0939715e-05,     0.008439169 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -42.783721,       61.323692 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       6.3101657,      -9.9794039 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.5088759,      -1.8548128 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.075735439,     0.094851874,     0.031153403,    -0.044813993,    0.0063187209,   0.00025097951 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.27209595,      0.75611179,     -0.73768324,      0.34020618,     -0.10351919,    0.0036456575 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam25_layer2_etaoffv5_37_e = [
 [
  [               0,               0,               0,    -0.087944335,     0.073468199,     0.074572162,    -0.024020901,     -0.07599763,     0.044070791,   -0.0086402553 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    -0.001493928,  -0.00070116599,    0.0012106856,    0.0056382066 ],
  [               0,               0,               0,               0,               0,               0,       185.50674,       94.309204,      -133.70264,      -597.76156 ],
  [               0,               0,               0,               0,               0,               0,      -11.658431,      -6.1201016,       7.7831067,       36.564747 ],
  [               0,               0,               0,               0,               0,               0,       10.061105,       4.9470387,       -7.561674,      -33.761814 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.013906251,    -0.010507036 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       55.647241,       -82.16428 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.39111146,     -0.67228898 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.1979398,      -1.7555067 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00063111961,   -0.0012488897,    0.0079418117 ],
  [               0,               0,               0,               0,               0,               0,               0,       5.1480032,      -17.199477,       17.643651 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.19693173,      0.42816981,      -1.4583678 ],
  [               0,               0,               0,               0,               0,               0,               0,  -0.00021394269,      0.17550767,     -0.22128789 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00068234828,   -0.0017770123,    0.0015927784 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.4236157,      -5.1273955,       5.2381963 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.025546769,      -1.6996537,      0.82064311 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,      0.01141877,    -0.029746811,     0.021583056 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.6519097,       -15.26214,       22.995651 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.61837566,      -1.7166702,     -0.34593477 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.9197475,       15.786091,      -15.758986 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.003184445,    -0.010848273,    0.0099690784 ],
  [               0,               0,               0,               0,               0,               0,               0,       11.533956,      -50.308363,       54.964063 ],
  [               0,               0,               0,               0,               0,               0,               0,       30.783016,      -124.95307,        124.6039 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0065741883,    -0.012954307,    -0.049555898,    -0.042455895,      0.36902034 ],
  [               0,               0,               0,               0,               0,       1.0985991,      -1.1591493,      -5.9245973,      -7.6805296,       34.247713 ],
  [               0,               0,               0,               0,               0,     -0.03742435,      -0.0116939,      0.22675377,      0.54588661,      -2.3452362 ],
  [               0,               0,               0,               0,               0,     -0.16174294,      0.17282948,      0.95553805,       1.3191307,      -5.6715397 ],
  [               0,               0,               0,               0,               0,    0.0012100887,  -0.00067455966,   -0.0052712742,   -0.0089996026,     0.019084304 ],
  [               0,               0,               0,               0,               0,    -0.025174332,     0.071567533,      0.26868139,      0.19408558,      -1.0209436 ],
  [               0,               0,               0,               0,               0,       0.1398249,     -0.14580989,      -0.8839697,      -1.4041116,       4.2920439 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00098843695,    0.0096819156 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.0001866,     -0.23494905 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.9093589,       -13.27121 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.24519753,       0.6699013 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     -0.70299836,      0.81539477,     -0.19837638,    0.0094782315,   -1.448946e-06 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]


###############################################################################
# Tables for v5 sshape correction, 50 GeV photon clusters
#


gam50_layer1_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,    -0.025961924,     0.065575384,    -0.052966646,     0.012699496 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00046821619,   0.00098413195 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.32090047,       2.7780659 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.13662308,     -0.95371356 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.54165239,     -0.79321175 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.21230078,     0.002618925 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.030964591,     0.005330558 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -56.728906,       45.662518 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       26.492431,      -21.569754 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       15.160908,      -12.058342 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0071449835,    -0.023900317,     0.021887189 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.2484121,       3.8733321,      -1.2802066 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.6374279,      -3.6158705,       1.0689578 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.8127692,      -4.3040601,        1.465103 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.21392467,      0.36166035,     -0.19234578 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.000888068,    0.0020434472,   -0.0011580722 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.1074322,       17.036597,      -12.301794 ],
  [               0,               0,               0,               0,               0,               0,               0,       102.01174,      -216.36727,       114.66124 ],
 ],
 [
  [               0,               0,               0,               0,     -0.10620319,      0.38731261,     -0.48728803,      0.27170061,    -0.075749889,    0.0048739711 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.10038296,      0.21246104,     -0.11581226,    -0.034828764,     0.042861537,   -0.0054022953 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0045713642,    0.0039092957 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.9893662,      -2.0039104 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -23.535205,       33.076286 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -66.060119,       92.619665 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.076292376,      -0.2108968 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer1_etaoffv5_55_e = [
 [
  [               0,               0,               0,               0,      0.43847009,     -0.86584144,      0.54021714,     -0.13172044,     0.011191557,    0.0040638964 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.007758881,    0.0084195605 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       6.4618944,      -7.9864193 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -7.4919357,        10.08336 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        2.994172,      -5.3403726 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.30254425,     -0.49569418 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.30923498,      0.46650539 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       16.404371,       -19.77391 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       43.900874,      -66.335852 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       10.637789,      -15.753222 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0034203182,    0.0028545037 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.30821097,       1.6559196 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.15295081,       -1.157789 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.14644061,      -1.1366404 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.021687287,     0.052612323 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   5.5109732e-05,   0.00026375378,   0.00014964069,   -0.0011570324 ],
  [               0,               0,               0,               0,               0,               0,      -1.0187061,     -0.38629266,       1.5539283,       10.065137 ],
  [               0,               0,               0,               0,               0,               0,       1.2464201,       1.2663073,      -2.4763579,      -19.581761 ],
  [               0,               0,               0,               0,               0,               0,       1.3234753,    -0.047383962,      -3.6609969,      -12.191439 ],
  [               0,               0,               0,               0,               0,               0,     -0.41826291,      0.16132785,       1.0381769,      0.96678918 ],
  [               0,               0,               0,               0,               0,               0,   0.00088811389,   0.00068858624,   -0.0005889013,   -0.0036746743 ],
  [               0,               0,               0,               0,               0,               0,    -0.062849256,   -0.0014961141,     -0.17895058,       1.2755063 ],
  [               0,               0,               0,               0,               0,               0,     -0.75885827,       1.5082088,       2.6066307,      -6.7918905 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0051260072,    -0.012475124,    -0.025761311,     0.063479845 ],
  [               0,               0,               0,               0,               0,               0,       4.5523191,      -8.9678655,      -20.316103,        42.81059 ],
  [               0,               0,               0,               0,               0,               0,       4.3302676,      -8.2352186,      -17.627442,       32.414011 ],
  [               0,               0,               0,               0,               0,               0,        2.973234,      -5.1800718,      -13.345886,       22.397031 ],
  [               0,               0,               0,               0,               0,               0,       -0.709041,       1.4707408,       3.8209777,      -7.8772356 ],
  [               0,               0,               0,               0,               0,               0,   -0.0010822656,    0.0024296072,    0.0055564052,    -0.012147638 ],
  [               0,               0,               0,               0,               0,               0,      -5.0768864,       11.094016,       24.591787,      -52.094955 ],
  [               0,               0,               0,               0,               0,               0,       2.7825164,      -6.4359967,      -13.046934,       28.872874 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0013399537,    0.0060444863 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -15.170371,       32.393718 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -21.61646,       51.318067 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -15.879755,       38.156425 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.2447837,      -2.7279402 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00034326054,   -0.0015372849 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.0805182,       1.0403569 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.15553472,      -1.3623979 ],
 ],
 [
  [               0,               0,               0,               0,               0,   1.2658992e-05,   3.2455009e-05,   8.2243201e-05,   0.00020636494,    0.0010134212 ],
  [               0,               0,               0,               0,               0,    -0.054722816,    -0.074272313,    -0.046818154,      0.18988645,       5.1473209 ],
  [               0,               0,               0,               0,               0,     0.028071267,     0.030158581,    -0.020577754,     -0.28021396,      -1.6426051 ],
  [               0,               0,               0,               0,               0,    0.0075956078,    0.0064802696,    -0.006661815,    -0.055868731,      -1.1997468 ],
  [               0,               0,               0,               0,               0,     -0.01079157,   -0.0091770318,     0.014733802,      0.11579375,      0.15300904 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   4.8878825e-05,   0.00061731329 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.9739347,       4.8131726 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.15618846,    -0.025393859 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.063022961,     -0.85059753 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.29069355,     -0.99719865 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0057091732,    0.0010986679,    -0.029578123 ],
  [               0,               0,               0,               0,               0,               0,               0,      -11.314515,        1.415915,       70.343857 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.92883259,       2.0752338,      -1.4513968 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.0524734,      0.17590213,      -6.3861491 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0021270598,   0.00010842848,    -0.011358156 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.83475609,      0.23699871,       5.2696581 ],
  [               0,               0,               0,               0,               0,               0,               0,       -2.755136,      -1.2058165,       10.365582 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.021612421,     0.029600315 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -31.641397,       81.266321 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.2046953,     -0.88435158 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -68.255222,        167.8414 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.23114744,      0.70891533 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer2_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0076394776,    0.0081153679 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.2083426,       3.5256905 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.39405798,      -1.2397536 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.14526604,    -0.036889308 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0024199812,   6.2440154e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.46128201,      0.81507025 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -1.061399,      -1.2728814 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0023512967,    0.0024242284,    0.0049321281 ],
  [               0,               0,               0,               0,               0,               0,               0,      -7.2250879,       6.1891517,       2.5856376 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.29164759,      0.13147061,      -1.3468002 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.1219014,     -0.96376924,      0.22615002 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0026992126,   -0.0041888164,    0.0019301396 ],
  [               0,               0,               0,               0,               0,               0,               0,       -2.119441,       1.5813013,      0.66788584 ],
  [               0,               0,               0,               0,               0,               0,               0,       -8.169575,       6.4121341,      -2.8050331 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.010669233,    0.0025812135,   -0.0022282564 ],
  [               0,               0,               0,               0,               0,               0,               0,       10.600701,      -1.2081174,      -2.4613068 ],
  [               0,               0,               0,               0,               0,               0,               0,       5.9303718,      -3.0727196,      -2.5535308 ],
  [               0,               0,               0,               0,               0,               0,               0,      -4.0709558,     -0.53645959,       2.9504982 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0060993588,   -0.0001433037,   -0.0031062311 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.7685025,    -0.020995654,       3.4823206 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.6950531,      0.17592593,       1.0385408 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0015920543,    0.0023354123,   -0.0010903812,    0.0091356623 ],
  [               0,               0,               0,               0,               0,               0,       4.8680624,      -14.448121,       13.601394,      -1.0884806 ],
  [               0,               0,               0,               0,               0,               0,       1.0454654,       -2.889943,       2.2165816,      -1.7126836 ],
  [               0,               0,               0,               0,               0,               0,      -1.8966885,       2.4067453,       4.3574373,      -4.1870625 ],
  [               0,               0,               0,               0,               0,               0,    -0.014990339,     0.013505631,      0.04391955,    -0.037154665 ],
  [               0,               0,               0,               0,               0,               0,       2.5152289,      -9.4429212,       11.674706,      -4.6562031 ],
  [               0,               0,               0,               0,               0,               0,      -14.097762,       51.182176,      -61.548094,       21.420852 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.16676089,      0.23680964 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -25.907688,       38.003607 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        5.208027,      -8.5452107 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.99636046,      -1.0355661 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,    0.0054523754,   -0.0074019451,   -0.0047648368,     0.004008056,    0.0071840393,   0.00010982539,    0.0014325507,    -0.006772569,   0.00033661981 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0081774779,    0.0053675961,    0.0014879436,   -0.0022365135 ],
  [               0,               0,               0,               0,               0,               0,     -0.37666535,       2.0136885,       4.6778861,       12.060458 ],
  [               0,               0,               0,               0,               0,               0,       9.1764412,      -14.230806,      -11.545339,       16.900824 ],
  [               0,               0,               0,               0,               0,               0,      0.68713125,      0.26129513,     -0.45924271,      -1.3976304 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer2_etaoffv5_55_e = [
 [
  [    -0.030994628,    0.0057707942,     0.022966091,     0.020790503,    0.0036847208,    -0.016226609,    -0.016442147,     0.021886017,    -0.012584956,   -0.0015272463 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00032903281,    0.0009118919,    0.0019438519,    0.0042057917 ],
  [               0,               0,               0,               0,               0,               0,       8.3601989,       4.4698271,      -6.2446782,      -26.170182 ],
  [               0,               0,               0,               0,               0,               0,      0.23956193,     0.068069074,     -0.31216597,      -1.4565159 ],
  [               0,               0,               0,               0,               0,               0,     -0.64987564,     -0.30690425,      0.51932326,       1.9150779 ],
  [               0,               0,               0,               0,               0,               0,    0.0036915332,    0.0018688197,   -0.0026570942,    -0.011110064 ],
  [               0,               0,               0,               0,               0,               0,       3.5268481,       1.6901462,      -2.9222548,      -10.314378 ],
  [               0,               0,               0,               0,               0,               0,       5.0159044,       2.5030365,      -3.5600784,       -15.99046 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0011502758,    0.0053533402 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       52.338887,      -76.663634 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.55297157,     -0.38372601 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.8046674,       2.8190572 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0015651094,   -0.0013616271,    0.0018930801,     0.015017544 ],
  [               0,               0,               0,               0,               0,               0,         5.39649,      -7.2236261,      -18.065108,       28.204098 ],
  [               0,               0,               0,               0,               0,               0,     -0.12613402,      0.39419005,     -0.90079715,      -0.3024938 ],
  [               0,               0,               0,               0,               0,               0,     0.089921231,     0.032786088,     -0.48539685,      0.38421781 ],
  [               0,               0,               0,               0,               0,               0,    0.0011903864,  -0.00096466001,   -0.0025639121,    0.0019511032 ],
  [               0,               0,               0,               0,               0,               0,    -0.029996054,      0.17621345,     -0.17438393,      0.86928047 ],
  [               0,               0,               0,               0,               0,               0,      0.15471403,      0.19373822,     -0.52963206,      -2.3216188 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   8.4847536e-05,   0.00032967891,    0.0010459409,    0.0053500628 ],
  [               0,               0,               0,               0,               0,               0,      0.16724606,     -0.43723196,      -2.3174211,       7.5604278 ],
  [               0,               0,               0,               0,               0,               0,      0.22880631,      -1.0107765,       1.5633596,      -2.1125422 ],
  [               0,               0,               0,               0,               0,               0,      -3.9124233,       7.7905024,       17.193856,      -33.936467 ],
  [               0,               0,               0,               0,               0,               0,   0.00077126958,   -0.0011008166,   -0.0022518086,     0.011507558 ],
  [               0,               0,               0,               0,               0,               0,     0.031251015,     -0.35943625,       1.0855509,     -0.99357059 ],
  [               0,               0,               0,               0,               0,               0,      -4.4106883,       9.3286584,       17.190159,      -40.841058 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -0.00094495446,   -0.0011865262,   5.4231956e-05,    0.0077826805,     0.039067322 ],
  [               0,               0,               0,               0,               0,       2.8932635,      -3.2213897,       -15.95058,      -19.835981,       90.074793 ],
  [               0,               0,               0,               0,               0,     -0.26049385,      0.21213186,       1.4662511,        2.224617,      -9.2055029 ],
  [               0,               0,               0,               0,               0,     -0.63645131,      0.74004924,       3.6953148,       4.6234314,      -21.398234 ],
  [               0,               0,               0,               0,               0,   0.00024287011,   7.1032909e-06,  -0.00068093211,    -0.001678412,   0.00072702451 ],
  [               0,               0,               0,               0,               0,        0.199588,     -0.35185896,      -1.2915359,      -1.2577322,       8.9109121 ],
  [               0,               0,               0,               0,               0,    -0.016251325,      0.68423987,       1.5841054,     -0.32817217,      -19.497581 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    -0.035255995,  -0.00067452866,       0.2284876 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.2474406,    -0.034789728,       15.688724 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.2281822,    -0.089558668,      -8.4594159 ],
  [               0,               0,               0,               0,               0,               0,               0,       0.2157105,     0.037120359,      -1.1664551 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.023002543,    -0.022290626,      -2.2523278,       1.9990566,     -0.44643891,     0.028920944,   0.00066183539 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer1_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,    -0.078600562,      0.18855678,     -0.14677724,     0.035957663 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00048508162,    0.0010017522 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.35747786,       2.7653838 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.16465307,      -0.9769305 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.51228933,     -0.81032353 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.20804075,    0.0032396096 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.027408356,    0.0042072306 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -61.962525,       49.854763 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       26.920173,       -21.93316 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       17.414748,      -13.850844 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,  -0.00061732901,  -2.4436364e-05,    0.0027389155 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.28566829,      0.89085155,         1.58867 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.19900998,      0.12688491,      -1.3225853 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.80316485,      -1.4723204,     -0.63136295 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.49565909,       1.0192494,     -0.63484528 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00012996965,  -0.00034166014,   0.00024990432 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.8898841,       -8.084955,       5.8238649 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.7493769,      -10.920669,       7.2878823 ],
 ],
 [
  [               0,               0,               0,               0,     -0.12365044,      0.41235016,     -0.49016389,      0.26135812,    -0.070554289,    0.0042104712 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.30941871,      0.72560902,     -0.55249157,      0.10858429,     0.033332431,   -0.0076397723 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0044483769,    0.0037626655 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.6564113,      -2.9120876 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -27.730506,       39.157272 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -60.733083,       85.001885 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.076943872,     -0.21845536 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer1_etaoffv5_35_e = [
 [
  [               0,               0,               0,               0,    -0.042046227,     0.042452571,     0.019356786,    -0.016409552,   -0.0065658953,   -0.0012230319 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.011605539,    0.0033694366 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.1913332,      -3.0909539 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.65264151,    -0.056743956 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.57963216,      -1.9358568 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.31491133,     -0.51166886 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.22410111,      0.33952001 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.6852816,       3.9110037 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       28.993283,      -44.384568 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       6.6324482,      -9.7710799 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.085721975,      0.27724023 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.16762076,      0.56327975 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.16356117,     -0.55180263 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.16439514,     -0.55302733 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00028633427,   0.00080808421 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   5.7498274e-05,   0.00024954636,   0.00014409049,   -0.0010220764 ],
  [               0,               0,               0,               0,               0,               0,      -1.0299709,     -0.42696948,       1.4994557,       10.246272 ],
  [               0,               0,               0,               0,               0,               0,      0.57146727,       0.9260757,      -1.3013387,       -13.26924 ],
  [               0,               0,               0,               0,               0,               0,      0.62984133,      -0.4686567,      -2.4234129,       -5.686866 ],
  [               0,               0,               0,               0,               0,               0,     -0.53841096,       0.1706195,       1.2429201,       1.3048181 ],
  [               0,               0,               0,               0,               0,               0,   0.00070370494,   0.00043622613,  -0.00055063736,   -0.0019582769 ],
  [               0,               0,               0,               0,               0,               0,     -0.01632872,    -0.025965737,     -0.29560523,       1.2496453 ],
  [               0,               0,               0,               0,               0,               0,     -0.87621348,       1.7878598,       3.0956042,      -7.9351311 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -6.1021368e-05,   2.6202327e-05,   0.00029425108,    0.0014125199 ],
  [               0,               0,               0,               0,               0,               0,    -0.033895153,    -0.099430823,     -0.12123915,       4.2808357 ],
  [               0,               0,               0,               0,               0,               0,     -0.39156252,      0.17162515,      0.90696997,      -1.0155267 ],
  [               0,               0,               0,               0,               0,               0,     0.088722985,     -0.59018302,     -0.97416628,       1.9813625 ],
  [               0,               0,               0,               0,               0,               0,    -0.080612259,      0.30654741,      0.57995538,      -1.7211713 ],
  [               0,               0,               0,               0,               0,               0,   0.00023694561,   0.00018121374,  -0.00032112075,   -0.0012040478 ],
  [               0,               0,               0,               0,               0,               0,    -0.029228715,     0.043865527,     -0.15819029,       1.0800056 ],
  [               0,               0,               0,               0,               0,               0,     -0.23824727,        0.439398,      0.78961565,      -2.9438009 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.053434241,     -0.11622871 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -23.116823,       50.989944 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -19.172968,       45.712994 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -15.396343,       36.905329 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.72546705,      -1.5154895 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.025969687,    -0.061394449 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.7888657,      -14.716161 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.2835564,       -11.74591 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00014781782,   0.00019933021,   0.00010746844,  -0.00062648096,   -0.0031103322 ],
  [               0,               0,               0,               0,               0,    -0.095568049,     -0.12363337,    -0.052641068,      0.43543352,       6.3314385 ],
  [               0,               0,               0,               0,               0,     0.014745231,     0.022681877,   -0.0014461626,     -0.19489848,        -1.45602 ],
  [               0,               0,               0,               0,               0,   -0.0022303766,    -0.011991816,    -0.023207561,     0.005404015,     -0.73500192 ],
  [               0,               0,               0,               0,               0,    -0.013091055,   -0.0093659165,     0.020535236,      0.13003892,      0.15509096 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.044250601,     -0.10571897 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.16648598,       5.0613843 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       32.983582,      -79.011746 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       9.9580081,      -26.577596 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.98236001,      -1.3868888 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0059186475,    0.0011085961,    -0.030657632 ],
  [               0,               0,               0,               0,               0,               0,               0,      -11.153166,       1.3461185,       69.443664 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.96852375,       2.5314009,      -2.2905108 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.97550984,      0.34190896,      -6.3472549 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0022944852,   8.7733683e-05,    -0.012375186 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.84725411,      0.25555984,       5.2838743 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.7864718,      -1.2676566,       10.654097 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.021250684,     0.027683769 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -37.124797,       94.930024 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.2479275,      -0.9521259 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -68.708464,       168.95791 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.24514343,      0.74777603 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer2_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0060872566,    0.0053873146 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.6759825,       5.3408303 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.17152462,      -1.3738313 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.13620578,   -0.0058711597 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0011876703,   0.00096311765 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.5405754,      0.96674267 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.5294879,      -1.8148215 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00088924474,   0.00043899902,    0.0065465024 ],
  [               0,               0,               0,               0,               0,               0,               0,      -8.6234457,       8.4654945,       2.5253354 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.70973716,      0.68874498,      -1.5486352 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.1300306,      -1.0445544,      0.25752432 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0030268006,   -0.0032082302,    0.0013928806 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.5862987,      0.74348449,      0.69818013 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.55071131,      -1.7313405,       -7.274146 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.014405721,     0.003026967,   -0.0032095604 ],
  [               0,               0,               0,               0,               0,               0,               0,       10.156707,      -3.7358434,      0.25369057 ],
  [               0,               0,               0,               0,               0,               0,               0,       17.684625,      -15.019374,     -0.63794444 ],
  [               0,               0,               0,               0,               0,               0,               0,      -16.020498,      -2.6534748,       12.324624 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.017873864,     0.001073553,   -0.0087387173 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.44332908,      -1.6411131,       1.6519117 ],
  [               0,               0,               0,               0,               0,               0,               0,       43.015479,       -13.43755,      -24.601852 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0017310664,   0.00088335385,    0.0041987422,    0.0046041375 ],
  [               0,               0,               0,               0,               0,               0,       2.7426284,      -6.4294558,        3.322994,       4.0909061 ],
  [               0,               0,               0,               0,               0,               0,     -0.34857412,       1.0512734,      -1.2830963,     -0.81937318 ],
  [               0,               0,               0,               0,               0,               0,     -0.51838937,       1.9743326,      -1.6425007,      0.34712845 ],
  [               0,               0,               0,               0,               0,               0,   0.00098187997,   0.00011231026,   3.2869142e-06,   -7.387351e-05 ],
  [               0,               0,               0,               0,               0,               0,      0.56640537,      -1.0954194,     0.080099657,      0.73447067 ],
  [               0,               0,               0,               0,               0,               0,       1.4230041,      -1.6480418,       -2.915138,      0.69193445 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.087631476,      0.12804626 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -17.413256,       27.084271 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       6.1437855,      -9.7609802 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.034150398,      0.11339992 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,     -0.28137903,      0.19230629,      0.20160264,     0.021070437,      -0.1193276,    -0.077844147,     0.097510025,    -0.028824728,    0.0015096697 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,      0.16155139,     0.082779947,    -0.097565509,     -0.45336604 ],
  [               0,               0,               0,               0,               0,               0,     -0.40594691,      0.05955747,       0.1834876,       3.6218263 ],
  [               0,               0,               0,               0,               0,               0,      -2.1986729,       5.9798559,       6.3214297,      -15.871625 ],
  [               0,               0,               0,               0,               0,               0,   -0.0082802116,     0.021003674,     0.064462599,     -0.29397154 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer2_etaoffv5_35_e = [
 [
  [      -1.9608015,        4.286523,     -0.97163854,      -3.2432001,       0.7418392,       3.0082415,      -2.5621836,      0.78928658,    -0.084556503,   -0.0052820518 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,      0.35693425,      0.17784248,     -0.25531058,      -1.1442648 ],
  [               0,               0,               0,               0,               0,               0,        3.081282,       3.0598507,      -1.0127372,      -14.765789 ],
  [               0,               0,               0,               0,               0,               0,      -1.8947551,      -1.1684902,       1.1997514,       6.4407529 ],
  [               0,               0,               0,               0,               0,               0,      -0.6976863,     -0.22719849,      0.65580939,       1.8307359 ],
  [               0,               0,               0,               0,               0,               0,    0.0025693842,    0.0012747313,   -0.0017352108,   -0.0067329054 ],
  [               0,               0,               0,               0,               0,               0,       10.423605,       5.1115511,      -8.0180544,      -32.618598 ],
  [               0,               0,               0,               0,               0,               0,       13.163732,       6.1989115,      -10.230243,      -43.479337 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0056995974,  -0.00083290998 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       77.317663,      -114.02685 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.1353224,      -7.5889828 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.1130617,       3.3181392 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   1.1886531e-05,   3.8156995e-06,    0.0010436236,    0.0062483146 ],
  [               0,               0,               0,               0,               0,               0,       1.1258847,      -1.8974664,      -4.0308937,       10.365256 ],
  [               0,               0,               0,               0,               0,               0,     -0.10504698,      0.11991164,       0.3787079,      -1.7690582 ],
  [               0,               0,               0,               0,               0,               0,     0.049312288,     -0.22820429,    -0.028729306,      0.62925276 ],
  [               0,               0,               0,               0,               0,               0,  -0.00042535019,  -0.00040162437,   0.00032025169,    0.0041689888 ],
  [               0,               0,               0,               0,               0,               0,        1.873919,      -3.3179055,      -3.8920195,       7.1494655 ],
  [               0,               0,               0,               0,               0,               0,      0.20994658,     -0.19071496,     -0.75431049,      -1.4182625 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00031919305,  -0.00070475941,  -0.00054234472,     0.010301923 ],
  [               0,               0,               0,               0,               0,               0,     0.031817066,      0.41468022,      -3.9398333,          9.0628 ],
  [               0,               0,               0,               0,               0,               0,     -0.10367605,         1.24165,      -3.5875744,       1.7657482 ],
  [               0,               0,               0,               0,               0,               0,      -2.6413444,       4.8231257,       7.3859202,      -10.074773 ],
  [               0,               0,               0,               0,               0,               0,   -0.0037744301,   -0.0021820567,    0.0084148931,     0.047908872 ],
  [               0,               0,               0,               0,               0,               0,    -0.035695634,     0.030640018,      0.34573304,     -0.50082508 ],
  [               0,               0,               0,               0,               0,               0,       2.5811399,      -5.0542051,      -10.744506,       18.158449 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -0.00050438402,  -0.00074669864,  -0.00017127811,    0.0046155185,     0.026155062 ],
  [               0,               0,               0,               0,               0,       3.3583581,      -3.7939352,      -18.716426,      -23.117532,       106.92139 ],
  [               0,               0,               0,               0,               0,      0.34616098,     -0.54598656,      -1.8142172,      -1.2793555,        8.349442 ],
  [               0,               0,               0,               0,               0,     -0.97457911,       1.1341195,       5.7804637,       7.4172833,      -33.519236 ],
  [               0,               0,               0,               0,               0,   2.2247429e-05,  -8.6170827e-06,    4.569357e-05,   0.00081466622,    0.0060155298 ],
  [               0,               0,               0,               0,               0,     -0.55767944,      0.78688582,       2.9172446,       2.4115635,      -14.873616 ],
  [               0,               0,               0,               0,               0,       1.2214875,      -1.4092522,      -6.1488949,      -6.9426803,        28.31599 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0079831984,    -0.041500392,      0.16195477 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.36511495,     -0.01524036,       4.6535939 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.0436568,    -0.065596689,      -7.3547288 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.12189011,      0.49477478,      -1.8838924 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      -147.27555,        190.9027,      -94.749099,       22.964999,      -2.7722794,      0.14476602,   -0.0026725298 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer1_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,     -0.03399653,     0.085441884,    -0.068779031,     0.016669208 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0004186053,   0.00090573884 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.27155694,       2.8900985 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.10738391,     -0.90440993 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.61492658,      -0.7367684 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.2289738,    0.0014934551 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.3306589,      -1.0415523 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -79.756209,       64.130976 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       28.284505,       -23.10248 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       22.605474,      -17.980483 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.026303664,    -0.072780312,     0.052324793 ],
  [               0,               0,               0,               0,               0,               0,               0,      -11.722011,       26.206517,      -12.433391 ],
  [               0,               0,               0,               0,               0,               0,               0,       8.3568273,      -17.680777,       8.0647458 ],
  [               0,               0,               0,               0,               0,               0,               0,       9.8048116,      -21.321296,       10.075727 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.088120449,  -0.00028044594,     0.038615375 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0016619058,    0.0051182134,   -0.0038413977 ],
  [               0,               0,               0,               0,               0,               0,               0,      -5.9926968,       15.959619,      -8.2145086 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.8336898,      -8.6387568,       9.3417016 ],
 ],
 [
  [               0,               0,               0,               0,     -0.16256646,       0.5278428,     -0.61804549,      0.32626239,    -0.085294838,    0.0053869099 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.29987084,      0.70252664,     -0.53287885,      0.10157296,     0.034306838,   -0.0076733382 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0044197186,    0.0037236631 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.9540038,      -3.3292708 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -28.975571,       40.962923 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -60.384469,       84.502734 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.076151867,     -0.22198023 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer1_etaoffv5_37_e = [
 [
  [               0,               0,               0,               0,      0.17030743,     -0.52803197,      0.59176779,     -0.27784315,     0.043939967,   -0.0039878208 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.010078368,    0.0013919483 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.5483484,      -6.5258451 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -12.050921,       16.742684 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       6.1982521,      -10.047853 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.23991041,     -0.41618272 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.20311329,      0.30796287 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.2588264,       3.3642534 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       32.219418,      -49.243964 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       6.9265618,      -10.183455 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0094388295,     0.033069634 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.33927862,       1.1572732 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.30279354,      -1.0556646 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.31038165,       -1.067064 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.002449846,    0.0068998423 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00020057876,   0.00026751665,  -8.2862446e-05,   -0.0015068878 ],
  [               0,               0,               0,               0,               0,               0,      -0.7302649,     -0.22121527,       1.0971707,       7.6654947 ],
  [               0,               0,               0,               0,               0,               0,      0.84870214,       1.4558112,     -0.95745442,      -15.773831 ],
  [               0,               0,               0,               0,               0,               0,       1.2989147,     -0.23894863,        -3.24118,      -7.8820394 ],
  [               0,               0,               0,               0,               0,               0,      -0.4307973,       0.2629603,       1.1237212,       0.2531311 ],
  [               0,               0,               0,               0,               0,               0,    0.0007934853,   0.00060539131,   -0.0005896414,   -0.0034322249 ],
  [               0,               0,               0,               0,               0,               0,    -0.090232787,    -0.037247898,     -0.16555378,       1.5533256 ],
  [               0,               0,               0,               0,               0,               0,     -0.81458335,       1.6383744,       2.8452105,      -7.3283813 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0097704715,    -0.023015366,    -0.050115611,      0.12070154 ],
  [               0,               0,               0,               0,               0,               0,     0.083178361,      0.68826081,    -0.034684131,      -1.6413175 ],
  [               0,               0,               0,               0,               0,               0,      -2.3197968,        3.092312,       13.752749,      -22.087565 ],
  [               0,               0,               0,               0,               0,               0,      -4.7008925,       10.011389,       16.647537,      -36.526404 ],
  [               0,               0,               0,               0,               0,               0,      -3.7922885,       6.9661507,       19.092112,       -35.59317 ],
  [               0,               0,               0,               0,               0,               0,   -0.0012185827,    0.0043873464,    0.0077170273,    -0.019429745 ],
  [               0,               0,               0,               0,               0,               0,     -0.35718399,       1.0176099,       1.7214875,       -4.832966 ],
  [               0,               0,               0,               0,               0,               0,      0.10278095,     0.017476515,      0.57953207,      -3.8604989 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.84035066,       1.9412098 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -19.211207,       43.781967 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -24.631053,       57.231841 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -4.4620287,       10.747448 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       11.054731,      -25.293887 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.031559075,     0.071900154 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       190.50915,      -439.67139 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       88.727277,      -206.59842 ],
 ],
 [
  [               0,               0,               0,               0,               0,   1.2540784e-05,   3.2189465e-05,   8.1641389e-05,   0.00020497981,    0.0010101623 ],
  [               0,               0,               0,               0,               0,    -0.059430739,    -0.079412051,    -0.045651106,      0.22205996,       5.2879795 ],
  [               0,               0,               0,               0,               0,     0.025520452,     0.027251194,    -0.020329692,     -0.26349284,       -1.566794 ],
  [               0,               0,               0,               0,               0,     0.017687174,     0.019041559,   -0.0051002942,     -0.12198696,      -1.5286825 ],
  [               0,               0,               0,               0,               0,   -0.0095245007,    -0.007852776,      0.01427733,      0.10709339,       0.1164478 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   7.6146138e-05,   0.00067877832 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.6324455,       5.5610661 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.063571391,     -0.70514122 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.17646419,      -1.2327402 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.24021518,     -0.87606767 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0064996558,    0.0011998767,    -0.034132994 ],
  [               0,               0,               0,               0,               0,               0,               0,      -11.162282,        1.968276,       67.760388 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.78588056,       3.8588828,      -6.6516618 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.81926069,      0.38549767,      -5.4959174 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0025925038,   0.00013710109,    -0.014049384 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.92641425,      0.23589708,       5.7682667 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.1520554,     -0.71350634,       5.3805561 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.023719385,      0.03783656 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -44.574205,       113.30257 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       554.46222,       -1383.859 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -71.058948,       174.74873 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.10323135,     -0.14805065 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer2_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0061391821,    0.0053731109 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -2.690154,       5.3438416 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.1737941,      -1.3748475 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.14010917,   -0.0048431598 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0011796365,   0.00098007484 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.5477141,      0.96600899 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.4841472,      -1.8135101 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00031829845,   -0.0010765616,    0.0080122235 ],
  [               0,               0,               0,               0,               0,               0,               0,      -8.1733399,       8.6978351,       2.0959798 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.4063349,       1.3872153,       -1.703824 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.27228348,      -0.0808261,     0.063015981 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0067567726,    0.0061904509,   4.9174274e-06 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.7723389,      -2.5427773,       0.7241967 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.82204945,      -2.1497648,      -7.4532173 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0068348572,   -0.0026244532,     0.014503619 ],
  [               0,               0,               0,               0,               0,               0,               0,       14.330428,      -5.5657024,     -0.85273579 ],
  [               0,               0,               0,               0,               0,               0,               0,       9.8678012,      -4.1931601,      -4.3109713 ],
  [               0,               0,               0,               0,               0,               0,               0,      -11.851559,     -0.43596012,       8.1342175 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.049202674,   -0.0038457712,    -0.030665127 ],
  [               0,               0,               0,               0,               0,               0,               0,       10.553046,        1.345647,      -7.6512226 ],
  [               0,               0,               0,               0,               0,               0,               0,       26.465869,      -5.9742658,      -24.064148 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    -0.001742601,    0.0007267713,    0.0045142168,    0.0045154342 ],
  [               0,               0,               0,               0,               0,               0,       3.7651842,      -9.7561259,       6.8869442,       2.8168436 ],
  [               0,               0,               0,               0,               0,               0,       0.3053185,     -0.84215838,      0.49985718,      -1.3648573 ],
  [               0,               0,               0,               0,               0,               0,     -0.47400738,       1.7889183,      -1.2930154,      0.18350935 ],
  [               0,               0,               0,               0,               0,               0,    0.0013486056,  -3.5321412e-05,  -0.00017822143,   0.00024748475 ],
  [               0,               0,               0,               0,               0,               0,      0.25561717,       -1.133114,       1.3873463,     -0.26795085 ],
  [               0,               0,               0,               0,               0,               0,       1.8297516,      -2.0211066,      -3.3125667,      0.96184598 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.090543006,      0.13213267 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -17.049902,       26.538935 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       6.2922748,      -9.9632045 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.15766678,     -0.05054453 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,     -0.33525865,      0.22392362,      0.23226507,     0.023602624,     -0.13409616,    -0.085339048,      0.10527801,    -0.030031947,    0.0015452421 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,       0.1749273,      0.08800067,     -0.10853251,     -0.49334678 ],
  [               0,               0,               0,               0,               0,               0,      -12.241508,       -5.644136,        8.335412,       38.609079 ],
  [               0,               0,               0,               0,               0,               0,       1.7160089,         6.76245,       2.9447727,      -23.811981 ],
  [               0,               0,               0,               0,               0,               0,   -0.0074701828,      0.02670468,      0.05406865,     -0.35352031 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam50_layer2_etaoffv5_37_e = [
 [
  [      0.37567024,     -0.43623933,     -0.33663228,      0.17595052,      0.44917061,     0.056881244,     -0.50514654,      0.26140098,    -0.034326639,    -0.006744982 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,     0.083275313,     0.042206821,    -0.058531814,     -0.26673351 ],
  [               0,               0,               0,               0,               0,               0,       13.314015,       4.3358826,      -11.238294,      -34.764015 ],
  [               0,               0,               0,               0,               0,               0,     -0.15051889,     -0.26736634,     -0.16111004,      0.15299906 ],
  [               0,               0,               0,               0,               0,               0,      -1.2439627,     -0.65062217,      0.92586204,       4.0545056 ],
  [               0,               0,               0,               0,               0,               0,     0.017609801,    0.0087552627,    -0.012882604,    -0.056641492 ],
  [               0,               0,               0,               0,               0,               0,       14.779263,       6.9358779,       -11.69447,      -46.841025 ],
  [               0,               0,               0,               0,               0,               0,       22.180293,       10.817697,      -16.456643,      -72.248417 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0050963752,   7.0898942e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       88.391023,      -130.76991 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.5692641,       -9.779114 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.0567766,       3.2337123 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   3.3512378e-05,  -7.5069474e-05,   0.00080602604,     0.005658358 ],
  [               0,               0,               0,               0,               0,               0,       1.1415576,      -2.0223796,      -3.7000423,       10.358098 ],
  [               0,               0,               0,               0,               0,               0,    -0.086959815,      0.15600654,      0.15216571,      -1.6187555 ],
  [               0,               0,               0,               0,               0,               0,   -0.0019636426,   -0.0060781807,     0.085231623,    -0.029429527 ],
  [               0,               0,               0,               0,               0,               0,   0.00014943835,  -0.00016619279,  -0.00042379312,    0.0011753118 ],
  [               0,               0,               0,               0,               0,               0,      0.93687276,      -1.7786808,      -2.0914293,       4.8201629 ],
  [               0,               0,               0,               0,               0,               0,       0.1950978,     -0.28000197,     -0.77358981,     -0.63884635 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -1.2138966e-05,   0.00046902583,    0.0015316571,    0.0038186452 ],
  [               0,               0,               0,               0,               0,               0,      0.29875996,      -0.4977498,      -4.2422142,       11.265279 ],
  [               0,               0,               0,               0,               0,               0,        0.285348,      -1.2164581,       1.6338454,      -1.9594462 ],
  [               0,               0,               0,               0,               0,               0,      -4.2610942,       7.7050701,       18.427652,       -32.10815 ],
  [               0,               0,               0,               0,               0,               0,   0.00097585071,   -0.0040909802,   -0.0062663224,      0.03876149 ],
  [               0,               0,               0,               0,               0,               0,    -0.047461352,      0.30816364,     -0.66259817,      0.48930661 ],
  [               0,               0,               0,               0,               0,               0,       3.1505479,      -6.5915762,      -11.083812,       20.990795 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -0.00039093127,  -0.00045717867,   0.00022820019,    0.0041306418,     0.019950105 ],
  [               0,               0,               0,               0,               0,       1.5139639,      -1.7082047,      -8.4512636,      -10.534579,       49.900877 ],
  [               0,               0,               0,               0,               0,      0.23627551,     -0.39048683,      -1.2540971,     -0.82771275,       5.6034488 ],
  [               0,               0,               0,               0,               0,     -0.35757244,      0.39294932,       2.1227242,       2.8473173,      -12.602817 ],
  [               0,               0,               0,               0,               0,   6.8151708e-05,  -3.3411934e-06,  -0.00023068105,  -0.00065511341,   0.00022906829 ],
  [               0,               0,               0,               0,               0,     0.099399843,    0.0030417852,     -0.27148825,      -0.7091002,       1.4667511 ],
  [               0,               0,               0,               0,               0,     -0.14509016,      0.27069638,      0.97090831,      0.61782733,      -7.8916299 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00082607444,    -0.053581616,       0.1380642 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.34481314,    -0.012896861,       4.5665334 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.1871315,    -0.085783036,      -8.1907467 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.13867201,      0.46214657,      -1.9034494 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      -40.283404,       33.145011,      -3.7528411,      -2.9623508,       1.0132473,     -0.11753103,    0.0039036597 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]


###############################################################################
# Tables for v5 sshape correction, 75 GeV photon clusters
#

gam75_layer1_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,    -0.017021493,      0.04813732,    -0.042528138,     0.010859923 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0004121916,   0.00090144586 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.72884927,       3.2487019 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.25702498,     -0.99855158 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.42997772,     -0.86087598 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.22623856,   -0.0029469815 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.6141542,      -2.8487542 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -48.361939,       39.034972 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       25.553125,      -20.845739 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       13.221213,      -10.517922 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0045572801,    -0.013352852,     0.010277775 ],
  [               0,               0,               0,               0,               0,               0,               0,       5.1619884,      -5.6141148,       3.5852872 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.7446617,      -7.7500636,       1.9515673 ],
  [               0,               0,               0,               0,               0,               0,               0,       5.6162688,      -11.560926,       4.3809831 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.2425508,        2.016987,     -0.93936869 ],
  [               0,               0,               0,               0,               0,               0,               0,  -0.00038068537,   0.00092836949,  -0.00062735083 ],
  [               0,               0,               0,               0,               0,               0,               0,      -4.7895401,       13.440559,      -6.0668187 ],
  [               0,               0,               0,               0,               0,               0,               0,       6.0918732,      -13.238266,       11.699891 ],
 ],
 [
  [               0,               0,               0,               0,     -0.13406412,      0.49237234,     -0.63232583,      0.36073232,    -0.099811428,    0.0072470322 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.058236963,     0.095629153,    0.0037635471,      -0.0899905,     0.053998473,   -0.0060947955 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0046010477,    0.0041723323 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.6040161,      -1.3174972 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -25.146092,        35.41966 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -65.79447,       92.156857 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.10066056,     -0.20722824 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer1_etaoffv5_55_e = [
 [
  [               0,               0,               0,               0,      0.60283979,      -1.2166371,      0.80175166,     -0.21951713,     0.027908442,    0.0023558972 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0070614967,    0.0089233212 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.9289144,      -2.7767276 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -8.5444025,       11.623229 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       6.8925094,      -10.993223 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.25923883,     -0.42304672 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0035313795,    0.0020842729 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.4668525,       2.3552533 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       22.341615,      -34.699702 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.5468887,      -8.1600211 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0013433287,    0.0014126428 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.23738728,       2.0432861 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00021126149,      -1.0647589 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.065809357,     -0.92634892 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.063419568,      0.14551598 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00082491863,   0.00013830612,    0.0018638194,    0.0039958125 ],
  [               0,               0,               0,               0,               0,               0,      0.35392174,     -0.24300705,      -1.0961504,       3.0654028 ],
  [               0,               0,               0,               0,               0,               0,     -0.51769491,     -0.75249963,     0.057454251,       4.6458449 ],
  [               0,               0,               0,               0,               0,               0,     -0.84269372,      -0.1701349,       1.2745495,       2.5394127 ],
  [               0,               0,               0,               0,               0,               0,    -0.072401016,     0.048206136,       0.2287157,    -0.061160448 ],
  [               0,               0,               0,               0,               0,               0,    0.0017690779,     0.001005305,   -0.0022846467,    -0.010756097 ],
  [               0,               0,               0,               0,               0,               0,     -0.42860761,      0.66132188,       1.1030728,      -1.1690901 ],
  [               0,               0,               0,               0,               0,               0,     -0.54535743,      0.94301044,       1.6594757,      -4.3020475 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0012115488,   -0.0046680596,   -0.0074486579,     0.028689226 ],
  [               0,               0,               0,               0,               0,               0,       3.3795035,      -6.4101436,      -13.631813,       28.001041 ],
  [               0,               0,               0,               0,               0,               0,       6.5208569,      -13.646674,      -29.289756,       59.747179 ],
  [               0,               0,               0,               0,               0,               0,        4.769619,      -9.4660459,      -21.419673,       41.203804 ],
  [               0,               0,               0,               0,               0,               0,      -3.3469504,       7.9064005,       18.291018,      -41.711365 ],
  [               0,               0,               0,               0,               0,               0,   0.00025797009,   -0.0007522216,  -0.00085346778,    0.0055688261 ],
  [               0,               0,               0,               0,               0,               0,       0.6603883,      -1.5095242,      -2.9829285,       6.6392253 ],
  [               0,               0,               0,               0,               0,               0,      -3.1715183,       7.8974288,       18.768818,      -48.873338 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00030833375,     0.001013831,    0.0035304493 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.0789759,     0.042260111,       17.774475 ],
  [               0,               0,               0,               0,               0,               0,               0,      -6.0129884,     -0.12987614,       30.805186 ],
  [               0,               0,               0,               0,               0,               0,               0,       -4.333302,     0.019262442,       21.982361 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.46786387,     0.077644665,      -2.4236551 ],
  [               0,               0,               0,               0,               0,               0,               0,  -2.6702454e-05,  -0.00016405324,   0.00038660144 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.28745857,      0.40133744,       2.9658085 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.34076203,      0.63687075,        1.126168 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0002745298,    0.0026075634 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.74495996,       4.3227825 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.55614095,      -2.5087129 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.1742583,     -0.71495638 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.32964127,       0.8275724 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      0.05696858 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -1.0165721 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      0.78359919 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      0.97991107 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,     0.018495453 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,    0.0026953993 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       1.3666171 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -2.2943167 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00095193475,    0.0010420649,  -0.00093209109,    -0.011095498 ],
  [               0,               0,               0,               0,               0,               0,      -2.8240481,      -1.7521177,       6.6040844,       42.589034 ],
  [               0,               0,               0,               0,               0,               0,     -0.78261893,      0.83294019,       3.1757562,      -3.2870927 ],
  [               0,               0,               0,               0,               0,               0,      0.32379437,      0.41510551,     -0.39664536,      -5.8961101 ],
  [               0,               0,               0,               0,               0,               0,   0.00054501479,   0.00047197444,   -0.0009974559,   -0.0068101644 ],
  [               0,               0,               0,               0,               0,               0,      -0.3027026,    -0.051630052,      0.77313085,       3.7595082 ],
  [               0,               0,               0,               0,               0,               0,      -0.8589622,     -0.73518482,       1.4056928,       10.935876 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,        1.577562,      0.01876632,      -9.7151792 ],
  [               0,               0,               0,               0,               0,               0,               0,      -86.586546,         22.4042,       482.51581 ],
  [               0,               0,               0,               0,               0,               0,               0,       565.85062,       6.9649043,      -3523.1113 ],
  [               0,               0,               0,               0,               0,               0,               0,      -34.639463,     -0.17384084,       211.60112 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.082583129,    -0.093895724,      0.80436283 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer2_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,    -0.042719185,     0.021847226,    0.0055364449 ],
  [               0,               0,               0,               0,               0,               0,               0,       14.101491,      -7.2587692,        4.536936 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.17980317,     -0.15469157,      -1.2742837 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.47950959,    -0.090800717,    -0.043741818 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0035795257,   -0.0020546253,   0.00062433493 ],
  [               0,               0,               0,               0,               0,               0,               0,       13.790923,      -7.5607516,        1.183198 ],
  [               0,               0,               0,               0,               0,               0,               0,      -32.577843,       13.128505,      -1.8977887 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0010583362,    0.0014715841,    0.0076414511 ],
  [               0,               0,               0,               0,               0,               0,               0,      -4.6386003,       4.4762778,       2.5741906 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.83548662,      0.90045729,      -1.5694609 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.33905478,     -0.23476874,      0.45639999 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0021871963,    0.0059172373,   -0.0044742314 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.39746935,     -0.72401453,      0.36795218 ],
  [               0,               0,               0,               0,               0,               0,               0,       11.823899,      -9.9513994,      -3.9649575 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.038755995,    -0.022038352 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       7.1795992,      -2.5807442 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.7892784,      -5.8510759 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -5.7228672,        4.519021 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00030961515,  -6.8517711e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -33.769756,        28.30204 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -15.692785,       12.843623 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00086089948,    0.0011398188,    0.0055984304 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.1849327,      -3.9090044,       5.9724076 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.72113549,      -1.7000254,     -0.41006994 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.51151395,       1.4348658,     -0.82187547 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00031046341,  -0.00096685096,    0.0010124193 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.4152814,       2.3968384,     -0.13186197 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.1175369,      0.22474343,      -1.1375463 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0053197963,    0.0019093806 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -97.659084,       135.86408 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       10.847144,      -16.211559 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        3.717218,      -4.7472756 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,     0.011668572,    -0.011204212,   -0.0093920562,    0.0027006769,    0.0089683203,    0.0025130441,    0.0020185692,   -0.0081620939,   0.00038139437 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,       0.1309008,     0.069298861,    -0.079452826,     -0.38168001 ],
  [               0,               0,               0,               0,               0,               0,       -10.36219,      -2.9345474,       13.288174,       44.650021 ],
  [               0,               0,               0,               0,               0,               0,      -4.6151646,        8.243949,        11.68536,      -23.198754 ],
  [               0,               0,               0,               0,               0,               0,     -0.75573419,     -0.29159538,      0.74724045,       2.3800822 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer2_etaoffv5_55_e = [
 [
  [     0.096711799,     -0.14215703,     -0.11560686,      0.14704535,       0.1277799,  -0.00053152828,     -0.23631959,      0.15330104,    -0.027611715,   -0.0036084762 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0061720507,    -0.002565195,    0.0057373241,      0.02280251 ],
  [               0,               0,               0,               0,               0,               0,       13.026156,       6.1969693,      -10.006552,      -39.020133 ],
  [               0,               0,               0,               0,               0,               0,      -1.6343601,     -0.84002685,       1.0494338,       4.5151523 ],
  [               0,               0,               0,               0,               0,               0,      0.78211084,      0.40407241,     -0.51769039,      -2.7066522 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0027127173,    0.0032462174 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       50.873156,      -74.268207 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.0399919,       3.3831709 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.5862809,       2.4901452 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0038810812,   -0.0030536608,    -0.012954637,     0.019652824 ],
  [               0,               0,               0,               0,               0,               0,      -1.0323944,        2.098657,     -0.28212535,       2.6439197 ],
  [               0,               0,               0,               0,               0,               0,     -0.23983068,      0.74577866,     -0.34840881,      -1.6647803 ],
  [               0,               0,               0,               0,               0,               0,      0.32894478,     -0.42181351,      -1.3458853,       1.9637674 ],
  [               0,               0,               0,               0,               0,               0,    0.0045837577,   -0.0075234389,    -0.014845577,     0.024748836 ],
  [               0,               0,               0,               0,               0,               0,        1.013451,      -1.6317244,      -4.7869971,       8.5241577 ],
  [               0,               0,               0,               0,               0,               0,      -4.2876326,        7.240956,        9.318916,      -17.656997 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00061040516,   0.00068281022,    0.0006787309 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.76734935,      0.20567882,       6.0675145 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.64708874,       3.4097865,      -5.6123273 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.65519555,       2.3902964,      -1.9861665 ],
  [               0,               0,               0,               0,               0,               0,               0,  -0.00032365191,  -3.2087204e-05,    0.0027242614 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.35786445,      -1.7385034,       3.0075718 ],
  [               0,               0,               0,               0,               0,               0,               0,        -0.14662,       1.0393078,      -3.1785846 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -0.00074369818,   -0.0010514021,  -0.00020192118,    0.0063287307,     0.034751261 ],
  [               0,               0,               0,               0,               0,       2.6378031,      -2.9180987,      -14.518292,      -18.136538,       82.188854 ],
  [               0,               0,               0,               0,               0,     -0.19190663,      0.10173986,       1.1338569,       2.0567976,      -7.6723712 ],
  [               0,               0,               0,               0,               0,     -0.42185744,       0.5282286,       2.6400316,       3.3249408,      -16.239976 ],
  [               0,               0,               0,               0,               0,   0.00062391148,   0.00011225815,   -0.0014897547,   -0.0040894278,   -0.0013277096 ],
  [               0,               0,               0,               0,               0,     0.083474681,     -0.15123742,     -0.61461304,     -0.77039572,       4.8235112 ],
  [               0,               0,               0,               0,               0,       0.1715097,      0.17540378,     -0.13744569,      -1.5509951,      -6.0440762 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0058144341,   -0.0019079058 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -44.720546,       112.13119 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       10.672618,      -27.531553 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.1117361,      -4.9448731 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      -31.671252,       20.956098,      -1.4353366,      -1.2457593,      0.31547784,    -0.027784142,    0.0014577952 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer1_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,    -0.025767391,      0.06872126,    -0.058159797,     0.014610614 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00040951148,   0.00086363996 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.78224355,       3.3412922 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.25704624,     -0.98143455 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.44641395,     -0.84334472 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.23426787,   -0.0030744437 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.023840719,    0.0064479343 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -15.073265,        12.54164 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -8.6585764,        6.378698 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       18.802254,      -14.956857 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0037591004,    -0.010922783,    0.0083308497 ],
  [               0,               0,               0,               0,               0,               0,               0,       5.1311681,      -4.2457083,       2.8489576 ],
  [               0,               0,               0,               0,               0,               0,               0,       6.8086264,      -10.913714,        3.139817 ],
  [               0,               0,               0,               0,               0,               0,               0,       7.6963972,      -15.247733,       5.9258578 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.6029746,       2.6162178,      -1.2250657 ],
  [               0,               0,               0,               0,               0,               0,               0,  -0.00013572266,   0.00038502194,  -0.00033320989 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.3946266,       8.2094975,      -3.0954438 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.7727486,      -8.2745451,       9.0896123 ],
 ],
 [
  [               0,               0,               0,               0,     -0.14172694,      0.47528737,     -0.57263816,      0.31008953,    -0.083320668,    0.0054302355 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.20247781,      0.43571958,     -0.26077635,    -0.025601591,     0.061186699,   -0.0095982239 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.004474376,    0.0040147436 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        4.410595,       -2.420288 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -29.242202,       41.371149 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -67.000171,       93.883676 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.095298635,     -0.22670521 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer1_etaoffv5_35_e = [
 [
  [               0,               0,               0,               0,    -0.067264227,     0.071818104,     0.031380762,    -0.035352682,   -0.0027322213,    -0.002523016 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0088444984,    0.0043339061 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.99842713,       3.1719403 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -7.4380548,       9.9139019 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.2030867,       3.5942962 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.24853524,     -0.41549474 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.090800007,      0.13994492 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -68.503656,       107.70104 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -35.932816,       52.003468 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       70.221056,      -104.30088 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0014546066,    0.0035798513 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.25292896,       1.7519883 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.11234964,      -1.1699439 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.10890303,       -1.146725 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.031497007,     0.076664318 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00051475176,  -7.1284424e-05,    0.0010527002,    0.0037625107 ],
  [               0,               0,               0,               0,               0,               0,      0.25468123,     -0.18615372,     -0.84457198,       3.2474538 ],
  [               0,               0,               0,               0,               0,               0,     -0.37536499,     -0.30936132,      0.21298339,       1.7362109 ],
  [               0,               0,               0,               0,               0,               0,     -0.32090618,     -0.31097619,      0.22780716,       1.4471416 ],
  [               0,               0,               0,               0,               0,               0,   -0.0031760598,     0.086505142,      0.13830627,     -0.46822915 ],
  [               0,               0,               0,               0,               0,               0,   0.00064222125,   0.00035995885,  -0.00088840556,   -0.0035379687 ],
  [               0,               0,               0,               0,               0,               0,     -0.36522294,      0.36901271,      0.73384709,      0.39123902 ],
  [               0,               0,               0,               0,               0,               0,     -0.28629789,      0.28842614,      0.58262149,      -1.7298344 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -6.822991e-05,   2.6062042e-06,   0.00026139604,    0.0014529017 ],
  [               0,               0,               0,               0,               0,               0,     0.051918387,     -0.11715014,     -0.27913208,       4.3591904 ],
  [               0,               0,               0,               0,               0,               0,     -0.35176921,    -0.080640695,      0.43566882,     -0.20812562 ],
  [               0,               0,               0,               0,               0,               0,     0.046572549,      -0.5338189,     -0.91198418,       1.1760441 ],
  [               0,               0,               0,               0,               0,               0,    -0.029827833,      0.28372099,      0.45216564,      -1.7444754 ],
  [               0,               0,               0,               0,               0,               0,   0.00021045749,   0.00018519381,  -0.00016020015,  -0.00048977563 ],
  [               0,               0,               0,               0,               0,               0,    -0.013189262,     0.040586757,      -0.2093425,      0.96143013 ],
  [               0,               0,               0,               0,               0,               0,     -0.18561562,      0.34307267,      0.60024142,      -2.6532299 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   1.3135379e-05,  -3.9864852e-05,   0.00046401576 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.96394502,    -0.079592462,       3.7462672 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.51694068,       1.5841314,       2.4340291 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.37384553,    -0.052616788,      -1.1160037 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0083581832,      0.04503654,     -0.18583609 ],
  [               0,               0,               0,               0,               0,               0,               0,   3.6456007e-05,   9.7087939e-05,    0.0011677722 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.47237329,     -0.20269144,       1.6815495 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.065533909,     0.019597959,     0.053005201 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00048148386,    0.0018667934 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.96041648,       4.9465822 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.0742214,      -3.7119648 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.55901465,      0.19870407 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.45681022,       1.1297266 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,    0.0014407906 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       3.0619185 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -1.2637132 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -2.5293515 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,     -0.88360546 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,    0.0001915076 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       2.1628744 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      0.67297776 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0012212153,    0.0013034344,   -0.0013441225,    -0.014864988 ],
  [               0,               0,               0,               0,               0,               0,      -3.0028596,      -1.7369389,       7.1075348,       43.402798 ],
  [               0,               0,               0,               0,               0,               0,     -0.71907705,        1.359825,       3.6295199,      -8.4187104 ],
  [               0,               0,               0,               0,               0,               0,      0.24021466,      0.37757767,     -0.19149831,      -4.9349145 ],
  [               0,               0,               0,               0,               0,               0,   0.00085227589,   0.00072852856,   -0.0015834102,    -0.011216274 ],
  [               0,               0,               0,               0,               0,               0,     -0.32322657,     -0.16728728,       0.6965368,       4.8067443 ],
  [               0,               0,               0,               0,               0,               0,     -0.53638607,     -0.46525284,       0.7423724,       5.9458876 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,     0.027025504 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       3.9099375 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       -8.583746 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,     -0.90014805 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       0.4824066 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer2_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.090307038,    -0.041878419,     0.013685596 ],
  [               0,               0,               0,               0,               0,               0,               0,      -16.367138,       7.4716201,        3.397752 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.60466439,     -0.37120889,      -1.2631217 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.15761237,    -0.031893433,    -0.016631801 ],
  [               0,               0,               0,               0,               0,               0,               0,       0.0284073,    -0.013230006,    0.0018538445 ],
  [               0,               0,               0,               0,               0,               0,               0,      -7.6960105,       3.3907366,       1.0183529 ],
  [               0,               0,               0,               0,               0,               0,               0,       11.048929,      -4.4310901,       -4.547836 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0066200992,    0.0085346515,    0.0061286753 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.8902238,      -5.2423232,       6.1147383 ],
  [               0,               0,               0,               0,               0,               0,               0,       -1.444265,       1.6598916,      -1.8081491 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.89668722,     -0.92498328,      0.23717746 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0017309989,   0.00061008302,   0.00035756562 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.37678467,       1.1433595,       1.1567698 ],
  [               0,               0,               0,               0,               0,               0,               0,      -38.744959,        35.63088,      -19.111183 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.014920879,   -0.0035032502 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       11.412419,      -5.2706566 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       8.2708487,      -7.9406562 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.5674723,       3.0264674 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0023248429,    0.0039545461 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.0281772,      -1.5038279 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -30.912195,       3.2068693 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00088981683,   0.00048204924,    0.0031828831,    0.0045352989 ],
  [               0,               0,               0,               0,               0,               0,       1.8573354,      -4.1403878,       1.2387387,       5.0594658 ],
  [               0,               0,               0,               0,               0,               0,      0.63434543,      -1.4636987,      0.61557733,      -1.2092578 ],
  [               0,               0,               0,               0,               0,               0,     -0.60583104,       1.6241589,     -0.96605708,     0.028474054 ],
  [               0,               0,               0,               0,               0,               0,   0.00051139036,  -0.00048575823,  -0.00045293078,   0.00098263042 ],
  [               0,               0,               0,               0,               0,               0,     -0.13219433,     -0.21354635,      0.48611079,      0.57300404 ],
  [               0,               0,               0,               0,               0,               0,       1.4896421,       -1.824794,      -3.0163545,       1.4117634 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.065777715,     0.098241565 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -12.602508,       20.669956 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.7758784,      -9.2668662 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.086178905,     0.022416166 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,     -0.74642698,        1.000695,      0.26219947,     -0.53799515,     -0.27347542,      0.43635851,     -0.14269064,    0.0089002119,  -0.00010478515 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00023129822,    0.0016679613,     0.008669115,    0.0074960138 ],
  [               0,               0,               0,               0,               0,               0,    -0.036881331,   -0.0045441863,    0.0067620856,       7.9734945 ],
  [               0,               0,               0,               0,               0,               0,     0.010694176,   -0.0057935083,   -0.0017407912,      -1.3615609 ],
  [               0,               0,               0,               0,               0,               0,    0.0068542912,    0.0005671524,    -0.044018157,     -0.14428869 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer2_etaoffv5_35_e = [
 [
  [     -0.10905074,     0.049306074,     0.088968317,     0.047198887,    -0.025212402,    -0.068771622,    -0.031953462,     0.070887935,    -0.014716587,   -0.0074508759 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,      -7.0099515,      -3.2752567,       5.4221007,       22.867384 ],
  [               0,               0,               0,               0,               0,               0,       28.177938,      -35.636675,      -57.234084,       71.639603 ],
  [               0,               0,               0,               0,               0,               0,       -2.835956,      -1.5748039,       1.8757585,        9.648061 ],
  [               0,               0,               0,               0,               0,               0,      0.92263054,      -1.3333964,      -2.0126481,       2.9084459 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0055987547,   -0.0011741118 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       77.433551,      -113.81954 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.9949356,      -5.8608046 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.5509323,       2.4721729 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0002150365,   -0.0004467145,  -0.00028251284,    0.0072950435 ],
  [               0,               0,               0,               0,               0,               0,       1.0228776,       -1.784462,      -3.5002203,       10.166286 ],
  [               0,               0,               0,               0,               0,               0,    -0.016745922,     0.060794271,    -0.086821105,      -1.3184517 ],
  [               0,               0,               0,               0,               0,               0,     -0.04231291,     0.058157827,      0.19065715,     -0.18100394 ],
  [               0,               0,               0,               0,               0,               0,   0.00034116652,  -0.00042326036,   -0.0011347917,    0.0022988433 ],
  [               0,               0,               0,               0,               0,               0,      0.22907986,     -0.29795868,     -0.93001532,       2.1585192 ],
  [               0,               0,               0,               0,               0,               0,      0.17866335,      -0.1753937,     -0.71508794,      -0.9362049 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.050509512,     -0.20847945,       0.2236837 ],
  [               0,               0,               0,               0,               0,               0,               0,      -9.4529323,       37.465146,      -33.919238 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.1373131,      -8.5714429,       7.2747254 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.7517435,       15.564172,       -15.95669 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.026388744,      0.11136278,     -0.11610545 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.3906426,      -14.873739,       16.565888 ],
  [               0,               0,               0,               0,               0,               0,               0,       8.7365134,      -34.807458,       32.158815 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -0.00021502579,   -0.0003621908,  -7.3946276e-05,    0.0026418611,     0.015511358 ],
  [               0,               0,               0,               0,               0,       1.3440015,      -1.5402964,      -7.5747948,       -9.390271,       45.558932 ],
  [               0,               0,               0,               0,               0,      0.18020794,     -0.27376143,     -0.90257402,     -0.66213615,       3.3498485 ],
  [               0,               0,               0,               0,               0,     -0.35153048,      0.66030603,        2.760839,       3.1172578,      -17.471929 ],
  [               0,               0,               0,               0,               0,   0.00051904043,   0.00076160746,   0.00085866561,   0.00022125346,   -0.0016677179 ],
  [               0,               0,               0,               0,               0,     -0.11277076,      0.17327923,      0.53104938,       0.1576305,      -2.0464854 ],
  [               0,               0,               0,               0,               0,       0.8120982,     -0.93494404,       -4.300067,      -5.2068961,       20.307477 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.013328923,    -0.020880941 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -29.265753,       74.515486 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       11.502661,      -29.524283 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.8573795,       7.1175567 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      -25.363535,       21.059556,       -1.282298,      -2.8003553,      0.92305613,     -0.11106461,    0.0036934173 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer1_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,     -0.01439278,     0.041008815,    -0.036102813,    0.0088828389 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00045025342,   0.00091869704 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.64999146,        3.144144 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.22185633,     -0.99392135 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.45227196,     -0.85450874 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.22124517,   -0.0010769524 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.023454539,    0.0059704866 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -15.361818,       12.777919 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -9.3326721,       6.9098553 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       19.460749,      -15.480723 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0011103203,    0.0001355322,    0.0034540364 ],
  [               0,               0,               0,               0,               0,               0,               0,        1.017248,      -1.2721814,       2.2169517 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.62991265,      -0.9630216,     -0.64329564 ],
  [               0,               0,               0,               0,               0,               0,               0,       0.7218872,      -1.5701416,     -0.40167548 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.55228195,       1.0862293,     -0.63091633 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00014630976,  -0.00034138979,   0.00017393093 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.49437083,     -0.10737922,       2.6186037 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.19475109,      0.70907371,        3.402137 ],
 ],
 [
  [               0,               0,               0,               0,     -0.17082681,      0.56813651,     -0.68530807,      0.37439072,     -0.10035016,    0.0071044729 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.20585044,       0.4445907,     -0.26920821,    -0.022102615,     0.060574362,   -0.0095606205 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0044875073,    0.0040651837 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.2223287,      -2.1580468 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -28.173497,       39.820966 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -67.528762,       94.639772 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.095471312,      -0.2240792 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer1_etaoffv5_37_e = [
 [
  [               0,               0,               0,               0,    -0.065938937,      0.07085079,     0.032209989,    -0.035404677,   -0.0051516859,   -0.0015651836 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0095570322,    0.0044326073 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.98969699,       3.1325818 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -5.8301161,       7.5423607 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.9667612,        4.713143 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.24921543,     -0.41365071 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0027749866,    0.0021800353 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.2259145,        3.481196 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       25.607639,      -39.692124 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       7.1836195,      -10.555904 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00074660187,    0.0028337537 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.17411072,       1.8627864 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.025344018,      -1.0944827 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.012999826,       -1.005266 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.048722672,      0.11667623 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00052300162,  -3.1474818e-05,   0.00093022415,    0.0025141219 ],
  [               0,               0,               0,               0,               0,               0,      0.81219574,      0.49790642,      -0.9685226,      -1.3468939 ],
  [               0,               0,               0,               0,               0,               0,     -0.37154065,     -0.18312102,      0.22143038,       0.2117933 ],
  [               0,               0,               0,               0,               0,               0,      0.15870945,    -0.059907019,     -0.46711232,      -1.8964172 ],
  [               0,               0,               0,               0,               0,               0,      0.17967442,      0.21694061,     -0.02890096,      -1.5959466 ],
  [               0,               0,               0,               0,               0,               0,  -3.2660721e-05,   8.1114991e-06,   3.7903429e-05,   0.00090654989 ],
  [               0,               0,               0,               0,               0,               0,      0.10557056,      0.15907326,     -0.27220411,      0.11878267 ],
  [               0,               0,               0,               0,               0,               0,     -0.24018496,      0.51690185,      0.72883645,      -3.2053014 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0030328964,    -0.017406142,     0.026503078 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.3816015,      -8.2668558,       9.4190411 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.4669889,       -6.564236,       5.9543084 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.30461574,      -1.1064614,     -0.31751962 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.46309906,       1.2030106,      -4.6621521 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00039171687,   -0.0049498356,     0.010987323 ],
  [               0,               0,               0,               0,               0,               0,               0,       0.4950989,      -1.9514536,       1.9949021 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.7578379,       11.460403,      -8.8362415 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   7.8612642e-05,   0.00029476952,   0.00093954923,    0.0032613677 ],
  [               0,               0,               0,               0,               0,               0,      -4.3300667,      -3.2743678,       7.2588158,       54.580247 ],
  [               0,               0,               0,               0,               0,               0,       3.2892275,        2.496414,      -5.9173175,      -40.827124 ],
  [               0,               0,               0,               0,               0,               0,     0.022764993,     0.038062159,     0.020660658,      -1.1844049 ],
  [               0,               0,               0,               0,               0,               0,     -0.23944249,     -0.14876346,      0.58173734,       3.1615379 ],
  [               0,               0,               0,               0,               0,               0,   7.9190362e-05,   0.00017324762,   0.00037800347,    0.0018222939 ],
  [               0,               0,               0,               0,               0,               0,     -0.72558705,     -0.46156649,       1.0156526,       9.1477264 ],
  [               0,               0,               0,               0,               0,               0,     0.024432747,    -0.076824191,     -0.48433798,      -1.8254585 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00051629924,    0.0019321573 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.54442901,       3.8951618 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.75129463,      -2.9642784 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.73293212,      0.60517467 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.44183632,       1.0916219 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,   0.00076315682 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       6.1492451 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -1.3277571 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -3.8260113 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -1.7031587 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,   0.00020342635 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       2.3066552 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      0.63031797 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0010385231,    0.0011699443,  -0.00099311043,    -0.012565689 ],
  [               0,               0,               0,               0,               0,               0,      -2.7840505,       -1.475804,       6.7833557,       39.545149 ],
  [               0,               0,               0,               0,               0,               0,       -0.777342,       1.2015576,        3.602372,      -6.6444577 ],
  [               0,               0,               0,               0,               0,               0,      0.25806744,      0.45559487,     -0.14746538,      -5.7415491 ],
  [               0,               0,               0,               0,               0,               0,   0.00089858272,   0.00076323786,   -0.0016656121,    -0.011765348 ],
  [               0,               0,               0,               0,               0,               0,     -0.29778457,    -0.052534137,      0.75914525,       3.6473676 ],
  [               0,               0,               0,               0,               0,               0,     -0.59427579,     -0.47348215,      0.90968722,       6.4687871 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,     0.027347638 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,       3.8065177 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      -2400.0679 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,     -0.90073876 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,      0.47543949 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer2_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,    -0.084588053,     0.037716187,    0.0074499034 ],
  [               0,               0,               0,               0,               0,               0,               0,        13.88147,      -6.3608907,       4.4702723 ],
  [               0,               0,               0,               0,               0,               0,               0,        3.252462,      -1.3593048,      -1.2188336 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.8553287,      0.79633368,    -0.067878593 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.027665616,      0.01221576,  -0.00015461609 ],
  [               0,               0,               0,               0,               0,               0,               0,       5.8401358,      -2.7075056,       1.4807402 ],
  [               0,               0,               0,               0,               0,               0,               0,       -19.03078,       8.2313677,      -5.4077029 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.012054481,    -0.011483411,     0.011491358 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.46671555,     -0.16416012,       4.2283975 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.66037928,      0.73607686,      -1.5382793 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.60292079,     -0.60598441,      0.15380649 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.012696834,     0.013298288,   -0.0034381577 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.6054355,      -4.6831548,     -0.46173631 ],
  [               0,               0,               0,               0,               0,               0,               0,       69.262138,      -99.537642,       18.483655 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.015981918,   -0.0043005052 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       11.513747,      -5.3742667 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       8.1401903,       -7.833513 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -4.1475277,       3.4462073 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0084193453,    0.0084864501 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.2513206,      -1.6658945 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -32.000581,      -2.1465382 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0018881044,    0.0013065548,    0.0049782217,    0.0031124757 ],
  [               0,               0,               0,               0,               0,               0,       1.5102394,      -2.4442089,      -1.2071703,       6.0986422 ],
  [               0,               0,               0,               0,               0,               0,      0.21187106,     -0.24593497,     -0.52393537,     -0.85738323 ],
  [               0,               0,               0,               0,               0,               0,     -0.23292588,      0.49501614,      0.18349989,     -0.36398102 ],
  [               0,               0,               0,               0,               0,               0,   0.00088646458,  -0.00088576875,  -0.00071429625,    0.0012580567 ],
  [               0,               0,               0,               0,               0,               0,       0.1304022,     -0.39887255,     -0.10452113,       1.0249086 ],
  [               0,               0,               0,               0,               0,               0,       1.9106601,      -2.3854397,      -3.4859815,       1.9863805 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.069137157,      0.10288121 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -13.596752,        22.00175 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        6.191888,      -9.8351072 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.17432749,    -0.092699377 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,      -1.1413507,       1.5648203,      0.33810915,     -0.84311577,     -0.37323393,      0.66811907,     -0.23507746,     0.021869278,  -0.00057344489 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0012236837,    0.0023036593,     0.015462035,    -0.005168939 ],
  [               0,               0,               0,               0,               0,               0,    -0.088261209,   -0.0055678385,     0.016123003,        6.086694 ],
  [               0,               0,               0,               0,               0,               0,    -0.025039941,     -0.13861267,     0.062707893,     -0.93354218 ],
  [               0,               0,               0,               0,               0,               0,     0.012614875,    0.0013980956,     0.057150939,      -0.3910055 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam75_layer2_etaoffv5_37_e = [
 [
  [      0.17119391,     -0.26361589,     -0.15211777,      0.17011111,      0.28198562,    -0.041131908,     -0.37060759,      0.26009304,    -0.049152198,   -0.0064200702 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,      -19.154782,      -9.0005124,       14.782902,       62.675002 ],
  [               0,               0,               0,               0,               0,               0,       11.900929,      -12.662984,         -23.174,       23.598956 ],
  [               0,               0,               0,               0,               0,               0,     -0.25275053,      -4.2756799,      -2.9145524,       14.354216 ],
  [               0,               0,               0,               0,               0,               0,     -0.36586912,      0.55453334,      0.83069152,      -1.2581024 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0081310867,   -0.0049233009 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       91.015265,      -134.34394 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        5.621402,      -9.8475949 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.4775939,       5.3836317 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0060884896,   -0.0050605112,    -0.023922669,     0.032091293 ],
  [               0,               0,               0,               0,               0,               0,      -2.2100729,       4.3541015,       0.6816333,       1.2540926 ],
  [               0,               0,               0,               0,               0,               0,      0.37144655,     -0.53449452,      -1.3543572,      0.64777638 ],
  [               0,               0,               0,               0,               0,               0,     0.050310533,    -0.091693278,    -0.047745526,       0.2025898 ],
  [               0,               0,               0,               0,               0,               0,    0.0024792559,   -0.0033620271,    -0.012092375,     0.018842662 ],
  [               0,               0,               0,               0,               0,               0,     -0.78239156,       3.6084094,      -7.0130354,       6.0744396 ],
  [               0,               0,               0,               0,               0,               0,      -2.9589203,       4.2913789,       8.3483997,      -13.880409 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.053630091,     -0.22130323,      0.23685877 ],
  [               0,               0,               0,               0,               0,               0,               0,      -9.9548194,       39.551765,      -36.080438 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.1800997,      -8.7503976,       7.4616109 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.7363666,       15.503957,      -15.898994 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.026863555,      0.11350253,     -0.11846361 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.4730874,      -15.225217,       16.936804 ],
  [               0,               0,               0,               0,               0,               0,               0,       7.8365153,      -31.102197,       28.385021 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -0.00058185222,  -0.00077779456,  -2.9366671e-05,    0.0051067812,     0.026824308 ],
  [               0,               0,               0,               0,               0,       2.7627674,      -3.0278325,      -15.219115,      -19.133119,       86.568568 ],
  [               0,               0,               0,               0,               0,     -0.16855147,     0.039277889,       1.0779755,       2.2786657,      -7.8589848 ],
  [               0,               0,               0,               0,               0,      -1.3264999,       1.5509017,       7.7584461,       9.8019773,      -44.183974 ],
  [               0,               0,               0,               0,               0,    1.849488e-05,   6.8798086e-05,   0.00056436992,     0.003144906,     0.014861157 ],
  [               0,               0,               0,               0,               0,    -0.064718782,      0.19456555,      0.20052706,     -0.68806996,    -0.033155027 ],
  [               0,               0,               0,               0,               0,      0.60397378,     -0.80070591,      -2.9929219,      -2.8265742,       13.005116 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00077842418,    0.0092411685 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.80298805,      0.55663105 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.8317402,      -15.606911 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.05049981,    -0.039235094 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      -26.687998,       21.957187,      -1.6969893,      -2.5504969,      0.84518731,     -0.10100794,     0.003308369 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]


###############################################################################
# Tables for v5 sshape correction, 100 GeV photon clusters
#


gam100_layer1_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,   -0.0041632198,     0.018609989,    -0.020203251,    0.0052605977 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.000415968,   0.00093024335 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.71155056,       3.2945708 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.27888597,      -1.0371927 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.47565274,     -0.88587208 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.22857244,   -0.0021868338 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.0017292,       -1.566228 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -53.719214,       43.300129 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       20.878202,      -17.128594 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       16.189022,      -12.877207 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0031556379,   -0.0095138338,    0.0079609415 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.7322078,     -0.31786313,      0.97472156 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.86737051,      -0.6743064,      -1.0616979 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.5774639,      -3.7952206,       0.9972915 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.42832775,      0.47106298,     -0.18575092 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00080769109,   -0.0020233741,    0.0012215759 ],
  [               0,               0,               0,               0,               0,               0,               0,       -17.16799,        42.28053,      -22.479699 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.2559197,       9.2450957,      -1.8388455 ],
 ],
 [
  [               0,               0,               0,               0,    -0.095910378,      0.36723674,     -0.47551338,      0.26860851,     -0.07538128,    0.0049729242 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.10496216,      0.21789099,     -0.10707516,    -0.050458725,     0.049833266,   -0.0061181833 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0045020693,    0.0037044877 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.2075405,       -3.273154 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -31.645839,       44.870033 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -82.241053,       115.60385 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.097259411,     -0.25395101 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer1_etaoffv5_55_e = [
 [
  [               0,               0,               0,               0,      0.38963785,     -0.79577528,      0.50740368,     -0.12144846,     0.013608033,    0.0032004145 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0050888213,    0.0067654964 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.55004023,       2.7167823 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -10.024399,       13.722105 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.5059215,       1.1050626 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.21508817,     -0.37307713 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.08189273,      0.12947489 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -32.141817,       52.573955 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       19.790993,      -30.867609 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.6145885,      -6.7574271 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00063240664,    0.0022554625 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.18168205,       1.4446115 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.23937293,     -0.69323745 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.27302274,     -0.60002713 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.049648993,      0.12724921 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   5.5868655e-05,   0.00020079026,   0.00025815519,   0.00020553466 ],
  [               0,               0,               0,               0,               0,               0,     -0.35860754,     -0.10045235,      0.42873169,        5.079609 ],
  [               0,               0,               0,               0,               0,               0,     -0.52749661,       0.3414617,       1.1059549,       -1.476217 ],
  [               0,               0,               0,               0,               0,               0,     0.039246284,     -0.38635942,      -0.5158382,      0.55799718 ],
  [               0,               0,               0,               0,               0,               0,     0.045807309,      0.33870836,      0.31676125,      -1.8799981 ],
  [               0,               0,               0,               0,               0,               0,    0.0005003551,   0.00032588917,  -0.00059167961,    -0.002458145 ],
  [               0,               0,               0,               0,               0,               0,      0.03948629,     0.083930405,     -0.30236909,      0.31918537 ],
  [               0,               0,               0,               0,               0,               0,     -0.51632729,        0.510013,       1.2031549,      -2.4343885 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0016816743,   -0.0030614719,   -0.0069889411,     0.014729315 ],
  [               0,               0,               0,               0,               0,               0,     -0.24344761,     -0.24859085,       2.2412318,       1.7407463 ],
  [               0,               0,               0,               0,               0,               0,      0.33390707,       1.3704031,       2.2271225,      -17.933788 ],
  [               0,               0,               0,               0,               0,               0,       -2.043711,        6.939211,       11.521126,      -39.724368 ],
  [               0,               0,               0,               0,               0,               0,      -1.2668806,       2.5806845,        6.479679,       -12.98685 ],
  [               0,               0,               0,               0,               0,               0,   -0.0013670888,   -0.0022628448,    0.0014867283,     0.026180638 ],
  [               0,               0,               0,               0,               0,               0,      0.15137659,     -0.21463049,     -0.88536624,       1.7279508 ],
  [               0,               0,               0,               0,               0,               0,     -0.59181733,       1.1238889,       2.7096272,       -6.785211 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0015679932,   0.00021519952,   -0.0066721647 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.1125258,      0.13884023,       18.354058 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.1487587,     -0.28524107,      -13.130312 ],
  [               0,               0,               0,               0,               0,               0,               0,       -2.459305,      0.09615073,        12.36729 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.011251684,       2.8271141,       12.632229 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0020513406,    0.0055358249,     0.015609518 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.4824165,     0.066777796,      -18.644299 ],
  [               0,               0,               0,               0,               0,               0,               0,       6.5599804,      0.43988212,      -32.673758 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00042704948,    0.0018188121 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.69584547,       4.5377071 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.2351488,       1.6638254 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.12672293,     -0.79008645 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.32893332,      0.84289706 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   6.3303844e-05,    0.0006497374 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.1635749,       5.9973895 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.16815976,     -0.85872826 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.2540215,      -1.5215751 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.23225125,     -0.85685136 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0055706262,    0.0010363596,    -0.029067805 ],
  [               0,               0,               0,               0,               0,               0,               0,      -13.396258,       1.4744302,       83.807058 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.61472645,       1.5045158,      -2.1159599 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.0444863,     -0.28432471,      -5.1204043 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0010964854,   0.00018804852,   -0.0050397068 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.0762069,       1.5343158,       3.5543849 ],
  [               0,               0,               0,               0,               0,               0,               0,        -2.49465,      -0.3753031,       13.086882 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0026612978,    0.0012074916,    0.0032143747 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0045654559,     0.008493174,       5.4503111 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.043062278,    -0.015511865,      -28.178608 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.11461034,     0.047218271,      -0.1883586 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.023864509,     0.012658065,      0.46655297 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer2_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0056554861,     0.007857905 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.5115015,       3.8339221 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.36252894,       -1.248989 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.13460352,    -0.041826028 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0020483181,   7.8068665e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.39971105,      0.84591999 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.5704694,      -1.1302181 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.010005471,    0.0011100013 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -4.4583052,       6.5954283 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.39410201,        -1.63257 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.93470248,      -1.8805284 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.24592895,     -0.11841864 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0037652151,    0.0028395422 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.42496738,       1.0332889 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.5114414,     -0.84867662 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.054383962,    -0.034536993 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.81844172,       3.8889705 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.8721337,      -5.1547295 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -4.7000131,       3.7155878 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0096276517,    0.0077404169 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -11.60047,        10.95025 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -8.1792655,       6.9607476 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00066077511,   0.00024358835,    0.0014802668,    0.0047400629 ],
  [               0,               0,               0,               0,               0,               0,       1.1588954,       -2.218142,      -1.0840152,       5.5586092 ],
  [               0,               0,               0,               0,               0,               0,      0.55538198,     -0.77452625,     -0.47773192,     -0.68237356 ],
  [               0,               0,               0,               0,               0,               0,     -0.26969528,       0.1647105,      0.95349261,     -0.76734274 ],
  [               0,               0,               0,               0,               0,               0,   0.00012463207,  -0.00036417349,  -0.00030849673,    0.0010057489 ],
  [               0,               0,               0,               0,               0,               0,      -1.2283655,       1.1170182,       1.3204986,     -0.28537007 ],
  [               0,               0,               0,               0,               0,               0,     -0.41401057,     -0.49515207,     -0.06270749,     -0.76830462 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0033953476,    0.0047960793 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -97.504154,       135.68043 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       11.076377,      -16.515941 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.8028675,      -3.5239116 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,    -0.012721245,    0.0051574594,     0.012293853,    0.0065900421,   -0.0069841122,    -0.010801101,     0.017010793,    -0.011895017,   0.00050700064 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0010894408,    0.0030456442,    0.0040668699,   0.00093220253,    0.0051407998,     0.010265006 ],
  [               0,               0,               0,               0,    -0.019792064,    -0.028064844,   -0.0028093886,     0.012318402,    0.0046689785,       32.755867 ],
  [               0,               0,               0,               0,    0.0036255369,    -0.019093777,    -0.021913075,    -0.010941261,    -0.014743047,      -2.1570142 ],
  [               0,               0,               0,               0,    -0.045293075,     0.023870812,    -0.038007115,    -0.034473044,    -0.039469631,       1.0764016 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer2_etaoffv5_55_e = [
 [
  [    -0.076462679,     0.030756641,     0.070445871,     0.049589935,    -0.010514586,    -0.066147156,    -0.051106517,     0.065624703,   -0.0073885025,   -0.0050348462 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00072199099,   0.00073773129,   0.00059511145,   0.00063534022 ],
  [               0,               0,               0,               0,               0,               0,       9.3040416,       4.6761826,      -7.0455254,      -27.873274 ],
  [               0,               0,               0,               0,               0,               0,      -1.6200014,     -0.78952862,       1.0769793,       4.3160684 ],
  [               0,               0,               0,               0,               0,               0,       1.1208512,      0.59732867,     -0.74579834,      -3.9032337 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0022694458,    0.0041984228 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       46.723566,      -67.906776 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.2406642,        3.683054 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.5250195,       2.3989281 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00016271007,   0.00031981618,    0.0011656935,    0.0045880751 ],
  [               0,               0,               0,               0,               0,               0,       1.1744201,      -2.1670893,      -4.1566398,       10.674701 ],
  [               0,               0,               0,               0,               0,               0,    -0.065772195,      0.20039362,    -0.064366784,      -1.4423816 ],
  [               0,               0,               0,               0,               0,               0,     0.031283045,    -0.028491796,      -0.1086373,      0.20097944 ],
  [               0,               0,               0,               0,               0,               0,   0.00019240871,  -0.00024565883,   -0.0005556189,    0.0010605507 ],
  [               0,               0,               0,               0,               0,               0,     -0.11401722,      0.11500634,     -0.15765898,       1.1518276 ],
  [               0,               0,               0,               0,               0,               0,    -0.092018467,     0.019115867,     -0.35295079,       -1.229646 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00045417295,   0.00065168374,    0.0013552311 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.66290216,      -0.0658126,       6.2858965 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.55890793,       3.0288732,       -5.195033 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.61827248,       2.2427288,      -1.8455886 ],
  [               0,               0,               0,               0,               0,               0,               0,  -0.00030512185,  -1.9461682e-05,    0.0026529507 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.46964614,      -2.1841886,       3.4508436 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.10354235,      0.89089826,      -3.0131403 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -0.00089582659,   -0.0013226212,  -0.00041672998,    0.0074153085,     0.042644903 ],
  [               0,               0,               0,               0,               0,       2.8339452,      -3.1180167,      -15.547971,      -19.454361,       87.522596 ],
  [               0,               0,               0,               0,               0,     -0.27813086,      0.20200717,       1.5920189,       2.5778275,      -10.085893 ],
  [               0,               0,               0,               0,               0,     -0.58836515,      0.66934496,       3.4351808,       4.4065467,      -19.984547 ],
  [               0,               0,               0,               0,               0,   0.00028000089,  -5.4092174e-05,  -0.00092873311,   -0.0019114122,    0.0024290284 ],
  [               0,               0,               0,               0,               0,     -0.06985991,     0.084098501,      0.24493582,     0.011834365,    -0.085500577 ],
  [               0,               0,               0,               0,               0,    -0.015345108,      0.64700256,       1.5246125,     -0.25464787,       -18.80298 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.007736988,   -0.0049398629 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -5.9493644,        16.53705 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.5372219,      -5.0348113 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.6070204,      -3.7402787 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,      -1.4198128,       1.5197725,     -0.37142206,     0.025994316,   0.00045403095 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer1_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,   -0.0069841477,     0.026817806,    -0.027762605,    0.0074247738 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00039022446,   0.00079443336 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.6944405,       3.5599567 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.1751339,     -0.91308029 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.65115138,     -0.75961038 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.25843721,   -0.0045463742 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.027443715,    0.0073936035 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -15.196189,       12.601719 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -7.1399836,       5.1952421 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       16.985232,      -13.509686 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   9.6511961e-05,  -3.2148446e-05,   0.00057125003 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.1011924,      0.15608079,       4.7937597 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.13853212,      0.18993231,     -0.26600649 ],
  [               0,               0,               0,               0,               0,               0,               0,       0.2796475,     -0.15373459,     -0.86746207 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.061454512,   -0.0042630164,     -0.45571622 ],
  [               0,               0,               0,               0,               0,               0,               0,   1.3516958e-05,   2.9926244e-05,  -5.4312648e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.9241829,     0.077405181,       2.2538439 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.3183862,       1.1659225,       0.5317308 ],
 ],
 [
  [               0,               0,               0,               0,     -0.11206752,      0.37249581,     -0.43513345,      0.22297029,    -0.058194425,    0.0028852129 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.15226772,      0.29727488,     -0.11664666,    -0.096146793,     0.077086688,    -0.010756896 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0045425471,    0.0039784263 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.6576805,       -2.605156 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -30.428693,       43.104321 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -78.720836,       110.58976 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.09678178,     -0.24127316 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer1_etaoffv5_35_e = [
 [
  [               0,               0,               0,               0,    -0.096507788,      0.11086074,     0.039750935,    -0.058686411,    0.0016856965,   -0.0017351241 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0080138812,    0.0059745915 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.78259859,       2.9196321 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -7.0418164,       9.3172646 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -4.4217574,       5.3304288 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.26356116,     -0.43482468 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.12038706,      0.18419737 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -152.70444,       233.04485 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -45.504481,       66.278231 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       76.808162,       -114.0743 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00051999165,    0.0019674267 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.26162301,       1.4455879 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -0.317632,     -0.57917584 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.31996776,     -0.53390554 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.053800221,      0.14150688 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00012067889,  -1.3647147e-06,   0.00027102729,    0.0012579178 ],
  [               0,               0,               0,               0,               0,               0,    -0.045279083,      0.11254003,      0.17589778,       3.4946831 ],
  [               0,               0,               0,               0,               0,               0,     -0.92322544,    -0.098847107,       1.3835763,       2.1613411 ],
  [               0,               0,               0,               0,               0,               0,    -0.084988302,     -0.38257432,     -0.29340343,      0.92998802 ],
  [               0,               0,               0,               0,               0,               0,     0.075313965,      0.30131381,      0.22005431,      -1.8196854 ],
  [               0,               0,               0,               0,               0,               0,   0.00030774819,   0.00018452518,   -0.0004371292,   -0.0013659024 ],
  [               0,               0,               0,               0,               0,               0,     -0.18019469,    -0.030765146,     0.057994394,       2.0990266 ],
  [               0,               0,               0,               0,               0,               0,     -0.52426813,      0.46671563,       1.2079545,      -1.9683545 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00054930418,  -0.00097719301,   -0.0021272111,    0.0050799935 ],
  [               0,               0,               0,               0,               0,               0,        1.770821,      -4.2681699,      -7.9844159,       22.883566 ],
  [               0,               0,               0,               0,               0,               0,       3.5351068,      -4.2458792,      -11.516973,       4.2354205 ],
  [               0,               0,               0,               0,               0,               0,      0.69541227,        2.379817,     -0.41976585,      -21.559154 ],
  [               0,               0,               0,               0,               0,               0,     -0.38034562,      0.60638554,       2.1889734,       -3.412274 ],
  [               0,               0,               0,               0,               0,               0,  -0.00094447651,   -0.0026657956,  -0.00028696345,     0.027144781 ],
  [               0,               0,               0,               0,               0,               0,      0.24729035,     -0.36218062,      -1.2065468,       2.2487276 ],
  [               0,               0,               0,               0,               0,               0,     -0.54788158,       1.1240133,       2.4717702,      -6.6807786 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0013080018,    0.0056666457 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -12.626461,       30.356017 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -25.711631,       58.020836 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -18.985774,       42.882278 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.5767919,      -5.8444796 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00010144263,     0.001120024 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.70455673,       1.6156981 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.2198815,       4.1650608 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00010560221,    0.0027723404 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.58516075,       4.1611427 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.3799037,       1.9960442 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.092304081,      -1.3194651 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.23136598,      0.60923164 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    6.597312e-05,    0.0006549894 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.1323117,       6.0476728 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.15879447,     -0.88864582 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.27433368,      -1.4852899 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.2310511,     -0.85396978 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0051599154,    0.0010812177,    -0.026680739 ],
  [               0,               0,               0,               0,               0,               0,               0,      -12.455364,       1.1582533,       78.896755 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.40825653,    -0.062306272,       0.4764627 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.0504453,     0.050772831,      -5.9743361 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0015889859,   0.00024681358,   -0.0081192535 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.81499262,    -0.015462728,       5.7937036 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.5078177,     -0.33434398,        13.05159 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0044616211,     0.011480798 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.029054108,       5.6163101 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.13598467,      -10.375458 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.26161489,     -0.12312112 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.10867028,      0.38021854 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer2_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0058797028,    0.0056267605 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.6283104,       5.4530736 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.14030117,      -1.3680528 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.097455657,    -0.013812444 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0013845982,   0.00087617474 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.4786719,      0.99954203 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.6546839,      -1.8060514 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0035487701,    0.0056392424 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.0313025,        6.214664 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.10254302,      -1.2406247 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.21914531,      -1.3991285 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.14015729,    -0.074270185 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00045130997,  -0.00024335558 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.6465866,       -1.228936 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -95.690201,      -126.99206 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.025485035,     0.029247534 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       12.481573,      -6.1468366 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       13.692288,      -12.228703 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -15.964688,        12.79101 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.022391842,    -0.019193121 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       15.030286,      -10.873926 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -169.42395,       120.86911 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   1.8247617e-05,   0.00050198721,    0.0019060293,    0.0042569329 ],
  [               0,               0,               0,               0,               0,               0,       1.4295055,      -2.6929188,      -1.1494713,        6.690252 ],
  [               0,               0,               0,               0,               0,               0,      0.22413744,      0.21176844,      -1.4880726,      -0.3781981 ],
  [               0,               0,               0,               0,               0,               0,     -0.49229504,       1.0979312,     -0.27349507,     -0.26663017 ],
  [               0,               0,               0,               0,               0,               0,   0.00041144711,  -0.00059213279,  -0.00062390124,    0.0015068962 ],
  [               0,               0,               0,               0,               0,               0,      -1.0015404,       0.9193755,      0.98375363,    -0.049699424 ],
  [               0,               0,               0,               0,               0,               0,      0.51247115,     -0.84096098,       -1.591759,      0.17374044 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.066486901,     0.099221533 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -12.326933,       20.364726 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.4839849,      -8.8648494 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.057362107,     0.048079273 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,     -0.96622076,       1.3811798,      0.26842926,     -0.76540275,     -0.31797271,       0.6051626,     -0.21549736,     0.019098684,  -0.00055976773 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.020094348,     0.017895727,    0.0099784081,   -0.0082102299,    -0.044095173,      -0.1091071 ],
  [               0,               0,               0,               0,      -3.5829459,       -2.982392,      -1.3103415,       2.1991635,       8.7534175,       24.233553 ],
  [               0,               0,               0,               0,     -0.82843851,       1.1264799,       2.9047726,        3.196528,      -0.7160563,      -14.446216 ],
  [               0,               0,               0,               0,    -0.012091122,    -0.010154065,    0.0033724467,     0.024265004,      0.02983589,     -0.34431071 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer2_etaoffv5_35_e = [
 [
  [     -0.13807458,      0.07004154,      0.11903809,     0.058890939,    -0.042099667,     -0.10061994,    -0.040811241,      0.11161414,    -0.029777552,   -0.0071677697 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,      -17.924302,      -8.5652801,         13.6758,        58.88819 ],
  [               0,               0,               0,               0,               0,               0,       8.5443153,      -8.9177228,      -16.429442,       16.361542 ],
  [               0,               0,               0,               0,               0,               0,       1.8208121,      -6.0953938,      -6.4823383,       16.833948 ],
  [               0,               0,               0,               0,               0,               0,      0.33650711,     -0.51615038,     -0.71612826,       1.0999949 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.098746561,      -0.1421585 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       28.402706,      -39.501794 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.2908986,      -6.3431709 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.010315133,      0.15281601 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00017907128,  -0.00013502705,    0.0011862664,    0.0070325816 ],
  [               0,               0,               0,               0,               0,               0,       1.0261529,      -1.8323767,      -3.3474231,       9.8136178 ],
  [               0,               0,               0,               0,               0,               0,      0.17310328,      -0.2405424,     -0.67470406,      -0.3681043 ],
  [               0,               0,               0,               0,               0,               0,    -0.085833889,      0.10270499,      0.27393593,     -0.22879337 ],
  [               0,               0,               0,               0,               0,               0,   7.7386054e-05,  -0.00017412585,  -0.00033004865,    0.0013364975 ],
  [               0,               0,               0,               0,               0,               0,      0.97659469,      -1.5264315,      -2.1225614,        3.736093 ],
  [               0,               0,               0,               0,               0,               0,      0.33529308,    -0.026455391,     -0.82333717,      -2.2775618 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    -0.012665686,     0.054897382,    -0.050186143 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.0426255,      -19.083455,       25.242675 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.14247372,     -0.33613429,      -1.1853392 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.3178644,       5.2678371,       5.7388547 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0052699186,    -0.041535962,      0.13187713 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0092347094,    -0.015326342,     0.014468822 ],
  [               0,               0,               0,               0,               0,               0,               0,       9.9026166,      -39.588122,       36.747628 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -0.00028396561,  -0.00046885234,  -0.00013520998,    0.0031517867,     0.018906328 ],
  [               0,               0,               0,               0,               0,       1.7525404,      -2.0020999,      -9.8640521,      -12.221965,       58.298858 ],
  [               0,               0,               0,               0,               0,      0.11070877,     -0.21223591,     -0.53987785,     -0.14599245,       1.4946098 ],
  [               0,               0,               0,               0,               0,     -0.49352197,      0.77107306,       3.4074569,       3.8994022,      -21.450815 ],
  [               0,               0,               0,               0,               0,   0.00075915371,   0.00091381958,   0.00030764147,   -0.0030556723,    -0.013636335 ],
  [               0,               0,               0,               0,               0,     -0.17906126,      0.25678687,      0.93299031,      0.62926069,      -4.4516195 ],
  [               0,               0,               0,               0,               0,      0.45261859,     -0.40324061,      -2.2254278,      -3.2496028,       8.4024206 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00074681305,    0.0086751198 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.3936996,       3.5995807 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       7.0339762,      -18.576192 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.061600748,    -0.066358472 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      -3.8966492,      -1.4546258,       6.8259656,      -3.8251739,      0.90004962,    -0.099026696,    0.0031685786 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer1_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,  -0.00034489719,     0.011443038,    -0.016281063,    0.0046676951 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00043461959,   0.00085962099 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.71693899,       3.3896906 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.22399892,     -0.98037115 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.55902659,     -0.82827346 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.23963803,   -0.0032144677 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.028659231,    0.0075642814 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -14.843775,       12.314719 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -6.3202717,       4.5484667 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       16.272078,      -12.942398 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0031268015,    -0.010472773,     0.011558804 ],
  [               0,               0,               0,               0,               0,               0,               0,       -1.352211,       3.4645049,     -0.59461184 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.88812064,      -1.8092325,    -0.028233419 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.3512094,      -3.0691523,      0.61342342 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.13603206,      0.23567623,     -0.15968843 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00066160615,     -0.00157017,    0.0009252957 ],
  [               0,               0,               0,               0,               0,               0,               0,       23.221552,      -43.723661,       20.476694 ],
  [               0,               0,               0,               0,               0,               0,               0,      -72.057835,       153.78813,      -73.714118 ],
 ],
 [
  [               0,               0,               0,               0,     -0.13193706,      0.43197647,     -0.50388432,      0.26115605,    -0.068294594,    0.0039018194 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.14827945,      0.28754195,      -0.1080747,    -0.099465604,     0.077628797,     -0.01078226 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     -0.16506042,     0.003591245,      0.34935061 ],
  [               0,               0,               0,               0,               0,               0,               0,       22.994579,    -0.089690948,      -43.981338 ],
  [               0,               0,               0,               0,               0,               0,               0,      -6.7868567,      -0.2137537,       13.524494 ],
  [               0,               0,               0,               0,               0,               0,               0,      -30.077962,    -0.039825615,       59.694103 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.019470432,    0.0071139529,      -0.2732862 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer1_etaoffv5_37_e = [
 [
  [               0,               0,               0,               0,    -0.075540264,     0.087570212,     0.027576053,    -0.035475743,   -0.0076967349,  -0.00076610107 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0080867387,     0.005173973 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.77250128,        2.924177 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -9.995366,       13.656568 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.9951037,       4.7211308 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.25179786,     -0.41975294 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.26052052,       0.3923067 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -124.52189,       191.25871 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -41.672478,       60.962752 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       99.736021,      -148.16506 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00056263257,    0.0021924361 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.14787643,       1.5503777 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.22576288,     -0.72267461 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.24209468,     -0.65820424 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.052214246,      0.13389635 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00010856761,   8.8598158e-05,   0.00031716489,   0.00055377582 ],
  [               0,               0,               0,               0,               0,               0,      -0.0422344,      0.27404152,      0.38233815,       3.0781784 ],
  [               0,               0,               0,               0,               0,               0,     -0.96387034,     -0.75695305,      0.24470765,       2.7408397 ],
  [               0,               0,               0,               0,               0,               0,     -0.51654913,     -0.84262166,     -0.50622208,       1.7941992 ],
  [               0,               0,               0,               0,               0,               0,     0.092073916,      0.38875256,      0.28635928,      -2.2831479 ],
  [               0,               0,               0,               0,               0,               0,   0.00060989053,   0.00040430033,  -0.00045856497,   -0.0017414433 ],
  [               0,               0,               0,               0,               0,               0,    -0.044721388,    0.0033162211,     -0.21124109,        1.187161 ],
  [               0,               0,               0,               0,               0,               0,     -0.55701023,      0.98023906,       1.7648891,      -4.6542913 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,  -1.5944046e-05,   -0.0021484541,    0.0064558273 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.9065834,       9.7655306,      -8.7817419 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.1771299,      -13.744808,       15.706591 ],
  [               0,               0,               0,               0,               0,               0,               0,      -5.0863957,       19.962626,      -22.815144 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.083629952,        2.699387,      -5.1546015 ],
  [               0,               0,               0,               0,               0,               0,               0,  -0.00075159582,    0.0040648226,   -0.0011752968 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0013426359,     0.082501968,      0.23977997 ],
  [               0,               0,               0,               0,               0,               0,               0,      -0.1496593,     -0.14514206,     -0.62114945 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.005383919,       0.0177807 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -21.244794,       50.232381 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -35.086476,         79.6319 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -29.055373,       66.167172 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.7163123,      -8.2408553 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.043472834,      0.10011458 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -50.248623,       117.80997 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -11.664427,       26.558587 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0018310581,     -0.00114236 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.19172854,       3.1623676 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.2836941,       1.7713113 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.9044359,       1.0079599 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.40359188,       1.0089176 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.000383731,    0.0022145863 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        1.585636,      -2.3333071 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.8703236,      -12.712325 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -7.7896438,       17.367054 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.016030046,     -0.33371279 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0055178504,     0.001203324,    -0.028989403 ],
  [               0,               0,               0,               0,               0,               0,               0,       -12.29613,       1.3482124,       77.223152 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.11667975,     -0.18080832,     -0.98492994 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.93020935,     0.062656344,      -5.2803718 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0014277927,   0.00027006141,   -0.0071414194 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.81932814,    0.0055419274,       5.7602876 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.2989709,     -0.30527155,        11.68164 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0049513889,     0.010911025 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.097183589,       5.3090405 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.17712596,       56.249889 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.21111187,     -0.27855053 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.13899462,       0.2646646 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer2_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0058153474,    0.0056527869 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.5871416,       5.4393617 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.14837518,      -1.3717022 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.10654481,    -0.010762217 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0013412214,    0.0008763356 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.4847925,       0.9939912 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.6313981,      -1.8185993 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0033336719,     0.005781422 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -2.888165,       6.1236125 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.10284478,      -1.2384808 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.21714761,      -1.3952323 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.14102634,    -0.074267629 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00038270135,   0.00021729335 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.2335919,      -2.1155562 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -60.476726,      -25.461397 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.020883985,   -0.0080731662 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       7.7366294,      -2.3220639 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        7.609419,      -7.3990708 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.1278305,       2.7176177 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0031099989,    0.0053899062 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.56663752,      0.50498574 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -18.638439,     -0.10548936 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -2.9109399e-05,   0.00048588747,    0.0019389418,    0.0043661515 ],
  [               0,               0,               0,               0,               0,               0,        1.384901,      -2.6331975,      -1.0777583,       6.5784386 ],
  [               0,               0,               0,               0,               0,               0,      0.22051059,      0.12104754,      -1.2824596,     -0.48807425 ],
  [               0,               0,               0,               0,               0,               0,     -0.44311576,       1.0002923,     -0.23426707,     -0.25696386 ],
  [               0,               0,               0,               0,               0,               0,    0.0004214269,  -0.00057161007,  -0.00062332778,    0.0014698822 ],
  [               0,               0,               0,               0,               0,               0,      -0.9709047,      0.89841009,      0.95658537,    -0.043907736 ],
  [               0,               0,               0,               0,               0,               0,      0.47874893,     -0.78926199,      -1.5120092,      0.07592885 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.061494321,     0.092569588 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -11.527937,       19.227172 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.2904994,      -8.6066137 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.011502364,      0.14541025 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,       -1.385495,       1.9937367,      0.33487781,      -1.1000862,     -0.41014673,      0.85451107,     -0.32066183,     0.034981472,   -0.0012031792 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.011633284,     0.010431812,    0.0059975486,   -0.0042444765,    -0.024491541,    -0.060983958 ],
  [               0,               0,               0,               0,     -0.66697755,     -0.53536145,     -0.18748951,      0.52975509,       1.8571337,       8.1703597 ],
  [               0,               0,               0,               0,      -2.0568586,      0.35873231,       2.8958114,       4.2824772,       1.7650373,      -10.478918 ],
  [               0,               0,               0,               0,   -0.0094473647,    -0.011053064,    0.0021471947,     0.029280185,     0.052050309,     -0.28914089 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam100_layer2_etaoffv5_37_e = [
 [
  [     -0.10745088,     0.050672878,     0.091257786,     0.049145201,    -0.027661954,    -0.076726112,    -0.034642225,     0.085545161,    -0.020696601,   -0.0080532474 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,      -22.637793,      -10.809952,        17.34151,       74.632039 ],
  [               0,               0,               0,               0,               0,               0,       9.8169971,      -10.005595,      -18.774177,       18.016848 ],
  [               0,               0,               0,               0,               0,               0,      0.37894881,      -4.8461737,      -4.0078049,       15.179603 ],
  [               0,               0,               0,               0,               0,               0,     -0.14003548,      0.21961224,      0.34550781,     -0.53869886 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.096939755,     -0.13932101 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       28.007916,      -38.942429 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.1097006,      -7.5826971 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        0.027046,     0.094695185 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00064792179,  -0.00076306909,   -0.0010034433,     0.008819831 ],
  [               0,               0,               0,               0,               0,               0,       1.0900694,      -2.0481872,      -3.9158409,       11.007846 ],
  [               0,               0,               0,               0,               0,               0,      0.10683247,     -0.12740242,     -0.48944013,     -0.67298964 ],
  [               0,               0,               0,               0,               0,               0,    -0.073192128,     0.030531175,      0.19749517,     0.092525126 ],
  [               0,               0,               0,               0,               0,               0,  -0.00057526219,   0.00036246778,   0.00098630169,    0.0011328255 ],
  [               0,               0,               0,               0,               0,               0,       1.1419639,      -1.7609301,      -2.8928915,       4.7033672 ],
  [               0,               0,               0,               0,               0,               0,      0.94982201,      -1.2066109,      -2.9562499,        1.473725 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,      0.02265556,    -0.076451605,     0.068387599 ],
  [               0,               0,               0,               0,               0,               0,               0,       0.6207894,      -8.4523116,       18.329198 ],
  [               0,               0,               0,               0,               0,               0,               0,       0.5311648,      -1.2638365,      -1.0405794 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.64059542,       2.3264334,      -1.9289871 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0056090223,    -0.026349667,     0.031430043 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.3015031,      -7.1137339,       9.8431161 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.6526578,      -16.474823,       16.431782 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -0.00031487851,  -0.00048731273,  -9.3225449e-05,    0.0033728001,     0.019584195 ],
  [               0,               0,               0,               0,               0,       1.5965999,      -1.8298938,      -8.9586917,      -11.047014,       53.038386 ],
  [               0,               0,               0,               0,               0,     -0.17957288,       0.1410741,       1.0940107,       1.7345693,      -7.6791694 ],
  [               0,               0,               0,               0,               0,     -0.61503484,      0.86846917,       3.9370034,       4.5167219,       -24.38912 ],
  [               0,               0,               0,               0,               0,   0.00088916794,   0.00095709707,   -0.0001471985,   -0.0052872272,    -0.021263445 ],
  [               0,               0,               0,               0,               0,      0.18431862,     -0.26082731,      -1.0907247,      -1.2918543,       6.9414167 ],
  [               0,               0,               0,               0,               0,     0.091837343,      0.10306847,      -0.0485683,     -0.92901722,      -4.5685704 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00075418981,    0.0087542313 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.1339614,     -0.17148953 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       6.8673015,      -18.167914 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.32140532,     -0.70554144 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.066175375,    -0.020755725,       1.1428938,     -0.54699369,      0.13401642,    -0.022288637,   0.00058830333 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]

###############################################################################
# Tables for v5 sshape correction, 200 GeV photon clusters
#


gam200_layer1_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,    -0.012186622,     0.039019136,    -0.037186446,    0.0098901951 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0004677741,    0.0007687623 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.3231517,       4.0835262 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.27451249,      -1.0074086 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.63501472,     -0.82565613 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.26540416,   -0.0058170235 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.015921082,     0.025012246 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -36.682674,       28.863362 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -6.3497526,       5.4273356 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       37.749898,       -30.07535 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.016283521,      -0.0122922 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.5928443,       3.0774736 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       19.001232,      -15.867564 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0005543665,    0.0016847278 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.65914981,       2.8091753 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.32399029,      -1.2885962 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.16881934,      -1.4544156 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.025095596,     -0.15248172 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -2.2013614e-06,   9.6200812e-07 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       30.397712,       2.0874458 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.2129798,      -2.2521226 ],
 ],
 [
  [               0,               0,      -0.1238139,      0.16650991,       0.1080069,     -0.11809562,     -0.17142242,       0.1998052,    -0.073308765,    0.0056516495 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.092761801,      0.17909169,    -0.061636067,    -0.075945892,     0.057042501,   -0.0068497891 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0022542099,    0.0033368033,    0.0054298958 ],
  [               0,               0,               0,               0,               0,               0,               0,       6.5334211,    -0.047641503,      -9.6410574 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.7260543,     -0.45336337,      -8.2406895 ],
  [               0,               0,               0,               0,               0,               0,               0,      -35.622329,      0.25972168,        70.51013 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.025085068,    0.0029114885,     -0.25676434 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer1_etaoffv5_55_e = [
 [
  [               0,               0,               0,               0,      0.13500736,     -0.25111381,     0.095739385,     0.017464384,    -0.004408882,    0.0033736957 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0028961109,    0.0053313214 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.67415138,       1.7523419 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -20.032222,       28.377504 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -24.610193,       34.651457 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.1487117,     -0.29460587 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.091380439,      0.14371604 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -34.260303,       56.219277 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       16.677245,      -26.240606 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.9840865,      -5.8240537 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00046720377,    0.0015416642 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.40372334,       1.4888248 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.2890497,     -0.67918458 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.39663577,     -0.42816371 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.084926365,      0.20806321 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00041758451,  -0.00012355575,    0.0008663366,    0.0038891678 ],
  [               0,               0,               0,               0,               0,               0,     0.094005114,      0.12730772,     -0.20558185,       2.1186552 ],
  [               0,               0,               0,               0,               0,               0,     -0.33545583,      0.30223355,      0.77230969,      -1.6271811 ],
  [               0,               0,               0,               0,               0,               0,     0.017158304,     -0.34979705,     -0.39769386,      0.66124701 ],
  [               0,               0,               0,               0,               0,               0,    -0.081539241,      0.18504612,      0.37658065,     -0.74515722 ],
  [               0,               0,               0,               0,               0,               0,   0.00029879615,   0.00012387782,   -0.0005321474,    -0.001347132 ],
  [               0,               0,               0,               0,               0,               0,     -0.31567634,    0.0010875953,      0.33823513,       2.2925194 ],
  [               0,               0,               0,               0,               0,               0,     -0.64343498,       0.4337783,       1.3855753,      -1.2237179 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0054907029,    -0.013696968,    -0.025459309,     0.064812884 ],
  [               0,               0,               0,               0,               0,               0,       4.1532289,      -7.2787687,      -15.971649,       30.822332 ],
  [               0,               0,               0,               0,               0,               0,       2.8057024,      -9.8206939,       -9.739363,       33.995254 ],
  [               0,               0,               0,               0,               0,               0,      -9.5757506,       18.483063,        37.07459,      -73.823456 ],
  [               0,               0,               0,               0,               0,               0,      -3.9050422,       7.1553588,       20.632025,       -38.23002 ],
  [               0,               0,               0,               0,               0,               0,     0.004471642,    -0.010883979,    -0.022639647,      0.05807143 ],
  [               0,               0,               0,               0,               0,               0,     -0.50063616,       1.2294692,       3.0744542,      -6.8339913 ],
  [               0,               0,               0,               0,               0,               0,      0.27040143,     -0.86732054,       1.3569189,      -3.2459194 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00044298226,    4.227497e-05,   -0.0016420276 ],
  [               0,               0,               0,               0,               0,               0,               0,      -107.62122,      -1.7894358,       570.96047 ],
  [               0,               0,               0,               0,               0,               0,               0,       35.722516,     -0.67847358,       -192.4617 ],
  [               0,               0,               0,               0,               0,               0,               0,      -13.798315,      0.34959377,        73.49065 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.45163935,      0.49790597,       4.3626725 ],
  [               0,               0,               0,               0,               0,               0,               0,  -0.00086172616,   0.00022751928,    0.0066101241 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.9524195,    -0.036636892,      -26.881529 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.3731902,      0.24915829,      -6.0489728 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00047364062,    0.0019255802 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.35063682,        3.772356 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.036971417,      -1.3623722 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.70072242,       0.4975764 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.42435785,       1.0613451 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00026138057,    0.0011632581 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.2904412,       5.4114528 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.16670534,     -0.71036849 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.14998638,       -1.728531 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.076982006,     -0.48452029 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0037513036,    0.0024646139,     -0.01950141 ],
  [               0,               0,               0,               0,               0,               0,               0,       -8.987274,       1.3550631,       56.354814 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.30455692,     0.026417165,      0.18401635 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.56430304,     0.061090742,      -3.4093911 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0023308383,    0.0043099387,    0.0075293377 ],
  [               0,               0,               0,               0,               0,               0,               0,      -7.7854202,      0.31650231,       53.027244 ],
  [               0,               0,               0,               0,               0,               0,               0,      -4.2568664,      -1.6984876,      -1.0490762 ],
  [               0,               0,               0,               0,               0,               0,               0,      -8.6787029,    -0.067957641,       52.430009 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.0903546,      0.13296509,      -6.5150178 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer2_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,      0.37021845,     -0.24999891,     0.049491761,    0.0049585255 ],
  [               0,               0,               0,               0,               0,               0,      -118.26623,       79.707013,      -15.936659,       4.9419716 ],
  [               0,               0,               0,               0,               0,               0,       -2.667615,       1.8517884,     -0.28248013,      -1.3062933 ],
  [               0,               0,               0,               0,               0,               0,        5.067245,       -3.464518,      0.64933247,    -0.052096615 ],
  [               0,               0,               0,               0,               0,               0,    -0.097028829,     0.067117683,    -0.013692335,   0.00096560487 ],
  [               0,               0,               0,               0,               0,               0,      -139.54419,       87.384814,      -15.139243,       1.3259044 ],
  [               0,               0,               0,               0,               0,               0,       39.313454,      -33.684758,        8.940809,      -1.8736261 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00031576289,    0.0019493088,    0.0054729529 ],
  [               0,               0,               0,               0,               0,               0,               0,      -4.5840265,       3.9839414,       3.4192168 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.52349405,    -0.056764136,      -1.1349215 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.0732072,      -1.2949737,     -0.91561919 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.98550403,     -0.90875395,      0.19999757 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.001607202,   -0.0010720967,   0.00059991208 ],
  [               0,               0,               0,               0,               0,               0,               0,      -0.1886124,      -1.4366952,       1.9054259 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.2726042,      -3.3687901,      -6.7785105 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0082585647,    0.0035038291,   0.00086883601 ],
  [               0,               0,               0,               0,               0,               0,               0,       6.7422736,      -1.3713125,      0.14235059 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.0905935,     -0.99383142,      -3.0672053 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.3881815,    -0.030828325,       2.1328791 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0039739096,  -0.00044608425,   -0.0021644218 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.35356686,      0.36965366,       1.9584523 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.9819069,     -0.60428763,    -0.097808374 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00087482623,   -0.0018724748,    0.0068142994,    0.0046781161 ],
  [               0,               0,               0,               0,               0,               0,       1.5876798,      -3.5821483,       1.5941836,       3.6161751 ],
  [               0,               0,               0,               0,               0,               0,      0.41026502,      -1.2678288,       1.0439246,      -1.5025294 ],
  [               0,               0,               0,               0,               0,               0,     -0.21117581,      -0.3194202,       1.7673749,      -1.1510153 ],
  [               0,               0,               0,               0,               0,               0,   -0.0017368353,    0.0004092013,    0.0056667875,   -0.0038165459 ],
  [               0,               0,               0,               0,               0,               0,     -0.29711365,       1.7032081,      -1.8557052,      0.65987444 ],
  [               0,               0,               0,               0,               0,               0,       7.4126029,      -10.488241,      -5.5198936,       6.0599174 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0012410903,    0.0076944767 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -130.53424,       180.85094 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       15.607505,      -22.655434 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.9556797,      -3.7567114 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,   -0.0072303753,     0.012961367,    -0.001723693,    0.0035863779,   -0.0085977106,   0.00014104419 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    -0.014306848,      0.01552624,    0.0025823959,    -0.010407676,     0.063944955 ],
  [               0,               0,               0,               0,               0,     -0.20923625,    -0.048874186,     -0.14678557,     -0.22720971,       41.543673 ],
  [               0,               0,               0,               0,               0,     0.055746075,     -0.13664474,   -0.0027215046,     0.024464199,       -2.061367 ],
  [               0,               0,               0,               0,               0,      0.27902372,    -0.036231144,    -0.062338529,    -0.067846195,     -0.15527308 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer2_etaoffv5_55_e = [
 [
  [               0,               0,               0,      -0.1878515,      0.36185606,     0.011769194,        -0.35568,      0.18468339,   -0.0054314266,   -0.0079750691 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00091270137,    0.0010809992,    0.0011943027,    0.0016582102 ],
  [               0,               0,               0,               0,               0,               0,       4.9622547,       2.3847549,      -4.0347756,      -13.507981 ],
  [               0,               0,               0,               0,               0,               0,      -2.1080123,      -1.0235655,       1.4585309,       5.9655765 ],
  [               0,               0,               0,               0,               0,               0,       1.2639365,      0.66708078,     -0.85094086,      -4.3645054 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.087379079,     -0.12441769 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       12.644756,      -16.178288 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.1638781,      0.50498993 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.72992652,       1.2054445 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00016964291,   0.00031035587,    0.0011672073,    0.0047091242 ],
  [               0,               0,               0,               0,               0,               0,        1.298046,      -2.4363916,      -4.5150652,        11.54538 ],
  [               0,               0,               0,               0,               0,               0,    -0.063661768,      0.18361851,    0.0093924532,      -1.5465047 ],
  [               0,               0,               0,               0,               0,               0,     0.025259627,    -0.031178824,    -0.066749569,      0.17209006 ],
  [               0,               0,               0,               0,               0,               0,    0.0001961026,  -0.00027258517,  -0.00057682987,    0.0011662396 ],
  [               0,               0,               0,               0,               0,               0,     -0.14254888,      0.09811745,     -0.12423877,       1.3237539 ],
  [               0,               0,               0,               0,               0,               0,     -0.11476351,    0.0041439172,     -0.32835377,      -1.0787158 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00023459549,   0.00066003766,    0.0025086731 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.59036165,     -0.35685756,       6.6670855 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.46818064,       2.5675689,      -4.6629735 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.51838654,       1.8987821,      -1.5529292 ],
  [               0,               0,               0,               0,               0,               0,               0,  -0.00019651267,  -2.9536496e-05,    0.0021939247 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.42795019,      -1.9705208,       3.1883346 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.022956413,      0.48824193,      -2.5632708 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -0.00054991005,  -0.00072816855,   2.6952677e-05,    0.0051097142,     0.026593091 ],
  [               0,               0,               0,               0,               0,       1.8205233,      -2.0447543,       -10.04318,      -12.450302,       57.753233 ],
  [               0,               0,               0,               0,               0,   -0.0034011581,    -0.070620124,     0.053001027,      0.45955043,      -1.4824768 ],
  [               0,               0,               0,               0,               0,      -0.1256276,      0.24954813,       1.0911929,       1.2345098,      -8.1262932 ],
  [               0,               0,               0,               0,               0,    0.0008021596,    0.0005661647,  -0.00089633707,   -0.0051360406,    -0.012678223 ],
  [               0,               0,               0,               0,               0,     -0.38234752,      0.68263943,       2.0567252,       1.1262978,      -11.377976 ],
  [               0,               0,               0,               0,               0,    -0.073112202,     -0.13579704,     -0.18733384,     0.030971421,       1.6796905 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0055556087,  -0.00091853179 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -38.500303,        96.85698 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       8.4355729,      -22.045046 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.9661738,      -4.5955823 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,      -1.4055911,       1.5219318,      -0.3784067,     0.024981545,   0.00046942879 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer1_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,   -0.0085434106,     0.032005134,    -0.033135108,    0.0092141607 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00050178355,   0.00073325866 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.4749028,       4.2087599 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.25416373,     -0.98631563 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.65608768,     -0.80758585 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.26418379,   -0.0087493737 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.7052081,       1.3944897 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.5591309,      0.91682704 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -17.476067,       13.455592 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       15.084489,      -11.999311 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,  -0.00096989157,  -0.00010218296,    0.0032939815 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.92496345,       2.8530905,      0.27298905 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.4169587,      -4.5517067,       1.1437742 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.9684349,      -4.2315852,      0.94012605 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.68604226,       1.3455766,     -0.78206895 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0012115773,   -0.0028082957,    0.0016100111 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.0232697,      -1.5720215,      0.36032473 ],
  [               0,               0,               0,               0,               0,               0,               0,         8.25187,        4.775672,      -4.8642503 ],
 ],
 [
  [               0,               0,     -0.10511204,      0.13353983,     0.086096982,    -0.085392775,     -0.12373971,      0.13132949,    -0.046944482,    0.0025147943 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,    -0.022532485,    -0.075800525,      0.28298292,     -0.29189564,        0.120049,    -0.013948441 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,      -0.1484112,    0.0027371512,      0.31076611 ],
  [               0,               0,               0,               0,               0,               0,               0,        79.81484,       0.1742455,      -161.10966 ],
  [               0,               0,               0,               0,               0,               0,               0,       -47.98437,     -0.13193667,       100.21443 ],
  [               0,               0,               0,               0,               0,               0,               0,      -34.875536,      0.82728669,       68.211877 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.052016561,    -0.045793941,     -0.41653497 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer1_etaoffv5_35_e = [
 [
  [               0,               0,               0,               0,     -0.18311105,      0.20387617,     0.075404993,     -0.10979512,    0.0080604707,   0.00021140503 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.003698355,    0.0053439776 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.059604457,       2.6914979 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -15.443709,       21.594579 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -25.484633,       35.891383 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.16190365,     -0.30584543 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.14879972,      0.22921782 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -47.549828,       75.949218 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       15.373294,      -24.268181 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.0097866,      -5.8594654 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00046572794,    0.0015420894 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.39174259,       1.5097548 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.2818754,     -0.69158425 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.39020654,     -0.43855747 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.085515465,      0.20921315 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00043466809,  -0.00012222138,   0.00090817672,    0.0039840204 ],
  [               0,               0,               0,               0,               0,               0,      0.07972658,      0.15485678,     -0.15766102,       1.9971776 ],
  [               0,               0,               0,               0,               0,               0,     -0.30406413,      0.33235324,      0.74437174,      -1.8801012 ],
  [               0,               0,               0,               0,               0,               0,     0.034832638,     -0.33840244,      -0.4157517,       0.5527961 ],
  [               0,               0,               0,               0,               0,               0,     -0.09074434,      0.19194253,      0.39964992,     -0.75172931 ],
  [               0,               0,               0,               0,               0,               0,   0.00027572732,    0.0001037194,  -0.00051223653,   -0.0011861396 ],
  [               0,               0,               0,               0,               0,               0,     -0.33489698,    -0.027062182,      0.34739966,        2.519198 ],
  [               0,               0,               0,               0,               0,               0,     -0.69473207,      0.41523397,       1.4850557,     -0.97205744 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0019860632,    -0.004584759,    -0.010402109,     0.026622761 ],
  [               0,               0,               0,               0,               0,               0,      0.40505635,     -0.64745975,      -2.7459442,       7.7722508 ],
  [               0,               0,               0,               0,               0,               0,      -0.7854627,       1.7202868,       4.1938103,      -10.309346 ],
  [               0,               0,               0,               0,               0,               0,      -2.7281459,        6.183137,       11.867981,        -28.0548 ],
  [               0,               0,               0,               0,               0,               0,     -0.56743733,       1.1432051,       3.0201359,      -6.0696003 ],
  [               0,               0,               0,               0,               0,               0,    -0.001443963,    0.0034095214,    0.0065247521,    -0.015404455 ],
  [               0,               0,               0,               0,               0,               0,      -13.782641,        29.07619,        68.97932,      -143.84498 ],
  [               0,               0,               0,               0,               0,               0,        -20.8144,       44.938096,       109.13293,      -236.79804 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    -0.011009118,   0.00061912329,     0.061640193 ],
  [               0,               0,               0,               0,               0,               0,               0,      -25.067513,     0.098840377,       133.86829 ],
  [               0,               0,               0,               0,               0,               0,               0,       27.123628,     -0.23223297,      -144.59446 ],
  [               0,               0,               0,               0,               0,               0,               0,      -9.7663384,      0.14258538,       50.799553 ],
  [               0,               0,               0,               0,               0,               0,               0,       -1.151751,      0.13640784,        6.403547 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00040365026,   3.3410832e-05,  -0.00098486191 ],
  [               0,               0,               0,               0,               0,               0,               0,       14.537667,    0.0037428962,      -77.471717 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.6854539,      0.36424569,       10.703428 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00052399805,    0.0020046542 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.14277854,       3.1776972 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.24499948,     -0.70818419 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.70390101,      0.49367407 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.38752538,      0.97146899 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.1044445,       2.6540642 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.3678236,       4.3838719 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,          -17.44,       40.887081 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       24.036341,      -58.695045 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.1866057,     -0.61222695 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0041531068,    0.0024766888,    -0.021947143 ],
  [               0,               0,               0,               0,               0,               0,               0,      -9.4805427,       1.3861132,       59.249359 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.29161979,     0.045774112,     0.063318789 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.56517741,     0.063538213,       -3.421045 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0083443261,    0.0045734705,    -0.028161831 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.7099073,     -0.44501623,      -4.0444949 ],
  [               0,               0,               0,               0,               0,               0,               0,       -3.543841,      -1.6347096,      -1.5888483 ],
  [               0,               0,               0,               0,               0,               0,               0,      -9.7231072,       1.4718688,        57.07634 ],
  [               0,               0,               0,               0,               0,               0,               0,       -32.22791,     0.014899074,       199.90384 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer2_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,     -0.13221126,      0.12450673,    -0.029187338,    0.0079382002 ],
  [               0,               0,               0,               0,               0,               0,       72.361662,       -65.39394,       14.958257,       4.5449679 ],
  [               0,               0,               0,               0,               0,               0,       -28.24946,       21.975542,      -4.7571601,       -1.132714 ],
  [               0,               0,               0,               0,               0,               0,        9.311525,      -8.0063368,       1.8337792,      -0.1111621 ],
  [               0,               0,               0,               0,               0,               0,     0.085590383,    -0.065652717,     0.013773054,  -3.4708529e-05 ],
  [               0,               0,               0,               0,               0,               0,      -27.219143,       7.2595187,     0.088199476,      0.83468259 ],
  [               0,               0,               0,               0,               0,               0,      -37.030862,       45.688148,      -12.197289,     -0.82540683 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    -0.001559779,    0.0021842797,    0.0070044654 ],
  [               0,               0,               0,               0,               0,               0,               0,      -4.4811253,       4.2044215,       3.7135295 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.72983891,      0.82719036,      -1.5336395 ],
  [               0,               0,               0,               0,               0,               0,               0,        1.579355,      -1.6226793,      -0.9073733 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.37256053,     -0.34948659,     0.072701207 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.011909347,     0.013478379,   -0.0037268321 ],
  [               0,               0,               0,               0,               0,               0,               0,       9.9913193,      -8.3856213,       1.2614242 ],
  [               0,               0,               0,               0,               0,               0,               0,      -424.89803,       505.93478,       -153.2826 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     -0.02823161,    0.0028406652,     0.024207974 ],
  [               0,               0,               0,               0,               0,               0,               0,        11.72523,     -0.39209169,      -3.0837856 ],
  [               0,               0,               0,               0,               0,               0,               0,       7.2970556,     0.030929013,      -5.9756959 ],
  [               0,               0,               0,               0,               0,               0,               0,      -6.4662471,      0.17300663,       3.9784929 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0015373934,  -0.00015721452,    0.0016906337 ],
  [               0,               0,               0,               0,               0,               0,               0,       23.571332,      -3.9434619,      -11.649095 ],
  [               0,               0,               0,               0,               0,               0,               0,       32.038934,     -0.13476387,      -22.581392 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00028693864,   0.00031751592,    0.0020374656,     0.005151756 ],
  [               0,               0,               0,               0,               0,               0,      0.98918434,      -1.7877405,     -0.85974016,       5.8999599 ],
  [               0,               0,               0,               0,               0,               0,      0.13223306,     -0.10922888,     -0.46691662,     -0.94637848 ],
  [               0,               0,               0,               0,               0,               0,     -0.17212083,      0.35277305,      0.13888783,      -0.2814454 ],
  [               0,               0,               0,               0,               0,               0,   0.00030177396,  -0.00032610658,  -0.00046635015,    0.0010450215 ],
  [               0,               0,               0,               0,               0,               0,     -0.46984624,      0.42641695,      0.34541081,      0.49409519 ],
  [               0,               0,               0,               0,               0,               0,      0.15478467,     -0.47452313,     -0.91517433,     -0.49970017 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0026166165,    0.0043709024 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.37332979,       4.4231329 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.49244143,      -2.2142085 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.3915359,     -0.16820828 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00032650196,    0.0015845585 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.92226729,       1.5924978 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -1.112034,      -1.3544998 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.069032694,    -0.029425722,    -0.032506926,      0.02793089,     -0.01190612,   0.00023860324 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0048108365,    0.0042001808,     0.002167653,   -0.0023985742,    -0.010803926 ],
  [               0,               0,               0,               0,               0,        1.599171,       2.3656086,       3.4915473,       5.1429321,        11.56071 ],
  [               0,               0,               0,               0,               0,      -2.1688213,     -0.99145463,      0.67909326,       2.6851026,       4.0969121 ],
  [               0,               0,               0,               0,               0,     0.029471157,     0.035128795,     0.053755164,     0.092856362,      -0.1403968 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer2_etaoffv5_35_e = [
 [
  [               0,               0,               0,      -2.1806195,       6.6928581,      -7.6717476,       3.9924265,     -0.88175126,     0.057765645,   -0.0065701983 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,     -0.70491366,     -0.33648282,       0.5411038,       2.3283714 ],
  [               0,               0,               0,               0,               0,               0,       4.5541699,       2.0042622,      -4.0483788,      -12.393537 ],
  [               0,               0,               0,               0,               0,               0,      -1.9185398,     -0.97154118,        1.340722,       5.7068779 ],
  [               0,               0,               0,               0,               0,               0,     -0.31357458,     -0.14807938,      0.28769181,      0.93588682 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.083448577,     -0.11868746 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       20.936861,      -28.059496 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.9677321,      -4.3362297 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.078669076,      0.25189703 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0055548383,   -0.0056851814,    -0.019434967,     0.029217074 ],
  [               0,               0,               0,               0,               0,               0,      -1.3403724,       2.3824564,     -0.38566022,       4.6387229 ],
  [               0,               0,               0,               0,               0,               0,      0.25765147,     -0.38426395,     -0.97046093,      0.10908565 ],
  [               0,               0,               0,               0,               0,               0,     0.024586302,    -0.037432768,     0.040574045,     0.020860378 ],
  [               0,               0,               0,               0,               0,               0,    0.0019242988,   -0.0027575706,   -0.0086819712,     0.013947414 ],
  [               0,               0,               0,               0,               0,               0,     0.044643072,       1.1257403,      -5.4135075,       6.4684926 ],
  [               0,               0,               0,               0,               0,               0,      -3.2476677,       4.8773462,       8.5033239,      -14.406641 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    -0.011006809,     0.048972128,    -0.044834921 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.4719993,      -21.142283,       27.721741 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.34824494,      -1.1508026,     -0.40326241 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.3284859,       3.0265075,      0.08668079 ],
  [               0,               0,               0,               0,               0,               0,               0,  -0.00081042599,    -0.015080835,     0.040528693 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.38687495,       1.6962503,      -1.7596448 ],
  [               0,               0,               0,               0,               0,               0,               0,       14.796635,      -59.530952,       56.980074 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -0.00011593106,  -0.00019621227,   3.3584462e-05,      0.00201499,     0.011667564 ],
  [               0,               0,               0,               0,               0,       1.2444078,      -1.4665886,      -7.0759576,      -8.6238773,       43.092685 ],
  [               0,               0,               0,               0,               0,     -0.10592468,     0.068557588,       0.6276873,       1.0360492,      -4.7695037 ],
  [               0,               0,               0,               0,               0,      -0.3360113,      0.56457513,       2.3926552,       2.5708982,      -15.829513 ],
  [               0,               0,               0,               0,               0,   0.00029344092,   0.00030593503,   1.5079894e-05,   -0.0011172999,   -0.0030757358 ],
  [               0,               0,               0,               0,               0,      0.34517788,     -0.43154998,       -1.783152,       -2.039078,       9.7429884 ],
  [               0,               0,               0,               0,               0,     -0.12098312,     -0.11244174,     0.040782644,      0.52197562,       1.4003195 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00011816855,     0.010400817 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.0905869,    -0.015232598 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.9958763,      -16.033797 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.12860543,     -0.22810348 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,      0.54724065,      0.11431454,     -0.12144987,     0.014068528,   -0.0010719038 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer1_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,   -0.0024270767,     0.018826081,    -0.024007899,    0.0071410249 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00046393903,   0.00072106131 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.3723672,       4.2121679 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.2674176,     -0.96827531 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.73624933,     -0.75115705 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.2829794,   -0.0041050148 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.027097435,    0.0093152465 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -15.41207,       12.778953 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -6.0505436,       4.3276371 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       16.128151,      -12.829652 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0015046786,   -0.0064444902,    0.0070898882 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.5288699,       4.9377785,      -1.1375239 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.2479929,      -5.9054061,       1.6584994 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.1429881,      -4.6225443,       1.1445654 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.79972459,       1.4048287,     -0.72464578 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00095029774,   -0.0021966278,    0.0012455053 ],
  [               0,               0,               0,               0,               0,               0,               0,        19.41148,      -43.988089,       23.321043 ],
  [               0,               0,               0,               0,               0,               0,               0,      -68.235803,       168.67993,      -89.008728 ],
 ],
 [
  [               0,               0,     -0.10606865,      0.13749086,     0.085819952,    -0.090652634,      -0.1245052,       0.1376279,     -0.05039683,    0.0030512114 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.017227902,     -0.17627995,      0.37435733,     -0.32818513,      0.12614056,    -0.014279072 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.72158615,       1.0531021 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       50.385246,      -69.168351 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -15.669245,       21.694254 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -94.499752,       133.22375 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.075767433,     -0.18478769 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer1_etaoffv5_37_e = [
 [
  [               0,               0,               0,               0,     -0.21141693,      0.25331424,     0.073722253,      -0.1415866,     0.021494614,  -0.00065269801 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0038308752,    0.0052927176 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.5679595,       1.7532913 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -17.708766,       24.927731 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -22.695556,       31.845068 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.16195078,     -0.30529834 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.14543535,      0.22417618 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -46.314347,       74.088913 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       16.429451,      -25.861437 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        3.811592,      -5.5512521 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00044908663,    0.0015501956 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.34622753,       1.5934167 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.25594791,     -0.73374784 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.36936878,     -0.47148077 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.086986757,      0.21229509 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0003493775,  -6.9932127e-05,   0.00078343509,    0.0033079401 ],
  [               0,               0,               0,               0,               0,               0,     0.026520919,      0.13905463,    -0.067048116,       2.3029675 ],
  [               0,               0,               0,               0,               0,               0,     -0.34891386,      0.29985836,      0.79422012,       -1.578377 ],
  [               0,               0,               0,               0,               0,               0,   -0.0070145159,     -0.28462338,     -0.29316389,      0.38400864 ],
  [               0,               0,               0,               0,               0,               0,    -0.068942974,      0.17856152,      0.34771127,     -0.75068495 ],
  [               0,               0,               0,               0,               0,               0,   0.00028536651,   0.00013003616,  -0.00050034592,   -0.0013124344 ],
  [               0,               0,               0,               0,               0,               0,     -0.26031038,    -0.069784466,       0.1578918,       2.5170777 ],
  [               0,               0,               0,               0,               0,               0,     -0.59735042,      0.33198456,       1.2247896,     -0.89589284 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0092353438,    -0.019142626,    -0.043061197,     0.092421665 ],
  [               0,               0,               0,               0,               0,               0,      -3.0912128,       6.2242058,       14.390634,      -26.200777 ],
  [               0,               0,               0,               0,               0,               0,       4.0576943,      -8.3270207,      -15.716975,       30.065589 ],
  [               0,               0,               0,               0,               0,               0,       1.6693584,      -2.4094678,      -8.3239179,       10.752948 ],
  [               0,               0,               0,               0,               0,               0,      -3.7552973,       6.5546656,       17.878867,      -30.572198 ],
  [               0,               0,               0,               0,               0,               0,    -0.002615834,   -0.0057948536,   0.00020954891,     0.058734176 ],
  [               0,               0,               0,               0,               0,               0,       0.2701598,     -0.47612062,       -1.323468,       2.4680391 ],
  [               0,               0,               0,               0,               0,               0,      -1.2927863,       2.2722459,       6.9295376,      -14.514037 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    -0.045692478,   0.00044716872,      0.24410031 ],
  [               0,               0,               0,               0,               0,               0,               0,       6.5046304,     0.034101491,      -32.660811 ],
  [               0,               0,               0,               0,               0,               0,               0,        36.07159,     -0.69237441,      -194.26949 ],
  [               0,               0,               0,               0,               0,               0,               0,       6.8824147,     -0.44549204,      -39.455704 ],
  [               0,               0,               0,               0,               0,               0,               0,      -3.2358173,      0.13142018,       17.418062 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0027871689,    0.0024690523,   -0.0023848773 ],
  [               0,               0,               0,               0,               0,               0,               0,       5.3723552,     0.065940099,      -28.797017 ],
  [               0,               0,               0,               0,               0,               0,               0,       -1.682004,      0.22231621,       9.9699798 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00065271974,    0.0022246898 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.1566221,        2.235269 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.43475124,     -0.26140393 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.4043091,     -0.22772453 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.25054207,      0.64052071 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0010116954,    0.0032311841 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.5616596,       5.0965112 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.25608799,     -0.33010074 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.0931691,      -3.9868042 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.015245703,     -0.26329357 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0040765035,    0.0024608808,    -0.021398217 ],
  [               0,               0,               0,               0,               0,               0,               0,      -9.1231311,       1.3189593,       57.176187 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.30730165,     0.035265086,       0.1839632 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.57619292,     0.065730228,      -3.4939219 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0019514105,    0.0026451838,    0.0073676456 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.014258906,       0.1172097,       6.1158858 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.047062173,     0.058121075,      -11.308513 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.089739183,     -0.16467564,      0.22244538 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.078217913,    -0.089460547,      0.46852041 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer2_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,     -0.12596214,      0.12019507,    -0.028406685,    0.0079319043 ],
  [               0,               0,               0,               0,               0,               0,       71.167044,      -64.342586,       14.728783,        4.544085 ],
  [               0,               0,               0,               0,               0,               0,      -27.835001,       21.626532,      -4.6825404,      -1.1328526 ],
  [               0,               0,               0,               0,               0,               0,       8.9214643,       -7.750811,       1.7863771,     -0.11060995 ],
  [               0,               0,               0,               0,               0,               0,     0.083732386,    -0.063961671,     0.013414701,  -1.8000029e-05 ],
  [               0,               0,               0,               0,               0,               0,      -32.450057,       10.478906,     -0.46862332,      0.85350571 ],
  [               0,               0,               0,               0,               0,               0,      -37.165185,        45.61244,      -12.168687,     -0.81567048 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0050391737,   -0.0051169876,    0.0084886884 ],
  [               0,               0,               0,               0,               0,               0,               0,      -9.4834488,       9.7174347,       2.6111603 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.91239598,      0.97387126,      -1.5505218 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.2712253,      -1.3140973,     -0.97387956 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.82706936,     -0.86757919,      0.21391381 ],
  [               0,               0,               0,               0,               0,               0,               0,  -0.00037208412,   0.00043253814,  -4.8728004e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,       -3.323666,       2.7868552,       2.4570593 ],
  [               0,               0,               0,               0,               0,               0,               0,      -16.185238,       18.166158,      -9.6500643 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    -0.036717489,    0.0027778957,     0.029682733 ],
  [               0,               0,               0,               0,               0,               0,               0,       14.362477,     -0.44627548,      -4.7196381 ],
  [               0,               0,               0,               0,               0,               0,               0,        7.521007,     0.044404833,      -6.1164279 ],
  [               0,               0,               0,               0,               0,               0,               0,       -6.833996,      0.17161193,       4.2088929 ],
  [               0,               0,               0,               0,               0,               0,               0,  -0.00023732061,  -6.0305543e-05,   0.00085993666 ],
  [               0,               0,               0,               0,               0,               0,               0,         21.2102,      -2.3014589,      -11.480125 ],
  [               0,               0,               0,               0,               0,               0,               0,       33.202296,       0.2729954,      -23.609775 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0016138398,   3.7639291e-05,     0.005584428,    0.0037593072 ],
  [               0,               0,               0,               0,               0,               0,       1.4811369,      -2.6976975,      -0.1058064,       5.4134531 ],
  [               0,               0,               0,               0,               0,               0,     0.057850227,     -0.13177061,     -0.22841374,       -1.072713 ],
  [               0,               0,               0,               0,               0,               0,     -0.19622538,      0.35661219,       0.2451292,     -0.36657411 ],
  [               0,               0,               0,               0,               0,               0,   0.00051951812,  -0.00071670445,   6.7061093e-05,   0.00058780223 ],
  [               0,               0,               0,               0,               0,               0,      0.59790753,      -0.9791994,      0.13118466,      0.87016409 ],
  [               0,               0,               0,               0,               0,               0,       2.3666681,      -2.9037091,      -3.6968229,        2.407339 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0026721316,    0.0043734514 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.29719345,       4.2845694 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.73896588,      -2.5526017 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.43397644,     -0.21164196 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00038218043,    0.0016065507 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.84409417,       1.4771658 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -1.100659,      -1.3832679 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.11791927,      0.42625071,     -0.43663005,      0.18611416,    -0.037455496,    0.0014310374 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0044525516,    0.0039903213,    0.0023100891,   -0.0015481865,   -0.0086437957 ],
  [               0,               0,               0,               0,               0,       1.4931372,       2.2880449,       3.4858347,       5.2843693,       11.972478 ],
  [               0,               0,               0,               0,               0,      -1.7987295,     -0.98392434,      0.31594411,        2.151808,       4.0432985 ],
  [               0,               0,               0,               0,               0,     0.034072009,     0.041634292,     0.060573687,     0.099016432,     -0.13209355 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam200_layer2_etaoffv5_37_e = [
 [
  [               0,               0,               0,      -1.6342888,       5.0185414,      -5.7328667,       2.9504148,     -0.62736206,     0.035512437,   -0.0068233157 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,      -17.936197,      -8.5831772,       13.716264,       59.143964 ],
  [               0,               0,               0,               0,               0,               0,       8.2144728,       -8.203577,       -15.53184,       14.509689 ],
  [               0,               0,               0,               0,               0,               0,         1.37255,      -5.7442695,      -5.6933427,       16.378046 ],
  [               0,               0,               0,               0,               0,               0,       1.2854135,      -1.8988088,      -2.7922663,       4.1244591 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.08599967,     -0.12252706 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       22.131016,      -29.859643 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.5063643,      -5.1563646 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.022669905,      0.16669143 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00011928223,  -0.00013269412,    0.0010228228,    0.0064705695 ],
  [               0,               0,               0,               0,               0,               0,       0.9849977,      -1.8552216,      -3.2387988,       10.196583 ],
  [               0,               0,               0,               0,               0,               0,    -0.034726524,     0.073986395,   0.00019232215,      -1.4247179 ],
  [               0,               0,               0,               0,               0,               0,    -0.032336842,     0.042573815,      0.17842063,     -0.16620843 ],
  [               0,               0,               0,               0,               0,               0,   0.00016531372,  -0.00011712638,    -0.000404407,   0.00089996574 ],
  [               0,               0,               0,               0,               0,               0,      0.78787498,      -1.3770371,       -2.002522,       4.2747242 ],
  [               0,               0,               0,               0,               0,               0,      0.11383881,     -0.11887177,     -0.53250398,       -1.134856 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,      0.02131511,    -0.073912349,     0.068511239 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.039113593,      -5.9533055,       15.852078 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.75652936,      -2.2942682,      0.11952498 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.49241067,       1.6664673,       -1.211229 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0028341069,      -0.0149856,     0.019833628 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.5005303,      -11.422853,         13.6969 ],
  [               0,               0,               0,               0,               0,               0,               0,       6.0514706,      -24.977398,       23.923117 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -0.00041055369,  -0.00056374661,   7.7760411e-07,    0.0040308819,     0.021605522 ],
  [               0,               0,               0,               0,               0,       1.9575238,      -2.1806832,      -10.840622,      -13.527925,       62.815342 ],
  [               0,               0,               0,               0,               0,    -0.071608803,    -0.023934661,      0.54569076,       1.3676908,      -4.9654403 ],
  [               0,               0,               0,               0,               0,      -0.3809879,      0.48240454,       2.4472995,       3.1533286,      -15.138811 ],
  [               0,               0,               0,               0,               0,   0.00034976807,   0.00039086109,   2.3230019e-06,   -0.0018628764,    -0.006963988 ],
  [               0,               0,               0,               0,               0,      0.09442273,     -0.15605866,      -0.6105853,     -0.74396774,       4.4302543 ],
  [               0,               0,               0,               0,               0,      0.73041915,     -0.71394669,      -3.6201785,      -4.8175458,       15.372124 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   4.2624769e-05,      0.01047762 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.1915925,      -0.2317316 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       6.7550127,      -17.911663 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.22885517,      -0.4720761 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,      0.15246492,      0.49556767,     -0.24188956,     0.029117385,   -0.0016552705 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]

###############################################################################
# Tables for v5 sshape correction, 500 GeV photon clusters
#


gam500_layer1_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,    -0.024671908,   -0.0027062792,      0.10739568,     -0.11178967,     0.031013653 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00044310091,   0.00066993864 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -1.566671,       4.7497915 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.1416989,     -0.88124731 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.9786085,     -0.65923136 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.27841971,      -0.0242201 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,        1.884966,     0.013232142,      -1.1796661 ],
  [               0,               0,               0,               0,               0,               0,               0,      -27.700716,      -1.3651305,       19.196888 ],
  [               0,               0,               0,               0,               0,               0,               0,       9.0925069,    -0.048073458,      -6.2419916 ],
  [               0,               0,               0,               0,               0,               0,               0,       10.712383,      0.13199741,      -6.8870396 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.015799858,    -0.034216762,     0.020498708 ],
  [               0,               0,               0,               0,               0,               0,               0,      -5.0714564,       10.951482,      -3.6255609 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.1835119,       2.7677941,      -2.5371817 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.67140005,       1.2688929,      -1.8831419 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.28250804,     -0.63576898,      0.22296826 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00033565821,  -0.00073539541,   0.00039764063 ],
  [               0,               0,               0,               0,               0,               0,               0,       13.295396,      -4.3142691,     -0.38021794 ],
  [               0,               0,               0,               0,               0,               0,               0,       -8.800078,       7.4440436,      -1.4736992 ],
 ],
 [
  [               0,               0,               0,    -0.088772224,      0.19865105,    -0.012321837,     -0.22930112,      0.17684294,    -0.054909692,    0.0028981899 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.10064501,      0.20310659,    -0.082460376,    -0.072918337,     0.059477202,    -0.007393442 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    -0.063913918,    -0.029664339,     0.045878265,      0.19239307 ],
  [               0,               0,               0,               0,               0,               0,       41.279269,       19.840646,      -28.328301,      -118.53885 ],
  [               0,               0,               0,               0,               0,               0,      -28.068787,      -13.647926,       19.560752,         85.1991 ],
  [               0,               0,               0,               0,               0,               0,      -18.327731,      -8.8453569,       12.225553,       52.114153 ],
  [               0,               0,               0,               0,               0,               0,    -0.033665151,    -0.037025609,    -0.070806737,     -0.47618463 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer1_etaoffv5_55_e = [
 [
  [               0,               0,               0,               0,     0.055861177,     -0.06027451,    -0.068006871,     0.077053318,    -0.012074657,     0.003145484 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0035936749,    0.0078692981 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.9694614,       8.1533626 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -22.359727,        31.78044 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -28.376562,       40.131957 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.16437421,     -0.28609409 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.12581171,      0.19491831 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -25.716371,       44.232146 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       14.636389,      -23.220625 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.6496544,      -6.8101953 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00057583586,    0.0010605343 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.53970127,       1.5241203 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.35708352,     -0.60768866 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.56698489,     -0.15867102 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.11495197,      0.26905449 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -6.5949717e-05,    0.0001369466,   0.00043001275,    0.0010006225 ],
  [               0,               0,               0,               0,               0,               0,      -0.3429452,   -0.0032742522,      0.52748479,       4.6736251 ],
  [               0,               0,               0,               0,               0,               0,     -0.35761727,      0.35098328,      0.84722696,      -1.8271746 ],
  [               0,               0,               0,               0,               0,               0,    -0.039730648,     -0.23542608,     -0.18030819,      0.22454104 ],
  [               0,               0,               0,               0,               0,               0,    -0.083363166,      0.17349375,      0.36819769,     -0.66461451 ],
  [               0,               0,               0,               0,               0,               0,    0.0002508957,   0.00013272532,  -0.00044029181,   -0.0012212797 ],
  [               0,               0,               0,               0,               0,               0,     -0.26880817,    -0.093685727,      0.15882018,       2.7387176 ],
  [               0,               0,               0,               0,               0,               0,     -0.88371271,      0.67515617,       2.1075841,      -1.7537834 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00045105021,   0.00037319641,    0.0018532793,    0.0020336961 ],
  [               0,               0,               0,               0,               0,               0,      0.18504068,     -0.19882651,      -0.7767067,       3.6364223 ],
  [               0,               0,               0,               0,               0,               0,     -0.20094345,      0.47049653,      0.92197361,      -3.4173156 ],
  [               0,               0,               0,               0,               0,               0,      0.36048683,     -0.68723071,      -1.6417202,       1.9705338 ],
  [               0,               0,               0,               0,               0,               0,       -0.196025,      0.42464538,      0.98564018,      -2.0652047 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -0.00017686928,  -0.00015077482,   0.00023948917,    0.0018932874,    0.0079284767 ],
  [               0,               0,               0,               0,               0,      -2.7943594,      -3.0876232,     0.026480136,       16.402391,       79.183859 ],
  [               0,               0,               0,               0,               0,       1.8487292,       2.1104997,    -0.063939508,      -11.441186,      -52.650695 ],
  [               0,               0,               0,               0,               0,      0.36567708,      0.43018727,     0.020366865,      -2.2372407,      -11.525844 ],
  [               0,               0,               0,               0,               0,     -0.17205759,     -0.18788676,     0.045654622,       1.2033448,       4.9924986 ],
  [               0,               0,               0,               0,               0,   2.1446767e-05,   7.2119143e-05,   0.00021840968,   0.00062343561,    0.0027130103 ],
  [               0,               0,               0,               0,               0,      0.10778996,      0.17992955,    -0.010812012,      -1.0046987,      -2.5716276 ],
  [               0,               0,               0,               0,               0,     0.087652268,     0.082408385,    -0.084180078,      -0.8221853,       -3.334461 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00015382522,    0.0020942265 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.9948623,       8.2620353 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.31797557,      -2.0207982 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.7385743,       2.9930916 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.90969137,       2.2243222 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.69984732,       1.6814778 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.3389674,       4.3662464 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -8.052845,       18.304734 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       74.318371,      -179.13159 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.12097089,     -0.52109724 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0064202879,    0.0021683053,    -0.034567545 ],
  [               0,               0,               0,               0,               0,               0,               0,      -12.332055,       1.0642432,       77.256255 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.24606495,       -0.113839,      0.16929653 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.54075258,     0.029661184,      -3.1852283 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0048728948,     0.016892468 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.2219738,       8.4103864 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.1703425,      -16.228219 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0099899429,     -0.58890018 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0092047911,      0.86110664 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer2_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0051989916,    0.0062754494 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.9234455,       4.7056698 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.028486155,      -1.2878299 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.005135097,    -0.048383261 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0011059567,   0.00052126876 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.6652595,       1.1777353 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.6042168,       -1.727717 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,     0.011817677,    -0.006052961,   -0.0062347308,    0.0097842664 ],
  [               0,               0,               0,               0,               0,               0,      -7.1840846,       5.0740441,       1.0407856,        3.576309 ],
  [               0,               0,               0,               0,               0,               0,     -0.62941778,     -0.72028202,       1.1248359,      -1.6016017 ],
  [               0,               0,               0,               0,               0,               0,       1.9295047,       -1.817538,      0.36049097,   -0.0061710915 ],
  [               0,               0,               0,               0,               0,               0,   -0.0069529042,    0.0063720734,   0.00035888287,   -0.0003770494 ],
  [               0,               0,               0,               0,               0,               0,      -6.2914532,       8.4686785,      -3.3510284,       1.3274827 ],
  [               0,               0,               0,               0,               0,               0,      -3.6134928,       1.9703191,    -0.056451762,      -1.4058887 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0062720129,     0.003266082,    0.0017161852 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.5523785,     -0.22417652,       0.8429033 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.4725689,      0.20886379,      -4.2046586 ],
  [               0,               0,               0,               0,               0,               0,               0,      -4.1288684,        0.164681,       2.4011468 ],
  [               0,               0,               0,               0,               0,               0,               0,  -0.00027963265,  -0.00031120241,   0.00070480332 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.37302939,    -0.016008755,       1.3614376 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.7666826,      0.32987067,      0.57677129 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00064142745,   -0.0015613058,   -0.0011318586,    0.0087636416 ],
  [               0,               0,               0,               0,               0,               0,      0.92202776,     -0.93820019,      -1.2783961,       5.0731572 ],
  [               0,               0,               0,               0,               0,               0,      0.58995173,      -1.7577526,        1.350225,      -1.4810262 ],
  [               0,               0,               0,               0,               0,               0,      0.14879289,     -0.78775152,       1.4902429,     -0.85064174 ],
  [               0,               0,               0,               0,               0,               0,  -0.00077563411,    0.0010649314,    0.0011562969,  -0.00082967217 ],
  [               0,               0,               0,               0,               0,               0,     -0.98523324,       2.2134369,      -1.2011678,      0.93189263 ],
  [               0,               0,               0,               0,               0,               0,       1.6624778,      -3.6268412,       1.2225105,     -0.68823345 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0043526083,     0.015232743 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        -150.302,       208.01037 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       16.709775,      -24.143433 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.3431864,      -2.9518369 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,    -0.023729774,     0.013591584,      0.02182289,     0.007275208,    -0.014342853,    -0.015726331,     0.024283677,    -0.014848741,   0.00047521282 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,     0.013958204,    0.0021936063,    -0.011630532,    0.0087470922 ],
  [               0,               0,               0,               0,               0,               0,    -0.093426925,     0.018847345,    0.0075127656,       40.208896 ],
  [               0,               0,               0,               0,               0,               0,    -0.014259499,    -0.023270405,    -0.016846525,      -2.1449688 ],
  [               0,               0,               0,               0,               0,               0,   -0.0086784966,     0.043168134,    -0.047491486,      0.63899919 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer2_etaoffv5_55_e = [
 [
  [               0,               0,               0,      -1.0146859,        3.044352,       -3.366326,       1.7269343,     -0.45826559,     0.077370303,   -0.0074821495 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00097216441,    0.0012487423,    0.0015640397,    0.0023903114 ],
  [               0,               0,               0,               0,               0,               0,          4.6698,       2.2040477,      -3.8225448,      -12.330153 ],
  [               0,               0,               0,               0,               0,               0,      -0.4382749,     -0.19559592,      0.22344706,       0.4571476 ],
  [               0,               0,               0,               0,               0,               0,     -0.12087937,    -0.030518493,      0.16216311,      0.22873389 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0030038328,  -0.00067372288 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.77377816,       3.2202587 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.54814227,     -0.43063578 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.12801848,     -0.01044973 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0013975496,  -0.00083045678 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -2.284143,       2.4454145 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.4494499,      0.78061172 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -4.0133641e-05,   0.00020874013,    0.0014680345,    0.0060673548 ],
  [               0,               0,               0,               0,               0,               0,       1.3497536,       -2.567862,      -4.6046577,       11.859086 ],
  [               0,               0,               0,               0,               0,               0,       0.0316051,     0.020006613,     -0.31444407,      -1.0120333 ],
  [               0,               0,               0,               0,               0,               0,   -0.0032672304,    -0.050639434,    -0.045145315,       0.3886954 ],
  [               0,               0,               0,               0,               0,               0,   0.00023525267,   0.00011002864,  -0.00050564137,    -0.001365385 ],
  [               0,               0,               0,               0,               0,               0,      0.10805577,      0.18689498,     -0.41017463,     0.022192842 ],
  [               0,               0,               0,               0,               0,               0,      0.45290848,      -0.3014352,      -1.7926044,       -4.214241 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -2.4941721e-05,  -1.2557681e-05,   0.00044017188,    0.0019565298,    0.0048741219 ],
  [               0,               0,               0,               0,               0,      0.10697241,     -0.11794703,     -0.73472635,      -1.1448654,       7.1718605 ],
  [               0,               0,               0,               0,               0,      0.00355393,     0.081947614,    -0.070387751,     -0.59703648,     -0.49427987 ],
  [               0,               0,               0,               0,               0,    -0.069387516,      0.04832773,      0.13618587,      0.12539237,      0.37642279 ],
  [               0,               0,               0,               0,               0,  -0.00017570749,  -0.00031024283,  -0.00021421742,    0.0010995755,    0.0078316972 ],
  [               0,               0,               0,               0,               0,     0.019841215,      0.11949873,     -0.29231402,      -1.1431388,       2.0696148 ],
  [               0,               0,               0,               0,               0,     -0.85426951,      0.78133873,       3.5489618,       4.1716595,      -12.138842 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   1.0038371e-05,   5.4010707e-05,   0.00020881855,   0.00070801786,    0.0027334415 ],
  [               0,               0,               0,               0,               0,     0.011853903,    -0.062775762,     -0.20791309,     -0.25607347,       4.8565336 ],
  [               0,               0,               0,               0,               0,   -0.0028467089,      0.01002479,     0.019922138,    -0.079664762,       -1.201715 ],
  [               0,               0,               0,               0,               0,     -0.00111161,    0.0078080701,       0.0300025,     0.059857185,     -0.29250036 ],
  [               0,               0,               0,               0,               0,  -2.6664077e-05,  -3.4271708e-05,  -5.5650349e-06,   0.00018285806,     0.001937104 ],
  [               0,               0,               0,               0,               0,    -0.015012706,     0.031912091,    0.0074882975,     -0.14455216,       1.2724703 ],
  [               0,               0,               0,               0,               0,     0.012655301,    0.0043105647,    -0.053923045,     -0.28812652,      -1.0839564 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0063085696,  -0.00084500989 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.7025612,       11.055184 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.58816097,      -2.7196532 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.7173633,      -1.5571354 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,     -0.67447856,      0.98312261,     -0.26355227,     0.017029017,   0.00043956048 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer1_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,    0.0043233704,    -0.040033513,     0.091822323,    -0.077046949,     0.020326802 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00048705511,   0.00089066214 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -1.342185,       4.0187755 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.36572868,      -1.1433151 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.57889019,     -0.93893668 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.22952094,    -0.014100548 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.9509436,      -2.3240845 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -62.558172,       50.370198 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       20.711465,      -17.011009 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       18.181548,      -14.467187 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0014387537,  -0.00017709549,    0.0041700928 ],
  [               0,               0,               0,               0,               0,               0,               0,       4.3093997,      -7.7161512,       5.4312244 ],
  [               0,               0,               0,               0,               0,               0,               0,       -1.684444,       4.1853695,      -3.4034657 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.6859124,       3.4154041,      -2.9696438 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.78642456,       1.8748011,      -1.3050822 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00021493907,    -0.001714919,    0.0018322369 ],
  [               0,               0,               0,               0,               0,               0,               0,       2.7596433,      -3.8301059,       1.2527944 ],
  [               0,               0,               0,               0,               0,               0,               0,       -9.023807,       21.194139,      -5.3029525 ],
 ],
 [
  [               0,               0,               0,     -0.15103085,      0.29612051,    -0.014037622,      -0.2939414,      0.21292278,    -0.061386991,    0.0032368319 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.19324413,     -0.63253183,      0.80991645,     -0.51708879,      0.16291205,    -0.016778712 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     -0.24590271,    0.0050722697,      0.51880923 ],
  [               0,               0,               0,               0,               0,               0,               0,       17.266715,     -0.10735748,      -32.165234 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.37157678,     -0.26174785,      0.11541834 ],
  [               0,               0,               0,               0,               0,               0,               0,      -37.656819,    -0.050048634,       75.107326 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.012543515,    0.0066015001,     -0.27686772 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer1_etaoffv5_35_e = [
 [
  [               0,               0,               0,               0,      0.57896655,      -1.7362659,       1.7876994,     -0.71505995,     0.080758871,   -0.0007603578 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.003668127,    0.0079649958 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.3971715,       7.2823295 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -22.631179,       32.181934 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -25.176218,       35.479005 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.16511399,     -0.28591026 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.1570345,       0.2414402 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -31.415033,       52.664946 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       14.283994,      -22.678976 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.4688598,      -6.5407454 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0005947122,    0.0010434681 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.54134653,       1.5119984 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.35300883,     -0.61475986 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -0.563472,     -0.16519967 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.11443107,      0.26764285 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -9.2327979e-05,   9.5754788e-05,    0.0004166834,    0.0011921849 ],
  [               0,               0,               0,               0,               0,               0,     -0.18799936,    -0.048914643,      0.21889235,        4.477379 ],
  [               0,               0,               0,               0,               0,               0,     -0.34301595,      0.17401451,      0.65899338,     -0.93716219 ],
  [               0,               0,               0,               0,               0,               0,    -0.044224988,     -0.17961537,     -0.12782458,    -0.075692358 ],
  [               0,               0,               0,               0,               0,               0,    -0.034437762,      0.13358484,      0.24243454,     -0.61776997 ],
  [               0,               0,               0,               0,               0,               0,   0.00022956018,   0.00011139622,  -0.00041605577,  -0.00099478387 ],
  [               0,               0,               0,               0,               0,               0,     -0.36646832,    -0.012487195,      0.42674619,       2.6760543 ],
  [               0,               0,               0,               0,               0,               0,      -1.0815946,       1.0422154,       2.5943249,      -2.6379402 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00025132347,  -6.5299417e-05,   0.00083930103,    0.0042514378 ],
  [               0,               0,               0,               0,               0,               0,      0.11720763,    -0.039876734,     -0.43995982,       2.8537251 ],
  [               0,               0,               0,               0,               0,               0,     -0.21317735,      0.49756683,      0.97328951,      -3.5307414 ],
  [               0,               0,               0,               0,               0,               0,      0.30688595,     -0.57434103,      -1.3833678,       1.4275452 ],
  [               0,               0,               0,               0,               0,               0,     -0.19283669,      0.41939163,       0.9664447,      -2.0314292 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -0.00019228075,  -0.00017572135,   0.00020659753,    0.0018747221,    0.0080188361 ],
  [               0,               0,               0,               0,               0,      -2.9768076,      -3.2959034,     0.031351559,       17.521985,       84.296812 ],
  [               0,               0,               0,               0,               0,       2.1769642,       2.4883345,    -0.062968381,      -13.437062,      -61.840474 ],
  [               0,               0,               0,               0,               0,      0.37944779,      0.44812472,     0.027810168,       -2.302993,      -11.879396 ],
  [               0,               0,               0,               0,               0,     -0.16878087,     -0.18447842,     0.044036413,       1.1779584,       4.8844861 ],
  [               0,               0,               0,               0,               0,   1.1291719e-05,   4.2904127e-05,   0.00013777864,   0.00040716797,    0.0021449207 ],
  [               0,               0,               0,               0,               0,     0.048791244,      0.11233718,    -0.009271975,     -0.63914619,     -0.89614766 ],
  [               0,               0,               0,               0,               0,     0.076614648,     0.069217428,     -0.08665187,     -0.76419711,      -3.0556139 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0001531162,    0.0020918118 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.9886495,       8.2524613 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.29294582,      -1.9618021 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.7255693,       2.9631373 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.90824054,       2.2210735 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   9.8505654e-05,   0.00077370861 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.0475541,       5.1753452 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.096015849,     -0.78594658 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.20456665,      -2.4963867 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.1467346,     -0.65492974 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0066618592,    0.0021534342,    -0.035992545 ],
  [               0,               0,               0,               0,               0,               0,               0,      -12.655559,       1.0657627,       79.206187 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.21257326,     -0.11517747,    -0.026116926 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.53727239,     0.029829284,       -3.165046 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0060195587,     0.011851175 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1026.1137,       2557.9616 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -78.480137,       184.20386 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -52.076075,       128.09996 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.21228308,      0.16201485 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer2_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0058387062,      0.00562757 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.9167583,       5.9078802 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.055145453,      -1.3295885 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.041830913,    -0.045678138 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0015913893,   0.00083956316 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.7812157,       1.1349988 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.3245662,      -1.7339166 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,     0.013819885,    -0.004449254,    -0.010264406,     0.011168461 ],
  [               0,               0,               0,               0,               0,               0,      -13.991364,       11.666481,      0.50856593,       3.8897785 ],
  [               0,               0,               0,               0,               0,               0,      -1.8729786,       1.2667097,      0.29174869,      -1.5657648 ],
  [               0,               0,               0,               0,               0,               0,       1.6513445,      -1.2366061,     -0.14510387,      0.13783553 ],
  [               0,               0,               0,               0,               0,               0,     -0.01005116,     0.010213834,  -0.00080745446,  -0.00033297102 ],
  [               0,               0,               0,               0,               0,               0,      -5.6303086,       4.4749307,        0.787584,      0.07145579 ],
  [               0,               0,               0,               0,               0,               0,      -3.8527909,      -1.0964971,       4.4885647,      -2.9068702 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    -0.050721083,    0.0026419049,     0.038518609 ],
  [               0,               0,               0,               0,               0,               0,               0,       16.279403,     -0.60710855,       -5.586131 ],
  [               0,               0,               0,               0,               0,               0,               0,       8.5429857,      0.46544908,      -7.0874155 ],
  [               0,               0,               0,               0,               0,               0,               0,      -6.7333212,      0.23993999,       4.0710108 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0048017823,   -0.0004986031,   -0.0020514436 ],
  [               0,               0,               0,               0,               0,               0,               0,       28.464944,      -3.6852186,      -14.965925 ],
  [               0,               0,               0,               0,               0,               0,               0,        25.58185,     -0.35331039,      -18.262647 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0015556933,    0.0011863102,    0.0036458062,    0.0046835229 ],
  [               0,               0,               0,               0,               0,               0,       1.1591331,      -1.5392381,      -1.5786904,        6.160978 ],
  [               0,               0,               0,               0,               0,               0,    -0.076151029,      0.11431291,     -0.14786894,      -1.2326777 ],
  [               0,               0,               0,               0,               0,               0,     -0.14597687,      0.29271165,      0.25149506,     -0.38196116 ],
  [               0,               0,               0,               0,               0,               0,  -0.00053483157,   0.00067478446,   0.00029159367,   -0.0011821743 ],
  [               0,               0,               0,               0,               0,               0,      0.30111745,     -0.12941854,     -0.62101472,      0.66248766 ],
  [               0,               0,               0,               0,               0,               0,      0.62548222,      -1.0814703,      -2.1750103,      -2.2332483 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0024612151,    0.0048751381 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.92847882,       5.2992873 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.9415588,      -4.1615196 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.37300555,     -0.31252017 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00018666902,    0.0011811277 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.7084956,        2.828789 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.2696415,      -1.1954569 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,    0.0078382282,     0.023711082,     0.015999436,   -0.0023079792,    -0.016498392,    -0.010481774,     0.021766871,    -0.013383114,   0.00048823978 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,     0.081668539,     0.046831687,    -0.029865441,     -0.18394148 ],
  [               0,               0,               0,               0,               0,               0,      -6.1334603,       5.9280847,      -3.2827024,       13.574266 ],
  [               0,               0,               0,               0,               0,               0,       5.0926989,       2.6081695,      -3.1024411,      -16.050176 ],
  [               0,               0,               0,               0,               0,               0,     0.035098582,     0.088049238,     0.094026619,     -0.50853446 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer2_etaoffv5_35_e = [
 [
  [               0,               0,               0,      -3.8857219,       11.890502,      -13.615672,       7.1224109,      -1.6184363,      0.11046673,   -0.0015860211 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0070638052,    0.0042675926,   -0.0029437047,    -0.017710168 ],
  [               0,               0,               0,               0,               0,               0,       9.1254369,       4.3922047,      -7.0367508,      -26.504794 ],
  [               0,               0,               0,               0,               0,               0,     0.064389332,      0.28582559,   -0.0044809521,      -2.0796439 ],
  [               0,               0,               0,               0,               0,               0,      -0.9543343,     -0.44539564,       0.7806622,       3.0018341 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0017894705,    0.0028978385 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.42274592,        3.855421 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.073606529,       -1.399803 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.22065884,     -0.20415905 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   2.2272736e-05,   0.00087710169 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.039591216,     -0.99896513 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.49238859,      0.51001724 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00010085632,  -0.00064382496,   1.8957312e-05,    0.0082945084 ],
  [               0,               0,               0,               0,               0,               0,      0.89524255,      -1.7905113,      -3.0729465,       10.432438 ],
  [               0,               0,               0,               0,               0,               0,     0.093144126,     -0.13794462,     -0.39330576,     -0.81385434 ],
  [               0,               0,               0,               0,               0,               0,    -0.072968407,      0.11126855,      0.30017592,     -0.35674816 ],
  [               0,               0,               0,               0,               0,               0,   0.00028378658,  -0.00021290856,  -0.00076053367,    0.0012747384 ],
  [               0,               0,               0,               0,               0,               0,      0.15759197,     -0.11853692,     -0.73361427,       1.6632319 ],
  [               0,               0,               0,               0,               0,               0,      0.30125928,      -0.3982065,      -1.1146582,     -0.27934051 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   1.6719013e-06,   8.6077548e-05,   0.00040330379,    0.0014194406,    0.0048948646 ],
  [               0,               0,               0,               0,               0,      0.11730451,     -0.19802942,     -0.76906244,     -0.80709708,         7.79266 ],
  [               0,               0,               0,               0,               0,    -0.016526854,     0.065489287,     0.058723618,     -0.25322454,      -1.2651094 ],
  [               0,               0,               0,               0,               0,    -0.019422492,     0.031808051,     0.055856459,    0.0064602279,      -0.1283826 ],
  [               0,               0,               0,               0,               0,   1.2812591e-05,   1.9431758e-06,  -4.2010062e-05,  -0.00016829687,   0.00051919152 ],
  [               0,               0,               0,               0,               0,     0.038819703,     0.040674733,     -0.16456437,     -0.79227373,     -0.45590552 ],
  [               0,               0,               0,               0,               0,      0.01270536,   -0.0094291431,    -0.076587152,     -0.23472361,      -0.5558456 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -0.00010136821,  -0.00010227204,   0.00022085142,    0.0019482602,    0.0092734735 ],
  [               0,               0,               0,               0,               0,      0.60867277,     -0.74151237,      -3.5332758,      -4.3079234,       23.417208 ],
  [               0,               0,               0,               0,               0,     0.088484511,     -0.11571193,     -0.46936792,     -0.53072465,       1.3939161 ],
  [               0,               0,               0,               0,               0,    -0.097519477,       0.1495471,      0.67979374,      0.85302329,      -4.0669173 ],
  [               0,               0,               0,               0,               0,  -0.00014439332,   -0.0001944582,  -0.00019489898,  -4.0772118e-05,    0.0013860374 ],
  [               0,               0,               0,               0,               0,    -0.095573775,      0.17226815,      0.48960794,     0.084761285,      -2.3296295 ],
  [               0,               0,               0,               0,               0,    -0.010167404,   -0.0076545315,    0.0030671396,      0.02923897,     0.092365406 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0010333542,    0.0083334467 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.83598963,      0.62283777 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.7809551,      -13.045986 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.1456313,     -0.26957242 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,       2.3881798,       -1.669375,      0.46231341,    -0.061190775,     0.001956606 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer1_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,    0.0021823503,    -0.037397953,     0.093093305,    -0.079679311,     0.021144788 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00044729847,   0.00075982722 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.4145919,       4.3967887 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.27015279,      -1.0151253 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.81798464,     -0.77791474 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.26139893,    -0.017839015 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.016478409,    0.0062645029,   2.5071941e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,      -7.6695211,      -2.3017607,       7.3483973 ],
  [               0,               0,               0,               0,               0,               0,               0,      -22.455884,     -0.18178499,       13.770939 ],
  [               0,               0,               0,               0,               0,               0,               0,       22.077195,      0.11691287,      -14.067842 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00026503942,   0.00028934702,    0.0020055982 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.3192985,      -3.3017484,       4.0408279 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.73161771,      -1.1336054,      -0.5445204 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.2677184,       4.5063587,      -3.5197594 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.43139767,      0.97714713,      -0.7118273 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00017842211,  -0.00046550788,   0.00043166035 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.0986848,      -1.7364179,      -1.2630797 ],
  [               0,               0,               0,               0,               0,               0,               0,      -6.5517309,       15.335991,       -2.458219 ],
 ],
 [
  [               0,               0,               0,     -0.13643282,       0.2703695,    -0.014444077,     -0.26890273,      0.19523319,    -0.056665262,    0.0027716013 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.19322597,     -0.63299686,      0.81091768,      -0.5177647,       0.1630742,    -0.016788934 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     -0.25235848,    0.0040164038,       0.5258777 ],
  [               0,               0,               0,               0,               0,               0,               0,       68.218013,     0.040098984,       -137.5192 ],
  [               0,               0,               0,               0,               0,               0,               0,      -38.211294,     -0.19176741,       79.884693 ],
  [               0,               0,               0,               0,               0,               0,               0,      -39.345607,      0.95220731,       77.216678 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.041752998,    -0.035594322,     -0.39384225 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer1_etaoffv5_37_e = [
 [
  [               0,               0,               0,               0,      0.46920813,      -1.4825616,       1.5910092,     -0.65863662,     0.076505435,  -0.00075194401 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0045626707,    0.0083343828 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -4.171582,       8.2726632 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -19.203705,       27.139632 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -20.823771,       29.152164 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.17563574,     -0.29499512 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.14294555,      0.22038576 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -28.757927,       48.775619 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       14.615803,      -23.192773 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.5292489,      -6.6211313 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0006147236,    0.0010386293 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.61006646,       1.3801625 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.39234823,     -0.54903425 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.60354082,    -0.099483391 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.11375765,      0.26550059 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00013446113,    5.040438e-05,   0.00041069085,     0.001356765 ],
  [               0,               0,               0,               0,               0,               0,     -0.23614527,      0.12339406,      0.49505788,       3.9286292 ],
  [               0,               0,               0,               0,               0,               0,     -0.58260213,      0.18698565,        1.093205,     -0.24843902 ],
  [               0,               0,               0,               0,               0,               0,     0.032418144,     -0.31604188,     -0.40110045,      0.39166605 ],
  [               0,               0,               0,               0,               0,               0,     -0.02022527,        0.233892,      0.31743687,      -1.1860093 ],
  [               0,               0,               0,               0,               0,               0,   0.00025483608,   0.00014842541,  -0.00041352165,   -0.0012062723 ],
  [               0,               0,               0,               0,               0,               0,     -0.28299055,    -0.064070747,      0.22686452,       2.7240902 ],
  [               0,               0,               0,               0,               0,               0,     -0.39593067,     -0.11018343,      0.41846036,      0.81636918 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00025536607,  -6.2583122e-05,   0.00085640216,    0.0042819358 ],
  [               0,               0,               0,               0,               0,               0,      0.11240887,    -0.026433395,     -0.41724996,       2.7678967 ],
  [               0,               0,               0,               0,               0,               0,     -0.23062682,      0.53164929,       1.0480026,      -3.6741513 ],
  [               0,               0,               0,               0,               0,               0,      0.32030869,     -0.60610584,      -1.4469929,       1.5766105 ],
  [               0,               0,               0,               0,               0,               0,      -0.1877543,      0.40877628,      0.94100845,      -1.9791915 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -0.00012176675,  -9.2640014e-05,   0.00021672374,    0.0014840424,    0.0061733072 ],
  [               0,               0,               0,               0,               0,      -2.7647557,      -3.0515383,     0.035311665,       16.251024,       78.438192 ],
  [               0,               0,               0,               0,               0,       1.9106723,       2.1910094,    -0.021304345,      -11.671026,       -53.93364 ],
  [               0,               0,               0,               0,               0,      0.14361749,      0.18456729,     0.064885449,     -0.73390003,      -4.8478251 ],
  [               0,               0,               0,               0,               0,       -0.184099,      -0.1971243,     0.066611067,       1.3480347,       5.5458831 ],
  [               0,               0,               0,               0,               0,  -4.8386861e-05,   1.5494478e-05,   0.00032630927,    0.0014169035,    0.0057866042 ],
  [               0,               0,               0,               0,               0,     0.047525439,      0.10649017,     -0.02866353,     -0.69717608,      -1.0599843 ],
  [               0,               0,               0,               0,               0,     0.068323874,     0.058893046,    -0.090271931,     -0.72620322,      -2.8616345 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   7.0803779e-05,    0.0023309003 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.3198158,       8.9841016 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.35944576,       -0.437326 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.4121506,       2.2164655 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.74928899,       1.8442087 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   2.6074809e-06,   0.00051061195 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.70225435,        6.307121 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.095796875,      -0.6398047 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.27876067,      -1.9463647 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.34929415,      -1.1381953 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0072657324,    0.0021950282,    -0.039593396 ],
  [               0,               0,               0,               0,               0,               0,               0,      -12.969292,       1.0476569,       81.014875 ],
  [               0,               0,               0,               0,               0,               0,               0,      -0.1814419,      -0.1112609,     -0.21825173 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.53878159,     0.033279884,      -3.1833178 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.24412332,      0.64107819 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -246.37676,       619.62703 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -4.6844793,      -2.2827323 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -49.581945,       122.00588 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.8955537,       10.369458 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer2_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0057981617,    0.0056482532 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.8784597,       5.8921399 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.063297711,      -1.3321804 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.049459785,    -0.043799642 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0015595254,   0.00083886072 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.7959432,        1.132389 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.2719701,      -1.7370651 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,     0.010261219,   -0.0023002329,   -0.0087800262,     0.010366712 ],
  [               0,               0,               0,               0,               0,               0,      -12.527159,        11.51281,     -0.99592751,       4.4795351 ],
  [               0,               0,               0,               0,               0,               0,      -2.3988386,       1.5537406,      0.49881248,         -1.6695 ],
  [               0,               0,               0,               0,               0,               0,       1.9802362,      -1.4773485,     -0.19862109,      0.18082442 ],
  [               0,               0,               0,               0,               0,               0,   -0.0096228516,     0.010833363,   -0.0020554099,   8.1678189e-05 ],
  [               0,               0,               0,               0,               0,               0,       1.1620447,      -7.2150193,       7.3588918,      -1.1417621 ],
  [               0,               0,               0,               0,               0,               0,       -7.390973,       1.5295602,       5.0703876,      -3.3881257 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00053160212,    0.0032258843,    0.0057329376 ],
  [               0,               0,               0,               0,               0,               0,               0,         5.57256,      -1.3215586,       1.7855207 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.93217241,       6.5339396,      -7.1011843 ],
  [               0,               0,               0,               0,               0,               0,               0,      -4.6542588,       1.1514773,       1.9947298 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0069485704,  -0.00047499118,   -0.0040559115 ],
  [               0,               0,               0,               0,               0,               0,               0,       8.8918644,       3.8819334,       -6.414505 ],
  [               0,               0,               0,               0,               0,               0,               0,      -20.988704,      -1.1146913,        11.11375 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   -0.0012082424,   0.00040371408,    0.0030161858,    0.0070267085 ],
  [               0,               0,               0,               0,               0,               0,      0.93628507,      -1.0780636,      -1.3264632,       5.4534468 ],
  [               0,               0,               0,               0,               0,               0,     -0.09272512,      0.12735403,    -0.035565537,      -1.3159961 ],
  [               0,               0,               0,               0,               0,               0,    -0.046857402,      0.11876749,     0.028629461,      -0.1022203 ],
  [               0,               0,               0,               0,               0,               0,   0.00013789451,   -0.0001483291,  -0.00037129845,   0.00066029646 ],
  [               0,               0,               0,               0,               0,               0,      0.55441377,      -0.2964383,     -0.68857884,       1.9290153 ],
  [               0,               0,               0,               0,               0,               0,      0.30355495,     -0.19117692,      -1.1387284,      -3.8188241 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0024406952,    0.0049640404 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.88826764,       5.2215973 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.0586146,      -4.3138248 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.42584145,     -0.34422753 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -7.6185381e-05,    0.0013545911 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.8722013,       2.9532332 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.2711147,      -1.2202697 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,    -0.092703962,      0.11160867,     0.080370779,    -0.016791921,    -0.066418328,     -0.02524165,     0.055763964,    -0.022866561,    0.0009927807 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,      0.10311702,     0.057248375,    -0.042497882,     -0.24856388 ],
  [               0,               0,               0,               0,               0,               0,      -5.7165857,       1.5603144,       5.1246137,       9.2878912 ],
  [               0,               0,               0,               0,               0,               0,       5.1520093,       2.6040194,      -3.1832474,      -16.118775 ],
  [               0,               0,               0,               0,               0,               0,     0.027100295,     0.072811131,     0.084438716,     -0.42772391 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam500_layer2_etaoffv5_37_e = [
 [
  [               0,               0,               0,      -3.9535387,       12.139782,      -13.976606,       7.3779712,      -1.7058905,      0.12231064,   -0.0020536471 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0058783106,    0.0037103001,   -0.0019769414,    -0.013599663 ],
  [               0,               0,               0,               0,               0,               0,       9.6485437,       4.6625174,      -7.4257088,      -28.296607 ],
  [               0,               0,               0,               0,               0,               0,      0.03677035,      0.27176679,      0.01612283,      -1.9860497 ],
  [               0,               0,               0,               0,               0,               0,      -1.0340418,     -0.48174521,      0.84144646,       3.2516033 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0016637394,    0.0028839462 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.29473679,       4.1401262 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.15282872,      -1.0797339 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       0.1754288,     -0.11932658 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   1.4230668e-05,   0.00097702087 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.7403046,       1.6961817 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.46354041,      0.63999735 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00020033681,  -0.00062327733,  -0.00016248596,    0.0081018883 ],
  [               0,               0,               0,               0,               0,               0,      0.81691977,      -1.7129691,      -2.8978358,        10.28392 ],
  [               0,               0,               0,               0,               0,               0,      0.10405945,     -0.14766831,     -0.41852254,     -0.79742739 ],
  [               0,               0,               0,               0,               0,               0,    -0.080904533,      0.12048737,      0.31813553,     -0.37414861 ],
  [               0,               0,               0,               0,               0,               0,   0.00024467957,  -0.00019587133,  -0.00071311809,    0.0013418453 ],
  [               0,               0,               0,               0,               0,               0,      0.15295145,     -0.12332997,     -0.72457598,       1.6795165 ],
  [               0,               0,               0,               0,               0,               0,      0.34075977,     -0.45913534,      -1.2569172,    -0.064311323 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -2.4027448e-05,    1.243459e-05,   0.00024326319,    0.0011611877,    0.0047226174 ],
  [               0,               0,               0,               0,               0,      0.11154964,     -0.20315923,     -0.74950919,     -0.71553034,        7.962588 ],
  [               0,               0,               0,               0,               0,    -0.022673275,     0.075506786,     0.081618619,     -0.26066936,      -1.4202441 ],
  [               0,               0,               0,               0,               0,    -0.022436367,     0.034732621,     0.068676476,     0.025311702,      -0.1360554 ],
  [               0,               0,               0,               0,               0,   5.4011011e-06,   3.3190995e-06,  -4.3439896e-05,  -0.00027392673,  -0.00012428386 ],
  [               0,               0,               0,               0,               0,    -0.023382166,     0.070618709,        0.102853,     -0.35319379,     -0.68586633 ],
  [               0,               0,               0,               0,               0,     0.012564815,   -0.0015034833,    -0.048656384,     -0.22062591,     -0.85104074 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -0.00019544193,  -0.00040550228,  -0.00023760462,    0.0025817066,     0.017533987 ],
  [               0,               0,               0,               0,               0,       1.9749285,      -2.2804642,      -11.119332,      -13.623486,        65.42486 ],
  [               0,               0,               0,               0,               0,     0.055018947,     -0.12417026,     -0.20561992,      0.14662238,     -0.51636287 ],
  [               0,               0,               0,               0,               0,     -0.44927328,      0.60314245,       2.8272691,       3.3586435,       -17.37342 ],
  [               0,               0,               0,               0,               0,   0.00064456128,   0.00059016015,  -0.00038147467,   -0.0040292247,    -0.013268493 ],
  [               0,               0,               0,               0,               0,      0.31565439,     -0.53185469,      -1.7555398,      -1.5183571,       10.737925 ],
  [               0,               0,               0,               0,               0,     0.011986987,      0.19245431,      0.36265493,     -0.45028477,      -6.6212312 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00094463206,    0.0082424134 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.95297825,      0.40736593 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.8838147,      -15.778676 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.26925323,     -0.56845537 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,        2.262074,      -1.5155212,      0.40565002,      -0.0529032,    0.0015783885 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]

###############################################################################
# Tables for v5 sshape correction, 1000 GeV photon clusters
#

gam1000_layer1_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,    -0.061282346,       0.1692842,     -0.15304919,     0.044335318 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0004802504,   0.00077837006 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.6840918,       4.6817591 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.37393465,      -1.1152123 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.89561415,     -0.79764569 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.2883723,   -0.0075952049 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.020074968,    0.0072897228,   -0.0006279828 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.8175954,     -0.35262605,       2.0834504 ],
  [               0,               0,               0,               0,               0,               0,               0,        -14.6432,     -0.14624243,       8.7818972 ],
  [               0,               0,               0,               0,               0,               0,               0,        16.12401,       0.1249923,      -10.298406 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00086254345,    0.0016863782 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.56471437,       2.7044474 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.25401349,      -1.2554517 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.061677925,      -1.2592478 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0073068492,     -0.11134513 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   7.3021756e-05,  -7.6262665e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.9379193,       3.3680893 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.2134679,       2.0371676 ],
 ],
 [
  [               0,               0,               0,     -0.11181089,      0.23618822,    -0.013208168,     -0.25408749,      0.18814309,    -0.055609226,    0.0028694511 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     -0.13672261,      0.27412378,     -0.12253271,    -0.070837438,     0.062933723,   -0.0079275695 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,     0.001267392,     0.002164863,    0.0035845309,    0.0063025011 ],
  [               0,               0,               0,               0,               0,               0,       7.0943833,       3.3802115,      -4.9638837,      -17.291437 ],
  [               0,               0,               0,               0,               0,               0,        4.410162,       1.7302768,      -3.4979733,      -13.096272 ],
  [               0,               0,               0,               0,               0,               0,      -17.535065,      -7.9278881,       12.092379,       48.232463 ],
  [               0,               0,               0,               0,               0,               0,    0.0013511449,    0.0027263642,    0.0033265563,     -0.29905724 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer1_etaoffv5_55_e = [
 [
  [               0,               0,               0,               0,       0.1898977,     -0.31886347,     0.082892093,     0.053982181,    -0.014068945,     0.003058365 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0032150405,    0.0067053869 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.44959025,       2.1661741 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -23.035931,       32.634144 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -25.191844,       35.486789 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.13456877,     -0.22439343 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.21025849,      0.32068439 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -40.369753,       66.355631 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       15.334398,      -24.232796 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.9649912,      -7.2697913 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00060932913,   0.00084135471 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.61714948,       1.5533423 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.3114553,     -0.69478191 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.51002422,     -0.25434407 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.12072455,      0.28526466 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00011455025,   2.8429552e-05,   0.00029995617,    0.0011013637 ],
  [               0,               0,               0,               0,               0,               0,      0.17875451,      0.22327102,    -0.048225295,       2.5792915 ],
  [               0,               0,               0,               0,               0,               0,     -0.80475956,     -0.19443527,       1.1222593,       2.4908798 ],
  [               0,               0,               0,               0,               0,               0,    -0.034128205,     -0.27757632,     -0.24228629,      0.41805751 ],
  [               0,               0,               0,               0,               0,               0,     0.071931027,      0.25466661,      0.17924085,      -1.5719431 ],
  [               0,               0,               0,               0,               0,               0,   0.00025178495,   0.00010791182,  -0.00043215642,  -0.00089244812 ],
  [               0,               0,               0,               0,               0,               0,      -0.2548145,    -0.043709973,      0.20001513,       2.5400406 ],
  [               0,               0,               0,               0,               0,               0,     -0.33221861,     -0.11918359,      0.28756575,      0.61868068 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00025108815,  -7.9219793e-05,   0.00081047597,    0.0042761582 ],
  [               0,               0,               0,               0,               0,               0,      0.21913323,     -0.18159939,     -0.81658289,       3.4597971 ],
  [               0,               0,               0,               0,               0,               0,     -0.22414496,      0.50035127,       1.0052761,      -3.5269758 ],
  [               0,               0,               0,               0,               0,               0,      0.43170828,     -0.83760066,      -1.9673756,       2.6508793 ],
  [               0,               0,               0,               0,               0,               0,     -0.23003416,      0.50175948,       1.1456314,      -2.4225716 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -4.9244383e-05,  -2.6463366e-05,   0.00013807431,   0.00077352606,    0.0033241019 ],
  [               0,               0,               0,               0,               0,     -0.65757856,     -0.63228195,     0.037124618,       3.5062618,       19.909712 ],
  [               0,               0,               0,               0,               0,     -0.56558342,     -0.66353757,    -0.060761336,       3.2295886,       14.775078 ],
  [               0,               0,               0,               0,               0,      -0.8183615,     -0.93523735,     0.022303429,       5.0390226,       22.036076 ],
  [               0,               0,               0,               0,               0,     0.098422938,      0.13562391,      0.10281832,     -0.24538052,       -1.974687 ],
  [               0,               0,               0,               0,               0,   5.7578049e-05,    0.0001222168,   0.00025778982,   0.00053971956,    0.0021199483 ],
  [               0,               0,               0,               0,               0,     -0.11010177,    -0.072066041,    -0.015258597,      0.31161925,       3.5162225 ],
  [               0,               0,               0,               0,               0,    -0.093691334,     -0.13271833,     -0.11506007,      0.17237806,       1.4006436 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00023772341,     0.002455267 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.7809323,       7.3244624 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.1143496,      -3.9206887 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.3745801,       2.0912636 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.88423512,       2.1485342 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00010422255,   0.00044839075 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        1.222138,       32.049477 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.43365898,      0.62127931 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.54406407,       -2.271881 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.0348904,      -2.7816731 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.012962384,    0.0015866075,    -0.071918927 ],
  [               0,               0,               0,               0,               0,               0,               0,       -19.68951,       2.4609001,        118.0896 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.3979558,      -6.6001196,       6.1850523 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.92950636,      -2.3600444,      0.34069632 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0025476648,    0.0036744893,    0.0029892713 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.005327898,    -0.012540196,       7.3170414 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0026646405,   -0.0016261251,       3.9728436 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.005061285,     0.011221731,     -0.71558517 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.076990763,    -0.021531621,       1.2919018 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer2_etaoffv5_55_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0084318279,    0.0085053361 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.3771109,       4.1706889 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.21358168,      -1.2919056 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.012650321,    -0.024873705 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0036802663,  -0.00036770226 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.54907932,      0.85632535 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.2147502,      -1.2395656 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0052217773,    0.0051195218 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.9922865,       5.2316574 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.24868613,      -1.1484047 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.24145313,     -0.17195618 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0017541403,    0.0010661561 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.6378431,     0.039902395 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.6604473,      0.59789618 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0041366872,    0.0058190924 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        5.905662,      -1.1052825 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       7.4999738,      -7.2674658 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -6.6840553,       5.2806726 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0025192795,    0.0022502686 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       32.008938,      -24.240663 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -3.4067607,     -0.96377073 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00080508501,   0.00073401897,    0.0031151427,    0.0055281169 ],
  [               0,               0,               0,               0,               0,               0,      0.78621983,      -1.2357598,     -0.96195015,       4.8870205 ],
  [               0,               0,               0,               0,               0,               0,      0.34128469,     -0.87032825,      0.62238804,      -1.3873435 ],
  [               0,               0,               0,               0,               0,               0,     -0.13701815,     -0.29707687,       1.0184691,      -0.5671648 ],
  [               0,               0,               0,               0,               0,               0,    0.0016567064,   -0.0013792924,   -0.0021477331,     0.002200826 ],
  [               0,               0,               0,               0,               0,               0,      0.31340612,     -0.27808658,     -0.99213383,       1.1024763 ],
  [               0,               0,               0,               0,               0,               0,       1.4517512,      0.59441854,      -1.3898179,      -8.9403811 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.17796185,       0.2512912 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -21.248444,        32.53987 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.3840745,      -3.3578394 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.4398639,      -4.4595905 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,    -0.016864053,     0.021333025,    0.0084006562,    -0.013050027,    0.0087433716,    -0.009505676,  -8.8493635e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0080776992,    0.0047396041,     0.011515664 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.11627649,     0.005743251,       40.219889 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.030148654,   -0.0045836345,      -2.1319057 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.021002427,     -0.01592795,      0.63448067 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer2_etaoffv5_55_e = [
 [
  [               0,               0,               0,     -0.23640026,      0.47439078,    -0.029764286,     -0.43291439,      0.27353338,    -0.047311454,     0.001528697 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0014736079,     0.001099476,  -5.6892783e-06,   -0.0019722988 ],
  [               0,               0,               0,               0,               0,               0,       3.3198066,       1.6384836,      -2.5719274,      -7.4280904 ],
  [               0,               0,               0,               0,               0,               0,      -1.8909575,     -0.87686962,       1.2809398,       4.9183376 ],
  [               0,               0,               0,               0,               0,               0,       1.8625858,      0.98581032,      -1.2531467,       -6.304487 ],
  [               0,               0,               0,               0,               0,               0,    0.0014410872,    0.0007045006,   -0.0010971242,    -0.003828744 ],
  [               0,               0,               0,               0,               0,               0,       2.8112609,       1.1714084,      -2.3262379,       -7.248718 ],
  [               0,               0,               0,               0,               0,               0,      -5.2489451,      -1.4794356,       3.9213635,       12.466472 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00076679324,    0.0070329838 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.4746623,      -4.9428221 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.3287563,      -3.3026622 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -1.155325,       1.8620281 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,  -0.00056303663,  -3.2379895e-05,     0.010232784 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.2048675,       -4.853979,        8.209334 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0015237877,    -0.084931411,      -1.2175587 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.098585611,       0.3496428,     -0.18598764 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00027268458,  -0.00057051703,   0.00065446009 ],
  [               0,               0,               0,               0,               0,               0,               0,       1.7595277,      -5.0212439,       4.0603676 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.88341956,      -1.5748405,      -2.3551429 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00070038639,  -0.00053696022,   -0.0011334824,    0.0079498375 ],
  [               0,               0,               0,               0,               0,               0,      0.23797241,     -0.87985102,      -1.8157678,       8.1600767 ],
  [               0,               0,               0,               0,               0,               0,     0.027969661,      0.05038071,     -0.16410091,      -1.4124251 ],
  [               0,               0,               0,               0,               0,               0,    -0.074836474,      0.14073942,      0.33501664,     -0.51481306 ],
  [               0,               0,               0,               0,               0,               0,   0.00011083268,  -0.00017698802,  -0.00052235201,     0.001345303 ],
  [               0,               0,               0,               0,               0,               0,      0.22768873,     -0.62783179,      -1.1843783,       3.6600572 ],
  [               0,               0,               0,               0,               0,               0,    0.0065195039,     -0.10512762,     -0.48649654,      -1.0440896 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   4.6464237e-05,   0.00010301779,   0.00029835699,    0.0010062616,    0.0040010311 ],
  [               0,               0,               0,               0,               0,      0.01985237,     -0.05890407,      -0.2275068,     -0.35417655,       4.5400745 ],
  [               0,               0,               0,               0,               0,     0.014951943,   0.00099627664,    -0.050946926,     -0.19534142,     -0.95521814 ],
  [               0,               0,               0,               0,               0,    0.0026118895,   -0.0037807241,   -0.0077111097,     0.036468075,     0.020603186 ],
  [               0,               0,               0,               0,               0,  -8.6026181e-06,  -2.3167845e-05,  -3.9652837e-05,  -4.2312544e-06,    0.0013331649 ],
  [               0,               0,               0,               0,               0,    -0.025984425,     0.010133351,    -0.018747515,     -0.11057098,       1.6650251 ],
  [               0,               0,               0,               0,               0,    -0.024797173,    -0.055477504,     -0.10763113,      -0.1612768,    -0.078641529 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0070226588,    -0.003342376 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -6.4395123,       17.856426 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.36844628,     -0.38206201 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.6026541,      -6.1907612 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,      -2.8221176,       2.8647081,     -0.79261415,     0.068190573,  -0.00083667836 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer1_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,    -0.059043404,      0.16912618,     -0.15675771,     0.046200276 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00053161728,   0.00081174147 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        -2.05165,       4.7361139 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.52513427,      -1.2298839 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.68847532,     -0.91855134 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.27114927,   -0.0074117029 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,      -752.69214,       1202.2962,      -480.09486 ],
  [               0,               0,               0,               0,               0,               0,               0,      -10.615563,      -64.372717,       58.758943 ],
  [               0,               0,               0,               0,               0,               0,               0,       122.55877,      -142.05419,        34.69424 ],
  [               0,               0,               0,               0,               0,               0,               0,       116.56319,      -165.00561,       57.504355 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0005154047,    0.0017988919 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.59399585,       2.8460629 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.29962247,      -1.2898154 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.050822334,      -1.3924189 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.018945965,     -0.11176079 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -3.9115982e-05,   4.4298078e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       12.795932,     -0.96828247 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.21384734,      0.23856507 ],
 ],
 [
  [               0,               0,               0,     -0.10183174,      0.21948957,     -0.01833666,     -0.23268694,      0.17611369,    -0.053407472,    0.0028002797 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,      0.15828492,     -0.57210875,      0.78983571,     -0.52943993,      0.17037444,    -0.017650284 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     -0.33408878,    0.0069718792,       0.7025052 ],
  [               0,               0,               0,               0,               0,               0,               0,       18.162246,     -0.16517831,      -34.248465 ],
  [               0,               0,               0,               0,               0,               0,               0,       0.3929882,     -0.30071774,      -1.4961998 ],
  [               0,               0,               0,               0,               0,               0,               0,      -36.028889,     0.057270296,        71.71351 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.022126526,     0.014445207,     -0.23238111 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer1_etaoffv5_35_e = [
 [
  [               0,               0,               0,               0,      0.85092411,      -2.3339355,       2.2056359,     -0.80416268,     0.078454978,  -0.00017895838 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0030860389,    0.0065000032 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.68634786,       1.8819358 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -24.942611,        35.43901 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -29.237025,       41.385312 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.13411632,      -0.2255276 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.22308959,      0.33983672 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -38.986963,       64.196801 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       15.099805,      -23.875531 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.9509693,      -7.2478244 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0006013284,   0.00084790006 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.61732222,        1.558058 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.30773916,     -0.70079966 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.51184288,      -0.2507443 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.12066779,      0.28549553 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00012479461,   4.8369549e-05,   0.00040266593,    0.0013976382 ],
  [               0,               0,               0,               0,               0,               0,    -0.041426095,    -0.036375342,    0.0038283224,       4.2004565 ],
  [               0,               0,               0,               0,               0,               0,     -0.44299094,      0.03287905,      0.69490275,     0.092248009 ],
  [               0,               0,               0,               0,               0,               0,    -0.080992952,     -0.32459184,     -0.20613029,      0.81217662 ],
  [               0,               0,               0,               0,               0,               0,    -0.029781065,       0.1815655,      0.28491491,     -0.87894312 ],
  [               0,               0,               0,               0,               0,               0,   0.00028628084,   0.00013827253,  -0.00048345371,    -0.001276541 ],
  [               0,               0,               0,               0,               0,               0,     -0.22899538,    -0.028383029,      0.14775349,       2.2518032 ],
  [               0,               0,               0,               0,               0,               0,     -0.33430963,     -0.10064267,      0.31685143,      0.56783558 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00013613241,  -0.00045708665,   0.00019559501,    0.0062149375 ],
  [               0,               0,               0,               0,               0,               0,      0.42506961,      -0.5442398,      -1.7133401,       5.0060221 ],
  [               0,               0,               0,               0,               0,               0,     -0.18274725,       0.4110103,      0.81823319,      -3.1239101 ],
  [               0,               0,               0,               0,               0,               0,      0.48890213,     -0.94851161,      -2.2372157,       3.1745599 ],
  [               0,               0,               0,               0,               0,               0,     -0.22735784,      0.49695887,       1.1424327,      -2.4202271 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   0.00069886602,  -2.6187709e-05,   -0.0033314406 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.63008512,     -0.34031199,       3.2644287 ],
  [               0,               0,               0,               0,               0,               0,               0,      -4.2280311,      -9.1334635,      -20.048343 ],
  [               0,               0,               0,               0,               0,               0,               0,      -2.7736252,      -5.8530688,      -13.253863 ],
  [               0,               0,               0,               0,               0,               0,               0,        1.070776,       2.4613633,        5.357373 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00094562658,    0.0022767621,     0.006470545 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.54113367,    -0.024844122,       3.2923697 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.21765374,     -0.25296933,    -0.013370614 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00027343432,    0.0022973923 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.9416728,       7.7541198 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.1894074,      -4.0958246 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.3816857,       2.1139403 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.90486608,       2.1988532 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00010422255,   0.00044839075 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        1.222138,       32.049477 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.43365898,      0.62127931 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.54406407,       -2.271881 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.0348904,      -2.7816731 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    0.0031033352,    0.0028985913,   -0.0048236015,    -0.041472864 ],
  [               0,               0,               0,               0,               0,               0,      -4.9937821,      -3.3146714,       11.125742,       72.033267 ],
  [               0,               0,               0,               0,               0,               0,      0.55814481,      -1.3090156,      -3.3800475,       6.3402839 ],
  [               0,               0,               0,               0,               0,               0,      0.32969844,     -0.45367991,      -1.4923605,       1.6682296 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0048616777,    0.0072649251,   -0.0088855965 ],
  [               0,               0,               0,               0,               0,               0,               0,      -0.0849801,    -0.010619632,       16.284129 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.002891472,   -0.0063796324,       6.5899155 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0052371102,      -0.1217376,     -0.12124953 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00027352354,    -0.002853824,        1.135593 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer2_etaoffv5_35_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0063872986,    0.0055131898 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -3.675439,       6.2266192 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.05047066,      -1.3395755 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.082164095,    -0.038578537 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0013648927,   0.00079030462 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.3791895,       1.2057092 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.5282606,      -1.7837694 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0061652117,    0.0052450674 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        -2.62862,       6.1564718 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,        0.206173,      -1.4804344 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.012392951,    -0.002183882 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0013639963,   0.00043427446 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.92303305,       1.8897041 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.3266181,       -18.76909 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.040886901,     0.041493623 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       30.140939,       -19.78399 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       7.8452974,      -7.5502312 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -6.7655019,       5.3449367 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.01058922,    0.0084179139 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.010820375,       2.2803608 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       14.830249,      -12.748372 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00095019421,    0.0070021052 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.68867987,       5.0066581 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.058360749,      -1.2822169 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00069788786,     0.015779991 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -0.00057312417,    0.0014193067 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.030137281,       0.1448907 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.94595961,      -15.295983 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0021028684,     0.011107737 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.31516065,       3.8107157 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.9699633,      -6.8090862 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.57667709,     -0.73273713 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,      0.30391904,     -0.36032991,    -0.059618365,      0.24169791,     -0.10107718,    0.0060908766,  -0.00051798095 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0064188549,    0.0094462839,    0.0051288896 ],
  [               0,               0,               0,               0,               0,               0,               0,   -0.0022204315,   -0.0011627622,       5.9626824 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.030423852,    -0.016538855,      -1.3347973 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.034110009,   0.00024947373,    -0.029578666 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer2_etaoffv5_35_e = [
 [
  [               0,               0,               0,      -3.9655415,        11.89795,      -13.266396,       6.6762506,      -1.4208008,     0.079751962,   0.00026585468 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    -0.019970947,   -0.0093651142,     0.017278568,     0.074058205 ],
  [               0,               0,               0,               0,               0,               0,       2.1846168,      0.97960547,      -1.8126764,      -3.3977285 ],
  [               0,               0,               0,               0,               0,               0,      0.40067517,      0.26745384,     -0.36847237,      -2.4916402 ],
  [               0,               0,               0,               0,               0,               0,      -1.1032354,     -0.52145774,      0.87746298,        3.446648 ],
  [               0,               0,               0,               0,               0,               0,    -0.023582121,    -0.011942374,     0.017298642,     0.079850593 ],
  [               0,               0,               0,               0,               0,               0,        4.878856,       2.9781668,       -3.379022,      -15.163129 ],
  [               0,               0,               0,               0,               0,               0,      -33.386414,      -17.177373,       23.207019,        108.0989 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00035142416,    0.0075060556 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.4212299,       1.8179822 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       2.8841072,      -5.7258463 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.40521052,      0.74940454 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,  -0.00093731782,  -0.00028153857,     0.010398383 ],
  [               0,               0,               0,               0,               0,               0,               0,       3.1819487,      -11.286349,       14.444423 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.31014513,      -1.1620687,     -0.36291464 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.34628503,       1.2434312,     -0.98706414 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00061148065,  -0.00062689957,   2.5151532e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.92586815,      -2.7548514,       2.8241305 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.33536241,     -0.52202219,      -2.0394718 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,    0.0003400289,  -0.00028593896,    -0.001359574,   -0.0007323679,      0.01264364 ],
  [               0,               0,               0,               0,               0,     0.022536136,    -0.080300644,     -0.36687335,     -0.48540917,       6.2464731 ],
  [               0,               0,               0,               0,               0,    -0.028747249,     0.075187306,      0.14297349,      -0.1262422,      -1.8231106 ],
  [               0,               0,               0,               0,               0,    -0.030276165,     0.016560329,     0.067036786,      0.10891822,     0.079844178 ],
  [               0,               0,               0,               0,               0,  -0.00022287857,    0.0001151792,    0.0007459249,    0.0010629541,   -0.0010319142 ],
  [               0,               0,               0,               0,               0,      0.12485087,    -0.092958604,     -0.57996646,     -0.96638562,       3.2819421 ],
  [               0,               0,               0,               0,               0,      0.25332915,     -0.07998409,     -0.90984621,      -1.7740163,       1.1248247 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -8.4739208e-05,  -0.00023712272,  -0.00015108784,    0.0018103402,     0.012780905 ],
  [               0,               0,               0,               0,               0,      0.10035729,     -0.11717535,     -0.59158048,     -0.84450854,       6.5032217 ],
  [               0,               0,               0,               0,               0,    -0.012005979,     0.014340321,       0.0555575,    -0.023203378,      -1.3473459 ],
  [               0,               0,               0,               0,               0,   -0.0044219779,     0.018357246,     0.065665117,     0.092110102,     -0.52126762 ],
  [               0,               0,               0,               0,               0,   0.00010567373,  -1.0037119e-05,  -0.00031906857,   -0.0006667396,    0.0015512956 ],
  [               0,               0,               0,               0,               0,     0.051021313,    -0.054952065,     -0.27209755,     -0.51362818,       2.1893609 ],
  [               0,               0,               0,               0,               0,     0.024910323,     0.009101661,    -0.096665689,      -0.5100736,      -1.8794048 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00077602314,    0.0087954529 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.2205285,     -0.24719403 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.2544622,      -14.226647 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.21338057,     -0.43448534 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,       4.1377956,       -3.177392,      0.90334936,     -0.11228106,    0.0038100321 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer1_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,    -0.058880902,      0.16718965,     -0.15438084,     0.045399983 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0005453701,   0.00079586088 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.0994422,       4.7802081 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.53191329,      -1.2288957 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.73832083,     -0.88858766 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -0.276499,   -0.0058454717 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,     0.012679138,    0.0052655822,   0.00084129596 ],
  [               0,               0,               0,               0,               0,               0,               0,      -1.8097404,      -1.2018865,       2.8427584 ],
  [               0,               0,               0,               0,               0,               0,               0,      -20.430848,     -0.17456132,       12.398317 ],
  [               0,               0,               0,               0,               0,               0,               0,       22.161595,      0.13312924,      -14.123694 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00035841959,    0.0018444659 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -0.4933305,       2.8096584 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.30723689,      -1.2915463 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0063852407,      -1.3467871 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.031686248,     -0.10573567 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,  -1.2103663e-05,   2.1380461e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       18.401436,      -5.3549928 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -4.2377092,       4.1756104 ],
 ],
 [
  [               0,               0,               0,     -0.12169922,      0.26699145,    -0.046398559,     -0.25337075,      0.20976002,    -0.068201111,    0.0048765203 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,     0.071469242,        -0.34352,      0.56940515,     -0.43444306,      0.15268833,     -0.01656087 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0059841406,    0.0057160567 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.5870748,       1.7464683 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -17.54828,       24.411254 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -113.58935,       160.28692 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.13816996,     -0.19610393 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer1_etaoffv5_37_e = [
 [
  [               0,               0,               0,               0,      0.80134075,      -2.2162048,       2.1071344,      -0.7683678,     0.072400417,   0.00019615722 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.003195386,    0.0066390945 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.33668725,       2.3328478 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -25.764217,       36.653315 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -30.626809,       43.417677 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.13487579,     -0.22580056 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.22419762,      0.34144634 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -38.860405,       64.004244 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       15.365674,      -24.281842 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       5.0684606,       -7.411962 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0005823217,   0.00088748782 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.59293335,       1.5936747 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.27961466,     -0.74822083 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.51832544,     -0.24089029 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.12043076,      0.28473956 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00015411753,    4.790359e-05,   0.00047234028,    0.0015957754 ],
  [               0,               0,               0,               0,               0,               0,     0.032052849,     -0.07902344,     -0.18762508,       4.0721736 ],
  [               0,               0,               0,               0,               0,               0,     -0.36512299,     0.055489341,      0.58029453,     -0.26672491 ],
  [               0,               0,               0,               0,               0,               0,    -0.068354286,     -0.26974448,     -0.17094299,      0.49921994 ],
  [               0,               0,               0,               0,               0,               0,    -0.032323147,      0.16229378,      0.27018853,      -0.7687827 ],
  [               0,               0,               0,               0,               0,               0,   0.00027039035,   0.00012389089,  -0.00047576461,   -0.0011856559 ],
  [               0,               0,               0,               0,               0,               0,     -0.24530836,    -0.045845518,       0.1701701,       2.3704467 ],
  [               0,               0,               0,               0,               0,               0,     -0.36463712,     -0.11710927,      0.35618992,      0.76019737 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00025295012,  -8.1449781e-05,   0.00081888595,    0.0043388544 ],
  [               0,               0,               0,               0,               0,               0,      0.21541027,     -0.17848347,      -0.8025176,       3.4218895 ],
  [               0,               0,               0,               0,               0,               0,     -0.20888063,      0.46816032,       0.9315614,      -3.3723931 ],
  [               0,               0,               0,               0,               0,               0,      0.40670747,     -0.78832676,      -1.8529087,        2.423635 ],
  [               0,               0,               0,               0,               0,               0,     -0.21219459,      0.46287863,       1.0601898,      -2.2381544 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,   0.00024737721,   0.00032043447,   0.00016530141,  -0.00043484592 ],
  [               0,               0,               0,               0,               0,               0,      -2.5858661,      -1.9188065,       4.2854376,       33.773931 ],
  [               0,               0,               0,               0,               0,               0,      -2.3576871,      -1.8463474,       3.9732183,       27.630505 ],
  [               0,               0,               0,               0,               0,               0,       -1.923814,      -1.4481084,       3.5021814,       22.739215 ],
  [               0,               0,               0,               0,               0,               0,      0.15315273,      0.15348896,      -0.1060496,      -1.6041695 ],
  [               0,               0,               0,               0,               0,               0,  -5.3819989e-05,  -1.5532045e-05,   0.00021323221,    0.0020629562 ],
  [               0,               0,               0,               0,               0,               0,       0.2734207,      0.32077385,      -0.6951861,      -2.9291716 ],
  [               0,               0,               0,               0,               0,               0,    -0.040963197,     -0.14313225,     -0.44114675,      -1.2707919 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   0.00025519354,    0.0022324471 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.9775614,       7.9194439 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.2170492,      -4.1549383 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -1.4848197,       2.3648239 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.99905613,       2.4219058 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   1.3686523e-05,   0.00065011104 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -6.0568895,       18.162579 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.037373552,     -0.49115854 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.26641992,      -1.6373951 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.26646041,     -0.93888826 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,      0.01047924,    0.0025463986,     -0.05948326 ],
  [               0,               0,               0,               0,               0,               0,               0,       -16.97131,       1.1949839,       104.88437 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.051657802,     -0.20958034,       -1.396167 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.47846317,    0.0095585095,      -2.7572483 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0051627951,     0.005057662,   -0.0020093442 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.10250553,     0.048898407,       15.620033 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.010585719,    0.0064165889,       -11.27489 ],
  [               0,               0,               0,               0,               0,               0,               0,     0.058036289,     -0.18416597,     -0.26761491 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.13354071,    -0.038462643,        2.074056 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer2_etaoffv5_37_b = [
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0063809689,    0.0055234356 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       -3.647731,       6.2148767 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     0.055709077,       -1.341061 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.088270709,    -0.037302624 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0013464984,   0.00079375342 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.3841946,       1.2028901 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.4865985,        -1.78642 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0029775994,    0.0057253095 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.5073734,       6.4919695 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.14342264,       -1.243511 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.31391726,     -0.24453467 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,   -0.0014798246,    0.0013299136 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    -0.063480653,      0.45716901 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -2.6707816,       -12.03035 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0066691414,    0.0036949349 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.0137655,       3.3620783 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       8.9254047,      -8.4104688 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      -6.5547143,        4.761706 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0023661993,    0.0018703525 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.33983442,     -0.24464123 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.87498653,      0.67826508 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,  -0.00097462375,   0.00057441289,    0.0029496477,    0.0064638686 ],
  [               0,               0,               0,               0,               0,               0,       1.1038441,      -1.4612754,      -1.5635065,       5.8928241 ],
  [               0,               0,               0,               0,               0,               0,     0.005886808,     -0.23216981,      0.45626613,      -1.5429976 ],
  [               0,               0,               0,               0,               0,               0,    -0.014993335,     0.036586199,     0.058496499,    -0.088308861 ],
  [               0,               0,               0,               0,               0,               0,  -5.4660262e-06,  -0.00010911547,  -0.00027216618,   0.00046230047 ],
  [               0,               0,               0,               0,               0,               0,     -0.58202305,      -0.3388353,    0.0021556309,       2.3977001 ],
  [               0,               0,               0,               0,               0,               0,      -1.7602833,      -2.1705679,      -3.0952998,      -5.2289868 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0028664619,      0.00408976 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.64373027,       3.4307119 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.99818839,      -2.7705785 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.0264107,      -5.4284964 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,       0.1139984,    -0.037398425,     -0.06972348,     -0.01331052,     0.061941691,    -0.027967155,    0.0013378976 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,    0.0040241715,    0.0053168104,    0.0042592852 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0011280335,   -0.0038205227,       20.712783 ],
  [               0,               0,               0,               0,               0,               0,               0,    0.0039839811,   -0.0020935165,      -1.4243466 ],
  [               0,               0,               0,               0,               0,               0,               0,    -0.014762889,     0.001296359,     -0.21367876 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]
gam1000_layer2_etaoffv5_37_e = [
 [
  [               0,               0,               0,       -3.954863,       11.899446,      -13.313813,        6.731335,      -1.4427924,     0.082345755,   0.00013725534 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,    -0.028062923,    -0.013433321,     0.023124077,      0.10052097 ],
  [               0,               0,               0,               0,               0,               0,       2.8122432,       1.3017733,      -2.2446731,      -5.3967589 ],
  [               0,               0,               0,               0,               0,               0,      0.46629093,      0.29092392,     -0.42036515,      -2.6638281 ],
  [               0,               0,               0,               0,               0,               0,      -1.0602548,     -0.50119644,      0.84377755,       3.3032497 ],
  [               0,               0,               0,               0,               0,               0,    -0.026209048,    -0.013421227,     0.019114582,     0.089126592 ],
  [               0,               0,               0,               0,               0,               0,       5.3385371,       3.2398318,      -3.6678694,      -16.790792 ],
  [               0,               0,               0,               0,               0,               0,      -33.439172,      -16.874767,       23.412938,        107.2847 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,   9.2326389e-05,    0.0078582486 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.6032645,       1.5708691 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       3.5827816,      -6.7874836 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,     -0.38494498,      0.71994282 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,   -0.0011271039,  -0.00020057602,     0.010845713 ],
  [               0,               0,               0,               0,               0,               0,               0,        2.957189,       -10.29555,       13.403936 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.48234531,        -1.75402,      0.14205735 ],
  [               0,               0,               0,               0,               0,               0,               0,     -0.42267595,       1.5016693,      -1.2027969 ],
  [               0,               0,               0,               0,               0,               0,               0,   0.00066883479,  -0.00068584543,    -3.07134e-05 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.88295458,      -2.5407649,       2.5764952 ],
  [               0,               0,               0,               0,               0,               0,               0,      0.37584355,     -0.44073257,      -2.3133278 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,   0.00079237235,   -0.0004265723,   -0.0028038957,   -0.0031861992,     0.015847317 ],
  [               0,               0,               0,               0,               0,     0.087802258,     -0.22650009,     -0.81521034,     -0.75623924,       9.0373387 ],
  [               0,               0,               0,               0,               0,    -0.031999539,      0.10031813,      0.18403436,     -0.15730976,       -2.146532 ],
  [               0,               0,               0,               0,               0,     -0.02433295,     0.021574897,     0.053305464,     0.052270305,     0.042247715 ],
  [               0,               0,               0,               0,               0,  -0.00018881286,   0.00013493947,   0.00069342356,   0.00081954841,   -0.0016298379 ],
  [               0,               0,               0,               0,               0,      0.21375619,     -0.21670506,      -1.0656749,      -1.4064931,       6.1316905 ],
  [               0,               0,               0,               0,               0,      0.34376941,      -0.1655315,      -1.5277668,      -2.6555566,       5.1594125 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,  -7.1201249e-06,     3.12219e-05,   0.00020163961,   0.00081248631,    0.0032934324 ],
  [               0,               0,               0,               0,               0,  -0.00041218324,    -0.050032009,     -0.12894331,    -0.072692614,       5.0010837 ],
  [               0,               0,               0,               0,               0,    -0.011040559,     0.033174908,     0.096639561,    -0.041820735,       -1.934275 ],
  [               0,               0,               0,               0,               0,    -0.015685185,     0.014096223,     0.086939909,      0.16283178,     -0.43853488 ],
  [               0,               0,               0,               0,               0,  -2.5581463e-05,   -3.310002e-05,  -2.1331274e-06,   0.00020208184,    0.0020317712 ],
  [               0,               0,               0,               0,               0,    0.0055173241,    -0.023810717,    -0.061583026,     -0.14275545,        1.705786 ],
  [               0,               0,               0,               0,               0,    0.0091187106,    0.0049429482,     -0.04712392,     -0.29414806,      -1.2249842 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,               0,               0,               0,    0.0010392265,    0.0084925691 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       1.0247788,       0.1894423 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,       4.8746257,       -13.28483 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,      0.14478557,     -0.26655617 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
 [
  [               0,               0,               0,               0,               0,       4.0979278,      -3.0913851,      0.86662244,     -0.10672495,    0.0035722258 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
  [               0,               0,               0,               0,               0,               0,               0,               0,               0,               0 ],
 ],
]


###############################################################################
# Now, organize these by type/size/barrel-endcap.
#


ele_layer1_etaoffv5_55_b = [
    ele5_layer1_etaoffv5_55_b,
    ele10_layer1_etaoffv5_55_b,
    ele25_layer1_etaoffv5_55_b,
    ele50_layer1_etaoffv5_55_b,
    ele75_layer1_etaoffv5_55_b,
    ele100_layer1_etaoffv5_55_b,
    ele200_layer1_etaoffv5_55_b,
    ele500_layer1_etaoffv5_55_b,
    ele1000_layer1_etaoffv5_55_b,
    ]

ele_layer1_etaoffv5_55_e = [
    ele5_layer1_etaoffv5_55_e,
    ele10_layer1_etaoffv5_55_e,
    ele25_layer1_etaoffv5_55_e,
    ele50_layer1_etaoffv5_55_e,
    ele75_layer1_etaoffv5_55_e,
    ele100_layer1_etaoffv5_55_e,
    ele200_layer1_etaoffv5_55_e,
    ele500_layer1_etaoffv5_55_e,
    ele1000_layer1_etaoffv5_55_e,
    ]



gam_layer1_etaoffv5_55_b = [
    gam5_layer1_etaoffv5_55_b,
    gam10_layer1_etaoffv5_55_b,
    gam25_layer1_etaoffv5_55_b,
    gam50_layer1_etaoffv5_55_b,
    gam75_layer1_etaoffv5_55_b,
    gam100_layer1_etaoffv5_55_b,
    gam200_layer1_etaoffv5_55_b,
    gam500_layer1_etaoffv5_55_b,
    gam1000_layer1_etaoffv5_55_b,
    ]

gam_layer1_etaoffv5_55_e = [
    gam5_layer1_etaoffv5_55_e,
    gam10_layer1_etaoffv5_55_e,
    gam25_layer1_etaoffv5_55_e,
    gam50_layer1_etaoffv5_55_e,
    gam75_layer1_etaoffv5_55_e,
    gam100_layer1_etaoffv5_55_e,
    gam200_layer1_etaoffv5_55_e,
    gam500_layer1_etaoffv5_55_e,
    gam1000_layer1_etaoffv5_55_e,
    ]



ele_layer1_etaoffv5_35_b = [
    ele5_layer1_etaoffv5_35_b,
    ele10_layer1_etaoffv5_35_b,
    ele25_layer1_etaoffv5_35_b,
    ele50_layer1_etaoffv5_35_b,
    ele75_layer1_etaoffv5_35_b,
    ele100_layer1_etaoffv5_35_b,
    ele200_layer1_etaoffv5_35_b,
    ele500_layer1_etaoffv5_35_b,
    ele1000_layer1_etaoffv5_35_b,
    ]

ele_layer1_etaoffv5_35_e = [
    ele5_layer1_etaoffv5_35_e,
    ele10_layer1_etaoffv5_35_e,
    ele25_layer1_etaoffv5_35_e,
    ele50_layer1_etaoffv5_35_e,
    ele75_layer1_etaoffv5_35_e,
    ele100_layer1_etaoffv5_35_e,
    ele200_layer1_etaoffv5_35_e,
    ele500_layer1_etaoffv5_35_e,
    ele1000_layer1_etaoffv5_35_e,
    ]



gam_layer1_etaoffv5_35_b = [
    gam5_layer1_etaoffv5_35_b,
    gam10_layer1_etaoffv5_35_b,
    gam25_layer1_etaoffv5_35_b,
    gam50_layer1_etaoffv5_35_b,
    gam75_layer1_etaoffv5_35_b,
    gam100_layer1_etaoffv5_35_b,
    gam200_layer1_etaoffv5_35_b,
    gam500_layer1_etaoffv5_35_b,
    gam1000_layer1_etaoffv5_35_b,
    ]

gam_layer1_etaoffv5_35_e = [
    gam5_layer1_etaoffv5_35_e,
    gam10_layer1_etaoffv5_35_e,
    gam25_layer1_etaoffv5_35_e,
    gam50_layer1_etaoffv5_35_e,
    gam75_layer1_etaoffv5_35_e,
    gam100_layer1_etaoffv5_35_e,
    gam200_layer1_etaoffv5_35_e,
    gam500_layer1_etaoffv5_35_e,
    gam1000_layer1_etaoffv5_35_e,
    ]



ele_layer1_etaoffv5_37_b = [
    ele5_layer1_etaoffv5_37_b,
    ele10_layer1_etaoffv5_37_b,
    ele25_layer1_etaoffv5_37_b,
    ele50_layer1_etaoffv5_37_b,
    ele75_layer1_etaoffv5_37_b,
    ele100_layer1_etaoffv5_37_b,
    ele200_layer1_etaoffv5_37_b,
    ele500_layer1_etaoffv5_37_b,
    ele1000_layer1_etaoffv5_37_b,
    ]

ele_layer1_etaoffv5_37_e = [
    ele5_layer1_etaoffv5_37_e,
    ele10_layer1_etaoffv5_37_e,
    ele25_layer1_etaoffv5_37_e,
    ele50_layer1_etaoffv5_37_e,
    ele75_layer1_etaoffv5_37_e,
    ele100_layer1_etaoffv5_37_e,
    ele200_layer1_etaoffv5_37_e,
    ele500_layer1_etaoffv5_37_e,
    ele1000_layer1_etaoffv5_37_e,
    ]



gam_layer1_etaoffv5_37_b = [
    gam5_layer1_etaoffv5_37_b,
    gam10_layer1_etaoffv5_37_b,
    gam25_layer1_etaoffv5_37_b,
    gam50_layer1_etaoffv5_37_b,
    gam75_layer1_etaoffv5_37_b,
    gam100_layer1_etaoffv5_37_b,
    gam200_layer1_etaoffv5_37_b,
    gam500_layer1_etaoffv5_37_b,
    gam1000_layer1_etaoffv5_37_b,
    ]

gam_layer1_etaoffv5_37_e = [
    gam5_layer1_etaoffv5_37_e,
    gam10_layer1_etaoffv5_37_e,
    gam25_layer1_etaoffv5_37_e,
    gam50_layer1_etaoffv5_37_e,
    gam75_layer1_etaoffv5_37_e,
    gam100_layer1_etaoffv5_37_e,
    gam200_layer1_etaoffv5_37_e,
    gam500_layer1_etaoffv5_37_e,
    gam1000_layer1_etaoffv5_37_e,
    ]



ele_layer2_etaoffv5_55_b = [
    ele5_layer2_etaoffv5_55_b,
    ele10_layer2_etaoffv5_55_b,
    ele25_layer2_etaoffv5_55_b,
    ele50_layer2_etaoffv5_55_b,
    ele75_layer2_etaoffv5_55_b,
    ele100_layer2_etaoffv5_55_b,
    ele200_layer2_etaoffv5_55_b,
    ele500_layer2_etaoffv5_55_b,
    ele1000_layer2_etaoffv5_55_b,
    ]

ele_layer2_etaoffv5_55_e = [
    ele5_layer2_etaoffv5_55_e,
    ele10_layer2_etaoffv5_55_e,
    ele25_layer2_etaoffv5_55_e,
    ele50_layer2_etaoffv5_55_e,
    ele75_layer2_etaoffv5_55_e,
    ele100_layer2_etaoffv5_55_e,
    ele200_layer2_etaoffv5_55_e,
    ele500_layer2_etaoffv5_55_e,
    ele1000_layer2_etaoffv5_55_e,
    ]


gam_layer2_etaoffv5_55_b = [
    gam5_layer2_etaoffv5_55_b,
    gam10_layer2_etaoffv5_55_b,
    gam25_layer2_etaoffv5_55_b,
    gam50_layer2_etaoffv5_55_b,
    gam75_layer2_etaoffv5_55_b,
    gam100_layer2_etaoffv5_55_b,
    gam200_layer2_etaoffv5_55_b,
    gam500_layer2_etaoffv5_55_b,
    gam1000_layer2_etaoffv5_55_b,
    ]

gam_layer2_etaoffv5_55_e = [
    gam5_layer2_etaoffv5_55_e,
    gam10_layer2_etaoffv5_55_e,
    gam25_layer2_etaoffv5_55_e,
    gam50_layer2_etaoffv5_55_e,
    gam75_layer2_etaoffv5_55_e,
    gam100_layer2_etaoffv5_55_e,
    gam200_layer2_etaoffv5_55_e,
    gam500_layer2_etaoffv5_55_e,
    gam1000_layer2_etaoffv5_55_e,
    ]


ele_layer2_etaoffv5_35_b = [
    ele5_layer2_etaoffv5_35_b,
    ele10_layer2_etaoffv5_35_b,
    ele25_layer2_etaoffv5_35_b,
    ele50_layer2_etaoffv5_35_b,
    ele75_layer2_etaoffv5_35_b,
    ele100_layer2_etaoffv5_35_b,
    ele200_layer2_etaoffv5_35_b,
    ele500_layer2_etaoffv5_35_b,
    ele1000_layer2_etaoffv5_35_b,
    ]

ele_layer2_etaoffv5_35_e = [
    ele5_layer2_etaoffv5_35_e,
    ele10_layer2_etaoffv5_35_e,
    ele25_layer2_etaoffv5_35_e,
    ele50_layer2_etaoffv5_35_e,
    ele75_layer2_etaoffv5_35_e,
    ele100_layer2_etaoffv5_35_e,
    ele200_layer2_etaoffv5_35_e,
    ele500_layer2_etaoffv5_35_e,
    ele1000_layer2_etaoffv5_35_e,
    ]


gam_layer2_etaoffv5_35_b = [
    gam5_layer2_etaoffv5_35_b,
    gam10_layer2_etaoffv5_35_b,
    gam25_layer2_etaoffv5_35_b,
    gam50_layer2_etaoffv5_35_b,
    gam75_layer2_etaoffv5_35_b,
    gam100_layer2_etaoffv5_35_b,
    gam200_layer2_etaoffv5_35_b,
    gam500_layer2_etaoffv5_35_b,
    gam1000_layer2_etaoffv5_35_b,
    ]

gam_layer2_etaoffv5_35_e = [
    gam5_layer2_etaoffv5_35_e,
    gam10_layer2_etaoffv5_35_e,
    gam25_layer2_etaoffv5_35_e,
    gam50_layer2_etaoffv5_35_e,
    gam75_layer2_etaoffv5_35_e,
    gam100_layer2_etaoffv5_35_e,
    gam200_layer2_etaoffv5_35_e,
    gam500_layer2_etaoffv5_35_e,
    gam1000_layer2_etaoffv5_35_e,
    ]


ele_layer2_etaoffv5_37_b = [
    ele5_layer2_etaoffv5_37_b,
    ele10_layer2_etaoffv5_37_b,
    ele25_layer2_etaoffv5_37_b,
    ele50_layer2_etaoffv5_37_b,
    ele75_layer2_etaoffv5_37_b,
    ele100_layer2_etaoffv5_37_b,
    ele200_layer2_etaoffv5_37_b,
    ele500_layer2_etaoffv5_37_b,
    ele1000_layer2_etaoffv5_37_b,
    ]

ele_layer2_etaoffv5_37_e = [
    ele5_layer2_etaoffv5_37_e,
    ele10_layer2_etaoffv5_37_e,
    ele25_layer2_etaoffv5_37_e,
    ele50_layer2_etaoffv5_37_e,
    ele75_layer2_etaoffv5_37_e,
    ele100_layer2_etaoffv5_37_e,
    ele200_layer2_etaoffv5_37_e,
    ele500_layer2_etaoffv5_37_e,
    ele1000_layer2_etaoffv5_37_e,
    ]


gam_layer2_etaoffv5_37_b = [
    gam5_layer2_etaoffv5_37_b,
    gam10_layer2_etaoffv5_37_b,
    gam25_layer2_etaoffv5_37_b,
    gam50_layer2_etaoffv5_37_b,
    gam75_layer2_etaoffv5_37_b,
    gam100_layer2_etaoffv5_37_b,
    gam200_layer2_etaoffv5_37_b,
    gam500_layer2_etaoffv5_37_b,
    gam1000_layer2_etaoffv5_37_b,
    ]

gam_layer2_etaoffv5_37_e = [
    gam5_layer2_etaoffv5_37_e,
    gam10_layer2_etaoffv5_37_e,
    gam25_layer2_etaoffv5_37_e,
    gam50_layer2_etaoffv5_37_e,
    gam75_layer2_etaoffv5_37_e,
    gam100_layer2_etaoffv5_37_e,
    gam200_layer2_etaoffv5_37_e,
    gam500_layer2_etaoffv5_37_e,
    gam1000_layer2_etaoffv5_37_e,
    ]


###############################################################################
# Region tables.
#

### v5 tables
layer1_etaoffv5_regions_b = [
  [ 0.0, 0.00625, -1, 0.003125, 0 ],
  [ 0.00625, 0.79375, -1, 0.003125, 0 ],
  [ 0.79375, 0.8, -1, 0.003125, 0 ],
  [ 0.8, 1.396875, -1, 0.003125, 0 ],
  [ 1.396875, 1.4125, -1, 0.025, 0 ],
  [ 1.4125, 1.425, -1, 0.025, 0 ],
  [ 1.425, 1.45, -1, 0.025, 0 ],
]
layer1_etaoffv5_regions_e = [
  [ 1.425, 1.454, -1, 0.025, 0 ],
  [ 1.454, 1.475, -1, 0.025, 0 ],
  [ 1.475, 1.503125, -1, 0.025, 0 ],
  [ 1.503125, 1.8, -1, 0.003125, 0 ],
  [ 1.8, 2.0, -1, 0.00416666666667, 0 ],
  [ 2.0, 2.29375, -1, 0.00625, 0 ],
  [ 2.29375, 2.30625, -1, 0.00625, 0 ],
  [ 2.30625, 2.39375, -1, 0.00625, 0 ],
  [ 2.39375, 2.4, -1, 0.00625, 0 ],
  [ 2.4, 2.475, -1, 0.025, 0 ],
  [ 2.475, 2.5, -1, 0.025, 0 ],
]
layer1_etaoffv5_regions_forms_b_ele55 = [
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 0, 0, 0, 0, 0, 0, 0, 11, 0 ],
  [ 3, 3, 3, 3, 3, 10, 3, 3, 3 ],
  [ 11, 11, 11, 11, 11, 11, 11, 11, 11 ],
  [ 13, 13, 13, 13, 13, 13, 11, 11, 11 ],
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
]
layer1_etaoffv5_regions_forms_e_ele55 = [
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 0, 0, 0, 0, 0, 11, 0, 0, 0 ],
  [ 3, 3, 3, 3, 3, 3, 3, 3, 3 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
  [ 11, 11, 11, 11, 11, 11, 11, 11, 11 ],
  [ 11, 11, 0, 11, 11, 11, 11, 0, 0 ],
  [ 11, 11, 11, 11, 11, 11, 11, 10, 11 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
  [ 0, 0, 13, 0, 11, 0, 0, 0, 0 ],
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
]
layer1_etaoffv5_regions_forms_b_ele35 = [
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
  [ 3, 3, 3, 3, 3, 3, 3, 3, 3 ],
  [ 11, 11, 11, 11, 11, 11, 11, 11, 11 ],
  [ 13, 13, 13, 13, 13, 13, 11, 11, 11 ],
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
]
layer1_etaoffv5_regions_forms_e_ele35 = [
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
  [ 3, 3, 3, 3, 3, 3, 3, 3, 3 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
  [ 11, 11, 11, 11, 11, 11, 11, 11, 11 ],
  [ 11, 11, 0, 11, 11, 11, 11, 0, 0 ],
  [ 11, 11, 11, 11, 11, 11, 11, 11, 11 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
  [ 0, 0, 13, 0, 11, 11, 0, 0, 0 ],
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
]
layer1_etaoffv5_regions_forms_b_ele37 = [
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
  [ 3, 3, 3, 3, 3, 3, 3, 3, 3 ],
  [ 11, 11, 11, 11, 11, 11, 11, 11, 11 ],
  [ 13, 13, 13, 13, 13, 11, 11, 11, 11 ],
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
]
layer1_etaoffv5_regions_forms_e_ele37 = [
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
  [ 3, 3, 3, 3, 3, 3, 3, 3, 3 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
  [ 11, 11, 11, 11, 11, 11, 11, 11, 11 ],
  [ 11, 11, 0, 11, 11, 11, 11, 0, 0 ],
  [ 11, 11, 11, 11, 11, 11, 11, 11, 11 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
  [ 0, 0, 13, 0, 11, 11, 0, 0, 0 ],
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
]
layer1_etaoffv5_regions_forms_b_gam55 = [
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
  [ 3, 3, 3, 3, 3, 3, 10, 3, 3 ],
  [ 11, 11, 11, 11, 11, 11, 11, 11, 11 ],
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
]
layer1_etaoffv5_regions_forms_e_gam55 = [
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
  [ 3, 3, 3, 3, 3, 3, 3, 3, 3 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
  [ 11, 11, 11, 11, 11, 11, 11, 11, 11 ],
  [ 11, 11, 0, 11, 11, 11, 11, 0, 0 ],
  [ 11, 11, 11, 11, 11, 11, 11, 11, 11 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
  [ 0, 0, 13, 0, 11, 0, 0, 0, 0 ],
  [ 10, 10, 3, 10, 10, 10, 3, 3, 3 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
]
layer1_etaoffv5_regions_forms_b_gam35 = [
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
  [ 3, 3, 3, 3, 3, 3, 3, 3, 3 ],
  [ 11, 11, 11, 11, 11, 11, 11, 11, 11 ],
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
]
layer1_etaoffv5_regions_forms_e_gam35 = [
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
  [ 3, 3, 3, 3, 3, 3, 3, 3, 3 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
  [ 11, 11, 11, 11, 11, 11, 11, 11, 11 ],
  [ 11, 11, 0, 11, 11, 11, 11, 0, 0 ],
  [ 11, 11, 11, 11, 11, 11, 11, 11, 11 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
  [ 0, 0, 13, 0, 11, 0, 0, 0, 0 ],
  [ 10, 10, 3, 10, 10, 10, 3, 3, 3 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
]
layer1_etaoffv5_regions_forms_b_gam37 = [
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
  [ 3, 3, 3, 3, 3, 3, 3, 3, 3 ],
  [ 11, 11, 11, 11, 11, 11, 11, 11, 11 ],
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
]
layer1_etaoffv5_regions_forms_e_gam37 = [
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
  [ 3, 3, 3, 3, 3, 3, 3, 3, 3 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
  [ 11, 11, 11, 11, 11, 11, 11, 11, 11 ],
  [ 11, 11, 0, 11, 11, 11, 11, 0, 0 ],
  [ 11, 11, 11, 11, 11, 11, 11, 11, 11 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
  [ 0, 0, 13, 0, 11, 0, 0, 0, 0 ],
  [ 10, 10, 3, 10, 10, 10, 3, 3, 3 ],
  [ 0, 0, 0, 0, 0, 0, 0, 0, 0 ],
]
layer2_etaoffv5_regions_b = [
  [ 0.0, 0.4, -1, 0.025, 0 ],
  [ 0.4, 0.775, -1, 0.025, 0 ],
  [ 0.775, 0.825, -1, 0.025, 0 ],
  [ 0.825, 1.35, -1, 0.025, 0 ],
  [ 1.35, 1.375, -1, 0.025, 0 ],
  [ 1.375, 1.3875, -1, 0.025, 0 ],
  [ 1.4, 1.437, -1, 0.075, -0.025 ],
]
layer2_etaoffv5_regions_e = [
  [ 1.425, 1.47, -1, 0.025, 0 ],
  [ 1.47, 1.5, -1, 0.025, 0 ],
  [ 1.5, 1.525, -1, 0.025, 0 ],
  [ 1.525, 1.9, -1, 0.025, 0 ],
  [ 1.9, 2.3, -1, 0.025, 0 ],
  [ 2.3, 2.45, -1, 0.025, 0 ],
  [ 2.45, 2.475, -1, 0.025, 0 ],
  [ 2.475, 2.5, -1, 0.025, 0 ],
]
layer2_etaoffv5_regions_forms_b_ele55 = [
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 3, 3, 3, 3, 3, 3, 3, 3, 3 ],
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 10, 3, 10, 3, 3, 3, 3, 10, 3 ],
]
layer2_etaoffv5_regions_forms_e_ele55 = [
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 3, 3, 3, 10, 3, 3, 3, 3, 10 ],
  [ 3, 3, 3, 3, 3, 3, 3, 10, 3 ],
  [ 10, 3, 10, 10, 10, 10, 10, 10, 10 ],
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 3, 3, 3, 3, 3, 3, 3, 3, 3 ],
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
]
layer2_etaoffv5_regions_forms_b_ele35 = [
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 10, 10, 10, 10, 10, 10, 10, 11, 10 ],
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 3, 3, 3, 3, 3, 3, 10, 10, 3 ],
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 10, 3, 10, 3, 3, 3, 3, 3, 3 ],
]
layer2_etaoffv5_regions_forms_e_ele35 = [
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 3, 3, 3, 10, 3, 3, 3, 3, 10 ],
  [ 3, 3, 3, 3, 3, 3, 3, 10, 3 ],
  [ 10, 3, 10, 10, 10, 10, 10, 10, 10 ],
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 3, 3, 3, 3, 3, 3, 3, 3, 3 ],
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
]
layer2_etaoffv5_regions_forms_b_ele37 = [
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 10, 10, 10, 10, 10, 10, 10, 11, 10 ],
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 3, 3, 3, 3, 3, 3, 10, 10, 10 ],
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 10, 3, 10, 3, 3, 3, 3, 3, 3 ],
]
layer2_etaoffv5_regions_forms_e_ele37 = [
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 3, 3, 3, 10, 3, 3, 3, 3, 10 ],
  [ 3, 3, 3, 3, 3, 3, 3, 10, 3 ],
  [ 10, 3, 10, 10, 10, 10, 10, 10, 10 ],
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 3, 3, 3, 3, 3, 3, 3, 3, 3 ],
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
]
layer2_etaoffv5_regions_forms_b_gam55 = [
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 10, 10, 11, 10, 10, 11, 11, 10, 10 ],
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 3, 3, 3, 3, 3, 3, 3, 3, 3 ],
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 10, 10, 13, 3, 3, 3, 3, 3, 3 ],
]
layer2_etaoffv5_regions_forms_e_gam55 = [
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 3, 3, 3, 10, 3, 3, 3, 3, 10 ],
  [ 3, 3, 3, 3, 3, 3, 3, 10, 3 ],
  [ 10, 3, 10, 10, 10, 10, 10, 10, 10 ],
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 3, 3, 3, 3, 3, 3, 3, 3, 3 ],
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
]
layer2_etaoffv5_regions_forms_b_gam35 = [
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 10, 10, 11, 10, 10, 11, 11, 10, 10 ],
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 3, 3, 3, 3, 3, 3, 10, 10, 3 ],
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 10, 10, 13, 3, 3, 3, 3, 3, 3 ],
]
layer2_etaoffv5_regions_forms_e_gam35 = [
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 3, 3, 3, 10, 3, 3, 3, 3, 10 ],
  [ 3, 3, 3, 3, 3, 3, 3, 10, 3 ],
  [ 10, 3, 10, 10, 10, 10, 10, 10, 10 ],
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 3, 3, 3, 3, 3, 3, 3, 3, 3 ],
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
]
layer2_etaoffv5_regions_forms_b_gam37 = [
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 10, 10, 11, 10, 10, 11, 11, 10, 10 ],
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 3, 3, 3, 3, 3, 3, 10, 10, 3 ],
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 10, 10, 13, 3, 3, 3, 3, 3, 3 ],
]
layer2_etaoffv5_regions_forms_e_gam37 = [
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
  [ 3, 3, 3, 10, 3, 3, 3, 3, 10 ],
  [ 3, 3, 3, 3, 3, 3, 3, 10, 3 ],
  [ 10, 3, 10, 10, 10, 10, 10, 10, 10 ],
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 10, 10, 10, 10, 10, 10, 10, 10, 10 ],
  [ 3, 3, 3, 3, 3, 3, 3, 3, 3 ],
  [ 13, 13, 13, 13, 13, 13, 13, 13, 13 ],
]



### old v5=v41
#layer1_etaoffv5_regions_b = [
#  [ 0.0, 0.00625, 5, 0.003125, 0 ],
#  [ 0.00625, 0.4, 11, 0.003125, 0 ],
#  [ 0.4, 0.79375, 11, 0.003125, 0 ],
#  [ 0.79365, 0.8, 3, 0.003125, 0 ],
#  [ 0.8, 1.39375, 11, 0.003125, 0 ],
#  [ 1.39375, 1.3955, 3, 0.003125, 0 ],
#  [ 1.3955, 1.4, 0, 0.025, 0 ],
#  [ 1.4, 1.45, 0, 0.025, 0 ],
#]
#layer1_etaoffv5_regions_e = [
#  [ 1.425, 1.454, 3, 0.025, 0 ],
#  [ 1.454, 1.475, 0, 0.025, 0 ],
#  [ 1.475, 1.5, 3, 0.025, 0 ],
#  [ 1.5, 1.503125, 0, 0.003125, 0 ],
#  [ 1.503125, 1.8, 0, 0.003125, 0 ],
#  [ 1.8, 2.0, 0, 0.00416666666667, 0 ],
#  [ 2.0, 2.29375, 0, 0.00625, 0 ],
#  [ 2.29375, 2.30625, 3, 0.00625, 0 ],
#  [ 2.30625, 2.39375, 0, 0.00625, 0 ],
#  [ 2.39375, 2.4, 0, 0.00625, 0 ],
#  [ 2.4, 2.475, 10, 0.025, 0 ],
#  [ 2.475, 2.5, 0, 0.025, 0 ],
#]
#
#layer2_etaoffv5_regions_b = [
#  [ 0.0, 0.4, 10, 0.025, 0 ],
#  [ 0.4, 0.775, 10, 0.025, 0 ],
#  [ 0.775, 0.825, 10, 0.025, 0 ],
#  [ 0.825, 1.35, 10, 0.025, 0 ],
#  [ 1.35, 1.4, 3, 0.025, 0 ],
#  [ 1.4, 1.437, 3, 0.075, -0.025 ],
#]
#layer2_etaoffv5_regions_e = [
#  [ 1.4, 1.45, 5, 0.025, 0 ],
#  [ 1.45, 1.5, 3, 0.025, 0 ],
#  [ 1.5, 1.525, 3, 0.025, 0 ],
#  [ 1.525, 1.9, 10, 0.025, 0 ],
#  [ 1.9, 2.3, 10, 0.025, 0 ],
#  [ 2.3, 2.45, 10, 0.025, 0 ],
#  [ 2.45, 2.475, 3, 0.025, 0 ],
#  [ 2.475, 2.5, 0, 0.025, 0 ],
#]


ele_etaoffv5_energies = [
    5*GeV, 10*GeV, 25*GeV, 50*GeV, 75*GeV, 100*GeV, 200*GeV, 500*GeV, 1000*GeV
    ]
gam_etaoffv5_energies = [
    5*GeV, 10*GeV, 25*GeV, 50*GeV, 75*GeV, 100*GeV, 200*GeV, 500*GeV, 1000*GeV
    ]
etaoffv5_energies = {'ele55' : ele_etaoffv5_energies,
                       'ele35' : ele_etaoffv5_energies,
                       'ele37' : ele_etaoffv5_energies,
                       'gam55' : gam_etaoffv5_energies,
                       'gam35' : gam_etaoffv5_energies,
                       'gam37' : gam_etaoffv5_energies}

etaoffv5_layer1b_forms = {'ele55' : layer1_etaoffv5_regions_forms_b_ele55,
                          'ele35' : layer1_etaoffv5_regions_forms_b_ele35,
                          'ele37' : layer1_etaoffv5_regions_forms_b_ele37,
                          'gam55' : layer1_etaoffv5_regions_forms_b_gam55,
                          'gam35' : layer1_etaoffv5_regions_forms_b_gam35,
                          'gam37' : layer1_etaoffv5_regions_forms_b_gam37}
etaoffv5_layer1e_forms = {'ele55' : layer1_etaoffv5_regions_forms_e_ele55,
                          'ele35' : layer1_etaoffv5_regions_forms_e_ele35,
                          'ele37' : layer1_etaoffv5_regions_forms_e_ele37,
                          'gam55' : layer1_etaoffv5_regions_forms_e_gam55,
                          'gam35' : layer1_etaoffv5_regions_forms_e_gam35,
                          'gam37' : layer1_etaoffv5_regions_forms_e_gam37}
etaoffv5_layer2b_forms = {'ele55' : layer2_etaoffv5_regions_forms_b_ele55,
                          'ele35' : layer2_etaoffv5_regions_forms_b_ele35,
                          'ele37' : layer2_etaoffv5_regions_forms_b_ele37,
                          'gam55' : layer2_etaoffv5_regions_forms_b_gam55,
                          'gam35' : layer2_etaoffv5_regions_forms_b_gam35,
                          'gam37' : layer2_etaoffv5_regions_forms_b_gam37}
etaoffv5_layer2e_forms = {'ele55' : layer2_etaoffv5_regions_forms_e_ele55,
                          'ele35' : layer2_etaoffv5_regions_forms_e_ele35,
                          'ele37' : layer2_etaoffv5_regions_forms_e_ele37,
                          'gam55' : layer2_etaoffv5_regions_forms_e_gam55,
                          'gam35' : layer2_etaoffv5_regions_forms_e_gam35,
                          'gam37' : layer2_etaoffv5_regions_forms_e_gam37}





class CaloSwEtaoff_v5_b1_parms:
    region = CALOCORR_EMB1
    regions = layer1_etaoffv5_regions_b
    energy_degree = 3
    forms = etaoffv5_layer1b_forms

    correction = {'ele55' : ele_layer1_etaoffv5_55_b,
                  'ele35' : ele_layer1_etaoffv5_35_b,
                  'ele37' : ele_layer1_etaoffv5_37_b,
                  'gam55' : gam_layer1_etaoffv5_55_b,
                  'gam35' : gam_layer1_etaoffv5_35_b,
                  'gam37' : gam_layer1_etaoffv5_37_b,

                  # Use 5x5 for other cluster sizes.
                  'ele33' : ele_layer1_etaoffv5_55_b,
                  'ele57' : ele_layer1_etaoffv5_55_b,
                  'ele77' : ele_layer1_etaoffv5_55_b,
                  'gam33' : gam_layer1_etaoffv5_55_b,
                  'gam57' : gam_layer1_etaoffv5_55_b,
                  'gam77' : gam_layer1_etaoffv5_55_b,
                  }

    energies = etaoffv5_energies


class CaloSwEtaoff_v5_b2_parms:
    region = CALOCORR_EMB2
    regions = layer2_etaoffv5_regions_b
    energy_degree = 3
    forms = etaoffv5_layer2b_forms

    correction = {'ele55' : ele_layer2_etaoffv5_55_b,
                  'ele35' : ele_layer2_etaoffv5_35_b,
                  'ele37' : ele_layer2_etaoffv5_37_b,
                  'gam55' : gam_layer2_etaoffv5_55_b,
                  'gam35' : gam_layer2_etaoffv5_35_b,
                  'gam37' : gam_layer2_etaoffv5_37_b,

                  # Use 5x5 for other cluster sizes.
                  'ele33' : ele_layer2_etaoffv5_55_b,
                  'ele57' : ele_layer2_etaoffv5_55_b,
                  'ele77' : ele_layer2_etaoffv5_55_b,
                  'gam33' : gam_layer2_etaoffv5_55_b,
                  'gam57' : gam_layer2_etaoffv5_55_b,
                  'gam77' : gam_layer2_etaoffv5_55_b,
                  }

    energies = etaoffv5_energies


class CaloSwEtaoff_v5_e1_parms:
    region = CALOCORR_EME1
    regions = layer1_etaoffv5_regions_e
    energy_degree = 3
    forms = etaoffv5_layer1e_forms

    correction = {'ele55' : ele_layer1_etaoffv5_55_e,
                  'ele35' : ele_layer1_etaoffv5_35_e,
                  'ele37' : ele_layer1_etaoffv5_37_e,
                  'gam55' : gam_layer1_etaoffv5_55_e,
                  'gam35' : gam_layer1_etaoffv5_35_e,
                  'gam37' : gam_layer1_etaoffv5_37_e,

                  # Use 5x5 for other cluster sizes.
                  'ele33' : ele_layer1_etaoffv5_55_e,
                  'ele57' : ele_layer1_etaoffv5_55_e,
                  'ele77' : ele_layer1_etaoffv5_55_e,
                  'gam33' : gam_layer1_etaoffv5_55_e,
                  'gam57' : gam_layer1_etaoffv5_55_e,
                  'gam77' : gam_layer1_etaoffv5_55_e,
                  }

    energies = etaoffv5_energies


class CaloSwEtaoff_v5_e2_parms:
    region = CALOCORR_EME2
    regions = layer2_etaoffv5_regions_e
    energy_degree = 3
    forms = etaoffv5_layer2e_forms


    correction = {'ele55' : ele_layer2_etaoffv5_55_e,
                  'ele35' : ele_layer2_etaoffv5_35_e,
                  'ele37' : ele_layer2_etaoffv5_37_e,
                  'gam55' : gam_layer2_etaoffv5_55_e,
                  'gam35' : gam_layer2_etaoffv5_35_e,
                  'gam37' : gam_layer2_etaoffv5_37_e,

                  # Use 5x5 for other cluster sizes.
                  'ele33' : ele_layer2_etaoffv5_55_e,
                  'ele57' : ele_layer2_etaoffv5_55_e,
                  'ele77' : ele_layer2_etaoffv5_55_e,
                  'gam33' : gam_layer2_etaoffv5_55_e,
                  'gam57' : gam_layer2_etaoffv5_55_e,
                  'gam77' : gam_layer2_etaoffv5_55_e,
                  }

    energies = etaoffv5_energies




CaloSwEtaoff_v5_parms = { EMB1 : CaloSwEtaoff_v5_b1_parms,
                            EMB2 : CaloSwEtaoff_v5_b2_parms,
                            EME1 : CaloSwEtaoff_v5_e1_parms,
                            EME2 : CaloSwEtaoff_v5_e2_parms,
                          }



#
# Notes on the derivation of the above tables.
# The following are the descriptions of the fits used.
# The D constructor has the signature:
#
#layer1_regions_v5 = [
#    D(EMB1, 0.025/8, 0.0,     0.00625, 
#      {'ele1000-35'     : 2,
#       'ele25-55'       : 2,
#       'ele500'         : 4,
#       'ele75-37'       : 2,
#       'gam500'         : 4,
#       None             : 3},
#      13, 0.004,
#          outlier_sigma = {'ele1000-35'  : 2.3,
#                           'ele1000-55'  : 2.3,
#                           'ele100-35'  : 1.7,
#                           'ele200-37'  : 2.3,
#                           'gam5-55'    : 5,
#                           None: 2},
#          outlier_window_cellsize_fraction = 
#                    {'ele25-55'    : 1.0/2.0,
#                     None          : 1.0/8.0   },
#          outliersInStats=0,
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10' : 100  }     ),                                                                   # 0
#
#    D(EMB1, 0.025/8, 0.00625, 0.79375,    1, {'ele500-55' : 11,
#                                              None : 0} ,           # 1
#      prange=(0.1, 0.15), psize=0.0001, 
#        outlier_sigma = {'gam1000'    : 7,
#                         'gam500'     : 9,
#                         None         : 5},
#        outlier_window_cellsize_fraction = 
#                    {'gam25-35'    : 1.0/4.0,
#                     'gam25-37'    : 1.0/4.0,
#                     None          : 1.0/8.0   },
#        residual_outlier_sigma = 
#                        {'gam1000'    : 8,
#                         'gam500'     : 8,
#                         None         : 5},
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10' : 100  } ),
#
#    D(EMB1, 0.025/8, 0.79375, 0.8,     
#              {'ele1000-55'      : 3,
#               'ele500-37'       : 2,
#               'ele200-55'       : 2,
#               'gam1000'         : 2,
#               'gam500-37'       : 2,
#               'gam500-55'       : 2,
#               None              : 1},
#              {'ele50-37'        : 3,
#               'ele100-55'       : 10,
#               'gam200-55'       : 10,
#               None              : 3},
#         outlier_sigma = {'gam200'  : 4,
#                          'gam100'  : 3,
#                          'gam500'  : 2.5,
#                          None: 5},
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10' : 100  }),                                    # 2
#
#    D(EMB1, 0.025/8, 0.8,   1.396875, 
#        {'ele200'     : 1, 
#         'gam200-55'   : 1, 
#         'gam1000'     : 1, 
#         'ele100-35'  : 1, 
#         'ele500-37'  : 1, 
#         'ele500-55'  : 1, 
#          None     : 2}, 11,                                               # 3
#      prange=(1.0, 1.05), psize=0.0001,
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10' : 100  }),
#
#
#
#    # note that this (without a phase) takes into account the two cell sizes and their positions - i.e. cellsize=0.025 dealt with for 0.025/8 cell
#    D(EMB1, 0.025,   1.396875, 1.4125,  
#         {'ele100-37'     : 4,
#          'ele200-35'       : 4,
#          'ele200-37'       : 4,
#          'ele200-55'       : 3,
#          'ele500-35'     : 3,
#          'ele500-37'     : 4,
#          'ele500-55'     : 4,
#          'ele1000-35'      : 3,
#          'ele1000-37'      : 4,
#          'ele1000-55'      : 4,
#          'gam1000'        : 6,
#          'gam200'        : 7,
#          'gam500'        : 6,
#          None            : 5}, 
#         {'ele100-37'     : 11,
#          'ele200'        : 11,
#          'ele500'        : 11,
#          'ele1000'       : 11,
#          None            : 13}, psize=0.0005, 
#            outlier_sigma = 2,
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10' : 100  }   ),                         # 4 
#         
#
#
#    D(EMB1, 0.025,   1.4125, 1.425,
#         {'ele1000-55'    : 7,
#          'ele500-55'     : 7,
#          None            : 5}, 13, 0.01, psize=0.0005,
#         outlier_window_cellsize_fraction = 1.0,
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10' : 100  }  ),                  # 5 
#
#    D(EMB1, 0.025,   1.425,  1.45,      
#             {'ele1000-37'     : 2,
#              'gam1000-55'      : 3,
#              'gam1000-35'      : 2,
#              'gam100-37'    : 2,
#              'gam200-55'      : 2,
#              'gam200-35'      : 2,
#              'gam200-37'    : 1,
#              'gam500-55'    : 3,
#              'gam500-35'    : 2,
#              'gam500-37'    : 2,
#              None             : 1},
#             {'gam500-37'     : 0,
#              None             : 0}, 0.01, psize=0.002,
#      outlier_sigma={'gam500-35'    : 5,
#                     'gam500-37'    : 5,
#                     None           : 5   },
#      outlier_window_cellsize_fraction = 
#                    {'gam500-37'    : 1.0/16.0,
#                     'gam500-35'    : 1.0/16.0,
#                     None           : 1.0/8.0   },
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10' : 100  }       ),                   # 6 
#
#
#
#
#    D(EME1, 0.025,   1.425,   1.454,   5, 13, 0.015, psize=0.005,
#      outlier_sigma={# 55
#                     'gam100-55'    : 1.75,
#                     'ele100-55'    : 1.75,
#                     'ele1000-55'   : 3,
#                     'gam1000-55'   : 3,
#                     'ele200-55'   : 3,
#                     'gam200-55'   : 3,
#                     'ele500-55'   : 3,
#                     'gam500-55'   : 3,
#                     'ele50-55'   : 3,
#                     'gam50-55'   : 3,
#                     'ele75-55'   : 3,
#                     'gam75-55'   : 3,
#                     # 35
#                     'ele75-35'   : 1.5,
#                     'gam75-35'   : 1.5,
#                     # 37
#                     'ele75-37'   : 1.5,
#                     'gam75-37'   : 1.5,
#                     #
#                     'ele25'   : 3,
#                     'gam25'   : 3,
#                     None           : 1.5   },
#      outlier_window_cellsize_fraction =
#                    {'gam100-55'    : 1.2,
#                     'ele100-55'    : 1.2,
#                     'gam50-55'     : 1.2,
#                     'ele50-55'     : 1.2,
#                     None           : 2.0   },
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10' : 100  }   ),         # 7
#
#    D(EME1, 0.025,   1.454,   1.475, 1,
#        {'ele100-55'     : 11,
#         None            : 0},  0.01, psize=0.004,
#      outlier_sigma={#'gam100-37'    : 5,
#                     None           : 2.0   },
#      outliersInStats=0,
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10-35' : 1e9,
#                       'gam10-37' : 1e9,
#                       'gam10-55' : 100  } ),                        # 8 
#
#    D(EME1, 0.025,   1.475,   1.503125,1, 3, 0.01, psize=0.004,
#      outlier_sigma={'gam100-37'    : 5,
#                     'gam50-35'        : 2.0,
#                     'gam50-37'        : 2.0,
#                     'gam50-55'        : 2.5,
#                     'ele50'        : 2.0,
#                     'gam75'        : 2.4,
#                     'ele75'        : 2.4,
#                     None           : 5   },
#      outlier_window_cellsize_fraction = 
#                    {'gam100-37'    : 1.0/12.0,
#                     'gam50-35'        : 0.8,
#                     'gam50-37'        : 0.8,
#                     'gam50-55'        : 1.0/2.0,
#                     'ele50'        : 0.8,
#                     'gam75'        : 1.0/2.0,
#                     'ele75'        : 1.0/2.0,
#                     None           : 1.0/8.0   },
#           minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10' : 100  } ),           # 9 
#
#    D(EME1, 0.025/8, 1.503125, 1.8, {'ele10-35' : 2, None : 1},
#                                      0, prange=(1.6, 1.65), psize=0.0002,
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10' : 100  }),# 10
#
#    D(EME1, 0.025/6, 1.8,     2.0,     
#      {'ele1000-55'     : 2,
#        None            : 3}, 11, prange=(1.85, 1.9),           #11
#      psize=0.0002,
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10' : 100  }),
#
#
#    D(EME1, 0.025/4, 2.0,     2.29375,     {'gam75-37'   : 2,
#                                            'gam100-37'   : 2,
#                                            None         : 3   },
#                                           {'ele25' :   0,
#                                           'gam25' :   0,
#                                           'ele500' :  0,
#                                           'gam500' :  0,
#                                           'ele1000' : 0,
#                                           'gam1000' : 0,
#                                           None :      11},
#                                       prange=(2.1, 2.15),
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10-35' : 1e9,
#                       'gam10-37' : 1e9,
#                       'gam10-55' : 1e9  }),                # 12
#
#
#    D(EME1, 0.025/4, 2.29375, 2.30625, {'gam1000-35' : 2,                     # 13
#                                        'gam1000-37' : 3,
#                                        'gam1000-55' : 4,
#                                        'gam500' : 4,
#                                        'gam200' : 2,
#                                        'gam100' : 1,
#                                        'gam100-55' : 2,
#                                        'gam75-35' : 2,
#                                        'gam75-37' : 3,
#                                        'gam75-55' : 2,
#                                        'ele100' : 2,
#                                        'ele200' : 2,
#                                        'ele500' : 2,
#                                        'ele1000' : 2,
#                                        None: 1}, 
#                                       {'ele500-55'   : 10,
#                                        None          : 11}, 0.003, psize=0.001,
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10' : 100  }),
#
#    D(EME1, 0.025/4, 2.30625, 2.39375, {'ele25' : 4,
#                                        'ele50' : 4,
#                                        'gam50' : 4,
#                                        'gam25' : 4,
#                                         None :   1}, 0, psize=0.0002,
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10' : 100  }),    # 14
#
#    D(EME1, 0.025/4, 2.39375, 2.4,     {'ele200-37' : 2,                   # 15
#                                        'ele25'  :    1,
#                                        'gam25'  :    1,
#                                        'ele75'  :    0,
#                                        'gam75'  :    0,
#                                        'ele100'  :   2,
#                                        'gam100'  :   1,
#                                        None: 1},
#                                       {'ele25' :  13,
#                                        'ele75' :  11,
#                                        'gam25' :  13,
#                                        'gam75' :  11,
#                                        'ele100-35' :  11,
#                                        'ele100-37' :  11,
#                                        None :     0},
#              0.004, psize=0.001,
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10' : 100  }),
#
#
#
#
#    D(EME1, 0.025,   2.4,     2.475,   {'gam200' : 2,                      # 16
#                                        'ele200' : 3,
#                                        'ele75' : 3,
#                                        'gam25' : 2,
#                                        'gam75' : 3,
#                                        'ele500-37' : 3,
#                                        'ele500-55' : 3,
#                                        'ele1000-35' : 2,
#                                        'gam1000-35' : 3,
#                                        'gam1000-37' : 2,
#                                        'gam500' : 2,
#                                        None:2},
#                                       {'gam200' : 3,
#                                        'gam1000' : 3,
#                                        'gam500' : 3,
#                                        'gam25' : 3,
#                                        None:10} , 0.008, psize=0.001,
#                    outlier_sigma = {'gam1000-37'    : 3,
#                                     None            : 5},
#                    outliersInStats=0,
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10' : 100  } ),
#
#
#    D(EME1, 0.025,   2.475,   2.5,     {'gam500' : 1,                     # 17 
#                                        'gam200' : 2,
#                                        'gam100-35' : 1,
#                                        'gam100-55' : 2,
#                                        'gam1000' : 2,
#                                        'gam75-55' : 2,
#                                        'gam75-35' : 0,
#                                        'gam75-37' : 0,
#                                        'ele25' : 0,
#                                        'ele50' : 2,
#                                        'ele75' : 2,
#                                        'ele100-35' : 1,
#                                        'ele100-55' : 2,
#                                        'ele200' : 2,
#                                        'ele500' : 2,
#                                        None:1},
#              0, 0.008, outlier_sigma=
#                    {'gam25'     : 3,
#                     'gam50'     : 3,
#                     'gam75'     : 2.5,
#                     None        : 2}, outliersInStats=0, outliersInStats_5sigma=0,
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10' : 100  }),
#    ]
#
#
#
#layer2_regions_v5 = [
#    D(EMB2, 0.025,   0.0,   0.4,       {'ele25'      : 0,                    # 0
#                                        'ele50-55'   : 0,
#                                        'ele50-35'   : 2,
#                                        'ele75'      : 2,
#                                        'gam75'      : 2,
#                                        'ele5'       : 0,
#                                        'ele100-35'  : 2,
#                                        'ele100-37'  : 3,
#                                        'ele200'     : 3,
#                                        'gam200'     : 3,
#                                        'gam25'      : 0,
#                                        'gam10-35'   : 2,
#                                        'gam10-37'   : 2,
#                                        None         : 1},
#      10, psize=0.0003,
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10' : 100  }),
#
#    D(EMB2, 0.025,   0.4,   0.775,     {'ele500-35'   : 3,                      # 1
#                                        'ele500-37'   : 3,
#                                        'ele500-55'   : 3,
#                                        'gam500'   : 3,
#                                        'ele75-35'    : 2,
#                                        'ele75-55'    : 2,
#                                        'ele10-37'    : 2,
#                                        'gam75'    : 2,
#                                        'ele200'   : 3,
#                                        'gam200'   : 2,
#                                        'ele50'       : 2,
#                                        'gam50'       : 2,
#                                        None:1}, 
#                                       {'ele500-35'   : 11,
#                                        'ele500-37'   : 11,
#                                        'gam200'   : 11,
#                                        'gam100'   : 11,
#                                        'gam25'   : 11,
#                                        None          : 10}, psize=0.0003,
#          minevents = {'ele5' : 1e9,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10' : 100  }),
#
#
#    D(EMB2, 0.025,   0.775, 0.825,     {'gam500'     : 2,                    # 2
#                                        'gam200'     : 2,
#                                        'ele50-35'   : 2,
#                                        'ele50-37'   : 3,
#                                        'ele50-55'   : 2,
#                                        'gam50'      : 2,
#                                        'ele75-35'   : 1,
#                                        'ele75-37'   : 2,
#                                        'ele75-55'   : 3,
#                                        'ele100-37'  : 2,
#                                        'ele200'     : 2,
#                                        'ele500'     : 2,
#                                        None: 1}, 10, 0.005, psize=0.001,
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10' : 100  }),
#
#
#    D(EMB2, 0.025,   0.825, 1.35,      {'ele1000-35'  : 1,
#                                        'gam1000-35'   : 1,
#                                        'gam25'   : 2,
#                                        'gam75-55'    : 2,
#                                        'ele500-35'   : 2,
#                                        'ele75-35'    : 2,
#                                        None:3},  10, 0.002, psize=0.0003,
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10' : 100  } ),     # 3
#
#
#    D(EMB2, 0.025,   1.35,  1.375,     1,                                  # 4
#      {'ele1000-37' : 10,
#       'ele200-35' : 10,
#       'ele200-37' : 10,
#       'gam200-35' : 10,
#       'gam200-37' : 10,
#       'ele500-35' : 10,
#       'ele500-37' : 10,
#       'gam500-35' : 10,
#       'gam500-37' : 10,
#       None: 3},
#      0.005, psize=0.0004,
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10' : 100  }),
#
#
#
#
#
#    D(EMB2, 0.025,   1.375,   1.3875, 
#       {'gam1000'    : 6,
#        'gam100'    : 8,
#        'gam500'    : 8,
#        'gam50'    : 8,
#        'gam75'    : 8,
#        None         : 5},13,                                         # 5
#      0.008, psize=0.005,
#      outlier_sigma={'gam1000-55'    : 2.0,
#                     'gam100-55'     : 1.0,
#                     'gam200-55'     : 2.0,
#                     'gam25-55'      : 2.2,
#                     'gam500-55'     : 2.0,
#                     'gam50-55'      : 1.2,
#                     'gam75-55'      : 1.6,
#                     None            : 5},
#      outlier_window_cellsize_fraction = 
#                    {'gam1000-55'   : 1.0/2.0,
#                     'gam100-55'    : 1.0/2.0,
#                     'gam200-55'    : 1.0/2.0,
#                     'gam25-55'     : 1.0/2.0,
#                     'gam500-55'    : 1.0/2.0,
#                     'gam50-55'     : 1.0/2.0,
#                     'gam75-55'     : 1.0/2.0,
#                     None           : 1.0/8.0   },
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10' : 100  }  ),
#
#
#
#
#
#
#    # Do not use 1.3875 < eta < 1.4 since it is fairly empty and causing issues with fitting
#
#    D(EMB2, 0.075,   1.4,   1.437,                                         # 6 
#      {'ele1000-35'   : 2,
#       'ele1000-37'   : 2,
#       'ele1000-55'   : 2,
#       'gam1000'    : 2,
#       'ele100-35'    : 2,
#       'ele100-37'    : 2,
#       'ele100-55'    : 3,
#       'gam100'       : 5,
#       'ele200' : 4,
#       'gam200' : 4,
#       'ele500-35'    : 3,
#       'ele500-37'    : 3,
#       'ele500-55'    : 2,
#       'gam500'       : 3,
#       'ele50'     : 3,
#       'gam50'     : 3,
#       'gam25'     : 5,
#       'ele75-37'  : 2,
#       'ele75-55'  : 3,
#       'gam75'  : 3,
#       None: 1},
#      {'ele1000'       : 3,
#       'ele100'       : 3,
#       'ele10'     : 3,
#       'ele200'       : 3,
#       'ele500-35'       : 3,
#       'ele500-37'       : 3,
#       'ele50'        : 3,
#       'ele75'     : 3,
#       'gam25'     : 13,
#       'gam50'        : 3,
#       'gam75'        : 3,
#       'gam100'       : 3,
#       'gam200'       : 3,
#       'gam500'       : 3,
#       'gam1000'       : 3,
#       None: 10},
#      0.008, psize=0.005, phase=-0.025,
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10' : 100  }),
#
#
#    D(EME2, 0.025,   1.425, 1.47,     {'ele1000' : 9,
#                                       'gam1000' : 6,
#                                       'gam200'  : 6,
#                                       'gam25'  : 6,
#                                       'gam500'  : 6,
#                                        None :   9},                                 # 7
#      13, 0.01, psize=0.005,
#      outlier_sigma={# 55
#                     'ele1000-55'   : 2,
#                     'gam1000-55'   : 2,
#                     'gam100-55'    : 1.5,
#                     'ele100-55'    : 1.5,
#                     'ele200-55'   : 1.75,
#                     'gam200-55'   : 1.75,
#                     'ele25-55'   : 3.5,
#                     'gam25-55'   : 3.5,
#                     'ele500-55'   : 2,
#                     'gam500-55'   : 2,
#                     'ele50-55'   : 2.5,
#                     'gam50-55'   : 2.5,
#                     'ele75-55'   : 2,
#                     'gam75-55'   : 2,
#                     # 35
#                     'ele75-35'   : 1.2,
#                     'ele25-35'   : 3,
#                     'gam25-35'   : 3,
#                     'gam75-35'   : 1.2,
#                     # 37
#                     'ele25-37'   : 3,
#                     'gam25-37'   : 3,
#                     'ele75-37'   : 1.2,
#                     'gam75-37'   : 1.2,
#                     None           : 1.5   },
#      outlier_window_cellsize_fraction = 2,
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10' : 100  } ),
#
#
#
#
#    D(EME2, 0.025,   1.47, 1.5,     
#      {'ele1000' : 3,
#       'gam1000' : 3,
#       'ele50'   : 3,
#       'gam50'   : 3,
#       None      : 3},                                   # 8
#      {'ele1000' : 10,
#       'gam1000' : 10,
#       'ele50'   : 10,
#       'gam50'   : 10,
#       None      : 3},
#      0.01, psize=0.005,
#      outlier_sigma =   {'gam25'       : 3,
#                         'gam25-55'    : 1.8,
#                         'gam50'       : 2.5,
#                         'gam50-55'    : 1.85,
#                         'gam75'       : 1.8,
#                         'gam75-55'    : 1.0,
#                         'gam100'      : 3,
#                         'gam100-37'   : 2.0,
#                         'ele25'       : 3,
#                         'ele25-55'    : 1.8,
#                         'ele50'       : 2.5,
#                         'ele50-55'    : 1.85,
#                         'ele75'       : 1.8,
#                         'ele75-55'    : 1.0,
#                         'ele100'      : 3,
#                         'gam100-37'   : 2.0,
#                         #breaking the pattern
#                         'ele100-55'    : 1.8,
#                         'ele200-37'    : 3.5,
#                         'gam25-35'     : 2.7,
#                         'gam75-35'     : 1.5,
#                         'ele75-35'     : 1.35,
#                         None          : 5},
#      outlier_window_cellsize_fraction = 
#                         {'gam75'    : 1.0/5.0,
#                          'ele75'    : 1.0/5.0,
#                          None       : 1.0/8.0  } ,
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10' : 1e9  } ),
#
#
#
#    D(EME2, 0.025,   1.5,    1.525,   1,
#       {'ele500' : 10,
#        'gam500' : 10,
#        None : 3}, 0.005, psize=0.003,
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10' : 100  }),                         # 9
#
#    D(EME2, 0.025,   1.525,  1.9,                               # 10
#      {'ele1000'   : 2,
#       'gam1000'   : 2,
#       'ele100-37'  : 2,
#       'ele25'     : 2,
#       'gam25'     : 2,
#       'ele50'     : 2,
#       'ele10'     : 2,
#       'gam10'     : 1,
#       None: 3},
#      {'ele10'     : 3,
#       'gam10'     : 3,
#       None        : 10},
#      0.005, psize=0.001,
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10' : 100  }),
#
#    D(EME2, 0.025,   1.9,  2.3,                                            # 11
#      {'ele1000'   : 5,
#       'gam1000'   : 4,
#       'gam1000-55'   : 3,
#       'ele500'    : 3,
#       'ele500-37' : 4,
#       'gam500'    : 4,
#       'ele100-37' : 3,
#       'ele100-55' : 3,
#       'ele50-35'     : 3,
#       'ele50-37'     : 3,
#       'ele50-55'     : 4,
#       'gam50'     : 3,
#       'ele25-37'  : 1,
#       None        : 2},
#      10, psize=0.0003,
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 1e9,
#                       'gam10' : 1e9  }),
#
#    D(EME2, 0.025,   2.3,  2.45,                                           # 12
#      {'ele75-55' : 2,
#       None       : 4},
#      10, psize=0.0003),
#    D(EME2, 0.025,   2.45, 2.475, 
#            {'ele50' : 2,
#             'gam50' : 2,
#             'ele500-55' : 3,
#             None : 1},
#          3, 0.005, psize=0.001,                                          # 13
#      outlier_sigma = {'ele25': 5,
#                       'gam25': 5,
#                       'ele50': 4,
#                       'gam50': 4,
#                       'ele75': 3,
#                       'gam75': 3,
#                       'gam1000': 4,
#                       'gam200': 4,
#                       'gam500': 4,
#                       None: 5}),
#    D(EME2, 0.025,   2.475,2.5,                                            # 14
#      {'ele1000-55'   : 6,
#       'gam100'   : 6,
#       'gam100-55'   : 4,
#       'gam50'   : 6,
#       'gam75'   : 6,
#       None           : 4}, 13,
#      0.005,
#      outlier_sigma = {'gam100-55'   : 2.3,
#                       None          : 2.0},
#      outlier_window_cellsize_fraction = 1.0/8.0,
#      outliersInStats=0,
#      initvals={
#       '35'       : [(0,l2r15_cc_35)],
#       'ele50-35' : [(0,l2r15_cc_35_50)],
#       'gam50-35' : [(0,l2r15_cc_35_50)],
#       'ele75-35' : [(0,l2r15_cc_35_50)],
#       'gam75-35' : [(0,l2r15_cc_35_50)],
#       '37'       : [(0,l2r15_cc_37)],
#       'ele50-35' : [(0,l2r15_cc_37_50)],
#       'gam50-35' : [(0,l2r15_cc_37_50)],
#       'ele75-35' : [(0,l2r15_cc_37_50)],
#       'gam75-35' : [(0,l2r15_cc_37_50)],
#       'ele25-35' : [(0,l2r15_cc_37_50)],
#       'gam25-35' : [(0,l2r15_cc_37_50)],
#        None      : [(0,l2r15_cc_55)],
#        },
#          minevents = {'ele5' : 100,
#                       'gam5' : 100,
#                       'ele10' : 100,
#                       'gam10' : 100  }     ),
#    ]
#


