#!/bin/env python

# Copyright (C) 2002-2017 CERN for the benefit of the ATLAS collaboration
import sys
from PyCool import cool,coral

TYPE="MC"
#TYPE="Data"



if TYPE=="MC":
    dbfile="mycool_MC.db"
    dbname="OFLP200" 
    TagToT = "TRTCalibPID_RToT-Col-MC-01"
    TagHT  = "TRTCalibPID-Col-MC-01"
else:
    dbfile="mycool_Data.db"
    dbname="COMP200" 
    TagToT = "TRTCalibPID_RToT-Col-01"
    TagHT  = "TRTCalibPID-Col-01"
    

def main():
    # remove the old db file so that we can write the new one
    try:
        import os
        os.remove(dbfile)
    except:
        pass
    # ----------------------------------------------------------------------- #
    # get database service and open database
    # ----------------------------------------------------------------------- #
    dbSvc=cool.DatabaseSvcFactory.databaseService()
    # database accessed via physical name
    dbstring="sqlite://;schema=%s;dbname=%s" % (dbfile,dbname)
    try:
        db=dbSvc.createDatabase(dbstring)
    except Exception,e:
        print 'Problem creating database',e
        sys.exit(-1)
    print "Created database",dbstring
    
    # ----------------------------------------------------------------------- #
    # setup a folder for ToT
    # ----------------------------------------------------------------------- #
    specHT=cool.RecordSpecification()
    specToT=cool.RecordSpecification()
    
    specHT.extend("TRT",cool.StorageType.Blob64k)    
    specToT.extend("TRT_RToT",cool.StorageType.Blob64k)
    
    # folder meta-data - note for Athena this has a special meaning
    descHT='<timeStamp>run-event</timeStamp><addrHeader><address_header service_type="71" clid="40774348" /></addrHeader><typeName>AthenaAttributeList</typeName>'
    descToT='<timeStamp>run-event</timeStamp><addrHeader><address_header service_type="71" clid="40774348" /></addrHeader><typeName>AthenaAttributeList</typeName>'
    
    # create the folder - single version
    # last argument is createParents - if true, automatically creates parent folders if needed
    # note this will not work if the database already exists - delete mycool.db first
    HTfolder=db.createFolder('/TRT/Calib/PIDver_New',specHT,descHT,cool.FolderVersioning.MULTI_VERSION,True)
    ToTfolder=db.createFolder('/TRT/Calib/PID_RToTver_New',specToT,descToT,cool.FolderVersioning.MULTI_VERSION,True)
        
    # ----------------------------------------------------------------------- #
    # now fill in some data - create a record and fill it
    # ----------------------------------------------------------------------- #
    data_HT=cool.Record(specHT)
    ToTdata=cool.Record(specToT)
    
    # get a reference to the blob
    HTblob=data_HT["TRT"]
    ToTblob=ToTdata['TRT_RToT']
    
    # set the size (in bytes)
    HTblob.resize(len(dataBytesHT))
    ToTblob.resize(len(dataBytesToT))

    print "Length data ToT",len(dataBytesToT)
    for i in xrange(0,len(dataBytesToT)):
        ToTblob[i]=dataBytesToT[i]

    print "Length data HT",len(dataBytesHT)
    for i in xrange(0,len(dataBytesHT)):
        HTblob[i]=dataBytesHT[i]


    print "Will store this object",data_HT #data_barrel,data_endcap
    print "Will store this object",ToTdata
    
    # store object with IOV valid from 0-10, channel 1
    HTfolder.storeObject(0,cool.ValidityKeyMax,data_HT,0,TagHT)
    ToTfolder.storeObject(0,cool.ValidityKeyMax,ToTdata,0,TagToT)
    print "Stored objects"

    db.closeDatabase()

#for MC:
if TYPE=="MC":
    dataBytesHT = [ 4, 27, 8, 10, 0, 0, 128, 63, 0, 0, 0, 0, 17, 235, 40, 61, 17, 108, 63, 59, 24, 236, 22, 62, 214, 57, 84, 64, 135, 220, 60, 62, 103, 133, 29, 61, 41, 246, 200, 59, 22, 246, 212, 61, 247, 117, 80, 64, 109, 113, 237, 61, 62, 211, 230, 60, 175, 67, 53, 60, 219, 110, 50, 62, 3, 62, 85, 64, 214, 200, 30, 62, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63,  ]
    dataBytesToT = [ 4, 27, 8, 10, 72, 225, 122, 63, 10, 215, 163, 60, 0, 0, 0, 0, 40, 112, 239, 62, 41, 150, 192, 64, 38, 198, 226, 62, 255, 238, 81, 60, 15, 156, 225, 64, 69, 158, 172, 63, 152, 219, 134, 62, 78, 98, 198, 64, 152, 161, 41, 63, 149, 74, 129, 62, 222, 171, 170, 64, 70, 96, 0, 63, 241, 189, 41, 63, 177, 80, 158, 64, 2, 126, 205, 62, 133, 245, 221, 59, 98, 74, 197, 64, 75, 4, 106, 63, 210, 0, 126, 65, 140, 202, 205, 65, 136, 22, 84, 74, 32, 170, 183, 73, 120, 209, 147, 63, 75, 200, 218, 65, 32, 224, 86, 74, 0, 212, 218, 74, 5, 197, 135, 65, 118, 55, 159, 64, 240, 30, 170, 73, 64, 181, 97, 73, 121, 233, 139, 65, 53, 99, 133, 64, 32, 41, 167, 73, 176, 186, 44, 73, 32, 65, 142, 65, 28, 235, 106, 65, 216, 89, 2, 74, 48, 123, 10, 73, 90, 117, 143, 65, 182, 33, 25, 65, 160, 88, 227, 73, 160, 40, 241, 72, 242, 48, 144, 65, 158, 94, 47, 65, 160, 92, 238, 73, 192, 211, 230, 72, 215, 180, 144, 65, 56, 243, 17, 64, 192, 184, 149, 73, 160, 76, 200, 72, 172, 139, 144, 65, 56, 16, 136, 64, 224, 76, 185, 73, 160, 77, 208, 72, 100, 221, 144, 65, 146, 150, 171, 64, 16, 133, 196, 73, 64, 4, 198, 72, 162, 69, 144, 65, 61, 126, 168, 64, 80, 125, 200, 73, 160, 59, 217, 72, 111, 129, 144, 65, 247, 117, 20, 65, 224, 160, 226, 73, 96, 250, 204, 72, 225, 218, 39, 196, 184, 142, 199, 68, 116, 68, 166, 77, 93, 214, 197, 77, 215, 131, 185, 195, 12, 226, 179, 67, 4, 160, 123, 203, 106, 96, 77, 77, 16, 216, 143, 65, 51, 0, 152, 71, 145, 216, 32, 87, 178, 157, 143, 65, 179, 61, 155, 71, 86, 85, 42, 88, 126, 140, 143, 65, 230, 75, 159, 71, 182, 160, 193, 86, 131, 175, 143, 65, 166, 111, 161, 71, 58, 17, 216, 87, 184, 175, 143, 65, 179, 81, 164, 71, 161, 227, 245, 86, 174, 199, 143, 65, 13, 80, 167, 71, 218, 93, 207, 87, 85, 176, 143, 65, 205, 223, 169, 71, 159, 62, 82, 86, 35, 91, 144, 65, 90, 187, 179, 71, 57, 1, 168, 85, 125, 208, 143, 65, 13, 67, 177, 71, 153, 48, 196, 87, 101, 25, 131, 65, 201, 214, 30, 67, 55, 121, 241, 67, 169, 36, 131, 65, 96, 197, 32, 67, 0, 48, 242, 67, 55, 9, 131, 65, 174, 135, 34, 67, 231, 43, 244, 67, 87, 219, 130, 65, 121, 137, 37, 67, 109, 199, 249, 67, 62, 104, 131, 65, 186, 169, 38, 67, 225, 74, 241, 67, 66, 79, 131, 65, 72, 1, 40, 67, 242, 242, 241, 67, 213, 137, 131, 65, 102, 166, 41, 67, 221, 52, 239, 67, 228, 148, 131, 65, 252, 41, 44, 67, 186, 137, 239, 67, 77, 149, 131, 65, 49, 104, 45, 67, 227, 5, 239, 67, 10, 104, 131, 65, 162, 229, 46, 67, 55, 121, 242, 67, 251, 220, 131, 65, 123, 244, 48, 67, 158, 143, 238, 67, 35, 202, 131, 65, 6, 225, 51, 67, 100, 11, 240, 67, 135, 150, 131, 65, 92, 15, 53, 67, 215, 243, 241, 67, 50, 213, 131, 65, 102, 6, 55, 67, 250, 30, 239, 67, 16, 233, 131, 65, 123, 180, 56, 67, 150, 99, 238, 67, 37, 245, 131, 65, 113, 157, 58, 67, 199, 43, 238, 67, 182, 243, 131, 65, 12, 162, 59, 67, 180, 40, 238, 67, 64, 36, 132, 65, 180, 72, 62, 67, 82, 216, 237, 67, 9, 138, 131, 65, 238, 220, 63, 67, 43, 135, 247, 67, 31, 244, 131, 65, 160, 250, 64, 67, 197, 32, 241, 67, 56, 231, 131, 65, 66, 160, 66, 67, 240, 199, 241, 67, 24, 21, 132, 65, 61, 10, 69, 67, 168, 102, 240, 67, 83, 5, 132, 65, 174, 167, 70, 67, 70, 134, 242, 67, 239, 201, 131, 65, 25, 36, 72, 67, 96, 197, 246, 67, 151, 255, 131, 65, 147, 184, 74, 67, 63, 21, 246, 67, 51, 196, 131, 65, 78, 66, 75, 67, 94, 186, 248, 67, 221, 36, 132, 65, 92, 207, 77, 67, 129, 5, 244, 67, 96, 229, 131, 65, 84, 3, 79, 67, 219, 185, 248, 67, 144, 177, 131, 65, 45, 114, 80, 67, 195, 165, 253, 67, 250, 254, 131, 65, 41, 124, 82, 67, 0, 16, 249, 67, 238, 218, 131, 65, 223, 175, 83, 67, 39, 129, 252, 67, 175, 148, 131, 65, 135, 214, 84, 67, 14, 45, 1, 68, 178, 174, 131, 65, 223, 175, 86, 67, 229, 96, 1, 68, 5, 35, 132, 65, 141, 55, 89, 67, 117, 83, 251, 67, 95, 24, 132, 65, 94, 90, 91, 67, 2, 219, 0, 68, 55, 154, 131, 65, 96, 37, 92, 67, 172, 100, 3, 68, 18, 3, 132, 65, 82, 24, 95, 67, 41, 52, 0, 68, 220, 232, 131, 65, 94, 250, 95, 67, 137, 81, 1, 68, 47, 238, 131, 65, 160, 186, 96, 67, 115, 88, 1, 68, 83, 22, 132, 65, 176, 50, 98, 67, 29, 90, 255, 67, 194, 6, 132, 65, 123, 20, 100, 67, 35, 235, 0, 68, 87, 219, 131, 65, 156, 228, 100, 67, 223, 183, 2, 68, 114, 249, 131, 65, 84, 195, 102, 67, 229, 144, 1, 68, 93, 237, 131, 65, 193, 138, 103, 67, 180, 216, 1, 68, 235, 226, 131, 65, 92, 239, 104, 67, 111, 234, 2, 68, 105, 239, 131, 65, 59, 159, 106, 67, 195, 109, 2, 68, 255, 33, 132, 65, 164, 208, 108, 67, 76, 151, 0, 68, 221, 53, 132, 65, 184, 30, 110, 67, 43, 167, 255, 67, 79, 47, 132, 65, 227, 5, 111, 67, 72, 105, 0, 68, 235, 98, 132, 65, 117, 115, 113, 67, 53, 126, 253, 67, 6, 1, 132, 65, 215, 227, 112, 67, 6, 241, 1, 68, 218, 44, 132, 65, 45, 146, 114, 67, 104, 9, 0, 68, 212, 43, 132, 65, 203, 161, 115, 67, 92, 31, 0, 68, 200, 7, 132, 65, 94, 26, 117, 67, 123, 220, 1, 68, 221, 53, 132, 65, 2, 43, 119, 67, 121, 161, 0, 68, 122, 37, 132, 65, 182, 179, 119, 67, 172, 212, 0, 68, 224, 45, 132, 65, 207, 23, 121, 67, 231, 203, 0, 68, 58, 18, 132, 65, 133, 75, 122, 67, 47, 29, 2, 68, 18, 148, 132, 65, 217, 46, 125, 67, 180, 232, 250, 67, 141, 151, 132, 65, 180, 104, 126, 67, 250, 110, 251, 67, 20, 63, 132, 65, 49, 72, 126, 67, 96, 101, 1, 68, 205, 59, 132, 65, 137, 65, 127, 67, 154, 201, 1, 68, 19, 97, 132, 65, 94, 218, 128, 67, 88, 177, 0, 68, 223, 96, 132, 65, 211, 189, 128, 67, 27, 167, 0, 68, 53, 222, 143, 65, 218, 135, 152, 71, 156, 205, 165, 88, 98, 161, 143, 65, 102, 60, 155, 71, 25, 243, 249, 87, 215, 146, 143, 65, 115, 110, 159, 71, 203, 201, 155, 87, 60, 78, 143, 65, 77, 186, 159, 71, 203, 124, 217, 88, 132, 158, 143, 65, 218, 202, 163, 71, 163, 91, 37, 87, 104, 162, 143, 65, 192, 56, 167, 71, 107, 22, 41, 88, 111, 129, 143, 65, 243, 159, 168, 71, 147, 48, 247, 87, 190, 31, 144, 65, 218, 29, 178, 71, 2, 1, 230, 86, 217, 223, 143, 65, 141, 183, 177, 71, 93, 171, 121, 86, 240, 39, 131, 65, 96, 229, 30, 67, 6, 225, 240, 67, 229, 80, 131, 65, 72, 129, 32, 67, 221, 116, 238, 67, 30, 39, 131, 65, 111, 50, 34, 67, 53, 174, 241, 67, 152, 221, 130, 65, 57, 52, 37, 67, 74, 220, 248, 67, 77, 21, 131, 65, 215, 195, 37, 67, 117, 3, 244, 67, 73, 46, 131, 65, 47, 125, 39, 67, 10, 231, 242, 67, 11, 70, 131, 65, 203, 129, 41, 67, 244, 77, 242, 67, 32, 65, 131, 65, 248, 51, 43, 67, 236, 193, 241, 67, 125, 80, 131, 65, 223, 15, 46, 67, 39, 145, 244, 67, 161, 120, 131, 65, 217, 110, 46, 67, 127, 202, 240, 67, 158, 222, 131, 65, 109, 7, 49, 67, 133, 187, 238, 67, 227, 165, 131, 65, 203, 97, 51, 67, 164, 128, 241, 67, 184, 158, 131, 65, 147, 216, 53, 67, 14, 93, 243, 67, 153, 187, 131, 65, 74, 12, 55, 67, 2, 139, 240, 67, 94, 203, 131, 65, 233, 6, 57, 67, 145, 13, 241, 67, 143, 228, 131, 65, 88, 25, 58, 67, 248, 19, 238, 67, 186, 218, 131, 65, 109, 167, 59, 67, 86, 110, 239, 67, 172, 11, 132, 65, 201, 118, 61, 67, 102, 198, 236, 67, 116, 198, 131, 65, 152, 206, 63, 67, 49, 232, 242, 67, 221, 36, 132, 65, 156, 68, 65, 67, 55, 89, 238, 67, 179, 12, 132, 65, 0, 0, 67, 67, 59, 95, 240, 67, 76, 166, 131, 65, 59, 191, 67, 67, 154, 89, 246, 67, 22, 251, 131, 65, 254, 148, 70, 67, 37, 134, 242, 67, 154, 136, 131, 65, 145, 237, 71, 67, 207, 167, 251, 67, 113, 172, 131, 65, 193, 74, 73, 67, 104, 113, 248, 67, 20, 208, 131, 65, 213, 120, 75, 67, 147, 248, 248, 67, 7, 223, 131, 65, 139, 44, 77, 67, 78, 2, 248, 67, 27, 141, 131, 65, 131, 96, 78, 67, 188, 20, 254, 67, 44, 229, 131, 65, 98, 208, 80, 67, 92, 159, 249, 67, 68, 233, 131, 65, 219, 121, 82, 67, 92, 47, 250, 67, 41, 203, 131, 65, 168, 102, 83, 67, 84, 83, 253, 67, 82, 201, 131, 65, 92, 207, 85, 67, 68, 235, 255, 67, 45, 161, 131, 65, 127, 74, 86, 67, 254, 92, 1, 68, 140, 202, 131, 65, 205, 12, 88, 67, 29, 26, 0, 68, 251, 58, 132, 65, 92, 47, 90, 67, 240, 7, 251, 67, 131, 209, 131, 65, 205, 44, 92, 67, 123, 244, 0, 68, 140, 185, 131, 65, 225, 26, 94, 67, 115, 136, 2, 68, 31, 5, 132, 65, 125, 223, 95, 67, 106, 236, 255, 67, 120, 250, 131, 65, 117, 51, 97, 67, 143, 202, 0, 68, 136, 227, 131, 65, 188, 20, 98, 67, 158, 167, 1, 68, 240, 5, 132, 65, 213, 152, 99, 67, 33, 176, 255, 67, 50, 213, 131, 65, 182, 243, 100, 67, 248, 251, 1, 68, 73, 29, 132, 65, 16, 216, 102, 67, 152, 78, 255, 67, 252, 24, 132, 65, 88, 249, 103, 67, 221, 20, 0, 68, 163, 18, 132, 65, 188, 148, 105, 67, 27, 159, 0, 68, 3, 9, 132, 65, 205, 204, 106, 67, 35, 203, 0, 68, 243, 31, 132, 65, 203, 97, 108, 67, 4, 126, 0, 68, 52, 17, 132, 65, 49, 136, 109, 67, 225, 162, 0, 68, 73, 29, 132, 65, 57, 180, 110, 67, 92, 175, 0, 68, 6, 18, 132, 65, 131, 64, 112, 67, 188, 92, 1, 68, 123, 3, 132, 65, 111, 114, 113, 67, 31, 245, 1, 68, 227, 71, 132, 65, 215, 3, 115, 67, 129, 149, 254, 67, 177, 80, 132, 65, 248, 19, 116, 67, 156, 52, 253, 67, 61, 44, 132, 65, 57, 84, 117, 67, 145, 69, 0, 68, 156, 68, 132, 65, 59, 95, 118, 67, 104, 209, 254, 67, 172, 45, 132, 65, 20, 110, 119, 67, 244, 53, 0, 68, 215, 35, 132, 65, 139, 236, 120, 67, 242, 106, 1, 68, 154, 25, 132, 65, 39, 81, 122, 67, 190, 95, 2, 68, 4, 86, 132, 65, 90, 164, 123, 67, 209, 50, 254, 67, 130, 115, 132, 65, 162, 229, 125, 67, 10, 167, 253, 67, 48, 76, 132, 65, 145, 13, 126, 67, 33, 208, 255, 67, 11, 70, 132, 65, 154, 153, 127, 67, 109, 23, 1, 68, 140, 57, 132, 65, 51, 3, 128, 67, 53, 78, 1, 68, 76, 38, 132, 65, 154, 233, 128, 67, 0, 32, 3, 68, 127, 106, 116, 65, 246, 23, 241, 65, 232, 215, 110, 74, 112, 28, 210, 73, 199, 41, 130, 64, 154, 247, 93, 66, 60, 172, 0, 75, 204, 96, 185, 74, 4, 231, 135, 65, 213, 231, 145, 65, 48, 124, 30, 74, 208, 9, 97, 73, 21, 12, 140, 65, 113, 172, 26, 65, 64, 31, 240, 73, 208, 203, 43, 73, 255, 33, 142, 65, 230, 174, 103, 65, 192, 148, 2, 74, 96, 47, 12, 73, 176, 131, 143, 65, 172, 45, 27, 65, 0, 107, 228, 73, 0, 200, 240, 72, 215, 52, 144, 65, 159, 60, 46, 65, 112, 30, 238, 73, 128, 251, 230, 72, 3, 137, 144, 65, 168, 215, 221, 65, 96, 59, 12, 74, 192, 70, 208, 72, 200, 135, 144, 65, 139, 113, 183, 64, 128, 252, 200, 73, 160, 211, 208, 72, 74, 251, 144, 65, 166, 208, 133, 64, 176, 27, 182, 73, 192, 188, 190, 72, 172, 45, 144, 65, 240, 191, 21, 64, 128, 90, 157, 73, 96, 6, 223, 72, 155, 102, 144, 65, 1, 19, 22, 64, 80, 29, 156, 73, 192, 240, 207, 72, 113, 93, 130, 196, 154, 183, 50, 70, 234, 106, 182, 78, 140, 103, 25, 78, 143, 202, 43, 197, 51, 212, 218, 70, 168, 63, 81, 79, 162, 195, 180, 78, 171, 62, 132, 65, 20, 63, 132, 65, 74, 251, 153, 65, 98, 161, 152, 65,  ]
else:
#for Data:
    dataBytesHT = [ 4, 27, 8, 10, 0, 0, 128, 63, 0, 0, 0, 0, 192, 131, 21, 61, 71, 192, 178, 59, 131, 164, 31, 62, 166, 126, 82, 64, 60, 192, 83, 62, 151, 213, 10, 61, 98, 227, 209, 59, 113, 198, 16, 62, 176, 85, 82, 64, 166, 11, 33, 62, 14, 67, 21, 61, 24, 1, 58, 59, 85, 250, 105, 62, 148, 193, 85, 64, 150, 236, 120, 62, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63, 0, 0, 128, 63,  ]
    dataBytesToT = [ 4, 27, 8, 10, 72, 225, 122, 63, 10, 215, 163, 60, 0, 0, 0, 0, 98, 31, 9, 61, 99, 238, 210, 64, 150, 207, 146, 191, 194, 250, 187, 60, 226, 6, 231, 64, 41, 61, 179, 62, 177, 0, 47, 63, 37, 35, 197, 64, 193, 2, 4, 63, 208, 229, 36, 63, 80, 1, 157, 64, 150, 122, 206, 190, 171, 120, 134, 62, 83, 5, 170, 64, 31, 187, 251, 62, 138, 180, 62, 60, 5, 221, 197, 64, 97, 252, 92, 63, 184, 12, 131, 197, 205, 84, 96, 70, 41, 241, 210, 78, 225, 249, 171, 78, 51, 251, 47, 198, 154, 143, 91, 70, 243, 249, 82, 78, 98, 15, 112, 79, 254, 84, 43, 195, 84, 227, 87, 67, 246, 253, 18, 75, 189, 46, 135, 76, 136, 133, 76, 65, 155, 85, 180, 65, 224, 42, 116, 74, 160, 113, 24, 74, 140, 219, 109, 65, 152, 23, 176, 64, 96, 136, 163, 73, 160, 197, 218, 73, 175, 148, 65, 65, 94, 58, 110, 66, 224, 133, 233, 74, 184, 174, 64, 74, 3, 9, 83, 65, 161, 45, 251, 63, 160, 193, 232, 201, 88, 113, 41, 74, 82, 39, 115, 65, 166, 27, 135, 65, 128, 228, 101, 74, 96, 140, 214, 73, 50, 119, 116, 65, 76, 55, 215, 65, 52, 231, 132, 74, 240, 56, 189, 73, 172, 11, 132, 65, 51, 220, 154, 64, 0, 215, 220, 73, 128, 40, 87, 73, 217, 62, 148, 195, 0, 143, 210, 70, 199, 218, 81, 78, 230, 115, 60, 77, 68, 235, 37, 196, 92, 87, 115, 68, 168, 60, 24, 77, 45, 184, 212, 77, 113, 137, 63, 197, 205, 148, 180, 69, 11, 42, 142, 78, 50, 21, 226, 78, 41, 150, 135, 197, 102, 184, 47, 70, 213, 74, 10, 79, 90, 197, 17, 79, 196, 194, 137, 65, 230, 172, 151, 71, 166, 164, 149, 85, 195, 245, 137, 65, 128, 229, 154, 71, 7, 35, 177, 85, 68, 105, 141, 65, 0, 85, 220, 71, 112, 69, 224, 73, 153, 187, 137, 65, 102, 20, 162, 71, 118, 187, 135, 75, 100, 204, 139, 65, 51, 35, 180, 71, 176, 181, 133, 74, 121, 216, 140, 65, 0, 93, 201, 71, 8, 122, 27, 74, 34, 236, 138, 65, 128, 148, 201, 71, 16, 15, 36, 74, 66, 96, 139, 65, 0, 66, 254, 71, 80, 156, 174, 73, 34, 14, 134, 65, 128, 170, 218, 71, 184, 50, 8, 74, 217, 206, 134, 65, 141, 215, 100, 67, 111, 34, 139, 67, 173, 122, 132, 65, 145, 45, 107, 67, 152, 94, 138, 67, 131, 175, 131, 65, 145, 45, 110, 67, 49, 88, 136, 67, 209, 145, 130, 65, 86, 14, 120, 67, 240, 39, 138, 67, 64, 36, 131, 65, 143, 130, 118, 67, 98, 96, 134, 67, 70, 165, 131, 65, 152, 174, 121, 67, 178, 45, 132, 67, 25, 226, 131, 65, 184, 190, 123, 67, 39, 1, 132, 67, 170, 2, 132, 65, 27, 15, 127, 67, 66, 48, 134, 67, 76, 166, 131, 65, 45, 114, 131, 67, 39, 161, 133, 67, 193, 185, 131, 65, 53, 62, 132, 67, 25, 100, 136, 67, 127, 123, 130, 65, 141, 71, 138, 67, 104, 225, 132, 67, 131, 192, 130, 65, 100, 59, 140, 67, 143, 66, 130, 67, 13, 113, 130, 65, 184, 126, 142, 67, 176, 66, 129, 67, 144, 160, 130, 65, 184, 14, 141, 67, 29, 250, 125, 67, 226, 88, 131, 65, 127, 10, 142, 67, 82, 248, 118, 67, 13, 241, 130, 65, 133, 123, 141, 67, 152, 238, 116, 67, 36, 40, 131, 65, 14, 93, 141, 67, 252, 169, 113, 67, 161, 214, 130, 65, 59, 191, 144, 67, 186, 201, 113, 67, 155, 230, 130, 65, 74, 28, 146, 67, 74, 44, 115, 67, 15, 156, 131, 65, 68, 187, 144, 67, 174, 71, 104, 67, 162, 69, 131, 65, 156, 84, 147, 67, 8, 140, 102, 67, 215, 163, 131, 65, 76, 87, 148, 67, 143, 98, 104, 67, 94, 58, 131, 65, 233, 246, 148, 67, 37, 70, 106, 67, 102, 119, 131, 65, 143, 114, 152, 67, 139, 204, 112, 67, 222, 19, 131, 65, 193, 202, 152, 67, 195, 53, 108, 67, 226, 88, 131, 65, 96, 245, 155, 67, 137, 193, 105, 67, 39, 160, 131, 65, 102, 134, 156, 67, 201, 54, 105, 67, 151, 144, 131, 65, 180, 104, 160, 67, 102, 198, 106, 67, 230, 46, 132, 65, 119, 62, 162, 67, 84, 99, 104, 67, 249, 32, 132, 65, 164, 144, 165, 67, 168, 134, 107, 67, 251, 186, 131, 65, 25, 100, 167, 67, 131, 64, 107, 67, 213, 103, 131, 65, 193, 234, 169, 67, 70, 118, 113, 67, 2, 188, 131, 65, 31, 229, 173, 67, 109, 135, 114, 67, 171, 190, 131, 65, 6, 17, 177, 67, 150, 227, 115, 67, 63, 87, 131, 65, 236, 49, 181, 67, 164, 240, 114, 67, 41, 109, 131, 65, 240, 215, 181, 67, 225, 90, 113, 67, 72, 225, 130, 65, 193, 26, 182, 67, 147, 24, 112, 67, 247, 245, 130, 65, 242, 34, 184, 67, 252, 73, 108, 67, 222, 147, 131, 65, 96, 165, 185, 67, 160, 26, 101, 67, 64, 36, 131, 65, 72, 113, 186, 67, 135, 182, 101, 67, 23, 200, 130, 65, 238, 92, 186, 67, 76, 119, 93, 67, 70, 20, 131, 65, 10, 71, 188, 67, 195, 181, 96, 67, 159, 205, 130, 65, 201, 102, 190, 67, 147, 88, 91, 67, 62, 249, 130, 65, 133, 91, 191, 67, 137, 193, 91, 67, 31, 116, 130, 65, 55, 153, 192, 67, 227, 197, 87, 67, 108, 137, 130, 65, 131, 64, 194, 67, 150, 35, 86, 67, 11, 70, 130, 65, 131, 176, 197, 67, 57, 180, 81, 67, 213, 137, 130, 65, 182, 211, 197, 67, 55, 41, 78, 67, 239, 184, 130, 65, 94, 42, 199, 67, 31, 229, 75, 67, 108, 248, 130, 65, 37, 214, 198, 67, 125, 31, 80, 67, 26, 64, 131, 65, 86, 30, 200, 67, 51, 211, 83, 67, 114, 121, 131, 65, 219, 185, 202, 67, 164, 240, 82, 67, 211, 205, 130, 65, 150, 211, 206, 67, 94, 218, 76, 67, 198, 109, 130, 65, 215, 67, 208, 67, 98, 80, 81, 67, 135, 22, 131, 65, 29, 58, 208, 67, 147, 184, 77, 67, 238, 252, 130, 65, 223, 159, 212, 67, 225, 122, 78, 67, 173, 122, 130, 65, 217, 158, 214, 67, 23, 185, 77, 67, 61, 155, 129, 65, 109, 23, 217, 67, 92, 15, 79, 67, 70, 182, 130, 65, 219, 249, 221, 67, 199, 11, 78, 67, 159, 205, 130, 65, 2, 171, 224, 67, 180, 200, 81, 67, 100, 221, 130, 65, 84, 243, 231, 67, 145, 237, 80, 67, 245, 202, 130, 65, 63, 21, 234, 67, 113, 29, 79, 67, 191, 14, 131, 65, 12, 162, 237, 67, 168, 6, 81, 67, 222, 113, 131, 65, 158, 127, 242, 67, 43, 103, 89, 67, 131, 81, 137, 65, 128, 220, 162, 71, 220, 228, 202, 74, 168, 87, 136, 65, 243, 80, 157, 71, 60, 46, 96, 75, 51, 68, 139, 65, 0, 184, 215, 71, 176, 128, 216, 73, 249, 143, 138, 65, 179, 148, 191, 71, 152, 87, 28, 74, 31, 244, 137, 65, 192, 74, 175, 71, 60, 230, 130, 74, 20, 46, 139, 65, 26, 254, 194, 71, 216, 159, 22, 74, 12, 2, 138, 65, 0, 58, 197, 71, 56, 73, 23, 74, 38, 83, 139, 65, 0, 170, 8, 72, 96, 116, 133, 73, 99, 255, 133, 65, 128, 194, 238, 71, 0, 230, 180, 73, 39, 49, 134, 65, 27, 207, 89, 67, 49, 72, 134, 67, 13, 224, 131, 65, 217, 174, 94, 67, 82, 40, 133, 67, 213, 231, 130, 65, 195, 53, 97, 67, 20, 174, 130, 67, 143, 100, 130, 65, 35, 219, 107, 67, 236, 1, 135, 67, 191, 253, 130, 65, 115, 232, 103, 67, 205, 220, 131, 67, 45, 33, 131, 65, 225, 154, 108, 67, 233, 214, 130, 67, 143, 100, 131, 65, 102, 38, 110, 67, 238, 108, 132, 67, 254, 84, 131, 65, 63, 117, 114, 67, 188, 84, 131, 67, 154, 153, 131, 65, 96, 37, 120, 67, 27, 159, 134, 67, 175, 148, 131, 65, 135, 54, 123, 67, 78, 82, 132, 67, 44, 84, 131, 65, 217, 238, 129, 67, 12, 50, 135, 67, 7, 206, 130, 65, 199, 251, 133, 67, 18, 35, 133, 67, 165, 78, 131, 65, 203, 97, 135, 67, 8, 92, 131, 67, 195, 100, 131, 65, 53, 174, 134, 67, 201, 6, 131, 67, 191, 125, 131, 65, 170, 33, 136, 67, 51, 147, 125, 67, 34, 236, 131, 65, 127, 58, 135, 67, 141, 23, 120, 67, 36, 168, 131, 65, 172, 220, 135, 67, 68, 107, 115, 67, 128, 55, 131, 65, 2, 91, 138, 67, 117, 83, 116, 67, 171, 79, 131, 65, 170, 33, 141, 67, 244, 29, 117, 67, 250, 109, 131, 65, 147, 152, 140, 67, 199, 171, 110, 67, 89, 23, 131, 65, 145, 189, 142, 67, 45, 242, 110, 67, 166, 138, 131, 65, 80, 29, 144, 67, 123, 180, 109, 67, 116, 36, 131, 65, 150, 147, 145, 67, 119, 30, 116, 67, 168, 70, 131, 65, 182, 131, 149, 67, 203, 1, 119, 67, 219, 249, 130, 65, 121, 153, 150, 67, 121, 41, 111, 67, 96, 101, 131, 65, 4, 70, 152, 67, 23, 25, 112, 67, 86, 159, 131, 65, 154, 89, 153, 67, 170, 81, 115, 67, 179, 12, 132, 65, 197, 128, 156, 67, 225, 154, 114, 67, 151, 144, 131, 65, 117, 243, 160, 67, 190, 95, 118, 67, 150, 33, 132, 65, 158, 111, 162, 67, 139, 12, 119, 67, 82, 167, 131, 65, 106, 124, 165, 67, 158, 143, 115, 67, 17, 54, 131, 65, 86, 222, 168, 67, 254, 84, 125, 67, 31, 244, 131, 65, 27, 127, 172, 67, 82, 120, 123, 67, 240, 167, 131, 65, 94, 26, 174, 67, 150, 99, 126, 67, 156, 51, 131, 65, 131, 96, 181, 67, 141, 151, 107, 67, 234, 21, 131, 65, 248, 67, 180, 67, 252, 41, 107, 67, 194, 151, 130, 65, 10, 103, 181, 67, 88, 249, 100, 67, 238, 235, 130, 65, 53, 142, 181, 67, 225, 122, 104, 67, 75, 106, 131, 65, 70, 118, 181, 67, 188, 244, 93, 67, 48, 76, 131, 65, 115, 8, 182, 67, 201, 54, 95, 67, 54, 43, 131, 65, 49, 104, 183, 67, 162, 165, 91, 67, 206, 8, 131, 65, 254, 132, 184, 67, 207, 247, 90, 67, 234, 21, 131, 65, 117, 163, 187, 67, 180, 232, 86, 67, 233, 38, 131, 65, 203, 97, 187, 67, 219, 89, 89, 67, 145, 126, 130, 65, 57, 68, 187, 67, 213, 248, 83, 67, 179, 140, 130, 65, 0, 0, 189, 67, 215, 99, 87, 67, 130, 226, 130, 65, 82, 72, 190, 67, 164, 16, 78, 67, 138, 159, 130, 65, 213, 248, 192, 67, 86, 78, 78, 67, 189, 99, 130, 65, 18, 163, 192, 67, 240, 103, 77, 67, 130, 243, 130, 65, 20, 78, 192, 67, 4, 150, 74, 67, 231, 12, 131, 65, 43, 231, 194, 67, 43, 39, 80, 67, 154, 25, 131, 65, 18, 131, 196, 67, 227, 69, 79, 67, 108, 120, 130, 65, 252, 121, 200, 67, 14, 77, 73, 67, 245, 185, 130, 65, 240, 71, 201, 67, 223, 15, 75, 67, 80, 13, 131, 65, 213, 8, 203, 67, 72, 193, 72, 67, 178, 29, 131, 65, 12, 178, 205, 67, 133, 107, 76, 67, 3, 137, 130, 65, 53, 190, 209, 67, 154, 249, 74, 67, 102, 119, 130, 65, 35, 171, 211, 67, 242, 50, 70, 67, 221, 164, 130, 65, 252, 89, 213, 67, 180, 136, 76, 67, 123, 3, 131, 65, 43, 119, 218, 67, 80, 141, 78, 67, 178, 174, 130, 65, 229, 112, 226, 67, 53, 126, 76, 67, 92, 160, 130, 65, 193, 154, 228, 67, 106, 156, 74, 67, 197, 15, 131, 65, 248, 83, 231, 67, 66, 0, 79, 67, 219, 10, 131, 65, 76, 183, 236, 67, 223, 15, 85, 67, 33, 128, 180, 195, 20, 6, 132, 68, 102, 249, 3, 77, 230, 152, 0, 77, 154, 133, 167, 197, 246, 54, 210, 69, 34, 93, 211, 77, 73, 25, 236, 78, 41, 174, 159, 64, 48, 59, 158, 65, 160, 242, 30, 74, 104, 115, 161, 74, 105, 29, 249, 63, 8, 253, 69, 66, 232, 58, 255, 74, 48, 57, 215, 74, 22, 251, 90, 65, 149, 212, 64, 65, 32, 120, 40, 74, 184, 106, 2, 74, 188, 5, 121, 65, 98, 21, 159, 64, 80, 139, 186, 73, 240, 200, 160, 73, 133, 139, 180, 197, 154, 8, 139, 70, 53, 2, 64, 79, 52, 33, 43, 79, 166, 155, 106, 65, 14, 79, 151, 64, 160, 104, 136, 73, 8, 55, 24, 74, 232, 234, 133, 65, 116, 36, 120, 65, 64, 185, 68, 74, 48, 182, 140, 73, 206, 42, 135, 65, 154, 95, 3, 65, 216, 193, 18, 74, 128, 112, 122, 73, 64, 164, 115, 65, 190, 65, 138, 65, 192, 226, 124, 74, 0, 96, 244, 73, 82, 32, 236, 196, 72, 105, 66, 69, 234, 124, 18, 78, 130, 152, 149, 78, 205, 215, 144, 198, 102, 130, 138, 70, 36, 123, 245, 205, 117, 24, 40, 80, 10, 143, 230, 196, 0, 34, 160, 69, 153, 162, 122, 78, 166, 131, 119, 78, 171, 62, 132, 65, 20, 63, 132, 65, 74, 251, 153, 65, 98, 161, 152, 65,  ]

if __name__=="__main__":
    main()
